/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 15 1.11.2.5 source/albert/graph2d/colprof.hpp, 2d, ioc.v400, 000919 
/*==========================================
||
||	File:	ColorProfile.h (colprof.hpp)
||
||	What:	ColorProfile routines for Color matching
||
||	Change History:
||
*/

#ifndef _COLPROF_
#define _COLPROF_

#ifndef NO_FAKE_ICC

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

//classes defined by ICC
class icHeader;
class icTagList;
typedef char iccTagData ;
typedef long iccTagSignature ;
typedef long icS15Fixed16Number;

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#else

#ifndef ICC_H
#include <ICC.h>
#endif

#endif

#include <ibcolor.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

// always <=1
typedef float ChromaCoord;

// used in setting the white  point of the monitor
typedef float Temperature;

// used in most device independent color spaces Generally > 1 can be negative
typedef float TristimulusCoord;

#pragma pack(push,4)

//----------------------------------------------------------------
class IColorProfile {

public:
    IColorProfile();
    IColorProfile(const IColorProfile& aColorProfile);

 	IColorProfile& operator=(const IColorProfile& source);
    virtual ~IColorProfile();

// Override these for converting between RGB and XYZ

	virtual	void convertRGBtoXYZ(
		GIntensity red,
		GIntensity green,
		GIntensity blue,
		GIntensity& X,
		GIntensity& Y,
		GIntensity& Z) const;
	
	virtual	void convertXYZtoRGB(
		GIntensity X,
		GIntensity Y,
		GIntensity Z,
		GIntensity& red,
		GIntensity& green,
		GIntensity& blue) const;

// These are common read only gets to the ICCcolor ICCface.

	const icHeader* header() const;
    const icTagList* tagList() const;

private:

// Get Tag from color profile returns a Tag from the Color Profile
// Throws an exception  if signature not in profile
    const iccTagData* tagFromProfile(const iccTagSignature& signature) const;

    icHeader* fheader;	
    icTagList* ficTagList;

	static const unsigned long fMatrixDimension;

// Stuff to go between RGB<->XYZ

	void makefXYZ2RGBMatrix()  ;
    void defineTransformationMatrix() ;

	double fRGB2XYZ[3][3];
	double fXYZ2RGB[3][3];
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _COLPROF_

