// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 89 1.2.2.4 source/albert/graph2d/deleter.hpp, 2d, ioc.v400, 000919 

#ifndef _DELETER_
#define _DELETER_

//===================================================================
// A smart pointer class for automatically allocate and
// release a POINTL pointer to an array of objects

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class AType>
class IDeleterForArrayOf
{
public:
	IDeleterForArrayOf();
	IDeleterForArrayOf(AType*);
	~IDeleterForArrayOf();
		
	void operator=(AType*);
	AType& operator[](unsigned long);
	operator AType*();

private:
	IDeleterForArrayOf(const IDeleterForArrayOf<AType>&);
	IDeleterForArrayOf<AType>& operator=(const IDeleterForArrayOf<AType>&);
	AType* fObject;
};

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

template <class AType>
inline
IDeleterForArrayOf<AType>::IDeleterForArrayOf() : fObject(0)
{}

template <class AType>
inline
IDeleterForArrayOf<AType>::IDeleterForArrayOf(AType* obj) : fObject(obj)
{}

template <class AType>
inline
IDeleterForArrayOf<AType>::~IDeleterForArrayOf()
{
	delete [] fObject;
}

template <class AType>
inline
void IDeleterForArrayOf<AType>::operator=(AType* newObject)
{
	if (newObject != fObject)
	{
		delete [] fObject;
		fObject = newObject;
	}
}

template <class AType>
inline
AType& IDeleterForArrayOf<AType>::operator[](unsigned long index)
{
	return fObject[index];
}

template <class AType>
inline
IDeleterForArrayOf<AType>::operator AType*()
{
	return fObject;
}

template <class AType>
inline
IDeleterForArrayOf<AType>& IDeleterForArrayOf<AType>::operator=(const IDeleterForArrayOf<AType>&)
{
	return *this;
}

template <class AType>
inline
IDeleterForArrayOf<AType>::IDeleterForArrayOf(const IDeleterForArrayOf<AType>&)
{}

#endif // _DELETER_
