// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 18  1.54.1.8  source/albert/graph2d/gdidev.hpp, 2d, ioc.v400, 000919  
/*================
||
||  File:       GdiDevice.h
||
||  What:       Routines which support the graphics device on Windows GDI platform
||
||  Change History:
||             ?           Isai Scheinberg -   Original
||             20 May 96   tt              -   First rewrite
||             16 Jan 97   xl              -   Added Win95 Transform Text Support
||
*/

#ifndef _GDIDEV_
#define _GDIDEV_

#include <ilanglvl.hpp>
#include <igrtypes.hpp>
#include <grafdevs.hpp>
#include <grstate.hpp>
#include <ibundles.hpp>
#include <iexgrprt.hpp>                        // for EDeviceMappingMode

#include <ipoint.hpp>
#include <irect.hpp>
#include <i2dghand.hpp>
#include <iexcept.hpp>
#include <windows.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGPoint2D;
class IGRect2D;
class IGLine2D;
class IGRect2D;
class IGPoint2D;
class IGEllipse2D;
class IGPolygon2D;
class IGPolyline2D;
class IGCurve2D;
class IGLoop2D;
class IGImage;
class IGTextRun;
class IColorTransferMode;
class IMaskPattern;

class IGrafState;
class IBaseColor;
class IPaint;
class ITextStyleSet;

class IRegionGrafExtractor;

class IGdiFontContext;
class IGdiNonUnicodeFontContext;
class IGdiWin95FontContext;
class IGdiWin95RotateFontContext;
class IGdiWin95ScaleFontContext;
class IGdiWin95OutlineFontContext;

class IPenContext;
class IBrushContext;

#pragma enum(4)
#pragma pack(push,4)

/*================================================================
||
||             IGdiDevice
||
*/

class IGdiDevice: public IGrafDevice {
public:
        IGdiDevice(
               const IPresSpaceHandle& deviceContext,
               bool baseRootFlag = true);
        IGdiDevice(
               const IPresSpaceHandle& deviceContext,
               EDeviceMappingMode mappingMode,
               bool baseRootFlag = true);

        IGdiDevice(
               const IPresSpaceHandle& deviceContext,
               const IGRect2D& clipRect,
               const IGrafMatrix& resolution = IGrafMatrix::identity());

        virtual ~IGdiDevice();

        virtual void renderLine(const IGLine2D& geometry, const IGrafState& grafState);
        virtual void renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState);
        virtual void renderCurve(const IGCurve2D& geometry, const IGrafState& grafState);
        virtual void renderRect(const IGRect2D& geometry, const IGrafState& grafState);
        virtual void renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState);
        virtual void renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState);
        virtual void renderLoop(const IGLoop2D& geometry, const IGrafState& grafState);
        virtual void renderArea(const IGArea& geometry, const IGrafState& grafState);

        virtual void renderImage(
               const IGImage& image,
               const IGRect2D& source,
               const IGrafState& grafState);

        virtual void renderMaskedImage(
               const IGImage& image,
               const IGImage& mask,
               const IGRect2D& source,
               const IGrafState& grafState);

        virtual IGPoint2D renderTextRun(const IGTextRun& geometry, const IGrafState& grafState);

        virtual const IGrafMatrix& deviceResolution(
               const IGRect2D& areaOfInterest = IGRect2D::infiniteRect()) const;

        virtual const IPresSpaceHandle& deviceContext() const;
        virtual void adoptDeviceContext(const IPresSpaceHandle& deviceContext);
        virtual IPresSpaceHandle orphanDeviceContext();

#if !IC_RTTI_ENABLED
        GrafDeviceTypeDeclarationsMacro(IGdiDevice);
#endif // IC_RTTI_ENABLED

private:
friend class IGTextRun;
        static void determineTextMetrics(IGTextRun& textRun, IGdiDevice*, const IGrafState*);

        static COLORREF convertToWRGB(const IBaseColor& color);

protected:
        // this is disabled ... it can only be called by disabled functions to assist compile...
        IGdiDevice();
        void SetWinWorldTransform(const IGrafMatrix* grafmatrix);

friend class ConicMaker;
friend class IGImagePixelAccessor;
friend class IPmPathVertexEngine;
        static int GCoordinateToInt(GCoordinate value);

        void setupTransforms(const IGrafState& state);
//      static enum EPlatform { kWin95, kWinNT, kOS2, kUndefined } fPlatform;
        int SetClipArea(const IGrafState& state);

        IStateChange fStateChange;                     // used for storing the delta state for each render call

private:
        void setupDevice(const IPresSpaceHandle& deviceHandle);
        void initialise(int mmode = MM_TEXT, const IGRect2D* clipRect = 0 /*NIL*/);
        void IGPoint2DToPOINTL(const IGPoint2D &p, POINTL &ptl);

        // methods
        void SetFillPaint(const IGrafState& state, bool nullPen = false);
        void SetFramePaint(const IGrafState& state, bool hollowBrush = false);
        void SetFillAndFramePaint(const IGrafState& state);

        HPEN CreatePen(const IGrafState& state);
        HBRUSH CreateFillBrush(const IGrafState& state);
        HBRUSH CreateFrameBrush(const IGrafState& state);
        HBRUSH CreateBrush(const LOGBRUSH& logbrush);
        LOGBRUSH CreateBrushStructure(const IPaint& paint);
        bool FrameEqualFillPaintModes(const IGrafState& state) const;

        void SetFillPaintModes(const IGrafState& state);
        void SetFramePaintModes(const IGrafState& state);
        void SetPaintModes(const IPaint* paint, const IColorTransferMode* xmode);


        void DrawArc(const IGCurve2D& curve, bool isLoop);

        int imageSamplingControl(const IGrafState& state);
        int imageTransferMode(const IGrafState& state);

        IBaseColor Win95ImageSamplingControl(
               IGImagePixelAccessor& source,
               IGImagePixelAccessor&  dest,
               IGPoint2D srcPoint,
               int destX,
               int destY,
               const IGrafState& state);

        void Win95RenderImage(
               const IGImage& image,
               const IGRect2D& source,
               const IGrafState& grafState);

        void Win95RenderMaskedImage(
               const IGImage& image,
               const IGImage& mask,
               const IGRect2D& source,
               const IGrafState& grafState);

/*
        void internalRenderImage(
               const IGRect2D& rect,
               long rasterOperation,
               IGImage* srcIGImage,
               IGImage* srcIGMatte);
*/

        // data:
        HDC fHDC;
        IPresSpaceHandle fPsCache;
        HDC fHDCOwned; // for situations that the given HDC is null, HDC of the desktop is pulled
        int fSavedDC;  // for saveDC/restoreDC to work well...
        IAttributeState::EDrawOperation fDrawOperation;
        IGrafMatrix* fResolution;

        IGrafMatrix fMatrix;                           // T(geometry) where T(model) = T(geometry) . T(view)
        const IGrafMatrix* fFrameMatrix;       // used for transform the frame geometry
        const IGrafMatrix* fViewMatrix;                // used for set to GDI WorldTransform
        const IGrafMatrix* fModelMatrix;       // used for transform the fill geometry
        const IGrafMatrix* fPenMatrix;         // used for calculate the pen width

        unsigned long fCachePaintModeStateChange;      // to cache paintMode state change
        unsigned long fCacheTransferMode;
        bool fBackModeSet;

        unsigned long fCachePenBrushStateChange;       // to cache paints state change
        bool fNullPenWhenFill;
        bool fHollowBrushWhenFrame;
        IPenContext* fPenCache;                                // used to cache the pen for paints
        IBrushContext* fBrushCache;                    // used to cache the brush for paints

        IGrafMatrix fMatrixCache;
        bool fMatrixCacheUsed;
        XFORM fXFORM;                                          // GDI matrix

        bool fClipAreaSet;
        bool fBaseRootDevice; // true if the device is created by an IBaseRootGrafPort, otherwise, false.
        bool  baseRootDevice() const;
  IGrafMatrix  *resolutionMatrix();


        static bool fInitialized;
        static GCoordinate gMaxCoord;
        static GCoordinate gMinCoord;

        friend class IRegionGrafExtractor;
        friend class IGdiFontContext;
        friend class IGdiNonUnicodeFontContext;
        friend class IGdiWin95FontContext;
        friend class IGdiWin95RotateFontContext;
        friend class IGdiWin95OutlineFontContext;
        friend class IGdiWin95ScaleFontContext;

        IGdiFontContext* fFontCache;
        IGdiFontContext& makeFontContext(
               const IGTextRun&,
               const IGrafMatrix*,
               const IGrafState*,
               bool unicodeSupported);

        // used for text colour caching:
        IBaseColor fTextColour;
        bool fTextColourCached;

        void setTextColour(const IBaseColor& target);

        // disabled functions:
        IGdiDevice(const IGdiDevice&);
        IGdiDevice& operator=(const IGdiDevice& Src);
};


inline COLORREF IGdiDevice::convertToWRGB(const IBaseColor& color)
{
        return PALETTERGB(color.redMix(), color.greenMix(), color.blueMix());
}

// --------------------------------------------------------------------------------
// this is the arbitrator object for making the right device for the root grafport on
// the right platforms... may be this should be in a independent header file

class IGrafDeviceMaker {
public:
        static IGrafDevice* makeDevice(
               const IPresSpaceHandle& deviceContext,
          const IGrafPort::EGrafPortType = IGrafPort::kIBaseRootGrafPort,
               EDeviceMappingMode mappingMode = kPixel);
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _GDIDEV_
