// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 17 1.4.1.7 source/albert/graph2d/gpidev.hpp, 2d, ioc.v400, 000919 
/*================
||
||  File:       GpiDevice.h
||
||  What:       Routines which support the graphics device on os2 GPI platform
||
||  Change History:
||		?           Isai Scheinberg -	Original
||		20 May 96   tt              -	First rewrite
||
*/

#ifndef _GPIDEV_
#define _GPIDEV_

#include <ilanglvl.hpp>
#include <igrtypes.hpp>
#include <grafdevs.hpp>
#include <grstate.hpp>
#include <ibundles.hpp>
#include <iexgrprt.hpp>			// for EDeviceMappingMode
#include <ipoint.hpp>
#include <irect.hpp>
#include <i2dghand.hpp>
#include <iexcept.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGPoint2D;
class IGRect2D;
class IGLine2D;
class IGRect2D;
class IGPoint2D;
class IGEllipse2D;
class IGPolygon2D;
class IGPolyline2D;
class IGCurve2D;
class IGLoop2D;
class IGImage;
class IGTextRun;
class IColorTransferMode;
class IMaskPattern;

class IGrafState;
class IBaseColor;
class IPaint;
class ITextStyleSet;

class IRegionGrafExtractor;

class IGdiFontContext;
class IGdiNonUnicodeFontContext;

class IPenContext;
class IBrushContext;

class IGpiEmulator;

#pragma enum(4)
#pragma pack(push,4)

/*================================================================
||
||		IGpiDevice
||
*/

class IGpiDevice: public IGrafDevice {
public:
	IGpiDevice(
		const IPresSpaceHandle& deviceContext,
		bool baseRootFlag = true);
		
	IGpiDevice(
		const IPresSpaceHandle& deviceContext,
		EDeviceMappingMode mappingMode,
		bool baseRootFlag = true);

	IGpiDevice(
		const IPresSpaceHandle& deviceContext,
		const IGRect2D& clipRect,
		const IGrafMatrix& resolution = IGrafMatrix::identity());

    virtual ICoordinateSystem::EOrientation coordinateSystem() const;
	virtual ~IGpiDevice();
	
	virtual void renderLine(const IGLine2D& geometry, const IGrafState& grafState);
	virtual void renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState);
	virtual void renderCurve(const IGCurve2D& geometry, const IGrafState& grafState);
	virtual void renderRect(const IGRect2D& geometry, const IGrafState& grafState);
	virtual void renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState);
	virtual void renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState);
	virtual void renderLoop(const IGLoop2D& geometry, const IGrafState& grafState);
	virtual void renderArea(const IGArea& geometry, const IGrafState& grafState);
	
	virtual void renderImage(
		const IGImage& image,
		const IGRect2D& source,
		const IGrafState& grafState);
	
	virtual void renderMaskedImage(
		const IGImage& image,
		const IGImage& mask,
		const IGRect2D& source,
		const IGrafState& grafState);

	virtual IGPoint2D renderTextRun(const IGTextRun& geometry, const IGrafState& grafState);

	virtual const IGrafMatrix& deviceResolution(
		const IGRect2D& areaOfInterest = IGRect2D::infiniteRect()) const;

	virtual const IPresSpaceHandle& deviceContext() const;
	virtual void adoptDeviceContext(const IPresSpaceHandle& deviceContext);
	virtual IPresSpaceHandle orphanDeviceContext();
	
#if !IC_RTTI_ENABLED
	GrafDeviceTypeDeclarationsMacro(IGpiDevice);
#endif // IC_RTTI_ENABLED

private:
friend class IGTextRun;
	static void determineTextMetrics(IGTextRun& textRun, IGpiDevice*, const IGrafState*);

protected:
	// this is disabled ... it can only be called by disabled functions to assist compile...
	IGpiDevice();
	void SetWinWorldTransform(const IGrafMatrix* grafmatrix);
	
friend class ConicMaker;
friend class IGImagePixelAccessor;
friend class IPmPathVertexEngine;
	static int GCoordinateToInt(GCoordinate value);

	void setupTransforms(const IGrafState& state);	
//	static enum EPlatform { kWin95, kWinNT, kOS2, kUndefined } fPlatform;
	int SetClipArea(const IGrafState& state);

	IStateChange fStateChange;			// used for storing the delta state for each render call

private:
	void setupDevice(const IPresSpaceHandle& deviceHandle);
	void initialise(int mmode = 0x0008L /*MM_TEXT*/, const IGRect2D* clipRect = 0 /*NIL*/);

	// methods
 	void SetFillPaint(const IGrafState& state, bool nullPen = false);
 	void SetFramePaint(const IGrafState& state, bool hollowBrush = false);
 	void SetFillAndFramePaint(const IGrafState& state);

	bool FrameEqualFillPaintModes(const IGrafState& state) const;
	
	void SetFillPaintModes(const IGrafState& state);
	void SetFramePaintModes(const IGrafState& state);
	void SetPaintModes(const IPaint* paint, const IColorTransferMode* xmode);


	void DrawArc(const IGCurve2D& curve, bool isLoop);
	
	int imageSamplingControl(const IGrafState& state);
	int imageTransferMode(const IGrafState& state);

	// data:
    //  On OS/2 the IPresSpaceHandle are used to preserve the window
    //  handles.
    //

	IPresSpaceHandle fHDC;
	IPresSpaceHandle fHDCOwned;

	int fSavedDC;	// for saveDC/restoreDC to work well...
	IAttributeState::EDrawOperation fDrawOperation;
	IGrafMatrix* fResolution;

	IGrafMatrix fMatrix;				// T(geometry) where T(model) = T(geometry) . T(view)
	const IGrafMatrix* fFrameMatrix;	// used for transform the frame geometry
	const IGrafMatrix* fViewMatrix;		// used for set to GDI WorldTransform
	const IGrafMatrix* fModelMatrix;	// used for transform the fill geometry
	const IGrafMatrix* fPenMatrix;		// used for calculate the pen width

	unsigned long fCachePaintModeStateChange;	// to cache paintMode state change
	unsigned long fCacheTransferMode;
 	bool fBackModeSet;

 	unsigned long fCachePenBrushStateChange;	// to cache paints state change
 	bool fNullPenWhenFill;
 	bool fHollowBrushWhenFrame;
 	IPenContext* fPenCache;				// used to cache the pen for paints
 	IBrushContext* fBrushCache;			// used to cache the brush for paints
	
	IGrafMatrix fMatrixCache;
	bool fMatrixCacheUsed;

	bool fClipAreaSet;
	bool fBaseRootDevice; // true if the device is created by an IBaseRootGrafPort, otherwise, false.
	bool  baseRootDevice() const;
  IGrafMatrix  *resolutionMatrix();
	
	
	static bool fInitialized;
	static GCoordinate gMaxCoord;
	static GCoordinate gMinCoord;

	friend class IRegionGrafExtractor;
	friend class IGdiFontContext;
	friend class IGdiNonUnicodeFontContext;

   friend class IGpiEmulator;
   IGpiEmulator* fGpiEmulator;

	IGdiFontContext* fFontCache;
	IGdiFontContext& makeFontContext(
		const IGTextRun&,
		const IGrafMatrix*,
		const IGrafState*,
		bool unicodeSupported);

	// used for text colour caching:
	IBaseColor fTextColour;
	bool fTextColourCached;

	void setTextColour(const IBaseColor& target);

	// disabled functions:
	IGpiDevice(const IGpiDevice&);
	IGpiDevice& operator=(const IGpiDevice& Src);
};


// --------------------------------------------------------------------------------
// this is the arbitrator object for making the right device for the root grafport on
// the right platforms... may be this should be in a independent header file

class IGrafDeviceMaker {
public:
	static IGrafDevice* makeDevice(
		const IPresSpaceHandle& deviceContext,
	  const IGrafPort::EGrafPortType = IGrafPort::kIBaseRootGrafPort,
		EDeviceMappingMode mappingMode = kPixel);
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _GPIDEV_
