// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 20 1.18.1.5 source/albert/graph2d/grafdevs.hpp, 2d, ioc.v400, 000919 
/*================
||
||  File:   GrafDevices.h
||
||  What:   Albert Graphic Devices
||
||  Change History:
||
*/

#ifndef _GRAFDEVS_
#define _GRAFDEVS_

#include <iexgrprt.hpp>
#include <igbase2d.hpp>      // for IGRect2D
#include <imatrix.hpp>      // for IGrafMatrix

#include <istring.hpp>
#include <ilanglvl.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGArea;
class IGCurve2D;
class IConicInfo;
class IGEllipse2D;
class IGLine2D;
class IGPolygon2D;
class IGLoop2D;
class IGPolyline2D;
class IGrafBundle;
class IPaint;
class IGImage;
class IGTextRun;
class IGrafState;
class IGrafDevice;
class IGrafExtractor;
class ICAGNode;

#pragma enum(4)
#pragma pack(push,4)

/*================
||
||  IGrafDevice
||
*/

#if !IC_RTTI_ENABLED

#define GrafDeviceTypeDeclarationsMacro(atype) \
    private: static const char* gDeviceClassName; \
            static const IString* gTypeNamePtr; \
    public: virtual const IString* deviceType() const \
            { if (!gTypeNamePtr) gTypeNamePtr = new IString(gDeviceClassName); return gTypeNamePtr; }

#define GrafDeviceTypeDefinitionsMacro(atype) \
    const IString* atype::gTypeNamePtr = 0 /*NIL*/; \
    const char* atype::gDeviceClassName = #atype

#endif // IC_RTTI_ENABLED

class IGrafDevice {
public:

    // === Constructors and Destructors ===

    virtual ~IGrafDevice();

    // === render calls ===

    virtual void renderLine(const IGLine2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderCurve(const IGCurve2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderRect(const IGRect2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderLoop(const IGLoop2D& geometry, const IGrafState& grafState) = 0;

    virtual void renderArea(const IGArea& geometry, const IGrafState& grafState) = 0;

    void renderImage(const IGImage& geometry, const IGrafState& grafState);

    void renderMaskedImage(
        const IGImage& image,
        const IGImage& mask,
        const IGrafState& grafState);

    virtual void renderImage(
        const IGImage& image,
        const IGRect2D& source,
        const IGrafState& grafState) = 0;

    virtual void renderMaskedImage(
        const IGImage& image,
        const IGImage& mask,
        const IGRect2D& source,
        const IGrafState& grafState) = 0;

    virtual IGPoint2D renderTextRun(const IGTextRun& geometry, const IGrafState& grafState) = 0;

    // === Physical Device Methods ===

    virtual IGRect2D worldBounds() const;
    virtual IGArea worldArea() const;

    // resolution (for device-dependent text & rendering)
    // This class returns ideal resolution; subclasses can override
    // for their specific resolution.
    virtual const IGrafMatrix& deviceResolution(const IGRect2D& areaOfInterest = IGRect2D::infiniteRect()) const;

    virtual void flush();

    virtual ICoordinateSystem::EOrientation coordinateSystem() const;

    virtual const IPresSpaceHandle& deviceContext() const = 0;
    virtual void adoptDeviceContext(const IPresSpaceHandle& deviceContext) = 0;
    virtual IPresSpaceHandle orphanDeviceContext() = 0;

#if !IC_RTTI_ENABLED
    virtual const IString* deviceType() const = 0;
#endif // IC_RTTI_ENABLED

    bool isValid() const;

protected:  //=== Start of public to protected scope change ===
    // An empty grafdevice has a invalid flag, and infinite bounds
    IGrafDevice();
    IGrafDevice(const IGRect2D& worldbounds);
    IGrafDevice(const IGrafDevice&);

    void setValidity(bool valid);

friend class IBaseRootGrafPort;
    void setWorldBounds(const IGRect2D& newBounds);

friend class IPaint;
friend class IMaskPattern;
friend class IMaskPatternStatics;
friend class IGraph2DTestHelper;
    enum EMaskPattern {
        kSolidPattern,
        kDiagonalDownPattern,
        kCrossPattern,
        kDiagonalCrossPattern,
        kDiagonalUpPattern,
        kHorizontalPattern,
        kVerticalPattern,
        kCustomPattern
    };
    static EMaskPattern maskPattern(const IMaskPattern* pattern);
    static const unsigned char* maskBits(const IMaskPattern* pattern);

    static void extract(
        const IGArea& area,
        IGrafExtractor& extractor,
        ICAGNode* parentNode,
        const IGrafMatrix& coordSys = IGrafMatrix::identity());

    static IConicInfo* conicInfo(IGCurve2D& curve);
    static void deleteConicInfo(IGCurve2D& curve);

private:
    bool fValid;               // indicate validity
    IGRect2D fWorldBounds;     // device bounds in world space

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _GRAFDEVS_
