/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 46 1.14.2.4 source/albert/graph2d/grafextr.hpp, 2d, ioc.v400, 000919 
/*================
||
||  File:   GrafExtractor.h
||
||  What:   Albert Graphics Extractor
||
||  Classes declared in this file:
||
||  *IGrafExtractor
||
||  Change History:
||
*/

#ifndef _GRAFEXTR_
#define _GRAFEXTR_

#ifndef IC_TRANSFORMS
#include <imatrix.hpp>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ICAGNode;
class ICAGUnary;
class ICAGRoot;
class IGLine2D;
class IGPolyline2D;
class IGCurve2D;
class IGRect2D;
class IGEllipse2D;
class IGPolygon2D;
class IGLoop2D;
#ifdef NO_OC_APPLICABLE
class IGGlyphRun;
#endif // NO_OC_APPLICABLE


/*================
||
||  IGrafExtractor
||
||  IGrafExtractor is the means by which geometry classes express their contents.
||  Their contents are expressed in terms of the basic set of geometries (see the
||  ExtractXXX calls below).
||
*/

#pragma enum(4)
#pragma pack(push,4)

class IGrafExtractor {
    public:
        virtual ~IGrafExtractor ();

        // === Path-defining geometries ===


#ifdef NO_OC_APPLICABLE
        virtual void    extractLine (
                            const IGLine2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;
        virtual void    extractPolyline (
                            const IGPolyline2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;
        virtual void    extractCurve (
                            const IGCurve2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;
#endif // NO_OC_APPLICABLE

        // === Area-defining geometries ===
        virtual void    extractRect (
                            const IGRect2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;
        virtual void    extractEllipse (
                            const IGEllipse2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;
        virtual void    extractPolygon (
                            const IGPolygon2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;

        virtual void    extractLoop (
                            const IGLoop2D&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;

#ifdef NO_OC_APPLICABLE
        virtual void    extractGlyphRun (
                            const IGGlyphRun&,
                            ICAGUnary*,
                            const IGrafMatrix& cs = IGrafMatrix::identity()) = 0;

        // === render ===
        virtual void    render (ICAGRoot&) = 0;
#endif // NO_OC_APPLICABLE


        // === cAG node adoption function ===
        void    adoptCAGNode (ICAGNode* node);  // adopt a cAG node

#ifdef IC_MOTIF
    private:
			// === render ===
        virtual void    render (ICAGRoot&) = 0;
#endif //IC_MOTIF

    protected:
        IGrafExtractor ();
        IGrafExtractor (const IGrafExtractor&);

        void deleteCAGNodes ();

        IGrafExtractor& operator= (const IGrafExtractor&);

    private:
        ICAGNode**  fCAGNodes;                  // list of cAG nodes
        long        fNumCAGNodes;               // number of nodes in the list
        long        fNumAllocatedCAGNodes;      // number of nodes allocated
        static const long   kCAGNodeIncrement;  // resize list by this amount
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _GRAFEXTR_




