// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 33 1.9.1.4 source/albert/graph2d/graftext/istdcrnd.hpp, 2d, ioc.v400, 000919 
#ifndef _ISTDCRND_
#define _ISTDCRND_

#include <igbase2d.hpp>
#include <itxtstyl.hpp>
#include <istylset.hpp>
#include <itxtiter.hpp>
#include <igrtypes.hpp>
#include <igrport.hpp>
#include <itxtplmt.hpp>
#include <icrend.hpp>

#include <igraphpl.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGTextRun;

#pragma pack(push,4)
#pragma enum(4)

class ICharacterRendererCache : public ICharacterRenderer {
public:
							ICharacterRendererCache();
							ICharacterRendererCache(const ICharacterRendererCache& source);
	virtual					~ICharacterRendererCache();

	virtual ICharacterRenderer* clone() const;
	virtual void			writeToStream(IDataStream& toWhere) const;
	virtual void			readFromStream(IDataStream& fromWhere);

	virtual void			setStyle(const ITextStyleSet* s);

	virtual GCoordinate		getAscent(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const;

	virtual GCoordinate		getDescent(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const;

	virtual GCoordinate		getLeading(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const;

	virtual GCoordinate		handleDrawChars(
								IGrafPort& g,
								const IGPoint2D& baseLine,
								ITextIterator textStart,
								const ITextIterator& textLimit) const;

	virtual GCoordinate		handleAdvance(
								IGrafPort& g,
								ITextIterator textStart,
								const ITextIterator& textLimit,
								GCoordinate tabAdvance) const;

	virtual GCoordinate		handleCharWidth(
								IGrafPort& g,
								const ITextIterator& iter) const;

	virtual ITextPlacement	handlePixelToIndex(
								IGrafPort& g,
								const IGPoint2D& pixel,
								ITextIterator textStart,
								const ITextIterator& textLimit) const;

	virtual IGPoint2D		handleIndexToPixel(
								IGrafPort& g,
								ITextIterator textStart,
								const ITextIterator& textLimit,
								const ITextPlacement& placementToSet) const;

	virtual bool 			handleIsWhiteSpace(
								const ITextIterator& charPos) const;

	virtual bool 			handleIsDecimal(
								const ITextIterator& charPos) const;

private:
	bool				fIsOwned;
	ICharacterRenderer*	fRenderer;
};

//................................................................................

class IStandardCharacterRenderer : public ICharacterRenderer {
public:
			IStandardCharacterRenderer();
			IStandardCharacterRenderer(const IStandardCharacterRenderer& source);
	virtual	~IStandardCharacterRenderer();
			IStandardCharacterRenderer& operator=(const IStandardCharacterRenderer& source);


	virtual ICharacterRenderer* clone() const;
	virtual void			writeToStream(IDataStream& toWhere) const;
	virtual void			readFromStream(IDataStream& fromWhere);

	virtual void			setStyle(const ITextStyleSet* s);

	virtual GCoordinate		getAscent(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const;

	virtual GCoordinate		getDescent(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const;

	virtual GCoordinate		getLeading(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const;

	virtual GCoordinate		handleDrawChars(
								IGrafPort& g,
								const IGPoint2D& baseLine,
								ITextIterator textStart,
								const ITextIterator& textLimit) const;

	virtual GCoordinate		handleAdvance(
								IGrafPort& g,
								ITextIterator textStart,
								const ITextIterator& textLimit,
								GCoordinate tabAdvance) const;

	virtual GCoordinate		handleCharWidth(
								IGrafPort& g,
								const ITextIterator& iter) const;

	virtual ITextPlacement	handlePixelToIndex(
								IGrafPort& g,
								const IGPoint2D& pixel,
								ITextIterator textStart,
								const ITextIterator& textLimit) const;

	virtual IGPoint2D		handleIndexToPixel(
								IGrafPort& g,
								ITextIterator textStart,
								const ITextIterator& textLimit,
								const ITextPlacement& placementToSet) const;

	virtual bool 			handleIsWhiteSpace(
								const ITextIterator& charPos) const;

	virtual bool 			handleIsDecimal(
								const ITextIterator& charPos) const;

private:
	IGTextRun*				fCache;
	const ITextStyleSet*	fStyle;

	GCoordinate				fBaseLineShift;

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTDCRND_
