// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 35 1.9.1.3 source/albert/graph2d/graftext/istdprnd.hpp, 2d, ioc.v400, 000919 
#ifndef _ISTDPRND_
#define _ISTDPRND_

#include <igbase2d.hpp>
#include <itxtstyl.hpp>
#include <itxtiter.hpp>
#include <igrtypes.hpp>
#include <igrport.hpp>
#include <igrcolor.hpp>
#include <itxtplmt.hpp>
#include <iprend.hpp>
#include <istyles.hpp>
#include <istylset.hpp>
#include <imtabrl.hpp>
#include <istyles.hpp>
#include <istdcrnd.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

//................................................................................
class IParagraphRendererCache : public IParagraphRenderer {
public:
							IParagraphRendererCache();
							IParagraphRendererCache(const IParagraphRendererCache& source);
							~IParagraphRendererCache();

	virtual IParagraphRenderer* clone() const;

	virtual void			writeToStream(IDataStream& toWhere) const;
	virtual void			readFromStream(IDataStream& fromWhere);

	virtual void			layout(
								IGrafPort& g,
								ILayoutInfo& infoOut,
								const GCoordinate& lineDimension,
								ITextIterator lineStart,
								const ITextIterator& paragraphStart,
								const ITextIterator& limit,
								GCoord averageCharWidth) const;

	virtual void			renderWithHighlight(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& baseLine,
								const ITextIterator& selStart,
								const ITextIterator& selEnd,
								const IColor& hilight) const;

	virtual void			render(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& baseLine) const;

	virtual ITextPlacement	pixelToIndex(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& where) const;

	virtual IGPoint2D		indexToPixel(
								IGrafPort& g,
								const ILayoutInfo& info,
								const ITextPlacement& index) const;

	virtual void			renderCaret(
								const ILayoutInfo& info,
								const ITextPlacement& index) const;

	virtual void			setStyle(const ITextStyleSet* style);

private:
	bool				fIsOwned;
	IParagraphRenderer*	fRenderer;
};

//................................................................................
class IStandardParagraphRenderer : public IParagraphRenderer {
public:
	static void				TextBox(const IText& text,
								IGrafPort& g,
								const IGRect2D& drawRect);

	static void				TextBox(const IText& text,
								IGrafPort& g,
								const IGRect2D& drawRect,
								bool fillInc,
								bool hLine);


							IStandardParagraphRenderer();
							IStandardParagraphRenderer(const ITextStyleSet* style);
							IStandardParagraphRenderer(const IStandardParagraphRenderer& source);
	virtual					~IStandardParagraphRenderer();

	virtual void			setStyle(const ITextStyleSet* style);

	virtual IParagraphRenderer* clone() const;
	virtual void			writeToStream(IDataStream& toWhere) const;
	virtual void			readFromStream(IDataStream& fromWhere);

	virtual void			layout(
								IGrafPort& g,
								ILayoutInfo& infoOut,
								const GCoordinate& lineDimension,
								ITextIterator lineStart,
								const ITextIterator& paragraphStart,
								const ITextIterator& limit,
								GCoord averageCharWidth) const;

    virtual void			setNextTab(ITabStop& tab, GCoordinate position) const;

	virtual void			renderWithHighlight(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& baseLine,
								const ITextIterator& selStart,
								const ITextIterator& selEnd,
								const IColor& hilight) const;

	virtual void			render(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& baseLine) const;

	virtual ITextPlacement	pixelToIndex(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& where) const;

	virtual IGPoint2D		indexToPixel(
								IGrafPort& g,
								const ILayoutInfo& info,
								const ITextPlacement& index) const;

	virtual void			renderCaret(
								const ILayoutInfo& info,
								const ITextPlacement& index) const;

private:
	void					initRenderer(const ITextStyleSet* style);
    GCoordinate				renderSegment(
    							IGrafPort& g,
    							ITextIterator start,
    							const ITextIterator& limit,
    							const IGPoint2D& p) const;

    GCoordinate				fLeadingMargin;
    GCoordinate				fTrailingMargin;
    GCoordinate				fFirstLineIndent;
    GCoordinate				fMinFirstLineSpacing; // first line
    GCoordinate				fExtraFirstLineSpacing; // first line
    GCoordinate				fMinLineSpacing; // subsequent lines
    GCoordinate				fExtraLineSpacing; // subsequent lines
    IParagraphJustificationStyle::EJustification  fFlush;
    IMTabRuler*				fTabRuler;

	const ITextStyleSet*	fCacheStyle;
    ICharacterRendererCache	fRenderer;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTDPRND_
