/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 25 1.19.1.14 source/albert/graph2d/ibcolor.cpp, 2d, ioc.v400, 000919 
/*==========================================
||
||  File:   Color.C
||
||  What:   Color used to define IColor abstract class
||
*/

#ifdef IC_PM
#ifdef INCL_NOPMAPI
#undef INCL_NOPMAPI
#endif //INCL_NOPMAPI
#ifndef INCL_PM
#define INCL_PM
#define INCL_WIN
#endif //INCL_PM
#define INCL_DOSPROCESS
#define INCL_GPI
#include <os2.h>
//#include <pm.h>
#endif //IC_PM

#include <ibcolor.hpp>
#include <colprof.hpp>
#include <grtype.hpp>
#include <idatstrm.hpp>
#include <grstrmod.hpp>
#include <igrexept.hpp>
#include <icolmap.hpp>
#include <itrace.hpp>
#include <ixdc.hpp>
#include <iprimlck.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _IBCOLOR_CPP_
#include <ipagetun.h>
#endif

StreamableDefinitionsMacro(IBaseColor, gGraphicsStreamModule);

#if (IC_OBSOLETE <= IC_OBSOLETE_3)
IBasicColor::~IBasicColor() {}
#endif // IC_OBSOLETE

//----------------------------------------------------------------

IBaseColor::IBaseColor() :
	fRed(0),
	fGreen(0),
	fBlue(0),
	fOpacity(255)
{}

IBaseColor::IBaseColor(const IBaseColor& aColor) :
	fRed(aColor.fRed),
	fGreen(aColor.fGreen),
	fBlue(aColor.fBlue),
	fOpacity(aColor.fOpacity)
{}

IBaseColor::IBaseColor(CharIntensity gray, CharIntensity opacity) :
	fRed(gray),
	fGreen(gray),
	fBlue(gray),
	fOpacity(opacity)
{}

IBaseColor::IBaseColor(CharIntensity red, CharIntensity green, CharIntensity blue, CharIntensity opacity) :
	fRed(red),
	fGreen(green),
	fBlue(blue),
	fOpacity(opacity)
{}

typedef struct _predefclrs{
unsigned char R;
unsigned char G;
unsigned char B;
} PreDefClrs;

PreDefClrs preDefClrs[16]={
	{255 , 255 , 255},	//White
	{0 , 0 , 255},		//Blue
	{255 , 0 , 0},		//Red
	{255 , 0 , 255},	//Pink
	{0 , 255 , 0},		//Green
	{0 , 255 , 255},	//Cyan
	{255 , 255 , 0},	//Yellow
	{0 , 0 , 0},		//Black
#ifdef IC_WIN
	{128 , 128 , 128},	//Dark Gray
	{0 , 0 , 128},		//Dark Blue
	{128 , 0 , 0},		//Dark Red
	{128 , 0 , 128},	//Dark Pink
	{0 , 128, 0},		//Dark Green
	{0 , 128, 128},		//Dark Cyan
	{128 , 128 , 0},	//Dark Brown
	{192 , 192, 192}	//Pale Gray
#else
	{131 , 131 , 131},	//Dark Gray
	{0 , 0 , 170},		//Dark Blue
	{128 , 0 , 0},		//Dark Red
	{170 , 0 , 170},	//Dark Pink
	{0 , 146, 0},		//Dark Green
	{0 , 146, 170},		//Dark Cyan
	{128 , 128 , 0},	//Dark Brown
	{204 , 204, 204}	//Pale Gray
#endif
};

IBaseColor::IBaseColor(EPredefinedColor commonlyUsedColor) :
        fOpacity(255)
{
	setRGB(preDefClrs[commonlyUsedColor].R,
		preDefClrs[commonlyUsedColor].G,
		preDefClrs[commonlyUsedColor].B);
}

IBaseColor::~IBaseColor()
{
}

//----------------------------------------------------------------
IBaseColor& IBaseColor::operator=(const IBaseColor& source)
{
        if (this != &source)
                copyRGBA(source);
        return *this;
}

void IBaseColor::redMix(GIntensity& aRed) const
{
        aRed = GIntensity(((GIntensity)fRed + 0.5) / 255.0);
}

void IBaseColor::greenMix(GIntensity& aGreen) const
{
        aGreen = GIntensity(((GIntensity)fGreen + 0.5) / 255.0);
}

void IBaseColor::blueMix(GIntensity& aBlue) const
{
        aBlue = GIntensity(((GIntensity)fBlue + 0.5) / 255.0);
}

void IBaseColor::opacity(GIntensity& opacity) const
{
        opacity = GIntensity(((GIntensity)fOpacity + 0.5) / 255.0);
}

// === Boilerplate functions ===

void IBaseColor::writeToStream( IDataStream& toWhere ) const
{
        IStreamOutFrame streamFrame(toWhere);
        fRed >>= toWhere;
        fGreen >>= toWhere;
        fBlue >>= toWhere;
        fOpacity >>= toWhere;
}

void IBaseColor::readFromStream( IDataStream& fromWhere )
{
        IStreamInFrame streamFrame(fromWhere);
        fRed <<= fromWhere;
        fGreen <<= fromWhere;
        fBlue <<= fromWhere;
        fOpacity <<= fromWhere;
}

// index color convertion operator
/*
IBaseColor::operator IColor()
{
        if (GraphicTypeid(*this) == GraphicTypeid(IColor))
                return IColor(*((IColor*)this));
        else
                return IColor(fRed, fGreen, fBlue, fOpacity);
}
*/

//----------------------------------------------------------------
const IColorProfile* IBaseColor::fgColorProfile = 0;

const IColorProfile* IBaseColor::colorProfile() const
{
    if (!fgColorProfile) {
        IPrimalLock lock;
        if (!fgColorProfile) {
            fgColorProfile = new IColorProfile;
        }
    }
    return fgColorProfile;
}

void IBaseColor::elementsInXYZColorSpace(
        GIntensity& x,
        GIntensity& y,
        GIntensity& z) const
{
        GIntensity r, g, b;
        redMix(r);
        greenMix(g);
        blueMix(b);
        colorProfile()->convertRGBtoXYZ(r, g, b, x, y, z);
}


void IBaseColor::setElementsInXYZColorSpace(
        GIntensity x,
        GIntensity y,
        GIntensity z)
{
        GIntensity r, g, b;
        colorProfile()->convertXYZtoRGB(x, y, z, r, g, b);
        setRed(CharIntensity(r));
        setGreen(CharIntensity(g));
        setBlue(CharIntensity(b));
}


long IBaseColor::createIndex(CharIntensity r, CharIntensity g, CharIntensity b)
{
long index;
#if defined(IC_WIN)
  index = (long) ((r & 0xFFul) | ((g & 0xFFul)<<8) | ((b & 0xFFul) << 16));
#elif defined(IC_PM)
  index = (long) ((b & 0xFFul) | ((g & 0xFFul)<<8) | ((r & 0xFFul) << 16));
#elif defined(IC_MOTIF)
  index = IXDisplay::rgbToPixel(r, g, b);
#endif
  return index;
}

void IBaseColor::rgbFromIndex(unsigned long index)
{
#if defined(IC_WIN)
  this->fRed = (CharIntensity)(index & 0x000000FF);
  this->fGreen = (CharIntensity)((index & 0x000FF00) >> 8);
  this->fBlue = (CharIntensity)((index & 0x00FF0000) >> 16);
#elif defined(IC_PM)
  this->fRed = (CharIntensity)(index & 0x000000FF);
  this->fGreen = (CharIntensity)((index & 0x0000FF00) >> 8);
  this->fBlue = (CharIntensity)((index & 0x00FF0000) >> 16);
#elif defined(IC_MOTIF)
	IXDisplay::pixelToRGB(index, &(this->fRed), &(this->fGreen), &(this->fBlue));
#endif
}

//----------------------------------------------------------------
// IColor
//----------------------------------------------------------------


//----------------------------------------------------------------
// for system color implementation
//

#include <limits.h>     // for LONGMIN & LONGMAX


#ifdef IC_MOTIF
#include <X11/Intrinsic.h>
#include <ixcolrns.hpp>
#include <isrtset.h>
#endif

#ifdef IC_WIN
#include <windows.h>
#include <iplatfrm.hpp>
/**********************************************************************/
/* Following are mappings for system color definitions in Windows NT  */
/**********************************************************************/
#define SYSCLR_MENUDISABLEDTEXT        COLOR_GRAYTEXT
#define SYSCLR_BUTTONDARK              COLOR_BTNSHADOW
#define SYSCLR_SHADOW                  COLOR_BTNHIGHLIGHT   // was COLOR_SHADOW
#define SYSCLR_HILITEFOREGROUND        COLOR_HIGHLIGHTTEXT
#define SYSCLR_HILITEBACKGROUND        COLOR_HIGHLIGHT
#define SYSCLR_INACTIVETITLETEXT       COLOR_INACTIVECAPTIONTEXT
#define SYSCLR_ACTIVETITLETEXT         COLOR_CAPTIONTEXT
#define SYSCLR_SCROLLBAR               COLOR_SCROLLBAR
#define SYSCLR_BACKGROUND              COLOR_BACKGROUND
#define SYSCLR_ACTIVETITLE             COLOR_ACTIVECAPTION
#define SYSCLR_INACTIVETITLE           COLOR_INACTIVECAPTION
#define SYSCLR_MENU                    COLOR_MENU
#define SYSCLR_WINDOW                  COLOR_WINDOW
#define SYSCLR_WINDOWFRAME             COLOR_WINDOWFRAME
#define SYSCLR_MENUTEXT                COLOR_MENUTEXT
#define SYSCLR_WINDOWTEXT              COLOR_WINDOWTEXT
#define SYSCLR_TITLETEXT               COLOR_CAPTIONTEXT
#define SYSCLR_ACTIVEBORDER            COLOR_ACTIVEBORDER
#define SYSCLR_INACTIVEBORDER          COLOR_INACTIVEBORDER
#define SYSCLR_APPWORKSPACE            COLOR_APPWORKSPACE
// Following is per 4.0 SDK, "System Bitmaps and Colors".  Note that
// COLOR_3DFACE is also the same as COLOR_BTNFACE in this SDK version.
#define SYSCLR_DIALOGBACKGROUND        COLOR_3DFACE

// The following color areas are not separate areas in Windows, or
// are not directly changable.  These are the mappings which in most
// cases result in similar behavior ... particularly if default colors
// set via properties/control panel are used.
#define SYSCLR_SHADOWHILITEBGND        COLOR_HIGHLIGHT
#define SYSCLR_SHADOWHILITEFGND        COLOR_HIGHLIGHTTEXT
#define SYSCLR_SHADOWTEXT              COLOR_WINDOWTEXT
#define SYSCLR_ENTRYFIELD              COLOR_WINDOW
#define SYSCLR_MENUHILITE              COLOR_HIGHLIGHTTEXT
#define SYSCLR_MENUHILITEBGND          COLOR_HIGHLIGHT
#define SYSCLR_PAGEBACKGROUND          SYSCLR_DIALOGBACKGROUND
#define SYSCLR_FIELDBACKGROUND         COLOR_INACTIVECAPTION
#define SYSCLR_BUTTONLIGHT             COLOR_BTNHIGHLIGHT
#define SYSCLR_BUTTONMIDDLE \
	( (IPlatform::isWin9x() || IPlatform::isNTNewShell()) ? \
           COLOR_3DFACE : COLOR_BTNFACE )

//no mapping  SYSCLR_BUTTONDEFAULT ... appears to be black or xor
#define SYSCLR_TITLEBOTTOM             COLOR_MENU
#define SYSCLR_ICONTEXT                COLOR_WINDOWTEXT
#define SYSCLR_INACTIVETITLETEXTBGND   COLOR_INACTIVECAPTION
#define SYSCLR_ACTIVETITLETEXTBGND     COLOR_ACTIVECAPTION
#define SYSCLR_OUTPUTTEXT              COLOR_WINDOWTEXT
#define SYSCLR_WINDOWSTATICTEXT        COLOR_WINDOWTEXT
// The help values could map to COLOR_INFOBK/COLOR_INFOTEXT, which apply
// to tooltip controls but not normal help
#define SYSCLR_HELPBACKGROUND          COLOR_WINDOW
#define SYSCLR_HELPTEXT                COLOR_WINDOWTEXT
#define SYSCLR_HELPHILITE              COLOR_WINDOWTEXT

#endif //IC_WIN


//static IBaseColor nativeSystemColor(IColor::ESystemColor systemColor)
IBaseColor IColor::nativeSystemColor(IColor::ESystemColor systemColor)
{
        long lClSysColor;

#ifndef IC_MOTIF

        switch (systemColor) {
        case IColor::kShadowIconHiliteBgnd:
                lClSysColor = SYSCLR_SHADOWHILITEBGND;
                break;
        case IColor::kShadowIconHiliteFgnd:
                lClSysColor = SYSCLR_SHADOWHILITEFGND;
                break;
        case IColor::kShadowIconText:
                lClSysColor = SYSCLR_SHADOWTEXT;
                break;
        case IColor::kEntryFieldBgnd:
        case IColor::kListBoxBgnd:
                lClSysColor = SYSCLR_ENTRYFIELD;
                break;
        case IColor::kDisableMenuText:
                lClSysColor = SYSCLR_MENUDISABLEDTEXT;
                break;
        case IColor::kMenuHiliteText:
                lClSysColor = SYSCLR_MENUHILITE;
                break;
        case IColor::kMenuHiliteBgnd:
                lClSysColor = SYSCLR_MENUHILITEBGND;
                break;
        case IColor::kNotebookPageBgnd:
                lClSysColor = SYSCLR_PAGEBACKGROUND;
                break;
        case IColor::kInactiveScrollBar:
        case IColor::kDefaultControl:
                lClSysColor = SYSCLR_FIELDBACKGROUND;
                break;
        case IColor::kButtonLight:
                lClSysColor = SYSCLR_BUTTONLIGHT;
                break;
        case IColor::kButtonMiddle:
                lClSysColor = SYSCLR_BUTTONMIDDLE;
                break;
        case IColor::kButtonDark:
                lClSysColor = SYSCLR_BUTTONDARK;
                break;
        case IColor::kDefaultButton:
#ifdef IC_PM
                lClSysColor = SYSCLR_BUTTONDEFAULT;
#else
                // No real mapping
                lClSysColor = SYSCLR_WINDOWTEXT;
#endif
                break;
        case IColor::kTitleLine:
                lClSysColor = SYSCLR_TITLEBOTTOM;
                break;
        case IColor::kMenuShadow:
        case IColor::kDialogShadow:
                lClSysColor = SYSCLR_SHADOW;
                break;
        case IColor::kIconText:
                lClSysColor = SYSCLR_ICONTEXT;
                break;
        case IColor::kDialogBgnd:
                lClSysColor = SYSCLR_DIALOGBACKGROUND;
                break;
        case IColor::kHiliteFgnd:
                lClSysColor = SYSCLR_HILITEFOREGROUND;
                break;
        case IColor::kHiliteBgnd:
                lClSysColor = SYSCLR_HILITEBACKGROUND;
                break;
        case IColor::kInactiveTitleTextBgnd:
                lClSysColor = SYSCLR_INACTIVETITLETEXTBGND;
                break;
        case IColor::kActiveTitleTextBgnd:
                lClSysColor = SYSCLR_ACTIVETITLETEXTBGND;
                break;
        case IColor::kInactiveTitleText:
                lClSysColor = SYSCLR_INACTIVETITLETEXT;
                break;
        case IColor::kActiveTitleText:
                lClSysColor = SYSCLR_ACTIVETITLETEXT;
                break;
        case IColor::kOutputText:
                lClSysColor = SYSCLR_OUTPUTTEXT;
                break;
        case IColor::kWindowStaticText:
                lClSysColor = SYSCLR_WINDOWSTATICTEXT;
                break;
        case IColor::kScrollBar:
                lClSysColor = SYSCLR_SCROLLBAR;
                break;
        case IColor::kDesktopBgnd:
                lClSysColor = SYSCLR_BACKGROUND;
                break;
        case IColor::kActiveTitleBgnd:
                lClSysColor = SYSCLR_ACTIVETITLE;
                break;
        case IColor::kInactiveTitleBgnd:
                lClSysColor = SYSCLR_INACTIVETITLE;
                break;
        case IColor::kMenuBgnd:
                lClSysColor = SYSCLR_MENU;
                break;
        case IColor::kWindowBgnd:
                lClSysColor = SYSCLR_WINDOW;
                break;
        case IColor::kFrameBorder:
                lClSysColor = SYSCLR_WINDOWFRAME;
                break;
        case IColor::kMenuText:
                lClSysColor = SYSCLR_MENUTEXT;
                break;
        case IColor::kWindowText:
                lClSysColor = SYSCLR_WINDOWTEXT;
                break;
        case IColor::kTitleText:
        case IColor::kSizeBar:
        case IColor::kScrollArrow:
                lClSysColor = SYSCLR_TITLETEXT;
                break;
        case IColor::kActiveFrameBorder:
                lClSysColor = SYSCLR_ACTIVEBORDER;
                break;
        case IColor::kInactiveFrameBorder:
                lClSysColor = SYSCLR_INACTIVEBORDER;
                break;
        case IColor::kMainWindowBgnd:
                lClSysColor = SYSCLR_APPWORKSPACE;
                break;
        case IColor::kHelpWindowBgnd:
                lClSysColor = SYSCLR_HELPBACKGROUND;
                break;
        case IColor::kHelpText:
                lClSysColor = SYSCLR_HELPTEXT;
                break;
        case IColor::kHelpHiliteText:
                lClSysColor = SYSCLR_HELPHILITE;
                break;
        default:
                throw
                        IGraphicException(
                                "Enum parametre cannot evaluated to be a system color",
                                IGraphicException::kInvalidValue);
        } /* endswitch */

#else //IC_MOTIF


        switch (systemColor) {
        case IColor::kShadowIconHiliteBgnd:
                return IBaseColor(128, 128, 128);
        case IColor::kShadowIconHiliteFgnd:
                return IBaseColor(255, 255 , 255);
        case IColor::kShadowIconText:
                return IBaseColor(128, 128, 128);
        case IColor::kEntryFieldBgnd:
        case IColor::kListBoxBgnd:
                return IBaseColor(255, 255, 204);
        case IColor::kDisableMenuText:
                return IBaseColor(128, 128, 128);
        case IColor::kMenuHiliteText:
                return IBaseColor(0, 0, 0);
        case IColor::kMenuHiliteBgnd:
                return IBaseColor(204, 204, 204);
        case IColor::kNotebookPageBgnd:
                return IBaseColor(255, 255, 255);
        case IColor::kInactiveScrollBar:
        case IColor::kDefaultControl:
                return IBaseColor(204, 204, 204);
        case IColor::kButtonLight:
                return IBaseColor(255, 255, 255);
        case IColor::kButtonMiddle:
                return IBaseColor(204, 204, 204);
	case IColor::kButtonDark:
                return IBaseColor(128, 128, 128);
        case IColor::kDefaultButton:
                return IBaseColor(0, 0, 0);
        case IColor::kTitleLine:
                return IBaseColor(128, 128, 128);
        case IColor::kMenuShadow:
        case IColor::kDialogShadow:
                return IBaseColor(128, 128, 128);
        case IColor::kIconText:
                return IBaseColor(0, 0, 0);
        case IColor::kDialogBgnd:
                return IBaseColor(204, 204, 204);
        case IColor::kHiliteFgnd:
                return IBaseColor(255, 255, 255);
        case IColor::kHiliteBgnd:
                return IBaseColor(128, 128, 128);
        case IColor::kInactiveTitleTextBgnd:
                return IBaseColor(204, 204, 204);
        case IColor::kActiveTitleTextBgnd:
                return IBaseColor(204, 204, 204);
        case IColor::kInactiveTitleText:
                return IBaseColor(128, 128, 128);
        case IColor::kActiveTitleText:
                return IBaseColor(255, 255, 255);
        case IColor::kOutputText:
                return IBaseColor(0, 0, 0);
        case IColor::kWindowStaticText:
                return IBaseColor(0, 0, 128);
        case IColor::kScrollBar:
                return IBaseColor(192, 192, 192);
        case IColor::kDesktopBgnd:
                return IBaseColor(0, 128, 128);
        case IColor::kActiveTitleBgnd:
                return IBaseColor(64, 128, 204);
        case IColor::kInactiveTitleBgnd:
                return IBaseColor(204, 204, 204);
        case IColor::kMenuBgnd:
                return IBaseColor(204, 204, 204);
        case IColor::kWindowBgnd:
                return IBaseColor(255, 255, 255);
        case IColor::kFrameBorder:
                return IBaseColor(128, 128, 128);
        case IColor::kMenuText:
                return IBaseColor(0, 0, 0);
        case IColor::kWindowText:
                return IBaseColor(0, 0, 0);
        case IColor::kTitleText:
        case IColor::kSizeBar:
        case IColor::kScrollArrow:
                return IBaseColor(255, 255, 255);
        case IColor::kActiveFrameBorder:
                return IBaseColor(255, 255, 128);
        case IColor::kInactiveFrameBorder:
                return IBaseColor(204, 204, 204);
        case IColor::kMainWindowBgnd:
                return IBaseColor(255, 255, 224);
        case IColor::kHelpWindowBgnd:
                return IBaseColor(255, 255, 255);
        case IColor::kHelpText:
                return IBaseColor(0, 0, 128);
        case IColor::kHelpHiliteText:
                return IBaseColor(0, 128, 128);
        default:
                throw
                        IGraphicException(
                                "Enum parametre cannot evaluated to be a system color",
                                IGraphicException::kInvalidValue);
        } /* endswitch */

#endif

        // find the system color index
#if defined(IC_PM)
#define IQUERYSYSCOLOR(idx) (WinQuerySysColor(HWND_DESKTOP, idx, 0L))
      long lRGBColor = IQUERYSYSCOLOR(lClSysColor);
      RGB2* prgb2 = (RGB2*)&lRGBColor;
      return IBaseColor(prgb2->bRed, prgb2->bGreen, prgb2->bBlue);
#elif defined(IC_WIN)
#define IQUERYSYSCOLOR(idx) (GetSysColor(idx))
      long lRGBColor = IQUERYSYSCOLOR((int)lClSysColor);
      return IBaseColor(GetRValue(lRGBColor), GetGValue(lRGBColor), GetBValue(lRGBColor));
#endif //IC_MOTIF
}

IBaseColor& IColor::evaluate()
{
        if (fType != kSystem)
                return *this;
        else
                return copyRGBA(nativeSystemColor((IColor::ESystemColor)fValue));
}

IBaseColor& IColor::validate() {

        if (fType == kIndex)
                fValue = LONG_MIN;      // need to re-evaluate...
        else if (fType == kSystem || fType == kPredefined)
                fType = kRGB;           // type change...

	if (IColorMap::hasColorMapSupport())
		fValue  = IColorMap::defaultColorMap().colorIndex(*this);
	else // no colormap support
		fValue = createIndex(fRed, fGreen, fBlue);

        fValueCache = true;
        return *this;
}


IColor::IColor(ESystemColor runtimeGuiColor) :
        IBaseColor(),
        fType(kSystem),
        fValue((long)runtimeGuiColor),
        fValueCache(false)
{
        evaluate();
}

IColor::IColor(EPredefinedColor color) :
        IBaseColor(color),
        fType(kPredefined),
        fValue((long)color),
        fValueCache(false)
{
        evaluate();
}

IColor::IColor(
        CharIntensity red,
        CharIntensity green,
        CharIntensity blue,
        CharIntensity opacity) :

        IBaseColor(red, green, blue, opacity),
        fType(kRGB),
        fValue(0),
        fValueCache(false)
{}

IColor::IColor() :
        IBaseColor(),
        fType(kRGB),
        fValue(0),
        fValueCache(false)
{}


IColor::IColor(unsigned long index, const IColorMap* colorMap) :
        fType(kIndex),
        fValue(index)
{
	if(IColorMap::hasColorMapSupport()){
        	const IColorMap* tColMap = ((colorMap) ? colorMap : &(IColorMap::defaultColorMap()));
		if(fValue >= tColMap->numberOfColors())
			rgbFromIndex((unsigned long)index);
		else
			IBaseColor::copyRGBA((*tColMap)[fValue]);
	} else {
		//no colormap support
		rgbFromIndex( (unsigned long)index);
	}
}

IColor::IColor(const IColor& src) :
        IBaseColor(src),
        fType(src.fType),
        fValue(src.fValue),
        fValueCache(src.fValueCache)
{
        evaluate();
}

IColor::~IColor()
{}

IColor& IColor::operator=(const IColor& source)
{
        if (this != &source)
        {
                fType = source.fType;
                fValue = source.fValue;
                fValueCache = source.fValueCache;
                if (fType == kSystem)
                        IBaseColor::copyRGBA(nativeSystemColor((IColor::ESystemColor)fValue));
                else
                        IBaseColor::copyRGBA(source);
        }
        return *this;
}

bool IColor::operator==(const IBaseColor& src) const
{
	if (GraphicTypeid(src) != GraphicTypeid(IColor))
	{
		return( (fRed == src.redMix())
			&& (fGreen == src.greenMix())
			&& (fBlue == src.blueMix())
			&& (fOpacity == src.opacity()));
	}
	else
	{
		const IColor* source = (const IColor*)&src;
		return
			(fType == source->fType) &&
			(fValue == source->fValue) &&
			(fType != kSystem && (fRed == source->fRed)
					&& (fGreen == source->fGreen)
					&& (fBlue == source->fBlue)
					&& (fOpacity == source->fOpacity));
	}
}

IColor::ESystemColor IColor::runtimeGuiColor() const
{
        if (fType != kSystem)
                throw
                        IGraphicException(
                                "IColor object is not a system color",
                                IGraphicException::kInvalidValue);
        return (ESystemColor)fValue;
}

IBaseColor::EPredefinedColor IColor::predefinedColor() const
{
	if(fType == kPredefined)
		return(IBaseColor::EPredefinedColor)fValue;
	else
	{
		IBaseColor::EPredefinedColor value;
		for(int i=0; i< 16; i++){
			
			if((fRed == preDefClrs[i].R) &&
				(fGreen == preDefClrs[i].G) &&
				(fBlue == preDefClrs[i].B)){
					value = (IBaseColor::EPredefinedColor)i;
					IColor* nonconst = (IColor*)this;
					nonconst->fType = kPredefined;
					nonconst->fValue = (long)value;
					return value;
			}
		}

		throw IGraphicException(
			   "IColor object cannot evaluate to be a predefined color",
			   IGraphicException::kInvalidValue);

	}
}

long IColor::asRGBLong() const
{
#if defined(IC_WIN)
        return RGB(fRed, fGreen, fBlue);
#elif defined(IC_PM)
        return ((fRed << 16) | (fGreen << 8) | fBlue);
#elif defined(IC_MOTIF)
        return ((fRed << 16) | (fGreen << 8) | fBlue);
#endif
}

long IColor::index() const
{
        if ((fType == kIndex) && (fValue != LONG_MIN))
        {       // it might be cached...
                        return fValue;
        }
        else    // oops, invalid index ... need to re-evaluate
        {
                if (!fValueCache)
                {
			if (IColorMap::hasColorMapSupport()){
                                ((IColor*)this)->fValueCache = true;
                                ((IColor*)this)->fValue = IColorMap::defaultColorMap().colorIndex(*this);
                        } else { // no colormap support
                                ((IColor*)this)->fValue = ((IColor*)this)->createIndex(
						fRed, fGreen, fBlue);
                        }
                }
    return fValue;
  }
}

CharIntensity IColor::redMix() const
{
        if (fType != kSystem)
                ((IColor*)this)->evaluate();
	return fRed;
}

CharIntensity IColor::greenMix() const
{
        if (fType != kSystem)
                ((IColor*)this)->evaluate();
	return fGreen;
}

CharIntensity IColor::blueMix() const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	return fBlue;
}

CharIntensity IColor::opacity() const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	return fOpacity;
}

CharIntensity IColor::gray() const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	return IBaseColor::gray();
}

void IColor::redMix(GIntensity& aRed) const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	IBaseColor::redMix(aRed);
}

void IColor::greenMix(GIntensity& aGreen) const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	IBaseColor::greenMix(aGreen);
}

void IColor::blueMix(GIntensity& aBlue) const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	IBaseColor::blueMix(aBlue);
}

void IColor::opacity(GIntensity& opacity) const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
	IBaseColor::opacity(opacity);
}

IBaseColor& IColor::setRed(CharIntensity red)
{
        fRed=red;
        return validate();
}

IBaseColor& IColor::setGreen(CharIntensity green)
{
        fGreen=green;
        return validate();
}

IBaseColor& IColor::setBlue(CharIntensity blue)
{
        fBlue=blue;
        return validate();
}

IBaseColor& IColor::setOpacity(CharIntensity opacity)
{
        fOpacity=opacity;
        return validate();
}

void IColor::elementsInXYZColorSpace(
        GIntensity& x,
        GIntensity& y,
        GIntensity& z) const
{
        if (fType == kSystem)
                ((IColor*)this)->evaluate();
        IBaseColor::elementsInXYZColorSpace(x, y, z);
}

void IColor::setElementsInXYZColorSpace(GIntensity x, GIntensity y, GIntensity z)
{
        IBaseColor::setElementsInXYZColorSpace(x, y, z);
        validate();
}

void IColor::writeToStream(IDataStream& toWhere) const
{
        IStreamOutFrame streamFrame(toWhere);

        IBaseColor::writeToStream(toWhere);
        long value = (long)fType;
        value >>= toWhere;
        if (fType != kRGB)
                fValue >>= toWhere;
}

void IColor::readFromStream(IDataStream& fromWhere)
{
        IStreamInFrame streamFrame(fromWhere);

        IBaseColor::readFromStream(fromWhere);

        long value;
        value <<= fromWhere;
        fType = (IColor::EColorType)value;
        if (fType != kRGB) {
                fValue <<= fromWhere;
                if (fType == kIndex)
                        fValue = LONG_MIN;      // need to re-evaluate...
                else
                        evaluate();
        }
}

//----------------------------------------------------------------
#if (IC_OBSOLETE <= IC_OBSOLETE_3)

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
|                                                                              |
| Construct an IColor from a Color enumerator value.                           |
------------------------------------------------------------------------------*/
IColor::IColor(Color clr) :
        IBaseColor((IBaseColor::EPredefinedColor)clr),
        fType(kPredefined),
        fValue((long)clr),
        fValueCache(false)
{}

bool IColor::operator==(const Color src) const
{
	const IColor source( src );
	return ((fType == source.fType) &&
		(fValue == source.fValue) &&
		(fType != kSystem) &&
		(fRed == source.fRed) &&
		(fGreen == source.fGreen) &&
		(fBlue == source.fBlue) &&
		(fOpacity == source.fOpacity));
}

IBaseColor::IBaseColor(Color commonlyUsedColor) :
		fOpacity(255)
{
	setRGB(preDefClrs[commonlyUsedColor].R,
		preDefClrs[commonlyUsedColor].G,
		preDefClrs[commonlyUsedColor].B);
}

bool IBaseColor::operator==(const IBaseColor::Color src) const
{
	const IBaseColor source( (IBaseColor::EPredefinedColor)src );

	return(fRed == source.fRed
		   &&  fGreen == source.fGreen
		   &&  fBlue == source.fBlue
		   &&  fOpacity == source.fOpacity);
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IColor::asPixel                                                              |
|                                                                              |
| Return the colormap index for the RGB color representation.                  |
------------------------------------------------------------------------------*/
unsigned long IColor::asPixel() const
{
        return index();
}
#endif

/*------------------------------------------------------------------------------
| IColor::Color IColor::value                                                  |
|                                                                              |
| Return the Color enumerator value.                                           |
------------------------------------------------------------------------------*/
IColor::Color IColor::value() const
{
        return (Color)predefinedColor();
}

/*------------------------------------------------------------------------------
| IColor::SystemColor IColor::systemColor                                      |
|                                                                              |
| Return the SystemColor enumerator value.                                     |
------------------------------------------------------------------------------*/
IColor::SystemColor IColor::systemColor() const
{
        return (SystemColor)runtimeGuiColor();
}

#endif // IC_OBSOLETE
