// Revision: 09 1.3.4.6 source/albert/graph2d/ibcolor.inl, 2d, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*                                                                                       *
*****************************************************************************************
*/
/*==========================================
||
||  File:   ColorImp.h
||
||  What:   Color used to define interfaces for IBasicColor
||
*/

#ifndef IC_ColorImp_INL
#define IC_ColorImp_INL 0

#ifndef _IBCOLOR_
#undef  IC_ColorImp_INL
#define IC_ColorImp_INL 1
#include <ibcolor.hpp>
#endif

#if IC_ColorImp_INL
#define inline
#endif

///////////// inline functions //////////////

// IBaseColor inlines

inline IBaseColor& IBaseColor::copyRGBA(const IBaseColor& source)
{
        fRed = source.fRed;
        fGreen = source.fGreen;
        fBlue = source.fBlue;
        fOpacity = source.fOpacity;
        return *this;
}

inline bool IBaseColor::operator==(const IBaseColor& source) const
{
        return (fRed == source.fRed && fGreen == source.fGreen && fBlue == source.fBlue && fOpacity == source.fOpacity);
}

inline bool IBaseColor::operator!=(const IBaseColor& obj) const
{
        return !operator==(obj);
}

inline CharIntensity IBaseColor::redMix() const
{
        return fRed;
}

inline CharIntensity IBaseColor::greenMix() const
{
        return fGreen;
}

inline CharIntensity IBaseColor::blueMix() const
{
        return fBlue;
}

inline CharIntensity IBaseColor::opacity() const
{
        return fOpacity;
}

inline CharIntensity IBaseColor::gray() const
{
        // using formular (red * 0.30 + green * 0.59 + blue * 0.11)
        return CharIntensity(
                         (((unsigned long)fRed) * 30 +
                         ((unsigned long)fGreen) * 59 +
                         ((unsigned long)fBlue) * 11) / 100);
}

inline IBaseColor& IBaseColor::setRed(CharIntensity red)
{
        fRed = red;
        return *this;
}

inline IBaseColor& IBaseColor::setGreen(CharIntensity green)
{
        fGreen = green;
        return *this;
}

inline IBaseColor& IBaseColor::setBlue(CharIntensity blue)
{
        fBlue = blue;
        return *this;
}

inline IBaseColor& IBaseColor::setOpacity(CharIntensity opacity)
{
        fOpacity = opacity;
        return *this;
}

inline void IBaseColor::setRGB(CharIntensity r, CharIntensity g,  CharIntensity b, CharIntensity o)
{
        fRed = r;
        fGreen = g;
        fBlue = b;
        fOpacity = o;
}


#if (IC_OBSOLETE <= IC_OBSOLETE_3)
inline IBasicColor::IBasicColor() : IBaseColor() {}

inline IBasicColor::IBasicColor(IBaseColor::EPredefinedColor c) : IBaseColor(c) {}

inline IBasicColor::IBasicColor(CharIntensity g, CharIntensity o) : IBaseColor(g, o) {}

inline IBasicColor::IBasicColor(const IBasicColor& aColor) : IBaseColor(aColor) {}

inline IBasicColor::IBasicColor(
        CharIntensity r,
        CharIntensity g,
        CharIntensity b,
        CharIntensity o) :

        IBaseColor(r, g, b, o) {}

inline CharIntensity IBasicColor::red() const {
    return redMix();
}

inline CharIntensity IBasicColor::green() const {
    return greenMix();
}

inline CharIntensity IBasicColor::blue() const {
    return blueMix();
}

inline void IBasicColor::red(GIntensity& r) const {
    redMix(r);
}

inline void IBasicColor::green(GIntensity& g) const {
    greenMix(g);
}

inline void IBasicColor::blue(GIntensity& b) const {
    blueMix(b);
}

inline IBasicColor::IBasicColor(const IColor& src) :
    IBaseColor(src.redMix(), src.greenMix(), src.blueMix())
{}

inline IBasicColor::operator IColor() { return IColor(red(), green(), blue()); }

#endif // IC_OBSOLETE

#if IC_ColorImp_INL
#undef inline
#endif

#endif // IC_ColorImp_INL

