#ifndef _ICOLMAPP_
#define _ICOLMAPP_
//NOSHIP
// Revision: 22 1.2 source/albert/graph2d/icolmapp.hpp, 2d, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icolmapp.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Private data for IColorMap.                                                *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*                                                                              *
*******************************************************************************/
#include <iksset2.h>   //sorted set

#define kMAXCACHEHITCOUNT 0x7FFFFFFF
#define kINVALIDINDEX     ((unsigned long)-1)
/*------------------------------------------------------------------------------
| Class IColorMapCacheRecord - an object of this class is stored in the        |
| color cache for each color that is requested.                                |
------------------------------------------------------------------------------*/

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IColorMapCacheRecord {
public:
IColorMapCacheRecord( unsigned long rgb=0,
                      unsigned long index=0,
                      unsigned long hitCount=0)
  : fRGB     (rgb),
    fIndex   (index),
    fHitCount(hitCount)
   { };

IColorMapCacheRecord( const IColorMapCacheRecord& another)
  : fRGB     (another.fRGB     ),
    fIndex   (another.fIndex   ),
    fHitCount(another.fHitCount)
   { };

~IColorMapCacheRecord( )
   { };

IColorMapCacheRecord& operator=( const IColorMapCacheRecord& another)
{
    fRGB      = another.fRGB     ;
    fIndex    = another.fIndex   ;
    fHitCount = another.fHitCount;
    return *this;
}

unsigned long  index( )
{
   if (fHitCount < kMAXCACHEHITCOUNT) fHitCount++;
   return fIndex;
}

unsigned long  fRGB;
unsigned long  fIndex;
unsigned long  fHitCount;
}; //IColorMapCacheRecord

/*------------------------------------------------------------------------------
| key  - key function for IColorMapCacheRecord                                 |
------------------------------------------------------------------------------*/
inline unsigned long const& key (IColorMapCacheRecord const& element )
{
   return element.fRGB;
}

/*------------------------------------------------------------------------------
| Class IColorMapData -                                                        |
| This class maintains a cache of colors whose indices have been requested.    |
| The cache is keyed by RGB value.                                             |
------------------------------------------------------------------------------*/
typedef IKeySortedSet<IColorMapCacheRecord, unsigned long> IColorMapCache;
class IColorMapData {
public:
/*------------------------------------------------------------------------------
  Construct new cache.  Size is maximum number of colors in cache.
------------------------------------------------------------------------------*/
   IColorMapData(unsigned long size = 512);
   ~IColorMapData( );

/*------------------------------------------------------------------------------
  Add color to cache.  RGB is 00rrggbb, index is colormap index.  Exactmatch
  should be true if it is an exact match, false if a closest match.
------------------------------------------------------------------------------*/
void           add        ( unsigned long rgb,
                            unsigned long index,
                            bool          exactMatch = false);

/*------------------------------------------------------------------------------
  Remove color from cache.  RGB is 00rrggbb.
------------------------------------------------------------------------------*/
void           remove     ( unsigned long rgb );

/*------------------------------------------------------------------------------
  Returns index of color with given RGB value.  Returns unsigned long(-1) if
  not found.
------------------------------------------------------------------------------*/
unsigned long  colorIndex ( unsigned long rgb );

private:
   //Explicitly not allowed to force regeneration for new colormap.
   IColorMapData( const IColorMapData& );
   IColorMapData operator=( const IColorMapData& );

   unsigned long          fMaxSize;
   unsigned long          fCurrentSize;
   IColorMapCache         fCache;
   IColorMapCache::Cursor fCursor;
};  //IColorMapData

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
