/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 31 1.11.1.4 source/albert/graph2d/igbase2d.inl, 2d, ioc.v400, 000919 
/*================
||
||  File:       BaseGeometryImp.h (igbase2d.inl)
||
||  What:       Inline implementation of trivial Point/Rect operations
||
||  Change History:
||
*/

#ifndef _IGBASE2D_INL_
#define _IGBASE2D_INL_

//=========================//
// Inline IGPoint2D routines //
//=========================//

inline IGPoint2D::IGPoint2D ()
{
    fX = 0.0;
    fY = 0.0;
}

inline IGPoint2D::IGPoint2D(const IGPoint2D& point)
{
    fX = point.fX;
    fY = point.fY;
}

inline IGPoint2D::IGPoint2D (
    GCoordinate x,
    GCoordinate y)
{
    fX = x;
    fY = y;
}

inline IGPoint2D::~IGPoint2D ()
{
}

inline IGPoint2D& IGPoint2D::operator= (const IGPoint2D& Src)
{
    fX = Src.fX;
    fY = Src.fY;
    return (*this);
}

inline bool IGPoint2D::operator==(const IGPoint2D& Src) const
{
    return ((fX==Src.fX)&&(fY==Src.fY));
}

inline bool IGPoint2D::operator!=(const IGPoint2D& Src) const
{
    return ((fX!=Src.fX)||(fY!=Src.fY));
}


/*================
||
||  IGPoint2D::operator-
||
*/

inline IGPoint2D IGPoint2D::operator-() const
{
    return (IGPoint2D(-fX,-fY));
}

/*================
||
||  IGPoint2D::operator+
||
*/

inline IGPoint2D operator+(const IGPoint2D& a, const IGPoint2D& b)
{
    return (IGPoint2D(a.fX + b.fX, a.fY + b.fY));
}

/*================
||
||  IGPoint2D::operator-
||
*/

inline IGPoint2D operator-(const IGPoint2D& a, const IGPoint2D& b)
{
    return (IGPoint2D(a.fX - b.fX, a.fY - b.fY));
}

/*================
||
||  IGPoint2D::operator*
||
*/

inline IGPoint2D operator*(const IGPoint2D& a, const IGPoint2D& b)
{
    return (IGPoint2D(a.fX * b.fX, a.fY * b.fY));
}

inline IGPoint2D operator*(const GCoordinate num, const IGPoint2D& pt)
{
    return IGPoint2D(pt.fX * num, pt.fY * num);
}

inline IGPoint2D operator*(const IGPoint2D& pt, const GCoordinate num)
{
    return IGPoint2D(pt.fX * num, pt.fY * num);
}

/*================
||
||  IGPoint2D::operator/
||
*/

inline IGPoint2D operator/(const IGPoint2D& a, const IGPoint2D& b)
{
    return (IGPoint2D(a.fX / b.fX, a.fY / b.fY));
}

inline IGPoint2D operator/(const IGPoint2D& pt, const GCoordinate num)
{
    return IGPoint2D(pt.fX / num, pt.fY / num);
}

/*================
||
||  IGPoint2D::operator+=
||
*/

inline IGPoint2D& IGPoint2D::operator+=(
    const IGPoint2D& Src)
{
    fX += Src.fX;
    fY += Src.fY;
    return (*this);
}

/*================
||
||  IGPoint2D::operator-=
||
*/

inline IGPoint2D& IGPoint2D::operator-=(
    const IGPoint2D& Src)
{
    fX -= Src.fX;
    fY -= Src.fY;
    return (*this);
}

/*================
||
||  IGPoint2D::operator*=
||
*/

inline IGPoint2D& IGPoint2D::operator*=(
    const IGPoint2D& Src)
{
    fX *= Src.fX;
    fY *= Src.fY;
    return (*this);
}

inline IGPoint2D& IGPoint2D::operator*=(
    const GCoordinate Src)
{
    fX *= Src;
    fY *= Src;
    return (*this);
}

/*================
||
||  IGPoint2D::operator/=
||
*/

inline IGPoint2D& IGPoint2D::operator/=(
    const IGPoint2D& Src)
{
    fX /= Src.fX;
    fY /= Src.fY;
    return (*this);
}

inline IGPoint2D& IGPoint2D::operator/=(
    const GCoordinate Src)
{
    fX /= Src;
    fY /= Src;
    return (*this);
}

/*================
||
||  IGPoint2D::interpolate
||
*/

inline IGPoint2D IGPoint2D::interpolate(const IGPoint2D& dest, GParametric u) const
{
    return (1.0-u) * (*this) + u * dest;
}

/*================
||
||  IGPoint2D::dotProduct
||
*/

inline GCoordinate IGPoint2D::dotProduct(
    const IGPoint2D& vector) const
{
    return (fX*vector.fX + fY*vector.fY);
}

/*================
||
||  IGPoint2D::crossProduct
||
||  | u0 u1 |
||  | v0 v1 | = u0*v1 - u1*v0
||
*/

inline GCoordinate IGPoint2D::crossProduct(
    const IGPoint2D& vector) const
{
    return (fX*vector.fY - vector.fX*fY);
}


//==========================//
// Inline IGRPoint2D routines //
//==========================//

/*================
||
||  IGRPoint2D::IGRPoint2D
||
*/
inline IGRPoint2D::IGRPoint2D()
{
    fX = 0.0;
    fY = 0.0;
    fW = 0.0;
}


inline IGRPoint2D::IGRPoint2D (
    GCoordinate x,
    GCoordinate y,
    GCoordinate w)
{
    fX = x;
    fY = y;
    fW = w;
}

inline IGRPoint2D::~IGRPoint2D()
{
}

/*================
||
||  IGRPoint2D::operator=
||
*/

inline IGRPoint2D& IGRPoint2D::operator= (const IGRPoint2D& Src)
{
    fX = Src.fX;
    fY = Src.fY;
    fW = Src.fW;
    return (*this);
}

/*================
||
||  IGRPoint2D::IGRPoint2D
||
*/
inline IGRPoint2D::IGRPoint2D(const IGRPoint2D& point)
{
    *this = point;
}

/*================
||
||  IGRPoint2D::IGPoint2D
||
*/

inline IGRPoint2D::IGRPoint2D(const IGPoint2D& point)
{
    fX = point.fX;
    fY = point.fY;
    fW = 1.0;
}

/*================
||
||  IGRPoint2D::operator==
||
*/

inline bool IGRPoint2D::operator==(const IGRPoint2D& Src) const
{
    return ((fX==Src.fX) && (fY==Src.fY) && (fW==Src.fW));
}

/*================
||
||  IGRPoint2D::operator!=
||
*/

inline bool IGRPoint2D::operator!=(const IGRPoint2D& Src) const
{
    return ((fX!=Src.fX) || (fY!=Src.fY) || (fW!=Src.fW));
}

/*================
||
||  Explicit conversions from IGRPoint2D to IGPoint2D
||
*/

inline IGPoint2D IGRPoint2D::divW() const
{
    return (IGPoint2D( fX / fW, fY / fW) );
}

inline IGPoint2D IGRPoint2D::dropW() const
{
    return (IGPoint2D( fX, fY) );
}

//========================//
// Inline IGRect2D routines //
//========================//


/*================
||
||  IGRect2D empty constructor taking two TGPoints
*/

inline IGRect2D::IGRect2D()
{
    fLeft = 0.0;
    fTop = 0.0;
    fRight = 0.0;
    fBottom = 0.0;
}

/*================
||
||  IGRect2D constructor taking two TGPoints
||
*/

inline IGRect2D::IGRect2D(
    const IGPoint2D& p0,
    const IGPoint2D& p1)
{
    fLeft = p0.fX;
    fTop = p0.fY;
    fRight = p1.fX;
    fBottom = p1.fY;
}

inline IGRect2D::IGRect2D(
    GCoordinate left,
    GCoordinate top,
    GCoordinate right,
    GCoordinate bottom)
{
    fLeft = left;
    fTop = top;
    fRight = right;
    fBottom = bottom;
}


/*================
||
||  IGRect2D constructor taking IGRect2D Reference
||
*/

inline IGRect2D::IGRect2D(const IGRect2D& rect)
{
    fLeft = rect.fLeft;
    fTop = rect.fTop;
    fRight = rect.fRight;
    fBottom = rect.fBottom;
}

/*================
||
||  IGRect2D destructor
||
*/

inline IGRect2D::~IGRect2D() {}


/*================
||
||  IGRect2D::topLeft
||
*/

inline IGPoint2D IGRect2D::topLeft() const
{
    return IGPoint2D(fLeft, fTop);
}

/*================
||
||  IGRect2D::bottomRight
||
*/

inline IGPoint2D IGRect2D::bottomRight() const
{
    return IGPoint2D(fRight, fBottom);
}

/*================
||
||  IGRect2D::bottomLeft
||
*/
inline IGPoint2D IGRect2D::bottomLeft() const
{
    return IGPoint2D(fLeft, fBottom);
}

/*================
||
||  IGRect2D::topRight
||
*/
inline IGPoint2D IGRect2D::topRight() const
{
    return IGPoint2D(fRight, fTop);
}

/*================
||
||  IGRect2D::isEmpty
||
*/

inline bool IGRect2D::isEmpty() const
{
    return ((fLeft >= fRight) || (fTop >= fBottom));
}

/*================
||
||  IGRect2D::size
||
*/

inline IGPoint2D IGRect2D::size() const
{
    return IGPoint2D(fRight-fLeft, fBottom-fTop);
}

inline GCoordinate IGRect2D::width() const
{
    return fRight - fLeft;
}

inline GCoordinate IGRect2D::height() const
{
    return fBottom-fTop;
}

/*================
||
||  IGRect2D::contains
||
*/

inline bool IGRect2D::contains(const IGPoint2D& p) const
{
    return ((p.fX >= fLeft) &&
        (p.fX < fRight) &&
        (p.fY >= fTop) &&
        (p.fY < fBottom));
}

inline bool IGRect2D::contains(const IGRect2D& g) const
{
    return ((g.fLeft >= fLeft) &&
        (g.fRight <= fRight) &&
        (g.fTop >= fTop) &&
        (g.fBottom <= fBottom));
}

// === operators ===

inline IGRect2D& IGRect2D::operator=(const IGRect2D& Src)
{
    fLeft = Src.fLeft;
    fTop = Src.fTop;
    fRight = Src.fRight;
    fBottom = Src.fBottom;
    return (*this);
}

#endif // _IGBASE2D_INL_
