// Revision: 28 1.2 source/albert/graph2d/igbidi.hpp, 2d, ioc.v400, 000919 
/* NOSHIP */
#ifndef _IGBIDI_
#define _IGBIDI_
/*******************************************************************************
* FILE NAME: igbidi.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IGBidiSettings                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp. 1992, 1997  All Rights Reserved.                   *
*                                                                              *
*******************************************************************************/

#include <ibase.hpp>
#include <igraphpl.hpp>

class IGBidiSettingsData;

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

class IGBidiSettings {

public:
/*------------------------------- Constructors -------------------------------*/
  IGBidiSettings ( );
  IGBidiSettings ( const IGBidiSettings& settings );
IGBidiSettings
 &operator=      ( const IGBidiSettings& settings );

 ~IGBidiSettings ( );

/*-------------------------------- Comparisons -------------------------------*/
bool
  operator== ( const IGBidiSettings& settings ) const;

/*---------------------------------- Testing ---------------------------------*/
static bool
  isBidiSupported ( );

/*------------------------ Setting and Querying Values -----------------------*/
enum EBidiTextType {
  kVisual,
  kImplicit
  };

enum EBidiTextOrientation {
  kTextLeftToRight,
  kTextRightToLeft,
  kTextContextual
  };

enum EBidiLayout {
  kLayoutLeftToRight,
  kLayoutRightToLeft
  };

enum EBidiNumeralType {
  kArabic,
  kAsStored,
  kNational,
  kContextual
  };

enum EBidiTextShape {
  kDisplayShaped,
  kSaveShaped,
  kNominalShape,
  kInitialShape,
  kMiddleShape,
  kFinalShape,
  kIsolatedShape
  };

#ifndef IC_WIN_FLAGNOP
IGBidiSettings
 &setTextType                   ( EBidiTextType        textType );
#endif
IGBidiSettings
 &setTextOrientation            ( EBidiTextOrientation textOrientation ),
 &setWindowLayout               ( EBidiLayout          windowLayout );
#ifndef IC_WIN_FLAGNOP
IGBidiSettings
 &setNumeralDisplay             ( EBidiNumeralType     numeralDisplay ),
 &enableSymmetricSwapping       ( bool                 enable = true ),
 &disableSymmetricSwapping      ( ),
 &enableWordByWordReordering    ( bool                 enable = true ),
 &disableWordByWordReordering   ( ),
 &setTextShape                  ( EBidiTextShape       textShape );
#endif

EBidiTextType
  textType                      ( ) const;
EBidiTextOrientation
  textOrientation               ( ) const;
EBidiLayout
  windowLayout                  ( ) const;
EBidiNumeralType
  numeralDisplay                ( ) const;
bool
  isSymmetricSwappingEnabled    ( ) const;
bool
  isWordByWordReorderingEnabled ( ) const;
EBidiTextShape
  textShape                     ( ) const;

/*----------------------------- Helper Functions -----------------------------*/
#ifdef IC_MOTIF
static void
  layoutObjectTransform ( const char* inputBuffer,
                          size_t*     inputBufferSize,
                          void*       outputBuffer,
                          size_t*     outputBufferSize );
#endif

/*------------------------------- Conversions --------------------------------*/
#ifdef IC_PM
IGBidiSettings
 &setBidiFlags ( unsigned long flags );
unsigned long
  bidiFlags    ( ) const;
#endif

private:
/*--------------------------------- Private ----------------------------------*/
IGBidiSettingsData
 *fBidiSettingsData;
}; // IGBidiSettings

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _IGBIDI_ */
