/*
******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation,  1997           *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
||
||  File:       ijpgconv.hpp
||
||  What:       Routines which support loading of jpeg Image format
||
*/

// Revision: 03 1.2.1.6 source/albert/graph2d/ijpgconv.hpp, 2d, ioc.v400, 000919 

#ifndef _IJPGCONV_
#define _IJPGCONV_

#include <imgconv.hpp>
#include <igimage.hpp>
#include <imghelp.h>

//***************************************************************************
//JPEG
//***************************************************************************
#include <setjmp.h>

#include <jpeglib.h>
#include <jerror.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

typedef struct _jpgErrMgr_{
	struct jpeg_error_mgr pub;
	jmp_buf setJmpBuffer;
}jpgErrMgr, *jpgErrPtr;

class IJPGConverter : public IImageConverter
{
public:
	IJPGConverter();
	~IJPGConverter();

	IGImage* createFromStream( IDataStream&);

	void writeTo( const IGImage&,
				IDataStream&) const;

protected:
	IGImage* createFromFile( const char*);
	int writeToFile( char *toFile, iBYTE *);
private:
	double imageQuality; //0.90
	long numberOfColors; // 16777216
	bool convertToNativeColorMap; //0
	struct jpeg_decompress_struct	dInfo;
	struct jpeg_compress_struct	cInfo;

	long readJpg(IDataStream &, iBYTE **);
	int writeJpg(int, iBYTE*);
	
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

//==============================================================================
#endif // _IJPGCONV_
