/*
******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation,  1997           *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
||
||  File:       Imgconv.hpp
||
||  What:       Routines which support loading of Different Image formats
||
*/

// Revision: 20 1.9.1.11 source/albert/graph2d/imgconv.hpp, 2d, ioc.v400, 000919 

#ifndef _IMAGECONVERTER_
#define _IMAGECONVERTER_

#include <igimage.hpp>
#include <icolmap.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

//==============================================================================
// image converters
class IImageConverter
{

public:
	virtual ~IImageConverter();

	virtual IGImage* createFromStream( IDataStream& fromWhere) = 0;


	virtual void writeTo( const IGImage& fromImage,
				IDataStream& toWhere) const = 0;


	static char *unCompressFile(const char* );
	static IGImagePixelAccessor::EImageFormat imageType( const char *fromFile);
	static IGImage* readImage(const char *fromFile);
	static int writeImage(char *, IGImage*, IGImagePixelAccessor::EImageFormat);

protected :
	/**
	* Default constructor. Constructs an IImageConverter with empty geometry and attributes.
	*/
	IImageConverter ();
	IPixelBuffer*	constructPixelBuffer(unsigned char *fImgData);
	virtual int writeToFile( char *toFile, unsigned char *data) = 0;
	virtual IGImage* createFromFile( const char* fromFile) = 0;
	void attachPixelBuffer(IGImage *, IPixelBuffer *);
	static void swapImageRBData( unsigned char*,
		const unsigned long,
		const unsigned long,
		const unsigned long);


	// Image related data
	
	IGImagePixelAccessor::EImageFormat fImgFormat;
	unsigned int	fBitsPerPixel;
	long		fWidth;
	long		fHeight;
	unsigned long xHot; //X Hot Spot
	unsigned long yHot; //Y Hot Spot
	unsigned long	fImageFileSize;
	bool		fHasColormap;
	unsigned long	fColormapSize;
	bool		fInterlace;
	bool		fIsTransparencyEnabled;
	IBaseColor 	fTransparency;
	bool		bFlipImage;
	IRGBAColorArray	*fPal;

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

//==============================================================================
#endif // _IMAGECONVERTER_
