/*
******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation,  1997           *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
||
||  File:       Imghelper.h
||
||  What:       Routines which support loading of Different Image formats
||
||
*/

// Revision: 79 1.3.1.5 source/albert/graph2d/imghelp.h, 2d, ioc.v400, 000919 

#ifndef _IMAGEHELPER_
#define _IMAGEHELPER_

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#ifndef IC_MOTIF
#include <io.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>

#ifdef IC_MOTIF
#define BINARYOPEN 0x0
#else
#define BINARYOPEN O_BINARY
#endif

#define iBYTE   unsigned char
#define iWORD   unsigned short
#define iCWORD  unsigned int
#define iDWORD  unsigned long

#define low_byte(w)     ((iBYTE)  ((w)&0x00ff)    )
#define high_byte(w)    ((iBYTE) (((w)&0xff00)>>8))
#define make_usword(a,b)  (((iWORD)a) + (((iWORD)b) << 8))
#define make_sword(a,b)  (((short)a) + (((short)b) << 8))
#define make_usdword(a,b)  (((iDWORD)a) + (((iDWORD)b) << 16))
#define make_sdword(a,b)  (((long)a) + (((long)b) << 16))

#define ABS(_val) (((_val) < 0) ? -(_val) : (_val))

#define PIXELSTRIDE(w, bpp)  (((((w) * (bpp)) + 31 ) >> 5 ) << 2)

typedef struct _RGBClr{
        iBYTE   R;
        iBYTE   G;
        iBYTE   B;
} RGBClr;

typedef struct _RGBAClr{
        iBYTE   R;
        iBYTE   G;
        iBYTE   B;
        iBYTE   A;
} RGBAClr;

typedef struct _R5G6B5_  //total 16 bits = sizeof (unsigned short)
{
        unsigned short R:5;
        unsigned short G:6;
        unsigned short B:5;
}R5G6B5;


#ifdef IC_MOTIF
#include <iprimlck.hpp>

static bool bPrivateDisplay=false;
static Display *privateDpy=NULL;

static Display* getDisplay()
{
	privateDpy = IXDisplay::display();
	if (privateDpy == NULL) {
        IPrimalLock lock;
        if (privateDpy == NULL) {
    		IXDisplay tmpDisp(NULL);
    		privateDpy = IXDisplay::display();
    		bPrivateDisplay = true;
        }
	}
	return privateDpy;
}

#endif //IC_MOTIF

#endif //_IMAGEHELPER
