/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 37 1.9.1.4 source/albert/graph2d/ipt2darr.cpp, 2d, ioc.v400, 000919 
/*================
||
||  File:   PointArray.c
||
||  What:   Class defining operations for arrays of TGPoints
||
||  Classes defined in this file:
||
||  Change History:
||
*/

//
// Something to think about:  Cache groups of points <= 4 in the object
//(i.e., don't hit the allocator for groups of points <= 4).
//
// Also note the two classes in this file may be folded together when
// Templates ever show up.
//

#include <ipt2darr.hpp>
#include <igbase2d.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _IPT2DARR_CPP_
#include <ipagetun.h>
#endif

IGPoint2DArray::IGPoint2DArray(unsigned long size) : IRawArray<IGPoint2D>( size )
{}

IGPoint2DArray::IGPoint2DArray(const IGPoint2DArray& source) : IRawArray<IGPoint2D>( source )
{}

IGPoint2DArray::~IGPoint2DArray()
{}

IGRect2D IGPoint2DArray::bounds() const
{
    unsigned long numPoints = numberOfValues();
    if (!numPoints)
        return IGRect2D::zeroRect();
    IGRect2D result((*this)[0],(*this)[0]);
    for (unsigned long i = 1; i < numPoints; i++)
        result.extendTo((*this)[i]);
    return result;
}


////////////////////////// IGRPoint2DArray /////////////////////////

// Constructors
//

IGRPoint2DArray::IGRPoint2DArray(unsigned long size) : IRawArray<IGRPoint2D>(size)
{}

IGRPoint2DArray::IGRPoint2DArray(const IGRPoint2DArray& source) : IRawArray<IGRPoint2D>(source)
{}

IGRPoint2DArray::IGRPoint2DArray(const IGPoint2DArray& source) :
	IRawArray<IGRPoint2D>(source.numberOfPoints())
{
    register unsigned long i;
    for (i = 0; i < source.numberOfPoints(); i++)
    	(*this)[i] = source[i];
}

IGRPoint2DArray::~IGRPoint2DArray()
{}

IGRect2D IGRPoint2DArray::bounds() const
{
    unsigned long numPoints = numberOfValues();
    if (!numPoints)
        return IGRect2D::zeroRect();
    IGRect2D result((*this)[0].divW(),(*this)[0].divW());
    for (unsigned long i = 1; i < numPoints; i++)
        result.extendTo((*this)[i].divW());
    return result;
}
