/*
******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation,  1997           *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
||
||  File:       ixpmconv.hpp
||
||  What:       Routines which support loading of xpm Image format
||
*/

// Revision: 04 1.2.1.9 source/albert/graph2d/ixpmconv.hpp, 2d, ioc.v400, 000919 

#ifndef _IXPMCONV_
#define _IXPMCONV_

#include <imgconv.hpp>
#include <igimage.hpp>
#include <imghelp.h>

//***************************************************************************
//XPM
//***************************************************************************

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#pragma pack(push,4)

#define TOKEN_LEN 8 //8 - Max length of color token
#define cv_index    color_val.index
#define cv_rgb      color_val.rgb

typedef struct hent {
	char        token[TOKEN_LEN];
	union {
		iBYTE      index;
		iBYTE      rgb[3];
	} color_val;
	struct hent *next;
} hEntry;

#ifndef IC_MOTIF
// Simulating Data structure used by Xcolor operations
typedef struct {
	unsigned long pixel;
	unsigned short red, green, blue;
	char flags;  // do_red, do_green, do_blue
	char pad;
} XColor;

#endif

class IXPMConverter : public IImageConverter
{
public:
	IXPMConverter();
	~IXPMConverter();

	IGImage* createFromStream( IDataStream& );

	void writeTo( const IGImage& ,
				IDataStream& ) const;
	IPixelBuffer* createPixBufFromResource(char *, unsigned long);

protected:
	IGImage* createFromFile( const char* );
	int writeToFile( char *, iBYTE *);
private:

	hEntry **hashtab;
	hEntry  *clmp;		// colormap hash-table
	hEntry  *c_sptr;	// cmap hash-table search pointer
	int hash_len;
	iBYTE hex[256]; 	//used for hex conversion


	int readXpm(IDataStream& , iBYTE **);
	int writeXpm(int , iBYTE* );

	//data readers
	int getString(IDataStream& , iBYTE *);
	iBYTE getChar(IDataStream&);
	//hash table helpers
	int hash(char*);
	int hashInit(int);
	int hashInsert(hEntry*);
	hEntry* hashSearch(char*);
	void hashDestroy();
	//simulating X11 functionality
	int parseColor(char* , XColor*);
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

//==============================================================================
#endif // _IXPMCONV_
