// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//

// Revision: 31 1.3.1.4 source/albert/graph2d/pmvtxeng.hpp, 2d, ioc.v400, 000919 
//

#ifndef _PMVTXENG_
#define _PMVTXENG_

#ifdef IC_PM

#include <gpidev.hpp>
#include <vrtxeng.hpp>
#include <sampextr.hpp>

// this file has os/2 specific polygon vertex engine for rendering IGArea ...

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IPmPoint {
public:
	POINTL fPoint;

	IPmPoint(LONG x, LONG y) ;
	IPmPoint() ;
	~IPmPoint() ;
	
	IPmPoint(const IPmPoint& src) ;

	operator POINTL() const ;

	IDataStream& operator>>= (IDataStream&) const ;
        IDataStream& operator<<= (IDataStream&) ;

	IPmPoint& operator=(const IPmPoint& src);
	bool operator==(const IPmPoint& src) const ;
	bool operator!=(const IPmPoint& src) const ;
};

class IPmPoly {
public:
    unsigned long fnPoints;
	long fIndex;

	IPmPoly(ULONG numPoints, long index) ;
	IPmPoly() ;
	~IPmPoly() ;
	
	IPmPoly(const IPmPoly& src) ;

	IDataStream& operator>>= (IDataStream&) const ;
        IDataStream& operator<<= (IDataStream&) ;

	IPmPoly& operator=(const IPmPoly& src) ;
	bool operator==(const IPmPoly& src) const ;
	bool operator!=(const IPmPoly& src) const ;
};

class IPmPathVertexEngine : public IVertexEngine {
public:
	IPmPathVertexEngine(IPresSpaceHandle hps) ;
	virtual ~IPmPathVertexEngine() ;
	virtual void SetCAGNode(ICAGUnary*) ;
	virtual void SetClosed(bool) ;
	virtual void Reset() ;
	virtual void AddVertex(const IGPoint2D& vertex, bool joint = true) ;
	virtual void End() ;
	virtual void Render(const ICAGRoot&) ;

protected:
	IPresSpaceHandle fHPS;

private:
	long fNumPoints;
	IRawArray<IPmPoint> fPoints;
	IRawArray<IPmPoly> fPolys;
};

class IPmClipPathVertextEngine : public IPmPathVertexEngine {
public:
	IPmClipPathVertextEngine(IPresSpaceHandle hps) ;
	virtual ~IPmClipPathVertextEngine() ;
	virtual void Render(const ICAGRoot& node) ;
};

#ifndef IC_BUNDLES
#include <ibundles.hpp>		// IAttributeState::EDrawOperation
#endif
class IPmRenderPathVertextEngine : public IPmPathVertexEngine {
public:
	IPmRenderPathVertextEngine(IPresSpaceHandle hps) ;
	virtual ~IPmRenderPathVertextEngine() ;
	virtual void Render(const ICAGRoot& node) ;
	int FillArea(IPresSpaceHandle hps) ;
	int StrokeArea(IPresSpaceHandle hps) ;
private:
	IAttributeState::EDrawOperation fDO;
};

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // IC_PM

#endif // _PMVTXENG_
