/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 29 1.9.2.4 source/albert/graph2d/prratopr.hpp, 2d, ioc.v400, 000919 
/*================
||
||  File:       prRationalOperators.h
||
||  What:       Internally used operators for TGRPoints
||
||  Change History:
||
*/

#ifndef _PRRATOPR_
#define _PRRATOPR_

#ifndef IC_BASEGEOMETRY
#include <igbase2d.hpp>          // For hard return values
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

//
// Arithmatic operators on rational points are not defined as part
// of the Pink API because they don't make clear sense geometrically
// and algebraicly.  There are numerous (and ambigous!) definitions
// for what they might do.
//
// However, for the internal implementation of the curve methods,
// their definition as straight vector operations is quite useful,
// and indeed necessary to implement similar 2D and 3D routines
// with the same code using templates.
//
// If the call overhead is a problem (and on a box with fast FP it might
// well be) these functions can easily be made inline.
//

//      2D     //

extern IGRPoint2D& operator*=(IGRPoint2D& dest, const IGRPoint2D& src);
extern IGRPoint2D& operator*=(IGRPoint2D& dest, GCoordinate src);
extern IGRPoint2D& operator/=(IGRPoint2D& dest, const IGRPoint2D& src);
extern IGRPoint2D& operator/=(IGRPoint2D& dest, GCoordinate src);
extern IGRPoint2D& operator+=(IGRPoint2D& dest, const IGRPoint2D& src);
extern IGRPoint2D& operator-=(IGRPoint2D& dest, const IGRPoint2D& src);

extern IGRPoint2D operator+(const IGRPoint2D& left, const IGRPoint2D& right);
extern IGRPoint2D operator-(const IGRPoint2D& left, const IGRPoint2D& right);
extern IGRPoint2D operator*(const IGRPoint2D& left, const IGRPoint2D& right);
extern IGRPoint2D operator/(const IGRPoint2D& left, const IGRPoint2D& right);
extern IGRPoint2D operator*(const IGRPoint2D& left, GCoordinate right);
extern IGRPoint2D operator*(GCoordinate right, const IGRPoint2D& left);
extern IGRPoint2D operator/(const IGRPoint2D& left, GCoordinate right);

extern void SetToZero(IGRPoint2D& p);

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _PRRATOPR_
