// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 01  1.4.2.5  source/albert/graph2d/sampextr.hpp, 2d, ioc.v400, 000919  

// sampextr.hpp

#ifndef _SAMPEXTR_
#define _SAMPEXTR_

#ifndef IC_GRAFEXTRACTOR
#include <grafextr.hpp>
#endif

#ifndef IC_IVERTEX
#include <vrtxeng.hpp>
#endif

#ifndef IC_NURBS
#include <nurbs.hpp>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ICAGNode;
class ICAGUnary;
class IGEllipse2D;
class IGPolygon2D;
class IGRect2D;
class IGLoop2D;

class ISamplingExtractor : public IGrafExtractor {
public:
    ISamplingExtractor(IVertexEngine* outputVertexEngine, IGRect2D clipRect);
	virtual ~ISamplingExtractor();
	
	// === Area-defining geometries ===
	virtual void  extractRect (
		const IGRect2D&,
		ICAGUnary*,
		const IGrafMatrix& cs = IGrafMatrix::identity());

	virtual void  extractEllipse (
		const IGEllipse2D&,
		ICAGUnary*,
		const IGrafMatrix& cs = IGrafMatrix::identity());

	virtual void  extractPolygon (
		const IGPolygon2D&,
		ICAGUnary*,
		const IGrafMatrix& cs = IGrafMatrix::identity());

	virtual void  extractLoop (
		const IGLoop2D&,
		ICAGUnary*,
		const IGrafMatrix& cs = IGrafMatrix::identity());

	// === Render ===
	virtual void render(ICAGRoot&);
	
	// === Sets/Gets ===
    void SetVertexEngine(IVertexEngine*);
    IVertexEngine* GetVertexEngine();

    bool isExtractedRectEmpty(); //to check if the extracted rect is empty or not

private:
	IVertexEngine*  fVertexEngine;  // accepts the samples
	IGRect2D fClipBounds;

	GCoordinate fResolution;  // discretizer resolution

	// resize the vertex array by this uch
	static const unsigned long  kVertexIncrement;

	IGRect2D fRect; //used to check the size of extracted rect
};

class IAreaDiscretiser : public INurbDiscretiser
{
public:
	IAreaDiscretiser(IVertexEngine* outputVertexEngine, ICAGUnary* node, IGRect2D clipBound);
	void begin ();
	void endPath();
	void addPoint(IGPoint2D vertex);
	void DiscretizeLoop(
		int order,
		const GParametric* knotsVector,
		IGRPoint2DArray controlPoints,
		bool closed);
		
private:
	IVertexEngine* fVertexEngine;
	ICAGUnary* fNode;
};	
	
#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _SAMPEXTR_
