/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 31 1.28.1.5 source/albert/graph2d/winemul.h, 2d, ioc.v400, 000919 
#ifndef _WINEMUL_
#define _WINEMUL_
// <windows.h> emulation layer for OS/2.

// Pull in standard types and Gpi stuff (which we'll need in the
// implementation files).

#define INCL_GPI
#define INCL_DOSPROCESS
#ifdef SS
#undef SS
#include <os2.h>
#define SS
#else
#include <os2.h>
#endif

#include <i2dghand.hpp>

#pragma enum(4)
#pragma pack(push,4)

// Windows-specific types:
//   WINAPI   -> _System
//   HANDLE   -> LHANDLE
//   LPVOID   -> void*
//   WORD     -> USHORT
//   DWORD    -> ULONG
//   FLOAT    -> float
//   POINT    -> POINTL
//   HPALETTE -> HPAL
//   SIZE     -> SIZEL
//   HPEN     -> LINEBUNDLE
//   HBRUSH   -> AREABUNDLE
//   COLORREF -> ULONG (holds RGB values).
//   HFONT    -> FATTRS*
//   LPSTR    -> PSZ
//   LPBITMAPINFO -> BITMAPINFO2*
//   RGBQUAD  -> as on Windows
//   RECT     -> as on Windows
//   LOGBRUSH -> as on Windows
//   XFORM    -> as on Windows
//   BITMAP   -> as on Windows
//   BITMAPINFOHEADER -> as on Windows (hides Gpi version)
//   BITMAPCOREHEADER -> as on Windows
//   RGBTRIPLE -> as on Windows

#define WINAPI      _System

typedef LHANDLE     HANDLE;
typedef void       *LPVOID;
typedef USHORT      WORD;
typedef ULONG       DWORD;
typedef float       FLOAT;
typedef POINTL      POINT;
typedef HPAL        HPALETTE;
typedef SIZEL       SIZE;
typedef LINEBUNDLE  PEN, *HPEN;
typedef AREABUNDLE  BRUSH, *HBRUSH;
typedef ULONG       COLORREF;
typedef FATTRS     *HFONT;
typedef PSZ         LPSTR;
typedef BITMAPINFO *LPBITMAPINFO;

struct RGBQUAD {
  unsigned char
    rgbBlue,
    rgbGreen,
    rgbRed,
    rgbReserved;
};

struct RECT {
  LONG
    left, top, right, bottom;
};

struct LOGBRUSH {
  unsigned int
    lbStyle;
  COLORREF
    lbColor;
  long
    lbHatch;
};

struct XFORM {
  FLOAT
    eM11, eM12, eM21, eM22, eDx, eDy;
};

struct BITMAP {
  LONG
    bmType,
    bmWidth,
    bmHeight,
    bmWidthBytes;
  WORD
    bmPlanes,
    bmBitsPixel;
  void
   *bmBits;
};

// Must redefine BITMAPINFOHEADER to avoid conflict with Gpi.
#define BITMAPINFOHEADER winemulBITMAPINFOHEADER
struct BITMAPINFOHEADER {
  DWORD
    biSize;
  LONG
    biWidth,
    biHeight;
  WORD
    biPlanes,
    biBitCount;
  DWORD
    _1,
    _2;
  LONG
    _3,
    _4;
  DWORD
    _5,
    _6;
};

struct BITMAPCOREHEADER {
  DWORD
    bcSize;
  WORD
    bcWidth,
    bcHeight,
    bcPlanes,
    bcBitCount;
};

struct RGBTRIPLE {
  unsigned char
    rgbtBlue,
    rgbtGreen,
    rgbtRed;
};

// Mappings from Windows constants to Gpi equivalents.
//   *REGION     -> RGN_*
//   RGN_*       -> CRGN_*
//   MM_*        -> PU_*
//   PS_*        -> LINETYPE_*
//   PS_ENDCAP_* -> LINEEND_*
//   PS_JOIN_*   -> LINEJOIN_*
//   BS_*        -> simple constants (for calls to CreateBrushStructure)
//   HS_*        -> PATSYM_*
//   R2_*        -> FM_*
//   (ROPs)      -> ROP_*
//   TA_BASELINE -> TA_BASE
enum { NULLREGION    = RGN_NULL,
       SIMPLEREGION  = RGN_RECT,
       COMPLEXREGION = RGN_COMPLEX };

enum { RGN_OR   = CRGN_OR,
       RGN_DIFF = CRGN_DIFF,
       RGN_AND  = CRGN_AND,
       RGN_XOR  = CRGN_XOR };

enum { MM_TEXT      = PU_PELS,
       MM_LOMETRIC  = PU_LOMETRIC,
       MM_HIMETRIC  = PU_HIMETRIC,
       MM_LOENGLISH = PU_LOENGLISH,
       MM_HIENGLISH = PU_HIENGLISH,
       MM_TWIPS     = PU_TWIPS };

#define LINETYPE_BIT_MASK 15
enum { PS_SOLID         = LINETYPE_SOLID,
       PS_DOT           = LINETYPE_DOT,
       PS_DASH          = LINETYPE_LONGDASH,
       PS_DASHDOT       = LINETYPE_DASHDOT,
       PS_DASHDOTDOT    = LINETYPE_DASHDOUBLEDOT,
       PS_NULL          = LINETYPE_INVISIBLE } ;

// We shift these left 4 bits to avoid overlap with the LINETYPE_* values.
// Note that these are or-ed together to pass to ExtCreatePen.
#define LINEEND_BIT_OFFSET 4
#define LINEEND_BIT_MASK   (3<<LINEEND_BIT_OFFSET)
enum { PS_ENDCAP_SQUARE = LINEEND_SQUARE<<LINEEND_BIT_OFFSET,
       PS_ENDCAP_ROUND  = LINEEND_ROUND<<LINEEND_BIT_OFFSET,
       PS_ENDCAP_FLAT   = LINEEND_FLAT<<LINEEND_BIT_OFFSET };

// We shift these left 6 bits to avoid overlap with the LINETYPE_* and
// LINEEND_* values (which are or-ed with these to pass to ExtCreatePen.
#define LINEJOIN_BIT_OFFSET 6
#define LINEJOIN_BIT_MASK   (3<<LINEJOIN_BIT_OFFSET)
enum { PS_JOIN_BEVEL    = LINEJOIN_BEVEL<<LINEJOIN_BIT_OFFSET,
       PS_JOIN_MITER    = LINEJOIN_MITRE<<LINEJOIN_BIT_OFFSET,
       PS_JOIN_ROUND    = LINEJOIN_ROUND<<LINEJOIN_BIT_OFFSET };

enum { BS_SOLID   = 1,
       BS_PATTERN = 2,
       BS_HATCHED = 3 };

enum { HS_BDIAGONAL  = PATSYM_DIAG1,
       HS_CROSS      = PATSYM_HATCH,
       HS_DIAGCROSS  = PATSYM_DENSE6,
       HS_FDIAGONAL  = PATSYM_DIAG3,
       HS_HORIZONTAL = PATSYM_HORIZ,
       HS_VERTICAL   = PATSYM_VERT };

enum { R2_COPYPEN     = FM_OVERPAINT,
       R2_NOTCOPYPEN  = FM_NOTCOPYSRC,
       R2_NOP         = FM_LEAVEALONE,
       R2_NOT         = FM_INVERT,
       R2_MERGEPEN    = FM_OR,
       R2_MASKPEN     = FM_AND,
       R2_XORPEN      = FM_XOR,
       R2_MASKNOTPEN  = FM_SUBTRACT,
       R2_MERGENOTPEN = FM_MERGENOTSRC,
       R2_MASKPENNOT  = FM_MASKSRCNOT,
       R2_MERGEPENNOT = FM_MERGESRCNOT,
       R2_NOTMASKPEN  = FM_NOTMASKSRC,
       R2_NOTMERGEPEN = FM_NOTMERGESRC,
       R2_NOTXORPEN   = FM_NOTXORSRC,
       R2_WHITE       = FM_ONE,
       R2_BLACK       = FM_ZERO };

enum { SRCCOPY     = ROP_SRCCOPY,
       SRCPAINT    = ROP_SRCPAINT,
       SRCAND      = ROP_SRCAND,
       SRCINVERT   = ROP_SRCINVERT,
       SRCERASE    = ROP_SRCERASE,
       NOTSRCCOPY  = ROP_NOTSRCCOPY,
       NOTSRCERASE = ROP_NOTSRCERASE,
       MERGECOPY   = ROP_MERGECOPY,
       MERGEPAINT  = ROP_MERGEPAINT,
       PATCOPY     = ROP_PATCOPY,
       PATPAINT    = ROP_PATPAINT,
       PATINVERT   = ROP_PATINVERT,
       DSTINVERT   = ROP_DSTINVERT,
       BLACKNESS   = ROP_ZERO,
       WHITENESS   = ROP_ONE };

enum { TA_BASELINE = TA_BASE };

// Win32 APIs...

// SetGraphicsMode:
//   Not required under OS/2 so we make it a nop (always succeeding).
enum { GM_ADVANCED = 1 };
#define SetGraphicsMode( hps, mode ) 1

// ModifyWorldTransform -> GpiSetModelTransformMatrix
enum { MWT_IDENTITY };
int ModifyWorldTransform( const IPresSpaceHandle&, XFORM*, DWORD );

// DeleteObject -> For pens/brushes, invoke delete operator.
//                 For bitmaps, invoke GpiDestroyBitmap.
//                 For regions, invoke GpiDestroyRegion.
//   Note: The latter requires the HPS.  To obtain it, we use
//         a macro that presumes the HPS in question is referenced
///        by the variable fHDC.
int DeleteObject2( HPEN, const IPresSpaceHandle& );
int DeleteObject2( HBRUSH, const IPresSpaceHandle& );

int DeleteObjectOS ( HPEN hPen, const IPresSpaceHandle& hps ) ;
int DeleteObjectOS ( HBRUSH hBrush, const IPresSpaceHandle& hps ) ;
int DeleteObjectOS ( IBitmapHandle hbm, const IPresSpaceHandle& ) ;
int DeleteObjectOS ( HFONT hFont, const IPresSpaceHandle& ) ;
int DeleteObjectOS ( IRegionHandle hrgn, const IPresSpaceHandle& hps ) ;

#define DeleteObject(x) (DeleteObjectOS)( x, fHDC )

// BeginPath -> GpiBeginPath
int BeginPath( const IPresSpaceHandle& );

// EndPath -> GpiEndPath
int EndPath( const IPresSpaceHandle& );

// CloseFigure -> GpiCloseFigure
int CloseFigure( const IPresSpaceHandle& );

// GetStockObject:
//   We create enum's for the different stock objects that can be
//   requested.  And overload based on enum type to return HPENs
//   vs. HBRUSHs.
enum EPenTag   { NULL_PEN = 1 };
enum EBrushTag { HOLLOW_BRUSH = 1 };

HPEN GetStockObject( EPenTag );
HBRUSH GetStockObject( EBrushTag );

// Rectangle -> GpiMove+GpiBox.
int Rectangle( const IPresSpaceHandle&, int, int, int, int );

// Ellipse -> GpiFullArc
int Ellipse( const IPresSpaceHandle&, int, int, int, int );

// Polygon -> GpiPolygons
enum { ALTERNATE = 0, WINDING = 1 };
int SetPolyFillMode( const IPresSpaceHandle&, int );
int Polygon( const IPresSpaceHandle&, const POINTL *, int );

// SelectClipPath -> GpiSetClipRegion
enum { RGN_COPY = 1 };
int SelectClipPath( const IPresSpaceHandle&, int );

// PathToRegion -> GpiPathToRegion
IRegionHandle PathToRegion( const IPresSpaceHandle& );

// CombineRgn -> GpiCombineRegion
//   Gpi requires HPS so we obtain it via extra-arg hack.
#define ERROR RGN_ERROR
int CombineRgn( IRegionHandle, IRegionHandle, IRegionHandle, int, const IPresSpaceHandle& );
#define CombineRgn(a,b,c,d) (CombineRgn)(a,b,c,d,fHDC)

// GetDC -> WinGetPS
IPresSpaceHandle GetDC( INativeWindowHandle );

// GetWindowDC -> WinGetPS
IPresSpaceHandle GetWindowDC( INativeWindowHandle );

// SetMapMode -> GpiSetPS
DWORD SetMapMode( const IPresSpaceHandle&, DWORD );

// GetMapMode -> GpiQueryPS
DWORD GetMapMode( const IPresSpaceHandle& );

// GetWindowRect -> WinQueryWindowRect
int GetWindowRect( INativeWindowHandle, RECT * );

// WindowFromDC -> WinWindowFromDC
INativeWindowHandle WindowFromDC( const IPresSpaceHandle& );

// SetMiterLimit -> nop (always return success; note that the function isn't)
//                  supported by Gpi)
enum { PS_GEOMETRIC     = 0,
       PS_COSMETIC      = 2 };
#define SetMiterLimit( hps, limit, pOld ) 1

long ColorToIndex( const IPresSpaceHandle& hps, COLORREF color );

void setPenBrush(HBRUSH penBrush);

// SetBrushOrgEx -> GpiSetPatternRefPoint
int SetBrushOrgEx( const IPresSpaceHandle&, int, int, POINTL* );

// SetBkMode -> GpiSetBackMix
enum { OPAQUE = BM_OVERPAINT, TRANSPARENT = BM_LEAVEALONE };
int SetBkMode( const IPresSpaceHandle&, int );

// SetROP2 -> GpiSetMix
int SetROP2( const IPresSpaceHandle&, int );

// ExtSelectClipRgn -> GpiSetClipRegion
int ExtSelectClipRgn( const IPresSpaceHandle&, IRegionHandle, int );

// SetWorldTransform -> GpiSetModelTransformMatrix
int SetWorldTransform( const IPresSpaceHandle&, XFORM* );

// SelectObject -> Overloaded based on object type and implemented by
//                 setting appropriate attribute/font/bitmap.
HBRUSH        SelectObject( const IPresSpaceHandle&, HBRUSH );
HPEN          SelectObject( const IPresSpaceHandle&, HPEN );
HFONT         SelectObject( const IPresSpaceHandle&, HFONT );
IBitmapHandle SelectObject( const IPresSpaceHandle&, IBitmapHandle );

// MoveToEx -> GpiMove
int MoveToEx( const IPresSpaceHandle&, int, int, POINTL* );

// LineTo -> GpiLine
int LineTo( const IPresSpaceHandle&, int, int );

// Polyline -> GpiPolyLine
int Polyline( const IPresSpaceHandle&, POINTL*, int );

// SetStretchBltMode -> Set corresponding IMAGEBUNDLE attribute
enum { COLORONCOLOR, BLACKONWHITE, WHITEONBLACK, HALFTONE };
int SetStretchBltMode( const IPresSpaceHandle&, int );

// FillPath -> GpiFillPath
int FillPath( const IPresSpaceHandle& );

// StrokePath -> GpiStrokePath
int StrokePath( const IPresSpaceHandle& );

// FillRgn -> Implemented by enumerating rects that comprise the region
//            and filling each using GpiBox.
int FillRgn( const IPresSpaceHandle&, IRegionHandle, HBRUSH );

// FrameRgn -> GpiFrameRegion
int FrameRgn( const IPresSpaceHandle&, IRegionHandle, HBRUSH, int, int );

// Pie -> GpiPartialArc
int Pie( const IPresSpaceHandle&, int, int, int, int, int, int, int, int );

// Chord -> GpiPartialArc
int Chord( const IPresSpaceHandle&, int, int, int, int, int, int, int, int );

// Arc -> GpiPartialArc
int Arc( const IPresSpaceHandle&, int, int, int, int, int, int, int, int );

// PolyBezier -> GpiPolySpline
int PolyBezier( const IPresSpaceHandle&, POINTL*, DWORD );

// SetBkColor -> GpiSetBackColor
int SetBkColor( const IPresSpaceHandle&, COLORREF );

// StretchBlt -> Implemented using GpiBitBlt
int StretchBlt( const IPresSpaceHandle&, int, int, int, int,
                const IPresSpaceHandle&, int, int, int, int, DWORD );

// MAKEROP4 -> Defined as on Windows
#define MAKEROP4( fg, bg ) ((((bg)<<8)&0xFF000000)|(fg))

// MaskBlt -> Implemented using GpiBitBlt
int MaskBlt( const IPresSpaceHandle&, int, int, int, int,
             const IPresSpaceHandle&, int, int,
             IBitmapHandle, int, int, DWORD );

// CreateFont -> GpiCreateLogFont
enum { FW_BOLD = 1 };


// ajd begin 3/17/1997
HFONT CreateFont2( const IPresSpaceHandle&, int, int, int, int, int,
                  DWORD,DWORD, DWORD, DWORD, DWORD, DWORD, DWORD, DWORD, DWORD, DWORD,
                  const char * );

// check for closest bitmap font match
HFONT  checkForBitmapMatch (const IPresSpaceHandle& hps,int hPointSize, char* faceName);

//check for closest transformable font match.
// There are two kinds of transformable font: vector font and outline font.
HFONT  checkForTransformableMatch (const IPresSpaceHandle& hps,int height, DWORD outline, char* faceName);


#define CreateFont( int1, int2, int3, int4, int5, DWORD1, DWORD2, DWORD3, DWORD4, DWORD5, DWORD6, DWORD7, DWORD8, DWORD9, DWORD10, ptr ) \
  CreateFont2( fHDC, int1, int2, int3, int4, int5, DWORD1, DWORD2, DWORD3, DWORD4, DWORD5, DWORD6, DWORD7, DWORD8, DWORD9,DWORD10, ptr )
// ajd end


// SetTextAlign -> GpiSetTextAlignment
enum { GDI_ERROR = FALSE };
unsigned int SetTextAlign( const IPresSpaceHandle&, unsigned int );

// ExtTextOut -> GpiCharStringPos
int ExtTextOut( const IPresSpaceHandle&, int, int, unsigned int,
                const RECTL*, const char*, unsigned int, int* );

// GetTextExtentPoint32 -> GpiQueryTextBox
int GetTextExtentPoint32( const IPresSpaceHandle&, const char*, int, SIZEL* );

// RGB -> Defined to match Windows (hides Gpi structure, though).
#define RGB(r,g,b) ((COLORREF)(((BYTE)(r)|((WORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(b))<<16)))

// SetTextColor -> GpiSetAttrs with PRIM_CHAR, CBB_COLOR
#define CLR_INVALID (-1)
COLORREF SetTextColor( const IPresSpaceHandle&, COLORREF );

// long SaveDC( HDC );

// DeleteDC -> GpiDestroyPS
int DeleteDC( const IPresSpaceHandle& );

// GetObject -> GpiQueryBitmap
//   Note: We only support get BITMAP objects (and only the header).
int GetObject( IBitmapHandle, int, void* );

// GetDIBits -> Return BITMAPINFO (including color table).
//   Note that this function does not return the bitmap bits!
enum { DIB_RGB_COLORS = 1 };
int GetDIBits( const IPresSpaceHandle&, IBitmapHandle,
               unsigned int, unsigned int, void*, LPBITMAPINFO, unsigned int );

// CreateCompatibleDC - Implemented per Windows spec.
IPresSpaceHandle CreateCompatibleDC( const IPresSpaceHandle& );

// CreateCompatibleBitmap - Implemented per Windows spec.
IBitmapHandle CreateCompatibleBitmap( const IPresSpaceHandle&, int, int );

// GetPixel -> GpiQueryPixel
COLORREF GetPixel( const IPresSpaceHandle&, int, int );

// SetPixel -> GpiSetPixel
int SetPixel( const IPresSpaceHandle&, int, int, COLORREF );

// ReleaseDC -> GpiDestroyPS
int ReleaseDC( INativeWindowHandle, const IPresSpaceHandle& );

// BitBlt -> GpiBitBlt
int BitBlt( const IPresSpaceHandle&, int, int, int, int, const IPresSpaceHandle&, int, int, DWORD );

// GetModuleHandle -> DosQueryModuleHandle
HMODULE GetModuleHandle( const char* );

// LoadBitmap -> WinLoadBitmap
//   Gpi needs an HPS so we contrive to pass it as an extra argument.
IBitmapHandle LoadBitmap( HMODULE, const char*, const IPresSpaceHandle& );
#define LoadBitmap( hmod, resName ) (LoadBitmap)( hmod, resName, fBitmapHDC )

// MAKEINTRESOURCE -> identity
#define MAKEINTRESOURCE(x) (void*)(x.id())

// SaveDC -> GpiSavePS
int SaveDC( const IPresSpaceHandle& );

// RestoreDC -> GpiRestorePS
int RestoreDC( const IPresSpaceHandle&, int );

// GetDeviceCaps -> DevQueryCaps
enum { HORZRES = 8, VERTRES = 10, TECHNOLOGY = 2, LOGPIXELSX = 88, LOGPIXELSY = 90 };
enum { DT_PLOTTER = 0, DT_RASDISPLAY = 1, DT_RASPRINTER = 2, DT_RASCAMERA = 3, DT_UNKNOWN = 7 };
int GetDeviceCaps( const IPresSpaceHandle&, int );

// CreateDIBitmap -> nop (never called on OS/2)
#define CreateDIBitmap(hdc,lpbmin,fdwInit,lpbInit,lpbm,fuUsage) (0)

// SetDIBits -> nop (never called on OS/2)
#define SetDIBits(hdc,hbmp,uStartScan,cScanLines,lpvBits,lpbmi,fuColorUse) (0)

// Sleep -> DosSleep
#define Sleep( x ) DosSleep( x )

typedef struct tagTEXTMETRIC
{
    long        tmHeight;
    long        tmAscent;
    long        tmDescent;
    long        tmInternalLeading;
    long        tmExternalLeading;
    long        tmAveCharWidth;
    long        tmMaxCharWidth;
    long        tmWeight;
    long        tmOverhang;
    long        tmDigitizedAspectX;
    long        tmDigitizedAspectY;
    long        tmFirstChar;
    long        tmLastChar;
    long        tmDefaultChar;
    long        tmBreakChar;
    long        tmItalic;
    long        tmUnderlined;
    long        tmStruckOut;
    long        tmPitchAndFamily;
    long        tmCharSet;
} TEXTMETRIC, *PTEXTMETRIC, *NPTEXTMETRIC, *LPTEXTMETRIC;

BOOL    GetTextMetrics( const IPresSpaceHandle&, LPTEXTMETRIC );

#define GetRValue(rgb)      ((BYTE)((rgb)>>16))
#define GetGValue(rgb)      ((BYTE)(((WORD)(rgb)) >> 8))
#define GetBValue(rgb)      ((BYTE)(rgb))

/* No longer needed because changes are made to IPresSpaceHandle to pass a windowhandle
RECT getDCRectSize( const IPresSpaceHandle& );
*/

int MulDiv(int m1, int m2, int d);

HBRUSH SelectBrush( const IPresSpaceHandle& hps, HBRUSH hBrush, bool fPenFlag );

#define OS2CHANGES 1

#pragma pack(pop)
#pragma enum(pop)

#endif // __WINEMUL__
