/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 09 1.10.1.5 source/albert/graph2d/ximdev.cpp, 2d, ioc.v400, 000919 

#include <ilanglvl.hpp>

#include <ximdev.hpp>    // IXImageDevice
#include <igimage.hpp>
#include <ibmpdata.hpp>
#include <igloop2d.hpp>
#include <igtxtrun.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _XIMDEV_CPP_
#include <ipagetun.h>
#endif
#include <pixbuf.hpp>

/*================================================================
||
||      IXImageDevice  implementation
||
*/

#if !IC_RTTI_ENABLED
GrafDeviceTypeDefinitionsMacro(IXImageDevice);
#endif // IC_RTTI_ENABLED

/*================================================================
||
||      Constructors
||
*/



IXImageDevice::IXImageDevice(IGImage* aliasImage, IPresSpaceHandle hdcMem):
	IXDevice(hdcMem, IGImage::imageBounds(*aliasImage)),
    fImage(aliasImage)
{}

// these three functions are disabled...
IXImageDevice::IXImageDevice()
{
// shouldn't be called
}

IXImageDevice::IXImageDevice(const IXImageDevice&)
{
// shouldn't be called
}

IXImageDevice& IXImageDevice::operator=(const IXImageDevice& Src )
{
// shouldn't be called
   return *this;
}


/*================================================================
||
||  IXImageDevice::~IXImageDevice
||
*/

IXImageDevice::~IXImageDevice ()
{
}

void IXImageDevice::renderLine(const IGLine2D& geometry,
		const IGrafState& grafState)
{
	IXDevice::renderLine( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState)
{
	IXDevice::renderPolyline( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderRect(const IGRect2D& geometry, const IGrafState& grafState)
{
	IXDevice::renderRect( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState)
{
	IXDevice::renderEllipse( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState) {
	IXDevice::renderPolygon( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderArea(const IGArea& geometry, const IGrafState& grafState) {
	IXDevice::renderArea( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderCurve(const IGCurve2D& geometry, const IGrafState& grafState) {
	IXDevice::renderCurve( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderLoop(const IGLoop2D& geometry, const IGrafState& grafState) {
	IXDevice::renderLoop( geometry, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderImage(
	const IGImage& image,
	const IGRect2D& source,
	const IGrafState& grafState)
{
	IXDevice::renderImage( image, source, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

void IXImageDevice::renderMaskedImage(
	const IGImage& image,
	const IGImage& mask,
	const IGRect2D& source,
	const IGrafState& grafState)
{
	IXDevice::renderMaskedImage( image, mask, source, grafState );
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
}

IGPoint2D IXImageDevice::renderTextRun(const IGTextRun& geometry, const IGrafState& grafState) {
	IGPoint2D pnt  = IXDevice::renderTextRun( geometry, grafState);
	fImage->fBmpData->pixelBuffer()->setXImageSyncFromPixmap(true);
	fImage->setImageSyncFlag(true);
	return pnt;
}
