/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 10 1.5.2.4 source/albert/graph2d/ximdev.hpp, 2d, ioc.v400, 000919 
/*

        name:   IXImageDevice.h
        type:   C++

        This is the definition of the IXImageDevice class.

Change History:
        Add support to Unix/X-Window environment (Xiangmin Zhang).
*/

#ifndef _XIMDEV_
#define _XIMDEV_

#include <ilanglvl.hpp>

#ifndef IC_XDEVICE
#include <xdevice.hpp>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGImage;

class IXImageDevice: public IXDevice
{
public:
    IXImageDevice(IGImage* aliasImage, IPresSpaceHandle hdcMem);
    virtual ~IXImageDevice ();
    IXImageDevice(IGImage* aliasImage, IWindowHandle windowHandle);
    virtual void renderLine(const IGLine2D& geometry, const IGrafState& grafState);

    virtual void renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState);

    virtual void renderRect(const IGRect2D& geometry, const IGrafState& grafState);

    virtual void renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState);

    virtual void renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState);

    virtual void renderArea(const IGArea& geometry, const IGrafState& grafState);

    virtual void renderCurve(const IGCurve2D& geometry, const IGrafState& grafState);

    virtual void renderLoop(const IGLoop2D& geometry, const IGrafState& grafState);

    virtual void renderImage(
        const IGImage& image,
                const IGRect2D& source,
                const IGrafState& grafState);

        virtual void renderMaskedImage(
                const IGImage& image,
                const IGImage& mask,
                const IGRect2D& source,
                const IGrafState& grafState);

        virtual IGPoint2D renderTextRun(const IGTextRun& geometry, const IGrafState& grafState);

#if !IC_RTTI_ENABLED
        GrafDeviceTypeDeclarationsMacro(IXImageDevice);
#endif // IC_RTTI_ENABLED

private:
    IGImage* fImage;

    IXImageDevice();
    IXImageDevice(const IXImageDevice&);
    IXImageDevice& operator=(const IXImageDevice& Src);
};

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _XIMDEV_

