// Revision: 15 1.2.1.4 source/core/attrib/iatname.cpp, attributes, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: iatname.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the classes:                                             *
*     IAttributeName                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/


#include <iatname.hpp>
#include <string.h>



#ifdef IC_PAGETUNE
    #define _ATNAME_CPP_
    #include <ipagetun.h>
#endif

IAttributeName::IAttributeName(const char* str) :
        fData(new IPrivateToken(str))
{ }

IAttributeName::IAttributeName(const IAttributeName& rhs) :
        fData(rhs.fData)
{
        ++(fData->fRefCount);
}

IAttributeName::~IAttributeName()
{
    if(--(fData->fRefCount) == 0)
                delete fData;
}


IAttributeName&    IAttributeName::operator=(const IAttributeName& rhs)
{
    if(this != &rhs) {
            if(--(fData->fRefCount) == 0)
                        delete fData;

                fData = rhs.fData;
                ++(fData->fRefCount);
    }

   return *this;
}


bool IAttributeName::operator==(const IAttributeName& rhs) const
{
        // do a quick pointer comparison
    if(fData == rhs.fData)
        return true;

        // do a quick comparison of hash values
    if(this->hash() != rhs.hash())
        return false;

        // do a full string comparison
        return( strcmp(getString(), rhs.getString()) == 0 );
}




/////////////////////////////////////////////////////////////
// Internal class for reference counting our string data.
//
// note: since we dont allow the internal string data to
// be modified we dont have to worry about providing
// an operator[].

IAttributeName::IPrivateToken::IPrivateToken(const char* s) :
        fRefCount(1)
{
    fString = new char[strlen(s) + 1];
    strcpy(fString, s);

    // calculate the hash value.
    unsigned long x = 0;
    for(; *s; ++s) {
        x = (x << 1) ^ (*s);
    }
    fHashValue = x;
}

IAttributeName::IPrivateToken::~IPrivateToken()
{
        delete[] fString;
}

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

