/*NOSHIP*/

// ----------------------------------------------------------------------------
// FILE NAME: iasynty.hpp                                                      
//                                                                             
// DESCRIPTION:                                                                
//   Declaration of the classes:                                               
//     INotificationProcessor                                                  
//     INotificationRequest                                                    
//                                                                             
// COPYRIGHT:                                                                  
//   IBM Open Class Library                                                    
//   (C) Copyright International Business Machines Corporation 1992, 1997      
//   Licensed Material - Program-Property of IBM - All Rights Reserved.        
//   US Government Users Restricted Rights - Use, duplication, or disclosure   
//   restricted by GSA ADP Schedule Contract with IBM Corp.                    
//                                                                             
// Revision: 75 1.8.1.3 source/core/base/iasynty.hpp, notification, ioc.v400, 000919  
// ----------------------------------------------------------------------------

#ifndef _IASYNTY_
#define _IASYNTY_

#include <ibase.hpp>
#include <iobservr.hpp>
#include <ireqproc.hpp>
#include <ingthrd.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

class INotificationProcessor : public IRequestQueue
{
public:
    INotificationProcessor();
    INotificationProcessor(IThreadId tid);
    ~INotificationProcessor();

    IRequestHandler* handlerAlias();

    //Reference counting
    virtual unsigned long addRef();
    virtual unsigned long removeRef();
    unsigned long refCount();

    bool isRunning() const;
    INotificationProcessor& setIsRunning(bool running);

    // Key
    const IThreadId& threadId() const;

private:
    INotificationProcessor(const INotificationProcessor&);
    INotificationProcessor&operator=( const INotificationProcessor&);

    IRequestHandler* fRequestHandler;
    IThreadId fThreadId;
    unsigned long fUsageCount;
    bool fIsRunning;
};


class INotificationRequest : public IRequest 
{
public:
    INotificationRequest(IObserver& theObserver, const INotificationEvent& anEvent);
    virtual ~INotificationRequest();

    virtual void Do();

private:
    INotificationRequest(const INotificationRequest&);
    INotificationRequest& operator=(const INotificationRequest&);

    INotificationEvent* fEvent;
    IObserver& fObserver;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
