//-----------------------------------------------------------------------------
// FILE NAME: ibhandle.cpp
//
// DESCRIPTION:
//   This file contains the implementation of classes/functions declared
//   in ibhandle.hpp.
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//
//   5645-001
//   (C) Copyright IBM Corporation 1992, 1997
//
//   US Government Users Restricted Rights - Use, duplication, or
//   disclosure restricted by GSA ADP Schedule Contract with IBM
//   Corp.
//
// Revision: 79 1.6.1.4 source/core/base/ibhandle.cpp, appsupport, ioc.v400, 000919 
// ----------------------------------------------------------------------------
extern "C"
{
#ifdef IC_POSIX
#include <pthread.h>
#endif

#ifdef IC_PMWIN
#include <malloc.h>
#endif
}

#include <ibhandle.hpp>
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
    #define _IBHANDLE_CPP_
    #include <ipagetun.h>
#endif

IString IHandle :: asString ( ) const
{
  return IString( (unsigned long)this->handle );
}

IString IHandle :: asDebugInfo ( ) const
{
  IString result( "IHandle(" );
  result += "handle=";
  result += this->asString();
  result += ")";
  return result;
}



/*------------------------------------------------------------------------------
| IThreadHandle::noHandle - Used in IThread constructor to represent the       |
|                           default value for the threadHandle parameter,      |
|                           which indicates that the thread is already         |
|                           accessible to the Open Class Library or is the     |
|                           current thread.                                    |

------------------------------------------------------------------------------*/
#ifdef IC_WIN
const IThreadHandle IThreadHandle::noHandle (0);
#endif

IString IThreadId::asString ( ) const
{
// #ifdef IC_POSIX
//   if (fSystemThreadId == 0 )
//       return IString( (unsigned long) 0 );
//   else
//       return IString::c2x(
//              IString( fSystemThreadId, sizeof(*fSystemThreadId) ) );
// #else
  return IString( asUnsigned() );
// #endif


}

IString IThreadId :: asDebugInfo ( ) const
{
  IString result( "IThreadId(" );
  result += this->asString();
  result += ")";
  return result;
}

unsigned long IThreadId::asUnsigned ( ) const
{
// #ifdef IC_POSIX
//      return  this->fSystemThreadId ?
//              (unsigned long) *(this->fSystemThreadId) : 0 ;
// #else
     return ( (unsigned long) this->fSystemThreadId );
// #endif
}


bool operator  ==   (const IThreadId  &threadId1,
                               const IThreadId  &threadId2)
{

   if ( threadId1.fSystemThreadId == 0 &&
          threadId2.fSystemThreadId == 0 ) return true;

   if ( threadId1.fSystemThreadId != 0 &&
          threadId2.fSystemThreadId != 0 )
   {
 #ifdef IC_POSIX
    return pthread_equal( (pthread_t)(threadId1.fSystemThreadId) ,
                          (pthread_t)(threadId2.fSystemThreadId) );
 #else
   return ( threadId1.fSystemThreadId == threadId2.fSystemThreadId);
 #endif
   } else {  // if one of the handle value is 0
      return false;
   }

}

bool  operator  !=   (const IThreadId  &threadId1,
                                const IThreadId  &threadId2)
{
   return !( threadId1 == threadId2 );
}


IThreadId& IThreadId::operator=( const IThreadId& tid )
{
  fSystemThreadId  = tid.fSystemThreadId;
  return *this;
}


bool  IThreadId::isValid( ) const
{
     return ( (this->fSystemThreadId) ? true : false);
}


IString IProcessId :: asString ( ) const
{
  return IString( (unsigned long)this->handle );
}

IString IProcessId :: asDebugInfo ( ) const
{
  IString result( "IProcessId(" );
  result += this->asString();
  result += ")";
  return result;
}


IString IEnumHandle :: asString ( ) const
{
  return IString( (unsigned long)this->handle );
}

IString IEnumHandle :: asDebugInfo ( ) const
{
  IString result( "IEnumHandle(" );
  result += this->asString();
  result += ")";
  return result;
}


IString ISharedSemaphoreHandle :: asString ( ) const
{
  return IString( (unsigned long)this->handle );
}

IString ISharedSemaphoreHandle :: asDebugInfo ( ) const
{
  IString result( "ISharedSemaphoreHandle(" );
  result += this->asString();
  result += ")";
  return result;
}


#ifdef IC_POSIX
IPrivateSemaphoreHandle&
 IPrivateSemaphoreHandle::operator=
                          (  void *privSemId )
{
#ifdef ASC_DEBUG
  printf ("Entering IPrivateSemaphoreHandle Oper= Value ... \n");
#endif // ASC_DEBUG

  fSystemPrivSemId  = (pthread_t *)privSemId;

  return *this;
}
#endif  //IC_POSIX


IString IPrivateSemaphoreHandle :: asString ( ) const
{

#ifdef IC_POSIX
  if (fSystemPrivSemId == 0 )
      return IString( (unsigned long) 0 );
  else
      return IString::c2x(
//              IString( fSystemPrivSemId, sizeof(*fSystemPrivSemId) ) );
             IString( fSystemPrivSemId, sizeof(fSystemPrivSemId) ) );
#else
  return IString( asUnsigned() );
#endif

}

IString IPrivateSemaphoreHandle :: asDebugInfo ( ) const
{
  IString result( "IPrivateSemaphoreHandle(" );
  result += this->asString();
  result += ")";
  return result;
}

// #ifdef IC_PMWIN
unsigned long IPrivateSemaphoreHandle :: asUnsigned ( ) const
{
  return ( (unsigned long)this->fSystemPrivSemId );
}
// #endif


IThreadId :: operator unsigned long  ( ) const
{
    return ( this->fSystemThreadId);
}

/*------------------------------------------------------------------------------
| IAnchorBlockHandle::asString                                                 |
------------------------------------------------------------------------------*/
IString IAnchorBlockHandle::asString ( ) const
{
  return IString( this->asUnsigned() );
}

/*------------------------------------------------------------------------------
| IAnchorBlockHandle::asDebugInfo                                              |
------------------------------------------------------------------------------*/
IString IAnchorBlockHandle::asDebugInfo ( ) const
{
  IString result( "IAnchorBlockHandle(" );
  result += ",handle=";
  result += this->asString();
  result += ")";
  return result;
}


/*------------------------------------------------------------------------------
| IModuleHandle::asString                                                      |
------------------------------------------------------------------------------*/
IString IModuleHandle::asString ( ) const
{
  return IString( this->asUnsigned() );
}

/*------------------------------------------------------------------------------
| IModuleHandle::asDebugInfo                                                   |
------------------------------------------------------------------------------*/
IString IModuleHandle::asDebugInfo ( ) const
{
  IString result( "IModuleHandle(" );
  result += ",handle=";
  result += this->asString();
  result += ")";
  return result;
}
