#ifndef _IDLLMOD_
#define _IDLLMOD_
/*******************************************************************************
* FILE NAME: idllmod.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declarations of the classes:                                               *
*     IDLLModule                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*                                                                              *
*******************************************************************************/
// Revision: 69  1.4.1.5  source/core/base/idllmod.hpp, appsupport, ioc.v400, 000919  

#include <ibase.hpp>
#include <ibasepl.hpp>
#include <ibhandle.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IDLLModulePrivateData;
class IRefCountedDLLImp;

#pragma enum(4)
#pragma pack(push,4)

class IC_EXPORTB IDLLModule
{
public:
enum ESearchLocation { 
   kOSDefaultSearch, 
   kNLSPathSearch 
};
/*------------------------------- Constructors -------------------------------*/
  IDLLModule   ( const char*      fileName,
                 ESearchLocation  searchOrder = kOSDefaultSearch );

  IDLLModule   ( const IDLLModule& dllLibrary );

#ifdef IC_PMWIN
  IDLLModule   ( const IModuleHandle& moduleHandle );
#endif

 ~IDLLModule   ( );

IDLLModule
 &operator=    ( const IDLLModule& dllLibrary );

/*-------------------------- Opening and Closing -----------------------------*/
IDLLModule
 &open                ( ),
 &close               ( );

/*----------------------- Procedure Address Loading --------------------------*/
void
 *procAddress       ( const char*   procedureName ) const;
#ifdef IC_PMWIN
void
 *procAddress       ( unsigned long procedureOrdinal ) const;
#endif

#ifdef IC_PMWIN
bool
  isEntryPoint32Bit ( const char*   procedureName ) const,
  isEntryPoint32Bit ( unsigned long procedureOrdinal ) const;
#endif

/*--------------------- Dynamic Link Library Information ---------------------*/
bool
  isOpen            ( ) const;

IString
  fileName          ( ) const;

IModuleHandle
  handle            ( ) const;

private:
/*-------------------------------- Private -----------------------------------*/
static unsigned long
  addRef          ( const IModuleHandle& moduleHandle ),
  removeRef       ( const IModuleHandle& moduleHandle );

IDLLModulePrivateData
 *fPrivateData;
IRefCountedDLLImp
 *fDLLImp;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IDLLMOD_
