/* NOSHIP */
// ----------------------------------------------------------------------------
// FILE NAME: idllstat.hpp
//
// DESCRIPTION:
//   Declarations of the classes:
//     IDLLStaticPtr
//
// COPYRIGHT:
//   IBM Open Class Library
//   (C) Copyright International Business Machines Corporation 1992, 1997
//   Licensed Material - Program-Property of IBM - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication, or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// Revision: 71 1.9.1.3 source/core/base/idllstat.hpp, appsupport, ioc.v400, 000919 
// ----------------------------------------------------------------------------

#ifndef _IDLLSTAT_
#define _IDLLSTAT_

#include <ibase.hpp>
#include <iset2.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

// Forward references
class IDLLSet;
class IPrivateResource;
class IRefCountedDLLImp;
class IModuleHandle;


#pragma enum(4)
#pragma pack(push,4)

class IDLLStaticPtr 
{
public:
    // ------------------------------------------------------------------------
    //  Destructor
    // ------------------------------------------------------------------------
                                ~IDLLStaticPtr();

    // ------------------------------------------------------------------------
    //  Accessor methods
    // ------------------------------------------------------------------------
    static IDLLSet&             dllSet();
    static IPrivateResource&    dllKey();

private:
    // ------------------------------------------------------------------------
    //  Static data
    // ------------------------------------------------------------------------
    static IDLLSet*             fgDLLSet;
    static IPrivateResource*    fgDLLKey;
};


class IDLLSet 
{
public:
    // ------------------------------------------------------------------------
    //  Constructors and destructors
    // ------------------------------------------------------------------------
                                IDLLSet();
                                ~IDLLSet();

    // ------------------------------------------------------------------------
    //  Search and ref counting methods
    // ------------------------------------------------------------------------
    IRefCountedDLLImp*          find(const IRefCountedDLLImp* dllImp);
    IRefCountedDLLImp*          find(const IModuleHandle& moduleHandle);

    void                        add(IRefCountedDLLImp* dllImp);
    void                        remove(IRefCountedDLLImp* dllImp);

private:
    // ------------------------------------------------------------------------
    //  Private data
    // ------------------------------------------------------------------------
    IVPtrSet<IRefCountedDLLImp*>    fDLLSet;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
