// Revision: 11 1.4.1.4 source/core/base/iexcept.cpp, exception, ioc.v400, 000919 
/*----------------------------------------------------------------------------*/
/* FILE NAME: iexcept.cpp                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of classes/functions declared      */
/*   in iexcept.hpp.                                                          */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"IEXCEPT")
    #pragma csect(static,"SEXCEPT")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif

#include <ibase.hpp>
#include <ilanglvl.hpp>

extern "C"
{
#ifndef IC_400
#ifdef IC_PM
   #define INCL_WINERRORS
   #define INCL_DOSFILEMGR
   #define INCL_DOSMISC
   #define INCL_WINWINDOWMGR
   #include <os2.h>
#endif
#ifdef IC_WIN
  #include <windows.h>
#endif
#ifdef IC_MOTIF
    #include <X11/Intrinsic.h>
#endif
#endif
  #include <string.h>
  #include <errno.h>
}

#include <iexcept.hpp>
#include <icconsta.h>

#ifdef IC_MOTIF
  #include <ingthrd.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IEXCEPT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBaseErrorInfo::IBaseErrorInfo                                               |
------------------------------------------------------------------------------*/
IBaseErrorInfo::IBaseErrorInfo ( )
{ }

/*------------------------------------------------------------------------------
| IBaseErrorInfo::~IBaseErrorInfo                                              |
------------------------------------------------------------------------------*/
IBaseErrorInfo::~IBaseErrorInfo ( )
{ }

/*------------------------------------------------------------------------------
| IBaseErrorInfo::throwError                                                   |
------------------------------------------------------------------------------*/
void IBaseErrorInfo :: throwError ( const IExceptionLocation&  location,
                                ExceptionType              name,
                                IException::Severity       severity,
                                IException::ErrorCodeGroup errorGroup )
{
  switch (name)
  {
    case accessError:
    {
      IAccessError exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case deviceError:
    {
      IDeviceError exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidParameter:
    {
      IInvalidParameter exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidRequest:
    {
      IInvalidRequest exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfSystemResource:
    {
      IOutOfSystemResource exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfWindowResource:
    {
      IOutOfWindowResource exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfMemory:
    {
      IOutOfMemory exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case resourceExhausted:
    {
      IResourceExhausted exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

#if ( defined(IC_PM) || defined (IC_WIN))
/*------------------------------------------------------------------------------
| IGUIErrorInfo::IGUIErrorInfo                                                 |
------------------------------------------------------------------------------*/
IGUIErrorInfo::IGUIErrorInfo ( const char *pGUIFunctionName )
{
  char *pErrText;

#ifdef IC_PM
  HAB habTemp = WinQueryAnchorBlock( HWND_DESKTOP );
  PERRINFO perri = WinGetErrorInfo( habTemp );
  this->perrinfoCl = (void*)perri;
  if ((perri != 0)  &&  (perri->cDetailLevel > 0UL))
  {
    unsigned long *pul = (unsigned long*)((char*)perri+ (perri->offaoffszMsg));
    pErrText = ((char*)perri + *pul);
  }
  else
    pErrText = "No error text is available.";
#endif
#ifdef IC_WIN
  this->ulClErrorId = GetLastError();
  char szText[512];
  /********************************************************************/
  /* Format the message from the system message table resources       */
  /********************************************************************/
  unsigned long ulMessageLength = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, this->ulClErrorId, GetSystemDefaultLangID(), (LPTSTR)szText, 255, NULL);
  if( ulMessageLength)
  {
     /********************************************************************/
     /* Remove CRLF control characters if found                          */
     /********************************************************************/
     if ((szText[ulMessageLength-2] == '\r')  &&
         (szText[ulMessageLength-1] == '\n'))
     {
       szText[ulMessageLength-2] = 0;
     }
     pErrText = szText;
  }
  else
     pErrText = "No error text is available.";
#endif


  if (pGUIFunctionName)
  {
    this->pClFuncNameText = new char[strlen(pGUIFunctionName) +
                                     strlen(pErrText)+2];
    strcpy(this->pClFuncNameText, pGUIFunctionName);
    strcat(this->pClFuncNameText, ":");
    strcat(this->pClFuncNameText, pErrText);
  }
  else
  {
    this->pClFuncNameText = new char[strlen(pErrText)+1];
    strcpy(this->pClFuncNameText, pErrText);
  }
}

#endif
/* for other platforms  */
#if defined(IC_MVS) || defined(IC_400) || defined(IC_MOTIF) || defined(IC_POSIX)

/*------------------------------------------------------------------------------
| IGUIErrorInfo::IGUIErrorInfo                                                 |
|                                                                              |
| This class does not really have meaning for other platforms. A default       |
| message will be used for instantiations of this class.                       |
------------------------------------------------------------------------------*/
IGUIErrorInfo::IGUIErrorInfo ( const char* GUIFunctionName ) :
    pClFuncNameText(0)

  {
  IMessageText errInfo(IC_GUI_ERROR);

  if (GUIFunctionName)
    {
    pClFuncNameText = new char[strlen(GUIFunctionName) +
      strlen( errInfo.text()) + 2];
    strcpy(pClFuncNameText, GUIFunctionName);
    strcat( pClFuncNameText, ":" );
    }
  else
    {
    pClFuncNameText = new char[ strlen(errInfo.text()) + 1];
    strcpy(pClFuncNameText, "");
    }

  strcat( pClFuncNameText, errInfo.text() );
  }
#endif


/*------------------------------------------------------------------------------
| IGUIErrorInfo::~IGUIErrorInfo                                                |
------------------------------------------------------------------------------*/
IGUIErrorInfo::~IGUIErrorInfo ( )
{
#ifdef IC_PM
  if (this->perrinfoCl)
    WinFreeErrorInfo((PERRINFO)this->perrinfoCl);
#endif

  if (this->pClFuncNameText)
    delete [] this->pClFuncNameText;
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::text                                                          |
------------------------------------------------------------------------------*/
const char* IGUIErrorInfo::text ( ) const
{
  return( this->pClFuncNameText );
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::errorId                                                       |
------------------------------------------------------------------------------*/
unsigned long IGUIErrorInfo::errorId ( ) const
{
#ifdef IC_PM
  PERRINFO perri = (PERRINFO)this->perrinfoCl;
  if (perri)
    return( (unsigned short)perri->idError );
  else
    return( 0 );
#endif
#ifdef IC_WIN
  return( this->ulClErrorId );
#endif

// For all other platforms
    return( 0 );

}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::operator const char*                                          |
------------------------------------------------------------------------------*/
IGUIErrorInfo::operator const char* () const
{
  return( this->pClFuncNameText );
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::isAvailable                                                   |
------------------------------------------------------------------------------*/
bool IGUIErrorInfo::isAvailable ( ) const
{
#ifdef IC_PM
  PERRINFO perri = (PERRINFO)this->perrinfoCl;
  if (perri == 0)
    return( 0 );
  else
  {
    if (perri->idError != 0)
      return( 1 );
    else
      return( 0 );
  }
#endif
#ifdef IC_WIN
  if (this->ulClErrorId != 0)
    return( 1 );
  else
    return( 0 );
#endif

// For all other platforms
    return( 0 );

}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::throwGUIError                                                 |
------------------------------------------------------------------------------*/
void IGUIErrorInfo :: throwGUIError ( const char*               pFunctionName,
                                      const IExceptionLocation  &location,
                                      IBaseErrorInfo::ExceptionType name,
                                      IException::Severity      severity )
{
  IGUIErrorInfo errInfo(pFunctionName);
  errInfo.throwError(location, name, severity, IException::presentationSystem );
}

#if ( defined(IC_PM) || defined (IC_WIN) )
/*------------------------------------------------------------------------------
| ISystemErrorInfo::ISystemErrorInfo                                           |
------------------------------------------------------------------------------*/
ISystemErrorInfo::ISystemErrorInfo ( unsigned long ulSystemErrorId,
                                     const char    *pSystemFunctionName )
                  : ulClErrorId(ulSystemErrorId),
                    bClError(true)
{
  char szText[512];
  unsigned long ulRc;
#ifdef IC_PM
  unsigned long ulBootDrive;                               /* QSV_BOOT_DRIVE */
  DosQuerySysInfo( 5, 5, &ulBootDrive, sizeof(ulBootDrive) );
  unsigned char ucA = 'A';
  unsigned char ucBootDrive = (unsigned char)(ucA + (ulBootDrive - 1));
  char szFileName[80];
  szFileName[0] = ucBootDrive;
  szFileName[1] = '\0';
  strcat(szFileName, "\\OS2\\SYSTEM\\OSO001.MSG");
  unsigned long ulMessageLength;
  ulRc = DosGetMessage( 0, 0, (PCHAR)szText, 255,
                        ulSystemErrorId, (PSZ)szFileName,
                        &ulMessageLength );
#endif
#ifdef IC_WIN
  /********************************************************************/
  /* Format the message from the system message table resources       */
  /********************************************************************/
  unsigned long ulMessageLength = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                                                0,
                                                ulSystemErrorId,
                                                GetSystemDefaultLangID(),
                                                (LPTSTR)szText,
                                                255,
                                                NULL);

  /********************************************************************/
  /* Handle message not found condition                               */
  /********************************************************************/
  if (ulMessageLength)
     ulRc = 0;
  else
     ulRc = 1;
#endif

  if (ulRc)
  {
    this->bClError = false;
    ulMessageLength = 27;
    strcpy(szText, "No error text is available.");
  }

  /********************************************************************/
  /* Remove CRLF control characters if found                          */
  /********************************************************************/
  if ((szText[ulMessageLength-2] == '\r')  &&
      (szText[ulMessageLength-1] == '\n'))
  {
    ulMessageLength = ulMessageLength - 2;
  }

  if (pSystemFunctionName)
  {
    this->pClErrorText = new char[ulMessageLength +
                                  strlen(pSystemFunctionName) + 2];
    strcpy(this->pClErrorText, pSystemFunctionName);
    strcat(this->pClErrorText, ":");
    strncat(this->pClErrorText, szText, ulMessageLength);
  }
  else
  {
    this->pClErrorText = new char[ulMessageLength + 1];
    strncpy(this->pClErrorText, szText, ulMessageLength);
    this->pClErrorText[ulMessageLength] = '\0';
  }
}

#endif
/* for other platforms  */
#if defined(IC_MVS) || defined(IC_400) || defined(IC_MOTIF) || defined(IC_POSIX)

/*------------------------------------------------------------------------------
| ISystemErrorInfo::ISystemErrorInfo                                           |
|                                                                              |
| This class does not really have meaning for other platforms. A default       |
| message will be used for instantiations of this class.                       |
------------------------------------------------------------------------------*/
ISystemErrorInfo::ISystemErrorInfo ( unsigned long systemErrorId,
                                     const char* systemFunctionName ) :
    pClErrorText(0)

  {
  IMessageText errInfo(IC_SYSTEM_ERROR);
  ulClErrorId = systemErrorId;
  bClError = true;

  if (systemFunctionName)
    {
    pClErrorText = new char[strlen(systemFunctionName) +
      strlen( errInfo.text()) + 2];
    strcpy(pClErrorText, systemFunctionName);
    strcat( pClErrorText, ":" );
    }
  else
    {
    pClErrorText = new char[ strlen(errInfo.text()) + 1];
    strcpy(pClErrorText, "");
    }

  strcat( pClErrorText, errInfo.text() );
  }
#endif

/*------------------------------------------------------------------------------
| ISystemErrorInfo::~ISystemErrorInfo                                          |
------------------------------------------------------------------------------*/
ISystemErrorInfo::~ISystemErrorInfo ( )
{
  if (this->pClErrorText)
    delete [] this->pClErrorText;
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::text                                                       |
------------------------------------------------------------------------------*/
const char* ISystemErrorInfo::text ( ) const
{
  return( (const char*)this->pClErrorText );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::operator const char*                                       |
------------------------------------------------------------------------------*/
ISystemErrorInfo::operator const char* () const
{
  return( text() );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::errorId                                                    |
------------------------------------------------------------------------------*/
unsigned long ISystemErrorInfo::errorId ( ) const
{
  return( this->ulClErrorId );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::isAvailable                                                |
------------------------------------------------------------------------------*/
bool ISystemErrorInfo::isAvailable ( ) const
{
  if (!this->bClError)
    return( 0 );

  return( 1 );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::throwSystemError                                           |
------------------------------------------------------------------------------*/
void ISystemErrorInfo ::
                 throwSystemError ( unsigned long             ulSystemErrorId,
                                    const char                *pFunctionName,
                                    const IExceptionLocation  &location,
                                    IBaseErrorInfo::ExceptionType name,
                                    IException::Severity      severity )
{
  ISystemErrorInfo errInfo(ulSystemErrorId, pFunctionName);

  switch (name)
  {
    case accessError:
    {
      IAccessError exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case deviceError:
    {
      IDeviceError exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidParameter:
    {
      IInvalidParameter exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidRequest:
    {
      IInvalidRequest exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfSystemResource:
    {
      IOutOfSystemResource exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfWindowResource:
    {
      IOutOfWindowResource exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfMemory:
    {
      IOutOfMemory exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case resourceExhausted:
    {
      IResourceExhausted exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::~IClibErrorInfo                                              |
------------------------------------------------------------------------------*/
ICLibErrorInfo::~ICLibErrorInfo ( )
{
  if (this->pClErrorText)
    delete [] this->pClErrorText;
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::text                                                         |
------------------------------------------------------------------------------*/
const char* ICLibErrorInfo::text ( ) const
{
  return( (const char*)this->pClErrorText );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::operator const char*                                         |
------------------------------------------------------------------------------*/
ICLibErrorInfo::operator const char* () const
{
  return( text() );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::errorId                                                      |
------------------------------------------------------------------------------*/
unsigned long ICLibErrorInfo::errorId ( ) const
{
  return( this->ulClErrno );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::isAvailable                                                  |
------------------------------------------------------------------------------*/
bool ICLibErrorInfo::isAvailable ( ) const
{
  return( this->bClError );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::ICLibErrorInfo                                               |
|                                                                              |
|  The strerror C function is used to retrieve the error text for errno.       |
------------------------------------------------------------------------------*/
ICLibErrorInfo::ICLibErrorInfo ( const char *pCLibFunctionName )
{
#ifdef IC_POSIX
  char pErrorStr[512];
  strerror_r(errno, pErrorStr, 512);
#else
  char *pErrorStr = strerror(errno);
#endif
  char noErrorText[] = "No Error text is available";

  this->ulClErrno = (unsigned long) errno;

  this->bClError = strlen(pErrorStr) ? true : false;

  if (pCLibFunctionName)
  {
    if (this->bClError)
    {
      this->pClErrorText = new char[strlen(pErrorStr) +
                                    strlen(pCLibFunctionName) + 2];
      strcpy(this->pClErrorText, pCLibFunctionName);
      strcat(this->pClErrorText, ":");
      strcat(this->pClErrorText, pErrorStr);
    }
    else
    {
      this->pClErrorText = new char[strlen(noErrorText) +
                                    strlen(pCLibFunctionName) + 2];
      strcpy(this->pClErrorText, pCLibFunctionName);
      strcat(this->pClErrorText, ":");
      strcat(this->pClErrorText, noErrorText);
    }
  }
  else
  {
    if (this->bClError)
    {
      this->pClErrorText = new char[strlen(pErrorStr) + 1];
      strcpy(this->pClErrorText, pErrorStr);
    }
    else
    {
      this->pClErrorText = new char[strlen(noErrorText) + 1];
      strcpy(this->pClErrorText, noErrorText);
    }
  }
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::throwCLibError                                               |
------------------------------------------------------------------------------*/
void ICLibErrorInfo::throwCLibError( const char                *pFunctionName,
                                     const IExceptionLocation  &location,
                                     IBaseErrorInfo::ExceptionType name,
                                     IException::Severity      severity )
{
  ICLibErrorInfo clibError( pFunctionName );
  clibError.throwError( location, name, severity, IException::CLibrary );
}


#if ( defined(IC_MOTIF) || defined(IC_MVS) )
/*------------------------------------------------------------------------------
| IXLibErrorInfo::IXLibErrorInfo                                               |
------------------------------------------------------------------------------*/
IXLibErrorInfo::IXLibErrorInfo ( const char* XLibFunctionName ) :
    pClErrorText(0)

  {
  char NoErrorText[] = "No error text is available.";  // Move to msg file
  char errorText[256] = "";  // X needs a size for this buffer


#ifndef IC_NONGUI
#if defined(IC_MOTIF) &&  !defined(__NO_ITHREAD__)

#ifdef IC_NOTYET
  bClInfoAvail = INonGUIThread::current().isXerrorCodeAvailable();
  if (bClInfoAvail)
    {
    XGetErrorText( XtDisplay( INonGUIThread::current().appShell() ),
                   INonGUIThread::current().XerrorCode(), errorText, 256);
    }
#endif

#endif    // IC_MOTIF && !__NO_ITHREAD__
#endif  // IC_NONGUI

  if (strlen(errorText) == 0 )
    strcpy(errorText, NoErrorText);

  if (XLibFunctionName)
    {
    pClErrorText = new char[strlen(XLibFunctionName) + strlen(errorText) + 2];
    strcpy(pClErrorText, XLibFunctionName);
    strcat(pClErrorText, ":");
    strcat(pClErrorText, errorText);
    }
  else
    {
    pClErrorText = new char[strlen(errorText) + 1];
    strcpy(pClErrorText, errorText);
    }
  }

#if IC_EXTENDED_RTTI_ENABLED
/*------------------------------------------------------------------------------
| IXLibErrorInfo::IXLibErrorInfo                                               |
------------------------------------------------------------------------------*/
IXLibErrorInfo::IXLibErrorInfo ( const IXLibErrorInfo& )

  {
  }
#endif // IC_EXTENDED_RTTI_ENABLED

/*------------------------------------------------------------------------------
| IXLibErrorInfo::~IXLibErrorInfo                                              |
------------------------------------------------------------------------------*/
IXLibErrorInfo::~IXLibErrorInfo ( )

  {
  if (pClErrorText)
    delete [] pClErrorText;
  }

/*------------------------------------------------------------------------------
| IXLibErrorInfo::throwXLibError                                               |
------------------------------------------------------------------------------*/
void IXLibErrorInfo :: throwXLibError(
                           const char*               functionName,
                           const IExceptionLocation& location,
                           IBaseErrorInfo::ExceptionType name,
                           IException::Severity      severity )

  {
  IXLibErrorInfo xlibError(functionName);
  xlibError.throwError( location, name, severity);
  }

/*------------------------------------------------------------------------------
| IXLibErrorInfo::errorId                                                      |
------------------------------------------------------------------------------*/
unsigned long IXLibErrorInfo::errorId ( ) const

  {
  return ulClErrorCode;
  }

/*------------------------------------------------------------------------------
| IXLibErrorInfo::text                                                         |
------------------------------------------------------------------------------*/
const char* IXLibErrorInfo::text ( ) const

  {
  return (const char*) pClErrorText;
  }

/*------------------------------------------------------------------------------
| IXLibErrorInfo::operator const char*                                         |
------------------------------------------------------------------------------*/
IXLibErrorInfo::operator const char* () const

  {
  return text();
  }


/*------------------------------------------------------------------------------
| IXLibErrorInfo::isAvailable                                                  |
------------------------------------------------------------------------------*/
bool IXLibErrorInfo::isAvailable ( ) const

  {
  return bClInfoAvail;
  }
#endif /* IC_MOTIF */


/*------------------------------------------------------------------------------
| IExcept__assertParameter                                                     |
------------------------------------------------------------------------------*/
void IExcept__assertParameter ( const char               *pAssertExpression,
                                const IExceptionLocation &location )
{
  IInvalidParameter invParm(IMessageText(IC_ASSERTPARM, IMessageText::messageFile(),
                                               pAssertExpression),
                            IC_ASSERTPARM,
                            IException::recoverable);
  invParm.setErrorCodeGroup(IException::other);
  invParm.addLocation(location);
  invParm.logExceptionData();

  #if (INO_EXCEPTIONS_SUPPORT)
    invParm.terminate();
  #else
    throw invParm;
  #endif
}

/*------------------------------------------------------------------------------
| IExcept__assertState                                                         |
------------------------------------------------------------------------------*/
void IExcept__assertState ( const char               *pAssertExpression,
                            const IExceptionLocation &location )
{
  IInvalidRequest invReq(IMessageText(IC_ASSERTPARM, IMessageText::messageFile(),
                                            pAssertExpression),
                         IC_ASSERTPARM,
                         IException::recoverable);
  invReq.setErrorCodeGroup(IException::other);
  invReq.addLocation(location);
  invReq.logExceptionData();

  #if (INO_EXCEPTIONS_SUPPORT)
    invReq.terminate();
  #else
    throw invReq;
  #endif
}

/*------------------------------------------------------------------------------
| IExcept__throwLibraryError                                                   |
------------------------------------------------------------------------------*/
void IExcept__throwLibraryError ( unsigned long             ulMessageId,
                                  const IExceptionLocation  &location,
                                  IBaseErrorInfo::ExceptionType name,
                                  IException::Severity      severity,
                                  const char                *pText )
{
  IMessageText errInfo(ulMessageId, IMessageText::messageFile(), pText);

  switch (name)
  {
    case IBaseErrorInfo::accessError:
    {
      IAccessError exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::deviceError:
    {
      IDeviceError exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::invalidParameter:
    {
      IInvalidParameter exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::invalidRequest:
    {
      IInvalidRequest exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::outOfSystemResource:
    {
      IOutOfSystemResource exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::outOfWindowResource:
    {
      IOutOfWindowResource exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::outOfMemory:
    {
      IOutOfMemory exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::resourceExhausted:
    {
      IResourceExhausted exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

/*------------------------------------------------------------------------------
| IExcept__throwError                                                          |
------------------------------------------------------------------------------*/
void IExcept__throwError ( unsigned long              messageId,
                           const IExceptionLocation&  location,
                           const char*                messageFile,
                           IException::ErrorCodeGroup errorGroup,
                           IBaseErrorInfo::ExceptionType  name,
                           IException::Severity       severity,
                           const char*                substitutionText )
{
  IMessageText errInfo = IMessageText( messageId,
                                       messageFile,
                                       substitutionText );

  switch (name)
  {
    case IBaseErrorInfo::accessError:
    {
      IAccessError exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::deviceError:
    {
      IDeviceError exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::invalidParameter:
    {
      IInvalidParameter exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::invalidRequest:
    {
      IInvalidRequest exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::outOfSystemResource:
    {
      IOutOfSystemResource exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::outOfWindowResource:
    {
      IOutOfWindowResource exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::outOfMemory:
    {
      IOutOfMemory exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IBaseErrorInfo::resourceExhausted:
    {
      IResourceExhausted exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}
