// ----------------------------------------------------------------------------
// FILE NAME: iextproc.inl
//
// DESCRIPTION:
//   This file contains the definition of the inline functions for the
//   classes declared in iextproc.hpp.
//
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//   (C) IBM Corporation 1992, 1997.
//
// Revision: 32 1.1 source/core/base/iextproc.inl, oss, ioc.v400, 000919 
// ----------------------------------------------------------------------------

#ifndef _IEXTPROC_INL_
#define _IEXTPROC_INL_

inline IPathName IExtProcImplBase::getPath() const
{
    return fPath;
}

inline IProcessId  IExtProcImplBase::getId() const
{
    return fId;
}

inline void IExtProcImplBase::setId(const IProcessId& newId)
{
    fId = newId;
}

//
//  All of these inlines are just delegating the request or action to
//  the implementation object, so they will effectively be optimized
//  away in the end.
//
inline IPathName IExternalProcess::getPath() const
{
    return fImpl->getPath();
}

inline IProcessId IExternalProcess::getId() const
{
    return fImpl->getId();
}

inline INonGUIApplication::PriorityClass IExternalProcess::getPriority() const
{
    return fImpl->getPriority();
}

inline bool IExternalProcess::exitStatus(unsigned long& status) const
{
    return fImpl->exitStatus(status);
}

inline bool IExternalProcess::isRunning() const
{
    return fImpl->isRunning();
}

inline void IExternalProcess::setPath(const IPathName& newName)
{
    fImpl->setPath(newName);
}

inline void IExternalProcess::setEnvironment(const IEnvironment& newEnv)
{
    fImpl->setEnvironment(newEnv);
}

inline bool
IExternalProcess::setPriority(INonGUIApplication::PriorityClass priority)
{
    return fImpl->setPriority(priority);
}

inline void IExternalProcess::kill()
{
    fImpl->kill();
}

inline bool IExternalProcess::wait(const long timeout) const
{
    return fImpl->wait(timeout);
}


//
//  These are just translation methods that call other methods, in order
//  to support a previous interface. They will probably be removed in the
//  future.
//
inline bool IExternalProcess::isStarted() const
{
    return this->isRunning();
}

inline void IExternalProcess::stop()
{
    this->kill();
}

inline IProcessId IExternalProcess::id() const
{
    return this->getId();
}


#endif // _IEXTPROC_INL_
