// ----------------------------------------------------------------------------
// FILE NAME: imsgtext.cpp
//
// DESCRIPTION:
//   This file contains the implementation of classes/functions declared
//   in imsgtext.hpp.
//
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//
//   5645-001
//   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.
//
//   US Government Users Restricted Rights - Use, duplication, or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
// Revision: 22 1.6.1.8 source/core/base/imsgtext.cpp, appsupport, ioc.v400, 000919 
// ----------------------------------------------------------------------------

#include <ibase.hpp>
#include <imsgtext.hpp>
#include <ireslock.hpp>
#include <iprimlck.hpp>
#include <imsgtext.h>
#include <string.h>

#ifdef IC_MVS
#include <fstream.h>
#include <stdlib.h>
#endif

#ifdef IC_400
#include <stdio.h>
//
// Replace sys/sysopapi.h with correct header files; sysopapi was not
// part of the supported AS/400 interface
//
#include <qmhrtvm.h>
#include <qmh.h>
#include <qtes.h>
#endif

extern "C"
{
#include <string.h>

#ifdef IC_PM
#define INCL_DOSMISC
#include <os2.h>
#endif
    
#ifdef IC_MOTIF
#include <limits.h>
#include <nl_types.h>
#endif
    
#ifdef IC_WIN
#include <windows.h>
#endif
}

// Segment definitions
#ifdef IC_PAGETUNE
#define _IMSGTEXT_CPP_
#include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Statics & functions related to message file handling                         |
------------------------------------------------------------------------------*/
static const char default_text[] = "Unable to load text from message file.";
const int IMessageTextDefaultSize = (sizeof(IMessageTextDefaultMessages)/sizeof(char *));
char *IMessageText::iocMessageFile = 0;

static IPrivateResource& msgFileNameLock();
static const char *getMessageFilename();
static char *getMessageFromDefault(int messageId, char *insertTable[], 
                                   unsigned long tableEntries);
static char *replaceVars(const char* varText, char *insertTable[],
                         unsigned long tableEntries);

#ifdef IC_MVS
static const char *defaultFile = "ibmcl.cat";
#endif

#ifdef IC_MOTIF
/* kMsgFileName is defined in the build process */
static const char defaultFile[] = kMsgFileName;
#endif

#ifdef IC_400
// must keep it as 10 character long for OS400
static const char defaultFile[] = "QCPPMSGF  ";
#endif

#if defined(IC_PM) || defined(IC_WIN)
#ifndef __JUNO__
static const char defaultFile[] = OPENCLASS_MESSAGE_LIBRARY_NAME_2(ocmsg,oa);
#else
static const char defaultFile[] = "CPPWOC3U.MSG";
#endif
#endif

char *
getMessageFromDefault(int messageId, char *insertTable[],
                      unsigned long tableEntries)
{
    // Assuming that we're in IOC space, load message text from
    // a static array of strings
    if (messageId > 0 && messageId < IMessageTextDefaultSize) {
        if (IMessageTextDefaultMessages[messageId] == NULL) {
            return NULL;
        } else {
            return replaceVars(IMessageTextDefaultMessages[messageId], 
                               insertTable, tableEntries);
        }
    }
    return NULL;
}

const char *
getMessageFilename()
{
    const char *pFileName = getenv("ICLUI MSGFILE");
    if (! pFileName)
        pFileName = getenv("ICLUI_MSGFILE");
    if (! pFileName)
        pFileName = defaultFile;
    return pFileName;
}

IPrivateResource& 
msgFileNameLock()
{
    static IPrivateResource *msgFileLock = 0;
    if (!msgFileLock) {
        IPrimalLock lockInit;
        if (!msgFileLock)
            msgFileLock = new IPrivateResource;
    }
    return *msgFileLock;
}

char *
IMessageText::messageFile()
{
    IResourceLock aLock(msgFileNameLock());
    if (!IMessageText::iocMessageFile) {
        IMessageText::setMessageFile(getMessageFilename());
    }
    return iocMessageFile;
}

void
IMessageText::setMessageFile(const char *msgFileName)
{
    IResourceLock aLock(msgFileNameLock());
    if (msgFileName) {
        if (IMessageText::iocMessageFile)
            delete [] IMessageText::iocMessageFile;
        IMessageText::iocMessageFile = new char[strlen(msgFileName) + 1];
        strcpy(IMessageText::iocMessageFile, msgFileName);
    }
}

IMessageText::IMessageText(unsigned long messageId,
                           const char*   messageFileName,
                           const char*   textInsert1,
                           const char*   textInsert2,
                           const char*   textInsert3,
                           const char*   textInsert4,
                           const char*   textInsert5,
                           const char*   textInsert6,
                           const char*   textInsert7,
                           const char*   textInsert8,
                           const char*   textInsert9) :
        bClSuccess(0), pClText(0)
{
    IFUNCTRACE_DEVELOP();
#ifndef IC_PMWIN
    if (messageFileName) {
#endif
    unsigned long ulCount = 0;
    char* insertTable[9] = {NULL};
    
    // Build variable text insertion table
    if (textInsert1) {
        ulCount++;
        insertTable[0] = (char*)textInsert1;
        if (textInsert2) {
            ulCount++;
            insertTable[1] = (char*)textInsert2;
            if (textInsert3) {
                ulCount++;
                insertTable[2] = (char*)textInsert3;
                if (textInsert4) {
                    ulCount++;
                    insertTable[3] = (char*)textInsert4;
                    if (textInsert5) {
                        ulCount++;
                        insertTable[4] = (char*)textInsert5;
                        if (textInsert6) {
                            ulCount++;
                            insertTable[5] = (char*)textInsert6;
                            if (textInsert7) {
                                ulCount++;
                                insertTable[6] = (char*)textInsert7;
                                if (textInsert8) {
                                    ulCount++;
                                    insertTable[7] = (char*)textInsert8;
                                    if (textInsert9) {
                                        ulCount++;
                                        insertTable[8] = (char*)textInsert9;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    unsigned long ulMessageLength;

#ifdef IC_MVS
#define IMSGTEXT 256
    char messageText[IMSGTEXT];
    char *szText = 0;
    bClSuccess = readMessageFromFile(messageText,messageFileName,
                                     messageId);
    if (bClSuccess) {
        szText = replaceVars(messageText, insertTable, ulCount);
        ulMessageLength = strlen (szText);
    } 
#endif

#ifdef IC_400
#define MSGDTASIZE    512
#define MSGINFOLENGTH 1024
#define MSGIDSIZE 7
// Max. message file name size, not including library specification
#define BASEMSGFNAMESIZE  10
#define MSGFSIZE 20
#define LIBRARYLIST "*LIBL     "
#define ANSWERNO    "*NO       "
#define ANSWERYES   "*YES      "

    // construct the substitution parameters
    char msgdta[MSGDTASIZE + 1];
    char msgf[MSGFSIZE + 1];
    char msginfo[MSGINFOLENGTH]; // limit to retrieving 1024 bytes
    char msgid[MSGIDSIZE + 1];
    // Replaced error_code_t with _TE_ERROR_CODE_T
    _TE_ERROR_CODE_T  rtverr;
    
    // Added IMSGTEXT_RTVM0100_t type definition; same as Qmh_Rtvm_RTVM0100_t
    // (defined in AS/400 system header file sys/qmhrtvm.h) with 'Message'
    // member uncommented out. Size of 'Message' array is specified as '1';
    // array is of variable length and actual size is unknown.
    // This struct definition is required now that the 'sys/sysopapi'
    // header file no longer exists; pmsginfo used to be defined as a
    // pointer to RTVM0100_t, a type that is identical to Qmh_Rtvm_RTVM0100_t,
    // except member names are different and member 'Message' is not commented
    // out.
    
    typedef _Packed struct
    {
        int  Bytes_Return;
        int  Bytes_Available;
        int  Length_Message_Returned;
        int  Length_Message_Available;
        int  Length_Help_Returned;
        int  Length_Help_Available;
        char Message[1];
        /*char Message_Help[];*/    /* Varying length                 */
    } IMSGTEXT_RTVM0100_t;
    
    // Replaced RTVM0100_t with IMSGTEXT_RTVM0100_t
    IMSGTEXT_RTVM0100_t    *pmsginfo = (IMSGTEXT_RTVM0100_t *)msginfo;
    // Member mesg_text of RTVM0100_t renamed to Message
    char *szText = pmsginfo->Message;
    msgdta[0] = '\0';
    for (unsigned long i = 0, totlength = 0;
         i < ulCount && totlength < MSGDTASIZE ; i++ ) { // build sub. data
        totlength += strlen(insertTable[i]);
        if (totlength < MSGDTASIZE) {
            strcat(msgdta, insertTable[i]);
        } /* endif */
    } /* endfor */
    
    // If message file name supplied by user is less than minimum
    // of 3 characters long or more than 10 chars long, then fail
    size_t msgfNameLength = strlen(messageFileName);
    
    if ( msgfNameLength < 3 || msgfNameLength > 10 )
        bClSuccess = 0;
    else
    {
        // Build 10 character base message file name, padding with
        // blanks as necessary
        char msgfName[BASEMSGFNAMESIZE+1] = "          ";
        memcpy(msgfName, messageFileName, msgfNameLength);
        
        // ASC itself may be specifying QCPPMSGF (the correct message
        // file for ASC - QYPPMSGF appears to have been a typo).
        // Existing users may be specifying QYPPMSGF and expecting to
        // get a default message prefix of 'TPP'. In both these cases,
        // message file will be QYPPMSGF and prefix will be 'TPP'.
        
        if (!strncmp(msgfName,"QCPPMSGF  ",msgfNameLength) ||
            !strncmp(msgfName,"QYPPMSGF  ",msgfNameLength)) {
            strcpy(msgf, "QYPPMSGF  ");
            sprintf(msgid, "TPP%04d", messageId);
        } else {
            strcpy(msgf, msgfName);
          // Prefix is first 3 characters of message file name
          sprintf(msgid, "%3.3s%04d", msgf, messageId);
        }

        // Message identifier must be no longer than 7 characters
        if ( strlen(msgid) > 7  )
            bClSuccess = 0;
        else
        {
            // Arguments OK - call API
            strcat(msgf, LIBRARYLIST);
            // Replaced byte_provided with BytesProvided
            rtverr.BytesProvided = sizeof(rtverr);
            // Added prefix QMH_ to FMT_RTVM0100
            QMHRTVM(msginfo, MSGINFOLENGTH, QMH_FMT_RTVM0100, msgid,
                    msgf, msgdta, strlen(msgdta), ANSWERYES, ANSWERNO,
                    &rtverr);
            // Replaced byte_available with BytesAvailable
            bClSuccess = !rtverr.BytesAvailable;
            // Replaced len_msg_returned with Length_Message_Returned
            ulMessageLength = pmsginfo->Length_Message_Returned;
            
        } // else after checking if incorrect msgid length
    } // else after checking if incorrect msgfile name length
#endif
    
#ifdef IC_PM
#define IMSGTEXTLEN 2048
    char szText[IMSGTEXTLEN];
    this->bClSuccess = (DosGetMessage(insertTable, ulCount, (PCHAR)szText,
                                     (unsigned long)IMSGTEXTLEN, messageId,
                                     (PSZ)messageFileName,
                                     &ulMessageLength) == 0);
#endif

#ifdef IC_MOTIF
    char *szText = 0;
    char *szVarText = 0;

    nl_catd catalog = catopen((char*)messageFileName, NL_CAT_LOCALE);
    if (catalog == (nl_catd)-1)
        catalog = catopen((char*)messageFileName, 0);

    if (catalog != (nl_catd)-1) {
        szVarText = catgets(catalog, 1, messageId, NULL);
        bClSuccess = szVarText != 0;
        if (bClSuccess) {
            szText = replaceVars(szVarText, insertTable, ulCount);
            ulMessageLength = strlen(szText);
        }
        catclose(catalog);
    }
#endif

#ifdef IC_WIN
#define IMSGTEXTLEN 2048
    char szText[IMSGTEXTLEN];
    this->bClSuccess = 0;
    HMODULE hMsgLibrary = GetModuleHandle( 0 );
    bool bFreeLib = False;
    if (messageFileName) {
        hMsgLibrary = LoadLibrary(messageFileName);
        bFreeLib = True;
    }
    if (hMsgLibrary) {
        /**************************************************************/
        /* Format the message from the supplied msg DLL               */
        /**************************************************************/
        ulMessageLength = FormatMessage(FORMAT_MESSAGE_FROM_HMODULE |
                                        FORMAT_MESSAGE_ARGUMENT_ARRAY,
                                        hMsgLibrary,
                                        messageId,
                                        0, // 26568 GetSystemDefaultLangID(),
                                        (LPTSTR)szText,
                                        IMSGTEXTLEN,
                                        insertTable);
        if (ulMessageLength)
            this->bClSuccess = 1;
        
        if (bFreeLib)
            FreeLibrary(hMsgLibrary);
    }
#endif

    if (bClSuccess) {
        pClText = new char[ulMessageLength + 1];
        if ((szText[ulMessageLength-2] == '\r') &&
            (szText[ulMessageLength-1] == '\n'))
            // get rid of CRLF
            ulMessageLength = ulMessageLength - 2;
        strncpy(pClText, szText, ulMessageLength);
        pClText[ulMessageLength] = '\0';
    } else {
        // Do not set bClSuccess to 1 because this is still considered failure
        // to load message from message file.
        char *tmp_msg = getMessageFromDefault(messageId, insertTable, ulCount);
        if (tmp_msg) {
            ITRACE_DEVELOP(IString("Using Open Class internal message: ") +
                           IString(tmp_msg));
            pClText = new char[strlen(tmp_msg) + 1];
            strcpy(pClText, tmp_msg);
            delete [] tmp_msg;
        }
    }


#if defined(IC_MVS) || defined(IC_MOTIF)
    if (szText)
        delete [] szText;
#endif

#ifndef IC_PMWIN
    }   //   if (messageFileName) ....
#endif

    // This is just the original design. In any case, we shouldn't be
    // falling into the block below unless users are not using our ICLUI messages
    // and fail to provide the message for his/her own message Id.

    if (! pClText) { //   load default text if message text load failed
        pClText = new char[sizeof(default_text)];
        strcpy(pClText, default_text);
    }
}

IMessageText::IMessageText(const IMessageText& text)
{
    pClText = 0;
    bClSuccess = text.bClSuccess;
    if (text.pClText) {
        pClText = new char[strlen(text.pClText) + 1];
        strcpy(pClText, text.pClText);
    }
}

IMessageText&
IMessageText::operator=(const IMessageText& text)
{
    
    bClSuccess = text.bClSuccess;
    if (pClText) {
        delete [] pClText;
        pClText = 0;
    }
    if (text.pClText) {
        pClText = new char[strlen(text.pClText) + 1];
        strcpy(pClText, text.pClText);
    }
    return *this;
}

IMessageText::~IMessageText()
{
    if (pClText)
        delete [] pClText;
}

IMessageText::operator const char* () const
{
    return text();
}

int IMessageText::successful() const
{
    return bClSuccess;
}

const char* IMessageText::text() const
{
    return pClText;
}

IMessageText&
IMessageText::setDefaultText(const char* defaultText)
{
    if (defaultText && !bClSuccess) {
        // only set if we couldn't load text from message file
        if (pClText)
            delete [] pClText;
        pClText = new char[strlen(defaultText) + 1];
        strcpy(pClText, defaultText);
    }
    return *this;
}

char *
replaceVars(const char *varText, char *insertTable[],
            unsigned long tableEntries)
{ 
    char* replText = 0;

    if (varText == NULL)
        return NULL;

    if (varText[0] == 0) {
        replText = new char (1);
        replText[0] = 0;
    } else {
        unsigned replTextLen = 0;
        unsigned to = 0;
        do {
            unsigned var = 0;
            unsigned from = to;
            
            for (; varText [to] != 0; to++) {
                if (varText [to] == '%') {
                    if (varText [to+1] == '%')
                        to++;
                    else if ('1' <= varText [to+1] && varText [to+1] <= '9') {
                        var = varText [to+1] - '0';
                        if (var <= tableEntries)
                            break;
                        else
                            var = 0;
                    }
                }
            }
            // to is index of '\0' or '%'
            
            char* oldReplText = replText;
            
            replTextLen += to - from;
            if (var != 0)
                replTextLen += strlen (insertTable [var-1]);
            
            replText = new char [replTextLen + 1];
            
            if (oldReplText) {
                strcpy (replText, oldReplText);
                delete [] oldReplText;
            }
            else
                replText [0] = 0;
            
            strncat (replText, &varText [from], to - from);
            
            if (var != 0) {
                strcat (replText, insertTable [var-1]);
                to += 2;
            }
            
        } while (varText [to] != 0);
        
    }
    return replText;
}

#if 0         // Bypass pick LE Message handling module

#ifdef IC_MVS

int
IMessageText :: readMessageFromFile(char*           messageText,
                                    const char*     messageFileName,
                                    unsigned long   messageId)
{
    static const unsigned int lprefix  =    3;
    static const unsigned int lmsgnum  =    4;
    static const unsigned int lsuffix  =    3;
    static const unsigned int ltext    =  256;
    static const unsigned int lmsgbuf  =  266;
    
    union Line
    {
        struct
        {
            char prefix[lprefix];
            char msgnum[lmsgnum];
            char suffix[lsuffix];
            char text[ltext];
        } record;
        
        char msgbuf[lmsgbuf];
    };
    
    Line line;
    unsigned int llength;
    
    ifstream messageFile(messageFileName);
    
    while (messageFile)
    {
        ws(messageFile).get(line.msgbuf, lmsgbuf, '\n');
        if (atol(line.record.msgnum) == messageId)
        {
            llength = strlen(line.record.text);
            strcpy(messageText, line.record.text);
            messageText[llength] = '\0';
            return 1;
        }
    } /* endwhile */
    
    return 0;
}

#endif
#endif
