// Revision: 26 1.8.1.3 source/core/base/ingapp.cpp, oss, ioc.v400, 000919 
/*----------------------------------------------------------------------------*/
/* FILE NAME: ingapp.cpp                                                       */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   Implementation of the class(es):                                         */
/*     INonGUIApplication                                                     */
/*     ICurrentNonGUIApplication                                              */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"INGAPP")
    #pragma csect(static,"SNGAPP")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif


#include <ibase.hpp>
#include <ingapp.hpp>

extern "C" {
#ifdef IC_POSIX
  #include <unistd.h>
#else
#ifdef IC_PMWIN
  #define INCL_DOSPROCESS
  #define INCL_DOSERRORS
#ifdef IC_WIN
  #include <windows.h>
#endif
#ifdef IC_PM
  #include <os2.h>
#endif
#endif // IC_PMWIN

  #include <stdlib.h>
#endif // else IC_POSIX
}

#include <ingthrd.hpp>
#include <istring.hpp>
#include <iexcept.hpp>

#ifdef IC_WIN
  #include <iplatfrm.hpp>
#endif  // IC_WIN

#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _INGAPP_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
  Static objects:
    IApplication::pCurrent - initially zero.
------------------------------------------------------------------------------*/
ICurrentNonGUIApplication
 *INonGUIApplication::pCurrent = 0;


// The following are added to stop the compiler statically
// generating them.
INonGUIApplication :: ~INonGUIApplication ( ) {}
ICurrentNonGUIApplication :: ~ICurrentNonGUIApplication ( )
{
  /* delete pCurrentApplicationData when it is defined. */
}


/*------------------------------------------------------------------------------
  asString - Returns "INonGUIApplication(id)".
------------------------------------------------------------------------------*/
IString INonGUIApplication :: asString ( ) const
{
  return "INonGUIApplication(" + IString( this->pid.asString() ) + ")";
}

/*------------------------------------------------------------------------------
  asDebugInfo - Returns "INonGUIApplication(id=xx)".
------------------------------------------------------------------------------*/
IString INonGUIApplication :: asDebugInfo ( ) const
{
  IString result( "INonGUIApplication(" );
  result += "id=";
  result += IString( this->pid.asString() );
  result += ")";
  return( result );
}

/*------------------------------------------------------------------------------
   id - return the pid data member.
------------------------------------------------------------------------------*/
IProcessId INonGUIApplication :: id ( ) const
{
  return( pid );
}

/*------------------------------------------------------------------------------
  setId - set the pid data member to the argument value.
------------------------------------------------------------------------------*/
INonGUIApplication& INonGUIApplication :: setId ( const IProcessId& id )
{
  this->pid = id;
  return( *this );
}

/*----------------------------------------------------------------------------
  setProcessPriority

              - In Motif, this function calls setPriority for the current
                (and only) thread.  Since threads are not implemented
                in AIX 3.2, the INonGUIThread::setPriority() member function
                will do nothing.
----------------------------------------------------------------------------*/
INonGUIApplication&
    INonGUIApplication::setProcessPriority(const EProcessPriority priority)
{
    IFUNCTRACE_DEVELOP();

#ifdef IC_PM
    unsigned long newClass;
    switch ( priority )
    {
    case belowNormal:
        newClass = PRTYC_IDLETIME;
        break;
    case normal:
        newClass = PRTYC_REGULAR;
        break;
    case aboveNormal:
        newClass = PRTYC_TIMECRITICAL;
        break;
    }

    //
    // We don't support any priority levels for priorities under the portable
    // API, so they all go at the 0 level within the new class.
    //
    unsigned long rc = DosSetPriority(PRTYS_PROCESS, newClass, 0, this->id());

    if (rc != NO_ERROR)
    {
      ITHROWSYSTEMERROR( rc,
                         "DosSetPriority",
                         IBaseErrorInfo::accessError,
                         IException::recoverable );
    }
#endif //IC_PM

#ifdef IC_WIN
    int newClass;
    switch ( priority )
    {
    case belowNormal:
      newClass = IDLE_PRIORITY_CLASS;
      break;
    case normal:
      newClass = NORMAL_PRIORITY_CLASS;
      break;
    case aboveNormal:
      newClass = HIGH_PRIORITY_CLASS;
      break;
    }

    if (!SetPriorityClass( GetCurrentProcess(), newClass))
    {
        ITHROWGUIERROR2("SetPriorityClass",
                        IBaseErrorInfo::accessError,
                        IException::recoverable );
    }
#endif //IC_WIN

#ifdef IC_MOTIF

    //
    //  NEED TO FIGURE OUT GOOD NUMBERS HERE! Right now the thread prio
    //  stuff does nothing on Posix, but might soon!
    //
    //  <TBD>
    //
    int level;
    switch ( priority )
    {
    case idleTime:
      level = 0;
      break;
    case regular:
      level = 16;
      break;
    case timeCritical:
      level = 31;
      break;
    }

  //INonGUIThread::current().setPriority(level);
#endif // IC_MOTIF

  return( *this );
}


/*----------------------------------------------------------------------------
 currentPID -  Return current process's id
----------------------------------------------------------------------------*/
IProcessId INonGUIApplication :: currentPID ( )
{
#ifdef IC_PM
  PPIB ppib;
  PTIB ptib;
  DosGetInfoBlocks( &ptib, &ppib );
  return( IProcessId( ppib->pib_ulpid ) );
#endif //IC_PM

#ifdef IC_WIN
  return( IProcessId( (void*)GetCurrentProcessId() ) );
#endif //IC_WIN

#ifdef IC_MOTIF
  return IProcessId( getpid() );
#endif // IC_MOTIF

#ifdef IC_POSIX
  return( IProcessId( getpid() ) );
#endif // IC_POSIX
}


/*------------------------------------------------------------------------------
  Construct current application with pib/pid set for the current process.
------------------------------------------------------------------------------*/
ICurrentNonGUIApplication :: ICurrentNonGUIApplication ( )
  : INonGUIApplication ( 0 )
  , argCount     ( 0 )
  , argValue     ( 0 )
  , pCurrentApplicationData ( 0 )
#ifdef IC_PM
  , pPIB         ( 0 )
#endif
{
  IFUNCTRACE_DEVELOP();

#ifdef IC_PM
  this->setId( this->pib().pib_ulpid );
#endif //IC_PM

#ifdef IC_WIN
  this->setId( this->currentPID() );
#endif //IC_WIN

#ifdef IC_MOTIF
  this->setId( getpid() );
#endif // IC_MOTIF

#ifdef  IC_MVS400
#ifdef IC_POSIX
  this->setId( getpid() );
#endif // IC_POSIX
#endif  // IC_MVS400
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
  pib - If not set previously, get pointer to PIB for this process.  It is
  "cached" in the ICurrentApplication object.
------------------------------------------------------------------------------*/
struct pib_s& ICurrentNonGUIApplication :: pib ( )
{
  if ( !this->pPIB )
  {
    PTIB ptib;
    DosGetInfoBlocks( &ptib, &this->pPIB );
  }

  return( *this->pPIB );
}
#endif //IC_PM


/*------------------------------------------------------------------------------
  argc - Return arg count data member.
------------------------------------------------------------------------------*/
int ICurrentNonGUIApplication :: argc ( ) const
{
  return( this->argCount );
}

/*------------------------------------------------------------------------------
  argv - Return argument array data member.
------------------------------------------------------------------------------*/
IString ICurrentNonGUIApplication :: argv ( int argNo ) const
{
  IString result;
  if (argNo < argCount)
    result = this->argValue[ argNo ];
  return( result );
}

/*------------------------------------------------------------------------------
  setArgs - If current data member isn't empty, delete old array of args.
  Allocate new array data member and initialize each element from input
  char* array.
------------------------------------------------------------------------------*/
ICurrentNonGUIApplication&
ICurrentNonGUIApplication::setArgs ( int argc, const char* const argv[] )
{
  if (this->argCount)
  {
    // free the old array
    delete [] this->argValue;
  }
  if (argc)
  {
    // Allocate new array of pointers and allocate buffers for args.
    this->argValue = new IString[ argc ];
    this->argCount = 0;
    while (this->argCount < argc)
    {
      this->argValue[ this->argCount ] = argv[ this->argCount ];
      this->argCount++;
    }
  }

  return( *this );
}

/*------------------------------------------------------------------------------
  exit - Exit current thread (which will exit application in a single-threaded
  app).
------------------------------------------------------------------------------*/
ICurrentNonGUIApplication& ICurrentNonGUIApplication::exit()
{
#ifndef __RRBC_LIB__   // INonGUIThread is not supported in SOM Environment
  INonGUIThread::current().exit(0);
#else
  ::exit(0);
#endif  // else ! __RRBC_LIB__
  return( *this );
}

/*------------------------------------------------------------------------------
  current - return pointer to current application.
------------------------------------------------------------------------------*/
ICurrentNonGUIApplication& INonGUIApplication :: current ( )
{
  if ( ! INonGUIApplication::pCurrent )
    INonGUIApplication::pCurrent = new ICurrentNonGUIApplication;

  return( *INonGUIApplication::pCurrent );
}

/*------------------------------------------------------------------------------
  Constructor, which only initializes the pid.
------------------------------------------------------------------------------*/
INonGUIApplication :: INonGUIApplication ( const IProcessId& id )
  : pid( id )
{
  IFUNCTRACE_DEVELOP();
}

/*------------------------------------------------------------------------------
  asDebugInfo - Generate string with current application contents.
------------------------------------------------------------------------------*/
IString ICurrentNonGUIApplication :: asDebugInfo ( ) const
{
  IString result( "ICurrentApplication(" );
  result += this->Inherited::asDebugInfo();
  result += ",argCount=" + IString( this->argCount );
  result += ",argValue=" + IString( (unsigned long) this->argValue );
  result += "->[";
  IString comma;
  for ( int n = 0; n < this->argCount; n++ )
  {
    result += comma;
    result += this->argValue[ n ].asDebugInfo();
    comma = ",";
  }

  result += "]";
#ifdef IC_PM
  result += ",pPIB=" + IString( (unsigned long) this->pPIB );
#endif //IC_PM
  result += ")";
  return( result );
}



#ifdef IC_PM

/*------------------------------------------------------------------------------
  Deprecated priority methods. These are only available under OS/2 for
  backwards compatibility.
------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  setPriority - Issue DosSetPriority using specified arguments.
----------------------------------------------------------------------------*/
INonGUIApplication&
INonGUIApplication::setPriority(PriorityClass priorityClass,
                                long          priorityLevel,
                                bool          setDescendents)
{
    IFUNCTRACE_DEVELOP();

    if ( priorityClass == noChange )
        priorityClass = INonGUIThread::current().priorityClass();

    unsigned long newClass;
    switch ( priorityClass )
    {
    case idleTime:
        newClass = PRTYC_IDLETIME;
        break;
    case regular:
        newClass = PRTYC_REGULAR;
        break;
    case timeCritical:
        newClass = PRTYC_TIMECRITICAL;
        break;
    case foregroundServer:
        newClass = PRTYC_FOREGROUNDSERVER;
        break;
  }

    unsigned long pscope = ( setDescendents ) ? PRTYS_PROCESSTREE : PRTYS_PROCESS,
    rc = DosSetPriority( pscope, newClass, priorityLevel, this->id() );

    if (rc != NO_ERROR)
    {
        ITHROWSYSTEMERROR( rc,
                           "DosSetPriority",
                           IBaseErrorInfo::accessError,
                           IException::recoverable );
    }
    return *this;
}


/*----------------------------------------------------------------------------
  adjustPriority - Issue DosSetPriority to adjust the priority level.
----------------------------------------------------------------------------*/
INonGUIApplication&
INonGUIApplication::adjustPriority(long adjustment, bool setDescendents)
{
    IFUNCTRACE_DEVELOP();

    unsigned long pscope = ( setDescendents ) ? PRTYS_PROCESSTREE : PRTYS_PROCESS,
    rc = DosSetPriority( pscope, PRTYC_NOCHANGE, adjustment, this->id() );
    if (rc != NO_ERROR)
    {
        ITHROWSYSTEMERROR(  rc,
                            "DosSetPriority",
                            IBaseErrorInfo::accessError,
                            IException::recoverable );
    }
    return *this;
}

#endif
