// Revision: 33 1.11.1.9 source/core/base/ingthrds.cpp, oss, ioc.v400, 000919 
/*----------------------------------------------------------------------------*/
/* FILE NAME: ingthrds.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of certain INonGUIThread member    */
/*   functions, removing the dependency on beginthread for single-threaded    */
/*   applications.                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997                                 */
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM              */
/*   Corp.                                                                    */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"INGTHRDS")
    #pragma csect(static,"SNGTHRDS")
    #pragma comment (copyright, \
    "5645-001 (C) Copyright IBM Corp. 1992, 1997.")
#endif

// If the compiler were to support:
//    #pragma options /Gm+
// then we wouldn't need the declaration of _beginthread below.
// The declaration is lifted from stdlib.h, because it won't get
// generated unless /Gm+ has been set for the compile.
// If the declaration changes in stdlib.h, then we have to update here.

//         INonGUIThread is not supported in MVS SOM Environment
#if !defined(__MVS__) || (defined(__MVS__) && !defined(__RRBC_LIB__))

// pthread.h should be the first one to be included in any source file.
#ifdef IC_POSIX
extern "C" 
{
#include <pthread.h>
}
#endif

#include <ibase.hpp>
#include <ingthrd.hpp>
#include <ingthrdp.hpp>

extern "C" {
#ifdef IC_PMWIN
    int  _Optlink _beginthread(void (* _Optlink)(void *), void *, 
                               unsigned, void *);
#endif // IC_PMWIN
#include <stdlib.h>
}

#include <iexcept.hpp>
#include <itrace.hpp>
#include <istring.hpp>

#ifdef IC_WIN
#include <iplatfrm.hpp>
#include <windows.h>
#endif  // IC_WIN

// Segment definitions.
#ifdef IC_PAGETUNE
#define _INGTHRDS_CPP_
#include <ipagetun.h>
#endif

class IStartedThread;

/*------------------------------------------------------------------------------
| initThread                                                                   |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
extern "C" void* __stdcall initThread(void* aThread);
#else
extern "C" void* initThread(void* aThread);
#endif


INonGUIThread::INonGUIThread (IThreadFn *pFnObj)
        : thread(INonGUIThread::newStartedThread())
{
    this->thread->addRef();
    this->start( pFnObj );
}

/*------------------------------------------------------------------------------
| INonGUIThread::INonGUIThread                                                 |
|                                                                              |
| INonGUIThread constructor for functions that use the _Optlink linkage        |
------------------------------------------------------------------------------*/
INonGUIThread::INonGUIThread(OptlinkFnPtr pfn, void* anArg)
        : thread(INonGUIThread::newStartedThread())
{
    this->thread->addRef();
    this->start( pfn, anArg );
}

/*------------------------------------------------------------------------------
| INonGUIThread::INonGUIThread                                                 |
|                                                                              |
| INonGUIThread constructor for functions that use _System linkage             |
------------------------------------------------------------------------------*/
INonGUIThread::INonGUIThread(SystemFnPtr pfn, unsigned long anArg)
  : thread(INonGUIThread::newStartedThread())
{
    this->thread->addRef();
    this->start( pfn, anArg );
}

///
/// INonGUIThread::start
///
/// All versions are basically implemented via the function that accepts
/// as argument a reference to an INonGUIThread::Function object.  The other
/// versions construct such an object from the other arguments.
///
/// The function is implemented by invoking _beginthread against the
/// static member function IStartedThread::initThread, passing as argument
/// the IStartedThread address.
///
/// The IStartedThread referenced by the receiver is added to the
/// static "started thread" array.
///
void INonGUIThread::start(IThreadFn *pFnObj)
{
    IFUNCTRACE_DEVELOP();
        
    IStartedThread* p = this->prepareToStart( pFnObj );
        
    ITRACE_DEVELOP( IString( "\tStarting thread id=" ) + (p->id).asString().d2x() +
                    IString(" p=") + IString((int)p).d2x());

#ifdef IC_PMWIN
#ifdef IC_PM
    int rc = _beginthread((void(*_Optlink)(void*))initThread,0,this->stackSize(),p);
    unsigned long th = rc;
#endif
#ifdef IC_WIN
    (IStartedThread::lock()).lock();  // Will cause the new thread to block
                                      //  if it tries to look itself up in
                                      //  the IStartedThreadList before we can
                                      //  stuff in the ID.
    unsigned long winThreadId;
    void* th = CreateThread(NULL, this->stackSize(), (LPTHREAD_START_ROUTINE)initThread,
                            p, 0, &winThreadId);
    int rc = (int)th;
#endif
#endif  // IC_PMWIN

#ifdef IC_POSIX
    unsigned long ulRC = 0;
    pthread_attr_t thread_attr;

    ulRC = pthread_attr_init(&thread_attr);
    if (ulRC != 0) {
#ifdef ASC_DEBUG
        printf("Error pthread_attr_init  retCode (%d) errno(%d) \n ",
               ulRC, errno );
#endif // ASC_DEBUG
        ITHROWCLIBERROR("pthread_attr_init", IBaseErrorInfo::accessError,
                        IException::recoverable);
    }

#ifdef __MVS__
#define  PTHREAD_CREATE_UNDETACHED     0
#endif //__MVS__

// On MVS/OE --  By default the Threads are  created in undetach state.
    ulRC = pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_UNDETACHED);
    if (ulRC != 0) {
#ifdef ASC_DEBUG
        printf("Error pthread_attr_setdetachstate  retCode (%d) errno(%d) \n",
               ulRC , errno );
#endif // ASC_DEBUG
        ITHROWCLIBERROR( "pthread_attr_setdetachstate", IBaseErrorInfo::accessError,
                         IException::recoverable );
    }

    if (pthread_attr_setstacksize(&thread_attr, this->stackSize()) != 0) {
        ITHROWCLIBERROR("pthread_attr_setstacksize", IBaseErrorInfo::accessError,
                        IException::recoverable);
    }

    pthread_t th;
    int rc = pthread_create(&th, &thread_attr, initThread, p);

#endif //IC_POSIX

#ifdef IC_WIN
    IThreadId thId(winThreadId);
#else
    IThreadId thId((unsigned long)th);   // build IThreadId  from system thread Id
#endif

    this->threadStarted(p, rc, thId);

#ifdef IC_WIN
    (IStartedThread::lock()).unlock();
#endif
}


/*------------------------------------------------------------------------------
| OptlinkFn class : An IThreadFn subclass to dispatch an optlink function      |
------------------------------------------------------------------------------*/
struct OptlinkFn : public IThreadFn 
{
    OptlinkFn(INonGUIThread::OptlinkFnPtr pfn, void* p);
    ~OptlinkFn();

void run()
#ifdef IC_MOTIF_AVAIL_NOW
        {
            function(arg);
        }
#else
    ;
#endif // else IC_MOTIF

    INonGUIThread::OptlinkFnPtr function;
    void *arg;

private:
    OptlinkFn(const OptlinkFn&) {}
    OptlinkFn &operator=(const OptlinkFn&);

};

/*------------------------------------------------------------------------------
| OptlinkFn::OptlinkFn                                                         |
------------------------------------------------------------------------------*/
OptlinkFn::OptlinkFn(INonGUIThread::OptlinkFnPtr pfn, void* p)
        : function(pfn), arg(p)
{
}

/*------------------------------------------------------------------------------
| OptlinkFn::~OptlinkFn                                                        |
------------------------------------------------------------------------------*/
OptlinkFn::~OptlinkFn()
{
}

/*------------------------------------------------------------------------------
| OptlinkFn::run                                                               |
------------------------------------------------------------------------------*/
void OptlinkFn::run()
{
    function(arg);
}

/*------------------------------------------------------------------------------
| INonGUIThread::start                                                         |
------------------------------------------------------------------------------*/
void INonGUIThread::start(OptlinkFnPtr  pfn,
                          void*         anArg)
{
  this->start(new OptlinkFn(pfn, anArg));
}

/*------------------------------------------------------------------------------
| SystemFn class : An IThreadFn subclass to dispatch a system function.        |
------------------------------------------------------------------------------*/
struct SystemFn : public IThreadFn 
{
    SystemFn(INonGUIThread::SystemFnPtr pfn, unsigned long p);
    ~SystemFn();
    
    void run ()
#ifdef IC_MOTIF_AVAIL_NOW
    { function( arg ); }
#else
    ;
#endif // else IC_MOTIF

    INonGUIThread::SystemFnPtr function;
    unsigned long arg;

#ifndef IC_MOTIF_AVAIL_NOW
    SystemFn (const SystemFn&) {}
    SystemFn &operator=(const SystemFn&);
#endif // !IC_MOTIF
};

/*------------------------------------------------------------------------------
| SystemFn::SystemFn                                                           |
------------------------------------------------------------------------------*/
SystemFn::SystemFn(INonGUIThread::SystemFnPtr pfn, unsigned long p)
  : function(pfn), arg(p)
{
}

/*------------------------------------------------------------------------------
| SystemFn::~SystemFn                                                          |
------------------------------------------------------------------------------*/
SystemFn::~SystemFn()
{
}

/*------------------------------------------------------------------------------
| SystemFn::run                                                                |
------------------------------------------------------------------------------*/
void SystemFn::run()
{
    function(arg);
}

/*------------------------------------------------------------------------------
| INonGUIThread::start                                                         |
------------------------------------------------------------------------------*/
void INonGUIThread::start(SystemFnPtr   pfn,
                          unsigned long anArg)
{
    this->start(new SystemFn(pfn, anArg));
}

#endif //!__MVS__ or (__MVS__ && !__RRBC_LIB__)  thread support NOT in MVS-SOM

