// ----------------------------------------------------------------------------
// FILE NAME: irect.cpp
//
// DESCRIPTION:
//   This file contains the implementation of classes/functions declared
//   in irect.hpp.
//
// COPYRIGHT:
//   IBM Open Class Library
//   (C) Copyright International Business Machines Corporation 1992, 1997
//   Licensed Material - Program-Property of IBM - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication, or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// Revision: 63 1.4.1.3 source/core/base/irect.cpp, appsupport, ioc.v400, 000919 
// ----------------------------------------------------------------------------

extern "C"
{
    #ifdef IC_WIN
    #include <windows.h>
    #endif

    #ifdef IC_PM
    #include <os2.h>
    #endif
}

#include <irect.hpp>
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRECT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IRectangle::asString                                                         |
|                                                                              |
| Return the rectangle contents as an IString in the format                    |
|   IRectangle(left, bottom, right, top)                                       |
------------------------------------------------------------------------------*/
IString IRectangle::asString() const
{
 IString str(IString("IRectangle(") +
        IString(minX()) +
        IString(',') +
        IString(minY())+
        IString(',')+
        IString(maxX())+
        IString(',')+
        IString(maxY())+
        IString(')'));
 return str;
}

/*------------------------------------------------------------------------------
| IRectangle::asDebugInfo                                                      |
------------------------------------------------------------------------------*/
IString IRectangle::asDebugInfo() const
  {
  IString result( "IRectangle(origin=" );
  result += this->origin.asString();
  result += "corner=";
  result += this->corner.asString();
  result += ")";
  return result;
  }


#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IRectangle::asRECTL                                                          |
------------------------------------------------------------------------------*/
RECTL IRectangle :: asRECTL ( ) const
  {
  RECTL rectl = { minX(), minY(), maxX(), maxY() };
  return rectl;
  }

#ifdef IC_WIN
IRectangle :: IRectangle ( const RECT &aRECTL)
  : origin( IPair( aRECTL.left, aRECTL.bottom ).
      minimum( IPair( aRECTL.right, aRECTL.top ) ) ),
    corner( IPair( aRECTL.left, aRECTL.bottom ).
      maximum( IPair( aRECTL.right, aRECTL.top  ) ) )
  {
  }
IRectangle :: IRectangle ( const RECTL &aRECTL)
  : origin( IPair( aRECTL.left, aRECTL.bottom ).
      minimum( IPair( aRECTL.right, aRECTL.top ) ) ),
    corner( IPair( aRECTL.left, aRECTL.bottom ).
      maximum( IPair( aRECTL.right, aRECTL.top  ) ) )
  {
  }
#endif	//IC_WIN

#ifdef IC_PM
IRectangle :: IRectangle ( const RECTL &aRECTL)
  : origin( IPair( aRECTL.xLeft, aRECTL.yBottom ).
      minimum( IPair( aRECTL.xRight, aRECTL.yTop ) ) ),
    corner( IPair( aRECTL.xLeft, aRECTL.yBottom ).
      maximum( IPair( aRECTL.xRight, aRECTL.yTop  ) ) )
  {
  }
#endif	//IC_PM
#endif	//IC_PMWIN
