// Revision: 66 1.3.1.6 source/core/base/irefcnt.cpp, appsupport, ioc.v400, 000919 
/*----------------------------------------------------------------------------*/
/* FILE NAME: irefcnt.cpp                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of classes/functions declared      */
/*   in irefcnt.hpp.                                                          */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"IREFCNT")
    #pragma csect(static,"SREFCNT")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif

#include <irefcnt.hpp>
#include <icconsta.h>

#ifdef IC_PM
unsigned long LockedIncrement(unsigned long * val);
unsigned long LockedDecrement(unsigned long * val);
#endif

#ifdef IC_WIN
#include <windows.h>
#endif

#ifdef IC_POSIX
#include <sys/atomic_op.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IREFCNT_CPP_
  #include <ipagetun.h>
#endif

#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )

/*------------------------------------------------------------------------------
| IRefCounted::IRefCounted                                                     |
|                                                                              |
| Constructor sets the the reference count to 1.                               |
------------------------------------------------------------------------------*/
IRefCounted :: IRefCounted ( )
  : refCount( 1 )
  {
  }

/*------------------------------------------------------------------------------
| IRefCounted::~IRefCounted                                                    |
------------------------------------------------------------------------------*/
IRefCounted :: ~IRefCounted ( )
  {
  }

/*------------------------------------------------------------------------------
| IRefCounted::addRef                                                          |
|                                                                              |
| Add a reference by incrementing the reference count                          |
------------------------------------------------------------------------------*/
void IRefCounted :: addRef ( )
  {
#ifdef IC_WIN
  InterlockedIncrement( LPLONG(&refCount) );
#endif
#ifdef IC_PM
   LockedIncrement( (unsigned long*) &refCount );
#endif
#ifdef IC_POSIX
   fetch_and_add((atomic_p)&refCount, 1);
#endif
  }

/*------------------------------------------------------------------------------
| IRefCounted::removeRef                                                       |
|                                                                              |
| Remove a reference by decrementing the reference count. If the refernce      |
| count goes to 0 then delete this.                                            |
------------------------------------------------------------------------------*/
void IRefCounted :: removeRef ( )
  {
#ifdef IC_WIN
    if (::InterlockedDecrement( LPLONG(&refCount) ) == 0)
        delete this;
#endif
#ifdef IC_PM
   if (LockedDecrement( (unsigned long *) &refCount ) == 0)
      delete this;
#endif
#ifdef IC_POSIX
   if (fetch_and_add( (atomic_p)&refCount, -1) == 1)
        delete this;
#endif
  }

/*------------------------------------------------------------------------------
| IRefCounted::throwNullIReference                                             |
|                                                                              |
| Out-of-line exception support for IReference template class.                 |
------------------------------------------------------------------------------*/
void IRefCounted :: throwNullIReference ( )
  {
  ITHROWLIBRARYERROR(IC_NULL_IREFERENCE, IBaseErrorInfo::invalidRequest,
                     IException::recoverable);
  }

#endif // ( IC_OBSOLETE <= IC_OBSOLETE_3 )
