// Revision: 51 1.4.1.2 source/core/base/ithrdexc.cpp, oss, ioc.v400, 000919 
/// ithrdexc.cpp
///
/// Contains: Exception classes specific to the base Interface
///     IConditionInvalid               - A condition is invalid
///
/// Author:
///     Michael Werts, Taligent Inc.
///
/// COPYRIGHT:
///     IBM Open Class Library
///     (C) Copyright International Business Machines Corporation 1997
///     Licensed Material - Program-Property of IBM - All Rights Reserved.
///     US Government Users Restricted Rights - Use, duplication, or disclosure
///     restricted by GSA ADP Schedule Contract with IBM Corp.
///
#include <imsgtext.hpp>
#include <ithrdexc.hpp>

//#include "filesys.hpp"
//#include "ifilemsg.h"
#include <istring.hpp>
#include "baseexc.h"

///
/// exceptionText
/// Handy utility class that returns the proper message string for an exception.
/// This lets me avoid lots of messy ?: expressions in my ctor initializer lists.
///
static IString
exceptionText(
        const char *textArg,
        unsigned long errorId,
        const char*   insert1 = 0,
        const char*   insert2 = 0,
        const char*   insert3 = 0,
        const char*   insert4 = 0,
        const char*   insert5 = 0,
        const char*   insert6 = 0,
        const char*   insert7 = 0,
        const char*   insert8 = 0,
        const char*   insert9 = 0 )
{
        IString result;
        if (textArg) {
                result = textArg;
        } else {
                IMessageText message( errorId, kMsgFileName,
                                                                insert1, insert2, insert3,
                                                                insert4, insert5, insert6,
                                                                insert7, insert8, insert9 );
                result = message.text();
        }
        return result;
}


///
/// IConditionInvalid
///
IConditionInvalid::IConditionInvalid(const char *textArg)
        : IInvalidParameter(exceptionText(textArg, kConditionInvalid),
                         kConditionInvalid,
                         recoverable)
{
}

IConditionInvalid::IConditionInvalid(const IConditionInvalid &copy)
        : IInvalidParameter(copy)
        {
}

IConditionInvalid::~IConditionInvalid()
{
}

const char * IConditionInvalid::name() const
{
        return "IConditionInvalid";
}

///
///
/// IWaitAbandoned
///
IWaitAbandoned::IWaitAbandoned(const char *textArg)
        : IException(exceptionText(textArg, kWaitAbandoned),
                         kConditionInvalid,
                         recoverable)
{
}

IWaitAbandoned::IWaitAbandoned(const IWaitAbandoned &copy)
        : IException(copy)
        {
}

IWaitAbandoned::~IWaitAbandoned()
{
}

const char * IWaitAbandoned::name() const
{
        return "IConditionInvalid";
}



