// Revision: 40 1.6.2.1 source/core/base/win32/iwcondtn.hpp, oss, ioc.v400, 000919 
//
// FILE NAME: iwcondtn.hpp
//
// DESCRIPTION:
//
//  The iwcondtn module provides the implementation classes for the
//  two derivaitves of the ICondition class, in base/iconditn.cpp. This
//  class provides the platform specific functionality, and the public
//  API is implemented in terms of this platform specific API.
//
//  There are are public and private conditions, and an implementation
//  class must be provided for each. Since the windows implementation
//  only differs in the naming or not naming of some internal system
//  entities, there is a common base class here and two derivatives that
//  just have constructors (to handle naming issues) and destructors. All
//  of the functionality is in the common base class.
//
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//   (C) Copyright IBM Corp. 1997
//

#ifndef _IWCONDTN_
#define _IWCONDTN_


// ----------------------------------------------------------------------------
//  Includes
// ----------------------------------------------------------------------------
#include <windows.h>
#include <iconditn.hpp>
#include <istring.hpp>


#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

// ----------------------------------------------------------------------------
//  Forward references
// ----------------------------------------------------------------------------
class IResourceLock;



// ----------------------------------------------------------------------------
//  CLASS: IConditionImpl
// ----------------------------------------------------------------------------
class IConditionImpl
{
    public :
        virtual         ~IConditionImpl();

        IString         keyName() const;
        bool            wait(long timeOut = -1);
        bool            wait(IResourceLock& alock, long timeOut = -1);
        void            signal();
        void            broadcast();

        static  bool    exists(const char* keyName);

    protected :
                        IConditionImpl() {}
                        IConditionImpl(const char* keyName);

    private :
                        IConditionImpl(const IConditionImpl& condition) {}
        ICondition&     operator=(const IConditionImpl& condition);

        IString         fKeyName;
        HANDLE          fSemaphore;
        HANDLE          fSemaCount;     // 28999 system wide counter semaphore
        HANDLE          fMutex;         // 28999 mutex to control access to
                                        //       fSemaCount
        static const int kMaxCount;
};


// ----------------------------------------------------------------------------
//  CLASS: IPrivCondImpl
// ----------------------------------------------------------------------------
class IPrivCondImpl : public IConditionImpl
{
    public:
                        IPrivCondImpl();
                        ~IPrivCondImpl();

    private:
                        IPrivCondImpl(const IPrivCondImpl&) {}
        IPrivCondImpl&  operator=(const IPrivCondImpl& condition);
};


// ----------------------------------------------------------------------------
//  CLASS: ISharedCondImpl
// ----------------------------------------------------------------------------
class ISharedCondImpl : public IConditionImpl
{
    public:
                        ISharedCondImpl(const char * keyName);
                        ~ISharedCondImpl();

    private:
                        ISharedCondImpl() {}
                        ISharedCondImpl(const ISharedCondImpl&) {}
        ISharedCondImpl& operator=(const ISharedCondImpl&);
};

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
