// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 14 1.6.1.3 source/core/collect/iacllct.h, collection, ioc.v400, 000919 
#ifndef _IACLLCT_
#define _IACLLCT_

#include <ibcllct.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

#if ! defined(__INO_L1_LOCKING__)
template <class Element>
class ICollectionGuard {
public:
  ICollectionGuard (IACollection<Element>&, long timeout=-1);

  ~ICollectionGuard ();

private:
  IACollection<Element>& ivCollection;

  ICollectionGuard (const ICollectionGuard<Element>&);

         ICollectionGuard<Element>
          &operator=      (const ICollectionGuard<Element>&);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
                          ICollectionGuard(IACollection<Element>&,\
                              long timeout=-1),\
                                                                                operator= \
                                                                                                            (const ICollectionGuard<Element>&)\
                        )
#endif
};
#endif

template <class Element>
class IElementCursor : public ICursor {

  typedef  ICursorImpl Implementation;

public:

  Element const&
           element          () const;

protected:

           IElementCursor   (Implementation*);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           element          () const,\
           IElementCursor   (Implementation*)\
                        )
#endif

};

template <class Element>
class IACollection : protected IACollectionBase {

  typedef  IACollectionImpl Implementation;

public:

#if ! defined (__INO_AVALON__)
           IACollection     (INotifier&);

#endif
  virtual ~IACollection     ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool add              (Element const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool add              (Element const&, ICursor&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addAllFrom       (IACollection <Element> const&);

  bool allElementsDo    (bool (*applicatorFunction)
                              (Element&, void*),
                             void* additionalArgument = 0);

  bool allElementsDo    (bool (*applicatorFunction)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  bool allElementsDo    (IApplicator <Element>&);

  bool allElementsDo    (IConstantApplicator <Element>&) const;

  Element const&
           any              () const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     copy             (IACollection <Element> const&);

#if ! defined (__INO_AVALON__)
  INotifier&
           disableNotification
                            ();

#endif

  Element& elementAt        (ICursor const&);

  Element const&
           elementAt        (ICursor const&) const;

#if ! defined (__INO_AVALON__)
  INotifier&
           enableNotification
                            (bool = true);

#endif
  bool isBounded        () const;

  bool isConsistent     () const;

  bool isEmpty          () const;

#if ! defined (__INO_AVALON__)
  bool isEnabledForNotification
                            () const;

#endif
  bool isFull           () const;

  INumber  maxNumberOfElements
                            () const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  ICursor* newCursor        () const;

#if ! defined (__INO_MNGCURSOR__)
  ICursor* newMngCursor     () const;

#endif
#if ! defined (__INO_AVALON__)
  INotifier&
           notifier         ();

  INotifier const&
           notifier         () const;

  INotifier&
           notifyObservers  (INotificationEvent const&);

#endif
  INumber  numberOfElements () const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  INumber  removeAll        ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  INumber  removeAll        (bool (*predicateFunction)
                              (Element const&, void*),
                             void* additionalArgument = 0);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     removeAt         (ICursor&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     replaceAt        (ICursor const&, Element const&);

  bool setToFirst       (ICursor&) const;

  bool setToNext        (ICursor&) const;

#if ! defined (__INO_AVALON__)
           operator INotifier&
                            () const;

#endif
  // for backward compatibility
  Element const&
           anyElement       () const;

protected:

#if ! defined(__INO_L1_LOCKING__)
        friend class ICollectionGuard<Element>;
#endif

           IACollection     ();

           IACollection     (IACollection <Element> const&);

  static
  Implementation&
           ImplOf           (IACollection <Element> const&);

private:

#if ! defined (__INO_STREAMING__)
public:
#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void   operator<<=      (IDataStream&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void   operator>>=      (IDataStream&);

  void   adoptStreamer    (IACollectionStreamer*);
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           add              (Element const&),\
           add              (Element const&, ICursor&),\
           addAllFrom       (IACollection <Element> const&),\
           allElementsDo    (bool (*) (Element&, void*), void*),\
           allElementsDo    (bool (*) (Element const&, void*),\
                             void*) const,\
           allElementsDo    (IApplicator <Element>&),\
           allElementsDo    (IConstantApplicator <Element>&) const,\
           anyElement       () const,\
           copy             (IACollection <Element> const&),\
           elementAt        (ICursor const&),\
           elementAt        (ICursor const&) const,\
           isBounded        () const,\
           isConsistent     () const,\
           isEmpty          () const,\
           isFull           () const,\
           maxNumberOfElements\
                            () const,\
           newCursor        () const,\
           numberOfElements () const,\
           removeAll        (),\
           removeAll        (bool (*) (Element const&, void*),\
                             void*),\
           removeAt         (ICursor&),\
           replaceAt        (ICursor const&, Element const&),\
           setToFirst       (ICursor&) const,\
           setToNext        (ICursor&) const,\
           ImplOf           (IACollection <Element> const&)\
           operator<<=      (IDataStream&),\
           operator>>=      (IDataStream&),\
           adoptStreamer    (IACollectionStreamer*)\
                       )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#include <iacllct.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IACLLCT_
