// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 35 1.4.1.3 source/core/collect/iakey.h, collection, ioc.v400, 000919 
#ifndef _IAKEY_
#define _IAKEY_

#include <iacllct.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key>
class IAKeyCollection : public virtual IACollection <Element> 
{
  typedef  IACollectionImpl Implementation;

public:

#if ! defined (__INO_AVALON__)
           IAKeyCollection  (INotifier&);

#endif
          ~IAKeyCollection  ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool addOrReplaceElementWithKey
                            (Element const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool addOrReplaceElementWithKey
                            (Element const&, ICursor&);

  bool containsAllKeysFrom
                            (IACollection <Element> const&) const; 

  bool containsElementWithKey
                            (Key const&) const;

  Element& elementWithKey   (Key const&);

  Element const&
           elementWithKey   (Key const&) const;

  Key const&
           key              (Element const&) const;

  bool locateElementWithKey
                            (Key const&, ICursor&) const;

  bool locateNextElementWithKey
                            (Key const&, ICursor&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool locateOrAddElementWithKey
                            (Element const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool locateOrAddElementWithKey
                            (Element const&, ICursor&);

  INumber  numberOfDifferentKeys
                            () const;

  INumber  numberOfElementsWithKey
                            (Key const&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  INumber  removeAllElementsWithKey
                            (Key const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool removeElementWithKey
                            (Key const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool replaceElementWithKey
                            (Element const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool replaceElementWithKey
                            (Element const&, ICursor&);

  bool setToNextWithDifferentKey
                            (ICursor&) const;

protected:

           IAKeyCollection  ();

           IAKeyCollection  (IAKeyCollection <Element, Key> const&);

private:

  static
  Implementation&
           ImplOf           (IAKeyCollection <Element, Key> const&);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           addOrReplaceElementWithKey\
                            (Element const&),\
           addOrReplaceElementWithKey\
                            (Element const&, ICursor&),\
           containsAllKeysFrom\
                            (IAKeyCollection\
                              <Element, Key> const&) const,\
           containsElementWithKey\
                            (Key const&) const,\
           elementWithKey   (Key const&),\
           elementWithKey   (Key const&) const,\
           key              (Element const&) const,\
           locateElementWithKey\
                            (Key const&, ICursor&) const,\
           locateNextElementWithKey\
                            (Key const&, ICursor&) const,\
           locateOrAddElementWithKey\
                            (Element const&),\
           locateOrAddElementWithKey\
                            (Element const&, ICursor&),\
           numberOfDifferentKeys\
                            () const,\
           numberOfElementsWithKey\
                            (Key const&) const,\
           removeAllElementsWithKey\
                            (Key const&),\
           removeElementWithKey\
                            (Key const&),\
           replaceElementWithKey\
                            (Element const&),\
           replaceElementWithKey\
                            (Element const&, ICursor&),\
           setToNextWithDifferentKey\
                            (ICursor&) const,\
           ImplOf           (IAKeyCollection <Element, Key> const&))
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iakey.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IAKEY_
