// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 54 1.3.1.2 source/core/collect/iaorder.inl, collection, ioc.v400, 000919 
#ifndef _IAORDER_INL_
#define _IAORDER_INL_

// --------------
// IOrderedCursor
// --------------

#include <ilanglvl.hpp>

// public members

template <class Element>
inline bool
IOrderedCursor <Element>::
setToLast ()
{ return this->ImplOf (*this).SetToLast ();
}


template <class Element>
inline bool
IOrderedCursor <Element>::
setToPrevious ()
{ return this->ImplOf (*this).SetToPrevious ();
}

// protected members

template <class Element>
inline
IOrderedCursor <Element>::
IOrderedCursor (Implementation* impl)
: IElementCursor <Element> (impl)
{
}

// -------------------
// IAOrderedCollection
// -------------------

// public members

#if ! defined (__INO_AVALON__)
template <class Element>
inline
IAOrderedCollection <Element>::
IAOrderedCollection (INotifier& notifier)
{ this->SetImpl ((Implementation*)&notifier);
}

#endif
template <class Element>
inline
IAOrderedCollection <Element>::
~IAOrderedCollection ()
{
}

template <class Element>
inline Element const&
IAOrderedCollection <Element>::
elementAtPosition (IPosition position) const
{ return *(Element const*) ImplOf (*this).
    ElementAtPosition (position);
}

template <class Element>
inline Element const&
IAOrderedCollection <Element>::
firstElement () const
{ return *(Element const*) ImplOf (*this).First ();
}

template <class Element>
inline Element const&
IAOrderedCollection <Element>::
first () const
{ return *(Element const*) ImplOf (*this).First ();
}

template <class Element>
inline bool
IAOrderedCollection <Element>::
isFirst (ICursor const& cursor) const
{ return this->ImplOf (*this).IsFirstAt (this->CrsrImplOf (cursor));
}

template <class Element>
inline bool
IAOrderedCollection <Element>::
isFirstAt (ICursor const& cursor) const
{ return this->ImplOf (*this).IsFirstAt (this->CrsrImplOf (cursor));
}

template <class Element>
inline bool
IAOrderedCollection <Element>::
isLast (ICursor const& cursor) const
{ return this->ImplOf (*this).IsLastAt (this->CrsrImplOf (cursor));
}

template <class Element>
inline bool
IAOrderedCollection <Element>::
isLastAt (ICursor const& cursor) const
{ return this->ImplOf (*this).IsLastAt (this->CrsrImplOf (cursor));
}

template <class Element>
inline Element const&
IAOrderedCollection <Element>::
lastElement () const
{ return *(Element const*) ImplOf (*this).Last ();
}

template <class Element>
inline Element const&
IAOrderedCollection <Element>::
last () const
{ return *(Element const*) ImplOf (*this).Last ();
}

template <class Element>
inline IPosition
IAOrderedCollection <Element>::
position (ICursor const& cursor) const
{ return this->ImplOf (*this).PositionAt (this->CrsrImplOf (cursor));
}

template <class Element>
inline IPosition
IAOrderedCollection <Element>::
positionAt (ICursor const& cursor) const
{ return this->ImplOf (*this).PositionAt (this->CrsrImplOf (cursor));
}

template <class Element>
inline void
IAOrderedCollection <Element>::
removeAtPosition (IPosition position)
{ this->ImplOf (*this).RemoveAtPosition (position);
}

template <class Element>
inline void
IAOrderedCollection <Element>::
removeFirst ()
{ this->ImplOf (*this).RemoveFirst ();
}

template <class Element>
inline void
IAOrderedCollection <Element>::
removeLast ()
{ this->ImplOf (*this).RemoveLast ();
}

template <class Element>
inline bool
IAOrderedCollection <Element>::
setToLast (ICursor& cursor) const
{ return this->ImplOf (*this).SetToLast (this->CrsrImplOf (cursor));
}

template <class Element>
inline void
IAOrderedCollection <Element>::
setToPosition (IPosition position, ICursor& cursor) const
{ this->ImplOf (*this).SetToPosition (position, this->CrsrImplOf (cursor));
}

template <class Element>
inline bool
IAOrderedCollection <Element>::
setToPrevious (ICursor& cursor) const
{ return this->ImplOf (*this).SetToPrevious (this->CrsrImplOf (cursor));
}

// protected members

template <class Element>
inline
IAOrderedCollection <Element>::
IAOrderedCollection ()
{
}

template <class Element>
inline
IAOrderedCollection <Element>::
IAOrderedCollection (IAOrderedCollection <Element> const&)
{
}

// private members

template <class Element>
inline TYPENAME IAOrderedCollection <Element>::Implementation&
IAOrderedCollection <Element>::
ImplOf (IAOrderedCollection <Element> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

#endif // _IAORDER_INL_
