// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 56 1.5.1.3 source/core/collect/iapqu.h, collection, ioc.v400, 000919 
#ifndef _IAPQU_
#define _IAPQU_

#include <iarstrct.h>
#include <iaorder.h>
#include <iiaksb.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key>
class IAPriorityQueue :
  public IARestrictedAccessCollection <Element> {

  typedef  IARestrictedAccessCollection <Element> Inherited;

  typedef  IAKeySortedBagImpl Implementation;

public:

#if ! defined (__INO_AVALON__)
           IAPriorityQueue
                            (INotifier&);

#endif
          ~IAPriorityQueue
                            ();

  long     compare          (IAPriorityQueue <Element, Key> const&,
                             long (*comparisonFunction)
                              (Element const&, Element const&)) const;
 
  bool     containsAllKeysFrom
                            (IARestrictedAccessCollection
                              <Element> const&) const;

  bool     containsElementWithKey
                            (Key const&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     dequeue          ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     dequeue          (Element&);

  Element const&
           elementWithKey   (Key const&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     enqueue          (Element const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     enqueue          (Element const&, ICursor&);

  Key const&
           key              (Element const&) const;

  bool     locateElementWithKey
                            (Key const&, ICursor&) const;

  bool     locateNextElementWithKey
                            (Key const&, ICursor&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool     locateOrAddElementWithKey
                            (Element const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  bool     locateOrAddElementWithKey
                            (Element const&, ICursor&);

  INumber  numberOfDifferentKeys
                            () const;

  INumber  numberOfElementsWithKey
                            (Key const&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     removeFirst      ();

  bool     setToNextWithDifferentKey
                            (ICursor&) const;

protected:

           IAPriorityQueue
                            ();

           IAPriorityQueue
                            (IAPriorityQueue <Element, Key> const&);

private:

  static
  Implementation&
           ImplOf           (IAPriorityQueue <Element, Key> const&);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           compare          (IAPriorityQueue <Element, Key> const&,\
                             long (*)\
                              (Element const&, Element const&) const,\
           containsAllKeysFrom\
                            (IAPriorityQueue\
                              <Element, Key> const&) const,\
           containsElementWithKey\
                            (Key const&) const,\
           dequeue          (),\
           dequeue          (Element&),\
           enqueue          (Element const&),\
           enqueue          (Element const&, ICursor),\
           key              (Element const&) const,\
           locateElementWithKey\
                            (Key const&, ICursor&) const,\
           locateNextElementWithKey\
                            (Key const&, ICursor&) const,\
           locateOrAddElementWithKey\
                            (Element const&),\
           locateOrAddElementWithKey\
                            (Element const&, ICursor&),\
           numberOfDifferentKeys\
                            () const,\
           numberOfElementsWithKey\
                            (Key const&) const,\
           removeFirst      (),\
           setToNextWithDifferentKey\
                            (ICursor&) const,\
           ImplOf           (IAPriorityQueue <Element, Key> const&)\
                        )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <iapqu.inl>

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IAPQU_
