// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 57 1.4.1.2 source/core/collect/iapqu.inl, collection, ioc.v400, 000919  
#ifndef _IAPQU_INL_
#define _IAPQU_INL_

// ---
// IAPriorityQueue
// ---

#include <ilanglvl.hpp>

#if ! defined (__INO_AVALON__)
template <class Element, class Key>
inline
IAPriorityQueue <Element, Key>::
IAPriorityQueue (INotifier& notifier)
{ this->SetImpl ((Implementation*)&notifier);
}

#endif
template <class Element, class Key>
inline
IAPriorityQueue <Element, Key>::
~IAPriorityQueue ()
{
}

template <class Element, class Key>
inline long
IAPriorityQueue <Element, Key>::
compare (IAPriorityQueue <Element, Key> const& collection,
         long (*compFunc) (Element const&, Element const&)) const
{ return this->ImplOf (*this).
    Compare (this->ImplOf (collection), (ICompFunc)compFunc, 0);
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
containsAllKeysFrom
  (IARestrictedAccessCollection <Element> const& collection) const
{ return this->ImplOf (*this).
    ContainsAllKeysFrom (Inherited::ImplOf (collection));
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
containsElementWithKey (Key const& key) const
{ return this->ImplOf (*this).ContainsElementWithKey (&key);
}

template <class Element, class Key>
inline void
IAPriorityQueue <Element, Key>::
dequeue ()
{ this->ImplOf (*this).RemoveFirst ();
}

template <class Element, class Key>
inline void
IAPriorityQueue <Element, Key>::
dequeue (Element& element)
{ element = *(Element*) this->ImplOf (*this).First ();
  this->ImplOf (*this).RemoveFirst ();
}

template <class Element, class Key>
inline Element const&
IAPriorityQueue <Element, Key>::
elementWithKey (Key const& key) const
{ return *(Element const*) this->ImplOf (*this).ElementWithKey (&key);
}

template <class Element, class Key>
inline void
IAPriorityQueue <Element, Key>::
enqueue (Element const& element)
{ this->ImplOf (*this).Add (&element);
}

template <class Element, class Key>
inline void
IAPriorityQueue <Element, Key>::
enqueue (Element const& element, ICursor& cursor)
{ this->ImplOf (*this).Add (&element, this->CrsrImplOf (cursor));
}

template <class Element, class Key>
inline Key const&
IAPriorityQueue <Element, Key>::
key (Element const& element) const
{ return *(Key const*) this->ImplOf (*this).Key (&element);
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
locateElementWithKey (Key const& key, ICursor& cursor) const
{ return this->ImplOf (*this).
    LocateElementWithKey (&key, this->CrsrImplOf (cursor));
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
locateNextElementWithKey (Key const& key, ICursor& cursor) const
{ return this->ImplOf (*this).
    LocateNextElementWithKey (&key, this->CrsrImplOf (cursor));
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
locateOrAddElementWithKey (Element const& element)
{ return this->ImplOf (*this).LocateOrAddElementWithKey (&element);
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
locateOrAddElementWithKey (Element const& element, ICursor& cursor)
{ return this->ImplOf (*this).
    LocateOrAddElementWithKey (&element, this->CrsrImplOf (cursor));
}

template <class Element, class Key>
inline INumber
IAPriorityQueue <Element, Key>::
numberOfDifferentKeys () const
{ return this->ImplOf (*this).NumberOfDifferentKeys ();
}

template <class Element, class Key>
inline INumber
IAPriorityQueue <Element, Key>::
numberOfElementsWithKey (Key const& key) const
{ return this->ImplOf (*this).NumberOfElementsWithKey (&key);
}

template <class Element, class Key>
inline void
IAPriorityQueue <Element, Key>::
removeFirst ()
{ this->ImplOf (*this).RemoveFirst ();
}

template <class Element, class Key>
inline bool
IAPriorityQueue <Element, Key>::
setToNextWithDifferentKey (ICursor& cursor) const
{ return this->ImplOf (*this).SetToNextWithDifferentKey (this->CrsrImplOf (cursor));
}

// protected members

template <class Element, class Key>
inline
IAPriorityQueue <Element, Key>::
IAPriorityQueue ()
{
}

template <class Element, class Key>
inline
IAPriorityQueue <Element, Key>::
IAPriorityQueue (IAPriorityQueue <Element, Key> const& collection)
: Inherited (collection)
{
}

// private members

template <class Element, class Key>
inline TYPENAME IAPriorityQueue <Element, Key>::Implementation&
IAPriorityQueue <Element, Key>::
ImplOf (IAPriorityQueue <Element, Key> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

#endif // _IAPQU_INL_
