// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 95 1.5.1.5 source/core/collect/ibagavl.h, collection, ioc.v400, 000919 
#ifndef _IBAGAVL_
#define _IBAGAVL_

#include <ilanglvl.hpp>
#include <iabag.h>
#include <iicbag.h>
#if ! defined (__INO_AVALON__)
#include <iivbag.h>
#endif
#include <iibaglst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGBagAsAvlTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGBagAsAvlTreeCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGBagAsAvlTree
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGBagAsAvlTreeCursor
                            (IGBagAsAvlTree
                              <
                               Element, ElementOps> const&);

  IGBagAsAvlTreeCursor <Element, ElementOps>&
           operator=        (IGBagAsAvlTreeCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsAvlTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsAvlTreeCursor\
                            (IGBagAsAvlTree\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGBagAsAvlTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGBagAsAvlTreeMngCursor :
  public IGBagAsAvlTreeCursor <Element, ElementOps>  {

  typedef  IGBagAsAvlTreeCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGBagAsAvlTree
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGBagAsAvlTreeMngCursor
                            (IGBagAsAvlTree
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsAvlTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsAvlTreeMngCursor\
                            (IGBagAsAvlTree\
                              <\
                               Element, ElementOps> const&),\
           IGBagAsAvlTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGBagAsAvlTree :
  public IABag <Element> {
protected:

  typedef  IGBagAsAvlTree <Element, ElementOps> Self;

  typedef  IBagAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICBagImpl
            <PureImplementation> Implementation;
#endif

#define IGBagAsAvlTreeTypedef(Element, ElementOps)\
  typedef  IBagAsListOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGBagAsAvlTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGBagAsAvlTreeCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGBagAsAvlTreeMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGBagAsAvlTree
                            (INumber = 100);

           IGBagAsAvlTree
                            (IGBagAsAvlTree
                              <
                               Element, ElementOps> const&);

  IGBagAsAvlTree <Element, ElementOps>&
           operator=        (IGBagAsAvlTree
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGBagAsAvlTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGBagAsAvlTree
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGBagAsAvlTreeCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGBagAsAvlTreeMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsAvlTree\
                            (INumber),\
           operator=,\
           IGBagAsAvlTree\
                            (PureImplementation&),\
           ImplOf           (IGBagAsAvlTree\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IBagAsAvlTree :
  public IGBagAsAvlTree
          <Element, ICOps <Element> > {

  typedef  IGBagAsAvlTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IBagAsAvlTree <Element> Self;

#define IBagAsAvlTreeTypedef(Element)\
  typedef  IBagAsListOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IBagAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IBagAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IBagAsAvlTree\
                              (INumber),\
           IBagAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGBagAsAvlTree :
  public IGBagAsAvlTree
          <Element, ElementOps > {

  typedef  IVGBagAsAvlTree
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGBagAsAvlTree
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICBagImpl
            <IVBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGBagAsAvlTreeTypedef(Element, ElementOps)\
  typedef  IBagAsListOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGBagAsAvlTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGBagAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGBagAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGBagAsAvlTree\
                              (INumber),\
           IVGBagAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVBagAsAvlTree :
  public IVGBagAsAvlTree
          <Element, ICOps <Element> > {

  typedef  IVGBagAsAvlTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVBagAsAvlTree <Element> Self;

#define IVBagAsAvlTreeTypedef(Element)\
  typedef  IVBagAsListOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVBagAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVBagAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVBagAsAvlTree\
                              (INumber),\
           IVBagAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ibagavl.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IBAGAVL_
