// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 96 1.4.1.4 source/core/collect/ibagavl.inl, collection, ioc.v400, 000919  
#ifndef _IBAGAVL_INL_
#define _IBAGAVL_INL_

// ---
// IGBagAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGBagAsAvlTreeCursor <Element, ElementOps>::
IGBagAsAvlTreeCursor
  (IGBagAsAvlTree
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGBagAsAvlTreeCursor <Element, ElementOps>&
IGBagAsAvlTreeCursor <Element, ElementOps>::
operator=
  (IGBagAsAvlTreeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsAvlTreeCursor <Element, ElementOps>::
IGBagAsAvlTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGBagAsAvlTreeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsAvlTreeMngCursor <Element, ElementOps>::
IGBagAsAvlTreeMngCursor
  (IGBagAsAvlTree
    <Element, ElementOps> const& collection)
: IGBagAsAvlTreeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsAvlTreeMngCursor <Element, ElementOps>::
IGBagAsAvlTreeMngCursor (ICursorImpl* impl)
: IGBagAsAvlTreeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGBagAsAvlTree
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsAvlTree <Element, ElementOps>::
IGBagAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGBagAsAvlTree <Element, ElementOps>::
IGBagAsAvlTree
  (IGBagAsAvlTree
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGBagAsAvlTree <Element, ElementOps>&
IGBagAsAvlTree <Element, ElementOps>::
operator= (IGBagAsAvlTree
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsAvlTree <Element, ElementOps>::
IGBagAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGBagAsAvlTree
 <Element, ElementOps>::Implementation&
IGBagAsAvlTree <Element, ElementOps>::
ImplOf (IGBagAsAvlTree
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IBagAsAvlTree
// ---

// public members

template <class Element>
inline
IBagAsAvlTree <Element>::
IBagAsAvlTree (INumber numberOfElements)
: IGBagAsAvlTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IBagAsAvlTree <Element>::
IBagAsAvlTree (PureImplementation& impl)
: IGBagAsAvlTree
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGBagAsAvlTree
// ---

// public members

template <class Element, class ElementOps>
inline
IVGBagAsAvlTree <Element, ElementOps>::
IVGBagAsAvlTree (INumber numberOfElements)
: IGBagAsAvlTree
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGBagAsAvlTree <Element, ElementOps>::
IVGBagAsAvlTree (PureImplementation& impl)
: IGBagAsAvlTree
    <Element, ElementOps> (impl)
{
}

// ---
// IVBagAsAvlTree
// ---

// public members

template <class Element>
inline
IVBagAsAvlTree <Element>::
IVBagAsAvlTree (INumber numberOfElements)
: IVGBagAsAvlTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVBagAsAvlTree <Element>::
IVBagAsAvlTree (PureImplementation& impl)
: IVGBagAsAvlTree
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _IBAGAVL_INL_
