// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 05 1.4.1.4 source/core/collect/ibaglst.inl, collection, ioc.v400, 000919  
#ifndef _IBAGLST_INL_
#define _IBAGLST_INL_

// ---
// IGBagAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGBagAsListCursor <Element, ElementOps>::
IGBagAsListCursor
  (IGBagAsList
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGBagAsListCursor <Element, ElementOps>&
IGBagAsListCursor <Element, ElementOps>::
operator=
  (IGBagAsListCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsListCursor <Element, ElementOps>::
IGBagAsListCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGBagAsListMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsListMngCursor <Element, ElementOps>::
IGBagAsListMngCursor
  (IGBagAsList
    <Element, ElementOps> const& collection)
: IGBagAsListCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsListMngCursor <Element, ElementOps>::
IGBagAsListMngCursor (ICursorImpl* impl)
: IGBagAsListCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGBagAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsList <Element, ElementOps>::
IGBagAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGBagAsList <Element, ElementOps>::
IGBagAsList
  (IGBagAsList
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGBagAsList <Element, ElementOps>&
IGBagAsList <Element, ElementOps>::
operator= (IGBagAsList
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsList <Element, ElementOps>::
IGBagAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGBagAsList
 <Element, ElementOps>::Implementation&
IGBagAsList <Element, ElementOps>::
ImplOf (IGBagAsList
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IBagAsList
// ---

// public members

template <class Element>
inline
IBagAsList <Element>::
IBagAsList (INumber numberOfElements)
: IGBagAsList
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IBagAsList <Element>::
IBagAsList (PureImplementation& impl)
: IGBagAsList
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGBagAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IVGBagAsList <Element, ElementOps>::
IVGBagAsList (INumber numberOfElements)
: IGBagAsList
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGBagAsList <Element, ElementOps>::
IVGBagAsList (PureImplementation& impl)
: IGBagAsList
    <Element, ElementOps> (impl)
{
}

// ---
// IVBagAsList
// ---

// public members

template <class Element>
inline
IVBagAsList <Element>::
IVBagAsList (INumber numberOfElements)
: IVGBagAsList
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVBagAsList <Element>::
IVBagAsList (PureImplementation& impl)
: IVGBagAsList
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _IBAGLST_INL_
