// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 10 1.4.1.4 source/core/collect/ibagtab.inl, collection, ioc.v400, 000919  
#ifndef _IBAGTAB_INL_
#define _IBAGTAB_INL_

// ---
// IGBagAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGBagAsTableCursor <Element, ElementOps>::
IGBagAsTableCursor
  (IGBagAsTable
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGBagAsTableCursor <Element, ElementOps>&
IGBagAsTableCursor <Element, ElementOps>::
operator=
  (IGBagAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsTableCursor <Element, ElementOps>::
IGBagAsTableCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGBagAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsTableMngCursor <Element, ElementOps>::
IGBagAsTableMngCursor
  (IGBagAsTable
    <Element, ElementOps> const& collection)
: IGBagAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsTableMngCursor <Element, ElementOps>::
IGBagAsTableMngCursor (ICursorImpl* impl)
: IGBagAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGBagAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsTable <Element, ElementOps>::
IGBagAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGBagAsTable <Element, ElementOps>::
IGBagAsTable
  (IGBagAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGBagAsTable <Element, ElementOps>&
IGBagAsTable <Element, ElementOps>::
operator= (IGBagAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsTable <Element, ElementOps>::
IGBagAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGBagAsTable
 <Element, ElementOps>::Implementation&
IGBagAsTable <Element, ElementOps>::
ImplOf (IGBagAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IBagAsTable
// ---

// public members

template <class Element>
inline
IBagAsTable <Element>::
IBagAsTable (INumber numberOfElements)
: IGBagAsTable
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IBagAsTable <Element>::
IBagAsTable (PureImplementation& impl)
: IGBagAsTable
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGBagAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGBagAsTable <Element, ElementOps>::
IVGBagAsTable (INumber numberOfElements)
: IGBagAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGBagAsTable <Element, ElementOps>::
IVGBagAsTable (PureImplementation& impl)
: IGBagAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVBagAsTable
// ---

// public members

template <class Element>
inline
IVBagAsTable <Element>::
IVBagAsTable (INumber numberOfElements)
: IVGBagAsTable
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVBagAsTable <Element>::
IVBagAsTable (PureImplementation& impl)
: IVGBagAsTable
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _IBAGTAB_INL_
