// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 16 1.5.1.3 source/core/collect/ibtree.h, collection, ioc.v400, 000919 
#ifndef _IBTREE_
#define _IBTREE_

#include <imtree.h>
#include <iglobals.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__RRBC_LIB__)         
#define ITreeCursor IRRBCTreeCursor
#define IATreeBase IARRBCTreeBase  
#endif                             

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

class IC_EXPORTC ITreeCursor
{

  typedef  ITreeCursorImpl Implementation;

public:

  virtual ~ITreeCursor      ();

  void     copy             (ITreeCursor const&);

  void     invalidate       ();

  bool isValid          () const;

  bool setToChild       (IPosition);

  bool setToFirstExistingChild
                            ();

  bool setToLastExistingChild
                            ();

  bool setToNextExistingChild
                            ();

  bool setToParent      ();

  bool setToPreviousExistingChild
                            ();

  bool setToRoot        ();

  void     operator=        (ITreeCursor const&);

  bool operator==       (ITreeCursor const&) const;

  bool operator!=       (ITreeCursor const&) const;

protected:

           ITreeCursor      (Implementation*);

           ITreeCursor      (ITreeCursor const&);

  static
  inline
  Implementation&
           ImplOf           (ITreeCursor const&);

private:

  Implementation*
           ivImpl;

  friend
  class    IATreeBase;

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           copy             (ITreeCursor const&),\
           invalidate       (),\
           isValid          () const,\
           setToChild       (IPosition),\
           setToFirstExistingChild\
                            (),\
           setToLastExistingChild\
                            (),\
           setToNextExistingChild\
                            (),\
           setToParent      (),\
           setToPreviousExistingChild\
                            (),\
           setToRoot        (),\
           operator==       (ITreeCursor const&) const,\
           operator!=       (ITreeCursor const&) const,\
           ITreeCursor      (Implementation*)\
                        )
#endif

};

class IC_EXPORTC IATreeBase : protected IMetaTree {
public:

protected:

           IATreeBase       ();

           IATreeBase       (IATreeBase const&);

          ~IATreeBase       ();

  static
  ITreeCursorImpl&
           CrsrImplOf       (ITreeCursor& cursor);

  static
  ITreeCursorImpl const&
           CrsrImplOf       (ITreeCursor const& cursor);

  static
  ITreeCursor*
           InterfaceFor     (ITreeCursorImpl* cursor);
 
#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           CrsrImplOf       (ITreeCursor&),\
           CrsrImplOf       (ITreeCursor const&),\
           InterfaceFor     (ITreeCursorImpl*)\
                        )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ibtree.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IBTREE_
