// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 23 1.6.1.4 source/core/collect/idqu.h, collection, ioc.v400, 000919 
#ifndef _IDQU_
#define _IDQU_

#include <ilanglvl.hpp>
#include <iadqu.h>
#include <iicseq.h>
#if ! defined (__INO_AVALON__)
#include <iivseq.h>
#endif
#include <iiseqlst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGDeque;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGDequeCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGDeque
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGDequeCursor
                            (IGDeque
                              <
                               Element, ElementOps> const&);

  IGDequeCursor <Element, ElementOps>&
           operator=        (IGDequeCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGDequeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGDequeCursor\
                            (IGDeque\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGDequeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGDequeMngCursor :
  public IGDequeCursor <Element, ElementOps>  {

  typedef  IGDequeCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGDeque
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGDequeMngCursor
                            (IGDeque
                              <
                               Element, ElementOps> const&);

protected:

           IGDequeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGDequeMngCursor\
                            (IGDeque\
                              <\
                               Element, ElementOps> const&),\
           IGDequeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGDeque :
  public IADeque <Element> {
protected:

  typedef  IGDeque <Element, ElementOps> Self;

  typedef  ISequenceAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSequenceImpl
            <PureImplementation> Implementation;
#endif

#define IGDequeTypedef(Element, ElementOps)\
  typedef  ISequenceAsListOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGDequeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

  typedef  IGDequeCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGDequeMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGDeque
                            (INumber = 100);

           IGDeque
                            (IGDeque
                              <
                               Element, ElementOps> const&);

  IGDeque <Element, ElementOps>&
           operator=        (IGDeque
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGDeque
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGDeque
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGDequeCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGDequeMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGDeque\
                            (INumber),\
           operator=,\
           IGDeque\
                            (PureImplementation&),\
           ImplOf           (IGDeque\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IDeque :
  public IGDeque
          <Element, IStdOps <Element> > {

  typedef  IGDeque
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IDeque <Element> Self;

#define IDequeTypedef(Element)\
  typedef  ISequenceAsListOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IDeque
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IDeque
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IDeque\
                              (INumber),\
           IDeque\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGDeque :
  public IGDeque
          <Element, ElementOps > {

  typedef  IVGDeque
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGDeque
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSequenceImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSequenceImpl
            <IVSequenceImpl
              <PureImplementation> > Implementation;
#endif

#define IVGDequeTypedef(Element, ElementOps)\
  typedef  ISequenceAsListOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGDequeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVGDeque
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGDeque
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGDeque\
                              (INumber),\
           IVGDeque\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVDeque :
  public IVGDeque
          <Element, IStdOps <Element> > {

  typedef  IVGDeque
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVDeque <Element> Self;

#define IVDequeTypedef(Element)\
  typedef  IVSequenceAsListOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVDeque
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVDeque
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVDeque\
                              (INumber),\
           IVDeque\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <idqu.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IDQU_
