// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 29 1.4.1.4 source/core/collect/idqulst.inl, collection, ioc.v400, 000919  
#ifndef _IDQULST_INL_
#define _IDQULST_INL_

// ---
// IGDequeAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGDequeAsListCursor <Element, ElementOps>::
IGDequeAsListCursor
  (IGDequeAsList
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGDequeAsListCursor <Element, ElementOps>&
IGDequeAsListCursor <Element, ElementOps>::
operator=
  (IGDequeAsListCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeAsListCursor <Element, ElementOps>::
IGDequeAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGDequeAsListMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGDequeAsListMngCursor <Element, ElementOps>::
IGDequeAsListMngCursor
  (IGDequeAsList
    <Element, ElementOps> const& collection)
: IGDequeAsListCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeAsListMngCursor <Element, ElementOps>::
IGDequeAsListMngCursor (ICursorImpl* impl)
: IGDequeAsListCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGDequeAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IGDequeAsList <Element, ElementOps>::
IGDequeAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGDequeAsList <Element, ElementOps>::
IGDequeAsList
  (IGDequeAsList
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGDequeAsList <Element, ElementOps>&
IGDequeAsList <Element, ElementOps>::
operator= (IGDequeAsList
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeAsList <Element, ElementOps>::
IGDequeAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGDequeAsList
 <Element, ElementOps>::Implementation&
IGDequeAsList <Element, ElementOps>::
ImplOf (IGDequeAsList
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IDequeAsList
// ---

// public members

template <class Element>
inline
IDequeAsList <Element>::
IDequeAsList (INumber numberOfElements)
: IGDequeAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IDequeAsList <Element>::
IDequeAsList (PureImplementation& impl)
: IGDequeAsList
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGDequeAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IVGDequeAsList <Element, ElementOps>::
IVGDequeAsList (INumber numberOfElements)
: IGDequeAsList
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGDequeAsList <Element, ElementOps>::
IVGDequeAsList (PureImplementation& impl)
: IGDequeAsList
    <Element, ElementOps> (impl)
{
}

// ---
// IVDequeAsList
// ---

// public members

template <class Element>
inline
IVDequeAsList <Element>::
IVDequeAsList (INumber numberOfElements)
: IVGDequeAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVDequeAsList <Element>::
IVDequeAsList (PureImplementation& impl)
: IVGDequeAsList
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IDQULST_INL_
