// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 31 1.4.1.4 source/core/collect/idqutab.inl, collection, ioc.v400, 000919  
#ifndef _IDQUTAB_INL_
#define _IDQUTAB_INL_

// ---
// IGDequeAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGDequeAsTableCursor <Element, ElementOps>::
IGDequeAsTableCursor
  (IGDequeAsTable
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGDequeAsTableCursor <Element, ElementOps>&
IGDequeAsTableCursor <Element, ElementOps>::
operator=
  (IGDequeAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeAsTableCursor <Element, ElementOps>::
IGDequeAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGDequeAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGDequeAsTableMngCursor <Element, ElementOps>::
IGDequeAsTableMngCursor
  (IGDequeAsTable
    <Element, ElementOps> const& collection)
: IGDequeAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeAsTableMngCursor <Element, ElementOps>::
IGDequeAsTableMngCursor (ICursorImpl* impl)
: IGDequeAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGDequeAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGDequeAsTable <Element, ElementOps>::
IGDequeAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGDequeAsTable <Element, ElementOps>::
IGDequeAsTable
  (IGDequeAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGDequeAsTable <Element, ElementOps>&
IGDequeAsTable <Element, ElementOps>::
operator= (IGDequeAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeAsTable <Element, ElementOps>::
IGDequeAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGDequeAsTable
 <Element, ElementOps>::Implementation&
IGDequeAsTable <Element, ElementOps>::
ImplOf (IGDequeAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IDequeAsTable
// ---

// public members

template <class Element>
inline
IDequeAsTable <Element>::
IDequeAsTable (INumber numberOfElements)
: IGDequeAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IDequeAsTable <Element>::
IDequeAsTable (PureImplementation& impl)
: IGDequeAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGDequeAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGDequeAsTable <Element, ElementOps>::
IVGDequeAsTable (INumber numberOfElements)
: IGDequeAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGDequeAsTable <Element, ElementOps>::
IVGDequeAsTable (PureImplementation& impl)
: IGDequeAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVDequeAsTable
// ---

// public members

template <class Element>
inline
IVDequeAsTable <Element>::
IVDequeAsTable (INumber numberOfElements)
: IVGDequeAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVDequeAsTable <Element>::
IVDequeAsTable (PureImplementation& impl)
: IVGDequeAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IDQUTAB_INL_
