// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 37 1.4.1.4 source/core/collect/ies.inl, collection, ioc.v400, 000919  
#ifndef _IES_INL_
#define _IES_INL_

// ---
// IGEqualitySequenceCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGEqualitySequenceCursor <Element, ElementOps>::
IGEqualitySequenceCursor
  (IGEqualitySequence
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGEqualitySequenceCursor <Element, ElementOps>&
IGEqualitySequenceCursor <Element, ElementOps>::
operator=
  (IGEqualitySequenceCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGEqualitySequenceCursor <Element, ElementOps>::
IGEqualitySequenceCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGEqualitySequenceMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGEqualitySequenceMngCursor <Element, ElementOps>::
IGEqualitySequenceMngCursor
  (IGEqualitySequence
    <Element, ElementOps> const& collection)
: IGEqualitySequenceCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGEqualitySequenceMngCursor <Element, ElementOps>::
IGEqualitySequenceMngCursor (ICursorImpl* impl)
: IGEqualitySequenceCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGEqualitySequence
// ---

// public members

template <class Element, class ElementOps>
inline
IGEqualitySequence <Element, ElementOps>::
IGEqualitySequence (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGEqualitySequence <Element, ElementOps>::
IGEqualitySequence
  (IGEqualitySequence
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGEqualitySequence <Element, ElementOps>&
IGEqualitySequence <Element, ElementOps>::
operator= (IGEqualitySequence
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGEqualitySequence <Element, ElementOps>::
IGEqualitySequence (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGEqualitySequence
 <Element, ElementOps>::Implementation&
IGEqualitySequence <Element, ElementOps>::
ImplOf (IGEqualitySequence
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IEqualitySequence
// ---

// public members

template <class Element>
inline
IEqualitySequence <Element>::
IEqualitySequence (INumber numberOfElements)
: IGEqualitySequence
   <Element, IEOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IEqualitySequence <Element>::
IEqualitySequence (PureImplementation& impl)
: IGEqualitySequence
   <Element, IEOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGEqualitySequence
// ---

// public members

template <class Element, class ElementOps>
inline
IVGEqualitySequence <Element, ElementOps>::
IVGEqualitySequence (INumber numberOfElements)
: IGEqualitySequence
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGEqualitySequence <Element, ElementOps>::
IVGEqualitySequence (PureImplementation& impl)
: IGEqualitySequence
    <Element, ElementOps> (impl)
{
}

// ---
// IVEqualitySequence
// ---

// public members

template <class Element>
inline
IVEqualitySequence <Element>::
IVEqualitySequence (INumber numberOfElements)
: IVGEqualitySequence
   <Element, IEOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVEqualitySequence <Element>::
IVEqualitySequence (PureImplementation& impl)
: IVGEqualitySequence
   <Element, IEOps <Element> > (impl)
{
}
#endif

#endif // _IES_INL_
