// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 56 1.4.1.4 source/core/collect/ihp.inl, collection, ioc.v400, 000919  
#ifndef _IHP_INL_
#define _IHP_INL_

// ---
// IGHeapCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGHeapCursor <Element, ElementOps>::
IGHeapCursor
  (IGHeap
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGHeapCursor <Element, ElementOps>&
IGHeapCursor <Element, ElementOps>::
operator=
  (IGHeapCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapCursor <Element, ElementOps>::
IGHeapCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGHeapMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGHeapMngCursor <Element, ElementOps>::
IGHeapMngCursor
  (IGHeap
    <Element, ElementOps> const& collection)
: IGHeapCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapMngCursor <Element, ElementOps>::
IGHeapMngCursor (ICursorImpl* impl)
: IGHeapCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGHeap
// ---

// public members

template <class Element, class ElementOps>
inline
IGHeap <Element, ElementOps>::
IGHeap (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGHeap <Element, ElementOps>::
IGHeap
  (IGHeap
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGHeap <Element, ElementOps>&
IGHeap <Element, ElementOps>::
operator= (IGHeap
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGHeap <Element, ElementOps>::
IGHeap (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGHeap
 <Element, ElementOps>::Implementation&
IGHeap <Element, ElementOps>::
ImplOf (IGHeap
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IHeap
// ---

// public members

template <class Element>
inline
IHeap <Element>::
IHeap (INumber numberOfElements)
: IGHeap
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IHeap <Element>::
IHeap (PureImplementation& impl)
: IGHeap
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGHeap
// ---

// public members

template <class Element, class ElementOps>
inline
IVGHeap <Element, ElementOps>::
IVGHeap (INumber numberOfElements)
: IGHeap
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGHeap <Element, ElementOps>::
IVGHeap (PureImplementation& impl)
: IGHeap
    <Element, ElementOps> (impl)
{
}

// ---
// IVHeap
// ---

// public members

template <class Element>
inline
IVHeap <Element>::
IVHeap (INumber numberOfElements)
: IVGHeap
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVHeap <Element>::
IVHeap (PureImplementation& impl)
: IVGHeap
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IHP_INL_
