// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 61 1.5.1.4 source/core/collect/ihptab.h, collection, ioc.v400, 000919 
#ifndef _IHPTAB_
#define _IHPTAB_

#include <ilanglvl.hpp>
#include <iahp.h>
#include <iichp.h>
#if ! defined (__INO_AVALON__)
#include <iivhp.h>
#endif
#include <iihptab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGHeapAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGHeapAsTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGHeapAsTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGHeapAsTableCursor
                            (IGHeapAsTable
                              <
                               Element, ElementOps> const&);

  IGHeapAsTableCursor <Element, ElementOps>&
           operator=        (IGHeapAsTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGHeapAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGHeapAsTableCursor\
                            (IGHeapAsTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGHeapAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGHeapAsTableMngCursor :
  public IGHeapAsTableCursor <Element, ElementOps>  {

  typedef  IGHeapAsTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGHeapAsTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGHeapAsTableMngCursor
                            (IGHeapAsTable
                              <
                               Element, ElementOps> const&);

protected:

           IGHeapAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGHeapAsTableMngCursor\
                            (IGHeapAsTable\
                              <\
                               Element, ElementOps> const&),\
           IGHeapAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGHeapAsTable :
  public IAHeap <Element> {
protected:

  typedef  IGHeapAsTable <Element, ElementOps> Self;

  typedef  IHeapAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICHeapImpl
            <PureImplementation> Implementation;
#endif

#define IGHeapAsTableTypedef(Element, ElementOps)\
  typedef  IHeapAsTableOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGHeapAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGHeapAsTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGHeapAsTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGHeapAsTable
                            (INumber = 100);

           IGHeapAsTable
                            (IGHeapAsTable
                              <
                               Element, ElementOps> const&);

  IGHeapAsTable <Element, ElementOps>&
           operator=        (IGHeapAsTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGHeapAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGHeapAsTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGHeapAsTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGHeapAsTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGHeapAsTable\
                            (INumber),\
           operator=,\
           IGHeapAsTable\
                            (PureImplementation&),\
           ImplOf           (IGHeapAsTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IHeapAsTable :
  public IGHeapAsTable
          <Element, IStdOps <Element> > {

  typedef  IGHeapAsTable
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IHeapAsTable <Element> Self;

#define IHeapAsTableTypedef(Element)\
  typedef  IHeapAsTableOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IHeapAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IHeapAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IHeapAsTable\
                              (INumber),\
           IHeapAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGHeapAsTable :
  public IGHeapAsTable
          <Element, ElementOps > {

  typedef  IVGHeapAsTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGHeapAsTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVHeapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICHeapImpl
            <IVHeapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGHeapAsTableTypedef(Element, ElementOps)\
  typedef  IHeapAsTableOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGHeapAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGHeapAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGHeapAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGHeapAsTable\
                              (INumber),\
           IVGHeapAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVHeapAsTable :
  public IVGHeapAsTable
          <Element, IStdOps <Element> > {

  typedef  IVGHeapAsTable
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVHeapAsTable <Element> Self;

#define IVHeapAsTableTypedef(Element)\
  typedef  IVHeapAsTableOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVHeapAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVHeapAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVHeapAsTable\
                              (INumber),\
           IVHeapAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ihptab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IHPTAB_
