// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 68 1.7.1.3 source/core/collect/iiacllct.h, collection, ioc.v400, 000919 
#ifndef _IIACLLCT_
#define _IIACLLCT_

#include <iiglobal.h>
#include <iexctxt.h>

#if ! defined(__INO_L1_LOCKING__)
#include <ireslock.hpp>
#endif

#if ! defined (__INO_AVALON__)
#include <iivcllct.h>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IACollectionImpl;

#pragma pack(push,4)
#pragma enum(4)

#if ! defined(__INO_L1_LOCKING__)
class IC_EXPORTC ICollectionResourceImpl 
{
public:
           ICollectionResourceImpl
                            ();

           ~ICollectionResourceImpl
                            ();

  void     lock             (long);

  void     unlock           ();

protected:
  IResource* ivResource;

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMAttribute (ivResource)
#pragma SOMReleaseOrder (\
                         lock (long),\
                         unlock (),\
                         ivResource\
                        )
#endif
};
#endif

class IC_EXPORTC ICursorImpl {
public:

           ICursorImpl      (IACollectionImpl const&);

  virtual ~ICursorImpl      ();

  virtual
  ICursorImpl*
           Clone            () const = 0;

  virtual
  void     Copy             (ICursorImpl const&) = 0;

  void const*
           Element          () const;

  virtual
  void     Invalidate       () = 0;

  bool IsFor            (IACollectionImpl const&) const;

  virtual
  bool IsInBetween      () const = 0;

  virtual
  bool IsMngCursor      () const;

  virtual
  bool IsValid          () const = 0;

  IPosition
           Position         () const;

  bool SetToFirst       ();

  bool SetToLast        ();

  bool SetToNext        ();

  void     SetToPosition    (IPosition);

  bool SetToPrevious    ();

  virtual
  bool operator==       (ICursorImpl const&) const;

  bool operator!=       (ICursorImpl const&) const;

protected:

  IACollectionImpl const*
           ivCollection;

  static
  void*    CheckPointer     (void*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMAttribute (ivCollection)
#pragma SOMReleaseOrder (\
           ICursorImpl      (IACollectionImpl const&),\
           Clone            () const,\
           Copy             (ICursorImpl const&),\
           Element          () const,\
           Invalidate       (),\
           IsFor            (IACollectionImpl const&) const,\
           IsInBetween      () const,\
           IsValid          () const,\
           Position         () const,\
           SetToFirst       (),\
           SetToLast        (),\
           SetToNext        (),\
           SetToPosition    (IPosition),\
           SetToPrevious    (),\
           operator==       (ICursorImpl const&) const,\
           operator!=       (ICursorImpl const&) const,\
           ivCollection,\
           CheckPointer     (void*),\
           IsMngCursor      () const\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
#define forMngCursors(c) \
  for (MngCursor* (c) = ivMngCursors; (c) != 0; (c) = (c)->ivNext)
#endif

class IACollectionOps;

#if ! defined (__INO_AVALON__)
class IC_EXPORTC IACollectionImpl : public IVCollectionImpl 
{
#else
class IC_EXPORTC IACollectionImpl 
{
#endif
public:

  typedef  IACollectionOps Ops;

          ~IACollectionImpl ();

  bool Add              (void const*);

  virtual
  bool Add              (void const*, ICursorImpl&) = 0;

  void     AddAsFirst       (void const*);

  virtual
  void     AddAsFirst       (void const*, ICursorImpl&); // = 0

  void     AddAsLast        (void const*);

  virtual
  void     AddAsLast        (void const*, ICursorImpl&); // = 0

  virtual
  void     AddAsNext        (void const*, ICursorImpl&); // = 0

  virtual
  void     AddAsPrevious    (void const*, ICursorImpl&); // = 0

  void     AddAtPosition    (IPosition, void const*);

  virtual
  void     AddAtPosition    (IPosition,
                             void const*, ICursorImpl&); // = 0

  bool AddOrReplaceElementWithKey
                            (void const*);

  virtual
  bool AddOrReplaceElementWithKey
                            (void const*, ICursorImpl&); // = 0

  virtual
  void     AddAllFrom       (IACollectionImpl const&);

  virtual
  bool AllElementsDo    (IApplFunc, void*);

  virtual
  void*    Any              () const = 0;

  static
  void*    CheckPointer     (void*);

  virtual
  char const*
           ClassName        () const = 0;

  virtual
  IACollectionImpl*
           Clone            () const = 0;

  virtual
  bool Contains         (void const*) const; // = 0

  virtual
  bool ContainsAllFrom  (IACollectionImpl const&) const; // = 0

  virtual
  bool ContainsAllKeysFrom
                            (IACollectionImpl const&) const; // = 0

  virtual
  bool ContainsElementWithKey
                            (void const*) const; // = 0

  virtual
  void     Copy             (IACollectionImpl const&);

  virtual
  ICursorImpl*
           CreateCursor     () const = 0;

  virtual
  ICursorImpl*
           CreateMngCursor  () const = 0;

  void     Delete           ();

  virtual
  void*    ElementAt        (ICursorImpl const&) const = 0;

  virtual
  void*    ElementAtPosition
                            (IPosition) const; // = 0

  virtual
  void*    ElementWithKey   (void const*) const; // = 0

  virtual
  void*    First            () const;

  void     Initialize       ();

  virtual
  bool IsBounded        () const = 0;

  virtual
  bool IsConsistent     () const = 0;

  virtual
  bool IsEmpty          () const;

  virtual
  bool IsFirstAt        (ICursorImpl const&) const;

  virtual
  bool IsFull           () const = 0;

  virtual
  bool IsLastAt         (ICursorImpl const&) const;

  virtual
  void*    Key              (void const*) const; // = 0

  virtual
  void*    Last             () const;

  virtual
  bool Locate           (void const*, ICursorImpl&) const; // = 0

  virtual
  bool LocateElementWithKey
                            (void const*, ICursorImpl&) const; // = 0

  virtual
  bool LocateNext       (void const*, ICursorImpl&) const; // = 0

  virtual
  bool LocateNextElementWithKey
                            (void const*, ICursorImpl&) const; // = 0

  bool LocateOrAdd      (void const*);

  virtual
  bool LocateOrAdd      (void const*, ICursorImpl&); // = 0

  bool LocateOrAddElementWithKey
                            (void const*);

  virtual
  bool LocateOrAddElementWithKey
                            (void const*, ICursorImpl&); // = 0

  virtual
  void     LockBeforeReading
                            (long = -1);

  virtual
  void     LockBeforeWriting
                            (long = -1);

  virtual
  INumber  MaxNumberOfElements
                            () const;

  virtual
  INumber  NumberOfDifferentKeys
                            () const; // = 0

  virtual
  INumber  NumberOfElements () const = 0;

  virtual
  INumber  NumberOfElementsWithKey
                            (void const*) const; // = 0

  virtual
  INumber  NumberOfOccurrences
                            (void const*) const; // = 0

  virtual
  IPosition
           PositionAt       (ICursorImpl const&) const; // = 0

  virtual
  void     Release          () const;

  virtual
  bool Remove           (void const*); // = 0

  virtual
  INumber  RemoveAll        () = 0;

  virtual
  INumber  RemoveAll        (IPredFunc, void*) = 0;

  virtual
  INumber  RemoveAllElementsWithKey
                            (void const*); // = 0

  virtual
  INumber  RemoveAllOccurrences
                            (void const*); // = 0

  virtual
  void     RemoveAt         (ICursorImpl&) = 0;


  virtual
  void     RemoveAtPosition (IPosition); // = 0

  virtual
  bool RemoveElementWithKey
                            (void const*); // = 0

  virtual
  void     RemoveFirst      ();

  virtual
  void     RemoveLast       ();

  virtual
  void     ReplaceAt        (ICursorImpl const&, void const*) = 0;

  virtual
  void     ReplaceAtPosition
                            (IPosition, void const*);

  bool ReplaceElementWithKey
                            (void const*);

  virtual
  bool ReplaceElementWithKey
                            (void const*, ICursorImpl&); // = 0

  virtual
  void     Reverse          (); // = 0

  virtual
  void     Seize            (long timeout = -1);

  virtual
  bool SetToFirst       (ICursorImpl&) const = 0;

  virtual
  bool SetToLast        (ICursorImpl&) const; // = 0

  virtual
  bool SetToNext        (ICursorImpl&) const = 0;

  virtual
  bool SetToNextWithDifferentKey
                            (ICursorImpl&) const; // = 0

  virtual
  void     SetToPosition    (IPosition, ICursorImpl&) const; // = 0

  virtual
  bool SetToPrevious    (ICursorImpl&) const; // = 0

  virtual
  void     Sort             (ICompFunc, void*); // = 0

  virtual
  void     UnlockAfterReading
                            ();

  virtual
  void     UnlockAfterWriting
                            ();


protected:

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (off)
#endif

  struct Predicate {

    static
    bool IsContained      (void const*, void*);

    static
    bool IsNotContained   (void const*, void*);

  };

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

           IACollectionImpl (Ops&);

           IACollectionImpl (IACollectionImpl const&);

  static
  inline
  ICursorImpl&
           CursorOf         (IACollectionImpl const&);

  static
  Ops&     OpsOf            (IACollectionImpl const&);
private:

  Ops*     ivOps;

  INumber  ivReferenceCount;

  ICursorImpl*
           ivDummyCursor;

#if ! defined(__INO_L1_LOCKING__)
  ICollectionResourceImpl*
           ivResource;
#endif

#if ! defined (__INO_STREAMING__)
protected:
  IStreamerPtr
           ivStreamer;

public:
  virtual
  IStreamerPtr&
           StreamerOf       (IACollectionImpl const&) = 0;

  virtual
  void     StreamIn         (IDataStream&) = 0;

  virtual
  void     StreamOut        (IDataStream&) const = 0;

  virtual
  void     AdoptStreamer    (IACollectionStreamer*) = 0;

  virtual
  void     WriteVersion     (IDataStream&) const;

  virtual
  void     ReadVersion      (IDataStream&);
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#if ! defined (__INO_STREAMING__)
#pragma SOMAttribute (ivStreamer)
#endif
#pragma SOMReleaseOrder (\
           Add              (void const*),\
           Add              (void const*, ICursorImpl&),\
           AddAllFrom       (IACollectionImpl const&),\
           AddAsFirst       (void const*),\
           AddAsFirst       (void const*, ICursorImpl&),\
           AddAsLast        (void const*),\
           AddAsLast        (void const*, ICursorImpl&),\
           AddAsNext        (void const*, ICursorImpl&),\
           AddAsPrevious    (void const*, ICursorImpl&),\
           AddAtPosition    (IPosition, void const*),\
           AddAtPosition    (IPosition, void const*, ICursorImpl&),\
           AddOrReplaceElementWithKey\
                            (void const*),\
           AddOrReplaceElementWithKey\
                            (void const*, ICursorImpl&),\
           AllElementsDo    (IApplFunc, void*),\
           Any              () const,\
           CheckPointer     (void*),\
           ClassName        () const,\
           Clone            () const,\
           Contains         (void const*) const,\
           ContainsAllFrom  (IACollectionImpl const&) const,\
           ContainsAllKeysFrom\
                            (IACollectionImpl const&) const ,\
           ContainsElementWithKey\
                            (void const*) const,\
           Copy             (IACollectionImpl const&),\
           CreateCursor     () const,\
           CreateMngCursor  () const,\
           Delete           (),\
           ElementAt        (ICursorImpl const&) const,\
           ElementAtPosition\
                            (IPosition) const,\
           ElementWithKey   (void const*) const,\
           First            () const,\
           Initialize       (),\
           IsBounded        () const,\
           IsConsistent     () const,\
           IsEmpty          () const,\
           IsFirstAt        (ICursorImpl const&) const,\
           IsFull           () const,\
           IsLastAt         (ICursorImpl const&) const,\
           Key              (void const*) const,\
           Last             () const,\
           Locate           (void const*, ICursorImpl&) const,\
           LocateElementWithKey\
                            (void const*, ICursorImpl&) const,\
           LocateNext       (void const*, ICursorImpl&) const,\
           LocateNextElementWithKey\
                            (void const*, ICursorImpl&) const,\
           LocateOrAdd      (void const*),\
           LocateOrAdd      (void const*, ICursorImpl&),\
           LocateOrAddElementWithKey\
                            (void const*),\
           LocateOrAddElementWithKey\
                            (void const*, ICursorImpl&),\
           LockBeforeReading\
                            (long),\
           LockBeforeWriting\
                            (long),\
           MaxNumberOfElements\
                            () const,\
           NumberOfDifferentKeys\
                            () const,\
           NumberOfElements () const,\
           NumberOfElementsWithKey\
                            (void const*) const,\
           NumberOfOccurrences\
                            (void const*) const,\
           PositionAt       (ICursorImpl const&) const,\
           Remove           (void const*),\
           RemoveAll        (),\
           RemoveAll        (IPredFunc, void*),\
           RemoveAllElementsWithKey\
                            (void const*),\
           RemoveAllOccurrences\
                            (void const*),\
           RemoveAt         (ICursorImpl&),\
           RemoveAtPosition (IPosition),\
           RemoveElementWithKey\
                            (void const*),\
           RemoveFirst      (),\
           RemoveLast       (),\
           ReplaceAt        (ICursorImpl const&, void const*),\
           ReplaceAtPosition\
                            (IPosition, void const*),\
           ReplaceElementWithKey\
                            (void const*),\
           ReplaceElementWithKey\
                            (void const*,ICursorImpl&),\
           Reverse          (),\
           SetToFirst       (ICursorImpl&) const,\
           SetToLast        (ICursorImpl&) const,\
           SetToNext        (ICursorImpl&) const,\
           SetToNextWithDifferentKey\
                            (ICursorImpl&) const,\
           SetToPosition    (IPosition, ICursorImpl&) const,\
           SetToPrevious    (ICursorImpl&) const,\
           Sort             (ICompFunc, void*),\
           UnlockAfterReading\
                            (),\
           UnlockAfterWriting\
                            (),\
           IACollectionImpl (Ops&),\
           CursorOf         (IACollectionImpl const&),\
           OpsOf            (IACollectionImpl const&),\
           cvExceptionText,\
           Release          () const,\
           Seize            (long)\
#if ! defined (__INO_STREAMING__)
           ,StreamerOf       (IACollectionImpl const&)\
           ,StreamIn         (IDataStream&)\
           ,StreamOut        (IDataStream&) const\
           ,AdoptStreamer    (IACollectionStreamer*)\
           ,WriteVersion     (IDataStream&) const\
           ,ReadVersion      (IDataStream&)\
           ,ivStreamer\
#endif
                        )
#endif

};

class IC_EXPORTC IACollectionOps 
{
public:

  virtual ~IACollectionOps  ();

protected:

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
                        )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#include <iiacllct.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIACLLCT_
