// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 69 1.2.1.2 source/core/collect/iiacllct.inl, collection, ioc.v400, 000919 
#ifndef _IIACLLCT_INL_
#define _IIACLLCT_INL_

#if !defined(__INO_L1_LOCKING__)
#include <iiexc.h>
// -----------------------
// ICollectionResourceImpl
// -----------------------

inline
ICollectionResourceImpl::
~ICollectionResourceImpl ()
{ delete ivResource;
}
#endif

// -----------
// ICursorImpl
// -----------

// public members

inline
ICursorImpl::
ICursorImpl (IACollectionImpl const& collection)
: ivCollection (&collection)
{
}

inline
ICursorImpl::
~ICursorImpl ()
{
}

inline void const*
ICursorImpl::
Element () const
{ return ivCollection->ElementAt (*this);
}

inline bool
ICursorImpl::
IsFor (IACollectionImpl const& collection) const
{ return (ivCollection == &collection);
}

inline IPosition
ICursorImpl::
Position () const
{ return ivCollection->PositionAt (*this);
}

inline bool
ICursorImpl::
SetToFirst ()
{ return ivCollection->SetToFirst (*this);
}

inline bool
ICursorImpl::
SetToLast ()
{ return ivCollection->SetToLast (*this);
}

inline bool
ICursorImpl::
SetToNext ()
{ return ivCollection->SetToNext (*this);
}

inline void
ICursorImpl::
SetToPosition (IPosition position)
{ ivCollection->SetToPosition (position, *this);
}

inline bool
ICursorImpl::
SetToPrevious ()
{ return ivCollection->SetToPrevious (*this);
}

inline bool
ICursorImpl::
operator== (ICursorImpl const& cursor) const
{ return (ivCollection == cursor.ivCollection);
}

inline bool
ICursorImpl::
operator!= (ICursorImpl const& cursor) const
{ return ! operator== (cursor);
}

// protected members

inline void*
ICursorImpl::
CheckPointer (void* ptr)
{ return IACollectionImpl::CheckPointer (ptr);
}

// ----------------
// IACollectionImpl
// ----------------

// public members

inline bool
IACollectionImpl::
Add (void const* element)
{ return Add (element, *ivDummyCursor);
}

inline void
IACollectionImpl::
AddAsFirst (void const* element)
{ AddAsFirst (element, CursorOf (*this));
}

inline void
IACollectionImpl::
AddAsLast (void const* element)
{ AddAsLast (element, CursorOf (*this));
}

inline void
IACollectionImpl::
AddAtPosition (IPosition position, void const* element)
{ AddAtPosition (position, element, CursorOf (*this));
}

inline bool
IACollectionImpl::
AddOrReplaceElementWithKey (void const* element)
{ return AddOrReplaceElementWithKey (element, CursorOf (*this));
}

inline bool
IACollectionImpl::
Contains (void const* element) const
{ return Locate (element, CursorOf (*this));
}

inline bool
IACollectionImpl::
LocateOrAdd (void const* element)
{ return LocateOrAdd (element, CursorOf (*this));
}

inline bool
IACollectionImpl::
LocateOrAddElementWithKey (void const* element)
{ return LocateOrAddElementWithKey (element, CursorOf (*this));
}

inline bool
IACollectionImpl::
ReplaceElementWithKey (void const* element)
{ return ReplaceElementWithKey (element, CursorOf (*this));
}

// protected members

inline
IACollectionImpl::
IACollectionImpl (IACollectionImpl const&)
{ // should never be executed
}

inline ICursorImpl&
IACollectionImpl::
CursorOf (IACollectionImpl const& collection)
{ return *collection.ivDummyCursor;
}

inline IACollectionImpl::Ops&
IACollectionImpl::
OpsOf (IACollectionImpl const& collection)
{ return *collection.ivOps;
}

#endif // _IIACLLCT_INL_
