// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 82 1.5.1.2 source/core/collect/iiamap.cpp, collection, ioc.v400, 000919  
#ifdef IC_PAGETUNE
  #define _IIAMAP_CPP_
  #include <ipagetun.h>
#endif

#include <iiamap.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif


// ---
// IAMapImpl
// ---

// public members

IAMapImpl::
~IAMapImpl ()
{
}

void
IAMapImpl::
AddAntivalence (IAMapImpl const& collection1,
                IAMapImpl const& collection2)
{
#if ! defined (__INO_INTERNAL__)
  if (&collection1 != &collection2) {
    bool hasNext;
    for (hasNext = CursorOf (collection1).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection1).SetToNext ()) {
      if (! collection2.Contains
           (collection1.ElementAt (CursorOf (collection1)))) {
        Add (collection1.ElementAt (CursorOf (collection1)),
             CursorOf (*this));
      }
    }
    for (hasNext = CursorOf (collection2).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection2).SetToNext ()) {
      if (! collection1.Contains
           (collection2.ElementAt (CursorOf (collection2)))) {
        Add (collection2.ElementAt (CursorOf (collection2)),
             CursorOf (*this));
      }
    }
  }
#endif
}

void
IAMapImpl::
AddDifference (IAMapImpl const& collection1,
               IAMapImpl const& collection2)
{
  if (this == &collection1)
    return;
  else if (this == &collection2)
    Union (collection1);
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (! collection2.Contains (element))
        Add (collection1.ElementAt (cursor1), CursorOf (*this));
    }
  }
}

void
IAMapImpl::
AddIntersection (IAMapImpl const& collection1,
                 IAMapImpl const& collection2)
{
  if (this != &collection1 && this != &collection2) {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (collection2.Contains (element)) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IAMapImpl::
AddUnion (IAMapImpl const& collection1,
          IAMapImpl const& collection2)
{
  Union (collection1);
  Union (collection2);
}

void
IAMapImpl::
Antivalence (IAMapImpl const& collection)
{
#if ! defined (__INO_INTERNAL__)
  if (this == &collection) {
    RemoveAll ();
  }
  else {
    for (CursorOf (collection).SetToFirst ();
         CursorOf (collection).IsValid ();
         CursorOf (collection).SetToNext ()) {
      if (! Contains
           (collection.ElementAt (CursorOf (collection)))) {
        Add (collection.ElementAt (CursorOf (collection)),
             CursorOf (*this));
      }
    }
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsContained, (void*)eq);
  }
#endif
}

void
IAMapImpl::
Difference (IAMapImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IAMapImpl::
Intersection (IAMapImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IAMapImpl::
Union (IAMapImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
}

bool
IAMapImpl::
operator!= (IAMapImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IAMapImpl::
IAMapImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

bool
IAMapImpl::
LocateNext (void const*, ICursorImpl& cursor) const
{ cursor.Invalidate ();
  return false;
}

bool
IAMapImpl::
LocateNextElementWithKey (void const*, ICursorImpl& cursor) const
{ cursor.Invalidate ();
  return false;
}

INumber
IAMapImpl::
NumberOfDifferentKeys () const
{ return NumberOfElements ();
}

INumber
IAMapImpl::
NumberOfElementsWithKey (void const* key) const
{ if (ContainsElementWithKey (key))
    return 1;
  else
    return 0;
}

INumber
IAMapImpl::
NumberOfOccurrences (void const* element) const
{ if (Contains (element))
    return 1;
  else
    return 0;
}

INumber
IAMapImpl::
RemoveAllOccurrences (void const* element)
{ if (Remove (element))
    return 1;
  else
    return 0;
}

INumber
IAMapImpl::
RemoveAllElementsWithKey (void const* key)
{ if (RemoveElementWithKey (key))
    return 1;
  else
    return 0;
}

bool
IAMapImpl::
SetToNextWithDifferentKey (ICursorImpl& cursor) const
{ return SetToNext (cursor);
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
