// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 90 1.5.1.2 source/core/collect/iiaset.cpp, collection, ioc.v400, 000919  
#ifdef IC_PAGETUNE
  #define _IIASET_CPP_
  #include <ipagetun.h>
#endif

#include <iiaset.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif


// ---
// IASetImpl
// ---

// public members

IASetImpl::
~IASetImpl ()
{
}

void
IASetImpl::
AddAntivalence (IASetImpl const& collection1,
                IASetImpl const& collection2)
{
#if ! defined (__INO_INTERNAL__)
  if (&collection1 != &collection2) {
    bool hasNext;
    for (hasNext = CursorOf (collection1).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection1).SetToNext ()) {
      if (! collection2.Contains
           (collection1.ElementAt (CursorOf (collection1)))) {
        Add (collection1.ElementAt (CursorOf (collection1)),
             CursorOf (*this));
      }
    }
    for (hasNext = CursorOf (collection2).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection2).SetToNext ()) {
      if (! collection1.Contains
           (collection2.ElementAt (CursorOf (collection2)))) {
        Add (collection2.ElementAt (CursorOf (collection2)),
             CursorOf (*this));
      }
    }
  }
#endif
}

void
IASetImpl::
AddDifference (IASetImpl const& collection1,
               IASetImpl const& collection2)
{
  if (this == &collection1)
    return;
  else if (this == &collection2)
    Union (collection1);
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (! collection2.Contains (element))
        Add (collection1.ElementAt (cursor1), CursorOf (*this));
    }
  }
}

void
IASetImpl::
AddIntersection (IASetImpl const& collection1,
                 IASetImpl const& collection2)
{
  if (this != &collection1 && this != &collection2) {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (collection2.Contains (element)) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IASetImpl::
AddUnion (IASetImpl const& collection1,
          IASetImpl const& collection2)
{
  Union (collection1);
  Union (collection2);
}

void
IASetImpl::
Antivalence (IASetImpl const& collection)
{
#if ! defined (__INO_INTERNAL__)
  if (this == &collection) {
    RemoveAll ();
  }
  else {
    for (CursorOf (collection).SetToFirst ();
         CursorOf (collection).IsValid ();
         CursorOf (collection).SetToNext ()) {
      if (! Contains
           (collection.ElementAt (CursorOf (collection)))) {
        Add (collection.ElementAt (CursorOf (collection)),
             CursorOf (*this));
      }
    }
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsContained, (void*)eq);
  }
#endif
}

void
IASetImpl::
Difference (IASetImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IASetImpl::
Intersection (IASetImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IASetImpl::
Union (IASetImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
}

bool
IASetImpl::
operator!= (IASetImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IASetImpl::
IASetImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

bool
IASetImpl::
LocateNext (void const*, ICursorImpl& cursor) const
{ cursor.Invalidate ();
  return false;
}

INumber
IASetImpl::
NumberOfOccurrences (void const* element) const
{ if (Contains (element))
    return 1;
  else
    return 0;
}

INumber
IASetImpl::
RemoveAllOccurrences (void const* element)
{ if (Remove (element))
    return 1;
  else
    return 0;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
