// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 46 1.5.1.2 source/core/collect/iicss.c, collection, ioc.v400, 000919 
#ifndef _IICSS_C_
#define _IICSS_C_

#include <iiexc.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// ICSortedSetImpl
// ---

// public members

template <class CInherited>
ICSortedSetImpl <CInherited>::
ICSortedSetImpl (Ops& ops, INumber numberOfElements)
: CInherited (ops, numberOfElements)
{
}

template <class CInherited>
ICSortedSetImpl <CInherited>::
ICSortedSetImpl
  (Ops& ops, ICSortedSetImpl <CInherited> const& collection)
: CInherited (ops, collection)
{
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
Add (void const* element, ICursorImpl& cursor)
{ ICHECK_IsFor      (this->ClassName (), "Add")

  bool hasBeenAdded = CInherited::Add (element, cursor);
  return hasBeenAdded;
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ ICHECK_Identical (this->ClassName (), "AddAllFrom")

  CInherited::AddAllFrom (collection);
}

template <class CInherited>
void*
ICSortedSetImpl <CInherited>::
Any () const
{ ICHECK_IsEmpty (this->ClassName (), "Any")

  return CInherited::Any ();
}

template <class CInherited>
void*
ICSortedSetImpl <CInherited>::
ElementAt (ICursorImpl const& cursor) const
{ ICHECK_IsFor        (this->ClassName (), "ElementAt")
  ICHECK_IsValid      (this->ClassName (), "ElementAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "ElementAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "ElementAt")
#endif

  return CInherited::ElementAt (cursor);
}

template <class CInherited>
void*
ICSortedSetImpl <CInherited>::
ElementAtPosition (IPosition position) const
{ ICHECK_Position (this->ClassName (), "ElementAtPosition")

  return CInherited::ElementAtPosition (position);
}

template <class CInherited>
void*
ICSortedSetImpl <CInherited>::
First () const
{ ICHECK_IsEmpty (this->ClassName (), "First")

  return CInherited::First ();
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
IsFirstAt (ICursorImpl const& cursor) const
{ ICHECK_IsFor        (this->ClassName (), "IsFirstAt")
  ICHECK_IsValid      (this->ClassName (), "IsFirstAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "IsFirstAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "IsFirstAt")
#endif

  return CInherited::IsFirstAt (cursor);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
IsLastAt (ICursorImpl const& cursor) const
{ ICHECK_IsFor        (this->ClassName (), "IsLastAt")
  ICHECK_IsValid      (this->ClassName (), "IsLastAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "IsLastAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "IsLastAt")
#endif

  return CInherited::IsLastAt (cursor);
}

template <class CInherited>
void*
ICSortedSetImpl <CInherited>::
Last () const
{ ICHECK_IsEmpty (this->ClassName (), "Last")

  return CInherited::Last ();
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
Locate (void const* element, ICursorImpl& cursor) const
{ ICHECK_IsFor (this->ClassName (), "Locate")

  return CInherited::Locate (element, cursor);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
LocateNext (void const* element, ICursorImpl& cursor) const
{ ICHECK_IsFor   (this->ClassName (), "LocateNext")
  ICHECK_IsValid (this->ClassName (), "LocateNext")
#if defined (IALL_CHECKS)
  ICHECK_Cursor  (this->ClassName (), "LocateNext")
#endif

  return CInherited::LocateNext (element, cursor);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
LocateOrAdd (void const* element, ICursorImpl& cursor)
{ ICHECK_IsFor (this->ClassName (), "LocateOrAdd")

  return CInherited::LocateOrAdd (element, cursor);
}

template <class CInherited>
IPosition
ICSortedSetImpl <CInherited>::
PositionAt (ICursorImpl const& cursor) const
{ ICHECK_IsFor        (this->ClassName (), "PositionAt")
  ICHECK_IsValid      (this->ClassName (), "PositionAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "PositionAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "PositionAt")
#endif

  return CInherited::PositionAt (cursor);
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
RemoveAt (ICursorImpl& cursor)
{ ICHECK_IsFor        (this->ClassName (), "RemoveAt")
  ICHECK_IsValid      (this->ClassName (), "RemoveAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "RemoveAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "RemoveAt")
#endif

  CInherited::RemoveAt (cursor);
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
RemoveAtPosition (IPosition position)
{ ICHECK_Position (this->ClassName (), "RemoveAtPosition")

  CInherited::RemoveAtPosition (position);
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
RemoveFirst ()
{ ICHECK_IsEmpty (this->ClassName (), "RemoveFirst")

  CInherited::RemoveFirst ();
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
RemoveLast ()
{ ICHECK_IsEmpty (this->ClassName (), "RemoveLast")

  CInherited::RemoveLast ();
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ ICHECK_IsFor        (this->ClassName (), "ReplaceAt")
  ICHECK_IsValid      (this->ClassName (), "ReplaceAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "ReplaceAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "ReplaceAt")
#endif
  ICHECK_Replacement  (this->ClassName (), "ReplaceAt")

  CInherited::ReplaceAt (cursor, element);
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
ReplaceAtPosition (IPosition position, void const* element)
{ ICHECK_Position    (this->ClassName (), "ReplaceAtPosition")
  ICursorImpl& cursor = this->CursorOf (*this);
  this->SetToPosition (position, cursor);
  ICHECK_Replacement (this->ClassName (), "ReplaceAtPosition")

  CInherited::ReplaceAtPosition (position, element);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
SetToFirst (ICursorImpl& cursor) const
{ ICHECK_IsFor (this->ClassName (), "SetToFirst")

  return CInherited::SetToFirst (cursor);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
SetToLast (ICursorImpl& cursor) const
{ ICHECK_IsFor (this->ClassName (), "SetToLast")

  return CInherited::SetToLast (cursor);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
SetToNext (ICursorImpl& cursor) const
{ ICHECK_IsFor   (this->ClassName (), "SetToNext")
  ICHECK_IsValid (this->ClassName (), "SetToNext")
#if defined (IALL_CHECKS)
  ICHECK_Cursor  (this->ClassName (), "SetToNext")
#endif

  return CInherited::SetToNext (cursor);
}

template <class CInherited>
void
ICSortedSetImpl <CInherited>::
SetToPosition (IPosition position, ICursorImpl& cursor) const
{ ICHECK_IsFor     (this->ClassName (), "SetToPosition")
  ICHECK_Position  (this->ClassName (), "SetToPosition")

  CInherited::SetToPosition (position, cursor);
}

template <class CInherited>
bool
ICSortedSetImpl <CInherited>::
SetToPrevious (ICursorImpl& cursor) const
{ ICHECK_IsFor   (this->ClassName (), "SetToPrevious")
  ICHECK_IsValid (this->ClassName (), "SetToPrevious")
#if defined (IALL_CHECKS)
  ICHECK_Cursor  (this->ClassName (), "SetToPrevious")
#endif

  return CInherited::SetToPrevious (cursor);
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IICSS_C_
