// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 07 1.5.1.4 source/core/collect/iiseqlst.h, collection, ioc.v400, 000919 
#ifndef _IISEQLST_
#define _IISEQLST_

#include <iiaseq.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (off)
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element>
struct ISequenceAsListNodeImpl;

#pragma enum (4)
#pragma pack (push,4)

template <class Element>
struct ISequenceAsListNodeImpl {

  ISequenceAsListNodeImpl <Element>
          *ivNext,
          *ivPrevious;

  Element  ivElement;

           ISequenceAsListNodeImpl
                            (Element const&);

          ~ISequenceAsListNodeImpl
                            ();

#if ! defined(__INO_STREAMING__)
  void     StreamIn         (IACollectionStreamer*, IDataStream&);
#endif

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ();
#endif

};

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

class IC_EXPORTC ISequenceAsListCursorImpl : public ICursorImpl {

  typedef  ICursorImpl Inherited;

  typedef  ISequenceAsListCursorImpl Self;

public:

           ISequenceAsListCursorImpl
                            (IACollectionImpl const&);

           ISequenceAsListCursorImpl
                            (ISequenceAsListCursorImpl const&);

          ~ISequenceAsListCursorImpl
                            ();

  ICursorImpl*
           Clone            () const;

  void     Copy             (ICursorImpl const&);

  void     Invalidate       ();

  bool IsInBetween      () const;

  bool IsValid          () const;

  void     operator=        (ISequenceAsListCursorImpl const&);

  bool operator==       (ICursorImpl const&) const;

protected:

private:

  ISequenceAsListNodeImpl <long>*
           ivNode;

  friend
  class    ISequenceAsListImpl;

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISequenceAsListCursorImpl\
                            (IACollectionImpl const&),\
           operator=\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
class IC_EXPORTC ISequenceAsListMngCursorImpl : public ISequenceAsListCursorImpl {

  typedef  ISequenceAsListCursorImpl Inherited;

  typedef  ISequenceAsListMngCursorImpl Self;

public:

           ISequenceAsListMngCursorImpl
                            (ISequenceAsListImpl const&);

          ~ISequenceAsListMngCursorImpl
                            ();

  ICursorImpl*
           Clone            () const;

  void     Copy             (ICursorImpl const&);

  bool IsMngCursor      () const;

  bool IsInBetween      () const;

  void     operator=        (Self const&);

protected:

private:

  static
  ISequenceAsListNodeImpl <long> const*const
           kIsInBetween;

  ISequenceAsListImpl*
           ivMngCollection;

  Self*    ivNext;

  ISequenceAsListNodeImpl <long>*
           ivNextNode;

  ISequenceAsListNodeImpl <long>*
           ivPreviousNode;

  friend
  class    ISequenceAsListImpl;

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISequenceAsListMngCursorImpl\
                            (ISequenceAsListImpl const&),\
           operator=,\
           kIsInBetween\
                        )
#endif

};
#endif

class IASequenceAsListOps;

class IC_EXPORTC ISequenceAsListImpl : public IASequenceImpl {

  typedef  IASequenceImpl Inherited;

  typedef  ISequenceAsListImpl Self;

  typedef  ISequenceAsListNodeImpl <long> Node;

public:

  typedef  IASequenceAsListOps Ops;

  typedef  ISequenceAsListCursorImpl Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  ISequenceAsListMngCursorImpl MngCursor;
#else
  typedef  ISequenceAsListCursorImpl MngCursor;
#endif

           ISequenceAsListImpl
                            (Ops&, INumber);

           ISequenceAsListImpl
                            (Ops&,
                             ISequenceAsListImpl const&);

          ~ISequenceAsListImpl
                            ();

  bool Add              (void const*, ICursorImpl&);

  void     AddAllFrom       (IACollectionImpl const&);

  void     AddAsFirst       (void const*, ICursorImpl&);

  void     AddAsLast        (void const*, ICursorImpl&);

  void     AddAsNext        (void const*, ICursorImpl&);

  void     AddAsPrevious    (void const*, ICursorImpl&);

  void     AddAtPosition    (IPosition, void const*, ICursorImpl&);

  bool AllElementsDo    (IApplFunc, void*);

  void*    Any              () const;

  bool CheckCursor      (ICursorImpl const&) const;

  char const*
           ClassName        () const;

  IACollectionImpl*
           Clone            () const;

  void     Copy             (IACollectionImpl const&);

  Cursor*  CreateCursor     () const;

  MngCursor*
           CreateMngCursor  () const;

  void*    ElementAt        (ICursorImpl const&) const;

  void*    ElementAtPosition
                            (IPosition) const;

  void*    First            () const;

  bool IsBounded        () const;

  bool IsConsistent     () const;

  bool IsEmpty          () const;

  bool IsFirstAt        (ICursorImpl const&) const;

  bool IsFull           () const;

  bool IsLastAt         (ICursorImpl const&) const;

  void*    Last             () const;

  INumber  NumberOfElements () const;

  IPosition
           PositionAt       (ICursorImpl const&) const;

  INumber  RemoveAll        ();

  INumber  RemoveAll        (IPredFunc, void*);

  void     RemoveAt         (ICursorImpl&);

  void     RemoveAtPosition (IPosition);

  void     RemoveFirst      ();

  void     RemoveLast       ();

  void     ReplaceAt        (ICursorImpl const&, void const*);

  void     ReplaceAtPosition
                            (IPosition, void const*);

  void     Reverse          ();

  bool SetToFirst       (ICursorImpl&) const;

  bool SetToLast        (ICursorImpl&) const;

  bool SetToNext        (ICursorImpl&) const;

  void     SetToPosition    (IPosition, ICursorImpl&) const;

  bool SetToPrevious    (ICursorImpl&) const;

  void     Sort             (ICompFunc, void*);

  void     operator=        (ISequenceAsListImpl const&);

protected:

  void     Assign           (void*, void const*) const;

  void*    CreateNode       (void const*) const;

  void     DeleteNode       (void*) const;


private:

  enum { kOriginalVersion };

  static
  char const*
           cvClassName;

  MngCursor*
           ivMngCursors;

  Node*    ivFirst;

  Node*    ivLast;

  INumber  ivNumberOfElements;

  void     AddAsNextNode    (Node*, Node*);

  void*    ElementOf        (Node const*) const;

  static
  bool IsInBetween      (ICursorImpl const&);

  static
  Node*&   NextNodeOf       (ICursorImpl const&);

  static
  Node*&   NodeOf           (ICursorImpl const&);

  static
  Ops&     OpsOf            (ISequenceAsListImpl const&);

  static
  Node*&   PreviousNodeOf   (ICursorImpl const&);

  void     RemoveNode       (Node*);

  void     SetInBetween     (MngCursor&) const;

  void     SetNodeToPosition
                            (IPosition, Node*&) const;

  friend
  class    ISequenceAsListMngCursorImpl;

#if ! defined (__INO_STREAMING__)
protected:
  void*    CreateNodeWithStreamer
                              (IDataStream&) const;

public:
  IStreamerPtr&
           StreamerOf       (IACollectionImpl const&);

  void     StreamIn         (IDataStream&);

  void     StreamOut        (IDataStream&) const;

  void     AdoptStreamer    (IACollectionStreamer*);
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISequenceAsListImpl\
                            (Ops&, INumber),\
           ISequenceAsListImpl\
                            (Ops&, ISequenceAsListImpl const&),\
           CheckCursor      (ICursorImpl const&) const,\
           operator=,\
           Assign           (void*, void const*) const,\
           CreateNode       (void const*) const,\
           DeleteNode       (void*) const,\
           cvClassName,\
           AddAsNextNode    (Node*, Node*),\
           ElementOf        (Node const*) const,\
           IsInBetween      (ICursorImpl const&),\
           NextNodeOf       (ICursorImpl const&),\
           NodeOf           (ICursorImpl const&),\
           OpsOf            (ISequenceAsListImpl const&),\
           PreviousNodeOf   (ICursorImpl const&),\
           RemoveNode       (Node*),\
           SetInBetween     (MngCursor&) const,\
           SetNodeToPosition\
                            (IPosition, Node*&) const\
#if ! defined (__INO_STREAMING__)
           ,CreateNodeWithStreamer \
                              (IDataStream&) const\
           ,StreamerOf       (IACollectionImpl const&)\
           ,StreamIn         (IDataStream&)\
           ,StreamOut        (IDataStream&) const\
           ,AdoptStreamer    (IACollectionStreamer*)\
#endif
                        )
#endif

};

class IC_EXPORTC IASequenceAsListOps {
public:

  virtual ~IASequenceAsListOps
                            ();

  virtual
  void     Assign           (void*, void const*) const = 0;

  virtual
  IACollectionImpl*
           Clone            () const = 0;

  virtual
  void*    CreateNode       (void const*) const = 0;

  virtual
  void     DeleteNode       (void*) const = 0;


protected:

           IASequenceAsListOps
                            (bool);

private:

  bool ivContainsDTSObjects;

  friend
  class    ISequenceAsListImpl;

#if ! defined (__INO_STREAMING__)
public:
  virtual
  void*    CreateNodeWithStreamer
                            (IACollectionStreamer*,
                             IDataStream&) const = 0;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMReleaseOrder (\
           Assign           (void*, void const*) const,\
           Clone            () const,\
           CreateNode       (void const*) const,\
           DeleteNode       (void*) const,\
           IASequenceAsListOps\
                            (bool)\
#if ! defined (__INO_STREAMING__)
           ,CreateNodeWithStreamer \
                            (IACollectionStreamer*, IDataStream&) const\
#endif
                        )
#endif

};

template <class Element, class ElementOps, class Implementation>
class IC_EXPORTC ISequenceAsListOps : public IASequenceAsListOps {

  typedef  IASequenceAsListOps Inherited;

  typedef  ISequenceAsListOps
            <Element, ElementOps, Implementation> Self;

  typedef  ISequenceAsListNodeImpl <Element> Node;

public:

  typedef  Implementation ImplType;

  typedef  Element ElementType;

  typedef  void KeyType;

           ISequenceAsListOps
                            (INumber);

           ISequenceAsListOps
                            (INumber, void*);

           ISequenceAsListOps
                            (ISequenceAsListOps
                              <Element, ElementOps,
                               Implementation> const&);

  void     Assign           (void*, void const*) const;

  IACollectionImpl*
           Clone            () const;

  void*    CreateNode       (void const*) const;

  void     DeleteNode       (void*) const;

           operator Implementation*
                            ();

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ();

  static
  void     os_mark_schema_type ()
           {
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
             OS_MARK_SCHEMA_TYPESPEC((Self::Node));
#else
             { Self* _OSMT_ = 0; }
             { Self::Node* _OSMT_ = 0; }
#endif
           }
#endif

protected:

private:

  ElementOps
           ivElementOps;

  Implementation
           ivImpl;

#if ! defined (__INO_STREAMING__)
public:
  void*    CreateNodeWithStreamer
                            (IACollectionStreamer*, IDataStream&) const;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMReleaseOrder (\
           ISequenceAsListOps\
                            (INumber),\
           ISequenceAsListOps\
                            (INumber, void*),\
           operator Implementation*\
                            ()\
                        )
#endif

};

#pragma enum (pop)
#pragma pack (pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#include <iiseqlst.inl>

#if defined (_AIX)        || defined (__MVS__)   ||\
    defined (__OS2__)     || defined (__OS400__) ||\
    defined (__WINDOWS__)

#ifndef IC_BUILD
#include <iiseqlst.c>
#else
#if ! defined (__TEMPINC__)
#include <iiseqlst.c>
#endif
#endif

#endif

#endif // _IISEQLST_
