// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 11 1.5.1.3 source/core/collect/iiseqtab.h, collection, ioc.v400, 000919 
#ifndef _IISEQTAB_
#define _IISEQTAB_

#include <iiaseq.h>
#include <iitable.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (off)
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum (4)
#pragma pack (push,4)

template <class Element>
struct ISequenceAsTableNodeImpl {

  Element  ivElement;

           ISequenceAsTableNodeImpl
                            (Element const&);

          ~ISequenceAsTableNodeImpl
                            ();

#if ! defined(__INO_STREAMING__)
  void     StreamIn         (IACollectionStreamer*, IDataStream&);
#endif

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ();
#endif

};

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

class IC_EXPORTC ISequenceAsTableCursorImpl : public ICursorImpl {

  typedef  ICursorImpl Inherited;

  typedef  ISequenceAsTableCursorImpl Self;

public:

           ISequenceAsTableCursorImpl
                            (IACollectionImpl const&);

           ISequenceAsTableCursorImpl
                            (ISequenceAsTableCursorImpl const&);

          ~ISequenceAsTableCursorImpl
                            ();

  ICursorImpl*
           Clone            () const;

  void     Copy             (ICursorImpl const&);

  void     Invalidate       ();

  bool IsInBetween      () const;

  bool IsValid          () const;

  void     operator=        (ISequenceAsTableCursorImpl const&);

  bool operator==       (ICursorImpl const&) const;

protected:

private:

  INumber  ivIndex;

  friend
  class    ISequenceAsTableImpl;

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISequenceAsTableCursorImpl\
                            (IACollectionImpl const&),\
           operator=\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
class IC_EXPORTC ISequenceAsTableMngCursorImpl :
  public ISequenceAsTableCursorImpl {

  typedef  ISequenceAsTableCursorImpl Inherited;

  typedef  ISequenceAsTableMngCursorImpl Self;

public:

           ISequenceAsTableMngCursorImpl
                            (ISequenceAsTableImpl const&);

           ~ISequenceAsTableMngCursorImpl
                            ();

  ICursorImpl*
           Clone            () const;

  void     Copy             (ICursorImpl const&);

  bool IsInBetween      () const;

  bool IsMngCursor      () const;

  void     operator=        (ISequenceAsTableMngCursorImpl const&);

protected:

private:

  static
  INumber const
           kIsInBetween;

  ISequenceAsTableImpl*
           ivMngCollection;

  Self*    ivNext;

  INumber  ivNextIndex,
           ivPreviousIndex;

  friend
  class    ISequenceAsTableImpl;

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISequenceAsTableMngCursorImpl\
                            (ISequenceAsTableImpl const&),\
           operator=,\
           kIsInBetween\
                        )
#endif

};
#endif

class IASequenceAsTableOps;

class IC_EXPORTC ISequenceAsTableImpl : public IASequenceImpl {

  typedef  IASequenceImpl Inherited;

  typedef  ISequenceAsTableImpl Self;

public:

  typedef  IASequenceAsTableOps Ops;

  typedef  ISequenceAsTableCursorImpl Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  ISequenceAsTableMngCursorImpl MngCursor;
#else
  typedef  ISequenceAsTableCursorImpl MngCursor;
#endif

           ISequenceAsTableImpl
                            (Ops&, INumber);

           ISequenceAsTableImpl
                            (Ops&,
                             ISequenceAsTableImpl const&);

          ~ISequenceAsTableImpl
                            ();

  bool Add              (void const*, ICursorImpl&);

  void     AddAllFrom       (IACollectionImpl const&);

  void     AddAsFirst       (void const*, ICursorImpl&);

  void     AddAsLast        (void const*, ICursorImpl&);

  void     AddAsNext        (void const*, ICursorImpl&);

  void     AddAsPrevious    (void const*, ICursorImpl&);

  void     AddAtPosition    (IPosition, void const*, ICursorImpl&);

  bool AllElementsDo    (IApplFunc, void*);

  void*    Any              () const;

  bool CheckCursor      (ICursorImpl const&) const;

  char const*
           ClassName        () const;

  IACollectionImpl*
           Clone            () const;

  void     Copy             (IACollectionImpl const&);

  Cursor*  CreateCursor     () const;

  MngCursor*
           CreateMngCursor  () const;

  void*    ElementAt        (ICursorImpl const&) const;

  void*    ElementAtPosition
                            (IPosition) const;

  void*    First            () const;

  bool IsBounded        () const;

  bool IsConsistent     () const;

  bool IsEmpty          () const;

  bool IsFirstAt        (ICursorImpl const&) const;

  bool IsFull           () const;

  bool IsLastAt         (ICursorImpl const&) const;

  void*    Last             () const;

  INumber  NumberOfElements () const;

  IPosition
           PositionAt       (ICursorImpl const&) const;

  INumber  RemoveAll        ();

  INumber  RemoveAll        (IPredFunc, void*);

  void     RemoveAt         (ICursorImpl&);

  void     RemoveAtPosition (IPosition);

  void     RemoveFirst      ();

  void     RemoveLast       ();

  void     ReplaceAt        (ICursorImpl const&, void const*);

  void     ReplaceAtPosition
                            (IPosition, void const*);

  void     Reverse          ();

  bool SetToFirst       (ICursorImpl&) const;

  bool SetToLast        (ICursorImpl&) const;

  bool SetToNext        (ICursorImpl&) const;

  void     SetToPosition    (IPosition, ICursorImpl&) const;

  bool SetToPrevious    (ICursorImpl&) const;

  void     Sort             (ICompFunc, void*);

  void     operator=        (ISequenceAsTableImpl const&);

protected:

  void     Assign           (void*, void const*) const;

  void     BlockLeft        (INumber, INumber);

  void     BlockRight       (INumber, INumber);

  void     CopyFrom         (ITableImpl const&);

  void     ConstructNode    (void*, void const*) const;

  void*    CreateNodes      (INumber, size_t&);

  void     DeleteNodes      (void*);

  void     DestroyNodes     (void*, INumber = 1) const;

  void     Swap             (INumber, INumber) const;

private:

  enum { kOriginalVersion };

  static
  char const*
           cvClassName;

  MngCursor*
           ivMngCursors;

  ITableImpl
           ivTable;

  void     GrowBy           (INumber);

  void*    ElementOf        (void const*) const;

  static
  INumber& IndexOf          (ICursorImpl const&);

  void     Initialize       (INumber);

  static
  bool IsInBetween      (ICursorImpl const&);

  static
  INumber& NextIndexOf      (ICursorImpl const&);

  static
  Ops&     OpsOf            (ISequenceAsTableImpl const&);

  static
  INumber& PreviousIndexOf  (ICursorImpl const&);

  void     RemoveAtIndex    (INumber);

  void     SetInBetween     (MngCursor&) const;

  void     Quicksort        (ICompFunc, void*, INumber, INumber);

  void     UpdateMngCursorsForAssign
                            (INumber, INumber) const;

  void     UpdateMngCursorsForCopyFrom
                            (INumber, INumber) const;

  void     UpdateMngCursorsForGrowBy
                            () const;

  void     UpdateMngCursorsForPredRemove
                            (INumber, INumber) const;

  void     UpdateMngCursorsForRemoveOf
                            (INumber) const;

  void     UpdateMngCursorsForShiftLeftOf
                            (INumber) const;

  void     UpdateMngCursorsForShiftRightOf
                            (INumber) const;

  void     UpdateMngCursorsForSwap
                            (INumber, INumber) const;

  friend
  class    ISequenceAsTableMngCursorImpl;

  friend
  class    IASequenceAsTableOps;

#if ! defined (__INO_STREAMING__)
protected:
  void     AddAsLastFromStreamer
                            (IDataStream&, ICursorImpl&);

  void     ConstructNodeWithStreamer
                            (void*, IDataStream&) const;

public:
  IStreamerPtr&
           StreamerOf       (IACollectionImpl const&);

  void     StreamIn         (IDataStream&);

  void     StreamOut        (IDataStream&) const;

  void     AdoptStreamer    (IACollectionStreamer*);
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISequenceAsTableImpl\
                            (Ops&, INumber),\
           ISequenceAsTableImpl\
                            (Ops&, ISequenceAsTableImpl const&),\
           CheckCursor      (ICursorImpl const&) const,\
           operator=,\
           Assign           (void*, void const*) const,\
           BlockLeft        (INumber, INumber),\
           BlockRight       (INumber, INumber),\
           CopyFrom         (ITableImpl const&),\
           ConstructNode    (void*, void const*) const,\
           CreateNodes      (INumber, size_t&),\
           DeleteNodes      (void*),\
           DestroyNodes     (void*, INumber) const,\
           Swap             (INumber, INumber) const,\
           cvClassName,\
           ElementOf        (void const*) const,\
           GrowBy           (INumber),\
           IndexOf          (ICursorImpl const&),\
           Initialize       (INumber),\
           IsInBetween      (ICursorImpl const&),\
           NextIndexOf      (ICursorImpl const&),\
           OpsOf            (ISequenceAsTableImpl const&),\
           PreviousIndexOf  (ICursorImpl const&),\
           RemoveAtIndex    (INumber),\
           SetInBetween     (MngCursor&) const,\
           Quicksort        (ICompFunc, void*, INumber, INumber),\
           UpdateMngCursorsForAssign\
                            (INumber, INumber) const,\
           UpdateMngCursorsForCopyFrom\
                            (INumber, INumber) const,\
           UpdateMngCursorsForGrowBy\
                            () const,\
           UpdateMngCursorsForPredRemove\
                            (INumber, INumber) const,\
           UpdateMngCursorsForRemoveOf\
                            (INumber) const,\
           UpdateMngCursorsForShiftLeftOf\
                            (INumber) const,\
           UpdateMngCursorsForShiftRightOf\
                            (INumber) const,\
           UpdateMngCursorsForSwap\
                            (INumber, INumber) const\
#if ! defined (__INO_STREAMING__)
           ,AddAsLastFromStreamer \
                            (IDataStream&, ICursorImpl&)\
           ,ConstructNodeWithStreamer \
                            (void*, IDataStream&) const\
           ,StreamerOf       (IACollectionImpl const&)\
           ,StreamIn         (IDataStream&)\
           ,StreamOut        (IDataStream&) const\
           ,AdoptStreamer    (IACollectionStreamer*)\
#endif
                       )
#endif

};

class IC_EXPORTC IASequenceAsTableOps {
public:

  virtual ~IASequenceAsTableOps
                            ();

  virtual
  void     Assign           (void*, void const*) const = 0;

  virtual
  void     BlockLeft        (INumber, INumber) = 0;

  virtual
  void     BlockRight       (INumber, INumber) = 0;

  virtual
  IACollectionImpl*
           Clone            () const = 0;

  virtual
  void     CopyFrom         (ITableImpl const&) = 0;

  virtual
  void     ConstructNode    (void*, void const*) const = 0;

  virtual
  void*    CreateNodes      (INumber, size_t&) = 0;

  virtual
  void     DeleteNodes      (void*) = 0;

  virtual
  void     DestroyNodes     (void*, INumber = 1) const = 0;

  virtual
  void     Swap             (INumber, INumber) const = 0;

protected:

           IASequenceAsTableOps
                            (bool);

  static
  ITableImpl&
           TableOf          (ISequenceAsTableImpl const&);

  void     UpdateMngCursorsForAssign
                            (ISequenceAsTableImpl const&,
                             INumber, INumber) const;

  void     UpdateMngCursorsForCopyFrom
                            (ISequenceAsTableImpl const&,
                             INumber, INumber) const;

  void     UpdateMngCursorsForPredRemove
                            (ISequenceAsTableImpl const&,
                             INumber, INumber) const;

  void     UpdateMngCursorsForRemoveOf
                            (ISequenceAsTableImpl const&,
                             INumber) const;

  void     UpdateMngCursorsForShiftLeftOf
                            (ISequenceAsTableImpl const&,
                             INumber) const;

  void     UpdateMngCursorsForShiftRightOf
                            (ISequenceAsTableImpl const&,
                             INumber) const;

  void     UpdateMngCursorsForSwap
                            (ISequenceAsTableImpl const&,
                             INumber, INumber) const;

private:

  bool ivContainsDTSObjects;

  friend
  class    ISequenceAsTableImpl;

#if ! defined (__INO_STREAMING__)
public:
  virtual
  void     ConstructNodeWithStreamer
                         (void*, IACollectionStreamer*, IDataStream&) const = 0;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMReleaseOrder (\
           Assign           (void*, void const*) const,\
           BlockLeft        (INumber, INumber),\
           BlockRight       (INumber, INumber),\
           Clone            () const,\
           CopyFrom         (ITableImpl const&),\
           ConstructNode    (void*, void const*) const,\
           CreateNodes      (INumber, size_t&),\
           DeleteNodes      (void*),\
           DestroyNodes     (void*, INumber) const,\
           Swap             (INumber, INumber) const,\
           IASequenceAsTableOps\
                            (bool),\
           TableOf          (ISequenceAsTableImpl const&),\
           UpdateMngCursorsForAssign\
                            (ISequenceAsTableImpl const&,\
                             INumber, INumber) const,\
           UpdateMngCursorsForCopyFrom\
                            (ISequenceAsTableImpl const&,\
                             INumber, INumber) const,\
           UpdateMngCursorsForPredRemove\
                            (ISequenceAsTableImpl const&,\
                             INumber, INumber) const,\
           UpdateMngCursorsForRemoveOf\
                            (ISequenceAsTableImpl const&,\
                             INumber) const,\
           UpdateMngCursorsForShiftLeftOf\
                            (ISequenceAsTableImpl const&,\
                             INumber) const,\
           UpdateMngCursorsForShiftRightOf\
                            (ISequenceAsTableImpl const&,\
                             INumber) const,\
           UpdateMngCursorsForSwap\
                            (ISequenceAsTableImpl const&,\
                             INumber, INumber) const\
#if ! defined (__INO_STREAMING__)
           ,ConstructNodeWithStreamer \
                         (void*, IACollectionStreamer*, IDataStream&) const\
#endif
                        )
#endif

};

template <class Element, class ElementOps, class Implementation>
class IC_EXPORTC ISequenceAsTableOps : public IASequenceAsTableOps {

  typedef  IASequenceAsTableOps Inherited;

  typedef  ISequenceAsTableOps
            <Element, ElementOps, Implementation> Self;

  typedef  ISequenceAsTableNodeImpl <Element> Node;

public:

  typedef  Implementation ImplType;

  typedef  Element ElementType;

  typedef  void KeyType;

           ISequenceAsTableOps
                            (INumber);

           ISequenceAsTableOps
                            (INumber, void*);

           ISequenceAsTableOps
                            (ISequenceAsTableOps
                              <Element, ElementOps,
                               Implementation> const&);

          ~ISequenceAsTableOps
                            ();

  void     Assign           (void*, void const*) const;

  void     BlockLeft        (INumber, INumber);

  void     BlockRight       (INumber, INumber);

  IACollectionImpl*
           Clone            () const;

  void     CopyFrom         (ITableImpl const&);

  void     ConstructNode    (void*, void const*) const;

  void*    CreateNodes      (INumber, size_t&);

  void     DeleteNodes      (void*);

  void     DestroyNodes     (void*, INumber) const;

  void     Swap             (INumber, INumber) const;

           operator Implementation*
                            ();

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ();

  static
  void     os_mark_schema_type ()
           {
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
             OS_MARK_SCHEMA_TYPESPEC((Self::Node));
#else
             { Self* _OSMT_ = 0; }
             { Self::Node* _OSMT_ = 0; }
#endif
           }
#endif

protected:

private:

  ElementOps
           ivElementOps;

  Implementation
           ivImpl;

#if ! defined (__INO_STREAMING__)
public:
  void     ConstructNodeWithStreamer
                            (void*, IACollectionStreamer*, IDataStream&) const;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMReleaseOrder (\
           ISequenceAsTableOps\
                            (INumber),\
           ISequenceAsTableOps\
                            (INumber, void*),\
           operator Implementation*\
                            ()\
                        )
#endif

};

#pragma pack (pop)
#pragma enum (pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#include <iiseqtab.inl>

#if defined (_AIX)        || defined (__MVS__)   ||\
    defined (__OS2__)     || defined (__OS400__) ||\
    defined (__WINDOWS__)
#if ! defined (__TEMPINC__)
#include <iiseqtab.c>
#endif
#endif

#endif // _IISEQTAB_
