// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 01 1.5.1.4 source/core/collect/iivkss.c, collection, ioc.v400, 000919 
#ifndef _IIVKSS_C_
#define _IIVKSS_C_

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// IVKeySortedSetImpl
// ---

// public members

template <class VInherited>
IVKeySortedSetImpl <VInherited>::
IVKeySortedSetImpl (Ops& ops, INumber numberOfElements)
: VInherited (ops, numberOfElements)
{
}

template <class VInherited>
IVKeySortedSetImpl <VInherited>::
IVKeySortedSetImpl
  (Ops& ops, IVKeySortedSetImpl <VInherited> const& collection)
: VInherited (ops, collection)
{
}

template <class VInherited>
bool
IVKeySortedSetImpl <VInherited>::
Add (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenAdded = VInherited::Add (element, cursor);
  this->EnableNotification (isEnabled);
  if (hasBeenAdded) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAllFrom (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
bool
IVKeySortedSetImpl <VInherited>::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ bool hasBeenAdded = false;
  if (VInherited::LocateElementWithKey (this->Key (element), cursor)) {
#if defined (ICLCC_COMPAT_IVB)
    this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
    this->NotifyOfReplaceAt (cursor, element);
#endif
    VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
    this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
  }
  else {
    hasBeenAdded = VInherited::Add (element, cursor);
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
Copy (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Copy (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
bool
IVKeySortedSetImpl <VInherited>::
LocateOrAddElementWithKey (void const* key, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenLocated =
    VInherited::LocateOrAddElementWithKey (key, cursor);
  this->EnableNotification (isEnabled);
  if (!hasBeenLocated) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenLocated;
}

template <class VInherited>
INumber
IVKeySortedSetImpl <VInherited>::
RemoveAll ()
{ INumber result = VInherited::RemoveAll ();
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVKeySortedSetImpl <VInherited>::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber result = VInherited::RemoveAll (predFunc, addArg);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVKeySortedSetImpl <VInherited>::
RemoveAllElementsWithKey (void const* key)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  INumber result = VInherited::RemoveAllElementsWithKey (key);
  this->EnableNotification (isEnabled);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
RemoveAt (ICursorImpl& cursor)
{ this->NotifyOfRemoveAt (cursor);
  VInherited::RemoveAt (cursor);
}

template <class VInherited>
bool
IVKeySortedSetImpl <VInherited>::
RemoveElementWithKey (void const* key)
{ if (this->LocateElementWithKey (key, this->CursorOf (*this))) {
    this->NotifyOfRemoveAt (this->CursorOf (*this));
  }
  return VInherited::RemoveElementWithKey (key);
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
RemoveAtPosition (IPosition position)
{ this->SetToPosition (position, this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveAtPosition (position);
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
RemoveFirst ()
{ this->SetToFirst (this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveFirst ();
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
RemoveLast ()
{ this->SetToLast (this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveLast ();
}

template <class VInherited>
void
IVKeySortedSetImpl <VInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
}

template <class VInherited>
bool
IVKeySortedSetImpl <VInherited>::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  bool hasBeenReplaced =
    VInherited::ReplaceElementWithKey (element, cursor);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
  return hasBeenReplaced;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIVKSS_C_
