// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 11 1.5.1.3 source/core/collect/iivseq.c, collection, ioc.v400, 000919 
#ifndef _IIVSEQ_C_
#define _IIVSEQ_C_

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// IVSequenceImpl
// ---

// public members

template <class VInherited>
IVSequenceImpl <VInherited>::
IVSequenceImpl (Ops& ops, INumber numberOfElements)
: VInherited (ops, numberOfElements)
{
}

template <class VInherited>
IVSequenceImpl <VInherited>::
IVSequenceImpl
  (Ops& ops, IVSequenceImpl <VInherited> const& collection)
: VInherited (ops, collection)
{
}

template <class VInherited>
bool
IVSequenceImpl <VInherited>::
Add (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenAdded = VInherited::Add (element, cursor);
  this->EnableNotification (isEnabled);
  if (hasBeenAdded) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAllFrom (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
AddAsFirst (void const* element, ICursorImpl& cursor)
{ VInherited::AddAsFirst (element, cursor);
  this->NotifyOfAddAt (cursor);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
AddAsLast (void const* element, ICursorImpl& cursor)
{ VInherited::AddAsLast (element, cursor);
  this->NotifyOfAddAt (cursor);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
AddAsNext (void const* element, ICursorImpl& cursor)
{ VInherited::AddAsNext (element, cursor);
  this->NotifyOfAddAt (cursor);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
AddAsPrevious (void const* element, ICursorImpl& cursor)
{ VInherited::AddAsPrevious (element, cursor);
  this->NotifyOfAddAt (cursor);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
AddAtPosition (IPosition position,
               void const* element,
               ICursorImpl& cursor)
{ VInherited::AddAtPosition (position, element, cursor);
  this->NotifyOfAddAt (cursor);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
Copy (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Copy (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
INumber
IVSequenceImpl <VInherited>::
RemoveAll ()
{ INumber result = VInherited::RemoveAll ();
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVSequenceImpl <VInherited>::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber result = VInherited::RemoveAll (predFunc, addArg);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
RemoveAt (ICursorImpl& cursor)
{ this->NotifyOfRemoveAt (cursor);
  VInherited::RemoveAt (cursor);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
RemoveAtPosition (IPosition position)
{ this->SetToPosition (position, this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveAtPosition (position);
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
RemoveFirst ()
{ this->SetToFirst (this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveFirst ();
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
RemoveLast ()
{ this->SetToLast (this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveLast ();
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
Reverse ()
{ VInherited::Reverse ();
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSequenceImpl <VInherited>::
Sort (ICompFunc compFunc, void* addArg)
{ VInherited::Sort (compFunc, addArg);
  this->NotifyOfModification ();
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIVSEQ_C_
