// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 19 1.5.1.3 source/core/collect/iivss.c, collection, ioc.v400, 000919 
#ifndef _IIVSS_C_
#define _IIVSS_C_

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// IVSortedSetImpl
// ---

// public members

template <class VInherited>
IVSortedSetImpl <VInherited>::
IVSortedSetImpl (Ops& ops, INumber numberOfElements)
: VInherited (ops, numberOfElements)
{
}

template <class VInherited>
IVSortedSetImpl <VInherited>::
IVSortedSetImpl
  (Ops& ops, IVSortedSetImpl <VInherited> const& collection)
: VInherited (ops, collection)
{
}

template <class VInherited>
bool
IVSortedSetImpl <VInherited>::
Add (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenAdded = VInherited::Add (element, cursor);
  this->EnableNotification (isEnabled);
  if (hasBeenAdded) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAllFrom (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
AddAntivalence (IASortedSetImpl const& collection1,
                IASortedSetImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAntivalence (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
AddDifference (IASortedSetImpl const& collection1,
               IASortedSetImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddDifference (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
AddIntersection (IASortedSetImpl const& collection1,
               IASortedSetImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddIntersection (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
AddUnion (IASortedSetImpl const& collection1,
          IASortedSetImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddUnion (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
Antivalence (IASortedSetImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Antivalence (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}
template <class VInherited>
void
IVSortedSetImpl <VInherited>::
Copy (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Copy (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
Difference (IASortedSetImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Difference (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
Intersection (IASortedSetImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Intersection (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}
template <class VInherited>
bool
IVSortedSetImpl <VInherited>::
LocateOrAdd (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenLocated = VInherited::LocateOrAdd (element, cursor);
  this->EnableNotification (isEnabled);
  if (! hasBeenLocated) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenLocated;
}

template <class VInherited>
bool
IVSortedSetImpl <VInherited>::
Remove (void const* element)
{ if (this->Locate (element, this->CursorOf (*this))) {
    this->NotifyOfRemoveAt (this->CursorOf (*this));
  }
  return VInherited::Remove (element);
}

template <class VInherited>
INumber
IVSortedSetImpl <VInherited>::
RemoveAll ()
{ INumber result = VInherited::RemoveAll ();
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVSortedSetImpl <VInherited>::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber result = VInherited::RemoveAll (predFunc, addArg);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVSortedSetImpl <VInherited>::
RemoveAllOccurrences (void const* element)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  INumber result = VInherited::RemoveAllOccurrences (element);
  this->EnableNotification (isEnabled);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
RemoveAt (ICursorImpl& cursor)
{ this->NotifyOfRemoveAt (cursor);
  VInherited::RemoveAt (cursor);
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
RemoveAtPosition (IPosition position)
{ this->SetToPosition (position, this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveAtPosition (position);
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
RemoveFirst ()
{ this->SetToFirst (this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveFirst ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
RemoveLast ()
{ this->SetToLast (this->CursorOf (*this));
  this->NotifyOfRemoveAt (this->CursorOf (*this));
  VInherited::RemoveLast ();
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
}

template <class VInherited>
void
IVSortedSetImpl <VInherited>::
Union (IASortedSetImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Union (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIVSS_C_
