// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 22 1.4.1.4 source/core/collect/ikb.inl, collection, ioc.v400, 000919  
#ifndef _IKB_INL_
#define _IKB_INL_

// ---
// IGKeyBagCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagCursor <Element, Key, ElementOps>::
IGKeyBagCursor
  (IGKeyBag
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeyBagCursor <Element, Key, ElementOps>&
IGKeyBagCursor <Element, Key, ElementOps>::
operator=
  (IGKeyBagCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagCursor <Element, Key, ElementOps>::
IGKeyBagCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeyBagMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagMngCursor <Element, Key, ElementOps>::
IGKeyBagMngCursor
  (IGKeyBag
    <Element, Key, ElementOps> const& collection)
: IGKeyBagCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagMngCursor <Element, Key, ElementOps>::
IGKeyBagMngCursor (ICursorImpl* impl)
: IGKeyBagCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeyBag
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBag <Element, Key, ElementOps>::
IGKeyBag (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeyBag <Element, Key, ElementOps>::
IGKeyBag
  (IGKeyBag
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeyBag <Element, Key, ElementOps>&
IGKeyBag <Element, Key, ElementOps>::
operator= (IGKeyBag
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBag <Element, Key, ElementOps>::
IGKeyBag (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeyBag
 <Element, Key, ElementOps>::Implementation&
IGKeyBag <Element, Key, ElementOps>::
ImplOf (IGKeyBag
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeyBag
// ---

// public members

template <class Element, class Key>
inline
IKeyBag <Element, Key>::
IKeyBag (INumber numberOfElements)
: IGKeyBag
   <Element, Key, IKEHOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeyBag <Element, Key>::
IKeyBag (PureImplementation& impl)
: IGKeyBag
   <Element, Key, IKEHOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeyBag
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeyBag <Element, Key, ElementOps>::
IVGKeyBag (INumber numberOfElements)
: IGKeyBag
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeyBag <Element, Key, ElementOps>::
IVGKeyBag (PureImplementation& impl)
: IGKeyBag
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeyBag
// ---

// public members

template <class Element, class Key>
inline
IVKeyBag <Element, Key>::
IVKeyBag (INumber numberOfElements)
: IVGKeyBag
   <Element, Key, IKEHOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeyBag <Element, Key>::
IVKeyBag (PureImplementation& impl)
: IVGKeyBag
   <Element, Key, IKEHOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKB_INL_
