// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 25 1.5.1.4 source/core/collect/ikbhsh.h, collection, ioc.v400, 000919 
#ifndef _IKBHSH_
#define _IKBHSH_

#include <ilanglvl.hpp>
#include <iakb.h>
#include <iickb.h>
#if ! defined (__INO_AVALON__)
#include <iivkb.h>
#endif
#include <iikbhsh.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeyBagAsHshTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeyBagAsHshTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGKeyBagAsHshTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeyBagAsHshTableCursor
                            (IGKeyBagAsHshTable
                              <
                               Element, Key, ElementOps> const&);

  IGKeyBagAsHshTableCursor <Element, Key, ElementOps>&
           operator=        (IGKeyBagAsHshTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeyBagAsHshTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeyBagAsHshTableCursor\
                            (IGKeyBagAsHshTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeyBagAsHshTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeyBagAsHshTableMngCursor :
  public IGKeyBagAsHshTableCursor <Element, Key, ElementOps>  {

  typedef  IGKeyBagAsHshTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeyBagAsHshTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeyBagAsHshTableMngCursor
                            (IGKeyBagAsHshTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeyBagAsHshTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeyBagAsHshTableMngCursor\
                            (IGKeyBagAsHshTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeyBagAsHshTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeyBagAsHshTable :
  public IAKeyBag <Element, Key> {
protected:

  typedef  IGKeyBagAsHshTable <Element, Key, ElementOps> Self;

  typedef  IKeyBagAsHshTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeyBagImpl
            <PureImplementation> Implementation;
#endif

#define IGKeyBagAsHshTableTypedef(Element, Key, ElementOps)\
  typedef  IKeyBagAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeyBagAsHshTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeyBagAsHshTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeyBagAsHshTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeyBagAsHshTable
                            (INumber = 100);

           IGKeyBagAsHshTable
                            (IGKeyBagAsHshTable
                              <
                               Element, Key, ElementOps> const&);

  IGKeyBagAsHshTable <Element, Key, ElementOps>&
           operator=        (IGKeyBagAsHshTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeyBagAsHshTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeyBagAsHshTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeyBagAsHshTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeyBagAsHshTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeyBagAsHshTable\
                            (INumber),\
           operator=,\
           IGKeyBagAsHshTable\
                            (PureImplementation&),\
           ImplOf           (IGKeyBagAsHshTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeyBagAsHshTable :
  public IGKeyBagAsHshTable
          <Element, Key, IKEHOps <Element, Key> > {

  typedef  IGKeyBagAsHshTable
            <Element, Key,
             IKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeyBagAsHshTable <Element, Key> Self;

#define IKeyBagAsHshTableTypedef(Element, Key)\
  typedef  IKeyBagAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, IKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeyBagAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeyBagAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeyBagAsHshTable\
                              (INumber),\
           IKeyBagAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeyBagAsHshTable :
  public IGKeyBagAsHshTable
          <Element, Key, ElementOps > {

  typedef  IVGKeyBagAsHshTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeyBagAsHshTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeyBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeyBagImpl
            <IVKeyBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeyBagAsHshTableTypedef(Element, Key, ElementOps)\
  typedef  IKeyBagAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeyBagAsHshTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeyBagAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeyBagAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeyBagAsHshTable\
                              (INumber),\
           IVGKeyBagAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeyBagAsHshTable :
  public IVGKeyBagAsHshTable
          <Element, Key, IKEHOps <Element, Key> > {

  typedef  IVGKeyBagAsHshTable
            <Element, Key,
             IKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeyBagAsHshTable <Element, Key> Self;

#define IVKeyBagAsHshTableTypedef(Element, Key)\
  typedef  IVKeyBagAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, IKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeyBagAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeyBagAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeyBagAsHshTable\
                              (INumber),\
           IVKeyBagAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ikbhsh.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKBHSH_
