// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 28 1.4.1.4 source/core/collect/ikblst.inl, collection, ioc.v400, 000919  
#ifndef _IKBLST_INL_
#define _IKBLST_INL_

// ---
// IGKeyBagAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsListCursor <Element, Key, ElementOps>::
IGKeyBagAsListCursor
  (IGKeyBagAsList
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeyBagAsListCursor <Element, Key, ElementOps>&
IGKeyBagAsListCursor <Element, Key, ElementOps>::
operator=
  (IGKeyBagAsListCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsListCursor <Element, Key, ElementOps>::
IGKeyBagAsListCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeyBagAsListMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsListMngCursor <Element, Key, ElementOps>::
IGKeyBagAsListMngCursor
  (IGKeyBagAsList
    <Element, Key, ElementOps> const& collection)
: IGKeyBagAsListCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsListMngCursor <Element, Key, ElementOps>::
IGKeyBagAsListMngCursor (ICursorImpl* impl)
: IGKeyBagAsListCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeyBagAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsList <Element, Key, ElementOps>::
IGKeyBagAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsList <Element, Key, ElementOps>::
IGKeyBagAsList
  (IGKeyBagAsList
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeyBagAsList <Element, Key, ElementOps>&
IGKeyBagAsList <Element, Key, ElementOps>::
operator= (IGKeyBagAsList
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsList <Element, Key, ElementOps>::
IGKeyBagAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeyBagAsList
 <Element, Key, ElementOps>::Implementation&
IGKeyBagAsList <Element, Key, ElementOps>::
ImplOf (IGKeyBagAsList
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeyBagAsList
// ---

// public members

template <class Element, class Key>
inline
IKeyBagAsList <Element, Key>::
IKeyBagAsList (INumber numberOfElements)
: IGKeyBagAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeyBagAsList <Element, Key>::
IKeyBagAsList (PureImplementation& impl)
: IGKeyBagAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeyBagAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeyBagAsList <Element, Key, ElementOps>::
IVGKeyBagAsList (INumber numberOfElements)
: IGKeyBagAsList
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeyBagAsList <Element, Key, ElementOps>::
IVGKeyBagAsList (PureImplementation& impl)
: IGKeyBagAsList
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeyBagAsList
// ---

// public members

template <class Element, class Key>
inline
IVKeyBagAsList <Element, Key>::
IVKeyBagAsList (INumber numberOfElements)
: IVGKeyBagAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeyBagAsList <Element, Key>::
IVKeyBagAsList (PureImplementation& impl)
: IVGKeyBagAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKBLST_INL_
