// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 29 1.5.1.4 source/core/collect/ikbtab.h, collection, ioc.v400, 000919 
#ifndef _IKBTAB_
#define _IKBTAB_

#include <ilanglvl.hpp>
#include <iakb.h>
#include <iickb.h>
#if ! defined (__INO_AVALON__)
#include <iivkb.h>
#endif
#include <iikbtab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeyBagAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeyBagAsTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGKeyBagAsTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeyBagAsTableCursor
                            (IGKeyBagAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGKeyBagAsTableCursor <Element, Key, ElementOps>&
           operator=        (IGKeyBagAsTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeyBagAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeyBagAsTableCursor\
                            (IGKeyBagAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeyBagAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeyBagAsTableMngCursor :
  public IGKeyBagAsTableCursor <Element, Key, ElementOps>  {

  typedef  IGKeyBagAsTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeyBagAsTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeyBagAsTableMngCursor
                            (IGKeyBagAsTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeyBagAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeyBagAsTableMngCursor\
                            (IGKeyBagAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeyBagAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeyBagAsTable :
  public IAKeyBag <Element, Key> {
protected:

  typedef  IGKeyBagAsTable <Element, Key, ElementOps> Self;

  typedef  IKeyBagAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeyBagImpl
            <PureImplementation> Implementation;
#endif

#define IGKeyBagAsTableTypedef(Element, Key, ElementOps)\
  typedef  IKeyBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeyBagAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeyBagAsTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeyBagAsTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeyBagAsTable
                            (INumber = 100);

           IGKeyBagAsTable
                            (IGKeyBagAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGKeyBagAsTable <Element, Key, ElementOps>&
           operator=        (IGKeyBagAsTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeyBagAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeyBagAsTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeyBagAsTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeyBagAsTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeyBagAsTable\
                            (INumber),\
           operator=,\
           IGKeyBagAsTable\
                            (PureImplementation&),\
           ImplOf           (IGKeyBagAsTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeyBagAsTable :
  public IGKeyBagAsTable
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGKeyBagAsTable
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeyBagAsTable <Element, Key> Self;

#define IKeyBagAsTableTypedef(Element, Key)\
  typedef  IKeyBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeyBagAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeyBagAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeyBagAsTable\
                              (INumber),\
           IKeyBagAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeyBagAsTable :
  public IGKeyBagAsTable
          <Element, Key, ElementOps > {

  typedef  IVGKeyBagAsTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeyBagAsTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeyBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeyBagImpl
            <IVKeyBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeyBagAsTableTypedef(Element, Key, ElementOps)\
  typedef  IKeyBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeyBagAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeyBagAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeyBagAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeyBagAsTable\
                              (INumber),\
           IVGKeyBagAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeyBagAsTable :
  public IVGKeyBagAsTable
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGKeyBagAsTable
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeyBagAsTable <Element, Key> Self;

#define IVKeyBagAsTableTypedef(Element, Key)\
  typedef  IVKeyBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeyBagAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeyBagAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeyBagAsTable\
                              (INumber),\
           IVKeyBagAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ikbtab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKBTAB_
