// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 34 1.4.1.4 source/core/collect/iks.inl, collection, ioc.v400, 000919  
#ifndef _IKS_INL_
#define _IKS_INL_

// ---
// IGKeySetCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetCursor <Element, Key, ElementOps>::
IGKeySetCursor
  (IGKeySet
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySetCursor <Element, Key, ElementOps>&
IGKeySetCursor <Element, Key, ElementOps>::
operator=
  (IGKeySetCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetCursor <Element, Key, ElementOps>::
IGKeySetCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySetMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetMngCursor <Element, Key, ElementOps>::
IGKeySetMngCursor
  (IGKeySet
    <Element, Key, ElementOps> const& collection)
: IGKeySetCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetMngCursor <Element, Key, ElementOps>::
IGKeySetMngCursor (ICursorImpl* impl)
: IGKeySetCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySet
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySet <Element, Key, ElementOps>::
IGKeySet (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySet <Element, Key, ElementOps>::
IGKeySet
  (IGKeySet
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySet <Element, Key, ElementOps>&
IGKeySet <Element, Key, ElementOps>::
operator= (IGKeySet
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySet <Element, Key, ElementOps>::
IGKeySet (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySet
 <Element, Key, ElementOps>::Implementation&
IGKeySet <Element, Key, ElementOps>::
ImplOf (IGKeySet
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySet
// ---

// public members

template <class Element, class Key>
inline
IKeySet <Element, Key>::
IKeySet (INumber numberOfElements)
: IGKeySet
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySet <Element, Key>::
IKeySet (PureImplementation& impl)
: IGKeySet
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySet
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySet <Element, Key, ElementOps>::
IVGKeySet (INumber numberOfElements)
: IGKeySet
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySet <Element, Key, ElementOps>::
IVGKeySet (PureImplementation& impl)
: IGKeySet
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySet
// ---

// public members

template <class Element, class Key>
inline
IVKeySet <Element, Key>::
IVKeySet (INumber numberOfElements)
: IVGKeySet
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySet <Element, Key>::
IVKeySet (PureImplementation& impl)
: IVGKeySet
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKS_INL_
