// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 36 1.4.1.4 source/core/collect/iksavl.inl, collection, ioc.v400, 000919  
#ifndef _IKSAVL_INL_
#define _IKSAVL_INL_

// ---
// IGKeySetAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>::
IGKeySetAsAvlTreeCursor
  (IGKeySetAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>&
IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>::
operator=
  (IGKeySetAsAvlTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>::
IGKeySetAsAvlTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySetAsAvlTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGKeySetAsAvlTreeMngCursor
  (IGKeySetAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGKeySetAsAvlTreeMngCursor (ICursorImpl* impl)
: IGKeySetAsAvlTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySetAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTree <Element, Key, ElementOps>::
IGKeySetAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTree <Element, Key, ElementOps>::
IGKeySetAsAvlTree
  (IGKeySetAsAvlTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsAvlTree <Element, Key, ElementOps>&
IGKeySetAsAvlTree <Element, Key, ElementOps>::
operator= (IGKeySetAsAvlTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsAvlTree <Element, Key, ElementOps>::
IGKeySetAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySetAsAvlTree
 <Element, Key, ElementOps>::Implementation&
IGKeySetAsAvlTree <Element, Key, ElementOps>::
ImplOf (IGKeySetAsAvlTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySetAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IKeySetAsAvlTree <Element, Key>::
IKeySetAsAvlTree (INumber numberOfElements)
: IGKeySetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySetAsAvlTree <Element, Key>::
IKeySetAsAvlTree (PureImplementation& impl)
: IGKeySetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySetAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsAvlTree <Element, Key, ElementOps>::
IVGKeySetAsAvlTree (INumber numberOfElements)
: IGKeySetAsAvlTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsAvlTree <Element, Key, ElementOps>::
IVGKeySetAsAvlTree (PureImplementation& impl)
: IGKeySetAsAvlTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySetAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IVKeySetAsAvlTree <Element, Key>::
IVKeySetAsAvlTree (INumber numberOfElements)
: IVGKeySetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySetAsAvlTree <Element, Key>::
IVKeySetAsAvlTree (PureImplementation& impl)
: IVGKeySetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSAVL_INL_
