// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 38 1.4.1.4 source/core/collect/iksb.inl, collection, ioc.v400, 000919  
#ifndef _IKSB_INL_
#define _IKSB_INL_

// ---
// IGKeySortedBagCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagCursor <Element, Key, ElementOps>::
IGKeySortedBagCursor
  (IGKeySortedBag
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedBagCursor <Element, Key, ElementOps>&
IGKeySortedBagCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedBagCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagCursor <Element, Key, ElementOps>::
IGKeySortedBagCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedBagMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagMngCursor <Element, Key, ElementOps>::
IGKeySortedBagMngCursor
  (IGKeySortedBag
    <Element, Key, ElementOps> const& collection)
: IGKeySortedBagCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagMngCursor <Element, Key, ElementOps>::
IGKeySortedBagMngCursor (ICursorImpl* impl)
: IGKeySortedBagCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedBag
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBag <Element, Key, ElementOps>::
IGKeySortedBag (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBag <Element, Key, ElementOps>::
IGKeySortedBag
  (IGKeySortedBag
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedBag <Element, Key, ElementOps>&
IGKeySortedBag <Element, Key, ElementOps>::
operator= (IGKeySortedBag
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBag <Element, Key, ElementOps>::
IGKeySortedBag (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedBag
 <Element, Key, ElementOps>::Implementation&
IGKeySortedBag <Element, Key, ElementOps>::
ImplOf (IGKeySortedBag
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedBag
// ---

// public members

template <class Element, class Key>
inline
IKeySortedBag <Element, Key>::
IKeySortedBag (INumber numberOfElements)
: IGKeySortedBag
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedBag <Element, Key>::
IKeySortedBag (PureImplementation& impl)
: IGKeySortedBag
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedBag
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedBag <Element, Key, ElementOps>::
IVGKeySortedBag (INumber numberOfElements)
: IGKeySortedBag
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedBag <Element, Key, ElementOps>::
IVGKeySortedBag (PureImplementation& impl)
: IGKeySortedBag
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedBag
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedBag <Element, Key>::
IVKeySortedBag (INumber numberOfElements)
: IVGKeySortedBag
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedBag <Element, Key>::
IVKeySortedBag (PureImplementation& impl)
: IVGKeySortedBag
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSB_INL_
