// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 46 1.4.1.4 source/core/collect/iksbst.inl, collection, ioc.v400, 000919  
#ifndef _IKSBST_INL_
#define _IKSBST_INL_

// ---
// IGKeySetAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTreeCursor <Element, Key, ElementOps>::
IGKeySetAsBstTreeCursor
  (IGKeySetAsBstTree
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsBstTreeCursor <Element, Key, ElementOps>&
IGKeySetAsBstTreeCursor <Element, Key, ElementOps>::
operator=
  (IGKeySetAsBstTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTreeCursor <Element, Key, ElementOps>::
IGKeySetAsBstTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySetAsBstTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTreeMngCursor <Element, Key, ElementOps>::
IGKeySetAsBstTreeMngCursor
  (IGKeySetAsBstTree
    <Element, Key, ElementOps> const& collection)
: IGKeySetAsBstTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTreeMngCursor <Element, Key, ElementOps>::
IGKeySetAsBstTreeMngCursor (ICursorImpl* impl)
: IGKeySetAsBstTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySetAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTree <Element, Key, ElementOps>::
IGKeySetAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTree <Element, Key, ElementOps>::
IGKeySetAsBstTree
  (IGKeySetAsBstTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsBstTree <Element, Key, ElementOps>&
IGKeySetAsBstTree <Element, Key, ElementOps>::
operator= (IGKeySetAsBstTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsBstTree <Element, Key, ElementOps>::
IGKeySetAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySetAsBstTree
 <Element, Key, ElementOps>::Implementation&
IGKeySetAsBstTree <Element, Key, ElementOps>::
ImplOf (IGKeySetAsBstTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySetAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IKeySetAsBstTree <Element, Key>::
IKeySetAsBstTree (INumber numberOfElements)
: IGKeySetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySetAsBstTree <Element, Key>::
IKeySetAsBstTree (PureImplementation& impl)
: IGKeySetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySetAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsBstTree <Element, Key, ElementOps>::
IVGKeySetAsBstTree (INumber numberOfElements)
: IGKeySetAsBstTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsBstTree <Element, Key, ElementOps>::
IVGKeySetAsBstTree (PureImplementation& impl)
: IGKeySetAsBstTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySetAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IVKeySetAsBstTree <Element, Key>::
IVKeySetAsBstTree (INumber numberOfElements)
: IVGKeySetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySetAsBstTree <Element, Key>::
IVKeySetAsBstTree (PureImplementation& impl)
: IVGKeySetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSBST_INL_
