// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 49 1.4.1.4 source/core/collect/iksbtab.inl, collection, ioc.v400, 000919  
#ifndef _IKSBTAB_INL_
#define _IKSBTAB_INL_

// ---
// IGKeySortedBagAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTableCursor <Element, Key, ElementOps>::
IGKeySortedBagAsTableCursor
  (IGKeySortedBagAsTable
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedBagAsTableCursor <Element, Key, ElementOps>&
IGKeySortedBagAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedBagAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTableCursor <Element, Key, ElementOps>::
IGKeySortedBagAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedBagAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTableMngCursor <Element, Key, ElementOps>::
IGKeySortedBagAsTableMngCursor
  (IGKeySortedBagAsTable
    <Element, Key, ElementOps> const& collection)
: IGKeySortedBagAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTableMngCursor <Element, Key, ElementOps>::
IGKeySortedBagAsTableMngCursor (ICursorImpl* impl)
: IGKeySortedBagAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedBagAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTable <Element, Key, ElementOps>::
IGKeySortedBagAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTable <Element, Key, ElementOps>::
IGKeySortedBagAsTable
  (IGKeySortedBagAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedBagAsTable <Element, Key, ElementOps>&
IGKeySortedBagAsTable <Element, Key, ElementOps>::
operator= (IGKeySortedBagAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedBagAsTable <Element, Key, ElementOps>::
IGKeySortedBagAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedBagAsTable
 <Element, Key, ElementOps>::Implementation&
IGKeySortedBagAsTable <Element, Key, ElementOps>::
ImplOf (IGKeySortedBagAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedBagAsTable
// ---

// public members

template <class Element, class Key>
inline
IKeySortedBagAsTable <Element, Key>::
IKeySortedBagAsTable (INumber numberOfElements)
: IGKeySortedBagAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedBagAsTable <Element, Key>::
IKeySortedBagAsTable (PureImplementation& impl)
: IGKeySortedBagAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedBagAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedBagAsTable <Element, Key, ElementOps>::
IVGKeySortedBagAsTable (INumber numberOfElements)
: IGKeySortedBagAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedBagAsTable <Element, Key, ElementOps>::
IVGKeySortedBagAsTable (PureImplementation& impl)
: IGKeySortedBagAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedBagAsTable
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedBagAsTable <Element, Key>::
IVKeySortedBagAsTable (INumber numberOfElements)
: IVGKeySortedBagAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedBagAsTable <Element, Key>::
IVKeySortedBagAsTable (PureImplementation& impl)
: IVGKeySortedBagAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSBTAB_INL_
