// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 52 1.6.1.4 source/core/collect/ikshsh.h, collection, ioc.v400, 000919 
#ifndef _IKSHSH_
#define _IKSHSH_

#include <ilanglvl.hpp>
#include <iaks.h>
#include <iicks.h>
#if ! defined (__INO_AVALON__)
#include <iivks.h>
#endif
#include <iikshsh.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeySetAsHshTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeySetAsHshTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGKeySetAsHshTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeySetAsHshTableCursor
                            (IGKeySetAsHshTable
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsHshTableCursor <Element, Key, ElementOps>&
           operator=        (IGKeySetAsHshTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsHshTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsHshTableCursor\
                            (IGKeySetAsHshTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeySetAsHshTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeySetAsHshTableMngCursor :
  public IGKeySetAsHshTableCursor <Element, Key, ElementOps>  {

  typedef  IGKeySetAsHshTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeySetAsHshTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeySetAsHshTableMngCursor
                            (IGKeySetAsHshTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsHshTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsHshTableMngCursor\
                            (IGKeySetAsHshTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeySetAsHshTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeySetAsHshTable :
  public IAKeySet <Element, Key> {
protected:

  typedef  IGKeySetAsHshTable <Element, Key, ElementOps> Self;

  typedef  IKeySetAsHshTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySetImpl
            <PureImplementation> Implementation;
#endif

#define IGKeySetAsHshTableTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeySetAsHshTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeySetAsHshTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeySetAsHshTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeySetAsHshTable
                            (INumber = 100);

           IGKeySetAsHshTable
                            (IGKeySetAsHshTable
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsHshTable <Element, Key, ElementOps>&
           operator=        (IGKeySetAsHshTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeySetAsHshTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeySetAsHshTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeySetAsHshTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeySetAsHshTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsHshTable\
                            (INumber),\
           operator=,\
           IGKeySetAsHshTable\
                            (PureImplementation&),\
           ImplOf           (IGKeySetAsHshTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeySetAsHshTable :
  public IGKeySetAsHshTable
          <Element, Key, IKEHOps <Element, Key> > {

  typedef  IGKeySetAsHshTable
            <Element, Key,
             IKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeySetAsHshTable <Element, Key> Self;

#define IKeySetAsHshTableTypedef(Element, Key)\
  typedef  IKeySetAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, IKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeySetAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeySetAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeySetAsHshTable\
                              (INumber),\
           IKeySetAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeySetAsHshTable :
  public IGKeySetAsHshTable
          <Element, Key, ElementOps > {

  typedef  IVGKeySetAsHshTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeySetAsHshTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySetImpl
            <IVKeySetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeySetAsHshTableTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeySetAsHshTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeySetAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeySetAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeySetAsHshTable\
                              (INumber),\
           IVGKeySetAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeySetAsHshTable :
  public IVGKeySetAsHshTable
          <Element, Key, IKEHOps <Element, Key> > {

  typedef  IVGKeySetAsHshTable
            <Element, Key,
             IKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeySetAsHshTable <Element, Key> Self;

#define IVKeySetAsHshTableTypedef(Element, Key)\
  typedef  IVKeySetAsHshTableOps\
            <Element, Key,\
             IWKEHOps <Element, Key, IKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeySetAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeySetAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeySetAsHshTable\
                              (INumber),\
           IVKeySetAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ikshsh.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKSHSH_
