// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 53 1.4.1.4 source/core/collect/ikshsh.inl, collection, ioc.v400, 000919  
#ifndef _IKSHSH_INL_
#define _IKSHSH_INL_

// ---
// IGKeySetAsHshTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTableCursor <Element, Key, ElementOps>::
IGKeySetAsHshTableCursor
  (IGKeySetAsHshTable
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsHshTableCursor <Element, Key, ElementOps>&
IGKeySetAsHshTableCursor <Element, Key, ElementOps>::
operator=
  (IGKeySetAsHshTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTableCursor <Element, Key, ElementOps>::
IGKeySetAsHshTableCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySetAsHshTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTableMngCursor <Element, Key, ElementOps>::
IGKeySetAsHshTableMngCursor
  (IGKeySetAsHshTable
    <Element, Key, ElementOps> const& collection)
: IGKeySetAsHshTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTableMngCursor <Element, Key, ElementOps>::
IGKeySetAsHshTableMngCursor (ICursorImpl* impl)
: IGKeySetAsHshTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySetAsHshTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTable <Element, Key, ElementOps>::
IGKeySetAsHshTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTable <Element, Key, ElementOps>::
IGKeySetAsHshTable
  (IGKeySetAsHshTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsHshTable <Element, Key, ElementOps>&
IGKeySetAsHshTable <Element, Key, ElementOps>::
operator= (IGKeySetAsHshTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsHshTable <Element, Key, ElementOps>::
IGKeySetAsHshTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySetAsHshTable
 <Element, Key, ElementOps>::Implementation&
IGKeySetAsHshTable <Element, Key, ElementOps>::
ImplOf (IGKeySetAsHshTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySetAsHshTable
// ---

// public members

template <class Element, class Key>
inline
IKeySetAsHshTable <Element, Key>::
IKeySetAsHshTable (INumber numberOfElements)
: IGKeySetAsHshTable
   <Element, Key, IKEHOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySetAsHshTable <Element, Key>::
IKeySetAsHshTable (PureImplementation& impl)
: IGKeySetAsHshTable
   <Element, Key, IKEHOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySetAsHshTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsHshTable <Element, Key, ElementOps>::
IVGKeySetAsHshTable (INumber numberOfElements)
: IGKeySetAsHshTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsHshTable <Element, Key, ElementOps>::
IVGKeySetAsHshTable (PureImplementation& impl)
: IGKeySetAsHshTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySetAsHshTable
// ---

// public members

template <class Element, class Key>
inline
IVKeySetAsHshTable <Element, Key>::
IVKeySetAsHshTable (INumber numberOfElements)
: IVGKeySetAsHshTable
   <Element, Key, IKEHOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySetAsHshTable <Element, Key>::
IVKeySetAsHshTable (PureImplementation& impl)
: IVGKeySetAsHshTable
   <Element, Key, IKEHOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSHSH_INL_
