// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 57 1.4.1.4 source/core/collect/ikss.inl, collection, ioc.v400, 000919  
#ifndef _IKSS_INL_
#define _IKSS_INL_

// ---
// IGKeySortedSetCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetCursor <Element, Key, ElementOps>::
IGKeySortedSetCursor
  (IGKeySortedSet
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetCursor <Element, Key, ElementOps>&
IGKeySortedSetCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedSetCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetCursor <Element, Key, ElementOps>::
IGKeySortedSetCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedSetMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetMngCursor <Element, Key, ElementOps>::
IGKeySortedSetMngCursor
  (IGKeySortedSet
    <Element, Key, ElementOps> const& collection)
: IGKeySortedSetCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetMngCursor <Element, Key, ElementOps>::
IGKeySortedSetMngCursor (ICursorImpl* impl)
: IGKeySortedSetCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedSet
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSet <Element, Key, ElementOps>::
IGKeySortedSet (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSet <Element, Key, ElementOps>::
IGKeySortedSet
  (IGKeySortedSet
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSet <Element, Key, ElementOps>&
IGKeySortedSet <Element, Key, ElementOps>::
operator= (IGKeySortedSet
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSet <Element, Key, ElementOps>::
IGKeySortedSet (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedSet
 <Element, Key, ElementOps>::Implementation&
IGKeySortedSet <Element, Key, ElementOps>::
ImplOf (IGKeySortedSet
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedSet
// ---

// public members

template <class Element, class Key>
inline
IKeySortedSet <Element, Key>::
IKeySortedSet (INumber numberOfElements)
: IGKeySortedSet
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedSet <Element, Key>::
IKeySortedSet (PureImplementation& impl)
: IGKeySortedSet
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedSet
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSet <Element, Key, ElementOps>::
IVGKeySortedSet (INumber numberOfElements)
: IGKeySortedSet
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSet <Element, Key, ElementOps>::
IVGKeySortedSet (PureImplementation& impl)
: IGKeySortedSet
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedSet
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedSet <Element, Key>::
IVKeySortedSet (INumber numberOfElements)
: IVGKeySortedSet
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedSet <Element, Key>::
IVKeySortedSet (PureImplementation& impl)
: IVGKeySortedSet
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSS_INL_
