// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 61 1.4.1.4 source/core/collect/ikssbst.inl, collection, ioc.v400, 000919  
#ifndef _IKSSBST_INL_
#define _IKSSBST_INL_

// ---
// IGKeySortedSetAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTreeCursor <Element, Key, ElementOps>::
IGKeySortedSetAsBstTreeCursor
  (IGKeySortedSetAsBstTree
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsBstTreeCursor <Element, Key, ElementOps>&
IGKeySortedSetAsBstTreeCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedSetAsBstTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTreeCursor <Element, Key, ElementOps>::
IGKeySortedSetAsBstTreeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedSetAsBstTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTreeMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsBstTreeMngCursor
  (IGKeySortedSetAsBstTree
    <Element, Key, ElementOps> const& collection)
: IGKeySortedSetAsBstTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTreeMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsBstTreeMngCursor (ICursorImpl* impl)
: IGKeySortedSetAsBstTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedSetAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTree <Element, Key, ElementOps>::
IGKeySortedSetAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTree <Element, Key, ElementOps>::
IGKeySortedSetAsBstTree
  (IGKeySortedSetAsBstTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsBstTree <Element, Key, ElementOps>&
IGKeySortedSetAsBstTree <Element, Key, ElementOps>::
operator= (IGKeySortedSetAsBstTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsBstTree <Element, Key, ElementOps>::
IGKeySortedSetAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedSetAsBstTree
 <Element, Key, ElementOps>::Implementation&
IGKeySortedSetAsBstTree <Element, Key, ElementOps>::
ImplOf (IGKeySortedSetAsBstTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedSetAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IKeySortedSetAsBstTree <Element, Key>::
IKeySortedSetAsBstTree (INumber numberOfElements)
: IGKeySortedSetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedSetAsBstTree <Element, Key>::
IKeySortedSetAsBstTree (PureImplementation& impl)
: IGKeySortedSetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedSetAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsBstTree <Element, Key, ElementOps>::
IVGKeySortedSetAsBstTree (INumber numberOfElements)
: IGKeySortedSetAsBstTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsBstTree <Element, Key, ElementOps>::
IVGKeySortedSetAsBstTree (PureImplementation& impl)
: IGKeySortedSetAsBstTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedSetAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedSetAsBstTree <Element, Key>::
IVKeySortedSetAsBstTree (INumber numberOfElements)
: IVGKeySortedSetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedSetAsBstTree <Element, Key>::
IVKeySortedSetAsBstTree (PureImplementation& impl)
: IVGKeySortedSetAsBstTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSSBST_INL_
