// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 73 1.4.1.4 source/core/collect/iksstab.inl, collection, ioc.v400, 000919  
#ifndef _IKSSTAB_INL_
#define _IKSSTAB_INL_

// ---
// IGKeySortedSetAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTableCursor <Element, Key, ElementOps>::
IGKeySortedSetAsTableCursor
  (IGKeySortedSetAsTable
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsTableCursor <Element, Key, ElementOps>&
IGKeySortedSetAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedSetAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTableCursor <Element, Key, ElementOps>::
IGKeySortedSetAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedSetAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTableMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsTableMngCursor
  (IGKeySortedSetAsTable
    <Element, Key, ElementOps> const& collection)
: IGKeySortedSetAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTableMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsTableMngCursor (ICursorImpl* impl)
: IGKeySortedSetAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedSetAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTable <Element, Key, ElementOps>::
IGKeySortedSetAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTable <Element, Key, ElementOps>::
IGKeySortedSetAsTable
  (IGKeySortedSetAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsTable <Element, Key, ElementOps>&
IGKeySortedSetAsTable <Element, Key, ElementOps>::
operator= (IGKeySortedSetAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsTable <Element, Key, ElementOps>::
IGKeySortedSetAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedSetAsTable
 <Element, Key, ElementOps>::Implementation&
IGKeySortedSetAsTable <Element, Key, ElementOps>::
ImplOf (IGKeySortedSetAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedSetAsTable
// ---

// public members

template <class Element, class Key>
inline
IKeySortedSetAsTable <Element, Key>::
IKeySortedSetAsTable (INumber numberOfElements)
: IGKeySortedSetAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedSetAsTable <Element, Key>::
IKeySortedSetAsTable (PureImplementation& impl)
: IGKeySortedSetAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedSetAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsTable <Element, Key, ElementOps>::
IVGKeySortedSetAsTable (INumber numberOfElements)
: IGKeySortedSetAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsTable <Element, Key, ElementOps>::
IVGKeySortedSetAsTable (PureImplementation& impl)
: IGKeySortedSetAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedSetAsTable
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedSetAsTable <Element, Key>::
IVKeySortedSetAsTable (INumber numberOfElements)
: IVGKeySortedSetAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedSetAsTable <Element, Key>::
IVKeySortedSetAsTable (PureImplementation& impl)
: IVGKeySortedSetAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSSTAB_INL_
