// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 76 1.4.1.4 source/core/collect/ikstab.inl, collection, ioc.v400, 000919  
#ifndef _IKSTAB_INL_
#define _IKSTAB_INL_

// ---
// IGKeySetAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTableCursor <Element, Key, ElementOps>::
IGKeySetAsTableCursor
  (IGKeySetAsTable
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsTableCursor <Element, Key, ElementOps>&
IGKeySetAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGKeySetAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTableCursor <Element, Key, ElementOps>::
IGKeySetAsTableCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySetAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTableMngCursor <Element, Key, ElementOps>::
IGKeySetAsTableMngCursor
  (IGKeySetAsTable
    <Element, Key, ElementOps> const& collection)
: IGKeySetAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTableMngCursor <Element, Key, ElementOps>::
IGKeySetAsTableMngCursor (ICursorImpl* impl)
: IGKeySetAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySetAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTable <Element, Key, ElementOps>::
IGKeySetAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTable <Element, Key, ElementOps>::
IGKeySetAsTable
  (IGKeySetAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsTable <Element, Key, ElementOps>&
IGKeySetAsTable <Element, Key, ElementOps>::
operator= (IGKeySetAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsTable <Element, Key, ElementOps>::
IGKeySetAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySetAsTable
 <Element, Key, ElementOps>::Implementation&
IGKeySetAsTable <Element, Key, ElementOps>::
ImplOf (IGKeySetAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySetAsTable
// ---

// public members

template <class Element, class Key>
inline
IKeySetAsTable <Element, Key>::
IKeySetAsTable (INumber numberOfElements)
: IGKeySetAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySetAsTable <Element, Key>::
IKeySetAsTable (PureImplementation& impl)
: IGKeySetAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySetAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsTable <Element, Key, ElementOps>::
IVGKeySetAsTable (INumber numberOfElements)
: IGKeySetAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsTable <Element, Key, ElementOps>::
IVGKeySetAsTable (PureImplementation& impl)
: IGKeySetAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySetAsTable
// ---

// public members

template <class Element, class Key>
inline
IVKeySetAsTable <Element, Key>::
IVKeySetAsTable (INumber numberOfElements)
: IVGKeySetAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySetAsTable <Element, Key>::
IVKeySetAsTable (PureImplementation& impl)
: IVGKeySetAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSTAB_INL_
