// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 79 1.4.1.4 source/core/collect/imap.inl, collection, ioc.v400, 000919  
#ifndef _IMAP_INL_
#define _IMAP_INL_

// ---
// IGMapCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapCursor <Element, Key, ElementOps>::
IGMapCursor
  (IGMap
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGMapCursor <Element, Key, ElementOps>&
IGMapCursor <Element, Key, ElementOps>::
operator=
  (IGMapCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapCursor <Element, Key, ElementOps>::
IGMapCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGMapMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapMngCursor <Element, Key, ElementOps>::
IGMapMngCursor
  (IGMap
    <Element, Key, ElementOps> const& collection)
: IGMapCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapMngCursor <Element, Key, ElementOps>::
IGMapMngCursor (ICursorImpl* impl)
: IGMapCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGMap
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMap <Element, Key, ElementOps>::
IGMap (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGMap <Element, Key, ElementOps>::
IGMap
  (IGMap
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGMap <Element, Key, ElementOps>&
IGMap <Element, Key, ElementOps>::
operator= (IGMap
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMap <Element, Key, ElementOps>::
IGMap (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGMap
 <Element, Key, ElementOps>::Implementation&
IGMap <Element, Key, ElementOps>::
ImplOf (IGMap
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IMap
// ---

// public members

template <class Element, class Key>
inline
IMap <Element, Key>::
IMap (INumber numberOfElements)
: IGMap
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IMap <Element, Key>::
IMap (PureImplementation& impl)
: IGMap
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGMap
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGMap <Element, Key, ElementOps>::
IVGMap (INumber numberOfElements)
: IGMap
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGMap <Element, Key, ElementOps>::
IVGMap (PureImplementation& impl)
: IGMap
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVMap
// ---

// public members

template <class Element, class Key>
inline
IVMap <Element, Key>::
IVMap (INumber numberOfElements)
: IVGMap
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVMap <Element, Key>::
IVMap (PureImplementation& impl)
: IVGMap
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IMAP_INL_
