// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 81 1.4.1.4 source/core/collect/imapavl.inl, collection, ioc.v400, 000919  
#ifndef _IMAPAVL_INL_
#define _IMAPAVL_INL_

// ---
// IGMapAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTreeCursor <Element, Key, ElementOps>::
IGMapAsAvlTreeCursor
  (IGMapAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGMapAsAvlTreeCursor <Element, Key, ElementOps>&
IGMapAsAvlTreeCursor <Element, Key, ElementOps>::
operator=
  (IGMapAsAvlTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTreeCursor <Element, Key, ElementOps>::
IGMapAsAvlTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGMapAsAvlTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGMapAsAvlTreeMngCursor
  (IGMapAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IGMapAsAvlTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGMapAsAvlTreeMngCursor (ICursorImpl* impl)
: IGMapAsAvlTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGMapAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTree <Element, Key, ElementOps>::
IGMapAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTree <Element, Key, ElementOps>::
IGMapAsAvlTree
  (IGMapAsAvlTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGMapAsAvlTree <Element, Key, ElementOps>&
IGMapAsAvlTree <Element, Key, ElementOps>::
operator= (IGMapAsAvlTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsAvlTree <Element, Key, ElementOps>::
IGMapAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGMapAsAvlTree
 <Element, Key, ElementOps>::Implementation&
IGMapAsAvlTree <Element, Key, ElementOps>::
ImplOf (IGMapAsAvlTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IMapAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IMapAsAvlTree <Element, Key>::
IMapAsAvlTree (INumber numberOfElements)
: IGMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IMapAsAvlTree <Element, Key>::
IMapAsAvlTree (PureImplementation& impl)
: IGMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGMapAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGMapAsAvlTree <Element, Key, ElementOps>::
IVGMapAsAvlTree (INumber numberOfElements)
: IGMapAsAvlTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGMapAsAvlTree <Element, Key, ElementOps>::
IVGMapAsAvlTree (PureImplementation& impl)
: IGMapAsAvlTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVMapAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IVMapAsAvlTree <Element, Key>::
IVMapAsAvlTree (INumber numberOfElements)
: IVGMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVMapAsAvlTree <Element, Key>::
IVMapAsAvlTree (PureImplementation& impl)
: IVGMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IMAPAVL_INL_
