// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 02 1.5.1.4 source/core/collect/imwt.h, collection, ioc.v400, 000919 
#ifndef _IMWT_
#define _IMWT_

#include <ilanglvl.hpp>
#include <iamwt.h>
#include <iicmwt.h>
#if ! defined (__INO_AVALON__)
#include <iivmwt.h>
#endif
#include <iimwt.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <INumber numOfChildren, class Element, class ElementOps>
class IGMultiwayTree;

#pragma pack(push,4)
#pragma enum(4)

template <INumber numOfChildren, class Element, class ElementOps>
class IGMultiwayTreeCursor :
  public IElementTreeCursor <Element> {

  typedef  IElementTreeCursor <Element> Inherited;

  typedef TYPENAME IGMultiwayTree
            <numOfChildren, Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGMultiwayTreeCursor
                            (IGMultiwayTree
                              <numOfChildren,
                               Element, ElementOps> const&);

  IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>&
           operator=        (IGMultiwayTreeCursor
                              <numOfChildren,
                               Element, ElementOps> const&);

protected:

           IGMultiwayTreeCursor
                            (ITreeCursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMultiwayTreeCursor\
                            (IGMultiwayTree\
                              <numOfChildren,\
                               Element, ElementOps> const&),\
           operator=,\
           IGMultiwayTreeCursor\
                            (ITreeCursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <INumber numOfChildren, class Element, class ElementOps>
class IGMultiwayTreeMngCursor :
  public IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>  {

  typedef  IGMultiwayTreeCursor
            <numOfChildren, Element, ElementOps> Cursor;

  typedef TYPENAME IGMultiwayTree
            <numOfChildren, Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGMultiwayTreeMngCursor
                            (IGMultiwayTree
                              <numOfChildren,
                               Element, ElementOps> const&);

protected:

           IGMultiwayTreeMngCursor
                            (ITreeCursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMultiwayTreeMngCursor\
                            (IGMultiwayTree\
                              <numOfChildren,\
                               Element, ElementOps> const&),\
           IGMultiwayTreeMngCursor\
                            (ITreeCursorImpl*)\
                        )
#endif

};

#endif
template <INumber numOfChildren, class Element, class ElementOps>
class IGMultiwayTree :
  public IAMultiwayTree <numOfChildren, Element> {
protected:

  typedef  IGMultiwayTree <numOfChildren, Element, ElementOps> Self;

  typedef  IMultiwayTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICMultiwayTreeImpl
            <PureImplementation> Implementation;
#endif

#define IGMultiwayTreeTypedef(Element, ElementOps)\
  typedef  IMultiwayTreeOps\
            <numOfChildren, Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGMultiwayTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ITreeCollectionGuard<Element> Guard;
#endif

  typedef  IGMultiwayTreeCursor
            <numOfChildren, Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGMultiwayTreeMngCursor
            <numOfChildren, Element, ElementOps> MngCursor;
#endif

           IGMultiwayTree
                            ();

           IGMultiwayTree
                            (IGMultiwayTree
                              <numOfChildren,
                               Element, ElementOps> const&);

  IGMultiwayTree <numOfChildren, Element, ElementOps>&
           operator=        (IGMultiwayTree
                              <numOfChildren,
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGMultiwayTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGMultiwayTree
                              <numOfChildren,
                               Element, ElementOps> const&);

private:

  friend
  class    IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGMultiwayTreeMngCursor
            <numOfChildren, Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMultiwayTree\
                            (),\
           operator=,\
           IGMultiwayTree\
                            (PureImplementation&),\
           ImplOf           (IGMultiwayTree\
                              <numOfChildren,\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <INumber numOfChildren, class Element>
class IMultiwayTree :
  public IGMultiwayTree
          <numOfChildren, Element, IStdOps <Element> > {

  typedef  IGMultiwayTree
            <numOfChildren, Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IMultiwayTree <numOfChildren, Element> Self;

#define IMultiwayTreeTypedef(Element)\
  typedef  IMultiwayTreeOps\
            <numOfChildren, Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ITreeCollectionGuard<Element> Guard;
#endif

           IMultiwayTree
                            ();

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IMultiwayTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IMultiwayTree\
                              (),\
           IMultiwayTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <INumber numOfChildren, class Element, class ElementOps>
class IVGMultiwayTree :
  public IGMultiwayTree
          <numOfChildren, Element, ElementOps > {

  typedef  IVGMultiwayTree
            <numOfChildren, Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGMultiwayTree
            <numOfChildren, Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVMultiwayTreeImpl
            <PureImplementation> Implementation;
#else
  typedef  ICMultiwayTreeImpl
            <IVMultiwayTreeImpl
              <PureImplementation> > Implementation;
#endif

#define IVGMultiwayTreeTypedef(Element, ElementOps)\
  typedef  IMultiwayTreeOps\
            <numOfChildren, Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGMultiwayTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ITreeCollectionGuard<Element> Guard;
#endif

           IVGMultiwayTree
                            ();

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGMultiwayTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGMultiwayTree\
                              (),\
           IVGMultiwayTree\
                              (PureImplementation&)\
                        )
#endif

};

template <INumber numOfChildren, class Element>
class IVMultiwayTree :
  public IVGMultiwayTree
          <numOfChildren, Element, IStdOps <Element> > {

  typedef  IVGMultiwayTree
            <numOfChildren, Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVMultiwayTree <numOfChildren, Element> Self;

#define IVMultiwayTreeTypedef(Element)\
  typedef  IVMultiwayTreeOps\
            <numOfChildren, Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ITreeCollectionGuard<Element> Guard;
#endif

           IVMultiwayTree
                            ();

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVMultiwayTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVMultiwayTree\
                              (),\
           IVMultiwayTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <imwt.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IMWT_
