// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 12 1.4.1.4 source/core/collect/ipqulst.inl, collection, ioc.v400, 000919  
#ifndef _IPQULST_INL_
#define _IPQULST_INL_

// ---
// IGPriorityQueueAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsListCursor <Element, Key, ElementOps>::
IGPriorityQueueAsListCursor
  (IGPriorityQueueAsList
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGPriorityQueueAsListCursor <Element, Key, ElementOps>&
IGPriorityQueueAsListCursor <Element, Key, ElementOps>::
operator=
  (IGPriorityQueueAsListCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsListCursor <Element, Key, ElementOps>::
IGPriorityQueueAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGPriorityQueueAsListMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsListMngCursor <Element, Key, ElementOps>::
IGPriorityQueueAsListMngCursor
  (IGPriorityQueueAsList
    <Element, Key, ElementOps> const& collection)
: IGPriorityQueueAsListCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsListMngCursor <Element, Key, ElementOps>::
IGPriorityQueueAsListMngCursor (ICursorImpl* impl)
: IGPriorityQueueAsListCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGPriorityQueueAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsList <Element, Key, ElementOps>::
IGPriorityQueueAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsList <Element, Key, ElementOps>::
IGPriorityQueueAsList
  (IGPriorityQueueAsList
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGPriorityQueueAsList <Element, Key, ElementOps>&
IGPriorityQueueAsList <Element, Key, ElementOps>::
operator= (IGPriorityQueueAsList
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsList <Element, Key, ElementOps>::
IGPriorityQueueAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGPriorityQueueAsList
 <Element, Key, ElementOps>::Implementation&
IGPriorityQueueAsList <Element, Key, ElementOps>::
ImplOf (IGPriorityQueueAsList
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IPriorityQueueAsList
// ---

// public members

template <class Element, class Key>
inline
IPriorityQueueAsList <Element, Key>::
IPriorityQueueAsList (INumber numberOfElements)
: IGPriorityQueueAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IPriorityQueueAsList <Element, Key>::
IPriorityQueueAsList (PureImplementation& impl)
: IGPriorityQueueAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGPriorityQueueAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGPriorityQueueAsList <Element, Key, ElementOps>::
IVGPriorityQueueAsList (INumber numberOfElements)
: IGPriorityQueueAsList
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGPriorityQueueAsList <Element, Key, ElementOps>::
IVGPriorityQueueAsList (PureImplementation& impl)
: IGPriorityQueueAsList
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVPriorityQueueAsList
// ---

// public members

template <class Element, class Key>
inline
IVPriorityQueueAsList <Element, Key>::
IVPriorityQueueAsList (INumber numberOfElements)
: IVGPriorityQueueAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVPriorityQueueAsList <Element, Key>::
IVPriorityQueueAsList (PureImplementation& impl)
: IVGPriorityQueueAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IPQULST_INL_
