// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 16 1.4.1.4 source/core/collect/ipqutab.inl, collection, ioc.v400, 000919  
#ifndef _IPQUTAB_INL_
#define _IPQUTAB_INL_

// ---
// IGPriorityQueueAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTableCursor <Element, Key, ElementOps>::
IGPriorityQueueAsTableCursor
  (IGPriorityQueueAsTable
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGPriorityQueueAsTableCursor <Element, Key, ElementOps>&
IGPriorityQueueAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGPriorityQueueAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTableCursor <Element, Key, ElementOps>::
IGPriorityQueueAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGPriorityQueueAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTableMngCursor <Element, Key, ElementOps>::
IGPriorityQueueAsTableMngCursor
  (IGPriorityQueueAsTable
    <Element, Key, ElementOps> const& collection)
: IGPriorityQueueAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTableMngCursor <Element, Key, ElementOps>::
IGPriorityQueueAsTableMngCursor (ICursorImpl* impl)
: IGPriorityQueueAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGPriorityQueueAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTable <Element, Key, ElementOps>::
IGPriorityQueueAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTable <Element, Key, ElementOps>::
IGPriorityQueueAsTable
  (IGPriorityQueueAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGPriorityQueueAsTable <Element, Key, ElementOps>&
IGPriorityQueueAsTable <Element, Key, ElementOps>::
operator= (IGPriorityQueueAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueAsTable <Element, Key, ElementOps>::
IGPriorityQueueAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGPriorityQueueAsTable
 <Element, Key, ElementOps>::Implementation&
IGPriorityQueueAsTable <Element, Key, ElementOps>::
ImplOf (IGPriorityQueueAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IPriorityQueueAsTable
// ---

// public members

template <class Element, class Key>
inline
IPriorityQueueAsTable <Element, Key>::
IPriorityQueueAsTable (INumber numberOfElements)
: IGPriorityQueueAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IPriorityQueueAsTable <Element, Key>::
IPriorityQueueAsTable (PureImplementation& impl)
: IGPriorityQueueAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGPriorityQueueAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGPriorityQueueAsTable <Element, Key, ElementOps>::
IVGPriorityQueueAsTable (INumber numberOfElements)
: IGPriorityQueueAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGPriorityQueueAsTable <Element, Key, ElementOps>::
IVGPriorityQueueAsTable (PureImplementation& impl)
: IGPriorityQueueAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVPriorityQueueAsTable
// ---

// public members

template <class Element, class Key>
inline
IVPriorityQueueAsTable <Element, Key>::
IVPriorityQueueAsTable (INumber numberOfElements)
: IVGPriorityQueueAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVPriorityQueueAsTable <Element, Key>::
IVPriorityQueueAsTable (PureImplementation& impl)
: IVGPriorityQueueAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IPQUTAB_INL_
