// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 29 1.5.1.4 source/core/collect/iqutab.h, collection, ioc.v400, 000919 
#ifndef _IQUTAB_
#define _IQUTAB_

#include <ilanglvl.hpp>
#include <iaqu.h>
#include <iicseq.h>
#if ! defined (__INO_AVALON__)
#include <iivseq.h>
#endif
#include <iiseqtab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGQueueAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGQueueAsTableCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGQueueAsTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGQueueAsTableCursor
                            (IGQueueAsTable
                              <
                               Element, ElementOps> const&);

  IGQueueAsTableCursor <Element, ElementOps>&
           operator=        (IGQueueAsTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGQueueAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGQueueAsTableCursor\
                            (IGQueueAsTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGQueueAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGQueueAsTableMngCursor :
  public IGQueueAsTableCursor <Element, ElementOps>  {

  typedef  IGQueueAsTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGQueueAsTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGQueueAsTableMngCursor
                            (IGQueueAsTable
                              <
                               Element, ElementOps> const&);

protected:

           IGQueueAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGQueueAsTableMngCursor\
                            (IGQueueAsTable\
                              <\
                               Element, ElementOps> const&),\
           IGQueueAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGQueueAsTable :
  public IAQueue <Element> {
protected:

  typedef  IGQueueAsTable <Element, ElementOps> Self;

  typedef  ISequenceAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSequenceImpl
            <PureImplementation> Implementation;
#endif

#define IGQueueAsTableTypedef(Element, ElementOps)\
  typedef  ISequenceAsTableOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGQueueAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

  typedef  IGQueueAsTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGQueueAsTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGQueueAsTable
                            (INumber = 100);

           IGQueueAsTable
                            (IGQueueAsTable
                              <
                               Element, ElementOps> const&);

  IGQueueAsTable <Element, ElementOps>&
           operator=        (IGQueueAsTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGQueueAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGQueueAsTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGQueueAsTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGQueueAsTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGQueueAsTable\
                            (INumber),\
           operator=,\
           IGQueueAsTable\
                            (PureImplementation&),\
           ImplOf           (IGQueueAsTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IQueueAsTable :
  public IGQueueAsTable
          <Element, IStdOps <Element> > {

  typedef  IGQueueAsTable
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IQueueAsTable <Element> Self;

#define IQueueAsTableTypedef(Element)\
  typedef  ISequenceAsTableOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IQueueAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IQueueAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IQueueAsTable\
                              (INumber),\
           IQueueAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGQueueAsTable :
  public IGQueueAsTable
          <Element, ElementOps > {

  typedef  IVGQueueAsTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGQueueAsTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSequenceImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSequenceImpl
            <IVSequenceImpl
              <PureImplementation> > Implementation;
#endif

#define IVGQueueAsTableTypedef(Element, ElementOps)\
  typedef  ISequenceAsTableOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGQueueAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVGQueueAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGQueueAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGQueueAsTable\
                              (INumber),\
           IVGQueueAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVQueueAsTable :
  public IVGQueueAsTable
          <Element, IStdOps <Element> > {

  typedef  IVGQueueAsTable
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVQueueAsTable <Element> Self;

#define IVQueueAsTableTypedef(Element)\
  typedef  IVSequenceAsTableOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVQueueAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVQueueAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVQueueAsTable\
                              (INumber),\
           IVQueueAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iqutab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IQUTAB_
