// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 30 1.4.1.4 source/core/collect/iqutab.inl, collection, ioc.v400, 000919  
#ifndef _IQUTAB_INL_
#define _IQUTAB_INL_

// ---
// IGQueueAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGQueueAsTableCursor <Element, ElementOps>::
IGQueueAsTableCursor
  (IGQueueAsTable
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGQueueAsTableCursor <Element, ElementOps>&
IGQueueAsTableCursor <Element, ElementOps>::
operator=
  (IGQueueAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueAsTableCursor <Element, ElementOps>::
IGQueueAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGQueueAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGQueueAsTableMngCursor <Element, ElementOps>::
IGQueueAsTableMngCursor
  (IGQueueAsTable
    <Element, ElementOps> const& collection)
: IGQueueAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueAsTableMngCursor <Element, ElementOps>::
IGQueueAsTableMngCursor (ICursorImpl* impl)
: IGQueueAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGQueueAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGQueueAsTable <Element, ElementOps>::
IGQueueAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGQueueAsTable <Element, ElementOps>::
IGQueueAsTable
  (IGQueueAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGQueueAsTable <Element, ElementOps>&
IGQueueAsTable <Element, ElementOps>::
operator= (IGQueueAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueAsTable <Element, ElementOps>::
IGQueueAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGQueueAsTable
 <Element, ElementOps>::Implementation&
IGQueueAsTable <Element, ElementOps>::
ImplOf (IGQueueAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IQueueAsTable
// ---

// public members

template <class Element>
inline
IQueueAsTable <Element>::
IQueueAsTable (INumber numberOfElements)
: IGQueueAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IQueueAsTable <Element>::
IQueueAsTable (PureImplementation& impl)
: IGQueueAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGQueueAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGQueueAsTable <Element, ElementOps>::
IVGQueueAsTable (INumber numberOfElements)
: IGQueueAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGQueueAsTable <Element, ElementOps>::
IVGQueueAsTable (PureImplementation& impl)
: IGQueueAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVQueueAsTable
// ---

// public members

template <class Element>
inline
IVQueueAsTable <Element>::
IVQueueAsTable (INumber numberOfElements)
: IVGQueueAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVQueueAsTable <Element>::
IVQueueAsTable (PureImplementation& impl)
: IVGQueueAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IQUTAB_INL_
