// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 35 1.5.1.4 source/core/collect/irel.h, collection, ioc.v400, 000919 
#ifndef _IREL_
#define _IREL_

#include <ilanglvl.hpp>
#include <iarel.h>
#include <iicrel.h>
#if ! defined (__INO_AVALON__)
#include <iivrel.h>
#endif
#include <iirelhsh.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGRelation;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGRelationCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGRelation
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGRelationCursor
                            (IGRelation
                              <
                               Element, Key, ElementOps> const&);

  IGRelationCursor <Element, Key, ElementOps>&
           operator=        (IGRelationCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGRelationCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGRelationCursor\
                            (IGRelation\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGRelationCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGRelationMngCursor :
  public IGRelationCursor <Element, Key, ElementOps>  {

  typedef  IGRelationCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGRelation
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGRelationMngCursor
                            (IGRelation
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGRelationMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGRelationMngCursor\
                            (IGRelation\
                              <\
                               Element, Key, ElementOps> const&),\
           IGRelationMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGRelation :
  public IARelation <Element, Key> {
protected:

  typedef  IGRelation <Element, Key, ElementOps> Self;

  typedef  IRelationAsHshTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICRelationImpl
            <PureImplementation> Implementation;
#endif

#define IGRelationTypedef(Element, Key, ElementOps)\
  typedef  IRelationAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGRelationTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGRelationCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGRelationMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGRelation
                            (INumber = 100);

           IGRelation
                            (IGRelation
                              <
                               Element, Key, ElementOps> const&);

  IGRelation <Element, Key, ElementOps>&
           operator=        (IGRelation
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGRelation
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGRelation
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGRelationCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGRelationMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGRelation\
                            (INumber),\
           operator=,\
           IGRelation\
                            (PureImplementation&),\
           ImplOf           (IGRelation\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IRelation :
  public IGRelation
          <Element, Key, IEKEHOps <Element, Key> > {

  typedef  IGRelation
            <Element, Key,
             IEKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IRelation <Element, Key> Self;

#define IRelationTypedef(Element, Key)\
  typedef  IRelationAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, IEKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IRelation
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IRelation
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IRelation\
                              (INumber),\
           IRelation\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGRelation :
  public IGRelation
          <Element, Key, ElementOps > {

  typedef  IVGRelation
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGRelation
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVRelationImpl
            <PureImplementation> Implementation;
#else
  typedef  ICRelationImpl
            <IVRelationImpl
              <PureImplementation> > Implementation;
#endif

#define IVGRelationTypedef(Element, Key, ElementOps)\
  typedef  IRelationAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGRelationTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGRelation
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGRelation
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGRelation\
                              (INumber),\
           IVGRelation\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVRelation :
  public IVGRelation
          <Element, Key, IEKEHOps <Element, Key> > {

  typedef  IVGRelation
            <Element, Key,
             IEKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVRelation <Element, Key> Self;

#define IVRelationTypedef(Element, Key)\
  typedef  IVRelationAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, IEKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVRelation
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVRelation
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVRelation\
                              (INumber),\
           IVRelation\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <irel.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IREL_
