// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 36 1.4.1.4 source/core/collect/irel.inl, collection, ioc.v400, 000919  
#ifndef _IREL_INL_
#define _IREL_INL_

// ---
// IGRelationCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGRelationCursor <Element, Key, ElementOps>::
IGRelationCursor
  (IGRelation
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGRelationCursor <Element, Key, ElementOps>&
IGRelationCursor <Element, Key, ElementOps>::
operator=
  (IGRelationCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGRelationCursor <Element, Key, ElementOps>::
IGRelationCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGRelationMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGRelationMngCursor <Element, Key, ElementOps>::
IGRelationMngCursor
  (IGRelation
    <Element, Key, ElementOps> const& collection)
: IGRelationCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGRelationMngCursor <Element, Key, ElementOps>::
IGRelationMngCursor (ICursorImpl* impl)
: IGRelationCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGRelation
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGRelation <Element, Key, ElementOps>::
IGRelation (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGRelation <Element, Key, ElementOps>::
IGRelation
  (IGRelation
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGRelation <Element, Key, ElementOps>&
IGRelation <Element, Key, ElementOps>::
operator= (IGRelation
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGRelation <Element, Key, ElementOps>::
IGRelation (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGRelation
 <Element, Key, ElementOps>::Implementation&
IGRelation <Element, Key, ElementOps>::
ImplOf (IGRelation
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IRelation
// ---

// public members

template <class Element, class Key>
inline
IRelation <Element, Key>::
IRelation (INumber numberOfElements)
: IGRelation
   <Element, Key, IEKEHOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IRelation <Element, Key>::
IRelation (PureImplementation& impl)
: IGRelation
   <Element, Key, IEKEHOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGRelation
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGRelation <Element, Key, ElementOps>::
IVGRelation (INumber numberOfElements)
: IGRelation
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGRelation <Element, Key, ElementOps>::
IVGRelation (PureImplementation& impl)
: IGRelation
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVRelation
// ---

// public members

template <class Element, class Key>
inline
IVRelation <Element, Key>::
IVRelation (INumber numberOfElements)
: IVGRelation
   <Element, Key, IEKEHOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVRelation <Element, Key>::
IVRelation (PureImplementation& impl)
: IVGRelation
   <Element, Key, IEKEHOps <Element, Key> > (impl)
{
}
#endif

#endif // _IREL_INL_
