// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 81 1.4.1.4 source/core/collect/isb.inl, collection, ioc.v400, 000919  
#ifndef _ISB_INL_
#define _ISB_INL_

// ---
// IGSortedBagCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSortedBagCursor <Element, ElementOps>::
IGSortedBagCursor
  (IGSortedBag
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSortedBagCursor <Element, ElementOps>&
IGSortedBagCursor <Element, ElementOps>::
operator=
  (IGSortedBagCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedBagCursor <Element, ElementOps>::
IGSortedBagCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedBagMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedBagMngCursor <Element, ElementOps>::
IGSortedBagMngCursor
  (IGSortedBag
    <Element, ElementOps> const& collection)
: IGSortedBagCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedBagMngCursor <Element, ElementOps>::
IGSortedBagMngCursor (ICursorImpl* impl)
: IGSortedBagCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedBag
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedBag <Element, ElementOps>::
IGSortedBag (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSortedBag <Element, ElementOps>::
IGSortedBag
  (IGSortedBag
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSortedBag <Element, ElementOps>&
IGSortedBag <Element, ElementOps>::
operator= (IGSortedBag
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedBag <Element, ElementOps>::
IGSortedBag (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSortedBag
 <Element, ElementOps>::Implementation&
IGSortedBag <Element, ElementOps>::
ImplOf (IGSortedBag
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedBag
// ---

// public members

template <class Element>
inline
ISortedBag <Element>::
ISortedBag (INumber numberOfElements)
: IGSortedBag
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISortedBag <Element>::
ISortedBag (PureImplementation& impl)
: IGSortedBag
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedBag
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSortedBag <Element, ElementOps>::
IVGSortedBag (INumber numberOfElements)
: IGSortedBag
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSortedBag <Element, ElementOps>::
IVGSortedBag (PureImplementation& impl)
: IGSortedBag
    <Element, ElementOps> (impl)
{
}

// ---
// IVSortedBag
// ---

// public members

template <class Element>
inline
IVSortedBag <Element>::
IVSortedBag (INumber numberOfElements)
: IVGSortedBag
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSortedBag <Element>::
IVSortedBag (PureImplementation& impl)
: IVGSortedBag
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISB_INL_
