// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 96 1.4.1.4 source/core/collect/iseq.inl, collection, ioc.v400, 000919  
#ifndef _ISEQ_INL_
#define _ISEQ_INL_

// ---
// IGSequenceCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSequenceCursor <Element, ElementOps>::
IGSequenceCursor
  (IGSequence
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSequenceCursor <Element, ElementOps>&
IGSequenceCursor <Element, ElementOps>::
operator=
  (IGSequenceCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSequenceCursor <Element, ElementOps>::
IGSequenceCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSequenceMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSequenceMngCursor <Element, ElementOps>::
IGSequenceMngCursor
  (IGSequence
    <Element, ElementOps> const& collection)
: IGSequenceCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSequenceMngCursor <Element, ElementOps>::
IGSequenceMngCursor (ICursorImpl* impl)
: IGSequenceCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSequence
// ---

// public members

template <class Element, class ElementOps>
inline
IGSequence <Element, ElementOps>::
IGSequence (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSequence <Element, ElementOps>::
IGSequence
  (IGSequence
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSequence <Element, ElementOps>&
IGSequence <Element, ElementOps>::
operator= (IGSequence
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSequence <Element, ElementOps>::
IGSequence (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSequence
 <Element, ElementOps>::Implementation&
IGSequence <Element, ElementOps>::
ImplOf (IGSequence
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISequence
// ---

// public members

template <class Element>
inline
ISequence <Element>::
ISequence (INumber numberOfElements)
: IGSequence
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISequence <Element>::
ISequence (PureImplementation& impl)
: IGSequence
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSequence
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSequence <Element, ElementOps>::
IVGSequence (INumber numberOfElements)
: IGSequence
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSequence <Element, ElementOps>::
IVGSequence (PureImplementation& impl)
: IGSequence
    <Element, ElementOps> (impl)
{
}

// ---
// IVSequence
// ---

// public members

template <class Element>
inline
IVSequence <Element>::
IVSequence (INumber numberOfElements)
: IVGSequence
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSequence <Element>::
IVSequence (PureImplementation& impl)
: IVGSequence
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _ISEQ_INL_
