// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 04 1.4.1.4 source/core/collect/iset.inl, collection, ioc.v400, 000919  
#ifndef _ISET_INL_
#define _ISET_INL_

// ---
// IGSetCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSetCursor <Element, ElementOps>::
IGSetCursor
  (IGSet
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSetCursor <Element, ElementOps>&
IGSetCursor <Element, ElementOps>::
operator=
  (IGSetCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSetCursor <Element, ElementOps>::
IGSetCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSetMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSetMngCursor <Element, ElementOps>::
IGSetMngCursor
  (IGSet
    <Element, ElementOps> const& collection)
: IGSetCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSetMngCursor <Element, ElementOps>::
IGSetMngCursor (ICursorImpl* impl)
: IGSetCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSet
// ---

// public members

template <class Element, class ElementOps>
inline
IGSet <Element, ElementOps>::
IGSet (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSet <Element, ElementOps>::
IGSet
  (IGSet
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSet <Element, ElementOps>&
IGSet <Element, ElementOps>::
operator= (IGSet
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSet <Element, ElementOps>::
IGSet (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSet
 <Element, ElementOps>::Implementation&
IGSet <Element, ElementOps>::
ImplOf (IGSet
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISet
// ---

// public members

template <class Element>
inline
ISet <Element>::
ISet (INumber numberOfElements)
: IGSet
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISet <Element>::
ISet (PureImplementation& impl)
: IGSet
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSet
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSet <Element, ElementOps>::
IVGSet (INumber numberOfElements)
: IGSet
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSet <Element, ElementOps>::
IVGSet (PureImplementation& impl)
: IGSet
    <Element, ElementOps> (impl)
{
}

// ---
// IVSet
// ---

// public members

template <class Element>
inline
IVSet <Element>::
IVSet (INumber numberOfElements)
: IVGSet
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSet <Element>::
IVSet (PureImplementation& impl)
: IVGSet
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISET_INL_
