// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 08 1.4.1.4 source/core/collect/isetbst.inl, collection, ioc.v400, 000919  
#ifndef _ISETBST_INL_
#define _ISETBST_INL_

// ---
// IGSetAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSetAsBstTreeCursor <Element, ElementOps>::
IGSetAsBstTreeCursor
  (IGSetAsBstTree
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSetAsBstTreeCursor <Element, ElementOps>&
IGSetAsBstTreeCursor <Element, ElementOps>::
operator=
  (IGSetAsBstTreeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSetAsBstTreeCursor <Element, ElementOps>::
IGSetAsBstTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSetAsBstTreeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSetAsBstTreeMngCursor <Element, ElementOps>::
IGSetAsBstTreeMngCursor
  (IGSetAsBstTree
    <Element, ElementOps> const& collection)
: IGSetAsBstTreeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSetAsBstTreeMngCursor <Element, ElementOps>::
IGSetAsBstTreeMngCursor (ICursorImpl* impl)
: IGSetAsBstTreeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSetAsBstTree
// ---

// public members

template <class Element, class ElementOps>
inline
IGSetAsBstTree <Element, ElementOps>::
IGSetAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSetAsBstTree <Element, ElementOps>::
IGSetAsBstTree
  (IGSetAsBstTree
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSetAsBstTree <Element, ElementOps>&
IGSetAsBstTree <Element, ElementOps>::
operator= (IGSetAsBstTree
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSetAsBstTree <Element, ElementOps>::
IGSetAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSetAsBstTree
 <Element, ElementOps>::Implementation&
IGSetAsBstTree <Element, ElementOps>::
ImplOf (IGSetAsBstTree
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISetAsBstTree
// ---

// public members

template <class Element>
inline
ISetAsBstTree <Element>::
ISetAsBstTree (INumber numberOfElements)
: IGSetAsBstTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISetAsBstTree <Element>::
ISetAsBstTree (PureImplementation& impl)
: IGSetAsBstTree
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSetAsBstTree
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSetAsBstTree <Element, ElementOps>::
IVGSetAsBstTree (INumber numberOfElements)
: IGSetAsBstTree
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSetAsBstTree <Element, ElementOps>::
IVGSetAsBstTree (PureImplementation& impl)
: IGSetAsBstTree
    <Element, ElementOps> (impl)
{
}

// ---
// IVSetAsBstTree
// ---

// public members

template <class Element>
inline
IVSetAsBstTree <Element>::
IVSetAsBstTree (INumber numberOfElements)
: IVGSetAsBstTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSetAsBstTree <Element>::
IVSetAsBstTree (PureImplementation& impl)
: IVGSetAsBstTree
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISETBST_INL_
