// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 22 1.4.1.4 source/core/collect/ism.inl, collection, ioc.v400, 000919  
#ifndef _ISM_INL_
#define _ISM_INL_

// ---
// IGSortedMapCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapCursor <Element, Key, ElementOps>::
IGSortedMapCursor
  (IGSortedMap
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapCursor <Element, Key, ElementOps>&
IGSortedMapCursor <Element, Key, ElementOps>::
operator=
  (IGSortedMapCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapCursor <Element, Key, ElementOps>::
IGSortedMapCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedMapMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapMngCursor <Element, Key, ElementOps>::
IGSortedMapMngCursor
  (IGSortedMap
    <Element, Key, ElementOps> const& collection)
: IGSortedMapCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapMngCursor <Element, Key, ElementOps>::
IGSortedMapMngCursor (ICursorImpl* impl)
: IGSortedMapCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedMap
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMap <Element, Key, ElementOps>::
IGSortedMap (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedMap <Element, Key, ElementOps>::
IGSortedMap
  (IGSortedMap
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedMap <Element, Key, ElementOps>&
IGSortedMap <Element, Key, ElementOps>::
operator= (IGSortedMap
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMap <Element, Key, ElementOps>::
IGSortedMap (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedMap
 <Element, Key, ElementOps>::Implementation&
IGSortedMap <Element, Key, ElementOps>::
ImplOf (IGSortedMap
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedMap
// ---

// public members

template <class Element, class Key>
inline
ISortedMap <Element, Key>::
ISortedMap (INumber numberOfElements)
: IGSortedMap
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedMap <Element, Key>::
ISortedMap (PureImplementation& impl)
: IGSortedMap
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedMap
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMap <Element, Key, ElementOps>::
IVGSortedMap (INumber numberOfElements)
: IGSortedMap
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMap <Element, Key, ElementOps>::
IVGSortedMap (PureImplementation& impl)
: IGSortedMap
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedMap
// ---

// public members

template <class Element, class Key>
inline
IVSortedMap <Element, Key>::
IVSortedMap (INumber numberOfElements)
: IVGSortedMap
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedMap <Element, Key>::
IVSortedMap (PureImplementation& impl)
: IVGSortedMap
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISM_INL_
