// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 30 1.4.1.4 source/core/collect/ismlst.inl, collection, ioc.v400, 000919  
#ifndef _ISMLST_INL_
#define _ISMLST_INL_

// ---
// IGSortedMapAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsListCursor <Element, Key, ElementOps>::
IGSortedMapAsListCursor
  (IGSortedMapAsList
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsListCursor <Element, Key, ElementOps>&
IGSortedMapAsListCursor <Element, Key, ElementOps>::
operator=
  (IGSortedMapAsListCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsListCursor <Element, Key, ElementOps>::
IGSortedMapAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedMapAsListMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsListMngCursor <Element, Key, ElementOps>::
IGSortedMapAsListMngCursor
  (IGSortedMapAsList
    <Element, Key, ElementOps> const& collection)
: IGSortedMapAsListCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsListMngCursor <Element, Key, ElementOps>::
IGSortedMapAsListMngCursor (ICursorImpl* impl)
: IGSortedMapAsListCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedMapAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsList <Element, Key, ElementOps>::
IGSortedMapAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsList <Element, Key, ElementOps>::
IGSortedMapAsList
  (IGSortedMapAsList
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsList <Element, Key, ElementOps>&
IGSortedMapAsList <Element, Key, ElementOps>::
operator= (IGSortedMapAsList
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsList <Element, Key, ElementOps>::
IGSortedMapAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedMapAsList
 <Element, Key, ElementOps>::Implementation&
IGSortedMapAsList <Element, Key, ElementOps>::
ImplOf (IGSortedMapAsList
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedMapAsList
// ---

// public members

template <class Element, class Key>
inline
ISortedMapAsList <Element, Key>::
ISortedMapAsList (INumber numberOfElements)
: IGSortedMapAsList
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedMapAsList <Element, Key>::
ISortedMapAsList (PureImplementation& impl)
: IGSortedMapAsList
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedMapAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsList <Element, Key, ElementOps>::
IVGSortedMapAsList (INumber numberOfElements)
: IGSortedMapAsList
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsList <Element, Key, ElementOps>::
IVGSortedMapAsList (PureImplementation& impl)
: IGSortedMapAsList
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedMapAsList
// ---

// public members

template <class Element, class Key>
inline
IVSortedMapAsList <Element, Key>::
IVSortedMapAsList (INumber numberOfElements)
: IVGSortedMapAsList
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedMapAsList <Element, Key>::
IVSortedMapAsList (PureImplementation& impl)
: IVGSortedMapAsList
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISMLST_INL_
