// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 35 1.4.1.4 source/core/collect/ismtab.inl, collection, ioc.v400, 000919  
#ifndef _ISMTAB_INL_
#define _ISMTAB_INL_

// ---
// IGSortedMapAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTableCursor <Element, Key, ElementOps>::
IGSortedMapAsTableCursor
  (IGSortedMapAsTable
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsTableCursor <Element, Key, ElementOps>&
IGSortedMapAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGSortedMapAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTableCursor <Element, Key, ElementOps>::
IGSortedMapAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedMapAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTableMngCursor <Element, Key, ElementOps>::
IGSortedMapAsTableMngCursor
  (IGSortedMapAsTable
    <Element, Key, ElementOps> const& collection)
: IGSortedMapAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTableMngCursor <Element, Key, ElementOps>::
IGSortedMapAsTableMngCursor (ICursorImpl* impl)
: IGSortedMapAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedMapAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTable <Element, Key, ElementOps>::
IGSortedMapAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTable <Element, Key, ElementOps>::
IGSortedMapAsTable
  (IGSortedMapAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsTable <Element, Key, ElementOps>&
IGSortedMapAsTable <Element, Key, ElementOps>::
operator= (IGSortedMapAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsTable <Element, Key, ElementOps>::
IGSortedMapAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedMapAsTable
 <Element, Key, ElementOps>::Implementation&
IGSortedMapAsTable <Element, Key, ElementOps>::
ImplOf (IGSortedMapAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedMapAsTable
// ---

// public members

template <class Element, class Key>
inline
ISortedMapAsTable <Element, Key>::
ISortedMapAsTable (INumber numberOfElements)
: IGSortedMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedMapAsTable <Element, Key>::
ISortedMapAsTable (PureImplementation& impl)
: IGSortedMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedMapAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsTable <Element, Key, ElementOps>::
IVGSortedMapAsTable (INumber numberOfElements)
: IGSortedMapAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsTable <Element, Key, ElementOps>::
IVGSortedMapAsTable (PureImplementation& impl)
: IGSortedMapAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedMapAsTable
// ---

// public members

template <class Element, class Key>
inline
IVSortedMapAsTable <Element, Key>::
IVSortedMapAsTable (INumber numberOfElements)
: IVGSortedMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedMapAsTable <Element, Key>::
IVSortedMapAsTable (PureImplementation& impl)
: IVGSortedMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISMTAB_INL_
