// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 37 1.4.1.4 source/core/collect/isr.inl, collection, ioc.v400, 000919  
#ifndef _ISR_INL_
#define _ISR_INL_

// ---
// IGSortedRelationCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationCursor <Element, Key, ElementOps>::
IGSortedRelationCursor
  (IGSortedRelation
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedRelationCursor <Element, Key, ElementOps>&
IGSortedRelationCursor <Element, Key, ElementOps>::
operator=
  (IGSortedRelationCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationCursor <Element, Key, ElementOps>::
IGSortedRelationCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedRelationMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationMngCursor <Element, Key, ElementOps>::
IGSortedRelationMngCursor
  (IGSortedRelation
    <Element, Key, ElementOps> const& collection)
: IGSortedRelationCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationMngCursor <Element, Key, ElementOps>::
IGSortedRelationMngCursor (ICursorImpl* impl)
: IGSortedRelationCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedRelation
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelation <Element, Key, ElementOps>::
IGSortedRelation (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedRelation <Element, Key, ElementOps>::
IGSortedRelation
  (IGSortedRelation
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedRelation <Element, Key, ElementOps>&
IGSortedRelation <Element, Key, ElementOps>::
operator= (IGSortedRelation
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelation <Element, Key, ElementOps>::
IGSortedRelation (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedRelation
 <Element, Key, ElementOps>::Implementation&
IGSortedRelation <Element, Key, ElementOps>::
ImplOf (IGSortedRelation
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedRelation
// ---

// public members

template <class Element, class Key>
inline
ISortedRelation <Element, Key>::
ISortedRelation (INumber numberOfElements)
: IGSortedRelation
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedRelation <Element, Key>::
ISortedRelation (PureImplementation& impl)
: IGSortedRelation
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedRelation
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedRelation <Element, Key, ElementOps>::
IVGSortedRelation (INumber numberOfElements)
: IGSortedRelation
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedRelation <Element, Key, ElementOps>::
IVGSortedRelation (PureImplementation& impl)
: IGSortedRelation
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedRelation
// ---

// public members

template <class Element, class Key>
inline
IVSortedRelation <Element, Key>::
IVSortedRelation (INumber numberOfElements)
: IVGSortedRelation
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedRelation <Element, Key>::
IVSortedRelation (PureImplementation& impl)
: IVGSortedRelation
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISR_INL_
