// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 45 1.5.1.4 source/core/collect/isrtab.h, collection, ioc.v400, 000919 
#ifndef _ISRTAB_
#define _ISRTAB_

#include <ilanglvl.hpp>
#include <iasr.h>
#include <iicsr.h>
#if ! defined (__INO_AVALON__)
#include <iivsr.h>
#endif
#include <iisrtab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGSortedRelationAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGSortedRelationAsTableCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedRelationAsTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedRelationAsTableCursor
                            (IGSortedRelationAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGSortedRelationAsTableCursor <Element, Key, ElementOps>&
           operator=        (IGSortedRelationAsTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedRelationAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedRelationAsTableCursor\
                            (IGSortedRelationAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGSortedRelationAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGSortedRelationAsTableMngCursor :
  public IGSortedRelationAsTableCursor <Element, Key, ElementOps>  {

  typedef  IGSortedRelationAsTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGSortedRelationAsTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedRelationAsTableMngCursor
                            (IGSortedRelationAsTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedRelationAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedRelationAsTableMngCursor\
                            (IGSortedRelationAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGSortedRelationAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGSortedRelationAsTable :
  public IASortedRelation <Element, Key> {
protected:

  typedef  IGSortedRelationAsTable <Element, Key, ElementOps> Self;

  typedef  ISortedRelationAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedRelationImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedRelationAsTableTypedef(Element, Key, ElementOps)\
  typedef  ISortedRelationAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGSortedRelationAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedRelationAsTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedRelationAsTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGSortedRelationAsTable
                            (INumber = 100);

           IGSortedRelationAsTable
                            (IGSortedRelationAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGSortedRelationAsTable <Element, Key, ElementOps>&
           operator=        (IGSortedRelationAsTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedRelationAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedRelationAsTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGSortedRelationAsTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedRelationAsTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedRelationAsTable\
                            (INumber),\
           operator=,\
           IGSortedRelationAsTable\
                            (PureImplementation&),\
           ImplOf           (IGSortedRelationAsTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class ISortedRelationAsTable :
  public IGSortedRelationAsTable
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IGSortedRelationAsTable
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedRelationAsTable <Element, Key> Self;

#define ISortedRelationAsTableTypedef(Element, Key)\
  typedef  ISortedRelationAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedRelationAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedRelationAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedRelationAsTable\
                              (INumber),\
           ISortedRelationAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGSortedRelationAsTable :
  public IGSortedRelationAsTable
          <Element, Key, ElementOps > {

  typedef  IVGSortedRelationAsTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedRelationAsTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedRelationImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedRelationImpl
            <IVSortedRelationImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedRelationAsTableTypedef(Element, Key, ElementOps)\
  typedef  ISortedRelationAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGSortedRelationAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedRelationAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedRelationAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedRelationAsTable\
                              (INumber),\
           IVGSortedRelationAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVSortedRelationAsTable :
  public IVGSortedRelationAsTable
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IVGSortedRelationAsTable
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedRelationAsTable <Element, Key> Self;

#define IVSortedRelationAsTableTypedef(Element, Key)\
  typedef  IVSortedRelationAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedRelationAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedRelationAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedRelationAsTable\
                              (INumber),\
           IVSortedRelationAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <isrtab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISRTAB_
