// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 46 1.4.1.4 source/core/collect/isrtab.inl, collection, ioc.v400, 000919  
#ifndef _ISRTAB_INL_
#define _ISRTAB_INL_

// ---
// IGSortedRelationAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTableCursor <Element, Key, ElementOps>::
IGSortedRelationAsTableCursor
  (IGSortedRelationAsTable
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedRelationAsTableCursor <Element, Key, ElementOps>&
IGSortedRelationAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGSortedRelationAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTableCursor <Element, Key, ElementOps>::
IGSortedRelationAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedRelationAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTableMngCursor <Element, Key, ElementOps>::
IGSortedRelationAsTableMngCursor
  (IGSortedRelationAsTable
    <Element, Key, ElementOps> const& collection)
: IGSortedRelationAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTableMngCursor <Element, Key, ElementOps>::
IGSortedRelationAsTableMngCursor (ICursorImpl* impl)
: IGSortedRelationAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedRelationAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTable <Element, Key, ElementOps>::
IGSortedRelationAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTable <Element, Key, ElementOps>::
IGSortedRelationAsTable
  (IGSortedRelationAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedRelationAsTable <Element, Key, ElementOps>&
IGSortedRelationAsTable <Element, Key, ElementOps>::
operator= (IGSortedRelationAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedRelationAsTable <Element, Key, ElementOps>::
IGSortedRelationAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedRelationAsTable
 <Element, Key, ElementOps>::Implementation&
IGSortedRelationAsTable <Element, Key, ElementOps>::
ImplOf (IGSortedRelationAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedRelationAsTable
// ---

// public members

template <class Element, class Key>
inline
ISortedRelationAsTable <Element, Key>::
ISortedRelationAsTable (INumber numberOfElements)
: IGSortedRelationAsTable
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedRelationAsTable <Element, Key>::
ISortedRelationAsTable (PureImplementation& impl)
: IGSortedRelationAsTable
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedRelationAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedRelationAsTable <Element, Key, ElementOps>::
IVGSortedRelationAsTable (INumber numberOfElements)
: IGSortedRelationAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedRelationAsTable <Element, Key, ElementOps>::
IVGSortedRelationAsTable (PureImplementation& impl)
: IGSortedRelationAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedRelationAsTable
// ---

// public members

template <class Element, class Key>
inline
IVSortedRelationAsTable <Element, Key>::
IVSortedRelationAsTable (INumber numberOfElements)
: IVGSortedRelationAsTable
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedRelationAsTable <Element, Key>::
IVSortedRelationAsTable (PureImplementation& impl)
: IVGSortedRelationAsTable
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISRTAB_INL_
