// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 53 1.5.1.4 source/core/collect/issavl.h, collection, ioc.v400, 000919 
#ifndef _ISSAVL_
#define _ISSAVL_

#include <ilanglvl.hpp>
#include <iass.h>
#include <iicss.h>
#if ! defined (__INO_AVALON__)
#include <iivss.h>
#endif
#include <iissavl.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSortedSetAsAvlTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSortedSetAsAvlTreeCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedSetAsAvlTree
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedSetAsAvlTreeCursor
                            (IGSortedSetAsAvlTree
                              <
                               Element, ElementOps> const&);

  IGSortedSetAsAvlTreeCursor <Element, ElementOps>&
           operator=        (IGSortedSetAsAvlTreeCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetAsAvlTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsAvlTreeCursor\
                            (IGSortedSetAsAvlTree\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSortedSetAsAvlTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSortedSetAsAvlTreeMngCursor :
  public IGSortedSetAsAvlTreeCursor <Element, ElementOps>  {

  typedef  IGSortedSetAsAvlTreeCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSortedSetAsAvlTree
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedSetAsAvlTreeMngCursor
                            (IGSortedSetAsAvlTree
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetAsAvlTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsAvlTreeMngCursor\
                            (IGSortedSetAsAvlTree\
                              <\
                               Element, ElementOps> const&),\
           IGSortedSetAsAvlTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSortedSetAsAvlTree :
  public IASortedSet <Element> {
protected:

  typedef  IGSortedSetAsAvlTree <Element, ElementOps> Self;

  typedef  ISortedSetAsAvlTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedSetAsAvlTreeTypedef(Element, ElementOps)\
  typedef  ISortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSortedSetAsAvlTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedSetAsAvlTreeCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedSetAsAvlTreeMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSortedSetAsAvlTree
                            (INumber = 100);

           IGSortedSetAsAvlTree
                            (IGSortedSetAsAvlTree
                              <
                               Element, ElementOps> const&);

  IGSortedSetAsAvlTree <Element, ElementOps>&
           operator=        (IGSortedSetAsAvlTree
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedSetAsAvlTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedSetAsAvlTree
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSortedSetAsAvlTreeCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedSetAsAvlTreeMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsAvlTree\
                            (INumber),\
           operator=,\
           IGSortedSetAsAvlTree\
                            (PureImplementation&),\
           ImplOf           (IGSortedSetAsAvlTree\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISortedSetAsAvlTree :
  public IGSortedSetAsAvlTree
          <Element, ICOps <Element> > {

  typedef  IGSortedSetAsAvlTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedSetAsAvlTree <Element> Self;

#define ISortedSetAsAvlTreeTypedef(Element)\
  typedef  ISortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedSetAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedSetAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedSetAsAvlTree\
                              (INumber),\
           ISortedSetAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSortedSetAsAvlTree :
  public IGSortedSetAsAvlTree
          <Element, ElementOps > {

  typedef  IVGSortedSetAsAvlTree
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedSetAsAvlTree
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedSetImpl
            <IVSortedSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedSetAsAvlTreeTypedef(Element, ElementOps)\
  typedef  ISortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSortedSetAsAvlTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedSetAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedSetAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedSetAsAvlTree\
                              (INumber),\
           IVGSortedSetAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSortedSetAsAvlTree :
  public IVGSortedSetAsAvlTree
          <Element, ICOps <Element> > {

  typedef  IVGSortedSetAsAvlTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedSetAsAvlTree <Element> Self;

#define IVSortedSetAsAvlTreeTypedef(Element)\
  typedef  IVSortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedSetAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedSetAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedSetAsAvlTree\
                              (INumber),\
           IVSortedSetAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <issavl.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISSAVL_
