// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 54 1.4.1.4 source/core/collect/issavl.inl, collection, ioc.v400, 000919  
#ifndef _ISSAVL_INL_
#define _ISSAVL_INL_

// ---
// IGSortedSetAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTreeCursor <Element, ElementOps>::
IGSortedSetAsAvlTreeCursor
  (IGSortedSetAsAvlTree
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSortedSetAsAvlTreeCursor <Element, ElementOps>&
IGSortedSetAsAvlTreeCursor <Element, ElementOps>::
operator=
  (IGSortedSetAsAvlTreeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTreeCursor <Element, ElementOps>::
IGSortedSetAsAvlTreeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedSetAsAvlTreeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTreeMngCursor <Element, ElementOps>::
IGSortedSetAsAvlTreeMngCursor
  (IGSortedSetAsAvlTree
    <Element, ElementOps> const& collection)
: IGSortedSetAsAvlTreeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTreeMngCursor <Element, ElementOps>::
IGSortedSetAsAvlTreeMngCursor (ICursorImpl* impl)
: IGSortedSetAsAvlTreeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedSetAsAvlTree
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTree <Element, ElementOps>::
IGSortedSetAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTree <Element, ElementOps>::
IGSortedSetAsAvlTree
  (IGSortedSetAsAvlTree
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSortedSetAsAvlTree <Element, ElementOps>&
IGSortedSetAsAvlTree <Element, ElementOps>::
operator= (IGSortedSetAsAvlTree
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsAvlTree <Element, ElementOps>::
IGSortedSetAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSortedSetAsAvlTree
 <Element, ElementOps>::Implementation&
IGSortedSetAsAvlTree <Element, ElementOps>::
ImplOf (IGSortedSetAsAvlTree
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedSetAsAvlTree
// ---

// public members

template <class Element>
inline
ISortedSetAsAvlTree <Element>::
ISortedSetAsAvlTree (INumber numberOfElements)
: IGSortedSetAsAvlTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISortedSetAsAvlTree <Element>::
ISortedSetAsAvlTree (PureImplementation& impl)
: IGSortedSetAsAvlTree
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedSetAsAvlTree
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSortedSetAsAvlTree <Element, ElementOps>::
IVGSortedSetAsAvlTree (INumber numberOfElements)
: IGSortedSetAsAvlTree
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSortedSetAsAvlTree <Element, ElementOps>::
IVGSortedSetAsAvlTree (PureImplementation& impl)
: IGSortedSetAsAvlTree
    <Element, ElementOps> (impl)
{
}

// ---
// IVSortedSetAsAvlTree
// ---

// public members

template <class Element>
inline
IVSortedSetAsAvlTree <Element>::
IVSortedSetAsAvlTree (INumber numberOfElements)
: IVGSortedSetAsAvlTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSortedSetAsAvlTree <Element>::
IVSortedSetAsAvlTree (PureImplementation& impl)
: IVGSortedSetAsAvlTree
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISSAVL_INL_
