// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 56 1.4.1.4 source/core/collect/issbst.inl, collection, ioc.v400, 000919  
#ifndef _ISSBST_INL_
#define _ISSBST_INL_

// ---
// IGSortedSetAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTreeCursor <Element, ElementOps>::
IGSortedSetAsBstTreeCursor
  (IGSortedSetAsBstTree
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSortedSetAsBstTreeCursor <Element, ElementOps>&
IGSortedSetAsBstTreeCursor <Element, ElementOps>::
operator=
  (IGSortedSetAsBstTreeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTreeCursor <Element, ElementOps>::
IGSortedSetAsBstTreeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedSetAsBstTreeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTreeMngCursor <Element, ElementOps>::
IGSortedSetAsBstTreeMngCursor
  (IGSortedSetAsBstTree
    <Element, ElementOps> const& collection)
: IGSortedSetAsBstTreeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTreeMngCursor <Element, ElementOps>::
IGSortedSetAsBstTreeMngCursor (ICursorImpl* impl)
: IGSortedSetAsBstTreeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedSetAsBstTree
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTree <Element, ElementOps>::
IGSortedSetAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTree <Element, ElementOps>::
IGSortedSetAsBstTree
  (IGSortedSetAsBstTree
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSortedSetAsBstTree <Element, ElementOps>&
IGSortedSetAsBstTree <Element, ElementOps>::
operator= (IGSortedSetAsBstTree
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsBstTree <Element, ElementOps>::
IGSortedSetAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSortedSetAsBstTree
 <Element, ElementOps>::Implementation&
IGSortedSetAsBstTree <Element, ElementOps>::
ImplOf (IGSortedSetAsBstTree
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedSetAsBstTree
// ---

// public members

template <class Element>
inline
ISortedSetAsBstTree <Element>::
ISortedSetAsBstTree (INumber numberOfElements)
: IGSortedSetAsBstTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISortedSetAsBstTree <Element>::
ISortedSetAsBstTree (PureImplementation& impl)
: IGSortedSetAsBstTree
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedSetAsBstTree
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSortedSetAsBstTree <Element, ElementOps>::
IVGSortedSetAsBstTree (INumber numberOfElements)
: IGSortedSetAsBstTree
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSortedSetAsBstTree <Element, ElementOps>::
IVGSortedSetAsBstTree (PureImplementation& impl)
: IGSortedSetAsBstTree
    <Element, ElementOps> (impl)
{
}

// ---
// IVSortedSetAsBstTree
// ---

// public members

template <class Element>
inline
IVSortedSetAsBstTree <Element>::
IVSortedSetAsBstTree (INumber numberOfElements)
: IVGSortedSetAsBstTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSortedSetAsBstTree <Element>::
IVSortedSetAsBstTree (PureImplementation& impl)
: IVGSortedSetAsBstTree
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISSBST_INL_
