// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 60 1.4.1.4 source/core/collect/isslst.inl, collection, ioc.v400, 000919  
#ifndef _ISSLST_INL_
#define _ISSLST_INL_

// ---
// IGSortedSetAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsListCursor <Element, ElementOps>::
IGSortedSetAsListCursor
  (IGSortedSetAsList
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSortedSetAsListCursor <Element, ElementOps>&
IGSortedSetAsListCursor <Element, ElementOps>::
operator=
  (IGSortedSetAsListCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsListCursor <Element, ElementOps>::
IGSortedSetAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedSetAsListMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsListMngCursor <Element, ElementOps>::
IGSortedSetAsListMngCursor
  (IGSortedSetAsList
    <Element, ElementOps> const& collection)
: IGSortedSetAsListCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsListMngCursor <Element, ElementOps>::
IGSortedSetAsListMngCursor (ICursorImpl* impl)
: IGSortedSetAsListCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedSetAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsList <Element, ElementOps>::
IGSortedSetAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSortedSetAsList <Element, ElementOps>::
IGSortedSetAsList
  (IGSortedSetAsList
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSortedSetAsList <Element, ElementOps>&
IGSortedSetAsList <Element, ElementOps>::
operator= (IGSortedSetAsList
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsList <Element, ElementOps>::
IGSortedSetAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSortedSetAsList
 <Element, ElementOps>::Implementation&
IGSortedSetAsList <Element, ElementOps>::
ImplOf (IGSortedSetAsList
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedSetAsList
// ---

// public members

template <class Element>
inline
ISortedSetAsList <Element>::
ISortedSetAsList (INumber numberOfElements)
: IGSortedSetAsList
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISortedSetAsList <Element>::
ISortedSetAsList (PureImplementation& impl)
: IGSortedSetAsList
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedSetAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSortedSetAsList <Element, ElementOps>::
IVGSortedSetAsList (INumber numberOfElements)
: IGSortedSetAsList
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSortedSetAsList <Element, ElementOps>::
IVGSortedSetAsList (PureImplementation& impl)
: IGSortedSetAsList
    <Element, ElementOps> (impl)
{
}

// ---
// IVSortedSetAsList
// ---

// public members

template <class Element>
inline
IVSortedSetAsList <Element>::
IVSortedSetAsList (INumber numberOfElements)
: IVGSortedSetAsList
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSortedSetAsList <Element>::
IVSortedSetAsList (PureImplementation& impl)
: IVGSortedSetAsList
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISSLST_INL_
