// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 64 1.5.1.4 source/core/collect/isstab.h, collection, ioc.v400, 000919 
#ifndef _ISSTAB_
#define _ISSTAB_

#include <ilanglvl.hpp>
#include <iass.h>
#include <iicss.h>
#if ! defined (__INO_AVALON__)
#include <iivss.h>
#endif
#include <iisstab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSortedSetAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSortedSetAsTableCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedSetAsTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedSetAsTableCursor
                            (IGSortedSetAsTable
                              <
                               Element, ElementOps> const&);

  IGSortedSetAsTableCursor <Element, ElementOps>&
           operator=        (IGSortedSetAsTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsTableCursor\
                            (IGSortedSetAsTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSortedSetAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSortedSetAsTableMngCursor :
  public IGSortedSetAsTableCursor <Element, ElementOps>  {

  typedef  IGSortedSetAsTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSortedSetAsTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedSetAsTableMngCursor
                            (IGSortedSetAsTable
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsTableMngCursor\
                            (IGSortedSetAsTable\
                              <\
                               Element, ElementOps> const&),\
           IGSortedSetAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSortedSetAsTable :
  public IASortedSet <Element> {
protected:

  typedef  IGSortedSetAsTable <Element, ElementOps> Self;

  typedef  ISortedSetAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedSetAsTableTypedef(Element, ElementOps)\
  typedef  ISortedSetAsTableOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSortedSetAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedSetAsTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedSetAsTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSortedSetAsTable
                            (INumber = 100);

           IGSortedSetAsTable
                            (IGSortedSetAsTable
                              <
                               Element, ElementOps> const&);

  IGSortedSetAsTable <Element, ElementOps>&
           operator=        (IGSortedSetAsTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedSetAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedSetAsTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSortedSetAsTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedSetAsTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsTable\
                            (INumber),\
           operator=,\
           IGSortedSetAsTable\
                            (PureImplementation&),\
           ImplOf           (IGSortedSetAsTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISortedSetAsTable :
  public IGSortedSetAsTable
          <Element, ICOps <Element> > {

  typedef  IGSortedSetAsTable
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedSetAsTable <Element> Self;

#define ISortedSetAsTableTypedef(Element)\
  typedef  ISortedSetAsTableOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedSetAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedSetAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedSetAsTable\
                              (INumber),\
           ISortedSetAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSortedSetAsTable :
  public IGSortedSetAsTable
          <Element, ElementOps > {

  typedef  IVGSortedSetAsTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedSetAsTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedSetImpl
            <IVSortedSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedSetAsTableTypedef(Element, ElementOps)\
  typedef  ISortedSetAsTableOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSortedSetAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedSetAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedSetAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedSetAsTable\
                              (INumber),\
           IVGSortedSetAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSortedSetAsTable :
  public IVGSortedSetAsTable
          <Element, ICOps <Element> > {

  typedef  IVGSortedSetAsTable
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedSetAsTable <Element> Self;

#define IVSortedSetAsTableTypedef(Element)\
  typedef  IVSortedSetAsTableOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedSetAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedSetAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedSetAsTable\
                              (INumber),\
           IVSortedSetAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <isstab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISSTAB_
