// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 65 1.4.1.4 source/core/collect/isstab.inl, collection, ioc.v400, 000919  
#ifndef _ISSTAB_INL_
#define _ISSTAB_INL_

// ---
// IGSortedSetAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsTableCursor <Element, ElementOps>::
IGSortedSetAsTableCursor
  (IGSortedSetAsTable
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSortedSetAsTableCursor <Element, ElementOps>&
IGSortedSetAsTableCursor <Element, ElementOps>::
operator=
  (IGSortedSetAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsTableCursor <Element, ElementOps>::
IGSortedSetAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedSetAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsTableMngCursor <Element, ElementOps>::
IGSortedSetAsTableMngCursor
  (IGSortedSetAsTable
    <Element, ElementOps> const& collection)
: IGSortedSetAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsTableMngCursor <Element, ElementOps>::
IGSortedSetAsTableMngCursor (ICursorImpl* impl)
: IGSortedSetAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedSetAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetAsTable <Element, ElementOps>::
IGSortedSetAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSortedSetAsTable <Element, ElementOps>::
IGSortedSetAsTable
  (IGSortedSetAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSortedSetAsTable <Element, ElementOps>&
IGSortedSetAsTable <Element, ElementOps>::
operator= (IGSortedSetAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetAsTable <Element, ElementOps>::
IGSortedSetAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSortedSetAsTable
 <Element, ElementOps>::Implementation&
IGSortedSetAsTable <Element, ElementOps>::
ImplOf (IGSortedSetAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedSetAsTable
// ---

// public members

template <class Element>
inline
ISortedSetAsTable <Element>::
ISortedSetAsTable (INumber numberOfElements)
: IGSortedSetAsTable
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISortedSetAsTable <Element>::
ISortedSetAsTable (PureImplementation& impl)
: IGSortedSetAsTable
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedSetAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSortedSetAsTable <Element, ElementOps>::
IVGSortedSetAsTable (INumber numberOfElements)
: IGSortedSetAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSortedSetAsTable <Element, ElementOps>::
IVGSortedSetAsTable (PureImplementation& impl)
: IGSortedSetAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVSortedSetAsTable
// ---

// public members

template <class Element>
inline
IVSortedSetAsTable <Element>::
IVSortedSetAsTable (INumber numberOfElements)
: IVGSortedSetAsTable
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSortedSetAsTable <Element>::
IVSortedSetAsTable (PureImplementation& impl)
: IVGSortedSetAsTable
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISSTAB_INL_
