// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 70 1.4.1.4 source/core/collect/istk.inl, collection, ioc.v400, 000919  
#ifndef _ISTK_INL_
#define _ISTK_INL_

// ---
// IGStackCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGStackCursor <Element, ElementOps>::
IGStackCursor
  (IGStack
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGStackCursor <Element, ElementOps>&
IGStackCursor <Element, ElementOps>::
operator=
  (IGStackCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGStackCursor <Element, ElementOps>::
IGStackCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGStackMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGStackMngCursor <Element, ElementOps>::
IGStackMngCursor
  (IGStack
    <Element, ElementOps> const& collection)
: IGStackCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGStackMngCursor <Element, ElementOps>::
IGStackMngCursor (ICursorImpl* impl)
: IGStackCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGStack
// ---

// public members

template <class Element, class ElementOps>
inline
IGStack <Element, ElementOps>::
IGStack (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGStack <Element, ElementOps>::
IGStack
  (IGStack
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGStack <Element, ElementOps>&
IGStack <Element, ElementOps>::
operator= (IGStack
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGStack <Element, ElementOps>::
IGStack (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGStack
 <Element, ElementOps>::Implementation&
IGStack <Element, ElementOps>::
ImplOf (IGStack
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IStack
// ---

// public members

template <class Element>
inline
IStack <Element>::
IStack (INumber numberOfElements)
: IGStack
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IStack <Element>::
IStack (PureImplementation& impl)
: IGStack
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGStack
// ---

// public members

template <class Element, class ElementOps>
inline
IVGStack <Element, ElementOps>::
IVGStack (INumber numberOfElements)
: IGStack
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGStack <Element, ElementOps>::
IVGStack (PureImplementation& impl)
: IGStack
    <Element, ElementOps> (impl)
{
}

// ---
// IVStack
// ---

// public members

template <class Element>
inline
IVStack <Element>::
IVStack (INumber numberOfElements)
: IVGStack
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVStack <Element>::
IVStack (PureImplementation& impl)
: IVGStack
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _ISTK_INL_
