// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 75 1.4.1.4 source/core/collect/istklst.inl, collection, ioc.v400, 000919  
#ifndef _ISTKLST_INL_
#define _ISTKLST_INL_

// ---
// IGStackAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGStackAsListCursor <Element, ElementOps>::
IGStackAsListCursor
  (IGStackAsList
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGStackAsListCursor <Element, ElementOps>&
IGStackAsListCursor <Element, ElementOps>::
operator=
  (IGStackAsListCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGStackAsListCursor <Element, ElementOps>::
IGStackAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGStackAsListMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGStackAsListMngCursor <Element, ElementOps>::
IGStackAsListMngCursor
  (IGStackAsList
    <Element, ElementOps> const& collection)
: IGStackAsListCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGStackAsListMngCursor <Element, ElementOps>::
IGStackAsListMngCursor (ICursorImpl* impl)
: IGStackAsListCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGStackAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IGStackAsList <Element, ElementOps>::
IGStackAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGStackAsList <Element, ElementOps>::
IGStackAsList
  (IGStackAsList
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGStackAsList <Element, ElementOps>&
IGStackAsList <Element, ElementOps>::
operator= (IGStackAsList
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGStackAsList <Element, ElementOps>::
IGStackAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGStackAsList
 <Element, ElementOps>::Implementation&
IGStackAsList <Element, ElementOps>::
ImplOf (IGStackAsList
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IStackAsList
// ---

// public members

template <class Element>
inline
IStackAsList <Element>::
IStackAsList (INumber numberOfElements)
: IGStackAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IStackAsList <Element>::
IStackAsList (PureImplementation& impl)
: IGStackAsList
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGStackAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IVGStackAsList <Element, ElementOps>::
IVGStackAsList (INumber numberOfElements)
: IGStackAsList
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGStackAsList <Element, ElementOps>::
IVGStackAsList (PureImplementation& impl)
: IGStackAsList
    <Element, ElementOps> (impl)
{
}

// ---
// IVStackAsList
// ---

// public members

template <class Element>
inline
IVStackAsList <Element>::
IVStackAsList (INumber numberOfElements)
: IVGStackAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVStackAsList <Element>::
IVStackAsList (PureImplementation& impl)
: IVGStackAsList
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _ISTKLST_INL_
