/*NOSHIP*/
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 60 1.6.1.4 source/core/filesys/filebuf.hpp, filesystem, ioc.v400, 000919 
// filebuf.h
// Created by laura on Fri, Feb 08, 1996 @ 11:58 AM.

#ifndef _FILEBUF_
#define _FILEBUF_

#include <iostream.h>
#include "ifilesys.hpp"

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IFSFileStream;

#pragma enum(4)
#pragma pack(push,4)

//-------------------------------------------------------------------------------
// Class:
//      IFSFileBuf
//
// Purpose:
//      streambuf subclass that works with the Pluto filesystem API
//
class IFSFileBuf : public streambuf {
public:
// Special functions...
                                                IFSFileBuf(const IFile &file, int myMode, int otherMode);
        virtual                         ~IFSFileBuf();

        void                            open(const IFile &file, int myMode, int otherMode);
        void                            close();

// Accessors...
        IFile                           file() const;
        int                                     mode() const;
        IFileSize                       position() const;

        IFileSize                       endOfFile() const;
        void                            setEndOfFile(IFileSize);

// Streambuf overrides...
        virtual int                     overflow(int c=EOF);
        virtual int                     underflow();
        virtual int                     pbackfail(int c);

        virtual int                     sync();

        virtual streampos       seekoff(streamoff offset, ios::seek_dir dir, int );
        virtual streampos       seekpos(streampos offset, int );

private:
// Disallowed functions...
                                                IFSFileBuf(const IFSFileBuf &copy);
        IFSFileBuf &            operator = (const IFSFileBuf &copy);

// Private functions...
        void                            syncout();
        void                            syncin();

// Private data...
        enum { kPutBackSize   = 4 };
        enum { kDefaultBuffer = 4096 };

        IFile                           fFile;
        int                                     fMode;

        size_t                          fBufferSize;
        char *                          fBuffer;
        const char *            fPbackEnd;
        IFileSize                       fFilePosition;

        IFSFileStream *         fByteStream;
};

// -----------------------------------------------------------------
// IFSFileBuf inlines...
//
inline IFile
IFSFileBuf::file() const
{
        return fFile;
}

inline int
IFSFileBuf::mode() const
{
        return fMode;
}

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _FILEBUF_
