// Revision: 61 1.11.1.4 source/core/filesys/filesys.hpp, filesystem, ioc.v400, 000919 
//NOSHIP
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// filesys.hpp
//
// Contains:
//    IFSFileSystem, the main internal implementation class for the
//    OpenClass file system interface.
//
//    Engineers porting the file system to a new platform must create a
//    subclass and override the pure virtual methods to provide implementation.
//    You must also provide an implementation for IFSFileSystem::initialize
//    which creates and returns an object of your subclass.
//
//    See the File System Interface ERS for more details.
//
#ifndef _FILESYS_
#define _FILESYS_

#include <ilanglvl.hpp>
#include <ifilesys.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IFSIterator;
class IFSAlias;

#pragma pack(push,4)
#pragma enum(4)

class IFSFileSystem
{
public:
    // Return a pointer to the one and only IFSFileSystem object...
    static IFSFileSystem* instance();

#if !IC_RTTI_ENABLED
    // TEMPORARY until we get rtti...
    virtual    IFileName        typeOf() const = 0;
#endif // IC_RTTI_ENABLED

// Entity lookup...
    // These three methods have default implementations.  Subclasses
    // can override them, but most shouldn't need to
    virtual IFileSystemEntity   lookUp(const IHostPathName& path,
                                        const ICompoundNameParser& parser,
                                        IFileSystemEntity::ECreateFlags    flags,
                                        IFileKind createKind ) const;

    virtual IFileSystemEntity   lookUp(const IHostPathName& path,
                                        IFileSystemEntity::ECreateFlags    flags,
                                        IFileKind createKind ) const;

    virtual IFileSystemEntity   lookUp( const IPathName& path,
                                        IFileSystemEntity::ECreateFlags    flags,
                                        IFileKind createKind ) const;

    // Support functions for entity lookup with automatic creation
    IFileSystemEntity           createPath( const IPathName& path,
                                            IFileKind createKind ) const;

    IFileSystemEntity           createPath( const IDirectory& parent,
                                            const IPathName& path,
                                            IFileKind createKind ) const;

// Iteration methods...
    virtual IFSIterator*        createVolumesIterator() const=0;
    virtual IFSIterator*        createRootDirectoryIterator() const=0;

// Path names...
    virtual const ICompoundNameParser&
                                getDefaultParser () const=0;
    virtual IDirectory          getCurrentDirectory() const=0;

// Temporary files...
    virtual IFile               createTempFile (const IFileName& name,
                                                const IFileSize size) const=0;

    virtual IDirectory          createTempDirectory (const IFileName& name) const=0;

// Streaming support
    virtual IFileSystemEntity   resolveAlias( const IFSAlias& ) const;

// File system exception handling utilities
    static const char *kMessageFile;

protected:
                                IFSFileSystem();
    virtual                     ~IFSFileSystem();

    // This function MUST be implemented by the platform-specific code.
    static    IFSFileSystem*    initialize();

// Subclass entity lookup.  Subclasses MUST override...
    virtual IFileSystemEntity   doLookUp( const IHostPathName& path ) const = 0;

private:
//    Disallowed functions...
                                IFSFileSystem(const IFSFileSystem& copy);
        IFSFileSystem&          operator=(const IFSFileSystem& copy);
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _FILESYS_
