// Revision: 64 1.8.1.4 source/core/filesys/fsfilrec.hpp, filesystem, ioc.v400, 000919 
//NOSHIP
// fsfilrec.hpp
//
// Contains:    IFSFileRecord
//                              Internal filesystem class that encapsulates file operations.
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#ifndef _FSFILREC_
#define _FSFILREC_

#include <ifilesys.hpp>
#include "fsrecord.hpp"

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IFSFileStream;

#pragma pack(push,4)

class IFSFileRecord : public IFSRecord {
public:
//      Special Functions
        virtual                                         ~IFSFileRecord ();

// Type Safety Functions
        // Subclasses may override kind to return their own kind.  If so, they
        // must override isA to check for own kind and call up if not equal.
        virtual IFileKind                       kind    () const;
        virtual bool                            isA (const IFileKind& kind) const;

// Access properties...
        virtual IFileSize                       size () const = 0;
        virtual void                            setSize (IFileSize newSize) = 0;

        virtual void                            setModificationTime(const ITimeStamp& modificationTime) = 0;

// Manipulation
        // copyTo just calls copySelf, since files don't have children.
        // Subclasses MUST override moveTo and copySelf.
        virtual IFileSystemEntity       copyTo(IDirectory &to, const IFileName &newName) const;

        virtual void                            doDeleteSelf () = 0;

// Streaming...
        virtual IFSFileStream*          createStream(int myMode, int otherMode) const = 0;

protected:
// Subclass interface...
                                                                IFSFileRecord ();

private:
// This class is ref-counted; copying and assignment aren't allowed...
                                                                IFSFileRecord (const IFSFileRecord& copy);
        IFSFileRecord&                          operator = (const IFSFileRecord& copy);
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

// This inline definition doesn't strictly belong here,  but it has
// to appear somewhere after the declaration of IFSFileRecord,
// and this is as good a place as any
inline IFSFileRecord*
IFile::getRecord() const {
        return (IFSFileRecord*) IFileSystemEntity::getRecord();
}

#endif // _FSFILREC_
