// Revision: 47 1.11.1.3 source/core/filesys/fsvolrec.cpp, filesystem, ioc.v400, 000919 
//NOSHIP
// fsvolrec.cpp
//
// Contains:    IFSVolumeRecord
//                              Internal filesystem class that encapsulates volume operations.
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#include <iprimtyp.hpp>
#include <ifilesys.hpp>
#include <iexcept.hpp>
#include <iexcfile.hpp>
#include <igenexc.hpp>  // For working ITHROW macro

#include "fsvolrec.hpp"
#include "filesys.hpp"
#include "ifilemsg.h"

#ifdef IC_PAGETUNE
    #define _FSVOLREC_CPP_
    #include <ipagetun.h>
#endif

//------------------------------------------------------------------------------
// Constructors, etc...
//
IFSVolumeRecord::IFSVolumeRecord()
{
}

IFSVolumeRecord::IFSVolumeRecord        (const IFSVolumeRecord& )
{
        // Disallowed.
}

IFSVolumeRecord::~IFSVolumeRecord()
{
}

IFSVolumeRecord&
IFSVolumeRecord::operator = (const IFSVolumeRecord& )
{
        // Disallowed.
        return (*this);
}


//------------------------------------------------------------------------------
// Type safety methods...
//
bool
IFSVolumeRecord::isA (const IFileKind& kind) const
{
        return kind == IVolume::kKind || IFSDirectoryRecord::isA(kind);
}

IFileKind
IFSVolumeRecord::kind () const
{
        return IVolume::kKind;
}


//------------------------------------------------------------------------------
// Volume manipulation...
//      By default, you're not allowed to move or delete a volume.
//
void
IFSVolumeRecord::deleteSelf(bool /*deleteChildren*/)
{
        IMessageText text(kCantDeleteVolume, IFSFileSystem::kMessageFile);
        IInvalidRequest e(text, kCantDeleteVolume, IException::recoverable);
        ITHROW(e);
}

void
IFSVolumeRecord::doDeleteSelf ()
{
        IMessageText text(kCantDeleteVolume, IFSFileSystem::kMessageFile);
        IInvalidRequest e(text, kCantDeleteVolume, IException::recoverable);
        ITHROW(e);
}

IFileSystemEntity IFSVolumeRecord::moveTo(IDirectory &to, const IFileName& newName)
{
        IMessageText text(kCantDeleteVolume, IFSFileSystem::kMessageFile);
        IInvalidRequest e(text, kCantDeleteVolume, IException::recoverable);
        ITHROW(e);
}


//------------------------------------------------------------------------------
// Access system-defined volume properties....
//
void
IFSVolumeRecord::setName(const IFileName& newName)
{
        // By default, you're not allowed to rename a volume
        ITHROWERROR(kCantRenameVolume,
                                IBaseErrorInfo::invalidRequest,
                                IException::recoverable,
                                IFSFileSystem::kMessageFile, 0);
}

IVolume::EState
IFSVolumeRecord::online () const
{
        try {
                // If the following operations work, assume it's online.
                freeSpace();
                remote();
                return IVolume::kTrue;
        }
        catch (const IVolumeOffline &) {
        }
        return IVolume::kFalse;
}

