/*NOSHIP*/
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

// Revision: 91 1.12.1.4 source/core/filesys/win32/winflsys.hpp, filesystem, ioc.v400, 000919 
// NIFileSystem.h
// Created by laura on Sat, Jan 19, 1996 @ 8:01 PM.

#ifndef _WINFLSYS_
#define _WINFLSYS_

#include <ilanglvl.hpp>
#include <ifilesys.hpp>
#include <ipathnam.hpp>

#ifdef __WINDOWS__      // ITimeStamp not supported on other platforms yet
#include <itmstamp.hpp>
#endif

#include "filesys.hpp"
#include "winpaths.hpp"

#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

class INTFileSystem : public IFSFileSystem {
public:
        // Standard methods
                                                                INTFileSystem();
        virtual                                         ~INTFileSystem();

// ------------------- IFSFileSystem overrides -------------------
public:
        virtual IFSIterator *           createRootDirectoryIterator() const;
        virtual IFSIterator *           createVolumesIterator() const;

        virtual IDirectory                      createTempDirectory (const IFileName& name) const;
        virtual IFile                           createTempFile (const IFileName& name,
                                                                                                const IFileSize initialSize) const;

        virtual IDirectory                      getCurrentDirectory() const;

        virtual const ICompoundNameParser&
                                                                getDefaultParser () const;

protected:
        virtual IFileSystemEntity       doLookUp( const IHostPathName& hostPath) const;



// -------------------  Win32 FileSystem calls -------------------
public:
        // Most of these are wrapper functions that Unicode and ANSI
        // transparently and throw exceptions for error conditions

        // Volume operations...
        static WCHAR *          copyLogicalDriveStrings();

        static void             getDiskFreeSpace(const INTPathName &path,
                                                                        DWORD &sectorsPerCluster, DWORD &nytesPerSector,
                                                                        DWORD &lpFreeClusters, DWORD &lpClusters);

        static UINT                     getDriveType( const INTPathName &path );

        static void                     getVolumeInformation( const INTPathName &path,
                                                                                        DWORD & lpVolumeSerialNumber,
                                                                                        DWORD & lpMaximumComponentLength,
                                                                                        DWORD & lpFileSystemFlags);

        // File operations
        static INTPathName      getFullPathName(const INTPathName &path);

        static DWORD            getFileAttributes( const INTPathName &path );

        static void                     getFileInformation( const INTPathName &path,
                                                                                BY_HANDLE_FILE_INFORMATION *info);

        static ITimeStamp       modificationTime( const INTPathName & );
        static void                     setModificationTime( const INTPathName &, const ITimeStamp &time );

        static void                     setFileTime( const INTPathName &path, const FILETIME *create,
                                                                        const FILETIME *access, const FILETIME *write);



        static void                     setCurrentDirectory(const INTPathName &path);

        static void                     removeDirectory(const INTPathName &path);
        static void                     deleteFile(const INTPathName &path);

        static IFile            createFile(const INTPathName& fullPath, IFileSize initialSize);

        static IDirectory       createDirectory(const INTPathName& fullPath);
        static IDirectory       createDirectoryEx( const INTPathName &oldPath,
                                                                                        const INTPathName &newPath,
                                                                                        LPSECURITY_ATTRIBUTES security );



        static void                     moveFile( const INTPathName &oldPath, const INTPathName &newPath );
        static void                     copyFile( const INTPathName &oldPath, const INTPathName &newPath,
                                                                        BOOL failIfExists );

        static HANDLE           createFile(const INTPathName &path, DWORD fdwAccess,
                                                                 DWORD fdwShareMode, LPSECURITY_ATTRIBUTES lpsa,
                                                                 DWORD fdwCreate, DWORD fdwAttrs, HANDLE hTemplate);

        static void                     setFilePointer( HANDLE handle, IFileSize newPointer );

        static HANDLE           findFirstFile( const INTPathName &path,
                                                                                WIN32_FIND_DATAW *data );

        static BOOL                     findNextFile( HANDLE, WIN32_FIND_DATAW *data );

// Exception reporting.
        static void                     throwLastError(DWORD error, const char *funcName);
        static void                     throwLastError(const char *funcName);

// Other assorted utilities...
        static IFileSystemEntity        lookUpFullPath( const INTPathName &path  );

        static FILETIME                         toFileTime( const ITimeStamp & dateTime) ;
        static ITimeStamp                       fromFileTime( const FILETIME & fileTime );


private:
// Disallowed...
                                                                INTFileSystem(const INTFileSystem &copy);
        INTFileSystem &                         operator = (const INTFileSystem &copy);

// Internal utilities...
        static IFileSystemEntity        handleLookup(const INTPathName& hostPath);

        static void                                     convertToUnicode(const WIN32_FIND_DATAA &ansi,
                                                                                                        WIN32_FIND_DATAW *data);

        static INTPathName                      getTempPath();
        static DWORD                            getTempSeed();

        static INTPathName                      getWinString(DWORD (* WINAPI ansiFunc)(DWORD, CHAR*),
                                                                                        DWORD (* WINAPI uniFunc) (DWORD, WCHAR*),
                                                                                        const char *funcName );

// Constants...
        static const char *                     kTmpPrefix;

        enum EWinVersion { kWinNT, kWin95, kWin32s };

// Data...
        IWin32PathNameParser    fDefaultParser;
        static EWinVersion              fWinVersion;
        static bool                             fUseUnicode;

#if !IC_RTTI_ENABLED
        // for rtti
        virtual IFileName               typeOf() const;
        static const IFileName  fType;
#endif // IC_RTTI_ENABLED
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _WINFLSYS_
