/*NOSHIP*/
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

// Revision: 90 1.7.1.4 source/core/filesys/win32/winvol.hpp, filesystem, ioc.v400, 000919 
// winvol.hpp
// Created by laura on Sat, Jan 26, 1996 @ 12:32 PM.

#ifndef _WINVOL_
#define _WINVOL_

#include "fsiterat.hpp"
#include "fsvolrec.hpp"
#include "winpaths.hpp"

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

class INTVolumeIterator : public IFSIterator {
public:
                                                                INTVolumeIterator();
        virtual                                         ~INTVolumeIterator();

        virtual IFileSystemEntity       first ();
        virtual IFileSystemEntity       next ();



private:
// Disallowed...
                                                INTVolumeIterator(const INTVolumeIterator &copy);
        INTVolumeIterator&      operator = (const INTVolumeIterator &copy);

// Private data....
        WCHAR *         fDriveStrings;
        WCHAR *         fCurrentDrive;
};

class INTVolumeRecord : public IFSVolumeRecord {
public:
// Standard methods....
                                                                INTVolumeRecord(const INTPathName &path);
        virtual                                         ~INTVolumeRecord();

// Comparison....
        virtual bool                            isEqual ( const IFSRecord *) const;
        virtual bool                            testConnection() const;

// Access  properties...
        // IFileSystemEntity properties...
        virtual IFileName                       name() const ;

        virtual IHostPathName           hostPath() const;

        virtual ITimeStamp                      modificationTime() const;


        // IVolume properties...
        virtual IFileSize                       freeSpace () const;
        virtual IFileSize                       totalSpace () const;
        virtual IVolume::EState         remote () const;
        virtual IVolume::EState         removable () const;

// Manipulation...
        virtual IFileSystemEntity       copySelf(IDirectory &to, const IFileName &name) const;

// Navigation...
        virtual IDirectory                      parent() const;
        virtual IDirectory                      topmostParent() const;
        virtual IVolume                         volume () const;

        void                                            setCurrent() const;

// Browsing...
        virtual IFileSystemEntity       lookUpByName(const IFileName& theName) const;
        virtual IFSIterator *           createIterator() const;

// Creation...
        virtual IDirectory                      createDirectory(const IFileName& name);
        virtual IFile                           createFile(const IFileName& name,
                                                                                IFileSize initialFileSize);

private:
// This class is ref-counted; copying and assignment aren't allowed...
                                                                INTVolumeRecord(const INTVolumeRecord &copy);
        INTVolumeRecord &                       operator = (const INTVolumeRecord &copy);

        INTPathName                                     fPath;
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _WINVOL_
