/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 63 1.9.1.3 source/core/streams/imstrmbf.cpp, oss, ioc.v400, 000919 
//
//
//
// imstrmbf.cpp
//
#include "imstrmbf.hpp"

#include <istrmexc.hpp>

#ifndef __MWERKS__
#include <string.h>
#include <stdlib.h>

static size_t   kBufferGrowthSize = 8192;
#endif

#ifdef IC_PAGETUNE
    #define _IMSTRMBF_CPP_
    #include <ipagetun.h>
#endif

IMemoryStreamBuffer::IMemoryStreamBuffer()
#ifndef __MWERKS__
        :       fBuffer(0)
        ,       fPointer(0)
        ,       fEndPointer(0)
#endif
{
}

IMemoryStreamBuffer::IMemoryStreamBuffer(const IMemoryStreamBuffer& other)
#ifndef __MWERKS__
        :       fBuffer(0)
        ,       fPointer(0)
        ,       fEndPointer(0)
#endif
{
        // not called...
}

IMemoryStreamBuffer::~IMemoryStreamBuffer()
{
#if defined(__MWERKS__)
        fBuffer.pubsync();
#else
        if (fBuffer != 0)
                free(fBuffer);
#endif
}

IDataStream*
IMemoryStreamBuffer::createWithStreamEncoding(int encoding)
{
        return IStreamBuffer::createWithStreamEncoding(encoding);
}

void
IMemoryStreamBuffer::writeBytes(const char* data, unsigned long length)
{
#if defined(__MWERKS__)
        if (fBuffer.sputn(data, (int)length) != length) {
                IEndOfStream exc;
                ITHROW(exc);
        }
#else
        if (fPointer + length > fEndPointer) {
                size_t currentOffset = fPointer - fBuffer;
                size_t currentSize = fEndPointer - fBuffer;
                size_t newSize = currentSize + length + kBufferGrowthSize + 1;
                newSize = newSize & (~(kBufferGrowthSize - 1));
                if (fBuffer == 0) {
                        fBuffer = (char*)malloc(newSize);
                } else {
                        fBuffer = (char*)realloc(fBuffer, newSize);
                }
                fPointer = fBuffer + currentOffset;
                fEndPointer = fBuffer + newSize;
        }
        memcpy(fPointer, data, (size_t)length);
        fPointer += length;
#endif
}

void
IMemoryStreamBuffer::readBytes(char* data, unsigned long length)
{
#if defined(__MWERKS__)
        if (fBuffer.sgetn(data, (int)length) != length) {
                IEndOfStream exc;
                ITHROW(exc);
        }
#else
        if (fPointer + length > fEndPointer) {
                IEndOfStream exc;
                ITHROW(exc);
        }
        memcpy(data, fPointer, (size_t)length);
        fPointer += length;
#endif
}

void
IMemoryStreamBuffer::flush()
{
#if defined(__MWERKS__)
        fBuffer.pubsync();
#endif
}

void
IMemoryStreamBuffer::reset()
{
#if defined(__MWERKS__)
        fBuffer.pubseekoff(0, ios::beg, 0);
#else
        fEndPointer = fPointer;
        fPointer = fBuffer;
#endif
}
