/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 45 1.10.1.3 source/core/streams/imstrmbl.cpp, oss, ioc.v400, 000919 
//
//
//
// imstrmbl.cpp
//
#include <imstrmbl.hpp>
#include <idatstrm.hpp>
#include <istrmmod.hpp>
#include <icntptr.hpp>
#include <imrefcnt.hpp>
#include <istrmexc.hpp>
#include "itypenam.hpp"

#ifdef IC_PAGETUNE
    #define _IMSTRMBL_CPP_
    #include <ipagetun.h>
#endif

IStreamableFactory::IStreamableFactory(IStreamModule* module)
        :       fTypeRep(this)
        ,       fModule(module)
        ,       fTypeName(0)
{
        module->registerType(&fTypeRep);
}

IStreamableFactory::IStreamableFactory()
        :       fTypeRep(0)
        ,       fModule(0)
        ,       fTypeName(0)
{
}

IStreamableFactory::IStreamableFactory(const IStreamableFactory& other)
        :       fTypeRep(0)
        ,       fModule(0)
        ,       fTypeName(0)
{
}

IStreamableFactory::~IStreamableFactory()
{
        // should never be called...
        delete fTypeName;
}

IStreamableFactory&
IStreamableFactory::operator=(const IStreamableFactory& other)
{
        return *this;
}

const IStreamableFactory&
IStreamableFactory::findFactory(const ITypeRepresentation& typeRepresentation)
{
        return typeRepresentation.streamableFactory();
}

const ITypeName&
IStreamableFactory::typeName()
{
        if (fTypeName == 0)
                fTypeName = new ITypeName(fModule->name(), name());
        return *fTypeName;
}

//
// IStreamOutFrame
//
IStreamOutFrame::IStreamOutFrame(IDataStream& aStream)
        :       fStream(&aStream)
{
        fStream->streamIO()->writeBeginFrame();
}

IStreamOutFrame::IStreamOutFrame()
        :       fStream(0)
{
}

IStreamOutFrame::~IStreamOutFrame()
{
        fStream->streamIO()->writeEndFrame();
}

//
// IStreamInFrame
//
IStreamInFrame::IStreamInFrame(IDataStream& aStream)
        :       fStream(&aStream)
{
        fStream->streamIO()->readBeginFrame();
}

IStreamInFrame::IStreamInFrame()
        :       fStream(0)
{
}

IStreamInFrame::~IStreamInFrame()
{
        fStream->streamIO()->readEndFrame();
}

bool
IStreamInFrame::atEnd()
{
        return fStream->streamIO()->atEndOfFrame();
}

//
// IMStreamable
//
IMStreamable::IMStreamable()
{
}

IMStreamable::IMStreamable(const IMStreamable& other)
{
}

IMStreamable::~IMStreamable()
{
}

void
IMStreamable::externalizeToStream(IDataStream& aStream) const
{
        writeToStream(aStream);
}

void
IMStreamable::internalizeFromStream(IDataStream& aStream)
{
        readFromStream(aStream);
}

//
// support for streamable objects.
//
void
IStreamableGlobalTemplateHelper::writeUnaliasedObject(IStreamIO& streamIO, const IMStreamable* source)
{
        streamIO.writeObject(source, 0);
}

void
IStreamableGlobalTemplateHelper::writeUnaliasedObject(IStreamIO& streamIO, const IMStreamable* source, const ITypeRepresentation& typeRep)
{
        streamIO.writeObject(source, &typeRep);
}

void
IStreamableGlobalTemplateHelper::writeAliasedObject(IStreamIO& streamIO, const IMStreamable* source, const IMRefCounted* refCounted)
{
        streamIO.writeAliasedObject(source, refCounted, 0);
}

void
IStreamableGlobalTemplateHelper::writeAliasedObject(IStreamIO& streamIO, const IMStreamable* source, const IMRefCounted* refCounted, const ITypeRepresentation& typeRep)
{
        streamIO.writeAliasedObject(source, refCounted, &typeRep);
}

IMStreamable*
IStreamableGlobalTemplateHelper::readUnaliasedObject(IStreamIO& streamIO)
{
        return streamIO.readObject(0, 0);
}

IMStreamable*
IStreamableGlobalTemplateHelper::readUnaliasedObject(IStreamIO& streamIO, const ITypeRepresentation& typeRep)
{
        return streamIO.readObject(0, &typeRep);
}

IMStreamable*
IStreamableGlobalTemplateHelper::readUnaliasedObject(IStreamIO& streamIO, const IStreamFinder& finder)
{
        return streamIO.readObject(&finder, 0);
}

IMStreamable*
IStreamableGlobalTemplateHelper::readUnaliasedObject(IStreamIO& streamIO, const IStreamFinder& finder, const ITypeRepresentation& typeRep)
{
        return streamIO.readObject(&finder, &typeRep);
}

IMStreamable*
IStreamableGlobalTemplateHelper::readAliasedObject(IStreamIO& streamIO)
{
        IMStreamable* result;
        IMRefCounted* refCounted;
        streamIO.readAliasedObject(result, refCounted, 0, 0);
        return result;
}

IMStreamable*
IStreamableGlobalTemplateHelper::readAliasedObject(IStreamIO& streamIO, const ITypeRepresentation& typeRep)
{
        IMStreamable* result;
        IMRefCounted* refCounted;
        streamIO.readAliasedObject(result, refCounted, 0, &typeRep);
        return result;
}

IMStreamable*
IStreamableGlobalTemplateHelper::readAliasedObject(IStreamIO& streamIO, const IStreamFinder& finder)
{
        IMStreamable* result;
        IMRefCounted* refCounted;
        streamIO.readAliasedObject(result, refCounted, &finder, 0);
        return result;
}

IMStreamable*
IStreamableGlobalTemplateHelper::readAliasedObject(IStreamIO& streamIO, const IStreamFinder& finder, const ITypeRepresentation& typeRep)
{
        IMStreamable* result;
        IMRefCounted* refCounted;
        streamIO.readAliasedObject(result, refCounted, &finder, &typeRep);
        return result;
}

void
IStreamableGlobalTemplateHelper::writeAliasedObject(IStreamIO& streamIO, const IMStreamable* source, const IMRefCounted* refCounted, bool isWrapped)
{
        char wrapped = (char)isWrapped;
        streamIO.writeByte(wrapped);
        if (isWrapped) {
                const IVoidMasterPointer* indirect = (const IVoidMasterPointer*)refCounted;
                if (streamIO.writeAliasedObject(indirect, refCounted, &IVoidMasterPointer::staticTypeRepresentation()))
                        streamIO.writeObject(source, 0);
        } else {
                streamIO.writeAliasedObject(source, refCounted, 0);
        }
}

IMStreamable*
IStreamableGlobalTemplateHelper::readAliasedObject(IStreamIO& streamIO, IVoidMasterPointer*& indirect, bool isWrapped)
{
        IMStreamable* result = 0;
        IMRefCounted* refCounted;
        char wasWrapped;
        wasWrapped = streamIO.readByte();
        if (wasWrapped != isWrapped) {
                IInvalidDataOnStream exc;
                ITHROW(exc);
        }
        if (isWrapped) {
                IMStreamable* indirectStreamable;
                IMRefCounted* indirectRefCounted;
                if (streamIO.readAliasedObject(indirectStreamable, indirectRefCounted, 0, &IVoidMasterPointer::staticTypeRepresentation()))
                        result = streamIO.readObject(0, 0);
                indirect = (IVoidMasterPointer*)indirectRefCounted;
        } else {
                streamIO.readAliasedObject(result, refCounted, 0, 0);
                indirect = 0;
        }
        return result;
}

IMStreamable*
IStreamableGlobalTemplateHelper::createCopy(const IMStreamable& source)
{
        return source.typeRepresentation().streamableFactory().createCopy(source);
}
