/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 75 1.13.1.4 source/core/streams/istrmctx.hpp, oss, ioc.v400, 000919 
//
//
//
// istrmctx.hpp
//
#ifndef _ISTRMCTX_
#define _ISTRMCTX_

#include <iprimtyp.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

//
// external class declarations.
//
class IMStreamable;
class IMRefCounted;
class IStreamFinder;
class ITypeRepresentation;

#pragma enum(4)
#pragma pack(push,4)

//
// IContextContainer
//
class IContextContainer
{
protected:
        IContextContainer();
        IContextContainer(const IContextContainer& other);
        virtual ~IContextContainer();

        virtual IMStreamable*   readAliasedObject(IMRefCounted*& refCounted, const IStreamFinder* finder, const ITypeRepresentation* typeRep) = 0;
        virtual const ITypeRepresentation*      readObjectType() = 0;
        virtual void    printAll() = 0;

private:
friend class IStreamContext;
        long    objectCount();
        void    setObjectCount(long objectCount);
        long    typeCount();
        void    setTypeCount(long typeCount);
        IContextContainer*      next();
        void    setNext(IContextContainer* nextContainer);

        long    fObjectCount;
        long    fTypeCount;
        IContextContainer*      fNext;
};

//
// IStreamContext
//
class IStreamContext
{
public:
        IStreamContext();
        virtual ~IStreamContext();

        long    addObject(const IMStreamable* object, const IMRefCounted* refCounted, bool& typeOnStream, bool& newEntry);
        long    addObjectType(const ITypeRepresentation* objectType, bool& newEntry);
        IMStreamable*   findObject(IMRefCounted*& refCounted, long objectNumber, const IStreamFinder* finder, const ITypeRepresentation* typeRep);
        const ITypeRepresentation*      findObjectType(long objectTypeNumber);
        IMStreamable*   nextObject(IMRefCounted*& refCounted);

        void    growObjects();
        void    growTypes();

        bool    establish();
        void    clear();
        void    reset();

        void    addObjectContainer(IContextContainer* container, bool addType);

protected:
        IStreamContext(const IStreamContext& other);
        IStreamContext& operator=(const IStreamContext& other);

private:
        typedef enum { kFastObjectsArraySize = 4 };
        typedef enum { kFastTypesArraySize = 4 };

        bool    fActive;
        bool    fUsingContainer;
        unsigned char   fFastObjectTypeOnStream[kFastObjectsArraySize];
        IMStreamable*   fFastObjects[kFastObjectsArraySize];
        IMRefCounted*   fFastRefCounted[kFastObjectsArraySize];
        const ITypeRepresentation*      fFastTypes[kFastTypesArraySize];
        unsigned char*  fObjectTypeOnStream;
        IMStreamable**  fObjects;
        IMRefCounted**  fRefCounted;
        const ITypeRepresentation**     fTypes;
        IContextContainer*      fContainers;
        IContextContainer*      fLastContainer;
        unsigned long   fObjectCount;
        unsigned long   fTypeCount;
        unsigned long   fMaxObjectCount;
        unsigned long   fMaxTypeCount;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTRMCTX_
