// ----------------------------------------------------------------------------
//  FILE: istrmfnd.cpp
//
//  COPYRIGHT:                                                                            *
//   IBM Open Class Library                                                              *
//   (C) Copyright International Business Machines Corporation,  1997                    *
//   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
//   US Government Users Restricted Rights - Use, duplication, or disclosure             *
//   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
//                                                                                       *
// Revision: 78 1.9.1.3 source/core/streams/istrmfnd.cpp, oss, ioc.v400, 000919 
// ----------------------------------------------------------------------------

#include <iprimlck.hpp>
#include <istrmfnd.hpp>
#include <imstrmbl.hpp>
#include <istrmexc.hpp>
#include "strmexc.h"

#ifdef IC_PAGETUNE
    #define _ISTRMFND_CPP_
    #include <ipagetun.h>
#endif


// ----------------------------------------------------------------------------
// IDefaultStreamFinder
// ----------------------------------------------------------------------------
class IDefaultStreamFinder : public IStreamFinder
{
public:
        IDefaultStreamFinder();
        IDefaultStreamFinder(const IDefaultStreamFinder& other);
        virtual ~IDefaultStreamFinder();
};

IDefaultStreamFinder::IDefaultStreamFinder(const IDefaultStreamFinder& other)
        :       IStreamFinder(other)
{
}

IDefaultStreamFinder::IDefaultStreamFinder()
        :       IStreamFinder()
{
}

IDefaultStreamFinder::~IDefaultStreamFinder()
{
}



// ----------------------------------------------------------------------------
// IStreamFinder
// ----------------------------------------------------------------------------
IStreamFinder::IStreamFinder()
{
}

IStreamFinder::IStreamFinder(const IStreamFinder& other)
{
}

IStreamFinder::~IStreamFinder()
{
}

const IStreamFinder& IStreamFinder::defaultFinder()
{
    // This static member is lazily evaluated
        static IDefaultStreamFinder* gDefaultFinder = 0;

    if (!gDefaultFinder)
    {
        IPrimalLock lockInit;

        if (!gDefaultFinder)
            gDefaultFinder = new IDefaultStreamFinder;
    }
        return *gDefaultFinder;
}

IMStreamable*
IStreamFinder::createUninitialized(const ITypeName& typeName, IMRefCounted*& refCounted) const
{
        return findFactory(typeName).createUninitialized(refCounted);
}

const ITypeRepresentation&
IStreamFinder::findTypeRepresentation(const ITypeName& typeName) const
{
        return findFactory(typeName).typeRepresentation();
}

const IStreamableFactory&
IStreamFinder::findFactory(const ITypeName& typeName) const
{
        const ITypeRepresentation* typeRepresentation = ITypeRepresentation::findTypeWithName(typeName);
        if (typeRepresentation == 0) {
                IUnknownTypeOnStream exc;
                ITHROW(exc);
        }
        return IStreamableFactory::findFactory(*typeRepresentation);
}

const ITypeRepresentation*
IStreamFinder::findTypeWithName(const ITypeName& typeName) const
{
        return ITypeRepresentation::findTypeWithName(typeName);
}

