/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 95 1.4.1.3 source/core/streams/itypenam.cpp, oss, ioc.v400, 000919 
//
//
//
// itypenam.cpp
//
#include "itypenam.hpp"
#include <ityperep.hpp>

#include <string.h>

#ifdef IC_PAGETUNE
    #define _ITYPENAM_CPP_
    #include <ipagetun.h>
#endif

//
// ITypeName
//
ITypeName::ITypeName(const char* formattedName)
        :       fNComponents(0)
        ,       fComponents(0)
        ,       fFormattedName(0)
{
        size_t length = 1;
        const char* temp = formattedName + length;
        while (*temp != 0) {
                if (*temp++ == '/')
                        fNComponents++;
                length++;
        }
        length++;
        fFormattedName = new char[length];
        memcpy(fFormattedName, formattedName, length);
        fComponents = new char*[fNComponents];
        temp = fFormattedName + 1;
        const char* name = temp;
        int comp = 0;
        while (*temp != 0) {
                if (*temp++ == '/') {
                        char* cp = new char[temp - name];
                        memcpy(cp, name, temp - name);
                        cp[temp - name - 1] = 0;
                        fComponents[comp++] = cp;
                        name = temp;
                }
        }
}

ITypeName::ITypeName(const char* moduleName, const char* className)
        :       fNComponents(0)
        ,       fComponents(0)
        ,       fFormattedName(0)
{
        fNComponents = 2;
        fComponents = new char*[fNComponents];
        size_t moduleNameLen = strlen(moduleName) + 1;
        size_t classNameLen = strlen(className) + 1;
        fComponents[0] = new char[moduleNameLen];
        fComponents[1] = new char[classNameLen];
        fFormattedName = new char[1 + moduleNameLen + classNameLen + 1];
        memcpy(fComponents[0], moduleName, moduleNameLen);
        memcpy(fComponents[1], className, classNameLen);
        char* temp = fFormattedName;
        *temp++ = '/';
        memcpy(temp, moduleName, moduleNameLen - 1);
        temp += moduleNameLen - 1;
        *temp++ = '/';
        memcpy(temp, className, classNameLen - 1);
        temp += classNameLen - 1;
        *temp++ = '/';
        *temp = 0;
}

ITypeName::ITypeName()
        :       fNComponents(0)
        ,       fComponents(0)
        ,       fFormattedName(0)
{
}

ITypeName::ITypeName(const ITypeName& other)
        :       fNComponents(0)
        ,       fComponents(0)
        ,       fFormattedName(0)
{
        fNComponents = other.fNComponents;
        fComponents = new char*[fNComponents];
        size_t length = strlen(other.fFormattedName) + 1;
        fFormattedName = new char[length];
        memcpy(fFormattedName, other.fFormattedName, length);
        for (int i = 0; i < fNComponents; i++) {
                length = strlen(other.fComponents[i]) + 1;
                fComponents[i] = new char[length];
                memcpy(fComponents[i], other.fComponents[i], length);
        }
}

ITypeName::~ITypeName()
{
        for (int i = 0; i < fNComponents; i++)
                delete [] fComponents[i];
        delete [] fComponents;
        delete [] fFormattedName;
}

ITypeName&
ITypeName::operator=(const ITypeName& other)
{
        if (this != &other) {
                delete [] fFormattedName;
                fFormattedName = 0;
                for (int j = 0; j < fNComponents; j++) {
                        delete [] fComponents[j];
                        fComponents[j] = 0;
                }
                delete [] fComponents;
                fComponents = 0;
                fNComponents = 0;

                size_t length = strlen(other.fFormattedName) + 1;
                fFormattedName = new char[length];
                memcpy(fFormattedName, other.fFormattedName, length);
                fNComponents = other.fNComponents;
                fComponents = new char*[fNComponents];
                for (int i = 0; i < fNComponents; i++) {
                        length = strlen(other.fComponents[i]) + 1;
                        fComponents[i] = new char[length];
                        memcpy(fComponents[i], other.fComponents[i], length);
                }
        }
        return *this;
}

const char*
ITypeName::formattedName() const
{
        return fFormattedName;
}

const char*
ITypeName::componentAt(unsigned long index) const
{
        if (index == 0 || index - 1 > fNComponents)
                return 0;
        else
                return fComponents[index - 1];
}

const ITypeName&
ITypeName::nameOfTypeRepresentation(const ITypeRepresentation& typeRep)
{
        return typeRep.typeName();
}
