/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 01 1.14.1.3 source/core/testfw/icmptest.c, testfw, ioc.v400, 000919 

#ifndef _ICMPTEST_C_
#define _ICMPTEST_C_

#include <ilanglvl.hpp>
#if IC_RTTI_ENABLED
#include <typeinfo.h>
#endif

#include "icmptest.hpp"
//#include "iheapchk.hpp"

//----------------------------------------------------------------------------
// Implementation for template follows.
//----------------------------------------------------------------------------

template <class AType>
IComparisonTestOf<AType>::IComparisonTestOf(AType* target, AType* unequalTarget)
: fTarget(target), fUnequalTarget(unequalTarget)
{
}

template <class AType>
IComparisonTestOf<AType>::~IComparisonTestOf()
{
        delete fTarget;
        delete fUnequalTarget;
}

template <class AType>
IComparisonTestOf<AType>::IComparisonTestOf(const IComparisonTestOf<AType>& h)
: ITest(h), fTarget(NULL), fUnequalTarget(NULL)
{
}

template <class AType>
IComparisonTestOf<AType>& IComparisonTestOf<AType>::operator=(const IComparisonTestOf<AType>& h)
{
    if (this != &h)
        {
                ITest::operator=(h);
                delete fTarget;
                delete fUnequalTarget;
                fTarget = NULL;
                fUnequalTarget = NULL;
        }
    return *this;
}

template <class AType>
void IComparisonTestOf<AType>::copyInfo()
{
        const char* className = targetClassNameAsString();
        addInfo(kDescriptionKey, IString("IComparisonTestOf tests the canonical methods of the target class."));
        addInfo(kTargetClassKey, IString(className ? className : "Unknown"));
}

template <class AType>
void IComparisonTestOf<AType>::test()
{
        ITieredTextBuffer out = outputTextStream();
        const char* targetClassName = targetClassNameAsString();
        out.pushTier(ITieredTextBuffer::kDetail);
        out << "IComparisonTestOf " << (targetClassName ? targetClassName:"Unknown")
        << " {\n";
        out.setRelativeIndent(1);

//      IHeapChecker leaks;             // Check for memory leaks;

        if (fTarget == NULL) {
                out.pushTier(ITieredTextBuffer::kHeadline);
                out     << "IComparisonTestOf - target object not specified.\n";
                out.popTier();
                setSuccess(false);

        } else {

                out << "same object\n";
                compare(fTarget, fTarget, true);

                out << "copy-constructed stack object\n";
                AType stackObject(*fTarget);
        compare(fTarget, &stackObject, true);

                out << "operator= assigned stack object\n";
                stackObject = *fTarget;
                compare(fTarget, &stackObject, true);

                out << "operator= self assigned stack object\n";
                stackObject = stackObject;
                compare(fTarget, &stackObject, true);

                AType* heapObject = new AType(*fTarget);
                out << "copy-constructed heap object\n";
                compare(heapObject, fTarget, true);

                *heapObject = *fTarget;
                out << "operator= assigned heap object\n";
                compare(heapObject, fTarget, true);

                *heapObject = *heapObject;
                out << "operator= self-assigned heap object\n";
                compare(heapObject, fTarget, true);

                if (fUnequalTarget) {
                        out << "separately constructed unequal object\n";
                        compare(fTarget, fUnequalTarget, false);
                }

                delete heapObject;
        }

//      setStickySuccess( leaks.check(out) );

        if (isSuccessUndecided()) setSuccess(true);
        out.setRelativeIndent(-1);
        out << "}\n";
        out.popTier();
}

template <class AType>
void IComparisonTestOf<AType>::compare(AType* obj, AType* target, bool same)
{
        ITieredTextBuffer out = outputTextStream();
        out.pushTier(ITieredTextBuffer::kDetail);
        out.setRelativeIndent(1);

        if (same){
                if (*obj == *target){
                        setSuccess(true);
                }
                else{
                        out << "operator== returned FALSE for equal objects" << "\n";
                        setSuccess(false);
                }
        }
        else{
                if (*obj == *target){
                        out << "operator== returned TRUE for unequal objects" <<"\n";
                        setSuccess(false);
                }
                else{
                        setSuccess(true);
                }
        }
        out.setRelativeIndent(-1);
        out.popTier();
}


template <class AType>
const char* IComparisonTestOf<AType>::targetClassNameAsString() const
{
        const char *name = NULL;
#if IC_RTTI_ENABLED && __IBMCPP__ >= 360 // Juno bug
        const type_info& info = typeid(AType);
        if (&info) {
                name = info.name();
        }
#endif
        return name;
}

// definition of the global function
template<class AType>
IComparisonTestOf<AType>* createComparisonTest(AType* target, AType* unequalTarget)
{
        return new IComparisonTestOf<AType>(target, unequalTarget);
}

#endif  // _ICMPTEST_C_
