// Revision: 71 1.5.1.3 source/core/testfw/iexctest.cpp, testfw, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// iexctest.cpp
//
// Laura Werner
//
// Contains the implementation of IExceptionTestBase
//

#include <iexctest.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
    #define _IEXCTEST_CPP_
    #include <ipagetun.h>
#endif

void            IExceptionTestBase::setup()
{
        ITest::setup ();

        // Do "first time" things that might allocate memory here in setup
        // so that they won't screw up my memory leak checking

        // Throw and catch one exception
        try {
                IException e("Hello there", 0, IException::recoverable);
                ITHROW(e);
        } catch (const IException& e) {
                ITieredTextBuffer out;
                verifyMessage(out, e);
        }
}

bool
IExceptionTestBase::verifyMessage(
        ITieredTextBuffer&      out,
        const IException&       e,
        const char *            expected)
{
        bool success = true;
        unsigned short oldIndent = out.indent();
        out.setIndent((unsigned short)(oldIndent + 1));

        // This constant will contain the "couldn't find the message" message.
        static const IString kNoMessage = IMessageText(0, "bogusmsg").text();

        // Fetch the text out of the exception in question.
        const IString eText = IString(e.text());

        if (expected)
        {
                if ( eText != IString(expected) ) {
                        out << "ERROR: wrong exception message\n"
                                << " expected: " << expected << "\n"
                                << " got:      " << e.text() << "\n";
                        success = false;
                }
        }
        // If we don't know exactly what string to expect,
        // we can at least verify that it's reasonable.
        else if ( eText.indexOf(kNoMessage) )
        {
                // Matches the default "couldn't find the message" message!
                out << "ERROR: " << kNoMessage << "\n";
                success = false;
        }
        else if ( *e.text() == 0 )
        {
                out << "ERROR: no exception message\n";
                success = false;
        }
        else if ( eText.length() < 5 ) {
                out << "ERROR: length < 5; probably a bug:\n";
                out.setIndent((unsigned short)(oldIndent + 2));
                out << e.text() << "\n";

                success = false;
        }
        else if ( eText.indexOf("(null)") ) {
                // If the message contains text like (null), replacement probably failed.
                out << "ERROR: text contains '(null)'; probably a bug:\n";
                out.setIndent((unsigned short)(oldIndent + 2));
                out << e.text() << "\n";

                success = false;
        }
        else if ( eText.indexOf("Drror") == 1 ) {
                // If the message starts with "Drror", you probably have a
                // reference-to-temporary bug
                out << "ERROR: text starts with 'Drror'; probably a ref-to-temp bug:\n";
                out.setIndent((unsigned short)(oldIndent + 2));
                out << e.text() << "\n";

                success = false;
        }

#if 0
        // In theory, we can also make sure that the exception has location info
        //
        // But right now, anyone who includes <iiexc.h> (an ICLCC header)
        // gets a definition of ITHROW that doesn't record the location info,
        // so testing this is pointless.
        //
        if ( e.locationCount() == 0 ) {
                out << "ERROR: exception has no location info\n";
                success = false;
        }
#endif

        out.setIndent(oldIndent);

        return success;
}

void
IExceptionTestBase::verifyMessage(
        const IException&       e,
        const char *            expected)
{
        setStickySuccess( verifyMessage(outputTextStream(), e, expected) );
}
