// NOSHIP
// Revision: 72 1.9.1.4 source/core/testfw/iexctest.hpp, testfw, ioc.v400, 000919 
#ifndef _IEXCTEST_
#define _IEXCTEST_
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// iexctest.hpp
//
// Contains:
//      IExceptionTestOf        - Templatized test for OpenClass exceptions.
//      IExceptionTestBase      - Has a handy exception message verification method
//                                                      that you can call from your own test classes.
//
// Laura Werner
//
//

#include <itest.hpp>
#include <iexcept.hpp>
#include <iheapchk.hpp>
#include <istring.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

//---------------------------------------------------------------------------------------
// IExceptionTestBase
//      Common code for type-specific exception testers
//
class IExceptionTestBase : public ITest {
public:
        // Public utility method for any test that wants to verify an
        // exception's message
        static bool     verifyMessage(ITieredTextBuffer& out,
                                                                const IException &e,
                                                                const char *expectText = 0);

protected:
                                IExceptionTestBase() {}
        void            setup();

        void            verifyMessage(const IException &e,
                                                                const char *expectText = 0);
};

//---------------------------------------------------------------------------------------
// IExceptionTestOf
//      Template for testing IOC's exception classes
//
template <class T>
class IExceptionTestOf : public IExceptionTestBase {
public:
                                        IExceptionTestOf();
protected:
        virtual void    test();
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

template <class T>
IExceptionTestOf<T>::IExceptionTestOf()
{
}

template <class T>
void
IExceptionTestOf<T>::test()
{
        setSuccess(true);

        ITieredTextBuffer& out = outputTextStream();
        unsigned short oldIndent = out.indent();
        out.setIndent((unsigned short)(oldIndent + 1));

        const char *kTestMessage = "test exception message ABCXYZ";

        out << "Testing exception class " << T().name() << "\n";

        IHeapChecker leaks;                                             // Check for memory leaks

        out << "throw/catch (default ctor)...\n";
        try {
                T exception;
                ITHROW(exception);
        }
        catch (const T& e) {
                verifyMessage(e);
        }

        out << "throw/catch (const char* ctor)...\n";
        try {
                T exception(kTestMessage);
                ITHROW(exception);
        }
        catch (const T& e) {
                verifyMessage(e, kTestMessage);
        }

        out << "throw/catch (copy ctor)...\n";
        try {
                T myException(kTestMessage);
                T myCopy(myException);
                ITHROW(myCopy);
        }
        catch (const T& e) {
                verifyMessage(e, kTestMessage);
        }

#ifndef _AIX
        setStickySuccess( leaks.check(out) );   // Check for memory leaks
#endif
        out.setIndent(oldIndent);
}

#endif // _IEXCTEST_
