// Revision: 03 1.6.1.3 source/core/testfw/issttest.cpp, testfw, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

#include <issttest.hpp>
#include <ismpclck.hpp>

#include <stdio.h>
#include <assert.h>

// Segment definitions
#ifdef IC_PAGETUNE
    #define _ISSTTEST_CPP_
    #include <ipagetun.h>
#endif

//-----------------------------------------------------------------------------
// IStartStopTimingTest: PUBLIC Special Methods
//-----------------------------------------------------------------------------

IStartStopTimingTest::~IStartStopTimingTest()
{       // Nothing to do here.
}

IStartStopTimingTest::IStartStopTimingTest(const IStartStopTimingTest& other):
        ITimingTest(other)
{
}

IStartStopTimingTest& IStartStopTimingTest::operator=(const IStartStopTimingTest& other)
{
        if (&other != this) ITimingTest::operator=(other);
        return *this;
}

//-----------------------------------------------------------------------------
// IStartStopTimingTest: PROTECTED Special member Functions
//-----------------------------------------------------------------------------

IStartStopTimingTest::IStartStopTimingTest(): ITimingTest()
{       // Nothing to do here.
}

IStartStopTimingTest::IStartStopTimingTest(unsigned long timingCount, unsigned
        long sampleCount): ITimingTest(timingCount, sampleCount)
{       // Nothing to do here.
}


//-----------------------------------------------------------------------------
// IStartStopTimingTest: Timing Framework Methods
//-----------------------------------------------------------------------------

void IStartStopTimingTest::emptyStartStopTimingTest(ITimingTestStopwatch& stopwatch)
/*
        emptyTimingTest is the routine which corresponds to timingTest, but is
        empty. Clients need NOT override this method. However, clients who wish to
        produce more accurate times should override this method and do everything
        that they do in timingTest but which they don't want to be timed.
    Specifically,any calls to ITimingTestStopwatch methods which are made in
    timingTest should also be made here.
*/
{
        // By default, we assume the client makes on pair of calls to start/stop.
        // If more than one pair of calls is made, clients should override this
        // method.
        stopwatch.start();
        stopwatch.stop();
}

//-----------------------------------------------------------------------------
// IStartStopTimingTest: PROTECTED Advanced Subclass Specialization Methods
//-----------------------------------------------------------------------------

double IStartStopTimingTest::oneSample()
{
        double elapsed;
        unsigned long iteration = timingCount();
        timingSetup();

        fStopwatch.fStartCount = 0;
        fStopwatch.fStopCount = 0;

        fStopwatch.reset();
        while (iteration-- > 0) startStopTimingTest(fStopwatch);
        fStopwatch.elapsedTime(elapsed);

        // We compute this after each sample. This is not ideal, but it is the
        // best solution without altering the ITimingTest framework extensively.
        fStopwatchOverhead = fStopwatch.fStartCount + fStopwatch.fStopCount;

        if (fStopwatch.fIsRunning)
                outputTextStream() << "Warning: stopwatch not stopped after sample\n";
        if (fStopwatch.fStartCount != fStopwatch.fStopCount)
                outputTextStream() << "Warning: stopwatch start/stop mismatch\n";

        timingCleanup();
        return elapsed / timingCount();
}


void IStartStopTimingTest::doUntimedTest()
{
        ITimingTestStopwatch dummyStopwatch;

        timingSetup();
        startStopTimingTest(dummyStopwatch);
        timingCleanup();
}



double IStartStopTimingTest::oneEmptySample()
{
        double elapsed;
        unsigned long iteration = timingCount();

        fStopwatch.reset();
        while (iteration-- > 0) emptyStartStopTimingTest(fStopwatch);
        fStopwatch.elapsedTime(elapsed);

        return elapsed / timingCount();
}

double IStartStopTimingTest::samplingErrorMicroseconds() const
{
        // For start-stop tests, the sampling error per sample is (k alpha), where
        // k is the number of calls to start or stop per call to
        // startStopTimingTest.

        // We divide by timingCount because the value in
        // fStopwatchOverhead is the accumulated count for an entire sample.

        return fStopwatchOverhead * clockNowOverhead() / timingCount();
}

//-----------------------------------------------------------------------------
// IStartStopTimingTest: PRIVATE Methods
//-----------------------------------------------------------------------------

void IStartStopTimingTest::timingTest()
{
         assert(false) ;  //IStartStopTimingTest::test() should never be called.
}

//-----------------------------------------------------------------------------
// ITimingTestStopwatch
//-----------------------------------------------------------------------------
ITimingTestStopwatch::ITimingTestStopwatch() : fElapsedTime(0)
/*
        Make a new stopped, reset stopwatch.
*/
{
        fIsRunning = false;
        fClock = new ISimpleClock;
        reset();

}

ITimingTestStopwatch::~ITimingTestStopwatch()
{
        delete fClock;
}

ITimingTestStopwatch::ITimingTestStopwatch(const ITimingTestStopwatch& rhs)
{
        fClock = new ISimpleClock;
        copyDataMembers(rhs);
}


ITimingTestStopwatch& ITimingTestStopwatch::operator=(const ITimingTestStopwatch& rhs)
{
        if (&rhs != this)
        {
                copyDataMembers(rhs);
        }
        return (*this);
}

void ITimingTestStopwatch::reset()
/*
        Resetting a running stopwatch is okay; it is equivalent to stopping it,
        resetting it, and starting it again.
*/
{
        if (fIsRunning) {
                fClock->timeNow(fStartTime);
        }
        fElapsedTime = 0;
}

void ITimingTestStopwatch::start()
/*
        start starts a stopwatch without resetting it, so that times can be
        accumulated into the stopwatch. Use reset to force back to zero.
*/
{
        if (!fIsRunning) {
                fClock->timeNow(fStartTime);
                fIsRunning = true;
                ++fStartCount;
        }
}

void ITimingTestStopwatch::stop()
{
        if (fIsRunning) {
                fClock->timeNow(fScratchTime);
                fScratchTime -= fStartTime; // Avoid generating temporaries here
                fElapsedTime += fScratchTime;
                fIsRunning = false;
                ++fStopCount;
        }
}

void ITimingTestStopwatch::elapsedTime(double& result)
{
        if (fIsRunning) {
                fClock->timeNow(result);
                result -= fStartTime;
                result += fElapsedTime;
        }
        else result = fElapsedTime;
}


void ITimingTestStopwatch::copyDataMembers(const ITimingTestStopwatch& rhs)
{
        fElapsedTime = rhs.fElapsedTime;
        fStartTime = rhs.fStartTime;
        fScratchTime = rhs.fScratchTime;
        fIsRunning = rhs.fIsRunning;
        fStartCount = rhs.fStartCount;
        fStopCount = rhs.fStopCount;
}
