/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 08 1.16.1.3 source/core/testfw/istrmtst.c, testfw, ioc.v400, 000919 

#ifndef _ISTRMTST_C_
#define _ISTRMTST_C_

#include <istrmtst.hpp>

#include <ilanglvl.hpp>
#include <idatstrm.hpp>
#include <imstrmbl.hpp>

template <class AType>
IStreamTestOf<AType>::IStreamTestOf(AType* target)
:fTarget(target), fStream(IDataStream::createMemoryStream())
{
}

template <class AType>
IStreamTestOf<AType>::~IStreamTestOf()
{
        delete fStream;
        delete fTarget;
}

template <class AType>
IStreamTestOf<AType>::IStreamTestOf(const IStreamTestOf<AType>& source)
: ITest(source), fTarget(source.fTarget? new AType(*source.fTarget):NULL),
        fStream(source.fStream? IDataStream::createMemoryStream() : NULL)
{
}

template <class AType>
IStreamTestOf<AType>& IStreamTestOf<AType>::operator=(const IStreamTestOf<AType>& other)
{
        if (this != &other)
        {
           ITest::operator=(other);
           delete fStream;
           delete fTarget;
           fStream = other.fStream? IDataStream::createMemoryStream() : NULL;
           fTarget = other.fTarget? new AType(*other.fTarget):NULL;
   }
        return *this;
}

template <class AType>
void IStreamTestOf<AType>::copyInfo()
{
        const char* className = targetClassNameAsString();

        addInfo(kDescriptionKey, IString("IStreamTest tests the streaming of the target class."));
        addInfo(kTargetClassKey, IString(className ? className : "Unknown"));
}


template <class AType>
void IStreamTestOf<AType>::test()
{
#if 0   // The new streams don't support this - laura 5/1/96
        unsigned long outLength, inLength;
#endif
        ITieredTextBuffer out = outputTextStream();
        const char* targetClassName = targetClassNameAsString();

        out.pushTier(ITieredTextBuffer::kDetail);
        out << "IStreamTestOf "  << (targetClassName ? targetClassName:"Unknown")
                        << "{\n";
        out.setRelativeIndent(1);

        if (fTarget == NULL) {
                out.pushTier(ITieredTextBuffer::kHeadline);
                out << "IStreamTest - target not specified\n";
                out.popTier();
                setSuccess(false);
        }
        else {
                fStream->reset();
                *fTarget >>= *fStream;
#if 0
        outLength = fStream->position();
#endif
                AType newObject(*fTarget);

                fStream->reset();
                newObject <<= *fStream;
#if 0
                inLength = fStream->position();
        if (outLength != inLength){
            out.pushTier(ITieredTextBuffer::kHeadline);
                        out << "Streaming failure: operator>>= wrote "
                                << outLength << " bytes, operator<<= read " << inLength << " bytes.\n";
                out.popTier();
                        setSuccess(false);
                }
#endif
                if (!(*fTarget == newObject)){
                        out.pushTier(ITieredTextBuffer::kHeadline);
                        out << "Streamed object is not equal to target\n";
            out.popTier();
                        setSuccess(false);
                }
                if (isSuccessUndecided()) setSuccess(true);
                out.setRelativeIndent(-1) ;
                out <<  "}\n";
                out.popTier();
        }
}


template <class AType>
const char* IStreamTestOf<AType>::targetClassNameAsString() const
{
        const char *name = NULL;
#if IC_RTTI_ENABLED && __IBMCPP__ >= 360 // Juno bug
        const type_info& info = typeid(AType);
        if (&info) {
                name = info.name();
        }
#endif
        return name;
}

// definition of global function
template<class AType>
IStreamTestOf<AType> *createStreamTest(AType* target)
{
        return new IStreamTestOf<AType>(target);
}

#endif // _ISTRMTST_C_
