/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 59 1.8.1.4 source/core/text/intl/compare/coliter.cpp, text, ioc.v400, 000919 
//===============================================================================
//
// File icolitr.inl : Collation Iterator inline functions
//
//
//
// Created by: Helena Shih
//
// Modification History:
//
//      Date         Name          Description
//
//===============================================================================
//

#ifndef IC_ILOCALE
#include <ilocale.hpp>
#endif

#ifndef IC_ICLITIMP
#include "iclitimp.hpp"
#endif

#ifdef IC_PAGETUNE
        #define _COLITER_CPP
        #include <ipagetun.h>
#endif

// ------------------------------------------------------------------------------------------
// IHostCollationIterator
// ------------------------------------------------------------------------------------------


IHostCollationIterator::IHostCollationIterator()
{
}


IHostCollationIterator::IHostCollationIterator(const IHostCollationIterator& that)
        :fLocales(that.fLocales)
{
}


IHostCollationIterator::~IHostCollationIterator()
{
}

// assignment
 IHostCollationIterator&
IHostCollationIterator::operator=(const IHostCollationIterator& that)
{
        if (this != &that) {
                fLocales = that.fLocales;
        }
        return *this;
}
// comparison
 bool
IHostCollationIterator::operator==(const IHostCollationIterator& that) const
{
        if (this == &that) return true;
        return (fLocales == that.fLocales);
}

 bool
IHostCollationIterator::operator!=(const IHostCollationIterator& that) const
{
        return !(*this == that);
}

ILocaleKeyIterator&
IHostCollationIterator::hostLocales()
{
        return fLocales;
}

// cursor actions
 void
IHostCollationIterator::reset()
{
        fLocales.reset();
}

IHostCollationIterator::operator bool() const
{
        return fLocales.operator bool();
}

 ICollation*
IHostCollationIterator::create(ICollation::ECollationStrength strength) const
{
        return (ICollation::createCollation(*fLocales, strength));
}

 IText
IHostCollationIterator::localePOSIXID() const
{
        return ((*fLocales).posixID());
}

 IHostCollationIterator&
IHostCollationIterator::operator++()     // prefix
{
        fLocales++;
        return *this;
}

 IHostCollationIterator
IHostCollationIterator::operator++(int) // postfix
{
        IHostCollationIterator tmp(*this);
        fLocales++;
        return tmp;
}
// displayable key name
 IText
IHostCollationIterator::displayName(const ILocaleKey& desiredLanguage,
                                                                        ILocaleKey::ELookupStrategy strategy) const
{
        return ((*fLocales).displayName(desiredLanguage, strategy));
}


// ------------------------------------------------------------------------------------------
// IPortableCollationIterator
// ------------------------------------------------------------------------------------------

IPortableCollationIterator::IPortableCollationIterator()
{
}

IPortableCollationIterator::IPortableCollationIterator(const IPortableCollationIterator& that)
{
}

IPortableCollationIterator::~IPortableCollationIterator()
{
}

// assignment
IPortableCollationIterator&
IPortableCollationIterator:: operator=(const IPortableCollationIterator& that)
{
        return *this;
}
// comparison
bool
IPortableCollationIterator::operator==(const IPortableCollationIterator& that) const
{
        return true;
}

bool
IPortableCollationIterator::operator!=(const IPortableCollationIterator& that) const
{
        return false;
}

// cursor actions
void
IPortableCollationIterator::reset()
{
}

IPortableCollationIterator::operator bool() const
{
        return false;
}

#pragma info(nouse)
IPortableCollation*
IPortableCollationIterator::create(ICollation::ECollationStrength strength) const
{
        return 0;
}
#pragma info(restore)

IText
IPortableCollationIterator::localePOSIXID() const
{
        return IText();
}

IPortableCollationIterator&
IPortableCollationIterator::operator++()         // prefix
{
        return *this;
}

IPortableCollationIterator
IPortableCollationIterator::operator++(int)     // postfix
{
        return *this;
}
// displayable key name
#pragma info(nouse)
IText
IPortableCollationIterator::displayName(const ILocaleKey& desiredLanguage,
                                                                                ILocaleKey::ELookupStrategy strategy) const
{
        return IText();
}
#pragma info(restore)

