/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 63 1.21.1.4 source/core/text/intl/compare/itxclimp.hpp, text, ioc.v400, 000919 
//===============================================================================
//
// File itxclimp.hpp : collation implementation classes
//
//
//
// Created by: Helena Shih
//
// Modification History:
//
//      Date         Name          Description
//
//===============================================================================
//
#ifndef _ITXCLIMP_
#define _ITXCLIMP_

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

#ifdef _WIN32
class IWin32Collation : public ICollation
{
        public:
                IWin32Collation(const ILocaleKey& localeKey,
                                                ICollation::ECollationStrength strength);  // default current locale
                ~IWin32Collation(void);
                IText                   displayName(const ILocaleKey& desiredLanguage,
                                                                        ILocaleKey::ELookupStrategy) const;
        protected:
                IWin32Collation();
                IWin32Collation(const IWin32Collation& copy);
                IWin32Collation& operator=(const IWin32Collation& that);

                virtual         int             do_compare(     const UniChar* sourceLow,
                                                                                const UniChar* sourceHigh,
                                                                                const UniChar* targetLow,
                                                                                const UniChar* targetHigh) const;

                virtual         IText   do_transform(const IText& source) const;


        private:
                int                     makeSortKey( unsigned long      flags,
                                                         const IText&   source,
                                                         UniChar*               sourceStr,
                                                         char*                  sourceBuffer,
                                                         int                    len) const;
#ifdef DEBUG
                static  void            printSortKeyStr(char* strBuffer, int len);
#endif
                char*   sortKeyToken(char *str, int len) const;

};
#endif

#if defined(__OS2__) || defined(_AIX) || defined(__AS400__)
#define __POSIX__
class IPOSIXCollation : public ICollation
{
        public:
                IPOSIXCollation(const ILocaleKey& localeId,
                                                ICollation::ECollationStrength strength);  // default current locale
                ~IPOSIXCollation(void);
                IText                           displayName(const ILocaleKey& desiredLanguage,
                                                                                ILocaleKey::ELookupStrategy) const;
        protected:
                IPOSIXCollation();
                IPOSIXCollation(const IPOSIXCollation& copy);
                IPOSIXCollation& operator=(const IPOSIXCollation& that);

                virtual         int     do_compare(     const UniChar* sourceLow,
                                                                                const UniChar* sourceHigh,
                                                                                const UniChar* targetLow,
                                                                                const UniChar* targetHigh) const;

                virtual         IText   do_transform(const IText& source) const;

        private:

};
#endif

class ITableBasedCollation : public IPortableCollation
{
        public:
                ITableBasedCollation ();
                virtual ~ITableBasedCollation();

        protected:
                ITableBasedCollation(const IText& collationName);
                ITableBasedCollation& operator=(const ITableBasedCollation& that);
                ITableBasedCollation (const ITableBasedCollation& order);

                virtual         int             do_matchCompare(const UniChar* sourceLow,
                                                                        const UniChar* sourceHigh,
                                                                        const UniChar* targetLow,
                                                                        const UniChar* targetHigh,
                                                                        IText::length_type& sourceCharactersMatched,
                                                                        IText::length_type& targetCharactersMatched) const;

                virtual         IText   do_transform(const IText& source) const;

    private:

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ITXCLIMP_
