// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 70 1.6.1.3 source/core/text/intl/findword/working/itxtbhdl.hpp, text, ioc.v400, 000919  
#ifndef _ITXTBHDL_
#define _ITXTBHDL_

#include <itxtbdry.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

class ITextBoundaryHandle {
public:

    static ITextBoundaryHandle characterBreak(const IText& text) { return ITextBoundaryHandle(ITextBoundary::createCharacterBreak(), text); }
    static ITextBoundaryHandle wordBreak(const IText& text) { return ITextBoundaryHandle(ITextBoundary::createWordBreak(), text); }
    static ITextBoundaryHandle lineBreak(const IText& text) { return ITextBoundaryHandle(ITextBoundary::createLineBreak(), text); }
    static ITextBoundaryHandle sentenceBreak(const IText& text) { return ITextBoundaryHandle(ITextBoundary::createSentenceBreak(), text); }

    ITextBoundaryHandle(ITextBoundary* b) : fb(b) {}
    ITextBoundaryHandle(ITextBoundary* b, const IText& text) : fb(b) { fb->setText(text); }
    ~ITextBoundaryHandle() { delete fb; }
    ITextBoundaryHandle(const ITextBoundaryHandle& rhs) { fb = rhs.fb->clone(); }
    ITextBoundaryHandle operator=(const ITextBoundaryHandle& rhs) { if (&rhs != this) { delete fb; fb = rhs.fb->clone(); } return *this; }

    IText text() const { return fb->text(); }
    void setText(const IText& text) { fb->setText(text); }

    void setToFirst() { fb->first(); }
    void setToLast() { fb->last(); }
    void setAfter(size_t offset) { fb->nextAfter(offset); }

    bool operator==(const ITextBoundaryHandle& rhs) const { return fb->operator==(*rhs.fb); }
    bool operator!=(const ITextBoundaryHandle& rhs) const { return fb->operator!=(*rhs.fb); }

    size_t operator*() const { return fb->current(); }
    
    ITextBoundaryHandle& operator++() { fb->next(); return *this; }
    ITextBoundaryHandle operator++(int) { ITextBoundaryHandle result(*this); fb->next(); return result; }
    ITextBoundaryHandle& operator--() { fb->previous(); return *this; }
    ITextBoundaryHandle operator--(int) { ITextBoundaryHandle result(*this); fb->previous(); return result; }
    ITextBoundaryHandle& operator+=(int n) { fb->nthFromCurrent(n); return *this; }
    ITextBoundaryHandle& operator-=(int n) { fb->nthFromCurrent(-n); return *this; }
    ITextBoundaryHandle operator+(int n) const { return ITextBoundaryHandle(*this) += n; }
    ITextBoundaryHandle operator-(int n) const { return ITextBoundaryHandle(*this) -= n; }

private:
    ITextBoundary* fb;
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

bool operator>(int n, const ITextBoundaryHandle& rhs) { return n > *rhs; }
bool operator>(const ITextBoundaryHandle& lhs, int n) { return *lhs > n; }
bool operator>(const ITextBoundaryHandle& lhs, const ITextBoundaryHandle& rhs) { return *lhs > *rhs; }

bool operator>=(int n, const ITextBoundaryHandle& rhs) { return n >= *rhs; }
bool operator>=(const ITextBoundaryHandle& lhs, int n) { return *lhs >= n; }
bool operator>=(const ITextBoundaryHandle& lhs, const ITextBoundaryHandle& rhs) { return *lhs >= *rhs; }

bool operator<(int n, const ITextBoundaryHandle& rhs) { return n < *rhs; }
bool operator<(const ITextBoundaryHandle& lhs, int n) { return *lhs < n; }
bool operator<(const ITextBoundaryHandle& lhs, const ITextBoundaryHandle& rhs) { return *lhs < *rhs; }

bool operator<=(int n, const ITextBoundaryHandle& rhs) { return n <= *rhs; }
bool operator<=(const ITextBoundaryHandle& lhs, int n) { return *lhs <= n; }
bool operator<=(const ITextBoundaryHandle& lhs, const ITextBoundaryHandle& rhs) { return *lhs <= *rhs; }

bool operator==(int n, const ITextBoundaryHandle& rhs) { return n == *rhs; }
bool operator==(const ITextBoundaryHandle& lhs, int n) { return *lhs == n; }

bool operator!=(int n, const ITextBoundaryHandle& rhs) { return n != *rhs; }
bool operator!=(const ITextBoundaryHandle& lhs, int n) { return *lhs != n; }

#endif // _ITXTBHDL_
