// Revision: 31 1.6.1.4 source/core/text/intl/transcod/icodecvt.cpp, text, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//  File: codecvt.cpp
//  Function: ANSI C++ Std Lib Compatible Transcoder APIs implementation
//  Author: Chen-Lieh Huang
//  Date created: 03/19/1996
//
#ifndef IC_ITRANCOD
#include "itrancod.hpp"
#endif

#ifdef IC_PAGETUNE
        #define _ICODECVT_CPP
        #include <ipagetun.h>
#endif


// The following two ANSI C++ compatible classes are optional for 96a release,
// although they worked.  After 96a, these two classes should be moved into
// itrancod.hpp, and made available as part of the Transcoder Framework APIs.

class ICodecvtToUnicode // : public codecvt<char, UniChar, ITranscoder*>
{
public:
        typedef codecvt_base::result    result;

        //      Standard member functions
        ICodecvtToUnicode();
        ICodecvtToUnicode(const ICodecvtToUnicode& source);
        ICodecvtToUnicode&              operator=(const ICodecvtToUnicode& right);
        virtual                                 ~ICodecvtToUnicode();

        // Characters are translated in the range [from, from_end), placing
        // the results in sequential position starting at "to". It converts
        // no more than (from_end - from) characters, and stores no more than
        // (to_limit - to) characters. The conversion stops
        // if it encounters a character it cannot convert. It always leaves
        // the from_next and to_next pointers pointing one beyond the last
        // character successfully converted.

        // ANSI C++ Std Lib codecvt class compatible methods
        result  convert(ITranscoder*& state,
                                                const char* from, const char* from_end, const char*& from_next,
                                                UniChar* to, UniChar* to_limit, UniChar*& to_next) const;
        bool            always_noconv() const throw();
        int                     length(const ITranscoder*& state,
                                                const char* from, const char* from_end, size_t max) const;
        int                     max_length() const throw();

protected:
        result  do_convert(ITranscoder*& state,
                                                const char* from, const char* from_end, const char*& from_next,
                                                UniChar* to, UniChar* to_limit, UniChar*& to_next) const;
        bool            do_always_noconv() const throw();
        int                     do_length(const ITranscoder*& state,
                                                const char* from, const char* from_end, size_t max) const;
        int                     do_max_length() const throw();
};


class ICodecvtFromUnicode // : public codecvt<UniChar, char, ITranscoder*>
{
public:
        typedef codecvt_base::result    result;

        //      Standard member functions
        ICodecvtFromUnicode();
        ICodecvtFromUnicode(const ICodecvtFromUnicode& source);
        ICodecvtFromUnicode&    operator=(const ICodecvtFromUnicode& right);
        virtual                                 ~ICodecvtFromUnicode();

        // Characters are translated in the range [from, from_end), placing
        // the results in sequential position starting at "to". It converts
        // no more than (from_end - from) characters, and stores no more than
        // (to_limit - to) characters. The conversion stops
        // if it encounters a character it cannot convert. It always leaves
        // the from_next and to_next pointers pointing one beyond the last
        // character successfully converted.

        // ANSI C++ Std Lib codecvt class compatible methods
        result  convert(ITranscoder*& state,
                                                const UniChar* from, const UniChar* from_end, const UniChar*& from_next,
                                                char* to, char* to_limit, char*& to_next) const;
        bool            always_noconv() const throw();
        int             length(const ITranscoder*& state,
                                                const UniChar* from, const UniChar* from_end, size_t max) const;
        int             max_length() const throw();

protected:
        result  do_convert(ITranscoder*& state,
                                                const UniChar* from, const UniChar* from_end, const UniChar*& from_next,
                                                char* to, char* to_limit, char*& to_next) const;
        bool            do_always_noconv() const throw();
        int                     do_length(const ITranscoder*& state,
                                                const UniChar* from, const UniChar* from_end, size_t max) const;
        int                     do_max_length() const throw();
};


//
//      ICodecvtToUnicode class implementation
//
ICodecvtToUnicode::ICodecvtToUnicode()
{
}


ICodecvtToUnicode::~ICodecvtToUnicode()
{
}


ICodecvtToUnicode::ICodecvtToUnicode(const ICodecvtToUnicode &source)
{
}


ICodecvtToUnicode &
ICodecvtToUnicode::operator=(const ICodecvtToUnicode &right)
{
        return *this;
}


codecvt_base::result
ICodecvtToUnicode::convert(ITranscoder*& state,
                const char* from, const char* from_end, const char*& from_next,
                UniChar* to, UniChar* to_limit, UniChar*& to_next) const
{
        return do_convert(state, from, from_end, from_next, to, to_limit, to_next);
}


bool
ICodecvtToUnicode::always_noconv() const throw()
{
        return do_always_noconv();
}


int
ICodecvtToUnicode::length(const ITranscoder*& state,
                const char* from, const char* from_end, size_t max) const
{
        return do_length(state, from, from_end, max);
}


int
ICodecvtToUnicode::max_length() const throw()
{
        return do_max_length();
}


codecvt_base::result
ICodecvtToUnicode::do_convert(ITranscoder*& state,
                const char* from, const char* from_end, const char*& from_next,
                UniChar* to, UniChar* to_limit, UniChar*& to_next) const
{
        if (state == NULL)
                ITHROW(IInvalidParameter("State not initialized!"));

        return state->toUnicode(from, from_end, from_next, to, to_limit, to_next);
}


bool
ICodecvtToUnicode::do_always_noconv() const throw()
{
        return false;
        // return true, if codecvt<char, char, mbstate_t> is instantiated.
}


int
ICodecvtToUnicode::do_length(const ITranscoder*& state,
                const char* from, const char* from_end, size_t max) const
{
        // Verifies preconditions
        if (state == NULL || from == NULL || from > from_end)
                ITHROW(IInvalidParameter("State not initialized or buffer range error!"));

        int length = state->uniCharBufferSize(from, from_end);
        return (length < max) ? length : max;
}


int
ICodecvtToUnicode::do_max_length() const throw()
{
        return 1;       // temp! But this should work.
//      return state->maximumUniCharsPerByte();
//
//      Above statement is the correct implementation of this function.
//      Since this function doesn't take a transcoder argument, there
//      is no way to make a call through the state/transcoder object.
//      We might have to adopt the transcoder object while this
//      ICodecvtToUnicode class is instantiated. For instance,
//      ICodecvtToUnicode::ICodecvtFromUnicode(ITranscoder* adopted).
//      And then we need to add another API ICodecvtToUnicode::orphan(),
//      which can be called to free up the storage of the adopted
//      transcoder object during ICodecvtFromUnicode instantiation.
//      The reason of using "adopt" is that whoever use this ANSI C++
//      APIs would not care to use IOC Transcoder APIs. So ANSI C++
//      clients should not have to worry about the IOC Transcoder
//      object at all after this ANSI codecvt object is created.
}


//
//      ICodecvtFromUnicode implementation
//
ICodecvtFromUnicode::ICodecvtFromUnicode()
{
}


ICodecvtFromUnicode::~ICodecvtFromUnicode()
{
}


ICodecvtFromUnicode::ICodecvtFromUnicode(const ICodecvtFromUnicode &source)
{
}


ICodecvtFromUnicode &
ICodecvtFromUnicode::operator=(const ICodecvtFromUnicode &right)
{
        return *this;
}


codecvt_base::result
ICodecvtFromUnicode::convert(ITranscoder*& state,
                const UniChar* from, const UniChar* from_end, const UniChar*& from_next,
                char* to, char* to_limit, char*& to_next) const
{
        return do_convert(state, from, from_end, from_next, to, to_limit, to_next);
}


bool
ICodecvtFromUnicode::always_noconv() const throw()
{
        return do_always_noconv();
}


int
ICodecvtFromUnicode::length(const ITranscoder*& state,
                const UniChar* from, const UniChar* from_end, size_t max) const
{
        return do_length(state, from, from_end, max);
}


int
ICodecvtFromUnicode::max_length() const throw()
{
        return do_max_length();
}


codecvt_base::result
ICodecvtFromUnicode::do_convert(ITranscoder*& state,
                const UniChar* from, const UniChar* from_end, const UniChar*& from_next,
                char* to, char* to_limit, char*& to_next) const
{
        if (state == NULL)
                ITHROW(IInvalidParameter("State not initialized!"));

        return state->fromUnicode(from, from_end, from_next, to, to_limit, to_next);
}


bool
ICodecvtFromUnicode::do_always_noconv() const throw()
{
        return false;
        // return true, if codecvt<char, char, mbstate_t> is instantiated.
}


int
ICodecvtFromUnicode::do_length(const ITranscoder*& state,
                const UniChar* from, const UniChar* from_end, size_t max) const
{
        // Verifies preconditions
        if (state == NULL || from == NULL || from > from_end)
                ITHROW(IInvalidParameter("State not initialized or buffer range error!"));

        int length = state->byteBufferSize(from, from_end);
        return (length < max) ? length : max;
}


int
ICodecvtFromUnicode::do_max_length() const throw()
{
        return 5;       // temp! But this should work.
//      return state->maximumBytesPerUniChar();
//
//      Above statement is the correct implementation of this function.
//      Since this function doesn't take a transcoder argument, there
//      is no way to make a call through the state/transcoder object.
//      We might have to adopt the transcoder object while this
//      ICodecvtFromUnicode class is instantiated. For instance,
//      ICodecvtFromUnicode::ICodecvtFromUnicode(ITranscoder* adopted).
//      And then we need to add another API ICodecvtFromUnicode::orphan(),
//      which can be called to free up the storage of the adopted
//      transcoder object during ICodecvtFromUnicode instantiation.
}
