// Revision: 37 1.9.1.4 source/core/text/intl/transcod/lbconvtr.cpp, text, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//  File: lbconvtr.cpp
//  Function: Line Break Converter implementation.
//  Author: Chen-Lieh Huang
//  Date created: 04/05/1996
//
#ifndef IC_ITRANCOD
#include "itrancod.hpp"
#endif

#ifdef IC_PAGETUNE
        #define _LBCONVTR_CPP
        #include <ipagetun.h>
#endif


void
ILineBreakConverter::convertInPlace(IText& string, ELineBreakConvention target)
{
        if (target == kHost)
        {
#ifdef _WIN32
                target = kCRLF;
#elif __OS2__
                target = kCRLF;
#elif _AIX
                target = kLF;
#else
                #error "Must port ILineBreakConverter::convert for this host!"
#endif
        }

        ITextIterator   iter = string.begin();
        UniChar         save;

        switch (target) {
                case kUnicode:
                        //
                        // Search for CR/LF, LF, CR, or VT, and replace them
                        // using the following rules:
                        //
                        // Host/App                             Host Line Breaks                                        Unicode Line Breaks
                        // ========                             ================                                        ===================
                        // DOS,OS/2,Win         kCRLF:0x000D 0x000A             <=>     U+2029 PARAGRAPH SEPARATOR
                        // Unix                                         kLF:0x000A                                              <=>     U+2029 PARAGRAPH SEPARATOR
                        // Mac                                          kCR:0x000D                                              <=>     U+2029 PARAGRAPH SEPARATOR
                        // Word/RichEdit        kCRLF_VT:0x000D 0x000A<=>       U+2029 PARAGRAPH SEPARATOR
                        // Word/RichEdit        kCRLF_VT:0x000B                         <=>     U+2028 LINE SEPARATOR
                        //
                        while (iter != string.end()) {
                                if (*iter == 0x000D || *iter == 0x000A) {
                    save = *iter;
                    *iter = 0x2029;
                                        ++iter;
                    if (iter == string.end()) break;
                    if (*iter == 0x000D || *iter == 0x000A) {
                        if (*iter != save) {
                            string.erase(iter);
                            continue;
                        }
                        else {
                            *iter = 0x2029;
                        }
                    }
                                }
                                else if (*iter == 0x000B) {
                                        *iter = 0x2028;
                                }
                                ++iter;
                        }
                        break;
                case kCRLF:
                        while (iter != string.end()) {
                                if (*iter == 0x2029) {
                                        *iter = 0x000D;
                                        ++iter;
                                        iter = string.insert(iter, 0x000A);
                                }
                                ++iter;
                        }
                        break;
                case kLF:
                        while (iter != string.end()) {
                                if (*iter == 0x2029)
                                        *iter = 0x000A;
                                ++iter;
                        }
                        break;
                case kCR:
                        while (iter != string.end()) {
                                if (*iter == 0x2029)
                                        *iter = 0x000D;
                                ++iter;
                        }
                        break;
                case kCRLF_VT:
                        while (iter != string.end()) {
                                if (*iter == 0x2029) {
                                        *iter = 0x000D;
                                        ++iter;
                                        iter = string.insert(iter, 0x000A);
                                }
                                else if (*iter == 0x2028)
                                        *iter = 0x000B;
                                ++iter;
                        }
                        break;
        }
}


IText
ILineBreakConverter::convert(const IText& string, ELineBreakConvention target)
{
        if (target == kHost)
        {
#ifdef _WIN32
                target = kCRLF;
#elif __OS2__
                target = kCRLF;
#elif _AIX
                target = kLF;
#else
                #error "Must port ILineBreakConverter::convert for this host!"
#endif
        }

        IText   aCopy = string;
        ITextIterator   iter = aCopy.begin();
        UniChar save;

        switch (target) {
                case kUnicode:
                        while (iter != aCopy.end()) {
                                if (*iter == 0x000D || *iter == 0x000A) {
                    save = *iter;
                    *iter = 0x2029;
                                        ++iter;
                    if (iter == aCopy.end()) break;
                    if (*iter == 0x000D || *iter == 0x000A) {
                        if (*iter != save) {
                            aCopy.erase(iter);
                            continue;
                        }
                        else {
                            *iter = 0x2029;
                        }
                    }
                                }
                                else if (*iter == 0x000B) {
                                        *iter = 0x2028;
                                }
                                ++iter;
                        }
                        break;
                case kCRLF:
                        while (iter != aCopy.end()) {
                                if (*iter == 0x2029) {
                                        *iter = 0x000D;
                                        ++iter;
                                        iter = aCopy.insert(iter, 0x000A);
                                }
                                ++iter;
                        }
                        break;
                case kLF:
                        while (iter != aCopy.end()) {
                                if (*iter == 0x2029)
                                        *iter = 0x000A;
                                ++iter;
                        }
                        break;
                case kCR:
                        while (iter != aCopy.end()) {
                                if (*iter == 0x2029)
                                        *iter = 0x000D;
                                ++iter;
                        }
                        break;
                case kCRLF_VT:
                        while (iter != aCopy.end()) {
                                if (*iter == 0x2029) {
                                        *iter = 0x000D;
                                        ++iter;
                                        iter = aCopy.insert(iter, 0x000A);
                                }
                                else if (*iter == 0x2028)
                                        *iter = 0x000B;
                                ++iter;
                        }
                        break;
        }

        return aCopy;
}

// Host-specific function implementation
// ELineBreakConvention ILineBreakConverter::hostConvention() {...}
