// Revision: 12 1.6.1.4 source/core/text/intl/transcod/utf8tran.hpp, text, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//  File: utf8tran.hpp
//  Function: Internal UTF-8 Transcoder Definition.
//  Author: Chen-Lieh Huang
//  Date created: 06/03/1996
//
#ifndef _UTF8TRAN_
#define _UTF8TRAN_

#include <itrancod.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

class UTF8Transcoder : public ITranscoder {
public:
        //      Standard member functions
        UTF8Transcoder();
        UTF8Transcoder(const UTF8Transcoder& source);
        UTF8Transcoder& operator=(const UTF8Transcoder& right);
        virtual                         ~UTF8Transcoder();

        // Overrides
        length_type             maximumBytesPerUniChar() const;
        length_type             maximumUniCharsPerByte() const;

        length_type             byteBufferSize(const IText& uniText) const;
        length_type             byteBufferSize(const UniChar* from, const UniChar* from_end) const;
        length_type             uniCharBufferSize(const IString& text) const;
        length_type             uniCharBufferSize(const char* from, const char* from_end) const;

protected:
        //      called by toUnicode
        result  doToUnicode(const char* from, const char* from_end,
                                                const char*& from_next,
                                                UniChar* to, UniChar* to_limit, UniChar*& to_next);

        //      called by fromUnicode
        result  doFromUnicode(const UniChar* from, const UniChar* from_end,
                                                const UniChar*& from_next,
                                                char* to, char* to_limit, char*& to_next);

        // Sets the maximum number of bytes generated by converting a UniChar.
        void            setMaximumBytesPerUniChar(length_type bytes);

        // Sets the maximum number of UniChars generated by converting
        // a multibyte char.
        void            setMaximumUniCharsPerByte(length_type bytes);

private:
        length_type                     fMaximumBytesPerUniChar;
        length_type                     fMaximumUniCharsPerByte;
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _UTF8TRAN_
