// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 13 1.16.1.2 source/core/text/itext/icrend.cpp, text, ioc.v400, 000919  
#include "icrend.hpp"

#include "istylset.hpp"

#include <ilanglvl.hpp>
#if IC_RTTI_ENABLED
#include <typeinfo.h>
#endif // IC_RTTI_ENABLED

#ifdef IC_PAGETUNE
	#define _ICREND_CPP
	#include <ipagetun.cpp>
#endif

ICharacterRenderer* ICharacterRenderer::extractFrom(const ITextStyleSet* s)
{
	ICharacterRenderer* result;

	if (s == NULL)
		result = NULL;
	else
		result = s->characterRenderer();

	if (result != NULL) {
		result->setStyle(s);
	}
	return result;
}

ICharacterRenderer::ICharacterRenderer()
	: ICharacterStyle()
{
}

bool
ICharacterRenderer::hasSameValueAs(const ICharacterRenderer& other) const
{
#if IC_RTTI_ENABLED
	return typeid(*this) == typeid(other);
#else
	return true;
#endif // IC_RTTI_ENABLED
}

void
ICharacterRenderer::operator>>=(IDataStream& toWhere) const
{
	writeToStream(toWhere);
}

void
ICharacterRenderer::operator<<=(IDataStream& fromWhere)
{
	readFromStream(fromWhere);
}

void
ICharacterRenderer::setStyleFrom(const ITextIterator& where)
{
	IText::offset_type start;
	IText::length_type length;
	const ITextStyleSet* styles = where.text()->stylesAt(where.offset(), start, length);
	setStyle(styles);
}

void
ICharacterRenderer::addToSet(ITextStyleSet&	set) const
{
	delete set.fCharacterRenderer;
	set.fCharacterRenderer = clone();
	set.fStyleContainmentFlags |= ITextStyleSet::kHasCharacterRenderer;
}

void
ICharacterRenderer::removeFromSet(ITextStyleSet& set) const
{
	delete set.fCharacterRenderer;
	set.fCharacterRenderer = NULL;
	set.fStyleContainmentFlags &= (unsigned long)~ITextStyleSet::kHasCharacterRenderer;
}

void
ICharacterRenderer::getValueFromSet(const ITextStyleSet& set)
{
}

bool
ICharacterRenderer::isContainedInSet(const ITextStyleSet& set) const
{
	return !!(set.fStyleContainmentFlags & ITextStyleSet::kHasCharacterRenderer);
}

bool
ICharacterRenderer::isValueContainedInSet(const ITextStyleSet& set) const
{
	return isContainedInSet(set) && hasSameValueAs(*set.fCharacterRenderer);
}
