// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 14 1.15.1.4 source/core/text/itext/icrend.hpp, text, ioc.v400, 000919 
#ifndef _ICREND_
#define _ICREND_


#include <itxtgrt.hpp>
#include <itxtstyl.hpp>	//for ICharacterStyle
#include <itxtplmt.hpp>	//for ITextPlacement

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ITextStyleSet;
class IGrafPort;
class IGPoint2D;

#pragma pack(push,4)
#pragma enum(4)

class ICharacterRenderer : public ICharacterStyle {
public:
	static ICharacterRenderer* extractFrom(const ITextStyleSet* s);

	virtual ICharacterRenderer* clone() const = 0;
	virtual bool			hasSameValueAs(const ICharacterRenderer& other) const;
	virtual void			operator>>=(IDataStream& toWhere) const;
	virtual void			operator<<=(IDataStream& fromWhere);
	virtual void			writeToStream(IDataStream& toWhere) const = 0;
	virtual void			readFromStream(IDataStream& fromWhere) = 0;

	virtual void			setStyle(const ITextStyleSet* s) = 0;
	void					setStyleFrom(const ITextIterator& where);

	virtual GCoord		getAscent(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const = 0;

	virtual GCoord		getDescent(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const = 0;

	virtual GCoord		getLeading(
								ITextIterator textStart,
								const ITextIterator& textLimit,
								IGrafPort& g) const = 0;

	virtual GCoord		handleDrawChars(
								IGrafPort& g,
								const IGPoint2D& baseLine,
								ITextIterator textStart,
								const ITextIterator& textLimit) const = 0;

	virtual GCoord		handleAdvance(
								IGrafPort& g,
								ITextIterator textStart,
								const ITextIterator& textLimit,
								GCoord tabAdvance) const = 0;

	virtual GCoord		handleCharWidth(
								IGrafPort& g,
								const ITextIterator& iter) const = 0;

	virtual ITextPlacement	handlePixelToIndex(
								IGrafPort& g,
								const IGPoint2D& pixel,
								ITextIterator textStart,
								const ITextIterator& textLimit) const = 0;

	virtual IGPoint2D		handleIndexToPixel(
								IGrafPort& g,
								ITextIterator textStart,
								const ITextIterator& textLimit,
								const ITextPlacement& placementToSet) const = 0;

	virtual bool 			handleIsWhiteSpace(
								const ITextIterator& charPos) const = 0;

	virtual bool 			handleIsDecimal(
								const ITextIterator& charPos) const = 0;


protected:
	ICharacterRenderer();

	virtual void			addToSet(ITextStyleSet&	set) const;
	virtual void			removeFromSet(ITextStyleSet& set) const;
	virtual void			getValueFromSet(const ITextStyleSet& set);
	virtual bool			isContainedInSet(const ITextStyleSet& set) const;
	virtual bool			isValueContainedInSet(const ITextStyleSet& set) const;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICREND_
