// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 15 1.22.1.3 source/core/text/itext/iprend.cpp, text, ioc.v400, 000919  
#include "iprend.hpp"

#include "istylset.hpp"

#include <ilanglvl.hpp>
#if IC_RTTI_ENABLED
#include <typeinfo.h>
#endif // IC_RTTI_ENABLED

#ifdef IC_PAGETUNE
	#define _IPREND_CPP
	#include <ipagetun.h>
#endif

ILayoutInfo::ILayoutInfo()
	: fText(NULL),
	fCharStart(0),
	fCharLength(0),
	fSegmentCount(0),
	fSegments(NULL)
{
}

ILayoutInfo::ILayoutInfo(const ITextIterator& start, const ITextIterator& limit)
	: fText(start.text()),
	fCharStart(start.offset()),
	fCharLength(limit.offset() - start.offset()),
	fSegmentCount(0),
	fSegments(NULL)
{
}

ILayoutInfo::ILayoutInfo(IText *text)
	: fText(text),
	fCharStart(text->begin().offset()),
	fCharLength(text->end().offset() - fCharStart),
	fSegmentCount(0),
	fSegments(NULL)
{
}

ILayoutInfo::~ILayoutInfo()
{
	delete [] fSegments;
}

ITextIterator ILayoutInfo::charStart() const
{
	ITextIterator r = fText->begin();
	if (fCharStart < 0)
		r += long(fText->length()) + fCharStart;
	else
		r += fCharStart;
	return r;
}

ITextIterator ILayoutInfo::charStart(IText::offset_type length) const
{
	ITextIterator r = fText->begin();
	if (fCharStart < 0)
		r += long(length) + fCharStart;
	else
		r += fCharStart;
	return r;
}

void ILayoutInfo::setCharStart(const ITextIterator& start)
{
	ITextIterator tempLimit = charLimit();
	fText = start.text();
	fCharStart = start.offset();
	setCharLimit(tempLimit);
//	clearSegments();
}

ITextIterator ILayoutInfo::charLimit() const
{
	return charStart() + fCharLength;
}

ITextIterator ILayoutInfo::charLimit(IText::offset_type length) const
{
	return charStart(length) + fCharLength;
}

void ILayoutInfo::setCharLimit(const ITextIterator& limit)
{
	fCharLength = limit.offset() - fCharStart;
//	clearSegments();
}

GCoord ILayoutInfo::height()
{
	return fAscent + fDescent + fLeading;
}

void ILayoutInfo::shiftCharStart(long textLength)
{
	fCharStart += textLength;
}

void ILayoutInfo::addSegment(const ISegmentInfo& segment)
{
	ISegmentInfo* newSegments = new ISegmentInfo[fSegmentCount+1];
	for (size_t i = 0; i < fSegmentCount; i++) {
		newSegments[i] = fSegments[i];
	}
	newSegments[fSegmentCount] = segment;
	fSegmentCount++;
	delete [] fSegments;
	fSegments = newSegments;
}

size_t ILayoutInfo::segmentCount() const
{
	return fSegmentCount;
}

void ILayoutInfo::clearSegments()
{
	fSegmentCount = 0;
	delete fSegments;
	fSegments = NULL;
}

ISegmentInfo& ILayoutInfo::operator[](size_t ndx)
{
//	ASSERT(ndx < fSegmentCount);
	return fSegments[ndx];
}

const ISegmentInfo& ILayoutInfo::operator[](size_t ndx) const
{
//	ASSERT(ndx < fSegmentCount);
	return fSegments[ndx];
}

//................................................................................


IParagraphRenderer* IParagraphRenderer::extractFrom(const ITextStyleSet* s)
{
	IParagraphRenderer* result = s->paragraphRenderer();
	if (result != NULL) {
		result->setStyle(s);
	}
	return result;
}

IParagraphRenderer::IParagraphRenderer()
{
}

IParagraphRenderer::~IParagraphRenderer()
{
}

bool
IParagraphRenderer::hasSameValueAs(const IParagraphRenderer& other) const
{
#if IC_RTTI_ENABLED
	return typeid(*this) == typeid(other);
#else
	return true;
#endif // IC_RTTI_ENABLED
}

void
IParagraphRenderer::operator>>=(IDataStream& toWhere) const
{
	writeToStream(toWhere);
}

void
IParagraphRenderer::operator<<=(IDataStream& fromWhere)
{
	readFromStream(fromWhere);
}

void
IParagraphRenderer::setStyleFrom(const ITextIterator& where)
{
	IText::offset_type start;
	IText::length_type length;
	const ITextStyleSet* styles = where.text()->stylesAt(where.offset(), start, length);
	setStyle(styles);
}

void
IParagraphRenderer::addToSet(ITextStyleSet&	set) const
{
	delete set.fParagraphRenderer;
	set.fParagraphRenderer = NULL;
	set.fParagraphRenderer = clone();
	set.fStyleContainmentFlags |= ITextStyleSet::kHasParagraphRenderer;
}

void
IParagraphRenderer::removeFromSet(ITextStyleSet& set) const
{
	delete set.fParagraphRenderer;
	set.fParagraphRenderer = NULL;
	set.fStyleContainmentFlags &= (unsigned long)~ITextStyleSet::kHasParagraphRenderer;
}

void
IParagraphRenderer::getValueFromSet(const ITextStyleSet& set)
{
}

bool
IParagraphRenderer::isContainedInSet(const ITextStyleSet& set) const
{
	return !!(set.fStyleContainmentFlags & ITextStyleSet::kHasParagraphRenderer);
}

bool
IParagraphRenderer::isValueContainedInSet(const ITextStyleSet& set) const
{
	return isContainedInSet(set) && hasSameValueAs(*set.fParagraphRenderer);
}
