// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 16 1.25.1.3 source/core/text/itext/iprend.hpp, text, ioc.v400, 000919  
#ifndef _IPREND_
#define _IPREND_

#include <itxtgrt.hpp> // for GCoord
#include <itxtstyl.hpp>	//for ICharacterStyle
#include <itxtplmt.hpp>	//for ITextPlacement

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ITextStyleSet;
class IGrafPort;
class IGPoint2D;
class IGRect2D;
class IColor;

#pragma pack(push,4)
#pragma enum(4)

class ISegmentInfo{
public:
	ISegmentInfo()
	{
	}

	ISegmentInfo(GCoord advance, IText::length_type length)
	{
		fAdvance = advance;
		fLength = length;
	}

	GCoord fAdvance; // anchor position to draw text from
	IText::length_type fLength; // length of text to draw
};


class ILayoutInfo {
public:
//    size_t  fCharStart;   // offset in text to start of line (was fStart)
                          // neg. values indicate distance from end of text
//    size_t  fCharLength;  // number of characters on line (was fLength)

	ILayoutInfo();
	ILayoutInfo(const ITextIterator& start, const ITextIterator& limit);
	//hey {jf} - trash this constructor
	ILayoutInfo(IText *text);
	~ILayoutInfo();

	ITextIterator charStart() const;
	ITextIterator charStart(IText::length_type length) const;

	void setCharStart(const ITextIterator& start);

	ITextIterator charLimit() const;
	ITextIterator charLimit(IText::length_type length) const;
	void setCharLimit(const ITextIterator& limit);

    GCoord height();
	void shiftCharStart(long textLength);

    GCoord  fAscent;
    GCoord  fDescent;
    GCoord  fLeading;
    GCoord  fAdvance;       // distance along line direction ie width
    GCoord  fTotalAdvance;  // distance along line direction including trailing whitespace
    GCoord  fGraphicStart;  // min pixel offset in fill direction
                              // negative values indicate distance from bottom
                              // of text view
    GCoord  fLeftMargin;  // distance in pixels from left/top margin
//    GCoord  fRightMargin;

	void addSegment(const ISegmentInfo& segment);
	size_t segmentCount() const;
    ISegmentInfo& operator[](size_t);
    const ISegmentInfo& operator[](size_t) const;
	void clearSegments();

	long getCharStart() { return fCharStart; }
	long getCharLength() { return fCharLength; }

private:
	long fCharStart, fCharLength;
	IText *fText;
	size_t fSegmentCount;
	ISegmentInfo* fSegments;
};


class IParagraphRenderer : public IParagraphStyle {
public:
	static IParagraphRenderer* extractFrom(const ITextStyleSet* s);

	virtual					~IParagraphRenderer();

	virtual IParagraphRenderer* clone() const = 0;
	virtual bool			hasSameValueAs(const IParagraphRenderer& other) const;
	virtual void			operator>>=(IDataStream& toWhere) const;
	virtual void			operator<<=(IDataStream& fromWhere);
	virtual void			writeToStream(IDataStream& toWhere) const = 0;
	virtual void			readFromStream(IDataStream& fromWhere) = 0;

	virtual void			layout(
								IGrafPort& g,
								ILayoutInfo& infoOut,
								const GCoord& lineDimension,
								ITextIterator lineStart,
								const ITextIterator& paragraphStart,
								const ITextIterator& limit,
								GCoord averageCharWidth) const = 0;

	virtual void			renderWithHighlight(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& baseLine,
								const ITextIterator& selStart,
								const ITextIterator& selEnd,
								const IColor& hilight) const = 0;

	virtual void			render(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& baseLine) const = 0;

	virtual ITextPlacement	pixelToIndex(
								IGrafPort& g,
								const ILayoutInfo& info,
								const IGPoint2D& where) const = 0;

	virtual IGPoint2D		indexToPixel(
								IGrafPort& g,
								const ILayoutInfo& info,
								const ITextPlacement& index) const = 0;

	virtual void			renderCaret(
								const ILayoutInfo& info,
								const ITextPlacement& index) const = 0;

	virtual void			setStyle(const ITextStyleSet* style) = 0;
	void					setStyleFrom(const ITextIterator& where);

protected:
	IParagraphRenderer();

	virtual void			addToSet(ITextStyleSet&	set) const;
	virtual void			removeFromSet(ITextStyleSet& set) const;
	virtual void			getValueFromSet(const ITextStyleSet& set);
	virtual bool			isContainedInSet(const ITextStyleSet& set) const;
	virtual bool			isValueContainedInSet(const ITextStyleSet& set) const;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IPREND_
