// Revision: 41 1.9.1.5 source/core/text/itext/itxtiter.cpp, text, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//      ITextIter.cpp
//
//      Contains: Out-of-line methods on ITextIterator. (will veyry likely delete this file)
//
// Author:
//      Richard Gillam
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef IC_ITXTITER
#include "itxtiter.hpp"
#endif

#ifndef IC_ITEXT
#include "itext.hpp"
#endif

#ifndef IC_ITXTSTOR
#include "itxtstor.hpp"
#endif

#include <iexcept.hpp>

#ifdef IC_PAGETUNE
        #define _ITXTITER_CPP
        #include <ipagetun.h>
#endif

//========================================================================================
// OUT-OF-LINE METHODS ON ITextIterator AND IConstTextIterator
//
// These methods are here because the IASSERTPARM macro expands to too long a sequence
// of statements to use in an inline function.  The inline functions on the iterator
// classes all do their own assert check and then call the throwException function
// if they fail.
//========================================================================================

#pragma info(none)
void
ITextIterator::throwException() const
{
        bool alwaysFalse = false;
        IASSERTPARM(alwaysFalse);
}

void
IConstTextIterator::throwException() const
{
        bool alwaysFalse = false;
        IASSERTPARM(alwaysFalse);
}

//========================================================================================
// OUT-OF-LINE METHODS ON IFastTextIterator
//========================================================================================

IFastTextIterator::IFastTextIterator(   const ITextIterator&    that,
                                                                                bool                                    willWrite)
: fChar(0),
  fGapStart(0),
  fGapLength(0)
{
        // if the client says he'll be writing through this iterator (or if he doesn't say),
        // make sure onl;y the one IText is pointing to the ITextStorage we're going to affect
#pragma info(nocnv)
        if (willWrite)
                ((IText*)(that.text()))->fStorage = that.text()->fStorage->cloneIfNecessary();
#pragma info(restore)
        *this = that.text()->fStorage->beginAt(that.offset());
}

IFastTextIterator::IFastTextIterator(   const IConstTextIterator&       that,
                                                                                bool                                            willWrite)
: fChar(0),
  fGapStart(0),
  fGapLength(0)
{
        // if the client says he'll be writing through this iterator (or if he doesn't say),
        // make sure onl;y the one IText is pointing to the ITextStorage we're going to affect
#pragma info(nocnv)
        if (willWrite)
                ((IText*)(that.text()))->fStorage = that.text()->fStorage->cloneIfNecessary();
#pragma info(restore)
        *this = that.text()->fStorage->beginAt(that.offset());
}

#pragma info(restore)
