// Revision: 70 1.4.1.4 source/core/text/styles/istylset.cpp, text, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
#ifndef _ISTYLSET_HPP
#include "istylset.hpp"
#endif

#ifdef IC_PAGETUNE
        #define _ISTYLSET_CPP
        #include <ipagetun.h>
#endif

ITextStyleSet::ITextStyleSet() :
                        fContainsTextSuperSubscriptStyle( false ),
                        fContainsTextBoldfaceStyle( false ),
                        fContainsTextItalicStyle( false ),
                        fContainsTextOutlineStyle( false ),
                        fContainsTextStrikethroughStyle( false ),
                        fContainsTextUnderlineStyle( false ),
                        fContainsTextUneditableStyle( false ),
                        fContainsTextPointSizeStyle( false ),
                        fContainsTextColorStyle( false ),
                        fContainsTextTypefaceStyle( false ),
                        fContainsParagraphStartIndentStyle( false ),
                        fContainsParagraphEndIndentStyle( false ),
                        fContainsParagraphFirstIndentStyle( false ),
                        fContainsParagraphJustificationStyle( false ),
                        fContainsParagraphSpreadStyle( false ),
                        fTextColorStyleValue( IColor::black )
{
}


ITextStyleSet::ITextStyleSet( const ITextStyle& style ) :
                        fContainsTextSuperSubscriptStyle( false ),
                        fContainsTextBoldfaceStyle( false ),
                        fContainsTextItalicStyle( false ),
                        fContainsTextOutlineStyle( false ),
                        fContainsTextStrikethroughStyle( false ),
                        fContainsTextUnderlineStyle( false ),
                        fContainsTextUneditableStyle( false ),
                        fContainsTextPointSizeStyle( false ),
                        fContainsTextColorStyle( false ),
                        fContainsTextTypefaceStyle( false ),
                        fContainsParagraphStartIndentStyle( false ),
                        fContainsParagraphEndIndentStyle( false ),
                        fContainsParagraphFirstIndentStyle( false ),
                        fContainsParagraphJustificationStyle( false ),
                        fContainsParagraphSpreadStyle( false ),
                        fTextColorStyleValue( IColor::black )
{
        add( style );
}


ITextStyleSet::ITextStyleSet( const ITextStyleSet& other ) :
                        fContainsTextSuperSubscriptStyle( false ),
                        fContainsTextBoldfaceStyle( false ),
                        fContainsTextItalicStyle( false ),
                        fContainsTextOutlineStyle( false ),
                        fContainsTextStrikethroughStyle( false ),
                        fContainsTextUnderlineStyle( false ),
                        fContainsTextUneditableStyle( false ),
                        fContainsTextPointSizeStyle( false ),
                        fContainsTextColorStyle( false ),
                        fContainsTextTypefaceStyle( false ),
                        fContainsParagraphStartIndentStyle( false ),
                        fContainsParagraphEndIndentStyle( false ),
                        fContainsParagraphFirstIndentStyle( false ),
                        fContainsParagraphJustificationStyle( false ),
                        fContainsParagraphSpreadStyle( false ),
                        fTextColorStyleValue( IColor::black )
{
        operator=( other );
}


ITextStyleSet::~ITextStyleSet()
{
}



ITextStyleSet&
ITextStyleSet::operator=( const ITextStyleSet& other )
{
        if ( &other != this ) {
                fContainsTextSuperSubscriptStyle = other.fContainsTextSuperSubscriptStyle;
                fContainsTextBoldfaceStyle = other.fContainsTextBoldfaceStyle;
                fContainsTextItalicStyle = other.fContainsTextItalicStyle;
                fContainsTextOutlineStyle = other.fContainsTextOutlineStyle;
                fContainsTextStrikethroughStyle = other.fContainsTextStrikethroughStyle;
                fContainsTextUnderlineStyle = other.fContainsTextUnderlineStyle;
                fContainsTextUneditableStyle = other.fContainsTextUneditableStyle;
                fContainsTextPointSizeStyle = other.fContainsTextPointSizeStyle;
                fContainsTextColorStyle = other.fContainsTextColorStyle;
                fContainsTextTypefaceStyle = other.fContainsTextTypefaceStyle;
                fContainsParagraphStartIndentStyle = other.fContainsParagraphStartIndentStyle;
                fContainsParagraphEndIndentStyle = other.fContainsParagraphEndIndentStyle;
                fContainsParagraphFirstIndentStyle = other.fContainsParagraphFirstIndentStyle;
                fContainsParagraphJustificationStyle = other.fContainsParagraphJustificationStyle;
                fContainsParagraphSpreadStyle = other.fContainsParagraphSpreadStyle;

                fTextSuperSubscriptStyleValue = other.fTextSuperSubscriptStyleValue;
                fTextBoldfaceStyleValue = other.fTextBoldfaceStyleValue;
                fTextItalicStyleValue = other.fTextItalicStyleValue;
                fTextOutlineStyleValue = other.fTextOutlineStyleValue;
                fTextStrikethroughStyleValue = other.fTextStrikethroughStyleValue;
                fTextUnderlineStyleValue = other.fTextUnderlineStyleValue;
                fTextUneditableStyleValue = other.fTextUneditableStyleValue;
                fTextPointSizeStyleValue = other.fTextPointSizeStyleValue;
                fTextColorStyleValue = other.fTextColorStyleValue;
                fTextTypefaceStyleValue = other.fTextTypefaceStyleValue;
                fParagraphStartIndentStyleValue = other.fParagraphStartIndentStyleValue;
                fParagraphEndIndentStyleValue = other.fParagraphEndIndentStyleValue;
                fParagraphFirstIndentStyleValue = other.fParagraphFirstIndentStyleValue;
                fParagraphJustificationStyleValue = other.fParagraphJustificationStyleValue;
                fParagraphSpreadStyleValue = other.fParagraphSpreadStyleValue;
                }
        return *this;
}



ITextStyleSet&
ITextStyleSet::operator=( const ITextStyle& style )
{
        removeAll();
        add( style );
        return *this;
}


ITextStyleSet
ITextStyleSet::operator+( const ITextStyleSet& other ) const
{
        ITextStyleSet returnValue( *this );
        returnValue += other;
        return returnValue;
}



ITextStyleSet
ITextStyleSet::operator+( const ITextStyle& style ) const
{
        ITextStyleSet returnValue( *this );
        returnValue += style;
        return returnValue;
}



ITextStyleSet
ITextStyleSet::operator-( const ITextStyleSet& other ) const
{
        ITextStyleSet returnValue( *this );
        returnValue -= other;
        return returnValue;
}



ITextStyleSet
ITextStyleSet::operator-( const ITextStyle& style ) const
{
        ITextStyleSet returnValue( *this );
        returnValue -= style;
        return returnValue;
}



bool
ITextStyleSet::operator==( const ITextStyleSet& other ) const
{
        return  fContainsTextSuperSubscriptStyle == other.fContainsTextSuperSubscriptStyle &&
                                        fContainsTextBoldfaceStyle == other.fContainsTextBoldfaceStyle &&
                                        fContainsTextItalicStyle == other.fContainsTextItalicStyle &&
                                        fContainsTextOutlineStyle == other.fContainsTextOutlineStyle &&
                                        fContainsTextStrikethroughStyle == other.fContainsTextStrikethroughStyle &&
                                        fContainsTextUnderlineStyle == other.fContainsTextUnderlineStyle &&
                                        fContainsTextUneditableStyle == other.fContainsTextUneditableStyle &&
                                        fContainsTextPointSizeStyle == other.fContainsTextPointSizeStyle &&
                                        fContainsTextColorStyle == other.fContainsTextColorStyle &&
                                        fContainsTextTypefaceStyle == other.fContainsTextTypefaceStyle &&
                                        fContainsParagraphStartIndentStyle == other.fContainsParagraphStartIndentStyle &&
                                        fContainsParagraphEndIndentStyle == other.fContainsParagraphEndIndentStyle &&
                                        fContainsParagraphFirstIndentStyle == other.fContainsParagraphFirstIndentStyle &&
                                        fContainsParagraphJustificationStyle == other.fContainsParagraphJustificationStyle &&
                                        fContainsParagraphSpreadStyle == other.fContainsParagraphSpreadStyle &&

                                        fTextSuperSubscriptStyleValue == other.fTextSuperSubscriptStyleValue &&
                                        fTextBoldfaceStyleValue == other.fTextBoldfaceStyleValue &&
                                        fTextItalicStyleValue == other.fTextItalicStyleValue &&
                                        fTextOutlineStyleValue == other.fTextOutlineStyleValue &&
                                        fTextStrikethroughStyleValue == other.fTextStrikethroughStyleValue &&
                                        fTextUnderlineStyleValue == other.fTextUnderlineStyleValue &&
                                        fTextUneditableStyleValue == other.fTextUneditableStyleValue &&
                                        fTextPointSizeStyleValue == other.fTextPointSizeStyleValue &&
                                        fTextColorStyleValue == other.fTextColorStyleValue &&
                                        fTextTypefaceStyleValue == other.fTextTypefaceStyleValue &&
                                        fParagraphStartIndentStyleValue == other.fParagraphStartIndentStyleValue &&
                                        fParagraphEndIndentStyleValue == other.fParagraphEndIndentStyleValue &&
                                        fParagraphFirstIndentStyleValue == other.fParagraphFirstIndentStyleValue &&
                                        fParagraphJustificationStyleValue == other.fParagraphJustificationStyleValue &&
                                        fParagraphSpreadStyleValue == other.fParagraphSpreadStyleValue;
}



bool
ITextStyleSet::operator!=( const ITextStyleSet& other ) const
{
        return !( operator==( other ) );
}



ITextStyleSet&
ITextStyleSet::operator+=( const ITextStyleSet& other )
{
        if ( other.fContainsTextSuperSubscriptStyle ) {
                fContainsTextSuperSubscriptStyle = true;
                fTextSuperSubscriptStyleValue = other.fTextSuperSubscriptStyleValue;
                }
        if ( other.fContainsTextBoldfaceStyle ) {
                fContainsTextBoldfaceStyle = true;
                fTextBoldfaceStyleValue = other.fTextBoldfaceStyleValue;
                }
        if ( other.fContainsTextItalicStyle ) {
                fContainsTextItalicStyle = true;
                fTextItalicStyleValue = other.fTextItalicStyleValue;
                }
        if ( other.fContainsTextOutlineStyle ) {
                fContainsTextOutlineStyle = true;
                fTextOutlineStyleValue = other.fTextOutlineStyleValue;
                }
        if ( other.fContainsTextStrikethroughStyle ) {
                fContainsTextStrikethroughStyle = true;
                fTextStrikethroughStyleValue = other.fTextStrikethroughStyleValue;
                }
        if ( other.fContainsTextUnderlineStyle ) {
                fContainsTextUnderlineStyle = true;
                fTextUnderlineStyleValue = other.fTextUnderlineStyleValue;
                }
        if ( other.fContainsTextUneditableStyle ) {
                fContainsTextUneditableStyle = true;
                fTextUneditableStyleValue = other.fTextUneditableStyleValue;
                }
        if ( other.fContainsTextPointSizeStyle ) {
                fContainsTextPointSizeStyle = true;
                fTextPointSizeStyleValue = other.fTextPointSizeStyleValue;
                }
        if ( other.fContainsTextColorStyle ) {
                fContainsTextColorStyle = true;
                fTextColorStyleValue = other.fTextColorStyleValue;
                }
        if ( other.fContainsTextTypefaceStyle ) {
                fContainsTextTypefaceStyle = true;
                fTextTypefaceStyleValue = other.fTextTypefaceStyleValue;
                }
        if ( other.fContainsParagraphStartIndentStyle ) {
                fContainsParagraphStartIndentStyle = true;
                fParagraphStartIndentStyleValue = other.fParagraphStartIndentStyleValue;
                }
        if ( other.fContainsParagraphEndIndentStyle ) {
                fContainsParagraphEndIndentStyle = true;
                fParagraphEndIndentStyleValue = other.fParagraphEndIndentStyleValue;
                }
        if ( other.fContainsParagraphFirstIndentStyle ) {
                fContainsParagraphFirstIndentStyle = true;
                fParagraphFirstIndentStyleValue = other.fParagraphFirstIndentStyleValue;
                }
        if ( other.fContainsParagraphJustificationStyle ) {
                fContainsParagraphJustificationStyle = true;
                fParagraphJustificationStyleValue = other.fParagraphJustificationStyleValue;
                }
        if ( other.fContainsParagraphSpreadStyle ) {
                fContainsParagraphSpreadStyle = true;
                fParagraphSpreadStyleValue = other.fParagraphSpreadStyleValue;
                }
        return *this;
}


ITextStyleSet&
ITextStyleSet::operator+=( const ITextStyle& style )
{
        switch( style.styleClass() ) {
                case ITextStyle::kTextTypefaceStyle                     :       fContainsTextTypefaceStyle = true;
                                                                                                                fTextTypefaceStyleValue = ((const ITextTypefaceStyle&) style).typeface();
                                                                                                                break;
                case ITextStyle::kTextPointSizeStyle            :       fContainsTextPointSizeStyle = true;
                                                                                                                fTextPointSizeStyleValue = ((const ITextPointSizeStyle&) style).pointSize();
                                                                                                                break;
                case ITextStyle::kTextBoldfaceStyle                     :       fContainsTextBoldfaceStyle = true;
                                                                                                                fTextBoldfaceStyleValue = ((const ITextBoldfaceStyle&) style).isBoldface();
                                                                                                                break;
                case ITextStyle::kTextItalicStyle                       :       fContainsTextItalicStyle = true;
                                                                                                                fTextItalicStyleValue = ((const ITextItalicStyle&) style).isItalic();
                                                                                                                break;
                case ITextStyle::kTextUnderlineStyle            :       fContainsTextUnderlineStyle = true;
                                                                                                                fTextUnderlineStyleValue = ((const ITextUnderlineStyle&) style).isUnderlined();
                                                                                                                break;
                case ITextStyle::kTextStrikethroughStyle        :       fContainsTextStrikethroughStyle = true;
                                                                                                                fTextStrikethroughStyleValue = ((const ITextStrikethroughStyle&) style).isStrikethrough();
                                                                                                                break;
                case ITextStyle::kTextOutlineStyle                      :       fContainsTextOutlineStyle = true;
                                                                                                                fTextOutlineStyleValue = ((const ITextOutlineStyle&) style).isOutlined();
                                                                                                                break;
                case ITextStyle::kTextUneditableStyle           :       fContainsTextUneditableStyle = true;
                                                                                                                fTextUneditableStyleValue = ((const ITextUneditableStyle&) style).isUneditable();
                                                                                                                break;
                case ITextStyle::kTextColorStyle                        :       fContainsTextColorStyle = true;
                                                                                                                fTextColorStyleValue = ((const ITextColorStyle&) style).color();
                                                                                                                break;
                case ITextStyle::kTextSuperSubscriptStyle       :       fContainsTextSuperSubscriptStyle = true;
                                                                                                                fTextSuperSubscriptStyleValue = ((const ITextSuperSubscriptStyle&) style).offset();
                                                                                                                break;
                case ITextStyle::kParagraphJustificationStyle:  fContainsParagraphJustificationStyle = true;
                                                                                                                fParagraphJustificationStyleValue = ((const IParagraphJustificationStyle&) style).justification();
                                                                                                                break;
                case ITextStyle::kParagraphSpreadStyle          :       fContainsParagraphSpreadStyle = true;
                                                                                                                fParagraphSpreadStyleValue = ((const IParagraphSpreadStyle&) style).spread();
                                                                                                                break;
                case ITextStyle::kParagraphStartIndentStyle     :       fContainsParagraphStartIndentStyle = true;
                                                                                                                fParagraphStartIndentStyleValue = ((const IParagraphStartIndentStyle&) style).startIndent();
                                                                                                                break;
                case ITextStyle::kParagraphEndIndentStyle       :       fContainsParagraphEndIndentStyle = true;
                                                                                                                fParagraphEndIndentStyleValue = ((const IParagraphEndIndentStyle&) style).endIndent();
                                                                                                                break;
                case ITextStyle::kParagraphFirstIndentStyle     :       fContainsParagraphFirstIndentStyle = true;
                                                                                                                fParagraphFirstIndentStyleValue = ((const IParagraphFirstIndentStyle&) style).firstIndent();
                                                                                                                break;
                }
        return *this;
}


void
ITextStyleSet::add( const ITextStyleSet& other )
{
        operator+=( other );
}



void
ITextStyleSet::add( const ITextStyle& style )
{
        operator +=( style );
}



ITextStyleSet&
ITextStyleSet::operator-=( const ITextStyleSet& other )
{
        if ( other.fContainsTextSuperSubscriptStyle )
                fContainsTextSuperSubscriptStyle = false;
        if ( other.fContainsTextBoldfaceStyle )
                fContainsTextBoldfaceStyle = false;
        if ( other.fContainsTextItalicStyle )
                fContainsTextItalicStyle = false;
        if ( other.fContainsTextOutlineStyle )
                fContainsTextOutlineStyle = false;
        if ( other.fContainsTextStrikethroughStyle )
                fContainsTextStrikethroughStyle = false;
        if ( other.fContainsTextUnderlineStyle )
                fContainsTextUnderlineStyle = false;
        if ( other.fContainsTextUneditableStyle )
                fContainsTextUneditableStyle = false;
        if ( other.fContainsTextPointSizeStyle )
                fContainsTextPointSizeStyle = false;
        if ( other.fContainsTextColorStyle )
                fContainsTextColorStyle = false;
        if ( other.fContainsTextTypefaceStyle )
                fContainsTextTypefaceStyle = false;
        if ( other.fContainsParagraphStartIndentStyle )
                fContainsParagraphStartIndentStyle = false;
        if ( other.fContainsParagraphEndIndentStyle )
                fContainsParagraphEndIndentStyle = false;
        if ( other.fContainsParagraphFirstIndentStyle )
                fContainsParagraphFirstIndentStyle = false;
        if ( other.fContainsParagraphJustificationStyle )
                fContainsParagraphJustificationStyle = false;
        if ( other.fContainsParagraphSpreadStyle )
                fContainsParagraphSpreadStyle = false;
        return *this;
}



ITextStyleSet&
ITextStyleSet::operator-=( const ITextStyle& style )
{
        return operator-=( ITextStyleSet( style ) );
}



void
ITextStyleSet::remove( const ITextStyleSet& other )
{
        operator-=( other );
}



void
ITextStyleSet::remove( const ITextStyle& style )
{
        operator-=( style );
}


void
ITextStyleSet::removeAll()
{
        fContainsTextSuperSubscriptStyle = false;
        fContainsTextBoldfaceStyle = false;
        fContainsTextItalicStyle = false;
        fContainsTextOutlineStyle = false;
        fContainsTextStrikethroughStyle = false;
        fContainsTextUnderlineStyle = false;
        fContainsTextUneditableStyle = false;
        fContainsTextPointSizeStyle = false;
        fContainsTextColorStyle = false;
        fContainsTextTypefaceStyle = false;
        fContainsParagraphStartIndentStyle = false;
        fContainsParagraphEndIndentStyle = false;
        fContainsParagraphFirstIndentStyle = false;
        fContainsParagraphJustificationStyle = false;
        fContainsParagraphSpreadStyle = false;
}



bool
ITextStyleSet::contains( const ITextStyle& style ) const
{
        bool returnValue = false;
        switch( style.styleClass() ) {
                case ITextStyle::kTextTypefaceStyle :                   returnValue = fContainsTextTypefaceStyle; break;
                case ITextStyle::kTextPointSizeStyle :                  returnValue = fContainsTextPointSizeStyle; break;
                case ITextStyle::kTextBoldfaceStyle :                   returnValue = fContainsTextBoldfaceStyle; break;
                case ITextStyle::kTextItalicStyle :                             returnValue = fContainsTextItalicStyle; break;
                case ITextStyle::kTextUnderlineStyle :                  returnValue = fContainsTextUnderlineStyle; break;
                case ITextStyle::kTextStrikethroughStyle :              returnValue = fContainsTextStrikethroughStyle; break;
                case ITextStyle::kTextOutlineStyle :                    returnValue = fContainsTextOutlineStyle; break;
                case ITextStyle::kTextUneditableStyle :                 returnValue = fContainsTextUneditableStyle; break;
                case ITextStyle::kTextColorStyle :                              returnValue = fContainsTextColorStyle; break;
                case ITextStyle::kTextSuperSubscriptStyle :             returnValue = fContainsTextSuperSubscriptStyle; break;
                case ITextStyle::kParagraphJustificationStyle : returnValue = fContainsParagraphJustificationStyle; break;
                case ITextStyle::kParagraphSpreadStyle :                returnValue = fContainsParagraphSpreadStyle; break;
                case ITextStyle::kParagraphStartIndentStyle :   returnValue = fContainsParagraphStartIndentStyle; break;
                case ITextStyle::kParagraphEndIndentStyle :     returnValue = fContainsParagraphEndIndentStyle; break;
                case ITextStyle::kParagraphFirstIndentStyle :   returnValue = fContainsParagraphFirstIndentStyle; break;
                }
        return returnValue;
}



bool
ITextStyleSet::extract( ITextStyle& style ) const
{
        bool returnValue = false;
        switch( style.styleClass() ) {
                case ITextStyle::kTextTypefaceStyle :                   returnValue = fContainsTextTypefaceStyle;
                                                                                                                (ITextTypefaceStyle&) style = ITextTypefaceStyle( fTextTypefaceStyleValue );
                                                                                                                break;
                case ITextStyle::kTextPointSizeStyle :                  returnValue = fContainsTextPointSizeStyle;
                                                                                                                (ITextPointSizeStyle&) style = ITextPointSizeStyle( fTextPointSizeStyleValue );
                                                                                                                break;
                case ITextStyle::kTextBoldfaceStyle :                   returnValue = fContainsTextBoldfaceStyle;
                                                                                                                (ITextBoldfaceStyle&) style = ITextBoldfaceStyle( fTextBoldfaceStyleValue );
                                                                                                                break;
                case ITextStyle::kTextItalicStyle :                             returnValue = fContainsTextItalicStyle;
                                                                                                                (ITextItalicStyle&) style = ITextItalicStyle( fTextItalicStyleValue );
                                                                                                                break;
                case ITextStyle::kTextUnderlineStyle :                  returnValue = fContainsTextUnderlineStyle;
                                                                                                                (ITextUnderlineStyle&) style = ITextUnderlineStyle( fTextUnderlineStyleValue );
                                                                                                                break;
                case ITextStyle::kTextStrikethroughStyle :              returnValue = fContainsTextStrikethroughStyle;
                                                                                                                (ITextStrikethroughStyle&) style = ITextStrikethroughStyle( fTextStrikethroughStyleValue );
                                                                                                                break;
                case ITextStyle::kTextOutlineStyle :                    returnValue = fContainsTextOutlineStyle;
                                                                                                                (ITextOutlineStyle&) style = ITextOutlineStyle( fTextOutlineStyleValue );
                                                                                                                break;
                case ITextStyle::kTextUneditableStyle :                 returnValue = fContainsTextUneditableStyle;
                                                                                                                (ITextUneditableStyle&) style = ITextUneditableStyle( fTextUneditableStyleValue );
                                                                                                                break;
                case ITextStyle::kTextColorStyle :                              returnValue = fContainsTextColorStyle;
                                                                                                                (ITextColorStyle&) style = ITextColorStyle( fTextColorStyleValue );
                                                                                                                break;
                case ITextStyle::kTextSuperSubscriptStyle :             returnValue = fContainsTextSuperSubscriptStyle;
                                                                                                                (ITextSuperSubscriptStyle&) style = ITextSuperSubscriptStyle( fTextSuperSubscriptStyleValue );
                                                                                                                break;
                case ITextStyle::kParagraphJustificationStyle : returnValue = fContainsParagraphJustificationStyle;
                                                                                                                (IParagraphJustificationStyle&) style = IParagraphJustificationStyle( fParagraphJustificationStyleValue );
                                                                                                                break;
                case ITextStyle::kParagraphSpreadStyle :                returnValue = fContainsParagraphSpreadStyle;
                                                                                                                (IParagraphSpreadStyle&) style = IParagraphSpreadStyle( fParagraphSpreadStyleValue );
                                                                                                                break;
                case ITextStyle::kParagraphStartIndentStyle :   returnValue = fContainsParagraphStartIndentStyle;
                                                                                                                (IParagraphStartIndentStyle&) style = IParagraphStartIndentStyle( fParagraphStartIndentStyleValue );
                                                                                                                break;
                case ITextStyle::kParagraphEndIndentStyle :     returnValue = fContainsParagraphEndIndentStyle;
                                                                                                                (IParagraphEndIndentStyle&) style = IParagraphEndIndentStyle( fParagraphEndIndentStyleValue );
                                                                                                                break;
                case ITextStyle::kParagraphFirstIndentStyle :   returnValue = fContainsParagraphFirstIndentStyle;
                                                                                                                (IParagraphFirstIndentStyle&) style = IParagraphFirstIndentStyle( fParagraphFirstIndentStyleValue );
                                                                                                                break;
                }
        return returnValue;
}


