// Revision: 71 1.6.1.4 source/core/text/styles/istylset.hpp, text, ioc.v400, 000919 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*                                                                                       *
*****************************************************************************************
*/
#ifndef _ISTYLSET_
#define _ISTYLSET_

#include <istyle.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

class ITextStyleSet {

        public:

                                                ITextStyleSet();
                                                ITextStyleSet( const ITextStyle& );
                                                ITextStyleSet( const ITextStyleSet& );

        virtual                         ~ITextStyleSet();

                        ITextStyleSet&          operator=( const ITextStyleSet& );
                        ITextStyleSet&          operator=( const ITextStyle& );

                        ITextStyleSet           operator+( const ITextStyleSet& ) const;
                        ITextStyleSet           operator+( const ITextStyle& ) const;

                        ITextStyleSet           operator-( const ITextStyleSet& ) const;
                        ITextStyleSet           operator-( const ITextStyle& ) const;

        virtual bool                    operator==( const ITextStyleSet& ) const;
        virtual bool                    operator!=( const ITextStyleSet& ) const;

        virtual ITextStyleSet&  operator+=( const ITextStyleSet& );
        virtual ITextStyleSet&  operator+=( const ITextStyle& );
        virtual void                    add( const ITextStyleSet& );
        virtual void                    add( const ITextStyle& );

        virtual ITextStyleSet&  operator-=( const ITextStyleSet& );
        virtual ITextStyleSet&  operator-=( const ITextStyle& );
        virtual void                    remove( const ITextStyleSet& );
        virtual void                    remove( const ITextStyle& );

        virtual void                    removeAll();

        virtual bool                    contains( const ITextStyle& ) const;
        virtual bool                    extract( ITextStyle& ) const;

        private:

        bool                                                    fContainsTextSuperSubscriptStyle : 1;
        bool                                                    fContainsTextBoldfaceStyle : 1;
        bool                                                    fContainsTextItalicStyle : 1;
        bool                                                    fContainsTextOutlineStyle : 1;
        bool                                                    fContainsTextStrikethroughStyle : 1;
        bool                                                    fContainsTextUnderlineStyle : 1;
        bool                                                    fContainsTextUneditableStyle : 1;
        bool                                                    fContainsTextPointSizeStyle : 1;
        bool                                                    fContainsTextColorStyle : 1;
        bool                                                    fContainsTextTypefaceStyle : 1;

        bool                                                    fContainsParagraphStartIndentStyle : 1;
        bool                                                    fContainsParagraphEndIndentStyle : 1;
        bool                                                    fContainsParagraphFirstIndentStyle : 1;
        bool                                                    fContainsParagraphJustificationStyle : 1;
        bool                                                    fContainsParagraphSpreadStyle : 1;

        bool                                                    fTextBoldfaceStyleValue : 1;
        bool                                                    fTextItalicStyleValue : 1;
        bool                                                    fTextOutlineStyleValue : 1;
        bool                                                    fTextStrikethroughStyleValue : 1;
        bool                                                    fTextUnderlineStyleValue : 1;
        bool                                                    fTextUneditableStyleValue : 1;

        IText                                                   fTextTypefaceStyleValue;
        double                                          fTextPointSizeStyleValue;
        IColor                                          fTextColorStyleValue;
        double                                          fTextSuperSubscriptStyleValue;

        double                                          fParagraphStartIndentStyleValue;
        double                                          fParagraphEndIndentStyleValue;
        double                                          fParagraphFirstIndentStyleValue;
        IParagraphJustificationStyle::EParagraphJustification
                                                                        fParagraphJustificationStyleValue;
        IParagraphSpreadStyle::EParagraphSpread
                                                                        fParagraphSpreadStyleValue;

};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTYLSET_

