// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1992, 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//

// ilonglng.inl
//
//      Implementation of ILongLong inline methods and global functions
//
// Revision: 56 1.7.1.3 source/core/utils/ilonglng.inl, utilities, ioc.v400, 000919 
//
#ifndef _ILONGLNG_INL_
#define _ILONGLNG_INL_

//      This type is ONLY needed with third-party compilers that do not support
//      VisualAge's "long long" extension.
//#ifndef __IBMCPP__
#if !defined(__IBMCPP__) || defined(__OS2__)

#include <limits.h>

/************************************************************************/
/*      PRIVATE FIELD ACCESS methods                                    */
/************************************************************************/
inline long ILongLong::GetMostSignificant() const
{
        return fMostSignificant;
}

inline unsigned long ILongLong::GetLeastSignificant() const
{
        return fLeastSignificant;
}

/************************************************************************/
/*      CONSTRUCTORs                                                    */
/************************************************************************/

inline ILongLong::ILongLong (long mostSignificant, unsigned long leastSignificant)
{
        fLeastSignificant = leastSignificant;
        fMostSignificant = mostSignificant;
}

inline ILongLong::ILongLong (unsigned long x)
{
        fLeastSignificant = x;
        fMostSignificant = 0;
}

inline ILongLong::ILongLong (long x)
{
        fLeastSignificant = x;
        fMostSignificant = (x < 0) ? -1 : 0;
}

inline ILongLong::ILongLong (unsigned int x)
{
        fLeastSignificant = x;
        fMostSignificant = 0;
}

inline ILongLong::ILongLong (int x)
{
        fLeastSignificant = x;
        fMostSignificant = (x < 0) ? -1 : 0;
}

/************************************************************************/
/*      ILongLong       Max and Min                                     */
/************************************************************************/

inline  ILongLong ILongLong::ILongLongMax (void) {      //largest ILongLong
        return ILongLong(LONG_MAX, ULONG_MAX);
}

inline  ILongLong ILongLong::ILongLongMin(void) {       //smallest ILongLong
        return ILongLong(LONG_MIN, 0);
}

/************************************************************************/
/*                      HASH                                            */
/************************************************************************/

inline HashResult ILongLong::Hash() const
{
        return ((unsigned long)fMostSignificant ^ fLeastSignificant) ;
}

/************************************************************************/
/* Increment operators                                                  */
/************************************************************************/

inline ILongLong ILongLong::operator++ (int)
{
   ILongLong temp = *this;
   if (++fLeastSignificant == 0) ++fMostSignificant;
   return(temp);
}

inline ILongLong& ILongLong::operator++ ()
{
   if (++fLeastSignificant == 0) ++fMostSignificant;
   return(*this);
}
/************************************************************************/
/* Decrement operators                                                  */
/************************************************************************/

inline ILongLong ILongLong::operator-- (int)
{
   ILongLong temp = *this;
   if (--fLeastSignificant == ULONG_MAX) --fMostSignificant;
   return(temp);
}

inline ILongLong& ILongLong::operator-- ()
{
   if (--fLeastSignificant == ULONG_MAX) --fMostSignificant;
   return(*this);
}
/************************************************************************/
/*      Assignment ADD operator                                         */
/************************************************************************/
inline ILongLong& ILongLong::operator+=(const ILongLong& x)
{
        fMostSignificant += x.fMostSignificant;
        if ((fLeastSignificant += x.fLeastSignificant) < x.fLeastSignificant)
                fMostSignificant++;
        return(*this);
}


/************************************************************************/
/*      Assignment SUBTRACT operator                                    */
/************************************************************************/
inline ILongLong& ILongLong::operator-=(const ILongLong& x)
{
        if(fLeastSignificant<x.fLeastSignificant) fMostSignificant -= x.fMostSignificant + 1;
        else fMostSignificant -= x.fMostSignificant;
        fLeastSignificant -= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      Assignmenti bitwise inclusive OR operator                       */
/************************************************************************/
inline ILongLong& ILongLong::operator|=(const ILongLong& x)
{
        fMostSignificant |= x.fMostSignificant;
        fLeastSignificant |= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      Assignment bitwise AND operator                                 */
/************************************************************************/
inline ILongLong& ILongLong::operator&=(const ILongLong& x)
{
        fMostSignificant &= x.fMostSignificant;
        fLeastSignificant &= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      Assignment bitwise exclusive XOR operator                       */
/************************************************************************/
inline ILongLong& ILongLong::operator^=(const ILongLong& x)
{
        fMostSignificant ^= x.fMostSignificant;
        fLeastSignificant ^= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      global method for unary MINUS Operator                          */
/************************************************************************/
inline ILongLong        operator-(const ILongLong& x)
{
        if((~ x.GetLeastSignificant()) + 1)
                return ILongLong( ~ x.GetMostSignificant(), (~ x.GetLeastSignificant()) + 1);
        else    return ILongLong( (~ x.GetMostSignificant()) + 1, (~ x.GetLeastSignificant()) + 1);
}
/************************************************************************/
/*      global method for unary ONE'S COMPLEMENT Operator               */
/************************************************************************/
inline ILongLong        operator~(const ILongLong& x)
{
        return ILongLong(~x.GetMostSignificant(), ~x.GetLeastSignificant());
}

/************************************************************************/
/*      global method for logical NEGATION Operator                     */
/************************************************************************/
inline bool             operator!(const ILongLong& x)
{
        return (x.GetMostSignificant()==0L && x.GetLeastSignificant()==0L);
}

/************************************************************************/
/*      global method for binary ADD Operator                           */
/************************************************************************/
inline ILongLong operator+(const ILongLong& x, const ILongLong& y)
{
        ILongLong result = x;
        result += y;
        return result;
}

inline ILongLong  operator+(const ILongLong& x, long a)
{
        ILongLong result = x;
        result += a;
        return result;
}

inline ILongLong  operator+(const ILongLong& x, unsigned long a)
{
        ILongLong result = x;
        result += a;
        return result;
}

inline ILongLong  operator+(const ILongLong& x, int a)
{
        ILongLong result = x;
        result += a;
        return result;
}

inline ILongLong  operator+(const ILongLong& x, unsigned int a)
{
        ILongLong result = x;
        result += a;
        return result;
}

inline ILongLong  operator+(long a, const ILongLong& x)
{
        return (x+a);                                   // addition is commutative
}

inline ILongLong  operator+(unsigned long a, const ILongLong& x)
{
        return (x+a);                                   // addition is commutative
}

inline ILongLong  operator+(int a, const ILongLong& x)
{
        return (x+a);                                   // addition is commutative
}

inline ILongLong  operator+(unsigned int a, const ILongLong& x)
{
        return (x+a);                                   // addition is commutative
}

/************************************************************************/
/*      global method for binary SUBTRACT Operator                      */
/************************************************************************/
inline ILongLong operator-(const ILongLong& x, const ILongLong& y)
{
        ILongLong result = x;
        result -= y;
        return result;
}

inline ILongLong  operator-(const ILongLong& x, long a)
{
        ILongLong result = x;
        result -= a;
        return result;
}

inline ILongLong  operator-(const ILongLong& x, unsigned long a)
{
        ILongLong result = x;
        result -= a;
        return result;
}

inline ILongLong  operator-(const ILongLong& x, int a)
{
        ILongLong result = x;
        result -= a;
        return result;
}

inline ILongLong  operator-(const ILongLong& x, unsigned int a)
{
        ILongLong result = x;
        result -= a;
        return result;
}

inline ILongLong  operator-(long a, const ILongLong& x)
{
        return (- x + a);
}

inline ILongLong  operator-(unsigned long a, const ILongLong& x)
{
        return (- x + a);
}

inline ILongLong  operator-(int a, const ILongLong& x)
{
        return (- x + a);
}

inline ILongLong  operator-(unsigned int a, const ILongLong& x)
{
        return (- x + a);
}


/**************************************************************************/
/*      global method for binary  == Operator                             */
/**************************************************************************/

inline bool operator== (const ILongLong& x, const ILongLong& y)
{
        return ((x.GetMostSignificant() == y.GetMostSignificant()) &&
                        (x.GetLeastSignificant() == y.GetLeastSignificant()));
}

inline bool operator== (const ILongLong& x, unsigned long a)
{
        return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
}

inline bool operator== (const ILongLong& x, long a)
{
        if(a>=0) return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
        else return ((x.GetMostSignificant()==-1) && (a == x.GetLeastSignificant()));
}


inline bool operator== (const ILongLong& x, unsigned int a)
{
        return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
}

inline bool operator== (const ILongLong& x, int a)
{
        if(a>=0) return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
        else return ((x.GetMostSignificant()==-1) && (a == x.GetLeastSignificant()));
}

inline bool operator== (unsigned long a, const ILongLong& x)
{
        return (x==a);
}

inline bool operator== (long a, const ILongLong& x)
{
        return (x==a);
}

inline bool operator== (unsigned int a, const ILongLong& x)
{
        return (x==a);
}

inline bool operator== (int a, const ILongLong& x)
{
        return (x==a);
}

/**************************************************************************/
/*      global method for binary  != Operator                             */
/**************************************************************************/


inline bool operator!= (const ILongLong& x, const ILongLong& y)
{
        return !(x==y);
}

inline bool operator!= (const ILongLong& x, unsigned long a)
{
        return !(x==a);
}

inline bool operator!= (const ILongLong& x, long a)
{
        return !(x==a);
}

inline bool operator!= (const ILongLong& x, unsigned int a)
{
        return !(x==a);
}

inline bool operator!= (const ILongLong& x, int a)
{
        return !(x==a);
}

inline bool operator!= (unsigned long a, const ILongLong& x)
{
        return !(x==a);
}

inline bool operator!= (long a, const ILongLong& x)
{
        return !(x==a);
}

inline bool operator!= (unsigned int a, const ILongLong& x)
{
        return !(x==a);
}

inline bool operator!= (int a, const ILongLong& x)
{
        return !(x==a);
}


/**************************************************************************/
/*      global method for binary  >  Operator                             */
/**************************************************************************/


inline bool operator> (const ILongLong& x, const ILongLong& y)
{
        long x1 = x.GetMostSignificant();
        long y1 = y.GetMostSignificant();
        return ((x1 > y1) || ((x1 == y1) && (x.GetLeastSignificant() > y.GetLeastSignificant())) );
}

inline bool operator> (unsigned long a, const ILongLong& x)
{
        long x1 = x.GetMostSignificant();
        return ((x1 < 0) || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}

inline bool operator> (long a, const ILongLong& x)
{
        long x1 = x.GetMostSignificant();
        if(a < 0)       return ((x1 < -1) || ((x1 == -1) && (a > x.GetLeastSignificant())) );
        else            return ((x1 < 0)  || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}

inline bool operator> (unsigned int a, const ILongLong& x)
{
        long x1 = x.GetMostSignificant();
        return ((x1 < 0) || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}

inline bool operator> (int a, const ILongLong& x)
{
        long x1 = x.GetMostSignificant();
        if(a < 0)       return ((x1 < -1) || ((x1 == -1) && (a > x.GetLeastSignificant())) );
        else            return ((x1 < 0)  || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}


inline bool operator> (const ILongLong& x, unsigned long a)
{
        long x1 = x.GetMostSignificant();
        return ((x1 > 0) || ((x1 == 0) && (a < x.GetLeastSignificant())) );
}

inline bool operator> (const ILongLong& x, long a)
{
        long x1 = x.GetMostSignificant();
        if (a >=0) return ((x1 > 0) || ((x1 == 0) && (x.GetLeastSignificant() > a)) );
        else return ((x1 > -1) || ((x1 == -1) && (x.GetLeastSignificant() > a)) );
}

inline bool operator> (const ILongLong& x, unsigned int a)
{
        long x1 = x.GetMostSignificant();
        return ((x1 > 0) || ((x1 == 0) && (a < x.GetLeastSignificant())) );
}

inline bool operator> (const ILongLong& x, int a)
{
        long x1 = x.GetMostSignificant();
        if (a >=0) return ((x1 > 0) || ((x1 == 0) && (x.GetLeastSignificant() > a)) );
        else return ((x1 > -1) || ((x1 == -1) && (x.GetLeastSignificant() > a)) );
}

/**************************************************************************/
/*      global method for binary  <  Operator                             */
/**************************************************************************/


inline bool operator< (const ILongLong& x, const ILongLong& y)
{
        return (y>x);
}

inline bool operator< (unsigned long a, const ILongLong& x)
{
        return (x>a);
}

inline bool operator< (long a, const ILongLong& x)
{
        return (x>a);
}

inline bool operator< (unsigned int a, const ILongLong& x)
{
        return (x>a);
}

inline bool operator< (int a, const ILongLong& x)
{
        return (x>a);
}

inline bool operator< (const ILongLong& x, unsigned long a)
{
        return (a>x);
}

inline bool operator< (const ILongLong& x, long a)
{
        return (a>x);
}

inline bool operator< (const ILongLong& x, unsigned int a)
{
        return (a>x);
}

inline bool operator< (const ILongLong& x, int a)
{
        return (a>x);
}

/**************************************************************************/
/*      global method for binary  >= Operator                             */
/**************************************************************************/

inline bool     operator>= (const ILongLong& x, const ILongLong& y)
{
        return !(y > x);
}

inline bool     operator>= (const ILongLong& x, unsigned long a)
{
        return !(x<a);
}

inline bool     operator>= (const ILongLong& x, long a)
{
        return !(x<a);
}

inline bool     operator>= (const ILongLong& x, unsigned int a)
{
        return !(x<a);
}

inline bool     operator>= (const ILongLong& x, int a)
{
        return !(x<a);
}

inline bool     operator>= (unsigned long a, const ILongLong& x)
{
        return !(x>a);
}

inline bool     operator>= (long a, const ILongLong& x)
{
        return !(x>a);
}

inline bool     operator>= (unsigned int a, const ILongLong& x)
{
        return !(x>a);
}

inline bool     operator>= (int a, const ILongLong& x)
{
        return !(x>a);
}

/**************************************************************************/
/*      global method for binary  <= Operator                             */
/**************************************************************************/

inline bool     operator<= (const ILongLong& x, const ILongLong& y)
{
        return !(x>y);
}

inline bool     operator<= (const ILongLong& x, unsigned long a)
{
        return !(x>a);
}

inline bool     operator<= (const ILongLong& x, long a)
{
        return !(x>a);
}

inline bool     operator<= (const ILongLong& x, unsigned int a)
{
        return !(x>a);
}

inline bool     operator<= (const ILongLong& x, int a)
{
        return !(x>a);
}

inline bool     operator<= (unsigned long a, const ILongLong& x)
{
        return !(x<a);
}

inline bool     operator<= (long a, const ILongLong& x)
{
        return !(x<a);
}
inline bool     operator<= (unsigned int a, const ILongLong& x)
{
        return !(x<a);
}

inline bool     operator<= (int a, const ILongLong& x)
{
        return !(x<a);
}


/************************************************************************/
/*      global method for binary logical OR Operator                    */
/************************************************************************/
inline bool operator||(const ILongLong& x, const ILongLong& y)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant()
        || y.GetMostSignificant() || y.GetLeastSignificant() );
}

inline bool operator||(const ILongLong& x, unsigned long a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(const ILongLong& x, long a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(const ILongLong& x, unsigned int a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(const ILongLong& x, int a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(unsigned long a, const ILongLong& x)
{
        return (x || a);
}

inline bool operator||(long a, const ILongLong& x)
{
        return (x || a);
}

inline bool operator||(unsigned int a, const ILongLong& x)
{
        return (x || a);
}

inline bool operator||(int a, const ILongLong& x)
{
        return (x || a);
}


/************************************************************************/
/*      global method for binary logical AND Operator                   */
/************************************************************************/
inline bool operator&&(const ILongLong& x, const ILongLong& y)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant())
        && (y.GetMostSignificant() || y.GetLeastSignificant()));
}

inline bool operator&&(const ILongLong& x, unsigned long a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(const ILongLong& x, long a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(const ILongLong& x, unsigned int a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(const ILongLong& x, int a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(unsigned long a, const ILongLong& x)
{
        return (x && a);
}

inline bool operator&&(long a, const ILongLong& x)
{
        return (x && a);
}

inline bool operator&&(unsigned int a, const ILongLong& x)
{
        return (x && a);
}

inline bool operator&&(int a, const ILongLong& x)
{
        return (x && a);
}

/************************************************************************/
/*      global method for binary bitwise OR Operator                    */
/************************************************************************/
inline ILongLong operator|(const ILongLong& x, const ILongLong& y)
{
        return(ILongLong(x.GetMostSignificant() | y.GetMostSignificant(),
                        x.GetLeastSignificant() | y.GetLeastSignificant()));
}

inline ILongLong operator|(const ILongLong& x, unsigned long a)
{
        return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline ILongLong operator|(const ILongLong& x, long a)
{
        if (a < 0)  return (ILongLong( -1, x.GetLeastSignificant() | a));
        else return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline ILongLong operator|(const ILongLong& x, unsigned int a)
{
        return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline ILongLong operator|(const ILongLong& x, int a)
{
        if (a < 0)  return (ILongLong( -1, x.GetLeastSignificant() | a));
        else return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline ILongLong operator|(unsigned long a, const ILongLong& x)
{
        return (x | a);
}

inline ILongLong operator|(long a, const ILongLong& x)
{
        return (x | a);
}

inline ILongLong operator|(unsigned int a, const ILongLong& x)
{
        return (x | a);
}

inline ILongLong operator|(int a, const ILongLong& x)
{
        return (x | a);
}


/************************************************************************/
/*      global method for binary bitwise AND Operator                   */
/************************************************************************/
inline ILongLong operator& (const ILongLong& x, const ILongLong& y)
{
        return(ILongLong(x.GetMostSignificant() & y.GetMostSignificant(),
                        x.GetLeastSignificant() & y.GetLeastSignificant()));
}

inline ILongLong operator&(const ILongLong& x, unsigned long a)
{
        return (ILongLong(0, x.GetLeastSignificant() & a));
}

inline ILongLong operator&(const ILongLong& x, long a)
{
        if ( a < 0) return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() & a));
        else return (ILongLong(0, x.GetLeastSignificant() & a));
}

inline ILongLong operator&(const ILongLong& x, unsigned int a)
{
        return (ILongLong(0, x.GetLeastSignificant() & a));
}

inline ILongLong operator&(const ILongLong& x, int a)
{
        if ( a < 0) return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() & a));
        else return (ILongLong(0, x.GetLeastSignificant() & a));
}


inline ILongLong operator&(unsigned long a, const ILongLong& x)
{
        return (x & a);
}

inline ILongLong operator&(long a, const ILongLong& x)
{
        return (x & a);
}

inline ILongLong operator&(unsigned int a, const ILongLong& x)
{
        return (x & a);
}

inline ILongLong operator&(int a, const ILongLong& x)
{
        return (x & a);
}

/************************************************************************/
/*      global method for binary bitwise XOR Operator                   */
/************************************************************************/
inline ILongLong operator^ (const ILongLong& x, const ILongLong& y)
{
        return(ILongLong(x.GetMostSignificant() ^ y.GetMostSignificant(),
                        x.GetLeastSignificant() ^ y.GetLeastSignificant()));
}

inline ILongLong operator^(const ILongLong& x, unsigned long a)
{
        return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline ILongLong operator^(const ILongLong& x, long a)
{
        if (a < 0 ) return (ILongLong(~x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
        else return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline ILongLong operator^(const ILongLong& x, unsigned int a)
{
        return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline ILongLong operator^(const ILongLong& x, int a)
{
        if (a < 0 ) return (ILongLong(~x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
        else return (ILongLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline ILongLong operator^(unsigned long a, const ILongLong& x)
{
        return (x ^ a);
}

inline ILongLong operator^(long a, const ILongLong& x)
{
        return (x ^ a);
}

inline ILongLong operator^(unsigned int a, const ILongLong& x)
{
        return (x ^ a);
}

inline ILongLong operator^(int a, const ILongLong& x)
{
        return (x ^ a);
}

inline ILongLong::operator long double() const
{
        long double most = (GetMostSignificant() * (2^32));

        if (most < 0 )
        {
                 return most - GetLeastSignificant()  ;
        }
        else
        {
                 return most + GetLeastSignificant()  ;
        }
}

#endif // __IBMCPP__
#endif // _ILONGLNG_INL_
