/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

// Revision: 77 1.4.1.3 source/core/utils/inumutil.cpp, utilities, ioc.v400, 000919 
/*-----------------------------------------------------------------------------------------
FILE:-

        numericutilities.c  --  general utility functions for internal useage


AUTHOR:-

        Sanjay Poonen


MODIFICATION HISTORY:-
         1 Aug  93 - SJP - Created _NaN function
        10 AUG  93 - SJP - Installed the _MULUL and _DIVUL functions
        13 Sep  93 - CLM - Fixed _DIVUL to return both quo _and_ remainder (JPO's fix)
        09 Feb  94 - SJP - Changed __PPC__ to _AIX_ for SDK1, revisit this again later.

--------------------------------------------------------------------------------------------*/
#ifndef Taligent_NUMERICUTILITIES
#include <inumutil.hpp>
#endif

#ifndef Taligent_FPBITDEF
#include <FPBitDef.h>
#endif

#ifdef IC_PAGETUNE
    #define _INUMUTIL_CPP_
    #include <ipagetun.h>
#endif

#ifdef __cplusplus
        extern "C" {
#endif

float   _NaN(unsigned short nanCode)
{
        _FloatOrLong x;
        x._I = 0x7FC00000;                      // quiet nan
        x._I |= nanCode;                        // insert the nan code
        return x._F;
}

#ifdef __cplusplus
        }
#endif


//#if defined(__X86__) || defined(_AIX_)        //implemented in assembler for 68k
/****************************************************************************/
/*      _MULUL emulates the MULUL instruction on the 68020.                                             */
/****************************************************************************/
#ifdef __cplusplus
        extern "C" {
#endif
void _MULUL(unsigned long x, unsigned long y, unsigned long *pmostSig, unsigned long *pleastSig)
{
        unsigned long carry = 0;
        unsigned long cross1, cross2, temp, lowprod;
        unsigned long xMostSigWord = x >> 16u;  // this is a PORTABLE way to access high-word of 32-bit
        unsigned long yMostSigWord = y >> 16u;

        x = (unsigned short) x;                                 // this is a PORTABLE way to access low-word of 32-bit
        y = (unsigned short) y;
        cross1 = xMostSigWord * y ;
        cross2 = x * yMostSigWord ;

        cross1 += cross2;                                               // modifications for double carry problem begin here
        if (cross1 < cross2)
                carry += 0x00010000UL;                          // carry out of cross terms affects middle of high result

        lowprod = x * y;
        temp = lowprod + (cross1 << 16u);               // single low term addition
        if (temp < lowprod)
                carry++;                                                        // carry out of low terms affects low end of high result
        *pleastSig = temp;
        *pmostSig = xMostSigWord * yMostSigWord + (cross1 >> 16) + carry;
}

#ifdef __cplusplus
        }
#endif

//#endif

//#if defined(__X86__) || defined(_AIX_)        //implemented in assembler for 68k
/*
 *      int _DIVUL (unsigned long dvsr, unsigned long *pdvndMostSig, unsigned long *dvndLeastSig)
 *
 *              mimics the MC68020 DIVU.L instruction by performing a 64-bit by 32-bit unsigned
 *              division:  pdvndMostSig:dvndLeastSig / dvsr, yielding 32-bit quotient and 32-bit
 *              remainder.  If no overflow occurs, this function returns 0 and overwrites
 *              pdvndMostSig with the quotient and dvndLeastSig with the remainder.  If overflow
 *              occurs, the function returns 1 and leaves pdvndMostSig and dvndLeastSig unchanged.
 */
#ifdef __cplusplus
        extern "C"{
#endif

int _DIVUL (unsigned long dvsr, unsigned long *pdvndMostSig, unsigned long *pdvndLeastSig)
{
        int i;
        unsigned long quo = 0;
        unsigned long dvndMostSig       = *pdvndMostSig;
        unsigned long dvndLeastSig      = *pdvndLeastSig;

        if (dvsr <= dvndMostSig) return 1;              // overflow will occur
        for (i = 1; i <= 8*sizeof(unsigned long); i++)
        {
                quo <<= 1;
                if ((dvndMostSig & 0x80000000) != 0)
                {
                        quo += 1;
                        dvndMostSig <<= 1;
                        if ((dvndLeastSig & 0x80000000) != 0)
                                (dvndMostSig)++;
                        dvndMostSig -= dvsr;
                }
                else
                {
                        dvndMostSig <<= 1;
                        if ((dvndLeastSig & 0x80000000) != 0)
                                (dvndMostSig)++;
                        if (dvsr <= dvndMostSig)
                        {
                                quo += 1;
                                dvndMostSig -= dvsr;
                        }
                }
                dvndLeastSig <<= 1;
        }
        *pdvndLeastSig = dvndMostSig;
        *pdvndMostSig = quo;
        return 0;
}
#ifdef __cplusplus
        }
#endif

//#endif        // __X86__ or _AIX_




