/* Revision: 79 1.6.1.3 source/core/utils/iprimtpi.h, utilities, ioc.v400, 000919  */
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

/*************************************************************************************

        Implementation of TDoubleLong inline methods and global functions

 *************************************************************************************/

/************************************************************************/
/*                                                                      */
/*  IMPLEMENTATION OF INLINE METHODS.                                   */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#ifndef _IPRIMTPI_
#define _IPRIMTPI_

#include <limits.h>

#ifndef Taligent_PRIMITIVETYPES
#include <Taligent/PrimitiveTypes.h>
#endif

#ifndef _IBASEPL_H
#include <ibasepl.hpp>          // Need the base library
#endif

#ifndef NO_Internal

/************************************************************************/
/*      PRIVATE FIELD ACCESS methods                                    */
/************************************************************************/
inline long TDoubleLong::GetMostSignificant() const
{
        return fMostSignificant;
}

inline unsigned long TDoubleLong::GetLeastSignificant() const
{
        return fLeastSignificant;
}

/************************************************************************/
/*      CONSTRUCTORs                                                    */
/************************************************************************/

inline TDoubleLong::TDoubleLong (long mostSignificant, unsigned long leastSignificant)
{
        fLeastSignificant = leastSignificant;
        fMostSignificant = mostSignificant;
}

inline TDoubleLong::TDoubleLong (unsigned long x)
{
        fLeastSignificant = x;
        fMostSignificant = 0;
}

inline TDoubleLong::TDoubleLong (long x)
{
        fLeastSignificant = x;
        fMostSignificant = (x < 0) ? -1 : 0;
}

inline TDoubleLong::TDoubleLong (unsigned int x)
{
        fLeastSignificant = x;
        fMostSignificant = 0;
}

inline TDoubleLong::TDoubleLong (int x)
{
        fLeastSignificant = x;
        fMostSignificant = (x < 0) ? -1 : 0;
}

/************************************************************************/
/*      TDoubleLong     Max and Min                                     */
/************************************************************************/

inline  TDoubleLong TDoubleLong::TDoubleLongMax (void) {        //largest TDoubleLong
        return TDoubleLong(LONG_MAX, ULONG_MAX);
}

inline  TDoubleLong TDoubleLong::TDoubleLongMin (void) {        //smallest TDoubleLong
        return TDoubleLong(LONG_MIN, 0);
}

/************************************************************************/
/*                      HASH                                            */
/************************************************************************/

inline HashResult TDoubleLong::Hash() const
{
        return ((unsigned long)fMostSignificant ^ fLeastSignificant) ;
}

/************************************************************************/
/* Increment operators                                                  */
/************************************************************************/
inline TDoubleLong TDoubleLong::operator++ (int)
{
        TDoubleLong temp = *this;
   if (++fLeastSignificant == 0) ++fMostSignificant;
   return temp;
}

inline TDoubleLong& TDoubleLong::operator++ ()
{
   if (++fLeastSignificant == 0) ++fMostSignificant;
   return(*this);
}
/************************************************************************/
/* Decrement operators                                                  */
/************************************************************************/
inline TDoubleLong TDoubleLong::operator-- (int)
{
   TDoubleLong temp = *this;
   if (--fLeastSignificant == ULONG_MAX) --fMostSignificant;
   return temp;
}

inline TDoubleLong& TDoubleLong::operator-- ()
{
   if (--fLeastSignificant == ULONG_MAX) --fMostSignificant;
   return(*this);
}

/************************************************************************/
/*      Assignment ADD operator                                         */
/************************************************************************/
inline TDoubleLong& TDoubleLong::operator+=(const TDoubleLong& x)
{
        fMostSignificant += x.fMostSignificant;
        if ((fLeastSignificant += x.fLeastSignificant) < x.fLeastSignificant)
                fMostSignificant++;
        return(*this);
}


/************************************************************************/
/*      Assignment SUBTRACT operator                                    */
/************************************************************************/
inline TDoubleLong& TDoubleLong::operator-=(const TDoubleLong& x)
{
        if(fLeastSignificant<x.fLeastSignificant) fMostSignificant -= x.fMostSignificant + 1;
        else fMostSignificant -= x.fMostSignificant;
        fLeastSignificant -= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      Assignmenti bitwise inclusive OR operator                       */
/************************************************************************/
inline TDoubleLong& TDoubleLong::operator|=(const TDoubleLong& x)
{
        fMostSignificant |= x.fMostSignificant;
        fLeastSignificant |= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      Assignment bitwise AND operator                                 */
/************************************************************************/
inline TDoubleLong& TDoubleLong::operator&=(const TDoubleLong& x)
{
        fMostSignificant &= x.fMostSignificant;
        fLeastSignificant &= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      Assignment bitwise exclusive XOR operator                       */
/************************************************************************/
inline TDoubleLong& TDoubleLong::operator^=(const TDoubleLong& x)
{
        fMostSignificant ^= x.fMostSignificant;
        fLeastSignificant ^= x.fLeastSignificant;
        return(*this);
}

/************************************************************************/
/*      global method for unary MINUS Operator                          */
/************************************************************************/
inline TDoubleLong      operator-(const TDoubleLong& x)
{
        if((~ x.GetLeastSignificant()) + 1)
                return TDoubleLong( ~ x.GetMostSignificant(), (~ x.GetLeastSignificant()) + 1);
        else    return TDoubleLong( (~ x.GetMostSignificant()) + 1, (~ x.GetLeastSignificant()) + 1);
}
/************************************************************************/
/*      global method for unary ONE'S COMPLEMENT Operator               */
/************************************************************************/
inline TDoubleLong      operator~(const TDoubleLong& x)
{
        return TDoubleLong(~x.GetMostSignificant(), ~x.GetLeastSignificant());
}

/************************************************************************/
/*      global method for logical NEGATION Operator                     */
/************************************************************************/
inline bool             operator!(const TDoubleLong& x)
{
        return (x.GetMostSignificant()==0L && x.GetLeastSignificant()==0L);
}

/************************************************************************/
/*      global method for binary ADD Operator                           */
/************************************************************************/
inline TDoubleLong operator+(const TDoubleLong& x, const TDoubleLong& y)
{
        TDoubleLong result = x;
        result += y;
        return result;
}

inline TDoubleLong  operator+(const TDoubleLong& x, long a)
{
        TDoubleLong result = x;
        result += a;
        return result;
}

inline TDoubleLong  operator+(const TDoubleLong& x, unsigned long a)
{
        TDoubleLong result = x;
        result += a;
        return result;
}

inline TDoubleLong  operator+(const TDoubleLong& x, int a)
{
        TDoubleLong result = x;
        result += a;
        return result;
}

inline TDoubleLong  operator+(const TDoubleLong& x, unsigned int a)
{
        TDoubleLong result = x;
        result += a;
        return result;
}

inline TDoubleLong  operator+(long a, const TDoubleLong& x)
{
        return (x+a);                                   // addition is commutative
}

inline TDoubleLong  operator+(unsigned long a, const TDoubleLong& x)
{
        return (x+a);                                   // addition is commutative
}

inline TDoubleLong  operator+(int a, const TDoubleLong& x)
{
        return (x+a);                                   // addition is commutative
}

inline TDoubleLong  operator+(unsigned int a, const TDoubleLong& x)
{
        return (x+a);                                   // addition is commutative
}

/************************************************************************/
/*      global method for binary SUBTRACT Operator                      */
/************************************************************************/
inline TDoubleLong operator-(const TDoubleLong& x, const TDoubleLong& y)
{
        TDoubleLong result = x;
        result -= y;
        return result;
}

inline TDoubleLong  operator-(const TDoubleLong& x, long a)
{
        TDoubleLong result = x;
        result -= a;
        return result;
}

inline TDoubleLong  operator-(const TDoubleLong& x, unsigned long a)
{
        TDoubleLong result = x;
        result -= a;
        return result;
}

inline TDoubleLong  operator-(const TDoubleLong& x, int a)
{
        TDoubleLong result = x;
        result -= a;
        return result;
}

inline TDoubleLong  operator-(const TDoubleLong& x, unsigned int a)
{
        TDoubleLong result = x;
        result -= a;
        return result;
}

inline TDoubleLong  operator-(long a, const TDoubleLong& x)
{
        return (- x + a);
}

inline TDoubleLong  operator-(unsigned long a, const TDoubleLong& x)
{
        return (- x + a);
}

inline TDoubleLong  operator-(int a, const TDoubleLong& x)
{
        return (- x + a);
}

inline TDoubleLong  operator-(unsigned int a, const TDoubleLong& x)
{
        return (- x + a);
}


/**************************************************************************/
/*      global method for binary  == Operator                             */
/**************************************************************************/

inline bool operator== (const TDoubleLong& x, const TDoubleLong& y)
{
        return ((x.GetMostSignificant() == y.GetMostSignificant()) &&
                        (x.GetLeastSignificant() == y.GetLeastSignificant()));
}

inline bool operator== (const TDoubleLong& x, unsigned long a)
{
        return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
}

inline bool operator== (const TDoubleLong& x, long a)
{
        if(a>=0) return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
        else return ((x.GetMostSignificant()==-1) && (a == x.GetLeastSignificant()));
}


inline bool operator== (const TDoubleLong& x, unsigned int a)
{
        return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
}

inline bool operator== (const TDoubleLong& x, int a)
{
        if(a>=0) return (!x.GetMostSignificant() && (a == x.GetLeastSignificant()));
        else return ((x.GetMostSignificant()==-1) && (a == x.GetLeastSignificant()));
}

inline bool operator== (unsigned long a, const TDoubleLong& x)
{
        return (x==a);
}

inline bool operator== (long a, const TDoubleLong& x)
{
        return (x==a);
}

inline bool operator== (unsigned int a, const TDoubleLong& x)
{
        return (x==a);
}

inline bool operator== (int a, const TDoubleLong& x)
{
        return (x==a);
}

/**************************************************************************/
/*      global method for binary  != Operator                             */
/**************************************************************************/


inline bool operator!= (const TDoubleLong& x, const TDoubleLong& y)
{
        return !(x==y);
}

inline bool operator!= (const TDoubleLong& x, unsigned long a)
{
        return !(x==a);
}

inline bool operator!= (const TDoubleLong& x, long a)
{
        return !(x==a);
}

inline bool operator!= (const TDoubleLong& x, unsigned int a)
{
        return !(x==a);
}

inline bool operator!= (const TDoubleLong& x, int a)
{
        return !(x==a);
}

inline bool operator!= (unsigned long a, const TDoubleLong& x)
{
        return !(x==a);
}

inline bool operator!= (long a, const TDoubleLong& x)
{
        return !(x==a);
}

inline bool operator!= (unsigned int a, const TDoubleLong& x)
{
        return !(x==a);
}

inline bool operator!= (int a, const TDoubleLong& x)
{
        return !(x==a);
}


/**************************************************************************/
/*      global method for binary  >  Operator                             */
/**************************************************************************/


inline bool operator> (const TDoubleLong& x, const TDoubleLong& y)
{
        long x1 = x.GetMostSignificant();
        long y1 = y.GetMostSignificant();
        return ((x1 > y1) || ((x1 == y1) && (x.GetLeastSignificant() > y.GetLeastSignificant())) );
}

inline bool operator> (unsigned long a, const TDoubleLong& x)
{
        long x1 = x.GetMostSignificant();
        return ((x1 < 0) || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}

inline bool operator> (long a, const TDoubleLong& x)
{
        long x1 = x.GetMostSignificant();
        if(a < 0)       return ((x1 < -1) || ((x1 == -1) && (a > x.GetLeastSignificant())) );
        else            return ((x1 < 0)  || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}

inline bool operator> (unsigned int a, const TDoubleLong& x)
{
        long x1 = x.GetMostSignificant();
        return ((x1 < 0) || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}

inline bool operator> (int a, const TDoubleLong& x)
{
        long x1 = x.GetMostSignificant();
        if(a < 0)       return ((x1 < -1) || ((x1 == -1) && (a > x.GetLeastSignificant())) );
        else            return ((x1 < 0)  || ((x1 == 0) && (a > x.GetLeastSignificant())) );
}


inline bool operator> (const TDoubleLong& x, unsigned long a)
{
        long x1 = x.GetMostSignificant();
        return ((x1 > 0) || ((x1 == 0) && (a < x.GetLeastSignificant())) );
}

inline bool operator> (const TDoubleLong& x, long a)
{
        long x1 = x.GetMostSignificant();
        if (a >=0) return ((x1 > 0) || ((x1 == 0) && (x.GetLeastSignificant() > a)) );
        else return ((x1 > -1) || ((x1 == -1) && (x.GetLeastSignificant() > a)) );
}

inline bool operator> (const TDoubleLong& x, unsigned int a)
{
        long x1 = x.GetMostSignificant();
        return ((x1 > 0) || ((x1 == 0) && (a < x.GetLeastSignificant())) );
}

inline bool operator> (const TDoubleLong& x, int a)
{
        long x1 = x.GetMostSignificant();
        if (a >=0) return ((x1 > 0) || ((x1 == 0) && (x.GetLeastSignificant() > a)) );
        else return ((x1 > -1) || ((x1 == -1) && (x.GetLeastSignificant() > a)) );
}

/**************************************************************************/
/*      global method for binary  <  Operator                             */
/**************************************************************************/


inline bool operator< (const TDoubleLong& x, const TDoubleLong& y)
{
        return (y>x);
}

inline bool operator< (unsigned long a, const TDoubleLong& x)
{
        return (x>a);
}

inline bool operator< (long a, const TDoubleLong& x)
{
        return (x>a);
}

inline bool operator< (unsigned int a, const TDoubleLong& x)
{
        return (x>a);
}

inline bool operator< (int a, const TDoubleLong& x)
{
        return (x>a);
}

inline bool operator< (const TDoubleLong& x, unsigned long a)
{
        return (a>x);
}

inline bool operator< (const TDoubleLong& x, long a)
{
        return (a>x);
}

inline bool operator< (const TDoubleLong& x, unsigned int a)
{
        return (a>x);
}

inline bool operator< (const TDoubleLong& x, int a)
{
        return (a>x);
}

/**************************************************************************/
/*      global method for binary  >= Operator                             */
/**************************************************************************/

inline bool     operator>= (const TDoubleLong& x, const TDoubleLong& y)
{
        return !(y > x);
}

inline bool     operator>= (const TDoubleLong& x, unsigned long a)
{
        return !(x<a);
}

inline bool     operator>= (const TDoubleLong& x, long a)
{
        return !(x<a);
}

inline bool     operator>= (const TDoubleLong& x, unsigned int a)
{
        return !(x<a);
}

inline bool     operator>= (const TDoubleLong& x, int a)
{
        return !(x<a);
}

inline bool     operator>= (unsigned long a, const TDoubleLong& x)
{
        return !(x>a);
}

inline bool     operator>= (long a, const TDoubleLong& x)
{
        return !(x>a);
}

inline bool     operator>= (unsigned int a, const TDoubleLong& x)
{
        return !(x>a);
}

inline bool     operator>= (int a, const TDoubleLong& x)
{
        return !(x>a);
}

/**************************************************************************/
/*      global method for binary  <= Operator                             */
/**************************************************************************/

inline bool     operator<= (const TDoubleLong& x, const TDoubleLong& y)
{
        return !(x>y);
}

inline bool     operator<= (const TDoubleLong& x, unsigned long a)
{
        return !(x>a);
}

inline bool     operator<= (const TDoubleLong& x, long a)
{
        return !(x>a);
}

inline bool     operator<= (const TDoubleLong& x, unsigned int a)
{
        return !(x>a);
}

inline bool     operator<= (const TDoubleLong& x, int a)
{
        return !(x>a);
}

inline bool     operator<= (unsigned long a, const TDoubleLong& x)
{
        return !(x<a);
}

inline bool     operator<= (long a, const TDoubleLong& x)
{
        return !(x<a);
}
inline bool     operator<= (unsigned int a, const TDoubleLong& x)
{
        return !(x<a);
}

inline bool     operator<= (int a, const TDoubleLong& x)
{
        return !(x<a);
}


/************************************************************************/
/*      global method for binary logical OR Operator                    */
/************************************************************************/
inline bool operator||(const TDoubleLong& x, const TDoubleLong& y)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant()
        || y.GetMostSignificant() || y.GetLeastSignificant() );
}

inline bool operator||(const TDoubleLong& x, unsigned long a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(const TDoubleLong& x, long a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(const TDoubleLong& x, unsigned int a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(const TDoubleLong& x, int a)
{
        return (x.GetMostSignificant() || x.GetLeastSignificant() || a);
}

inline bool operator||(unsigned long a, const TDoubleLong& x)
{
        return (x || a);
}

inline bool operator||(long a, const TDoubleLong& x)
{
        return (x || a);
}

inline bool operator||(unsigned int a, const TDoubleLong& x)
{
        return (x || a);
}

inline bool operator||(int a, const TDoubleLong& x)
{
        return (x || a);
}


/************************************************************************/
/*      global method for binary logical AND Operator                   */
/************************************************************************/
inline bool operator&&(const TDoubleLong& x, const TDoubleLong& y)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant())
        && (y.GetMostSignificant() || y.GetLeastSignificant()));
}

inline bool operator&&(const TDoubleLong& x, unsigned long a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(const TDoubleLong& x, long a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(const TDoubleLong& x, unsigned int a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(const TDoubleLong& x, int a)
{
        return ((x.GetMostSignificant() || x.GetLeastSignificant()) && a );
}

inline bool operator&&(unsigned long a, const TDoubleLong& x)
{
        return (x && a);
}

inline bool operator&&(long a, const TDoubleLong& x)
{
        return (x && a);
}

inline bool operator&&(unsigned int a, const TDoubleLong& x)
{
        return (x && a);
}

inline bool operator&&(int a, const TDoubleLong& x)
{
        return (x && a);
}

/************************************************************************/
/*      global method for binary bitwise OR Operator                    */
/************************************************************************/
inline TDoubleLong operator|(const TDoubleLong& x, const TDoubleLong& y)
{
        return(TDoubleLong(x.GetMostSignificant() | y.GetMostSignificant(),
                        x.GetLeastSignificant() | y.GetLeastSignificant()));
}

inline TDoubleLong operator|(const TDoubleLong& x, unsigned long a)
{
        return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline TDoubleLong operator|(const TDoubleLong& x, long a)
{
        if (a < 0)  return (TDoubleLong( -1, x.GetLeastSignificant() | a));
        else return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline TDoubleLong operator|(const TDoubleLong& x, unsigned int a)
{
        return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline TDoubleLong operator|(const TDoubleLong& x, int a)
{
        if (a < 0)  return (TDoubleLong( -1, x.GetLeastSignificant() | a));
        else return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() | a));
}

inline TDoubleLong operator|(unsigned long a, const TDoubleLong& x)
{
        return (x | a);
}

inline TDoubleLong operator|(long a, const TDoubleLong& x)
{
        return (x | a);
}

inline TDoubleLong operator|(unsigned int a, const TDoubleLong& x)
{
        return (x | a);
}

inline TDoubleLong operator|(int a, const TDoubleLong& x)
{
        return (x | a);
}


/************************************************************************/
/*      global method for binary bitwise AND Operator                   */
/************************************************************************/
inline TDoubleLong operator& (const TDoubleLong& x, const TDoubleLong& y)
{
        return(TDoubleLong(x.GetMostSignificant() & y.GetMostSignificant(),
                        x.GetLeastSignificant() & y.GetLeastSignificant()));
}

inline TDoubleLong operator&(const TDoubleLong& x, unsigned long a)
{
        return (TDoubleLong(0, x.GetLeastSignificant() & a));
}

inline TDoubleLong operator&(const TDoubleLong& x, long a)
{
        if ( a < 0) return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() & a));
        else return (TDoubleLong(0, x.GetLeastSignificant() & a));
}

inline TDoubleLong operator&(const TDoubleLong& x, unsigned int a)
{
        return (TDoubleLong(0, x.GetLeastSignificant() & a));
}

inline TDoubleLong operator&(const TDoubleLong& x, int a)
{
        if ( a < 0) return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() & a));
        else return (TDoubleLong(0, x.GetLeastSignificant() & a));
}


inline TDoubleLong operator&(unsigned long a, const TDoubleLong& x)
{
        return (x & a);
}

inline TDoubleLong operator&(long a, const TDoubleLong& x)
{
        return (x & a);
}

inline TDoubleLong operator&(unsigned int a, const TDoubleLong& x)
{
        return (x & a);
}

inline TDoubleLong operator&(int a, const TDoubleLong& x)
{
        return (x & a);
}

/************************************************************************/
/*      global method for binary bitwise XOR Operator                   */
/************************************************************************/
inline TDoubleLong operator^ (const TDoubleLong& x, const TDoubleLong& y)
{
        return(TDoubleLong(x.GetMostSignificant() ^ y.GetMostSignificant(),
                        x.GetLeastSignificant() ^ y.GetLeastSignificant()));
}

inline TDoubleLong operator^(const TDoubleLong& x, unsigned long a)
{
        return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline TDoubleLong operator^(const TDoubleLong& x, long a)
{
        if (a < 0 ) return (TDoubleLong(~x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
        else return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline TDoubleLong operator^(const TDoubleLong& x, unsigned int a)
{
        return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline TDoubleLong operator^(const TDoubleLong& x, int a)
{
        if (a < 0 ) return (TDoubleLong(~x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
        else return (TDoubleLong(x.GetMostSignificant(), x.GetLeastSignificant() ^ a));
}

inline TDoubleLong operator^(unsigned long a, const TDoubleLong& x)
{
        return (x ^ a);
}

inline TDoubleLong operator^(long a, const TDoubleLong& x)
{
        return (x ^ a);
}

inline TDoubleLong operator^(unsigned int a, const TDoubleLong& x)
{
        return (x ^ a);
}

inline TDoubleLong operator^(int a, const TDoubleLong& x)
{
        return (x ^ a);
}


/********************************************************************************/
/*  END OF INLINE METHODS.                                                      */
/********************************************************************************/

#endif   /* NO_Internal */

#endif // _IPRIMTPI_
