// Revision: 88 1.4.1.7 source/ui/baseapp/ibmphdr.cpp, staticctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ibmphdr.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibmphdr.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// #define IC_TRACE_ALL
extern "C"
  {
  #define INCL_GPIBITMAPS
  #include <iwindefs.h>
  }

#include <ibmpctl.hpp>
#include <ibmphdr.hpp>
#include <ibmpstat.hpp>
#include <ibundles.hpp>
#ifdef IC_MOTIF
  #include <icustbdd.hpp>
#endif //IC_MOTIF
#include <iexcept.hpp>
#include <igarea2d.hpp>
#include <igimage.hpp>
#include <igrport.hpp>
#include <ihandle.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBMPHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBitmapHandler::IBitmapHandler                                               |
------------------------------------------------------------------------------*/
IBitmapHandler :: IBitmapHandler() {;}

/*------------------------------------------------------------------------------
| IBitmapHandler::~IBitmapHandler                                              |
------------------------------------------------------------------------------*/
IBitmapHandler :: ~IBitmapHandler() {;}

/*------------------------------------------------------------------------------
| IBitmapHandler::drawBackground                                               |
------------------------------------------------------------------------------*/
bool IBitmapHandler::drawBackground( IStaticText* window,
                                     IPresSpaceHandle& ps,
                                     const IRectangle& updateRect )
{
   IMODTRACE_ALL("IBitmapHandler::drawBackground");
   bool result(false);

#ifdef IC_PMWIN
   // Let the IStaticTextHandler fill the background if necessary.
   IStaticTextHandler::drawBackground( window, ps, updateRect );

   ISize windowSize ( window->size() );
   IGImage* cache = captureImage( ps, windowSize );

   if (this->drawImage( window, *cache ))
   {
       IGrafBundle bundle;
       if ( !window->isEnabled() || window->isHalftone() )
          bundle.setImageTransferMode( IImageTransferMode::kOCHalfTone );
       else
          bundle.setImageTransferMode( IImageTransferMode::kSrcCopy );

       // Set a clip region so that text (or other non-invalidated part)
       // of the control is not overwritten.
       IGArea updateRegion( updateRect );

       // Must use native orientation because the updateRect comes directly
       // from the OS.
       IExtendedRootGrafPort port(ps,
                                  ICoordinateSystem::nativeOrientation());
       ILinkedGrafPort linkedPort( &port, &updateRegion );
       linkedPort.draw( *cache, bundle );
       result = true;
   }

   delete cache;

#endif //IC_PMWIN

#ifdef IC_MOTIF
   if (window->isValid())
   {
      // The image needs to be re-drawn so that it is sized to the
      // current size of the control to prevent tiling.
      // Get a pointer to the cached image information.
      ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
         ( window->windowData( ICustomButtonDrawData::dataHandle() ) );
      if (drawData)
      {
         bool refreshWidget (drawData->invalidateAll( 0 ));
         ISize windowSize ( window->size() );
         ITRACE_ALL( IString("windowSize=") + windowSize.asString() );
         if ( windowSize != ISize() )
         {
            // Create an image in memory sized to the window.
            IGImage* cache = captureImage(ps, windowSize);
            result = drawImage( window, *cache );
            if (result)
            {
               ITRACE_ALL("Setting widget values");
               //Save the cached image and provide its pixmap to the widget.
               drawData->adoptImage( drawData->buttonState(), cache);
               IBitmapHandle widgetbmap(
                  drawData->imageHandle( drawData->buttonState() ) );
               //Get correct pixmap for enabled/disabled widget.
               Pixmap backgroundPixmap(
                  ( window->isEnabled() && !window->isHalftone() ) ?
                   (IBitmapHandle::Value) widgetbmap :
                   IBitmapStaticPtr::insensitivePixmap( widgetbmap ) );
               XtVaSetValues(
                  window->handle(),
                  XmNbackgroundPixmap, backgroundPixmap,
                  0);
            }
            else
            {
               // Not processed.  Delete the cache IGImage.
               delete cache;
               if (refreshWidget)
               {
                  // Reset widget information if we invalidated the cache.
                  ITRACE_ALL("Resetting widget values");
                  XtVaSetValues(window->handle(),
                                XmNbackgroundPixmap, XmUNSPECIFIED_PIXMAP,
                                0);
               }
            }
         }
         else
         {
            //Window 0 sized.  All done.
            result = true;
            if (refreshWidget)
            {
               // Reset widget information if we invalidated the cache.
               ITRACE_ALL("Resetting widget values");
               XtVaSetValues(window->handle(),
                             XmNbackgroundPixmap, XmUNSPECIFIED_PIXMAP,
                             0);
            }
         }
      }
   }
#endif //IC_MOTIF
   if (!result)
      // Not drawn.  Let the base class draw the background.
      result = Inherited::drawBackground(window, ps, updateRect);

   return result;
}

/*------------------------------------------------------------------------------
| IBitmapHandler::captureImage                                                 |
------------------------------------------------------------------------------*/
IGImage* IBitmapHandler::captureImage( IPresSpaceHandle& ps,
                                       const ISize&      size )
{
   // Simply create a new image, ignoring background
   return new IGImage( (long)size.width(),
                       (long)size.height(),
                       IGImage::kTrueColor24Bit,
                       ICoordinateSystem::kOriginUpperLeft);
}

/*------------------------------------------------------------------------------
| IBitmapHandler::drawImage                                                    |
------------------------------------------------------------------------------*/
bool IBitmapHandler::drawImage( IStaticText* window, IGImage& image )
{
   IMODTRACE_ALL("IBitmapHandler::drawImage");
   bool result(false);
   IBitmapControl* pBitmap = dynamic_cast<IBitmapControl*>(window);
   IBitmapHandle bmp ( pBitmap ? pBitmap->bitmap() : IBitmapHandle() );
   if ( (IBitmapHandle::Value)bmp != 0)
   {
      ITRACE_ALL( IString("drawing bmp=") +
                  IString((unsigned long)(IBitmapHandle::Value)bmp).d2x() );

      // Construct a image from the client specified bitmap. Stretch or
      // shrink the image to fit the window.
      IGImage  bitmapImage( bmp );
      IGRect2D imageRect( image.rasterBounds() );
      IGRect2D targetRect(
          targetRectangle( pBitmap, imageRect, bitmapImage.rasterBounds() ));

      if (imageRect != targetRect )
      {
         //Have to fill area not occupied by bitmap.
         IFillBundle fillBundle( pBitmap->fillColor() );
         image.grafPort()->draw( imageRect, fillBundle );
      }
      IGrafBundle bundle;
      bundle.setImageTransferMode( IImageTransferMode::kSrcCopy );

      IGrafMatrix matrix( bitmapImage.rasterBounds(), targetRect );
      image.grafPort()->draw( bitmapImage, bundle, matrix );

      result = true;
   }
   return result;
}

/*------------------------------------------------------------------------------
| IBitmapHandler::targetRectangle                                              |
------------------------------------------------------------------------------*/
IGRect2D IBitmapHandler::targetRectangle( IBitmapControl* window,
                                          const IGRect2D& windowRect,
                                          const IGRect2D& bitmapRect)
{
   IGRect2D targetRect;
   if ( window->isSizeImageToWindowEnabled() )
   {
      // Render image to the size of the supplied windowRect, adjusted for
      // any system drawn areas.
#ifdef IC_PMWIN
      targetRect = windowRect;
#endif //IC_PMWIN
#ifdef IC_MOTIF
      Dimension shadowThickness(0), borderWidth(0),
                highlightThickness(0);

      XtVaGetValues(window->handle(),
                    XmNshadowThickness, &shadowThickness,
                    XmNborderWidth,     &borderWidth,
                    XmNhighlightThickness, &highlightThickness,
                    NULL);

      int edges = 2*( shadowThickness + borderWidth +
                      highlightThickness );

      // Compute new size for image.  Minimum size is 1,1.
      IGPoint2D newSize( windowRect.size() - IGPoint2D(edges,edges) );
      if (newSize.fX < 1) newSize.fX = 1;
      if (newSize.fY < 1) newSize.fY = 1;
      targetRect = IGRect2D( IGPoint2D(0,0), newSize );
#endif //IC_MOTIF
   }
   else
   {
      // Render image to its natural size, centered on the supplied IGImage.
      targetRect = bitmapRect;
   }
   targetRect.setCenter( windowRect.center() );
   return targetRect;
}

