// Revision: 90 1.9.1.2 source/ui/baseapp/icmd.cpp, genhdr, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icmd.cpp                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icmd.hpp.                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define INCL_WINFRAMEMGR              // For SC_xxx.
  #include <iwindefs.h>
  }

#include <icmd.hpp>
#include <icconst.h>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICMD_CPP_
  #include <ipagetun.h>
#endif

/*----------------------- Common Application Commands ------------------------*/
const ICommand::CommandId
  ICommand::kCancelOperationId       ( IC_ID_CANCELOPERATION ),
  ICommand::kOpenId                  ( IC_ID_OPEN ),
  ICommand::kSaveId                  ( IC_ID_SAVE ),
  ICommand::kPrintId                 ( IC_ID_PRINT ),
  ICommand::kLocateId                ( IC_ID_LOCATE ),
  ICommand::kCutId                   ( IC_ID_CUT ),
  ICommand::kCopyId                  ( IC_ID_COPY ),
  ICommand::kPasteId                 ( IC_ID_PASTE ),
  ICommand::kHelpId                  ( IC_ID_HELP ),
  ICommand::kBoldId                  ( IC_ID_BOLD ),
  ICommand::kItalicId                ( IC_ID_ITALIC ),
  ICommand::kUnderscoreId            ( IC_ID_UNDERSCORE ),
  ICommand::kDoubleUnderscoreId      ( IC_ID_DOUBLEUNDERSCORE ),
  ICommand::kNormalId                ( IC_ID_NORMAL ),
  ICommand::kSettingsId              ( IC_ID_SETTINGS ),
#if (IC_OBSOLETE <= IC_OBSOLETE_3)
  // CopyTo "means" the same thing as SaveAs, "SaveAs" is the standard term.
  // For back compatibility, keep the old label but change the value to match SaveAs
  ICommand::kCopyToId                ( IC_ID_SAVE_AS ),
#endif
  ICommand::kPropertiesId            ( IC_ID_SETTINGS ),
//  ICommand::kPropertiesOfSelectedId  ( IC_ID_PROPERTIESOFSELECTED ),
  ICommand::kOkId                    ( IC_ID_OK ),
  ICommand::kCancelId                ( IC_ID_CANCEL ),
  ICommand::kClearId                 ( IC_ID_CLEAR ),
#ifdef IC_PMWIN
  ICommand::kFileOkId                ( IC_ID_FILEOK ),
  ICommand::kFileCancelId            ( IC_ID_FILECANCEL ),
  ICommand::kFileApplyId             ( IC_ID_FILEAPPLY ),
  ICommand::kFontOkId                ( IC_ID_FONTOK ),
  ICommand::kFontCancelId            ( IC_ID_FONTCANCEL ),
  ICommand::kFontApplyId             ( IC_ID_FONTAPPLY ),
#endif
  ICommand::kNewId                   ( IC_ID_NEW ),
  ICommand::kSaveAsId                ( IC_ID_SAVE_AS ),
  ICommand::kExitId                  ( IC_ID_EXIT ),
  ICommand::kUpdateId                ( IC_ID_UPDATE ),
  ICommand::kSaveCopyAsId            ( IC_ID_SAVE_COPY_AS ),
  ICommand::kUndoId                  ( IC_ID_UNDO ),
  ICommand::kRedoId                  ( IC_ID_REDO ),
  ICommand::kPasteSpecialId          ( IC_ID_PASTESPECIAL ),
  ICommand::kPasteLinkId             ( IC_ID_PASTELINK ),
  ICommand::kSelectAllId             ( IC_ID_SELECTALL ),
  ICommand::kInsertId                ( IC_ID_INSERT ),
  ICommand::kEditLinksId             ( IC_ID_EDITLINKS ),
  ICommand::kDeleteId                ( IC_ID_DELETE ),
  ICommand::kConvertId               ( IC_ID_CONVERT ),
  ICommand::kViewToolbarId           ( IC_ID_TOOLBAR ),
  ICommand::kAboutId                 ( IC_ID_ABOUT ),
  ICommand::kJustifyId               ( IC_ID_JUSTIFY ),
  ICommand::kCenterId                ( IC_ID_CENTER ),
  ICommand::kLeftId                  ( IC_ID_LEFT ),
  ICommand::kRightId                 ( IC_ID_RIGHT ),
  ICommand::kCheckSpellingId         ( IC_ID_CHECKSPELLING ),
  ICommand::kDeselectAllId           ( IC_ID_DESELECTALL ),
  ICommand::kInsertColumnId          ( IC_ID_INSERTCOLUMN ),
  ICommand::kInsertRowId             ( IC_ID_INSERTROW ),
  ICommand::kDeleteColumnsId         ( IC_ID_DELETECOLUMNS ),
  ICommand::kDeleteRowsId            ( IC_ID_DELETEROWS ),
  ICommand::kLeftAlignObjectsId      ( IC_ID_LEFTALIGNOBJ ),
  ICommand::kRightAlignObjectsId     ( IC_ID_RIGHTALIGNOBJ ),
  ICommand::kMailSendId              ( IC_ID_MAILSEND ),
  ICommand::kMovePagesId             ( IC_ID_MOVEPAGES ),
  ICommand::kPrintPreviewId          ( IC_ID_PRINTPREVIEW ),
  ICommand::kPrintSetupId            ( IC_ID_PRINTSETUP ),
  ICommand::kToolbarSetupId          ( IC_ID_TOOLBARSETUP ),
  ICommand::kSortAZId                ( IC_ID_SORTAZ ),
  ICommand::kSortZAId                ( IC_ID_SORTZA ),
  ICommand::kSortUpId                ( IC_ID_SORTUP ),
  ICommand::kSortDownId              ( IC_ID_SORTDOWN ),
  ICommand::kZoomId                  ( IC_ID_ZOOM ),
  ICommand::kZoominId                ( IC_ID_ZOOMIN ),
  ICommand::kZoomOutId               ( IC_ID_ZOOMOUT ),
  ICommand::kTourId                  ( IC_ID_TOUR ),
  ICommand::kGeneralHelpId           ( IC_ID_GENERALHELP ),
  ICommand::kHelpIndexId             ( IC_ID_HELPINDEX ),
  ICommand::kHelpTutorialId          ( IC_ID_HELPTUTORIAL ),
  ICommand::kUsingHelpId             ( IC_ID_USINGHELP ),
  ICommand::kLogonId                 ( IC_ID_LOGON ),
  ICommand::kLogoffId                ( IC_ID_LOGOFF ),
  ICommand::kRunScriptId             ( IC_ID_RUNSCRIPT ),
  ICommand::kVPAPaletteId            ( IC_ID_VPAPALETTE );

/*-------------------------- Common System Commands --------------------------*/
const ICommand::CommandId
  ICommand::kCloseId      ( IC_ID_CLOSE );
#ifndef IC_MOTIF_FLAGNOP
const ICommand::CommandId
  ICommand::kMoveId       ( IC_ID_MOVE ),
  ICommand::kSizeId       ( IC_ID_SIZE ),
  ICommand::kMinimizeId   ( IC_ID_MINIMIZE ),
  ICommand::kMaximizeId   ( IC_ID_MAXIMIZE ),
  ICommand::kHideId       ( IC_ID_SYSHIDE ),
  ICommand::kRestoreId    ( IC_ID_RESTORE ),
  ICommand::kWindowListId ( IC_ID_WINDOWLIST );
#endif
