// Revision: 06 1.3.2.2 source/ui/baseapp/ictlevt.cpp, genhdr, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ictlevt.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ictlevt.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#include <ictlevt.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <istring.hpp>
#include <iwindow.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICTLEVT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IControlEvent::IControlEvent                                                 |
|                                                                              |
| Copy constructor here for page tuning                                        |
------------------------------------------------------------------------------*/
IControlEvent::IControlEvent ( const IControlEvent& event )
  : IControlEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IControlEvent::IControlEvent                                                 |
|                                                                              |
| Construct IControlEvent object from an IEvent instance                       |
------------------------------------------------------------------------------*/
IControlEvent::IControlEvent ( const IEvent& event )
  : IControlEvent::Inherited( event )
{
#ifdef IC_PMWIN
  // Reset the control handle, so it isn't the owner window handle.
  // Once the right control handle is set, IEvent::controlHandle and
  // IEvent::controlWindow will give the right results.

  // Start searching for the control.  We only know its window ID and
  // owner window.  For performance reasons, first search the
  // children of the owner window.
  IWindowHandle controlHandle =
                   IWindow::handleWithParent( this->controlId(),
                                              this->handle() );

  // In Windows, the above check will have found the control in cases
  // where the control is a child.  The only case where more work
  // is needed is top level windows which are owned.
  if ( controlHandle  &&
#ifdef IC_WIN
       ( !(ISTYLEOF( controlHandle ) & WS_CHILD) ) &&
#endif
       ( IOWNEROF( controlHandle ) != this->handle() ))
  {
     // We don't have the right control, since the owner window is
     // not correct.
     controlHandle = 0;
  }

  if ( ! controlHandle )
  {
     // Now search the window list for a control wrappered with an
     // IWindow.
     IWindow *owner = this->window();
     if ( owner )
     {
        // Note that the owner window must have an IWindow, or the
        // IWindow subclass procedure (_pfnwpICWinProc) would never
        // have been called, to in turn call the handler that
        // constructs this control event.
        IWindow *control =
                  IWindow::windowWithOwner( this->controlId(), owner );
        if ( control )
        {          // Found a match.
           controlHandle = control->handle();
        }
     }
  }  // End searching window list.

  // If we haven't yet found the control, we are pretty much out of
  // luck now.  If the control had an IWindow, we should have found
  // it in the window list (unless the IWindow for the owner was
  // deleted).  If the control doesn't have an IWindow, we have no
  // guaranteed way to find the control, since PM gives us no way to
  // trace down the owner chain.  We have already checked child
  // windows (although we could use a ChildCursor to check all child
  // windows, rather than just stopping at the first match).  We
  // could also check siblings of the owner, but this doesn't seem a
  // likely place to find an owned control.

  this->setControlHandle( controlHandle );
#endif
#ifdef IC_MOTIF
// Motif does nothing here; the event comes to the control itself in Motif
#endif
}

/*------------------------------------------------------------------------------
| IControlEvent::~IControlEvent                                                |
|                                                                              |
| Empty destructor here for page tuning                                        |
------------------------------------------------------------------------------*/
IControlEvent::~IControlEvent ( )
{ }

/*------------------------------------------------------------------------------
| IControlEvent::controlId                                                     |
------------------------------------------------------------------------------*/
unsigned long IControlEvent::controlId ( ) const
{
#ifdef IC_PMWIN
  return this->parameter1().number1();
#endif
#ifdef IC_MOTIF
  IString resid( XtName( (Widget)controlHandle() ) );
  return (resid.asInt() );
#endif
}
