// Revision: 25 1.4.1.5 source/ui/baseapp/ifocshdr.cpp, genhdr, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ifocshdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifocshdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define IC_MULTIMEDIA
#define INCL_CIRCULARSLIDER
#define INCL_SW
#define INCL_WINDIALOGS
#define INCL_WINENTRYFIELDS
#define INCL_WINLISTBOXES
#define INCL_WINMLE
#define INCL_WINSTDCNR
#define INCL_WINSTDSLIDER
#define INCL_WINSTDSPIN
#define INCL_WINSTDVALSET
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifdef IC_WIN
  #ifndef _ICLSLIDERW_
    #include <iclsldw.h>
  #endif
  #ifndef _ICLSPINBTNW_
    #include <iclspbw.h>
  #endif
  #ifndef _ICLCONTAINERW_
    #include <iclcnrw.h>
  #endif
  #ifndef _ICLCSLIDERW_
    #include <iclcsldw.h>
  #endif
#endif
#include <ifocshdr.hpp>
#include <ictlevt.hpp>
#include <ievent.hpp>
#include <iwindow.hpp>
#include <iwcname.hpp>
#include <icconst.h>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IFOCSHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IFocusHandler::IFocusHandler                                                 |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IFocusHandler::IFocusHandler ( )
  : IFocusHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IFocusHandler::~IFocusHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IFocusHandler::~IFocusHandler ( )
{ }

/*------------------------------------------------------------------------------
| IFocusHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
bool IFocusHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

#ifdef IC_PMWIN
  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_ENTRYFIELD )
        {                              // Entry field case.
           if ( subMsg == EN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == EN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
        else if ( className == WC_LISTBOX )
        {                              // List box case.
           if ( subMsg == LN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == LN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#ifdef IC_WIN
        else if ( className == WC_COMBOBOX )
        {
           if (subMsg == CBN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == CBN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#endif
#ifndef IC_WIN
        // Don't process MLE separately, since on Windows it is the
        // same as the entry field, which was just processed above.
        else if ( className == WC_MLE )
        {                              // Multi-line edit case.
           if ( subMsg == MLN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == MLN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#endif
#ifdef IC_WIN
        else if (( className == WC_SPINBUTTON) ||
                 ( className == WC_NATIVESPINBUTTON) )
#else
        else if ( className == WC_SPINBUTTON )
#endif
        {                              // Spin button case.
           if ( subMsg == SPBN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == SPBN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#ifdef IC_WIN
        else if ( (className == WC_CONTAINER) ||
                  (className == WC_NATIVECONTAINER))
#else
        else if ( className == WC_CONTAINER )
#endif
        {                              // Container case.
           if ( subMsg == CN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == CN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#ifdef IC_WIN
        else if ( className == WC_SLIDER ||
                  className == WC_NATIVESLIDER )
#else
        else if ( className == WC_SLIDER )
#endif
        {                              // Slider case.
           if ( subMsg == SLN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == SLN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#ifdef WC_CIRCULARSLIDER
        else if ( className == WC_CIRCULARSLIDER )
        {                              // Circular slider case.
           if ( subMsg == CSN_SETFOCUS )
           {
              if ( ctlevt.parameter2().number1() )
              {
                 bRc = this->gotFocus( ctlevt );
              }
              else
              {
                 bRc = this->lostFocus( ctlevt );
              }
           }
        }
#endif
#ifndef IC_WIN
        else if ( className == WC_VALUESET )
        {                              // Value set case.
           if ( subMsg == VN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == VN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#endif

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  } /* endif WM_CONTROL */
#endif

#ifdef IC_MOTIF
  // for montana we had to add allowing the focus events to also propagate to the
  // owner of the control. This was to allow them to store away the focus control
  // when the user moves focus to another pane. If the dispatching window is not
  // equal to the control window then we know that the event has propagated up
  // one level and we need to ignore that event here. Montana has there own handler
  // for this processing, they don't use our focus handler.
  if (event.controlWindow() == event.dispatchingWindow() )
  {
    // This code relies upon the fact that IWindow filters focus events such
    // that only focus events which indicate a change to this widget are
    // dispatched.  The ancestor/inferior notifications generated by the
    // X server are not dispatched.
    switch (event.eventId())
    {
       case xEvent( FocusIn ):
          {
            /* Motif doesn't have any special problems with who
            /  the handler is attached to. This is because a) Motif
            /  always routes events & callbacks to the control first
            /  and, b) because we do owner routing ourselves.    */
  
            // We only want to process normal focus changes and ignore
            // grab/ungrab notifications.
            XEvent *pxevent = (XEvent *)(char*)event.parameter2();
            if (pxevent->xfocus.mode == NotifyNormal)
            {
               IControlEvent ctlevt(event);
               bRc = gotFocus(ctlevt);
               if (bRc)
                  event.setResult(ctlevt.result());  // copy result
            }
          }
          break;
       case xEvent( FocusOut ):
          {
            /* Motif doesn't have any special problems with who
            /  the handler is attached to. This is because a) Motif
            /  always routes events & callbacks to the control first
            /  and, b) because we do owner routing ourselves.    */
  
            // We only want to process normal focus changes and ignore
            // grab/ungrab notifications.
            XEvent *pxevent = (XEvent *)(char*)event.parameter2();
            if (pxevent->xfocus.mode == NotifyNormal)
            {
               IControlEvent ctlevt(event);
               bRc = lostFocus(ctlevt);
               if (bRc)
                  event.setResult(ctlevt.result());  // copy result
            }
          }
          break;
  
       default:
          break;
    }   //end switch .eventId
  }
#endif

  return bRc;
}

bool IFocusHandler::gotFocus ( IControlEvent& event )
{
  return false;
}

bool IFocusHandler::lostFocus ( IControlEvent& event )
{
  return false;
}
