// Revision: 38 1.8.1.4 source/ui/baseapp/iframe1.cpp, frame, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: iframe1.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iframnhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINFRAMEMGR

extern "C" {
   #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <iframe.hpp>
#include <iframnhd.hpp>
#include <iframprv.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFRAME1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static notification strings.
INotificationId const IFrameWindow::closeId = "IFrameWindow::close";
INotificationId const IFrameWindow::activateId = "IFrameWindow::activate";
INotificationId const IFrameWindow::deactivateId = "IFrameWindow::deactivate";
INotificationId const IFrameWindow::titleTextId = "IFrameWindow::titleText";


/*------------------------------------------------------------------------------
| IFrameWindow::enableNotification                                             |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IFrameWindow, does not send notifications to observer objects until it is    |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IFrameWindow& IFrameWindow::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new IFrameWindowNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IFrameWindowNotifyHandler::IFrameWindowNotifyHandler                         |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IFrameWindowNotifyHandler::IFrameWindowNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IFrameWindowNotifyHandler::~IFrameWindowNotifyHandler                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IFrameWindowNotifyHandler::~IFrameWindowNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IFrameWindowNotifyHandler::dispatchHandlerEvent                              |
|                                                                              |
| This notifies IFrameWindow Windows of the following events:                  |
|                                                                              |
| close(Id)                                                                    |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN

bool IFrameWindowNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_ACTIVATE:

#ifdef IC_WIN
        /*******************************************************************************/
	/* Only suppress notification if the event is the first of two WA_ACTIVEs sent */
	/* when the frame is restored from minimized.                                  */ 
	/* This is indicated by anEvent.parameter1().number2() being non-zero.           */ 
        /*******************************************************************************/

        if( !( anEvent.parameter1().number1() == WA_ACTIVE && anEvent.parameter1().number2() ))
#endif
#ifdef IC_PM
        if (anEvent.parameter2().asUnsignedLong() == thisWindow->handle())
#endif
        {
          /************************************************************/
          /* Post private message to delay notification until after   */
          /* the selection state has been modified                    */
          /************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_FRAME_ACTIVE ));
        }
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_FRAME_ACTIVE)
        {
          thisWindow->notifyObservers(INotificationEvent(
#ifdef IC_PM
                     (anEvent.parameter1().asUnsignedLong() ?
                                 IFrameWindow::activateId :
                                 IFrameWindow::deactivateId),
#endif
#ifdef IC_WIN
                     (anEvent.parameter1().number1() == WA_INACTIVE ?
                                 IFrameWindow::deactivateId :
                                 IFrameWindow::activateId),
#endif
                     *thisWindow, false));
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif  // IC_PMWIN

#ifdef IC_MOTIF

bool IFrameWindowNotifyHandler::dispatchHandlerEvent(IEvent& anEvent)
{
   // Code for IFrameWindow::closeId in IFRAMHDR.CPP IFrameWindow::closed
   // IFrameWindow::deactivateId is flagged NOPed in AIX
   // Following is code for IFrameWindow::activateId
   IWindow* thisWindow = anEvent.window();

   if (thisWindow)
   {
     if (anEvent.eventId() == xEvent(ClientMessage))
     {
       XClientMessageEvent *xevent =
         (XClientMessageEvent *)anEvent.parameter2().asUnsignedLong();
       if (xevent->type == ClientMessage)
       {
           if (xevent->data.l[0] == IFrameWindowData::wmTakeFocus)
           {
               thisWindow->notifyObservers(INotificationEvent(
                 IFrameWindow::activateId, *thisWindow, false));
           } // if wmTakeFocus (which = PM's WM_ACTIVATE)
       } /* if ClientMessage  */
     }
   }
   return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif
