// Revision: 55 1.6.1.5 source/ui/baseapp/ihandle2.cpp, datatype, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ihandle2.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions for Bitmap      *
*   handles declared ihandle.hpp.                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_WINPOINTERS
  #define INCL_GPIBITMAPS
  #define INCL_WINMESSAGEMGR
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  }

#include <ihandle.hpp>

#include <icconst.h>
#include <iexcept.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <ibmpstat.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHANDLE2_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
 Note that bitmap handles use references to reduce
  duplicate bitmaps for better storage utilization.
  See IReference, IRefCounted classes for more info.
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
| ISystemBitmapHandle::ISystemBitmapHandle                                     |
------------------------------------------------------------------------------*/
ISystemBitmapHandle :: ISystemBitmapHandle(Identifier bitmapId)
         : IBitmapHandle(0)
{
  long lPMId(0);
#ifdef IC_PMWIN
  switch(bitmapId)
  {
    case  systemMenu:
          lPMId = SBMP_SYSMENU;
          break;
    case  systemMenuPressed:
          lPMId = SBMP_SYSMENUDEP;
          break;
    case  scrollBarUpArrow:
          lPMId = SBMP_SBUPARROW;
          break;
    case  scrollBarUpArrowPressed:
          lPMId = SBMP_SBUPARROWDEP;
          break;
    case  scrollBarUpArrowDisabled:
          lPMId = SBMP_SBUPARROWDIS;
          break;
    case  scrollBarDownArrow:
          lPMId = SBMP_SBDNARROW;
          break;
    case  scrollBarDownArrowPressed:
          lPMId = SBMP_SBDNARROWDEP;
          break;
    case  scrollBarDownArrowDisabled:
          lPMId = SBMP_SBDNARROWDIS;
          break;
    case  scrollBarRightArrow:
          lPMId = SBMP_SBRGARROW;
          break;
    case  scrollBarRightArrowPressed:
          lPMId = SBMP_SBRGARROWDEP;
          break;
    case  scrollBarRightArrowDisabled:
          lPMId = SBMP_SBRGARROWDIS;
          break;
    case  scrollBarLeftArrow:
          lPMId = SBMP_SBLFARROW;
          break;
    case  scrollBarLeftArrowPressed:
          lPMId = SBMP_SBLFARROWDEP;
          break;
    case  scrollBarLeftArrowDisabled:
          lPMId = SBMP_SBLFARROWDIS;
          break;
    case  menuCheckMark:
          lPMId = SBMP_MENUCHECK;
          break;
    case  menuAttached:               /* cascading menu mark */
          lPMId = SBMP_MENUATTACHED;
          break;
    case  checkBoxCheck:
          lPMId = SBMP_CHECKBOXES;
          break;
    case  comboBoxDownArrow:
          lPMId = SBMP_COMBODOWN;
          break;
    case  pushButtonCorners:
          lPMId = SBMP_BTNCORNERS;
          break;
    case  minimizeButton:
          lPMId = SBMP_MINBUTTON;
          break;
    case  minimizeButtonPressed:
          lPMId = SBMP_MINBUTTONDEP;
          break;
    case  maximizeButton:
          lPMId = SBMP_MAXBUTTON;
          break;
    case  maximizeButtonPressed:
          lPMId = SBMP_MAXBUTTONDEP;
          break;
    case  restoreButton:
          lPMId = SBMP_RESTOREBUTTON;
          break;
    case  restoreButtonPressed:
          lPMId = SBMP_RESTOREBUTTONDEP;
          break;
    case  childSystemMenu:
#ifdef IC_WIN
          lPMId = SBMP_SYSMENU;
#else
          lPMId = SBMP_CHILDSYSMENU;
#endif
          break;
    case  childSystemMenuPressed:
#ifdef IC_WIN
          lPMId = SBMP_SYSMENUDEP;
#else
          lPMId = SBMP_CHILDSYSMENUDEP;
#endif
          break;
#ifndef IC_WIN
    case  drive:
          lPMId = SBMP_DRIVE;
          break;
    case  file:
          lPMId = SBMP_FILE;
          break;
    case  folder:
          lPMId = SBMP_FOLDER;
          break;
    case  collapsedTree:
          lPMId = SBMP_TREEPLUS;
          break;
    case  expandedTree:
          lPMId = SBMP_TREEMINUS;
          break;
    case  program:
          lPMId = SBMP_PROGRAM;
          break;
#endif  // !IC_WIN
    case  sizeBox:
          lPMId = SBMP_SIZEBOX;
          break;
  } /* end switch */
  if (lPMId)
  {
    fhandle = (IBitmapHandle::Value)IBitmapStaticPtr::systemBitmap(lPMId);
    IBitmapStaticPtr::addBitmapRef(fhandle);
  }
  else
#endif // IC_PMWIN
  {
#ifdef IC_WIN
    bool       useIcon(false);
    bool       useShell32(false);
#endif
    switch(bitmapId)
      {
#ifdef IC_PMWIN
      case  smallSystemMenu:
        // Load from IOC resource lib
        lPMId = IC_SMALLSYSTEMMENU;
        break;
#endif
#ifdef IC_WIN
      case  drive:
        lPMId =  IC_DRIVE_SHELL32;
        useShell32 = true;
        useIcon = true;
        break;
      case  file:
        lPMId = IC_FILE_SHELL32;
        useShell32 = true;
        useIcon = true;
        break;
      case  folder:
        lPMId = IC_FOLDER_SHELL32;
        useShell32 = true;
        useIcon = true;
        break;
      case  collapsedTree:
        lPMId = IC_COLLAPSEICON;
        useIcon = true;
        break;
      case  expandedTree:
        lPMId = IC_EXPANDICON;
        useIcon = true;
        break;
      case  program:
        lPMId = IC_PROGRAM_SHELL32;
        useShell32 = true;
        useIcon = true;
        break;
#endif
#ifdef IC_MOTIF
    // Load from IOC resource lib
    case  systemMenu:
      lPMId = IC_SYSBMP_SYSTEMMENU;
      break;
    case  minimizeButton:
      lPMId = IC_SYSBMP_MINIMIZE;
      break;
    case  maximizeButton:
      lPMId = IC_SYSBMP_MAXIMIZE;
      break;
    case  menuCheckMark:
       lPMId = IC_SYSBMP_MENUCHECKMARK;
       break;
    case  restoreButton:
       lPMId = IC_SYSBMP_RESTOREBUTTON;
       break;
    case  file:
       lPMId = IC_SYSBMP_FILE;
       break;
#endif // IC_MOTIF
      } // switch


#ifdef IC_WIN
    if (useIcon)
       {
       IPointerHandle hicon(0);
       try
          {
          if (useShell32)
             hicon =
                IDynamicLinkLibrary( IC_SHELL32_DLL ).tryToLoadIcon( lPMId );
          else
             hicon =
                IApplication::current().resourceLibrary().tryToLoadIcon( lPMId );
          }
       catch (...)
          {
          // this is to catch nonexistant DLL errors
          }
       if (!hicon)
          {
          // could not load ... use a default
          hicon =
            ISystemPointerHandle( ISystemPointerHandle::standardApplication) ;
          }
       // extract the bitmap portion of the icon
       ICONINFO    pointerInfo;
       IQUERYPOINTERINFO( hicon, &pointerInfo );
       fhandle = pointerInfo.hbmColor;
       DeleteObject( pointerInfo.hbmMask );     // throw away mask part
       // Add a reference since this one must be deleted.
       IBitmapStaticPtr::addBitmapRef(fhandle);
       }
    else
#endif
       {
       // Use a temporary bitmap handle so that the reference count does not
       // drop to 0 (and delete the bitmap) before we are able to call addRef
       IBitmapHandle bmpHandle =
         IApplication::current().resourceLibrary().loadBitmap( lPMId );
       fhandle = bmpHandle;
       IBitmapStaticPtr::addBitmapRef(fhandle);
       }
  } /* endif */

}

/*------------------------------------------------------------------------------
| ISystemBitmapHandle::~ISystemBitmapHandle                                    |
------------------------------------------------------------------------------*/
ISystemBitmapHandle :: ~ISystemBitmapHandle( )
{
}


