// Revision: 56 1.5.1.7 source/ui/baseapp/ihandle3.cpp, datatype, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ihandle3.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions for Pointer     *
*   handles declared ihandle.hpp.                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_WINPOINTERS
  #define INCL_WINMESSAGEMGR
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  }

#include <ihandle.hpp>

#include <ireslib.hpp>
#include <iexcept.hpp>
#include <istring.hpp>
#include <icconst.h>
#include <itrace.hpp>
#include <ibmpstat.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHANDLE3_CPP_
  #include <ipagetun.h>
#endif


#if (IC_OBSOLETE <= IC_OBSOLETE_3)
/*------------------------------------------------------------------------------
| ISystemPointerHandle::ISystemPointerHandle                                   |
------------------------------------------------------------------------------*/
ISystemPointerHandle:: ISystemPointerHandle(Identifier     pointerId,
                                            bool fMakeCopy)
   : IPointerHandle(0)
{
bool fUseLoadIcon = false;
long lPMId = 0;
#ifdef IC_PMWIN
// Flag for Windows macro to determine correct api to use in loading resource
bool fIconApi = true;
switch(pointerId)
{
  case  arrow:
        lPMId = SPTR_ARROW;
        break;
  case  text:
        lPMId = SPTR_TEXT;
        break;
  case  wait:
        lPMId = SPTR_WAIT;
        break;
  case  move:
        lPMId = SPTR_MOVE;
        break;
  case  sizeNWSE:
        lPMId = SPTR_SIZENWSE;
        break;
  case  sizeNESW:
        lPMId = SPTR_SIZENESW;
        break;
  case  sizeHorizontal:
        lPMId = SPTR_SIZEWE;
        break;
  case  sizeVertical:
        lPMId = SPTR_SIZENS;
        break;
  case  standardApplication:
        lPMId = SPTR_APPICON;
        fIconApi = false;
        break;
  case  information:
        lPMId = SPTR_ICONINFORMATION;
        fIconApi = false;
        break;
  case  question:
        lPMId = SPTR_ICONQUESTION;
        fIconApi = false;
        break;
  case  error:
        lPMId = SPTR_ICONERROR;
        fIconApi = false;
        break;
  case  warning:
        lPMId = SPTR_ICONWARNING;
        fIconApi = false;
        break;
  case  illegalOperation:
        lPMId = SPTR_ILLEGAL;
        break;
#ifndef IC_WIN
  case  singleFile:
        lPMId = SPTR_FILE;
        break;
  case  multipleFile:
        lPMId = SPTR_MULTFILE;
        break;
  case  folder:
        lPMId = SPTR_FOLDER;
        break;
  case  program:
        lPMId = SPTR_PROGRAM;
        break;
#endif  // !IC_WIN
} /* end switch */

  if (lPMId)
  {
    fhandle = (IPointerHandle::Value)
                IBitmapStaticPtr::systemPointer( lPMId, fMakeCopy, fIconApi );

    /* If we return a copy we will manage reference count for this resource */
    /* otherwise not (don't want to destroy a system pointer).          */
    if(fMakeCopy)
    {
      IBitmapStaticPtr::addPointerRef(fhandle);
    }
  }
  /* This is for icons that are not part of the operating system.  We
   * need to load them ourselves.  I have set up a switch statement so
   * that when we add more icons, it will be easily expandable.
   */
  else
#endif // IC_PMWIN
  {
#ifdef IC_WIN
    bool useShell32 = false;
#endif
    switch(pointerId)
    {
      case  dragFade:
        lPMId = IC_FADEICON;
        fMakeCopy = true;
        break;
#ifdef IC_MOTIF
      case  arrow:
         lPMId = IC_SYSPTR_ARROW;
         fMakeCopy = true;
         break;
      case  text:
         lPMId = IC_SYSPTR_TEXT;
         fMakeCopy = true;
         break;
      case  wait:
         lPMId = IC_SYSPTR_WAIT;
         fMakeCopy = true;
         break;
      case  move:
         lPMId = IC_SYSPTR_MOVE;
         fMakeCopy = true;
         break;
      case  information:
         lPMId = IC_SYSPTR_INFORMATION;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  question:
         lPMId = IC_SYSPTR_QUESTION;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  error:
         lPMId = IC_SYSPTR_ERROR;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  warning:
         lPMId = IC_SYSPTR_WARNING_ILLEGAL;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  illegalOperation:
         lPMId = IC_SYSPTR_WARNING_ILLEGAL;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  sizeHorizontal:
         lPMId = IC_SYSPTR_SIZEHORIZONTAL;
         fMakeCopy = true;
         break;
      case  sizeVertical:
         lPMId = IC_SYSPTR_SIZEVERTICAL;
         fMakeCopy = true;
         break;
#endif // IC_MOTIF
#ifdef IC_WIN
      // These we attempt to load from shell32.dll.  The resource Ids
      // are NOT documented, but were determined by experimentation.
      case  singleFile:
        lPMId = IC_FILE_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
      case  multipleFile:
        lPMId = IC_MULTIFILE_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
      case  folder:
        lPMId = IC_FOLDER_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
      case  program:
        lPMId = IC_PROGRAM_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
#endif // IC_WIN
    }   // switch
#ifdef IC_WIN
    if (useShell32)
       {
       // Try to load the icon from shell32.dll.  If that fails, then
       // punt and return the standard application icon.
       IPointerHandle ph(0);
       try
          {
          ph =
            IDynamicLinkLibrary(IC_SHELL32_DLL).tryToLoadIcon( lPMId,
                                                               fMakeCopy );
          }
       catch (...)
          {
          // this is to catch nonexistant DLL errors
          }

       if (ph)
          {
          // Got it.  return icon from shell32
          fhandle = ph;
          }
       else
          {
          // Not found.  This may happen on NT 3.51 without newshell
          fhandle = (IPointerHandle::Value)
             IBitmapStaticPtr::systemPointer( SPTR_APPICON,
                                              fMakeCopy,
                                              false );
          }
       }
    else
#endif
       {
       IPointerHandle ph;

       if (fUseLoadIcon) 
          ph = IApplication::current().resourceLibrary().loadIcon(lPMId);
       else
          ph = IApplication::current().resourceLibrary().loadPointer(lPMId);

       fhandle = ph;
       if(fMakeCopy)
         {
         IBitmapStaticPtr::addPointerRef(fhandle);
         }
       }
  }   // else nonstandard icons

}
#endif // IC_OBSOLETE 

/*------------------------------------------------------------------------------
| ISystemPointerHandle::ISystemPointerHandle                                   |
------------------------------------------------------------------------------*/
ISystemPointerHandle:: ISystemPointerHandle(EPointerType     pointerId,
                                            bool fMakeCopy)
   : IPointerHandle(0)
{
bool fUseLoadIcon = false;
long lPMId = 0;
#ifdef IC_PMWIN
// Flag for Windows macro to determine correct api to use in loading resource
bool fIconApi = true;
switch(pointerId)
{
  case  kArrow:
        lPMId = SPTR_ARROW;
        break;
  case  kText:
        lPMId = SPTR_TEXT;
        break;
  case  kWait:
        lPMId = SPTR_WAIT;
        break;
  case  kMove:
        lPMId = SPTR_MOVE;
        break;
  case  kSizeUpLeft:
  case  kSizeDownRight:
        lPMId = SPTR_SIZENWSE;
        break;
  case  kSizeUpRight:
  case  kSizeDownLeft:
        lPMId = SPTR_SIZENESW;
        break;
  case  kSizeLeft:
  case  kSizeRight:
  case  kSizeHorizontal:
        lPMId = SPTR_SIZEWE;
        break;
  case  kSizeUp:
  case  kSizeDown:
  case  kSizeVertical:
        lPMId = SPTR_SIZENS;
        break;
  case  kStandardApplication:
        lPMId = SPTR_APPICON;
        fIconApi = false;
        break;
  case  kInformation:
        lPMId = SPTR_ICONINFORMATION;
        fIconApi = false;
        break;
  case  kQuestion:
        lPMId = SPTR_ICONQUESTION;
        fIconApi = false;
        break;
  case  kError:
        lPMId = SPTR_ICONERROR;
        fIconApi = false;
        break;
  case  kWarning:
        lPMId = SPTR_ICONWARNING;
        fIconApi = false;
        break;
  case  kIllegalOperation:
        lPMId = SPTR_ILLEGAL;
        break;
#ifndef IC_WIN
  case  kSingleFile:
        lPMId = SPTR_FILE;
        break;
  case  kMultipleFile:
        lPMId = SPTR_MULTFILE;
        break;
  case  kFolder:
        lPMId = SPTR_FOLDER;
        break;
  case  kProgram:
        lPMId = SPTR_PROGRAM;
        break;
#endif  // !IC_WIN
} /* end switch */

  if (lPMId)
  {
    fhandle = (IPointerHandle::Value)
                IBitmapStaticPtr::systemPointer( lPMId, fMakeCopy, fIconApi );

    /* If we return a copy we will manage reference count for this resource */
    /* otherwise not (don't want to destroy a system pointer).          */
    if(fMakeCopy)
    {
      IBitmapStaticPtr::addPointerRef(fhandle);
    }
  }
  /* This is for icons that are not part of the operating system.  We
   * need to load them ourselves.  I have set up a switch statement so
   * that when we add more icons, it will be easily expandable.
   */
  else
#endif // IC_PMWIN
  {
#ifdef IC_WIN
    bool useShell32 = false;
#endif
    switch(pointerId)
    {
      case  kDragFade:
        lPMId = IC_FADEICON;
        fMakeCopy = true;
        break;
#ifdef IC_MOTIF
      case  kArrow:
         lPMId = IC_SYSPTR_ARROW;
         fMakeCopy = true;
         break;
      case  kText:
         lPMId = IC_SYSPTR_TEXT;
         fMakeCopy = true;
         break;
      case  kWait:
         lPMId = IC_SYSPTR_WAIT;
         fMakeCopy = true;
         break;
      case  kMove:
         lPMId = IC_SYSPTR_MOVE;
         fMakeCopy = true;
         break;
      case  kSizeUpLeft:
         lPMId = IC_SYSPTR_SIZEUPLEFT;
         fMakeCopy = true;
         break;
      case  kSizeUpRight:
         lPMId = IC_SYSPTR_SIZEUPRIGHT;
         fMakeCopy = true;
         break;
      case  kSizeDownLeft:
         lPMId = IC_SYSPTR_SIZEDOWNLEFT;
         fMakeCopy = true;
         break;
      case  kSizeDownRight:
         lPMId = IC_SYSPTR_SIZEDOWNRIGHT;
         fMakeCopy = true;
         break;
      case  kSizeLeft:
         lPMId = IC_SYSPTR_SIZELEFT;
         fMakeCopy = true;
         break;
      case  kSizeRight:
         lPMId = IC_SYSPTR_SIZERIGHT;
         fMakeCopy = true;
         break;
      case  kSizeUp:
         lPMId = IC_SYSPTR_SIZEUP;
         fMakeCopy = true;
         break;
      case  kSizeDown:
         lPMId = IC_SYSPTR_SIZEDOWN;
         fMakeCopy = true;
         break;
      case  kInformation:
         lPMId = IC_SYSPTR_INFORMATION;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  kQuestion:
         lPMId = IC_SYSPTR_QUESTION;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  kError:
         lPMId = IC_SYSPTR_ERROR;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  kWarning:
         lPMId = IC_SYSPTR_WARNING_ILLEGAL;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  kIllegalOperation:
         lPMId = IC_SYSPTR_WARNING_ILLEGAL;
         fUseLoadIcon = true;
         fMakeCopy = true;
         break;
      case  kSizeHorizontal:
         lPMId = IC_SYSPTR_SIZEHORIZONTAL;
         fMakeCopy = true;
         break;
      case  kSizeVertical:
         lPMId = IC_SYSPTR_SIZEVERTICAL;
         fMakeCopy = true;
         break;
#endif // IC_MOTIF
#ifdef IC_WIN
      // These we attempt to load from shell32.dll.  The resource Ids
      // are NOT documented, but were determined by experimentation.
      case  kSingleFile:
        lPMId = IC_FILE_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
      case  kMultipleFile:
        lPMId = IC_MULTIFILE_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
      case  kFolder:
        lPMId = IC_FOLDER_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
      case  kProgram:
        lPMId = IC_PROGRAM_SHELL32;
        fMakeCopy = false;
        useShell32 = true;
        break;
#endif // IC_WIN
    }   // switch
#ifdef IC_WIN
    if (useShell32)
       {
       // Try to load the icon from shell32.dll.  If that fails, then
       // punt and return the standard application icon.
       IPointerHandle ph(0);
       try
          {
          ph =
            IDynamicLinkLibrary(IC_SHELL32_DLL).tryToLoadIcon( lPMId,
                                                               fMakeCopy );
          }
       catch (...)
          {
          // this is to catch nonexistant DLL errors
          }

       if (ph)
          {
          // Got it.  return icon from shell32
          fhandle = ph;
          }
       else
          {
          // Not found.  This may happen on NT 3.51 without newshell
          fhandle = (IPointerHandle::Value)
             IBitmapStaticPtr::systemPointer( SPTR_APPICON,
                                              fMakeCopy,
                                              false );
          }
       }
    else
#endif
       {
       IPointerHandle ph;

       if (fUseLoadIcon) 
          ph = IApplication::current().resourceLibrary().loadIcon(lPMId);
       else
          ph = IApplication::current().resourceLibrary().loadPointer(lPMId);

       fhandle = ph;
       if(fMakeCopy)
         {
         IBitmapStaticPtr::addPointerRef(fhandle);
         }
       }
  }   // else nonstandard icons

}

/*------------------------------------------------------------------------------
| ISystemPointerHandle::~ISystemPointerHandle                                  |
------------------------------------------------------------------------------*/
ISystemPointerHandle :: ~ISystemPointerHandle( )
{
}


