// Revision: 75 1.8.1.4 source/ui/baseapp/ihelptbl.hpp, help, ioc.v400, 000919  
/* NOSHIP */
#ifndef _IHELPTBL_
#define _IHELPTBL_
/*******************************************************************************
* FILE NAME: ihelptbl.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the declaration(s) of the class(es):                    *
*     IHelpTable                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <irefcnt.hpp>
#include <imrefcnt.hpp>  //24527
#include <icntptr.hpp>   //24527
#include <ireslib.hpp>
#include <iresprv.hpp>
#include <iseq2.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)


class IHelpSubItem  {
/*******************************************************************************
| The IHelpSubItem  class is a help subtable entry.  Instances of this class   |
| are contained in an IHelpSubTable object.   This class currently does not    |
| support the ability to have additional integer values associated with each   |
| subitem as the PM implementation does.                                       |
*******************************************************************************/
public:
  IHelpSubItem  ( unsigned short id, unsigned short help ) :
      controlID( id ), helpID(help)  {};
  ~IHelpSubItem () {};
  IHelpSubItem ( )  : controlID(0), helpID(0) {}

  bool operator< ( const IHelpSubItem& subitem )  const
  {  return helpID < subitem.helpID; }
  bool operator== ( const IHelpSubItem& subitem ) const
  {  return helpID == subitem.helpID &&
            controlID == subitem.controlID; }

IString
  asString() const;

// instance data
  unsigned short   controlID;
  unsigned short   helpID;
}; // IHelpSubItem

// IHelpSubItemList is the list of IHelpSubItem objects contained within
// a particular help subtable.
typedef ISequence< IHelpSubItem> IHelpSubItemList;


//24527 class IHelpSubTable : public IRefCounted  {
class IHelpSubTable : public IMRefCounted  {
/*******************************************************************************
| The IHelpSubTable  class is a help subtable.  Each instance of this class    |
| is pointed to by the subTable entry in an IHelpTableItem object.             |
*******************************************************************************/
public:
  IHelpSubTable  ( unsigned short id );

IString
  asString() const;

// instance data
  unsigned short   subTableID;         // id of the subTable
  unsigned short   subItemSize;        // size of each subItem (always 2 now)
  IHelpSubItemList subItemList;        // list of subitems

#ifdef IC_MOTIF
void
  processHelpSubtableData( rthelpsubtable * pRtHelpsubtable );

IHelpSubTable
 &addItem                ( const IHelpSubItem& subitem );

#endif

//24527 protected:
  ~IHelpSubTable();
}; // IHelpSubTable

//24527 typedef IReference <IHelpSubTable> IHelpSubTableRef;
typedef ICountedPointerTo <IHelpSubTable> IHelpSubTableRef;

class IHelpTableItem {
/*******************************************************************************
| The IHelpTableItem class is an entry in the help table.  Its function        |
| is similar to the OS/2 toolkit structure HELPTABLE.                          |
*******************************************************************************/
public:
  IHelpTableItem ( const IHelpTableItem& helpTableItem);

  IHelpTableItem ( unsigned short    window,
                   IHelpSubTableRef  subtable,
                   unsigned short    helpID);
  IHelpTableItem ( ) {}
  ~IHelpTableItem();

  bool operator< ( const IHelpTableItem& item )  const
  {  return extendedHelpID < item.extendedHelpID; }
  bool operator== ( const IHelpTableItem& item ) const
  {  return extendedHelpID == item.extendedHelpID &&
            windowID == item.windowID; }

IHelpSubTable&
  subTable() const;

IString
  asString() const;

// instance data
  unsigned short    windowID;           // id of the application window
  IHelpSubTableRef  subTableRef;        // pointer to the HELPSUBTABLE
  unsigned short    extendedHelpID;     // res ID of the extended help panel
}; // IHelpTableItem

// IHelpTableItemList is the list of IHelpSubItem objects contained within
// a particular help subtable.
typedef ISequence< IHelpTableItem > IHelpTableItemList;


class IHelpTable  {
/*******************************************************************************
| The IHelpTable class represents a help table.   The basic function of a      |
| help table object is to map help requests from a specific control or menu    |
| item to a corresponding help panel ID.                                       |
*******************************************************************************/
public:

/*------------------------- Constructor/Destructor ----------------------------
| You can construct an IHelpTable from:                                        |
|    - An ID of the help table and a pointer to the resource library from      |
|      which the help table is to be read.                                     |
------------------------------------------------------------------------------*/
   IHelpTable           ( IResourceId tableId );
   ~IHelpTable          ( );

/*-------------------------------- Accessors -----------------------------------
| In either of the retrieval functions below, not found returns a value        |
| = IHelpTable::noHelpFound.                                                   |
|  extendedHelpForWindowID   - given a frame window ID, searches the           |
|                      help table and returns the extended help ID for         |
|                      the frame                                               |
|  helpForWindowID   - given a control ID and a frame window ID, searches the  |
|                      help table and returns the help panel ID for the        |
|                      control                                                 |
|  addItem           - adds a helpitem with the specified frame id,            |
|                      subtableid, and help ID                                 |
|  addSubItem        - adds a helpsubitem to the specified subtable            |
|                      for the controlid.                                      |
------------------------------------------------------------------------------*/
unsigned short
   extendedHelpForWindowID( unsigned short frameID );
unsigned short
   helpForWindowID( unsigned short controlID, unsigned short frameID);

//static const unsigned short noHelpFound;
enum helpRC { noHelpFound = 0xffff };

IHelpTable
 &addItem   ( unsigned short id,
              unsigned short subtableid,
              unsigned short helpid ),
#ifdef IC_MOTIF
 &addItem   ( const IHelpTableItem& helpItem ),
#endif
 &addSubItem( unsigned short subtableid,
              unsigned short controlid,
              unsigned short helpid );

#ifdef IC_MOTIF
void
  processHelpTableData( rthelptable * helpTableData );

IHelpSubTable
 *subtable( unsigned short subtableId );
#endif

IString
  asString() const;

private:
// copying is not allowed
  IHelpTable   (const IHelpTable&);
IHelpTable
  &operator=   (const IHelpTable&);

// instance data
  unsigned long        tableID;
  IHelpTableItemList   itemList;
  IModuleHandle        hInstance;
#ifdef IC_MOTIF
  IAixResourceImp*     aixResource;
#endif
}; // IHelpTable

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IHELPTBL_
