// Revision: 76 1.5.1.6 source/ui/baseapp/iiconhdr.cpp, staticctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: iiconhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iiconhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// #define IC_TRACE_ALL

extern "C"
  {
  #define INCL_GPIBITMAPS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINPOINTERS
  #define INCL_GPILOGCOLORTABLE
  #include <iwindefs.h>
  }

#include <iiconhdr.hpp>

  // NOTYET
  #include <ibmpstat.hpp>
  #include <ibmpdata.hpp>

#include <ibundles.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <igimage.hpp>
#include <igrport.hpp>
#include <grafdevs.hpp>
#include <iiconctl.hpp>
#include <iiconctp.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IICONHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IIconHandler::IIconHandler                                                   |
------------------------------------------------------------------------------*/
IIconHandler::IIconHandler() {;}

/*------------------------------------------------------------------------------
| IIconHandler::~IIconHandler                                                  |
------------------------------------------------------------------------------*/
IIconHandler::~IIconHandler() {;}


/*------------------------------------------------------------------------------
| IIconHandler::captureImage                                                   |
------------------------------------------------------------------------------*/
IGImage* IIconHandler::captureImage( IPresSpaceHandle& ps,
                                     const ISize&      size )
{
#ifdef IC_PMWIN
   // Get the previous contents of the screen for transparency.
   IGRect2D srcRect( 0, 0, size.width(), size.height() );
   IGImage* image = new IGImage( IGImage::captureImage( ps,srcRect ) );
   return image;
#endif //IC_PMWIN
#ifdef IC_MOTIF
   return Inherited::captureImage( ps, size );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| IIconHandler::drawImage                                                      |
------------------------------------------------------------------------------*/
bool IIconHandler::drawImage( IStaticText* window, IGImage& image )
{
  IMODTRACE_ALL("IIconHandler::drawImage");
  bool result(false);
  // Insure it is an icon control and that it has an icon set.
  IIconControl* icon = dynamic_cast<IIconControl*>( window );
  IPointerHandle hicon = icon ? icon->icon() : IPointerHandle();
  if ( ((IPointerHandle::Value)hicon != 0) &&
       ((IBitmapHandle::Value)icon->bitmap() == 0) )
  {
     ITRACE_ALL( IString("drawing hicon=") +
                 IString((unsigned long)(IPointerHandle::Value)hicon).d2x() );

     IGRect2D imageRect( image.rasterBounds() );

     ISize    imageSize = IIconControlData::iconSize( hicon );
     IGRect2D targetRect(
         targetRectangle( icon, imageRect,
                          IGRect2D(0,0, imageSize.width(), imageSize.height() )));
#ifdef IC_WIN
     // 32 bit Windows.  Use system API to render.
     HDC     hdc = image.grafPort()->device()->deviceContext();
     HBITMAP bmp = image.handle();
     HGDIOBJ oldbmp = SelectObject( hdc, bmp );
     if (!DrawIconEx(hdc,
                     (int)targetRect.fLeft,
                     (int)targetRect.fTop,
                     hicon,
                     (int)targetRect.width(),
                     (int)targetRect.height(),
                     0,0, DI_NORMAL ) )
        ITHROWGUIERROR("DrawIconEx");

     SelectObject(hdc, oldbmp);
     result = true;
#endif //IC_WIN



#ifdef IC_PM
     POINTERINFO     pointerinfo;
     IQUERYPOINTERINFO(hicon,&pointerinfo);
     IGImage         bmMask ( IBitmapHandle(pointerinfo.hbmPointer) );

     ISize bmSize  = ISize( bmMask.rasterBounds().width(),
                            bmMask.rasterBounds().height() );
     // set up location of AND mask (upper part of bitmap)
     // PM... XOR mask present in all icons
     long bmHeight = bmSize.height() * .5;
     IGRect2D srcRect( 0, 0, bmSize.width(), bmHeight);
     // Move to AND mask in upper part of bitmap.
     srcRect.offset( IGPoint2D(0, bmHeight) );

     //Set up the AND MASK colors
     // Can't get hps attributes from a grafport...
     IGrafPort& port( *(image.grafPort()) );
     IPresSpaceHandle hps = port.device()->deviceContext();
     LONG lcolor  = GpiQueryColor( hps );
     LONG lbcolor = GpiQueryBackColor( hps );
     GpiSetColor( hps, CLR_WHITE );
     GpiSetBackColor( hps, CLR_BLACK );

     IGrafBundle bundle;
     {
        IGrafMatrix matrix( srcRect, targetRect );
        bundle.setImageTransferMode( IImageTransferMode::kSrcANDDst );
        port.draw( bmMask, srcRect, bundle, matrix );
     }

     //The following values are for the color bitmap and the XOR MASK
     srcRect.offset( IGPoint2D(0, -bmHeight) );

     IGrafMatrix matrix( srcRect, targetRect );
     // Paint the color bitmap if there is one.
     if ( pointerinfo.hbmColor != 0 )
     {
        // restore original colors
        GpiSetBackColor( hps, lbcolor );
        GpiSetColor( hps, lcolor );

        // draw the color bitmap
        IGImage   bmColor(
          IBitmapHandle(pointerinfo.hbmColor) );

        bundle.setImageTransferMode( IImageTransferMode::kSrcORDst );
        port.draw( bmColor, srcRect, bundle, matrix );
        GpiSetBackColor( hps, CLR_BLACK );
        GpiSetColor( hps, CLR_WHITE );
     } // Color

     bundle.setImageTransferMode( IImageTransferMode::kSrcXORDst );
     port.draw( bmMask, srcRect, bundle, matrix );

     // restore original colors
     GpiSetBackColor( hps, lbcolor );
     GpiSetColor( hps, lcolor );

     result = true;
#endif //IC_PM

#ifdef IC_MOTIF
     // Paint the entire background with background color so that
     // transparency works. A background other than the background color
     // will not show up, however.
     IFillBundle fillBundle( icon->fillColor() );
     image.grafPort()->draw( imageRect, fillBundle );

     //Construct the IGImage to draw.
     IPointerData* node = IBitmapStaticPtr::pointerSet().find( hicon );
     if (node)
     {
        IGImage       bitmapImage( node->handle() );
        IGRect2D      bitmapRect (bitmapImage.rasterBounds());
        if (bitmapRect != IGRect2D() )
        {
           IGrafBundle bundle;
           IGrafMatrix matrix( bitmapRect, targetRect );
           if ( node->maskImage() != 0 )   //mask pixmap invalid if no XImage
           {
              IBitmapHandle maskbmp ( node->maskPixmap() );
              IGImage       maskImage  (maskbmp);
              maskImage.setTransparencyColor( IBaseColor( IBaseColor::kWhite ) );
              maskImage.setTransparencyFlag ( true );
              image.grafPort()->draw( bitmapImage, maskImage, bundle, matrix );
           }
           else
           {
              image.grafPort()->draw( bitmapImage, bundle, matrix );
           }
        }
     }
     result = true;
#endif //IC_MOTIF
  }
  else
  {
     // No icon set.  Use base class drawing.
     result = Inherited::drawImage( window, image );
  }
  return result;
}
