// Revision: 84 1.2.2.2 source/ui/baseapp/ilbdievt.cpp, listctls, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ilbdievt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilbdievt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <stdio.h>
  #define INCL_WINMENUS
  #include <iwindefs.h>
  }

#include <ilbdievt.hpp>

#ifndef  _IRECT_
  #include <irect.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILBDIEVT_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
  #define OIDATA ((LPDRAWITEMSTRUCT)ownerItemData())
#else
  #define OIDATA ((POWNERITEM)ownerItemData())
#endif

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::IListBoxDrawItemEvent                                 |
|                                                                              |
| Construct IListBoxDrawItemEvent object from an IEvent instance               |
------------------------------------------------------------------------------*/
IListBoxDrawItemEvent :: IListBoxDrawItemEvent ( IEvent& event )
  : IListBoxDrawItemEvent::Inherited( event ),
    fSelectionStateDrawn( false )
{ }

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::~IListBoxDrawItemEvent                                |
|                                                                              |
| Empty destructor here for page tuning                                        |
------------------------------------------------------------------------------*/
IListBoxDrawItemEvent :: ~IListBoxDrawItemEvent ( )
{ }

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isDrawItem                                            |
|                                                                              |
| Test to see if the event is for drawing the item.                            |
------------------------------------------------------------------------------*/
bool IListBoxDrawItemEvent::isDrawItem ( ) const
{
#ifdef IC_WIN
  return (OIDATA->itemAction & ODA_DRAWENTIRE) ? true : false;
#else
  return (OIDATA->fsState == OIDATA->fsStateOld) ? true : false;
#endif
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isSelected                                            |
------------------------------------------------------------------------------*/
bool IListBoxDrawItemEvent::isSelected ( ) const
{
#ifdef IC_WIN
  return (OIDATA->itemState & ODS_SELECTED) ? true : false;
#else
  return (OIDATA->fsState) ? true : false;
#endif
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isSetHighlight                                        |
|                                                                              |
| Test to see if the event is for highlighting the item.                       |
------------------------------------------------------------------------------*/
bool IListBoxDrawItemEvent::isSetHighlight ( ) const
{
#ifdef IC_WIN
  return ((OIDATA->itemAction & ODA_SELECT) &&
          (OIDATA->itemState & ODS_SELECTED)) ? true : false;
#else
  return (OIDATA->fsState && !OIDATA->fsStateOld) ? true : false;
#endif
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isRemoveHighlight                                     |
------------------------------------------------------------------------------*/
bool IListBoxDrawItemEvent::isRemoveHighlight ( ) const
{
#ifdef IC_WIN
  return ((OIDATA->itemAction & ODA_SELECT) &&
          !(OIDATA->itemState & ODS_SELECTED)) ? true : false;
#else
  return (!OIDATA->fsState && OIDATA->fsStateOld) ? true : false;
#endif
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::setSelectionStateDrawn                                |
------------------------------------------------------------------------------*/
IListBoxDrawItemEvent&
  IListBoxDrawItemEvent::setSelectionStateDrawn ( bool drawn )
{
  fSelectionStateDrawn = drawn;
  return *this;
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isSelectionStateDrawn                                 |
------------------------------------------------------------------------------*/
bool IListBoxDrawItemEvent::isSelectionStateDrawn  ( ) const
{
  return fSelectionStateDrawn;
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::IListBoxSizeItemEvent                                 |
------------------------------------------------------------------------------*/
IListBoxSizeItemEvent::IListBoxSizeItemEvent ( IEvent& event )
  : IListBoxSizeItemEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::~IListBoxSizeItemEvent                                |
------------------------------------------------------------------------------*/
IListBoxSizeItemEvent::~IListBoxSizeItemEvent ( )
{ }

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::itemIndex                                             |
------------------------------------------------------------------------------*/
unsigned long IListBoxSizeItemEvent::itemIndex ( ) const
{
#ifdef IC_PM
  return this->parameter2().number1();
#endif
#ifdef IC_WIN
  return ((LPMEASUREITEMSTRUCT)(void*)this->parameter2())->itemID;
#endif
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::itemSize                                              |
------------------------------------------------------------------------------*/
ISize IListBoxSizeItemEvent::itemSize ( ) const
{
#ifdef IC_PM
  return ISize( this->result().number2(), this->result().number1() );
#endif
#ifdef IC_WIN
  return  ISize(
             ((LPMEASUREITEMSTRUCT)(void*)this->parameter2())->itemWidth,
             ((LPMEASUREITEMSTRUCT)(void*)this->parameter2())->itemHeight);
#endif
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::setItemSize                                           |
------------------------------------------------------------------------------*/
IListBoxSizeItemEvent&
  IListBoxSizeItemEvent::setItemSize ( const ISize& itemSize )
{
#ifdef IC_PM
  this->setResult( IEventResult( (unsigned short)itemSize.height(),
                                 (unsigned short)itemSize.width() ));
#endif
#ifdef IC_WIN
  MEASUREITEMSTRUCT* pmis = (LPMEASUREITEMSTRUCT)(void*)this->parameter2();
  pmis->itemHeight = itemSize.height();
  pmis->itemWidth  = itemSize.width();
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::setResult                                             |
------------------------------------------------------------------------------*/
IEvent& IListBoxSizeItemEvent::setResult ( const IEventResult& eventResult )
{
  return Inherited::setResult( eventResult );
}
