// Revision: 89 1.4.1.3 source/ui/baseapp/imbpriv.hpp, views, ioc.v400, 000919  
/* NOSHIP */
#ifndef _IMBPRIV_
#define _IMBPRIV_
/*******************************************************************************
* FILE NAME: imbpriv.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private to the       *
*   implementation of IMessageBox.                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <imsgbox.hpp>                  // IMessageBox
#include <icconst.h>                    // Application Constants
#include <icmdhdr.hpp>                  // ICommandHandler
#include <iframe.hpp>                   // IFrameWindow
#include <iiconctl.hpp>                 // IIconControl
#include <imcelcv.hpp>                  // IMultiCellCanvas
#include <ipushbut.hpp>                 // IPushButton
#include <isetcv.hpp>                   // ISetCanvas
#include <istattxt.hpp>                 // IStaticText
#include <ifont.hpp>                    // IFont

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

/*------------------------------------------------------------------------------
| Private defines.  Constants and message box control IDs.                     |
------------------------------------------------------------------------------*/
const unsigned MB_MESSAGEID_LIMIT (12);
const unsigned MB_MAIN_MCCANVAS   (IC_ID_BASE + 0x1000);
#ifdef IC_WIN
const unsigned MB_BUTTON_MCCANVAS (IC_ID_BASE + 0x1001);
#endif // IC_WIN
const unsigned MB_BUTTON_SETCANVAS(IC_ID_BASE + 0x1002);
const unsigned MB_ICON_SETCANVAS  (IC_ID_BASE + 0x1003);
const unsigned MB_ICON            (IC_ID_BASE + 0x1004);
const unsigned MB_MESSAGE_TEXT    (IC_ID_BASE + 0x1005);
const unsigned MB_MESSAGE_ID      (IC_ID_BASE + 0x1006);
#ifdef IC_PM
const unsigned MB_STATIC_TITLE    (IC_ID_BASE + 0x1007);
#endif // IC_PM

/*------------------------------------------------------------------------------
| The IMessageBoxPushButtonHandler class handles button presses on the message |
| box and returns the correct MB_XXX value.                                    |
------------------------------------------------------------------------------*/
class IMessageBoxPushButtonHandler : public ICommandHandler
{
  public:
    IMessageBoxPushButtonHandler(IFrameWindow* ownerWindow,
                                 IMessageBox::Style mbStyle)
      : window(ownerWindow),
        messageBoxStyle(mbStyle)
        {;}
  protected:
    virtual bool
      command( ICommandEvent& evt );
  private:
    IFrameWindow
     *window;
    IMessageBox::Style
      messageBoxStyle;
};

class IMessageBoxData  {
/*------------------------------------------------------------------------------
| The IMessageBoxData class encapsulates private data and functions            |
| used by the IMessageBox class.  An object of this class is created in the    |
| IMessageBox constructors.                                                    |
------------------------------------------------------------------------------*/
public:
  IMessageBoxData ( );

virtual
 ~IMessageBoxData ( );

// Visual components of the message box.
IFrameWindow
 *messageBox;

IIconControl
 *msgIcon;

IMultiCellCanvas
 *clientCanvas;

// MCCanvas to hold the buttons in Windows so they can be centered.
#ifdef IC_WIN
IMultiCellCanvas
 *buttonMCCanvas;
#endif // IC_WIN

IPushButton
 *pushButtons[4];

ISetCanvas
 *buttonSetCanvas,
 *iconSetCanvas;

IStaticText
 *messageText,
 *messageId;

// Static title for OS/2 message boxes with no title bar.
#ifdef IC_PM
IStaticText
 *staticTitle;
#endif // IC_PM

// Handler for the pushbuttons
IMessageBoxPushButtonHandler
 *pushButtonHandler;

// String for the message id.  Message text is in the IMessageBox class.
IString
  strMessageId;

// Global data.
bool
   useIcon
#ifdef IC_PM
   ,useStaticTitle
#endif
   ;

unsigned long
  numberOfLines,
  aveWidth;

// The message box font.
IFont
  font;
}; // IMessageBoxData

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif


/*------------------------------------------------------------------------------
| IMessageBoxData::IMessageBoxData                                             |
| Private data for the IMessageBox class.  Contains all controls that make up  |
| the message box plus handlers.                                               |
| Initialize all pointers to zero.  They are allocated when the message box is |
| shown.                                                                       |
------------------------------------------------------------------------------*/
IMessageBoxData::IMessageBoxData ( )
  : msgIcon(0)
    ,clientCanvas(0)
#ifdef IC_WIN
    ,buttonMCCanvas(0)
#endif // IC_WIN
    ,buttonSetCanvas(0)
    ,messageText(0)
    ,messageId(0)
    ,pushButtonHandler(0)
    ,strMessageId()
#ifdef IC_PM
    ,staticTitle(0)
    ,useStaticTitle(false)
#endif // IC_PM
    ,useIcon(false)
    ,numberOfLines(0)
    ,aveWidth(0)
#ifdef IC_PM
    ,font()
#endif // IC_PM
#ifdef IC_WIN
    ,font("MS Sans Serif", 8)
#endif // IC_WIN
{ }

/*------------------------------------------------------------------------------
| IMessageBoxData::~IMessageBoxData                                            |
| Delete only the pushbutton handler.  All controls are auto delete object.    |
------------------------------------------------------------------------------*/
IMessageBoxData::~IMessageBoxData ( )
{
   if (pushButtonHandler)
      delete pushButtonHandler;
}


#endif
