// Revision: 08 1.3.2.10 source/ui/baseapp/imndihdr.cpp, menu, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: imndihdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imndihdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

// #define IC_TRACE_ALL

extern "C"
{
  #include <stdio.h>
  #define INCL_WINMENUS
  #include <iwindefs.h>
}

#include <imndihdr.hpp>
#include <icconst.h>
#include <iwindow.hpp>
#include <imenu.hpp>
#include <imenuprv.hpp>
#ifdef IC_MOTIF
#include <ixdc.hpp>
#include <ibcolor.hpp>
#include <icolmap.hpp>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/MenuShell.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMNDIHDR_CPP_
  #include <ipagetun.h>
#endif

const IMenuDrawItemHandler::DrawFlag
  IMenuDrawItemHandler::drewChecked   ( 0, IDIF_CHECKED ),
  IMenuDrawItemHandler::drewDisabled  ( 0, IDIF_DISABLED ),
  IMenuDrawItemHandler::drewSelected  ( 0, IDIF_SELECTED );

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::dispatchHandlerEvent                                   |
------------------------------------------------------------------------------*/
bool IMenuDrawItemHandler::dispatchHandlerEvent(IEvent& evt)
{
   IMenuDrawItemEvent mndievt(evt);
   switch (evt.eventId()) {
   case WM_MEASUREITEM:
   {
      ISize siz(1,1);
      if (setSize(mndievt, siz) ) {
#ifdef IC_WIN
         ((LPMEASUREITEMSTRUCT)(void*)evt.parameter2())->itemWidth =
                                                           siz.width();
         ((LPMEASUREITEMSTRUCT)(void*)evt.parameter2())->itemHeight =
                                                           siz.height();
#else
         ((POWNERITEM)(void*)evt.parameter2())->rclItem.xRight = siz.width();
         ((POWNERITEM)(void*)evt.parameter2())->rclItem.yTop = siz.height();
#endif
         return true;
      } /* endif */
      break;
   } /* endcase */
   case WM_DRAWITEM:
      if (mndievt.isDrawItem()) {
         IMenuDrawItemHandler::DrawFlag flag = 0;

         if (draw(mndievt, flag) ) {
            evt.setResult(true);

#ifndef IC_WIN
// In Windows, the ownerdraw data is passed for information purposes
// only and does not require modification to complete drawing actions
            flag & IMenuDrawItemHandler::drewChecked ?
             ((POWNERITEM)(void*)evt.parameter2())->fsAttribute =
             ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                        ~MIA_CHECKED : 0;

            flag & IMenuDrawItemHandler::drewSelected ?
             ((POWNERITEM)(void*)evt.parameter2())->fsAttribute =
             ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                         ~MIA_HILITED : 0;

            flag & IMenuDrawItemHandler::drewDisabled ?
             ((POWNERITEM)(void*)evt.parameter2())->fsAttribute =
             ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                        ~MIA_DISABLED : 0;
#endif
            return true;
         } /* endif */
      }
      else
         if (mndievt.isSetHighlight()) {
            if(highlight(mndievt) ) {
               evt.setResult(true);
#ifndef IC_WIN
// In Windows, the ownerdraw data is passed for information purposes
// only and does not require modification to complete drawing actions
               ((POWNERITEM)(void*)evt.parameter2())->fsAttribute &=
                                                                  ~MIA_HILITED;
               ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                                                  ~MIA_HILITED;
#endif
               return true;
            }
         }
         else
           if (mndievt.isRemoveHighlight()) {
              if(unhighlight(mndievt) ) {
                 evt.setResult(true);
#ifndef IC_WIN
// In Windows, the ownerdraw data is passed for information purposes
// only and does not require modification to complete drawing actions
                 ((POWNERITEM)(void*)evt.parameter2())->fsAttribute &=
                                                                  ~MIA_HILITED;
                 ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                                                  ~MIA_HILITED;
#endif
                 return true;
              }
           }

      break;
   } /* endswitch */

   return false;
}
#endif // IC_PMWIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::dispatchHandlerEvent                                   |
------------------------------------------------------------------------------*/
bool IMenuDrawItemHandler::dispatchHandlerEvent(IEvent& evt)
{
  IMenuDrawItemEvent mndievt(evt);
  switch (evt.eventId())
  {
    case IC_UM_MEASUREITEM:
    {
      ITRACE_DEVELOP("Get the messure item");
      ISize siz(1,1);
      if (setSize(mndievt, siz) )
      {
        Widget tmp = (Widget) evt.parameter1().asUnsignedLong();
        XtVaSetValues( (Widget)tmp,
                       XmNwidth, siz.width(),
                       XmNheight, siz.height(),
                       XmNrecomputeSize, false,
                       NULL );
      }
    }
    break;

    case IC_UM_DRAWITEM:
    {
      ITRACE_DEVELOP("Got a draw event");
      if (mndievt.isDrawItem())
      {
        IMenuDrawItemHandler::DrawFlag flag = 0;
        bool result = draw(mndievt, flag);   //call override to let client do drawing
        ITRACE_ALL(IString("draw result=") + IString(result ? "true" : "false") +
                   IString("drawFlag=") + flag.asString()) ;
        if (result)
          evt.setResult(true);
        //********************************************************************************
        // Client code has finished drawing. Now we draw the cascade arrow for those
        //  items which have a cascade (submenu), except for top-level menubar items.
        //********************************************************************************
        drawArrowOnCascades(mndievt);
        // Return whatever was returned by the call to "draw"
        return result;
      }
      else if (mndievt.isSetHighlight())
      {
        bool result = highlight(mndievt);   //call override to let client draw highlight
        ITRACE_ALL(IString("highlight result=") + IString(result ? "true" : "false") );
        if (result)
          evt.setResult(true);
        //********************************************************************************
        // Client code has finished drawing. Now we draw the cascade arrow for those
        //  items which have a cascade (submenu), except for top-level menubar items.
        //********************************************************************************
        drawArrowOnCascades(mndievt);
        // Return whatever was returned by the call to "highlight"
        return result;
      }
      else if (mndievt.isRemoveHighlight())
      {
        bool result = unhighlight(mndievt);   //call override to let client undraw highlight
        ITRACE_ALL(IString("unhighlight result=") + IString(result ? "true" : "false") );
        if (result)
          evt.setResult(true);
        //********************************************************************************
        // Client code has finished drawing. Now we draw the cascade arrow for those
        //  items which have a cascade (submenu), except for top-level menubar items.
        //********************************************************************************
        drawArrowOnCascades(mndievt);
        // Return whatever was returned by the call to "unhighlight"
        return result;
      }
    }
    break;
  }
  return false;
}


/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::drawArrowOnCascades       PRIVATE      27005
|
| If the menu item involved with mndievt is a cascade (has a submenu) and is not
| a top-level menubar item, draw an arrow similar to a non-owner draw cascade button.
------------------------------------------------------------------------------*/
void IMenuDrawItemHandler::drawArrowOnCascades( IMenuDrawItemEvent mndievt )
{
  // See if this (owner-draw) item has a submenu (check at the Motif level,
  //   since we don't have its parent IMenu)
  Widget itemHWND = (Widget)mndievt.parameter1().asUnsignedLong();
  Widget subMenuId = 0;
  XtVaGetValues( itemHWND,
                 XmNsubMenuId, &subMenuId,
                 NULL );
  // Find out if this item's parent is the menubar
  bool isTopLevelMenubarItem = false;
  Widget itemParent = XtParent(itemHWND);
  unsigned char rcType;
  if ( XmIsRowColumn(itemParent) ) {
    XtVaGetValues( itemParent,
                   XmNrowColumnType, &rcType,
                   NULL);
    if ( rcType == XmMENU_BAR )
      isTopLevelMenubarItem = true;
  } //endif XmIsRowColumn

  // If this (owner-draw) item has a submenu and is NOT a top-level menubar item,
  //       draw the arrow
  if ( subMenuId &&
       !isTopLevelMenubarItem ) {
    IXDC *dc = (IXDC*) mndievt.itemPresSpaceHandle();
    Display *dpy = dc->xDisplay;
    Drawable win = dc->xDrawable;
    GC gc = dc->gc();
    // Get current value for "foreground", so I can restore it later
    XGCValues gcValues;
    XGetGCValues(dpy, gc, GCForeground , &gcValues);  //reuse

    IRectangle rect = mndievt.itemRect();
    //render right rectangle
    //Approximately calculate the position and coordinates
    int x=rect.maxX()-15, y=4;
    XPoint pt[3];
    pt[0].x = x;
    pt[0].y = y+12;
    pt[1].x = x;
    pt[1].y = y;
    pt[2].x = x+10;
    pt[2].y = y+6;
    XPoint ptUnderline[2];
    ptUnderline[0] = pt[0];
    ptUnderline[1] = pt[2];

    // From the default colormap, obtain the base arrow color (pale grey) and
    //   the light and dark shadow colors (white and black, respectively).
    IColorMap colorMap = IColorMap::defaultColorMap();
    IBaseColor baseArrow(160, 160, 160);    // grey, between kPaleGrey and kDarkGrey
    IBaseColor topShadow(IBaseColor::kWhite);
    IBaseColor bottomShadow(IBaseColor::kBlack);
    if ( mndievt.isSetHighlight() )
    {
      IBaseColor temp = topShadow;
      topShadow = bottomShadow;
      bottomShadow = temp;
    }
    // Convert our three colors into their respective color indices, as X prefers
    unsigned long baseArrowUL = colorMap.colorIndex(baseArrow);
    unsigned long topShadowUL = colorMap.colorIndex(topShadow);
    unsigned long bottomShadowUL = colorMap.colorIndex(bottomShadow);

   ITRACE_ALL(IString("drawArrowOnCascades") +
              IString(" isSetHighlight=") +
              IString(mndievt.isSetHighlight() ? "true" : "false") +
              IString(" isRemoveHighlight=") +
              IString(mndievt.isRemoveHighlight() ? "true" : "false") +
              IString(" baseArrowUL=") + IString(baseArrowUL).d2x() +
              IString(" topShadowUL=") + IString(topShadowUL).d2x() +
              IString(" bottomShadowUL=") + IString(bottomShadowUL).d2x() );

    //set the appropriate foreground color for the triangle & shadows, draw each.
    //    base arrow
    XSetForeground(dpy, gc, baseArrowUL);
    XFillPolygon(dpy, win, gc, pt, 3, Convex, CoordModeOrigin);
    //    2-segment shadow overline
    XSetForeground(dpy, gc, topShadowUL);
    XDrawLines(dpy, win, gc, pt, 3, CoordModeOrigin);
    //    one-segment shadow underline
    XSetForeground(dpy, gc, bottomShadowUL);
    XDrawLines(dpy, win, gc, ptUnderline, 2, CoordModeOrigin);
    // Restore the original foreground color value, for later use by the rest of IOC
    XSetForeground(dpy, gc, gcValues.foreground);       //reuse
  } //endif "owner-draw item with submenu"
  return;
}
#endif // IC_MOTIF


/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::IMenuDrawItemHandler                                   |
------------------------------------------------------------------------------*/
IMenuDrawItemHandler::IMenuDrawItemHandler()
{}


/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::~IMenuDrawItemHandler                                  |
------------------------------------------------------------------------------*/
IMenuDrawItemHandler::~IMenuDrawItemHandler()
{
}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::setSize                                                |
------------------------------------------------------------------------------*/
bool IMenuDrawItemHandler::setSize(IMenuDrawItemEvent& evt, ISize& newSize)
{ return false; }

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::draw                                                   |
------------------------------------------------------------------------------*/
bool IMenuDrawItemHandler::draw(IMenuDrawItemEvent& evt, DrawFlag& flag)
{return false;}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::highlight                                              |
------------------------------------------------------------------------------*/
bool IMenuDrawItemHandler::highlight(IMenuDrawItemEvent& evt)
{return false;}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::unhighlight                                            |
------------------------------------------------------------------------------*/
bool IMenuDrawItemHandler::unhighlight(IMenuDrawItemEvent& evt)
{return false;}
