// Revision: 15 1.5.1.2 source/ui/baseapp/imoushdr.c, userinput, ioc.v400, 000919 
#ifndef _IMOUSHDR_C_
#define _IMOUSHDR_C_
/*******************************************************************************
* FILE NAME: imoushdr.c                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of the template functions                *
*   declared in imoushdr.hpp.                                                  * 
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*                                                                              *
*******************************************************************************/
#include <imoushdr.hpp>

template<class ATarget>
IMouseClickConnectionTo<ATarget>::IMouseClickConnectionTo
                                    ( ATarget&            target,
                                      ClickMemberFunction clickMemberFunction )
  : IMouseClickConnectionTo<ATarget>::Inherited( IMouseHandler::noMouseMoves ),
    fTarget( target ),
    fConnectedClickMember( clickMemberFunction )
{ }

template<class ATarget>
IMouseClickConnectionTo<ATarget>::~IMouseClickConnectionTo ( )
{ }

template<class ATarget>
bool IMouseClickConnectionTo<ATarget>::mouseClicked ( IMouseClickEvent& event )
{
  if ( fConnectedClickMember )
     return ( fTarget.*fConnectedClickMember ) ( event );
  else
     return false;
}

template<class ATarget>
IMouseMoveConnectionTo<ATarget>::IMouseMoveConnectionTo
                                   ( ATarget&            target,
                                     MoveMemberFunction  moveMemberFunction,
                                     EnterMemberFunction enterMemberFunction,
                                     LeaveMemberFunction leaveMemberFunction )
  : IMouseMoveConnectionTo<ATarget>::Inherited
                                   ( styleForFunctions( moveMemberFunction,
                                                        enterMemberFunction,
                                                        leaveMemberFunction ) ),
    fTarget( target ),
    fConnectedMoveMember( moveMemberFunction ),
    fConnectedEnterMember( enterMemberFunction ),
    fConnectedLeaveMember( leaveMemberFunction )
{ }

template<class ATarget>
IMouseMoveConnectionTo<ATarget>::~IMouseMoveConnectionTo ( )
{ }

template<class ATarget>
bool IMouseMoveConnectionTo<ATarget>::mouseMoved ( IMouseEvent& event )
{
  if ( fConnectedMoveMember )
     return ( fTarget.*fConnectedMoveMember ) ( event );
  else
     return false;
}

template<class ATarget>
bool IMouseMoveConnectionTo<ATarget>::mouseEnter ( IEvent& event )
{
  if ( fConnectedEnterMember )
     return ( fTarget.*fConnectedEnterMember ) ( event );
  else
     return false;
}

template<class ATarget>
bool IMouseMoveConnectionTo<ATarget>::mouseLeave ( IEvent& event )
{
  if ( fConnectedLeaveMember )
     return ( fTarget.*fConnectedLeaveMember ) ( event );
  else
     return false;
}

template<class ATarget>
IMouseHandler::Style IMouseMoveConnectionTo<ATarget>::styleForFunctions
                                   ( MoveMemberFunction  moveMemberFunction,
                                     EnterMemberFunction enterMemberFunction,
                                     LeaveMemberFunction leaveMemberFunction )
{
  Style
    style( IMouseHandler::noMouseMoves );
  if ( moveMemberFunction )
  {
     // Enable the handler to receive mouse move events.  Request the type
     // of moves specified by the default class style.
     if ( IMouseHandler::defaultStyle() & IMouseHandler::allMouseMoves ) 
     {
        style |= IMouseHandler::allMouseMoves;
     }
     else
     {
        style |= IMouseHandler::someMouseMoves;
     }
     style &= ~IMouseHandler::noMouseMoves;
  }

  if ( enterMemberFunction || leaveMemberFunction )
  {
     // Enable the handler to receive mouse enter and leave events.
     style |= IMouseHandler::mouseEntersLeaves;
  }

  return style;
}


#if (IC_OBSOLETE <= IC_OBSOLETE_3)
#ifdef IC_PMWIN

template<class ATarget>
IMouseConnectionTo<ATarget>::IMouseConnectionTo( ATarget&              target,
                                                 ClickMemberFunction   clickMemberFunction,
                                                 MoveMemberFunction    moveMemberFunction, 
                                                 PointerMemberFunction pointerMemberFunction )
: fTarget(target), 
  fConnectedClickMember(clickMemberFunction),
  fConnectedMoveMember(moveMemberFunction), 
  fConnectedPointerMember(pointerMemberFunction) 
{ }

template<class ATarget>
IMouseConnectionTo<ATarget>::~IMouseConnectionTo()
{ }

template<class ATarget>
bool IMouseConnectionTo<ATarget>::mouseClicked ( IMouseClickEvent& event )
{
  if ( fConnectedClickMember )
     return ( fTarget.*fConnectedClickMember ) ( event );
  else
     return false;
}
    
template<class ATarget>
bool
IMouseConnectionTo<ATarget>::mouseMoved ( IMouseEvent& event )
{
    if ( fConnectedMoveMember )
       return ( fTarget.*fConnectedMoveMember ) ( event );
    else
       return false;
}
    
template<class ATarget>
bool
IMouseConnectionTo<ATarget>::mousePointerChange( IMousePointerEvent& event )
{   
    if ( fConnectedPointerMember )
       return ( fTarget.*fConnectedPointerMember ) ( event );
    else
       return false;
}

#endif // IC_PMWIN
#endif // IC_OBSOLETE

#endif // _IMOUSHDR_C_
