// Revision: 32 1.10 source/ui/baseapp/ipagehdr.cpp, notebook, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ipagehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IPageHandler                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO.
#define INCL_WINHELP
#define INCL_WINSTDBOOK
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #ifdef IC_PMWIN
    #include <stdlib.h>
  #endif //IC_PMWIN

  #ifdef IC_WIN
    #ifndef _WIN32
      #define _WIN32
    #endif
    #include <commctrl.h>
  #endif
}

#ifdef IC_MOTIF
  #include <notebk.h>
#endif //IC_MOTIF
#ifdef IC_WIN
  #include <iclnbw.h>
#endif

#include <ipagehdr.hpp>
#include <ictlevt.hpp>
#include <ihelp.hpp>
#include <inbdievt.hpp>
#include <inotebk.hpp>
#include <ipageevt.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>
#include <icconst.h>

#ifdef IC_PMWIN
  #include <iwcname.hpp>
  #include <ireslib.hpp>
#endif //IC_PMWIN

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IPAGEHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IPageHandler::IPageHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IPageHandler:: IPageHandler ( )
#ifdef IC_PMWIN
  : IPageHandler::Inherited( )
#endif
{ }

/*------------------------------------------------------------------------------
| IPageHandler::~IPageHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IPageHandler::~IPageHandler ( )
{ }

/*------------------------------------------------------------------------------
| IPageHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
bool IPageHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

#ifdef IC_PMWIN
  switch ( event.eventId() )
  {
     case WM_CONTROL:
     case WM_DRAWITEM:
     {                                 // Control/draw-item event.
        IControlEvent ctlevt( event );
        IWindowHandle hwndControl = ctlevt.controlHandle();

#ifdef IC_WIN
        if ( hwndControl  &&
            (( IWindowClassName( hwndControl ) == WC_TABCONTROL ) ||
            ( IWindowClassName( hwndControl ) == WC_NOTEBOOK )) )
#endif
#ifdef IC_PM
        if ( hwndControl  &&
            ( IWindowClassName( hwndControl ) == WC_NOTEBOOK ) )
#endif
        {                              // Notebook case.
           if ( event.eventId() == WM_CONTROL )
           {
              switch ( ctlevt.parameter1().number2() )
              {
                 case BKN_NEWPAGESIZE:
                 {
                    IPageEvent pageevt( ctlevt );
                    bRc = this->resize( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 case BKN_PAGESELECTED:
                 {
                    IPageSelectEvent pageevt( ctlevt );
                    bRc = this->select( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 case BKN_PAGESELECTEDPENDING:
                 {
                    IPageSelectEvent pageevt( ctlevt );
                    bRc = this->selectPending( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );

                       /******************************************************/
                       /* If the notebook is a CUA notebook and the event    */
                       /* result was set to true, set the new page id field  */
                       /* in PAGSELECTNOTIFY to 0 to indicate to the CUA     */
                       /* control that the selection of the new page is      */
                       /* not permitted.                                     */
                       /******************************************************/
                       if ( (IWindowClassName( hwndControl ) == WC_NOTEBOOK) &&
                            event.result().asUnsignedLong() )
                       {
                         PAGESELECTNOTIFY *pPageSelectNotify =
                                 (PAGESELECTNOTIFY*)(void*)ctlevt.parameter2();
                         pPageSelectNotify->ulPageIdNew = 0;
                       }
                    }
                    break;
                 }
                 case BKN_PAGEDELETED:
                 {
                    IPageRemoveEvent pageevt( ctlevt );
                    bRc = this->remove(pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 case BKN_HELP:
                 {
                    IPageHelpEvent pageevt( ctlevt );
                    bRc = this->help( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 default:
                    break;
              } /* endswitch */
           } /* endif WM_CONTROL */
           else if ( event.eventId() == WM_DRAWITEM )
           {
              INotebookDrawItemEvent nbdievt( event );
              bRc = this->drawTab( nbdievt );
              if ( bRc )
              {
                 event.setResult( nbdievt.result() );
              }
           }
        } /* endif notebook control */
        break;
     } /* endcase WM_CONTROL, WM_DRAWITEM */

     default:
        break;
  } /* endswitch */
#endif //IC_PMWIN

#ifdef IC_MOTIF
  unsigned long reason = event.eventId();

  switch (reason)
  {
    case motifEvent(XmCR_MAJOR_TAB):
    case motifEvent(XmCR_MINOR_TAB):
    case motifEvent(XmCR_PAGE_SCROLLER_INCREMENT):
    case motifEvent(XmCR_PAGE_SCROLLER_DECREMENT):
    case motifEvent( XmCR_NONE ):
    {                               //braces avoid compiler warning re: bypassed decl.
      IPageSelectEvent selEvt(event);
      bRc = select(selEvt);
      if (bRc)
        event.setResult(selEvt.result());  // copy result
    }
    break;

    default:
      break;
  } /* endswitch */
#endif //IC_MOTIF

  return bRc;
}

/*------------------------------------------------------------------------------
| IPageHandler::resize                                                         |
|                                                                              |
| Implemented by derived classes to process a page size change notification.   |
------------------------------------------------------------------------------*/
bool IPageHandler::resize ( IPageEvent& event )
{
  ITRACE_MOTIF_NOP();

  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::select                                                         |
|                                                                              |
| Implemented by derived classes to process the notification that a new page   |
| is at the top of the notebook.                                               |
------------------------------------------------------------------------------*/
bool IPageHandler::select ( IPageSelectEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::selectPending                                                  |
|                                                                              |
| Implemented by derived classes to process the notification that selection    |
| is pending on another page in the notebook.                                  |
------------------------------------------------------------------------------*/
bool IPageHandler::selectPending ( IPageSelectEvent& event )
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();

  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::remove                                                         |
|                                                                              |
| Implemented by derived classes to process a page deletion notification.      |
------------------------------------------------------------------------------*/
bool IPageHandler::remove ( IPageRemoveEvent& event )
{
  ITRACE_MOTIF_NOP();

  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::help                                                           |
|                                                                              |
| Implemented by derived classes to process a notification that help has       |
| been requested for a notebook page.                                          |
------------------------------------------------------------------------------*/
bool IPageHandler::help ( IPageHelpEvent& event )
{
  ITRACE_MOTIF_NOP();

#ifdef IC_PMWIN
  INotebook *notebook = event.notebook();
  IHelpWindow *helpWin = event.helpWindow();

  if (notebook && helpWin && notebook->helpId() != 0)
  {
    helpWin->show( IResourceId(notebook->helpId()) );
    return true;
  }
#endif
  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::drawTab                                                        |
|                                                                              |
| Implemented by derived classes to draw the page tab.                         |
------------------------------------------------------------------------------*/
bool IPageHandler::drawTab ( INotebookDrawItemEvent& event )
{
  ITRACE_MOTIF_NOP();

  return false;
}
