// Revision: 34 1.10.1.12 source/ui/baseapp/ipainevt.cpp, genhdr, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ipainevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipainevt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_GPIREGIONS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #include <string.h>
}

#include <ipainevt.hpp>
#include <ibundles.hpp>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <iexgrprt.hpp>
#include <ifont.hpp>
#include <igraftxt.hpp>
#include <igrport.hpp>
#include <ihandle.hpp>
#include <irect.hpp>
#include <istring.hpp>
#include <istyles.hpp>
#include <istylset.hpp>
#include <ireslib.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>
#include <grafdevs.hpp>

#ifdef IC_MOTIF
#include <ixdc.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IPAINEVT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IPaintEvent::IPaintEvent                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IPaintEvent::IPaintEvent ( const IEvent& evt )
  : IEvent(evt), hpsCl(), rectCl(), setGC(false)
{
  // The invalid rectangle is in native coordinates.
  rectCl = this->window()->invalidatedRect();
#ifdef IC_PM
  //--------------------------------------------------------------------------
  // On OS/2 BIDI systems, we need to copy the BIDI attributes from the
  // window to the HPS (since OS/2 does not do this for us).
  //--------------------------------------------------------------------------
  IWindow* win = this->window();
  if ( IBidiSettings::isBidiSupported() )
  {
     IBidiSettings settings(*win);
     settings.apply(this->presSpaceHandle());
  }
#endif // IC_PM
}

/*------------------------------------------------------------------------------
| IPaintEvent::~IPaintEvent                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPaintEvent::~IPaintEvent ( )
{
#ifdef IC_PM
  HRGN oldrgn;
  if ( hpsCl )
  {
     if ( GpiSetClipRegion( hpsCl, NULLHANDLE, &oldrgn ) == RGN_ERROR )
        ITHROWGUIERROR( "GpiSetClipRegion" );
     if ( oldrgn )
     {
        if ( !( GpiDestroyRegion( hpsCl, oldrgn )))
           ITHROWGUIERROR( "GpiDestroyRegion" );
     }
     if ( !setGC )
     {
        this->window()->releasePresSpace( hpsCl );
     }
  }
#endif // IC_PM

#ifdef IC_MOTIFWIN
  if ( hpsCl )
  {
#ifdef IC_WIN
     SelectClipRgn ( hpsCl, 0L );
     if ( !setGC )
#endif // IC_WIN
        this->window()->releasePresSpace( hpsCl );
  } /* endif */
#endif // IC_MOTIFWIN
}

/*------------------------------------------------------------------------------
| IPaintEvent::presSpaceHandle                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPresSpaceHandle IPaintEvent::presSpaceHandle ( )
{
  if (!hpsCl)
  {
     hpsCl = window()->presSpace();
     this->initializeGC();
  }
  return hpsCl;
}

/*------------------------------------------------------------------------------
| IPaintEvent::setGraphicContext                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPaintEvent&
  IPaintEvent::setGraphicContext ( const IPresSpaceHandle& presHandle,
                                   bool setClipRegion )
{
#ifdef IC_PM
  HRGN oldrgn;
  if ( hpsCl )
  {
     if ( GpiSetClipRegion( hpsCl, NULLHANDLE, &oldrgn ) == RGN_ERROR )
        ITHROWGUIERROR( "GpiSetClipRegion" );
     if ( oldrgn )
     {
        if ( !( GpiDestroyRegion( hpsCl, oldrgn )))
           ITHROWGUIERROR( "GpiDestroyRegion" );
     }
     if ( !setGC )
     {
        this->window()->releasePresSpace( hpsCl );
     }
  }
#endif // IC_PM

  hpsCl = presHandle;
  if ( setClipRegion )
  {
     this->initializeGC();
  }
  setGC = true;
  return *this;
}


/*------------------------------------------------------------------------------
| IPaintEvent::setGrafPort                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPaintEvent& IPaintEvent::setGrafPort ( IGrafPort& gc,
                                        bool       setClipRegion )
{
  return this->setGraphicContext( gc.device()->deviceContext(),
                                  setClipRegion );
}

/*------------------------------------------------------------------------------
| IPaintEvent::clearBackground                                                 |
|                                                                              |
| Clear and fill a given rectangle area                                        |
------------------------------------------------------------------------------*/
IPaintEvent& IPaintEvent::clearBackground ( const IRectangle& rect,
                                            const IColor& clr )
{
  // Use the native coordinate system for compatibility with
  // previous releases, and clip any painting to the invalid
  // region.
  IExtendedRootGrafPort
    grafPort( this->presSpaceHandle(),
              ICoordinateSystem::nativeOrientation() );
  IGArea
    clipRegion( this->rect() );
  ILinkedGrafPort
    clippedPort( &grafPort, &clipRegion );

  // Set up and paint the rectangle to fill.
  IGRect2D
    fillBox( rect );
  IBaseColor
    fillColor( clr );
  IFillBundle
    bgndFill( fillColor );
  ILinkedGrafPort
    fillPort( &clippedPort, &bgndFill );
  fillPort
   .draw( fillBox );

  return *this;
}

/*------------------------------------------------------------------------------
| IPaintEvent::drawText                                                        |
|                                                                              |
| Draw a line of text starting at a given position.                            |
------------------------------------------------------------------------------*/
IPaintEvent& IPaintEvent::drawText ( const char* pszText,
                                     const IPoint& atPoint,
                                     const IColor& clr )
{
  // Add font and color information to the text.
  IFont
    windowFont( this->dispatchingWindow()->font() );
  ITextStyleSet
    textStyles = windowFont.styleSet();
  textStyles
   .add( ITextColorStyle( clr.redMix(),
                          clr.greenMix(),
                          clr.blueMix() ) );
  IText
    text( pszText );
  text
   .addStyles( textStyles );
  IGraphicText
    drawText( text, IGraphicText::kSingleLine );
  drawText
   .setLocation( IGPoint2D( atPoint ) );

  // Use the native coordinate system for compatibility with
  // previous releases, and clip any drawing to the invalid
  // rectangle.
  IExtendedRootGrafPort
    grafPort( this->presSpaceHandle(),
              ICoordinateSystem::nativeOrientation() );
  IGArea
    clipRegion( this->rect() );
  ILinkedGrafPort
    clippedPort( &grafPort, &clipRegion );
  drawText
   .draw( clippedPort );

  return *this;
}

/*------------------------------------------------------------------------------
| IPaintEvent::initializeGC                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPaintEvent& IPaintEvent::initializeGC ( )
{
  if ( rectCl.area() )
  {
     IRegionHandle hrgn( this->window()->invalidatedRegion() );
#ifdef IC_PM
     HRGN oldrgn;
     if ( GpiSetClipRegion( hpsCl, hrgn, &oldrgn ) == RGN_ERROR )
        ITHROWGUIERROR( "GpiSetClipRegion" );
     if ( oldrgn == HRGN_ERROR )
        ITHROWGUIERROR( "GpiSetClipRegion" );
     if ( oldrgn != NULLHANDLE  &&  !( GpiDestroyRegion( hpsCl, oldrgn )))
        ITHROWGUIERROR( "GpiDestroyRegion" );
#endif
#ifdef IC_WIN
     SelectClipRgn ( hpsCl, hrgn );
//   DeleteObject ( hrgn );
#endif
#ifdef IC_MOTIF
     XSetRegion( XtDisplay( (Widget) this->handle() ),
                 ( (IXDC*) hpsCl )->gc(),
                 (Region)(void *)hrgn.asUnsigned() );
#endif
  }
  return *this;
}
