// Revision: 40 1.5.1.3 source/ui/baseapp/ipmbidi.cpp, staticctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ipmbidi.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipmbidi.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINCOUNTRY
#define INCL_WINHOOKS
extern "C"
{
  #include <iwindefs.h>
  #include <layout.h>
  #include <pmbidi.h>
}

#include <ireslib.hpp>
#include <ipmbidi.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IPMBIDI_CPP_
    #include <ipagetun.h>
#endif


typedef struct _REORDERINFO
{
   ULONG  cb;
   ULONG  CodePage;
   ULONG  Reserved;
   ULONG  CursorPos;
   ULONG  flFlags;
   UCHAR* LevelMap;
   ULONG* SrcToTrgMap;
   ULONG* TrgToSrcMap;
}
REORDERINFO;

typedef unsigned long
     (_System  ILayoutCreateObject) ( char*          locale_name,
                                      LAYOUT_OBJECT* layout );
typedef unsigned long
     (_System  ILayoutDeleteObject) ( LAYOUT_OBJECT  layout );

typedef unsigned long
     (_System  ILayoutSetValues)    ( LAYOUT_OBJECT  layout,
                                      PLAYOUT_VALUES values,
                                      ULONG*         index_returned );
typedef unsigned long
     (_System  ILayoutTransformText)( LAYOUT_OBJECT  layout,
                                      UCHAR*         InpBuf,
                                      ULONG*         InpSize,
                                      PVOID          OutBuf,
                                      ULONG*         OutSize,
                                      ULONG*         InpToOut,
                                      ULONG*         OutToInp,
                                      UCHAR*         BidiLevel );

#pragma enum(4)
#pragma pack(push,4)

class IPMBidiData
{
public:
static IDynamicLinkLibrary
  *bidiDLL;
static ILayoutCreateObject
  *layoutCreateObject;
static ILayoutDeleteObject
  *layoutDeleteObject;
static ILayoutSetValues
  *layoutSetValues;
static ILayoutTransformText
  *layoutTransformText;
};

#pragma pack(pop)
#pragma enum(pop)

IDynamicLinkLibrary*  IPMBidiData::bidiDLL(0);
ILayoutCreateObject*  IPMBidiData::layoutCreateObject (0);
ILayoutDeleteObject*  IPMBidiData::layoutDeleteObject (0);
ILayoutSetValues*     IPMBidiData::layoutSetValues    (0);
ILayoutTransformText* IPMBidiData::layoutTransformText(0);


/*------------------------------------------------------------------------------
| Load PM BIDI APIs                                                            |
------------------------------------------------------------------------------*/
static void loadBidiDLL ( )
{
   //--------------------------------------------------------------------------
   // Check for BIDI APIs (use ordinals to load procedure addresses)
   //--------------------------------------------------------------------------
   if ( !IPMBidiData::bidiDLL )
   {
     IPMBidiData::bidiDLL = new IDynamicLinkLibrary("pmbidi");
     IPMBidiData::layoutCreateObject  = (ILayoutCreateObject*)
                                        IPMBidiData::bidiDLL->procAddress(306);
     IPMBidiData::layoutDeleteObject  = (ILayoutDeleteObject*)
                                        IPMBidiData::bidiDLL->procAddress(307);
     IPMBidiData::layoutSetValues     = (ILayoutSetValues*)
                                        IPMBidiData::bidiDLL->procAddress(310);
     IPMBidiData::layoutTransformText = (ILayoutTransformText*)
                                        IPMBidiData::bidiDLL->procAddress(308);
   }
}

/*------------------------------------------------------------------------------
| Convert an IWindow::BidiSettings object to an unsigned long                  |
------------------------------------------------------------------------------*/
static unsigned long bidiAttributes ( IBidiSettings& settings )
{
   unsigned long attr = 0L;

   switch ( settings.textType() )
   {
      case IBidiSettings::implicit:
           attr |= BDA_TEXTTYPE_IMPLICIT;
           break;
      case IBidiSettings::visual:
           attr |= BDA_TEXTTYPE_VISUAL;
           break;
   }

   switch ( settings.textOrientation() )
   {
      case IBidiSettings::textLeftToRight:
           attr |= BDA_TEXT_ORIENT_LTR;
           break;
      case IBidiSettings::textRightToLeft:
           attr |= BDA_TEXT_ORIENT_RTL;
           break;
      case IBidiSettings::textContextual:
           attr |= BDA_TEXT_ORIENT_DYNAMIC;
           break;
   }

   switch ( settings.windowLayout() )
   {
      case IBidiSettings::layoutLeftToRight:
           attr |= BDA_WND_ORIENT_LTR;
           break;
      case IBidiSettings::layoutRightToLeft:
           attr |= BDA_WND_ORIENT_RTL;
           break;
   }

   switch ( settings.numeralDisplay() )
   {
      case IBidiSettings::arabic:
           attr |= BDA_NUMERALS_NOMINAL;
           break;
      case IBidiSettings::asStored:
           attr |= BDA_NUMERALS_PASSTHRU;
           break;
      case IBidiSettings::national:
           attr |= BDA_NUMERALS_NATIONAL;
           break;
      case IBidiSettings::contextual:
           attr |= BDA_NUMERALS_CONTEXTUAL;
           break;
   }

   if ( settings.isSymmetricSwappingEnabled() )
      attr |= BDA_SYM_SWAP_ON;
   else
      attr |= BDA_SYM_SWAP_OFF;

   if ( settings.isWordByWordReorderingEnabled() )
      attr |= BDA_WORD_BREAK_ON;
   else
      attr |= BDA_WORD_BREAK_OFF;

   switch ( settings.textShape() )
   {
      case IBidiSettings::displayShaped:
           attr |= BDA_TEXT_DISPLAY_SHAPED;
           break;
      case IBidiSettings::saveShaped:
           attr |= BDA_TEXT_SAVE_SHAPED;
           break;
      case IBidiSettings::nominalShape:
           attr |= BDA_TEXT_NOMINAL;
           break;
      case IBidiSettings::initialShape:
           attr |= BDA_TEXT_INITIAL;
           break;
      case IBidiSettings::middleShape:
           attr |= BDA_TEXT_MIDDLE;
           break;
      case IBidiSettings::finalShape:
           attr |= BDA_TEXT_FINAL;
           break;
      case IBidiSettings::isolatedShape:
           attr |= BDA_TEXT_ISOLATED;
           break;
   }

   return attr;
}

/*------------------------------------------------------------------------------
| LayoutConvert                                                                |
------------------------------------------------------------------------------*/
static void bidiLayoutConvert ( LAYOUT_OBJECT layoutObject,
                                IString       bidiText,
                                unsigned long sourceAttributes,
                                unsigned long targetAttributes,
                                REORDERINFO*  reorderInfo )

{

  LAYOUT_VALUES   Layout[6];
  ULONG           AttrMask     = BDAM_ALL  & ~BDAM_TEXT_SHAPE & ~BDAM_NUMERALS;
  PUCHAR          LevelMap     = NULL;
  PULONG          SrcTrgMap    = NULL;
  PULONG          TrgSrcMap    = NULL;
  ULONG           Increment    = 1L;
  bool            InpMode      = FALSE;
  ULONG           Length       = bidiText.length();
  ULONG           index        = 0;

  //---------------------------------------------------------------------------
  // Get maps and input mode from reorderinfo
  //---------------------------------------------------------------------------
  LevelMap  = reorderInfo->LevelMap;
  SrcTrgMap = reorderInfo->SrcToTrgMap;
  TrgSrcMap = reorderInfo->TrgToSrcMap;

  //---------------------------------------------------------------------------
  // Check for Arabic specific handling
  //---------------------------------------------------------------------------
  if ( reorderInfo->CodePage == 864 )
  {
    //-------------------------------------------------------------------------
    // Add back masks for text shape and numerals
    //-------------------------------------------------------------------------
    AttrMask |= (BDAM_TEXT_SHAPE | BDAM_NUMERALS);

    //-------------------------------------------------------------------------
    // If target is text shaped then set target to source
    //-------------------------------------------------------------------------
    if (( targetAttributes & BDAM_TEXT_SHAPE ) == BDA_TEXT_SAVE_SHAPED )
    {
      //-----------------------------------------------------------------------
      // If source is also text shaped, change it to nominal first
      //-----------------------------------------------------------------------
      if (( sourceAttributes & BDAM_TEXT_SHAPE ) == BDA_TEXT_SAVE_SHAPED )
      {
        sourceAttributes &= ~BDA_TEXT_SAVE_SHAPED;
        sourceAttributes |= ~BDA_TEXT_NOMINAL;
      }
      targetAttributes &= ~BDAM_TEXT_SHAPE;
      targetAttributes |= ( sourceAttributes & BDAM_TEXT_SHAPE );
    }
    else
    {
      //-----------------------------------------------------------------------
      // Otherwise ensure that source and target text shape is different
      //-----------------------------------------------------------------------
      sourceAttributes &= ~BDAM_TEXT_SHAPE;
      if (( targetAttributes & BDAM_TEXT_SHAPE ) == BDA_TEXT_NOMINAL )
         sourceAttributes |= BDA_TEXT_DISPLAY_SHAPED;
      else
         sourceAttributes |= BDA_TEXT_NOMINAL;
    }

    //-------------------------------------------------------------------------
    // If target is numerals passthru then set target to source
    //-------------------------------------------------------------------------
    if (( targetAttributes & BDAM_NUMERALS ) == BDA_NUMERALS_PASSTHRU )
    {
      //-----------------------------------------------------------------------
      // If source is also passthru, change it to nominal first
      //-----------------------------------------------------------------------
      if (( sourceAttributes & BDAM_NUMERALS ) == BDA_NUMERALS_PASSTHRU )
      {
        sourceAttributes &= ~BDA_NUMERALS_PASSTHRU;
        sourceAttributes |= ~BDA_NUMERALS_NOMINAL;
      }
      targetAttributes &= ~BDAM_NUMERALS;
      targetAttributes |= ( sourceAttributes & BDAM_NUMERALS );
    }
    else
    {
      //-----------------------------------------------------------------------
      // Otherwise ensure that source and target numerals shape is different
      //-----------------------------------------------------------------------
      sourceAttributes &= ~BDAM_NUMERALS;
      if (( targetAttributes & BDAM_NUMERALS ) == BDA_NUMERALS_NOMINAL )
         sourceAttributes |= BDA_NUMERALS_NATIONAL;
      else
         sourceAttributes |= BDA_NUMERALS_NOMINAL;
    }
  }

  //---------------------------------------------------------------------------
  // Initialize layout array
  //---------------------------------------------------------------------------
  Layout[0].name  =  InOutTextDescrMask;
  Layout[0].value =  &AttrMask;
  Layout[1].name  =  InOnlyTextDescr;
  Layout[1].value =  &sourceAttributes;
  Layout[2].name  =  OutOnlyTextDescr;
  Layout[2].value =  &targetAttributes;
  Layout[3].name  =  InputMode;
  Layout[3].value =  &InpMode;
  Layout[4].name  =  CellSize;
  Layout[4].value =  &Increment;
  Layout[5].name = 0;

  //---------------------------------------------------------------------------
  // Set layout values
  //---------------------------------------------------------------------------
  IPMBidiData::layoutSetValues ( layoutObject, Layout, &index );

  //---------------------------------------------------------------------------
  // Convert the text
  //---------------------------------------------------------------------------
  IPMBidiData::layoutTransformText ( layoutObject,
                                     (UCHAR*) (const char *) bidiText,
                                     &Length,
                                     (UCHAR*) (const char *) bidiText,
                                     &Length,
                                     SrcTrgMap, TrgSrcMap, LevelMap);

}

/*------------------------------------------------------------------------------
| CreateLayoutObject                                                           |
------------------------------------------------------------------------------*/
LAYOUT_OBJECT bidiCreateLayoutObject ()
{
   LAYOUT_OBJECT layout = 0L;

   //--------------------------------------------------------------------------
   // Load BIDI APIs
   //--------------------------------------------------------------------------
   loadBidiDLL();

   if ( IPMBidiData::layoutCreateObject )
   {
     //------------------------------------------------------------------------
     // Get the current codepage
     //------------------------------------------------------------------------
     unsigned long codePage = WinQueryCp(HMQ_CURRENT);

     //------------------------------------------------------------------------
     // Get a layout object to use when converting the text
     //------------------------------------------------------------------------
     if ( codePage == 862 )
       IPMBidiData::layoutCreateObject(Locale_Hebrew,&layout);
     if ( codePage == 864 )
       IPMBidiData::layoutCreateObject(Locale_Arabic,&layout);
   }

   return layout;
}

/*------------------------------------------------------------------------------
| DeleteLayoutObject                                                           |
------------------------------------------------------------------------------*/
void bidiDeleteLayoutObject ( LAYOUT_OBJECT layoutObject )
{
  if ( layoutObject && IPMBidiData::layoutDeleteObject )
    IPMBidiData::layoutDeleteObject(layoutObject);
}

/*------------------------------------------------------------------------------
| PrepareForDisplay                                                            |
------------------------------------------------------------------------------*/
void bidiPrepareForDisplay ( LAYOUT_OBJECT   layoutObject,
                             IString&        bidiText,
                             IBidiSettings&  settings,
                             unsigned long&  tildePosition )
{
   ULONG                sourceAttributes;
   ULONG                targetAttributes;
   REORDERINFO          reorderInfo;

   //--------------------------------------------------------------------------
   // Load BIDI APIs
   //--------------------------------------------------------------------------
   loadBidiDLL();

   //--------------------------------------------------------------------------
   // Get BIDI attributes
   //--------------------------------------------------------------------------
   sourceAttributes = bidiAttributes(settings);
   targetAttributes = sourceAttributes;

   //--------------------------------------------------------------------------
   // If symmetric swapping is enabled, disable in source
   //--------------------------------------------------------------------------
   if ( settings.isSymmetricSwappingEnabled() )
   {
      sourceAttributes &= ~BDA_SYM_SWAP_ON;
      sourceAttributes |= BDA_SYM_SWAP_OFF;
   }

   //--------------------------------------------------------------------------
   // Initialize REORDERINFO structure
   //--------------------------------------------------------------------------
   reorderInfo.cb          = sizeof(REORDERINFO);
   reorderInfo.CodePage    = WinQueryCp(HMQ_CURRENT);
   reorderInfo.Reserved    = NULL ;
   reorderInfo.CursorPos   = bidiText.length() + 1;
   reorderInfo.flFlags     = 0L;
   reorderInfo.LevelMap    = NULL;
   reorderInfo.TrgToSrcMap = NULL;

   //--------------------------------------------------------------------------
   // Changes to the tilde index are reflected in the SrcToTrgMap
   //--------------------------------------------------------------------------
   if ( tildePosition )
     reorderInfo.SrcToTrgMap = (ULONG*) new ULONG[bidiText.length()+1];
   else
     reorderInfo.SrcToTrgMap = NULL;

   //--------------------------------------------------------------------------
   // On Hebrew systems insure that implicit text uses contextual orientation
   //--------------------------------------------------------------------------
   if (( reorderInfo.CodePage == 862 ) &&
       ( settings.textType() == IBidiSettings::implicit ))
   {
      sourceAttributes &= ~BDA_TEXT_ORIENT_LTR & ~BDA_TEXT_ORIENT_RTL;
      sourceAttributes |= BDA_TEXT_ORIENT_DYNAMIC;
   }

   //--------------------------------------------------------------------------
   // Call convertBidiString to do the shaping
   //--------------------------------------------------------------------------
   bidiLayoutConvert ( layoutObject, bidiText, sourceAttributes,
                       targetAttributes, &reorderInfo );

   //--------------------------------------------------------------------------
   // Clean up
   //--------------------------------------------------------------------------
   if ( reorderInfo.SrcToTrgMap )
     delete reorderInfo.SrcToTrgMap;

}

/*------------------------------------------------------------------------------
| IsLamAlef                                                                    |
------------------------------------------------------------------------------*/
static bool bidiIsLamAlef (char ch)
{
  char LamAlefShape []  = {'','','','','','','\0'};
  int i;
  for (i=0; i<6;i++ ) {
    if (ch == LamAlefShape [i])
       return (true);
  }
  return (false);
}

/*------------------------------------------------------------------------------
| IsLamAlefBlank                                                               |
------------------------------------------------------------------------------*/
static bool bidiIsLamAlefBlank ( IString  bidiText,
                                    unsigned long pos,
                                    bool  rightToLeft )
{
  bool lamAlef = false;
  if ( bidiText[pos] == ' ' )
  {
    if (rightToLeft && bidiIsLamAlef(bidiText[pos-1]))
      lamAlef = true;
    if (!rightToLeft && bidiIsLamAlef(bidiText[pos+1]))
      lamAlef = true;
  }
  return lamAlef;
}

/*------------------------------------------------------------------------------
| LamAlefWrapOrClip                                                            |
------------------------------------------------------------------------------*/
static LONG LamAlefWrapOrClip ( IString       bidiText,
                                unsigned long startPos,
                                LONG          numChar,
                                bool       rightToLeft )
{
   //--------------------------------------------------------------------------
   // If less than 2 characters there is only room for a lam-alef and space
   //--------------------------------------------------------------------------
   if ( numChar < 2 )
      return numChar;

   //--------------------------------------------------------------------------
   // If the remaining characters all fit, just return
   //--------------------------------------------------------------------------
   if ( numChar >= ( bidiText.length() - startPos ))
      return numChar;

   //--------------------------------------------------------------------------
   // Check to be sure the space is an lam-alef blank
   //--------------------------------------------------------------------------
   if ( !bidiIsLamAlefBlank( bidiText, (startPos + numChar), rightToLeft ))
      return numChar;

   //--------------------------------------------------------------------------
   // Search backward in the string until a non-lam-alef blank is found
   //--------------------------------------------------------------------------
   LONG    oldNumChar = numChar;
   bool loop       = true;
   while ( loop )
   {
     numChar--;
     if ( numChar > 1 )
     {
       if (( bidiText[numChar+startPos] == (char) 0x0A ) ||
           ( bidiText[numChar+startPos] == (char) 0x0D ))
       {
          loop = false; // We found a CRLF, so stop looping
       }
       if ( bidiText[numChar+startPos] == ' ' )
       {
          if ( bidiText[numChar+startPos+1] == ' ' )
            loop = false; // We found two consecutive spaces, so stop looping
          if ( !bidiIsLamAlefBlank ( bidiText, (startPos+numChar), rightToLeft ))
            loop = false; // We found a non-lam-alef blank
       }
     }
     else
     {
       loop = false; // We reached the first character in the word
     }
   }

   //--------------------------------------------------------------------------
   // If the search found a non-lam-alef blank then return
   //--------------------------------------------------------------------------
   if ( numChar > 1 )
      return numChar - 1;

   //--------------------------------------------------------------------------
   // We reached the beginning of the text, so look for the first blank after
   // the word instead (the text will be clipped for this case)
   //--------------------------------------------------------------------------
   numChar = oldNumChar;
   loop    = true;
   while ( loop )
   {
     numChar++;
     if (( numChar + startPos ) <= bidiText.length() )
     {
       if (( bidiText[numChar+startPos] == (char) 0x0A ) ||
           ( bidiText[numChar+startPos] == (char) 0x0D ))
       {
          loop = false; // We found a CRLF, so stop looping
       }
       if ( bidiText[numChar+startPos] == ' ' )
       {
          if ( bidiText[numChar+startPos-1] == ' ' )
            loop = false; // We found two consecutive spaces, so stop looping
          if ( !bidiIsLamAlefBlank ( bidiText, (startPos+numChar), rightToLeft ))
            loop = false; // We found a non-lam-alef blank
       }
     }
     else
     {
       loop = false; // We reached the last character in the text
     }
   }

   return numChar - 1;
}

/*------------------------------------------------------------------------------
| DrawText                                                                     |
------------------------------------------------------------------------------*/
unsigned long bidiDrawText ( IPresSpaceHandle  presSpace,
                             IString           bidiText,
                             unsigned long     index,
                             LAYOUT_OBJECT     layoutObject,
                             IBidiSettings&    settings,
                             unsigned long     flags,
                             IColor            foregroundColor,
                             IColor            backgroundColor,
                             IRectangle        rectangle )
{

   RECTL  rectl = rectangle.asRECTL();

   LONG   numChar = 0;
   LONG   numCRLF = 0;

   //---------------------------------------------------------------------
   // Get the starting address for the text
   //---------------------------------------------------------------------
   char* txt = (char*) (const char*) bidiText;
   txt = txt + index;

   //--------------------------------------------------------------------------
   // Use WinDrawText to determine the number of characters that will fit
   //--------------------------------------------------------------------------
   numChar = WinDrawText ( presSpace,
                           bidiText.length() - index,
                           txt,
                           &rectl,
                           foregroundColor.asRGBLong(),
                           backgroundColor.asRGBLong(),
                           flags | DT_WORDBREAK | DT_QUERYEXTENT );

   //--------------------------------------------------------------------------
   // For Arabic systems a blank may not be the end of a word if it follows
   // a LAM-ALEF character.
   //--------------------------------------------------------------------------
   if ( WinQueryCp(HMQ_CURRENT) == 864 )
   {
      if (( settings.textType() == IBidiSettings::implicit ) ||
          ( settings.textOrientation() == IBidiSettings::textRightToLeft ))
      {
         numChar = LamAlefWrapOrClip ( bidiText, index, numChar, true );
      }
      else
      {
         numChar = LamAlefWrapOrClip ( bidiText, index, numChar, false );
      }
   }

   //---------------------------------------------------------------------
   // Remove the tilde from the string before shaping
   //---------------------------------------------------------------------
   unsigned long tilde = bidiText.indexOf('~');
   if (tilde)
   {
      bidiText.remove(tilde,1);
      if ( tilde <= ( index + numChar ))
         numChar--;
   }

   //---------------------------------------------------------------------
   // Shape the text
   //---------------------------------------------------------------------
   bidiPrepareForDisplay ( layoutObject, bidiText, settings, tilde );

   //---------------------------------------------------------------------
   // Restore the tilde to the string
   //---------------------------------------------------------------------
   if (tilde)
   {
      bidiText.insert("~",tilde);
      if ( tilde <= ( index + numChar + 1 ))
         numChar++;
   }

   //---------------------------------------------------------------------
   // Strip off CRLF since we do not use the DT_WORDBREAK when drawing
   //---------------------------------------------------------------------
   if ( numChar )
   {
     while (((bidiText[index+numChar-numCRLF]==0x0D) ||
             (bidiText[index+numChar-numCRLF]==0x0A)) &&
             (numChar - numCRLF))
     {
       numCRLF++;
     }
   }
   numChar -= numCRLF;

   //---------------------------------------------------------------------
   // Draw the text background
   //---------------------------------------------------------------------
   rectl = rectangle.asRECTL();
   WinDrawText ( presSpace,
                 numChar,
                 txt, &rectl, 0, 0,
                 (flags & ~DT_WORDBREAK) | DT_QUERYEXTENT );
   WinFillRect( presSpace, &rectl, backgroundColor.asRGBLong() );

   //---------------------------------------------------------------------
   // Draw the text
   //---------------------------------------------------------------------
   rectl = rectangle.asRECTL();
   WinDrawText ( presSpace,
                 numChar,
                 txt, &rectl,
                 foregroundColor.asRGBLong(),
                 backgroundColor.asRGBLong(),
                 flags & ~DT_WORDBREAK);

   return (numChar + numCRLF);
}

