// Revision: 51 1.10.1.5 source/ui/baseapp/irecohdr.cpp, basewin, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: irecohdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in irecohdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )


#define INCL_WINWINDOWMGR      // WinSetMultWindowPos, WinQueryWindow
#define INCL_WINFRAMEMGR       // WM_WINDOWPOSCHANGED
#define INCL_WINMESSAGEMGR          // for WM_CALCVALIDRECTS
extern "C" {
  #include <iwindefs.h>
}

#include <irecohdr.hpp>
#include <icoordsy.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <iframe.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <iwindow.hpp>
#include <itrace.hpp>
#include <ithread.hpp>
#include <iswp.hpp>
#ifdef IC_MOTIF
  #include <isizrect.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRECOHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Default IRecoordHandler object                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointed               |
| to object when the wrapper is destructed.                                    |
------------------------------------------------------------------------------*/
static struct DefaultRecoordHandler
{
  operator IRecoordHandler* ( );

IRecoordHandler
 *operator -> ( )
  {
    return( *this );
  }

  ~DefaultRecoordHandler ( )
   {
     IMODTRACE_DEVELOP("DefaultRecoordHandler::~DefaultRecoordHandler");
#if IC_STATIC_PRIORITY_SUPPORTED
     // Delete is only safe when static init/term ordering can be controlled.
     if (ptr)
     {
       delete ptr;
       ptr = 0;
     }
#endif
   }

IRecoordHandler
 *ptr;

} gdefaultHandler;

/*------------------------------------------------------------------------------
| DefaultRecoordHandler::operator IRecoordHandler*                             |
------------------------------------------------------------------------------*/
DefaultRecoordHandler :: operator IRecoordHandler* ( )
{
  if (!ptr)
    ptr = new IRecoordHandler;

  return( ptr );
}

/*------------------------------------------------------------------------------
| IRecoordHandler::defaultHandler                                              |
------------------------------------------------------------------------------*/
IRecoordHandler*  IRecoordHandler::defaultHandler( )
{
   return gdefaultHandler;
}

/*------------------------------------------------------------------------------
| IRecoordHandler::IRecoordHandler                                             |
------------------------------------------------------------------------------*/
IRecoordHandler :: IRecoordHandler ( )
  : IRecoordHandler::Inherited ( )
{}

/*------------------------------------------------------------------------------
| IRecoordHandler::~IRecoordHandler                                            |
------------------------------------------------------------------------------*/
IRecoordHandler :: ~IRecoordHandler ( )
{}


#ifdef IC_PMWIN
/************************************************************************/
/* dispatchhandlerEvent - Dispatch command functions for this handler.  */
/************************************************************************/
bool  IRecoordHandler::dispatchHandlerEvent ( IEvent& evt )
{
   bool bRc = false;
#ifdef IC_WIN
   if (evt.eventId() == WM_NCCALCSIZE)
#endif
#ifdef IC_PM
   // IRecoordHandler needs to  see resize events to work properly.
   // In PM, it is possible that a user provided IResizeHandler could
   // stop the WM_WINDOWPOSCHANGED message that we look for.  Therefore,
   // IResizeHandler generates a substitute in this case.
   // Note that this logic relies on the fact that there is
   // no ICLUI handler of WM_WINDOWPOSCHANGED which returns
   // true.  This means that the IC_UM... message ONLY occurs
   // if a user supplied handler returns true before the
   // IRecoordHandler has run.
   // This is not a problem in Windows since we use different
   // messages in IRecoordHandler and IResizeHandler.
   if ((evt.eventId() == WM_WINDOWPOSCHANGED ) ||
       (evt.eventId() == IC_UM_WINDOWPOSCHANGED ) )
#endif
      {
      // Got a CALCSIZE message
#ifdef IC_WIN
        NCCALCSIZE_PARAMS* pncsp   =  (NCCALCSIZE_PARAMS*)
           (void*)evt.parameter2();
        if ( !(pncsp->lppos->flags & SWP_NOSIZE) )
        {
          // resizing occurred
          IRectangle oldRect( pncsp->rgrc[1] );
          IRectangle newRect( pncsp->rgrc[0] );
          bRc = windowSizeChanged( evt.handle(), newRect, oldRect );
        }  // resizing occurred
#endif
#ifdef IC_PM
      SWP* pswp =  (SWP*)
         (void*)evt.parameter1();
      if ( (pswp->fl & SWP_SIZE) )
         {
         // resizing occurred
         SWP*    pswp = (SWP*)(void*)evt.parameter1();
         IRectangle newRect( IPoint(pswp[0].x, pswp[0].y),
                             ISize(pswp[0].cx, pswp[0].cy)  );
         IRectangle oldRect( IPoint(pswp[1].x, pswp[1].y),
                             ISize(pswp[1].cx, pswp[1].cy)  );
         bRc = windowSizeChanged( evt.handle(), newRect, oldRect );
         }  // resizing occurred
#endif
      if ( bRc )
         {                        // event has been processed
         /*******************************************************/
         /* The handler has said not to pass on the event, so   */
         /* don't pass it to another handler.  However, we do   */
         /* go ahead and pass it to the window procedure of the */
         /* current window, since it is only a notification     */
         /* message and we could break something by not doing   */
         /* this.                                               */
         /*******************************************************/
         defaultProcedure(evt);
         }  // event processed
      } //

   return bRc;
}

/*------------------------------------------------------------------------------
| IRecoordHandler::windowSizeChanged                                           |
------------------------------------------------------------------------------*/
bool IRecoordHandler::windowSizeChanged (
                      IWindowHandle     handle,
                      const IRectangle& newWindowRect,
                      const IRectangle& oldWindowRect  )
{
   IMODTRACE_ALL("IRecoordHandler::windowSizeChanged");
   if ( ICoordinateSystem::isConversionNeeded() )
      {
      // Only need to do this if operating in non-native coordinates
      // delta is value to be added to current y position of child windows
      long delta = newWindowRect.height() - oldWindowRect.height();

      // Determine if the window is a frame.  Need this later for
      // handling extensions.
      IWindow* pwin = IWindow::windowWithHandle( handle );
      bool isFrame = false;
      if (pwin)
         isFrame = pwin->isFrameWindow();
#ifdef IC_WIN

      // We don't use IWindow::ChildCursor here because this code can
      // be used against non-IWindow objects.
      IWindowHandle childHwnd = GetWindow( handle, GW_CHILD ) ;

      HDWP hdwp = 0;
      if (childHwnd)
         {
         hdwp = BeginDeferWindowPos( 1 ); // don't know how many children
         if (!hdwp)
            {
            ITHROWGUIERROR2("BeginDeferWindowPos",
                            IBaseErrorInfo::outOfSystemResource,
                            IException::unrecoverable);
            }
         }  // at least 1 child
#endif
#ifdef IC_PM
      HENUM henum = WinBeginEnumWindows( handle );
      IWindowHandle childHwnd = WinGetNextWindow( henum ) ;
      unsigned long maxChildren = 10;
      ISWP* swpArray = new ISWP[maxChildren];
      unsigned long numChildren = 0;
#endif

      while (childHwnd != 0)
         {
         bool processChild = true;
         // We need to exclude frame extensions and the client window
         // from adjustment by this handler.   They are handled by
         // the frame extensions code and/or the system.
         if (isFrame)
            {
            unsigned long id = IIDOF( childHwnd );
            switch (id)
               {
#ifdef  IC_PM
               case FID_SYSMENU     :
               case FID_TITLEBAR    :
               case FID_MINMAX      :
               case FID_VERTSCROLL  :
               case FID_HORZSCROLL  :
#endif
               case FID_MENU        :
               case FID_CLIENT      :
                  // skip system extensions and client
                  processChild = false;
                  break;
               default:
                  {
                  // check for IFrameWindow extensions
                  IWindow* childWin = IWindow::windowWithHandle( childHwnd );
                  if (childWin)
                     {
                     if ( ((IFrameWindow*)pwin)->isAnExtension( childWin ) )
                        processChild = false;
                     }
                  }
                  break;
               }
            }

         if (processChild)
            {
         // get current position of child window
#ifdef IC_WIN
            RECT rect;
            GetWindowRect( childHwnd, &rect );
            POINT origin = {rect.left, rect.top};
            if ( !ScreenToClient( handle, &origin) )
               {
               ITHROWGUIERROR2("ScreenToClient",
                               IBaseErrorInfo::invalidParameter,
                               IException::unrecoverable);
               }
            // adjust origin to keep relative to logical origin
            origin.y = origin.y + delta;
            // set new origin in child window
            ITRACE_ALL( IString("DeferWindowPos hwnd=") +
                        IString( childHwnd.asUnsigned() ).d2x() +
                        IString(" delta=") + IString(delta) +
                        IString(" origin=") +
                        IString(origin.x) + IString(",") + IString(origin.y) );
            hdwp = DeferWindowPos( hdwp,
                                   childHwnd, 0,
                                   origin.x,
                                   origin.y,
                                   0,
                                   0,
                                   SWP_NOSIZE | SWP_NOZORDER) ;
            if (!hdwp)
               {
               ITHROWGUIERROR2("DeferWindowPos",
                               IBaseErrorInfo::invalidParameter,
                               IException::unrecoverable);
               }

#endif
#ifdef   IC_PM
            if ( maxChildren <= numChildren+1 )
               {
               // reallocate the swpArray
               maxChildren += 10;
               ISWP*  temp = swpArray;
               swpArray = new ISWP[maxChildren];
               unsigned long i;
               for ( i=0; i < numChildren; i++)
                  swpArray[i] = temp[i];
               delete [] temp;
               }
            if (! WinQueryWindowPos( childHwnd,
                                     (SWP*)&(swpArray[numChildren]) ) )
               {
               ITHROWGUIERROR2("WinQueryWindowPos",
                               IBaseErrorInfo::invalidParameter,
                               IException::unrecoverable);
               }
            // MOVE & SIZE on in return value of WinQueryWindowPos.  Not
            // resizing or messing with Z order
            swpArray[numChildren].setSizeFlag(false);

            // adjust origin to keep relative to logical origin
            IPoint   pos = swpArray[numChildren].position();
            swpArray[numChildren].setPosition( IPoint(pos.x(), pos.y() + delta) );

            ITRACE_ALL( IString("WindowPos hwnd=") +
                        IString( childHwnd.asUnsigned() ).d2x() +
                        IString(" delta=") + IString(delta) +
                        IString(" origin=") +
                        swpArray[numChildren].position().asString()  );

            numChildren++;
#endif
            }  // if process child

         childHwnd =
#ifdef IC_WIN
            GetWindow( childHwnd, GW_HWNDNEXT );
#endif
#ifdef IC_PM
            WinGetNextWindow( henum ) ;
#endif
         }   // while

#ifdef IC_WIN
      if (hdwp)
         {
         if ( !EndDeferWindowPos( hdwp ) )
            {
            ITHROWGUIERROR2("EndDeferWindowPos",
                            IBaseErrorInfo::invalidParameter,
                            IException::unrecoverable);
            }
         }
#endif   //IC_WIN
#ifdef   IC_PM
      if (numChildren)
         {
         if (! WinSetMultWindowPos(IThread::current().anchorBlock(),
                                   (SWP*)swpArray,
                                   numChildren) )
            {                         // Error from WinSetMultWindowPos
            ITHROWGUIERROR2("WinSetMultWindowPos",
                            IBaseErrorInfo::invalidParameter,
                            IException::unrecoverable);
            }
         }

      delete [] swpArray;
      WinEndEnumWindows( henum );
#endif   //IC_PM
      }
   return false;      //  pass on to other handlers
}
#endif //IC_PMWIN

#ifdef IC_MOTIF
/************************************************************************/
/* dispatchhandlerEvent - Dispatch command functions for this handler.  */
/************************************************************************/
bool  IRecoordHandler::dispatchHandlerEvent ( IEvent& evt )
{
  bool bRc = false;
  // IRecoordHandler needs to  see resize events to work properly.
  // It is possible that a user provided IResizeHandler could
  // stop the WM_SIZE( ConfigureNotify) message that we look for.  Therefore,
  // IResizeHandler generates a substitute in this case.
  // Note that this logic relies on the fact that there is
  // no ICLUI handler of WM_WINDOWPOSCHANGED which returns
  // true.  This means that the IC_UM... message ONLY occurs
  // if a user supplied handler returns true before the
  // IRecoordHandler has run.
  if ( (evt.eventId() == xEvent(ConfigureNotify) )  ||
       (evt.eventId() == IC_UM_WINDOWPOSCHANGED) )
  {
    ISizeRectangle *sizeRect =
        (ISizeRectangle *) evt.parameter1().asUnsignedLong();
    bRc = windowSizeChanged( evt.handle(),
                             sizeRect->newRectangle(),
                             sizeRect->oldRectangle() );
  }  // resizing occurred

  // Note: Don't try to use the IC_UM_SIZE message for this processing.
  // A resize handler may see that message first and position will be
  // wrong for lower left. The map notify will be seen before the IC_UM_SIZE
  // so the coordinates of the widget will be correctly set after the
  // following code.
  if ( (evt.eventId() == xEvent(MapNotify)) &&
       (evt.parameter1().asUnsignedLong()))
  {
    IWindow *parent = evt.window();

    // We will place the children of this handler when the coordinate system is
    // not native the first time a control is shown. Typically that would be for
    // children of a frame and a canvas.
    if (ICoordinateSystem::nativeOrientation() !=
              ICoordinateSystem::applicationOrientation() )
    {
      // Cursor the children and place them at the coordinates saved by IWindow.
      IWindow::ChildCursor cursor( *parent );

      // We have a special case for frames so look for a frame now.
      bool parentIsFrame( false );
      if (parent->isFrameWindow() )
        parentIsFrame = true;

      bool moveChild( true );

      for ( cursor.setToFirst();
            cursor.isValid();
            cursor.setToNext())
      {

        IWindow *win = parent->childWindowAt( cursor );
        if (win)
        {
          // We don't want to place clients and extensions. Let frame handler
          // do it.
          moveChild = true;
          if (parentIsFrame )
          {
            if (win == ((IFrameWindow *)parent)->client() )
              moveChild = false;
            else if ( ((IFrameWindow *)parent)->isAnExtension( win ) )
              moveChild = false;
          }
          // don't move clients and extensions of a frame.
          if (moveChild )
            win->moveSizeTo( win->rect() );
        }
      }
    }
    // leave bRc false so other handlers can see the map notification.
  }
  return bRc;
}

/*------------------------------------------------------------------------------
| IRecoordHandler::windowSizeChanged                                           |
------------------------------------------------------------------------------*/
bool IRecoordHandler::windowSizeChanged (
                      IWindowHandle     handle,
                      const IRectangle& newWindowRect,
                      const IRectangle& oldWindowRect  )
{
  IMODTRACE_ALL("IRecoordHandler::windowSizeChanged");
  if ( ICoordinateSystem::isConversionNeeded() )
  {
    // Only need to do this if operating in non-native coordinates
    // delta is value to be added to current y position of child windows
    long delta = newWindowRect.height() - oldWindowRect.height();

    // Determine if the window is a frame.  Need this later for
    // handling extensions.
    IWindow* pwin = IWindow::windowWithHandle( handle );
    bool isFrame = false;
    if (pwin)
    {

      isFrame = pwin->isFrameWindow();

      // Only manager widgets for now!!
      if (XmIsManager( handle ) )
      {
        // Use IWindows child cursor. This handles the aggregate control
        // for cursoring children

        IWindow::ChildCursor cursor( *pwin );
        for( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
        {
          IWindowHandle childHwnd = pwin->childAt( cursor );
          bool processChild = true;

          // We need to exclude frame extensions and the client window
          // from adjustment by this handler.   They are handled by
          // the frame extensions code and/or the system.
          if (isFrame)
          {
            // check for IFrameWindow extensions.
            IWindow* childWin = IWindow::windowWithHandle( childHwnd );
            if (childWin)
            {
              if ( ((IFrameWindow*)pwin)->isAnExtension( childWin ) )
                processChild = false;
            }
            if ( ((IFrameWindow*)pwin)->clientHandle() == childHwnd )
              processChild = false;
          }

          if (processChild)
          {
            Position y;
            Dimension fHeight;
            Dimension fParentHeight;
            // get current position of child window
            // By directly setting the y we bypass IWindow::moveSizeTo. The
            // logic is only executed when the native orientation does not
            // match the application orientation. The will cause
            // IWindowPrivateData::nativePosition to be incorrect, but
            // IWindowPrivateData::position will be correct.

            // ok, the plot thickens. I got a test case where they
            // are positioning the child during a resize for a canvas.
            // This means that we are battling it out with there
            // handler. In this case we need to do some checking 
            // before we can modify the y.
            XtVaGetValues( childHwnd,
              XmNy, &y,
              XmNheight, &fHeight,
              NULL );
            Widget fParent;
            if ((fParent = XtParent(childHwnd)) == IWindow::desktopWindow()->handle())
              fParentHeight = IWindow::desktopWindow()->size().height();
            else
            {
              XtVaGetValues( fParent,
                             XmNheight, &fParentHeight,
                             NULL );
            }
            
            if (fParentHeight - fHeight - y != pwin->position().y() )
            {
              XtVaSetValues( childHwnd,
                XmNy, y + delta,
                NULL );
            }
          }
        }
      }   // for
    }
  }
  return false;      //  pass on to other handlers
}

#endif // IC_MOTIF
