/*******************************************************************************
* FILE NAME: ireslib.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IResourceLibrary                                                         *
*     IDynamicLinkLibrary                                                      *
*     IResourceId                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 53  1.9.1.11  source/ui/baseapp/ireslib.cpp, resource, ioc.v400, 000919  

#pragma priority( -2147482112 )

extern "C" {
  #define INCL_DOSMODULEMGR
  #define INCL_DOSRESOURCES    /* Resource types */
  #define INCL_DOSINFOSEG
  #define INCL_DOSPROCESS
  #define INCL_WINERRORS
  #define INCL_WINPOINTERS
  #define INCL_WINMENUS
  #define INCL_WINHELP
  #define INCL_WINACCELERATORS
  #define INCL_DEV                /* Device Function definitions  */
  #define INCL_GPICONTROL         /* GPI control Functions        */
  #define INCL_GPIBITMAPS
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <X11/StringDefs.h>
  #include <X11/Xresource.h>
  #include <stdlib.h>
  #include <sys/ldr.h>
  #include <errno.h>
  #include <aouthdr.h>
  #include <loader.h>
  #include <filehdr.h>
  #include <scnhdr.h>
#endif //IC_MOTIF

#include <ibase.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <istring.hpp>
#include <ireslib.hpp>
#include <iresstat.hpp>
#include <iresprv.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>
#include <iwindow.hpp>
#include <iapp.hpp>
#include <ithread.hpp>
#include <ireslock.hpp>
#include <ihelpsta.hpp>
#include <iplatfrm.hpp>
#include <imenuprv.hpp>
#include <iacceltb.hpp>

#ifdef IC_MOTIFWIN
  #include <ihelptbl.hpp>
#endif

#ifdef IC_MOTIF
  #include <imenubld.hpp>
  #include <iaccelbd.hpp>
#endif //IC_MOTIF

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IRESLIB_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_MOTIF

static void* IResourceLibrary__resourceAddress = 0;
static char* IResourceLibrary__pResourceFileName = 0;
static char IResourceLibrary__resourceFileName[PATH_MAX] = "";

struct hdr {
   struct filehdr f;
   struct aouthdr a;
   struct scnhdr s[1];
   };

/*------------------------------------------------------------------------------
| IResourceLibrary__findResources                                              |
|    Return the address of the resources in this shared library if found.  The |
|    resources are stored with the symbol __rscs.
------------------------------------------------------------------------------*/
static void* IResourceLibrary__findResources(struct ld_info *info) {
   int i;

   struct hdr *hdr = (struct hdr *) info->ldinfo_textorg;
   char *data = (char *) info->ldinfo_dataorg;
   ulong datareloc = (ulong)data - hdr->s[hdr->a.o_sndata-1].s_vaddr;
   struct ldhdr *ldhdr;
   struct ldsym *ldsym;
   unsigned long *textOffset;

   /* find the loader symbol table */
   ldhdr = (struct ldhdr *)
               ((char *)hdr + hdr->s[hdr->a.o_snloader-1].s_scnptr);
   ldsym = (struct ldsym *) (ldhdr + 1);

   /* walk through the symbol table */
   for (i = 0; i < ldhdr->l_nsyms; ldsym++, i++) {
      /* only look at exports which are NOT imports - that
       * is are defined by this module
       */
      if (L_EXPORT!=(ldsym->l_smtype & (L_IMPORT | L_EXPORT)))
            continue;

      /* check that it is a valid symbol class */
      if (ldsym->l_smclas != XMC_RW && ldsym->l_smclas != XMC_RO)
         continue;

      if (strncmp(ldsym->l_name, "__rsrc", 6) == 0) {
         /* check that it is in .data */
         if (ldsym->l_scnum != hdr->a.o_sndata)
            continue;

         textOffset = (unsigned long *)(ldsym->l_value + datareloc);
              return (void *)(*textOffset);
         }
      }
   return 0;
   }

/*------------------------------------------------------------------------------
| IResourceLibrary::resourceAddress - Returns the address of resources in the  |
|                          executable file.                                    |
------------------------------------------------------------------------------*/
void* IResourceLibrary::resourceAddress ( ) const
{
   IFUNCTRACE_ALL();
   if(!IResourceLibrary__pResourceFileName)
   {
      // Query the loader info and find the symbol __rsrc
      struct ld_info *info;
      void* buf = 0;
      size_t bufsize = sizeof(struct ld_info)+2048;
      int forever = 1;

      // Get the loader info into a suitably sized buffer
      for (;;) {
         buf = realloc(buf, bufsize);
         if (buf == 0)
            return 0;
         if (-1 != loadquery(L_GETINFO, buf, bufsize))
            break;
         IASSERTSTATE(errno == ENOMEM);
         bufsize *= 2;
         }

      info = (struct ld_info*)buf;
      strcpy(IResourceLibrary__resourceFileName, info->ldinfo_filename);
      IResourceLibrary__pResourceFileName = IResourceLibrary__resourceFileName;
      IResourceLibrary__resourceAddress =
                                    IResourceLibrary__findResources(info);
      ITRACE_DEVELOP(IString("Resource address is ")
                     +IString((unsigned long)IResourceLibrary__resourceAddress).d2x());
      free(buf);
   }

   return IResourceLibrary__resourceAddress;
}

#ifndef IC_NOOLDRESOURCES
/*------------------------------------------------------------------------------
| mergeOverrideResources - Merges the application resource database with the   |
|                          specified resource file.                            |
------------------------------------------------------------------------------*/
void IResLibStaticPtr::mergeOverrideResources(const char * resourceFile)
{
  Display* display;
  XrmDatabase resourceDb, overrideDb;
  char *envVar, *resourceFilePathname;

  // If make sure resource database is loaded before
  // we merge/override the resources.
  //
  if (!IThread::current().isGUIInitialized()) {
    IThread::current().initializeGUI();
  }

  // Get reference to current application resource database.
  //
  display = XtDisplay( (IWindowHandle::Value)IThread::current().applicationShell());
  resourceDb = XtDatabase(display);

  // Get reference to resource file.
  // Search rules are as follows:
  //  1. find resource file treating given absolute <or> relative
  //     pathname asis.
  //  2. if given pathname is not absolute, look under directory
  //     specified by environment variable XAPPLRESDIR.
  //
  overrideDb = XrmGetFileDatabase(resourceFile);
  if (overrideDb == 0 && resourceFile[0] != '/') {
    if ((envVar = getenv("XAPPLRESDIR")) != 0) {
      resourceFilePathname = (char *)malloc(strlen(envVar)+strlen(resourceFile)+2);
      (void)strcpy(resourceFilePathname, envVar);
      (void)strcat(resourceFilePathname, "/");
      (void)strcat(resourceFilePathname, resourceFile);
      overrideDb = XrmGetFileDatabase(resourceFilePathname);
      free(resourceFilePathname);
    }
  }

  if (overrideDb == 0) {
    // Couldn't find resource file given above search rules.
    //
    ITHROWGUIERROR(IString("Can't read user resource file: ") +
                   IString(resourceFile));
  }

  // Merge/override resources.
  //
  XrmMergeDatabases(overrideDb, &resourceDb);

}

/*------------------------------------------------------------------------------
| getAppRes - Loads specified resource value from Xt resource database         |
------------------------------------------------------------------------------*/
static char* nullStr = ""; // d6944
char* IResLibStaticPtr::getAppRes(unsigned  long resId,
                                  char*          resPreamble,
                                  bool throwException)
{
  char* result;
  IString resName(resPreamble);
  resName = resName + IString(resId);

  // If somehow the first thing done in a client app is to load some
  // resource, then the following is needed to get the resource
  // database loaded.
  if (!IThread::current().isGUIInitialized())
    IThread::current().initializeGUI();

  XtResource resDef = {
    (char*)resName,
    (char*)resName,
    XtRString,
    sizeof(char*),
    0,
    XtRString,
    "\x01"             // if the resource can't be found, we'll get back
                       // this string
  };

  // Note that we are passing a char** as the base address in the 2nd
  // parm to XtGetApplicationResources.
  // What is happening is that we are not getting a copy of the string
  // resource, rather we are getting the address where the Resource
  // Manager has this particular resource tucked away.
  // Could not find any info in O'Reilly or elsewhere about how to
  // retrieve string application resources.
  // What we do below works, but has to be considered an undocumented
  // interface that could someday be broken. But if they do change it
  // they'd have to change the interface to tell you how long the
  // string is so that you could allocate storage for the copy.
  XtGetApplicationResources(IThread::current().applicationShell(),
                            &result,
                            &resDef, 1,
                            0, 0);

  if ( strcmp(result, "\x01") == 0 ) {
     if ( throwException)
        ITHROWGUIERROR(IString("Failed to load resource: ") +
                       IString(resPreamble) +
                       IString(resId) );
     result = nullStr;  // d6944
  }
  return result;
}
#endif // IC_NOOLDRESOURCES

#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IResourceLibrary::IResourceLibrary                                           |
------------------------------------------------------------------------------*/
IResourceLibrary::IResourceLibrary()
  : fExtendedData(0),
    fModuleHandle(0)
{
  IFUNCTRACE_ALL();

#ifdef IC_WIN
  fModuleHandle = GetModuleHandle( 0 );
#endif
}

/*------------------------------------------------------------------------------
| IResourceLibrary::IResourceLibrary                                           |
| Copy constructor                                                             |
------------------------------------------------------------------------------*/
IResourceLibrary::IResourceLibrary(const IResourceLibrary& resLibrary)
{
  IFUNCTRACE_ALL();
#ifdef IC_PMWIN
  fExtendedData = resLibrary.fExtendedData;
  // When IResLibPrivateData is defined be sure to copy contents.
#endif
  fModuleHandle = resLibrary.fModuleHandle;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::operator=                                                  |
------------------------------------------------------------------------------*/
IResourceLibrary& IResourceLibrary::operator= (const IResourceLibrary& resLibrary)
{
  IFUNCTRACE_ALL();
  if (this != &resLibrary)
  {
#ifdef IC_PMWIN
     fExtendedData = resLibrary.fExtendedData;
     // When IResLibPrivateData is defined be sure to copy contents.
#endif
     fModuleHandle = resLibrary.fModuleHandle;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::~IResourceLibrary                                          |
------------------------------------------------------------------------------*/
IResourceLibrary::~IResourceLibrary   ( )
{
// delete fExtendedData; when IResLibPrivateData is defined.
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IResourceLibrary::loadText                                                   |
------------------------------------------------------------------------------*/
IString IResourceLibrary::loadText(bool isString,
                                unsigned long stringId,
                                bool       throwException) const
{
  IFUNCTRACE_ALL();
  unsigned long returnLength;

  // Win32 LoadString() is failing for some DBCS strings when buffer
  // length does not accommodate the string.  Increasing buffer to work around
  // the problem.  Only happens on Windows '95
  unsigned long bufferLength = (IPlatform::isWin9x()) ? 1024 : 100;

  char* pszString = 0;
  IAnchorBlockHandle habTemp = IQUERYANCHOR(HWND_DESKTOP);

  pszString = new char[bufferLength];      /* work buffer  */

  while (1)
  {
     if(isString==true)
     {
        returnLength = ILOADSTRING( habTemp,
                          handle(), stringId, (int)bufferLength,
                          (PSZ)pszString);
        /* if an error, save it and quit */
        if (!returnLength)
        {
           /* Free memory if we allocated it. */
           if (pszString)
           {
             delete (pszString);
           }
           if(throwException)
              ITHROWGUIERROR(IString("WinLoadString(")+
                        "IResourceId:"+IString(stringId)+")");
        }
     }
     else
     {
        returnLength = ILOADMESSAGE(habTemp,
                          handle(), stringId, bufferLength,
                          (PSZ)pszString);
        /* if an error, save it and quit */
        if (!returnLength)
        {
           /* Free memory if we allocated it. */
           if (pszString)
           {
             delete (pszString);
           }
           if(throwException)
              ITHROWGUIERROR(IString("WinLoadMessage(")+
                        "IResourceId:"+IString(stringId)+")");
        }
     }



     /* if (possibly) truncated, get more buffer and retry */
     if (((returnLength == (bufferLength-1)) ||
          (returnLength == (bufferLength-2))) )
     {
       delete(pszString);
       bufferLength += bufferLength;
       pszString = new char[bufferLength];
     }
     else  /* All O.K. get out  */
        break;
  }
  if (returnLength)
  {
     IString returnString(pszString);
     delete [] pszString;
     return returnString;
  }
  else
  {
     return IString();
  }
}
#endif //IC_PMWIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IResourceLibrary::loadText                                                   |
------------------------------------------------------------------------------*/
IString IResourceLibrary::loadText(bool isString,
                                unsigned long stringId,
                                bool throwException) const
{
  IFUNCTRACE_ALL();

  char* text = 0;
  char* textResources = 0;

#ifndef IC_NOOLDRESOURCES
  /*------------------------------------------------------------------*/
  /* Try to load the string resource first from memory. If this       */
  /* fails, get the string resource from the database.                */
  /*------------------------------------------------------------------*/
#endif
  IAixResourceImp xResource(*this);
  if(isString)
    textResources = xResource.resourceByteAddress (RT_STRING, stringId);
  else
    textResources = xResource.resourceByteAddress (RT_MESSAGE, stringId);

  if (textResources)
    text = xResource.resourceData((rtstring*) textResources, stringId);
#ifndef IC_NOOLDRESOURCES
  else
    text = IResLibStaticPtr::getAppRes(stringId, "ICLSTRING_", throwException);
#endif   // IC_NOOLDRESOURCES

  if(!text && throwException == true)
     ITHROWGUIERROR(IString("Resource load failed ")+
               "IResourceId:"+IString(stringId));

  return IString(text);
}
#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IResourceLibrary::loadString                                                 |
------------------------------------------------------------------------------*/
IString IResourceLibrary::loadString(unsigned long stringId) const
{
  IFUNCTRACE_ALL();

  return(loadText(true, stringId));
}


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadString                                            |
------------------------------------------------------------------------------*/
IString IResourceLibrary::tryToLoadString(unsigned long stringId) const
{
  IFUNCTRACE_ALL();

  return(loadText(true, stringId, false));
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadMessage                                                |
------------------------------------------------------------------------------*/
IString IResourceLibrary::loadMessage(unsigned long messageId) const
{
  IFUNCTRACE_ALL();

  return(loadText(false, messageId));
}


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadMessage                                           |
------------------------------------------------------------------------------*/
IString IResourceLibrary::tryToLoadMessage(unsigned long messageId) const
{
  IFUNCTRACE_ALL();

  return(loadText(false, messageId, false));
}


#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IResourceLibrary::loadMenu                                                   |
------------------------------------------------------------------------------*/
IMenuHandle IResourceLibrary::loadMenu ( unsigned long menuId,
                                           IWindow* pwinMenuOwner) const
{
  IFUNCTRACE_ALL();

  IMenuHandle hwndMenu = ILOADMENU(pwinMenuOwner->handle(), handle(), menuId);

  if (!hwndMenu)
  {
     ITHROWGUIERROR(IString("WinLoadMenu(")+
                  "IResourceId:"+IString(menuId)+")");
  }
  return (hwndMenu);
}
#endif // IC_PMWIN


#ifdef IC_MOTIF
void IResourceLibrary::loadMenu(unsigned long        menuId,
                                const IWindowHandle& menuHandle,
                                IMenu*               parentMenu) const
{
  char* result;
  IMODTRACE_ALL("ResLib::loadMenu");
  IASSERTPARM(parentMenu!=0);

  /*------------------------------------------------------------------*/
  /* Try to load the menu resource first from memory. If this         */
  /* fails, get the menu resource from the database.                  */
  /*------------------------------------------------------------------*/
  IAixResourceImp xResource(*this);

  result = xResource.resourceByteAddress (RT_MENU, menuId);

  if (!result)
  {
    result = IResLibStaticPtr::getAppRes(menuId, "ICLMENU_");
    IMenuBld newMenu(result, menuHandle, parentMenu);
  }
  else
  {
    IAixMenuImp newMenu(result, menuHandle, parentMenu);
  }


  return;
}
#endif //IC_MOTIF


#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IResourceLibrary::loadDialog                                                 |
------------------------------------------------------------------------------*/
IWindowHandle IResourceLibrary::loadDialog(unsigned long dialogId,
                      IWindow* dialogParent, IWindow* dialogOwner,
                      IWinProc* dialogWindowProc,
                      void* dialogCreateParameters) const
{
  IFUNCTRACE_ALL();

  IWindowHandle hwndDlg = 0L;

  IWindowHandle hwndOwner = 0;
  IWindowHandle hwndParent = HWND_DESKTOP;
   if (dialogOwner != 0) {
      hwndOwner = dialogOwner->handle();
   } /* endif */
   if (dialogParent != 0) {
      hwndParent = dialogParent->handle();
   } /* endif */
#ifdef IC_WIN
   hwndDlg = ILOADDLG (hwndParent, hwndOwner,
                         (DLGPROC)dialogWindowProc,
                         handle(), dialogId, (LPARAM)dialogCreateParameters);
#else
   hwndDlg = ILOADDLG (hwndParent, hwndOwner,
                         dialogWindowProc,
                         handle(), dialogId, dialogCreateParameters);
#endif
  if (!hwndDlg)
  {
     ITHROWGUIERROR(IString("WinLoadDlg(")+
                  "IResourceId:"+IString(dialogId)+")");
  }
  return (hwndDlg);
}
#endif //IC_PMWIN

/*------------------------------------------------------------------------------
| IResourceLibrary::loadAccelTable                                             |
------------------------------------------------------------------------------*/
IAccelTblHandle IResourceLibrary::loadAccelTable ( unsigned long accelTableId ) const
{
  IFUNCTRACE_ALL();

  IAccelTblHandle haccel;
#ifdef IC_PMWIN
  IAnchorBlockHandle habTemp = IQUERYANCHOR( HWND_DESKTOP );

  haccel = ILOADACCELTABLE( habTemp, this->handle(), accelTableId );
#ifdef IC_WIN

  if ( IPlatform::isWin9x() )
  {
     // For Windows 95, avoid the use of LoadAccelerators.  MSDN
     // documentation warns against using DestroyAcceleratorTable on
     // handles created with LoadAccelerators (which returns the same
     // handle when called twice for the same accelerator table resource),
     // but requires DestroyAcceleratorTable be called for handles
     // created by CreateAcceleratorTable.  To be able to do clean-up in
     // a reasonable fashion, remap any handle created by LoadAccelerators
     // into one created by CreateAcceleratorTable.  Windows NT seems to
     // allow DestroyAcceleratorTable to be used with LoadAccelerators.
     IAcceleratorTable accelTable( haccel );
     haccel = accelTable.handle();
  }
#endif // IC_WIN
#endif // IC_PMWIN

#ifdef IC_MOTIF
  IAixResourceImp
    xResource( *this );
  char
   *accelData = xResource.resourceByteAddress( RT_ACCELTABLE,
                                               accelTableId );
  haccel = IAccelBld::rtacceltableToHandle( (rtacceltable*) accelData );
#endif // IC_MOTIF

  if ( ! haccel )
  {
     ITHROWGUIERROR( IString( "LoadAccelTable(" )+
                     "IResourceId:" + IString( accelTableId ) + ")" );
  }

  return haccel;
}

#ifdef IC_MOTIF
#if (IC_OBSOLETE <= IC_OBSOLETE_3)
void IResourceLibrary::loadAccelTable ( unsigned long accelTableId,
                                        const IWindowHandle& menu ) const
{
  IFUNCTRACE_DEVELOP();

  IAccelTblHandle
    haccel = this->loadAccelTable( accelTableId );
  IWindow
   *menuWindow = IWindow::windowWithHandle( menu );
  if ( menuWindow )
  {  // Assign the accelerator table to the owner of the menu.
     IWindow
      *menuOwner = menuWindow->owner();
     if ( menuOwner )
     {
        menuOwner->setAcceleratorHandle( haccel );
     }
  }

#ifndef IC_NOOLDRESOURCES
  if ( haccel == 0 )
  {  // Could not find an accelerator table using the new style of
     // resource loading.  Try the old style now, loading from the
     // resource database.
     char* result = IResLibStaticPtr::getAppRes( accelTableId, "ICLACCEL_" );
     IAccelBld newAccel( result, menu );
  }
#endif

  return;
}
#endif // IC_OBSOLETE
#endif //IC_MOTIF

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IResourceLibrary::loadHelpTable                                              |
------------------------------------------------------------------------------*/
IResourceLibrary& IResourceLibrary::loadHelpTable(IWindow* helpInstance,
                          unsigned long helpTableId) const
{
  IFUNCTRACE_DEVELOP();

  rthelptable* result;

  /*------------------------------------------------------------------*/
  /* Try to load the string resource first from memory. If this       */
  /* fails, get the string resource from the database.                */
  /*------------------------------------------------------------------*/
  IAixResourceImp xResource(*this);

  result =
    (rthelptable*) xResource.resourceByteAddress (RT_HELPTABLE,
                                                      helpTableId);

  if (!result)
  {
    if (IHelpStatics::loadHelpTable
           (helpInstance->handle(), helpTableId, handle())==0)
    {
       ITHROWGUIERROR(IString("WinLoadHelpTable(")+
                    "IResourceId:"+IString(helpTableId)+")");
    }
  }
  else
  {
    // get the pointer to any previous help table
    IHelpTable  *oldHT = (IHelpTable *)IHelpStatics::callIPF( helpInstance->handle(),
                                                HM_QUERY,
                                                IEventParameter1( 0, USERDATA),
                                                IEventParameter2(0) );
    // create the new helptable
    IHelpTable *ht = new IHelpTable( helpTableId );
    ht->processHelpTableData( result );

    // store a pointer to it in the help widget
    if (0 == IHelpStatics::callIPF( helpInstance->handle(),
                       HM_SET_USERDATA,
                      IEventParameter1(0),
                      IEventParameter2( (void *) ht) ) )
    {
     // clean up the old helptable
     if (oldHT)
        delete oldHT;
    }

  }


  return (IResourceLibrary&)*this;
}
#endif

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IResourceLibrary::loadHelpTable                                              |
------------------------------------------------------------------------------*/
IResourceLibrary& IResourceLibrary::loadHelpTable(IWindow* helpInstance,
                          unsigned long helpTableId) const
{
  IFUNCTRACE_DEVELOP();

  if (IHelpStatics::loadHelpTable
           (helpInstance->handle(), helpTableId, handle())==0)
  {
    ITHROWGUIERROR(IString("WinLoadHelpTable(")+
                  "IResourceId:"+IString(helpTableId)+")");
  }

  return (IResourceLibrary&)*this;
}
#endif

/*------------------------------------------------------------------------------
| IResourceLibrary::fileName                                                   |
|   Return the Dll File Name                                                   |
------------------------------------------------------------------------------*/
IString IResourceLibrary::fileName() const
{

  IFUNCTRACE_ALL();

#ifdef IC_PM
  PTIB    ptib;      /*  Thread Information Block  */
  PPIB    ppib;      /*  Process Information Block */
  DosGetInfoBlocks(&ptib, &ppib);
  IString str(ppib->pib_pchcmd);
#endif

#ifdef IC_WIN
  IString str(GetCommandLine());
#endif

#ifdef IC_MOTIF
  IString str;
  if(resourceAddress())
     str = IResourceLibrary__resourceFileName;
#ifndef IC_NOOLDRESOURCES
  else
    str = "XResources";
#endif

#endif //IC_MOTIF

  return (str);
};


/*------------------------------------------------------------------------------
| IResourceLibrary::isOpen                                                     |
------------------------------------------------------------------------------*/
bool IResourceLibrary::isOpen() const
{
  return true;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::handle                                                     |
------------------------------------------------------------------------------*/
IModuleHandle IResourceLibrary::handle() const
{
  return fModuleHandle;
}

