/*******************************************************************************
* FILE NAME: ireslib4.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the Resource Library              *
*   diagnostic functions.                                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 59 1.6.1.2 source/ui/baseapp/ireslib4.cpp, resource, ioc.v400, 000919  

extern "C" {
  #include <stdio.h>
  #define INCL_
  #include <iwindefs.h>
}

#include <ireslib.hpp>
#include <istring.hpp>
#include <irect.hpp>
#include <itrace.hpp>
#include <iset2.h>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRESLIB4_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IResourceLibrary::asDebugInfo                                                |
------------------------------------------------------------------------------*/
IString IResourceLibrary :: asDebugInfo() const
{
  return asString();
}

/*------------------------------------------------------------------------------
| IResourceLibrary::asString                                                   |
------------------------------------------------------------------------------*/
IString IResourceLibrary :: asString() const
{
  IMODTRACE_ALL("ResLib::asString");
  IString str = "IResourceLibrary(Filename(";
  str += fileName();
  str += "))";
  ITRACE_ALL(str);
  return str;
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::asDebugInfo                                             |
------------------------------------------------------------------------------*/
IString IDynamicLinkLibrary :: asDebugInfo() const
{
  return asString();
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::asString                                                |
------------------------------------------------------------------------------*/
IString IDynamicLinkLibrary :: asString() const
{
  IMODTRACE_ALL("DynLnkLib::asString");
  IString str = "IDynamicLinkLibrary(Filename(";
  str += fileName();
  str += "))";
  ITRACE_ALL(str);
  return str;
}

/*------------------------------------------------------------------------------
| IResourceId::IResourceId                                                     |
------------------------------------------------------------------------------*/
IResourceId :: IResourceId(unsigned long resourceId)
                       : fResLibrary ( IApplication::current().userResourceLibrary() ),
                         fResId   ( resourceId )
{ }

/*------------------------------------------------------------------------------
| IResourceId::asDebugInfo                                                     |
------------------------------------------------------------------------------*/
IString IResourceId :: asDebugInfo() const
{
   IMODTRACE_ALL("ResId::asDebugInfo");
   IString str = "IResourceId(Id(";
   str += fResId;
   str += "),";
   str += fResLibrary.asDebugInfo();
   str += ")";
   ITRACE_ALL(str);
   return str;
}

/*------------------------------------------------------------------------------
| IResourceId::asString                                                        |
------------------------------------------------------------------------------*/
IString IResourceId :: asString() const
{
   IMODTRACE_ALL("ResId::asString");
   IString str = "IResourceId(Id(";
   str += fResId;
   str += ")";
   ITRACE_ALL(str);
   return str;
}

