/* NOSHIP */
#ifndef _IRESPRV_
#define _IRESPRV_
/*******************************************************************************
* FILE NAME: iresprv.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IAixResourceImp                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 88  1.4.1.5  source/ui/baseapp/iresprv.hpp, resource, ioc.v400, 000919  

#ifdef IC_MOTIF
#include <ireslib.hpp>

#pragma enum(4)
/* #pragma pack(push,1) */
#pragma options align=packed


#define RT_POINTER       1
#define RT_BITMAP        2
#define RT_MENU          3
#define RT_DIALOG        4   /* not supported yet */
#define RT_STRING        5
#define RT_ACCELTABLE    8
#define RT_MESSAGE       10 
#define RT_HELPTABLE     18
#define RT_HELPSUBTABLE  19

#define BFT_BITMAPARRAY  0x4142   // 'BA'
#define BFT_BMAP         0x4D42   // 'BM'
#define BFT_ICON         0x4349   // 'IC'
#define BFT_POINTER		 0x5450   // 'PT'
#define BFT_COLORPTR	    0x5043   // 'CP'
#define BFT_COLORICON    0x4943   // 'CI'

// Note: We changed the format of the resource file for images after
//       the portapak release shipped.  Prior to the change, an XPM file
//       existed for the color image and another image for the AND/XOR
//       mask.  After the change (26188), just a single image exists
//       that uses the XPM format for defining the transparent color.
//
//       We also changed the data stored with each image and introduced
//       the sturct xpmbitmaparray2 for the new definition.  If 
//       restypeid = 0x02, we use the new struct, otherwise we use
//       xpmbitmaparray (portapak ship level only).

/**********************************************************************/
/* Resource Table Structure                                           */
/**********************************************************************/
struct restable
{
  unsigned short restypecount;          /* number of resource types   */
};

/**********************************************************************/
/* Resource Types                                                     */
/**********************************************************************/
struct restype
{
  unsigned short restype;               /* resource type              */
  unsigned long  count;                 /* total number resource id's */
  unsigned long  strcount;              /* number string resource id's*/
  unsigned long  offset;                /* offset from resource data  */
                                        /* start to resource id's for */
                                        /* this resource type         */
};

/**********************************************************************/
/* Ordinal id                                                         */
/**********************************************************************/
struct ordresid
{
  unsigned short resourceBundleId;      /* resource id (1 - 65535)    */
  unsigned long  offset;                /* offset from resource data  */
                                        /* start for this resource    */
};

/**********************************************************************/
/* Named id                                                           */
/**********************************************************************/
struct nameresid
{
  char*   name;                         /* null terminated string name*/
  unsigned long offset;                 /* offset from resource data  */
                                        /* start for this resource    */
};

/**********************************************************************/
/* Resource                                                           */
/**********************************************************************/
struct resources
{
  unsigned char restype;                /* 0xFF in portapak, 0x02 after */
  unsigned short restypeid;             /* 1-65535                    */
  unsigned char residtype;              /* if 0xFF then its a short   */
                                        /* otherwise its a string     */
  unsigned short resid;
  unsigned short options;               /* AIX ignores                */
  unsigned long  cb;                    /* size of resource in bytes  */
  char     bytes;                       /* resource                   */
};

/**********************************************************************/
/* RT_STRING                                                          */
/**********************************************************************/
struct stringdata
{
  unsigned char stringLen;              /* resource text length       */
  char data;                            /* resource text              */
};

struct rtstring
{
  unsigned short ccsid;                 /* id of the codeset          */
  stringdata stringData;                /* len/data string array[16]  */
};

struct xpmbitmaparray
{
  unsigned short type;                  /* BFT_ICON or BFT_BMAP       */
  unsigned long size;                   /* length in bytes of the data*/
  unsigned long offsetNext;             /* offset to the beginning of */
                                        /* the next xp or 0 if no more*/
};

struct xpmbitmaparray2
{
	unsigned short type;		             // the type of the bitmap, BFT_ICON, erc
	unsigned long  size;	                // size of the XpmBitmapArray Structure
	unsigned short width;                // width of the bitmap
	unsigned short height;	             // height of the bitmap
	unsigned short bitCount;	          // 1 = b/w >1 = color
	unsigned short colors;	             // colors used(including color none)
	unsigned long  len;		             // length of the actual Xpm data
	unsigned long offsetNext;	// offset to next xpmbitmaparray2 
                              // from the start of this one or 0
};

struct rtbitmap
{
  xpmbitmaparray xpmba;                 /* xpmbitmaparray (or 2)      */
};

struct rtmenu
{
  unsigned short length;                /* length of template         */
  unsigned short version;               /* 1-V1.2 otherwise V1.1      */
  unsigned short ccsid;                 /* id of the codeset          */
  unsigned short itemOffset;            /* relative to here           */
  unsigned short itemCount;             /* number of items            */
};

struct rtmenuv1
{
  unsigned short length;                /* length of template         */
  unsigned short version;               /* 1-V1.2 otherwise V1.1      */
  unsigned short ccsid;                 /* id of the codeset          */
  unsigned short itemOffset;            /* relative to here           */
  unsigned short itemCount;             /* number of items            */
  unsigned short presParamOffset;       /* offset to pres. parm data  */
};

struct menuitem
{
  unsigned short style;
  unsigned short attributes;
  unsigned short id;
  char text;
};

struct rthelptable
{
  unsigned short appwindowId;
  unsigned short helpsubtableId;
  unsigned short thirdItem;
  unsigned short exthelppanelId;
};

struct helpsubtabledata
{
  unsigned short childId;
  unsigned short helpPanelId;
};

struct rthelpsubtable
{
  unsigned short subitemsize;
  helpsubtabledata theHelpSubtableData;
};

struct accelerator
{
   unsigned short   options;   /*  Options (fs).      */
   unsigned short   key;       /*  Key.               */
   unsigned short   commandd;  /*  Command code(cmd). */
};
struct rtacceltable
 {
   unsigned short    accelNumber;     /*  Number of accelerator entries.(CAccel) */
   unsigned short    codepage;        /*  Code page for accelerator entries. */
   accelerator       accelerators[1]; /*  Accelerator entries. */
 };
 
struct res
{
  restable  resTable;
  restype   resType;
  ordresid  ordResId;
  nameresid nameResId;
  resources resource;
};

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

// #pragma pack(pop)
#pragma options align=reset
#pragma enum(pop)

#pragma enum(4)
#pragma pack(push,4)

class IAixResourceImp {
public:
/*------------------------------- Constructors -------------------------------*/
  IAixResourceImp  ( const IResourceLibrary& resourceLibrary );


 ~IAixResourceImp  ( );


restype
 *resourceTypeAddress      ( unsigned short resType );

ordresid
 *resourceOrdinalIdAddress ( unsigned short resType,
                             unsigned short resourceId );

char
 *resourceByteAddress      ( unsigned short resType,
                             unsigned short resourceId );

char
 *resourceData             ( rtstring* pResource,
                             unsigned short resourceId );

void
  resourceData             ( rtbitmap* pResource,
                             char** xpmDataColor);

private:

const IResourceLibrary
  &fResourceLibrary;

unsigned short
  fResourceTypeCount;

res
 *fResourceStart;
unsigned long
  fVersion;
};  // IAixResourceImp

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // IC_MOTIF

#endif /* _IRESPRV_ */
