// Revision: 63 1.10 source/ui/baseapp/iscrlhdr.cpp, slidectls, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: iscrlhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iscrlhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINMESSAGEMGR
#define INCL_WINSCROLLBARS
extern "C"
  {
  #include <iwindefs.h>
  #ifdef IC_MOTIF
    #include<Xm/ScrollBar.h>
  #endif //IC_MOTIF
  }

#include <iscrlhdr.hpp>
#include <ievent.hpp>
#include <iscrlevt.hpp>
#include <iscroll.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISCRLHDR_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_MOTIF

#pragma enum(4)
#pragma pack(push,4)

class IScrollHandlerPrivateData
  {
  public:
    IScrollHandlerPrivateData()        :
      bClOkForScrollEnd( false)
      { ; }
    // IThread will dispatch a SB_ENDSCROLL anytime a button 1 release is
    // detected        on a scroll bar. This flag will determine if it is
    // appropriate to call the scrollEnd virtual function.
    IScrollHandlerPrivateData( const IScrollHandlerPrivateData &pData)
      {
      bClOkForScrollEnd        = pData.bClOkForScrollEnd;
      }
    bool bClOkForScrollEnd;
  };

#pragma pack(pop)
#pragma enum(pop)

#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IScrollHandler::IScrollHandler                                               |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IScrollHandler::IScrollHandler ( )
  : IScrollHandler::Inherited( )
#ifdef IC_MOTIF
    ,pData( new IScrollHandlerPrivateData )
#endif //IC_MOTIF
{
}

/*------------------------------------------------------------------------------
| IScrollHandler::~IScrollHandler                                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IScrollHandler::~IScrollHandler ( )
{
#ifdef IC_MOTIF
  delete pData;
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| IScrollHandler::dispatchHandlerEvent                                         |
|                                                                              |
| Dispatch scroll event handler functions.                                     |
| UNIX Notes:                                                                                             |
|  1) IThread will generate a SB_ENDSCROLL if button 1 release is detected on  |
|     on a scroll bar. The handler will        keep track whether it is appropriate   |
|     to dispatch to the handler or not.                                                      |
------------------------------------------------------------------------------*/
bool IScrollHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

#ifdef IC_PMWIN
  switch ( event.eventId() )
  {
     case WM_VSCROLL:
     {
        IScrollEvent scrlevt( event );
#ifdef IC_WIN
        switch ( event.parameter1().number1() )
#else
        switch ( event.parameter2().number2() )
#endif
        {
           case SB_LINEUP:
              bRc = this->lineUp( scrlevt );
              break;
           case SB_LINEDOWN:
              bRc = this->lineDown( scrlevt );
              break;
           case SB_PAGEUP:
              bRc = this->pageUp( scrlevt );
              break;
           case SB_PAGEDOWN:
              bRc = this->pageDown( scrlevt );
              break;
           case SB_SLIDERTRACK:
              bRc = this->scrollBoxTrack( scrlevt );
              break;
           case SB_SLIDERPOSITION:
              bRc = this->scrollBoxTrackEnd( scrlevt );
              break;
           case SB_ENDSCROLL:
              bRc = this->scrollEnd( scrlevt );
              break;
           default:
              break;
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( scrlevt.result());  // Copy event result.
        }
        break;
     }

     case WM_HSCROLL:
     {
        IScrollEvent scrlevt( event );
#ifdef IC_WIN
        switch ( event.parameter1().number1() )
#else
        switch ( event.parameter2().number2() )
#endif
        {
           case SB_LINELEFT:
              bRc = this->lineLeft( scrlevt );
              break;
           case SB_LINERIGHT:
              bRc = this->lineRight( scrlevt );
              break;
           case SB_PAGELEFT:
              bRc = this->pageLeft( scrlevt );
              break;
           case SB_PAGERIGHT:
              bRc = this->pageRight( scrlevt );
              break;
           case SB_SLIDERTRACK:
              bRc = this->scrollBoxTrack( scrlevt );
              break;
           case SB_SLIDERPOSITION:
              bRc = this->scrollBoxTrackEnd( scrlevt );
              break;
           case SB_ENDSCROLL:
              bRc = this->scrollEnd( scrlevt );
              break;
           default:
              break;
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( scrlevt.result() );  // Copy event result.
        }
        break;
     }

     default:
        break;
  }
#endif //IC_PMWIN

#ifdef IC_MOTIF
  if ( XtIsSubclass( event.controlHandle(), xmScrollBarWidgetClass ))
  {
     IScrollEvent scrlevt(event);
     XEvent* xEvent = (XEvent*) event.parameter2().asUnsignedLong();

     switch (event.eventId())
     {
       case motifEvent(XmCR_INCREMENT):
         event.controlWindow()->postEvent(IC_UM_CR_INCREMENT);
         bRc = true;
         break;
       case motifEvent(XmCR_DECREMENT):
         event.controlWindow()->postEvent(IC_UM_CR_DECREMENT);
         bRc = true;
         break;
       case motifEvent(XmCR_PAGE_DECREMENT):
       {
         if ( xEvent && xEvent->type == ButtonPress )
         {
           IEventParameter1 eparm1(0);
           IEventParameter2 eparm2(true);
           XButtonEvent* xButtonEvt =
             (XButtonEvent*) event.parameter2().asUnsignedLong();
           if ( xButtonEvt && xButtonEvt->button == Button1 )
             event.controlWindow()->postEvent(IC_UM_CRPAGE_DECREMENT,
               eparm1, eparm2);
         }
         else
         {
           event.controlWindow()->postEvent(IC_UM_CRPAGE_DECREMENT);
         }
         bRc = true;
         break;
       }
       case motifEvent(XmCR_PAGE_INCREMENT):
       {
         if ( xEvent && xEvent->type == ButtonPress )
         {
           IEventParameter1 eparm1(0);
           IEventParameter2 eparm2(true);
           XButtonEvent* xButtonEvt =
             (XButtonEvent*) event.parameter2().asUnsignedLong();
           if ( xButtonEvt && xButtonEvt->button == Button1 )
             event.controlWindow()->postEvent(IC_UM_CRPAGE_INCREMENT,
               eparm1, eparm2);
         }
         else
         {
           event.controlWindow()->postEvent(IC_UM_CRPAGE_INCREMENT);
         }
         bRc = true;
         break;
       }
       case IC_UM_CR_INCREMENT:                                         
       {
         IScrollBar* pscrbar = scrlevt.scrollBarWindow();
         if (pscrbar->isHorizontal())
            bRc = lineRight( scrlevt );
         else
            bRc = lineDown( scrlevt );
         pData->bClOkForScrollEnd = true;
         break;
       }
       case IC_UM_CR_DECREMENT:                                         
       {
         IScrollBar* pscrbar = scrlevt.scrollBarWindow();
         if (pscrbar->isHorizontal())
            bRc = lineLeft( scrlevt );
         else
            bRc = lineUp( scrlevt );
         pData->bClOkForScrollEnd = true;
         break;
       }
       case motifEvent(XmCR_DRAG):
       {
         IScrollBar* pscrbar = scrlevt.scrollBarWindow();
         bRc = scrollBoxTrack( scrlevt );
         pData->bClOkForScrollEnd = false;
         break;
       }
       case IC_UM_CRPAGE_DECREMENT:                                    
       {
         IScrollBar* pscrbar = scrlevt.scrollBarWindow();
         if (pscrbar->isHorizontal())
            bRc = pageLeft( scrlevt );
         else
            bRc = pageUp( scrlevt );
         pData->bClOkForScrollEnd = true;
         break;
       }
       case IC_UM_CRPAGE_INCREMENT:                                    
       {
         IScrollBar* pscrbar = scrlevt.scrollBarWindow();
         if (pscrbar->isHorizontal())
            bRc = pageRight( scrlevt );
         else
            bRc = pageDown( scrlevt );
         pData->bClOkForScrollEnd = true;
         break;
       }
       case motifEvent(XmCR_TO_BOTTOM):
       case motifEvent(XmCR_TO_TOP):        // Treat this as a drag
       case motifEvent(XmCR_VALUE_CHANGED):
       {
         IScrollBar* pscrbar = scrlevt.scrollBarWindow();
         bRc = scrollBoxTrackEnd( scrlevt);
         pData->bClOkForScrollEnd = false;
         break;
       }
       case SB_ENDSCROLL:
       {
         if (pData->bClOkForScrollEnd)
         {
            pData->bClOkForScrollEnd = false;
            IScrollBar* pscrbar = scrlevt.scrollBarWindow();
            bRc = scrollEnd( scrlevt );
         }
         break;
       }
       default:
         break;
     } /* endswitch */

     if (bRc)
        event.setResult(scrlevt.result());  //        copy result
  }
#endif //IC_MOTIF

  return bRc;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineUp                                                       |
------------------------------------------------------------------------------*/
bool IScrollHandler::lineUp ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineDown                                                     |
------------------------------------------------------------------------------*/
bool IScrollHandler::lineDown ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageUp                                                       |
------------------------------------------------------------------------------*/
bool IScrollHandler::pageUp ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageDown                                                     |
------------------------------------------------------------------------------*/
bool IScrollHandler::pageDown ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineLeft                                                     |
------------------------------------------------------------------------------*/
bool IScrollHandler::lineLeft ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineRight                                                    |
------------------------------------------------------------------------------*/
bool IScrollHandler::lineRight ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageLeft                                                     |
------------------------------------------------------------------------------*/
bool IScrollHandler::pageLeft ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageRight                                                    |
------------------------------------------------------------------------------*/
bool IScrollHandler::pageRight ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::scrollBoxTrack                                               |
------------------------------------------------------------------------------*/
bool IScrollHandler::scrollBoxTrack ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::scrollBoxTrackEnd                                            |
------------------------------------------------------------------------------*/
bool IScrollHandler::scrollBoxTrackEnd ( IScrollEvent& event )
{
#ifdef IC_PMWIN
  this->moveScrollBox( event );
#endif //IC_PMWIN
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::scrollEnd                                                    |
------------------------------------------------------------------------------*/
bool IScrollHandler::scrollEnd ( IScrollEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IScrollHandler::moveScrollBox                                                |
|                                                                              |
| Move the scroll box according to the information in the scroll event.        |
------------------------------------------------------------------------------*/
IScrollHandler& IScrollHandler::moveScrollBox ( IScrollEvent& event )
{
  IScrollBar* pscrbar = event.scrollBarWindow();
  if ( pscrbar )
  {
#ifdef IC_PMWIN
#ifdef IC_WIN
     switch ( event.parameter1().number1() )
#else
     switch ( event.parameter2().number2() )
#endif
     {                                 // Switch on scroll bar event.
        case SB_LINEUP:                // (Same as SB_LINELEFT.)
        case SB_LINEDOWN:              // (Same as SB_LINERIGHT.)
        case SB_PAGEUP:                // (Same as SB_PAGELEFT.)
        case SB_PAGEDOWN:              // (Same as SB_PAGERIGHT.)
           pscrbar->moveScrollBoxTo( pscrbar->scrollBoxPosition() +
                                       event.scrollAmount() );
           break;
        case SB_SLIDERPOSITION:        // Fall into next case.
        case SB_SLIDERTRACK:           // Fall into next case.
           pscrbar->moveScrollBoxTo( event.newScrollBoxPosition() );
           break;
        case SB_ENDSCROLL:             // Fall into next case.
        default:
           break;
     }
#endif //IC_PMWIN

#ifdef IC_MOTIF
    switch (event.eventId())
      {
      case motifEvent(XmCR_DECREMENT):                                  
      case IC_UM_CR_DECREMENT:                                  
      case motifEvent(XmCR_INCREMENT):                                  
      case IC_UM_CR_INCREMENT:                                  
      case motifEvent(XmCR_PAGE_DECREMENT):                             
      case IC_UM_CRPAGE_DECREMENT:                                  
      case motifEvent(XmCR_PAGE_INCREMENT):                             
      case IC_UM_CRPAGE_INCREMENT:                                  
            pscrbar->moveScrollBoxTo(pscrbar->prevScrollBoxPosition() +
              event.scrollAmount() );
            break;
      case motifEvent(XmCR_TO_BOTTOM):                                  
      case motifEvent(XmCR_TO_TOP):                                     
      case motifEvent(XmCR_VALUE_CHANGED):                              
            pscrbar->moveScrollBoxTo(event.newScrollBoxPosition());
            break;
      default:
            break;
      } /* endswitch */
#endif //IC_MOTIF
  }

  return *this;
}
