// Revision: 69 1.5.1.5 source/ui/baseapp/isizehdr.cpp, genhdr, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: isizehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isizehdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINFRAMEMGR          // for WM_WINDOWPOSCHANGED
extern "C" 
  {
  #include <iwindefs.h>
  }

#include <icconst.h>
#include <icoordsy.hpp>
#include <isizehdr.hpp>
#include <isizeevt.hpp>
#include <iwindow.hpp>
#include <irect.hpp>
#ifdef IC_MOTIF
  #include <isizrect.hpp>
#endif
// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISIZEHDR_CPP_
  #include <ipagetun.h>
#endif


/*******************************************************************/
/* Default constructor here for page tuning.                       */
/*******************************************************************/
IResizeHandler :: IResizeHandler ( )
  : IResizeHandler::Inherited ( )
{}


/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IResizeHandler :: ~IResizeHandler ( )
{}


/************************************************************************/
/* dispatchhandlerEvent - Dispatch command functions for this handler.  */
/************************************************************************/
bool  IResizeHandler :: dispatchHandlerEvent ( IEvent& evt )
{
   bool bRc = false;

#ifdef IC_WIN
   // Windows does not provide the old size on WM_WINDOWPOSCHANGED.
   // We save the old size in IWindow private data and use it on a user message
   // which is sent and processed by IResizeHandler.  User message is
   // as follows
   //    msgid    IC_UM_WINDOWPOSCHANGED
   //    wparam   (unused)
   //    lparam   address of array of 2 SIZE structures with new size
   //             and old size in that order
   if (evt.eventId() == IC_UM_WINDOWPOSCHANGED)
      {
      // don't need checking or default procedure since IWindow does
      // checking and the message is a user message.
      IResizeEvent resevt(evt);
      bRc = windowResize(resevt);      // call derived class (if any)
      if (bRc)
         evt.setResult(resevt.result());  // copy the derived result
      }
#endif //IC_WIN

#ifdef IC_PM
   if (evt.eventId() == WM_WINDOWPOSCHANGED)
   {                              // size/move event found
      PSWP pswp = (PSWP)(char*)evt.parameter1();
      if (pswp->fl & SWP_SIZE)
      {                           // size event found
         IResizeEvent resevt(evt);
         bRc = windowResize(resevt);      // call derived class (if any)
         if ( bRc )       // call derived class (if any)
         {                        // event has been processed
            // Generate a user message if the resize handler is stopping
            // processing of the message.   IRecoordHandler needs to
            // see resize events to work properly.  If this handler
            // is stopping the message we generate a substitute.
            // Note that this logic relies on the fact that there is
            // no ICLUI handler of WM_WINDOWPOSCHANGED which returns
            // true.  This means that the IC_UM... message ONLY occurs
            // if a user supplied handler returns true before the
            // IRecoordHandler has run.
            // This is not a problem in Windows since we use different
            // messages in IRecoordHandler and IResizeHandler.
            resevt.handle().sendEvent(
               IC_UM_WINDOWPOSCHANGED,
               resevt.parameter1(),
               resevt.parameter2() );

            /*******************************************************/
            /* The handler has said not to pass on the event, so   */
            /* don't pass it to another handler.  However, we do   */
            /* go ahead and pass it to the window procedure of the */
            /* current window, since it is only a notification     */
            /* message and we could break something by not doing   */
            /* this (this message is PM's way of generating the    */
            /* WM_SIZE, WM_SHOW, and WM_HIDE messages, if the      */
            /* message gets sent on to the default window          */
            /* procedure, and there's no harm in not eliminating   */
            /* these messages).                                    */
            /*******************************************************/
            defaultProcedure(evt);
            evt.setResult(resevt.result());  // copy the derived result
         }
      } /* endif SWP_SIZE */
   } /* endif WM_WINDOWPOSCHANGED */
#endif //IC_PM

#ifdef IC_MOTIF
  if ( ( evt.eventId() == xEvent(ConfigureNotify)) ||
       ( evt.eventId() == IC_UM_SIZE ) )
  {
    ISizeRectangle 
     *sizeRect((ISizeRectangle *) evt.parameter1().asUnsignedLong() );
    if ( sizeRect->sizeChanged() )    
    {
      IResizeEvent resevt (evt);
      bRc = windowResize (resevt);
      if ( bRc )       // call derived class (if any)
      {    
        // event has been processed
        // Generate a user message if the resize handler is stopping
        // processing of the message. IRecoordHandler needs to
        // see resize events to work properly.  If this handler
        // is stopping the message we generate a substitute.
        // Note that this logic relies on the fact that there is
        // no ICLUI handler of WM_WINDOWPOSCHANGED which returns
        // true.  This means that the IC_UM... message ONLY occurs
        // if a user supplied handler returns true before the
        // IRecoordHandler has run.
        // This is not a problem in Windows since we use different
        // messages in IRecoordHandler and IResizeHandler.
        if (evt.eventId() == xEvent(ConfigureNotify) )
        {
          resevt.handle().sendEvent(
            IC_UM_WINDOWPOSCHANGED,
            resevt.parameter1(),
            resevt.parameter2() );
        }

         evt.setResult(resevt.result());  // copy the derived result
      }
    }
  }
#endif //IC_MOTIF

   return bRc;
}
