// Revision: 77 1.4.1.4 source/ui/baseapp/ispinhdr.cpp, listctls, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ispinhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDSPIN
#define INCL_WINWINDOWMGR
extern "C"
  {
  #include <iwindefs.h>
  #include <stdlib.h>
  }

#ifdef IC_WIN
  #include <iclspbw.h>
#endif

#include <ispinhdr.hpp>
#include <ictlevt.hpp>
#include <ievent.hpp>
#include <iwindow.hpp>
#include <iwcname.hpp>
#include <icconst.h>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISPINHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISpinHandler::ISpinHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISpinHandler::ISpinHandler ( )
  : ISpinHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| ISpinHandler::~ISpinHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISpinHandler::~ISpinHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISpinHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
bool ISpinHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

#ifdef IC_PMWIN
  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl  &&
#ifdef IC_WIN
         ((IWindowClassName( hwndControl ) == WC_SPINBUTTON) ||
          (IWindowClassName( hwndControl ) == WC_NATIVESPINBUTTON)) )
#else
         IWindowClassName( hwndControl ) == WC_SPINBUTTON )
#endif
     {                                 // Spin button case.
        unsigned long subMsg = ctlevt.parameter1().number2();

        if ( subMsg == SPBN_UPARROW )
        {
           bRc = this->arrowUp( ctlevt );
        }
        else if ( subMsg == SPBN_DOWNARROW )
        {
           bRc = this->arrowDown( ctlevt );
        }
        else if ( subMsg == SPBN_ENDSPIN )
        {
           bRc = this->spinEnded( ctlevt );
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  }
#endif //IC_PMWIN

#ifdef IC_MOTIF
  // Check the notify code.
  if (event.eventId() == WM_CONTROL)
  {
     // The event might be for a servant which has focus, but the user
     // is spinning via the master's arrow buttons.  In any case, the
     // intended spin button will have focus.  Construct a new event
     // identical to the original, but use the window of the
     // current focus widget.
     IEvent adjustedEvent = IEvent (IPARENTOF ( XmGetFocusWidget (event.controlHandle())),
                                   event.eventId(),
                                   event.parameter1(),
                                   event.parameter2());

     IControlEvent ctlevt (adjustedEvent);

     switch (IEventData        (event.parameter1()).number2()) {
       case SPBN_UPARROW:
         if (bRc = arrowUp (ctlevt))
            event.setResult(ctlevt.result());  //      copy result
         break;
       case SPBN_DOWNARROW:
         if (bRc = arrowDown (ctlevt))
            event.setResult(ctlevt.result());  //      copy result
         break;
       case SPBN_ENDSPIN:
         if (bRc = spinEnded (ctlevt))
            event.setResult(ctlevt.result());  //      copy result
         break;
     }
  }
#endif //IC_MOTIF

  return bRc;
}

/*------------------------------------------------------------------------------
| ISpinHandler::arrowUp                                                        |
------------------------------------------------------------------------------*/
bool ISpinHandler::arrowUp ( IControlEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISpinHandler::arrowDown                                                      |
------------------------------------------------------------------------------*/
bool ISpinHandler::arrowDown ( IControlEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISpinHandler::spinEnd                                                        |
------------------------------------------------------------------------------*/
bool ISpinHandler::spinEnded ( IControlEvent& event )
{
  return false;
}
