// Revision: 80 1.4.1.4 source/ui/baseapp/istathdr.hpp, staticctls, ioc.v400, 000919 
/* NOSHIP */
#ifndef _ISTATHDR_
  #define _ISTATHDR_
/*******************************************************************************
* FILE NAME: istathdr.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IStaticTextHandler                                                       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ipainhdr.hpp>
#include <isizehdr.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IStaticText;

#pragma enum(4)
#pragma pack(push,4)

#ifdef IC_PMWIN
class IStaticTextHandler : public IPaintHandler {
typedef IPaintHandler
  Inherited;
#endif // IC_PMWIN
#ifdef IC_MOTIF
class IStaticTextHandler : public IResizeHandler {
typedef IResizeHandler
  Inherited;
#endif // IC_MOTIF

public:
/*------------------------------- Constructor ----------------------------------
| The only way to construct instances of this class is to use the default      |
| constructor.                                                                 |
------------------------------------------------------------------------------*/
  IStaticTextHandler (  );

virtual
 ~IStaticTextHandler ( );

/*-------------------------------- Drawing -------------------------------------
  drawBackground - draws the background area of the control, such as any
                   image or fill color.
  drawForeground - draws the foreground area of the control.  This is
                   typically the text.
------------------------------------------------------------------------------*/
virtual bool
  drawBackground ( IStaticText*      staticText,
                   IPresSpaceHandle& ps,
                   const IRectangle& updateRect );
virtual bool
  drawForeground ( IStaticText*      staticText,
                   IPresSpaceHandle& ps,
                   const IRectangle& updateRect );

protected:
/*---------------------------- Event Dispatching -------------------------------
| We override dispatchHandlerEvent to process several                          |
| messages not usually handled by a paint handler.                             |
------------------------------------------------------------------------------*/
#ifdef IC_MOTIFWIN
virtual bool
  dispatchHandlerEvent ( IEvent& event );
#endif

/*-------------------------------- Overrides -----------------------------------
| These inherited function overrides cause the drawing of the control to       |
| occur by calling drawBackground and drawForeground.                          |
|   paintWindow - (PMWIN) Process the paint events for a static text control   |
|                 window.                                                      |
|   windowResize - (MOTIF) Process a resize or initial show event for a static |
|                 text control window.                                         |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
virtual bool
  paintWindow          ( IPaintEvent& evt );
#endif
#ifdef IC_MOTIF
virtual bool
  windowResize         ( IResizeEvent& evt );
#endif //IC_MOTIF

/*-------------------------------- Text Control --------------------------------
| These functions provide access to a pointer to the control for which this    |
| handler is handling events.  This is an optimization which relies upon the   |
| fact that a new instance of this handler is created for each IStaticText     |
| object.                                                                      |
------------------------------------------------------------------------------*/
IStaticText
  *staticText( ) const                     { return fstaticText; };
void
  setStaticText( IStaticText* staticText ) { fstaticText = staticText; };

private:
  IStaticText
     *fstaticText;
}; // class IStaticTextHandler

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTATHDR_
