// Revision: 85 1.5.1.2 source/ui/baseapp/iswp.inl, datatype, ioc.v400, 000919 
#ifndef _ISWP_INL_
#define _ISWP_INL_
/*******************************************************************************
* FILE NAME: iswp.inl                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of the inline functions for the          *
*   classes declared in iswp.hpp.                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp. 1992, 1997  All Rights Reserved                    *
*                                                                              *
*******************************************************************************/

#include <ipoint.hpp>

/*------------------------------------------------------------------------------
| ISWP::ISWP                                                                   |
------------------------------------------------------------------------------*/
inline ISWP::ISWP ( )
  : fl          ( 0 ),
    cy          ( 0 ),
    cx          ( 0 ),
    y           ( 0 ),
    x           ( 0 ),
    behind      ( 0 ),
    hwnd        ( 0 )
#ifdef IC_PM
  , ulReserved1 ( 0 ),
    ulReserved2 ( 0 )
#endif
{ }

/*------------------------------------------------------------------------------
| ISWP::ISWP                                                                   |
------------------------------------------------------------------------------*/
inline ISWP::ISWP ( const ISWP& original )
  : fl          ( original.fl     ),
    cy          ( original.cy     ),
    cx          ( original.cx     ),
    y           ( original.y      ),
    x           ( original.x      ),
    behind      ( original.behind ),
    hwnd        ( original.hwnd   )
#ifdef IC_PM
  , ulReserved1 ( original.ulReserved1 ),
    ulReserved2 ( original.ulReserved2 )
#endif
{ }

/*------------------------------------------------------------------------------
| ISWP::~ISWP                                                                  |
------------------------------------------------------------------------------*/
inline ISWP::~ISWP ( )
{ }

/*------------------------------------------------------------------------------
| ISWP::operator=                                                              |
------------------------------------------------------------------------------*/
inline ISWP& ISWP::operator= ( const ISWP& original )
{
   if (this != &original)
      {
      fl          = original.fl     ;
      cy          = original.cy     ;
      cx          = original.cx     ;
      y           = original.y      ;
      x           = original.x      ;
      behind      = original.behind ;
      hwnd        = original.hwnd   ;
#ifdef IC_PM
      ulReserved1 = original.ulReserved1;
      ulReserved2 = original.ulReserved2;
#endif
      }
   return *this;
}

inline IWindowHandle ISWP::windowHandle ( ) const
  {
  return this->hwnd;
  }

inline IPoint ISWP::position ( ) const
  {
  return IPoint( this->x, this->y );
  }

inline ISWP& ISWP::setPosition ( const IPoint& point )
  {
  this->x = point.x();
  this->y = point.y();
  return *this;
  }

inline ISize ISWP::size ( ) const
  {
  return ISize( this->cx, this->cy );
  }

inline ISWP& ISWP::setSize ( const ISize& size )
  {
  this->cx = size.width();
  this->cy = size.height();
  return *this;
  }

inline ISWPArray::ISWPArray ( ISWP* array, unsigned dimension )
  : p( array ),
    dim( dimension )
  {
  }

inline unsigned long& ISWP::flags ( )
  {
  return this->fl;
  }

inline unsigned ISWPArray::size ( ) const
  {
  return this->dim;
  }

inline ISWP& ISWPArray::operator [] ( unsigned index )
  {
  return p[index];
  }

#endif // _ISWP_INL_
