// Revision: 09 1.7.1.3 source/ui/baseapp/iwindow7.cpp, basewin, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: iwindow7.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions used for the    *
*   IWindow notifcation handler in OS/2 and Windows.                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINFRAMEMGR
#define INCL_WINSYS

extern "C" {
   #include <iwindefs.h>
}

#include <iwinnhdr.hpp>
#include <iwindow.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <irect.hpp>
#ifdef IC_MOTIF
  #include <isizrect.hpp>
#endif
// Segment definitions
#ifdef IC_PAGETUNE
  #define _IWINDOW7_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IWindowNotifyHandler::IWindowNotifyHandler                                   |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IWindowNotifyHandler::IWindowNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IWindowNotifyHandler::~IWindowNotifyHandler                                  |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IWindowNotifyHandler::~IWindowNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IWindowNotifyHandler::dispatchHandlerEvent                                   |
|                                                                              |
| This notifies IWindows of the following events:                              |
|                                                                              |
| size(Id)                                                                     |
| position(Id)                                                                 |
| visible(Id)                                                                  |
| enable(Id)                                                                   |
| destroy(Id)                                                                  |
| command(Id)                                                                  |
| sysCommand(Id)                                                               |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN

#ifdef IC_PM
bool IWindowNotifyHandler::dispatchHandlerEvent( IEvent& anEvent)
{
   IWindow* thisWindow = anEvent.window();

   if (thisWindow)
   {
     switch (anEvent.eventId())
     {
       case WM_WINDOWPOSCHANGED:
       {
         PSWP pswpNew = (PSWP)(char*)anEvent.parameter1();
         if (pswpNew->fl & SWP_SIZE)
         {
            // Send Size changed notification
            ISize siz(pswpNew->cx, pswpNew->cy);
            thisWindow->notifyObservers(
                        INotificationEvent(IWindow::sizeId,
                                           *thisWindow, true, &siz));
         }
         if (pswpNew->fl & SWP_MOVE)
         {
            // Send Move notification
            IPoint pt(pswpNew->x, pswpNew->y);
            thisWindow->notifyObservers(
                   INotificationEvent(IWindow::positionId,
                                      *thisWindow, true, &pt));
         }
         if (pswpNew->fl & SWP_SHOW)
         {
            // Send Show notification
            thisWindow->notifyObservers(
                   INotificationEvent(IWindow::visibleId,
                                      *thisWindow, true, true));
         }
         else if (pswpNew->fl & SWP_HIDE)
         {
            // Send Hide notification
            thisWindow->notifyObservers(
                   INotificationEvent(IWindow::visibleId,
                                      *thisWindow, true, false));
         }
         break;
       } /* endcase */
       case WM_ENABLE:
       {
          bool fEnable = anEvent.parameter1().number1();
          thisWindow->notifyObservers(
                 INotificationEvent(IWindow::enableId,
                                    *thisWindow, true, (void*)fEnable));
          break;
       } /* endcase */

       case WM_SETFOCUS:
       {
          bool focus = anEvent.parameter2().number1();
          thisWindow->notifyObservers(
                   INotificationEvent(IWindow::focusId,
                                      *thisWindow, false, (void*)focus));
          break;
       } /* endcase */

       case WM_PRESPARAMCHANGED:
       {
         if (anEvent.parameter1() == PP_FOREGROUNDCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::foregroundColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_BACKGROUNDCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::backgroundColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_BORDERCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::borderColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_SHADOW)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::shadowColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_DISABLEDFOREGROUNDCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::disabledForegroundColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_DISABLEDBACKGROUNDCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::disabledBackgroundColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_ACTIVECOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::activeColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_INACTIVECOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::inactiveColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_HILITEFOREGROUNDCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::hiliteForegroundColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_HILITEBACKGROUNDCOLOR)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                   IWindow::hiliteBackgroundColorId,
                                   *thisWindow));
         }
         else if (anEvent.parameter1() == PP_FONTNAMESIZE)
         {
           thisWindow->notifyObservers(INotificationEvent(
                                       IWindow::fontId, *thisWindow));
         }
         break;
       }

       case WM_COMMAND:
       {
          thisWindow->notifyObservers(
                   INotificationEvent(IWindow::commandId,
                                      *thisWindow, false, (void*)&anEvent));
          break;
       } /* endcase */

       case WM_SYSCOMMAND:
       {
          thisWindow->notifyObservers(
                   INotificationEvent(IWindow::systemCommandId,
                                      *thisWindow, false, (void*)&anEvent));
          break;
       } /* endcase */

       default:
       break;
     }  /* end switch */
   }

   return false;  // We never "process" an event
}
#endif // IC_PM

#ifdef IC_WIN
bool IWindowNotifyHandler::dispatchHandlerEvent( IEvent& anEvent)
{
   IWindow* thisWindow = anEvent.window();

   if (thisWindow)
   {
     switch (anEvent.eventId())
     {
        case WM_WINDOWPOSCHANGED:
        {
          PWINDOWPOS pswpNew = (PWINDOWPOS)(char*)anEvent.parameter2();
          ISize siz(pswpNew->cx, pswpNew->cy);
          if (!(pswpNew->flags & SWP_NOSIZE) )
          {
             // Send Size changed notification
             thisWindow->notifyObservers(
                    INotificationEvent(
                         IWindow::sizeId,
                         *thisWindow,
                         true,
                         &siz));
          }
          if (!(pswpNew->flags & SWP_NOMOVE))
          {
             // Send Move notification
             IPoint pt(pswpNew->x, pswpNew->y);
             thisWindow->notifyObservers(
                    INotificationEvent(
                         IWindow::positionId,
                         *thisWindow,
                         true,
                         &pt));
          }
          break;
        } /* endcase */

        case WM_SHOWWINDOW:
        {
          /**************************************************************/
          /* Post private message to delay notification until after     */
          /* the selection state has been modified                      */
          /**************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_SHOW_WINDOW ));
          break;
        } /* endcase */

        case WM_ENABLE:
        {
           bool fEnable = anEvent.parameter1().number1();
           thisWindow->notifyObservers(
                  INotificationEvent(
                       IWindow::enableId,
                       *thisWindow,
                       true,
                       (void*)fEnable));

           break;
        } /* endcase */

        case WM_SETFOCUS:
        {
          thisWindow->notifyObservers(
                   INotificationEvent(IWindow::focusId,
                                      *thisWindow, false, (void*)true));
          break;
        } /* endcase */

        case WM_SETFONT:
        {
          /**************************************************************/
          /* Post private message to delay notification until after     */
          /* the selection state has been modified                      */
          /**************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_WINDOW_FONT ));
          break;
        }

        /****************************************************************/
        /* This is the delayed notification message.  Check to see which*/
        /* notification should be sent to the observers based on info   */
        /* stored in parameter 2 of the private message.  Parameter 1   */
        /* contains the information passed in the original event that   */
        /* needs to be sent with the notification                       */
        /****************************************************************/
        case IC_UM_DELAYNOTIFY:

          if (anEvent.parameter2() == IC_NOTDEL_WINDOW_FONT)
          {
            thisWindow->notifyObservers( INotificationEvent(
                                         IWindow::fontId,
                                         *thisWindow, true,
                                         (void*)anEvent.parameter1()));
          }
          else if (anEvent.parameter2() == IC_NOTDEL_SHOW_WINDOW)
          {
            bool fShow = anEvent.parameter1().number1();
            thisWindow->notifyObservers( INotificationEvent(
                                         IWindow::visibleId,
                                         *thisWindow, true,
                                         (void*)fShow));
          }
          else if (anEvent.parameter2() == IC_NOTDEL_WINDOW_COLOR)
          {
            if (anEvent.parameter1() == PP_FOREGROUNDCOLOR)
            {
              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::foregroundColorId,
                                          *thisWindow));
            }
            else if (anEvent.parameter1() == PP_BACKGROUNDCOLOR)
            {
              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::backgroundColorId,
                                          *thisWindow));
            }
          }
          break;

        case WM_KILLFOCUS:
        {
          thisWindow->notifyObservers(
                   INotificationEvent(IWindow::focusId,
                                      *thisWindow, false, (void*)false));
          break;
        } /* endcase */

        case WM_COMMAND:
        {
           thisWindow->notifyObservers(
                    INotificationEvent(IWindow::commandId,
                                       *thisWindow, false, (void*)&anEvent));
           break;
        } /* endcase */

        case WM_SYSCOMMAND:
        {
           thisWindow->notifyObservers(
                    INotificationEvent(IWindow::systemCommandId,
                                       *thisWindow, false, (void*)&anEvent));
           break;
        } /* endcase */

     }  /* end switch */
   }

   return false;  // We never "process" an event
}

#endif // IC_WIN
#endif // IC_PMWIN

#ifdef IC_MOTIF

bool IWindowNotifyHandler::dispatchHandlerEvent( IEvent& anEvent)
{
   IWindow* thisWindow = anEvent.window();

   if (thisWindow)
   {
     XEvent *pxevent =
       (XEvent *)(char*)anEvent.parameter2();
     switch (anEvent.eventId())
     {
       case xEvent( FocusIn ):
       case xEvent( FocusOut ):
       {
          // This code relies upon the fact that IWindow filters focus events such
          // that only focus events which indicate a change to this widget are
          // dispatched.  The ancestor/inferior notifications generated by the
          // X server are not dispatched.
          bool focus = (pxevent->xany.type == FocusIn) ? true : false;
          thisWindow->notifyObservers( INotificationEvent(
                IWindow::focusId, *thisWindow,
                false, (void*)focus));
          break;
       } /* endcase */


       case WM_COMMAND:
       {
          thisWindow->notifyObservers(INotificationEvent(
             IWindow::commandId, *thisWindow,
                 false, (void*)&anEvent) );
          break;
       } /* endcase */

       case WM_SYSCOMMAND:
       {
          thisWindow->notifyObservers(INotificationEvent(
             IWindow::systemCommandId, *thisWindow,
                false, (void*)&anEvent) );
          break;
       } /* endcase */

       // New Window VISIBILITY (visibleId)
       case xEvent(VisibilityNotify):
       {
         if (pxevent->xvisibility.state ==
           VisibilityFullyObscured)
         {
           thisWindow->notifyObservers(INotificationEvent(
             IWindow::visibleId, *thisWindow,
             true, false));
         }
         else if (pxevent->xvisibility.state ==
           VisibilityUnobscured)
         {
           thisWindow->notifyObservers(INotificationEvent(
             IWindow::visibleId, *thisWindow,
             true, true));
         }
         break;
       } /* endcase */
       case xEvent(ConfigureNotify):
       {
         ISizeRectangle *sizeRect =
           (ISizeRectangle *) anEvent.parameter1().asUnsignedLong();
         if ( sizeRect->sizeChanged() )
         {
           //case WM_SIZE:
// d6993 - size() is virtual.  Don't count on it being the same as the widget.
//            if (anEvent.parameter2() != NULL)
//            {
//                  // XEvent is second parameter
//              XEvent *pxevent =
//                (XEvent*)(anEvent.parameter2().asUnsignedLong());
//              ISize siz( (pxevent->xconfigure.width +
//                          2*pxevent->xconfigure.border_width),
//                         (pxevent->xconfigure.height +
//                          2*pxevent->xconfigure.border_width) );
//              thisWindow->notifyObservers(INotificationEvent(
//                IWindow::sizeId, *thisWindow,
//                true, &siz));
//            }   // endif
           ISize siz = thisWindow->size();
           thisWindow->notifyObservers(INotificationEvent(
             IWindow::sizeId, *thisWindow,
             true, &siz));
         }
         if ( sizeRect->rectangleMoved() )
         {
           // case WM_MOVE:
// d6993 - fix coordinate conversion.  Also, position() is virtual...
//            if (anEvent.parameter2() != NULL)
//            {
//                  // XEvent is second parameter
//              XEvent *pxevent =
//                (XEvent*)(anEvent.parameter2().asUnsignedLong());
//              IPoint pt(pxevent->xconfigure.x,
//                        (thisWindow->parent()->size().height() -
//                         pxevent->xconfigure.y -
//                         thisWindow->size().height()) );
//              thisWindow->notifyObservers(INotificationEvent(
//                IWindow::positionId, *thisWindow,
//                true, &pt));
//            }   // endif
           IPoint pt = thisWindow->position();
           thisWindow->notifyObservers(INotificationEvent(
             IWindow::positionId, *thisWindow,
             true, &pt));
         }
         break;
       }
       default:
         break;
     }  /* end switch */
   } /* end if */

   return false;  // We never "process" an event
}

#endif //IC_MOTIF
