// Revision: 10 1.3.1.2 source/ui/baseapp/iwindow8.cpp, basewin, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: iwindow8.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions used for the    *
*   IWindow bidirectional support.                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#if (IC_OBSOLETE <= IC_OBSOLETE_2)

#include <icconst.h>
#include <iwindow.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IWINDOW8_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
bool IWindow::isBidiSupported () const
{
  return IBidiSettings::isBidiSupported();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow& IWindow::applyBidiSettings ( const IWindow::BidiSettings& settings,
                                      bool childInherit, bool refresh )
{
  IBidiSettings* fBidiSettings = (IBidiSettings*) settings.fBidiSettings;
  IWindowHandle handle(this->handle());
  fBidiSettings->apply(handle,childInherit,refresh);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings::BidiSettings ( const IWindow& window )
                      :fBidiSettings(0)
{
  if ( IBidiSettings::isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }
  fBidiSettings = new IBidiSettings(window);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings::~BidiSettings ()
{
   delete fBidiSettings;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setTextType
                       ( IWindow::BidiTextType textType )
{
  switch ( textType )
  {
    case visual:
      fBidiSettings->setTextType(IBidiSettings::visual);
      break;
    case implicit:
      fBidiSettings->setTextType(IBidiSettings::implicit);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setTextOrientation
                       ( IWindow::BidiTextOrientation textOrientation )
{
  switch ( textOrientation )
  {
    case textLeftToRight:
      fBidiSettings->setTextOrientation(IBidiSettings::textLeftToRight);
      break;
    case textRightToLeft:
      fBidiSettings->setTextOrientation(IBidiSettings::textRightToLeft);
      break;
    case textContextual:
      fBidiSettings->setTextOrientation(IBidiSettings::textContextual);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setWindowLayout
                       ( IWindow::BidiLayout windowLayout )
{
  switch ( windowLayout )
  {
    case layoutLeftToRight:
      fBidiSettings->setWindowLayout(IBidiSettings::layoutLeftToRight);
      break;
    case layoutRightToLeft:
      fBidiSettings->setWindowLayout(IBidiSettings::layoutRightToLeft);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setNumeralDisplay
                       ( IWindow::BidiNumeralType numeralDisplay )
{
  switch ( numeralDisplay )
  {
    case arabic:
      fBidiSettings->setNumeralDisplay(IBidiSettings::arabic);
      break;
    case asStored:
      fBidiSettings->setNumeralDisplay(IBidiSettings::asStored);
      break;
    case national:
      fBidiSettings->setNumeralDisplay(IBidiSettings::national);
      break;
    case contextual:
      fBidiSettings->setNumeralDisplay(IBidiSettings::contextual);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::enableSymmetricSwapping
                       ( bool enable )
{
  fBidiSettings->enableSymmetricSwapping(enable);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::disableSymmetricSwapping ( )
{
   return enableSymmetricSwapping(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::enableWordByWordReordering
                       ( bool enable )
{
  fBidiSettings->enableWordByWordReordering(enable);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::disableWordByWordReordering ( )
{
   return enableWordByWordReordering(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setTextShape
                       ( IWindow::BidiTextShape textShape )
{
  switch ( textShape )
  {
    case displayShaped:
      fBidiSettings->setTextShape(IBidiSettings::displayShaped);
      break;
    case saveShaped:
      fBidiSettings->setTextShape(IBidiSettings::saveShaped);
      break;
    case nominalShape:
      fBidiSettings->setTextShape(IBidiSettings::nominalShape);
      break;
    case initialShape:
      fBidiSettings->setTextShape(IBidiSettings::initialShape);
      break;
    case middleShape:
      fBidiSettings->setTextShape(IBidiSettings::middleShape);
      break;
    case finalShape:
      fBidiSettings->setTextShape(IBidiSettings::finalShape);
      break;
    case isolatedShape:
      fBidiSettings->setTextShape(IBidiSettings::isolatedShape);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiTextType IWindow::BidiSettings::textType () const
{
  if ( fBidiSettings->textType() == IBidiSettings::implicit )
    return implicit;
  else
    return visual;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiTextOrientation IWindow::BidiSettings::textOrientation () const
{
  if ( fBidiSettings->textOrientation() == IBidiSettings::textContextual )
    return textContextual;
  if ( fBidiSettings->textOrientation() == IBidiSettings::textRightToLeft )
    return textRightToLeft;
  return textLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiLayout IWindow::BidiSettings::windowLayout () const
{
  if ( fBidiSettings->windowLayout() == IBidiSettings::layoutRightToLeft )
    return layoutRightToLeft;
  else
    return layoutLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiNumeralType IWindow::BidiSettings::numeralDisplay () const
{
  if ( fBidiSettings->numeralDisplay() == IBidiSettings::contextual )
    return contextual;
  if ( fBidiSettings->numeralDisplay() == IBidiSettings::asStored )
    return asStored;
  if ( fBidiSettings->numeralDisplay() == IBidiSettings::national )
    return national;
  return arabic;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
bool IWindow::BidiSettings::isSymmetricSwappingEnabled () const
{
  return fBidiSettings->isSymmetricSwappingEnabled();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
bool IWindow::BidiSettings::isWordByWordReorderingEnabled () const
{
  return fBidiSettings->isWordByWordReorderingEnabled();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiTextShape IWindow::BidiSettings::textShape () const
{
  if ( fBidiSettings->textShape() == IBidiSettings::isolatedShape )
    return isolatedShape;
  if ( fBidiSettings->textShape() == IBidiSettings::finalShape )
    return finalShape;
  if ( fBidiSettings->textShape() == IBidiSettings::middleShape )
    return middleShape;
  if ( fBidiSettings->textShape() == IBidiSettings::initialShape )
    return initialShape;
  if ( fBidiSettings->textShape() == IBidiSettings::nominalShape )
    return nominalShape;
  if ( fBidiSettings->textShape() == IBidiSettings::saveShaped )
    return saveShaped;
  return displayShaped;
}

#endif // IC_OBSOLETE
