// Revision: 13 1.30.1.12 source/ui/baseapp/iwinpriv.hpp, basewin, ioc.v400, 000919 
/* NOSHIP */
#ifndef _IWINPRIV_
#define _IWINPRIV_
/*******************************************************************************
* FILE NAME: iwinpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of iwindow.                                          *
*                                                                              *
*   It also contains a cross reference containing the location of functions    *
*   used to implement IWindow.                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iatname.hpp>
#include <icolor.hpp>
#include <ihandle.hpp>
#include <ikeyset2.h>
#include <ipoint.hpp>
#include <ireslock.hpp>
#include <iwindow.hpp>
#ifdef IC_PMWIN
  #include <iwinmphd.hpp>
#endif

#ifdef IC_MOTIF
extern "C" {
  #include <Xm/Xm.h>
}
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IAttribute;
class IHandlerList;
class INotifierImplementation;
class IWindowNotifyHandler;

#pragma enum(4)
#pragma pack(push,4)

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IXmMainWindow class                                                          |
------------------------------------------------------------------------------*/
class IXmMainWindow {
public:
static Widget
  getMainWindowChild ( Widget parent );
}; // IXmMainWindow

/*------------------------------------------------------------------------------
| IXmColor Class                                                               |
|                                                                              |
| This class provides some methods used to change colors of widgets.           |
|                                                                              |
------------------------------------------------------------------------------*/
class IXmColor {
public:
static void
  setColor           ( Widget        wid,
                       const bool    setBackground,
                       const Pixel   backgroundColor,
                       const Pixel   foregroundColor );
static void
  setBackgroundColor ( Widget        wid,
                       const Pixel   backgroundColor );
}; // IXmColor
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| ITheDesktopWindow class                                                      |
------------------------------------------------------------------------------*/
class ITheDesktopWindow : public IWindow {
typedef IWindow
  Inherited;

public:
  ITheDesktopWindow ( );
virtual
 ~ITheDesktopWindow ( );

virtual IWindowHandle
  handle   ( ) const;
#ifdef IC_MOTIF
virtual ISize
  size     ( ) const;
virtual IPoint
  position ( ) const;
#endif // IC_MOTIF

private:
  ITheDesktopWindow ( const ITheDesktopWindow& );
ITheDesktopWindow
 &operator=         ( const ITheDesktopWindow& );

IWindowHandle
  hwnd;
}; // ITheDesktopWindow

/*------------------------------------------------------------------------------
| ITheObjectWindow class                                                       |
------------------------------------------------------------------------------*/
class ITheObjectWindow : public IWindow {
typedef IWindow
  Inherited;

public:
  ITheObjectWindow ( );
virtual
 ~ITheObjectWindow ( );
virtual IWindowHandle
  handle ( ) const;

private:
  ITheObjectWindow ( const ITheObjectWindow& );
ITheObjectWindow
 &operator=        ( const ITheObjectWindow& );

IWindowHandle
  hwnd;
}; // ITheObjectWindow

/*------------------------------------------------------------------------------
| IObserverData class                                                          |
| The IObserverData member will contain an INotifierImplementation following   |
| the first INotifier::add(IObserver) request and an IWindowNotifyHandler      |
| following the first INotifier::enableNotification.                           |
------------------------------------------------------------------------------*/
class IObserverData  {
public:
  IObserverData ( );
 ~IObserverData ( );

static IPrivateResource
 &dictionaryKey( );

INotifierImplementation
 *fNotifierImp;

IWindowNotifyHandler
 *notifyHandler;

bool
  enabled;
}; // IObserverData

/*------------------------------------------------------------------------------
| IChildCursorData class                                                       |
------------------------------------------------------------------------------*/
class IChildCursorData {
public:
  IChildCursorData ( IWindowHandle parentHandle );
virtual
 ~IChildCursorData ( );

bool
  IChildCursorData::setToFirst ( IWindowHandle &hwnd );
bool
  IChildCursorData::setToNext  ( IWindowHandle &hwnd );

void
  IChildCursorData::invalidate ( IWindowHandle &hwnd );

IWindowHandle
  hwndParent;

#ifdef IC_MOTIF
_WidgetRec
 **children;
unsigned int
  num_children;
int
  index;
#endif

#ifdef IC_PM
IEnumHandle
  enumHandle;
#endif
}; // IChildCursorData

class IWindowAttributeName : public IAttributeName {
typedef IAttributeName
  Inherited;
public:

  IWindowAttributeName ( const IAttributeName&       name );

  IWindowAttributeName ( );
  IWindowAttributeName ( const IWindowAttributeName& name );
~IWindowAttributeName();

// the collections use the < operator for the element comparisons.
bool
  operator < (IWindowAttributeName const& k) const;
}; // IWindowAttributeName

/* Note that this class does NOT copy the attribute. It is up to the caller to
   manage the lifetime. This is to save time.
*/
class INamedAttribute {
public:
/*------------------------------- Constructors -------------------------------*/
  INamedAttribute ( );
  INamedAttribute ( const IAttribute*           attribute,
                    const IWindowAttributeName& name );
 ~INamedAttribute ( );

  INamedAttribute ( const INamedAttribute&      key );
INamedAttribute
 &operator=       ( const INamedAttribute&      key );

/*------------------------------- Comparisons --------------------------------*/
bool
  operator<  ( const INamedAttribute& key ) const,
  operator== ( const INamedAttribute& key ) const,
  operator!= ( const INamedAttribute& key ) const;

/*------------------------------- Accessors ----------------------------------*/
const IAttribute
 *attribute     ( ) const;
const IAttributeName
  attributeName ( ) const;

protected:
/*-------------------------------- Identifier --------------------------------*/
static IWindowAttributeName
  uniqueKeyFor ( const INamedAttribute& key );

private:
/*--------------------------------- Private ----------------------------------*/
friend IWindowAttributeName const
 &key ( INamedAttribute const& item );

IAttribute
 *fAttribute;
IWindowAttributeName
  fAttributeName;
}; // INamedAttribute

typedef IKeySet< INamedAttribute, IWindowAttributeName>
  IWindowAttributeKeySet;

/*------------------------------------------------------------------------------
| IWindowPrivateData class                                                     |
------------------------------------------------------------------------------*/
class IWindowPrivateData {
public:
  IWindowPrivateData ( IWinProc*     windowProcedure,
                       unsigned long defState );
 ~IWindowPrivateData ( );
IObserverData
 &observerData ( );

IWinProc
 *defaultProc;
unsigned long
  state,
  layoutChange,
  helpId,
  ulExtGUIStyle;
IHandlerList
 *handlerList;
ISize
  minimumSize,
  fCachedMinimumSize,
  sizClChar;       // Character size in the current font.
IObserverData
 *_observerData;
bool
  fHandlerRemovePending;
bool
  fSetMinSizeInProgress;
IAccelTblHandle
  fAccelHandle;
IWindowAttributeKeySet
  fAttributeKeySet;
IRegionHandle
  fInvalidatedRegion;
bool
  fUpdateEnabled;

#ifdef IC_MOTIFWIN
IWindowHandle      // Handle of owner window for WS_CHILD styles.
  fhwndOwner;

long
  oldHeight,       // Saved by WM_WINDOWPOSCHANGED for last size
  oldWidth;        // to make resize handler work.
                   //saved between ConfigureRequest and ConfigureNotify X events,
                   //... so we can determine if a resize, a move, or both, occured.
                   // Also used to spot height changes, to recoordinate only when needed.

/*--- support for color ---------------------------*/
IColor
  clrforeground,
  clrbackground,
  fPrevForegroundColor,
  fPrevBackgroundColor;
#endif // IC_MOTIFWIN

#ifdef IC_WIN
long
  lastButtonTime;
IWindowHandle
  lastButtonEventHandle;
unsigned long
  lastButtonEventId;
IPoint
  lastButtonPos;
bool
  eat_BN_CLICKED;

/*--- support for detecting the start of a drag ---*/
IPoint
  pointLButtonDown;
bool
  bCheckingDragStart;

IFont              // Need to save any IFonts set to prevent font handle
 *fFont;           // from being destroyed
#endif // IC_WIN

#ifdef IC_MOTIF
unsigned long
  ulStyle;
bool
  windowGrabbed;
// The following supports recoordination for Motif.
ISize
  windowSize;
IPoint
  windowPosition,
  nativeWindowPosition;

int
  oldX,
  oldY;

unsigned long
  motifState;        // member and enum are needed for recoordination...
                     // dispite name.  Better name is guiState.

enum MotifState { frameShow = 1, frameResize = 2, visible = 4,
                  inCreate = 8, checkedValidity = 16, handleValid = 32,
                  hasFocus = 64, disabled = 128, usedIWindowCreate = 256,
                  neverBeenShown = 512, realized = 1024 };

// MDM - Moved these to IWindow as private functions in Windows
static IPoint
  mapXdt2PM( const Position& xPos,
             const Position& yPos,
             const IWindowHandle& fromHandle );

static Widget
  realSizeWidget       ( const IWindow* win );

static ISize
  realSize             ( const IWindow *win );

static IPoint
  realPosition         ( const IWindow *win );

static IRectangle
  realRect             ( const IWindow *win );

static ISize
  parentSize           ( const IWindow* win,
                         bool   mustBeReal = false );

IPresSpaceHandle
  gc;              // X graphics context, rough (real) equivalent to presentation space handle
int
  gcUseCount;      // simple use count (don't need IRefCounted since API defines
                   // "get" and "free" methods, and "gc" is not a C++ object)
static void
  processPendingMsgs ( );

// The following two methods register and unregister callbacks for widgets
// wrappered by IWindow. The registered callbacks are common callbacks needed
// for geometry management.
void
  registerCallbacks   ( IWindow *win ),
  unregisterCallbacks ( IWindow *win );

void
  positionOnSiblings  ( IWindowHandle whnd );

#endif // IC_MOTIF

static bool
  bProcessMsgsNoWait;

static IWindow::ExceptionFn
 *exceptionFunction;

static IWindow::SiblingOrder
  siblingCreateOrder;

static bool
  fgFastWindowProcEnabled;

static void
  perfEventTrace ( unsigned long ulMsg );   // Writes event name to iperf log

#ifdef IC_WIN
static unsigned long
  matchMnemonicHwnd ( unsigned short character,
                      IWindowHandle  hwnd );
#endif

unsigned long
  fSomeMouseMovesCount,
  fAllMouseMovesCount,
  fMouseEntersLeavesCount;

IPointerHandle
  fMousePtr;

#ifdef IC_PMWIN
IWindowMousePointerHandler
 *fMousePointerHandler;
#endif

enum EWindowDirection {
  kLeftToRight,
  kRightToLeft,
  kDefaultDirection
  };
EWindowDirection
  fWindowDirection;
}; // IWindowPrivateData

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _IWINPRIV_ */
