// Revision: 19 1.3.1.6 source/ui/baseapp/ixmlabel.cpp, basewin, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ixmlabel.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ixmlabel.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482132 )
extern "C"
{
  #include <iwindefs.h>
}

#include <ixmlabel.hpp>
#include <istring.hpp>
#include <imstring.hpp>
#include <istrenum.hpp> // 27965 A

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IXMLABEL_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IXmLabel::initializeXmLabelString                                            |
------------------------------------------------------------------------------*/
bool IXmLabel::initializeXmLabelString( IWindowHandle initializeThis,
                                            const IString& setBlankIfThis,
                                            bool bRemoveMnemonic )
{
 IMString labelString( initializeThis, XmNlabelString );
 bool bReplaced
 = initializeXmLabelString( labelString, setBlankIfThis, bRemoveMnemonic );
 if ( bReplaced )
   XtVaSetValues (initializeThis,
                  XmNlabelString, (XmString)labelString,
                  NULL);
 return bReplaced;
}

/*------------------------------------------------------------------------------
| IXmLabel::initializeXmLabelString                                            |
------------------------------------------------------------------------------*/
bool IXmLabel::initializeXmLabelString( IMString& initializeThis,
                                            const IString& setBlankIfThis,
                                            bool bRemoveMnemonic )
{
 //
 // If the label string is equal to the specified string (normally the widget
 // name), set it to a blank.  Use this method after creating a widget
 // which is a (subclass of) XmLabel, so that if the XmNlabelString resource
 // has an initial value in a resource file, it will be kept.
 //
 // The key to understanding this is that if there is no value for the
 // XmNlabelString resource, it will default to the widget name.
 //
 // Also, if bRemoveMnmonics, then remove mnemonic sub string if there is one.
 //

 IString label = initializeThis.asString();
 bool bReplace = false;
 if ( bReplace = (label == setBlankIfThis) )
   label = IString();
 else
   bReplace = bRemoveMnemonic && removeMnemonic( label );
 if ( bReplace )
   initializeThis = IMString( label );
 return bReplace;
}

/*------------------------------------------------------------------------------
| IXmLabel::removeMnemonic                                                     |
------------------------------------------------------------------------------*/
bool IXmLabel::removeMnemonic( IString& label )
{
 //
 // Remove the first tilde.  If it is immediately preceeded by open paren, and
 // followed eventually by a close paren, then remove from the open paren
 // to the close paren inclusive.
 //
 // Refer to feature f5101.
 //
 // NLS considerations:
 //
 // IString::indexOf handles MBCS case, so a tilde as the second char of
 // an MBCS character will not be located by indexOf.
 // The IString::operator[] will give the previous byte, but it is not a
 // problem in this case, since the open paren encoding x28, which is less
 // than x3F, so open paren will never occur as the second char of an MBCS char.
 //
 // Tilde, open paren and close paren are all part of the
 // X Portable Character Set, so their encoding must be the same for all
 // locales running on a given host machine.  (O'Reilly V1 10.3.5. pg 344).
 //
 int start = label.indexOf( '~' );
 if ( start )
  {
   int end = start;
   if ( start > 1 && label[start-1] == '(' ) // Found a preceeding open paren.
    {
     end = label.indexOf( ')', start );
     if ( end ) // Found the close paren.
       start--; // Remove starting with the open paren.
     else // There is no close paren.
       end = start; // Just remove the tilde.
    }
   label.remove( start, end-start+1 );
  }
 return start != 0;
}

/*------------------------------------------------------------------------------
| IXmLabel::applyWordBreak                                                     |
------------------------------------------------------------------------------*/
bool IXmLabel::applyWordBreak( long                 maxWidth,
                               const IWindowHandle& handle,
                               IString&             text,
                               IString*             originalText)
{
  IFUNCTRACE_DEVELOP();
  bool modified(false);
  if( (maxWidth >= 0) && (text.length() > 0) ) // 27770 C
  {  // Skip if there is no text or no room for any text.  27770 C
     unsigned textPos(1);
     do
     {
        unsigned newlinePos = text.indexOf('\n', textPos);
        if (newlinePos == 0)
           newlinePos = text.length();
        else
           newlinePos++;
        // A paragraph here is like a word processor paragraph...delimited
        // by a newline or end of text.
        IString paragraph( text.subString(textPos, newlinePos - textPos) );
        ITRACE_ALL(IString("textPos=") + IString(textPos) +
                   IString(" paragraph=>") + paragraph + IString("<"));
        while (paragraph.length() > 0)
        {
           // Starting with the whole string, see if the text fits.  If not,
           // reduce its length by a word until it either fits or only 1
           // word is left.
           unsigned words     (paragraph.numWords());
           unsigned wordsLeft (words);
           unsigned paraPos   (paragraph.length());
           bool     wordsFit  (true);   // 27965 A
           ITRACE_ALL(IString("paraPos=") + IString(paraPos) +
                      IString(" words=") + IString(words) +
                      IString(" paragraph=>") + paragraph + IString("<"));
           while ((wordsLeft > 1) &&
                  (IMString( paragraph.subString(1, paraPos) ).size( handle ).width() > maxWidth))
           {
              paraPos = paragraph.indexOfWord(wordsLeft) - 1;
              wordsLeft--;
           }
          // if ((wordsLeft < 1) || ((wordsLeft == words) && (!text.includesDBCS())) ) / 27770 C
           if ( (wordsLeft == 1) &&
                (IMString( paragraph.subString(1, paraPos) ).size( handle ).width() > maxWidth)  )                    // 27965 A
           {
               wordsFit = false;    // 27965 A  
           }
           if ((wordsLeft < 1) || ((wordsLeft == words) && (wordsFit)) ) // 27965 C
           {  // Empty, single word, or whole thing fits. Use paragraph intact.
              textPos += paragraph.length();
              paragraph = IString();
           }
           else
           {  // Need to wrap lines.
              if ((!modified) && (originalText))
              {  // Save off original text before changing it if requested
                 // by caller.
                  *originalText = text;
              }
              modified = true;
              if( wordsFit )             // 27965 C
              {                          // 27770 A
                 // Replace last whitespace character before start of current         
                 // word with a newline.
                 text[textPos + paraPos - 1] = '\n';
                 paragraph.remove(1, paraPos);
                 textPos += paraPos;
              }                          // 27770 A
              else                       // 27770 A
              {                          // 27770 A
                 unsigned breakPos( paragraph.length() );   // 27770 A
                 while( IMString( paragraph.subString(1, breakPos) ).size( handle ).width() > maxWidth)                  // 27770 A
                 {                       // 27770 A
                    breakPos--;          // 27770 A
                 }
                 if (paragraph.charType(breakPos) == IStringEnum::dbcs2)  // 27965 A
                 {
                    breakPos--;          // 27965 A
                 }
                 IMString temp(text.subString(textPos + breakPos - 1, text.length() - textPos - breakPos + 2) );
                 text[textPos + breakPos - 1] = '\n';  // 27770 A
                 text = text.subString(1, textPos + breakPos - 1);
                 text += temp;
                 textPos += breakPos + 1;    // 27770 A
                 paragraph.remove( 1, breakPos);
              }                          // 27770 A 
           }
           ITRACE_ALL( IString("text=>") + text + IString("<"));
        }
     } while ( textPos < text.length() );
  }

  if (!modified && originalText)    //Clean up old values.
    *originalText = IString();

  return modified;
}
