// Revision: 31 1.9.1.8 source/ui/basectl/ibutton.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ibutton.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibutton.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C"
{
  #define INCL_WINBUTTONS
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #ifdef IC_MOTIF
     #include <Xm/PushB.h>
  #endif
}
#include <ibase.hpp>
#include <ibutton.hpp>
#include <icconst.h>
#include <icmnfun.hpp>
#include <icolor.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <ihandler.hpp>
#include <inotifev.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <itrace.hpp>

#ifdef IC_MOTIF
  #include <ixmlabel.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IBUTTON_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Public button styles                                                         |
------------------------------------------------------------------------------*/
const IButton::Style
#ifdef IC_MOTIFPM
  IButton::noPointerFocus = BS_NOPOINTERFOCUS;
#endif
#ifdef IC_WIN
  IButton::noPointerFocus (0, IBS_NOPOINTERFOCUS);
#endif

#ifdef IC_MOTIFWIN
#pragma enum(4)
#pragma pack(push,4)

class IButtonHandler : public IHandler {
typedef IHandler
  Inherited;
/*------------------------------------------------------------------------------
| This handler implements the BS_NOPOINTERFOCUS style for buttons.             |
------------------------------------------------------------------------------*/
public:

/*------------------------------- Constructors -------------------------------*/
  IButtonHandler ( );
virtual
 ~IButtonHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual bool
  dispatchHandlerEvent ( IEvent& event );
}; // IButtonHandler

class IButtonData {
/*------------------------------------------------------------------------------
| The IButtonData class encapsulates private data and functions                |
| used by the IButton class.  An object of this class is created in            |
| the IButton constructors.                                                    |
------------------------------------------------------------------------------*/
public:
  IButtonData ( );
 ~IButtonData ( );

bool
  fCheckingFocus;
#ifdef IC_WIN
IWindowHandle::Value
  fOldFocusHandle;
#endif //IC_WIN
#ifdef IC_MOTIF
bool
  fCallbackAdded;
#endif //IC_MOTIF
IButtonHandler
  fdefaultHandler;
}; // IButtonData

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IButtonData::IButtonData                                                     |
------------------------------------------------------------------------------*/
IButtonData::IButtonData ( )
  : fCheckingFocus( false ),
#ifdef IC_WIN
    fOldFocusHandle( 0 ),
#endif //IC_WIN
#ifdef IC_MOTIF
    fCallbackAdded( false ),
#endif //IC_MOTIF
    fdefaultHandler( )
{ }

/*------------------------------------------------------------------------------
| IButtonData::~IButtonData                                                    |
------------------------------------------------------------------------------*/
IButtonData::~IButtonData ( )
{ }


/*------------------------------------------------------------------------------
| IButtonHandler::IButtonHandler                                               |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
IButtonHandler::IButtonHandler()
{ }

/*------------------------------------------------------------------------------
| IButtonHandler::~IButtonHandler                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IButtonHandler::~IButtonHandler()
{ }

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IButton_disarmCallback                                                       |
------------------------------------------------------------------------------*/
static void IButton_disarmCallback( Widget     handle,
                                    XtPointer  client_data,
                                    XtPointer  call_data)
{
    // This callback is installed when noPointerFocus is active and a button
    // down has been detected in the button.  It restores the correct value
    // of XmNtraversalOn so that keyboard navigation works correctly.
    IFUNCTRACE_DEVELOP();
    IButtonData* buttonData = (IButtonData*)client_data;
    if ( (buttonData) && buttonData->fCheckingFocus)
    {
       //Restore XmNtraversalOn to true.  Note that we would not get here
       //if it was false.
       buttonData->fCheckingFocus = false;
       XtVaSetValues( handle, XmNtraversalOn, True, 0);
    }
    return;
}
#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IButtonHandler::dispatchHandlerEvent                                         |
------------------------------------------------------------------------------*/
bool IButtonHandler::dispatchHandlerEvent( IEvent& event )
{
  bool
    result(false);
  IButton*
    button(0);
  switch( event.eventId() )
     {
#ifdef IC_WIN
     case WM_LBUTTONDOWN:
        button = dynamic_cast<IButton*>(event.window());
        if ( (button) && !button->allowsMouseClickFocus()  )
           {
           button->fButtonData->fCheckingFocus = true;
           button->fButtonData->fOldFocusHandle = 0;
           }
        break;
     case WM_LBUTTONUP:
        button = dynamic_cast<IButton*>(event.window());
        if ( (button) && (button->fButtonData->fCheckingFocus) )
           {
           button->postEvent( IC_UM_CANVAS_SETFOCUS );
           }
        break;
     case WM_SETFOCUS:
        button = dynamic_cast<IButton*>(event.window());
        if ( (button) && (button->fButtonData->fCheckingFocus) )
           {
           button->fButtonData->fOldFocusHandle = (HWND)event.parameter1();
           }
        break;
     case WM_KILLFOCUS:
        button = dynamic_cast<IButton*>(event.window());
        if (button)
           {
           // Unhilite any depressed buttons to prevent the generation
           // of a command event because of a Windows flaw.
           if (button->isHighlighted())
             button->unhighlight();
           button->fButtonData->fCheckingFocus = false;
           }
        break;
     case IC_UM_CANVAS_SETFOCUS:
        button = dynamic_cast<IButton*>(event.window());
        if ( (button) && (button->fButtonData->fCheckingFocus) )
           {
           // if we get here we need to give focus back to fOldFocusHandle
           if ( button->fButtonData->fOldFocusHandle )
              {
              // have a handle to give focus back to
              SetFocus( button->fButtonData->fOldFocusHandle );
              }
           else
              {
              // give focus to top level window for this thread
              SetFocus( GetActiveWindow() );
              }

           button->fButtonData->fCheckingFocus = false;
           }
        result = true;
        break;

    // Code to trap a mnemonic key press and send it directly to
    // the owner/parent window (allowing it to naturally progress
    // through the dispatcher results in an audible beep caused
    // by the default window procedure, because it cannot find a
    // match for a mnemonic that we process in ICanvas or
    // IFrameWindow), eating original event.
     case WM_SYSCHAR:
        button = dynamic_cast<IButton*>(event.window());
        if (button)
        {
           IWindow* owner = button->owner();
           if ( ! owner )
           {
              owner = button->parent();
           }
           if ( owner )
           {
              owner->sendEvent( event );
              result = true;
           }
        }
        break;
#endif //IC_WIN

#ifdef IC_MOTIF
     // Use the synthetic event to get control before the widget sees the
     // button down.  XmCR_ARM is too late, because the processing we want to
     // alter is in the widget's Arm routine.
     case WM_BUTTON1DBLCLK:
        ITRACE_DEVELOP("IButtonData WM_BUTTON1DBLCLK");
        // Intentional fall thru to next case.

     case WM_BUTTON1DOWN:
        ITRACE_DEVELOP("IButtonData WM_BUTTON1DOWN");
        button = dynamic_cast<IButton*>(event.window());
        if ( (button) && !button->allowsMouseClickFocus()  )
        {
           // If the button is traversable, temporarily turn it off so that
           // the widget will not take focus.
           if (XmIsTraversable( event.handle() ) )
           {
               ITRACE_DEVELOP("IButtonData fCheckingFocus=true");
               if (!button->fButtonData->fCallbackAdded)
               {
                  // Add our private callback for disarm.  We cannot rely
                  // upon an IWindow disarm callback being installed
                  // by derived classes, and cannot install it ourselves
                  // because that may cause there to be more than one.
                  XtAddCallback( event.handle(),
                                 XmNdisarmCallback,
                                 IButton_disarmCallback,
                                 button->fButtonData);
                  button->fButtonData->fCallbackAdded = true;
               }
               button->fButtonData->fCheckingFocus = true;
               XtVaSetValues( event.handle(), XmNtraversalOn, False, 0);
           }
        }
        break;
#endif //IC_MOTIF

     default:
       break;
     }
  return result;
}

#endif //IC_MOTIFWIN


/*------------------------------------------------------------------------------
| IButton::IButton                                                             |
------------------------------------------------------------------------------*/
IButton::IButton ( )
    : fButtonData(0)
{
#ifdef IC_MOTIFWIN
  fButtonData = new IButtonData( );
  fButtonData->fdefaultHandler.handleEventsFor( this );
#endif
}

/*------------------------------------------------------------------------------
| IButton::~IButton                                                            |
------------------------------------------------------------------------------*/
IButton::~IButton ( )
{
#ifdef IC_MOTIF
   if (this->isValid() && this->fButtonData->fCallbackAdded)
      XtRemoveCallback( this->handle(),
                        XmNdisarmCallback,
                        IButton_disarmCallback,
                        this->fButtonData);
#endif
#ifdef IC_MOTIFWIN
   fButtonData->fdefaultHandler.stopHandlingEventsFor( this );
   delete fButtonData;
#endif
}

/*------------------------------------------------------------------------------
| IButton::enableMouseClickFocus                                               |
|                                                                              |
| Enables the button to receive the focus when the user clicks on the button   |
| using the mouse.                                                             |
------------------------------------------------------------------------------*/
IButton& IButton::enableMouseClickFocus ( bool bTurnOn )
{
#ifdef IC_MOTIFPM   //Motif is using the PM value for the style flag
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= ~noPointerFocus.asUnsignedLong();
  }
  else
  {
    ulStyle |= noPointerFocus.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
#endif
#ifdef IC_WIN
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= ~noPointerFocus.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle |= noPointerFocus.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle(ulStyle);
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IButton::disableMouseClickFocus                                              |
------------------------------------------------------------------------------*/
IButton& IButton::disableMouseClickFocus ( )
{
  enableMouseClickFocus(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IButton::allowsMouseClickFocus                                               |
------------------------------------------------------------------------------*/
bool IButton::allowsMouseClickFocus ( ) const
{
#ifdef IC_WIN
  if (extendedStyle() & noPointerFocus.asExtendedUnsignedLong())
    return false;
  else
    return true;
#endif
#ifdef IC_PM
  if (style() & noPointerFocus.asUnsignedLong())
    return false;
  else
    return true;
#endif
#ifdef IC_MOTIF
  // Will not accept pointer focus if traversal is off or if the IOC
  // style flag is set.
  ::Boolean traversalOn(true);
  if ( this->isValid() && XmIsPrimitive( this->handle() ) )
     XtVaGetValues( this->handle(),
                    XmNtraversalOn, &traversalOn,
                    0 );

  if (!traversalOn || (style() & noPointerFocus.asUnsignedLong()))
    return false;
  else
    return true;
#endif
}

/*------------------------------------------------------------------------------
| IButton::highlight                                                           |
|                                                                              |
| Highlight the button.  A highlighted button appears as if the mouse          |
| selection button were pressed with the mouse pointer over the button.        |
------------------------------------------------------------------------------*/
IButton& IButton::highlight ( bool highlight )
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  handle().sendEvent(BM_SETHILITE, IEventParameter1(highlight),
                                   IEventParameter2(0));
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IButton::unhighlight                                                         |
------------------------------------------------------------------------------*/
IButton& IButton::unhighlight ( )
{
  ITRACE_MOTIF_NOP();
  return highlight(false);
}

/*------------------------------------------------------------------------------
| IButton::isHighlighted                                                       |
------------------------------------------------------------------------------*/
bool IButton::isHighlighted ( ) const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  IEventResult evt = handle().sendEvent(BM_QUERYHILITE,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
#ifdef IC_WIN
  if (evt.asUnsignedLong() & 0x00000004)
    return true;
 else
    return false;
#endif
#ifdef IC_PM
  if (evt.asUnsignedLong())
     return true;
  else
     return false;
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  // Always return false on Motif.
  return false;
#endif
}

/*------------------------------------------------------------------------------
| IButton::click                                                               |
------------------------------------------------------------------------------*/
IButton& IButton::click ( )
{
  clickButton(handle());
  return *this;
}

/*------------------------------------------------------------------------------
| IButton::foregroundColor                                                     |
|                                                                              |
| Returns the foreground color of the button.                                  |
------------------------------------------------------------------------------*/
IColor IButton::foregroundColor () const
{
#ifdef IC_PMWIN
  return this->color( PP_FOREGROUNDCOLOR,
                      IColor( IColor::kMenuText ) );
#endif
#ifdef IC_MOTIF
  return Inherited::foregroundColor();
#endif
}

/*------------------------------------------------------------------------------
| IButton::backgroundColor                                                     |
|                                                                              |
| Returns the background color of the button.                                  |
------------------------------------------------------------------------------*/
IColor IButton::backgroundColor () const
{
#ifdef IC_PMWIN
  // Return the correct background for a pushbutton.
#ifdef IC_PM
  if (style() & BS_PRIMARYSTYLES)
    {
      return this->color( PP_BACKGROUNDCOLOR,
                          IColor( IColor::kWindowBgnd ) );
    }
#endif
#ifdef IC_WIN
  if (!(((style() & BS_PRIMARYSTYLES) == BS_PUSHBUTTON) ||
        ((style() & BS_PRIMARYSTYLES) == BS_DEFPUSHBUTTON) ||
        ((style() & BS_PRIMARYSTYLES) == BS_OWNERDRAW)))
    {
      return this->color( PP_BACKGROUNDCOLOR,
                          IColor( IColor::kWindowBgnd ) );
    }
#endif
  return this->color( PP_BACKGROUNDCOLOR,
                      IColor( IColor::kButtonMiddle ) );
#endif // IC_PMWIN
#ifdef IC_MOTIF
  return Inherited::backgroundColor();
#endif
}

/*------------------------------------------------------------------------------
| IButton::disabledForegroundColor                                             |
|                                                                              |
| Returns the disabled foreground color of the button.                         |
------------------------------------------------------------------------------*/
IColor IButton::disabledForegroundColor () const
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PM
  return this->color( PP_DISABLEDFOREGROUNDCOLOR,
                      IColor( IColor::kMenuText ) );
#endif
#ifdef IC_MOTIFWIN
  return IColor( IColor::kMenuText );
#endif
}

/*------------------------------------------------------------------------------
| IButton::hiliteForegroundColor                                               |
|                                                                              |
| Returns the hilite foreground color of the button.                           |
------------------------------------------------------------------------------*/
IColor IButton::hiliteForegroundColor () const
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PM
  return this->color( PP_HILITEFOREGROUNDCOLOR,
                      IColor( IColor::kMenuText ) );
#endif
#ifdef IC_MOTIFWIN
  return IColor( IColor::kMenuText );
#endif
}

/*------------------------------------------------------------------------------
| IButton::hiliteBackgroundColor                                               |
|                                                                              |
| Returns the hilite background color of the button.                           |
------------------------------------------------------------------------------*/
IColor IButton::hiliteBackgroundColor () const
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PM
  return this->color( PP_HILITEBACKGROUNDCOLOR,
                      IColor( IColor::kMenuText ) );
#endif
#ifdef IC_MOTIFWIN
  return this->backgroundColor();
#endif
}

/*------------------------------------------------------------------------------
| IButton::setText                                                             |
|                                                                              |
| Sets the text used by the button, and notifies the parent canvas.            |
------------------------------------------------------------------------------*/
IButton& IButton::setText ( const char* pszText )
{
  IString labelString( pszText );
#ifdef IC_MOTIF
  IXmLabel::removeMnemonic( labelString );
#endif
  Inherited::setText(labelString);
  setLayoutDistorted(IWindow::minimumSizeChanged, 0);
  return *this;
}

/*------------------------------------------------------------------------------
| IButton::setText                                                             |
|                                                                              |
| Sets the button text from a string table.                                    |
------------------------------------------------------------------------------*/
IButton& IButton::setText ( const IResourceId& residText )
{
  Inherited::setText(residText);
  return *this;
}
