// Revision: 33 1.7.1.3 source/ui/basectl/ibutton1.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ibutton1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibtnnhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
#ifdef IC_PM
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
#else
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
#endif
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WININPUT
#define INCL_WINBUTTONS

extern "C" {
  #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <ibutton.hpp>
#include <ibtnnhdr.hpp>
#include <ievent.hpp>
#ifndef IC_PM
  #include <ictlevt.hpp>
  #include <iwcname.hpp>
#endif //!IC_PM
#include <inotifev.hpp>
#include <icolor.hpp>
#include <irect.hpp>
#include <icconst.h>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IBUTTON1_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PM
#include <ikeyevt.hpp>
#endif

// Initialize static obects and notification strings.
INotificationId const IButton::buttonClickId = "IButton::buttonClick";


/*------------------------------------------------------------------------------
| IButton::enableNotification                                                  |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IButton, does not send notifications to observer objects until it is         |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IButton& IButton::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new IButtonNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IButtonNotifyHandler::IButtonNotifyHandler                                   |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IButtonNotifyHandler::IButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IButtonNotifyHandler::~IButtonNotifyHandler                                  |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IButtonNotifyHandler::~IButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IButtonNotifyHandler::dispatchHandlerEvent                                   |
|                                                                              |
| This notifies IButton Windows of the following events:                       |
|                                                                              |
| buttonClicked(Id)                                                            |
------------------------------------------------------------------------------*/
bool IButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
#ifdef IC_PM
  IWindow* thisWindow = anEvent.window();
  static bool button1down = false;
  static bool spaceBar = false;

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_CHAR:
      {
        if (thisWindow->isEnabled())
        {
           IKeyboardEvent keyevt(anEvent);
           if ((keyevt.isVirtual())  && (keyevt.virtualKey() == IKeyboardEvent::space) && !keyevt.isUpTransition())
           {
              spaceBar = true;
              thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                     IEventParameter1( 0 ),
                                     IEventParameter2( IC_NOTDEL_PUSHBT_SELECT ));
           }
        }
        break;
      }
      case WM_BUTTON1DBLCLK:
      case WM_BUTTON1DOWN:
      {
        if (thisWindow->isEnabled())
        {
          button1down = true;
        }
      break;
      }

      case WM_BUTTON1UP:
      {
        if (button1down)
        {
          /**************************************************************/
          /* Post private message to delay notification until after     */
          /* the selection state has been modified                      */
          /**************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_PUSHBT_SELECT ));
          button1down = false;
        }
      break;
      } /* end case */


      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_PUSHBT_SELECT)
        {
           if (spaceBar)
           {
              spaceBar = false;
              thisWindow->notifyObservers(INotificationEvent(
                                          IButton::buttonClickId,
                                          *thisWindow, false, 0));
           }
           else
           {
              IRectangle buttonRect = thisWindow->rect();
              IEventData mX = anEvent.parameter1().lowNumber();
              IEventData mY = anEvent.parameter1().highNumber();
              unsigned long mouseX = mX.asUnsignedLong();
              unsigned long mouseY = mY.asUnsignedLong();
              mouseX += buttonRect.left();
              mouseY += buttonRect.bottom();

              if (buttonRect.contains(IPoint(mouseX, mouseY)))
              {
                thisWindow->notifyObservers(INotificationEvent(
                                            IButton::buttonClickId,
                                            *thisWindow, false, 0));
              }
           }
        }
      break;

      default:
        break;
    }  /* end switch */
  }
#endif // IC_PM
#ifdef IC_WIN
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      /***********************************************************************/
      /* In Windows, all button click events (mouse, space bar, and mnemonic)*/
      /* can be obtained through WM_CONTROL since we are mapping all         */
      /* WM_COMMAND to WM_CONTROL.  Not so in PM.                            */
      /***********************************************************************/
      case WM_CONTROL:
      {
        IControlEvent ctlevt( anEvent );
        IWindowHandle hwndControl = ctlevt.controlHandle();
        IWindow* controlWindow = thisWindow->windowWithHandle(ctlevt.controlHandle());
        if (hwndControl)
        {
          unsigned long subMsg = ctlevt.parameter1().number2();
          IWindowClassName className( hwndControl );
          if ((className == WC_BUTTON) && (subMsg == BN_CLICKED))
          {
          /**************************************************************/
          /* Post private message to delay notification until after     */
          /* the selection state has been modified                      */
          /**************************************************************/
            controlWindow->postEvent( IC_UM_DELAYNOTIFY,
                                   IEventParameter1( anEvent.parameter2() ),
                                   IEventParameter2( IC_NOTDEL_PUSHBT_SELECT ));
          }
        } /* endif */
      }
      break;
      case IC_UM_DELAYNOTIFY:
      {
        if (anEvent.parameter2() == IC_NOTDEL_PUSHBT_SELECT)
        {
            thisWindow->notifyObservers(INotificationEvent(
                                        IButton::buttonClickId,
                                        *thisWindow, false, 0));
        }
      }
      break;
      default:
        break;
    }  /* end switch */
  }
#endif // IC_WIN

#ifdef IC_MOTIF
  switch (anEvent.eventId())
  {
    case motifEvent(XmCR_ACTIVATE):       //XmPushButton
    case motifEvent(XmCR_VALUE_CHANGED):  //XmToggleButton
       {
          //These events indicate that the button has been clicked.  Generate
          //the notification.

          // This code relies on derived classes to install the standard
          // iwindowMotifCallback function as an XmNactivateCallback
          // (IPushButton, ICustomButton, and derived classes) or
          // XmNvalueChangedCallback (ISettingButton derived classes).
          // This should be done wherever the widget is actually created.
          IWindow* thisWindow = anEvent.window();
          if (thisWindow)
             thisWindow->notifyObservers(
                INotificationEvent(IButton::buttonClickId,
                                   *thisWindow, false, 0));
       }
       break;

    default:
       break;
  }  /* end switch */
#endif // IC_MOITF
  return (Inherited::dispatchHandlerEvent(anEvent));
}

