// Revision: 40 1.16.1.9 source/ui/basectl/icheckbx.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icheckbx.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icheckbx.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

// #define IC_TRACE_ALL

extern "C"
{
  #define INCL_WINBUTTONS
  #define INCL_WINERRORS
  #define INCL_WINPOINTERS
  #define INCL_GPIBITMAPS
  #define INCL_WINSYS
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/Label.h>
  #include <Xm/ToggleB.h>
  #include <imstring.hpp>
  #include <ixmlabel.hpp>
#endif

#include <ibase.hpp>
#include <ibidiset.hpp>
#include <icheckbx.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <icolor.hpp>
#include <ihandle.hpp>
#include <ihandler.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <istring.hpp>
#include <inotifev.hpp>
#include <icconst.h>
#ifdef IC_WIN
  #include <ibrushdr.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICHECKBX_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public check box styles                                                      |
------------------------------------------------------------------------------*/
const ICheckBox::Style
  ICheckBox::autoSelect        ( 0, IBS_AUTOCHECKBOX ),
  ICheckBox::classDefaultStyle ( WS_VISIBLE, IBS_AUTOCHECKBOX );


/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
  ICheckBox::Style
        ICheckBox::currentDefaultStyle ( WS_VISIBLE, IBS_AUTOCHECKBOX );

#ifdef IC_PM
/*------------------------------------------------------------------------------
| The ICheckBoxData class only contains static data for OS/2 platform, so      |
| an object of this class is not required.  Once non-static data is added      |
| to this class, then the ICheckBoxData constructors must be changed to        |
| create an object of this class.                                              |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ICheckBoxData  {
public:
static ISize
  gBitmapSize;
}; // ICheckBoxData

#pragma pack(pop)
#pragma enum(pop)

#endif // IC_PM

#ifdef IC_WIN
#pragma enum(4)
#pragma pack(push,4)

class ICheckHandler : public IHandler {
typedef IHandler
  Inherited;
public:

/*------------------------------- Constructors -------------------------------*/
  ICheckHandler ( );

virtual
  ~ICheckHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual bool
  dispatchHandlerEvent ( IEvent& event );

}; // class ICheckHandler

/*------------------------------------------------------------------------------
| The ICheckBoxData class encapsulates private data and functions              |
| used by the ICheckBox class.  An object of this class is created in          |
| the ICheckBox constructors.  This is the windows version.                    |
------------------------------------------------------------------------------*/
class ICheckBoxData {
public:
  ICheckBoxData ( );

virtual
 ~ICheckBoxData ( );

static ISize
  gBitmapSize;

IBrushHandler
  fBrushHandler;
}; // ICheckBoxData

#pragma pack(pop)
#pragma enum(pop)

#endif // IC_WIN

#ifdef IC_PMWIN
  ISize ICheckBoxData::gBitmapSize ( 0, 0 );
#endif

#ifdef IC_WIN

/*------------------------------------------------------------------------------
| ICheckBoxData::ICheckBoxData                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
ICheckBoxData::ICheckBoxData ( )
  : fBrushHandler( )
{ }

/*------------------------------------------------------------------------------
| ICheckBoxData::~ICheckBoxData                                                |
|                                                                              |
------------------------------------------------------------------------------*/
ICheckBoxData::~ICheckBoxData ( )
{
}
#endif //IC_WIN

/*------------------------------------------------------------------------------
| ICheckBox::ICheckBox                                                         |
|                                                                              |
| Constructor to create a check box on a standard window.                      |
------------------------------------------------------------------------------*/
ICheckBox::ICheckBox( unsigned long ulId,
                      IWindow* pwndParent,
                      IWindow* pwndOwner,
                      const IRectangle& rectInit,
                      const Style& chbsStyle)
#ifdef IC_WIN
                     : fCheckBoxData( new ICheckBoxData )
#else
                     : fCheckBoxData( 0 )
#endif // IC_WIN
{

  IASSERTPARM(pwndParent!=0);          //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | chbsStyle.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (pwndParent)
    parentHandle = pwndParent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();

  unsigned long
    createStyle = this->convertToGUIStyle( chbsStyle );

#ifdef IC_WIN
  // Check if we need to supply bidi support to turn the check box
  // into a right-to-left one.
  if ( IBidiSettings::isBidiSupported() )
  {
     // IWindow bidi styles take precedence over inherited bidi
     // attributes.
     if ( chbsStyle & IWindow::rightToLeft )
     {  // Change the button into a right-to-left one.
        createStyle |= BS_RIGHTBUTTON;
     }
     else if ( ! ( chbsStyle & IWindow::leftToRight ) )
     {
        // The check box inherits bidi attributes from its parent
        // window.
        IBidiSettings
          bidiValues( *pwndParent );
        if ( bidiValues.windowLayout() ==
                           IBidiSettings::layoutRightToLeft )
        {
           // Change the control into a right-to-left one.
           createStyle |= BS_RIGHTBUTTON;
        }
     }
  }
#endif // IC_WIN

  //Create the check box control
  IWindowHandle whCheckBox =
      this -> create( ulId,
                      0,
                      createStyle,
                      WC_BUTTON,
                      parentHandle,
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( chbsStyle, true ) );
  startHandlingEventsFor(whCheckBox);
#ifdef IC_WIN
  fCheckBoxData->fBrushHandler.handleEventsFor( this );
#endif
#endif

#ifdef IC_MOTIF
  Arg argList[4];
  int i(0);
  IWindowHandle widgetID;

  // Determine the effect of recompute size on the button
  XtSetArg( argList[i], XmNrecomputeSize, false);i++;

  // Left align button text
  XtSetArg( argList[i], XmNalignment, XmALIGNMENT_BEGINNING); i++;

  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (pwndParent)
    parentHandle = pwndParent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();

  widgetID =
    Inherited::create(
      ulId,
      NULL,
      convertToGUIStyle(chbsStyle),
      (IXmCreateFunction)XmCreateToggleButton,
      parentHandle,
      ownerHandle,
      rectInit,
      argList,
      i);

  if (widgetID == 0)
     ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "check box toggle");

  startHandlingEventsFor(widgetID);

  // Set up pointerFocus based on the IButton::noPointerFocus style.
  this->enableMouseClickFocus( (chbsStyle & noPointerFocus) ? false : true );

  // If the label string is equal to the widget name, set it to the null
  // string.  MUST do this conditionally after create to allow the use of
  // resource files.
  IXmLabel::initializeXmLabelString( widgetID, IString( ulId ), true );

  // Register callbacks.
//Widget checkBox=widgetID->handle();

  // Register the valueChanged callback.
  XtAddCallback (widgetID,
                 XmNvalueChangedCallback,
                 iwindowMotifCallback,
                 (XtPointer) this );
#endif
}


/*------------------------------------------------------------------------------
| ICheckBox::ICheckBox                                                         |
|                                                                              |
| Constructor to instantiate an object for a dialog template.                  |
------------------------------------------------------------------------------*/
ICheckBox::ICheckBox(unsigned long ulId,
                     IWindow* pdlgwndParent)
#ifdef IC_WIN
                     : fCheckBoxData( new ICheckBoxData )
#else
                     : fCheckBoxData( 0 )
#endif // IC_WIN
{
  // assertions on input parms
  IASSERTPARM(pdlgwndParent!=0);

  setAutoDestroyWindow(false);
#ifdef IC_PMWIN
  reserveUserWindowWord( false );
  startHandlingEventsFor(ulId, pdlgwndParent);
#endif
#ifdef IC_WIN
  fCheckBoxData->fBrushHandler.handleEventsFor( this );
#endif

#ifdef IC_MOTIF
  IString widgetName = IString (ulId);
  IWindowHandle whCheckBox =
                  XtNameToWidget(pdlgwndParent->handle(), widgetName);

  if (whCheckBox == 0)
      ITHROWLIBRARYERROR1( IC_CONTROL_NOT_FOUND,
                           IBaseErrorInfo::accessError,
                           IException::recoverable,
                           "XtNameToWidget (check box toggle");

   if ( !(XtIsSubclass(whCheckBox, xmToggleButtonWidgetClass)) )
     ITHROWLIBRARYERROR1( IC_WRONG_CONTROL_TYPE,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "XmToggleButton");

  prepareForUse(whCheckBox);
  startHandlingEventsFor(whCheckBox);

  // Register callbacks.
//Widget checkBox=whCheckBox->handle();

  // Register the valueChanged callback.
  XtAddCallback (whCheckBox,
                 XmNvalueChangedCallback,
                 iwindowMotifCallback,
                 (XtPointer) this );
#endif
}


/*------------------------------------------------------------------------------
| ICheckBox::ICheckBox                                                         |
|                                                                              |
| Constructor to instantiate an object from an existing check box control.     |
------------------------------------------------------------------------------*/
ICheckBox::ICheckBox(const IWindowHandle& wh)
#ifdef IC_WIN
                     : fCheckBoxData( new ICheckBoxData )
#else
                     : fCheckBoxData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow(false);
#ifdef IC_PMWIN
  reserveUserWindowWord( false );
  startHandlingEventsFor(wh);
#endif
#ifdef IC_WIN
  fCheckBoxData->fBrushHandler.handleEventsFor( this );
#endif
#ifdef IC_MOTIF
  IASSERTPARM(wh!=0);

  if ( !(XtIsSubclass(wh, xmToggleButtonWidgetClass)) )
    ITHROWLIBRARYERROR1( IC_WRONG_CONTROL_TYPE,
                         IBaseErrorInfo::accessError,
                         IException::recoverable,
                         "XmToggleButton");

  setAutoDestroyWindow(false);
  prepareForUse(wh);    // prepareForUse will test for valid handle
  startHandlingEventsFor(wh);

  // Register callbacks.
//Widget checkBox=wh->handle();

  // Register the valueChanged callback.
  XtAddCallback (wh,
                 XmNvalueChangedCallback,
                 iwindowMotifCallback,
                 (XtPointer) this );
#endif
}


/*------------------------------------------------------------------------------
| ICheckBox::~ICheckBox                                                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ICheckBox::~ICheckBox()
{
#ifdef IC_WIN
  fCheckBoxData->fBrushHandler.stopHandlingEventsFor( this );
#endif //IC_WIN
#ifdef IC_PMWIN
  if ( fCheckBoxData != 0 )
    delete fCheckBoxData;
#endif
#ifdef IC_MOTIF
  if (this->isValid())
  {
     Widget checkBox=handle();

     // Remove the valueChanged callback.
     XtRemoveCallback (checkBox,
                       XmNvalueChangedCallback,
                       iwindowMotifCallback,
                       (XtPointer) this );
  }
#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| ICheckBox::defaultStyle                                                      |
|                                                                              |
| Return the default style for new check box objects.                          |
------------------------------------------------------------------------------*/
ICheckBox::Style  ICheckBox::defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| ICheckBox::setDefaultStyle                                                   |
------------------------------------------------------------------------------*/
void  ICheckBox::setDefaultStyle(const Style& chbsStyle)
{
  currentDefaultStyle = chbsStyle;
}


/*------------------------------------------------------------------------------
| ICheckBox::convertToGUIStyle                                                 |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ICheckBox::convertToGUIStyle(const IBitFlag& guiStyle,
                                           bool bExtOnly) const
{
  // Obtain the style from the class (ISettingButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXTCHECKBX;

    // Handle overlapping styles
    if (ulTmp & IBS_AUTOCHECKBOX)
      ulStyle |= BS_AUTOCHECKBOX;
    else
      ulStyle |= BS_CHECKBOX;

#ifdef IC_WIN
    ulStyle |= WS_CHILD;
#endif
  }


  return( ulStyle );
}


/*------------------------------------------------------------------------------
| ICheckBox::enableAutoSelect                                                  |
|                                                                              |
| Sets or removes the autoSelect style for the check box control.              |
------------------------------------------------------------------------------*/
ICheckBox&  ICheckBox::enableAutoSelect(bool bTurnOn)
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;  //Use masking flag from pmwin.h
    ulStyle |= BS_AUTOCHECKBOX;
  }
  else
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;  //Use masking flag from pmwin.h
    ulStyle |= BS_CHECKBOX;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                        //Force repaint of control
  }
  return *this;
}


/*------------------------------------------------------------------------------
| ICheckBox::disableAutoSelect                                                 |
------------------------------------------------------------------------------*/
ICheckBox&  ICheckBox::disableAutoSelect()
{
  enableAutoSelect(false);
  return *this;
}


/*------------------------------------------------------------------------------
| ICheckBox::isAutoSelect                                                      |
------------------------------------------------------------------------------*/
bool  ICheckBox::isAutoSelect() const
{
  return( (style() & BS_AUTOCHECKBOX & (unsigned long)~(BS_CHECKBOX)) ? true : false );
}


/*------------------------------------------------------------------------------
| ICheckBox::calcMinimumSize                                                   |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The minimum size   |
| is based on the maximum height of the font and the font width of the         |
| check box text (plus the size of the check box itself).                      |
------------------------------------------------------------------------------*/
ISize  ICheckBox::calcMinimumSize() const
{
   unsigned long i        ( textLength() );
   ISize         size     ( i ? displaySize(text()) : ISize());
   unsigned long ulWidth  ( size.width() );
   unsigned long ulHeight ( size.height() );

#ifdef IC_PMWIN
   ISize bmpSize = ICheckBoxData::gBitmapSize;
   if ( bmpSize == ISize() )
   {                    // Find size of check box bitmap.
   #ifdef IC_PM
      BITMAPINFOHEADER pbmpData;
      pbmpData.cbFix=sizeof(BITMAPINFOHEADER);
      HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
      IQUERYBITMAPPARAMETERS(hbit, &pbmpData);    //Retrieve size of check
      IDELETEBITMAP(hbit);                        //box bitmap
      ICheckBoxData::gBitmapSize.setWidth( pbmpData.cx );
      ICheckBoxData::gBitmapSize.setHeight( pbmpData.cy );
   #endif
   #ifdef IC_WIN
      BITMAP           pbmpData;
      HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
      IQUERYBITMAPPARAMETERS(hbit, &pbmpData);    //Retrieve size of check
      IDELETEBITMAP(hbit);                        //box bitmap
      ICheckBoxData::gBitmapSize.setWidth( pbmpData.bmWidth );
      ICheckBoxData::gBitmapSize.setHeight( pbmpData.bmHeight );
   #endif
      bmpSize = ICheckBoxData::gBitmapSize;
   }

   if (ulHeight < (bmpSize.height()/4) )
     ulHeight = bmpSize.height()/4;
   ulHeight += (2 * IQUERYSYSVALUE(SV_CYBORDER));

   if (i!=0)
     ulWidth += ulWidth/i;             //Has text
   ulWidth += ((bmpSize.width()/4) + 2 + (2 * IQUERYSYSVALUE(SV_CXBORDER)));
#endif  //IC_PMWIN

#ifdef IC_MOTIF
   Dimension shadowThickness, borderWidth, highlightThickness,
             marginHeight, marginWidth,
             marginLeft, marginRight, marginTop, marginBottom,
             indicatorSize;

   XtVaGetValues(this->handle(),
                 XmNshadowThickness,    &shadowThickness,
                 XmNborderWidth,        &borderWidth,
                 XmNhighlightThickness, &highlightThickness,
                 XmNmarginHeight,       &marginHeight,
                 XmNmarginWidth,        &marginWidth,
                 XmNmarginLeft,         &marginLeft,
                 XmNmarginRight,        &marginRight,
                 XmNmarginTop,          &marginTop,
                 XmNmarginBottom,       &marginBottom,
                 XmNindicatorSize,      &indicatorSize,
                 0);
  ITRACE_ALL(IString("CheckBox:") +
             IString(" ulWidth=") +               IString(ulWidth) +
             IString(" ulHeight=") +              IString(ulHeight) +
             IString(" XmNshadowThickness=") +    IString(shadowThickness) +
             IString(" XmNborderWidth=") +        IString(borderWidth) +
             IString(" XmNhighlightThickness=") + IString(highlightThickness) +
             IString(" XmNmarginHeight=") +       IString(marginHeight) +
             IString(" XmNmarginWidth=") +        IString(marginWidth) +
             IString(" XmNmarginLeft=") +         IString(marginLeft) +
             IString(" XmNmarginRight=") +        IString(marginRight) +
             IString(" XmNmarginTop=") +          IString(marginTop) +
             IString(" XmNmarginBottom=") +       IString(marginBottom) +
             IString(" XmNindicatorSize=") +      IString(indicatorSize) );

   unsigned long edges = 2*(shadowThickness+borderWidth+highlightThickness);
   ulHeight += 2*marginHeight + marginTop + marginBottom;
   if (ulHeight < indicatorSize)
     ulHeight = indicatorSize;
   ulHeight += edges;
   ulWidth += 2*marginWidth;
   if (i==0)
      ulWidth += indicatorSize;             //no text
   else
      ulWidth += marginLeft + marginRight;  //has text
   ulWidth  += edges;
#endif  //IC_MOTIF
   return ISize(ulWidth, ulHeight);
}
