// Revision: 66 1.16 source/ui/basectl/idrawcv.cpp, canvas, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idrawcv.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in idrawcv.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

#define INCL_WINSTATICS
extern "C" {
  #include <iwindefs.h>
}

#include <iexgrprt.hpp>
#include <igraph2d.hpp>
#include <idrawcv.hpp>
#include <icconst.h>
#include <ihandle.hpp>
#include <ipainhdr.hpp>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>
#include <imngpres.hpp>

#ifdef IC_PMWIN
#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )

// Note:  The following function pointers exist to keep this module
//        from binding a reference to the old graphics library. The
//        functions themselves are in graphic\iuiobs.cpp and are
//        established in calls to setGraphicList or setGraphicContext
//        (also in graphic\iuiobs.cpp).
static PTROldCalcMinimumSize gOldCalcMinimumSize = 0;
static PTROldPaintWindow     gOldPaintWindow = 0;
void IDrawingCanvas::setOldCalcMinimumSize (PTROldCalcMinimumSize ptrOldCalcMinimumSize)
{
   gOldCalcMinimumSize = ptrOldCalcMinimumSize;
}

void IDrawingCanvas::setOldPaintWindow (PTROldPaintWindow ptrOldPaintWindow)
{
   gOldPaintWindow = ptrOldPaintWindow;
}

#endif // IC_OBSOLETE
#endif // IC_PMWIN

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDRAWCV_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public drawingarea styles                                                    |
------------------------------------------------------------------------------*/
const IDrawingCanvas::Style
  IDrawingCanvas::useDefaultPaintHandler ( 0, ICNV_ATTACHDEFAULTHANDLER ),
  IDrawingCanvas::classDefaultStyle    ( (WS_VISIBLE | WS_CLIPSIBLINGS),
                                       ICNV_ATTACHDEFAULTHANDLER );
#pragma enum(4)
#pragma pack(push,4)

class IDrawingCanvasDrawHandler : public IPaintHandler
{
typedef IPaintHandler
  Inherited;
/*******************************************************************************
* The IDefaultDrawHandle is a paint handler that is used to handle paint       *
* events for the IDrawingCanvas class.  This handler is automatically created  *
* and added to an IDrawingCanvas object unless you create the IDrawingCanvas   *
* without the IDrawingCanvas style useDefaultPaintHandler.                       *
*                                                                              *
* The IDrawingCanvasDrawHandler overrides the paintWindow function.  When the paint  *
* window function is called, this handler will set the clip region to the      *
* windows update region and redraw the affected graphic in the graphic list    *
* of the draw canvas.                                                          *
*******************************************************************************/
public:
/*------------------------------- Constructor ----------------------------------
| The only way to construct an instance of this class is from the              |
| default constructor.                                                         |
------------------------------------------------------------------------------*/
  IDrawingCanvasDrawHandler();
virtual
  ~IDrawingCanvasDrawHandler();

protected:
/*----------------------------- Event Processing -------------------------------
| paintWindow - Processes paint events.  Iterate through graphic objects in    |
|               the drawing area and call the draw function for each.          |
------------------------------------------------------------------------------*/
virtual bool
  paintWindow( IPaintEvent& event );

};

class IDrawingCanvasData {
/*******************************************************************************
* The IDrawingCanvasData class is private data for IDrawingCanvas.             *
*******************************************************************************/
public:
   IDrawingCanvasData();
   ~IDrawingCanvasData();
IDrawingCanvasDrawHandler
  fdefaultHandler;
};

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IDrawingCanvasData::IDrawingCanvasData                                       |
------------------------------------------------------------------------------*/
IDrawingCanvasData::IDrawingCanvasData()
  : fdefaultHandler( )
{}

/*------------------------------------------------------------------------------
| IDrawingCanvasData::~IDrawingCanvasData                                      |
------------------------------------------------------------------------------*/
IDrawingCanvasData::~IDrawingCanvasData()
{}

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
IDrawingCanvas::Style
  IDrawingCanvas::fgCurrentDefaultStyle ( (WS_VISIBLE | WS_CLIPSIBLINGS),
                                          ICNV_ATTACHDEFAULTHANDLER );





/*------------------------------------------------------------------------------
| IDrawingCanvas::IDrawingCanvas                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDrawingCanvas::IDrawingCanvas( unsigned long windowIdentifier,
                                IWindow* parent,
                                IWindow* owner,
                                const IRectangle& initial,
                                const Style& style)
  : IDrawingCanvas::Inherited( ),
    fGroup(0),
    fPort(0),
#ifdef IC_PMWIN
#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )
    fList(0),
    fGc(0),
#endif // IC_OBSOLETE
#endif // IC_PMWIN
    fDrawingCanvasData( new IDrawingCanvasData ),
    fContextType(kNoneSpecified),
    fListType(kNoneSpecified)
{
  // Save the extended style to make sure we have a copy of it stored.
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  // Create the presentation system window.
  this->initialize( windowIdentifier,
                    parent,
                    owner,
                    initial,
                    this->convertToGUIStyle( style ),
                    this->convertToGUIStyle( style, true ) );

  // Attach the default draw handler if the style is set.
  if ( style & useDefaultPaintHandler )
  {
    fDrawingCanvasData->fdefaultHandler.handleEventsFor( this );
  }
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::~IDrawingCanvas                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDrawingCanvas::~IDrawingCanvas( )
{
  // Remove and delete the default handler if it exists.
  if (extendedStyle() & useDefaultPaintHandler.asExtendedUnsignedLong())
  {
    fDrawingCanvasData->fdefaultHandler.stopHandlingEventsFor( this );
  }
  delete fDrawingCanvasData;
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::defaultStyle                                                 |
|                                                                              |
| Return the current default style for the class.                              |
------------------------------------------------------------------------------*/
IDrawingCanvas::Style IDrawingCanvas::defaultStyle ( )
{
  return fgCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::convertToGUIStyle                                            |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                                 |
------------------------------------------------------------------------------*/
unsigned long IDrawingCanvas :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                  bool bExtOnly) const
{
  // Obtain the style from the class (ICanvas) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Replace canvas window style with drawing canvas window style.
#ifdef IC_PMWIN
    ulStyle |= ( guiStyle.asUnsignedLong() & ~IC_CVS_CANVAS ) |
                                              IC_CVS_DRAWINGCANVAS |
#else
    // No styles are defined for the drawing area, so we do not mask.
    ulStyle |= guiStyle.asUnsignedLong() |
#endif
#ifdef IC_WIN
               WS_CHILD |
#endif
    // Add the static background rectangle and group styles.
               IWindow::group.asUnsignedLong();
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| ICanvas::calcMinimumSize                                                     |
|                                                                              |
| Calculate the minimum screen size needed by the canvas.  This is the size    |
| needed to display all child windows (as determined by the layout routine).   |
------------------------------------------------------------------------------*/
ISize  IDrawingCanvas::calcMinimumSize ( ) const
{
  ISize      minSize(0,0);

  if ((fListType == kNewGraphics) || (fListType == kNoneSpecified)) {
     IGrafPort *gp(this->grafPort());
     bool createdGP(false);
     IManagedPresSpaceHandle* presSpace=0;

     IRectangle groupBoundingRect;

     if (!gp)
     {
       presSpace = new IManagedPresSpaceHandle((IWindow*)this);
       gp = new IRootGrafPort(*presSpace);
       createdGP = true;
     }

     if (this->graphicGroup() && this->graphicGroup()->count()!=0)
     {
       groupBoundingRect = this->graphicGroup()->looseFitBounds( gp );
       minSize.setWidth( groupBoundingRect.maxX() );
       minSize.setHeight( groupBoundingRect.maxY() );
     }

     if (createdGP)
     {
       delete gp;
       if (presSpace)
          delete presSpace;
     }

     minSize = minSize.maximum( Inherited::calcMinimumSize() );
  }
#ifdef IC_PMWIN
  else {
     IASSERTSTATE(gOldCalcMinimumSize!=0);
     minSize = (*gOldCalcMinimumSize)(this,Inherited::calcMinimumSize());
  }
#endif

  return minSize;
}

/*------------------------------------------------------------------------------
| IDrawingCanvasDrawHandler::IDrawingCanvasDrawHandler                         |
|                                                                              |
------------------------------------------------------------------------------*/
IDrawingCanvasDrawHandler::IDrawingCanvasDrawHandler( )
{ }

/*------------------------------------------------------------------------------
| IDrawingCanvasDrawHandler::~IDrawingCanvasDrawHandler                        |
|                                                                              |
| Empty destructor for page tuning.                                            |
------------------------------------------------------------------------------*/
IDrawingCanvasDrawHandler::~IDrawingCanvasDrawHandler( )
{ }

/*------------------------------------------------------------------------------
| IDrawingCanvasDrawHandler::paintWindow                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDrawingCanvasDrawHandler::paintWindow( IPaintEvent& paintEvent )
{
   bool          didDraw(false);
   bool          createdGC(false);
   IDrawingCanvas*  drawingCanvas((IDrawingCanvas*)paintEvent.window());

   if (drawingCanvas->fListType == IDrawingCanvas::kNewGraphics)
      IASSERTSTATE(drawingCanvas->fContextType != IDrawingCanvas::kOldGraphics);
   if (drawingCanvas->fListType == IDrawingCanvas::kOldGraphics)
      IASSERTSTATE(drawingCanvas->fContextType != IDrawingCanvas::kNewGraphics);

   if (drawingCanvas->fListType == IDrawingCanvas::kNewGraphics) {
       if ( drawingCanvas->graphicGroup() ) {
          // Draw the group.
          IGrafPort* port =
             drawingCanvas->grafPort();
          if ( port ) {
             // Tell paint event to use this port.
             paintEvent.setGrafPort( *port );
          } else {
             // Create port from paint event's handle.
             port =
                new IExtendedRootGrafPort(
                   paintEvent.presSpaceHandle(),
                   ICoordinateSystem::applicationOrientation() ==
                                      ICoordinateSystem::originUpperLeft ?
                   kLeftHand : kRightHand );
             createdGC = true;
          }
          if ( paintEvent.rect().area() ) {
             paintEvent.clearBackground( drawingCanvas->backgroundColor() );
             drawingCanvas->graphicGroup()->draw( *port );
             didDraw = true;
          }
          if ( createdGC ) {
             delete port;
          }
       }
   }
#ifdef IC_PMWIN
   else if (drawingCanvas->fListType == IDrawingCanvas::kOldGraphics) {
      if (drawingCanvas->fList !=0) {
          IASSERTSTATE(gOldPaintWindow!=0);
          didDraw = (*gOldPaintWindow)(drawingCanvas, paintEvent);
      }
   }
#endif

  return didDraw;
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::graphicGroup                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicGroup* IDrawingCanvas::graphicGroup( ) const
{
  return fGroup;
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::setGraphicGroup                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IDrawingCanvas::setGraphicGroup( IGraphicGroup* group )
{
  fGroup = group;
  fListType = kNewGraphics;
  setLayoutDistorted(IWindow::layoutChanged, 0);
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::setGrafPort                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
void IDrawingCanvas::setGrafPort( IGrafPort* pGP )
{
  fPort = pGP;
  fContextType = kNewGraphics;
}

/*------------------------------------------------------------------------------
| IDrawingCanvas::grafPort                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IGrafPort* IDrawingCanvas::grafPort( ) const
{
  return fPort;
}
