// Revision: 77 1.5.2.6 source/ui/basectl/igraphdr.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: igraphdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igraphdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_PM
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifdef IC_MOTIF
  #include <Xm/PushB.h>
#endif // IC_MOTIF

#include <icconst.h>
#include <icoordsy.hpp>
#include <ifont.hpp>
#include <ievent.hpp>
#include <igraphdr.hpp>
#include <igraphbt.hpp>
#include <iiconctl.hpp>
#include <ireslib.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRAPHDR_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IGraphicPushButtonHandler::IGraphicPushButtonHandler                         |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IGraphicPushButtonHandler::IGraphicPushButtonHandler ( )
{ ; }


/*------------------------------------------------------------------------------
| IGraphicPushButtonHandler::~IGraphicPushButtonHandler                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IGraphicPushButtonHandler::~IGraphicPushButtonHandler ( )
{ ; }


/************************************************************************/
/* dispatchHandler Event - Dispatch command functions for this handler. */
/************************************************************************/
bool IGraphicPushButtonHandler :: dispatchHandlerEvent(IEvent & evt)
{
   IGraphicPushButton* graphicbutton (0);
   bool bMovedImage (false);
   switch (evt.eventId())
   {
      case BM_SETHILITE:
         {
           // We keep track of whether the button is up or down in
           // the control's private data.  We need this since there are
           // scenarios where multiple BM_SETSTATE (BM_SETHILITE) messages
           // will be received - such as mouse movement w/ button 1 down.
           // Without this state data the graphic moves off the button.
           int yDeltaDown = 1;
           if (ICoordinateSystem::applicationOrientation() ==
               ICoordinateSystem::originLowerLeft )
             yDeltaDown = -1;
           graphicbutton = dynamic_cast<IGraphicPushButton*>(evt.window());
           if (graphicbutton)
           {
              IPoint location = graphicbutton->graphicWindow().position();
              if (evt.parameter1().asUnsignedLong() == 0)
              {
                 // draw button up
                 if (graphicbutton->fGraphicPushButtonData->fDrawDown)
                 {
                    graphicbutton->fGraphicPushButtonData->fDrawDown = false;
                    bMovedImage = true;
                    location.setX(location.x()-1);
                    location.setY(location.y()-yDeltaDown);
                 }
   #ifdef IC_PM
                 // There is an OS/2 bug that does not repaint the top
                 // and left outermost edge of the graphic when no-border
                 // and no-margin is set.  So... we need to be sure that
                 // the entire graphic is invalidated.
                 if ( !graphicbutton->hasBorder() &&
                    ( graphicbutton->marginSize() == ISize(0,0) ))
                 {
                      IRectangle rect1 =
                         graphicbutton->graphicWindow().nativeRect();
                      IRectangle rect2(rect1.left(),  rect1.top(),
                                       rect1.right()+1, rect1.top()+1);
                      graphicbutton->graphicWindow().refresh(
                         ICoordinateSystem::convertToApplication(
                            rect2, graphicbutton->size() ) );
                      rect2 = IRectangle(rect1.left(),  rect1.bottom(),
                                         rect1.left()+1, rect1.top()+1);
                      graphicbutton->graphicWindow().refresh(
                         ICoordinateSystem::convertToApplication(
                            rect2, graphicbutton->size() ) );
                 }
   #endif
              }
              else
              {
                 // draw button down
                 if (!graphicbutton->fGraphicPushButtonData->fDrawDown)
                 {
                    graphicbutton->fGraphicPushButtonData->fDrawDown = true;
                    bMovedImage = true;
                    location.setX(location.x()+1);
                    location.setY(location.y()+yDeltaDown);
                 } /* endif */
              }
              if (bMovedImage)
                 graphicbutton->graphicWindow().moveTo(location);
           }  // if graphicbutton
         }

#ifdef IC_PM
       // intentional fall through to next case
      case WM_SETWINDOWPARAMS:
      case BM_SETDEFAULT:
      case WM_SYSVALUECHANGED:
         if (!graphicbutton)
           graphicbutton = (IGraphicPushButton*)evt.window();

         graphicbutton->setStyle(graphicbutton->style() |
                                 IWindow::clipChildren.asUnsignedLong());
         // Pass setText operation to the graphicWindow
         if (evt.eventId() == WM_SETWINDOWPARAMS)
         {
             PWNDPARAMS params = (PWNDPARAMS)evt.parameter1().asUnsignedLong();
             if (params->fsStatus == WPM_TEXT)
                 graphicbutton->graphicWindow().setText( params->pszText );
         }
         defaultProcedure(evt);
         graphicbutton->setStyle(graphicbutton->style() &
                                 ~IWindow::clipChildren.asUnsignedLong());
#else
         defaultProcedure( evt );
         if (bMovedImage)
           graphicbutton->graphicWindow().refresh( IWindow::paintAllImmediate);
#endif
         return true;

#ifdef IC_WIN
      case WM_ENABLE:
         graphicbutton = (IGraphicPushButton*)evt.window();
         if (graphicbutton)
            graphicbutton->graphicWindow().enable(
               evt.parameter1().asUnsignedLong() );

         break;
      case WM_SETTEXT:
         graphicbutton = (IGraphicPushButton*)evt.window();
         if (graphicbutton)
         {
            char* text = evt.parameter2();
            graphicbutton->graphicWindow().setText( text );
            evt.setResult(1);  // text has been set
            return true;       // no more handlers
         }
         break;
      case WM_SETFONT:
         graphicbutton = (IGraphicPushButton*)evt.window();
         if (graphicbutton)
         {
            // Pass the font change to the graphicWindow.
            graphicbutton->graphicWindow().setFont( graphicbutton->font() );
         }
         break;
      case WM_GETFONT:
      case WM_GETTEXT:
      case WM_GETTEXTLENGTH:
         graphicbutton = (IGraphicPushButton*)evt.window();
         if (graphicbutton)
         {
            evt.setResult(
              graphicbutton->graphicWindow().sendEvent( evt ) );
            return true;       // no more handlers
         }
         break;
#endif

      default:
         break;
   } /* endswitch */
   return false;
}
#endif // IC_PMWIN

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IGraphicPushButtonIconHandler::IGraphicPushButtonIconHandler                 |
------------------------------------------------------------------------------*/
IGraphicPushButtonIconHandler::IGraphicPushButtonIconHandler ()
{ }

/*------------------------------------------------------------------------------
| IGraphicPushButtonIconHandler::~IGraphicPushButtonIconHandler                |
------------------------------------------------------------------------------*/
IGraphicPushButtonIconHandler::~IGraphicPushButtonIconHandler ()
{ }

/*------------------------------------------------------------------------------
| IGraphicPushButtonIconHandler::dispatchHandlerEvent                          |
------------------------------------------------------------------------------*/
bool IGraphicPushButtonIconHandler::dispatchHandlerEvent ( IEvent& event )
{
   bool rc = false;
   switch ( event.eventId() )
   {
   case WM_CONTEXTMENU:
   case WM_INITMENU:
   case WM_MENUSELECT:
   case WM_MENUEND:
        event.setResult(event.window()->parent()->sendEvent(event));
        rc = true;
        break;

   }
   return rc;
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| IGraphicPushButtonData :: IGraphicPushButtonData                             |
------------------------------------------------------------------------------*/
IGraphicPushButtonData :: IGraphicPushButtonData ( )
    : fsizeHandler ()
    , fsizeImageToWindow( false )
    , fsizeWindowToImage( false )
#ifdef IC_MOTIF
    , contains( IGraphicPushButton::bitmapType )
    , fbitmap( 0 )
    , ficon( 0 )
#endif
#ifdef IC_PMWIN
    , fpaintHandler ()
    , ficonHandler ()
    , fDrawDown( false )
#endif
{
}


/*------------------------------------------------------------------------------
| IGraphicPushButtonData :: ~IGraphicPushButtonData                            |
------------------------------------------------------------------------------*/
IGraphicPushButtonData :: ~IGraphicPushButtonData ()
{
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IGraphicPushButtonData :: setFromWidget                                      |
------------------------------------------------------------------------------*/
void IGraphicPushButtonData::setFromWidget ( Widget wid )
{
  // Remove fillOnArm so that image stays in the button when it is clicked.
  XtVaSetValues( wid,  XmNfillOnArm, False, 0);
  Pixmap labelPixmap;
  XtVaGetValues( wid,
                 XmNlabelPixmap, &labelPixmap,
                 NULL);

  contains = IGraphicPushButton::bitmapType;
  ficon = IPointerHandle(0);

  if ( labelPixmap==XmUNSPECIFIED_PIXMAP )
    fbitmap = IBitmapHandle();
  else
    fbitmap = (IBitmapHandle)labelPixmap;
}
#endif // IC_MOTIF

