/* NOSHIP */
// Revision: 78 1.4.1.5 source/ui/basectl/igraphdr.hpp, buttonctls, ioc.v400, 000919 
#ifndef _IGRAPHDR_
  #define _IGRAPHDR_
/*******************************************************************************
* FILE NAME: igraphdr.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IGraphicPushButtonHandler                                                *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandler.hpp>
#include <igraphbt.hpp>
#include <igrszhdr.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IEvent;

#pragma enum(4)
#pragma pack(push,4)

class IGraphicPushButtonHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* Instances of the IGraphicPushButtonHandler class process the BM_SETHILITE    *
* event for IGraphicPushButtons.                                               *
*******************************************************************************/
public:
/*------------------------------- Constructor ----------------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
  IGraphicPushButtonHandler       ( );
virtual
 ~IGraphicPushButtonHandler       ( );

protected:
/*---------------------------- Event Dispatching -------------------------------
| This function evaluates the event to determine if it is appropriate for      |
| this handler object to process.                                              |
|   dispatchHandlerEvent - If it finds the BM_SETHILITE message, then it forces|
|                          a repaint of the graphic.  This function always     |
|                          returns false so that if there are any other        |
|                          handlers, they can process the messages.            |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent ( IEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/

}; // IGraphicPushButtonHandler

#ifdef IC_PMWIN
class IGraphicPushButtonIconHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* Instances of the IGraphicPushButtonIconHandler class route menu messages     *
* from the IIconControl to the IGraphicPushButton.                             *
*******************************************************************************/
public:
/*------------------------------- Constructor --------------------------------*/
  IGraphicPushButtonIconHandler       ( );
virtual
 ~IGraphicPushButtonIconHandler       ( );

protected:
/*---------------------------- Event Dispatching -----------------------------*/
virtual bool
  dispatchHandlerEvent ( IEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/

}; // IGraphicPushButtonIconHandler
#endif //IC_PMWIN

class IGraphicPushButtonData {
/*******************************************************************************
* Private data for IGraphicPushButton.                                         *
*******************************************************************************/
public:
  IGraphicPushButtonData ( );
  ~IGraphicPushButtonData( );

IGraphicPushButtonResizeHandler
  fsizeHandler;
bool
  fsizeImageToWindow,
  fsizeWindowToImage;

#ifdef IC_MOTIF
const IBitmapHandle&
  bitmap() const { return fbitmap; }
const IPointerHandle&
  icon() const { return ficon; }
void
  setBitmap( const IBitmapHandle& bitmaphandle ) {fbitmap=bitmaphandle; };
void
  setIcon( const IPointerHandle& pointerhandle ) {ficon=pointerhandle; };

void
  setFromWidget ( Widget wid );

IGraphicPushButton::GraphicType
   contains;
IBitmapHandle
   fbitmap;
IPointerHandle
   ficon;
#endif // IC_MOTIF

#ifdef IC_PMWIN
IGraphicPushButtonHandler
  fpaintHandler;
IGraphicPushButtonIconHandler
  ficonHandler;
bool
  fDrawDown;          // the button is pressed
#endif

private:
  IGraphicPushButtonData::IGraphicPushButtonData (
          const IGraphicPushButtonData&);
  IGraphicPushButtonData& operator= (const IGraphicPushButtonData&);

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _IGRAPHDR_ */
