// Revision: 81 1.1.2.8 source/ui/basectl/igrszhdr.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: igrszhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igrszhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINFRAMEMGR          // for WM_WINDOWPOSCHANGED
#define INCL_WINSYS
extern "C" {
   #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/PushB.h>
#endif

#include <ibmpdata.hpp>
#include <ibmpstat.hpp>
#include <icconst.h>
#include <icustbdd.hpp>
#include <igraphbt.hpp>
#include <igrszhdr.hpp>
#include <iiconctl.hpp>
#ifdef IC_MOTIF
   #include <imstring.hpp>
#endif //IC_MOTIF
#include <istring.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRSZHDR_CPP_
  #include <ipagetun.h>
#endif


/*******************************************************************/
/* Default constructor here for page tuning.                       */
/*******************************************************************/
IGraphicPushButtonResizeHandler :: IGraphicPushButtonResizeHandler ( )
{}


/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IGraphicPushButtonResizeHandler :: ~IGraphicPushButtonResizeHandler ( )
{}

/*********************************************************************/
/* IGraphicPushButtonResizeHandler :: windowResize                   */
/*                                                                   */
/* Sizes the graphic window portion of the IGraphicPushButton control*/
/* to be within the border of the push button.                       */
/*********************************************************************/
bool IGraphicPushButtonResizeHandler :: windowResize ( IResizeEvent& evt )
{
  IGraphicPushButton* button = dynamic_cast<IGraphicPushButton*>(evt.window());
  ITRACE_ALL( IString("IGraphicPushButtonResizeHandler hwnd=") +
              IString( evt.handle().asUnsigned() ).d2x() );
  if (button)
  {
#ifdef IC_PMWIN
     ISize border=button->marginSize();
     long inith  = border.height(),
          initw  = border.width(),
          width  = -initw,
          height = -inith;
     if ((width += evt.newSize().width() ) < 0)
        width = 0;
     if ((height+= evt.newSize().height()) < 0)
        height = 0;
     ITRACE_ALL( IString("IGraphicPushButtonResizeHandler hwnd=") +
                 IString( evt.handle().asUnsigned() ).d2x() +
                 IString(" newRect=") +
                 IRectangle(initw,inith,width,height).asString() );
     button->graphicWindow().moveSizeTo(IRectangle(initw,inith,width,height));
     return true;
#endif // IC_PMWIN
#ifdef IC_MOTIF
     return this->drawBackground( button );
#endif //IC_MOTIF
  }
  return false;
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IGraphicPushButtonResizeHandler::dispatchHandlerEvent                        |
------------------------------------------------------------------------------*/
bool IGraphicPushButtonResizeHandler::dispatchHandlerEvent( IEvent& event )
{
   if ( (event.eventId() == IC_UM_PAINT) ||
        (event.eventId() == xEvent(MapNotify)) )
   {   // Respond to refresh() by redrawing button.
       IGraphicPushButton* button =
           dynamic_cast<IGraphicPushButton*>(event.window());
       if (button)
       {
          return this->drawBackground( button );
       }
   }
   return Inherited::dispatchHandlerEvent( event );
}
#endif //IC_MOTIF

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IGraphicPushButtonResizeHandler::drawBackground                              |
------------------------------------------------------------------------------*/
bool IGraphicPushButtonResizeHandler::drawBackground( IGraphicPushButton* window)
{
   IMODTRACE_ALL("IGraphicPushButtonResizeHandler::drawBackground");
   bool result(false);
   if (window->isValid())
   {
      // The image needs to be re-drawn so that it is sized to the
      // current size of the control to prevent tiling.
      // Get a pointer to the cached image information.
      ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
         ( window->windowData( ICustomButtonDrawData::dataHandle() ) );
      if (drawData)
      {
         bool refreshWidget( drawData->invalidateAll( 0 ) );
         ISize windowSize ( window->size() );
         ITRACE_ALL( IString("windowSize=") + windowSize.asString() );
         if ( windowSize != ISize() )
         {
            // Create an image in memory sized to the window.
            IGImage* cache = new IGImage( (long)windowSize.width(),
                                          (long)windowSize.height(),
                                          IGImage::kTrueColor24Bit,
                                          ICoordinateSystem::kOriginUpperLeft);

            IGImage* bitmapImage = 0;
            IGImage* maskImage = 0;
            IGRect2D bitmapRect;
            // Construct an IGImage for the client supplied image.
            if (window->currentGraphicType() == IGraphicPushButton::bitmapType)
            {
               IBitmapHandle bmp(window->bitmap());
               ITRACE_ALL( IString("drawing bmp=") +
                           IString((unsigned long)(IBitmapHandle::Value)bmp).d2x() );
               if ((IBitmapHandle::Value)bmp != 0)
               {
                  bitmapImage = new IGImage( bmp );
                  bitmapRect = bitmapImage->rasterBounds();
               }
            }
            else
            {
               IPointerHandle icon(window->icon());
               ITRACE_ALL( IString("drawing icon=") +
                           IString((unsigned long)(IPointerHandle::Value)icon).d2x() );
               if ((IPointerHandle::Value)icon != 0)
               {
                  IPointerData* node = IBitmapStaticPtr::pointerSet().find( icon );
                  if (node)
                  {
                     // Construct the color image.
                     bitmapImage = new IGImage( node->handle() );
                     bitmapRect = bitmapImage->rasterBounds();
#if 0 //SAS - The mask infrmatin is in Image and will be taken care at render
                     if ( node->maskImage() != 0 )   //mask pixmap invalid if no XImage
                     {
                        // Construct the mask image.
                        IBitmapHandle maskbmp( node->maskPixmap() );
                        maskImage = new IGImage( maskbmp );
                        maskImage->setTransparencyColor(
                           IBaseColor( IBaseColor::kWhite ) );
                        maskImage->setTransparencyFlag ( true );
                     }
#endif //SAS
                  }
               }
            }

            if ( bitmapRect != IGRect2D() )
            {
               IGRect2D cacheRect( cache->rasterBounds() );

               IGRect2D targetRect;
               // Adjust for size to window.
               if ( window->isSizeImageToWindowEnabled() )
               {
                  // Resize bitmap to fit the entire window.
                  Dimension shadowThickness(0), borderWidth(0),
                            highlightThickness(0),
                            defaultButtonShadowThickness(0), showAsDefault(0);

                  XtVaGetValues(window->handle(),
                                XmNshadowThickness, &shadowThickness,
                                XmNborderWidth,     &borderWidth,
                                XmNhighlightThickness, &highlightThickness,
                                XmNdefaultButtonShadowThickness, &defaultButtonShadowThickness,
                                XmNshowAsDefault, &showAsDefault,
                                NULL);


                  // Hack.  There is a private value Xm3D_ENHANCE_PIXEL with
                  // value of 2 which is added to the highlight thickness
                  // value in XmPushButton.  This occurs ONLY if
                  // defaultButtonShadowThickness is != 0.  There are also
                  // some differences in the button size when there is
                  // no default button shadow at all.
                  int enhance3D   ( defaultButtonShadowThickness ? 2 : 0 );
                  int defThickness( enhance3D ?
                                    defaultButtonShadowThickness : showAsDefault);
                  int edges;
                  if (defThickness)
                     edges = 2*(shadowThickness + defThickness) +
                             highlightThickness + enhance3D + borderWidth;
                  else
                     edges =  shadowThickness + highlightThickness + borderWidth;
                  edges = edges * 2;

                  IGPoint2D margins = IPoint(window->marginSize());
                  // Compute new size for image.  Minimum size is 1,1.
                  IGPoint2D newSize( cacheRect.size() -
                                    IGPoint2D(edges,edges) - margins * 2);
                  if (newSize.fX < 1) newSize.fX = 1;
                  if (newSize.fY < 1) newSize.fY = 1;
                  targetRect = IGRect2D( IGPoint2D(0,0), newSize );
               }
               else
               {
                  targetRect = bitmapRect;
               }

               // Center the bitmap in the area allotted for it.
               targetRect.setCenter( cacheRect.center() );
               if (cacheRect != targetRect )
               {
                  //Have to fill area not occupied by bitmap.
                  IFillBundle fillBundle( window->backgroundColor() );
                  cache->grafPort()->draw( cacheRect, fillBundle );
               }

               IGrafBundle bundle;
               IGrafMatrix matrix( bitmapRect, targetRect );
               if (maskImage)
                  cache->grafPort()->draw( *bitmapImage, *maskImage, bundle, matrix );
               else
                  cache->grafPort()->draw( *bitmapImage, bundle, matrix );

               result = true;
            }

            if (bitmapImage)
               delete bitmapImage;
            if (maskImage)
               delete maskImage;

            if (result)
            {
               ITRACE_ALL("Setting widget values");
               //Save the cached image and provide its pixmap to the widget.
               drawData->adoptImage( drawData->buttonState(), cache);
               IBitmapHandle widgetbmap(
                  drawData->imageHandle( drawData->buttonState() ) );
               //Get correct pixmap for enabled/disabled widget.
               Pixmap backgroundPixmap(
                   window->isEnabled()  ?
                   (IBitmapHandle::Value) widgetbmap :
                   IBitmapStaticPtr::insensitivePixmap( widgetbmap ) );
               XtVaSetValues(
                  window->handle(),
                  XmNbackgroundPixmap, backgroundPixmap,
                  0);
               refreshWidget = false;
            }
            else
            {
               // Not processed.  Delete the cache IGImage.
               delete cache;
               if (refreshWidget)
               {
                  // Reset widget information if we invalidated the cache.
                  ITRACE_ALL("Resetting widget values");
                  XtVaSetValues(window->handle(),
                                XmNbackgroundPixmap, XmUNSPECIFIED_PIXMAP,
                                0);
               }
            }
         }
         else
         {
            //Window 0 sized.  All done.
            result = true;
            if (refreshWidget)
            {
               // Reset widget information if we invalidated the cache.
               ITRACE_ALL("Resetting widget values");
               XtVaSetValues(window->handle(),
                             XmNbackgroundPixmap, XmUNSPECIFIED_PIXMAP,
                             0);
            }
         }
         if (refreshWidget)
         {
            // Force the widget to update itself by changing the background
            // color.  Appears to  be a bug in XmPushButton when setting
            // background pixmap to XmUNSPECIFIED_PIXMAP.
            Pixel  realBackground(0), dummy(0);
            XtVaGetValues( window->handle(), XmNbackground, &realBackground, 0);
            if (realBackground == dummy)
               dummy = 1;     // Have to have a different value.
            XtVaSetValues( window->handle(), XmNbackground, dummy, 0);
            XtVaSetValues( window->handle(), XmNbackground, realBackground, 0);
         }
      }
   }
   return result;
}
#endif //IC_MOTIF
