/* NOSHIP */
// Revision: 82 1.3.1.4 source/ui/basectl/igrszhdr.hpp, buttonctls, ioc.v400, 000919 
#ifndef _IGRSZHDR_
  #define _IGRSZHDR_
/*******************************************************************************
* FILE NAME: igrszhdr.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IGraphicPushButtonResizeHandler                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <isizehdr.hpp>
#include <isizeevt.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IEvent;

#pragma enum(4)
#pragma pack(push,4)

class IGraphicPushButtonResizeHandler : public IResizeHandler {
typedef IResizeHandler
  Inherited;
/*******************************************************************************
* The IGraphicPushButtonResizeHandler class handles the notification that an   *
* IGraphicPushButton has been resized.                                         *
*                                                                              *
* A resizing event notifies a window or control that it has grown or shrunk    *
* on the screen.  If the IGraphicPushButtonResizeHandler object is attached to *
* an IGraphicPushButton control, it will be called to process the resizing event.*
*                                                                              *
* IGraphicPushButtonResizeHandler processes the resizing event by creating an  *
* IResizeEvent object and routing it to the virtual windowResize function.  This*
* virtual function automatically sizes the drawing window that makes up the    *
* IGraphicPushButton control.
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is by using the default    |
| constructor, which takes no arguments.                                       |
------------------------------------------------------------------------------*/
  IGraphicPushButtonResizeHandler       ( );
virtual
 ~IGraphicPushButtonResizeHandler       ( );

/*-------------------------------- Drawing -------------------------------------
  drawBackground - draws the background area of the control, such as any
                   image or fill color.
------------------------------------------------------------------------------*/
#ifdef IC_MOTIF
virtual bool
  drawBackground ( IGraphicPushButton*      button);
#endif //IC_MOTIF

protected:
/*---------------------------- Event Dispatching -------------------------------
| We override dispatchHandlerEvent to process messages not usually             |
| handled by a resize handler.                                                 |
------------------------------------------------------------------------------*/
#ifdef IC_MOTIF
virtual bool
  dispatchHandlerEvent ( IEvent& event );
#endif

/*----------------------------- Event Processing -------------------------------
|   windowResize - This handles the resizing event, by sizing the drawing      |
|                  window portion of the IGraphicPushButton.                   |
------------------------------------------------------------------------------*/
virtual bool
  windowResize         ( IResizeEvent& event );
}; // IGraphicPushButtonResizeHandler

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _IGRSZHDR_ */
