// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 83 1.4 source/ui/basectl/ihitdtec.cpp, slidectls, ioc.v400, 000919 


#ifndef IC_HIT_DETECT
#include <ihitdtec.hpp>
#endif

#ifndef IC_GRAPHICEXCEPTION
#include <igrexept.hpp>
#endif

#include <assert.h>

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IHITDTEC_CPP_
    #include <ipagetun.h>
#endif



IHitGraphicGroupIterator::IHitGraphicGroupIterator (
                            const IHitGraphicGroup *  hitGroup)
  : IGraphicGroupIterator (hitGroup),
    fRect (),
    fMode (kAll)
{
}


IHitGraphicGroupIterator::IHitGraphicGroupIterator (
                            const IHitGraphicGroup *  hitGroup,
               const IGPoint2D &         point)
  : IGraphicGroupIterator (hitGroup),
    fRect (point, point),
    fMode (kHitRectangle)
{
}


IHitGraphicGroupIterator::IHitGraphicGroupIterator (
                            const IHitGraphicGroup *  hitGroup,
               const IGRect2D &          rect)
  : IGraphicGroupIterator (hitGroup),
    fRect (rect),
    fMode (kHitRectangle)
{
}


IHitGraphicGroupIterator::~IHitGraphicGroupIterator ()
{
}


IMGraphic *  IHitGraphicGroupIterator::first ()
{
  bool         done = false;
  IMGraphic *  dummy =  IGraphicGroupIterator::first ();
  IMGraphic *  retval = (IMGraphic *) dummy;

  while (! done && (retval != 0))
  {
    switch (fMode)
    {
      case kAll:
   done = true;
   break;


      case kHitRectangle:
   if ((retval->isHitEnabled ()) &&
       (retval->intersects (fRect) == true))
   {
     done = true;
   }
   break;


      default:             // Should never come here.
   break;

    } // switch

    if (! done)
    {
      dummy =  IGraphicGroupIterator::next ();
      retval = (IMGraphic *) dummy;
    }

  } // while not done

  return (retval);

} // IHitGraphicGroupIterator::first


IMGraphic *  IHitGraphicGroupIterator::last ()
{
  bool         done = false;
  IMGraphic *  dummy =  IGraphicGroupIterator::last ();
  IMGraphic *  retval = (IMGraphic *) dummy;

  while (! done && (retval != 0))
  {
    switch (fMode)
    {
      case kAll:
   done = true;
   break;


      case kHitRectangle:
   if ((retval->isHitEnabled ()) &&
       (retval->intersects (fRect) == true))
   {
     done = true;
   }
   break;


      default:             // Should never come here.
   break;

    } // switch

    if (! done)
    {
      dummy =  IGraphicGroupIterator::previous ();
      retval = (IMGraphic *) dummy;
    }

  } // while not done

  return (retval);

} // IHitGraphicGroupIterator::last


IMGraphic *  IHitGraphicGroupIterator::next ()
{
  bool         done = false;
  IMGraphic *  dummy =  IGraphicGroupIterator::next ();
  IMGraphic *  retval = (IMGraphic *) dummy;

  while (! done && (retval != 0))
  {
    switch (fMode)
    {
      case kAll:
   done = true;
   break;


      case kHitRectangle:
   if ((retval->isHitEnabled ()) &&
       (retval->intersects (fRect) == true))
   {
     done = true;
   }
   break;


      default:             // Should never come here.
   break;

    } // switch

    if (! done)
    {
      dummy =  IGraphicGroupIterator::next ();
      retval = (IMGraphic *) dummy;
    }

  } // while not done

  return (retval);

} // IHitGraphicGroupIterator::next



IMGraphic *  IHitGraphicGroupIterator::previous ()
{
  bool         done = false;
  IMGraphic *  dummy =  IGraphicGroupIterator::previous ();
  IMGraphic *  retval = (IMGraphic *) dummy;

  while (! done && (retval != 0))
  {
    switch (fMode)
    {
      case kAll:
   done = true;
   break;


      case kHitRectangle:
   if ((retval->isHitEnabled ()) &&
       (retval->intersects (fRect) == true))
   {
     done = true;
   }
   break;


      default:             // Should never come here.
   break;

    } // switch

    if (! done)
    {
      dummy =  IGraphicGroupIterator::previous ();
      retval = (IMGraphic *) dummy;
    }

  } // while not done

  return (retval);

} // IHitGraphicGroupIterator::previous





IHitGraphicGroup::IHitGraphicGroup ()
  : IGraphicGroup ()
{
}


IHitGraphicGroup::IHitGraphicGroup (const IHitGraphicGroup &  src)
  : IGraphicGroup (src)
{
}


IHitGraphicGroup::IHitGraphicGroup (IGrafBundle *  bundle)
  : IGraphicGroup (bundle)
{
}


IHitGraphicGroup::~IHitGraphicGroup ()
{
}


IHitGraphicGroup &  IHitGraphicGroup::operator= (const IHitGraphicGroup &  src)
{
  if (&src != this)
  {
    IGraphicGroup::operator= (src);
  }
  return *this;
}


bool IHitGraphicGroup::operator== (const IMDrawable&  other) const
{
  return IGraphicGroup::operator== (other);
}


bool IHitGraphicGroup::operator!= (const IMDrawable&  other) const
{
  return ! operator== (other);
}


void  IHitGraphicGroup::adoptBefore (const IMGraphic &  befObj,
                          IMGraphic *  newObj)
{
  IGraphicGroup             tempGp;
  IGraphicGroupIterator     iterator (this);
  IMGraphic *               obj = iterator.last ();

  while (obj && (*obj != befObj))
  {
    tempGp.adoptFirst (IGraphicGroup::orphanLast ());
    obj = iterator.previous ();
  }
  IGraphicGroup::adoptLast (newObj);
  IGraphicGroupIterator  cursor (&tempGp);
  for (obj = cursor.first (); obj; obj = cursor.next ())
  {
    IGraphicGroup::adoptLast (tempGp.orphanFirst ());
  }

} // IHitGraphicGroup::adoptBefore


void  IHitGraphicGroup::adoptAfter (const IMGraphic &  aftObj,
                         IMGraphic *  newObj)
{
  IGraphicGroup          tempGp;
  IGraphicGroupIterator  iterator (this);
  IMGraphic *            obj = iterator.first ();

  while (obj && (*obj != aftObj))
  {
    tempGp.adoptLast (IGraphicGroup::orphanFirst ());
    obj = iterator.next ();
  }
  IGraphicGroup::adoptFirst (newObj);
  IGraphicGroupIterator  cursor (&tempGp);
  for (obj = cursor.last (); obj; obj = cursor.previous ())
  {
    IGraphicGroup::adoptFirst (tempGp.orphanLast ());
  }

} // IHitGraphicGroup::adoptAfter


const IMGraphic *
IHitGraphicGroup::bottomGraphicUnderPoint (const IGPoint2D &  point)
{
  IHitGraphicGroupIterator    iterator (this, point);
  IMGraphic *  retval = iterator.last ();
  return retval;

} // IHitGraphicGroup::bottomGraphicUnderPoint


const IMGraphic *
IHitGraphicGroup::topGraphicUnderPoint (const IGPoint2D &  point)
{
  IHitGraphicGroupIterator    iterator (this, point);
  IMGraphic *  retval = iterator.first ();
  return retval;

} // IHitGraphicGroup::topGraphicUnderPoint



IKeyGraphicPair::IKeyGraphicPair ()
  : fKey (0),
    fObj (0)
{
}


IKeyGraphicPair::~IKeyGraphicPair ()
{
  // Don't free any memory here.
}


IKeyGraphicPair &  IKeyGraphicPair::operator= (const IKeyGraphicPair &  src)
{
  if (&src != this)
  {
    fKey = src.fKey;
    fObj = src.fObj;
  }
  return (*this);
}


bool IKeyGraphicPair::operator== (const IKeyGraphicPair & other) const
{
  // intentionally stubbed out.
  return false;
}


IDataStream &  IKeyGraphicPair::operator>>= (IDataStream & toWhere) const
{
  // intentionally stubbed out.
  return toWhere;
}


IDataStream &  IKeyGraphicPair::operator<<= (IDataStream & fromWhere)
{
  // intentionally stubbed out.
  return fromWhere;
}
