// Revision: 94 1.7.1.4 source/ui/basectl/ilistbo1.cpp, listctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ilistbo1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilistbnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINLISTBOXES

extern "C" {
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/List.h>
  #include <Xm/ScrolledW.h>
#endif //IC_MOTIF

#include <iwindow.hpp>
#include <ilistbox.hpp>
#include <ilistbnh.hpp>
#include <ievent.hpp>
#include <ictlevt.hpp>
#include <inotifev.hpp>
#include <icconst.h>

#ifndef IC_MOTIF
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILISTBO1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const IListBox::addId    = "IListBox::add";
INotificationId const IListBox::removeId = "IListBox::remove";
INotificationId const IBaseListBox::selectId = "IBaseListBox::select";
INotificationId const IBaseListBox::enterId  = "IBaseListBox::enter";
INotificationId const IBaseListBox::itemChangedId  = "IBaseListBox::itemChangedId";
INotificationId const IBaseListBox::itemsId  = "IBaseListBox::itemsId";
INotificationId const IBaseListBox::extendedSelectChangedId  = "IBaseListBox::extendedSelectChangedId";


/*------------------------------------------------------------------------------
| IBaseListBox::enableNotification                                                 |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IBaseListBox, does not send notifications to observer objects until it is        |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new IListBoxNotifyHandler() );
  }
  IWindow::enableNotification(enabled);

  return (*this);
}

/*------------------------------------------------------------------------------
| IListBoxNotifyHandler::IListBoxNotifyHandler                                 |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IListBoxNotifyHandler::IListBoxNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IListBoxNotifyHandler::~IListBoxNotifyHandler                                |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IListBoxNotifyHandler::~IListBoxNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IListBoxNotifyHandler::dispatchHandlerEvent                                  |
|                                                                              |
| This notifies IBaseListBox Windows of the following events:                      |
|                                                                              |
| select(Id)                                                                   |
------------------------------------------------------------------------------*/
bool IListBoxNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
#ifdef IC_PMWIN
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
        if (className == WC_LISTBOX)
        {
#ifdef IC_WIN
          bool isMultiple = ((IBaseListBox*)pwndControl)->isMultipleSelect(),
                  isExtended = ((IBaseListBox*)pwndControl)->isExtendedSelect();

          if (ulMsg == IC_UM_LN_SELECT ||
             (ulMsg == LN_SELECT &&
               (!isMultiple || (isMultiple && isExtended))))
#else
          if (ulMsg == LN_SELECT)
#endif
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseListBox::selectId,
                                        *thisWindow));
          }
          else if (ulMsg == LN_ENTER)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseListBox::enterId,
                                        *thisWindow));
          }
        }
      }
    break;
    } /* endcase */

#ifdef IC_WIN
    case WM_CHAR:
    {
      if ((unsigned long)anEvent.parameter1() == VK_RETURN)
      {
        IWindow* thisWindow = anEvent.window();
        if (thisWindow)
        {
          thisWindow->notifyObservers(INotificationEvent(
                                      IBaseListBox::enterId,
                                      *thisWindow));
        }
      }
      else if ((unsigned long)anEvent.parameter1() == VK_SPACE)
      {
         IBaseListBox *listBox = (IBaseListBox*)(anEvent.controlWindow());
         IWindow* thisWindow = anEvent.window();
         if (listBox->isMultipleSelect() && !listBox->isExtendedSelect())
         {
            // Since Windows sends LBN_SELCHANGE notifications on multiple
            // select listboxes whenever the user presses the arrow keys,
            // we need to handle the key/mouse events responsible for
            // selection.  This will ensure correct behavior of ISelectHandler
            // and selectId notifications.
            thisWindow->notifyObservers( INotificationEvent(
                                         IBaseListBox::selectId,
                                         *listBox ));
         }
      }
    break;
    }

    case WM_LBUTTONUP:
    {
      IBaseListBox *listBox = (IBaseListBox*)(anEvent.controlWindow());
      IWindow* thisWindow = anEvent.window();
      if (listBox->isMultipleSelect() && !listBox->isExtendedSelect())
      {
         thisWindow->notifyObservers( INotificationEvent(
                                      IBaseListBox::selectId,
                                      *listBox ));
      }
    break;
    }
#endif

    default:
      break;
  }  /* end switch */
#endif // IC_PMWIN

#ifdef IC_MOTIF
     switch ( anEvent.eventId() )
     {
       case motifEvent(XmCR_EXTENDED_SELECT):
       case motifEvent(XmCR_MULTIPLE_SELECT):
       case motifEvent(XmCR_BROWSE_SELECT):
       {
         IControlEvent ctlevt(anEvent);
         IWindow* pwndControl = ctlevt.controlWindow();
         IWindow* thisWindow;
         IWindowHandle hwndControl;

         if (pwndControl)
         {
           hwndControl = pwndControl->handle();
           thisWindow = pwndControl;
         }
         else
         {
           hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                              ctlevt.handle());
           thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
         }
         if ( (hwndControl) && (thisWindow) )
         {
           if ( XtIsSubclass((Widget)hwndControl,xmListWidgetClass) ||
                XtIsSubclass((Widget)hwndControl,xmScrolledWindowWidgetClass))
           {
             thisWindow->notifyObservers(INotificationEvent(
               IBaseListBox::selectId, *thisWindow));
           }
         }
       break;
       }
     }  /* end switch */
#endif //IC_MOTIF

  return (Inherited::dispatchHandlerEvent(anEvent));
}
