// Revision: 09 1.8 source/ui/basectl/imcelcvl.hpp, canvas, ioc.v400, 000919 
/* NOSHIP */
#ifndef _IMCELCVL_
#define _IMCELCVL_
/*******************************************************************************
* FILE NAME: imcelcvl.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IMultiCellCanvasList                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ikeyset2.h>
#include <imcelcvc.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IMCCvCell;
class IWindow;

#pragma enum(4)
#pragma pack(push,4)

class IMultiCellCanvasList : public IVPtrKeySet<IMCCvCell*,
                                            unsigned long>
{
/*******************************************************************************
* The IMultiCellCanvasList class is a dictionary to hold the Cells. Note that  *
* this collection contains both cell records and column and row heading        *
* records.                                                                     *
* Cell records start from (1, 1) and heading records are before these (e.g.    *
* (0, 1) or (1, 0) ). This means that all the records in a particular column   *
* can be iterated by putting the column number in the first part of the key    *
* and incrementing the second part of the key up to the number of rows in the  *
* canvas.                                                                      *
*******************************************************************************/
public:

  IMultiCellCanvasList    ( );

  IMultiCellCanvasList :: ~IMultiCellCanvasList( );

/*-------------------------------- Accessors -----------------------------------
| These functions provide a means of getting and setting the accessible        |
| attributes of instances of this class:                                       |
|  operator()    -  Returns a cell in the multi-cell canvas:                   |
|                     1) From the column and row                               |
|                     2) From the IWindow stored in the cell.                  |
|  updateHeaders -  Ensures the header records exist for the number of cells.  |
|                                                                              |
|    Note: The Canvas collection has a "null" cell that is used for unfilled   |
|          cells in the collection.  IMultiCellCanvasCell::isEmpty() can be    |
|          used to determine if the cell returned is the "null" cell.          |
------------------------------------------------------------------------------*/
IMCCvCell
 &operator()        ( unsigned long column, unsigned long row),
 &operator()        ( const IWindow* childWindow);

}; /* end class IMultiCellCanvasList */

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IMCELCVL_
