// Revision: 21 1.5 source/ui/basectl/inotebk0.cpp, notebook, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: inotebk0.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     INotebook::IPageSettings                                                 *
*     INotebook::Cursor                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINSTDBOOK
  #define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
  #include <iwindefs.h>
}

#ifdef IC_WIN
  #ifndef _ICLNOTEBOOKW_
  #include <iclnbw.h>
  #endif
#endif
#include <inotebk.hpp>
#include <inotebk0.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTEBK0_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_WIN
/*------------------------------------------------------------------------------
| INotebookPageSettingsData::INotebookPageSettingsData                         |
|                                                                              |
| This class contains the tab control item structure.                          |
------------------------------------------------------------------------------*/
INotebookPageSettingsData::INotebookPageSettingsData()
{ }

/*------------------------------------------------------------------------------
| INotebookPageSettingsData::~INotebookPageSettingsData                        |
|                                                                              |
| Destructor here for page tuning.                                             |
------------------------------------------------------------------------------*/
INotebookPageSettingsData::~INotebookPageSettingsData()
{ }
#endif


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using the default values.                |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( )
  : fPageSettingsData( 0 )
  , savedStatusText(IString ())
  , savedTabText(IString ())
  , pageStyle(0)
  , savedUserData(0)
{
#ifdef IC_WIN
  fPageSettingsData = new INotebookPageSettingsData();
  fPageSettingsData->tabCtrlItem.itemHeader.mask = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.pszText = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.cchTextMax = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.iImage = -1;
  fPageSettingsData->tabCtrlItem.hPageWindow = 0;
  fPageSettingsData->tabCtrlItem.hPageWindowParent = 0;
  fPageSettingsData->tabCtrlItem.ulPageStyle = 0;
  fPageSettingsData->tabCtrlItem.ulUserData = 0;
#endif
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using one or more page attributes.       |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( const Attribute& newPageStyle )
  : fPageSettingsData( 0 )
  , savedStatusText(IString ())
  , savedTabText(IString ())
  , pageStyle(newPageStyle)
  , savedUserData(0)
{
#ifdef IC_WIN
  fPageSettingsData = new INotebookPageSettingsData();
  fPageSettingsData->tabCtrlItem.itemHeader.mask = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.pszText = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.cchTextMax = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.iImage = -1;
  fPageSettingsData->tabCtrlItem.hPageWindow = 0;
  fPageSettingsData->tabCtrlItem.hPageWindowParent = 0;
  fPageSettingsData->tabCtrlItem.ulPageStyle = newPageStyle.asUnsignedLong();
  fPageSettingsData->tabCtrlItem.ulUserData = 0;
#endif // IC_WIN
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using the tab text, status text, and     |
| one or more attributes.                                                      |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( const char* tabText,
                                          const char* statusText,
                                          const Attribute& newPageStyle )
  : fPageSettingsData( 0 )
  , savedStatusText(IString (statusText))
  , pageStyle(newPageStyle)
  , savedTabText(IString (tabText))
  , savedUserData(0)
{
#ifdef IC_WIN
  fPageSettingsData = new INotebookPageSettingsData();
#endif // IC_WIN
  if (statusText)                                     // Make sure attribute is
    pageStyle |= (PageSettings::statusTextOn);        // on if text specified

  if (tabText)
  {                                                   // Make sure attribute is
    if ((!(pageStyle & (PageSettings::majorTab))) &&  // on if tab text is
        (!(pageStyle & (PageSettings::minorTab))))    // specified
      pageStyle |= (PageSettings::majorTab);
  }

#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem.itemHeader.mask = TCIF_TEXT;
  fPageSettingsData->tabCtrlItem.itemHeader.pszText = this->savedTabText;
  fPageSettingsData->tabCtrlItem.itemHeader.cchTextMax = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.iImage = -1;
  fPageSettingsData->tabCtrlItem.hPageWindow = 0;
  fPageSettingsData->tabCtrlItem.hPageWindowParent = 0;
  fPageSettingsData->tabCtrlItem.ulPageStyle = pageStyle.asUnsignedLong();
  fPageSettingsData->tabCtrlItem.ulUserData = 0;
#endif
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using the tab bitmap, status text,       |
| and one or more attributes.                                                  |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( const IBitmapHandle& tabBitmap,
                                          const char* statusText,
                                          const Attribute& newPageStyle )
  : fPageSettingsData( 0 )
  , savedStatusText(IString (statusText))
  , pageStyle(newPageStyle)
  , savedTabText(IString ())
  , savedTabBitmap(tabBitmap)
  , savedUserData(0)
{
#ifdef IC_WIN
  fPageSettingsData = new INotebookPageSettingsData();
#endif // IC_WIN
  if (statusText)                                     // Make sure attribute is
    pageStyle |= (PageSettings::statusTextOn);        // on if text specified

  if (tabBitmap)
  {                                                   // Make sure attribute is
    if ((!(pageStyle & (PageSettings::majorTab))) &&  // on if tab text is
        (!(pageStyle & (PageSettings::minorTab))))    // specified
      pageStyle |= (PageSettings::majorTab);
  }

#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem.itemHeader.mask = TCIF_IMAGE;
  fPageSettingsData->tabCtrlItem.itemHeader.pszText = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.cchTextMax = 0;
  fPageSettingsData->tabCtrlItem.itemHeader.iImage = -1;
  fPageSettingsData->tabCtrlItem.hPageWindow = 0;
  fPageSettingsData->tabCtrlItem.hPageWindowParent = 0;
  fPageSettingsData->tabCtrlItem.ulPageStyle = pageStyle.asUnsignedLong();
  fPageSettingsData->tabCtrlItem.ulUserData = 0;
#endif
}

/*------------------------------------------------------------------------------
| INotebook::PageSettings::~PageSettings                                       |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
INotebook :: PageSettings :: ~PageSettings ( )
{
#ifdef IC_WIN
  if (fPageSettingsData)
    delete fPageSettingsData;
#endif
}

/*------------------------------------------------------------------------------
| INotebook::PageSettings::PageSettings                                        |
|                                                                              |
| Copy constructor.                                                            |
------------------------------------------------------------------------------*/
INotebook::PageSettings ::
     PageSettings (const INotebook::PageSettings& pageSettings)
  : fPageSettingsData( 0 )
  , pageStyle      (pageSettings.pageStyle)
  , savedTabBitmap (pageSettings.savedTabBitmap)
  , savedUserData  (pageSettings.savedUserData)
  , savedStatusText(pageSettings.savedStatusText)
  , savedTabText   (pageSettings.savedTabText )
{
#ifdef IC_WIN
  fPageSettingsData = new INotebookPageSettingsData();
  fPageSettingsData->tabCtrlItem = pageSettings.fPageSettingsData->tabCtrlItem;
  fPageSettingsData->tabCtrlItem.hPageWindow =
                     pageSettings.fPageSettingsData->tabCtrlItem.hPageWindow;
  fPageSettingsData->tabCtrlItem.hPageWindow =
               pageSettings.fPageSettingsData->tabCtrlItem.hPageWindowParent;
  fPageSettingsData->tabCtrlItem.ulPageStyle =
                     pageSettings.fPageSettingsData->tabCtrlItem.ulPageStyle;
  fPageSettingsData->tabCtrlItem.ulUserData =
                     pageSettings.fPageSettingsData->tabCtrlItem.ulUserData;
#endif
}

/*------------------------------------------------------------------------------
| INotebook::PageSettings::PageSettings                                        |
|                                                                              |
| Assignment operator.                                                         |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                      operator = (const INotebook::PageSettings& pageSettings)
{
  pageStyle       = pageSettings.pageStyle;
  savedTabBitmap  = pageSettings.savedTabBitmap;
  savedUserData   = pageSettings.savedUserData;
  savedStatusText = pageSettings.savedStatusText;
  savedTabText    = pageSettings.savedTabText;

#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem = pageSettings.fPageSettingsData->tabCtrlItem;
  fPageSettingsData->tabCtrlItem.hPageWindow =
                     pageSettings.fPageSettingsData->tabCtrlItem.hPageWindow;
  fPageSettingsData->tabCtrlItem.hPageWindow =
               pageSettings.fPageSettingsData->tabCtrlItem.hPageWindowParent;
  fPageSettingsData->tabCtrlItem.ulPageStyle =
                     pageSettings.fPageSettingsData->tabCtrlItem.ulPageStyle;
  fPageSettingsData->tabCtrlItem.ulUserData =
                     pageSettings.fPageSettingsData->tabCtrlItem.ulUserData;
#endif
  return( *this );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setStatusText                                       |
|                                                                              |
| Store the status text in the PageSetting object's instance data.             |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                                     setStatusText ( const char* text )
{
  ITRACE_WIN_NOP();

  pageStyle |= (PageSettings::statusTextOn);
  savedStatusText = IString (text);
#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem.ulPageStyle = pageStyle.asUnsignedLong();
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setStatusText                                       |
|                                                                              |
| Store the status text in the PageSetting object's instance data.             |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                           setStatusText ( const IResourceId& resId )
{
  ITRACE_WIN_NOP();

  return setStatusText(resId.resourceLibrary().loadString (resId.id()));
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabText                                          |
|                                                                              |
| Store the tab text in the PageSetting object's instance data.                |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                                      setTabText ( const char* text )
{
  savedTabText = IString (text);

#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem.itemHeader.mask |= TCIF_TEXT;
  fPageSettingsData->tabCtrlItem.itemHeader.pszText = savedTabText;
#endif

  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabText                                          |
|                                                                              |
| Store the tab text in the PageSetting object's instance data.                |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                              setTabText ( const IResourceId& resId )
{
  return setTabText(resId.resourceLibrary().loadString (resId.id()));
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabBitmap                                        |
|                                                                              |
| Store the tab bitmap in the PageSetting object's instance data.              |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                         setTabBitmap ( const IBitmapHandle& bitmap )
{
#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem.itemHeader.mask |= TCIF_IMAGE;
  fPageSettingsData->tabCtrlItem.itemHeader.iImage = -1;
#endif

  savedTabBitmap = bitmap;
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabBitmap                                        |
|                                                                              |
| Store the tab bitmap in the PageSetting object's instance data.              |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                            setTabBitmap ( const IResourceId& resId )
{
  return setTabBitmap(resId.resourceLibrary().loadBitmap (resId.id()));
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setUserData                                         |
|                                                                              |
| Store the user data in the PageSetting object's instance data.               |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                                 setUserData ( unsigned long ulData )
{
  savedUserData = ulData;

#ifdef IC_WIN
  fPageSettingsData->tabCtrlItem.itemHeader.mask |= TCIF_PARAM;
  fPageSettingsData->tabCtrlItem.ulUserData = savedUserData;
#endif

  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::statusText                                          |
|                                                                              |
| Return the text of the status line associated with this page.                |
------------------------------------------------------------------------------*/
IString INotebook::PageSettings :: statusText ( ) const
{
  return savedStatusText;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::tabText                                             |
|                                                                              |
| Return the text of the tab associated with this page.                        |
------------------------------------------------------------------------------*/
IString INotebook::PageSettings :: tabText ( ) const
{
  return savedTabText;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::tabBitmap                                           |
|                                                                              |
| Return the bitmap of the tab associated with this page.                      |
------------------------------------------------------------------------------*/
IBitmapHandle INotebook::PageSettings :: tabBitmap ( ) const
{
  return savedTabBitmap;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::userData                                            |
|                                                                              |
| Return the application data in the page's reserved storage.                  |
------------------------------------------------------------------------------*/
unsigned long INotebook::PageSettings :: userData ( ) const
{
  return savedUserData;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isAutoSize                                          |
------------------------------------------------------------------------------*/
bool INotebook::PageSettings :: isAutoSize ( ) const
{
  return( (pageStyle & PageSettings::autoPageSize) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isStatusTextOn                                      |
------------------------------------------------------------------------------*/
bool INotebook::PageSettings :: isStatusTextOn ( ) const
{
  ITRACE_WIN_NOP();

  return( (pageStyle & PageSettings::statusTextOn) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isMajorTab                                          |
|                                                                              |
| Query whether this page has a major tab.                                     |
------------------------------------------------------------------------------*/
bool INotebook::PageSettings :: isMajorTab ( ) const
{
  return( (pageStyle & PageSettings::majorTab) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isMinorTab                                          |
|                                                                              |
| Query whether this page has a minor tab.                                     |
------------------------------------------------------------------------------*/
bool INotebook::PageSettings :: isMinorTab ( ) const
{
  return( (pageStyle & PageSettings::minorTab) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::Cursor                                                    |
|                                                                              |
| Cursor class constructor                                                     |
------------------------------------------------------------------------------*/
INotebook::Cursor :: Cursor ( const INotebook& notebook )
  : nbCl (notebook)
  , ulClCurrentPageId (0)
  , ulClReferences (0)
{ }

/*------------------------------------------------------------------------------
| INotebook::Cursor::~Cursor                                                   |
|                                                                              |
| Default destructor used for page tuning.                                     |
------------------------------------------------------------------------------*/
INotebook :: Cursor :: ~Cursor ( )
{ }


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToFirst                                                |
|                                                                              |
| Sets the cursor to point to the first page of the notebook.                  |
------------------------------------------------------------------------------*/
bool INotebook::Cursor :: setToFirst ( )
{
  if (!nbCl.isEmpty())
  {
#ifdef IC_WIN
     ulClCurrentPageId = (unsigned long)(void*)nbCl.firstPage();
#endif // IC_WIN
#ifdef IC_MOTIFPM
     ulClCurrentPageId = nbCl.firstPage();
#endif // IC_MOTIFPM
     ulClReferences    = nbCl.ulClValidate;   //Make cursor valid
     return true;
  }
  else
  {
     invalidate();
     return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToNext                                                 |
|                                                                              |
| Sets the cursor to point to the next page of the notebook.                   |
------------------------------------------------------------------------------*/
bool INotebook::Cursor :: setToNext ( )
{
#ifdef IC_WIN
  IPageHandle thisPage ((void*)ulClCurrentPageId);
#endif // IC_WIN
#ifdef IC_MOTIFPM
  IPageHandle thisPage (ulClCurrentPageId);
#endif // IC_MOTIFPM
  IPageHandle nextPage = nbCl.nextPage( thisPage );

  if ( nextPage )
  {
#ifdef IC_WIN
    ulClCurrentPageId = (unsigned long)(void*)nextPage;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    ulClCurrentPageId = nextPage;
#endif // IC_MOTIFPM
    ulClReferences    = nbCl.ulClValidate;   //Make cursor valid
    return true;
  }
  else
  {
    invalidate();
    return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToPrevious                                             |
|                                                                              |
| Sets the cursor to point to the previous page in the notebook.               |
------------------------------------------------------------------------------*/
bool INotebook::Cursor :: setToPrevious ( )
{
#ifdef IC_WIN
  IPageHandle thisPage ((void*)ulClCurrentPageId);
#endif // IC_WIN
#ifdef IC_MOTIFPM
  IPageHandle thisPage (ulClCurrentPageId);
#endif // IC_MOTIFPM
  IPageHandle prevPage = nbCl.previousPage( thisPage );

  if ( prevPage )
  {
#ifdef IC_WIN
    ulClCurrentPageId = (unsigned long)(void*)prevPage;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    ulClCurrentPageId = prevPage;
#endif // IC_MOTIFPM
    ulClReferences    = nbCl.ulClValidate;   //Make cursor valid
    return true;
  }
  else
  {
    invalidate();
    return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToLast                                                 |
|                                                                              |
| Sets the cursor to point to the last page in the notebook.                   |
------------------------------------------------------------------------------*/
bool INotebook::Cursor :: setToLast ( )
{
  if (!nbCl.isEmpty())
  {
#ifdef IC_WIN
     ulClCurrentPageId = (unsigned long)(void*)nbCl.lastPage();
#endif // IC_WIN
#ifdef IC_MOTIFPM
     ulClCurrentPageId = nbCl.lastPage();
#endif // IC_MOTIFPM
     ulClReferences    = nbCl.ulClValidate;
     return true;
  }
  else
  {
     invalidate();
     return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::first                                                     |
|                                                                              |
| Set the cursor to the first page of the notebook and return an IPageHandle   |
| object referencing the first notebook page.                                  |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: first ( )
{
  IPageHandle returnHandle (0);

  if (setToFirst())
#ifdef IC_WIN
    returnHandle = (void*)ulClCurrentPageId;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    returnHandle = ulClCurrentPageId;
#endif // IC_MOTIFPM

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::last                                                      |
|                                                                              |
| Set the cursor to the last page of the notebook and return an IPageHandle    |
| object referencing the last notebook page.                                   |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: last ( )
{
  IPageHandle returnHandle (0);

  if (setToLast())
#ifdef IC_WIN
    returnHandle = (void*)ulClCurrentPageId;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    returnHandle = ulClCurrentPageId;
#endif

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::next                                                      |
|                                                                              |
| Set the cursor to the next page of the notebook and return an IPageHandle    |
| object referencing that next page.                                           |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: next ( )
{
  IPageHandle returnHandle(0);

  if (setToNext())
#ifdef IC_WIN
    returnHandle = (void*)ulClCurrentPageId;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    returnHandle = ulClCurrentPageId;
#endif

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::previous                                                  |
|                                                                              |
| Set the cursor to the previous page of the notebook and return an            |
| IPageHandle object referencing that previous page.                           |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: previous ( )
{
  IPageHandle returnHandle(0);

  if (setToPrevious())
#ifdef IC_WIN
    returnHandle = (void*)ulClCurrentPageId;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    returnHandle = ulClCurrentPageId;
#endif

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::current                                                   |
|                                                                              |
| Return an IPageHandle referencing the page currently pointed to by the       |
| cursor.                                                                      |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: current ( ) const
{
  IPageHandle currHandle(0);

  if (isValid())
#ifdef IC_WIN
    currHandle = (void*)ulClCurrentPageId;
#endif // IC_WIN
#ifdef IC_MOTIFPM
    currHandle = ulClCurrentPageId;
#endif

  return currHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setCurrent                                                |
|                                                                              |
| Set the cursor to point to the provided page.                                |
------------------------------------------------------------------------------*/
void INotebook::Cursor :: setCurrent ( const IPageHandle& current )
{
#ifdef IC_WIN
  ulClCurrentPageId = (unsigned long)(void*)current;
#endif // IC_WIN
#ifdef IC_MOTIFPM
  ulClCurrentPageId = current;
#endif
  ulClReferences    = nbCl.ulClValidate;   //Make cursor valid
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::isValid                                                   |
|                                                                              |
| Queries whether the cursor points to a valid item.                           |
------------------------------------------------------------------------------*/
bool INotebook::Cursor :: isValid ( ) const
{
  if ((ulClCurrentPageId) && (ulClReferences==nbCl.ulClValidate))
    return true;
  else
    return false;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::invalidate                                                |
|                                                                              |
| Flags this cursor as not valid.                                              |
------------------------------------------------------------------------------*/
void INotebook::Cursor :: invalidate ( )
{
  ulClCurrentPageId = 0;
}
