// Revision: 23 1.8 source/ui/basectl/inotebk1.cpp, notebook, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: inotebk1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in inotebnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINSTDBOOK
#define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
extern "C" {
  #include <iwindefs.h>
}

#include <iwindow.hpp>

#ifdef IC_WIN
  #include <iclnbw.h>
#endif

#include <inotebk.hpp>
#include <inotebnh.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTEBK1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const INotebook::pageBackgroundColorId
                                       = "INotebook::pageBackgroundColor";
INotificationId const INotebook::majorTabBackgroundColorId
                                       = "INotebook::majorTabBackgroundColor";
INotificationId const INotebook::minorTabBackgroundColorId
                                       = "INotebook::minorTabBackgroundColor";
INotificationId const INotebook::majorTabForegroundColorId
                                       = "INotebook::majorTabForegroundColor";
INotificationId const INotebook::minorTabForegroundColorId
                                       = "INotebook::minorTabForegroundColor";
INotificationId const INotebook::orientationId
                                       = "INotebook::orientation";

/*------------------------------------------------------------------------------
| INotebook::enableNotification                                                |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| INotebook, does not send notifications to observer objects until it is       |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IWindow& INotebook::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new INotebookNotifyHandler() );
  }
  return(IWindow::enableNotification(enabled));
}

/*------------------------------------------------------------------------------
| INotebookNotifyHandler::INotebookNotifyHandler                               |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
INotebookNotifyHandler::INotebookNotifyHandler()
{}

/*------------------------------------------------------------------------------
| INotebookNotifyHandler::~INotebookNotifyHandler                              |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
INotebookNotifyHandler::~INotebookNotifyHandler()
{}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| INotebookNotifyHandler::dispatchHandlerEvent                                 |
|                                                                              |
| This notifies INotebook Windows of the following events:                     |
|                                                                              |
| pageBackgroundColor(Id)                                                      |
| majorTabBackgroundColor(Id)                                                  |
| minorTabBackgroundColor(Id)                                                  |
| majorTabForegroundColor(Id)                                                  |
| minorTabForegroundColor(Id)                                                  |
------------------------------------------------------------------------------*/
bool INotebookNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case BKM_SETNOTEBOOKCOLORS:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the selection state has been modified                      */
        /* Put color identifier into parameter 1 for later use        */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter2() ),
                               IEventParameter2( IC_NOTDEL_BOOK_COLOR ));
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* is used to determine which notification id to send           */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_BOOK_COLOR)
        {
          switch (anEvent.parameter1())
          {
            case BKA_BACKGROUNDPAGECOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::pageBackgroundColorId,
                                          *thisWindow));
            break;

            case BKA_BACKGROUNDMAJORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::majorTabBackgroundColorId,
                                          *thisWindow));
            break;

            case BKA_BACKGROUNDMINORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::minorTabBackgroundColorId,
                                          *thisWindow));
            break;

            case BKA_FOREGROUNDMAJORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::majorTabForegroundColorId,
                                          *thisWindow));
            break;

            case BKA_FOREGROUNDMINORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::minorTabForegroundColorId,
                                          *thisWindow));
            break;
          }
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| INotebookNotifyHandler::dispatchHandlerEvent                                 |
|                                                                              |
| This override calls the Inherited function. In Motif, all INotebook specific |
| notifications are generated in the INotebook::setOrientation and             |
| INotebookData::setColor member functions.                                    |
------------------------------------------------------------------------------*/
bool INotebookNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_MOTIF
