// Revision: 25 1.16 source/ui/basectl/ioutlbox.cpp, staticctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ioutlbox.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ioutlbox.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINERRORS
  #define INCL_WINSTATICS
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/DrawingA.h>
  #include <Xm/Separator.h>
  #include <ixconst.h>
  #include <istring.hpp>
#endif // IC_MOTIF

#include <ioutlbox.hpp>
#include <ibidiset.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <inotifev.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <iwindow.hpp>

#ifdef IC_WIN
  #include <icmnfun.hpp>
  #include <iplatfrm.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IOUTLBOX_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public outline box styles.                                                   |
------------------------------------------------------------------------------*/
const IOutlineBox::Style
  IOutlineBox::backgroundFrame   ( 0, IOBS_BACKGROUNDFRAME ),
  IOutlineBox::foregroundFrame   ( 0, IOBS_FOREGROUNDFRAME ),
  IOutlineBox::halftoneFrame     ( 0, IOBS_HALFTONEFRAME ),
#ifdef IC_WIN
  IOutlineBox::border3D          ( 0, WS_EX_STATICEDGE ),
#endif
#ifdef IC_MOTIFPM
  IOutlineBox::border3D          ( 0 ),
#endif
#ifdef IC_WIN
  IOutlineBox::classDefaultStyle ( WS_VISIBLE,
                                   IOBS_FOREGROUNDFRAME | WS_EX_STATICEDGE );
#endif
#ifdef IC_MOTIFPM
  IOutlineBox::classDefaultStyle ( WS_VISIBLE, IOBS_FOREGROUNDFRAME );
#endif

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
IOutlineBox::Style
  IOutlineBox::currentDefaultStyle ( WS_VISIBLE,
                                     IOBS_FOREGROUNDFRAME | WS_EX_STATICEDGE );
#endif
#ifdef IC_MOTIFPM
IOutlineBox::Style
  IOutlineBox::currentDefaultStyle ( WS_VISIBLE, IOBS_FOREGROUNDFRAME );
#endif
#ifdef IC_MOTIF
#pragma enum(4)
#pragma pack(push,4)

class IOutlineBoxData
{
/*------------------------------------------------------------------------------
| The IOutlineBoxData class encapsulates private data and functions used by    |
| the IOutlineBox class.  An object of this class is created in the IOutlineBox|
| constructors.                                                                |
------------------------------------------------------------------------------*/
public:
  IOutlineBoxData ( );

 ~IOutlineBoxData ( );

Widget
  whOutlineBox;
/*------------------------------- Private -----------------------------------*/
private:
  IOutlineBoxData ( const IOutlineBoxData& obd )
  { }
IOutlineBoxData
 &operator=       ( const IOutlineBoxData& obd );
}; // IOutlineBoxData

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IOutlineBoxData::IOutlineBoxData                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IOutlineBoxData::IOutlineBoxData ( )
{ }

/*------------------------------------------------------------------------------
| IOutlineBoxData::~IOutlineBoxData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IOutlineBoxData::~IOutlineBoxData ( )
{ }

void _System iobExposeCallback (_WidgetRec *w,
                                void       *client_data,
                                void       *call_data);

/*------------------------------------------------------------------------------
| IOutlineBox::IOutlineBox                                                     |
|                                                                              |
| Constructor to create an outline box control on a standard window.           |
------------------------------------------------------------------------------*/
IOutlineBox :: IOutlineBox(unsigned long ulId,
                           IWindow* pwndParent,
                           IWindow* pwndOwner,
                           const IRectangle& rectInit,
                           const Style& obsStyle)
  : fOutlineBoxData ( new IOutlineBoxData )
{
  // assertions on input parms
  IASSERTPARM(pwndParent!=0);
  IASSERTPARM((obsStyle & (backgroundFrame | foregroundFrame |
                           halftoneFrame))!=0);

  Arg    args[1];
  int    n = 0;

  // Create the Outline Box.
  XtSetArg( args[n], XmNborderWidth, 1); n++;

  fOutlineBoxData->whOutlineBox = XmCreateDrawingArea(
         pwndParent->handleForChildCreation(),
         (char *)( IString(ulId) ),
         args,
         n);

  if (fOutlineBoxData->whOutlineBox == 0)
  {
     ITHROWGUIERROR("XmCreateDrawingArea");
  }
  else
  {
     startHandlingEventsFor (fOutlineBoxData->whOutlineBox);
     setOwner (pwndOwner);
  }

  // Handle IOutlineBox styles.
  if ( obsStyle != IWindow::noStyle )
  {
     //retrieve owner colors to use for outline box
     IColor bgColor = pwndOwner->backgroundColor();
     IColor fgColor = pwndOwner->foregroundColor();

     // Interpret style flags.
     if (obsStyle & backgroundFrame)
     {
        XtVaSetValues ( fOutlineBoxData->whOutlineBox,
                        XmNborderColor, bgColor.index(),
                        NULL);
     }
     else
     {   // Default is IOutlineBox::foregroundFrame;
         // halftone is treated as foreground.
        XtVaSetValues ( fOutlineBoxData->whOutlineBox,
                        XmNborderColor, fgColor.index(),
                        NULL);
     }
  }
  if ( (obsStyle != IWindow::noStyle) && (obsStyle & IWindow::visible) )
  {
     XtManageChild(fOutlineBoxData->whOutlineBox);
  }
  XtAddCallback (fOutlineBoxData->whOutlineBox,
                 XmNexposeCallback,
                 iobExposeCallback,
                 NULL);
  moveSizeTo(rectInit);
}
#endif // IC_MOTIF

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IOutlineBox::IOutlineBox                                                     |
|                                                                              |
| Constructor to create an outline box control on a standard window.           |
------------------------------------------------------------------------------*/
IOutlineBox :: IOutlineBox(unsigned long ulId,
                           IWindow* pwndParent,
                           IWindow* pwndOwner,
                           const IRectangle& rectInit,
                           const Style& obsStyle)
{
  // assertions on input parms
  IASSERTPARM(pwndParent!=0);
  IASSERTPARM((obsStyle & (backgroundFrame | foregroundFrame |
                           halftoneFrame))!=0);

  // Save the extended style to make sure we have a copy of it stored.
  setExtendedStyle( extendedStyle() | obsStyle.asExtendedUnsignedLong() );

  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (pwndParent)
    parentHandle = pwndParent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();

  IWindowHandle whOutlineBox =
      this -> create( ulId,
                      0,
                      convertToGUIStyle( obsStyle ),
                      WC_STATIC,
                      parentHandle,
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( obsStyle, true ) );

  this->startHandlingEventsFor( whOutlineBox );

#ifdef IC_WIN
  if ( IBidiSettings::isBidiSupported() )
  {  // Explicitly set the outline type.  Not doing this on WinNT 4.0
     // with the WS_EX_RIGHT and/or WS_EX_RTLREADING styles results
     // in corrupted styles, which cause the control to paint its
     // interior.
     this->setOutlineType( this->outlineType() );
  }
#endif
}


/*------------------------------------------------------------------------------
| IOutlineBox::IOutlineBox                                                     |
|                                                                              |
| Constructor to create instance for a template window.                        |
------------------------------------------------------------------------------*/
IOutlineBox :: IOutlineBox(unsigned long ulId,
                           IWindow* pdlgwndParent)
{
  setAutoDestroyWindow( false );
  reserveUserWindowWord( false );
  startHandlingEventsFor(ulId, pdlgwndParent);

#ifdef IC_WIN
  // Save extended style information.
  unsigned long ulStyle = style();
  unsigned long ulExtStyle = extendedStyle();
  if (ulStyle & WS_EX_STATICEDGE)
     ulExtStyle |= border3D.asExtendedUnsignedLong();
  if (ulStyle & SS_BKGNDFRAME)
     ulExtStyle |= backgroundFrame.asExtendedUnsignedLong();
  if (ulStyle & SS_FGNDFRAME)
     ulExtStyle |= foregroundFrame.asExtendedUnsignedLong();
  if (ulStyle & SS_HALFTONEFRAME)
     ulExtStyle |= halftoneFrame.asExtendedUnsignedLong();
  setExtendedStyle (ulExtStyle);
#endif
}


/*------------------------------------------------------------------------------
| IOutlineBox::IOutlineBox                                                     |
|                                                                              |
| Constructor to instantiate from an existing outline box control.             |
------------------------------------------------------------------------------*/
IOutlineBox :: IOutlineBox(const IWindowHandle& wh)
{
  setAutoDestroyWindow( false );
  reserveUserWindowWord( false );
  startHandlingEventsFor(wh);

#ifdef IC_WIN
  // Save extended style information.
  unsigned long ulStyle = style();
  unsigned long ulExtStyle = extendedStyle();
  if (ulStyle & WS_EX_STATICEDGE)
     ulExtStyle |= border3D.asExtendedUnsignedLong();
  if (ulStyle & SS_BKGNDFRAME)
     ulExtStyle |= backgroundFrame.asExtendedUnsignedLong();
  if (ulStyle & SS_FGNDFRAME)
     ulExtStyle |= foregroundFrame.asExtendedUnsignedLong();
  if (ulStyle & SS_HALFTONEFRAME)
     ulExtStyle |= halftoneFrame.asExtendedUnsignedLong();
  setExtendedStyle (ulExtStyle);
#endif
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| IOutlineBox::~IOutlineBox                                                    |
|                                                                              |
| Destructor for cleanup.                                                      |
------------------------------------------------------------------------------*/
IOutlineBox :: ~IOutlineBox( )
{
#ifdef IC_MOTIF
  delete fOutlineBoxData;
#endif
}


/*------------------------------------------------------------------------------
| IOutlineBox::defaultStyle                                                    |
|                                                                              |
| Return the default style for new outline box objects.                        |
------------------------------------------------------------------------------*/
IOutlineBox::Style  IOutlineBox :: defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IOutlineBox::setDefaultStyle                                                 |
|                                                                              |
| Replace the default style for new outline box objects.                       |
------------------------------------------------------------------------------*/
void  IOutlineBox :: setDefaultStyle(const IOutlineBox::Style& obsStyle)
{
  currentDefaultStyle = obsStyle;
}

/*------------------------------------------------------------------------------
| IOutlineBox::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IOutlineBox::convertToGUIStyle(const IBitFlag& guiStyle,
                                             bool bExtOnly) const
{
  // Obtain the style from the class (IControl) that we inherit from.
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range.
    ulStyle |= guiStyle.asExtendedUnsignedLong() & ( IS_EXTMASK | IOB_EXTGUIMASK);
  }
  else
  {
    // Let the clean styles flow thru.
    ulStyle |= guiStyle.asUnsignedLong();

    // Get the extended styles and set the GUI style accordingly ...
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong();

    // Enforce that only one of the outline styles gets set.
    ulStyle &= ~( SS_HALFTONEFRAME | SS_BKGNDFRAME | SS_FGNDFRAME );
    if ( ulTmp & backgroundFrame.asExtendedUnsignedLong() )
    {
       ulStyle |= SS_BKGNDFRAME;
    }
    else if ( ulTmp & foregroundFrame.asExtendedUnsignedLong() )
    {
       ulStyle |= SS_FGNDFRAME;
    }
    else if ( ulTmp & halftoneFrame.asExtendedUnsignedLong() )
    {
       ulStyle |= SS_HALFTONEFRAME;
    }
#ifdef IC_WIN
    ulStyle |= WS_CHILD;
#endif
  }

  return ulStyle;
}


/*------------------------------------------------------------------------------
| IOutlineBox::setOutlineType                                                  |
|                                                                              |
| Sets the type of outline.                                                    |
------------------------------------------------------------------------------*/
IOutlineBox&  IOutlineBox :: setOutlineType(const OutlineType& outlineType)
{
#ifdef IC_PMWIN
  unsigned long ulStyles = style();
  unsigned long ulOldStyles = ulStyles;
  ulStyles &= ~(SS_HALFTONEFRAME | SS_BKGNDFRAME | SS_FGNDFRAME);

  /*************************************************************/
  /* Replace with new outline type.                            */
  /*************************************************************/
  switch (outlineType)
  {
    case foreground:
      ulStyles |= SS_FGNDFRAME;
    break;

    case background:
      ulStyles |= SS_BKGNDFRAME;
    break;

    case halftone:
      ulStyles |= SS_HALFTONEFRAME;
    break;

    default:                          //Uninitialized enum
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
    break;
  }

  if (ulStyles != ulOldStyles)
  {
    setStyle(ulStyles);                 //Change style
    refresh();                          //Force repaint
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  bool styleChanged = false;
  IColor pfColor = parent()->foregroundColor();
  IColor pbColor = parent()->backgroundColor();

  /*************************************************************/
  /* Replace with new outline type.                            */
  /*************************************************************/
  switch (outlineType)
  {
    case foreground:
    case halftone:
      if (this->outlineType() != foreground)
      {
        XtVaSetValues ( fOutlineBoxData->whOutlineBox,
                        XmNborderColor, pfColor.index(),
                        NULL);
        styleChanged = true;
      }
    break;

    case background:
      if (this->outlineType() != background)
      {
        XtVaSetValues ( fOutlineBoxData->whOutlineBox,
                        XmNborderColor, pbColor.index(),
                        NULL);
        styleChanged = true;
      }
    break;

    default:                          //Uninitialized enum
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
    break;
  }

  if (styleChanged)
  {
    refresh();                          //Force repaint
  }
#endif // IC_MOTIF
  return *this;
}

/*------------------------------------------------------------------------------
| IOutlineBox::outlineType                                                     |
------------------------------------------------------------------------------*/
IOutlineBox::OutlineType  IOutlineBox :: outlineType() const
{
#ifdef IC_PMWIN
  unsigned long ulStyles =
     style() & (SS_HALFTONEFRAME | SS_BKGNDFRAME | SS_FGNDFRAME);
  if (ulStyles == SS_BKGNDFRAME)
     return background;
  if (ulStyles == SS_HALFTONEFRAME)
     return halftone;
  return foreground;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Pixel outcolor;
  IColor pbcolor = parent()->backgroundColor();
  XtVaGetValues ( fOutlineBoxData->whOutlineBox,
                  XmNborderColor, &outcolor,
                  NULL ) ;
  if (outcolor == pbcolor.index())
     return background;
  else
     return foreground;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| WIN32 Color Support                                                          |
|                                                                              |
| Windows has no API equivalent for Presentation Parameters.  The following    |
| outline box color function is NOPed for Windows.  The query function returns |
| an appropriate default IColor object.                                        |
------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| IOutlineBox::foregroundColor                                                 |
|                                                                              |
| Return the foreground color of the outline box based on the outline type.    |
------------------------------------------------------------------------------*/
IColor IOutlineBox::foregroundColor () const
{
#ifdef IC_PMWIN
  IColor
    guiColor( IColor::kWindowBgnd );
  switch (outlineType())
  {
    case background:
#ifdef IC_WIN
      return defaultbackgroundcolor(handle());
#endif
      break;
    case foreground:
      guiColor = IColor( IColor::kFrameBorder );
   break;
   case halftone:
      guiColor = IColor( IColor::kDesktopBgnd );
   break;
  }
#ifdef IC_PM
  return this->color( PP_FOREGROUNDCOLOR, guiColor );
#endif
#ifdef IC_WIN
  return guiColor;
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Pixel outcolor;
  XtVaGetValues ( fOutlineBoxData->whOutlineBox,
                  XmNborderColor, &outcolor,
                  NULL ) ;
  return IColor(outcolor, 0);
#endif // IC_MOTIF
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IOutlineBox::visibleRectangle                                                |
|                                                                              |
| Returns the area painted by the outline box.                                 |
------------------------------------------------------------------------------*/
IRectangle IOutlineBox :: visibleRectangle () const
{
  return IRectangle(position(),ISize(0,0));
}
#endif // IC_PMWIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IOutlineBox::setForegroundColor                                              |
|                                                                              |
| Sets the foreground color of the outline box                                 |
------------------------------------------------------------------------------*/
IOutlineBox& IOutlineBox :: setForegroundColor    (const IColor &color)
{
  // Should really call inherited setForegroundColor.
  XtVaSetValues( fOutlineBoxData->whOutlineBox,
                 XmNborderColor,
                 color.index(),
                 NULL ) ;
  refresh();
  return *this;
}

/*------------------------------------------------------------------------------
| IOutlineBox::setColor                                                        |
|                                                                              |
| Sets the foreground color of the outline box, for other PP_ areas call the   |                                 |
| inherited setColor
------------------------------------------------------------------------------*/
IOutlineBox& IOutlineBox :: setColor ( unsigned long colorArea,
                                       const IColor &color)
{
  if (colorArea == PP_FOREGROUNDCOLOR )
  {
    XtVaSetValues ( fOutlineBoxData->whOutlineBox,
                    XmNborderColor,
                    color.index(),
                    NULL ) ;
    refresh();
  }
  else
    Inherited::setColor( colorArea, color );
  return *this;
}

#pragma options info=nouse
extern void _System iobExposeCallback (Widget    w,
                                       XtPointer client_data,
                                       XtPointer call_data)
{
  XLowerWindow( XtDisplay(w), XtWindow (w) );
  return;
}

#endif // IC_MOTIF
