// Revision: 27 1.27.1.11 source/ui/basectl/ipushbut.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ipushbut.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipushbut.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #define INCL_WINBUTTONS
  #define INCL_WINDIALOGS         // DLGC_DEFPUSHBUTTON, etc.
  #define INCL_WINERRORS
  #define INCL_WINFRAMEMGR        // WM_QUERYFOCUSCHAIN
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR       // QWL_xxx
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/Label.h>
  #include <Xm/PushB.h>
  #include <Xm/BulletinB.h>
#endif // IC_MOTIF

#include <ipushbut.hpp>
#include <ictlevt.hpp>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <iframe.hpp>
#include <ifont.hpp>
#include <istring.hpp>
#include <ihandle.hpp>
#include <ihandler.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <itrace.hpp>
#include <inotifev.hpp>
#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif
#ifdef IC_WIN
  #include <ihelpsta.hpp>
#endif
#include <icvhdr.hpp>
#include <icconst.h>
#include <iinhratt.hpp>

#ifdef IC_MOTIF
#include <ihelp.hpp>
#include <icmdevt.hpp>
#include <imstring.hpp>
#include <ixmlabel.hpp>
//#include <iwinpriv.hpp>
#endif // IC_MOTIF


#ifdef IC_PAGETUNE
  #define _IPUSHBUT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Public push button styles.                                                   |
------------------------------------------------------------------------------*/

const IPushButton::Style
#ifdef IC_PM
  IPushButton::help              = BS_HELP,
  IPushButton::systemCommand     = BS_SYSCOMMAND,
  IPushButton::defaultButton     = BS_DEFAULT,
  IPushButton::noBorder          = BS_NOBORDER,
#endif
#ifdef IC_MOTIFWIN
  IPushButton::help          (0, IBS_HELP       ),
  IPushButton::systemCommand (0, IBS_SYSCOMMAND ),
  IPushButton::defaultButton (0, IBS_DEFAULT    ),
#ifdef IC_WIN
  IPushButton::noBorder          = 0,  // can this be done??
#endif // IC_WIN
#ifdef IC_MOTIF
  IPushButton::noBorder      (0, IBS_NOBORDER   ),
#endif // IC_MOTIF
#endif // IC_MOTIFWIN
  IPushButton::classDefaultStyle = WS_VISIBLE;

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
  IPushButton::Style IPushButton::currentDefaultStyle = WS_VISIBLE;

#pragma enum(4)
#pragma pack(push,4)

class IPushButtonHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* The IPushButtonHandler class is used in Windows to support the generation    *
* of non standard events as a result of a button press.  The events this       *
* handler generates are:                                                       *
*    WM_COMMAND    - when neither IPushButton::systemCommand or                *
*                    IPushButton::help styles are specified.                   *
*    WM_SYSCOMMAND - when the IPushButton::systemCommand style is specified.   *
*    WM_HELP       - when the IPushButton::help style is specified.            *
* An instance of this handler is attached to all IPushButtons as a default     *
* handler.                                                                     *
*******************************************************************************/
public:
  IPushButtonHandler  ( );
virtual
  ~IPushButtonHandler ( );

/*------------------------------ Event Handling --------------------------------
|   dispatchHandlerEvent is overridden and performs all the processing for     |
|   this class.                                                                |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent ( IEvent& event );

#ifdef IC_WIN
private:
HANDLE
  lastFocusHwnd;
#endif

}; //class IPushButtonHandler

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IPushButtonHandler::IPushButtonHandler                                       |
------------------------------------------------------------------------------*/
IPushButtonHandler::IPushButtonHandler ( ) :
  IHandler( )
#ifdef IC_WIN
, lastFocusHwnd( 0 )
#endif
{ }

/*------------------------------------------------------------------------------
| IPushButtonHandler::~IPushButtonHandler                                      |
------------------------------------------------------------------------------*/
IPushButtonHandler::~IPushButtonHandler ( )
{ }

/*------------------------------------------------------------------------------
| IPushButtonHandler::dispatchHandlerEvent                                     |
------------------------------------------------------------------------------*/
bool IPushButtonHandler::dispatchHandlerEvent ( IEvent& event )
{
   bool bProcessed = false;
#ifdef IC_PMWIN
  #ifdef IC_WIN
   if (event.eventId() == WM_CONTROL)
      {
      IMODTRACE_ALL("IPushButtonHandler::dispatchHandlerEvent");
      ITRACE_ALL( IString("wparam=") +
                  IString( event.parameter1().asUnsignedLong() ).d2x() +
                  IString(" lparam=") +
                  IString( event.parameter2().asUnsignedLong() ).d2x() );
      unsigned long ulMsg = event.parameter1().number2();
      if (ulMsg == BN_CLICKED)
         {
         IControlEvent ctlevt(event);
         IPushButton* button = dynamic_cast<IPushButton*>(ctlevt.controlWindow());
         // generate the needed events due to a button click
         if (button)
            {
            ITRACE_ALL( IString("button=") +
                        IString( (unsigned long)((void*)button) ).d2x() );
            unsigned long newMsg = 0;
            IEventData    wparam(ctlevt.controlId(), 0);
            IEventData    lparam(0);
            IWindow* postTo = button->owner();
            if      ( button->isHelp() )
               {
               newMsg = IC_UM_CONTEXT_HELP;
               wparam = 0;
               if (lastFocusHwnd && !button->allowsMouseClickFocus())
                 lparam = lastFocusHwnd;
               else
                 lparam = (HWND) button->handle();
               postTo = (IWindow *)ihelpwindowSearchChain( lparam, 0 );
               }
            else if ( button->isSystemCommand() )
               {
               // note that lparam is 0, implying that syscommand was
               // selected with keyboard.  Win puts mouse coordinates
               // in lparam if chose with mouse
               newMsg = WM_SYSCOMMAND;
               }
            else
               {
               newMsg = WM_COMMAND;
               wparam = IEventData(ctlevt.controlId(), CMDSRC_PUSHBUTTON );
               }

            if ((postTo == 0) || (postTo == IWindow::desktopWindow()) )
               {
               postTo = button->parent();
               }
            if (postTo)
               {
               ITRACE_ALL( IString("postTo=") +
                           IString( (unsigned long)((void*)owner) ).d2x() +
                           IString(" hwnd=") +
                           IString( owner->handle().asUnsigned() ).d2x() );
               postTo->postEvent( newMsg,
                                 wparam,
                                 lparam );
               } // if owner
            }   // if button
         }      // if BN_CLICKED
      }         // if WM_CONTROL
   else if (event.eventId() == WM_KILLFOCUS)
      {
      // Push Button losing focus here.  Remove emphasis only if new
      // focus window is in the same frame.  (WINDOWS ONLY)
      IPushButton* button = dynamic_cast<IPushButton*>(event.window());
      IWindowHandle hwndOther((HWND)event.parameter1());
      if (button &&
          hwndOther &&
          (hwndOther.sendEvent( WM_QUERYFOCUSCHAIN,
                                IEventParameter1( QFC_FRAME ),
                                IEventParameter2( 0 ))
                    .asUnsignedLong()
           ==
           button->handle().sendEvent( WM_QUERYFOCUSCHAIN,
                                 IEventParameter1( QFC_FRAME ),
                                 IEventParameter2( 0 ))
                     .asUnsignedLong()))
         {
         button->addEmphasis(false);

         // Assume new focus window is not a Push Button control.
         // Reset emphasis to the default if necessary.
         bool setDefButton = true;
         IWindowClassName otherClassName(hwndOther);
         if (otherClassName == WC_BUTTON)
            {
            unsigned long ulOtherStyle = ISTYLEOF(hwndOther);
            if ((((ulOtherStyle & BS_PRIMARYSTYLES) == BS_PUSHBUTTON) ||
                 ((ulOtherStyle & BS_PRIMARYSTYLES) == BS_DEFPUSHBUTTON)))
               setDefButton = false;
            }
         if ( setDefButton )
            {
            IWindowHandle hwndDefBut(button->defaultPushButton());
            IPushButton* defBut((IPushButton*)IWindow::windowWithHandle(hwndDefBut));
            if (defBut)
               defBut->addEmphasis(true);
            }
         }
      }
  #endif // IC_WIN
  #ifdef IC_PM
   if (event.eventId() == WM_SETFOCUS)
  #else
   else if (event.eventId() == WM_SETFOCUS)
  #endif // IC_PM
      // We process this message to insure that there is only one
      // pushbutton with "default" state.
      {
      IPushButton* button = dynamic_cast<IPushButton*>(event.window());
      if (button)
         {
         IWindowHandle hwndDefBut(button->defaultPushButton());
         // save handle of window that last had focus for
         // help button context help processing.
  #ifdef IC_WIN
         if (button->isHelp())
            {
              lastFocusHwnd = event.parameter1();
            }
         // Push Button gaining focus here.  Add emphasis. (WINDOWS ONLY)
         button->addEmphasis(true);
  #endif // IC_WIN
  #ifdef IC_PM
         bool gainingFocus(event.parameter2().number1());
         if ( gainingFocus )
            {
            // Push Button gaining focus here.  Add emphasis. (PM ONLY)
            button->addEmphasis(true);
            }
         else if (!gainingFocus)
            {
            // Push Button losing focus here.  Remove emphasis only if
            // new focus window is in the same frame.  (PM ONLY)
            IWindowHandle hwndOther((HWND)event.parameter1());
            if (hwndOther &&
                (hwndOther.sendEvent( WM_QUERYFOCUSCHAIN,
                                      IEventParameter1( QFC_FRAME ),
                                      IEventParameter2( 0 ))
                          .asUnsignedLong()
                 ==
                 button->handle().sendEvent( WM_QUERYFOCUSCHAIN,
                                       IEventParameter1( QFC_FRAME ),
                                       IEventParameter2( 0 ))
                           .asUnsignedLong()))
               {
               button->addEmphasis(false);

               // Assume new focus window is not a Push Button control.
               // Reset emphasis to the default if necessary.
               bool setDefButton = true;
               IWindowClassName otherClassName(hwndOther);
               if (otherClassName == WC_BUTTON)
                  {
                  unsigned long ulOtherStyle = ISTYLEOF(hwndOther);
                  if ((ulOtherStyle & BS_PRIMARYSTYLES) == BS_PUSHBUTTON)
                     setDefButton = false;
                  }
               if ( setDefButton )
                  {
                  IPushButton* defBut(
                      (IPushButton*)IWindow::windowWithHandle(hwndDefBut));
                  if (defBut)
                     defBut->addEmphasis(true);
                  }
               }
            }
  #endif // IC_PM
         }
      }
   else if ( (event.eventId() == IC_UM_DEFAULT_EMPHASIS) ||
             (event.eventId() == IC_UM_DEFAULT_BUTTON) )
      {
      IPushButton* button = dynamic_cast<IPushButton*>(event.window());
      if (button)
         {
         IWindowHandle hwndBut(button->handle());
         unsigned long ulStyle = ISTYLEOF(hwndBut);
         bool emphasis(event.parameter1().number1());
         if (!emphasis)
            {
            // Clear emphasis.  This takes care of removing emphasis
            //     from the pushbutton.
  #ifdef IC_WIN
            hwndBut.sendEvent( BM_SETSTYLE,
                     (ulStyle & ~((unsigned long)BS_PRIMARYSTYLES)) |
                                                 BS_PUSHBUTTON, true );
  #endif // IC_WIN
  #ifdef IC_PM
            hwndBut.sendEvent( BM_SETDEFAULT, false, 0 );
  #endif // IC_PM
            bProcessed = true;  // do not pass on
            }
         else
            {
            // Set emphasis.  This takes care of adding emphasis to the
            //      pushbutton.
  #ifdef IC_WIN
            hwndBut.sendEvent( BM_SETSTYLE,
                     (ulStyle & ~((unsigned long)BS_PRIMARYSTYLES)) |
                                                 BS_DEFPUSHBUTTON, true );
  #endif // IC_WIN
  #ifdef IC_PM
            hwndBut.sendEvent( BM_SETDEFAULT, true, 0 );
  #endif // IC_PM
            bProcessed = true;  // do not pass on
            }
         }
      }
#endif // IC_PMWIN

#ifdef IC_MOTIF
   switch (event.eventId() )
   {
      // This code relies upon the fact that IWindow filters focus events such
      // that only focus events which indicate a change to this widget are
      // dispatched.  The ancestor/inferior notifications generated by the
      // X server are not dispatched.
      case xEvent( FocusIn ):
         {
            IMODTRACE_DEVELOP("IPushButton - FocusIn");
            IPushButton* button =
               dynamic_cast<IPushButton*>(event.window());
            if (button)
            {
               // Always set emphasis when this button gets focus.
               button->addEmphasis( true );
            }
         }
         break;
      case xEvent( FocusOut ):
         {
            IMODTRACE_DEVELOP("IPushButton - FocusOut");
            // Post message to allow X focus change processing to complete before
            // we attempt to query what happened.  We process message below.
            event.handle().postEvent( IC_UM_CANVAS_SETFOCUS );
         }
         break;

      case IC_UM_CANVAS_SETFOCUS:
         {
            IMODTRACE_DEVELOP("IC_UM_CANVAS_SETFOCUS");
            // This event is posted to the button when focus is lost
            // from the button.  After the focus notifications have been
            // done, we need to find the current focus window and adjust
            // emphasis accordingly.
            IPushButton* button =
               dynamic_cast<IPushButton*>(event.window());
            if (button)
            {
               // Find the focus in the current hierarchy that
               // has the focus.  If it is this widget, then focus
               // has moved to a heirachy under another shell and
               // we do not need to change the default.
               Widget hwndButton = event.handle();
               Widget newFocus = XmGetFocusWidget( hwndButton );
               ITRACE_DEVELOP( IString("hwndButton=") +
                               IString((unsigned long)hwndButton).d2x() +
                               IString(" newFocus=") +
                               IString((unsigned long)newFocus).d2x() );
               if  ((newFocus != hwndButton) &&
                    (newFocus != button->defaultEmphasisButton() ) )
               {
                   // Focus has moved to another widget in the same
                   // hierarchy and it (apparently) is not a push button.
                   // Remove default emphasis on this button and set
                   // the default button if there is one.
                   button->addEmphasis( false );

                   IPushButton* defBut =
                       dynamic_cast<IPushButton*>( IWindow::windowWithHandle(
                           button->defaultPushButton() ) );
                   if (defBut)
                      defBut->addEmphasis(true);
               }
            }
            // Always processed.
            bProcessed = true;
         }
         break;

      case IC_UM_DEFAULT_EMPHASIS:
      case IC_UM_DEFAULT_BUTTON:
         {
            IPushButton* button = dynamic_cast<IPushButton*>(event.window());
            if (button)
            {
               // event parameter 1 will indicate whether the button is
               // becoming the default button or not.
               Dimension emphasis( event.parameter1().number1() ? 1 : 0);
               XtVaSetValues( button->handle(), XmNshowAsDefault, emphasis, 0 );
               bProcessed = true;
            }
         }
         break;

      case  motifEvent(XmCR_ACTIVATE):
         {
            IPushButton* button =  dynamic_cast<IPushButton*>(event.window());
            if ( button && button->isHelp() )
            {
               // If this is a help pushbutton, we have to generate a correct
               // WM_HELP event.

               // Set up HELPINFO structure.
               HELPINFO helpinfo;
               helpinfo.cbSize       = sizeof(HELPINFO);
               helpinfo.iContextType = HELPINFO_WINDOW;
               // If this help button allows the focus, then they will get
               // help on the button itself.
               if (button->allowsMouseClickFocus())
               {
                  helpinfo.iCtrlId   = button->id();
                  helpinfo.hItemHandle  = button->handle();
               }
               // Otherwise, look for the widget with the focus so we can
               // provide help on that.
               else
               {
                  IWindowHandle
                     windowHandleWithFocus( XmGetFocusWidget(button->handle()) );
                  IWindow* windowWithFocus =
                     IWindow::windowWithHandle(windowHandleWithFocus);
                  // If we got a widget handle but not an IWindow, this may
                  // be a composite - try to get the parent handle.
                  if ((windowHandleWithFocus != (IWindowHandle::Value)0) &&
                      (windowWithFocus == 0))
                  {
                     windowHandleWithFocus = IPARENTOF(windowHandleWithFocus);
                     windowWithFocus =
                        IWindow::windowWithHandle( windowHandleWithFocus );
                  }
                  // If a widget with focus was found, provide its info in the
                  // HELPINFO structure.
                  if ((windowHandleWithFocus != (IWindowHandle::Value)0) &&
                       windowWithFocus)
                  {
                     helpinfo.iCtrlId = windowWithFocus->id();
                     helpinfo.hItemHandle = windowHandleWithFocus;
                  }
                  // Otherwise, provide help on the button.
                  else
                  {
                     helpinfo.iCtrlId = button->id();
                     helpinfo.hItemHandle = button->handle();
                  }
               }

               helpinfo.dwContextId  = 0;

               button->sendEvent( WM_HELP, 0, &helpinfo );
            }
            else if (button)
            {  // Post a command event to the owner.
               IWindow * powner = button->owner();
               if (powner)
                  powner->postEvent ( button->isSystemCommand() ?
                                      IWindow::systemCommand : IWindow::command,
                                      button->id(),
                                      IEventData(ICommandEvent::pushButton, 0) );
            }
         }
         break;
   }

#endif // IC_MOTIF
   return bProcessed;   // pass all events on
}

#pragma enum(4)
#pragma pack(push,4)

class IPushButtonData  {
public:
  IPushButtonData( );
  ~IPushButtonData( );

#ifdef IC_MOTIF
  unsigned long flags;

  static const unsigned
     helpButton,
     issueSysCommand;
#endif // IC_MOTIF
  IPushButtonHandler
     fdefaultHandler;
};

#pragma pack(pop)
#pragma enum(pop)


#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| Static IPushButton Private Data members.                                     |
------------------------------------------------------------------------------*/
const unsigned
  IPushButtonData::helpButton       = 0x00000001,
  IPushButtonData::issueSysCommand = 0x00000002;
#endif

/*------------------------------------------------------------------------------
| IPushButtonData::IPushButtonData                                             |
------------------------------------------------------------------------------*/
IPushButtonData::IPushButtonData()
#ifdef IC_MOTIF
  : flags(0)
#endif
#ifdef IC_PMWIN
  : fdefaultHandler()
#endif
{ }

/*------------------------------------------------------------------------------
| IPushButtonData::~IPushButtonData                                            |
------------------------------------------------------------------------------*/
IPushButtonData::~IPushButtonData()
{ }



/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to create a push button control on a standard window.            |
------------------------------------------------------------------------------*/
IPushButton::IPushButton ( unsigned long ulId,
                           IWindow* parent,
                           IWindow* pwndOwner,
                           const IRectangle& rectInit,
                           const Style& pbsStyle )
  : fPushButtonData( new IPushButtonData )
{
  IASSERTPARM(parent!=0);

#ifdef IC_PMWIN
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | pbsStyle.asExtendedUnsignedLong() );

  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (parent)
    parentHandle = parent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();
  IWindowHandle whPushButton =
      this -> create( ulId,
                      0,
                      convertToGUIStyle( pbsStyle ),
                      WC_BUTTON,
                      parentHandle,
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( pbsStyle, true ) );

  startHandlingEventsFor(whPushButton);

  #ifdef IC_WIN
    // if the given button has default style, set extended style
    // and enable as default
    unsigned long ulStyle = style() & BS_PRIMARYSTYLES;
    if ( ulStyle & BS_DEFPUSHBUTTON )
    {
      setExtendedStyle( extendedStyle() |
                        defaultButton.asExtendedUnsignedLong() );
      enableDefault(true);
    }
  #endif
  #ifdef IC_PM
    unsigned long ulStyle = style();
    if ( ulStyle & BS_DEFAULT )
      enableDefault(true);
  #endif // IC_PM
  fPushButtonData->fdefaultHandler.handleEventsFor( this );
#endif // IC_PMWIN

#ifdef IC_MOTIF
   IArgList argList;
   argList.setResource( XmNdefaultButtonShadowThickness, 1 );
   argList.setResource( XmNrecomputeSize, false );

   if (pbsStyle & IPushButton::noBorder)
     argList.setResource( XmNshadowThickness, 0 );
   else
     argList.setResource( XmNshadowThickness, 2);

   /***************************************************************/
   /* Determine if button should send system command events       */
   /***************************************************************/
   if (pbsStyle & IPushButton::systemCommand)
     fPushButtonData->flags |= IPushButtonData::issueSysCommand;
   if (pbsStyle & IPushButton::help)
     fPushButtonData->flags |= IPushButtonData::helpButton;

   Widget widgetID = Inherited::create(
      ulId,
      0,
      pbsStyle,
      (IXmCreateFunction)XmCreatePushButton,
      parent,
      pwndOwner,
      rectInit);
   XtSetValues( widgetID, (ArgList)argList.argList(), argList.numberOfArgs() );

   startHandlingEventsFor(widgetID);
   fPushButtonData->fdefaultHandler.handleEventsFor( this );

   if (pbsStyle & IButton::noPointerFocus)
     disableMouseClickFocus();


   // If the label string is equal to the widget name, set it to the null
   // string.  MUST do this conditionally after create to allow the use of
   // resource files.
   IXmLabel::initializeXmLabelString( widgetID, IString( ulId ), true );

   if (pbsStyle & IPushButton::defaultButton)
   {
      // frame needs to know that this is the default button.
      enableDefault( true );
   }

   // Register the activate callback.  Both the default IPushButtonHandler
   // and the notification handler in IButton depend on this callback for
   // proper operation.
   XtAddCallback (widgetID,
                  XmNactivateCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);

#endif // IC_MOTIF
#ifdef IC_MOTIFWIN
  // add the inheritColor attribute to the window.  PushButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );

#endif // IC_MOTIFWIN
#ifdef IC_WIN
  setColor( PP_BACKGROUNDCOLOR, IColor(IColor::kButtonMiddle));
#endif // IC_WIN
}


/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to instantiate an object for a dialog template.                  |
------------------------------------------------------------------------------*/
IPushButton::IPushButton ( unsigned long ulId,
                           IWindow* pdlgwndParent )
  : fPushButtonData( new IPushButtonData )
{
#ifdef IC_PMWIN
  setAutoDestroyWindow  ( false );
  reserveUserWindowWord( false );
  startHandlingEventsFor( ulId,
                          pdlgwndParent );
  #ifdef IC_WIN
    // if the given button is a default then set extended style
    unsigned long ulStyle = style() & BS_PRIMARYSTYLES;
    if ( ulStyle & BS_DEFPUSHBUTTON )
    {
      setExtendedStyle( extendedStyle() |
                        defaultButton.asExtendedUnsignedLong() );
      enableDefault(true);
    }
  #endif
  #ifdef IC_PM
    unsigned long ulStyle = style();
    if ( ulStyle & BS_DEFAULT )
      enableDefault(true);
  #endif // IC_PM
  fPushButtonData->fdefaultHandler.handleEventsFor( this );
#endif // IC_PMWIN

#ifdef IC_MOTIF
  // assertions on input parms
  IASSERTPARM(pdlgwndParent!=0);

  Widget    whPushButton;

  IString widgetName = IString (ulId);
  whPushButton = XtNameToWidget(pdlgwndParent->handle(), widgetName);

  if (whPushButton == NULL)
    ITHROWLIBRARYERROR1( IC_CONTROL_NOT_FOUND,
                         IBaseErrorInfo::accessError,
                         IException::recoverable,
                         "XtNameToWidget (push button)");

  if ( !(XtIsSubclass(whPushButton, xmPushButtonWidgetClass)) )
    ITHROWLIBRARYERROR1( IC_WRONG_CONTROL_TYPE,
                         IBaseErrorInfo::accessError,
                         IException::recoverable,
                         "XmPushButton");

  setAutoDestroyWindow(false);
  prepareForUse(whPushButton);
  startHandlingEventsFor(whPushButton);
  fPushButtonData->fdefaultHandler.handleEventsFor( this );

  // Register the activate callback.  Both the default IPushButtonHandler
  // and the notification handler in IButton depend on this callback for
  // proper operation.
  XtAddCallback (whPushButton,
                 XmNactivateCallback,
                 iwindowMotifCallback,
                 (XtPointer)this);

  IWindow::setOwner( pdlgwndParent );

  // If this is a default button, insure it is the only one.
  Dimension showAsDefault(0);
  XtVaGetValues(whPushButton, XmNshowAsDefault, &showAsDefault, 0);
  if ( showAsDefault )
     enableDefault( true );

#endif // IC_MOTIF
#ifdef IC_MOTIFWIN
  // add the inheritColor attribute to the window.  PushButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );
#endif
}


/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to instantiate from an existing push button control.             |
------------------------------------------------------------------------------*/
IPushButton::IPushButton ( const IWindowHandle& wh )
  : fPushButtonData( new IPushButtonData )
{
#ifdef IC_PMWIN
  setAutoDestroyWindow  ( false );
  reserveUserWindowWord( false );
  startHandlingEventsFor( wh );
  #ifdef IC_WIN
    // if the given button is a default then set extended style
    unsigned long ulStyle = style() & BS_PRIMARYSTYLES;
    if ( ulStyle & BS_DEFPUSHBUTTON )
    {
      setExtendedStyle( extendedStyle() |
                        defaultButton.asExtendedUnsignedLong() );
      enableDefault(true);
    }
  #endif
  #ifdef IC_PM
    unsigned long ulStyle = style();
    if ( ulStyle & BS_DEFAULT )
      enableDefault(true);
  #endif // IC_PM
  fPushButtonData->fdefaultHandler.handleEventsFor( this );
#endif // IC_PMWIN

#ifdef IC_MOTIF
  IASSERTPARM(wh!=0);

  if ( !(XtIsSubclass(wh, xmPushButtonWidgetClass)) )
    ITHROWLIBRARYERROR1( IC_WRONG_CONTROL_TYPE,
                         IBaseErrorInfo::accessError,
                         IException::recoverable,
                         "XmPushButton");

  setAutoDestroyWindow(false);
  prepareForUse(wh);
  startHandlingEventsFor(wh);
  fPushButtonData->fdefaultHandler.handleEventsFor( this );

  // Register the activate callback.  Both the default IPushButtonHandler
  // and the notification handler in IButton depend on this callback for
  // proper operation.
  XtAddCallback (wh,
                 XmNactivateCallback,
                 iwindowMotifCallback,
                 (XtPointer)this);

  // If this is a default button, insure it is the only one.
  Dimension showAsDefault(0);
  XtVaGetValues(wh, XmNshowAsDefault, &showAsDefault, 0);
  if ( showAsDefault )
     enableDefault( true );

#endif // IC_MOTIF
#ifdef IC_MOTIFWIN
  // add the inheritColor attribute to the window.  PushButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );
#endif
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to create a push button control with no widget.                  |
------------------------------------------------------------------------------*/
IPushButton::IPushButton ( )
                            : fPushButtonData( new IPushButtonData )
{
  // add the inheritColor attribute to the window.  PushButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );
}
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| IPushButton::~IPushButton                                                    |
------------------------------------------------------------------------------*/
IPushButton::~IPushButton ( )
{
  fPushButtonData->fdefaultHandler.stopHandlingEventsFor( this );
#ifdef IC_MOTIF
  if (this->isValid())
  {
     // Remove the activate callback.
     XtRemoveCallback (handle(),
                       XmNactivateCallback,
                       iwindowMotifCallback,
                       (XtPointer)this);
  }
#endif // IC_MOTIF

  delete fPushButtonData;
}


/*------------------------------------------------------------------------------
| IPushButton::defaultStyle                                                    |
------------------------------------------------------------------------------*/
IPushButton::Style  IPushButton::defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IPushButton::setDefaultStyle                                                 |
------------------------------------------------------------------------------*/
void  IPushButton::setDefaultStyle( const Style& pbsStyle )
{
  currentDefaultStyle = pbsStyle;
}


/*------------------------------------------------------------------------------
| IPushButton::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IPushButton::convertToGUIStyle( const IBitFlag& guiStyle,
                                              bool bExtOnly ) const
{
  // Obtain the style from the class (IButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // Set the GUI pushbutton style

#ifdef IC_WIN
    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXTPUSH;

    // Handle overlapping styles
    if (ulTmp & IBS_DEFAULT)
      ulStyle |= BS_DEFPUSHBUTTON | WS_CHILD;
    else
#endif

      ulStyle |= BS_PUSHBUTTON
#ifdef IC_WIN
                 | BS_CENTER
                 | WS_CHILD
#endif
      ;
  }

  return ulStyle;
}


/*------------------------------------------------------------------------------
| IPushButton::enableDefault                                                   |
|                                                                              |
| Sets the push button to be the default button.                               |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::enableDefault ( bool enable )
{
  // Give the button the default style, as appropriate.
  bool bDefault( this->isDefault() );
  if ( (enable && !bDefault) || (!enable & bDefault ) )
     {
     // Need to change it.  This takes care of removing default from
     // another button if needed.
     if (enable)
       setDefaultPushButton(handle());
     else
       setDefaultPushButton(0);
#ifdef IC_MOTIFWIN
     // update the extended style bits
     unsigned long ulExtStyle = extendedStyle();
     if (enable)
       ulExtStyle |= defaultButton.asExtendedUnsignedLong();
     else
       ulExtStyle &= ~defaultButton.asExtendedUnsignedLong();
     setExtendedStyle(ulExtStyle);

//     refresh();                //Force refresh
#endif


#ifdef IC_MOTIF
     XtVaSetValues( (Widget)handle(),
        XmNshowAsDefault, enable,
        NULL );
#endif // IC_MOTIF
    }  // changed

  return *this;
}


/*------------------------------------------------------------------------------
| IPushButton::disableDefault                                                  |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::disableDefault()
{
  enableDefault(false);
  return *this;
}


/*------------------------------------------------------------------------------
| IPushButton::isDefault                                                       |
------------------------------------------------------------------------------*/
bool  IPushButton::isDefault() const
{
#ifdef IC_PM
  if ( (style() & defaultButton.asUnsignedLong()) &&
       (handle() == defaultPushButton()) )
     return true;
  else
     return false;
#endif
#ifdef IC_WIN
  if ( ((style() & BS_PRIMARYSTYLES) == BS_DEFPUSHBUTTON ) &&
       (handle() == defaultPushButton()) )
     return true;
  else
     return false;
#endif
#ifdef IC_MOTIF
  Dimension showAsDefault(0);
  XtVaGetValues((Widget) handle(), XmNshowAsDefault, &showAsDefault, 0);
  if (showAsDefault && (handle() == defaultPushButton()) )
     return true;
  else
     return false;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IPushButton::enableHelp                                                      |
|                                                                              |
| Sets the help style which causes a help event to generated when the push     |
| button is pressed.                                                           |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::enableHelp(bool bTurnOn)
{
#ifdef IC_PMWIN
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long helpStyle = help.asUnsignedLong();
#endif
#ifdef IC_WIN
  unsigned long ulStyle = extendedStyle();
  unsigned long helpStyle = help.asExtendedUnsignedLong();
#endif
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle |= helpStyle;
  }
  else
  {
    ulStyle &= ~helpStyle;
  }

  if (ulStyle != ulOldStyle)
  {
#ifdef IC_PM
    setStyle(ulStyle);
#endif
#ifdef IC_WIN
    setExtendedStyle(ulStyle);
#endif
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (bTurnOn)
    fPushButtonData->flags |= IPushButtonData::helpButton;
  else
    fPushButtonData->flags &= ~IPushButtonData::helpButton;
#endif  // IC_MOTIF

  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::disableHelp                                                     |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::disableHelp()
{
  enableHelp(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::isHelp                                                          |
------------------------------------------------------------------------------*/
bool  IPushButton::isHelp() const
{
#ifdef IC_PM
  return( (style() & help.asUnsignedLong()) ? true : false );
#endif
#ifdef IC_WIN
  return( (extendedStyle() & help.asExtendedUnsignedLong()) ? true : false );
#endif
#ifdef IC_MOTIF
  if ( fPushButtonData->flags & IPushButtonData::helpButton )
    return true;
  else
    return false;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IPushButton::enableSystemCommand                                             |
|                                                                              |
| Sets the system command style which causes a system command to be generated  |
| when the push button is pressed.                                             |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::enableSystemCommand(bool bTurnOn)
{
#ifdef IC_PMWIN
  #ifdef IC_PM
    unsigned long ulStyle = style();
    unsigned long newStyle = systemCommand.asUnsignedLong();
  #endif
  #ifdef IC_WIN
    unsigned long ulStyle = extendedStyle();
    unsigned long newStyle = systemCommand.asExtendedUnsignedLong();
  #endif
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle |= newStyle;
  }
  else
  {
    ulStyle &= ~newStyle;
  }

  if (ulStyle != ulOldStyle)
  {
  #ifdef IC_PM
      setStyle(ulStyle);
  #endif
  #ifdef IC_WIN
      setExtendedStyle(ulStyle);
  #endif
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (bTurnOn)
    fPushButtonData->flags |=  IPushButtonData::issueSysCommand;
  else
    fPushButtonData->flags &= ~IPushButtonData::issueSysCommand;
#endif // IC_MOTIF
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::disableSystemCommand                                            |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::disableSystemCommand()
{
  enableSystemCommand(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::isSystemCommand                                                 |
------------------------------------------------------------------------------*/
bool  IPushButton::isSystemCommand() const
{
#ifdef IC_PM
  return( (style() & systemCommand.asUnsignedLong()) ? true : false );
#endif
#ifdef IC_WIN
  return( (extendedStyle() & systemCommand.asExtendedUnsignedLong()) ? true
                                                                     : false );
#endif
#ifdef IC_MOTIF
  if ( fPushButtonData->flags & IPushButtonData::issueSysCommand )
    return true;
  else
    return false;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IPushButton::addBorder                                                       |
|                                                                              |
| Sets the noBorder style indicating the presence of a border around the       |
| push button.                                                                 |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::addBorder( bool add )
{
  ITRACE_WIN_NOP();
#ifdef IC_PMWIN
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if ( add )
  {
    ulStyle &= ~noBorder.asUnsignedLong();
  }
  else
  {
    ulStyle |= noBorder.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                           // Force refresh of control window
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Dimension shadowT;
  bool bcSize;
  Arg args [3];
  int i=0;

  /*******************************************/
  /* Save current recomputeSize              */
  /*******************************************/
  XtVaGetValues( handle(),
                 XmNrecomputeSize,
                 &bcSize,
                 NULL);


  /*******************************************/
  /* Disable/Enable button border            */
  /*******************************************/
  if ( add )
  {
     shadowT = 2;
     XtSetArg(args[i],
              XmNshadowThickness,
              shadowT); i++;
  }
  else
  {
    shadowT = 0;
    XtSetArg(args[i],
             XmNshadowThickness,
             shadowT); i++;
   }
  XtSetArg(args[i],
           XmNrecomputeSize,
           FALSE); i++;
  XtSetValues( (Widget) handle(), args, i);

  /*******************************************/
  /* Restore recomputeSize                   */
  /*******************************************/
  i=0;
  XtSetArg(args[i],
           XmNrecomputeSize,
           bcSize); i++;
  XtSetValues( (Widget) handle(), args, i);
#endif // IC_MOTIF
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::removeBorder                                                    |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::removeBorder( )
{
  ITRACE_WIN_NOP();
  addBorder(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::hasBorder                                                       |
------------------------------------------------------------------------------*/
bool  IPushButton::hasBorder ( ) const
{
  ITRACE_WIN_NOP();
#ifdef IC_PMWIN
  return( (style() & noBorder.asUnsignedLong()) ? false : true );
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Dimension shadowT;

  XtVaGetValues( (Widget) handle(),
                 XmNshadowThickness,
                 &shadowT,
                 NULL);
  if (shadowT)
     return true;
  else
     return false;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IPushButton::hasEmphasis                                                     |
------------------------------------------------------------------------------*/
bool  IPushButton::hasEmphasis() const
{
  return( (handle() == defaultEmphasisButton()) ? true : false );
}

/*------------------------------------------------------------------------------
| IPushButton::removeEmphasis                                                  |
|                                                                              |
| Sets the push button to have emphasis off.                                   |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton::removeEmphasis ( )
{
  addEmphasis(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::addEmphasis                                                     |
|                                                                              |
| Sets the push button to have emphasis on.                                    |
------------------------------------------------------------------------------*/
IPushButton& IPushButton::addEmphasis ( bool enable )
{
  // Give the button the emphasis, as appropriate.
  bool bEmphasis( this->hasEmphasis() );
  if ( (enable && !bEmphasis) || (!enable & bEmphasis) )
     // Need to change it.  This takes care of removing emphasis from
     // another button if needed.
     setDefaultEmphasisButton(handle(), enable);
  return *this;
}



/*------------------------------------------------------------------------------
| IPushButton::calcMinimumSize                                                 |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The height is      |
| based on the font height and width is based on the average font width        |
| multiplied by the width of the push button text.                             |
------------------------------------------------------------------------------*/
ISize  IPushButton::calcMinimumSize() const
{
#ifdef IC_PMWIN
  ISize size = displaySize(text());
  unsigned long ulWidth = size.width();
  unsigned long i=textLength();

  if (i==0)
     ulWidth += (2 + (9 * IQUERYSYSVALUE( SV_CXBORDER )));
  else
     ulWidth += (ulWidth/i  + 2 + (9 * IQUERYSYSVALUE( SV_CXBORDER )));
  unsigned long ulHeight = size.height() + (9 * IQUERYSYSVALUE( SV_CYBORDER ));
  return ISize(ulWidth, ulHeight);
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Dimension shadowThickness(0), borderWidth(0), highlightThickness(0),
            defaultButtonShadowThickness(0), showAsDefault(0),
//            marginHeight(0), marginWidth(0),
            marginLeft(0),   marginRight(0), marginTop(0), marginBottom(0);

  XtVaGetValues(this->handle(),
                XmNshadowThickness,    &shadowThickness,
                XmNborderWidth,        &borderWidth,
                XmNhighlightThickness, &highlightThickness,
                XmNdefaultButtonShadowThickness, &defaultButtonShadowThickness,
                XmNshowAsDefault,      &showAsDefault,
//                XmNmarginHeight,       &marginHeight,
//                XmNmarginWidth,        &marginWidth,
                XmNmarginLeft,         &marginLeft,
                XmNmarginRight,        &marginRight,
                XmNmarginTop,          &marginTop,
                XmNmarginBottom,       &marginBottom,
                0);

  // Hack.  There is a private value Xm3D_ENHANCE_PIXEL with
  // value of 2 which is added to the highlight thickness
  // value in XmPushButton.  This occurs ONLY if
  // defaultButtonShadowThickness is != 0.  There are also some differences
  // in the button size when there is no default button shadow at all.
  int enhance3D   ( defaultButtonShadowThickness ? 2 : 0 );
  int defThickness( enhance3D ?
                    defaultButtonShadowThickness : showAsDefault);
  int edges;
  if (defThickness)
     edges = 2*(shadowThickness + defThickness) +
             highlightThickness + enhance3D + borderWidth;
  else
     edges =  shadowThickness + highlightThickness + borderWidth;
  ISize size(2*edges,2*edges);   //Base size of widget

//  if (this->textLength() > 0)     //  27736
//  {
     ISize textSize( displaySize( text() ) );
     textSize += ISize( marginLeft + marginRight,
                        marginTop + marginBottom);
     size += textSize;
//  }

  return size;
#endif // IC_MOTIF
}

#ifdef IC_MOTIFWIN
/*------------------------------------------------------------------------------
| IPushButton::passEventToOwner                                                |
|                                                                              |
| Returns whether or not the event can be passed up to the owner of this       |
| control.                                                                     |
------------------------------------------------------------------------------*/
bool IPushButton::passEventToOwner( IEvent& event )

{
  switch ( event.eventId() )
  {
    case WM_BUTTON1UP:
    case WM_BUTTON2UP:
    case WM_BUTTON3UP:
    case WM_BUTTON1DOWN:
    case WM_BUTTON2DOWN:
    case WM_BUTTON3DOWN:
    case WM_BUTTON1CLICK:
    case WM_BUTTON2CLICK:
    case WM_BUTTON3CLICK:
    case WM_CHORD:
      event.setPassToOwner( false );
      break;
    default:
      Inherited::passEventToOwner( event );
      break;
    }
  return event.passToOwner();
}
#endif  // IC_MOTIFWIN

