// Revision: 39 1.7.1.3 source/ui/basectl/isetbut1.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: isetbut1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isetbnhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINBUTTONS

extern "C" {
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/ToggleB.h>
#endif // IC_MOTIF

#include <iwindow.hpp>
#include <isetbut.hpp>
#include <isetbnhd.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <icconst.h>
#ifdef IC_MOTIF
  #include <ictlevt.hpp>
#endif // IC_MOTIF

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISETBUT1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const ISettingButton::selectId = "ISettingButton::select";

/*------------------------------------------------------------------------------
| ISettingButton::enableNotification                                           |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ISettingButton, does not send notifications to observer objects until it is  |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ISettingButton& ISettingButton::enableNotification ( bool enabled )
{
  if (!notificationHandler())
  {
    setNotificationHandler( new ISettingButtonNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ISettingButtonNotifyHandler::ISettingButtonNotifyHandler                     |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ISettingButtonNotifyHandler::ISettingButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ISettingButtonNotifyHandler::~ISettingButtonNotifyHandler                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ISettingButtonNotifyHandler::~ISettingButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ISettingButtonNotifyHandler::dispatchHandlerEvent                            |
|                                                                              |
| This notifies ISettingButton Windows of the following events:                |
|                                                                              |
| select(Id)                                                                   |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
bool ISettingButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case BM_SETCHECK:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the selection state has been modified                      */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_SETBT_SELECT ));
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_SETBT_SELECT)
        {
          thisWindow->notifyObservers(INotificationEvent(
                                ISettingButton::selectId,
                                *thisWindow, true,
                                (void*)anEvent.parameter1()));
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

#ifdef IC_MOTIF
bool ISettingButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  if (anEvent.eventId() == motifEvent(XmCR_VALUE_CHANGED))
  {
     IWindowHandle hwndControl(anEvent.handle());
     IWindow*      thisWindow (anEvent.window());
     if ( (thisWindow) && (hwndControl) )
     {
        if (XtIsSubclass((Widget)hwndControl, xmToggleButtonWidgetClass))
        {
           // Use the actual button state instead of the callback structure
           // to account for radio button resetting state.
           bool buttonState = XmToggleButtonGetState( hwndControl );
           thisWindow->notifyObservers(
              INotificationEvent(ISettingButton::selectId, *thisWindow, true,
                                 buttonState) );
        }
     }
   } /* end if */
   return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_MOTIF
