// Revision: 44 1.7 source/ui/basectl/isetcvob.hpp, canvas, ioc.v400, 000919  
/* NOSHIP */
#ifndef _ISETCVOB_
  #define _ISETCVOB_
/*******************************************************************************
* FILE NAME: isetcvob.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ISetCanvasObject                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <irect.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IPoint;
class ISize;
class IWindow;

#pragma enum(4)
#pragma pack(push,4)

class ISetCanvasObject {
/*******************************************************************************
* This class is used to represent a child window of a set canvas.  An instance *
* of this class is created for each window in the set canvas, and is added to  *
* a Collection Class library sequence.  These objects are iterated using a     *
* cursor to appropriately size and position all child windows of the canvas in *
* the ISetCanvas::layout() function.                                           *
*                                                                              *
* Example:                                                                     *
* ISetCanvasObject *scvobj = new ISetCanvasObject(pwin);                       *
* ...   (add scvobj to sequence, etc.)                                         *
*******************************************************************************/
public:
/*------------------------ Constructor -----------------------------------------
| There is 1 way to construct instances of this class:                         |
|   1. Create an object from a pointer to a window.                            |
------------------------------------------------------------------------------*/
ISetCanvasObject ( IWindow *childWindow );

/*-------------------------------- Accessors -----------------------------------
| These functions provide the means of getting and setting accessible          |
| attributes of instances of this class:                                       |
|   window         - returns a pointer to the window object assigned to this   |
|                    ISetCanvasObject object                                   |
|   isFirstInDeck  - returns whether the window object represented by this     |
|                    object is the first in its deck                           |
|   deckWidth      - returns the width of vertical deck or height of the       |
|                    horizontal deck that the object belongs to (only accurate |
|                    if the object is the first in the deck)                   |
|   setFirstInDeck - sets whether the window represented by this object is the |
|                    first in its deck (it is assumed that the caller is using |
|                    this function correctly, since no error checking is done) |
|   setDeckWidth   - sets the width of the vertical deck or height of the      |
|                    horizontal deck that the window belongs to                |
------------------------------------------------------------------------------*/
const IWindow
 *window              ( ) const;
bool
  isFirstInDeck       ( ) const;
unsigned long
  deckWidth           ( ) const;
ISetCanvasObject
 &setFirstInDeck      ( bool first = true ),
 &setDeckWidth        ( unsigned long width );

/*------------------------------- Move / Size ----------------------------------
| These functions provide the means of querying and changing the size or       |
| position of a child window:                                                  |
|   topLeft        - returns the upper-left corner position for the window     |
|                    represented by this object                                |
|   size           - returns the width and height for the window represented   |
|                    by this object                                            |
|   rect           - returns the position and size for the window maintained   |
|                    by this object                                            |
|   moveTo         - sets the upper-left corner position for the window        |
|                    represented by this object (no change is made to the size |
|                    of the window)                                            |
|   moveTopTo      - sets the position of the top edge for the window          |
|                    represented by this object (no change is made to the size |
|                    of the window)                                            |
|   moveLeftTo     - sets the position of the left edge for the window         |
|                    represented by this object (no change is made to the size |
|                    of the window)                                            |
|   sizeTo         - sets the height and width for the window represented by   |
|                    this object (no change is made to the top-left corner of  |
|                    the window)                                               |
|   sizeWidthTo    - sets the width for the window represented by this object  |
|                    (no change is made to the top-left corner of the window)  |
|   sizeHeightTo   - sets the height for the window represented by this object |
|                    (no change is made to the top-left corner of the window)  |
------------------------------------------------------------------------------*/
IPoint
  topLeft             ( ) const;
ISize
  size                ( ) const;
IRectangle
  rect                ( ) const;
ISetCanvasObject
 &moveTo              ( const IPoint& pt ),
 &moveTopTo           ( IPair::Coord y ),
 &moveLeftTo          ( IPair::Coord x ),
 &sizeTo              ( const ISize& siz ),
 &sizeWidthTo         ( IPair::Coord width ),
 &sizeHeightTo        ( IPair::Coord height );

private:
/*--------------------------------- Private ----------------------------------*/
IRectangle
  rectCl;
IWindow
 *pwinCl;
bool
  bClFirstInDeck;
unsigned long
  ulClDeckWidth;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

/*----------------------------- Inline Functions -----------------------------*/
#include <isetcvob.inl>

#endif /* ISETCVOB */
