// Revision: 52 1.7 source/ui/basectl/isetcvwd.hpp, canvas, ioc.v400, 000919  
/* NOSHIP */
#ifndef _ISETCVWD_
  #define _ISETCVWD_
/*******************************************************************************
* FILE NAME: isetcvwd.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ISetCanvasWindowData                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iwindow.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <irect.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

class ISetCanvasWindowData : public IWindowData {
typedef IWindowData
  Inherited;
/*******************************************************************************
* This class is used to store data with the child windows of a set canvas.     *
* ISetCanvas uses this data to lay out its child windows.                      *
*******************************************************************************/
public:
/*------------------------ Constructor -----------------------------------------
| There is 1 way to construct instances of this class:                         |
|   1. Create an object from the window handle of the parent set canvas.       |
------------------------------------------------------------------------------*/
  ISetCanvasWindowData ( const IWindowHandle& parentSetCanvasHandle );

/*---------------------------- Layout Contribution -----------------------------
| These functions identify the parent set canvas and whether it lays out the   |
| child window:                                                                |
|   parentHandle     - returns the window handle of the set canvas that        |
|                      created this object, and which uses this object to      |
|                      manage the layout of the child window storing this      |
|                      object                                                  |
|   setParentHandle  - sets the window handle of the parent set canvas of the  |
|                      child window                                            |
|   addToLayout      - explicitly adds the child window to the layout of the   |
|                      parent set canvas                                       |
|   removeFromLayout - removes the child window from the layout of the parent  |
|                      set canvas                                              |
|   isAddedToLayout  - identifies if the child window has been explicitly      |
|                      added to the layout of the parent set canvas, although  |
|                      this does not necessarily identify if the set canvas is |
|                      managing the layout of the child window or whether the  |
|                      child window has not been removed from the set canvas   |
|   isRemovedFromLayout - identifies if the child window has been removed from |
|                      the layout of the parent set canvas, although this does |
|                      not necessarily identify if the set canvas has not been |
|                      added to the set canvas                                 |
------------------------------------------------------------------------------*/
ISetCanvasWindowData
 &setParentHandle     ( const IWindowHandle& handle ),
 &addToLayout         ( ),
 &removeFromLayout    ( );
IWindowHandle
  parentHandle        ( ) const;
bool
  isAddedToLayout     ( ) const,
  isRemovedFromLayout ( ) const;

#if 0                                  // IC_NOTYET
/*------------------------ Caching of Deck Information -------------------------
| These functions support the saving of values from the previous layout of the |
| parent set canvas, for enhancing the performance of subsequent layouts:      |
|   setFirstInDeck - sets whether the child window storing this object is the  |
|                    first in a deck of the parent set canvas                  |
|   setDeckWidth   - sets the width of the vertical deck or height of the      |
|                    horizontal deck that the window belongs to                |
|   isFirstInDeck  - returns whether the window object represented by this     |
|                    object is the first in a deck of the parent set canvas    |
|   deckWidth      - returns the width of vertical deck or height of the       |
|                    horizontal deck that the object belongs to (this value is |
|                    accurate only if the object is the first in the deck)     |
------------------------------------------------------------------------------*/
ISetCanvasWindowData
 &setFirstInDeck      ( bool          first = true ),
 &setDeckWidth        ( unsigned long width );
bool
  isFirstInDeck       ( ) const;
unsigned long
  deckWidth           ( ) const;

/*----------------------- Caching of Window Coordinates ------------------------
| These functions support the saving of sizes and positions of child windows   |
| from the previous layout of the parent set canvas, for enhancing the         |
| performance of subsequent layouts:                                           |
|   topLeftOffset   - returns the offset to the upper-left corner of the       |
|                     window storing this object                               |
|   size            - returns the width and height of the window storing this  |
|                     object                                                   |
|   minimumSize     - returns the minimum size of the window storing this      |
|                     object                                                   |
|   rect            - returns the size and offset to the upper-left corner of  |
|                     the window storing this object                           |
|   saveTopLeftOffset - saves the offset to the upper-left corner of the       |
|                     window storing this object                               |
|   saveTopOffset   - saves the offset of the top edge of the window storing   |
|                     this object                                              |
|   saveLeftOffset  - saves the offset of the left edge of the window storing  |
|                     this object                                              |
|   saveSize        - saves the height and width of the window storing this    |
|                     object                                                   |
|   saveMinimumSize - saves the height and width of the window storing this    |
|                     object                                                   |
------------------------------------------------------------------------------*/
IPoint
  topLeftOffset       ( ) const;
ISize
  size                ( ) const,
  minimumSize         ( ) const;
IRectangle
  rect                ( ) const;
ISetCanvasWindowData
 &saveTopLeftOffset   ( const IPoint& offsetToTopLeft ),
 &saveTopOffset       ( IPair::Coord  offsetToTop ),
 &saveLeftOffset      ( IPair::Coord  offsetToLeft ),
 &saveSize            ( const ISize&  size ),
 &saveMinimumSize     ( const ISize&  minSize );
#endif                                 // IC_NOTYET

private:
/*--------------------------------- Private ----------------------------------*/
enum AddStatus {
  eAddedToLayout,
  eRemovedFromLayout,
  eUnknown
};

IWindowHandle
  fParentHandle;
AddStatus
  fInLayout;
#if 0                                  // IC_NOTYET
bool
  fFirstInDeck;
unsigned long
  fDeckWidth;
IRectangle
  fRectangle;
ISize
  fMinimumSize;
#endif                                 // IC_NOTYET
}; // ISetCanvasWindowData

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

/*----------------------------- Inline Functions -----------------------------*/
#include <isetcvwd.inl>

#endif /* ISETCVWD */
