// Revision: 60 1.7.1.3 source/ui/basectl/ispinbdh.cpp, listctls, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ispinbdh.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinbdh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#include <ilanglvl.hpp>

extern "C" {
  #include <Xm/Xm.h>
  #include <Xm/Text.h>
}

#include <ispinbdh.hpp>
#include <ispbdata.hpp>
#include <ispinbas.hpp>
#include <ispinnum.hpp>
#include <ispintxt.hpp>
#include <ikeyevt.hpp>
#include <ixconst.h>
#include <isizrect.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPINBDH_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBaseSpinDefaultHandler::IBaseSpinDefaultHandler                             |
------------------------------------------------------------------------------*/
IBaseSpinDefaultHandler :: IBaseSpinDefaultHandler() { }

#if IC_EXTENDED_RTTI_ENABLED
IBaseSpinDefaultHandler :: IBaseSpinDefaultHandler(const IBaseSpinDefaultHandler&) { }
#endif // IC_EXTENDED_RTTI_ENABLED

/*------------------------------------------------------------------------------
| IBaseSpinDefaultHandler::~IBaseSpinDefaultHandler                            |
------------------------------------------------------------------------------*/
IBaseSpinDefaultHandler :: ~IBaseSpinDefaultHandler() { }

/*------------------------------------------------------------------------------
| IBaseSpinDefaultHandler::virtualKeyPress                                     |
|                                                                              |
| Process the up and down virtual keys for the XmText widget in the            |
| spin button.                                                                 |
------------------------------------------------------------------------------*/
bool IBaseSpinDefaultHandler :: virtualKeyPress(IKeyboardEvent& event)
{
  IKeyboardEvent::VirtualKey keyPressed = event.virtualKey();
  IBaseSpinButton* spinButton = (IBaseSpinButton*) event.controlWindow();
  Widget spinWidget = (Widget) event.controlHandle();
  XKeyboardState keyboardState;
  IBaseSpinButtonData *pMasterData;

  // Determine whether spin button is already spinning via arrow buttons or
  // "opposite" cursor key.
  XtPointer userData;
  XtVaGetValues (spinButton->fBaseSpinButtonData->efHandle,
                 XmNuserData, &userData,
                 NULL);
  IBaseSpinButtonData::SpinType spinType =
      (IBaseSpinButtonData::SpinType)userData;
  if (spinType != IBaseSpinButtonData::notSpinning)
     // Return true to prevent processing tab characters during spinning.
     return true;

  if (keyPressed == IKeyboardEvent::up ||
      keyPressed == IKeyboardEvent::pageUp ||
      keyPressed == IKeyboardEvent::down ||
      keyPressed == IKeyboardEvent::pageDown)
  {
     // Grab input to insure that we get the key release event.
     XKeyEvent* keyEvent = (XKeyEvent*) event.parameter2().asUnsignedLong(); 
     if (XtGrabKeyboard (spinWidget,     // Grab widget.
                         True,           // Report events to grabbing window.
                         GrabModeSync,   // Pointer mode.
                         GrabModeAsync,  // Keyboard mode.
                         keyEvent->time) // Use XKeyEvent time.
         != GrabSuccess)
        return false;

     if (keyPressed == IKeyboardEvent::up ||
         keyPressed == IKeyboardEvent::pageUp)
        XtVaSetValues (spinButton->fBaseSpinButtonData->efHandle,
                       XmNuserData, (XtPointer)IBaseSpinButtonData::keyUp,
                       NULL);
     else
        XtVaSetValues (spinButton->fBaseSpinButtonData->efHandle,
                       XmNuserData, (XtPointer)IBaseSpinButtonData::keyDown,
                       NULL);
     

     pMasterData = spinButton->fBaseSpinButtonData;

     // Query auto repeat setting and turn off if it is currently on.
     // Since we are grabbing input, just use the global auto repeat.
     XGetKeyboardControl (XtDisplay (spinWidget), &keyboardState);
     if (keyboardState.global_auto_repeat == AutoRepeatModeOn)
     {
        XAutoRepeatOff (XtDisplay (spinWidget));
        spinButton->fBaseSpinButtonData->globalAutoRepeat = true;
     }
     else
        spinButton->fBaseSpinButtonData->globalAutoRepeat = false;

     pMasterData->currentDelay = pMasterData->initialDelay;

     if (keyPressed == IKeyboardEvent::up ||
         keyPressed == IKeyboardEvent::pageUp)
     {
        // Spin up one position.
        if (spinButton->fBaseSpinButtonData->isNumericOnly)
           ((INumericSpinButton*)spinButton)->spinUp();
        else
           ((ITextSpinButton*)spinButton)->spinUp();
    
        pMasterData->timer = XtAppAddTimeOut (
                                XtWidgetToApplicationContext (spinWidget),
                                pMasterData->currentDelay,
                                (XtTimerCallbackProc)ibsbSpinUp,
                                pMasterData);

        // Generate SPBN_UPARROW event.
        spinButton->postEvent (IWindow::control,
                    IEventData ((unsigned short)spinButton->id(), SPBN_UPARROW),
                    IEventData (spinWidget));
     }
     else if (keyPressed == IKeyboardEvent::down ||
              keyPressed == IKeyboardEvent::pageDown)
     {
        // Spin down one position.
        if (spinButton->fBaseSpinButtonData->isNumericOnly)
           ((INumericSpinButton*)spinButton)->spinDown();
        else
           ((ITextSpinButton*)spinButton)->spinDown();
    
        pMasterData->timer = XtAppAddTimeOut (
                                XtWidgetToApplicationContext (spinWidget),
                                pMasterData->currentDelay,
                                (XtTimerCallbackProc)ibsbSpinDown,
                                pMasterData);

        // Generate SPBN_DOWNARROW event.
        spinButton->postEvent (IWindow::control,
                    IEventData ((unsigned short)spinButton->id(), SPBN_DOWNARROW),
                    IEventData (spinWidget));
     }

     // If it is a fast spin button, start timer to increase spin speed after
     // 2 seconds.
     if (spinButton->isFastSpinEnabled())
        pMasterData->accelerationTimer = XtAppAddTimeOut (
                                    XtWidgetToApplicationContext (spinWidget),
                                    2000,    // every 2 seconds
                                    (XtTimerCallbackProc)ibsbFastSpin,
                                    pMasterData);
  }

  return false;
}

/*------------------------------------------------------------------------------
| IBaseSpinDefaultHandler::key                                                 |
|                                                                              |
| Process the key release for the up and down virtual keys.                    |
------------------------------------------------------------------------------*/
bool IBaseSpinDefaultHandler :: key (IKeyboardEvent& event)
{
  IBaseSpinButton* spinButton = (IBaseSpinButton*) event.controlWindow();
  Widget spinWidget = (Widget) event.controlHandle();

  if (event.isVirtual() && event.isUpTransition())
  {
     IKeyboardEvent::VirtualKey keyReleased = event.virtualKey();
     if (keyReleased == IKeyboardEvent::up ||
         keyReleased == IKeyboardEvent::pageUp ||
         keyReleased == IKeyboardEvent::down ||
         keyReleased == IKeyboardEvent::pageDown)
     {
        // Check whether spinning via keyboard.
        XtPointer userData;
        XtVaGetValues (spinButton->fBaseSpinButtonData->efHandle,
                       XmNuserData, &userData,
                       NULL);
        IBaseSpinButtonData::SpinType spinType =
            (IBaseSpinButtonData::SpinType)userData;
        if (((keyReleased == IKeyboardEvent::up ||
                 keyReleased == IKeyboardEvent::pageUp) && 
                 spinType != IBaseSpinButtonData::keyUp) ||
            ((keyReleased == IKeyboardEvent::down ||
                 keyReleased == IKeyboardEvent::pageDown) && 
                 spinType != IBaseSpinButtonData::keyDown))
           return false;

        // Restore global auto repeat.
        if (spinButton->fBaseSpinButtonData->globalAutoRepeat)
           XAutoRepeatOn (XtDisplay (spinWidget));

        // Cancel the spin timer.
        IBaseSpinButtonData *pMasterData = spinButton->fBaseSpinButtonData;

        XtRemoveTimeOut (pMasterData->timer);
        pMasterData->timer = 0;

        // If it is a fast spin button, remove acceleration timer as well.
        if (spinButton->isFastSpinEnabled())
        {
           XtRemoveTimeOut (pMasterData->accelerationTimer);
           pMasterData->accelerationTimer = 0;
        }


        // Remove the input grab.
        XtUngrabKeyboard (spinWidget, CurrentTime);

        XtVaSetValues (spinButton->fBaseSpinButtonData->efHandle,
                       XmNuserData, (XtPointer)IBaseSpinButtonData::notSpinning,
                       NULL);

        // Generate SPBN_ENDSPIN event.
        spinButton->postEvent (IWindow::control,
                    IEventData ((unsigned short)spinButton->id(), SPBN_ENDSPIN),
                    IEventData (spinWidget));
     }
  }

  return false;
} 
  
/*------------------------------------------------------------------------------
| IBaseSpinDefaultHandler::windowResize                                        |
------------------------------------------------------------------------------*/
bool  IBaseSpinDefaultHandler::windowResize(IResizeEvent& event)
{       
                                 
#if 0
  if ( event.eventId() == xEvent(ConfigureNotify) )
  {   
    ISizeRectangle *sizeRect = 
      (ISizeRectangle *) event.parameter1().asUnsignedLong();
                                                 
    // if the size has changed
    if ( sizeRect->sizeChanged() )
    {
      IBaseSpinButton *spinButton = (IBaseSpinButton*)event.controlWindow();
  
      Dimension     margins, efHeight, shadowThickness, highlightThickness;
      unsigned char maxCharHeight = spinButton->characterSize().height();

      // Need to recalculate the margin height.
      XtVaGetValues (spinButton->fBaseSpinButtonData->efHandle,
                     XmNheight,             &efHeight,
                     XmNshadowThickness,    &shadowThickness,
                     XmNhighlightThickness, &highlightThickness,
                     NULL);
      margins = efHeight - maxCharHeight -
                   2 * (shadowThickness + highlightThickness);
      XtVaSetValues (spinButton->fBaseSpinButtonData->efHandle,
                     XmNmarginHeight, margins / 2,
                     NULL);      
    }
  }
#endif          

  return false;
} 
