// Revision: 62 1.10.1.9 source/ui/basectl/ispinnum.cpp, listctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ispinnum.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinnum.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define INCL_WINSTDSPIN
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}

#ifdef IC_WIN
  #include <iclspbw.h>
  #include <commctrl.h>
#endif
#include <ispinnum.hpp>
#ifdef IC_PMWIN
  #include <ispinszh.hpp>
#endif
#include <icconst.h>
#include <iexcept.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <ilanglvl.hpp>
#include <inotifev.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPINNUM_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_MOTIF
#include <X11/IntrinsicP.h>     // for CoreP.h
#include <X11/CoreP.h>          // for core.being_destroyed
#include <Xm/Text.h>
#endif
/*------------------------------------------------------------------------------
| Public spin button styles.                                                   |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
const INumericSpinButton::Style
  INumericSpinButton::padWithZeros      (0, ISPBS_PADWITHZEROS),
  INumericSpinButton::classDefaultStyle ( WS_VISIBLE,
                                          ISPBS_MASTER   |
                                          ISPBS_JUSTLEFT |
                                          ISPBS_NUMERICONLY |
                                          WS_EX_CLIENTEDGE );
#endif
#ifdef IC_MOTIFPM
const INumericSpinButton::Style
  INumericSpinButton::padWithZeros      (0, ISPBS_PADWITHZEROS),
  INumericSpinButton::classDefaultStyle ( WS_VISIBLE,
                                          ISPBS_MASTER   |
                                          ISPBS_JUSTLEFT |
                                          ISPBS_NUMERICONLY );
#endif // IC_MOTIFPM

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
INumericSpinButton::Style
  INumericSpinButton::currentDefaultStyle ( WS_VISIBLE,
                                            ISPBS_MASTER   |
                                            ISPBS_JUSTLEFT |
                                            ISPBS_NUMERICONLY |
                                            WS_EX_CLIENTEDGE );
#endif
#ifdef IC_MOTIFPM
INumericSpinButton::Style
  INumericSpinButton::currentDefaultStyle ( WS_VISIBLE,
                                            ISPBS_MASTER   |
                                            ISPBS_JUSTLEFT |
                                            ISPBS_NUMERICONLY );
#endif // IC_MOTIFPM

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| INumericSpinButtonData class                                                 |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class INumericSpinButtonData {
public:
  INumericSpinButtonData() : isPadWithZeros (false),
                             currentValue (0),
                             maxValue (0),
                             minValue (0),
                             efHandle (0)
  { };

  ~INumericSpinButtonData() { };

  bool
    isPadWithZeros;    // True if pad with zeros.
  long
    currentValue,      // Current numeric value.
    maxValue,          // Max value.
    minValue;          // Min value.

  Widget
    textWidget (Widget topParent);

  void registerCallbacks(INumericSpinButton* spinButton);
  void unregisterCallbacks(INumericSpinButton* spinButton);

  IWindowHandle
    efHandle;          // Handle for the text widget

  friend void _System insbModifyVerifyCallback (Widget w,
                                                void *client_data,
                                                void *call_data );

private:
#if IC_EXTENDED_RTTI_ENABLED
  INumericSpinButtonData::INumericSpinButtonData (const INumericSpinButtonData&) { };
#else
  INumericSpinButtonData::INumericSpinButtonData (const INumericSpinButtonData&);
#endif // IC_EXTENDED_RTTI_ENABLED
  INumericSpinButtonData& operator= (const INumericSpinButtonData&);
};    // INumericSpinButtonData

#pragma pack(pop)
#pragma enum(pop)


/*------------------------------------------------------------------------------
| INumericSpinButtonData::textWidget( Widget topParent )                           |
|                                                                              |
------------------------------------------------------------------------------*/
Widget INumericSpinButtonData :: textWidget ( Widget topParent )
{
  Cardinal numChildren;
  WidgetList children;
  XtVaGetValues( ( Widget) topParent,
    XmNnumChildren, &numChildren,
    XmNchildren, &children,
    NULL );

  for ( int i = 0; i < numChildren; i++ )
    {
      if (XtIsSubclass((Widget)children[i], xmTextWidgetClass))
         {
//         XtManageChild(children[i]);
         return (Widget)children[i];
         }
    }
  return 0;
}

/*------------------------------------------------------------------------------
| INumericSpinButtonData::registerCallbacks                                         |
------------------------------------------------------------------------------*/
void INumericSpinButtonData :: registerCallbacks (INumericSpinButton* spinButton)
{
  XtAddCallback ((Widget) efHandle,
                 XmNmodifyVerifyCallback,
                 insbModifyVerifyCallback,
                 (XtPointer) spinButton);
}

/*------------------------------------------------------------------------------
| INumericSpinButtonData::unregisterCallbacks                                         |
------------------------------------------------------------------------------*/
void INumericSpinButtonData :: unregisterCallbacks (INumericSpinButton* spinButton)
{
  if (spinButton->isValid())
  {
    Widget wHandle = (Widget)spinButton->handle();
    // Only unregister callbacks if widget not to be destroyed
    if (!wHandle->core.being_destroyed)
    {
      XtRemoveCallback ((Widget) efHandle,
                        XmNmodifyVerifyCallback,
                        insbModifyVerifyCallback,
                        (XtPointer) spinButton );
    }
  }
}
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| INumericSpinButton::INumericSpinButton                                       |
|                                                                              |
| Constructor to create a spin button control on a standard window.            |
------------------------------------------------------------------------------*/
INumericSpinButton :: INumericSpinButton ( unsigned long id,
                                           IWindow* parent,
                                           IWindow* pwndOwner,
                                           const IRectangle& initial,
                                           const Style& style )
                          : IBaseSpinButton()
{
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  IWindowHandle owner = (pwndOwner == 0) ? IWindowHandle(0)
                                         : pwndOwner->handle();
  initialize ( id,
               parent->handleForChildCreation(),
               owner,
               convertToGUIStyle( style ),
               initial );
#endif // IC_PMWIN

#ifdef IC_MOTIF
  unsigned long sbStyle = style.asUnsignedLong();
  IWindowHandle hwndOwner = 0;

  fNumericSpinButtonData = new INumericSpinButtonData;

  if (pwndOwner != 0)
     hwndOwner = pwndOwner->handle();

  initialize (id,
              parent->handleForChildCreation(),
              hwndOwner,
              convertToGUIStyle( style ),
              initial);

  if (XtIsSubclass((Widget)handle(), xmTextWidgetClass))
    fNumericSpinButtonData->efHandle = handle();
  else
    fNumericSpinButtonData->efHandle = fNumericSpinButtonData->textWidget(handle());

  // Register callbacks
  fNumericSpinButtonData->registerCallbacks(this);

  // Remember is this is pad with zeros.
  fNumericSpinButtonData->isPadWithZeros = (extendedStyle() & ISPBS_PADWITHZEROS) ? true : false;

  // Set the initial value to zero.
  setValue (0);
#endif // IC_MOTIF
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| INumericSpinButton::INumericSpinButton                                       |
|                                                                              |
| Constructor to create a spin button control on a standard window.            |
------------------------------------------------------------------------------*/
INumericSpinButton :: INumericSpinButton ( unsigned long id, IWindow* parent )
      :  IBaseSpinButton()
{
   IWindowHandle
     hwnd = IWindow::handleWithParent( id, parent->handle() );

   if ( !(ISTYLEOF(hwnd) & SPBS_NUMERICONLY) )
      ITHROWLIBRARYERROR(IC_NUMSPIN_WRAP_INVALID,
                         IBaseErrorInfo::invalidRequest, IException::recoverable);
   setAutoDestroyWindow(false);
   setExtendedStyle(
     ISpinButtonStatics::convertToExtendedStyle(ISTYLEOF(hwnd)));
   initialize();
   reserveUserWindowWord( false );
   startHandlingEventsFor( hwnd );
}

/*------------------------------------------------------------------------------
| INumericSpinButton::INumericSpinButton                                       |
|                                                                              |
| Constructor to instantiate spin button object from window handle.            |
------------------------------------------------------------------------------*/
INumericSpinButton :: INumericSpinButton ( const IWindowHandle& handle )
     :  IBaseSpinButton()
{
   if ( !(ISTYLEOF(handle) & SPBS_NUMERICONLY) )
      ITHROWLIBRARYERROR(IC_NUMSPIN_WRAP_INVALID,
                         IBaseErrorInfo::invalidRequest, IException::recoverable);
   setAutoDestroyWindow(false);
   setExtendedStyle(
     ISpinButtonStatics::convertToExtendedStyle(ISTYLEOF(handle)));
   initialize();
   reserveUserWindowWord( false );
   startHandlingEventsFor(handle);
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| INumericSpinButton::~INumericSpinButton                                      |
------------------------------------------------------------------------------*/
INumericSpinButton::~INumericSpinButton ( )
{
#ifdef IC_MOTIF
  fNumericSpinButtonData->unregisterCallbacks(this);
  delete fNumericSpinButtonData;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| INumericTextSpinButton::defaultStyle                                         |
|                                                                              |
| Return the default style for new spin button objects.                        |
------------------------------------------------------------------------------*/
INumericSpinButton::Style INumericSpinButton::defaultStyle()
{
  return currentDefaultStyle;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| INumericTextSpinButton::setDefaultStyle                                      |
|                                                                              |
| Sets the default style for new spin button objects.                          |
------------------------------------------------------------------------------*/
void INumericSpinButton::setDefaultStyle( const INumericSpinButton::Style& stsStyle )
{
  currentDefaultStyle = stsStyle;
}
#endif

/*------------------------------------------------------------------------------
| INumericSpinButton::convertToGUIStyle                                        |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long INumericSpinButton::convertToGUIStyle( const IBitFlag& guiStyle,
                                                     bool bExtOnly ) const
{
  // Obtain the style from the class that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    unsigned long guiExtUL = guiStyle.asExtendedUnsignedLong();
#ifdef IC_WIN
    if ( guiExtUL & IBaseSpinButton::pmCompatible.asExtendedUnsignedLong() )
#endif
    {
       // PM Compatible spinbutton
       ulStyle |= SPBS_NUMERICONLY;
       if ( guiExtUL & INumericSpinButton::padWithZeros.asExtendedUnsignedLong())
          ulStyle |= SPBS_PADWITHZEROS;

    }
#ifdef IC_WIN
    else
    {
       // In native Windows up/down control, we use these flags in
       // our wrapper control.
       ulStyle |= INSPBS_NUMERICONLY;
       if ( guiExtUL & INumericSpinButton::padWithZeros.asExtendedUnsignedLong())
          ulStyle |= INSPBS_PADWITHZEROS;
    }
#endif
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| INumericSpinButton::setRange                                                 |
|                                                                              |
| Sets the lower and upper range of the spin button numbers.  If the value in  |
| the spin field does not match the range, PM sets the spin field to the       |
| closest limit.  If the range is empty, PM puts 0 in the spin field.  If      |
| override is set and the current value in the spin field is outside the       |
| new range, the current value in the spin field will not change.              |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton::setRange( const IRange& aNewRange,
                                                  bool override )
{
  IMODTRACE_DEVELOP("INumericSpinButton::setRange");

  int lower = aNewRange.lowerBound();
  int upper = aNewRange.upperBound();

#ifdef IC_PMWIN
  // If lower is greater than upper, operation will fail.
  IASSERTPARM( lower <= upper );

  // Check if this is a pmCompatible spin button.
  if (this->isPMCompatible())
  {
    if (override)
      handle().sendEvent( SPBM_OVERRIDESETLIMITS,
                          IEventParameter1( upper ),
                          IEventParameter2( lower ));
    else
      handle().sendEvent( SPBM_SETLIMITS,
                          IEventParameter1( upper ),
                          IEventParameter2( lower ));
  }
#ifdef IC_WIN
  // Otherwise native Up/Down control
  else
  {
    // Native control has tighter range limits, so check those first
    IASSERTPARM( (lower >= UD_MINVAL) && (upper <= UD_MAXVAL)
                  && ((upper - lower) <= UD_MAXVAL) );

    long currentValue = this->value();
    IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
    IEventData newRange( (short)upper, (short)lower );
    ISETRANGE( this->handle(), newRange.asLong() );

    // If current spinbutton not active one don't mess with updown
    if ( (updown.asUnsigned()) 
// This comparison is invalid if spinbutton is client of a frame
//        &&
//         (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
//          ISpinButtonStatics::buddyHandle( this->handle() ) ) 
       )
    {
      updown.sendEvent( UDM_SETRANGE,
                        IEventParameter1(0),
                        newRange );
    }
    // If override, leave current value as is
    if (!override)
    {
      // Otherwise, move the current value inside the new range limits
      if ( currentValue < lower )
        this->spinTo( lower );
      else if ( currentValue > upper )
        this->spinTo( upper );
    }
    this->refresh();
  }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (lower > upper)
  {
     ITHROWLIBRARYERROR (IC_VALUE_OUT_OF_RANGE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
  }

  fNumericSpinButtonData->minValue = lower;
  fNumericSpinButtonData->maxValue = upper;

  // Reset the current value if it is outside the new range and override
  // was not requested.
  if (!override)
     if (value() < fNumericSpinButtonData->minValue)
        setValue (fNumericSpinButtonData->minValue);
     else if (value() > fNumericSpinButtonData->maxValue)
        setValue (fNumericSpinButtonData->maxValue);
  else if (!fNumericSpinButtonData->minValue && !fNumericSpinButtonData->maxValue)
     setValue (0);
#endif // IC_MOTIF

  return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::range                                                    |
|                                                                              |
| Query lower and upper bounds of the numeric range.                           |
------------------------------------------------------------------------------*/
IRange INumericSpinButton :: range ( ) const
{
   IMODTRACE_DEVELOP("INumericSpinButton::range");
#ifdef IC_PMWIN
   long lLow = 0;
   long lHigh = 0;
   if (this->isPMCompatible())
   {
     unsigned long ulOk = handle().sendEvent(SPBM_QUERYLIMITS,
                                             IEventParameter1((void*)&lHigh),
                                             IEventParameter2((void*)&lLow));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_QUERYLIMITS",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IEventResult result = IQUERYRANGE( this->handle() );
     lLow = (short)result.highNumber();
     lHigh = (short)result.lowNumber();
   }
#endif
   return IRange(lLow, lHigh);
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return IRange (fNumericSpinButtonData->minValue, fNumericSpinButtonData->maxValue);
#endif // IC_MOTIF

}

/*------------------------------------------------------------------------------
| INumericSpinButton::value                                                    |
|                                                                              |
| Return the number in the spin field.                                         |
------------------------------------------------------------------------------*/
long INumericSpinButton :: value ( ) const
{
   IMODTRACE_DEVELOP("INumericSpinButton::value");

#ifdef IC_PMWIN
   IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
   unsigned long ulClBufLen =
         (unsigned long)IQUERYWINDOWTEXTLENGTH(ef) + 1;
   char* pszClBuf = new char[ulClBufLen];
   IQUERYWINDOWTEXT(ef, ulClBufLen, pszClBuf);
   IString str(pszClBuf);
   delete [] pszClBuf;
   return str.asInt();
#endif // IC_PMWIN

#ifdef IC_MOTIF
  char    *spinValue;
  XtVaGetValues (fNumericSpinButtonData->efHandle,
                 XmNvalue, &spinValue,
                 NULL);

  return IString (spinValue).asInt();
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| INumericSpinButton::setValue                                                 |
|                                                                              |
| Set the spin field with the specified value.                                 |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: setValue ( long aValue )
{
  IMODTRACE_DEVELOP("INumericSpinButton::setValue");

#ifdef IC_PMWIN
  if (aValue != value())
  {
    IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
    ITRACE_DEVELOP(IString("Value to be set is ") + IString(aValue));
    ISETWINDOWTEXT(ef, (const char*)IString(aValue));
#ifdef IC_WIN
    if (!this->isPMCompatible())
    {
      if ( this->range().includes( aValue ) )
      {
         IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
         // If current spinbutton not active one don't mess with updown
         if ( (updown.asUnsigned()) &&
              (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
               ISpinButtonStatics::buddyHandle( this->handle() ) ) )
            updown.sendEvent( UDM_SETPOS,
                              IEventParameter1(0),
                              IEventParameter2((short)aValue, 0) );
         ISETLASTPOS(this->handle(), aValue);
      }
    }

    // send the edit field changed message.
    IWindowHandle parent = IPARENTOF( this->handle() );
    parent.sendEvent(
          WM_COMMAND,
          IEventParameter1( (short)this->id() , SPBN_CHANGE),
          IEventParameter2( this->handle().asUnsigned() ) );
#endif
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  unsigned long width = limit();
  if (width > 11)
    width = 11;

  // Set the text field regardless of validity of data.
  IString newValue (aValue);
  if (newValue.length() > limit())
     newValue.remove (limit() + 1);
  if (fNumericSpinButtonData->isPadWithZeros)
  {
     if (newValue[1] == '-')
        newValue = "-" +
                   newValue.subString(2).rightJustify (width - 1, '0');
     else
        newValue.rightJustify (width, '0');
  }

  // Set text value; be sure that columns > 0 to avoid Motif warning message.
  short columns;
  Arg   args[5];
  int   n = 0;
  XtVaGetValues (fNumericSpinButtonData->efHandle,
                 XmNcolumns, &columns,
                 NULL);
  if (columns < 1)
  {
     XtSetArg (args[n], XmNcolumns, 1); n++;
  }
  XtSetArg (args[n], XmNvalue, (char*)newValue); n++;
  XtSetValues (fNumericSpinButtonData->efHandle, args, n);

  this->notifyObservers(INotificationEvent(
    IBaseSpinButton::valueId, *this));

#endif // IC_MOTIF

  return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::spinTo                                                   |
|                                                                              |
| Set the spin field to the specified numeric value.                           |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: spinTo ( long    aValue,
                                                   bool spinToClosest )
{
   IMODTRACE_DEVELOP("INumericSpinButton::spinTo");

#ifdef IC_PMWIN
   if (this->isPMCompatible())
   {
     unsigned long ulOk =
          handle().sendEvent(SPBM_SETCURRENTVALUE,
                             IEventParameter1((int)aValue),
                             IEventParameter2(0));
     if (!ulOk)
     {
        if (spinToClosest)
        {
           long lUpper, lLower, lValue;
           ulOk = handle().sendEvent( SPBM_QUERYLIMITS,
                                      IEventParameter1( &lUpper ),
                                      IEventParameter2( &lLower ));
           if (!ulOk)
              ITHROWGUIERROR2( "SPBM_QUERYLIMITS",
                               IBaseErrorInfo::invalidParameter,
                               IException::recoverable );
           if (aValue < lLower)
              lValue = lLower;
           else
              lValue = lUpper;
           ulOk = handle().sendEvent( SPBM_SETCURRENTVALUE,
                                      IEventParameter1( (int)lValue ),
                                      IEventParameter2( 0 ));
        }
     }
     if (!ulOk)
     {
        ITHROWGUIERROR2("SPBM_SETCURRENTVALUE",
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
     }
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IRange range = this->range();
     if ( !range.includes( aValue ) )
        {
        if (spinToClosest)
           {
           if ( aValue > range.upperBound() )
              aValue = range.upperBound();
           else
              aValue = range.lowerBound();
           }
        else
           {
           ITHROWGUIERROR2("UDM_SETPOS",
                           IBaseErrorInfo::invalidParameter,
                           IException::recoverable);
           }
        }
     IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
           ISpinButtonStatics::buddyHandle( this->handle() ) ) )
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2((short)aValue, 0) );
     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)aValue ),
                      IEventParameter2( updown.asUnsigned() ) );
   }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  long newSpinValue = aValue;
  if (spinToClosest)
     if (aValue < fNumericSpinButtonData->minValue)
        newSpinValue = fNumericSpinButtonData->minValue;
     else if (aValue > fNumericSpinButtonData->maxValue)
        newSpinValue = fNumericSpinButtonData->maxValue;
  else if (aValue < fNumericSpinButtonData->minValue || aValue > fNumericSpinButtonData->maxValue)
  {
     ITHROWLIBRARYERROR (IC_VALUE_OUT_OF_RANGE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
  }
  return setValue (newSpinValue);
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::isSpinFieldValid                                         |
|                                                                              |
| Validate that the contents of the spin field are in the spin field number    |
| range.                                                                       |
------------------------------------------------------------------------------*/
bool INumericSpinButton :: isSpinFieldValid (
                                         bool /*caseSensitive*/) const
{
   IMODTRACE_DEVELOP("INumericSpinButton::isSpinFieldValid");

#ifdef IC_PMWIN
   unsigned long ulOk;
   if (this->isPMCompatible())
   {
     long lValue;
     ulOk = handle().sendEvent(SPBM_QUERYVALUE,
                               IEventParameter1((void*)&lValue),
                               IEventParameter2(0,
                                                SPBQ_DONOTUPDATE));
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IRange range = this->range();
     ulOk = range.includes( this->value() );
   }
#endif

   return (ulOk) ? true : false;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  bool valid = fNumericSpinButtonData->minValue <= value() && value() <= fNumericSpinButtonData->maxValue;

  return valid;
#endif // IC_MOTIF

}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| INumericSpinButton::setLimit                                                 |
|                                                                              |
| Sets the number of characters permitted in a spin field.                     |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: setLimit ( unsigned long aNumber )
{
  IMODTRACE_DEVELOP("INumericSpinButton::setLimit");

  Inherited::setLimit (aNumber);

  char* currentValue;
  XtVaGetValues (fNumericSpinButtonData->efHandle,
                 XmNvalue, &currentValue,
                 NULL);
  IString curString = IString(currentValue);
  unsigned long curLength = curString.length();
  if (curLength > aNumber || (fNumericSpinButtonData->isPadWithZeros && curLength != aNumber))
     setValue (curString.asInt());

  return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::spinDown                                                 |
|                                                                              |
| Spin the button down (backward) the specified amount of times.               |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: spinDown ( unsigned long spinBy )
{
  IMODTRACE_DEVELOP("INumericSpinButton::spinDown");

  long rangeExtent, netSpin;

  // If the number in the spin field is valid and it is not equal to the
  // current value, set the current value to the spin field value.
  if (isSpinFieldValid())
     fNumericSpinButtonData->currentValue = value();

  rangeExtent = fNumericSpinButtonData->maxValue - fNumericSpinButtonData->minValue + 1;
  netSpin = spinBy % rangeExtent;
  if (fNumericSpinButtonData->currentValue - netSpin < fNumericSpinButtonData->minValue)
     fNumericSpinButtonData->currentValue += rangeExtent - netSpin;
  else
     fNumericSpinButtonData->currentValue -= netSpin;
  setValue (fNumericSpinButtonData->currentValue);
  return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::spinUp                                                   |
|                                                                              |
| Spin the button up (forward) the specified amount of times.                  |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: spinUp   ( unsigned long spinBy )
{
  IMODTRACE_DEVELOP("INumericSpinButton::spinUp");

  int rangeExtent, netSpin;

  // If the number in the spin field is valid and it is not equal to the
  // current value, set the current value to the spin field value.
  if (isSpinFieldValid())
     fNumericSpinButtonData->currentValue = value();

  rangeExtent = fNumericSpinButtonData->maxValue - fNumericSpinButtonData->minValue + 1;
  netSpin = spinBy % rangeExtent;
  if (fNumericSpinButtonData->currentValue + netSpin > fNumericSpinButtonData->maxValue)
     fNumericSpinButtonData->currentValue += netSpin - rangeExtent;
  else
     fNumericSpinButtonData->currentValue += netSpin;
  setValue (fNumericSpinButtonData->currentValue);

  return *this;
}

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  isbModifyVerifyCallback is called before text is deleted from or inserted   |
|  into the XmText widget in the ISpinButton.  The callback is used to force   |
|  only numeric data for an ISpinButton with the numericOnly style.            |
------------------------------------------------------------------------------*/
extern void _System insbModifyVerifyCallback (Widget w,
                                              XtPointer client_data,
                                              XtPointer call_data)
{
   XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *) call_data;
  
   if (cbs->text->length == 0 ||                     // deletion
       cbs->startPos < cbs->currInsert)              // backspace
      return;
 
   IString spinText = IString (cbs->text->ptr, cbs->text->length);
   if (!spinText.isDigits())
   {
     INumericSpinButton* spinButton = (INumericSpinButton*)client_data;
     IString spinVal = IString(spinButton->value());

     // if 1st character not '-'
     if ( !(spinText[1] == '-') )
       cbs->doit = False;
     // if length is 1, we know that value from keyboard, so check
     // if '-' not 1st position of text field or '-' already exists in spin button
     else if ((cbs->text->length == 1) && (cbs->startPos != 0 || spinVal.includes("-")))
       cbs->doit = False;
     // Make sure remainder of string valid numeric
     else if (cbs->text->length > 1)  
     {     
       if ( !(spinText.subString(2, cbs->text->length - 1).isDigits()))
         cbs->doit = False;
     }
   }
   return;
}
#pragma options info=reset
#endif // IC_MOTIF
