// Revision: 67 1.9.1.6 source/ui/basectl/ispintxt.cpp, listctls, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ispintxt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispintxt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define INCL_WINSTDSPIN
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
#ifdef IC_WIN
  #include <iclspbw.h>
  #include <commctrl.h>
#endif
}

#include <ispintxt.hpp>
#ifdef IC_PMWIN
#include <ispinszh.hpp>
#endif
#include <icconst.h>
#include <iexcept.hpp>
#include <inotifev.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <itrace.hpp>

#ifdef IC_MOTIF
#include <Xm/Text.h>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISPINTXT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public spin button styles.                                                   |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
const ITextSpinButton::Style
  ITextSpinButton::classDefaultStyle ( WS_VISIBLE,
                                       ISPBS_MASTER   |
                                       ISPBS_JUSTLEFT |
                                       WS_EX_CLIENTEDGE );
#endif
#ifdef IC_MOTIFPM
const ITextSpinButton::Style
  ITextSpinButton::classDefaultStyle ( WS_VISIBLE,
                                       ISPBS_MASTER   |
                                       ISPBS_JUSTLEFT );
#endif

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
ITextSpinButton::Style
  ITextSpinButton::currentDefaultStyle ( WS_VISIBLE,
                                         ISPBS_MASTER   |
                                         ISPBS_JUSTLEFT |
                                         WS_EX_CLIENTEDGE );
#endif
#ifdef IC_MOTIFPM
ITextSpinButton::Style
  ITextSpinButton::currentDefaultStyle ( WS_VISIBLE,
                                         ISPBS_MASTER   |
                                         ISPBS_JUSTLEFT );
#endif

/*------------------------------------------------------------------------------
| The spin button private data class.                                          |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ITextSpinButtonData {
public:
  ITextSpinButtonData ( );
 ~ITextSpinButtonData ( );

unsigned long
  textCount;
//#ifdef IC_PMWIN
unsigned long
  fChangeCount;
//#endif // IC_PMWIN
#ifdef IC_MOTIF
unsigned long
  position;          // Current position (0-based).

Widget
  textWidget (Widget topParent);

IWindowHandle
  efHandle;          // handle for the text widget
#endif // IC_MOTIF
IString
  spinText;
#ifdef IC_PMWIN
char
 **pList;
#endif // IC_PMWIN
}; // ITextSpinButtonData

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| ITextSpinButtonData::ITextSpinButtonData                                     |
------------------------------------------------------------------------------*/
ITextSpinButtonData::ITextSpinButtonData( )
  : textCount(0)
//#ifdef IC_PMWIN
  , fChangeCount(0)
//#endif // IC_PMWIN
#ifdef IC_MOTIF
  , position(0)
  , efHandle(0)
#endif // IC_MOTIF
  , spinText()
#ifdef IC_PMWIN
  , pList(0)
#endif // IC_PMWIN
{ }

/*------------------------------------------------------------------------------
| ITextSpinButtonData::~ITextSpinButtonData                                    |
------------------------------------------------------------------------------*/
ITextSpinButtonData::~ITextSpinButtonData( )
{
#ifdef IC_PMWIN
  if (pList)
    delete [] pList;
#endif // IC_PMWIN
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ITextSpinButtonData::textWidget( Widget topParent )                          |
|                                                                              |
------------------------------------------------------------------------------*/
Widget ITextSpinButtonData :: textWidget ( Widget topParent )
{
  Cardinal numChildren;
  WidgetList children;
  XtVaGetValues( ( Widget) topParent,
    XmNnumChildren, &numChildren,
    XmNchildren, &children,
    NULL );

  for ( int i = 0; i < numChildren; i++ )
    {
      if (XtIsSubclass((Widget)children[i], xmTextWidgetClass))
         {
//         XtManageChild(children[i]);
         return (Widget)children[i];
         }
    }
  return 0;
}
#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| The spin button cursor private data class.                                   |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ITextSpinButtonCursorData {
public:
  ITextSpinButtonCursorData (const ITextSpinButton& spinbt)
    : lCurrent(0),
//#ifdef IC_PMWIN
      fValidCount(0),
//#endif // IC_PMWIN
#ifdef IC_MOTIF
      ulCurCount (spinbt.fTextSpinButtonData->textCount),
#endif // IC_MOTIF
      rSpBut(spinbt)
  { }
long
  lCurrent;
//#ifdef IC_PMWIN
unsigned long
  fValidCount;
//#endif // IC_PMWIN
#ifdef IC_MOTIF
unsigned long
  ulCurCount;
#endif // IC_MOTIF
const ITextSpinButton
 &rSpBut;
private:
  ITextSpinButtonCursorData ( const ITextSpinButtonCursorData& cursor );
ITextSpinButtonCursorData
 &operator=  ( const ITextSpinButtonCursorData& cursor );
}; // ITextSpinButtonCursorData

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| ITextSpinButton::ITextSpinButton                                             |
|                                                                              |
| Constructor to create a spin button control on a standard window.            |
------------------------------------------------------------------------------*/
ITextSpinButton :: ITextSpinButton ( unsigned long id,
                                     IWindow* parent,
                                     IWindow* pwndOwner,
                                     const IRectangle& initial,
                                     const Style& style )
               :   IBaseSpinButton()
{
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  if (this->isPMCompatible())
  {
    if (style.asUnsignedLong() & SPBS_NUMERICONLY)
       ITHROWLIBRARYERROR(IC_TXTSPIN_WRAP_INVALID,
                          IBaseErrorInfo::invalidRequest, IException::recoverable);
  }

  IWindowHandle owner = (pwndOwner == 0) ? IWindowHandle(0)
                                         : pwndOwner->handle();
   initialize ( id,
                parent->handleForChildCreation(),
                owner,
                convertToGUIStyle( style ),
                initial );
   fTextSpinButtonData = new ITextSpinButtonData();
#ifdef IC_WIN
   if (!(this->isPMCompatible()))
     ISETEMPTYFLAG(this->handle(),1);
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  fTextSpinButtonData = new ITextSpinButtonData();

  IWindowHandle hwndOwner = 0;
  if (pwndOwner != 0)
     hwndOwner = pwndOwner->handle();

  initialize ( id,
               parent->handleForChildCreation(),
               hwndOwner,
               convertToGUIStyle( style ),
               initial );

  if (XtIsSubclass((Widget)handle(), xmTextWidgetClass))
    fTextSpinButtonData->efHandle = handle();
  else
    fTextSpinButtonData->efHandle = fTextSpinButtonData->textWidget(handle());

#endif // IC_MOTIF

}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| ITextSpinButton::ITextSpinButton                                             |
|                                                                              |
| Constructor to create a spin button control on a standard window.            |
------------------------------------------------------------------------------*/
ITextSpinButton :: ITextSpinButton ( unsigned long id, IWindow* parentDialog )
      :  IBaseSpinButton()
{
   IWindowHandle
     hwnd = IWindow::handleWithParent( id, parentDialog->handle() );

   if ( ISTYLEOF(hwnd) & SPBS_NUMERICONLY )
      ITHROWLIBRARYERROR(IC_TXTSPIN_WRAP_INVALID,
                         IBaseErrorInfo::invalidRequest, IException::recoverable);
   setAutoDestroyWindow(false);
   setExtendedStyle(
     ISpinButtonStatics::convertToExtendedStyle(ISTYLEOF(hwnd)));
   initialize();
   fTextSpinButtonData = new ITextSpinButtonData();
   reserveUserWindowWord( false );
   startHandlingEventsFor( hwnd );
}

/*------------------------------------------------------------------------------
| ITextSpinButton::ITextSpinButton                                             |
|                                                                              |
| Constructor to instantiate spin button object from window handle.            |
------------------------------------------------------------------------------*/
ITextSpinButton :: ITextSpinButton ( const IWindowHandle& handle )
     :  IBaseSpinButton()
{
   if ( ISTYLEOF(handle) & SPBS_NUMERICONLY )
      ITHROWLIBRARYERROR(IC_TXTSPIN_WRAP_INVALID,
                         IBaseErrorInfo::invalidRequest, IException::recoverable);
   setAutoDestroyWindow(false);
   setExtendedStyle(
     ISpinButtonStatics::convertToExtendedStyle(ISTYLEOF(handle)));
   initialize();
   fTextSpinButtonData = new ITextSpinButtonData();
   reserveUserWindowWord( false );
   startHandlingEventsFor(handle);
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| ITextSpinButton::~ITextSpinButton                                            |
------------------------------------------------------------------------------*/
ITextSpinButton::~ITextSpinButton ( )
{
   delete fTextSpinButtonData;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::defaultStyle                                                |
|                                                                              |
| Return the default style for new spin button objects.                        |
------------------------------------------------------------------------------*/
ITextSpinButton::Style  ITextSpinButton :: defaultStyle()
{
  return currentDefaultStyle;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITextSpinButton::setDefaultStyle                                             |
|                                                                              |
| Sets the default style for new spin button objects.                          |
------------------------------------------------------------------------------*/
void ITextSpinButton::setDefaultStyle( const ITextSpinButton::Style& stsStyle )
{
  currentDefaultStyle = stsStyle;
}
#endif

/*------------------------------------------------------------------------------
| ITextSpinButton::convertToGUIStyle                                           |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ITextSpinButton :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   bool bExtOnly) const
{
  // Obtain the style from the class that we inherit from.
  // IBaseSpinButton does all the work.
  return Inherited::convertToGUIStyle( guiStyle, bExtOnly );

}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ITextSpinButton::setLimit                                                    |
|                                                                              |
| Sets the number of characters permitted in a spin field.                     |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: setLimit ( unsigned long aNumber )
{
  Inherited::setLimit (aNumber);

  if (text().length() > limit())
     setText (text());

  setLayoutDistorted (IWindow::minimumSizeChanged, 0);
  return *this;
}
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| ITextSpinButton::text                                                        |
|                                                                              |
| Query the spin field text                                                    |
------------------------------------------------------------------------------*/
IString ITextSpinButton :: text  ( ) const
{
#ifdef IC_PMWIN
   IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );

   unsigned long ulClBufLen =
         (unsigned long)IQUERYWINDOWTEXTLENGTH(ef) + 1;

   char* pszClBuf = new char[ulClBufLen];
   IQUERYWINDOWTEXT(ef, ulClBufLen, pszClBuf);
   IString str(pszClBuf);
   delete [] pszClBuf;
   return str;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  char *text;

  XtVaGetValues (fTextSpinButtonData->efHandle,
                 XmNvalue, &text,
                 NULL);
  return IString (text);
#endif // IC_MOTIF

}

/*------------------------------------------------------------------------------
| ITextSpinButton::setText                                                     |
|                                                                              |
| Set the spin field text.                                                     |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: setText ( const char* string )
{
#ifdef IC_PMWIN
   IString strOrigText = this->text();
   if (strOrigText != string)
   {
#ifdef IC_PM
      IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
      ISETWINDOWTEXT(ef, string);
#endif //IC_PM
#ifdef IC_WIN
      IEventResult result = handle().sendEvent( IC_UM_EDIT_SETTEXT,
                                     IEventParameter1( (char *) string ),
                                     IEventParameter2( handle() ) );
      // if settext not handled by text change handler
      if (!result.asUnsignedLong())
      {
        IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
        ISETWINDOWTEXT(ef, string);
      }
      // If this is a native spin button, update the internal position
      // if the string matches one of the values in the spin array
      bool found = false;
      if (!this->isPMCompatible() )
      {
        if (fTextSpinButtonData->textCount)
        for (int i = 0; i < fTextSpinButtonData->textCount && !found; i++)
        {
          if (elementAt(i) == string)
          {
            found = true;
            ISETLASTPOS( this->handle(), i );
          }
        }
      }
      // Send the edit field changed message.
      IWindowHandle parent = IPARENTOF( this->handle() );
      parent.sendEvent(
           WM_COMMAND,
           IEventParameter1( (short)this->id(), SPBN_CHANGE),
           IEventParameter2( this->handle().asUnsigned()) );
#endif //IC_WIN
   }
#endif //IC_PMWIN

#ifdef IC_MOTIF
  IString newText (string);
  if (newText.length() > limit())
     newText.remove (limit() + 1);

  // Set text value; be sure that columns > 0 to avoid Motif warning message.
  short columns;
  Arg   args[5];
  int   n = 0;
  XtVaGetValues (fTextSpinButtonData->efHandle,
                 XmNcolumns, &columns,
                 NULL);
  if (columns < 1)
  {
     XtSetArg (args[n], XmNcolumns, 1); n++;
  }
  XtSetArg (args[n], XmNvalue, (String)newText); n++;
  XtSetValues (fTextSpinButtonData->efHandle, args, n);

  this->notifyObservers(INotificationEvent(
    IBaseSpinButton::textId, *this));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::setText                                                     |
|                                                                              |
| Set the spin field text.                                                     |
------------------------------------------------------------------------------*/
ITextSpinButton&  ITextSpinButton :: setText(const IResourceId& residText)
{
  IString str = residText.resourceLibrary().loadString(residText);
  return setText((char*)str);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::isSpinFieldValid                                            |
|                                                                              |
| Validate that the contents of the spin field match one of the spin text      |
| array values.                                                                |
------------------------------------------------------------------------------*/
bool ITextSpinButton :: isSpinFieldValid ( bool caseSensitive ) const
{
   IMODTRACE_DEVELOP("ITextSpinButton::isSpinFieldValid");

#ifdef IC_PMWIN
   long lValue;
   unsigned long valid = 0;
   if ((caseSensitive) || (!this->isPMCompatible()) )
   {
      IString str = text();
      if (!caseSensitive)
         str = str.upperCase();
      if (fTextSpinButtonData->textCount)
         for (int i = 0; i < fTextSpinButtonData->textCount && !valid; i++)
            {
            if ((caseSensitive) && (elementAt(i) == str))
               valid = true;
            else if ((!caseSensitive) && (elementAt(i).upperCase() == str))
               valid = true;
            }
   }
   else
   {
      valid = handle().sendEvent(SPBM_QUERYVALUE,
                                 IEventParameter1((void*)&lValue),
                                 IEventParameter2(0,SPBQ_DONOTUPDATE));
   }

   return (valid) ? true : false;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  bool valid = false;

  if (fTextSpinButtonData->textCount)
     for (int i = 0; i < fTextSpinButtonData->textCount && !valid; i++)
        if ((caseSensitive && elementAt(i) == text()) ||
            (!caseSensitive && elementAt(i).upperCase() == text().upperCase()))
           valid = true;

  return valid;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITextSpinButton::elementAt                                                   |
|                                                                              |
| Return the text element at cursor position.                                  |
------------------------------------------------------------------------------*/
IString ITextSpinButton::elementAt ( const Cursor& cursor ) const
{
   IMODTRACE_DEVELOP("ITextSpinButton::elementAt");
   IASSERTPARM(cursor.isValid());
   unsigned lIndex = findStringIndex(cursor);
   return IString((char*)fTextSpinButtonData->spinText.subString(lIndex));
}

/*------------------------------------------------------------------------------
| ITextSpinButton::elementAt                                                   |
|                                                                              |
| Return the text element at the index position.                               |
------------------------------------------------------------------------------*/
IString ITextSpinButton::elementAt ( unsigned long index ) const
{
   IMODTRACE_DEVELOP("ITextSpinButton::elementAt");
   IASSERTPARM(index < fTextSpinButtonData->textCount);
   unsigned lIndex = findStringIndex(index);
   return IString((char*)fTextSpinButtonData->spinText.subString(lIndex));
}

/*------------------------------------------------------------------------------
| ITextSpinButton::replaceAt                                                   |
|                                                                              |
| Replace the text element at the cursor position.                             |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::replaceAt  ( const IResourceId& resid,
                                               Cursor &cursor )
{
   IString newText = resid.resourceLibrary().loadString(resid);
   return replaceAt(newText,cursor);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::replaceAt                                                   |
|                                                                              |
| Replace the text element at the cursor position.                             |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::replaceAt  ( const char* newString,
                                               Cursor &cursor )
{
   IMODTRACE_DEVELOP("ITextSpinButton::replaceAt");
   ITRACE_DEVELOP( "Cursor at item: " + IString( cursor.fCursorData->lCurrent ));

   IASSERTPARM(cursor.isValid());

#ifdef IC_PMWIN
   //   remove the item at the cursor
   unsigned lIndex = findStringIndex(cursor);
   IString subStr = (char*)fTextSpinButtonData->spinText.subString(lIndex);
   fTextSpinButtonData->spinText.remove(lIndex, subStr.length()+1);
   --fTextSpinButtonData->textCount;
   if ( cursor.fCursorData->lCurrent > fTextSpinButtonData->textCount )
   {          // Replacing the last item.
      addAsLast(newString);
      cursor.setToLast();
   }
   else if (fTextSpinButtonData->textCount == 0 || cursor.fCursorData->lCurrent == 1)
   {          // Replacing the first item.
      addAsFirst(newString);
      cursor.setToFirst();
   }
   else
   {          // Replacing an item in the middle somewhere.
      --cursor.fCursorData->lCurrent;
      addAsNext(newString, cursor);
   }

   // Undo the add's increment of the change count, since we don't
   // want other cursors to be invalidated by a replace.
   --fTextSpinButtonData->fChangeCount;
   cursor.fCursorData->fValidCount = fTextSpinButtonData->fChangeCount;  // Keep cursor valid.
#endif // IC_PMWIN

#ifdef IC_MOTIF
  bool bLast = false;
  if (cursor.fCursorData->lCurrent == fTextSpinButtonData->textCount)
     bLast = true;

  unsigned long lIndex1, lIndex2;

  lIndex1 = findStringIndex (cursor);
  lIndex2 = fTextSpinButtonData->spinText.indexOf ('\0', lIndex1);

  fTextSpinButtonData->spinText.remove (lIndex1, lIndex2 - lIndex1 + 1);
  cursor.fCursorData->ulCurCount--;                //Decrement count
  fTextSpinButtonData->textCount--;

  if (fTextSpinButtonData->textCount == 0 ||
      cursor.fCursorData->lCurrent == 1)
  {
     addAsFirst (newString);
     cursor.setToFirst();
     if (fTextSpinButtonData->position == 0)
        setText (newString);
  }
  else if (bLast)
  {
     addAsNext (newString, cursor);
     cursor.setToLast();
     if (fTextSpinButtonData->position == fTextSpinButtonData->textCount - 1)
        setText (newString);
  }
  else
  {
     add (newString, cursor);
  }
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::removeAt                                                    |
|                                                                              |
| Remove the text element at cursor position.                                  |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::removeAt( Cursor& cursor )
{
   IMODTRACE_DEVELOP("ITextSpinButton::removeAt");
   IASSERTPARM(cursor.isValid());

#ifdef IC_PMWIN
   unsigned lIndex = findStringIndex(cursor);
   IString subStr = (char*)fTextSpinButtonData->spinText.subString(lIndex);
   fTextSpinButtonData->spinText.remove(lIndex, subStr.length()+1);
   --fTextSpinButtonData->textCount;            // One less element now.
   ++fTextSpinButtonData->fChangeCount;

   if ( cursor.fCursorData->lCurrent <= fTextSpinButtonData->textCount )
   {          // Did not remove last element.
      cursor.fCursorData->fValidCount = fTextSpinButtonData->fChangeCount;  // Keep cursor valid.
      ITRACE_DEVELOP( "Cursor at  = " + IString( cursor.fCursorData->lCurrent ));
   }
   else
   {     // Let cursor remain invalidated, since we removed the last element.
      ITRACE_DEVELOP( "Removed item " + IString( cursor.fCursorData->lCurrent ));
      ITRACE_DEVELOP( "Cursor now invalid." );
   }

   ITRACE_DEVELOP( "New number of items = " + IString( fTextSpinButtonData->textCount ));
   refreshText();
#endif // IC_PMWIN

#ifdef IC_MOTIF
  unsigned long lIndex1, lIndex2;

  lIndex1 = findStringIndex (cursor);
  lIndex2 = fTextSpinButtonData->spinText.indexOf ('\0', lIndex1);

  fTextSpinButtonData->spinText.remove (lIndex1, lIndex2 - lIndex1 + 1);
  cursor.fCursorData->ulCurCount--;                //Decrement count
  fTextSpinButtonData->textCount--;

  if (fTextSpinButtonData->position &&
      (fTextSpinButtonData->position == fTextSpinButtonData->textCount))
  {
     // Adjust the current position if it is now out of range.
     fTextSpinButtonData->position--;
     setText (elementAt (fTextSpinButtonData->position));
  }
  else if (!fTextSpinButtonData->textCount)
     // Clear the text field if the only item was removed.
     setText ("");
  else if (cursor.fCursorData->lCurrent - 1 <=
           fTextSpinButtonData->position)
     // Set the text field, if the item at the current position was removed or
     // followed the removed item.
     setText (elementAt (fTextSpinButtonData->position));

  if (cursor.fCursorData->lCurrent > fTextSpinButtonData->textCount)
     cursor.invalidate();
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::removeAll                                                   |
|                                                                              |
| Remove all spin text elements.                                               |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: removeAll ( )
{
#ifdef IC_PMWIN
   char* nullList[1];
   nullList[0] = "";
   if (this->isPMCompatible())
   {
     unsigned long ulOk =
          handle().sendEvent(SPBM_SETARRAY,
                             IEventParameter1(nullList),
                             IEventParameter2(1));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SETARRAY",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) 
// This comparison is invalid if spinbutton is client of a frame
//         &&
//          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
//           ISpinButtonStatics::buddyHandle( this->handle() ) ) 
        )
     {
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2( 0, 0 ) );
        updown.sendEvent( UDM_SETRANGE,
                          IEventParameter1(0),
                          IEventParameter2(0,0) );
     }

     ISETRANGE(this->handle(), 0 );
     ISETEMPTYFLAG(this->handle(),1);
     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, 0 ),
                      IEventParameter2( updown.asUnsigned() ) );
     refresh();
   }
#endif

   fTextSpinButtonData->spinText = "";
   fTextSpinButtonData->textCount = 0;
   if (fTextSpinButtonData->pList)
   {
      delete [] fTextSpinButtonData->pList;
      fTextSpinButtonData->pList = 0;
   }
   ++fTextSpinButtonData->fChangeCount;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  fTextSpinButtonData->textCount = fTextSpinButtonData->position = 0;
  fTextSpinButtonData->spinText = "";
  setText ("");
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::add                                                         |
|                                                                              |
| Add a new item at the cursor location by pushing all the following items     |
| after the cursor.                                                            |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: add ( const char* string, Cursor& cursor )
{
   IMODTRACE_DEVELOP("ITextSpinButton::add");
   IASSERTPARM(cursor.isValid());

#ifdef IC_PMWIN
   ITRACE_DEVELOP( "Previous number of items = " + IString( fTextSpinButtonData->textCount ));
   ITRACE_DEVELOP( "Cursor previously at item: " + IString( cursor.fCursorData->lCurrent ));

   if (cursor.fCursorData->lCurrent == 1)
   {
      addAsFirst(string);
      cursor.setToFirst();
   }
   else
   {
      unsigned ulLen1 = findStringIndex(cursor) - 1;
      unsigned ulLen2 = strlen(string) + 1;
      unsigned ulLen3 = fTextSpinButtonData->spinText.length() - ulLen1;
      fTextSpinButtonData->spinText = IString((char*)fTextSpinButtonData->spinText,
                                ulLen1,
                                string,
                                ulLen2,
                                (char*)fTextSpinButtonData->spinText.subString(ulLen1 + 1),
                                ulLen3);
      ++fTextSpinButtonData->textCount;                     // Increment counts.
      ++fTextSpinButtonData->fChangeCount;
      cursor.fCursorData->fValidCount = fTextSpinButtonData->fChangeCount;  // Keep cursor valid.
      refreshText();
   }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  unsigned long ulLen1 = findStringIndex(cursor) - 1;
  unsigned long ulLen2 = strlen(string) + 1;
  unsigned long ulLen3 = fTextSpinButtonData->spinText.length() - ulLen1;
  fTextSpinButtonData->spinText =
    IString((char*)fTextSpinButtonData->spinText,
                   ulLen1,
                   string,
                   ulLen2,
                   (char*)fTextSpinButtonData->spinText.subString (ulLen1 + 1),
                   ulLen3);
  cursor.fCursorData->ulCurCount++;             //Increment counts
  fTextSpinButtonData->textCount++;

  // Set the text if the item at the current position is new or follows the
  // new one.
  if (cursor.fCursorData->lCurrent >= fTextSpinButtonData->position + 1)
     setText (elementAt (fTextSpinButtonData->position));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::add                                                         |
|                                                                              |
| Add a new item at the cursor location by pushing all the following items     |
| after the cursor.                                                            |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: add ( const IResourceId& resid,
                                          Cursor& cursor )
{
   IString newText = resid.resourceLibrary().loadString(resid);
   return add(newText, cursor);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::add                                                         |
|                                                                              |
| Add a new item at the index location by pushing all the following items      |
| after the index.                                                             |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: add ( const char* string,
                                          unsigned long index )
{
   IMODTRACE_DEVELOP("ITextSpinButton::add");

#ifdef IC_PMWIN
   if ( index >= fTextSpinButtonData->textCount )
   {
      addAsLast(string);
   }
   else
   {
      unsigned ulLen1 = findStringIndex(index) - 1;
      unsigned ulLen2 = strlen(string) + 1;
      unsigned ulLen3 = fTextSpinButtonData->spinText.length() - ulLen1;
      fTextSpinButtonData->spinText = IString((char*)fTextSpinButtonData->spinText,
                                ulLen1,
                                string,
                                ulLen2,
                                (char*)fTextSpinButtonData->spinText.subString(ulLen1 + 1),
                                ulLen3);
      ++fTextSpinButtonData->textCount;
      ++fTextSpinButtonData->fChangeCount;
      refreshText();
   }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (index >= fTextSpinButtonData->textCount)
     addAsLast(string);
  else
  {
     unsigned long ulLen1 = findStringIndex(index) - 1;
     unsigned long ulLen2 = strlen(string) + 1;
     unsigned long ulLen3 = fTextSpinButtonData->spinText.length() - ulLen1;
     fTextSpinButtonData->spinText =
       IString((char*)fTextSpinButtonData->spinText,
                  ulLen1,
                  string,
                  ulLen2,
                  (char*)fTextSpinButtonData->spinText.subString(ulLen1 + 1),
                  ulLen3);
     fTextSpinButtonData->textCount++;
  }

  if (fTextSpinButtonData->position >= index ||
      fTextSpinButtonData->textCount == 1)
     setText (elementAt (fTextSpinButtonData->position));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::add                                                         |
|                                                                              |
| Add a new item at the index location by pushing all the following items      |
| after the index.                                                             |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: add ( const IResourceId& resid,
                                          unsigned long index )
{
   IString newText = resid.resourceLibrary().loadString(resid);
   return add(newText, index);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::add                                                         |
|                                                                              |
| Add a list of new items at the index location by pushing all the following   |
| items after the index.                                                       |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: add ( const char* const * stringArray,
                                          unsigned long index,
                                          unsigned long count )
{
   IMODTRACE_DEVELOP("ITextSpinButton::add");

#ifdef IC_PMWIN
   if (count == 0)
      return *this;

   IString newString;
   for (int i=0; i < count; i++ )
   {
      newString+=stringArray[i];
      newString+=IString('\0');
   }
   if ( index >= fTextSpinButtonData->textCount )
      fTextSpinButtonData->spinText += newString;
   else if ( index == 0)
      fTextSpinButtonData->spinText = newString + fTextSpinButtonData->spinText;
   else
   {
      unsigned ulLen1 = findStringIndex(index) - 1;
      unsigned ulLen2 = newString.length();
      unsigned ulLen3 = fTextSpinButtonData->spinText.length() - ulLen1;
      fTextSpinButtonData->spinText = IString((char*)fTextSpinButtonData->spinText,
                                ulLen1,
                                (char*)newString,
                                ulLen2,
                                (char*)fTextSpinButtonData->spinText.subString(ulLen1 + 1),
                                ulLen3);
   }
   fTextSpinButtonData->textCount += count;
   ++fTextSpinButtonData->fChangeCount;
   refreshText();
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (count == 0)
     return *this;

  IString newString;
  for (int i = 0; i < count; i++ )
  {
     newString += stringArray[i];
     newString += IString('\0');
  }
  if (index >= fTextSpinButtonData->textCount)
     fTextSpinButtonData->spinText += newString;
  else
  {
     unsigned long ulLen1 = findStringIndex(index) - 1;
     unsigned long ulLen2 = newString.length();
     unsigned long ulLen3 = fTextSpinButtonData->spinText.length() - ulLen1;
     fTextSpinButtonData->spinText =
       IString((char*)fTextSpinButtonData->spinText,
                  ulLen1,
                  (char*)newString,
                  ulLen2,
                  (char*)fTextSpinButtonData->spinText.subString(ulLen1 + 1),
                  ulLen3);
  }
  fTextSpinButtonData->textCount += count;

  // If the item at the current position changed, reset the text.
  if (fTextSpinButtonData->position >= index ||
      fTextSpinButtonData->textCount == count)
     setText (elementAt (fTextSpinButtonData->position));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::addAsNext                                                   |
|   Add a text next to the current cursor position.                            |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: addAsNext  ( const char*   string,
                                                 Cursor& cursor )
{
   IMODTRACE_DEVELOP("ITextSpinButton::addAsNext");
   IASSERTPARM(cursor.isValid());

#ifdef IC_PMWIN
   if (cursor.fCursorData->lCurrent == fTextSpinButtonData->textCount)
   {
      addAsLast(string);
      cursor.setToLast();
   }
   else
   {
      ++cursor.fCursorData->lCurrent;   // bump the cursor.
      add(string, cursor);
   }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  // Add the item following the cursor.  (lCurrent is one-based; indices are
  // zero-based.
  add (string, cursor.fCursorData->lCurrent);
  cursor.fCursorData->ulCurCount++;
  cursor.fCursorData->lCurrent++;
#endif // IC_MOTIF

   return *this;
}

ITextSpinButton& ITextSpinButton :: addAsNext  ( const IResourceId& resid,
                                                 Cursor& cursor )
{
   IString newText = resid.resourceLibrary().loadString(resid);
#ifdef IC_PMWIN
   return add(newText,cursor);
#endif // IC_PMWIN
#ifdef IC_MOTIF
  return addAsNext (newText, cursor);
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITextSpinButton::addAsFirst                                                  |
|                                                                              |
| Adds the text as the first item.                                             |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::addAsFirst ( const IResourceId& resid )
{
   IString newText = resid.resourceLibrary().loadString(resid);
   return addAsFirst(newText);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::addAsFirst                                                  |
|                                                                              |
| Adds the text as the first item.                                             |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::addAsFirst ( const char* string )
{
   IMODTRACE_DEVELOP("ITextSpinButton::addAsFirst");

#ifdef IC_PMWIN
   unsigned ulLen1 = strlen(string) + 1;
   unsigned ulLen2 = fTextSpinButtonData->spinText.length();
   if (ulLen2)
      fTextSpinButtonData->spinText = IString(string, ulLen1,
                          (char*)fTextSpinButtonData->spinText, ulLen2);
   else
      fTextSpinButtonData->spinText = IString(string, ulLen1);
   ++fTextSpinButtonData->textCount;
   ++fTextSpinButtonData->fChangeCount;
   refreshText();
#endif // IC_PMWIN

#ifdef IC_MOTIF
  unsigned long ulLen1 = strlen(string) + 1;
  unsigned long ulLen2 = fTextSpinButtonData->spinText.length();
  if (ulLen2)
     fTextSpinButtonData->spinText =
       IString(string, ulLen1, (char*)fTextSpinButtonData->spinText, ulLen2);
  else
     fTextSpinButtonData->spinText = IString(string, ulLen1);
  fTextSpinButtonData->textCount++;

  setText (elementAt (fTextSpinButtonData->position));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::addAsLast                                                   |
|                                                                              |
| Adds the text as the last item.                                              |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::addAsLast  ( const IResourceId& resid )
{
   IString newText = resid.resourceLibrary().loadString(resid);
   return addAsLast(newText);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::addAsLast                                                   |
|                                                                              |
| Adds the text as the last item.                                              |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::addAsLast  ( const char* string )
{
   IMODTRACE_DEVELOP("ITextSpinButton::addAsLast");

#ifdef IC_PMWIN
   unsigned ulLen1 = fTextSpinButtonData->spinText.length();
   unsigned ulLen2 = strlen(string) + 1;
   if (ulLen1)
      fTextSpinButtonData->spinText = IString(
          (char*)fTextSpinButtonData->spinText, ulLen1, string, ulLen2);
   else
      fTextSpinButtonData->spinText = IString(string, ulLen2);
   ++fTextSpinButtonData->textCount;
   ++fTextSpinButtonData->fChangeCount;
   refreshText();
#endif // IC_PMWIN

#ifdef IC_MOTIF
  unsigned long ulLen1 = fTextSpinButtonData->spinText.length();
  unsigned long ulLen2 = strlen(string) + 1;
  if (ulLen1)
     fTextSpinButtonData->spinText =
       IString((char*)fTextSpinButtonData->spinText, ulLen1, string, ulLen2);
  else
     fTextSpinButtonData->spinText = IString(string, ulLen2);
  fTextSpinButtonData->textCount++;

  if (fTextSpinButtonData->textCount == 1)
     setText (elementAt (0));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::spinTo                                                      |
|                                                                              |
|   Set the current text to the text at the cursor position.                   |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: spinTo ( const Cursor& cursor )
{
   IASSERTPARM(cursor.isValid());
   return (spinTo((unsigned long)(cursor.fCursorData->lCurrent - 1)));
}

/*------------------------------------------------------------------------------
| ITextSpinButton::spinTo                                                      |
|                                                                              |
|   Set the current text to the text at the index position.                    |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: spinTo ( unsigned long index )
{
   IMODTRACE_DEVELOP("ITextSpinButton::spinTo");

#ifdef IC_PMWIN
   if (this->isPMCompatible())
   {
     handle().sendEvent(SPBM_SETCURRENTVALUE,
                        IEventParameter1(index),
                        IEventParameter2(0));
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
           ISpinButtonStatics::buddyHandle( this->handle() ) ) )
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2((short)index, 0) );

     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)index ),
                      IEventParameter2( updown.asUnsigned() ) );
   }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (index >= fTextSpinButtonData->textCount)
     ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
  fTextSpinButtonData->position = index;
  setText ((char*)elementAt (index));
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::spinTo                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: spinTo ( const char* string, bool caseSensitive )
{
   IMODTRACE_DEVELOP("ITextSpinButton::spinTo");

#ifdef IC_PMWIN
   IString arrayString;
   IString inputString(string);
   if (!caseSensitive)
       inputString.upperCase();
   int j;
   bool bFound = false;
   for (int i=0; i < fTextSpinButtonData->textCount && bFound==false; i++ )
   {
       arrayString = elementAt(i);
       if (!caseSensitive)
           arrayString.upperCase();
       if (arrayString == inputString)
       {
          j=i;
          bFound = true;
       }
   }
   if (bFound)
   {
     if (this->isPMCompatible())
     {
        handle().sendEvent(SPBM_SETCURRENTVALUE,
                           IEventParameter1(j),
                           IEventParameter2(0));
     }
#ifdef IC_WIN
     else
     {
        // Native Up/Down control
        IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
        // If current spinbutton not active one don't mess with updown
        if ( (updown.asUnsigned()) &&
             (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
              ISpinButtonStatics::buddyHandle( this->handle() ) ) )
           updown.sendEvent( UDM_SETPOS,
                             IEventParameter1(0),
                             IEventParameter2((short)j, 0) );

        // Generate a message to be sent.  This simulates the message that
        // is sent when the user clicks on the arrow buttons.
        this->sendEvent( IC_UM_VSCROLL,
                         IEventParameter1( SB_THUMBPOSITION, (short)j ),
                         IEventParameter2( updown.asUnsigned() ) );

     }
#endif
   }
   else
   {
      ITHROWLIBRARYERROR(IC_SPINTO_TEXT_INVALID,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
   }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  IString arrayString;
  IString inputString (string);
  if (!caseSensitive)
     inputString.upperCase();
  int j;
  bool bFound = false;
  for (int i = 0; i < fTextSpinButtonData->textCount && !bFound; i++ )
  {
     arrayString = elementAt (i);
     if (!caseSensitive)
        arrayString.upperCase();
     if (arrayString == inputString)
     {
        j=i;
        bFound = true;
     }
  }
  if (bFound)
  {
     fTextSpinButtonData->position = j;
     setText (elementAt (j));
  }
  else
     ITHROWLIBRARYERROR (IC_SPINTO_TEXT_INVALID,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
#endif // IC_MOTIF

   return *this;
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| ITextSpinButton::refreshText                                                 |
|                                                                              |
| Adds or replaces elements in the spin button.                                |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::refreshText ( )
{
   IMODTRACE_DEVELOP("ITextSpinButton::refreshText");

   if ( fTextSpinButtonData->textCount )
   {

      if (fTextSpinButtonData->pList)
      {
         delete [] fTextSpinButtonData->pList;
      }

      fTextSpinButtonData->pList = new char*[fTextSpinButtonData->textCount];
      unsigned int lIndex = 0;

      for (int ix = 0; ix < fTextSpinButtonData->textCount ; ix++ )
      {
                                       //Use the string buffer address to setup
                                       //pList char* address.
         fTextSpinButtonData->pList[ix] = (char*)fTextSpinButtonData->spinText + lIndex;
         lIndex = fTextSpinButtonData->spinText.indexOf('\0', lIndex + 1);
      }

      if (this->isPMCompatible())
      {
        unsigned long ulOk =
             handle().sendEvent(SPBM_SETARRAY,
                                IEventParameter1((char*)fTextSpinButtonData->pList),
                                IEventParameter2(fTextSpinButtonData->textCount));
        if (!ulOk)
            ITHROWLIBRARYERROR(IC_SET_SPIN_ARRAY_FAIL,
                               IBaseErrorInfo::invalidRequest,
                               IException::recoverable);
      }
#ifdef IC_WIN
      else
      {
        // Native Up/Down control

        // Position the up/down control
        IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
        // If current spinbutton not active one don't mess with updown
        if ( (updown.asUnsigned()) &&
             (IQUERYACTIVEBUDDY( IPARENTOF( updown )) !=
              ISpinButtonStatics::buddyHandle( this->handle() ) ) )
           updown = 0;
        IEventResult pos;
        if (updown.asUnsigned())
           pos = updown.sendEvent( UDM_GETPOS );
        else
           pos = IQUERYRANGE( this->handle() );

        if ( pos.highNumber() ||
             (fTextSpinButtonData->textCount > UD_MAXVAL) )
           ITHROWLIBRARYERROR(IC_SET_SPIN_ARRAY_FAIL,
                              IBaseErrorInfo::invalidRequest,
                              IException::recoverable);

        IEventData newRange( (short)(fTextSpinButtonData->textCount-1), 0 );

        if (updown.asUnsigned())
           updown.sendEvent( UDM_SETRANGE,
                             IEventParameter1(0),
                             newRange );
        ISETRANGE(this->handle(), newRange.asLong() );
        ISETEMPTYFLAG(this->handle(),0);

        unsigned newpos = pos.lowNumber();
        if ( newpos >= fTextSpinButtonData->textCount )
           {
           newpos = (unsigned) (fTextSpinButtonData->textCount - 1);
           if (updown.asUnsigned())
              updown.sendEvent( UDM_SETPOS,
                                IEventParameter1(0),
                                IEventParameter2( (short)newpos, 0 ) );
           }

        // Generate a message to be sent.  This simulates the message that
        // is sent when the user clicks on the arrow buttons.
        this->sendEvent( IC_UM_VSCROLL,
                         IEventParameter1( SB_THUMBPOSITION, (short)newpos ),
                         IEventParameter2( updown.asUnsigned() ) );
        refresh();
      }
#endif

   }
   else                                //If we have removed all of the
                                       //elements simply call removeAll
      removeAll();

   return *this;
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| ITextSpinButton::findStringIndex                                             |
|                                                                              |
| Locate the index of the string corresponding to the cursor position.         |
------------------------------------------------------------------------------*/
unsigned ITextSpinButton::findStringIndex(const Cursor& cursor) const
{
   // Assume cursor is validated by the caller.
   return this->findStringIndex( cursor.fCursorData->lCurrent - 1 );
}

/*------------------------------------------------------------------------------
| ITextSpinButton::findStringIndex                                             |
|                                                                              |
| Locate the index of the string corresponding to the index position.          |
------------------------------------------------------------------------------*/
unsigned ITextSpinButton::findStringIndex( unsigned long index ) const
{
   IMODTRACE_DEVELOP("ITextSpinButton::findStringIndex");
   /***************************************************/
   /* Loop to locate index of string to parse string. */
   /***************************************************/
   unsigned lIndex = 1;
   for (int i=1; i < index + 1; i++ ) {
     lIndex = fTextSpinButtonData->spinText.indexOf('\0', lIndex);
     ++lIndex;
     ITRACE_DEVELOP( "Index = " + IString( lIndex ));
   }
   return lIndex;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::Cursor::Cursor                                              |
|                                                                              |
| Cursor constructor.                                                          |
------------------------------------------------------------------------------*/
ITextSpinButton::Cursor::Cursor(const ITextSpinButton& spBut )
{
   IMODTRACE_DEVELOP("ITextSpinButton::Cursor::ctor");
   fCursorData = new ITextSpinButtonCursorData(spBut);
}

/*------------------------------------------------------------------------------
| ITextSpinButton::Cursor::~Cursor                                             |
------------------------------------------------------------------------------*/
ITextSpinButton::Cursor::~Cursor ( )
{
   delete fCursorData;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::Cursor::setToFirst                                          |
|                                                                              |
| Set the cursor to the first element.                                         |
------------------------------------------------------------------------------*/
bool ITextSpinButton::Cursor::setToFirst ()
{
  if ( fCursorData->rSpBut.fTextSpinButtonData->textCount )
  {      // Spin button has a first element to point to.
     fCursorData->fValidCount =
                  fCursorData->rSpBut.fTextSpinButtonData->fChangeCount;
#ifdef IC_MOTIF
     fCursorData->ulCurCount =
                  fCursorData->rSpBut.fTextSpinButtonData->textCount;
#endif // IC_MOTIF
     fCursorData->lCurrent = 1;
     return true;
  }
  else
  {
     fCursorData->lCurrent = 0;
     return false;
  }
}

/*------------------------------------------------------------------------------
| ITextSpinButton::Cursor::setToNext                                           |
|                                                                              |
| Set the cursor to the next element.                                          |
------------------------------------------------------------------------------*/
bool ITextSpinButton::Cursor :: setToNext ()
{
  if ( ! this->isValid() )
  {          // Reset invalid cursor to first item.
     return this->setToFirst();
  }
#ifdef IC_PMWIN
  else if ( fCursorData->lCurrent <
                    fCursorData->rSpBut.fTextSpinButtonData->textCount )
  {          // Point to next item.
     ++fCursorData->lCurrent;
     return true;
  }
#endif // IC_PMWIN
#ifdef IC_MOTIF
  else if (this->isValid() && fCursorData->lCurrent < fCursorData->ulCurCount)
     {
        fCursorData->lCurrent++;
        return true;
     }
#endif // IC_MOTIF
  else
  {          // No more items, so invalidate the cursor.
     fCursorData->lCurrent = 0;
     return false;
  }
}

/*------------------------------------------------------------------------------
| ITextSpinButton::Cursor::setToPrevious                                       |
|                                                                              |
| Set the cursor to the previous element.                                      |
------------------------------------------------------------------------------*/
bool ITextSpinButton::Cursor :: setToPrevious ()
{
  if ( ! this->isValid() )
  {          // Reset invalid cursor to last item.
     return this->setToLast();
  }
  else if ( fCursorData->lCurrent > 1 )
  {          // Point to previous item.
     --fCursorData->lCurrent;
     return true;
  }
  else
  {          // No more items, so invalidate the cursor.
     fCursorData->lCurrent = 0;
     return false;
  }
}

/*------------------------------------------------------------------------------
| ITextSpinButton::Cursor::setToLast                                           |
|                                                                              |
| Set the cursor to the last element.                                          |
------------------------------------------------------------------------------*/
bool ITextSpinButton::Cursor :: setToLast ()
{
#ifdef IC_PMWIN
  if ( fCursorData->rSpBut.fTextSpinButtonData->textCount )
  {      // Spin button has a last element to point to.
     fCursorData->fValidCount =
                  fCursorData->rSpBut.fTextSpinButtonData->fChangeCount;
     fCursorData->lCurrent =
                  fCursorData->rSpBut.fTextSpinButtonData->textCount;
     return true;
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  if ((fCursorData->ulCurCount =
      fCursorData->rSpBut.fTextSpinButtonData->textCount) != 0)
  {
     //Validate the cursor
     fCursorData->lCurrent = fCursorData->ulCurCount;
     return true;
  }
#endif // IC_MOTIF
  else
  {
     fCursorData->lCurrent = 0;
     return false;
  }
}

/*------------------------------------------------------------------------------
  Invalidate the cursor
------------------------------------------------------------------------------*/
void ITextSpinButton::Cursor :: invalidate ()
{
   fCursorData->lCurrent = 0;
}

/*------------------------------------------------------------------------------
  Is the cursor valid?
------------------------------------------------------------------------------*/
bool ITextSpinButton::Cursor :: isValid () const
{
#ifdef IC_PMWIN
   return ( fCursorData->lCurrent > 0 && fCursorData->fValidCount ==
       fCursorData->rSpBut.fTextSpinButtonData->fChangeCount) ? true : false;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return (fCursorData->lCurrent > 0 &&
          fCursorData->ulCurCount ==
          fCursorData->rSpBut.fTextSpinButtonData->textCount) ? true : false;
#endif // IC_MOTIF
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ITextSpinButton::spinDown                                                    |
|                                                                              |
| Spin the button down (backward) the specified amount of times.               |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: spinDown ( unsigned long spinBy )
{
  IMODTRACE_DEVELOP("ITextSpinButton::spinDown");

  if (!fTextSpinButtonData->textCount)
     return *this;

  // If the text in the spin field is valid and it is not equal to the
  // text pointed to by the current position, adjust the current position
  // prior to the spin.
  if (text() != elementAt (fTextSpinButtonData->position) &&
      fTextSpinButtonData->textCount)
     for (int i = 0; i < fTextSpinButtonData->textCount; i++)
        if (IString (fTextSpinButtonData->spinText[i]).upperCase() ==
              text().upperCase())
        {
           fTextSpinButtonData->position = i;
           break;
        }

  if (((long)(fTextSpinButtonData->position) -
       (long)(spinBy % fTextSpinButtonData->textCount)) < 0)
     // At first item in list, force wrap.
     fTextSpinButtonData->position +=
       fTextSpinButtonData->textCount -
       (spinBy % fTextSpinButtonData->textCount);
  else
     fTextSpinButtonData->position -= spinBy % fTextSpinButtonData->textCount;
  setText (elementAt (fTextSpinButtonData->position));

  return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButton::spinUp                                                      |
|                                                                              |
| Spin the button up (forward) the specified amount of times.                  |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton :: spinUp   ( unsigned long spinBy )
{
  IMODTRACE_DEVELOP("ITextSpinButton::spinUp");

  if (!fTextSpinButtonData->textCount)
     return *this;

  // If the text in the spin field is valid and it is not equal to the
  // text pointed to by the current position, adjust the current position
  // prior to the spin.
  if (text() != elementAt (fTextSpinButtonData->position) &&
      fTextSpinButtonData->textCount)
     for (int i = 0; i < fTextSpinButtonData->textCount; i++)
        if (IString (fTextSpinButtonData->spinText[i]).upperCase() ==
              text().upperCase())
        {
           fTextSpinButtonData->position = i;
           break;
        }

  if (fTextSpinButtonData->position + (spinBy % fTextSpinButtonData->textCount) >
        fTextSpinButtonData->textCount - 1)
     // At last item in list, force wrap.
     fTextSpinButtonData->position +=
      (spinBy % fTextSpinButtonData->textCount) -
      fTextSpinButtonData->textCount;
  else
     fTextSpinButtonData->position += spinBy % fTextSpinButtonData->textCount;
  setText (elementAt (fTextSpinButtonData->position));

  return *this;
}
#endif // IC_MOTIF
