// Revision: 77 1.16 source/ui/basectl/ispltbar.cpp, canvas, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: ispltbar.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispltbar.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#include <ilanglvl.hpp>

extern "C"
  {
  #include <Xm/Xm.h>
  #include <Xm/Form.h>
  #include <Xm/SashP.h>
  #include <Xm/Separator.h>
  }

#include <ispltbar.hpp>
#include <icolor.hpp>
#include <ihandler.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>
#include <icconst.h>
#ifdef IC_MOTIF
  #include <isizrect.hpp>
  #include <ixconst.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPLITCV_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Public canvas styles.                                                        |
|                                                                              |
| Notes: The below style bits must not conflict with those used by inherited   |
|          class.                                                              |
------------------------------------------------------------------------------*/
#define PSBS_VERT    0x00000001L
#define PSBS_HORZ    0x00000002L
const ISplitBar::Style ISplitBar::vertical(PSBS_VERT);
const ISplitBar::Style ISplitBar::horizontal(PSBS_HORZ);
const ISplitBar::Style ISplitBar::classDefaultStyle =
                                         (WS_VISIBLE | PSBS_VERT);


#define OFFSET 10

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
ISplitBar::Style ISplitBar::currentDefaultStyle =
                                      ISplitBar::classDefaultStyle;

#pragma enum(4)
#pragma pack(push,4)

class ISplitBarHandler : public IHandler {
typedef IHandler
  Inherited;
public:

/*------------------------------- Constructors -------------------------------*/
  ISplitBarHandler ( );

virtual
  ~ISplitBarHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual bool
  dispatchHandlerEvent ( IEvent& event );

}; // class ISplitBarHandler

class ISplitBarPrivate  {
public:
  unsigned long
    ulClSeparatorThickness;

  bool
    bClVertical;

  IWindowHandle
    iwhClSash,
    iwhClSeparator,
    iwhClForm;
  IWindow
    *iwClPrevControl,
    *iwClNextControl;
  ISplitBarHandler
    fdefaultHandler;
};

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| ISplitBarHandler::ISplitBarHandler                                           |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
ISplitBarHandler::ISplitBarHandler()
{ }

/*------------------------------------------------------------------------------
| ISplitBarHandler::~ISplitBarHandler                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISplitBarHandler::~ISplitBarHandler()
{ }

/*------------------------------------------------------------------------------
| ISplitBarHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Intercepts ConfigureNotify (WM_SIZE) events                                  |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
bool ISplitBarHandler::dispatchHandlerEvent( IEvent& event )
{
  bool  result = false;

  switch( event.eventId() )
  {
    case xEvent(ConfigureNotify):
    {
      ISizeRectangle* sizeRect =
         (ISizeRectangle*)event.parameter1().asUnsignedLong();
      // if the size has changed then run WM_SIZE logic
      if ( sizeRect->newRectangle() != IRectangle() ) // Sizing to 0,0 gives
                                                      // me problems
      {
//      if (sizeRect->sizeChanged() )
//      {
          // offset might have changed
          bool isSplitBar=false;
          IWindowHandle wh = (IWindowHandle) event.controlHandle();
          if ( (wh) && (XmIsForm((Widget)wh)) )
            {
            Cardinal numChildren;
            WidgetList children;
            XtVaGetValues ((Widget) wh,
              XmNnumChildren, &numChildren,
              XmNchildren, &children,
              NULL);
            for (int i=0; i < numChildren; i++)
              {
                if ( (children) &&
                     ( XmIsSash((Widget)children[i]) ) )
                  isSplitBar=true;
              }
            }
          if ( isSplitBar )
          {
            ISplitBar* spltBar = (ISplitBar*)event.controlWindow();
            ISize aRectSize = sizeRect->newRectangle().size();
            if (spltBar->pData->bClVertical)
            {
              int separatorOffset =
                ( aRectSize.width() - spltBar->pData->ulClSeparatorThickness) / 2;
              XtVaSetValues( spltBar->pData->iwhClSeparator,
                XmNleftOffset, separatorOffset,
                NULL);
              XtVaSetValues( spltBar->pData->iwhClSash,
                XmNheight, OFFSET,
                NULL );
            }
            else // horizontal
            {
              int separatorOffset =
                ( aRectSize.height() - spltBar->pData->ulClSeparatorThickness) / 2;
              XtVaSetValues( spltBar->pData->iwhClSeparator,
                XmNtopOffset, separatorOffset,
                NULL);
              XtVaSetValues( spltBar->pData->iwhClSash,
                XmNwidth, OFFSET,
                NULL );
            }
          }
//      }
      }
    }
    break;

    default:
      break;
  }
  return result;
}


/*------------------------------------------------------------------------------
| ISplitBar::ISplitBar                                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
ISplitBar::ISplitBar(unsigned long windowId, IWindow* parent,
                           IWindow* owner,
                           const IRectangle &initialSize, const Style &style)
  : pData(0)

  {
  IMODTRACE_ALL("ISplitBar constructor");
  pData = new ISplitBarPrivate();
  pData->bClVertical = (style & horizontal) ? false : true;
  unsigned long ulStyle = (style &
                     ~(horizontal | vertical)).asUnsignedLong();

  Arg args[10];
  int n;

  n = 0;
  // use a minimum size for now.
  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (parent)
    parentHandle = parent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (owner)
    ownerHandle = owner->handle();

  pData->iwhClForm =
    Inherited::create(
      windowId,
      NULL,
      style.asUnsignedLong(),
      (IXmCreateFunction)XmCreateForm,
      parentHandle,
      ownerHandle,
      initialSize,
      args,
      n);

  if (pData->iwhClForm == 0)
     ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "sash form");

  // Add default handler for processing WM_SIZE (moveSizeTo, etc.)
  pData->fdefaultHandler.handleEventsFor( this );

  n = 0;
  XtSetArg( args[n], XmNtraversalOn, False ); n++;
  pData->iwhClSash = XtCreateManagedWidget(
               IString( windowId) + IString("sash"),
               xmSashWidgetClass,
               pData->iwhClForm,
               args, n);

  // Register the any callback.
  XtAddCallback(
    pData->iwhClSash,      // Sash
    XmNcallback,           // callback name
    iwindowSashCallback,
    (XtPointer) this);     // client data

  n = 0;
  if ( pData->bClVertical )
    {
    XtSetArg( args[n], XmNorientation, XmVERTICAL ); n++;
    }
  else
    {
    XtSetArg( args[n], XmNorientation, XmHORIZONTAL ); n++;
    }
  pData->iwhClSeparator = XtCreateManagedWidget(
               IString( windowId) + IString("Sep"),
               xmSeparatorWidgetClass,
               pData->iwhClForm,
               args, n);

  Dimension thickness;
  if (pData->bClVertical )
    XtVaGetValues( pData->iwhClSeparator,
      XmNwidth, &thickness,
      NULL );
  else
    XtVaGetValues( pData->iwhClSeparator,
      XmNheight, &thickness,
      NULL );
  pData->ulClSeparatorThickness = thickness;

  startHandlingEventsFor( pData->iwhClForm );

  // setup Attachments
  int separatorOffset = 
     ( OFFSET - pData->ulClSeparatorThickness )/2;
  if ( pData->bClVertical )
    {
    if ( initialSize.width() != 0)
      separatorOffset =
        ( initialSize.width() - pData->ulClSeparatorThickness )/2;
    XtVaSetValues( pData->iwhClSeparator,
      XmNtopAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNleftAttachment, XmATTACH_FORM,
      XmNleftOffset, separatorOffset,
      NULL);
    XtVaSetValues( pData->iwhClSash,
      XmNleftAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      XmNtopAttachment, XmATTACH_FORM,
      XmNtopOffset, OFFSET,
      XmNheight, OFFSET,
      NULL );
    }
  else
    {
    if ( initialSize.height() != 0)
      separatorOffset =
        ( initialSize.height() - pData->ulClSeparatorThickness )/2;
    XtVaSetValues( pData->iwhClSeparator,
      XmNleftAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      XmNtopAttachment, XmATTACH_FORM,
      XmNtopOffset, separatorOffset,
      NULL);
    XtVaSetValues( pData->iwhClSash,
      XmNtopAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      XmNrightOffset, OFFSET,
      XmNwidth, OFFSET,
      NULL );
    }

  setOwner( parent );

  // show or hide the control according to the styles.
  setStyle(ulStyle);
  }

#if IC_EXTENDED_RTTI_ENABLED
ISplitBar::ISplitBar(const ISplitBar&)
  { }
#endif // IC_EXTENDED_RTTI_ENABLED

/*------------------------------------------------------------------------------
| ISplitBar::~ISplitBar                                                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ISplitBar::~ISplitBar()
  {
  IMODTRACE_ALL("ISplitBar destructor");
   if ( (this->isValid()) && (pData->iwhClSash) )
     {
     XtRemoveCallback(
        pData->iwhClSash,      // Sash
        XmNcallback,           // callback name
        iwindowSashCallback,
        (XtPointer) this );    // client data
     }

  pData->fdefaultHandler.stopHandlingEventsFor( this );
  if (pData)
    delete pData;
  }

/*------------------------------------------------------------------------------
| ISplitBar::Style  ISplitBar::defaultStyle                                    |
------------------------------------------------------------------------------*/
ISplitBar::Style  ISplitBar::defaultStyle ( )
  {
  return currentDefaultStyle;
  }

/*------------------------------------------------------------------------------
| ISplitBar::separator                                                         |
------------------------------------------------------------------------------*/
IWindowHandle ISplitBar::separator() const

  {
  return pData->iwhClSeparator;
  }

/*------------------------------------------------------------------------------
| ISplitBar::sash                                                              |
------------------------------------------------------------------------------*/
IWindowHandle ISplitBar::sash() const

  {
  return pData->iwhClSash;
  }

/*------------------------------------------------------------------------------
| ISplitBar::orientation                                                       |
------------------------------------------------------------------------------*/
ISplitBar::Orientation ISplitBar::orientation() const

  {
  return ( pData->bClVertical ? verticalSplit : horizontalSplit );
  }

/*------------------------------------------------------------------------------
| ISplitBar::setOrientation                                                    |
------------------------------------------------------------------------------*/
ISplitBar& ISplitBar::setOrientation( const ISplitBar::Orientation &value )

  {
  if ( value == ISplitBar::verticalSplit )
    {
    if (!pData->bClVertical)
      {
      pData->bClVertical = true;
      int separatorOffset =
        ( size().height() - pData->ulClSeparatorThickness )/2;
      Dimension height;
      XtVaGetValues( pData->iwhClSeparator,
        XmNheight, &height,
        NULL );
      XtVaSetValues( pData->iwhClSeparator,
        XmNtopAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_NONE,
        XmNleftOffset, separatorOffset,
        XmNtopOffset, 0,
        XmNwidth, height,
        XmNorientation, XmVERTICAL,
        NULL );

      XtVaSetValues( pData->iwhClSash,
        XmNtopAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_NONE,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNtopOffset, OFFSET,
        XmNheight, OFFSET,
        XmNrightOffset, 0,
        NULL );
      }
    }
  else if (pData->bClVertical )
    {
    pData->bClVertical = false;
    int separatorOffset =
       ( size().width() - pData->ulClSeparatorThickness )/2;
    Dimension width;
    XtVaGetValues( pData->iwhClSeparator,
      XmNwidth, &width,
      NULL );
    XtVaSetValues( pData->iwhClSeparator,
      XmNtopAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_NONE,
      XmNleftAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      XmNtopOffset, separatorOffset,
      XmNleftOffset, 0,
      XmNorientation, XmHORIZONTAL,
      XmNheight, width,
      NULL );

    XtVaSetValues( pData->iwhClSash,
      XmNtopAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNleftAttachment, XmATTACH_NONE,
      XmNrightAttachment, XmATTACH_FORM,
      XmNrightOffset, OFFSET,
      XmNwidth, OFFSET,
      XmNtopOffset, 0,
      NULL );
    }
  return *this;
  }

/*------------------------------------------------------------------------------
| ISplitBar::prevControl                                                       |
------------------------------------------------------------------------------*/
IWindow* ISplitBar::prevControl( ) const

  {
  return pData->iwClPrevControl;
  }

/*------------------------------------------------------------------------------
| ISplitBar::nextControl                                                       |
------------------------------------------------------------------------------*/
IWindow* ISplitBar::nextControl( ) const

  {
  return pData->iwClNextControl;
  }

/*------------------------------------------------------------------------------
| ISplitBar::setPrevControl                                                    |
------------------------------------------------------------------------------*/
ISplitBar& ISplitBar::setPrevControl( IWindow *win)

  {
  pData->iwClPrevControl = win;
  return *this;
  }

/*------------------------------------------------------------------------------
| ISplitBar::setNextControl                                                    |
------------------------------------------------------------------------------*/
ISplitBar& ISplitBar::setNextControl( IWindow *win)

  {
  pData->iwClNextControl = win;
  return *this;
  }

/*------------------------------------------------------------------------------
| ISplitBar::setNextControl                                                    |
------------------------------------------------------------------------------*/
IColor ISplitBar::splitBarColor   ( ) const

  {
  Pixel backgroundColor;

  XtVaGetValues( pData->iwhClSash,
                 XmNbackground, &backgroundColor,
                 NULL );
  return IColor( backgroundColor );
  }

/*------------------------------------------------------------------------------
| ISplitBar::setNextControl                                                    |
------------------------------------------------------------------------------*/
ISplitBar &ISplitBar::setSplitBarColor ( const IColor &color )

  {
  XmChangeColor( pData->iwhClSeparator, color.asPixel() );
  XmChangeColor( pData->iwhClSash, color.asPixel() );
  return *this;
  }
