// Revision: 78 1.11 source/ui/basectl/ispltbar.hpp, canvas, ioc.v400, 000919  
/* NOSHIP */
#ifndef _ISPLTBAR_
  #define _ISPLTBAR_
/*******************************************************************************
* FILE NAME: ispltbar.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     ISplitBar - This class defines the splitbar used in the split canvas.    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <icontrol.hpp>
#include <irect.hpp>

#ifdef IC_MOTIF
  extern "C"
    {
    #include <Xm/Xm.h>
    #include <Xm/SashP.h>
    }
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ISplitBarPrivate;

#pragma enum(4)
#pragma pack(push,4)

class ISplitBar : public IControl {
typedef IControl
  Inherited;
/*******************************************************************************
*                                                                              *
*******************************************************************************/
public:
/*-------------------------------- Style ----------------------------------
  The following functions provide a means to set and query split canvas
  styles:

    Style - Nested class that provides static members that define the set of
            valid split bar styles.  These styles can be used in
            conjunction with the styles defined by the nested classes
            IWindow::Style.

            The declaration of the ISplitBar::Style nested class is
            generated by the INESTEDBITFLAGCLASSDEF2 macro.

  The valid split canvas styles are:
    classDefaultStyle - Original default style for this class, which is
                        vertical | IWindow::visible.
    horizontal        - The split bar is create horizontally.
    vertical          - The split bar is created vertically.

  The following functions provide a means of getting and setting the default
  style for this class:
    defaultStyle    - Returns the current default style.  This is the same as
                      classDefaultStyle unless setDefaultStyle has been
                      called.
    setDefaultStyle - Sets the default style for all subsequent split
                      canvases.
-------------------------------------------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, ISplitBar, IWindow, IControl);
                                  // style class definition
static const Style
  horizontal,
  vertical,
  classDefaultStyle;

static Style
  defaultStyle    ( );
static void
  setDefaultStyle ( const Style& style );

/*-------------------------- Constructor/Destructor ----------------------------
| You can construct instances of this class from a window ID, parent window,   |
| owner window, rectangle, and style.  This creates the specified split        |
| canvas window.                                                               |
------------------------------------------------------------------------------*/
  ISplitBar       ( unsigned long windowId,
                    IWindow* parent,
                    IWindow* owner,
                    const IRectangle& initialSize = IRectangle(),
                    const Style& style = defaultStyle() );

virtual
 ~ISplitBar    ( );

/*------------------------------- Enumerators ----------------------------------
| The following enumerations are defined for this class:                       |
|   Orientation  - Enumeration that specifies the direction of the split bar.  |
|                  Values are:                                                 |
|                    horizontalSplit - The split bar is drawn horizontally.    |
|                    verticalSplit   - The split bar is drawn vertically.      |
------------------------------------------------------------------------------*/
enum Orientation  { horizontalSplit, verticalSplit };

/*---------------------------- Style Manipulation ------------------------------
| These functions are used to set and query the default style for new objects  |
| of this class or the styles of an existing instance:                         |
|   orientation    - Returns the direction in which the split-bar is drawn.    |
------------------------------------------------------------------------------*/
Orientation
  orientation     ( ) const;

ISplitBar
  &setOrientation( const Orientation& value );

virtual IWindow
  *prevControl( ) const,
  *nextControl( ) const;

virtual ISplitBar
  &setPrevControl( IWindow *win),
  &setNextControl( IWindow *win);

#ifdef IC_MOTIF
IWindowHandle
  separator() const,
  sash() const;
#endif

virtual IColor
  splitBarColor   ( ) const;

virtual ISplitBar
  &setSplitBarColor ( const IColor &color );

private:
/*--------------------------------- Private ----------------------------------*/
#ifdef IC_MOTIF
friend class ISplitBarHandler;
#endif //IC_MOTIF

  ISplitBar          ( const ISplitBar& );

static Style
  currentDefaultStyle;

ISplitBar
 &operator=             ( const ISplitBar& );

ISplitBarPrivate
  *pData;
}; // ISplitBar

INESTEDBITFLAGCLASSFUNCS(Style, ISplitBar)
                                  // global style functions

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

/*----------------------------- Inline Functions -----------------------------*/
//  #include <isplitcv.inl>

#endif /* _ISPLTBAR_ */

