// Revision: 72 1.4 source/ui/basectl/itabctl1.cpp, notebook, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: itabctl1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itabnh.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp.  1997  All Rights Reserved.                        *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINSTDBOOK
#define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
extern "C" {
  #include <iwindefs.h>
#ifdef IC_WIN
  #include <commctrl.h>
#endif
}

#include <iwindow.hpp>
#include <itabctl.hpp>
#include <itabnh.hpp>
#include <ictlevt.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <iwcname.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITABCTL1_CPP_
  #include <ipagetun.h>
#endif

// Initialize notification strings.
INotificationId const ITabControl::pageAddId
                                       = "ITabControl::pageAdd";
INotificationId const ITabControl::pageRemoveId
                                       = "ITabControl::pageRemove";
INotificationId const ITabControl::buttonAddId
                                       = "ITabControl::buttonAdd";
INotificationId const ITabControl::buttonRemoveId
                                       = "ITabControl::buttonRemove";
INotificationId const ITabControl::pageSelectId
                                       = "ITabControl::pageSelect";
INotificationId const ITabControl::pageBackgroundColorId
                                       = "ITabControl::pageBackgroundColor";
INotificationId const ITabControl::tabBackgroundColorId
                                       = "ITabControl::tabBackgroundColor";

/*------------------------------------------------------------------------------
| ITabControl::enableNotification                                              |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ITabControl, does not send notifications to observer objects until it is     |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IWindow& ITabControl::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new ITabControlNotifyHandler() );
  }
  return IWindow::enableNotification(enabled);
}

/*------------------------------------------------------------------------------
| ITabControlNotifyHandler::ITabControlNotifyHandler                           |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITabControlNotifyHandler::ITabControlNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITabControlNotifyHandler::~ITabControlNotifyHandler                          |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITabControlNotifyHandler::~ITabControlNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITabControlNotifyHandler::dispatchHandlerEvent                               |
|                                                                              |
| This notifies ITabControl windows of the following events:                   |
|                                                                              |
| select(Id)                                                                   |
| pageBackgroundColor(Id)                                                      |
| pageRemove(Id)                                                               |
------------------------------------------------------------------------------*/
bool ITabControlNotifyHandler::dispatchHandlerEvent (IEvent& event)
{
  switch (event.eventId())
  {
#ifdef IC_PM
    case BKM_SETNOTEBOOKCOLORS:
    {
      ITabControl
         *tabControl = dynamic_cast<ITabControl*>(event.controlWindow());

      if (tabControl)
      {
         // Post private message to delay notification until after
         // the selection state has been modified.
         // Put color identifier into parameter 1 for later use.
         tabControl->postEvent( IC_UM_DELAYNOTIFY,
                                IEventParameter1( event.parameter2() ),
                                IEventParameter2( IC_NOTDEL_BOOK_COLOR ));
         break;
      }
    }

    case IC_UM_DELAYNOTIFY:
    {
      ITabControl
         *tabControl = dynamic_cast<ITabControl*>(event.controlWindow());
 
      if (tabControl)
      {
         // This is the delayed notification message.  Check to see which
         // notification should be sent to the observers based on info
         // stored in parameter 2 of the private message.  Parameter 1
         // contains the information passed in the original event that
         // is used to determine which notification id to send
         if (event.parameter2().asUnsignedLong() == IC_NOTDEL_BOOK_COLOR)
         {
           switch (event.parameter1())
           {
             case BKA_BACKGROUNDPAGECOLOR:
               tabControl->notifyObservers( INotificationEvent(
                                            ITabControl::pageBackgroundColorId,
                                            *tabControl ));
               break;


             default:
               break;
           }
         }
      }
    }
#endif // IC_PM

    case WM_CONTROL:
    {
      IControlEvent ctlEvent( event );
      ITabControl
         *tabControl = dynamic_cast<ITabControl*>(ctlEvent.controlWindow());

      if (tabControl)
      {
         unsigned long msgId = ctlEvent.parameter1().highNumber();
         switch (msgId)
         {
#ifdef IC_PM
           case BKN_PAGESELECTED:
           {
             tabControl->notifyObservers( INotificationEvent(
                                             ITabControl::pageSelectId,
                                             *tabControl ));
             break;
           }
#endif

#ifdef IC_WIN
           case IC_UM_TCN_PAGEDELETED:
#elif IC_PM
           case BKN_PAGEDELETED:
#endif
           {
             ITabPage *removePage;
#ifdef IC_WIN
             removePage = (ITabPage*)(void*)ctlEvent.parameter2();
#elif IC_PM
             DELETENOTIFY *pNotifyInfo = (DELETENOTIFY*)(void*)ctlEvent.parameter2();
             removePage = (ITabPage*)(void*)(pNotifyInfo->ulAppPageData);
#endif
             tabControl->notifyObservers( INotificationEvent(
                                             ITabControl::pageRemoveId,
                                             *tabControl,
                                             false,
                                             (void*)removePage ));
           }

           default:
             break;
         }
       }
    }

    default:
      break;
  }
  return (Inherited::dispatchHandlerEvent( event ));
}

