// Revision: 88 1.6 source/ui/basectl/itabpage.cpp, notebook, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: itabpage.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     ITabPage                                                                 *
*                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp.  1997  All Rights Reserved.                        *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR         // WinQueryWindowULong
  #define INCL_WINBUTTONS           // BS_NOTEBOOKBUTTON
  #define INCL_WINSTDBOOK
  #define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
  #include <iwindefs.h>
#ifdef IC_WIN
  #include <commctrl.h>
#endif // IC_WIN
}

#include <ibase.hpp>
#include <idefstyl.h>
#include <itabctl.hpp>
#include <itabpage.hpp>
#include <itabpriv.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <icoordsy.hpp>
#include <iframe.hpp>
#include <ihandler.hpp>
#include <inotifev.hpp>
#include <ipushbut.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <isetcv.hpp>
#include <itext.hpp>
#include <itrace.hpp>

#ifdef IC_WIN
#include <igimage.hpp>
#endif // IC_WIN

#ifdef IC_PM
#include <ibmpstat.hpp>
#endif // IC_PM

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ITABPAGE_CPP_
  #include <ipagetun.h>
#endif

// Make private data pointer names less verbose.
#define tcd fTabControlData
#define tpd fTabPageData

/*------------------------------------------------------------------------------
| Public tab page styles.                                                      |
------------------------------------------------------------------------------*/
const ITabPage::Style
  ITabPage::autoPageSize          ( 1, ITabPage__autoPageSize ),
  ITabPage::noStyle               ( 1, ITabPage__noStyle ),
  ITabPage::classDefaultStyle     ( 1, ITabPage__classDefaultStyle );

/*------------------------------------------------------------------------------
| Default style for new tab page objects (initial value).                      |
------------------------------------------------------------------------------*/
ITabPage::Style
  ITabPage::fCurrentDefaultStyle  ( 1, ITabPage__classDefaultStyle );

/*------------------------------------------------------------------------------
| ITabPage::ITabPage                                                           |
|                                                                              |
| Construct a tab page object given an initial style.                          |
------------------------------------------------------------------------------*/
ITabPage :: ITabPage ( const Style& style )
  : fTabPageData( new ITabPageData( this ) )
{
  // Save the style.
  tpd->fStyle = convertToGUIStyle( style );

#ifdef IC_WIN
  tpd->fAutoPageSize = (style & autoPageSize) ? true : false;
#endif // IC_WIN
}


/*------------------------------------------------------------------------------
| ITabPage::ITabPage                                                           |
|                                                                              |
| Construct a tab page object given tab text and an initial style.             |
------------------------------------------------------------------------------*/
ITabPage :: ITabPage ( const IText& tabText,
                       const Style& style )
  : fTabPageData( new ITabPageData( this ) )
{
  // Save the style.
  tpd->fStyle = convertToGUIStyle( style );

#ifdef IC_WIN
  tpd->fAutoPageSize = (style & autoPageSize) ? true : false;
#endif // IC_WIN
  
  tpd->fTabText = tabText;
}


/*------------------------------------------------------------------------------
| ITabPage::ITabPage                                                           |
|                                                                              |
| Construct a tab page object given a bitmap and an initial style.             |
------------------------------------------------------------------------------*/
ITabPage :: ITabPage ( const IBitmapHandle& tabBitmap,
                       const Style&         style )
  : fTabPageData( new ITabPageData( this ) )
{
  // Save the style.
  tpd->fStyle = convertToGUIStyle( style );

#ifdef IC_WIN
  tpd->fAutoPageSize = (style & autoPageSize) ? true : false;
#endif // IC_WIN
 
  tpd->fTabBitmap = tabBitmap;
}


/*------------------------------------------------------------------------------
| ITabPage::ITabPage                                                           |
|                                                                              |
| Construct a tab page object given tab text, a bitmap, and an initial style.  |
------------------------------------------------------------------------------*/
ITabPage :: ITabPage ( const IText&         tabText,
                       const IBitmapHandle& tabBitmap,
                       const Style&         style )
  : fTabPageData( new ITabPageData( this ) )
{
  // Save the style.
  tpd->fStyle = convertToGUIStyle( style );

#ifdef IC_WIN
  tpd->fAutoPageSize = (style & autoPageSize) ? true : false;
#endif // IC_WIN

  tpd->fTabText = tabText;
  tpd->fTabBitmap = tabBitmap;
}


/*------------------------------------------------------------------------------
| ITabPage::~ITabPage                                                          |
|                                                                              |
| Tab page destructor.                                                         |
------------------------------------------------------------------------------*/
ITabPage :: ~ITabPage ( )
{
  // If this page hasn't been removed from the tab control, remove it now.
  if (tpd->fTabControl && IWindow::isWindowValid( tpd->fTabControl ))
  {
     tpd->fTabControl->removePage( this );
  }

  delete fTabPageData;
}


/*------------------------------------------------------------------------------
| ITabPage::defaultStyle                                                       |
|                                                                              |
| Return the default style for new tab page objects.                           |
------------------------------------------------------------------------------*/
ITabPage::Style ITabPage :: defaultStyle ( )
{
  return fCurrentDefaultStyle;
}


#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITabPage::setDefaultStyle                                                    |
|                                                                              |
| Sets the default style.                                                      |
|                                                                              |
| Note: This function is inlined on all platforms except Windows due to        |
|       variables it references not being exported.                            |
------------------------------------------------------------------------------*/
void ITabPage::setDefaultStyle ( const ITabPage::Style& style )
{
  fCurrentDefaultStyle = style;
}
#endif //IC_WIN


/*------------------------------------------------------------------------------
| ITabPage::convertToGUIStyle                                                  |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ITabPage :: convertToGUIStyle ( const IBitFlag& style,
                                              bool  extendedOnly ) const
{
  IMODTRACE_DEVELOP( "ITabPage::convertToGUIStyle" );

  unsigned long ulStyle = 0;

  if (!extendedOnly)
  {
#ifdef IC_PM
     ITabPage::Style& bitStyle = (ITabPage::Style &)style;
     if (bitStyle & ITabPage::autoPageSize)
        ulStyle |= BKA_AUTOPAGESIZE;

     ulStyle |= BKA_MAJOR;
#endif // IC_PM
  }

  return ulStyle;
}


/*------------------------------------------------------------------------------
| ITabPage::tabControl                                                         |
|                                                                              |
| Returns the tab control for which this is a valid tab page.                  |
------------------------------------------------------------------------------*/
ITabControl* ITabPage :: tabControl ( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::tabControl" );

  return tpd->fTabControl;
}


/*------------------------------------------------------------------------------
| ITabPage::pageWindow                                                         |
|                                                                              |
| Returns the window used by the tab control page.                             |
------------------------------------------------------------------------------*/
IWindow* ITabPage :: pageWindow ( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::pageWindow" );

  return tpd->fPageWindow;
}


/*------------------------------------------------------------------------------
| ITabPage::createPageWindow                                                   |
|                                                                              |
| Virtual function intended to be overridden by derived classes.  This         |
| function will be called by the default handler when the associated tab       |
| control page is first selected and the page window has not yet been set.     |
| It allows for delayed page window creation.                                  |                                                            |
------------------------------------------------------------------------------*/
IWindow* ITabPage :: createPageWindow ( )
{
  IMODTRACE_DEVELOP( "ITabPage::createPageWindow" );

  return (IWindow*)0;
}


/*------------------------------------------------------------------------------
| ITabPage::updatePageWindow                                                   |
|                                                                              |
| Virtual function intended to be overridden by derived classes.  This         |
| function will be called by the default handler when the associated tab       |
| control page is selected and the page window has already been set.  It       |
| provides the ability to update/paint the page window when the page is        |
| selected.                                                                    |
------------------------------------------------------------------------------*/
void ITabPage :: updatePageWindow ( )
{
  IMODTRACE_DEVELOP( "ITabPage::updatePageWindow" );

  return;
}


/*------------------------------------------------------------------------------
| ITabPage::setPageWindow                                                      |
|                                                                              |
| Sets the page window for the tab control page.                               |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setPageWindow ( IWindow* pageWindow )
{
  IMODTRACE_DEVELOP( "ITabPage::setPageWindow" );

  // Throw assertion if window isn't valid.
  IASSERTPARM( pageWindow != 0 )

  // We may be called after a createPageWindow where the user application
  // went ahead and set the page window.  If it is already set, just return.
  if (pageWindow == tpd->fPageWindow)
     return *this;

#ifdef IC_PM
  // If the page window is a frame, we need to set the owner to NULL.  The
  // notebook will set the parent to the page window (id 8006) and then we
  // can set the owner to the notebook.  (We cannot set the owner to the
  // notebook initially because this might cause the frame to have the same
  // parent and owner and cause problems).  If the owner of the frame is
  // not the notebook, a focus loop problem will occur.
  if (pageWindow->isFrameWindow())
     pageWindow->setOwner( NULL );
      
  // Remove the page window resize handler for the previous page window if
  // one exists.  Only remove it if the page window is not also the page
  // window for another page.
  if (tpd->fTabControl && tpd->fPageWindow)
  {
     ITabControl::Cursor cursor( *(tpd->fTabControl) );
     bool pageWindowFound = false;
     for (cursor.setToFirst();
          cursor.isValid() && !pageWindowFound;
          cursor.setToNext())
     {
        ITabPage *tabPage = cursor.current();
        if (tabPage != this &&
            tpd->fPageWindow == tabPage->tpd->fPageWindow)
           pageWindowFound = true;
     }
     if (!pageWindowFound)
     {
        tpd->fTabControl->tcd->fTabPageResizeHandler->
              stopHandlingEventsFor( tpd->fPageWindow );
     }
  }
#endif // IC_PM

#ifdef IC_WIN
  // If a page window already exists and the page has been added to a
  // tab control, reparent it back to its original parent.
  if (tpd->fTabControl && tpd->fPageWindow)
  {
     tpd->fPageWindow->setParent( tpd->fPageWindowParent );
     tpd->fPageWindow->hide();
  }
#endif // IC_WIN

  tpd->fPageWindow = pageWindow;

  // If the page has already been added to a tab control, set the page
  // window in the tab control.
#ifdef IC_WIN
  if (tpd->fTabControl)
#elif IC_PM
  if (tpd->fPageId)
#endif // IC_WIN
  {
     ITabControl *tabControl = tpd->fTabControl;
     ITabControlData *tabControlData = tabControl->tcd;

#ifdef IC_PM
     // If this page is not auto sized, save the current position of the
     // page window.  We need to reset this position after the page window
     // has been set since its parent will change.  This is only necessary
     // when the application orientation is upperLeft.
     IPoint posPageWindow;
     bool   bPosPageWindow = false;
     if ((ICoordinateSystem::applicationOrientation() ==
          ICoordinateSystem::kOriginUpperLeft) &&
         !isAutoPageSize())
     {
        bPosPageWindow = true;
        posPageWindow = pageWindow->position();
     }

     // If there are buttons for this page, add them now as page buttons.
     ITabControlButtonSequence 
        *buttons = (tpd->fPageButtons) ?
                       tpd->fPageButtons : tabControl->tcd->fCommonButtons;
     if (buttons)
     {
        // Add the buttons and set the page window.
        tpd->setButtons( buttons );
     }
     else
     {
        // Set the page window.
        if (!tabControl->sendEvent( BKM_SETPAGEWINDOWHWND,
                                    (unsigned long)(tpd->fPageId),
                                    pageWindow->handle().asUnsigned() ).asUnsignedLong())
        {
           ITHROWGUIERROR( "BKM_SETPAGEWINDOWHWND" );
        }
     }

     // If this is not an auto page size window, reset the window's position now that
     // its parent has been changed to the PM notebook's page clipping window.
     if (bPosPageWindow)
        pageWindow->moveTo( posPageWindow );

     // Add the page window resize handler.  Only add it if this page
     // window is not already a page window for another page.
     ITabControl::Cursor cursor( *(tpd->fTabControl) );
     ITabPage *tabPage = 0;
     bool isHandled = false;
     for (cursor.setToFirst();
          cursor.isValid() && !isHandled;
          cursor.setToNext())
     {
        tabPage = cursor.current();
        if (tabPage != this &&
            tabPage->tpd->fPageWindow == pageWindow)
           isHandled = true;
     }
     if (!isHandled)
        tabControlData->fTabPageResizeHandler->handleEventsFor( pageWindow );

     // If there is no owner for the page, set it to the tab control.
     if (pageWindow->owner() == NULL)
         pageWindow->setOwner( tabControl );
#endif // IC_PM

#ifdef IC_WIN
     // Save the original parent for restoration when the page is removed.
     tpd->fPageWindowParent = pageWindow->parent();

     // Make the page clipping window the parent of the application page
     // window.
     pageWindow->setParent( tabControlData->fClipWindow );

     // Size and show the application page window if it is on the top page.
     if (this == tabControl->topPage())
     {
        // Show the top page window.
        tabControlData->showTopPage( tabControlData->tabPageIndex( this ) );
     }
     else
     {
        // Make sure the application window is hidden since it is not
        // the top page.
        pageWindow->hide();
     }
#endif // IC_WIN

     // Indicate that the tab control's minimum size may have changed.
     // We don't want to change the minimum size every time a page is
     // created dynamically.  Therefore, only call setLayoutDistorted
     // if we're not creating a page window dynamically or the tab
     // control is not visible.
#ifdef IC_WIN
     if (!tabControlData->fInCreate || !tabControl->isVisible()) 
#else
     if (!tabControl->isVisible()) 
#endif
        tabControl->setLayoutDistorted( IWindow::minimumSizeChanged, 0 );
  }

  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::validatePageWindow                                                 |
|                                                                              |
| Virtual function intended to be overridden by derived classes.  This         |
| function will be called by the default handler when a selection change event |
| is received for the associated tab control page. It allows for page          |
| validation without the use of an application defined handler.                |
------------------------------------------------------------------------------*/
bool ITabPage :: validatePageWindow ( )
{
  IMODTRACE_DEVELOP( "ITabPage::validatePageWindow" );

  return true;
}


/*------------------------------------------------------------------------------
| ITabPage::addPageButton                                                      |
|                                                                              |
| Add a button which is unique for this tab page.                              |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: addPageButton ( IPushButton* button )
{
  IMODTRACE_DEVELOP( "ITabPage::addPageButton" );

  IASSERTPARM( button != 0 )

  if (tpd->fTabControl)
  {
     // Throw an exception if the tab control was not constructed with the buttons
     // style or the button's parent is not the button canvas.
#ifdef IC_WIN
     if (!tpd->fTabControl->tcd->fButtonCanvas)
#elif IC_PM
     if (~tpd->fTabControl->style() & BKS_BUTTONAREA)
#endif // IC_WIN
     {
        ITHROWLIBRARYERROR1( IC_INVALID_STYLE_FOR_FUNCTION,
                             IBaseErrorInfo::invalidRequest,
                             IException::recoverable,
                             "ITabControl::buttons" );
     }

     if (button->parent() != tpd->fTabControl->buttonParent())
     {
        ITHROWLIBRARYERROR( IC_INVALID_BUTTON_PARENT,
                            IBaseErrorInfo::invalidRequest,
                            IException::recoverable );
     }
  }

  // Create a tab control button and add it.
  ITabControlButton *tcPageButton = new ITabControlButton();
  tcPageButton->fApplicationButton = button;
  tpd->addPageButton( tcPageButton, button->isDefault() );

  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::addPageButton                                                      |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: addPageButton ( ITabControl::EButton buttonType,
                                      bool                 isDefaultButton )
{
  IMODTRACE_DEVELOP( "ITabPage::addPageButton" );

  if (tpd->fTabControl)
  {
     // Throw an exception if the tab control was not constructed with the buttons
     // style.
#ifdef IC_WIN
     if (!tpd->fTabControl->tcd->fButtonCanvas)
#elif IC_PM
     if (!(tpd->fTabControl->style() & BKS_BUTTONAREA))
#endif // IC_WIN
     {
        ITHROWLIBRARYERROR1( IC_INVALID_STYLE_FOR_FUNCTION,
                             IBaseErrorInfo::invalidRequest,
                             IException::recoverable,
                             "ITabControl::buttons" );
     }
  }


  // Create a tab control button and add it.
  ITabControlButton *tcPageButton = new ITabControlButton();
  tcPageButton->fStandardButton = buttonType;
  tpd->addPageButton( tcPageButton, isDefaultButton );

  return *this;
}

  
/*------------------------------------------------------------------------------
| ITabPage::removePageButton                                                   |
|                                                                              |
| Remove a button which is unique for this tab page.                           |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: removePageButton ( IPushButton* button )
{
  IMODTRACE_DEVELOP( "ITabPage::removePageButton" );

  // Find this button in the page button sequence.
  ITabControlButton *tcPageButton = 0;
  if (tpd->fPageButtons)
  {
     ITabControlButtonSequence::Cursor cursor( *(tpd->fPageButtons) );
     forCursor (cursor)
     {
        ITabControlButton *tcButton = tpd->fPageButtons->elementAt( cursor );
        if (tcButton->fApplicationButton == button ||
            (tpd->fTabControl && tcButton->button( tpd->fTabControl ) == button))
        {
           tcPageButton = tcButton;
           cursor.setToLast();
        }
     }
  }

  if (!tcPageButton)
  {
     // Throw an exception if we didn't find the button.
     ITHROWLIBRARYERROR1( IC_CONTROL_NOT_FOUND,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "IPushButton*");
  }

  // Remove this button from the tab control.
  tpd->removePageButton( tcPageButton );
  delete tcPageButton;

  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::removePageButton                                                   |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: removePageButton ( ITabControl::EButton buttonType )
{
  IMODTRACE_DEVELOP( "ITabPage::removePageButton" );

  // Find this button in the page button sequence.
  ITabControlButton *tcPageButton = 0;
  if (tpd->fPageButtons)
  {
     ITabControlButtonSequence::Cursor cursor( *(tpd->fPageButtons) );
     forCursor (cursor)
     {
        ITabControlButton *tcButton = tpd->fPageButtons->elementAt( cursor );
        if (tcButton->fStandardButton == buttonType)
        {
           tcPageButton = tcButton;
           cursor.setToLast();
        }
     }
  }
  
  if (!tcPageButton)
  {
     // Throw an exception if we didn't find the button.
     ITHROWLIBRARYERROR1( IC_CONTROL_NOT_FOUND,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "IPushButton*");
  }

  // Remove this button from the tab control.
  tpd->removePageButton( tcPageButton );
  delete tcPageButton;

  return *this;
}

  
/*------------------------------------------------------------------------------
| ITabPage::setTabText                                                         |
|                                                                              |
| Set the tab text for this tab page.                                          |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setTabText( const IText& tabText )
{
  IMODTRACE_DEVELOP( "ITabPage::setTabText" );

  ITabControl     *tabControl = tpd->fTabControl;
  tpd->fTabText = tabText;

  if (tabControl)
  {
     // If this page has already been added to the tab control, set the tab text
     // in the tab control.
#ifdef IC_WIN
     TC_ITEM tabItem;
     tabItem.mask = TCIF_TEXT;
     tabItem.pszText = (char*)(const char*)tabText;

     ITabControlData *tabControlData = tabControl->tcd;
     unsigned long pageIndex = tabControlData->tabPageIndex( this );
     if (!TabCtrl_SetItem( tabControl->handle(),
                           pageIndex-1,
                           &tabItem ))
     {
        ITHROWGUIERROR("TCM_INSERTITEM");
     }
#endif // IC_WIN

#ifdef IC_PM
     if (!tabControl->sendEvent( BKM_SETTABTEXT,
                                 tpd->fPageId,
                                 (void*)(const char*)tabText ).asUnsignedLong())
        ITHROWGUIERROR( "BKM_SETTABTEXT" );
#endif // IC_PM
  }

  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::setTabText                                                         |
|                                                                              |
| Set the tab text for this tab page.                                          |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setTabText( const IResourceId& resourceId )
{
  return setTabText( resourceId.resourceLibrary().loadString( resourceId.id() ));
}


/*------------------------------------------------------------------------------
| ITabPage::tabText                                                            |
|                                                                              |
| Return the tab text for this tab page.                                       |
------------------------------------------------------------------------------*/
IText ITabPage :: tabText( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::tabText" );

  // Just return the saved tab text since it will always be valid.
  return tpd->fTabText;
}


/*------------------------------------------------------------------------------
| ITabPage::setTabBitmap                                                       |
|                                                                              |
| Set the tab bitmap for this tab page.                                        |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setTabBitmap( const IBitmapHandle& bitmap )
{
  IMODTRACE_DEVELOP( "ITabPage::setTabBitmap" );

  ITabControl     *tabControl = tpd->fTabControl;
  
  tpd->fTabBitmap = bitmap;

  if (tabControl)
  {
     // If this page has already been added to the tab control, set the tab bitmap
     // in the tab control.
#ifdef IC_WIN
     // Get icon/bitmap dimensions.
     long lCXIcon = GetSystemMetrics( SM_CXICON );
     long lCYIcon = GetSystemMetrics( SM_CYICON );

     // Create an image list if it does not exist.                                               */
     ITabControlData *tabControlData = tabControl->tcd;
     if (!tabControlData->fImageList)
     {
        // Create the image list to initially contain 10 bitmaps and grow it
        // by an increment of 10 whenever the current increment list size is
        // exceeded.  Also, set the bitmap to the system icon size.
        tabControlData->fImageList =
              (void*)ImageList_Create( (int)lCXIcon, (int)lCYIcon, ILC_COLOR24, 10, 10 );

        if (!tabControlData->fImageList)
           ITHROWGUIERROR("ImageList_Create");

        // Set the tab control's image list.
        TabCtrl_SetImageList( tabControl->handle(),
                              (HIMAGELIST)tabControlData->fImageList );
     }

     // Size the new bitmap. 
     IGImage bmpImage( bitmap );
     bmpImage.sizeTo( IGPoint2D( (GCoordinate)lCXIcon, (GCoordinate)lCYIcon ));

     // Get the image index in the tab control item that identifies the
     // current bitmap's index in the image list.
     TC_ITEM tabItem;
     tabItem.mask = TCIF_IMAGE;
     unsigned long pageIndex = tabControlData->tabPageIndex( this );
     if (!TabCtrl_GetItem( tabControl->handle(),
                           pageIndex-1,
                           &tabItem ))
     {
        ITHROWGUIERROR("TabCtrl_GetItem");
     }

     // If the image for the specified page does not exist, add the bitmap
     // to the image list.
     if (tabItem.iImage == -1)
     {
        int imageIndex = ImageList_Add( (HIMAGELIST)tabControlData->fImageList,
                                        bmpImage.handle(),
                                        NULL );
        if (imageIndex == -1)
        {
           ITHROWGUIERROR("ImageList_Add");
        }

        // Use the index returned from the add to update the tab control item.
        tabItem.iImage = imageIndex;
     }
     else
     {
        // If the image for the specified page exists,  replace it with the
        // specified bitmap.
        if (!ImageList_Replace( (HIMAGELIST)tabControlData->fImageList,
                                tabItem.iImage,
                                bmpImage.handle(),
                                NULL ))
        {
           ITHROWGUIERROR("ImageList_Replace");
        }
     }

     // Set the image index in the tab control item that identifies the
     // bitmap's index in the image list.
     if (!TabCtrl_SetItem( tabControl->handle(),
                           pageIndex-1,
                           &tabItem ))
     {
        ITHROWGUIERROR("TabCtrl_SetItem");
     }
#endif // IC_WIN

#ifdef IC_PM
     IBitmapHandle
        desktopBitmap = IBitmapStaticPtr::desktopCompatibleBitmap( bitmap );

     if (!tabControl->sendEvent( BKM_SETTABBITMAP,
                                 (unsigned long)(tpd->fPageId),
                                 (unsigned long)desktopBitmap ).asUnsignedLong())
     {
       ITHROWGUIERROR( "BKM_SETTABBITMAP" );
     }
#endif // IC_PM
  }

  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::setTabBitmap                                                       |
|                                                                              |
| Set the tab bitmap for this tab page.                                        |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setTabBitmap( const IResourceId& resourceId )
{
  return setTabBitmap(
               resourceId.resourceLibrary().loadBitmap( resourceId.id() ));
}


/*------------------------------------------------------------------------------
| ITabPage::tabBitmap                                                          |
|                                                                              |
| Return the tab bitmap for this tab page.                                     |
------------------------------------------------------------------------------*/
IBitmapHandle ITabPage :: tabBitmap( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::tabBitmap" );

  // Just return the saved bitmap handle since it will always be valid.
  return tpd->fTabBitmap;
}


/*------------------------------------------------------------------------------
| ITabPage::setHelpId                                                          |
|                                                                              |
| Set the help panel id to be used for this tab page.                          |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setHelpId( unsigned long helpId )
{
  IMODTRACE_DEVELOP( "ITabPage::setHelpId" );

  tpd->fHelpId = helpId;
  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::helpId                                                             |
|                                                                              |
| Return the help panel id to be used for this tab page.                       |
------------------------------------------------------------------------------*/
unsigned long ITabPage :: helpId( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::helpId" );

  return tpd->fHelpId;
}


/*------------------------------------------------------------------------------
| ITabPage::setTabBackgroundColor                                              |
|                                                                              |
| Sets the tab background color.                                               |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: setTabBackgroundColor ( const IColor &color )
{
  ITRACE_WIN_NOP();

#ifdef IC_PM
  if (!tpd->fTabBkgrndColorSet || color != tpd->fTabBkgrndColor)
  {
     tpd->fTabBkgrndColorSet = true;
     tpd->fTabBkgrndColor = color;

     if (tpd->fTabControl)
     {
        ITabControl *tabControl = tpd->fTabControl;
        if (!tabControl->sendEvent( BKM_SETTABCOLOR,
                                    (unsigned long)(tpd->fPageId),
                                    (unsigned long)(color.asRGBLong()) )
            .asUnsignedLong())
        {
           ITHROWGUIERROR( "BKM_SETTABCOLOR" );
        }
   
        // Send a notification to all observers.
        tpd->fTabControl->notifyObservers(
                             INotificationEvent( ITabControl::tabBackgroundColorId,
                                                 *(tpd->fTabControl),
                                                 false,
                                                 (void*)this ));
     }
  }
#endif // IC_PM
  
  return *this;
}

/*------------------------------------------------------------------------------
| ITabPage::resetTabBackgroundColor                                            |
|                                                                              |
| Resets the tab background color by undoing a previous set.                   |
------------------------------------------------------------------------------*/
ITabPage& ITabPage :: resetTabBackgroundColor ( )
{
  ITRACE_WIN_NOP();

#ifdef IC_PM
  if (tpd->fTabBkgrndColorSet)
  {
     tpd->fTabBkgrndColorSet = false;

     ITabControl *tabControl = tpd->fTabControl;
     unsigned long pageCount = 0;
     if (tabControl)
     {
        pageCount = tabControl->totalPages();
     
        // Determine the relative position of this page in the tab control.
        ITabControl::Cursor pageCursor( *tabControl );
        unsigned long pageIndex = 0;
        for (pageCursor.setToFirst();
             pageCursor.isValid();
             pageCursor.setToNext())
        {
           if (pageCursor.current() == this)
              pageCursor.invalidate();
           else
              pageIndex++;
        }
        
        // Reset the tabs to the original auto color value.
        tabControl->sendEvent( BKM_SETTABCOLOR,
                               tpd->fPageId,
                               tpd->autoColors[pageIndex % 10] );
     }
  }
#endif // IC_PM

  return *this;
}


/*------------------------------------------------------------------------------
| ITabPage::tabBackgroundColor                                                 |
|                                                                              |
| Returns the tab background color.                                            |
------------------------------------------------------------------------------*/
IColor ITabPage :: tabBackgroundColor ( ) const
{
#ifdef IC_WIN
  return( IGUIColor(IGUIColor::notebookPageBgnd) );
#endif // IC_WIN

#ifdef IC_PM
  if (!tpd->fTabBkgrndColorSet)
  {
     ITabControl *tabControl = tpd->fTabControl;
     unsigned long pageCount = 0;
     if (tabControl)
        pageCount = tabControl->totalPages();
     
     if (pageCount)
     {
        // Determine the relative position of this page in the tab control.
        // If the page hasn't been added to the tab control, return the first
        // default tab color.
        ITabControl::Cursor pageCursor( *tabControl );
        unsigned long pageIndex = 0;
        unsigned long colorIndex;
        for (pageCursor.setToFirst();
             pageCursor.isValid();
             pageCursor.setToNext())
        {
           if (pageCursor.current() == this)
              pageCursor.invalidate();
           else
              pageIndex++;
        }
        if (pageIndex < pageCount)
           colorIndex = pageIndex % 10;
        
        // Return the default tab color.
        return IColor( tpd->autoColors[colorIndex] );
     }
     return IColor( tpd->autoColors[0] );
  }

  return IColor( tpd->fTabBkgrndColor );
#endif // IC_PM
}


/*------------------------------------------------------------------------------
| ITabPage::isAutoPageSize                                                     |
|                                                                              |
| Returns whether the page window for this tab page is to be autosized.        |
------------------------------------------------------------------------------*/
bool ITabPage :: isAutoPageSize( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::isAutoPageSize" );

#ifdef IC_WIN
  return tpd->fAutoPageSize;
#endif // IC_WIN

#ifdef IC_PM
  unsigned long ulPageStyle;
  if (tpd->fTabControl)
  {
     ulPageStyle = (unsigned long)(void*)(tpd->fTabControl->sendEvent(
                                             BKM_QUERYPAGESTYLE,
                                             tpd->fPageId,
                                             0 ));
     if ((long)ulPageStyle == BOOKERR_INVALID_PARAMETERS)
     {
        ITHROWGUIERROR( "BKM_QUERYPAGESTYLE:BOOKERR_INVALID_PARAMETERS" );
     }
  }
  else
     ulPageStyle = tpd->fStyle;
  
  return (ulPageStyle & BKA_AUTOPAGESIZE) ? true : false;
#endif // IC_PM
}

/*------------------------------------------------------------------------------
| ITabPage::minimumPageWindowSize                                              |
|                                                                              |
| Returns the minimum size of the page window if set; otherwise, a default     |
| size is returned.
------------------------------------------------------------------------------*/
ISize ITabPage :: minimumPageWindowSize( ) const
{
  IMODTRACE_DEVELOP( "ITabPage::minimumPageWindowSize" );
  
  ISize pageSize;

  // If there is no page window set, use the default minimum window size.
  if (tpd->fPageWindow == 0)
  {
     pageSize = ISize( 100, 100 );
  }
  else
  {
     // Try a dynamic cast to determine whether the page window is a frame
     // window.
     if (dynamic_cast<IFrameWindow*>(tpd->fPageWindow))
     {
        // For dialog page windows, use their actual size.
        pageSize = tpd->fPageWindow->size();
     }
     else
     {
        if (!isAutoPageSize())
        {
           // If the page window is not auto sized, then use its actual
           // size.
           pageSize = tpd->fPageWindow->size();
        }
        if (pageSize == ISize())
        {
           // Use minimum size if the page window has either not been sized yet
           // or it is an auto size window.  In the latter case, we may have
           // changed the size and should reflect the actual minimum size.
           pageSize = tpd->fPageWindow->minimumSize();
        }
     }
  }
  return pageSize;
}

