// Revision: 86 1.7.1.3 source/ui/basectl/itextct1.cpp, control, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: itextct1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itextcnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINWINDOWMGR
#define INCL_WINENTRYFIELDS
#define INCL_WINMLE
extern "C" {
  #include <iwindefs.h>
}


#include <icconst.h>
#include <icontrol.hpp>
#include <ictlevt.hpp>
#include <ientryfd.hpp>
#include <ievent.hpp>
#include <imle.hpp>
#include <inotifev.hpp>
#include <itextcnh.hpp>
#include <itextctl.hpp>
#include <iwindow.hpp>

#ifndef IC_MOTIF
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITEXTCT1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const ITextControl::textId = "ITextControl::text";

/*------------------------------------------------------------------------------
| ITextControlNotifyHandler::ITextControlNotifyHandler                         |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITextControlNotifyHandler::ITextControlNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextControlNotifyHandler::~ITextControlNotifyHandler                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITextControlNotifyHandler::~ITextControlNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextControlNotifyHandler::dispatchHandlerEvent                              |
|                                                                              |
| This notifies ITextControl Windows of the following events:                  |
|                                                                              |
| text(Id)                                                                     |
------------------------------------------------------------------------------*/
bool ITextControlNotifyHandler::dispatchHandlerEvent ( IEvent& anEvent )
{
#ifdef IC_PMWIN
  IWindow* thisWindow = 0;

  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      /**************************************************************/
      /* Never notify if we receive this message from the setText() */
      /* function.  The setText() function will send a notification */
      /* itself and we do not want to notify twice.                 */
      /**************************************************************/
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow  = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow  = IWindow::windowWithOwner(ctlevt.controlId(),
                                               ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        if (!(((ITextControl*)thisWindow)->fromSetText()))
        {
          IWindowClassName className(hwndControl);
          if (((className == WC_ENTRYFIELD) && (ulMsg == EN_CHANGE)) ||
#ifdef IC_PM
              ((className == WC_MLE) && (ulMsg == MLN_CHANGE))       ||
#endif
              ((className == WC_COMBOBOX)
#ifdef IC_WIN
                && (ulMsg == CBN_EFCHANGE || ulMsg == IC_UM_CBN_EFCHANGE)))
#else
                && (ulMsg == CBN_EFCHANGE)))
#endif
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        ITextControl::textId,
                                        *thisWindow));
          }
        }
      }
    break;
    } /* endcase */

    default:
      break;
  }  /* end switch */

#endif //IC_PMWIN

#ifdef IC_MOTIF
  if (anEvent.eventId() == motifEvent(XmCR_VALUE_CHANGED))
     {
       ITextControl* thisWindow =
          dynamic_cast<ITextControl*>( anEvent.dispatchingWindow() );

       // If the event comes from a text widget then do the notification.
       // We want to filter any of these events that come from buttons.
       if ( dynamic_cast<IEntryField*>(thisWindow) ||
            dynamic_cast<IMultiLineEdit*>(thisWindow) )
       {
         if (!thisWindow->fromSetText())
         {
             thisWindow->notifyObservers(INotificationEvent(
               ITextControl::textId, *thisWindow));
         }
       }  /* end if */
     }  /* end if */
#endif //IC_MOTIF

  return (Inherited::dispatchHandlerEvent(anEvent));
}

