// Revision: 87 1.11.1.10 source/ui/basectl/itextctl.cpp, control, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: itextctl.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itextctl.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
{
  #define INCL_GPIPRIMITIVES
  #define INCL_WINCLIPBOARD
  #define INCL_WINSYS
  #include <iwindefs.h>
  #include <string.h>
}

#ifdef IC_MOTIF
  #include <Xm/Text.h>
  #include <Xm/TextF.h>
  #include <Xm/CutPaste.h>
  #include <Xm/Label.h>
  #include <X11/Shell.h>
#endif //IC_MOTIF

#include <ibase.hpp>
#include <itextctl.hpp>
#include <itxtcprv.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <ipoint.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <ithread.hpp>
#include <ifont.hpp>
#include <inotifev.hpp>
#include <iwcname.hpp>
#include <iplatfrm.hpp>
#include <icombobs.hpp>
#ifdef IC_MOTIF
  #include <imstring.hpp>
  #include <ixmlabel.hpp>
#endif //IC_MOTIF

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITEXTCTL_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ITextControlData::ITextControlData                                           |
|                                                                              |
| This class contains special flags used by the text control                   |
------------------------------------------------------------------------------*/
ITextControlData::ITextControlData()
  : setTextFunction(false),
    containsMnemonic(false)
{}


/*------------------------------------------------------------------------------
| ITextControlData::~ITextControlData                                          |
|                                                                              |
| Destructor here for page tuning.                                             |
------------------------------------------------------------------------------*/
ITextControlData::~ITextControlData()
{}


#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ITextControlData::textWidget( Widget topParent )                             |
|                                                                              |
------------------------------------------------------------------------------*/
Widget ITextControlData :: textWidget ( Widget topParent )
{
  Cardinal numChildren;
  WidgetList children;

  // If widget is text, textField, or label, then not composite.
  // Just return widget
  if ( XtIsSubclass ( topParent, xmTextWidgetClass) ||
       XtIsSubclass ( topParent, xmTextFieldWidgetClass) ||
       XtIsSubclass ( topParent, topLevelShellWidgetClass) ||
       XtIsSubclass ( topParent, xmLabelWidgetClass) )
    return topParent;

  XtVaGetValues( ( Widget) topParent,
    XmNnumChildren, &numChildren,
    XmNchildren, &children,
    NULL );

  for ( int i = 0; i < numChildren; i++ )
    {
    if (XtIsSubclass((Widget)children[i], xmTextWidgetClass))
       {
       return (Widget)children[i];
       }
    // Only check if child is composite
    if ( XtIsComposite((Widget)children[i]) )
      {
      // Check to see if child is a form and has children.  Composite
      // is at most two deep, so just manually check each child for
      // to see if it's a parent
      Cardinal numChildren2;
      WidgetList children2;

      XtVaGetValues( ( Widget) children[i],
	XmNnumChildren, &numChildren2,
	XmNchildren, &children2,
	NULL );

      for ( int j = 0; j < numChildren2; j++ )
	{
	if (XtIsSubclass((Widget)children2[j], xmTextWidgetClass))
	   {
	   return (Widget)children2[j];
	   }
	}
      }
    }
  return 0;
}
#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| ITextControl::ITextControl                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ITextControl :: ITextControl ( )
  : fTextControlData(new ITextControlData())
{}


/*------------------------------------------------------------------------------
| ITextControl::~ITextControl                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ITextControl :: ~ITextControl ( )
{
  delete fTextControlData;
}


/*------------------------------------------------------------------------------
| ITextControl::setText                                                        |
|                                                                              |
| Treat a font change like a minimum size change.                              |
------------------------------------------------------------------------------*/
ITextControl&  ITextControl :: setText(const char* pszText)
{
  IString strOrigText = text();

#ifdef IC_PMWIN
  bool newTextMnemonic = false;
  IString newText = pszText;
#ifdef IC_WIN
  if ( newText.includes('&') )
#endif
#ifdef IC_PM
  if ( newText.includes('~') )
#endif
    newTextMnemonic = true;

  // If a mnemonic is involved in either then new or old string or
  // the strings are not the same, then update the text value
  if ( newTextMnemonic || fTextControlData->containsMnemonic ||
      (strOrigText != pszText))
  {
     fTextControlData->setTextFunction = true;
     ISize siz          = size ( );
     bool wasvisible = isVisible ( );
#ifdef IC_WIN
     IWindowClassName className( handle() );
#endif

     if (wasvisible && ((siz.width()==0) || (siz.height()==0)))
	hide();
#ifdef IC_WIN
     else if (wasvisible)
	{
	// Check for groupbox.  For this control, we do not want to disable
	// update so that the control paints correctly.  For some other
	// controls (static text, graphic buttons), we need to disable update
	// and do a refresh so our paint handlers get called.
	if ( ((ISTYLEOF( handle() ) & BS_PRIMARYSTYLES) == BS_GROUPBOX) &&
	     (className == WC_BUTTON) )
	   {
	   wasvisible = false;
	   }
	else
	   {
	   enableUpdate(false);
	   }
	}
#endif

#ifdef IC_WIN
     // If this is a Windows readonly combo box then we have to select
     // an item in the list rather than set the window text.
     if (className == WC_COMBOBOX)
     {
	unsigned long ulStyle = ISTYLEOF( handle() );
	if ((ulStyle & CBS_DROPDOWNLIST) == CBS_DROPDOWNLIST)
	{
	   if (newText == IString())
	   {
	      // Just deselect all items if null string specified.
	      handle().sendEvent( CB_SETCURSEL,
				  IEventParameter1( -1 ),
				  IEventParameter2( 0 ));
	   }
	   else
	   {
	      long match = handle().sendEvent( CB_FINDSTRINGEXACT,
				       IEventParameter1( (void*)-1 ),
				       IEventParameter2( (char*)newText ));
		 //

	      if (match != CB_ERR)
	      {
		 handle().sendEvent( CB_SETCURSEL,
				     IEventParameter1( (void*)match ),
				     IEventParameter2( 0 ));
	      }
	      else if (wasvisible)
	      {
		    enableUpdate( true );
		    refresh( IWindow::paintAll );
		    ITHROWLIBRARYERROR( IC_INVALID_SETTEXT,
					IBaseErrorInfo::invalidParameter,
					IException::recoverable);
	      }
	   }
	}
	else
	{
#ifdef IC_WIN
	   IEventResult result = handle().sendEvent( IC_UM_EDIT_SETTEXT,
					  IEventParameter1( (char *) pszText ),
					  IEventParameter2( handle() ) );

	   // if settext not handled by text change handler
	   if (!result.asUnsignedLong())
#endif //IC_WIN
	     ISETWINDOWTEXT( handle(), (PSZ)pszText );
	}
     ((IBaseComboBox *)this)->setTextChangedFlag( true );
     }
     else
#endif

     {
#ifdef IC_WIN
       IEventResult result = handle().sendEvent( IC_UM_EDIT_SETTEXT,
					  IEventParameter1( (char *) pszText ),
					  IEventParameter2( handle() ) );

       // if settext not handled by text change handler
       if (!result.asUnsignedLong())
#endif //IC_WIN
	 ISETWINDOWTEXT(handle(), (PSZ)pszText);
     }

     if (wasvisible && ((siz.width()==0) || (siz.height()==0)))
	show();
#ifdef IC_WIN
     else if (wasvisible)
	{
	enableUpdate(true);
	refresh(IWindow::paintAll);
	}
#endif

#ifdef IC_WIN
     unsigned short notifyMsg = 0;

     /*****************************************************************/
     /* Get window class name and set up notification message based   */
     /*  on which type of window is having its text changed           */
     /*****************************************************************/
     if ( className == WC_COMBOBOX )
       notifyMsg = CBN_EDITCHANGE;
     else if ( className == WC_ENTRYFIELD )
       notifyMsg = EN_CHANGE;

     /*****************************************************************/
     /* Only send notification if it is one of the above window types */
     /*****************************************************************/
     if (notifyMsg)
     {
       /***************************************************************/
       /* Since Windows only notifies on user changes, send an update */
       /* ourself to let any handlers know the text has changed       */
       /***************************************************************/
       fTextControlData->setTextFunction = false;
       parent()->handle().sendEvent( WM_COMMAND,
				     IEventParameter1(id(), notifyMsg),
				     IEventParameter2(handle()) );
     }
     else if ((className == WC_BUTTON) || (className == WC_STATIC))
	this->notifyObservers(INotificationEvent(
			      ITextControl::textId, *this));
#endif
#ifdef IC_PM
     this->notifyObservers(INotificationEvent(
			   ITextControl::textId, *this));
#endif

     // Modify the mnemonic flag based on previous check of new text string
     fTextControlData->containsMnemonic = newTextMnemonic;
     fTextControlData->setTextFunction = false;
  }

#endif //IC_PMWIN

#ifdef IC_MOTIF
  if (strOrigText != pszText)
  {                               // change in text
     fTextControlData->setTextFunction = true;
     ISize siz = size();
     bool wasvisible=isVisible();
     if (wasvisible && ((siz.width()==0) || (siz.height()==0)))
	hide();

     // Get proper widget (text, text field, or label)
     Widget wh = fTextControlData->textWidget((Widget)this->handle());

     /*************************************************************/
     /* Set the text value.  It's different for a text widget vs. */
     /* a label widget.                                           */
     /*************************************************************/
     if ( XtIsSubclass( wh,
			xmTextFieldWidgetClass ) ||
	  XtIsSubclass( wh,
			xmTextWidgetClass ) )
	{
	XmTextSetString( wh, (char *)pszText);
	}
     else  // use XmNlabelString
	{
	IMString  theText = IMString( IString(pszText) );
	XtVaSetValues(
	   wh,
	   XmNlabelString, (XmString)theText,
	   NULL );
	}

     if (wasvisible && ((siz.width()==0) || (siz.height()==0)))
	show();
     this->notifyObservers(INotificationEvent(
       ITextControl::textId, *this));
     fTextControlData->setTextFunction = false;
  } /* endif */
#endif //IC_MOTIF

  return *this;
}


/*------------------------------------------------------------------------------
| ITextControl::setText                                                        |
|                                                                              |
| Sets the control window text using string table.                             |
------------------------------------------------------------------------------*/
ITextControl&  ITextControl :: setText(const IResourceId& residText)
{
   // Try to load the string from the specified resource library.  If this
   // fails and the ID is in the range of the resource IDs used by IOC,
   // try to load the string from the resourceLibrary, where the default
   // strings for things like toolbar buttons are stored.
   IString str;
   if ( residText.id() >= IC_ID_BASE )
       {
       str = residText.resourceLibrary().tryToLoadString( residText.id() );
       if (str.length() == 0)
	   {
	   // Load from class library resource
	   str = IApplication::current()
		     .resourceLibrary()
		     .loadString(residText.id());
	   }
       }
   else
       {
       str = residText.resourceLibrary().loadString( residText.id() );
       }
   setText((char*)str);
   return *this;
}


/*------------------------------------------------------------------------------
| ITextControl::text                                                           |
|                                                                              |
| Returns the control window's text.                                           |
------------------------------------------------------------------------------*/
IString  ITextControl :: text() const
{
#ifdef IC_PMWIN
  unsigned long ulClBufLen =
	   (unsigned long)IQUERYWINDOWTEXTLENGTH(handle()) + 1;

  char* pszClBuf = new char[ulClBufLen];
  IQUERYWINDOWTEXT(handle(), ulClBufLen, (PSZ)pszClBuf);

  IString str(pszClBuf);

  delete [] pszClBuf;
  return str;
#endif //IC_PMWIN

#ifdef IC_MOTIF
   IString         textString;

   // Get proper widget (text, text field, or label)
   Widget wh = fTextControlData->textWidget((Widget)this->handle());

   if ( XtIsSubclass( wh,
		      xmTextFieldWidgetClass ) ||
	XtIsSubclass( wh,
		      xmTextWidgetClass ) )
   {
      // d7950 - Need to free result from XmTextGetString.
      // textString = XmTextGetString( this->handle() );
      char* result = XmTextGetString( wh );
      textString = result;
      XtFree( result );
   }
   else  // use XmNlabelString
   {
      textString = IMString( wh, XmNlabelString).asString( ) ;
   }

   return textString;
#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| ITextControl::textLength                                                     |
|                                                                              |
| Returns the current length of the control window text, in bytes.             |
------------------------------------------------------------------------------*/
unsigned long  ITextControl :: textLength() const
{
#ifdef IC_PMWIN
  return (unsigned long)IQUERYWINDOWTEXTLENGTH(handle());
#endif //IC_PMWIN

#ifdef IC_MOTIF
  return strlen( text() );
#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| ITextControl::displaySize                                                    |
|                                                                              |
| Returns the width and height of the rectangle that encloses the string       |
| when it is displayed on the window's presentation space.  This function      |
| takes into account the font attribute of the presentation space, including   |
| sheared or rotated string.                                                   |
------------------------------------------------------------------------------*/
ISize ITextControl:: displaySize(const char* pszText) const
{
#ifdef IC_PMWIN
   IString theText(pszText);

   if (pszText == 0)
      theText = text();

   IFont  font(this);
   ISize  result(font.textWidth( theText ), font.maxCharHeight()  );
#ifdef IC_WIN
   DWORD  dwStyle = GetWindowLong( handle(), GWL_EXSTYLE );
/*
   if (dwStyle & WS_EX_CLIENTEDGE)
       // 3d Border in effect, so add 4 and return information from font
       result += ISize( 4, 4);
*/
   if ( (dwStyle & WS_EX_CLIENTEDGE) &&
	( IPlatform::isWin9x() || IPlatform::isNTNewShell() ) )
   {
     // 22733 : y fudge factor of at least border+2 required, for resizing
     //         edit area which was created too small for its text.
     //         (using *2 instead allows for both edge borders, and returns
     //         approximately the same height as calcMinimumSize())
     int cx = IQUERYSYSVALUE( SV_CXSIZEBORDER )*2;
     int cy = IQUERYSYSVALUE( SV_CYSIZEBORDER )*2;

     // Add border width to information from font
     result += ISize( cx, cy);
   }

#endif   //IC_WIN

     // Return information from font
   return result;
#endif //IC_PMWIN

#ifdef IC_MOTIF
   if (pszText == 0)
      {
      pszText = (char*)text();
      } /* endif */

   // create an XmString and compute its size
   IMString theText = IMString( IString( pszText ) );

   // Get proper widget (text, text field, or label)
   Widget wh = fTextControlData->textWidget((Widget)this->handle());

   return  theText.size( wh ) ;
#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| ITextControl::clipboardHasTextFormat                                         |
|                                                                              |
| Queries whether the clipboard is in text format.                             |
------------------------------------------------------------------------------*/
bool  ITextControl :: clipboardHasTextFormat ( )
{
#ifdef IC_PMWIN
  unsigned long userInfo;

  bool bOpen =
	      IQUERYCLIPBRDFMTINFO(IThread::current().anchorBlock(),
				   CF_TEXT, &userInfo) ?
					       true : false;
  return bOpen;
#endif //IC_PMWIN

#ifdef IC_MOTIF
   int            rc, formatCount;
   ulong          maxLength, copiedLength;
   Display        *display;
   Window         window;
   char           *formatNameBuf = NULL;
   bool        result = false;

   IWindowHandle appShell = IWindow::desktopWindow()->handle();
   display = XtDisplay ((Widget) appShell );
   window = XtWindow ((Widget) appShell );

   // Get the number of registered formats for the clipboard.
   if ((rc = XmClipboardInquireCount (display,
				      window,
				      &formatCount,
				      &maxLength)) == ClipboardSuccess)
     {
      formatNameBuf = XtMalloc (maxLength);
      for (int i=1; i<=formatCount && rc==ClipboardSuccess && !result; i++)
	{
	 rc = XmClipboardInquireFormat (display,
					window,
					i,
					formatNameBuf,
					maxLength,
					&copiedLength);
	 // If the format type is "STRING", it is text format.
	 if (!strncmp (formatNameBuf, "STRING", copiedLength))
	    // Data in clipboard is text format.
	    result = true;
	}
      XtFree (formatNameBuf);
     }

   if (rc == ClipboardLocked)
      ITHROWLIBRARYERROR (IC_CLIPBOARD_LOCKED,
			  IBaseErrorInfo::accessError,
			  IException::recoverable);

   if (rc == ClipboardNoData)
      ITHROWLIBRARYERROR (IC_CLIPBOARDEMPTY,
			  IBaseErrorInfo::accessError,
			  IException::recoverable);

   if (rc == ClipboardTruncate)
      ITHROWLIBRARYERROR (IC_CLIPBOARD_TRUNCATE,
			  IBaseErrorInfo::accessError,
			  IException::recoverable);

   return result;
#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| ITextControl::setLayoutDistorted                                             |
|                                                                              |
| Queries whether the clipboard is in text format.                             |
------------------------------------------------------------------------------*/
ITextControl &  ITextControl :: setLayoutDistorted(
					      unsigned long layoutAttributeOn,
					      unsigned long layoutAttributeOff )
{
  unsigned long ulFlagsOn = layoutAttributeOn;
  if (layoutAttributeOn & IWindow::fontChanged)
  {                         // set flag for IControl::characterSize()
     ulFlagsOn |= IWindow::minimumSizeChanged;  // flag for ICanvas
  }
  Inherited::setLayoutDistorted(ulFlagsOn, layoutAttributeOff);

  return *this;
}

/*------------------------------------------------------------------------------
| ITextControl::fromSetText                                                    |
|                                                                              |
| Queries whether communication was from setText() function.                   |
------------------------------------------------------------------------------*/
bool ITextControl :: fromSetText ( ) const
{
  return fTextControlData->setTextFunction;
}
