/* Revision: 06 1.3.1.2 source/ui/cclnt/cnr/cnrreg.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                             IBM Confidential                       */
/* ------------------------------------------------------------------ */
/*                                                                    */
/*                                                                    */
/*   Graphic and Workplace API                                        */
/*                                                                    */
/*                                                                    */
/*   Services:                                                        */
/*                                                                    */
/*   Source File Name:  CNRREG.C                                      */
/*                                                                    */
/*   Copyright: (c) IBM Corp. 1981, 1997                              */
/*                                                                    */
/*   Status:    OS/2 Release 2.0                                      */
/*                                                                    */
/*   Functional Description:                                          */
/*                                                                    */
/*   Notes:  None                                                     */
/*                                                                    */
/*      Dependencies:  None                                           */
/*                                                                    */
/*               Defines:  INCL_WIN                                   */
/*                                                                    */
/*              Includes:  <os2.h>                                    */
/*                                                                    */
/*      Restrictions:  None                                           */
/*                                                                    */
/*   Entry Points:                                                    */
/*                                                                    */
/*   External References:                                             */
/*                     [Presentation Manager API's]                   */
/*                                                                    */
/*                     [Dos API's]                                    */
/*                                                                    */
/*                     [Gpi API's]                                    */
/*                                                                    */
/*                     [C functions]                                  */
/*                                                                    */
/*                     [Macros]                                       */
/*                                                                    */
/*  Change History:                                                   */
/*                                                                    */
/*  Rel  Programmer    Date      Description                          */
/*  ---  ----------    ----      -----------                          */
/* pfh01 Peter Haggar  8/21/91   Created.  This function used to      */
/*                               exist in cnrs3p4.c and was moved to  */
/*                               its own C file for the 1 DLL work.   */
/* xxxxx Ray Kwok      4/30/96   Add changes required for Wind/U.     */
/**********************************************************************/
#include <windows.h>
#include "cnr2win.h"
#include "iclcnrw.h"
#include "cnrcbn.h"

// Segment definitions
#ifdef IC_PAGETUNE
  #define _CNRREG_C_
  #include "icltune.h"
#endif

typedef struct _REGDATA {
  LPSTR    pszClass;
  WNDPROC  lpfnWndProc;
  WORD     wClassStyle;
  WORD     cbExtra;
  BOOL     fRegistered;
} REGDATA;
typedef REGDATA FAR *PREGDATA;

#ifdef SEGMENTOR
#define CNRREG_C
#include "cnrpragw.h"
#endif

BOOL CnrIsRegistered (HANDLE hInstance, PREGDATA prdat);

/**********************************************************************/
/* Globals                                                            */
/**********************************************************************/
HANDLE   vhCNRInstance = NULL;
//HBITMAP  vhbmExpand = NULL;
//HBITMAP  vhbmCollapse = NULL;
//HCURSOR  vhcurSplitPtr = NULL;
int      viUseCount;

#if !defined(CCL_SEPARATE_DLL)
#define LibMain ContainerLibMain
#endif

INT  APIENTRY LibMain(HANDLE hInstance, DWORD ul_reason_being_called, LPVOID lpReserved);

#if defined(CCL_SEPARATE_DLL)

#if defined(__IBMC__)

/* _CRT_init is the C run-time environment initialization function.         */
/* It will return 0 to indicate success and -1 to indicate failure.         */
int _CRT_init(void);

/* _DLL_InitTerm is the function that gets called by the operating system   */
/* loader when it loads and frees this DLL for each process that accesses   */
/* this DLL.  However, it only gets called the first time the DLL is loaded */
/* and the last time it is freed for a particular process.  The system      */
/* linkage convention MUST be used because the operating system loader is   */
/* calling this function.                                                   */
/* See DllEntryPoint in SDK documentation                                   */

INT APIENTRY          _DLL_InitTerm(HANDLE hInstance,
                                    DWORD  reasonCalled,
                                    LPVOID reserved );

INT APIENTRY          _DLL_InitTerm(HANDLE hInstance,
                                    DWORD  reasonCalled,
                                    LPVOID reserved )
{
   INT  rc = FALSE;     /* failed */
   switch (reasonCalled)
      {
      case  DLL_PROCESS_ATTACH:
         /*******************************************************************/
         /* The C run-time environment initialization function must be      */
         /* called before any calls to C run-time functions that are not    */
         /* inlined.                                                        */
         /*******************************************************************/

         if (_CRT_init() != -1)
            {
            /* initialize the controls */
            rc = LibMain( hInstance, reasonCalled, reserved );
            }
         break;

      case  DLL_THREAD_ATTACH:
      case  DLL_THREAD_DETACH:
      case  DLL_PROCESS_DETACH:
         /* We currently don't do anything with these */
         break;
      }
   return rc;
}

#endif  // IBMC

#endif  //CCL_SEPARATE_DLL

/****************************************************************************
   FUNCTION: LibMain(HANDLE, DWORD, LPVOID)

   PURPOSE:  LibMain is called by Windows when
             the DLL is initialized, Thread Attached, and other times.
             Refer to SDK documentation, as to the different ways this
             may be called.

             The LibMain function should perform additional initialization
             tasks required by the DLL.  In this example, no initialization
             tasks are required.  LibMain should return a value of 1 if
             the initialization is successful.

*******************************************************************************/
INT  APIENTRY LibMain(HANDLE hInstance,
                      DWORD  ul_reason_being_called,
                      LPVOID lpReserved)
{
  WNDCLASS wndclass;
  WORD i;
  BOOL rc;

  static REGDATA rdat[] = {
    { ICL_CONTAINER,        CnrGateProc,              0, 12, 0 },
    { WC_EDGE,              CnrEdgeProc,              0, 0, 0 },
    { WC_CNRTITLE,          CnrTitleProc,             0, 0, 0 },
    { WC_LDETAIL,           CnrLeftDetailsWndProc,    0, 0, 0 },
    { WC_RDETAIL,           CnrRightDetailsWndProc,   0, 0, 0 },
    { WC_LCOLTITLE,         CnrLeftColumnTitlesProc,  0, 0, 0 },
    { WC_RCOLTITLE,         CnrRightColumnTitlesProc, 0, 0, 0 }};

  /********************************************************************/
  /* Save Instance handle                                             */
  /********************************************************************/
  vhCNRInstance = hInstance;

  for (i=0; i < (sizeof (rdat)/sizeof (REGDATA)); i++)
  {
    if (!CnrIsRegistered (hInstance, rdat+i))
    {

      /****************************************************************/
      /* Initialize use count                                         */
      /****************************************************************/
      viUseCount = 0;

      /****************************************************************/
      /* Initialize registration structure                            */
      /****************************************************************/
      wndclass.style         = rdat[i].wClassStyle |
                               CS_HREDRAW | CS_VREDRAW |
                               CS_GLOBALCLASS | CS_DBLCLKS;
      wndclass.lpfnWndProc   = rdat[i].lpfnWndProc;
      wndclass.cbClsExtra    = 0;
      wndclass.cbWndExtra    = rdat[i].cbExtra;
      wndclass.hInstance     = hInstance;
#ifdef IC_WU
      wndclass.hIcon         = LoadIcon( (HINSTANCE) NULL, IDI_APPLICATION);
      wndclass.hCursor       = LoadCursor( (HINSTANCE) NULL, IDC_ARROW);
#else
      wndclass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
      wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
#endif
      wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
      wndclass.lpszMenuName  = NULL;
      wndclass.lpszClassName = rdat[i].pszClass;

      /****************************************************************/
      /* Register the container class                                 */
      /****************************************************************/
      rc = RegisterClass(&wndclass);

      if (!rc)
      {
        return(FALSE);
      }

      rdat[i].fRegistered = TRUE;
    }
  }

  /********************************************************************/
  /* Load Split cursor                                                */
  /********************************************************************/
//vhcurSplitPtr = LoadCursor (hInstance,
//                                  MAKEINTRESOURCE(SPLITBARPTR));

  /********************************************************************/
  /* Load default bitmaps for TreeView                                */
  /********************************************************************/
//vhbmCollapse = LoadBitmap (hInstance, MAKEINTRESOURCE(TREEMINUS));
//vhbmExpand = LoadBitmap (hInstance, MAKEINTRESOURCE(TREEPLUS));

  return(TRUE);
  ul_reason_being_called;
  lpReserved;
}


BOOL CnrIsRegistered (HANDLE hInstance, PREGDATA prdat)
{
  WNDCLASS clsi;

  return (GetClassInfo (hInstance, prdat->pszClass, &clsi));
}
