/* Revision: 08 1.3.1.2 source/ui/cclnt/cnr/cnrs1p2.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS1P2.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS1P2.C - Details view and column positioning routines */
/*                       for the container control                    */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* DAB2  10/11/95  DAB  Fixed problem with ownerdraw cnr id in mp1    */
/* D6386  1/16/96  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/**********************************************************************/
#include <windows.h>
#include <iclccl.h>     /* CCL common include file  */ 
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef SEGMENTOR
#define CNRS1P2_C
#include "cnrpragw.h"
#endif

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrQueryColumnPosInDV                             */
/*                                                                    */
/*  Descriptive Name:  Query the column position of a specific        */
/*                     column in Details View.                        */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function returns the column position of a specific column.   */
/*  It will return either the position of the left side or the        */
/*  position of the right side.                                       */
/*                                                                    */
/*  Notes:  If Last is True, all other parameters are ignored.        */
/*                                                                    */
/*  Input:  hwndCnr    - Handle of the Container Window.              */
/*          pStartFI   - Pointer to a FieldInfo structure in which    */
/*                       this function is to start from.              */
/*          pFieldInfo - Pointer to a FieldInfo structure in which    */
/*                       the position is requested.                   */
/*          LeftSide   - Boolean value to determine if the function   */
/*                       is to return the position of the left side   */
/*                       of the column or the right side.             */
/*          Last       - Boolean value to give the function a fast    */
/*                       way to return the rightmost point of the     */
/*                       last column.                                 */
/*                                                                    */
/*  Output:  The numeric column position of the specifed column.      */
/*                                                                    */
/*  Exit-Normal: The column position of the specified column is       */
/*               returned.                                            */
/*                                                                    */
/*  Exit-Error:  NULL is returned.                                    */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
int FAR CnrQueryColumnPosInDV (hwndCnr, pStartFI, pFieldInfo,
                               LeftSide, Last)

HANDLE      hwndCnr;     /* Handle of the Container Window.           */
LPFIELDINFO pStartFI;    /* Pointer to a FieldInfo structure in which */
                         /* to start from.                            */
LPFIELDINFO pFieldInfo;  /* Pointer to a FieldInfo structure in which */
                         /* the position is requested from.           */
BOOL        LeftSide;    /* Indicates if we are to return the left    */
                         /* side of the column or the right side.     */
BOOL        Last;        /* Indicates if we are to return the right-  */
                         /* most point of the last column.            */

{
 LPCNRINFON  pccinfon;       /* Pointer to Internal Control info.     */
 LPFIELDINFO pFI;            /* Pointer to a FieldInfo structure.     */
 int         ReturnVal;      /* Position of column to be returned.    */
 int         CxBetweenCols;  /* Distance between columns.             */

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Initially set the Return value to be the distance between the     */
 /* left border of the window and the first column.                   */
 /*********************************************************************/
 ReturnVal = pccinfon->lAveCharWidth;

 /*********************************************************************/
 /* The distance between the columns is 3 times the average character */
 /* width.                                                            */
 /*********************************************************************/
 CxBetweenCols = 3 * (pccinfon->lAveCharWidth);

 /*********************************************************************/
 /* Point to the FieldInfo structure the we are to start calculating  */
 /* from.                                                             */
 /*********************************************************************/
 pFI = pStartFI;

 /*********************************************************************/
 /* First take care of the case where we want to get the rightmost    */
 /* position of the last column in Details View.                      */
 /*********************************************************************/
 if (Last)
 {
    /******************************************************************/
    /* Start at the starting column that was passed in, and continue  */
    /* for all of the FieldInfo's currently inserted.                 */
    /******************************************************************/
    while (pFI)
    {
       /***************************************************************/
       /* If the application specified the width of their columns,    */
       /* add this amount.  Also, set our flag to FALSE.              */
       /***************************************************************/
       if (pFI->cxWidth)
       {
          ReturnVal += pFI->cxWidth + CxBetweenCols;
       }
       else
       /***************************************************************/
       /* Otherwise the width of the columns are determined by the    */
       /* longest line in the column.  Also, we need to add in the    */
       /* distance between the columns.                               */
       /***************************************************************/
       {
          ReturnVal += PFINFROMX(pFI)->usLongestLine + CxBetweenCols;
       }

       pFI = NEXTVISIBLEFI(pFI);
    }

   /*******************************************************************/
   /* Subtract off the distance between the columns that was added to */
   /* the end since we are returning the rightmost point of the       */
   /* column.                                                         */
   /*******************************************************************/
   ReturnVal -= CxBetweenCols;

   return (ReturnVal);
 }

 if (pFieldInfo)
 {
    /******************************************************************/
    /* If we are finding the left side of the column......            */
    /******************************************************************/
    if (LeftSide)
    {
       while ((pFI) && (pFI != pFieldInfo))
       {
          if (pFI->cxWidth)
          {
             ReturnVal += pFI->cxWidth + CxBetweenCols;
          }
          else
          {
             ReturnVal += PFINFROMX(pFI)->usLongestLine +
                          CxBetweenCols;
          }

          pFI = NEXTVISIBLEFI(pFI);
       }

     return (ReturnVal);

    }
    else
    /******************************************************************/
    /* We are finding the right side of a column.......               */
    /******************************************************************/
    {
       while ((pFI) && (pFI != NEXTVISIBLEFI(pFieldInfo)))
       {
          if (pFI->cxWidth)
          {
             ReturnVal += pFI->cxWidth + CxBetweenCols;
          }
          else
          {
             ReturnVal += PFINFROMX(pFI)->usLongestLine +
                          CxBetweenCols;
          }

          pFI = NEXTVISIBLEFI(pFI);
       }

       ReturnVal -= CxBetweenCols;

       return (ReturnVal);
    }
 }

  return (0);                                                           /*DAB1*/
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrQueryColumnTitleWndRect (hwndCnr, LeftWindow, TitleWndRect)

HANDLE  hwndCnr;
BOOL    LeftWindow;
LPRECT  TitleWndRect;

{
 LPCNRINFON  pccinfon;    /* Pointer to Internal control information. */
 LPCNRINFO   pccinfo;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 if (pccinfo->flWindowAttr & CA_DETAILSVIEWTITLES)
 {
    if (LeftWindow)
    {
       PMGetClientRect( GetDlgItem( hwndCnr, CID_LEFTCOLTITLEWND ),
                        TitleWndRect );
    }
    else
    {
       PMGetClientRect( GetDlgItem( hwndCnr, CID_RIGHTCOLTITLEWND ),
                        TitleWndRect );
    }
 }
 else
 {
    TitleWndRect->left   = 0;
    TitleWndRect->right  = 0;
    TitleWndRect->bottom = 0;
    TitleWndRect->top    = 0;
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LPFIELDINFO FAR CnrQueryLastColumnInView (hwndCnr, pFirstInView,
                                          LeftWindow)

HANDLE      hwndCnr;
LPFIELDINFO pFirstInView;
BOOL        LeftWindow;

{
 LPCNRINFON  pccinfon;
 LPFIELDINFO pFieldInfo;
 LPFIELDINFO pLast;
 long        right;
 long        CxBetweenCols;
 RECT        ViewPort;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 if ((pccinfon->pFirstVisibleFI) && (pFirstInView))
 {
    CxBetweenCols = 3 * (pccinfon->lAveCharWidth);

    if (LeftWindow)
    {
       /***************************************************************/
       /* Get the viewport of the Left window in Details View.        */
       /***************************************************************/
       PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                           &ViewPort);

       /***************************************************************/
       /* Get the right side of the first column in view in the left  */
       /* window.                                                     */
       /***************************************************************/
       right = CnrQueryColumnPosInDV (hwndCnr,
                                       pccinfon->pFirstVisibleFI,
                                       pFirstInView,
                                       FALSE,
                                       FALSE) + (CxBetweenCols / 2);

       /***************************************************************/
       /* Convert right to be relative to the ViewPort.  It is       */
       /* returned relative to the Origin.                            */
       /***************************************************************/
       right -= pccinfon->ptlNonIconOrg.x;

       /***************************************************************/
       /* Loop until we find a column thats right side is to the right*/
       /* of the viewport.  This is our last column in View.          */
       /***************************************************************/
       if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
         (CnrVerifyFieldInfoPtrs (hwndCnr, pccinfon->pFieldInfoLast, 1)))
       {
          pFieldInfo = pFirstInView;

          while ((pFieldInfo) &&
                 (pFieldInfo != (pccinfon->pFieldInfoLast)) &&
                 (right < ViewPort.right))
          {
             pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

             right = CnrQueryColumnPosInDV (hwndCnr,
                                             pccinfon->pFirstVisibleFI,
                                             pFieldInfo,
                                             FALSE,
                                             FALSE) +
                                             (CxBetweenCols / 2);

             /*********************************************************/
             /* Convert right to be relative to the ViewPort.  It is */
             /* returned relative to the Origin.                      */
             /*********************************************************/
             right -= pccinfon->ptlNonIconOrg.x;
          }

          return (pFieldInfo);
       }
       else
       /***************************************************************/
       /* There is no data in the right window.                       */
       /***************************************************************/
       {
          pFieldInfo = pFirstInView;

          while ((pFieldInfo) && (right < ViewPort.right))
          {
             pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

             right = CnrQueryColumnPosInDV (hwndCnr,
                                             pccinfon->pFirstVisibleFI,
                                             pFieldInfo,
                                             FALSE,
                                             FALSE) +
                                             (CxBetweenCols / 2);

             /*********************************************************/
             /* Convert right to be relative to the ViewPort.  It is */
             /* returned relative to the Origin.                      */
             /*********************************************************/
             right -= pccinfon->ptlNonIconOrg.x;
          }

          /************************************************************/
          /* If pFieldInfo is valid, return it.  Otherwise it is NULL */
          /* which means that the last FieldInfo in the list is to    */
          /* left of the window boundary.  So, we get the last one in */
          /* the list and return it.                                  */
          /************************************************************/
          if (pFieldInfo)
          {
             return (pFieldInfo);
          }
          else
          {
             pLast = pFirstInView;

             while (NEXTVISIBLEFI(pLast))
             {
                pLast = NEXTVISIBLEFI(pLast);
             }

             return (pLast);
          }
       }
    }
    else
    /*********************************************************************/
    /* We are looking in the right window.                               */
    /*********************************************************************/
    {
       /***************************************************************/
       /* Get the viewport of the Right window in Details View.       */
       /***************************************************************/
       PMGetClientRect (GetDlgItem (hwndCnr, CID_RIGHTDVWND),
                           &ViewPort);

       /***************************************************************/
       /* Get the right side of the first column in view in the right */
       /* window.                                                     */
       /***************************************************************/
       right = CnrQueryColumnPosInDV (hwndCnr,
                         NEXTVISIBLEFI(pccinfon->pFieldInfoLast),
                                       pFirstInView,
                                       FALSE,
                                       FALSE) + (CxBetweenCols / 2);

       /***************************************************************/
       /* Convert right to be relative to the ViewPort.  It is       */
       /* returned relative to the Origin.                            */
       /***************************************************************/
       right -= pccinfon->RightWndxOrg;

       /***************************************************************/
       /* Loop until we find a column thats right side is to the right*/
       /* of the viewport.  This is our last column in View.          */
       /***************************************************************/
       pFieldInfo = pFirstInView;

       while ((pFieldInfo) && (right < ViewPort.right))
       {
          pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

          right = CnrQueryColumnPosInDV (hwndCnr,
                            NEXTVISIBLEFI(pccinfon->pFieldInfoLast),
                                          pFieldInfo,
                                          FALSE,
                                          FALSE) + (CxBetweenCols / 2);

          /************************************************************/
          /* Convert right to be relative to the ViewPort.  It is    */
          /* returned relative to the Origin.                         */
          /************************************************************/
          right -= pccinfon->RightWndxOrg;
       }

       /***************************************************************/
       /* If pFieldInfo is valid, return it.  Otherwise it is NULL    */
       /* which means the last column in view is to the left of the   */
       /* window boundary.  Therefore return the last column in the   */
       /* right window.                                               */
       /***************************************************************/
       if (pFieldInfo)
       {
          return (pFieldInfo);
       }
       else
       {
          pLast = pFirstInView;

          while (NEXTVISIBLEFI(pLast))
          {
             pLast = NEXTVISIBLEFI(pLast);
          }

          return (pLast);
       }
    }
 }

 return (NULL);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
int FAR CnrQueryRowPosInDV (pRecord, TopOrBot)

LPRECORDCOREN pRecord;
long          TopOrBot;

{
  int         ReturnVal=0;

  /********************************************************************/
  /* Check for a valid record passed in                               */
  /********************************************************************/
  if (pRecord)
  {
    if (TopOrBot == PT_TOP)
    {
      ReturnVal = pRecord->ptlNonIconPos.y -
                  (pRecord->RowHeight - pRecord->slTextSize.cy) / 2;    /*DAB1*/

      /* Stupid hack needed.....
       */
      if ((pRecord->RowHeight - pRecord->slTextSize.cy) & NUM_ODD)      /*DAB1*/
      {
        ReturnVal--;
      }
    }
    else if (TopOrBot == PT_BOTTOM)
    {
      ReturnVal = pRecord->ptlNonIconPos.y + (pRecord->slTextSize.cy +  /*DAB1*/
                     (pRecord->RowHeight - pRecord->slTextSize.cy) / 2);/*DAB1*/

      /* Stupid hack needed.....
       */
      if ((pRecord->RowHeight - pRecord->slTextSize.cy) & NUM_ODD)      /*DAB1*/
      {
        ReturnVal++;
      }
    }
  }
  return (ReturnVal);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrRowHeightChanged                               */
/*                                                                    */
/*  Descriptive Name:  Did the height of the specified row change?    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will calculate the height of the row that is        */
/*  specfied by pRecord.                                              */
/*  Then the function will compare this value to the stored value     */
/*  that represents the previous height of the row.  If the row       */
/*  height has changed, TRUE is returned.  Otherwise FALSE is         */
/*  returned.                                                         */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  When pRecord is NULL, the height of the row occupied by the       */
/*  Column Titles is calculated.                                      */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*          pRecord - Pointer to a record that we want to calculate   */
/*                    the row height for.                             */
/*                                                                    */
/*  Output:  TRUE  - The height of the row has changed.               */
/*           FALSE - The height of the row has not changed.           */
/*                                                                    */
/*  Exit-Normal:  TRUE or FALSE                                       */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrRowHeightChanged (hwndCnr, pRecord)

HANDLE        hwndCnr;  /* Handle of the container window.            */
LPRECORDCOREN pRecord;  /* Pointer to a record that we want to calc.  */
                        /* the row height for.                        */

{
 LPCNRINFON  pccinfon;       /* Pointer to Internal Container Info.   */
 LPFIELDINFO pFieldInfo;     /* Pointer to a FieldInfo structure.     */
 POINT       IconSize;       /* The width and height of an Icon.      */
 SIZEL       TextSize;       /* The width and height of the Text in   */
                             /* the row.                              */
 LPSTR       String;         /* The string that we are calculating    */
                             /* the TextSize for.                     */
 HDC         hps;            /* Handle to a presentation space.       */
// LPVOID      pSatPtr;        /* Pointer to the storage space.         */
//ERRORID     ErrorID;        /* ID of an error which may be returned. */
 int         RowHeight = 0;  /* Height of the row which we are        */
                             /* calculating.                          */

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 pFieldInfo = pccinfon->pFirstVisibleFI;

 if (!pFieldInfo)
 {
    return (FALSE);
 }

 hps = PMGetFlippedDC (hwndCnr);
 CnrCalcIconSize (hwndCnr, &IconSize);

 /*********************************************************************/
 /* Loop through all of the columns for the specified row, and        */
 /* calculate the new height of the row.                              */
 /*********************************************************************/
 while (pFieldInfo)
 {
    if (pFieldInfo->flData & CFA_BITMAPORICON)
    {
       if (IconSize.y > RowHeight)
       {
          RowHeight = IconSize.y;
       }
    }
    else
    {
       if (pRecord)
       {
          /************************************************************/
          /* If the string is NULL, we need to check to see why it is */
          /* NULL.  If there is no memory left in the system, then we */
          /* need to return NULL, otherwise it is just a NULL string, */
          /* and we can keep processing.                              */
          /************************************************************/
          if (!(String = CnrGetColumnData (hwndCnr,
                                           pRecord->preccX,
                                           pFieldInfo)))
          {
//           ErrorID = GetLastError();
//
//           if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//           {
//              SETERROR(ErrorID);
//              ReleaseDC (hps);
//              return (FALSE);
//           }

             /*********************************************************/
             /* After we have looked at the error, we need to reset   */
             /* it.  Yeah, it's a waste of time, but PM doesn't give  */
             /* you a direct way to peek at Error information.        */
             /*********************************************************/
//           SETERROR(ErrorID);
          }
       }
       else
       {
          ReleaseDC (hwndCnr, hps);
          return (FALSE);
       }

       IcnCalcTextSize (hps, IconSize, pccinfon,
                        String, IS_SIDETEXT, &TextSize);

       /***************************************************************/
       /* Free the storage that was allocated for String only if we   */
       /* are processing anything but a Bitmap or a String.  This is  */
       /* because in these 2 cases, there is no extra memory that is  */
       /* allocated.  Also, if pRecord doesn't exist, that means we   */
       /* are processing Column Title Data, and no extra memory was   */
       /* allocated.                                                  */
       /***************************************************************/
       if ((pRecord) && (!(pFieldInfo->flData & (CFA_BITMAPORICON |
                                                 CFA_STRING))))
       {
//        pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
//        StmFreeStorage (pSatPtr, (LPVOID)String);
          GlobalFree( (LPVOID)String );                                 /*DAB1*/
       }

       if (TextSize.cy > RowHeight)                                     /*DAB1*/
       {
          RowHeight = TextSize.cy;                                      /*DAB1*/
       }
    }

    pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
 }

 ReleaseDC (hwndCnr, hps);

 /*********************************************************************/
 /* If the new height of the row is different than the old height, we */
 /* return TRUE.  Otherwise, return FALSE.                            */
 /*********************************************************************/
 if (RowHeight != (int)pRecord->RowHeight)
 {
    return (TRUE);
 }
 else
 {
    return (FALSE);
 }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrSetFirstColInView                              */
/*                                                                    */
/*  Descriptive Name:  Set the First column in View for the left and  */
/*                     right Details View windows.                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function determines the first column that is currently in    */
/*  view for the left and right Details View windows.  For the left   */
/*  window, it starts with the first column and gets its rightmost    */
/*  point.  It checks to see if this point is in the left viewport,   */
/*  and continues until it finds the correct column.  For the right   */
/*  window, it does the same thing, but starts with the first column  */
/*  in the right window.                                              */
/*                                                                    */
/*  Notes:  This function is called each time the left or right       */
/*          Details View windows are scrolled horizontally.           */
/*                                                                    */
/*  Input:  hwndCnr    - Handle of the Container Window.              */
/*          WindowLeft - Boolean value to determine which window we   */
/*                       are working with.                            */
/*                                                                    */
/*  Output:  VOID                                                     */
/*                                                                    */
/*  Exit-Normal: VOID                                                 */
/*                                                                    */
/*  Exit-Error:  VOID                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrSetFirstColInView (hwndCnr, WindowLeft)

HANDLE      hwndCnr;     /* Handle of the Container Window.           */
BOOL        WindowLeft;  /* Determines if we are setting the first in */
                         /* view for the left window or right window. */

{
 LPCNRINFON  pccinfon;    /* Pointer to Internal control information. */
 LPFIELDINFO pFieldInfo;  /* Pointer to a FieldInfo structure.        */
 long        right;       /* The rightmost point of a specific column */
 long        CxBetweenCols;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 CxBetweenCols = 3 * (pccinfon->lAveCharWidth);

 /*********************************************************************/
 /* Check to see if we are setting the first in view for the left     */
 /* window.                                                           */
 /*********************************************************************/
 if (WindowLeft)
 {
    /******************************************************************/
    /* Start searching from the very first FieldInfo structure        */
    /* (column).                                                      */
    /******************************************************************/
    pFieldInfo = pccinfon->pFirstVisibleFI;

    if (pFieldInfo)
    {
       /***************************************************************/
       /* Get the right side of the first column.                     */
       /***************************************************************/
       right = CnrQueryColumnPosInDV(hwndCnr, pccinfon->pFirstVisibleFI,
                                     pFieldInfo, FALSE, FALSE );

       /***************************************************************/
       /* We must take into account the distance between the columns. */
       /* Therefore we add half the distance between the columns to   */
       /* this value.                                                 */
       /***************************************************************/
       right += (CxBetweenCols / 2);
    }

    /******************************************************************/
    /* If the right side of the column is not in the viewport, then   */
    /* go to the next column (FieldInfo) and check it.                */
    /******************************************************************/
    while ((pFieldInfo) && (pFieldInfo != pccinfon->pFieldInfoLast) &&
           (right < pccinfon->ptlNonIconOrg.x))
    {
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

       right = CnrQueryColumnPosInDV(hwndCnr, pccinfon->pFirstVisibleFI,
                                     pFieldInfo, FALSE, FALSE );

       right += (CxBetweenCols / 2);
    }

    /******************************************************************/
    /* If we found the first in view column, set it.  Otherwise, set  */
    /* the first in view to be the first column in the list.          */
    /******************************************************************/
    if (pFieldInfo)
    {
       pccinfon->pFIFirstColLeftInDV = pFieldInfo;
    }
    else
    {
       pccinfon->pFIFirstColLeftInDV = pccinfon->pFirstVisibleFI;
    }
 }
 else
 /*********************************************************************/
 /* We scrolled the right window in Details View, so we have to find  */
 /* the first column in view for the right window.                    */
 /*********************************************************************/
 {
    /******************************************************************/
    /* Start with the first column in the right window.               */
    /* pccinfon->pFieldInfoLast represents the rightmost column in the*/
    /* left window.  Therefore, we start 1 FieldInfo beyond that one. */
    /******************************************************************/
    pFieldInfo = NEXTVISIBLEFI(pccinfon->pFieldInfoLast);

    if (pFieldInfo)
    {
       /***************************************************************/
       /* Get the rightmost of this column.                           */
       /***************************************************************/
       right = CnrQueryColumnPosInDV (hwndCnr,
                         NEXTVISIBLEFI(pccinfon->pFieldInfoLast),
                                       pFieldInfo,
                                       FALSE,
                                       FALSE);

       /***************************************************************/
       /* We must take into account the distance between the columns. */
       /* Therefore we add half the distance between the columns to   */
       /* this value.                                                 */
       /***************************************************************/
       right += (CxBetweenCols / 2);
    }

    /******************************************************************/
    /* If the right side of the column is not in the viewport, then   */
    /* go to the next column (FieldInfo) and check it.                */
    /******************************************************************/
    while ((pFieldInfo) && (right < pccinfon->RightWndxOrg))
    {
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

       right = CnrQueryColumnPosInDV (hwndCnr,
                         NEXTVISIBLEFI(pccinfon->pFieldInfoLast),
                                       pFieldInfo,
                                       FALSE,
                                       FALSE);

       right += (CxBetweenCols / 2);
    }

    /******************************************************************/
    /* If we found the first in view column, set it.  Otherwise, set  */
    /* the first in view to be the first column in the right window.  */
    /******************************************************************/
    if (pFieldInfo)
    {
       pccinfon->pFIFirstColRightInDV = pFieldInfo;
    }
    else
    {
       pccinfon->pFIFirstColRightInDV =
                                NEXTVISIBLEFI(pccinfon->pFieldInfoLast);
    }
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrSetFirstRowInView (hwndCnr, preccCurrent, FromAbove)

HANDLE        hwndCnr;
LPRECORDCOREN preccCurrent;
BOOL          FromAbove;

{
 LPCNRINFON    pccinfon;
 LPCNRINFO     pccinfo;
 long          bottom;
 long          top;
 RECT          ViewPort;
 long          CyBetweenRows;
 LPRECORDCOREN preccNext;
 BOOL          Found = FALSE;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* Set the distance between the rows.  If the application has        */
 /* specified it, use it...otherwise use our constant.                */
 /*********************************************************************/
 if (pccinfo->cyLineSpacing >= 0)
 {
    CyBetweenRows = pccinfo->cyLineSpacing;
 }
 else
 {
    CyBetweenRows = MINCY_BETWEENITEMS;
 }

 /*********************************************************************/
 /* Get the View port of the left window.  We could use either window */
 /* we just want to know what its top is.                            */
 /*********************************************************************/
 PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                     &ViewPort);

 /*********************************************************************/
 /* If no current first-in-view exists, set it to the first record in */
 /* the list.                                                         */
 /*********************************************************************/
 if (!preccCurrent)
 {
    preccCurrent = pccinfon->preccItem;
 }

 /*********************************************************************/
 /* If the window moved down, we search from above the current record */
 /* downward.                                                         */
 /*********************************************************************/
 if (FromAbove)
 {
    while ((preccCurrent) && (!Found))
    {
       bottom = CnrQueryRowPosInDV (preccCurrent, PT_BOTTOM) +
                 (CyBetweenRows / 2);

       if (!(ISRECORDVISIBLE(preccCurrent)) ||
           (bottom < (ViewPort.top + pccinfon->ptlNonIconOrg.y)))
       {
         preccCurrent = preccCurrent->preccNextItem;
       }
       else
       {
         Found = TRUE;
       }
    }

    /******************************************************************/
    /* If we found one, set this record to be the first in view.      */
    /* If we didn't, set the first record in view to be the first     */
    /* record in the list.                                            */
    /******************************************************************/
    if (preccCurrent)
    {
       pccinfon->preccFirstRowInDV = preccCurrent;
    }
    else
    {
       pccinfon->preccFirstRowInDV = pccinfon->preccItem;
    }
 }
 else
 /*********************************************************************/
 /* We are searching from below.  This means that the window was      */
 /* scrolled upwards, and we search backwards in the list of records. */
 /*********************************************************************/
 {
    /******************************************************************/
    /* If preccCurrent is NULL, we set it to the last record and      */
    /* search backwards from there.                                   */
    /******************************************************************/
    if (!preccCurrent)
    {
       preccCurrent = pccinfon->preccLastItem;
    }

    preccNext = pccinfon->preccFirstRowInDV;

    /* ****************************************************************/
    /* Find the first record (row) whose top is above the viewport.   */
    /* This could be the first in view record (row).                  */
    /* ****************************************************************/
    while ((preccCurrent) && (!Found))
    {
       top = CnrQueryRowPosInDV (preccCurrent, PT_TOP) -
              (CyBetweenRows / 2);

       if (!(ISRECORDVISIBLE(preccCurrent)) ||
           (top > (ViewPort.top + pccinfon->ptlNonIconOrg.y)))
       {
          if (ISRECORDVISIBLE(preccCurrent))
          {
            preccNext = preccCurrent;
          }

          preccCurrent = preccCurrent->preccPrevItem;
       }
       else
       {
          Found = TRUE;
       }
    }

    /******************************************************************/
    /* Now we have the record whose row's top is above the View Port. */
    /* We now have to check to see if its bottom is partially in      */
    /* view.  If it is, this is the first item in view.  If its       */
    /* bottom is not in view, then the first in view record is the    */
    /* next one in the list.                                          */
    /******************************************************************/
    if (preccCurrent)
    {
       bottom = CnrQueryRowPosInDV (preccCurrent, PT_BOTTOM) +
                 (CyBetweenRows / 2);

       if (bottom > (ViewPort.top + pccinfon->ptlNonIconOrg.y))
       {
          pccinfon->preccFirstRowInDV = preccCurrent;
       }
       else
       {
          pccinfon->preccFirstRowInDV = preccCurrent->preccNextItem;
       }
    }
    else
    /******************************************************************/
    /* We searched the whole list and didn't find a first in view     */
    /* record (row).  So, we set it to be the first record in the     */
    /* list.                                                          */
    /******************************************************************/
    {
       pccinfon->preccFirstRowInDV = pccinfon->preccItem;
    }
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrSizeColumnTitleWindows (hwndCnr, right, bottom)

HANDLE   hwndCnr;
int      right;
int      bottom;
{
 LPCNRINFON pccinfon;      /* Pointer to Internal control information */
 LPCNRINFO  pccinfo;
 RECT       TitleRect;
 int        cxSizeBorder;
 WORD       cyTitleWnd;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* If Details View Titles are to be displayed, and there are some    */
 /* FieldInfos inserted, we set the size and position of the column   */
 /* title windows.                                                    */
 /*********************************************************************/
 if ((pccinfo->flWindowAttr & CA_DETAILSVIEWTITLES) &&
     (pccinfon->pFirstVisibleFI))
 {
    CnrQueryTitleRect (hwndCnr, &TitleRect, NULL);
    cxSizeBorder = (int)GetSystemMetrics (SM_CXFRAME);

    cyTitleWnd   = pccinfon->MaxColTitleHeight +
                   (WORD)(2 * (pccinfon->lAveCharWidth));

    /******************************************************************/
    /* If Details View has a splitbar, ie. it has 2 windows, set      */
    /* their Column Title windows.                                    */
    /******************************************************************/
    if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
    {
       /***************************************************************/
       /* Set the size of the Left Title Window.                      */
       /***************************************************************/
       MoveWindow (GetDlgItem(hwndCnr, CID_LEFTCOLTITLEWND),
                   0,
                   TitleRect.bottom,
                   pccinfo->xVertSplitbar,
                   min(bottom, (int)cyTitleWnd),
                   TRUE);

       /***************************************************************/
       /* Set the size of the Right Title Window.                     */
       /***************************************************************/
       MoveWindow (GetDlgItem(hwndCnr, CID_RIGHTCOLTITLEWND),
                   (pccinfo->xVertSplitbar + cxSizeBorder),
                   TitleRect.bottom,
                   (right - pccinfo->xVertSplitbar - cxSizeBorder),
                   min(bottom, (int)cyTitleWnd), TRUE );
    }
    else
    /******************************************************************/
    /* Details View is unsplit which means there is no right window.  */
    /* Therefore we only set the size of the left title window.  We   */
    /* set to the width of the container window.  We also take away   */
    /* the right title window in case it was there.                   */
    /******************************************************************/
    {
       MoveWindow (GetDlgItem (hwndCnr, CID_LEFTCOLTITLEWND),
                   0, TitleRect.bottom, right,
                   min(bottom, (int)cyTitleWnd), TRUE);
#ifdef IC_WU
       SetWindowPos( GetDlgItem(hwndCnr, CID_RIGHTCOLTITLEWND),	
		     NULL, 0, 0, 0, 0, SWP_NOZORDER );		/*RKK1*/
#else
       MoveWindow (GetDlgItem (hwndCnr, CID_RIGHTCOLTITLEWND),
                   0, 0, 0, 0, TRUE);
#endif
    }
 }
 else
 /*********************************************************************/
 /* Details View titles are not to be displayed, so we take away the  */
 /* column title windows.                                             */
 /*********************************************************************/
 {
#ifdef IC_WU
    SetWindowPos (GetDlgItem (hwndCnr, CID_LEFTCOLTITLEWND),
		NULL, 0, 0, 0, 0, SWP_NOZORDER );		/*RKK1*/

    SetWindowPos (GetDlgItem (hwndCnr, CID_RIGHTCOLTITLEWND),
		NULL, 0, 0, 0, 0, SWP_NOZORDER );		/*RKK1*/
#else
    MoveWindow (GetDlgItem (hwndCnr, CID_LEFTCOLTITLEWND),
                0, 0, 0, 0, TRUE);

    MoveWindow (GetDlgItem (hwndCnr, CID_RIGHTCOLTITLEWND),
                0, 0, 0, 0, TRUE);
#endif
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrVerifyFieldInfoPtrs                            */
/*                                                                    */
/*  Descriptive Name:  Verify FieldInfo Pointers                      */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function determines if a particular FieldInfo structure is   */
/*  currently inserted in the linked list of FieldInfo structures or  */
/*  not.  This function is called by the FreeFieldInfo code because   */
/*  memory for a FieldInfo is not freed if it is still in the List.   */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr    - Handle of the Container Window.              */
/*          pFieldInfo - Pointer to a FieldInfo structure which is    */
/*                       to be verified.                              */
/*                                                                    */
/*  Output:  TRUE  - The FieldInfo structure is currently in the      */
/*                   linked list of FieldInfo's.                      */
/*           FALSE - The FieldInfo structure is not in the list of    */
/*                   FieldInfo's and its memory can be freed.         */
/*                                                                    */
/*  Exit-Normal:  TRUE or FALSE is returned.                          */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrVerifyFieldInfoPtrs(				/*RKK1*/
 HANDLE      hwndCnr,    	/* Handle of the Container window.    */
 LPFIELDINFO pFieldInfo, 	/* Pointer to a FieldInfo structure   */
                                /* which is to be verified.           */
 WORD        NumFieldInfo  )					/*RKK1*/
{

 LPCNRINFON  pccinfon;     /* Pointer to Internal control info.       */
 LPFIELDINFO pFI;          /* Pointer to a FieldInfo structure.       */
 BOOL        Found = FALSE;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 if ((!pFieldInfo) || (NumFieldInfo < 1))
 {
    return (FALSE);
 }

 while ((pFieldInfo) && (NumFieldInfo > 0))
 {
    pFI = pccinfon->pFirstFI;

    while ((pFI) && (!Found))
    {
       if (pFieldInfo == pFI)
       {
          Found = TRUE;
       }

       pFI = NEXTFI(pFI);
    }

    if (Found)
    {
       Found = FALSE;
       NumFieldInfo--;
       pFieldInfo = NEXTFI(pFieldInfo);
    }
    else
    {
       return (FALSE);
    }
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrProcessTitleData (hwndCnr, hps, pFieldInfo, xColumnPos,
                               InvalidRect, LeftWindow, FirstColLeft)

HANDLE       hwndCnr;
HDC          hps;
LPFIELDINFO  pFieldInfo;
long         xColumnPos;
RECT         InvalidRect;
BOOL         LeftWindow;
BOOL         FirstColLeft;

{
 LPCNRINFON      pccinfon;
 LPCNRINFO       pccinfo;
 SIZEL           ItemSize;
 POINT           IconSize;
 RECT            ItemRect;
 LPICONRESIZED   pIrs;
// LPVOID          pSatPtr;
 HBITMAP         hbmdraw;
 HANDLE          hwndOwner;
 UINT            wWinID;                                                /*DAB1*/
 DRAWITEMSTRUCT  DrawItem;
 RECT            aRectl[1];
 HRGN            hRegion = NULL;
 BOOL            OwnerDraw = FALSE;
 HANDLE          hwndTitleWnd;
 DWORD           TextPosition;
// ERRORID       ErrorID;
 CNRDRAWITEMINFO CnrDrawItemInfo;
 int             nSavedDC;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 CnrCalcIconSize (hwndCnr, &IconSize);

 if (LeftWindow)
 {
    hwndTitleWnd = GetDlgItem (hwndCnr, CID_LEFTCOLTITLEWND);
 }
 else
 {
    hwndTitleWnd = GetDlgItem (hwndCnr, CID_RIGHTCOLTITLEWND);
 }

 /*********************************************************************/
 /* Set rcItem as the rectangle of the title.                         */
 /*********************************************************************/
 CnrCalcCellRectForDV (hwndCnr, NULL, pFieldInfo,
                       LeftWindow, TRUE, &(DrawItem.rcItem));

 if (pFieldInfo->flTitle & CFA_BITMAPORICON)
 {
    ItemSize.cx = IconSize.x;                                           /*DAB1*/
    ItemSize.cy = IconSize.y;                                           /*DAB1*/
 }
 else
 {
   IcnCalcTextSize (hps, IconSize, pccinfon,
                    (LPSTR)pFieldInfo->pTitleData, IS_SIDETEXT,
                    &ItemSize);
 }

 CnrCalcRectForDVTitles (hwndCnr, pFieldInfo, ItemSize,
                         FirstColLeft, DrawItem.rcItem, &ItemRect);

 /*********************************************************************/
 /* If the item is in view, then we process it.                       */
 /*********************************************************************/
 if (CnrInViewRect (hwndCnr, ItemRect, InvalidRect))
 {
   /*******************************************************************/
   /* First check to see if this column is OwnerDraw.  If it is, let  */
   /* the application have at it.  If they don't do anything, then I  */
   /* do my thing.                                                    */
   /*******************************************************************/
   if ((pFieldInfo->flData & CFA_OWNER) ||
       (pccinfo->flWindowAttr & CA_OWNERDRAW) ||
       ((pFieldInfo->flTitle & CFA_BITMAPORICON) &&
       (!pFieldInfo->pTitleData)))
   {
      /****************************************************************/
      /* Save the current DC attributes.                              */
      /****************************************************************/
      nSavedDC = SaveDC (hps);
      hwndOwner = PARENTOF (hwndCnr);
      wWinID   = getWindowId(hwndCnr);                       /*DAB1*/

      /****************************************************************/
      /* Set DrawItem data by calling CnrSetDrawItem.                 */
      /****************************************************************/
      CnrSetDrawItem (hwndTitleWnd, hps, &DrawItem, 0);

      /****************************************************************/
      /* Set container control ID.                                    */
      /****************************************************************/
      DrawItem.CtlID = wWinID;

      /****************************************************************/
      /* Set up the CnrDrawItemInfo structure, and pass it back in    */
      /* the hItem field.                                             */
      /****************************************************************/
      CnrDrawItemInfo.pRecord = NULL;
      CnrDrawItemInfo.pFieldInfo = pFieldInfo;
      DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

      /****************************************************************/
      /* Send the application this message and let them draw anything */
      /* they want in the rectangle I provided them.                  */
      /****************************************************************/
      OwnerDraw = SENDMESSAGE( hwndOwner, WM_DRAWITEM, (WPARAM)wWinID,  /*DAB2*/
                               (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));

      /****************************************************************/
      /* Restore the device context.                                  */
      /****************************************************************/
      if (nSavedDC)
        RestoreDC (hps, nSavedDC);
   }

   /*******************************************************************/
   /* If the application did not specify OwnerDraw for this column or */
   /* they did, but didn't draw anything, I do the drawing.           */
   /*******************************************************************/
   if (!OwnerDraw)
   {
       if (pFieldInfo->flTitle & CFA_BITMAPORICON)
       {
          /************************************************************/
          /* For a Bitmap or an Icon, we need to create a clipping    */
          /* region.  This is so that if the Bitmap or Icon is too    */
          /* wide to fit into the width of the column, it will be     */
          /* clipped so it won't over write the next columns data.    */
          /************************************************************/
          DrawItem.rcItem.right +=
                           (3 * pccinfon->lAveCharWidth) / 2;
          DrawItem.rcItem.top--;                                //FLIP ++ to --
          aRectl[0] = DrawItem.rcItem;

          /************************************************************/
          /* Save the current DC attributes.                          */
          /************************************************************/
          nSavedDC = SaveDC (hps);

          if (hRegion = CreateRectRgn (aRectl[0].left, aRectl[0].top,
                                   aRectl[0].right, aRectl[0].bottom))
          {
            SelectClipRgn (hps, hRegion);
          }

          if ((pccinfo->flWindowAttr & CA_DRAWICON) ||
              !(pccinfo->flWindowAttr & CA_DRAWBITMAP))
          {
             /*********************************************************/
             /* If the Icon begin used was not resized, ie. the app is*/
             /* using the system default size, we create a dummy Irs  */
             /* structure to be passed into IcnDrawIcon.              */
             /*********************************************************/
             if ((pccinfo->slBitmapOrIcon.x == 0) &&
                 (pccinfo->slBitmapOrIcon.y == 0))
             {
//              pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);

//              if (!(pIrs = StmAllocateStorage (pSatPtr,
                if (!(pIrs = (LPICONRESIZED)GlobalAlloc( GPTR,          /*DAB1*/
                                           (long)sizeof(ICONRESIZED))))
                {
                   if (nSavedDC)
                     RestoreDC (hps, nSavedDC);

                   return (FALSE);
                }

                /******************************************************/
                /* Since we are not using the resized Icon, we create */
                /* a dummy Irs structure to pass to IcnDrawIcon.      */
                /******************************************************/
                pIrs->pirsNext   = NULL;
                pIrs->hptrOrigin = (HICON)(DWORD)pFieldInfo->pTitleData;
                pIrs->hbmPointer = NULL;
                pIrs->hbmColor   = NULL;
                pIrs->cUsage     = 1;
                pIrs->slIcon     = IconSize;

                hbmdraw = NULL;
             }
             else
             /*********************************************************/
             /* If we get to here, we need to get the resized Icon to */
             /* draw.                                                 */
             /*********************************************************/
             {
                if (!(pIrs = FindPirsForDV (hwndCnr,
                                 (HICON)(DWORD)pFieldInfo->pTitleData)))
                {
//                 ErrorID = GetLastError();
//
//                 if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//                 {
//                    SETERROR(ErrorID);
//                    return (FALSE);
//                 }
//
//                 SETERROR(ErrorID);
                }

                hbmdraw = NULL;
             }
          }
          else
          /************************************************************/
          /* The app didn't specify using an Icon, but instead is     */
          /* using a bitmap.                                          */
          /************************************************************/
          {
             pIrs = NULL;
             hbmdraw = (HBITMAP)(DWORD)pFieldInfo->pTitleData;
          }

          IcnDrawIcon( hps, pIrs, hbmdraw, 0L, ItemRect, pccinfon);

          /************************************************************/
          /* Free the storage allocated for the Irs Structure.        */
          /************************************************************/
          if ((pccinfo->slBitmapOrIcon.x == 0) &&
              (pccinfo->slBitmapOrIcon.y == 0))
          {
//           StmFreeStorage (pSatPtr, (LPVOID)pIrs);
             GlobalFree( (LPVOID)pIrs );                                /*DAB1*/
          }

          /************************************************************/
          /* Restore the device context.                              */
          /************************************************************/
          if (nSavedDC)
            RestoreDC (hps, nSavedDC);

          /************************************************************/
          /* Destroy the current clipping region                      */
          /************************************************************/
          if (hRegion)
            DeleteObject (hRegion);
       }
       else
       {
          if (pFieldInfo->flTitle & CFA_RIGHT)
          {
             TextPosition = IS_SIDERIGHTTEXT;
          }
          else if (pFieldInfo->flTitle & CFA_CENTER)
          {
             TextPosition = IS_BENEATHTEXT;
          }
          else
          {
             TextPosition = IS_SIDETEXT;
          }

          IcnDrawText (hps, (LPSTR)pFieldInfo->pTitleData, ItemRect, 0L,
                       TextPosition, pccinfon);
       }
    }
 }

 /******************************************************************/
 /* Draw the horizontal and vertical separators if they were       */
 /* specified.                                                     */
 /******************************************************************/
 if (pFieldInfo->flData & CFA_HORZSEPARATOR)
 {
    CnrDrawHorzSep (hwndCnr,
                    hps,
                    pFieldInfo,
                    xColumnPos,
                    LeftWindow);
 }

 if (pFieldInfo->flData & CFA_SEPARATOR)
 {
    CnrDrawVertSep (hwndCnr, hps, LeftWindow, TRUE);
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrProcessColumnData (hwndCnr, hps, pRecord, pFieldInfo,
                                InvalidRect, LeftWindow, FirstColLeft)

HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN pRecord;
LPFIELDINFO   pFieldInfo;
RECT          InvalidRect;
BOOL          LeftWindow;
BOOL          FirstColLeft;

{
 LPCNRINFON      pccinfon;
 LPCNRINFO       pccinfo;
 SIZEL           ItemSize;
 POINT           IconSize;
 RECT            ItemRect;
 RECT            SelectionRect;
 RECT            CellRect;
 LPICONRESIZED   pIrs;
// LPVOID          pSatPtr;
 HBITMAP         hbmDraw;
 HANDLE          hwndViewPort;
 HANDLE          hwndOwner;
 UINT            wWinID;                                                /*DAB1*/
 DRAWITEMSTRUCT  DrawItem;
 LPSTR           String;
 RECT            aRectl[1];
 HRGN            hRegion = NULL;
 DWORD           TextPosition;
// ERRORID       ErrorID;
 LPVOID          Image;
 BOOL            OwnerDraw = FALSE;
 BOOL            DrawEmph = FALSE;
 CNRDRAWITEMINFO CnrDrawItemInfo;
 int             nSavedDC;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 if (!(pccinfo->pFieldInfoObject))
 {
    pccinfo->pFieldInfoObject = pccinfon->pFirstVisibleFI;
 }

 if (LeftWindow)
 {
    hwndViewPort = GetDlgItem (hwndCnr, CID_LEFTDVWND);
 }
 else
 {
    hwndViewPort = GetDlgItem (hwndCnr, CID_RIGHTDVWND);
 }

 /*********************************************************************/
 /* First calculate the selection rectangle for this record (rcItem)  */
 /*********************************************************************/
 CnrCalcSelectionRect (hwndCnr, pRecord, pFieldInfo,
                       LeftWindow, &(DrawItem.rcItem));

 SelectionRect = DrawItem.rcItem;

 /*********************************************************************/
 /* Check to see if the selection rectangle is contained in the       */
 /* boundaries of the rectangle that was invalidated.  If it was,     */
 /* then process it.                                                  */
 /*********************************************************************/
 if (CnrInViewRect (hwndCnr, SelectionRect, InvalidRect))
 {
   /*******************************************************************/
   /* First check to see if this column is OwnerDraw.  If it is, let  */
   /* the application have at it.  If they don't do anything, then I  */
   /* do my thing.                                                    */
   /*******************************************************************/
   if ((pFieldInfo->flData & CFA_OWNER) ||
       (pccinfo->flWindowAttr & CA_OWNERDRAW) ||
       ((pFieldInfo->flData & CFA_BITMAPORICON) &&
        (!(Image = CnrGetColumnData (hwndCnr,
                                     pRecord->preccX,
                                     pFieldInfo)))))
   {
     /*****************************************************************/
     /* Save the current DC attributes.                               */
     /*****************************************************************/
     nSavedDC = SaveDC (hps);
     hwndOwner = PARENTOF (hwndCnr);
     wWinID   = getWindowId(hwndCnr);                        /*DAB1*/

     /*****************************************************************/
     /* Set DrawItem data by calling CnrSetDrawItem                   */
     /*****************************************************************/
     CnrSetDrawItem (hwndViewPort, hps, &DrawItem, 0);

     /*****************************************************************/
     /* Set container control ID.                                     */
     /*****************************************************************/
     DrawItem.CtlID = wWinID;

     /*****************************************************************/
     /* Set up the CnrDrawItemInfo structure, and pass it back in the */
     /* hItem field.                                                  */
     /*****************************************************************/
     COPYNDATATOX(pRecord);
     CnrDrawItemInfo.pRecord = pRecord->preccX;
     CnrDrawItemInfo.pFieldInfo = pFieldInfo;
     DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

     /*****************************************************************/
     /* Send the application this message and let them draw anything  */
     /* they want in the rectangle I provided them.                   */
     /*****************************************************************/
     OwnerDraw = SENDMESSAGE( hwndOwner, WM_DRAWITEM, (WPARAM)wWinID,   /*DAB2*/
                              (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));

     /*****************************************************************/
     /* Restore the device context.                                   */
     /*****************************************************************/
     if (nSavedDC)
       RestoreDC (hps, nSavedDC);
   }

   /*********************************************************************/
   /* If the application did not specify OwnerDraw for this column or   */
   /* they did, but didn't draw anything, I do the drawing.             */
   /*********************************************************************/
   if (!OwnerDraw)
   {
       /******************************************************************/
       /* Check to see if the selection rectangle is contained in the    */
       /* boundaries of the rectangle that was invalidated.  If it was,  */
       /* we get the data for the column, calculate the item rectangle,  */
       /* and draw it.                                                   */
       /******************************************************************/
        CnrCalcIconSize (hwndCnr, &IconSize);

        if (pFieldInfo->flData & CFA_BITMAPORICON)
        {
           ItemSize.cx = IconSize.x;                                    /*DAB1*/
           ItemSize.cy = IconSize.y;                                    /*DAB1*/
           Image = CnrGetColumnData (hwndCnr,
                                     pRecord->preccX,
                                     pFieldInfo);
        }
        else
        {
           if (pFieldInfo->flData & CFA_STRING)
           {
              String = CnrGetColumnData (hwndCnr,
                                         pRecord->preccX,
                                         pFieldInfo);
           }
           else
           {
              if (!(String = CnrGetColumnData (hwndCnr,
                                               pRecord->preccX,
                                               pFieldInfo)))
              {
//              ErrorID = GetLastError();
//
//              if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//              {
//                 SETERROR(ErrorID);
//                 return (FALSE);
//              }
//
//              SETERROR(ErrorID);
              }
           }

           IcnCalcTextSize (hps, IconSize, pccinfon,
                            String, IS_SIDETEXT, &ItemSize);
        }

        /**************************************************************/
        /* Get the Cell rectangle for this column cell.  This is the  */
        /* drawable area that we can draw the data in.                */
        /**************************************************************/
        CnrCalcCellRectForDV (hwndCnr, pRecord, pFieldInfo, LeftWindow,
                              FALSE, &CellRect);

        /**************************************************************/
        /* Now we have the item size, so we can calculate the         */
        /* Rectangle that the item will occupy.  This function calls  */
        /* another function to do any data justification.             */
        /**************************************************************/
        CnrCalcRectForDV (hwndCnr, pRecord, pFieldInfo, ItemSize,
                          FirstColLeft, CellRect, &ItemRect);

        /**************************************************************/
        /* Now we have the rectangle for the item that is to be drawn.*/
        /* So we draw it.                                             */
        /**************************************************************/
        if (pFieldInfo->flData & CFA_BITMAPORICON)
        {
          /************************************************************/
          /* For a Bitmap or an Icon, we need to create a clipping    */
          /* region.  This is so that if the Bitmap or Icon is too    */
          /* wide to fit into the width of the column, it will be     */
          /* clipped so it won't over write the next columns data.    */
          /************************************************************/
          aRectl[0] = SelectionRect;

          /************************************************************/
          /* Save the current DC attributes.                          */
          /************************************************************/
          nSavedDC = SaveDC (hps);

          if (hRegion = CreateRectRgn (aRectl[0].left, aRectl[0].top,
                                   aRectl[0].right, aRectl[0].bottom))
          {
            SelectClipRgn (hps, hRegion);
          }

          if ((pccinfo->flWindowAttr & CA_DRAWICON) ||
              !(pccinfo->flWindowAttr & CA_DRAWBITMAP))
          {
             /*********************************************************/
             /* If the Icon begin used was not resized, ie. the app is*/
             /* using the system default size, we create a dummy Irs  */
             /* structure to be passed into IcnDrawIconInDV.          */
             /*********************************************************/
             if ((pccinfo->slBitmapOrIcon.x == 0) &&
                 (pccinfo->slBitmapOrIcon.y == 0))
             {
//              pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
//              if (!(pIrs = StmAllocateStorage (pSatPtr,
                if (!(pIrs = (LPICONRESIZED)GlobalAlloc( GPTR,          /*DAB1*/
                                           (long)sizeof(ICONRESIZED))))
                {
                   SETERROR(PMERR_INSUFFICIENT_MEMORY);
                   if (nSavedDC)
                     RestoreDC (hps, nSavedDC);

                   return (FALSE);
                }

                pIrs->pirsNext = NULL;
                pIrs->hptrOrigin = (HICON)(DWORD)Image;
                pIrs->hbmPointer = NULL;
                pIrs->hbmColor   = NULL;
                pIrs->cUsage = 1;
                pIrs->slIcon = IconSize;

                hbmDraw = NULL;
             }
             else
             /*********************************************************/
             /* If we get to here, we need to get the resized Icon to */
             /* draw.                                                 */
             /*********************************************************/
             {
                if (!(pIrs = FindPirsForDV (hwndCnr, (HICON)(DWORD)Image)))
                {
//                 ErrorID = GetLastError();
//
//                 if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//                 {
//                    SETERROR(ErrorID);
//                    return (FALSE);
//                 }
//
//                 SETERROR(ErrorID);
                }

                hbmDraw = NULL;
             }
          }
          else
          {
             pIrs = NULL;
             hbmDraw = (HBITMAP)(DWORD)Image;
          }

          if (pFieldInfo == pccinfo->pFieldInfoObject)
          {
             DrawEmph = TRUE;
          }

          /************************************************************/
          /* Now draw the icon or bitmap in Details View.             */
          /************************************************************/
          IcnDrawIconInDV (hps, pIrs, hbmDraw, pRecord->flRecordAttr,
                           ItemRect, SelectionRect, DrawEmph,
                           pccinfon);

          /************************************************************/
          /* Free the storage allocated for the Irs Structure.        */
          /************************************************************/
          if ((pccinfo->slBitmapOrIcon.x == 0) &&
              (pccinfo->slBitmapOrIcon.y == 0))
          {
//           StmFreeStorage (pSatPtr, (LPVOID)pIrs);
             GlobalFree( (LPVOID)pIrs );                                /*DAB1*/
          }

          /************************************************************/
          /* Restore the device context.                              */
          /************************************************************/
          if (nSavedDC)
            RestoreDC (hps, nSavedDC);

          /************************************************************/
          /* Destroy the current clipping region                      */
          /************************************************************/
          if (hRegion)
            DeleteObject (hRegion);
        }
        else
        {
          if (pFieldInfo->flData & CFA_RIGHT)
          {
             TextPosition = IS_SIDERIGHTTEXT;
          }
          else if (pFieldInfo->flData & CFA_CENTER)
          {
             TextPosition = IS_BENEATHTEXT;
          }
          else
          {
             TextPosition = IS_SIDETEXT;
          }

          /************************************************************/
          /* Draw the text string in Details View.                    */
          /************************************************************/
          IcnDrawTextInDV (hps, String, ItemRect, SelectionRect,
                           pRecord->flRecordAttr, TextPosition,
                           pccinfon);

          /************************************************************/
          /* Free the storage allocated for data that was not         */
          /* CFA_STRING.                                              */
          /************************************************************/
          if (!(pFieldInfo->flData & CFA_STRING))
          {
//           pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
//           StmFreeStorage (pSatPtr, (LPVOID)String);
             GlobalFree( (LPVOID)String );                              /*DAB1*/
          }
        }
   }
 }
 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrProcessVisibleColumns                          */
/*                                                                    */
/*  Descriptive Name:  Process the visible columns that are inserted  */
/*                     in the container for Details View.             */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will go through the list of FieldInfo's that have   */
/*  been inserted for Details View, and assign their Visible          */
/*  pointers.  Each FieldInfo has a pointer to the next FieldInfo     */
/*  logically in the list, and a pointer to the next Visible          */
/*  FieldInfo.  The only FieldInfo's that are displayed in Detials    */
/*  View are the visible one's.                                       */
/*                                                                    */
/*  Notes:  N/A                                                       */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*  Output:  void                                                     */
/*                                                                    */
/*  Exit-Normal:  N/A                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  The Visible pointers will be assinged for every    */
/*                 inserted FieldInfo.                                */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrProcessVisibleColumns( HANDLE hwndCnr )
{
 LPCNRINFON  pccinfon;    /* Pointer to Internal Container Info.      */
 LPCNRINFO   pccinfo;     /* Pointer to External Container Info.      */
 LPFIELDINFO pFieldInfo;  /* Pointer to a FieldInfo structure.        */
 LPFIELDINFO pTempFI;     /* Pointer to a FieldInfo structure.        */

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 pFieldInfo = pccinfon->pFirstFI;

 /*********************************************************************/
 /* Find the first visible column, and set the global pointer to it.  */
 /*********************************************************************/
 if (pFieldInfo)
 {
    while ((pFieldInfo) && (pFieldInfo->flData & CFA_INVISIBLE))
    {
       if (pFieldInfo == pccinfon->pFIFirstColLeftInDV)
       {
          pccinfon->pFIFirstColLeftInDV = NULL;
       }

       if (pFieldInfo == pccinfon->pFIFirstColRightInDV)
       {
          pccinfon->pFIFirstColRightInDV = NULL;
       }

       /************************************************************/
       /* If the app. is making pFieldInfoLast invisible, and      */
       /* there are not previous visible columns, we make          */
       /* pFieldInfoLast NULL, so everything is in the left        */
       /* window.                                                  */
       /************************************************************/
       if (pFieldInfo == pccinfon->pFieldInfoLast)
       {
          pccinfon->pFieldInfoLast = NULL;
       }

       pFieldInfo = NEXTFI(pFieldInfo);
    }

    pccinfon->pFirstVisibleFI = pFieldInfo;
    pTempFI = pFieldInfo;

    if (pFieldInfo == pccinfo->pFieldInfoLast)
    {
       pccinfon->pFieldInfoLast = pFieldInfo;
    }

    if (pFieldInfo)
    {
       pFieldInfo = NEXTFI(pFieldInfo);
    }

    /******************************************************************/
    /* Walk the rest of the list and assign the visible pointers as   */
    /* we go.                                                         */
    /******************************************************************/
    while (pFieldInfo)
    {
       if (pFieldInfo->flData & CFA_INVISIBLE)
       {
          if (pFieldInfo == pccinfon->pFIFirstColLeftInDV)
          {
             pccinfon->pFIFirstColLeftInDV = NULL;
          }

          if (pFieldInfo == pccinfon->pFIFirstColRightInDV)
          {
             pccinfon->pFIFirstColRightInDV = NULL;
          }

          /************************************************************/
          /* If the app. is making pFieldInfoLast invisible, we need  */
          /* to reassign it to be the previous visible column.        */
          /************************************************************/
          if (pFieldInfo == pccinfon->pFieldInfoLast)
          {
             pccinfon->pFieldInfoLast = pTempFI;
          }

          pFieldInfo = NEXTFI(pFieldInfo);

          /************************************************************/
          /* Loop until we find a visible FieldInfo.                  */
          /************************************************************/
          while ((pFieldInfo) && (pFieldInfo->flData & CFA_INVISIBLE))
          {
             if (pFieldInfo == pccinfon->pFIFirstColLeftInDV)
             {
                pccinfon->pFIFirstColLeftInDV = NULL;
             }

             if (pFieldInfo == pccinfon->pFIFirstColRightInDV)
             {
                pccinfon->pFIFirstColRightInDV = NULL;
             }

             /*********************************************************/
             /* If the app. is making pFieldInfoLast invisible, we    */
             /* need to reassign it to be the previous visible        */
             /* column.                                               */
             /*********************************************************/
             if (pFieldInfo == pccinfon->pFieldInfoLast)
             {
                pccinfon->pFieldInfoLast = pTempFI;
             }

             pFieldInfo = NEXTFI(pFieldInfo);
          }
       }

       /***************************************************************/
       /* Assign the visible pointer.                                 */
       /***************************************************************/
       NEXTVISIBLEFI(pTempFI) = pFieldInfo;
       pTempFI = pFieldInfo;

       if (pFieldInfo == pccinfo->pFieldInfoLast)
       {
          pccinfon->pFieldInfoLast = pFieldInfo;
       }

       if (pFieldInfo)
       {
          pFieldInfo = NEXTFI(pFieldInfo);
       }
    }

    if (pTempFI)
    {
       NEXTVISIBLEFI(pTempFI) = pFieldInfo;
    }
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrInvalidateDetailFieldInfo                      */
/*                                                                    */
/*  Descriptive Name:  Invalidate all of the FieldInfo structures     */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will reposition and repaint both the left and       */
/*  right Details View windows.                                       */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*  Output:  TRUE is returned to the Gate Proc.                       */
/*                                                                    */
/*  Exit-Normal:  TRUE                                                */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrInvalidateDetailFieldInfo( HANDLE hwndCnr )
{
 LPCNRINFON  pccinfon;
 LPCNRINFO   pccinfo;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* Call this function to set the visible pointers for the list       */
 /* of FieldInfo's.                                                   */
 /*********************************************************************/
 CnrProcessVisibleColumns (hwndCnr);

 /*********************************************************************/
 /* Make sure we are in details view before we try to redo            */
 /* everything.  If we are not, just return TRUE.                     */
 /*********************************************************************/
 if (pccinfo->flWindowAttr & CV_DETAIL)
 {
    /******************************************************************/
    /* Set all the flags so we re-do everything.                      */
    /******************************************************************/
    pccinfon->ulFlagSet &= ~( FL_SETCNRTITLE | FL_POSITIONCOLTITLES |
                   FL_SIZECOLTITLES | FL_SIZECHILDREN | FL_CNRPOSITION |
                   FL_NEWVIEWSCROLLSET | FL_SETSCROLL );

#if 0
    PMGetClientRect (hwndCnr, &CnrRect);

    /******************************************************************/
    /* If the window is unsplit, or the splitbar is not visible, ie.  */
    /* to the right of the window, we need to explicitly send the     */
    /* WM_PAINT message to the control proc.                          */
    /******************************************************************/
    if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
        ((long)pccinfo->xVertSplitbar > CnrRect.right))
    {
       CnrDetailsVCtlProc (hwndCnr, WM_PAINT, NULL, NULL);
    }

    InvalidateRect (hwndCnr, NULL, TRUE);
#endif

    InvalidateChildren (hwndCnr, FALSE, FALSE);
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrInViewRect( HANDLE hwndCnr, RECT ItemRect, RECT ViewRect )
{
 LPCNRINFON pccinfon;       /* Pointer to Internal Control Info.      */
 BOOL       VertInView = FALSE;
 BOOL       HorzInView = FALSE;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* If any part of the item falls within the boundaries of the        */
 /* invalid rectangle, return TRUE and draw it.                       */
 /*********************************************************************/
 if ((ItemRect.top < ViewRect.bottom) &&
     (ItemRect.left < ViewRect.right))
 {
    return (TRUE);
 }
 else
 {
    /******************************************************************/
    /* Determine if the item is below and/or to the right of the      */
    /* invalid rectangle.  Then set the appropriate flag.  These      */
    /* flags optimize the painting process by telling us to stop      */
    /* processing items to be painted because everything else is out  */
    /* of view.                                                       */
    /******************************************************************/
    if (ItemRect.top > ViewRect.bottom)
    {
       pccinfon->ulFlagSet |= FL_BELOWINVRECT;
    }

    if (ItemRect.left > ViewRect.right)
    {
       pccinfon->ulFlagSet |= FL_RIGHTOFINVRECT;
    }

    return (FALSE);
  }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrPaintSplitbar                                  */
/*                                                                    */
/*  Descriptive Name:  Paint the Splitbar in Details View.            */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function paints a vertical Splitbar in Details View to       */
/*  separate the two split windows.                                   */
/*                                                                    */
/*  Notes:  The Splitbar spearates the left and right split windows.  */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*          hps     - Device context for painting                     */
/*                                                                    */
/*  Output:  VOID                                                     */
/*                                                                    */
/*  Exit-Normal: VOID                                                 */
/*                                                                    */
/*  Exit-Error:  VOID                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrPaintSplitbar( HANDLE hwndCnr, HDC hps )
{
 LPCNRINFON pccinfon;       /* Pointer to Internal control info.      */
 LPCNRINFO  pccinfo;
 RECT       rcl;            /* Rectangle of the Container window.     */
 RECT       TitleWinRect;   /* Rectangle of the Title window.         */
 RECT       rctlHScroll;    /* Rectangle of the horz scroll window.   */
 int        cxBorder;       /* Width of a window border.              */
 int        cxSizeBorder;   /* Width of a window sizing border.       */
 HBRUSH     hBrush;

  pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Get Rectangle of Title Window and Column Title window if it      */
  /* exists.                                                          */
  /********************************************************************/
  CnrQuerySizeRect(hwndCnr, NULL, &TitleWinRect, NULL, NULL,
                   &rctlHScroll) ;
  PMGetClientRect (hwndCnr, &rcl);

  /********************************************************************/
  /* Get the width of a window border, and the width of a window size */
  /* border.  These widths will be used to make up the splitbar.  The */
  /* split bar is painted in 3 steps.  The left and right sides are   */
  /* the width of a window border, with the center being the width of */
  /* a window sizing border, minus the width of 2 window borders.     */
  /* A rough picture may help:                                        */
  /*                                                                  */
  /*           Width of a size border                                 */
  /*           minus the width of 2                                   */
  /*           window borders                                         */
  /*                   |                                              */
  /*   Width of a      |      Width of a                              */
  /*   Window     \    |    / Window                                  */
  /*   Border      ||     ||  Border                                  */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /********************************************************************/
  cxBorder = (int)GetSystemMetrics (SM_CXBORDER);
  cxSizeBorder = (int)GetSystemMetrics (SM_CXFRAME);

  /********************************************************************/
  /* The Top of the Details View windows, and thus the Splitbar, is   */
  /* always below the Title Window if it exists.                      */
  /********************************************************************/
  rcl.top += TitleWinRect.bottom;

  /********************************************************************/
  /* Paint the left edge of the splitbar                              */
  /********************************************************************/
  rcl.left  = pccinfo->xVertSplitbar;
  rcl.right = rcl.left + cxBorder;

  if (hBrush = CreateSolidBrush (
                         pccinfon->ulColorPPSupport[CMA_SEPARATORCOLOR]))
  {
    PMFillRect (hps, &rcl, hBrush);
    DeleteObject(hBrush);
  }

  /********************************************************************/
  /* Paint the middle of the splitbar                                 */
  /********************************************************************/
  rcl.left  = rcl.right;
  rcl.right += cxSizeBorder - (2 * cxBorder);
  if (hBrush = CreateSolidBrush (
                     pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
  {
    PMFillRect (hps, &rcl, hBrush);
    DeleteObject(hBrush);
  }

  /********************************************************************/
  /* Paint the right edge of the splitbar                             */
  /********************************************************************/
  rcl.left  = rcl.right;
  rcl.right += cxBorder;
  if (hBrush = CreateSolidBrush (
                          pccinfon->ulColorPPSupport[CMA_SEPARATORCOLOR]))
  {
    PMFillRect (hps, &rcl, hBrush);
    DeleteObject(hBrush);
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrPositionColumnTitles( HANDLE hwndCnr, HDC hps )
{
 LPCNRINFON  pccinfon;
 POINT       IconSize;
 SIZEL       TextSize;
 LPFIELDINFO pFieldInfo;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Get the width and height of an icon.                              */
 /*********************************************************************/
 CnrCalcIconSize (hwndCnr, &IconSize);

 pFieldInfo = pccinfon->pFirstVisibleFI;

 if (pFieldInfo)
 {
    if (pFieldInfo->flTitle & CFA_BITMAPORICON)
    {
       pccinfon->slFirstTitleSize.cx = IconSize.x;                      /*DAB1*/
       pccinfon->slFirstTitleSize.cy = IconSize.y;                      /*DAB1*/
       pccinfon->MaxColTitleHeight = (WORD)IconSize.y;
       PFINFROMX(pFieldInfo)->usLongestLine = (WORD)IconSize.x;
    }
    else
    /******************************************************************/
    /* The Column Title is a text string.                             */
    /******************************************************************/
    {
       IcnCalcTextSize (hps, IconSize, pccinfon,
                        (LPSTR)pFieldInfo->pTitleData,
                        IS_SIDETEXT, &TextSize);

       pccinfon->slFirstTitleSize = TextSize;
       pccinfon->MaxColTitleHeight = (USHORT)TextSize.cy;               /*DAB1*/
       PFINFROMX(pFieldInfo)->usLongestLine = (USHORT)TextSize.cx;      /*DAB1*/
    }

    /******************************************************************/
    /* Go through the rest of the column titles, and compute the      */
    /* maximum height for an item, and update the longest line field. */
    /******************************************************************/
    pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

    while (pFieldInfo)
    {
       if (pFieldInfo->flTitle & CFA_BITMAPORICON)
       {
          if ((WORD)IconSize.y > pccinfon->MaxColTitleHeight)
          {
             pccinfon->MaxColTitleHeight = (WORD)IconSize.y;
          }

          PFINFROMX(pFieldInfo)->usLongestLine = (WORD)IconSize.x;
       }
       else
       /***************************************************************/
       /* The Column Title is a text string.                          */
       /***************************************************************/
       {
          IcnCalcTextSize (hps, IconSize, pccinfon,
                           (LPSTR)pFieldInfo->pTitleData,
                           IS_SIDETEXT, &TextSize);

          if ((WORD)TextSize.cy > pccinfon->MaxColTitleHeight)          /*DAB1*/
          {
             pccinfon->MaxColTitleHeight = (WORD)TextSize.cy;           /*DAB1*/
          }

          PFINFROMX(pFieldInfo)->usLongestLine = (WORD)TextSize.cx;     /*DAB1*/
       }

       /***************************************************************/
       /* Go to the next visible FieldInfo to process it's title data.*/
       /***************************************************************/
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
    }

    /******************************************************************/
    /* Now we have the height of the tallest column title.  From this */
    /* we set the position of the first Column Title item. Then we    */
    /* can draw the other column titles relative to this using the    */
    /* tallest column title, and the width and height of the first    */
    /* column title.                                                  */
    /******************************************************************/
    pccinfon->FirstTitlePos.x = pccinfon->lAveCharWidth;
    pccinfon->FirstTitlePos.y = pccinfon->MaxColTitleHeight +
                                pccinfon->lAveCharWidth -
                               (pccinfon->MaxColTitleHeight +           /*DAB1*/
                                pccinfon->slFirstTitleSize.cy) / 2;     /*DAB1*/
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrPositionDetailsView( HANDLE hwnd, HDC hps,
                                 LPRECORDCOREN LastValidRec )
{
 HANDLE        hwndCnr;        /* Handle of the Container Window      */
 LPCNRINFON    pccinfon;
 LPCNRINFO     pccinfo;
 LPFIELDINFO   pFieldInfo;     /* Pointer to FieldInfo structures.    */
 LPRECORDCOREN pRecord;
 LPSTR         Text;
 POINT         IconSize;       /* Width and Height of an Icon.        */
 SIZEL         TextSize;       /* Width and Height of Text.           */
 int           CyBetweenRows;  /* y distance between 2 items.         */
 int           Prevtop;        /* Top of previous item.               */
 int           Prevheight;     /* Top of previous item.               */
 RECT          Rect;
 RECT          CnrTitle;
 RECT          ColTitleRect;
// LPVOID        pSatPtr;
// ERRORID       ErrorID;
 BOOL          PositionAll = TRUE;

 hwndCnr  = PARENTOF (hwnd);
 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /******************************************************************/
 /* Set the distance between the rows.  If the application has     */
 /* specified it, use it...otherwise use our constant.             */
 /******************************************************************/
 if (pccinfo->cyLineSpacing >= 0)
 {
    CyBetweenRows = pccinfo->cyLineSpacing;
 }
 else
 {
    CyBetweenRows = MINCY_BETWEENITEMS;
 }

 /*********************************************************************/
 /* Get the Icon Size.                                                */
 /*********************************************************************/
 CnrCalcIconSize (hwndCnr, &IconSize);

 /*********************************************************************/
 /* Start at the first FieldInfo (column).                            */
 /*********************************************************************/
 pFieldInfo = pccinfon->pFirstVisibleFI;

 /*********************************************************************/
 /* If there are no FieldInfo's then we have no business trying to    */
 /* position this thing.                                              */
 /*********************************************************************/
 if (!pFieldInfo)
 {
    return (TRUE);
 }

 /*********************************************************************/
 /* Initialize the usLongestLine field for each column.               */
 /*********************************************************************/
 while (pFieldInfo)
 {
    if (pFieldInfo->flTitle & CFA_BITMAPORICON)
    {
       PFINFROMX(pFieldInfo)->usLongestLine = (USHORT)IconSize.x;       /*DAB1*/
    }
    else
    /******************************************************************/
    /* The Column Title is a text string.                             */
    /******************************************************************/
    {
       IcnCalcTextSize (hps, IconSize, pccinfon,
                        (LPSTR)pFieldInfo->pTitleData,
                        IS_SIDETEXT, &TextSize);

       PFINFROMX(pFieldInfo)->usLongestLine = (WORD)TextSize.cx;        /*DAB1*/
    }
    pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
 }

 /*********************************************************************/
 /* Start at the first FieldInfo (column).                            */
 /*********************************************************************/
 pFieldInfo = pccinfon->pFirstVisibleFI;

 PMGetClientRect (hwndCnr, &Rect);
 CnrQueryTitleRect (hwndCnr, &CnrTitle, NULL);
 CnrQueryColumnTitleWndRect (hwndCnr, TRUE, &ColTitleRect);

 pRecord = LastValidRec;

  /********************************************************************/
  /* If pRecord is NULL, this indicates we must position all of the   */
  /* records.  Otherwise, we position the next record after the one   */
  /* passed in as LastValidRec.  We do this because we don't want to  */
  /* waste our time positioning a valid record do we?  The next record*/
  /* after LastValidRec is the first invalid one.  We start there.    */
  /********************************************************************/
  if (!pRecord)
  {
    pRecord = pccinfon->preccItem;

    /******************************************************************/
    /* If there are no records, then we have no business trying to    */
    /* position this thing.                                           */
    /******************************************************************/
    if (!pRecord)
    {
       return (TRUE);
    }

    /******************************************************************/
    /* This is the imaginary Bottom of the topmost item in the        */
    /* container.  The first item is positoned relative to this.      */
    /******************************************************************/
    if (pFieldInfo->flData & CFA_BITMAPORICON)
       Prevtop    = (-IconSize.y)  - (CyBetweenRows / 2);
    else
    {
       if (Text = CnrGetColumnData (hwndCnr,
                                    pRecord->preccX,
                                      pFieldInfo))
       {
          IcnCalcTextSize (hps, IconSize, pccinfon,
                           Text, IS_SIDETEXT, &TextSize);
          Prevtop    =  (-(int)TextSize.cy) - (CyBetweenRows / 2);      /*DAB1*/
       }
       else
          Prevtop    =  (-pRecord->RowHeight) - (CyBetweenRows / 2);
    }
  }
  else
  {
    PositionAll = FALSE;
    pRecord = LastValidRec->preccNextItem;

    /******************************************************************/
    /* We want to start positioning after the record pointed to by    */
    /* LastValidRec.  First, get the bottom of the row for            */
    /* LastValidRec and assign Prevbottom to it.  Then get the       */
    /* next record and start positioning from there.                  */
    /******************************************************************/
    while (LastValidRec && (!ISRECORDVISIBLE(LastValidRec)))
    {
      LastValidRec = LastValidRec->preccPrevItem;
    }

    if (LastValidRec)
    {
      Prevtop    = CnrQueryRowPosInDV (LastValidRec, PT_TOP);
      Prevheight = LastValidRec->RowHeight;
    }
    else
    {
      if (pFieldInfo->flData & CFA_BITMAPORICON)
         Prevtop    = (-IconSize.y)  - (CyBetweenRows / 2);
      else
      {
         if (Text = CnrGetColumnData (hwndCnr,
                                      pRecord->preccX,
                                      pFieldInfo))
         {
            IcnCalcTextSize (hps, IconSize, pccinfon,
                             Text, IS_SIDETEXT, &TextSize);
            Prevtop    =  (-(int)TextSize.cy) - (CyBetweenRows / 2);    /*DAB1*/
         }
         else
            Prevtop    =  (-pRecord->RowHeight) - (CyBetweenRows / 2);
      }
    }
  }

  while (pRecord)
  {
    if (!ISRECORDVISIBLE(pRecord))
    {
      if (pRecord->preccPrevItem)
      {
        pRecord->ptlNonIconPos = pRecord->preccPrevItem->ptlNonIconPos;
      }
      else
      {
        pRecord->ptlNonIconPos.x = pccinfon->lAveCharWidth;
        pRecord->ptlNonIconPos.y = Prevtop + CyBetweenRows +
                                   IconSize.y;
      }

      pRecord->RowHeight = 0;
    }
    else
    {
       /***************************************************************/
       /* For the first item in each row we need to save its width    */
       /* and height so we can position it after we get the MaxHeight */
       /* of all the items in the row.  We save this value in the     */
       /* slTextSize field of RECORDVIEWN.  We can save the width and */
       /* height of an Icon here too because slTextSize is of type    */
       /* POINT so it will wash.                                      */
       /***************************************************************/
       if (pFieldInfo->flData & CFA_BITMAPORICON)
       {
          pRecord->slTextSize.cx = IconSize.x;                          /*DAB1*/
          pRecord->slTextSize.cy = IconSize.y;                          /*DAB1*/
          pRecord->RowHeight  = (WORD)IconSize.y;

          /************************************************************/
          /* Check and update the longestline field.                  */
          /************************************************************/
          if ((WORD)IconSize.x >
                      PFINFROMX(pFieldInfo)->usLongestLine)
          {
             PFINFROMX(pFieldInfo)->usLongestLine = (WORD)IconSize.x;
          }
       }
       else
       {
          if (!(Text = CnrGetColumnData (hwndCnr,
                                         pRecord->preccX,
                                         pFieldInfo)))
          {
//           ErrorID = GetLastError();
//
//           if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//           {
//              SETERROR(ErrorID);
//              return (FALSE);
//           }
//
//           SETERROR(ErrorID);
          }

          IcnCalcTextSize (hps, IconSize, pccinfon,
                           Text, IS_SIDETEXT, &TextSize);

          /************************************************************/
          /* Free the storage that was allocated for Text only if     */
          /* Text was not type CFA_STRING.                            */
          /************************************************************/
          if (!(pFieldInfo->flData & CFA_STRING))
          {
//           pSatPtr  = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
//           StmFreeStorage (pSatPtr, (LPVOID)Text);
             GlobalFree( (LPVOID)Text );                                /*DAB1*/
          }

          pRecord->slTextSize = TextSize;
          pRecord->RowHeight = (WORD)TextSize.cy;                       /*DAB1*/

          /************************************************************/
          /* Check and update the longestline field.                  */
          /************************************************************/
          if ((WORD)TextSize.cx >                                       /*DAB1*/
                      PFINFROMX(pFieldInfo)->usLongestLine)
          {
             PFINFROMX(pFieldInfo)->usLongestLine = (WORD)TextSize.cx;  /*DAB1*/
          }
       }

       /***************************************************************/
       /* Zip through the rest of the columns in this row for this    */
       /* record and figure out the maximum height for an item in the */
       /* row.  In the meantime, update the usLongestLine field and   */
       /* keep it current.                                            */
       /***************************************************************/
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

       while (pFieldInfo)
       {
          if (pFieldInfo->flData & CFA_BITMAPORICON)
          {
             if ((WORD)IconSize.y > pRecord->RowHeight)
             {
                pRecord->RowHeight = (WORD)IconSize.y;
             }

             if ((WORD)IconSize.x >
                         PFINFROMX(pFieldInfo)->usLongestLine)
             {
                PFINFROMX(pFieldInfo)->usLongestLine =
                                       (WORD)IconSize.x;
             }
          }
          else
          {
             if (!(Text = CnrGetColumnData (hwndCnr,
                                            pRecord->preccX,
                                            pFieldInfo)))
             {
//              ErrorID = GetLastError();
//
//              if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//              {
//                 SETERROR(ErrorID);
//                 return (FALSE);
//              }
//
//              SETERROR(ErrorID);
             }

             IcnCalcTextSize (hps, IconSize, pccinfon,
                              Text, IS_SIDETEXT, &TextSize);

             /*********************************************************/
             /* Free the storage that was allocated for Text only if  */
             /* Text was not type CFA_STRING.                         */
             /*********************************************************/
             if (!(pFieldInfo->flData & CFA_STRING))
             {
//              pSatPtr  = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
//              StmFreeStorage (pSatPtr, (LPVOID)Text);
                GlobalFree( (LPVOID)Text );                             /*DAB1*/
             }

             if ((WORD)TextSize.cy > pRecord->RowHeight)                /*DAB1*/
             {
                pRecord->RowHeight = (WORD)TextSize.cy;                 /*DAB1*/
             }

             if ((WORD)TextSize.cx >                                    /*DAB1*/
                         PFINFROMX(pFieldInfo)->usLongestLine)
             {
                PFINFROMX(pFieldInfo)->usLongestLine =
                                       (WORD)TextSize.cx;               /*DAB1*/
             }
          }

          pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
       }

       /***************************************************************/
       /* Now we have the Maximum height for any item in this row.    */
       /* From this we calculate the position of this record in the   */
       /* row.  We store this (x,y) value in ptlNonIconPos of         */
       /* RECORDVIEWN.                                                */
       /***************************************************************/
       pRecord->ptlNonIconPos.x = pccinfon->lAveCharWidth;

       if (PositionAll)
       {
         pRecord->ptlNonIconPos.y = Prevtop + CyBetweenRows +
                                    ((pRecord->RowHeight +              /*DAB1*/
                                     pRecord->slTextSize.cy) / 2);      /*DAB1*/

         Prevtop    += (CyBetweenRows + pRecord->RowHeight);
       }
       else
       {
         pRecord->ptlNonIconPos.y = Prevtop + CyBetweenRows + Prevheight;
         if (IconSize.y < pRecord->RowHeight)                           /*DAB1*/
         {
           pRecord->ptlNonIconPos.y += (((int)pRecord->RowHeight -
                                              IconSize.y) / 2);
         }

         Prevtop    += (CyBetweenRows + Prevheight);
         Prevheight = pRecord->RowHeight;
       }
    }

    /******************************************************************/
    /* Go to the next record (row), and go back to the first          */
    /* FieldInfo structure, and play it again.                        */
    /******************************************************************/
    pRecord = pRecord->preccNextItem;
    pFieldInfo = pccinfon->pFirstVisibleFI;
  } /* end while */

  pccinfon->preccLastValid = NULL;

 /*******************************************************************/
 /* The items are positioned relative to (0,0), so we need move the */
 /* window by the amount of the distance by which it is already     */
 /* scrolled                                                        */
 /*******************************************************************/
 if (PositionAll)
 {
    pccinfon->ptlNonIconOrg.y = (int)((long)pccinfon->sVscrollPos *
                                (long)pccinfon->sVscrollFactor);

    pccinfon->ptlNonIconOrg.x = (int)((long)pccinfon->sHscrollPos *
                                (long)pccinfon->sHscrollFactor);
 }

 return (TRUE);
}
