/* Revision: 12 1.2.2.2 source/ui/cclnt/cnr/cnrs2p1.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS2P1.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS2P1.C - Name/Text and Icon view routines for the     */
/*                       container control                            */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/**********************************************************************/
#include <windows.h>
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

#ifdef SEGMENTOR
#define CNRS2P1_C
#include "cnrpragw.h"
#endif


/**********************************************************************/
/*                                                                    */
/* FUNCTION NAME:  CnrNameTextVCtlProc                                */
/*                                                                    */
/* USAGES : External                                                  */
/*                                                                    */
/* DESCRIPTIVE NAME:  Container Control Name/Text View Control Proc   */
/*                                                                    */
/* FUNCTIONAL DESCRIPTION:  This procedure will handle all messages   */
/*   for the name/text view of the container.  Any messages that      */
/*   require special handling for the name/text view will be handled  */
/*   here                                                             */
/*                                                                    */
/* NOTES:                                                             */
/*    FORWARD DECLARATION:                                            */
/*                                                                    */
/* INPUT:  HANDLE hwndCnr - window handle for the container.          */
/*         WORD msg - message.                                        */
/*         WORD wParam - message parameter 1.                         */
/*         DWORD lParam - message parameter 2.                        */
/*                                                                    */
/* OUTPUT:                                                            */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:                                                       */
/*                                                                    */
/* EXIT-ERROR:                                                        */
/*                                                                    */
/* SIDE EFFECTS:                                                      */
/*                                                                    */
/**********************************************************************/
LRESULT CALLBACK CnrNameTextVCtlProc( HWND hwndCnr, UINT msg,           /*DAB1*/
                                      WPARAM wParam, LPARAM lParam )    /*DAB1*/
{
  HDC       hps;
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT      rctlWin;
  RECT      rctlInvalid;
  PAINTSTRUCT paint;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Process the messages sent to this view                           */
  /********************************************************************/
  switch (msg)
  {
    case WM_CREATE:
      return FALSE;

    case WM_SIZE:
      pccinfon->ulFlagSet2 |= FL2_WINDOWSIZED;

      if (wParam == SIZEICONIC)
      {
        pccinfon->ulFlagSet2 |= FL2_MINIMIZED;
        pccinfon->ulFlagSet &= ~FL_CNRWINOPEN;
      }
      else
      {
        if (pccinfon->ulFlagSet2 & FL2_MINIMIZED)
        {
          pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;
          pccinfon->ulFlagSet &= ~( FL_CNRWINOPEN | FL_CNRPOSITION |
                              FL_SETSCROLL | FL_NEWVIEWSCROLLSET |
                              FL_SETCNRTITLE);
        } /* endif */
      } /* endif */

      /***************************************************************/
      /* Need resize child window.                                   */
      /***************************************************************/
      pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;

      /***************************************************************/
      /* If window is already open....                               */
      /***************************************************************/
      if ((pccinfon->ulFlagSet & FL_CNRWINOPEN))
      {
        /*************************************************************/
        /* Only when cy of window changes we need to consider the    */
        /* position of the items                                     */
        /*************************************************************/

          /***********************************************************/
          /* If view is flowed and the height of the window changed  */
          /* we need repositioning in name and text view             */
          /***********************************************************/
          if (pccinfo->flWindowAttr & CV_FLOW)
          {
            /*********************************************************/
            /* New view scroll bar is not set.                       */
            /*********************************************************/
            pccinfon->ulFlagSet &= ~(FL_CNRPOSITION | FL_NEWVIEWSCROLLSET);

          } else {
            /*********************************************************/
            /* If window is already open, as cy of viewport in name/ */
            /* text view inc, so does y pos of item  to keep the     */
            /* relative pos of the item to the top left coner of the */
            /* the window(only in current view). The way to do is    */
            /* changing ptlNonIconOrg(when it is not flowed)         */
            /*********************************************************/
          } /* endif */
          /***********************************************************/
          /* cy of viewport in any  view inc, so does y of item      */
          /* to keep the relative pos of the item to the top left    */
          /* coner of the window in icon view                        */
          /***********************************************************/

        /*************************************************************/
        /* If CCS_AUTOPOSITION is set on, and the width of the window */
        /* chnaged, we need rearrange icons. We also need to reset   */
        /* title window.                                             */
        /*************************************************************/
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
          } /* endif */

          pccinfon->ulFlagSet &= ~FL_SETCNRTITLE;

      } else {
      /***************************************************************/
      /* If window is not open, this msg is not for the resizing     */
      /* of window. So, don't change origin and set flag on so       */
      /* that next time we get this msg, we update the origin        */
      /***************************************************************/
        pccinfon->ulFlagSet |= FL_CNRWINOPEN;
      } /* endif */

      PMInvalidateRect( hwndCnr, NULL, TRUE );
      return 0;

    case WM_VSCROLL:
    {                                                                   /*DAB1*/
      HWND hwndScroll = (HWND)(UINT)lParam;                             /*DAB1*/
      int  nThumbPos  = HIWORD(wParam);                                 /*DAB1*/
      WORD wNotifyCode= LOWORD(wParam);                                 /*DAB1*/

      CnrINTViewVScroll(hwndCnr, hwndScroll, nThumbPos, wNotifyCode);   /*DAB1*/
      return 0;
    }                                                                   /*DAB1*/
    case WM_HSCROLL:
    {                                                                   /*DAB1*/
      HWND hwndScroll = (HWND)(UINT)lParam;                             /*DAB1*/
      int  nThumbPos  = HIWORD(wParam);                                 /*DAB1*/
      WORD wNotifyCode= LOWORD(wParam);                                 /*DAB1*/

      CnrINTViewHScroll(hwndCnr, hwndScroll, nThumbPos, wNotifyCode);   /*DAB1*/
      return 0;
    }                                                                   /*DAB1*/

    case WM_PAINT:
      hps = BeginPaint(hwndCnr, &paint);
      rctlInvalid = paint.rcPaint;

      PMFlipDC(hwndCnr, hps);
      PMFlipRect((LPRECT)&rctlInvalid);

      pccinfon->ulFlagSet |= FL_CNRWINOPEN;

      if (!(IsIconic(hwndCnr)))
      {
        pccinfon->ulFlagSet |= FL_CNRWINOPEN;
        pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;

        if (!(pccinfon->ulFlagSet & FL_SETCNRTITLE)) {

          PMGetClientRect(hwndCnr, &rctlWin);

          /*************************************************************/
          /* We need reset Cnr Title Window. (The cnr window is not    */
          /* initialized or the font of the Title Window is changed)   */
          /*************************************************************/
          CnrSizeCnrTitle(hwndCnr, hps, rctlWin);

          /************************************************************/
          /* Because Cnr Title Window is reset, the size of the       */
          /* client area might change, so, we need reposition of      */
          /* the items.                                               */
          /************************************************************/
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
          } /* endif */
        } /* endif */

        /**************************************************************/
        /* If repositioning is necessary, reposition items.           */
        /**************************************************************/
        if (!(pccinfon->ulFlagSet & FL_CNRPOSITION)) {
          CnrPositionNameTextView(hwndCnr, hps, NULL);
          pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;

          /************************************************************/
          /* Make sure FL_CNRWINOPEN is on(for the case of the        */
          /* initial painting without sending WM_SIZE                 */
          /************************************************************/
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;
          pccinfon->ulFlagSet |= (FL_CNRPOSITION | FL_TEXTCALC );
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_SIZECHILDREN))
        {
          /************************************************************/
          /* resize the scroll bars                                   */
          /************************************************************/
          PMGetClientRect(hwndCnr, &rctlWin);
          CnrSize(hwndCnr, rctlWin.right, rctlWin.bottom);       //FLIP

          /************************************************************/
          /* Set the scroll bar                                       */
          /************************************************************/
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;
          pccinfon->ulFlagSet |= FL_SIZECHILDREN;
        } /* endif */


        if (!(pccinfon->ulFlagSet & FL_SETSCROLL))
        {
          /************************************************************/
          /* Set the scroll bar                                       */
          /************************************************************/
          CnrSetNameTextViewScroll(hwndCnr, FALSE);
          pccinfon->ulFlagSet |= FL_SETSCROLL;
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_SETFIRSTINVIEW))
        {
          CnrSetFirstItemInNTV (hwndCnr, pccinfon->preccItem,
                                TRUE);
          pccinfon->ulFlagSet |= FL_SETFIRSTINVIEW;
        }

        CnrEraseRect(hwndCnr, hps, rctlInvalid);
        if (!(CnrPaintNameTextView( hwndCnr, hps, &rctlInvalid)))
        {
           EndPaint(hwndCnr, &paint);
           return ((long)NULL);
        }

        EndPaint(hwndCnr, &paint);

        if (pccinfon->ulFlagSet2 & FL2_WINDOWSIZED)
        {
           /***********************************************************/
           /* Windows will call CnrEnumWindowsProc for each child     */
           /***********************************************************/
           EnumChildWindows(hwndCnr, CnrEnumWindowsProc, 0);            /*DAB1*/

           pccinfon->ulFlagSet2 &= ~FL2_WINDOWSIZED;
        }
        else
        {
          PMInvalidateRect (GetDlgItem (hwndCnr, CID_VSCROLL),
                             NULL, FALSE);

          PMInvalidateRect (GetDlgItem (hwndCnr, CID_HSCROLL),
                             NULL, FALSE);
        }
      }
      else
      {
        EndPaint (hwndCnr, &paint);
      }

      pccinfon->ulFlagSet2 |= FL2_INITPAINT;
      return 0;

    /*****************************************************************/
    /* All other messages are sent on to the superclass              */
    /*****************************************************************/
    default:
      return ((long)CnrGateProc( hwndCnr, msg, wParam, lParam ));
  }
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CalcTextSizeInIV                                  */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
void FAR CalcTextSizeInIV(hwnd, hps)
HANDLE     hwnd;
HDC        hps;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  precord;           /* record to be checked */
  POINT         iconSize;          /* size of the icon */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get icon size.                                                  */
  /*******************************************************************/
  CnrCalcIconSize(hwnd, &iconSize);

  precord = pccinfon->preccItem;
  while (precord)
  {
    IcnCalcTextSize(hps, iconSize, pccinfon,
                     (pccinfo->flWindowAttr & CV_TREE) ?
                                        GETLPSTRFORTRV(precord->preccX):
                                        precord->preccX->pszIcon,
                     (pccinfo->flWindowAttr & CV_TREE) ?
                                        IS_SIDETEXT:
                                        IS_BENEATHTEXT,
                      &(precord->slTextSize));
    precord = GetNextItem(pccinfon, precord);
  }

  return;
}
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrPositionNameTextView                           */
/*                                                                   */
/* USAGES : External                                                 */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Position Name/Text View      */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function positions name/text view   */
/*   records. The precord parameter is used as a starting point to   */
/*   begin positioning.  If the starting record is NULL, then        */
/*   positioning will begin with the first record in the container.  */
/*   Otherwise, the next record after the specified record is        */
/*   positioned based on the location of the specified record.       */
/*   If the view is flowed, the starting record is first             */
/*   "backtracked" to find the longest length entry to this point.   */
/*   The longest length is maintained to determine the starting      */
/*   point of the next Column.                                       */
/*   Records are then positioned, textSize fields are filled until   */
/*   the current position goes to the next column and fill the       */
/*   longestLine fields of the column. Positioning the columns       */
/*   continues until no records available.                           */
/*   If the view is not flowed, all records are positioned first and */
/*   the longestLine fields are filled.                              */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle of the container window           */
/*         LPRECORDCORE precord - pointer to external record core     */
/*             data structure                                        */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrPositionNameTextView( hwndCnr, hps, precord)
HANDLE       hwndCnr;
HDC          hps;
LPRECORDCOREN precord;
{
  WORD           maxTextWidth;     /* max width of a column            */
  int            cyBetween;        /* space between rows               */
  int            bottom;           /* Bottom of an item                */
  int            cyHscroll;        /* height of the H scroll bar       */
  int            prevy;            /* the previous bottom              */
  int            topMost;          /* the possible highest top         */
  int            top;
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  POINT          currentPos;       /* the position of the current item */
  LPRECORDCOREN  precPos;          /* curr record for longestLine      */
  LPRECORDCOREN  newColumnRecord;  /* first record in a column         */
  LPSTR          pszString;        /* text string                      */
  RECT           rectlWindow;      /* rectangle of the window          */
  RECT           rctlLTitle;       /* rectangle of the list title      */
  POINT          iconSize;         /* Icon size for Name view          */
  SIZEL          currTextSize;     /* Text size of the current record  */
  SIZEL          prevTextSize;     /* Text size of the previous record *///FLIP
  DWORD          FlowView;         /* Is flowview ?                    */
  BOOL           noVisibleItems = TRUE;

  PMGetClientRect( hwndCnr, &rectlWindow );
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  cyHscroll = GetSystemMetrics(SM_CYHSCROLL);
  FlowView = pccinfo->flWindowAttr & CV_FLOW;

  /*******************************************************************/
  /* Set cyBetween                                                   */
  /*******************************************************************/
  if (pccinfo->cyLineSpacing >= 0)
  {
    cyBetween = pccinfo->cyLineSpacing;
  }
  else
  {
    cyBetween = MINCY_BETWEENITEMS;
  }

  /*******************************************************************/
  /* Set icon dimensions.                                            */
  /*******************************************************************/
  if (pccinfo->flWindowAttr & CV_TEXT)
  {
    iconSize.x = 0;
    iconSize.y = 0;
  }
  else
  {
    CnrCalcIconSize( hwndCnr, &iconSize );
  }

  /*******************************************************************/
  /* Get List title box size                                         */
  /*******************************************************************/
  CnrQueryTitleRect ( hwndCnr, &rctlLTitle, NULL);

  /*******************************************************************/
  /* Imaginary top of the window which the first item in a column    */
  /* is positioned relative to.  The top of the first item in a      */
  /* column will be (top - cyBetween) which is                      */
  /* lAveCharWidth below  than top of the client area                */
  /*******************************************************************/
  topMost = rectlWindow.top + rctlLTitle.bottom + (cyBetween / 2) +
            pccinfon->lAveCharWidth;

  /*******************************************************************/
  /* Get bottom and x position of the previous item of the first item*/
  /* to be positioned. If no previous is available, set the bottom   */
  /* cyBetween higher than top of the first item.                    */
  /*******************************************************************/
  if (precord)
  {
    currentPos = precord->ptlNonIconPos;
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      pszString = GETLPSTRFORTV(precord->preccX);
    }
    else
    {
      pszString = GETLPSTRFORNV(precord->preccX);
    }

    maxTextWidth = precord->usLongestLine;

    /*****************************************************************/
    /* Find topmost point of the current record                      */
    /*****************************************************************/
    top = CnrQueryPtOfItemInNTV(hwndCnr, precord, &iconSize,
                                PT_TOP);
  }
  else
  {
    /******************************************************************/
    /* There is no current record. So set bottom the topmost point   */
    /* of the window. cyBetween is added because in positioning       */
    /* the first item in a column it is subtracted from the prev      */
    /* bottom and we don't want to have that space on the top.        */
    /* @left margin = lAveCharWidth                                   */
    /* @top margin = lAveCharWidth                                    */
    /* @cy margin between two items = max(9pixels, lExternamleading)  */
    /* @cx margin between two items = 3 * lAveCharWidth (Text)        */
    /* @cx margin between two items = 2 * lAveCharWidth (Name)        */
    /* @cx margin between icon & text = lAveCharWidth (Name)          */
    /******************************************************************/
    precord = pccinfon->preccItem;

    pccinfon->lMaxHeight = 0;

    maxTextWidth = 0;
    currentPos.x = pccinfon->lAveCharWidth;
    top = topMost;
  }

  if (precord)
  {
    /*****************************************************************/
    /* Initialize lMaxHeight.                                        */
    /*****************************************************************/
    if ((pccinfo->flWindowAttr & CV_NAME) &&
         (pccinfon->lMaxHeight == 0))
    {
      CnrCalcIconSize (hwndCnr, &iconSize);
      pccinfon->lMaxHeight = iconSize.y;
    }

     /****************************************************************/
     /* Set the first record to be positioned in the column.         */
     /****************************************************************/
     precPos = precord;
     if (FlowView)
     {
       /**************************************************************/
       /* If the view has CV_FLOW, then backtrack from the current   */
       /* precord to the top of the column  to  reposition from the  */
       /* top of the column.                                         */
       /**************************************************************/
       precPos = precord = FindFirstIteminColumn(hwndCnr, precord);
     } /* endif */

     /****************************************************************/
     /* Initialize text size                                         */
     /****************************************************************/
     currTextSize.cx = currTextSize.cy = 0;                             /*DAB1*/

     /****************************************************************/
     /* Position records                                             */
     /****************************************************************/
     while (precord)
     {
       /**************************************************************/
       /* Save previous record's text size to properly calulate top  */
       /* of the next item.                                          */
       /**************************************************************/
       prevTextSize = currTextSize;

       /**************************************************************/
       /* If this is the first positioning after changing view to    */
       /* the current one, calculate the text size.                  */
       /**************************************************************/
       if (!(pccinfon->ulFlagSet & FL_TEXTCALC))
       {
          if (pccinfo->flWindowAttr & CV_TEXT)
          {
            pszString = GETLPSTRFORTV(precord->preccX);
          }
          else
          {
            pszString = GETLPSTRFORNV(precord->preccX);
          }

          IcnCalcTextSize (hps, iconSize, pccinfon,
                           pszString, IS_SIDETEXT,
                           &(precord->slTextSize));

       }
       currTextSize = precord->slTextSize;
       currTextSize.cy += ICN_SELSPACE;                                 /*DAB1*/

       /**************************************************************/
       /* ulMaxTextHeight is used to decide the need of the V        */
       /* scroll bar in the flowed name/text view and to calculate   */
       /* the range of the scroll bar. (We provide V scroll bar      */
       /* when the height of an item in the list is larger than      */
       /* the height of the client area.).                           */
       /**************************************************************/
       if (ISRECORDVISIBLE(precord) &&
           (pccinfon->lMaxHeight < precord->slTextSize.cy))             /*DAB1*/
       {
         pccinfon->lMaxHeight = precord->slTextSize.cy;                 /*DAB1*/
       } /* endif */

       if (!ISRECORDVISIBLE(precord))
       {
         /************************************************************/
         /* If it is filtered, it's position is same as prev         */
         /* item. If there is no prev item, the position is the      */
         /* first unfiltered item's position.                        */
         /************************************************************/
         if (precord->preccPrevItem)
         {
           precord->ptlNonIconPos =
                    precord->preccPrevItem->ptlNonIconPos;
           currTextSize = precord->preccPrevItem->slTextSize;
           currTextSize.cy += ICN_SELSPACE;                             /*DAB1*/
         }
         else
         {
           /**********************************************************/
           /* Calculate the position of the first unfiltered item.   */
           /**********************************************************/
           if ((pccinfo->flWindowAttr & CV_NAME) &&
               ((iconSize.y + 2 * ICN_EMPHSPACE) > prevTextSize.cy) )   /*DAB1*/
           {
             precord->ptlNonIconPos.y = topMost + ICN_EMPHSPACE;
           }
           else
           {
             precord->ptlNonIconPos.y = topMost;
           }
           precord->ptlNonIconPos.x = pccinfon->lAveCharWidth;

         }
         precord->usLongestLine = 0;

         if (top == topMost &&
            (!precord->preccPrevItem || noVisibleItems))
         {
           currTextSize.cx = 0;                                         /*DAB1*/
           currTextSize.cy = ICN_SELSPACE;                              /*DAB1*/
         }
       }
       else
       {
         noVisibleItems = FALSE;
         /************************************************************/
         /* Save top as a previous top                               */
         /************************************************************/
         prevy = top;

         /************************************************************/
         /* Calculate possible top of the current record.            */
         /************************************************************/
         if (pccinfo->flWindowAttr & CV_TEXT)
         {
           if ((prevTextSize.cy - ICN_SELSPACE) > 0)                    /*DAB1*/
           {
             top += (cyBetween + prevTextSize.cy) +                     /*DAB1*/
                    (TXT_BTMMARGIN + TXT_TOPMARGIN);
           }
           bottom = top + currTextSize.cy;                              /*DAB1*/
         }
         else
         {
           /**********************************************************/
           /* Handle multiple possibilities for Name View            */
           /**********************************************************/
           if ((iconSize.y + 2 * ICN_EMPHSPACE) > prevTextSize.cy)      /*DAB1*/
           {
             if ((prevTextSize.cy - ICN_SELSPACE) > 0)                  /*DAB1*/
               top += (cyBetween + iconSize.y + 2 * ICN_EMPHSPACE);
             else
               top += ICN_EMPHSPACE;
           }
           else
           {
             if ((prevTextSize.cy - ICN_SELSPACE) > 0)                  /*DAB1*/
             {
               top += (cyBetween + prevTextSize.cy) +                   /*DAB1*/
                      (TXT_BTMMARGIN + TXT_TOPMARGIN);
             }
           }

           if ((iconSize.y + 2 * ICN_EMPHSPACE) > currTextSize.cy)      /*DAB1*/
             bottom = top + iconSize.y;
           else
             bottom = top + currTextSize.cy;                            /*DAB1*/
         }

         /************************************************************/
         /* If the bottom of the current record is lower than the    */
         /* bottom of the viewport, CV_FLOW bit is on and it is not  */
         /* the first item in the column,  position the records in   */
         /* the column and set the current record as the first item  */
         /* for the next column.                                     */
         /************************************************************/
         if ((bottom > (rectlWindow.bottom - cyHscroll - 1)) &&
             FlowView && (prevy >= topMost))
         {
           newColumnRecord = precord;

           /**********************************************************/
           /* Set usLongestLine for each item in the column.         */
           /**********************************************************/
           while (precPos != newColumnRecord)
           {
             if (ISRECORDVISIBLE(precPos))
             {
               precPos->usLongestLine = (WORD)maxTextWidth;
             }
             precPos = precPos->preccNextItem;
           }

           /**********************************************************/
           /* Set the first record to be drawn in the new column.    */
           /**********************************************************/
           precPos = newColumnRecord;

           /**********************************************************/
           /* Get the imaginary bottom and x position to calculate   */
           /* the position of the first record in the new column.    */
           /* Set the bottom cyBetween higher than top of the        */
           /* first item.                                            */
           /**********************************************************/
           currentPos.x += (maxTextWidth + iconSize.x +
                           (3 * pccinfon->lAveCharWidth));
           top = topMost;

           /**********************************************************/
           /* Calculate the position and the top of the first        */
           /* record                                                 */
           /**********************************************************/
           if (pccinfo->flWindowAttr & CV_NAME)
           {
             if ((iconSize.y + 2 * ICN_EMPHSPACE) < currTextSize.cy)    /*DAB1*/
             {
               currentPos.y = (top + (currTextSize.cy - iconSize.y)/2); /*DAB1*/

               /******************************************************/
               /* Reset the top position                             */
               /******************************************************/
               top = currentPos.y;
             }
             else
             {
               currentPos.y = top;
             }
           }
           else
           {
             currentPos.y = top;
           }


           /**********************************************************/
           /* Reset the maximum width for the new column.            */
           /**********************************************************/
           maxTextWidth = (WORD)currTextSize.cx;                        /*DAB1*/

         /************************************************************/
         /* It is the name view or the first item in a column is     */
         /* bigger than the height of the window. So continue        */
         /* positioning even though it is below the viewport.        */
         /************************************************************/
         }
         else
         {
           /**********************************************************/
           /* Calculate CurrentPos.y from the top of the item        */
           /**********************************************************/
           if (pccinfo->flWindowAttr & CV_NAME)
           {
             if ((iconSize.y + ICN_EMPHSPACE) < currTextSize.cy)        /*DAB1*/
             {
              currentPos.y = (top + (currTextSize.cy - iconSize.y) / 2);/*DAB1*/
             }
             else
             {
               currentPos.y = top - ICN_EMPHSPACE;
             }

             /********************************************************/
             /* Reset the top position                               */
             /********************************************************/
             top = currentPos.y;
           }
           else
           {
             currentPos.y = top;
           }

           maxTextWidth = max (maxTextWidth, (WORD)currTextSize.cx);    /*DAB1*/

         } /* endif */

         /************************************************************/
         /* Save calculated position                                 */
         /************************************************************/
         precord->ptlNonIconPos = currentPos;

       } /* endif */

       precord= precord->preccNextItem;
     }  /* end while */

     /****************************************************************/
     /* Set longestLine for  the last column for the flowed view     */
     /* and all items in the nonflowed view.                         */
     /****************************************************************/
     if (!FlowView)
     {
       precPos = pccinfon->preccItem;
     }

     while (precPos)
     {
       if (ISRECORDVISIBLE(precPos))
       {
         precPos->usLongestLine = (WORD)maxTextWidth;
       }
       precPos = precPos->preccNextItem;
     }
  } /* endif */

  /*******************************************************************/
  /* It is reposioned relative to the (0,0), so, we need move        */
  /* the window by the amount of the distance by which the           */
  /* is already scrolled                                             */
  /*******************************************************************/
  pccinfon->ptlNonIconOrg.y = (int)((long)pccinfon->sVscrollPos *
                              (long)pccinfon->sVscrollFactor);
  pccinfon->ptlNonIconOrg.x =  (int)((long)pccinfon->sHscrollPos *
                               (long)pccinfon->sHscrollFactor);

  /*******************************************************************/
  /* Text is calculated, and list is positioned. So, change the flag */
  /* accordingly.                                                    */
  /*******************************************************************/
  pccinfon->ulFlagSet |= (FL_TEXTCALC | FL_CNRPOSITION);

  return;
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrQueryPtOfItemInNTV                             */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Query the Pt  of Item in Name/Text View        */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculate pt  of the item  */
/*     specified in the name/text view                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
int  FAR  CnrQueryPtOfItemInNTV(hwndCnr, precord, piconSize, lDir)
HANDLE         hwndCnr;
LPRECORDCOREN  precord;
LPPOINT        piconSize;
long           lDir;            /* flag for the desired x/y pos      */
                                /*  Can be one of PT_TOP, PT_BOTTOM, */
                                /*              PT_LEFT or PT_RIGHT  */

{
  int          lPtofItem=0;        /* Desired x/y pos to get */
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  RECT         rctlIcon;
  RECT         rctlText;

  /********************************************************************/
  /* Check to insure that a valid record pointer was passed           */
  /********************************************************************/
  if (precord)
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    /*************************************************************/
    /* Get the rectangle of the icon and the rectangle of the    */
    /* the text which is relative to the window, not the         */
    /* workplace.                                                */
    /*************************************************************/
    CnrQueryIconTextRect( hwndCnr, precord, piconSize, TRUE,
                          &rctlIcon, &rctlText);

    /******************************************************************/
    /* Check if the topmost point of the item was requested           */
    /******************************************************************/
    if (lDir == PT_TOP)
    {
      /****************************************************************/
      /* Check if name view is the current view                       */
      /****************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        /**************************************************************/
        /* If cy of the text rect is bigger than the cy of the icon   */
        /* the top of the item is the top of the text rectangle.      */
        /* Else, the top of the item is top of the icon.              */
        /**************************************************************/
        lPtofItem = min(rctlIcon.top, rctlText.top) +
                    pccinfon->ptlNonIconOrg.y;
      }
      /****************************************************************/
      /* Otherwise this is text view                                  */
      /****************************************************************/
      else if (pccinfo->flWindowAttr & CV_TEXT)
      {
        /**************************************************************/
        /* The top of the item in the text view is the top of the     */
        /* text rectangle                                             */
        /**************************************************************/
        lPtofItem = precord->ptlNonIconPos.y;
      }
    }
    /******************************************************************/
    /* Otherwise check if the bottommost point of the item was req.   */
    /******************************************************************/
    else if (lDir == PT_BOTTOM)
    {
      /****************************************************************/
      /* Check if name view is the current view                       */
      /****************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        /**************************************************************/
        /* If cy of the text rect is bigger than the cy of the        */
        /* icon, the bottom of the item is the bottom of the text     */
        /* rectangle. Else, the top of the item is top of the icon.   */
        /**************************************************************/
        lPtofItem = max(rctlIcon.bottom, rctlText.bottom) +
                    pccinfon->ptlNonIconOrg.y;
      }
      /****************************************************************/
      /* Otherwise this is text view                                  */
      /****************************************************************/
      else if (pccinfo->flWindowAttr & CV_TEXT)
      {
        /**************************************************************/
        /* The bottom of the item in the text view is the bottom      */
        /* of the text rectangle                                      */
        /**************************************************************/
        lPtofItem = rctlText.bottom + pccinfon->ptlNonIconOrg.y;
      }
    }
    /******************************************************************/
    /* Otherwise check if the leftmost point of the item was requested*/
    /******************************************************************/
    else if (lDir == PT_LEFT)
    {
      /****************************************************************/
      /* If it is name view the leftmost point is left of the        */
      /* icon rect and if it is text view left of the text rect      */
      /* is the leftmost.  The point is relative to the workplace.    */
      /****************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
        lPtofItem = rctlIcon.left + pccinfon->ptlNonIconOrg.x;
      else
        lPtofItem = rctlText.left + pccinfon->ptlNonIconOrg.x;
    }
    /******************************************************************/
    /* Otherwise check if the rightmost point of item was requested   */
    /******************************************************************/
    else if (lDir == PT_RIGHT)
    {
      /****************************************************************/
      /* In both views, the right boundary of the text is the         */
      /* rightmost point                                              */
      /****************************************************************/
      lPtofItem = rctlText.right + pccinfon->ptlNonIconOrg.x;
    }
  }

  return lPtofItem;
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetFirstItemInNTV                              */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set the First Item in the Name/Text Viewport   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in    */
/*     the viewport searching backward or forward depending on       */
/*     the flag flFromAboveLeft. If we want to search forward,       */
/*     we should set flFromAboveLeft = TRUE.                         */
/*     If preccCurrent is NULL, search from the last item            */
/*     (flFromAboveLeft = FALSE) or from the first item              */
/*     (flFromAboveLeft = TRUE)  of the list.                        */
/*     Then, set the first item in the cnrinfon.                     */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetFirstItemInNTV(hwndCnr, preccCurrent, flFromAboveLeft)
HANDLE          hwndCnr;
LPRECORDCOREN  preccCurrent;
BOOL          flFromAboveLeft;

{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_FLOW)
  {
    CnrSetFirstItemInFlowedNTV(hwndCnr, preccCurrent, flFromAboveLeft);
  }
  else
  {
    CnrSetFirstItemInNFNTV(hwndCnr, preccCurrent, flFromAboveLeft);
  }
  return;
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetFirstItemInNFNTV                            */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set the First Item in the Name/Text Viewport   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in    */
/*     viewport searching backward or forward depending on the flag. */
/*     If preccCurrent is NULL, search from the last item            */
/*     (flFromabove = FALSE) or from the first item (flFromAbove =   */
/*     TRUE)  of the list.                                           */
/*     Then, set the first item in the cnrinfon.                     */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetFirstItemInNFNTV(hwndCnr, preccCurrent, flFromAbove)
HANDLE          hwndCnr;
LPRECORDCOREN  preccCurrent;
BOOL          flFromAbove;

{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  RECT         rctlHscroll;        /* H scroll bar rectangle */
  RECT         rctlViewport;       /* rectangle of the viewport */
  RECT         rctlTitle;                                        //FLIP
  POINT        iconSize;           /* size of the icon */
  LPRECORDCOREN preccNext;          /* next item of the current item */
  LPRECORDCOREN preccFirstinView;   /* first item in the viewport */
  long         top;               /* top of the certain range */
  long         bottom;            /* bottom of the certain range */
  BOOL         found = FALSE;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlViewport, NULL,
                   &rctlHscroll);

  if (flFromAbove)
  {
    /*******************************************************************/
    /* If preccCurrent is NULL set it as as the first item in the      */
    /* list.                                                           */
    /*******************************************************************/
    if (!preccCurrent)
    {
      preccCurrent = pccinfon->preccItem;
    }

    while (preccCurrent && !found)
    {
      /*****************************************************************/
      /* Get the top of the item to check if it is in the              */
      /* viewport                                                      */
      /*****************************************************************/
      bottom = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrent,
                                     &iconSize, PT_BOTTOM);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
           (bottom < (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom)))
      {
        preccCurrent = preccCurrent->preccNextItem;
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    if (preccCurrent)
    {
      preccFirstinView = preccCurrent;
    }
    else
    {
      /***************************************************************/
      /* If no item is in the viewport set the first item as the     */
      /* first item of the list.                                     */
      /***************************************************************/
      preccFirstinView = pccinfon->preccItem;
    }
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_NAME)
    {
      preccNext = pccinfon->preccFirstinNV;
    }
    else
    {
      preccNext = pccinfon->preccFirstinTV;
    }

    /*****************************************************************/
    /* Find the first item of which top  boundary is out of the      */
    /* viewport.                                                     */
    /*****************************************************************/
    while (preccCurrent && !found)
    {
      top = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrent,
                                  &iconSize, PT_TOP);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
          (top > (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom)))
      {
        /*************************************************************/
        /* Save preccCurrent as preccPrev for the case that          */
        /* preccCurrent is totally out of viewport.(Above the        */
        /* viewport.)                                                */
        /*************************************************************/
        if (ISRECORDVISIBLE(preccCurrent))
        {
          preccNext = preccCurrent;
        }
        preccCurrent = preccCurrent->preccPrevItem;
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    if (preccCurrent)
    {
      /***************************************************************/
      /* If the first item whose top is above the viewport has       */
      /* bottom in the viewport, it is the first item.(Patially      */
      /* shown) Else, the first item in the viewport is the          */
      /* next item of the current item.                              */
      /***************************************************************/
      bottom = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrent, &iconSize,
                                      PT_BOTTOM);
      if (bottom > (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom))
      {
        preccFirstinView = preccCurrent;
      }
      else
      {
        preccFirstinView = preccNext;
      }
    }
    else
    {
      /***************************************************************/
      /* No item is above the viewport.                              */
      /* The first item in viewport is the first item in the         */
      /* list.                                                       */
      /***************************************************************/
      preccFirstinView = pccinfon->preccItem;
    }
  }

  if (pccinfo->flWindowAttr & CV_NAME)
  {
    pccinfon->preccFirstinNV = preccFirstinView;
  }
  else
  {
    pccinfon->preccFirstinTV = preccFirstinView;
  }

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetFirstItemInFlowedNTV                        */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set the First Item in the Flowed Name/Text     */
/*     Viewport                                                      */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in    */
/*     viewport searching backward or forward depending on the flag. */
/*     If preccCurrent is NULL, search from the last item            */
/*     (flFromLeft = FALSE) or from the first item (flFromLeft =     */
/*     TRUE)  of the list.                                           */
/*     Then, set the first item in the cnrinfon.                     */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetFirstItemInFlowedNTV(hwndCnr, preccCurrent, flFromLeft)
HANDLE         hwndCnr;
LPRECORDCOREN preccCurrent;
BOOL         flFromLeft;

{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  POINT        iconSize;           /* size of the icon */
  LPRECORDCOREN preccFirstinView;   /* first item in the viewport */
  LPRECORDCOREN preccNext;
  long         left;              /* Left of the certain range */
  long         right;             /* right of the certain range */
  BOOL         found = FALSE;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get the icon size                                               */
  /*******************************************************************/
  CnrCalcIconSize( hwndCnr, &iconSize );

  if (flFromLeft)
  {
    /*****************************************************************/
    /* If preccCurrent is NULL set it as as the first item in the    */
    /* list.                                                         */
    /*****************************************************************/
    if (!preccCurrent)
    {
      preccCurrent = pccinfon->preccItem;
    } /* endif */

    while (preccCurrent && !found)
    {
      /***************************************************************/
      /* Get the right of the item to check if it is in the          */
      /* viewport                                                    */
      /***************************************************************/
      right = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrent, &iconSize,
                                    PT_RIGHT);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
          (right < pccinfon->ptlNonIconOrg.x ))
      {
        preccCurrent = preccCurrent->preccNextItem;
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    if (preccCurrent)
    {
      preccFirstinView = preccCurrent;
    }
    else
    {
      /***************************************************************/
      /* If no item is in the viewport set the first item as the     */
      /* first item of the list.                                     */
      /***************************************************************/
      preccFirstinView = pccinfon->preccItem;
    }
  }
  else
  {
    /*****************************************************************/
    /* If preccCurrent is NULL set it as as the last  item in the    */
    /* list(because we need to search backward).                     */
    /*****************************************************************/
    if (!preccCurrent)
    {
      preccCurrent = pccinfon->preccLastItem;
    } /* endif */

    if (pccinfo->flWindowAttr & CV_NAME)
    {
      preccNext = pccinfon->preccFirstinFNV;
    }
    else
    {
      preccNext = pccinfon->preccFirstinFTV;
    }

    /*****************************************************************/
    /* Find the first item of which left boundary is out of the      */
    /* viewport.                                                     */
    /*****************************************************************/
    while (preccCurrent && !found)
    {
      /***************************************************************/
      /* Get the left of the item to check if it is in the           */
      /* viewport                                                    */
      /***************************************************************/
      left = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrent, &iconSize,
                                  PT_LEFT);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
          (left > pccinfon->ptlNonIconOrg.x))
      {
        /*************************************************************/
        /* Save preccCurrent as preccPrev for the case that          */
        /* preccCurrent is totally out of viewport.(Above the        */
        /* viewport.)                                                */
        /*************************************************************/
        if (ISRECORDVISIBLE(preccCurrent))
        {
          preccNext = preccCurrent;
        }
        preccCurrent = preccCurrent->preccPrevItem;
      }
      else
      {
        found = TRUE;
      }
    }

    if (preccCurrent)
    {
      /***************************************************************/
      /* If the first item whose left boundary is out of             */
      /* the viewport has the right boundary which is in the         */
      /* the viewport, it is in the the first column. (Patially      */
      /* shown) Else, the first item in the viewport is the          */
      /* next item.                                                  */
      /***************************************************************/
      right = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrent, &iconSize,
                                    PT_RIGHT);

      if (right > pccinfon->ptlNonIconOrg.x )
      {
        /*************************************************************/
        /* Find the first item in the first column.                  */
        /*************************************************************/
        preccFirstinView = FindFirstIteminColumn(hwndCnr,preccCurrent);
      }
      else
      {
        preccFirstinView = preccNext;
      }
    }
    else
    {
      /***************************************************************/
      /* No item is left  to the viewport.                           */
      /* The first item in viewport is the first item in the list.   */
      /***************************************************************/
      preccFirstinView = pccinfon->preccItem;
    }
  }

  if (pccinfo->flWindowAttr & CV_NAME)
  {
    pccinfon->preccFirstinFNV = preccFirstinView;
  }
  else
  {
    pccinfon->preccFirstinFTV = preccFirstinView;
  } /* endif */
  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrGetLastItemInView                              */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set the pointer to the last item in the        */
/*                    viewport for Non-flowed Name or Text view.     */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the last item in     */
/*     the viewport and sets a global variable to this.              */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LPRECORDCOREN FAR CnrGetLastItemInView (HWND hwndCnr)
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  RECT          rctlHscroll;       /* H scroll bar rectangle          */
  POINT         iconSize;          /* size of the icon                */
  LPRECORDCOREN preccCurrinView;   /* next item of the current item   */
  LPRECORDCOREN preccLast;         /* prev item of the current item   */
  long          top;               /* bottom of the record            */
  long          left;              /* Left  of the record             */
  long          CyBetweenRows;
  RECT          rctlViewPort;
  RECT          rctlTitle;

  /********************************************************************/
  /* Get pointer to Container internal data.                          */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /******************************************************************/
  /* Calculate the size of the scroll bars and the viewport.        */
  /******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlViewPort,
                   NULL, &rctlHscroll);

  rctlViewPort.right += pccinfon->ptlNonIconOrg.x;
  rctlViewPort.bottom += pccinfon->ptlNonIconOrg.y + rctlTitle.bottom;

  /*******************************************************************/
  /* Initialize the pointer to the current record                    */
  /*******************************************************************/
  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
    /*****************************************************************/
    /* Query the left details viewport (window)                      */
    /*****************************************************************/
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                     &rctlViewPort);
    preccCurrinView = pccinfon->preccFirstRowInDV;
    rctlViewPort.bottom += pccinfon->ptlNonIconOrg.y;
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_FLOW)
    {
      /************************************************************/
      /* Flowed Name View                                         */
      /************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        preccCurrinView = pccinfon->preccFirstinFNV;
      }
      /************************************************************/
      /* Flowed Text View                                         */
      /************************************************************/
      else
      if (pccinfo->flWindowAttr & CV_TEXT)
      {
        preccCurrinView = pccinfon->preccFirstinFTV;
      }
    }
    else    //non-flow view
    {
      /************************************************************/
      /* Non-Flowed Name View                                     */
      /************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        preccCurrinView = pccinfon->preccFirstinNV;
      }
      /************************************************************/
      /* Non-Flowed Text View                                     */
      /************************************************************/
      else
      if (pccinfo->flWindowAttr & CV_TEXT)
      {
        preccCurrinView = pccinfon->preccFirstinTV;
      }
    }
  }

  if (!preccCurrinView)
  {
    pccinfon->preccItem;
  }

  /********************************************************************/
  /* Find the last record in the viewport.                            */
  /********************************************************************/
  if (preccCurrinView)
  {
    /******************************************************************/
    /* Get the coordinates of the record.                             */
    /******************************************************************/
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       /***************************************************************/
       /* Set the distance between the rows.  If the application has  */
       /* specified it, use it...otherwise use our constant.          */
       /***************************************************************/
       CyBetweenRows = 0;

       top = CnrQueryRowPosInDV (preccCurrinView, PT_TOP) -
                 (CyBetweenRows / 2);
    }
    else
    {
      top = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrinView,
                                      &iconSize, PT_TOP);

      if (pccinfo->flWindowAttr & CV_FLOW)
      {
        left  = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrinView,
                                        &iconSize, PT_LEFT);
      }
    }

    preccLast = preccCurrinView;
    /******************************************************************/
    /* Loop until the last record in view is found                    */
    /******************************************************************/
    while (preccCurrinView && ((!(pccinfo->flWindowAttr & CV_FLOW) &&
           (top < rctlViewPort.bottom)) ||
           ((pccinfo->flWindowAttr & CV_FLOW) &&
           (left < rctlViewPort.right))))
    {
      preccLast = preccCurrinView;

      /****************************************************************/
      /* Index to the next item and reset the record bottom coordinate*/
      /****************************************************************/
      if (preccCurrinView = preccCurrinView->preccNextItem)
      {
        /**************************************************************/
        /* Get the bottom coordinate of the record.                   */
        /**************************************************************/
        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          top = CnrQueryRowPosInDV (preccCurrinView,
                    PT_TOP) - (CyBetweenRows / 2);
        }
        else
        {
          left  = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrinView,
                                          &iconSize, PT_LEFT);

          top = CnrQueryPtOfItemInNTV(hwndCnr, preccCurrinView,
                                          &iconSize, PT_TOP);
        }
      }
    }
  }

  return (preccLast);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetNameTextViewVScroll                         */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set Scroll Bar Range and Pos                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the name/text view                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetNameTextViewScroll(hwndCnr, flChangeHeight  )
HANDLE        hwndCnr;
BOOL        flChangeHeight;

{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_FLOW)
  {
     CnrSetFlowedNameTextViewScroll(hwndCnr, flChangeHeight);
  }
  else
  {
     CnrSetNFNameTextViewScroll(hwndCnr);
  }

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetNFNameTextViewVScroll                       */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set Scroll Bar Range and Pos                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the name/text view                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetNFNameTextViewScroll(hwndCnr)
HANDLE        hwndCnr;
{
  HANDLE       hwndHscroll;         /* handle of H scroll window */
  HANDLE       hwndVscroll;         /* handle of V scroll window */
  int          bottom;              /* bottom of the certain range */
  int          top;                 /* top of the certain range */
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccFirstinView;    /* first record in the current view */
  LPRECORDCOREN preccLastItem;       /* last item in the list */
  LPRECORDCOREN preccTemp;
  WORD         usLongestLine;       /* longest line of the column */
  RECT         rctlClient;          /* rectangle of the client area */
  RECT         rctlHScroll;         /* rectangle of the H scroll bar */
  RECT         rctlTitle;           /* rectangle of the title window */
  POINT        iconSize;            /* size of the icon */
  int          sRBPos;              /* range of the scroll down or right */
  int          sLTPos;              /* range of the scroll up or left */

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlClient, NULL,
                   &rctlHScroll) ;

  /*******************************************************************/
  /* Get handle of the scroll bar                                    */
  /*******************************************************************/
  hwndHscroll = GetDlgItem (hwndCnr, CID_HSCROLL);
  hwndVscroll = GetDlgItem (hwndCnr, CID_VSCROLL);

  /*******************************************************************/
  /* Set the first item in the viewport in the current view          */
  /*******************************************************************/
  if (pccinfo->flWindowAttr & CV_NAME)
  {
    preccFirstinView = pccinfon->preccFirstinNV;
  }
  else
  {
    preccFirstinView = pccinfon->preccFirstinTV;
  } /* endif */

  if (pccinfon->ulFlagSet & FL_NEWVIEWSCROLLSET )
  {
    /*****************************************************************/
    /* If this is not the first setting of the scroll bar in the     */
    /* current view, just consider the scroll pos to get down        */
    /* scroll range in pixels.                                       */
    /*****************************************************************/
    sLTPos = pccinfon->sVscrollPos * pccinfon->sVscrollFactor;
  }
  else
  {
    sLTPos = 0;
    /*****************************************************************/
    /* If the first item in the view is not specified, assign        */
    /* the first item in the list as the first item in the view.     */
    /* And no more scroll up (sRTPos = 0 )                           */
    /*****************************************************************/
    if (!preccFirstinView)
    {
      preccFirstinView = pccinfon->preccItem;
    }
    else
    {
      /***************************************************************/
      /* Get the distance from top of list and top of the first      */
      /* item in viewport. This is the current pos of slider,        */
      /* and shows the possible distance to scroll up(sRTPos)        */
      /***************************************************************/
      if (pccinfon->preccItem &&
         (pccinfon->preccItem != preccFirstinView))
      {
        top = CnrQueryPtOfItemInNTV(hwndCnr, pccinfon->preccItem,
                                     &iconSize, PT_TOP);
        bottom = CnrQueryPtOfItemInNTV(hwndCnr, preccFirstinView,
                                       &iconSize, PT_TOP);

        sLTPos = bottom - top;
      } /* endif */
    } /* endif */
    /*****************************************************************/
    /* Move the window by the amount the slider moved down           */
    /* (sLTPos)                                                      */
    /*****************************************************************/
    pccinfon->ptlNonIconOrg.y = sLTPos;

    pccinfon->ulFlagSet |= FL_NEWVIEWSCROLLSET;
  } /* endif */


  if (pccinfon->preccLastItem)
  {
    preccLastItem =  pccinfon->preccLastItem;

    /******************************************************************/
    /* Get the last unfiltered item in the list                       */
    /******************************************************************/
    while (preccLastItem->preccPrevItem &&
           !ISRECORDVISIBLE(preccLastItem))
    {
      preccLastItem = preccLastItem->preccPrevItem;
    }

    /*****************************************************************/
    /* Get the bottom of the item to calculate sRBpos                */
    /*****************************************************************/
    bottom = CnrQueryPtOfItemInNTV(hwndCnr, preccLastItem,
                                    &iconSize, PT_BOTTOM);

    /*****************************************************************/
    /* Let's have some margin on the bottom.                         */
    /*****************************************************************/
    bottom += pccinfon->lAveCharWidth;

    /*****************************************************************/
    /* Set the scroll range below the window (the distance from      */
    /* bottom of the client area and bottom of the last item.        */
    /* Minimum is 0.                                                 */
    /*****************************************************************/
    sRBPos = bottom - pccinfon->ptlNonIconOrg.y -
             rctlClient.bottom - rctlTitle.bottom;
  }
  else
  {
    sRBPos = 0;
  }

  SetVscrollInNFView(hwndCnr, hwndVscroll, sRBPos, sLTPos, rctlClient);

  /*******************************************************************/
  /* Set sHscrollFactor as 1 so that the minimum scrolling unit      */
  /* can be a pixels.                                                */
  /*******************************************************************/
  pccinfon->sHscrollFactor = 1;
  usLongestLine = 0;
  if (pccinfon->preccItem)
  {
    preccTemp = pccinfon->preccItem;

    while (preccTemp && !ISRECORDVISIBLE(preccTemp))
    {
      preccTemp = preccTemp->preccNextItem;
    }

    if (preccTemp)
    {
      /***************************************************************/
      /* The longest line in the column is the rightmost point of an */
      /* item plus lAveCharWidth for the left margin.                */
      /***************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        usLongestLine = (WORD)(preccTemp->usLongestLine +
                        3 * pccinfon->lAveCharWidth + iconSize.x);
      }
      else
      {
        usLongestLine = (WORD)(preccTemp->usLongestLine +
                            2 * pccinfon->lAveCharWidth);
      } /* endif */
    } /* endif */

  } /* endif */

  if (usLongestLine > (WORD)rctlClient.right)
  {
    /*****************************************************************/
    /* Calculate sLBPos and sRTPos.                                  */
    /*****************************************************************/
    sLTPos = pccinfon->ptlNonIconOrg.x;
    sRBPos = usLongestLine - pccinfon->ptlNonIconOrg.x - rctlClient.right;

    SetHscrollInNFView(hwndHscroll, pccinfon, sLTPos, sRBPos,
                       rctlClient);
  }
  else
  {
    pccinfon->ptlNonIconOrg.x = 0;
    pccinfon->sHscrollPos = 0;
    pccinfon->sHscrollMax = 0;

  } /* endif */

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  SetVscrollInNFView                                */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set V Scroll Bar                               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the name, text and tree view          */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
void FAR SetVscrollInNFView(hwnd, hwndVscroll, sRBPos, sLTPos,
                            rctlClient)
HANDLE        hwnd;
HANDLE        hwndVscroll;
int           sRBPos;
int           sLTPos;
RECT          rctlClient;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  int           sLTPosTemp;
  int           sLTPosMove;
  int           sMaxPos;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Calculate total scroll range in pixels.                         */
  /*******************************************************************/
  sLTPosTemp = max(0, min(sLTPos, sLTPos + sRBPos));
  sLTPosMove = sLTPosTemp - sLTPos;
  pccinfon->ptlNonIconOrg.y += sLTPosMove;
  sLTPos = sLTPosTemp;
  sRBPos = max(0, sRBPos);
  if (sLTPosMove < 0)
  {
    if (pccinfo->flWindowAttr & CV_TREE)
    {
      CnrTRVSetFirstItem( hwnd, pccinfon->preccFirstinTRV, FALSE );
    }
    else if (pccinfo->flWindowAttr & CV_TEXT)
    {
      CnrSetFirstItemInNTV( hwnd, pccinfon->preccFirstinTV, FALSE );
    }
    else
    {
      CnrSetFirstItemInNTV( hwnd, pccinfon->preccFirstinNV, FALSE );
    }
  }

  sMaxPos = sLTPos + sRBPos;

  /*******************************************************************/
  /* Calculate the minimum unit of scrolling.                        */
  /*******************************************************************/
  pccinfon->sVscrollFactor = CalcScrollFactor(hwnd, sMaxPos, TRUE);

  /*******************************************************************/
  /* Calculate the scroll ranges  in the unit.                       */
  /*******************************************************************/
  if (sRBPos % (long)(pccinfon->sVscrollFactor))
  {
    sRBPos = sRBPos / pccinfon->sVscrollFactor + 1;
  }
  else
  {
    sRBPos = sRBPos / pccinfon->sVscrollFactor;
  } /* endif */

  if (sLTPos % pccinfon->sVscrollFactor)
  {
    sLTPos = sLTPos / pccinfon->sVscrollFactor + 1;
  }
  else
  {
    sLTPos = sLTPos / pccinfon->sVscrollFactor;
  } /* endif */

  /*******************************************************************/
  /* Calculate total scroll range in the unit.                       */
  /*******************************************************************/
  sMaxPos = sRBPos + sLTPos;

  if (sMaxPos)
  {
    /*****************************************************************/
    /* Enable the scroll bar if something outside viewport           */
    /*****************************************************************/
     EnableWindow (hwndVscroll, TRUE);

    /*****************************************************************/
    /* Set scroll bar. Range is 0 to sMaxPos. Current Pos is         */
    /* sLTPos                                                        */
    /*****************************************************************/
    SetScrollRange(hwndVscroll, SB_CTL, 0, sMaxPos, FALSE);
    SetScrollPos  (hwndVscroll, SB_CTL, sLTPos, TRUE);
    pccinfon->sVscrollPos = sLTPos;
  }
  else
  {
    /*****************************************************************/
    /* Disable the scroll bar if nothing outside viewport            */
    /*****************************************************************/
     EnableWindow (hwndVscroll, FALSE);
     pccinfon->sVscrollPos = 0;
  } /* endif */

  pccinfon->sVscrollMax = sMaxPos;

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  SetHscrollInNFView                                */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set H Scroll Bar                               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the name, text and tree view          */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
void FAR SetHscrollInNFView(hwndHscroll, pccinfon, sLTPos, sRBPos,
                           rctlClient)
HANDLE         hwndHscroll;
LPCNRINFON     pccinfon;
int            sLTPos;
int            sRBPos;
RECT           rctlClient;
{
  int           sLTPosTemp;
  int           sLTPosMove;
  int           sMaxPos;

  /*******************************************************************/
  /* Calculate total scroll range in pixels.                         */
  /*******************************************************************/
  sLTPosTemp = max(0, min(sLTPos, sLTPos + sRBPos));
  sLTPosMove = sLTPosTemp - sLTPos;
  pccinfon->ptlNonIconOrg.x += sLTPosMove;
  sLTPos = sLTPosTemp;
  sRBPos = max(0, sRBPos);

  /*******************************************************************/
  /* Calculate total scroll range                                    */
  /*******************************************************************/
  sMaxPos = sLTPos + sRBPos;

  if (sMaxPos)
  {
    /*****************************************************************/
    /* Enable the scroll bar if something outside viewport           */
    /*****************************************************************/
     EnableWindow (hwndHscroll, TRUE);

    /****************************************************************/
    /* Set scroll bar. Range is 0 to sMaxPos. Current Pos is        */
    /* sRTPos                                                       */
    /****************************************************************/
    SetScrollRange(hwndHscroll, SB_CTL, 0, sMaxPos, FALSE);
    SetScrollPos  (hwndHscroll, SB_CTL, sLTPos, TRUE);
    pccinfon->sHscrollPos = sLTPos;
  }
  else
  {
    /*****************************************************************/
    /* Disable the scroll bar if nothing outside viewport            */
    /*****************************************************************/
     EnableWindow (hwndHscroll, FALSE);
     pccinfon->sHscrollPos = 0;
  } /* endif */

  pccinfon->sHscrollMax = sMaxPos;

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetFlowedNameTextViewVScroll                   */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set Scroll Bar Range and Pos                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the name/text view                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetFlowedNameTextViewScroll(hwndCnr, flChangeHeight )
HANDLE      hwndCnr;
BOOL        flChangeHeight;
{
  HANDLE         hwndHscroll;       /* handle of H scroll window     */
  HANDLE         hwndVscroll;       /* handle of V scroll window     */
  int            left;              /* Left of the certain range     */
  int            right;             /* Right of the certain range    */
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  preccFirstinView;  /* first rec in the current view */
  LPRECORDCOREN  preccLastItem;     /* last item in the list         */
  RECT           rctlClient;        /* rectangle of the client area  */
  RECT           rctlHScroll;       /* rectangle of the H scroll bar */
  RECT           rctlVScroll;       /* rectangle of the V scroll bar */
  POINT          iconSize;          /* size of the icon              */
  int            sRBPos;            /* range of the scroll up/right  */
  int            sLTPos;            /* range of the scrall down/left */
  int            sMaxPos;           /* total range (sRBPos + sLTPos) */
  int            sLTPosTemp;
  int            sLTPosMove;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, NULL, &rctlClient, &rctlVScroll,
                   &rctlHScroll);

  /*******************************************************************/
  /* Get handle of the scroll bar                                    */
  /*******************************************************************/
  hwndHscroll = GetDlgItem (hwndCnr, CID_HSCROLL);
  hwndVscroll = GetDlgItem (hwndCnr, CID_VSCROLL);

  /*******************************************************************/
  /* Set the first item in the viewport in the current view          */
  /*******************************************************************/
  if (pccinfo->flWindowAttr & CV_NAME)
  {
    /*****************************************************************/
    /* Find the first item of the column in which the current        */
    /* the first item in the viewport. (because of repositioning     */
    /* the current first item can be in the middle of the column     */
    /* and it is the first column, we reset First Item in the        */
    /* viewport.                                                     */
    /*****************************************************************/
    pccinfon->preccFirstinFNV = FindFirstIteminColumn (hwndCnr,
                                            pccinfon->preccFirstinFNV);
    preccFirstinView = pccinfon->preccFirstinFNV;
  }
  else
  {
    pccinfon->preccFirstinFTV = FindFirstIteminColumn (hwndCnr,
                                            pccinfon->preccFirstinFTV);
    preccFirstinView = pccinfon->preccFirstinFTV;
  }

  if ((pccinfon->ulFlagSet & FL_NEWVIEWSCROLLSET) && !flChangeHeight)
  {
    /*****************************************************************/
    /* If this is not the first setting of the scroll bar in the     */
    /* current view, just consider the scroll pos to get left        */
    /* scroll range.                                                 */
    /*****************************************************************/
    sLTPos = pccinfon->sHscrollPos * pccinfon->sHscrollFactor;

  }
  else
  {
    sLTPos = 0;
    /*****************************************************************/
    /* If the first item in the view is not specified, assign        */
    /* the first item in the list as the first item in the view.     */
    /* And no more scroll left(sLTPos = 0 )                          */
    /*****************************************************************/
    if (!preccFirstinView)
    {
      preccFirstinView = pccinfon->preccItem;
    }
    else
    {
      /***************************************************************/
      /* Get the distance from the leftmost point of the list to     */
      /* the left boundary of the item in the first column in        */
      /* in the viewport. This is the current pos of slider,         */
      /* and shows the possible distance to scroll left(sLTPos)      */
      /***************************************************************/
      if (pccinfon->preccItem &&
         (pccinfon->preccItem != preccFirstinView))
      {
        left = CnrQueryPtOfItemInNTV(hwndCnr, pccinfon->preccItem,
                                     &iconSize, PT_LEFT);
        right = CnrQueryPtOfItemInNTV(hwndCnr, preccFirstinView,
                                       &iconSize, PT_LEFT);
        sLTPos = right - left;
      }
    }

    /*****************************************************************/
    /* Move the window by the amount the slider moved right          */
    /* (sLTPos)                                                      */
    /*****************************************************************/
    pccinfon->ptlNonIconOrg.x = sLTPos;


    pccinfon->ulFlagSet |= FL_NEWVIEWSCROLLSET;
  }


  if (pccinfon->preccLastItem)
  {
    preccLastItem =  pccinfon->preccLastItem;

    /*****************************************************************/
    /* Get the last unfiltered item in the list                      */
    /*****************************************************************/
    while (preccLastItem->preccPrevItem &&
           !ISRECORDVISIBLE(preccLastItem))
    {
      preccLastItem = preccLastItem->preccPrevItem;
    }

    /*****************************************************************/
    /* Get the bottom of the item to calculate sRBPos                */
    /*****************************************************************/
    right = CnrQueryPtOfItemInNTV(hwndCnr, preccLastItem, &iconSize,
                                   PT_RIGHT);

    /*****************************************************************/
    /* Let's have right margin.                                      */
    /*****************************************************************/
    right += pccinfon->lAveCharWidth;

    /*****************************************************************/
    /* Set the scroll range below the window (the distance from      */
    /* bottom of the client area and bottom of the last item.        */
    /* Minimum is 0.                                                 */
    /*****************************************************************/
    sRBPos = right - (pccinfon->ptlNonIconOrg.x + rctlClient.right);
  }
  else
  {
    sRBPos = 0;
  }

  /*******************************************************************/
  /* Calculate total scroll range in pixels.                         */
  /*******************************************************************/
  sLTPosTemp = max(0, min(sLTPos, sLTPos + sRBPos));
  sLTPosMove = sLTPosTemp - sLTPos;
  pccinfon->ptlNonIconOrg.x += sLTPosMove;
  sLTPos = sLTPosTemp;
  sRBPos = max(0, sRBPos);

  if (sLTPosMove < 0)
  {
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      CnrSetFirstItemInNTV(hwndCnr, pccinfon->preccFirstinFTV,
                           FALSE);
    }
    else
    {
      CnrSetFirstItemInNTV(hwndCnr, pccinfon->preccFirstinFNV,
                           FALSE);
    } /* endif */
  } /* endif */

  /*******************************************************************/
  /* Calculate total scroll range                                    */
  /*******************************************************************/
  sMaxPos = sRBPos + sLTPos;

  /*******************************************************************/
  /* Calculate the minimum unit of scrolling.                        */
  /*******************************************************************/
  pccinfon->sHscrollFactor = CalcScrollFactor(hwndCnr, sMaxPos, FALSE);

  /*******************************************************************/
  /* Calculate the scroll ranges  in the unit.                       */
  /*******************************************************************/
  if (sRBPos % (long)(pccinfon->sHscrollFactor))
  {
    sRBPos = sRBPos / pccinfon->sHscrollFactor + 1;
  }
  else
  {
    sRBPos = sRBPos / pccinfon->sHscrollFactor;
  } /* endif */

  if (sLTPos % pccinfon->sHscrollFactor)
  {
    sLTPos = sLTPos / pccinfon->sHscrollFactor + 1;
  }
  else
  {
    sLTPos = sLTPos / pccinfon->sHscrollFactor ;
  } /* endif */

  /*******************************************************************/
  /* Calculate total scroll range in the unit.                       */
  /*******************************************************************/
  sMaxPos = sRBPos + sLTPos;

  if (sMaxPos)
  {
    /*****************************************************************/
    /* Enable the scroll bar if something outside viewport           */
    /*****************************************************************/
    EnableWindow (hwndHscroll, TRUE);

    /*****************************************************************/
    /* Set scroll bar. Range is 0 to sMaxPos. Current Pos is         */
    /* sLTPos                                                        */
    /*****************************************************************/
    SetScrollRange(hwndHscroll, SB_CTL, 0, sMaxPos, FALSE);
    SetScrollPos  (hwndHscroll, SB_CTL, sLTPos, TRUE);
    pccinfon->sHscrollPos = sLTPos;
  }
  else
  {
    /*****************************************************************/
    /* Disable the scroll bar if nothing outside viewport            */
    /*****************************************************************/
    EnableWindow (hwndHscroll, FALSE);
    pccinfon->sHscrollPos = 0;
  } /* endif */

  pccinfon->sHscrollMax = sMaxPos;


  /*******************************************************************/
  /* Set sHscrollFactor as 1 so that the minimum scrolling unit      */
  /* can be a pixels.                                                */
  /*******************************************************************/
  pccinfon->sVscrollFactor = 1;

  /*******************************************************************/
  /* If the height of the tallest item in the list plus top          */
  /* and bottom margin (lAveCharWidth) is larger than                */
  /* the height of the client area, provide V scroll bar.            */
  /*******************************************************************/
  if ((pccinfon->lMaxHeight + 2 * pccinfon->lAveCharWidth) >
       rctlClient.bottom)
  {
    /*****************************************************************/
    /* Calculate sLTPos and sRBPos.                                  */
    /*****************************************************************/
    sLTPos = pccinfon->sVscrollPos;

    /*****************************************************************/
    /* Get the distance to move to see the bottom of the item        */
    /* with max height. lAveCharWidth is the top margin of the       */
    /* window (the distance between top of the item and top          */
    /* boundary of the client area ).                                */
    /*****************************************************************/
    sRBPos = pccinfon->lMaxHeight + 2 * pccinfon->lAveCharWidth -
             rctlClient.bottom - pccinfon->ptlNonIconOrg.y;

    /*****************************************************************/
    /* Calculate total scroll range in pixels.                       */
    /*****************************************************************/
    sLTPosTemp = max(0, min(sLTPos, sLTPos + sRBPos));
    sLTPosMove = sLTPosTemp - sLTPos;
    pccinfon->ptlNonIconOrg.y += sLTPosMove;
    sLTPos = sLTPosTemp;
    sRBPos = max(0, sRBPos);

    /*****************************************************************/
    /* Calculate total scroll range in the unit.                     */
    /*****************************************************************/
    sMaxPos = sRBPos + sLTPos;

    if (sMaxPos)
    {
      /***************************************************************/
      /* Enable the scroll bar if something outside viewport         */
      /***************************************************************/
      EnableWindow (hwndVscroll, TRUE);

      /***************************************************************/
      /* Set scroll bar. Range is 0 to sMaxPos. Current Pos is       */
      /* sRTPos                                                      */
      /***************************************************************/
      SetScrollRange(hwndVscroll, SB_CTL, 0, sMaxPos, FALSE);
      SetScrollPos  (hwndVscroll, SB_CTL, sLTPos, TRUE);
      pccinfon->sVscrollPos = sLTPos;
    }
    else
    {
      /***************************************************************/
      /* Disable the scroll bar if nothing outside viewport          */
      /***************************************************************/
      EnableWindow (hwndVscroll, FALSE);
      pccinfon->sVscrollPos = 0;
    } /* endif */

    pccinfon->sVscrollMax = sMaxPos;
  }
  else
  {
    pccinfon->ptlNonIconOrg.y = 0;
    pccinfon->sVscrollPos = 0;
    pccinfon->sVscrollMax = 0;
  } /* endif */

  return;
}

/**********************************************************************/
/*                                                                    */
/* FUNCTION NAME:  CnrEnumWindowsProc                                 */
/*                                                                    */
/* USAGES : External                                                  */
/*                                                                    */
/* DESCRIPTIVE NAME:  Container Control Proc for enumerating windows  */
/*                                                                    */
/* FUNCTIONAL DESCRIPTION:                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/* NOTES:                                                             */
/*    FORWARD DECLARATION:                                            */
/*                                                                    */
/* INPUT:  HWND  hwnd   - window handle for the container.            */
/*         DWORD lParam -                                             */
/*                                                                    */
/* OUTPUT:                                                            */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:                                                       */
/*                                                                    */
/* EXIT-ERROR:                                                        */
/*                                                                    */
/* SIDE EFFECTS:                                                      */
/*                                                                    */
/*********************************************************************/
BOOL FAR PASCAL CnrEnumWindowsProc(HWND hwndChild, DWORD lParam)
{
  if (hwndChild)
  {
     PMInvalidateRect (hwndChild, NULL, TRUE);
  }
  return (TRUE);
}
