/* Revision: 13 1.4.1.2 source/ui/cclnt/cnr/cnrs2p2.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS2P2.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS2P2.C - Workplace and icon resizing routines for the */
/*                       container control                            */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* D6386  1/16/96  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/* DAB2   8/01/96  DAB  Defect 8172 - Fixed CM_PAINTBACKGROUND parms  */
/* D7092  8/06/96  LJO  Add support for icon resizing.                */
/**********************************************************************/
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef SEGMENTOR
#define CNRS2P2_C
#include "cnrpragw.h"
#endif

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrUpdateDetailsViewIcons (hwndCnr)

HANDLE  hwndCnr;

{
 LPCNRINFON     pccinfon;
 LPFIELDINFO    pFieldInfo;
 LPRECORDCOREN  pRecord;
 HICON      Icon;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 pFieldInfo = pccinfon->pFirstFI;
 pRecord    = pccinfon->preccItem;

 /*********************************************************************/
 /* For every record currently in the container, process each         */
 /* FieldInfo.  If any of the FieldInfo's are BitMaps or Icons, we    */
 /* insert them in the Irs Metrics.                                   */
 /*********************************************************************/
 while (pRecord)
 {
    while (pFieldInfo)
    {
       if (pFieldInfo->flData & CFA_BITMAPORICON)
       {
          Icon = (HICON)(long)CnrGetColumnData (hwndCnr,
                                   pRecord->preccX,
                                   pFieldInfo);

          if (!(CnrInsertDeleteResizedIconInDV (hwndCnr,
                                             (HANDLE)(long)Icon, TRUE)))
          {
             return (FALSE);
          }
       }

       if (pFieldInfo->flTitle & CFA_BITMAPORICON)
       {
          if (!(CnrInsertDeleteResizedIconInDV (hwndCnr,
                           (HANDLE)(long)pFieldInfo->pTitleData, TRUE)))
          {
             return (FALSE);
          }
       }

       pFieldInfo = NEXTFI(pFieldInfo);
    }

    pFieldInfo = pccinfon->pFirstFI;
    pRecord    = pRecord->preccNextItem;
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrOverRecord                                  */
/*                                                                    */
/*  Descriptive Name:  Determine if activitity occurred over record.  */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:  long CnrOverRecord(                      */
/*                               HANDLE       hwnd                    */
/*                               )                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd    - container control handle                       */
/*                                                                    */
/*  Output:  TRUE  - An error occurred, terminate window creation     */
/*           FALSE - No error, continue creation                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:   FALSE                                              */
/*                                                                    */
/*  Exit-Error:    TRUE                                               */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR CnrOverRecord(HANDLE hwnd, int xPos, int yPos)
{
  LPRECORDCOREN pRecord = NULL;
  LPRECORDCOREN preccTemp = NULL;
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  POINT iconSize;
  RECT rctlText;
  RECT rctlIcon;
  RECT  rctlHScroll;
  RECT  rctlVScroll;
  RECT  rctlViewPort;
  RECT  rowRect;         /* Rectangle for a row when in Details View. */
  POINT adjPoint;
  RECT  rctlRec;
  int   cyBetween;
  RECT  ViewPortRight;
  RECT  rctlLTitle;

  /*******************************************************************/
  /* Get pointer to Container internal data                          */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    return (CnrTRVOverRecord(hwnd, xPos, yPos));
  }

  if (pccinfon->preccItem)
  {
    adjPoint.x = xPos;
    adjPoint.y = yPos;

    /******************************************************************/
    /* Calculate the size of the scroll bars and the viewport.        */
    /******************************************************************/
    CnrQuerySizeRect(hwnd, &iconSize, &rctlLTitle, &rctlViewPort,
                     &rctlVScroll, &rctlHScroll);

    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                           &rctlViewPort);

       PMGetClientRect (GetDlgItem (hwnd, CID_RIGHTDVWND),
                           &ViewPortRight);

       rctlViewPort.right += ViewPortRight.right +
                      GetSystemMetrics ( SM_CXFRAME);

       rctlLTitle.bottom = 0;
    }

    if ( (adjPoint.x < 0 ) || (adjPoint.x > rctlViewPort.right ) ||
         (adjPoint.y > (rctlViewPort.bottom + rctlLTitle.bottom) ||
         (adjPoint.y < rctlLTitle.bottom )))
    {
       return(NULL);
    }

    pRecord = GetFirstItemInView(hwnd);

    if (!pRecord)
    {
       pRecord = pccinfon->preccItem;
    }

    /****************************************************************/
    /* Enumerate all records in the viewport to find the item under */
    /* the point                                                    */
    /****************************************************************/
    while (pRecord)
    {
      if (ISRECORDVISIBLE(pRecord))
      {

        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
           CnrQueryRowRectForDV (hwnd, pRecord, &rowRect);
        }
        else
        {
          if (pccinfo->flWindowAttr & CV_ICON)
          {
            CnrQueryIconTextRect(hwnd, pRecord, &iconSize, FALSE,
                                 &rctlIcon, &rctlText);
            rctlText.top -= (2 * ICN_EMPHSPACE);
            rctlIcon.bottom += (2 * ICN_EMPHSPACE);
          }
          else
          {
            if (pccinfo->cyLineSpacing >= 0)
            {
              cyBetween = pccinfo->cyLineSpacing;
            }
            else
            {
              cyBetween = MINCY_BETWEENITEMS;
            } /* endif */

            /********************************************************/
            /* Get the proper rectangle for the item.               */
            /********************************************************/
            rctlRec.bottom = CnrQueryPtOfItemInNTV( hwnd, pRecord,
                    &iconSize, PT_BOTTOM) - pccinfon->ptlNonIconOrg.y +
                    cyBetween / 2 + (cyBetween & 1) + 1;

            /********************************************************/
            /* If the record is not on the   top  of the column     */
            /* (the  x position of the record is tha same as the    */
            /* one of the prev record) the   top  of the rectangle  */
            /* of the record is half way up   to the prev item,     */
            /* else, it is the   top  of the client area.           */
            /********************************************************/
            preccTemp = GetPrevUnfilteredItem (pRecord);

            if (preccTemp)
            {
              if (preccTemp->ptlNonIconPos.x == pRecord->ptlNonIconPos.x)
              {
                rctlRec.top = CnrQueryPtOfItemInNTV( hwnd, pRecord,
                            &iconSize, PT_TOP) -
                            pccinfon->ptlNonIconOrg.y -
                            (cyBetween / 2) - 1;
              }
              else
              {
                rctlRec.top = rctlLTitle.bottom;
              }
            }
            else
            {
              rctlRec.top = rctlLTitle.bottom;
            }
                rctlRec.right = CnrQueryPtOfItemInNTV(hwnd, pRecord,
                                                   &iconSize, PT_RIGHT) -
                             pccinfon->ptlNonIconOrg.x +
                             2*(pccinfon->lAveCharWidth);

            rctlRec.left = CnrQueryPtOfItemInNTV(hwnd, pRecord,
                                                &iconSize, PT_LEFT) -
                            pccinfon->ptlNonIconOrg.x -
                            pccinfon->lAveCharWidth;


          }
        }

        if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
          {
            /************************************************************/
            /* If icon view with autoposition then stop when out of view*/
            /************************************************************/
            if (rctlIcon.top > (rctlLTitle.bottom + rctlViewPort.bottom))
            {
              return(NULL);
            }
            else
            {
              if (PMPtInRect( &rctlIcon, adjPoint) ||
                  PMPtInRect( &rctlText, adjPoint))
              {
                return (pRecord);
              }
            }
          }
          else
          {
            /************************************************************/
            /* Only check items that are in view to ensure valid        */
            /* rectangle for PMPtInRect                                */
            /************************************************************/
            if ((rctlIcon.top < (rctlLTitle.bottom + rctlViewPort.bottom)) &&
                (rctlText.bottom > rctlLTitle.bottom) &&
                (rctlText.right > 0) &&
                (rctlText.left < rctlViewPort.right ))

            {
              if (PMPtInRect( &rctlIcon, adjPoint) ||
                  PMPtInRect( &rctlText, adjPoint))
              {
                return (pRecord);
              }
            }
          }
        }
        else
        {
          if (pccinfo->flWindowAttr & (CV_TEXT | CV_NAME))
          {
            /********************************************************/
            /* Stop when you reach the first item not in view       */
            /* rectangle for PMPtInRect                             */
            /********************************************************/
            if ( (rctlRec.top > (rctlLTitle.bottom + rctlViewPort.bottom)) ||
                 (rctlRec.left > rctlViewPort.right) )
            {
              return(NULL);
            }
            if (PMPtInRect( &rctlRec, adjPoint) )
            {
              return (pRecord);
            }
          }
          else
          {
            if (pccinfo->flWindowAttr & CV_DETAIL)
            {
                if (pRecord == pccinfon->preccFirstRowInDV)
                {
                  rowRect.top -= pccinfon->lAveCharWidth;
                }

              if (rowRect.top > (rctlLTitle.bottom + rctlViewPort.bottom))
              {
                return(NULL);
              }
              if (PMPtInRect( &rowRect, adjPoint) )
              {
                return (pRecord);
              }
            }
          }
        }
      }
      /*****************************************************************/
      /* Try next record in linked list.                               */
      /*****************************************************************/
      if ( (pccinfo->flWindowAttr & CV_ICON) &&
           !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION) )
      {
        pRecord = pRecord->preccPrevZ;
      }
      else
      {
        pRecord = pRecord->preccNextItem;
      }
    }
      return (pRecord);
  }
  else
  {
    return (NULL);
  }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrTRVOverRecord                               */
/*                                                                    */
/*  Descriptive Name:  Get Record Under Given Point                   */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR  CnrTRVOverRecord(HANDLE hwnd, int xPos, int yPos)
{
  LPRECORDCOREN  preccEnum;
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  POINT         slIcon;
  POINT         slTreeIcon;
  RECT          rctlText;
  RECT          rctlIcon;
  RECT          rctlTreeIcon;
  RECT          rctlHscroll;
  RECT          rctlVscroll;
  RECT          rctlViewport;
  POINT         ptlOver;
  RECT          rctlRec;
  long          cyBetween;
  RECT          rctlLTitle;

  /*******************************************************************/
  /* Get pointer to Container internal data and user defined data.   */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfon->preccItem)
  {
    ptlOver.x = xPos;
    ptlOver.y = yPos;

    /******************************************************************/
    /* Calculate the size of the scroll bars and the viewport.        */
    /******************************************************************/
    CnrQuerySizeRect(hwnd, &slIcon, &rctlLTitle, &rctlViewport,
                     &rctlVscroll, &rctlHscroll);

    CnrTRVCalcTreeIconSize(hwnd, &slTreeIcon);

    /******************************************************************/
    /* If ptlOver is out of the container client area limit,          */
    /* return NULL.                                                   */
    /******************************************************************/
    if ( (ptlOver.x < 0 ) || (ptlOver.x > rctlViewport.right )  ||
         (ptlOver.y > (rctlLTitle.bottom + rctlViewport.bottom) ||
         (ptlOver.y < rctlLTitle.bottom )))
    {
      return(NULL);
    }

    preccEnum = pccinfon->preccFirstinTRV;

    if (!preccEnum)
    {
      preccEnum = pccinfon->preccItem;
      if (preccEnum && !(ISRECORDVISIBLE(preccEnum)))
      {
        TRVGetNextVisualItem (preccEnum);
      }
    }

    if (pccinfo->cyLineSpacing >= 0)
    {
      cyBetween = (long)pccinfo->cyLineSpacing;
    }
    else
    {
      cyBetween = (long)MINCY_BETWEENITEMS;
    }

    while (preccEnum)
    {
      CnrTRVQueryIconTextRect(hwnd, preccEnum, TRUE, slIcon, slTreeIcon,
                              &rctlIcon, &rctlTreeIcon, &rctlText);

      /***************************************************************/
      /* Get the bottom of the rectangle of the item.                */
      /***************************************************************/
        rctlRec.bottom = (CnrTRVQueryPtOfItem( hwnd, rctlIcon,          /*DAB1*/
                                rctlTreeIcon, rctlText, PT_BOTTOM) -
                            pccinfon->ptlNonIconOrg.y +
                            cyBetween / 2 + (cyBetween % 2));

      /***************************************************************/
      /* If the record is not the first visible item,                */
      /* the top of the rectangle of the record is                   */
      /* is half way down to the prev item.                          */
      /* Else, it is the top of the client area.                     */
      /***************************************************************/
      if (TRVGetPrevVisualItem(hwnd,preccEnum))
      {
        rctlRec.top = (CnrTRVQueryPtOfItem( hwnd, rctlIcon,             /*DAB1*/
                                    rctlTreeIcon, rctlText, PT_TOP) -
                          pccinfon->ptlNonIconOrg.y - cyBetween / 2);
      }
      else
      {
        rctlRec.top = rctlLTitle.bottom;
      }

      rctlRec.right = rctlText.right ;

      rctlRec.left = 0;

      if (rctlRec.top > (rctlLTitle.bottom + rctlViewport.bottom))
      {
        return(NULL);
      }

      /***************************************************************/
      /* If ptlOver is over rctlRec(rect of the preccEnum),......    */
      /***************************************************************/
      if (PMPtInRect( &rctlRec, ptlOver) )
      {
        pccinfon->ulFlagSet2 &= ~FL2_ONTREENODE;

        /*************************************************************/
        /* If preccEnum is not a leaf, .....                         */
        /*************************************************************/
        if (TRVhasVisibleChild (preccEnum))
        {
          /***********************************************************/
          /* Check if the ptlOver is over tree node icon(+, - icon)  */
          /***********************************************************/
          if (((pccinfo->flWindowAttr & CV_NAME) &&
               PMPtInRect( &rctlIcon, ptlOver)) ||
              (!(pccinfo->flWindowAttr & CV_NAME) &&
               PMPtInRect( &rctlTreeIcon, ptlOver)))
          {
            pccinfon->ulFlagSet2 |= FL2_ONTREENODE;
          }
        }
        return (preccEnum);
      }
      preccEnum = TRVGetNextVisualItem (preccEnum);
    }
  }
  return (NULL);
}

/*********************************************************************/
/* FUNCTION NAME:  CalcScrollFactor                                  */
/*                                                                   */
/* DESCRIPTIVE NAME: Calculate the scroll factor                     */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculates the scroll      */
/*   factor considering the range of the scrolling.                  */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
int FAR  CalcScrollFactor(hwndCnr, lScrollRange, isV)
HANDLE hwndCnr;
int    lScrollRange;
BOOL   isV;
{
  LPCNRINFON    pccinfon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);

  if (lScrollRange <= 32000)
  {
    if (isV)
    {
      pccinfon->ulFlagSet2 |= FL2_VDYNAMICSCROLL;
    }
    else
    {
      pccinfon->ulFlagSet2 |= FL2_HDYNAMICSCROLL;
    } /* endif */
  }
  else
  {
    if (isV)
    {
      pccinfon->ulFlagSet2 &= ~FL2_VDYNAMICSCROLL;
    }
    else
    {
      pccinfon->ulFlagSet2 &= ~FL2_HDYNAMICSCROLL;
    } /* endif */
  } /* endif */
  if (lScrollRange == 0)
  {
    return (int)1;
  }
  else
  {
    if (lScrollRange % 32000)
    {
      return(lScrollRange / 32000 + 1 );
    }
    else
    {
      return( lScrollRange / 32000 );
    } /* endif */
  } /* endif */

  return 0;
}


/*********************************************************************/
/* FUNCTION NAME:  GetFirstItemInView                                */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*   xtai                                                            */
/*********************************************************************/
LPRECORDCOREN FAR GetFirstItemInView(hwndCnr)
HANDLE   hwndCnr;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  pRecord;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_TREE)
  {
     if (!(pRecord = pccinfon->preccFirstinTRV))
     {
       pRecord = pccinfon->preccFirstinTRV = pccinfon->preccItem;
     }
  }
  else if (pccinfo->flWindowAttr & CV_DETAIL)
  {
     if (!(pRecord = pccinfon->preccFirstRowInDV))
     {
       pRecord = pccinfon->preccFirstRowInDV = pccinfon->preccItem;
     }
  }
  else if (pccinfo->flWindowAttr & CV_NAME)
  {
     if (pccinfo->flWindowAttr & CV_FLOW)
     {
        if (!(pRecord = pccinfon->preccFirstinFNV))
        {
          pRecord = pccinfon->preccFirstinFNV = pccinfon->preccItem;
        }
     }
     else
     {
        if (!(pRecord = pccinfon->preccFirstinNV))
        {
          pRecord = pccinfon->preccFirstinNV = pccinfon->preccItem;
        }
     }
  }
  else if (pccinfo->flWindowAttr & CV_TEXT)
  {
     if (pccinfo->flWindowAttr & CV_FLOW)
     {
        if (!(pRecord = pccinfon->preccFirstinFTV))
        {
          pRecord = pccinfon->preccFirstinFTV = pccinfon->preccItem;
        }
     }
     else
     {
        if (!(pRecord = pccinfon->preccFirstinTV))
        {
          pRecord = pccinfon->preccFirstinTV = pccinfon->preccItem;
        }
     }
  }
  else if ((pccinfo->flWindowAttr & CV_ICON) &&
           (GetWindowLong( hwndCnr, GWL_STYLE ) &
            CCS_AUTOPOSITION))
  {
     if (!(pRecord = pccinfon->preccFirstinAutoPosIV))
     {
       pRecord = pccinfon->preccFirstinAutoPosIV = pccinfon->preccItem;
     }
  }
  else
  {
    if (!(pRecord = pccinfon->preccZTail))
    {
      pRecord = pccinfon->preccZTail = pccinfon->preccItem;
    }
  }

  return (pRecord);
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CnrChangeSizeInIRSMetrics                        */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Change Size In Resized Icon Metrics          */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function changes the size of the bitmaps in the          */
/*     ICONRESIZE metrics.                                           */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrChangeSizeInIRSMetrics(hwnd)
HANDLE        hwnd;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPICONRESIZED  pirsCurrent;
  POINT          slIcon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  CnrCalcIconSize( hwnd, &slIcon );

  pirsCurrent = pccinfon->pirsMetrics;

  while (pirsCurrent)
  {
    if (pirsCurrent->hbmPointer)
    {
      DeleteObject(pirsCurrent->hbmPointer);
    } /* endif */
    if (pirsCurrent->hbmColor)
    {
      DeleteObject(pirsCurrent->hbmColor);
    } /* endif */

    pirsCurrent->slIcon = slIcon;
    IcnResizeIcon(pirsCurrent, hwnd, pccinfo->slBitmapOrIcon);      /* D7092 */
    pirsCurrent = pirsCurrent->pirsNext;
  } /* endwhile */
  return TRUE;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CnrChangeSizeInIRSTreeMetrics                    */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Change Size In Resized Icon Metrics          */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function changes the size of the bitmaps in the          */
/*     ICONRESIZE metrics.                                           */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrChangeSizeInIRSTreeMetrics(hwnd)
HANDLE        hwnd;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPICONRESIZED  pirsCurrent;
  POINT          slTreeIcon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);

  pirsCurrent = pccinfon->pirsTreeMetrics;

  while (pirsCurrent)
  {
    if (pirsCurrent->hbmPointer)
    {
      DeleteObject(pirsCurrent->hbmPointer);
    } /* endif */
    if (pirsCurrent->hbmColor)
    {
      DeleteObject(pirsCurrent->hbmColor);
    } /* endif */

    pirsCurrent->slIcon = slTreeIcon;
    IcnResizeIcon (pirsCurrent, hwnd, pccinfo->slTreeBitmapOrIcon); /* D7092 */
    pirsCurrent = pirsCurrent->pirsNext;
  } /* endwhile */
  return TRUE;
}


/**********************************************************************/
/* FUNCTION NAME:  CnrSetupIconPirsInfo                               */
/*                                                                    */
/* DESCRIPTIVE NAME: Setup icon and pirs information depending on view*/
/*                                                                    */
/* FUNCTIONAL DESCRIPTION:  This function will setup the correct      */
/*                          values of pirsCurrent and hptrFind based  */
/*                          on the fsRegular flag passed in           */
/*                                                                    */
/* NOTES:                                                             */
/*    FORWARD DECLARATION:                                            */
/*                                                                    */
/* INPUT:  HANDLE hwndCnr - handle to the container window            */
/*         LPRECORDCORE precord - pointer to the record structure     */
/*         HBITMAP *hbmDraw - pointer to handle of bitmap to return   */
/*         LPICONRESIZED *pirsDraw - pointer of resized icon to return*/
/*                                                                    */
/* OUTPUT:                                                            */
/*                                                                    */
/* EXIT-NORMAL:                                                       */
/*                                                                    */
/* EXIT-ERROR:                                                        */
/*                                                                    */
/* SIDE EFFECTS:                                                      */
/**********************************************************************/
VOID FAR CnrSetupIconPirsInfo (HANDLE hwndCnr, LPRECORDCORE pRecord,
                                BOOL fsRegular, HICON FAR *hptrCurr,
                                LPICONRESIZED FAR * FAR *pirsCurr)
{
  LPCNRINFON     pccinfon;

  /********************************************************************/
  /* Get internal pointer to container information from window words  */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

  /********************************************************************/
  /* Check if a normal size record is being used                      */
  /********************************************************************/
  if (pRecord->cb == sizeof(RECORDCORE))
  {
    /******************************************************************/
    /* Check if a regular size icon is to be drawn                    */
    /******************************************************************/
    if (fsRegular)
    {
      /****************************************************************/
      /* If looking for the regular size, search in pirsMetrics and ..*/
      /****************************************************************/
      *pirsCurr = &(pccinfon->pirsMetrics);
      if (pRecord->hptrIcon)
      {
        /**************************************************************/
        /* If pRecord has regular size icon, find it in the metrics   */
        /**************************************************************/
        *hptrCurr = pRecord->hptrIcon;
      }
      else
      {
        /**************************************************************/
        /* Otherwise, find the mini icon in the regular size metrics. */
        /**************************************************************/
        *hptrCurr = pRecord->hptrMiniIcon;
      }
    }
    else
    {
      /****************************************************************/
      /* If looking for the mini size, search in pirsMiniMetrics and .*/
      /****************************************************************/
      *pirsCurr = &(pccinfon->pirsMiniMetrics);
      if (pRecord->hptrMiniIcon)
      {
        /**************************************************************/
        /* If pRecord has minisize icon, find it in the mini-metrics  */
        /**************************************************************/
        *hptrCurr = pRecord->hptrMiniIcon;
      }
      else
      {
        /**************************************************************/
        /* Otherwise, find the regular icon in the mini size metrics. */
        /**************************************************************/
        *hptrCurr = pRecord->hptrIcon;
      }
    }
  }
  /********************************************************************/
  /* Otherwise it is a small record so only one choice for icon to use*/
  /********************************************************************/
  else
  {
    *pirsCurr = &(pccinfon->pirsMetrics);
    *hptrCurr = pRecord->hptrIcon;
  }
  return;
}


/*********************************************************************/
/*                                                                   */
/*  Function Name:  FindPirs                                         */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Find LPICONRESIZED for The Record            */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function searchs for the LPICONRESIZED in the pirsMetrics*/
/*     (if fsRecular = TRUE) or in the pirsMiniMetrics (if           */
/*     (fsRegular = FALSE) for the record passed as a parameter (    */
/*     preccObject).                                                 */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
LPICONRESIZED FAR  FindPirs(hwnd, preccObject, fsRegular)
HANDLE        hwnd;
LPRECORDCORE  preccObject;
BOOL          fsRegular;

{
  BOOL           found = FALSE;
  HICON          hptrFind;
  LPICONRESIZED  pirsCurrent;
  LPICONRESIZED FAR *ppirsCurrent;

  /********************************************************************/
  /* Get which pirs to search in and which hptr to search for         */
  /********************************************************************/
  CnrSetupIconPirsInfo (hwnd, preccObject, fsRegular, &hptrFind,
                        &ppirsCurrent);
  pirsCurrent = *ppirsCurrent;

  /*******************************************************************/
  /* Search for the hptrFind in the metrics.                         */
  /*******************************************************************/
  if (hptrFind)
  {
    while (pirsCurrent)
    {
      if (pirsCurrent->hptrOrigin == hptrFind)
      {
        found = TRUE;
        return pirsCurrent;
      }
      pirsCurrent = pirsCurrent->pirsNext;
    }

    if (!found)
    {
      if (fsRegular)
      {
        if (!(CnrInsertDeleteResizedIcon (hwnd,
                                          preccObject,
                                          TRUE,
                                          TRUE)))
        {
           return ((LPICONRESIZED)NULL);
        }
      }
      else
      {
        if (!(CnrInsertDeleteResizedIcon (hwnd,
                                          preccObject,
                                          TRUE,
                                          FALSE)))
        {
           return ((LPICONRESIZED)NULL);
        }
      } /* endif */

      return FindPirs(hwnd, preccObject, fsRegular);
    } /* endif */
  }
  return 0;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  FindPirsForDV                                    */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Find LPICONRESIZED for HICON In DV           */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function searchs for the LPICONRESIZED in the pirsMetrics*/
/*     that has hptrFind as hptrOrigin.                              */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
LPICONRESIZED FAR  FindPirsForDV(hwnd, hptrFind)
HANDLE   hwnd;
HICON    hptrFind;

{
  BOOL           found = FALSE;
  LPCNRINFON     pccinfon;
  LPICONRESIZED  pirsCurrent;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  pirsCurrent = pccinfon->pirsMetrics;

  /*******************************************************************/
  /* Search for the hptrFind in the metrics.                         */
  /*******************************************************************/
  if (hptrFind)
  {
    while (pirsCurrent)
    {
      if (pirsCurrent->hptrOrigin == hptrFind)
      {
        found = TRUE;
        return pirsCurrent;
      } /* endif */
      pirsCurrent = pirsCurrent->pirsNext;
    } /* endwhile */
    if (!found)
    {
      if (!(CnrInsertDeleteResizedIconInDV(hwnd, hptrFind, TRUE)))
      {
         return ((LPICONRESIZED)NULL);
      }

      return(FindPirsForDV(hwnd, hptrFind));
    } /* endif */
  }
  return 0;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CnrOpenIRSMetrics                                */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Open New ICONRESIZED Mertics For Existing    */
/*                      Linked List                                  */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function open new   ICONRESIZED metrics for the existing */
/*     linked list when flAttr changes to use icons instead of       */
/*     bitmaps to draw the icons for the records in the name / icon  */
/*     view.                                                         */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrOpenIRSMetrics(hwnd)
HANDLE        hwnd;

{
  LPCNRINFON     pccinfon;
  LPRECORDCOREN  preccCurrent;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  preccCurrent = pccinfon->preccItem;
  while (preccCurrent)
  {
    /*****************************************************************/
    /* Insert the ICONRESIZED into the metrics for the regular       */
    /* size.                 (in cnrcfunc.c)                         */
    /*****************************************************************/
    if (!(CnrInsertDeleteResizedIcon (hwnd, preccCurrent->preccX,
                                      TRUE, TRUE)))
    {
       return (FALSE);                                                  /*DAB1*/
    }

    /*****************************************************************/
    /* Insert the ICONRESIZED into the metrics for the mini          */
    /* size.                 (in cnrcfunc.c)                         */
    /*****************************************************************/
    if (!(CnrInsertDeleteResizedIcon (hwnd, preccCurrent->preccX,
                                      TRUE, FALSE)))
    {
       return (FALSE);                                                  /*DAB1*/
    }

    preccCurrent = preccCurrent->preccNextItem;
  }

  return TRUE;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CnrInsertDeleteResizedIcon                       */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Add Or Delete Resized Icon From Metrics      */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function search for ICONRESIZED which has the hPointer   */
/*     passed to the function. If the ICONRESIZE exists add 1 to     */
/*     the counter(cUsage) which keeps the info how many records     */
/*     share the specific ICONSIZED for insertion (flInsert = TRUE)  */
/*     and subtract 1 from the counter for deletion (flInsert =      */
/*     FALSE). If no specific ICONRESIZED exists, create one for     */
/*     the hPointer for insertion and do nothing for the deletion    */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*    xtai                                                           */
/*********************************************************************/
BOOL FAR CnrInsertDeleteResizedIcon (hwnd, preccObject,
                                      fsInsert, fsRegular)
HANDLE        hwnd;
LPRECORDCORE preccObject;
BOOL        fsInsert;
BOOL        fsRegular;

{
  HICON          hptrResize;      /* handle of the pointer to be resized   */
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPICONRESIZED  FAR *ppirsFirst; /* ptr to the first IRS in the metrics.  */
  POINT          iconSize;        /* size of icon                          */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Get which pirs to insert into and which hptr to insert           */
  /********************************************************************/
  CnrSetupIconPirsInfo (hwnd, preccObject, fsRegular, &hptrResize,
                        &ppirsFirst);

  /*-----------------------------------------------------------------*/
  /* WinQueryPointerInfo in IcnResizeIcon will rip if no pointer was */
  /* passed in for this record.  If there is no pointer, Insert-     */
  /* DeleteResizedIcon will do a lot of unnecessary work anyway, so  */
  /* we can just bail out here and just pretend we were successful.  */
  /*-----------------------------------------------------------------*/
  if (!hptrResize)
  {
    return (TRUE);
  }

  /********************************************************************/
  /* Get icon size based on type of icon                              */
  /********************************************************************/
  if (fsRegular)
  {
    if ((pccinfo->slBitmapOrIcon.x <= 0) &&
        (pccinfo->slBitmapOrIcon.y <= 0))
    {
      iconSize.x = GetSystemMetrics( SM_CXICON);
      iconSize.y = GetSystemMetrics( SM_CYICON);
    }
    else
    {
      iconSize.x = pccinfo->slBitmapOrIcon.x;
      iconSize.y = pccinfo->slBitmapOrIcon.y;
    } /* endif */
  }
  else
  {
    if ((pccinfo->flWindowAttr & CA_DRAWICON) &&
        !(pccinfo->flWindowAttr & CV_MINI))                         /* D7092 */
    {
      iconSize.x = GetSystemMetrics(SM_CXICON);     /* use icons,    */
      iconSize.y = GetSystemMetrics(SM_CYICON);     /* not bitmaps   */
    }
    else
    {
      iconSize.x = GetSystemMetrics (SM_CXSMICON);
      iconSize.y = GetSystemMetrics (SM_CYSMICON);                  /* D7092 */
    }
  }

  return (InsertDeleteResizedIcon(hwnd, ppirsFirst,
                                  hptrResize, iconSize, fsInsert));
}

/*********************************************************************/
/*                                                                   */
/*  Function Name:  CnrTRVInsDelResizedIcon                          */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Add Or Delete Resized Icon From Metrics      */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function search for ICONRESIZED which has the hPointer   */
/*     passed to the function. If the ICONRESIZE exists add 1 to     */
/*     the counter(cUsage) which keeps the info how many records     */
/*     share the specific ICONSIZED for insertion (flInsert = TRUE)  */
/*     and subtract 1 from the counter for deletion (flInsert =      */
/*     FALSE). If no specific ICONRESIZED exists, create one for     */
/*     the hPointer for insertion and do nothing for the deletion    */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrTRVInsDelResizedIcon(hwnd, preccObj, fsInsert)
HANDLE        hwnd;
LPRECORDCORE preccObj;
BOOL        fsInsert;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  POINT         iconSize;

  if ((preccObj->cb == sizeof(RECORDCORE)) && preccObj->pTreeItemDesc)
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    if (pccinfo->flWindowAttr & CV_MINI)
    {
      iconSize.x = GetSystemMetrics (SM_CXSMICON);
      iconSize.y = GetSystemMetrics (SM_CYSMICON);                  /* D7092 */
    }
    else
    if ((pccinfo->slBitmapOrIcon.x <= 0) &&
        (pccinfo->slBitmapOrIcon.y <= 0))
    {
      iconSize.x = GetSystemMetrics( SM_CXICON);
      iconSize.y = GetSystemMetrics( SM_CYICON);
    }
    else
    {
      iconSize.x = pccinfo->slBitmapOrIcon.x;
      iconSize.y = pccinfo->slBitmapOrIcon.y;
    } /* endif */

    if (!(InsertDeleteResizedIcon(hwnd, &(pccinfon->pirsMetrics),
                                 preccObj->pTreeItemDesc->hptrCollapsed,
                                 iconSize, fsInsert)))
    {
      return FALSE;
    }

    if (!(InsertDeleteResizedIcon(hwnd, &(pccinfon->pirsMetrics),
                                 preccObj->pTreeItemDesc->hptrExpanded,
                                 iconSize, fsInsert)))
    {
      return FALSE;
    }
  } /* endif */
  return TRUE;
}

/*********************************************************************/
/*                                                                   */
/*  Function Name:  CnrTRVInsDelResizedTreeIcon                      */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Add Or Delete Resized Icon From Metrics      */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function search for ICONRESIZED which has the hPointer   */
/*     passed to the function. If the ICONRESIZE exists add 1 to     */
/*     the counter(cUsage) which keeps the info how many records     */
/*     share the specific ICONSIZED for insertion (flInsert = TRUE)  */
/*     and subtract 1 from the counter for deletion (flInsert =      */
/*     FALSE). If no specific ICONRESIZED exists, create one for     */
/*     the hPointer for insertion and do nothing for the deletion    */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrTRVInsDelResizedTreeIcon(hwnd, fsInsert)
HANDLE        hwnd;
BOOL        fsInsert;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  POINT         slTreeIcon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);

  if (!(InsertDeleteResizedIcon(hwnd, &(pccinfon->pirsTreeMetrics),
                               pccinfo->hptrCollapsed,
                               slTreeIcon, fsInsert)))
  {
    return FALSE;
  }

  if (!(InsertDeleteResizedIcon(hwnd, &(pccinfon->pirsTreeMetrics),
                               pccinfo->hptrExpanded,
                               slTreeIcon, fsInsert)))
  {
    return FALSE;
  }
  return TRUE;
}

/*********************************************************************/
/*                                                                   */
/*  Function Name:  CnrInsertDeleteResizedIconInDV                   */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Add Or Delete Resized Icon From Metrics      */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function search for ICONRESIZED which has the hPointer   */
/*     passed to the function. If the ICONRESIZE exists add 1 to     */
/*     the counter(cUsage) which keeps the info how many records     */
/*     share the specific ICONSIZED for insertion (flInsert = TRUE)  */
/*     and subtract 1 from the counter for deletion (flInsert =      */
/*     FALSE). If no specific ICONRESIZED exists, create one for     */
/*     the hPointer for insertion and do nothing for the deletion    */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*     xtai                                                          */
/*********************************************************************/
BOOL FAR CnrInsertDeleteResizedIconInDV(hwnd, hptrObject, fsInsert)
HANDLE      hwnd;
HICON       hptrObject;
BOOL        fsInsert;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  POINT          iconSize;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_MINI)
  {
    iconSize.x = GetSystemMetrics (SM_CXSMICON);
    iconSize.y = GetSystemMetrics (SM_CYSMICON);                    /* D7092 */
  }
  else
  if ((pccinfo->slBitmapOrIcon.x <= 0) &&
      (pccinfo->slBitmapOrIcon.y <= 0))
  {
    iconSize.x = GetSystemMetrics( SM_CXICON);
    iconSize.y = GetSystemMetrics( SM_CYICON);
  }
  else
  {
    iconSize.x = pccinfo->slBitmapOrIcon.x;
    iconSize.y = pccinfo->slBitmapOrIcon.y;
  } /* endif */

  return (InsertDeleteResizedIcon(hwnd, &(pccinfon->pirsMetrics),
                                  hptrObject, iconSize, fsInsert));
}


/* *******************************************************************/
/*                                                                   */
/*  Function Name:  InsertDeleteResizedIcon                          */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Add Or Delete Resized Icon From Metrics      */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function search for ICONRESIZED which has the hPointer   */
/*     passed to the function. If the ICONRESIZE exists add 1 to     */
/*     the counter(cUsage) which keeps the info how many records     */
/*     share the specific ICONSIZED for insertion (flInsert = TRUE)  */
/*     and subtract 1 from the counter for deletion (flInsert =      */
/*     FALSE). If no specific ICONRESIZED exists, create one for     */
/*     the hPointer for insertion and do nothing for the deletion    */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR InsertDeleteResizedIcon(hwnd, ppirsFirst, hptrResize,
                             iconSize, fsInsert)
HANDLE              hwnd;
LPICONRESIZED FAR  *ppirsFirst;
HICON               hptrResize;
POINT               iconSize;
BOOL                fsInsert;


{
  BOOL             found = FALSE;
  LPICONRESIZED    pirsCurrent;
  LPICONRESIZED    pirsPrev;
//LPVOID           pSatPtr;

  pirsCurrent = *ppirsFirst;
  pirsPrev = pirsCurrent;
  while (pirsCurrent)
  {
    if (pirsCurrent->hptrOrigin == hptrResize)
    {
      found = TRUE;
      if (fsInsert)
      {
        pirsCurrent->cUsage++;
      }
      else
      {
        pirsCurrent->cUsage--;
        if (!pirsCurrent->cUsage)
        {
          if (pirsCurrent == *ppirsFirst) {
            *ppirsFirst = pirsCurrent->pirsNext;
          } else {
            pirsPrev->pirsNext = pirsCurrent->pirsNext;
          } /* endif */
          if (pirsCurrent->hbmPointer) {
            DeleteObject(pirsCurrent->hbmPointer);
          } /* endif */
          if (pirsCurrent->hbmColor) {
            DeleteObject(pirsCurrent->hbmColor);
          } /* endif */
//        pSatPtr = (LPVOID)GetWindowLong (hwnd, QWL_STORAGEPTR);
//        StmFreeStorage (pSatPtr, (LPVOID)pirsCurrent);
          GlobalFree( (LPVOID)pirsCurrent );                            /*DAB1*/
        } /* endif */

      } /* endif */
      return TRUE;
    } /* endif */
    pirsPrev = pirsCurrent;
    pirsCurrent = pirsCurrent->pirsNext;

  } /* endwhile */

  if (!found && fsInsert)
  {
//  pSatPtr = (LPVOID)GetWindowLong (hwnd, QWL_STORAGEPTR);
    if (pirsPrev) {
//    if (!(pirsPrev->pirsNext = StmAllocateStorage (pSatPtr,
      if (!(pirsPrev->pirsNext = (LPICONRESIZED)GlobalAlloc( GPTR,      /*DAB1*/
                                         (DWORD)(sizeof(ICONRESIZED)))))
      {
         SETERROR(PMERR_INSUFFICIENT_MEMORY);
         return (FALSE);                                                /*DAB1*/
      }

      (pirsPrev->pirsNext)->pirsNext = NULL;
      (pirsPrev->pirsNext)->hptrOrigin = hptrResize;
      (pirsPrev->pirsNext)->cUsage = 1;
      (pirsPrev->pirsNext)->slIcon = iconSize;
      IcnResizeIcon(pirsPrev->pirsNext, hwnd, iconSize);            /* D7092 */
    } else {
//    if (!(*ppirsFirst = StmAllocateStorage (pSatPtr,
      if (!(*ppirsFirst = (LPICONRESIZED)GlobalAlloc( GPTR,             /*DAB1*/
                                       (DWORD)(sizeof(ICONRESIZED)))))
      {
         SETERROR(PMERR_INSUFFICIENT_MEMORY);
         return (FALSE);                                                /*DAB1*/
      }

      (*ppirsFirst)->pirsNext = NULL;
      (*ppirsFirst)->hptrOrigin = hptrResize;
      (*ppirsFirst)->cUsage = 1;
      (*ppirsFirst)->slIcon = iconSize;
      IcnResizeIcon(*ppirsFirst, hwnd, iconSize);                   /* D7092 */
    } /* endif */
    return TRUE;
  }
  else
  {
    return FALSE;
  } /* endif */

}






/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CnrDeleteAllResizedIcon                          */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Delete All Resized Icon From Metrics         */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function destroys   ICONRESIZED metrics.                 */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrDeleteAllResizedIcon(hwnd)
HANDLE        hwnd;

{
  LPCNRINFON     pccinfon;
  LPICONRESIZED  pirsCurrent;
  LPICONRESIZED  pirsTemp;
//LPVOID         pSatPtr;
  int            i;


  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
//pSatPtr = (LPVOID)GetWindowLong (hwnd, QWL_STORAGEPTR);

  pirsCurrent = pccinfon->pirsMetrics;
  for (i = 0; i < 2 ; i++) {
    while (pirsCurrent)
    {
      if (pirsCurrent->hbmPointer)
      {
        DeleteObject(pirsCurrent->hbmPointer);
      } /* endif */
      if (pirsCurrent->hbmColor)
      {
        DeleteObject(pirsCurrent->hbmColor);
      } /* endif */
      pirsCurrent = pirsCurrent->pirsNext;
    } /* endwhile */

    if (!i)
    {
      pirsCurrent = pccinfon->pirsMetrics;
    }
    else
    {
      pirsCurrent = pccinfon->pirsMiniMetrics;
    } /* endif */

    while (pirsCurrent)
    {
       pirsTemp = pirsCurrent->pirsNext;
//     StmFreeStorage (pSatPtr, (LPVOID)pirsCurrent);
       GlobalFree( (LPVOID)pirsCurrent );                               /*DAB1*/
       pirsCurrent = pirsTemp;
    }

    pirsCurrent = pccinfon->pirsMiniMetrics;
  } /* endfor */
  pccinfon->pirsMetrics = NULL;
  pccinfon->pirsMiniMetrics = NULL;
  return TRUE;
}

/*********************************************************************/
/* FUNCTION NAME:  CnrEraseRect                                      */
/*                                                                   */
/* DESCRIPTIVE NAME: Erase Rectangle                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function erases the rectangle       */
/*                          passed.                                  */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         HDC  hps - handle of the presentation space               */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/

VOID FAR  CnrEraseRect(hwndCnr, hps, rctlObj)
HANDLE      hwndCnr;
HDC         hps;
RECT        rctlObj;
{
  OWNERBACKGROUND ownerBack;
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  ownerBack.hwnd = hwndCnr;
  ownerBack.hps = hps;
  ownerBack.rclBackground = rctlObj;
  ownerBack.rclBackground.bottom++;
  ownerBack.rclBackground.right++;
  ownerBack.idWindow = getWindowId(hwndCnr);                            /*DAB1*/

  if (!(pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND) ||
      !(SENDMESSAGE (hwndCnr, CM_PAINTBACKGROUND,                       /*DAB1*/
                    (DWORD)((LPOWNERBACKGROUND)&ownerBack), 0 )))       /*DAB2*/
  {
    CnrPaintBackground (hwndCnr, hps, &rctlObj);
  }
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrProcessIconsForDV (hwndCnr, pRecord, pFieldInfo,
                               Insert, FromRecordInsert)

HANDLE          hwndCnr;
LPRECORDCOREN   pRecord;
LPFIELDINFO     pFieldInfo;
BOOL            Insert;
BOOL            FromRecordInsert;

{
 LPCNRINFON   pccinfon;
 HICON        Icon;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Check to see if this routine was called because of a record being */
 /* inserted or deleted.  We use this algorithm which will go through */
 /* all of the existing FieldInfo structures and store the Icons that */
 /* each of them point to in the record.                              */
 /*********************************************************************/
 if (FromRecordInsert)
 {
    pFieldInfo = pccinfon->pFirstFI;

    if (pRecord)
    {
       while (pFieldInfo)
       {
          if (pFieldInfo->flData & CFA_BITMAPORICON)
          {
             Icon = (HICON)(long)CnrGetColumnData (hwndCnr,
                                      pRecord->preccX,
                                      pFieldInfo);

             /*********************************************************/
             /* Make sure this guy is not a bitmap.  If it is, return */
             /* TRUE so we will continue to process without a rip.    */
             /*********************************************************/
             if (!(CnrInsertDeleteResizedIconInDV (hwndCnr,
                                                   (HANDLE)(long)Icon,
                                                   Insert)))
             {
                return (FALSE);
             }

             /*********************************************************/
             /* If we are removing a record, delete the Icon for the  */
             /* Title if it existed.                                  */
             /*********************************************************/
             if (!Insert)
             {
                if (pFieldInfo->flTitle & CFA_BITMAPORICON)
                {
                   /***************************************************/
                   /* Make sure this guy is not a bitmap.  If it is,  */
                   /* return TRUE so we will continue to process      */
                   /* without a rip.                                  */
                   /***************************************************/
                   if (!(CnrInsertDeleteResizedIconInDV (hwndCnr,
                                       (HANDLE)(long)pFieldInfo->pTitleData,
                                        FALSE)))
                   {
                      return (FALSE);
                   }
                }
             }
          }

          pFieldInfo = NEXTFI(pFieldInfo);
       }
    }
 }
 else
 /*********************************************************************/
 /* Otherwise we were called when we inserted a specific FieldInfo.   */
 /* Check to see if this FieldInfo (column) contains an Icon.  If it  */
 /* is, we have to insert the Icon for each record in the container   */
 /* into the IRS Metrics.  We only delete an Icon from the IRS        */
 /* Metrics when the record is deleted, not when a FieldInfo is       */
 /* deleted.                                                          */
 /*********************************************************************/
 {
    if (pFieldInfo)
    {
       if (pFieldInfo->flData & CFA_BITMAPORICON)
       {
          pRecord = pccinfon->preccItem;

          /************************************************************/
          /* Loop through all of the records inserting the Icon into  */
          /* the IRS Metrics.                                         */
          /************************************************************/
          while (pRecord)
          {
             Icon = (HICON)(long)CnrGetColumnData (hwndCnr,
                                      pRecord->preccX,
                                      pFieldInfo);

             /*********************************************************/
             /* Make sure this guy is not a bitmap.  If it is, return */
             /* TRUE so we will continue to process without a rip.    */
             /*********************************************************/
             if (!(CnrInsertDeleteResizedIconInDV (hwndCnr,
                                                   (HANDLE)(long)Icon,
                                                   TRUE)))
             {
                return (FALSE);
             }

             pRecord = pRecord->preccNextItem;
          }
       }
    }
    /******************************************************************/
    /* Now check to see if the column Title is to contain an Icon.    */
    /* If it is, we need to store its Icon in the IRS Metrics too.    */
    /******************************************************************/
    if (pFieldInfo->flTitle & CFA_BITMAPORICON)
    {
       /***************************************************************/
       /* Make sure this guy is not a bitmap.  If it is, return       */
       /* TRUE so we will continue to process without a rip.          */
       /***************************************************************/
       if (!(CnrInsertDeleteResizedIconInDV (hwndCnr,
                                       (HANDLE)(long)pFieldInfo->pTitleData,
                                       TRUE)))
       {
          return (FALSE);
       }
    }
 }

 return (TRUE);
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:     CnrSizeCnrTitle                               */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Set Cnr Title Window                         */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function updates the size of the Cnr title window        */
/*     writes the title in the center of the window                  */
/*                                                                   */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*           hps     - device context handle                         */
/*           rectCnr - container window handle                       */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSizeCnrTitle(hwnd, hps, rectCnr)
HANDLE  hwnd;
HDC     hps;
RECT    rectCnr;

{
  HANDLE        hwndTitle;         /* handle of the title window */
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  SIZEL         slTitle;           /* size of the title string */
  int           right;             /* right of the title window */
  int           top;               /* top of the title window */
                                   /* in the Cnr window */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get handle of the title window                                  */
  /*******************************************************************/
  hwndTitle = GetDlgItem(hwnd, CID_CNRTITLEWND);

  if (!(pccinfon->ulFlagSet2 & FL2_TITLESIZECALC))
  {
    CnrUpdateTitleSize(hwnd, hps);
    pccinfon->ulFlagSet2 |= FL2_TITLESIZECALC;
  }

  if (!(pccinfo->flWindowAttr & CA_CONTAINERTITLE))
  {
    /*****************************************************************/
    /* if container title window is not necessary, size the title    */
    /* window so that cx = 0, cy = 0.                                */
    /*****************************************************************/
    slTitle.cx = 0;                                                     /*DAB1*/
    slTitle.cy = 0;                                                     /*DAB1*/
    top = 0;
    right = 0;
  }
  else
  {
    /*****************************************************************/
    /* We need margin on the top and the bottom. (lAveCharWidth)     */
    /*****************************************************************/
    slTitle.cy = pccinfon->slCnrTitle.cy + 2 * pccinfon->lAveCharWidth; /*DAB1*/

    /*****************************************************************/
    /* The bottom of the title window is below the top of the        */
    /* Cnr window by cy of the title size but not below the          */
    /* bottom of Cnr window.                                         */
    /*****************************************************************/
    top = rectCnr.top;
    right = rectCnr.right;
  }

  /*******************************************************************/
  /* Set the title window on the top of the Cnr window.              */
  /*******************************************************************/
#ifdef IC_WU
     // Wind/U does not like zero for width or height in MoveWindow()
     SetWindowPos( GetDlgItem(hwnd, CID_CNRTITLEWND),
                 NULL,
                 rectCnr.left, top, right,
                 min(rectCnr.bottom, slTitle.cy),
                 SWP_NOZORDER);                                /*RKK1*/
#else
     MoveWindow (GetDlgItem(hwnd, CID_CNRTITLEWND),
                 rectCnr.left, top, right,
                 min(rectCnr.bottom, slTitle.cy),                       /*DAB1*/
                 TRUE);
#endif

  /*******************************************************************/
  /* Set FL_SETCNRTITLE on                                           */
  /*******************************************************************/
  pccinfon->ulFlagSet |= FL_SETCNRTITLE;

}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:     CnrUpdateTitleSize              NEW FUNCTION  */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Calculate And Set Title Size In PCCINFON     */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function updates the size of the Cnr title.              */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR   CnrUpdateTitleSize(hwnd, hDC)
HWND    hwnd;
HDC     hDC;
{
  HWND          hwndTitle;         /* handle of the title window */
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  POINT         iconSize;          /* size of the icon */
                                   /* in the Cnr window */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get handle of the title window                                  */
  /*******************************************************************/
  hwndTitle = GetDlgItem(hwnd, CID_CNRTITLEWND);

  if (!(pccinfo->flWindowAttr & CA_CONTAINERTITLE))
  {
    /*****************************************************************/
    /* if container title window is not necessary, size the title    */
    /* window so that cx = 0, cy = 0.                                */
    /*****************************************************************/
    pccinfon->slCnrTitle.cx = 0;                                        /*DAB1*/
    pccinfon->slCnrTitle.cy = 0;                                        /*DAB1*/
  }
  else
  {
    if ((!pccinfo->pszCnrTitle) || (*pccinfo->pszCnrTitle == '\0'))
    {
      /*****************************************************************/
      /* No string, no window.                                         */
      /*****************************************************************/
      pccinfon->slCnrTitle.cx = 0;                                      /*DAB1*/
      pccinfon->slCnrTitle.cy = 0;                                      /*DAB1*/
    } else {
      /*****************************************************************/
      /* No icon exists in the title. So, initialize as (0,0)          */
      /*****************************************************************/
      iconSize.x = 0;
      iconSize.y = 0;

      /*****************************************************************/
      /* Calculate text size. (cnriconc.c)                             */
      /* No icon exists in the title. So, we use IS_SIDETEXT to        */
      /* calculate text size.                                          */
      /*****************************************************************/
      IcnCalcTextSize(hDC, iconSize, pccinfon,
                      pccinfo->pszCnrTitle, IS_SIDETEXT,
                      &pccinfon->slCnrTitle);

    }
  }
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrUpdateWorkplace                                */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Calculate Workplace Rectangle        */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function updates    the rectangle   */
/*   of the workplace in the icon view  for addition(flInsert =      */
/*   TRUE) or deletion (flInsert = FALSE) of an item (preccObject)   */
/*   or get new workplace (preccObject = NULL, flInsert = FALSE)     */
/*   disregarding the previous workplace.                            */
/*        NOTE: you can update workplace for addition or deletion    */
/*              in any view. But you can not use this function       */
/*              to get new workplace in other than icon view.        */
/*              Instead you can set FL_SETWORKPLACE off so that      */
/*              when view changes to the icon view, this function    */
/*              is called to get new workplace.                      */
/*              This is done to save icon text size calculation of   */
/*              all items to get workplace while in the other view.  */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwnd - handle of the cnr window                    */
/*         LPRECORDCORE preccObject - pointer to the record to be     */
/*              added or deleted.                                    */
/*         BOOL flInsert - flag for insertion.(TRUE : insert,        */
/*              FALSE delete or new workplace)                       */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrUpdateWorkplace(hwnd, hps, preccnObject, flInsert)
HANDLE         hwnd;
HDC            hps;
LPRECORDCOREN  preccnObject;
BOOL           flInsert;
{

  BOOL        ischanged = FALSE;        /* flag for the change of workplace */
  LPCNRINFON  pccinfon;                 /* Internal CNR control data  */
  LPCNRINFO   pccinfo;
  RECT        rctlIcon;                 /* rectangle of the icon */
  RECT        rctlText;                 /* rectangle of the icon text */
  POINT       iconSize;                 /* size of an icon */
  SIZEL       textSize;                 /* size of the icon text */

  pccinfon = PCCINFONOF( hwnd );
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get icon size                                                   */
  /*******************************************************************/
  CnrCalcIconSize(hwnd, &iconSize);
  if (preccnObject)
  {
    if (pccinfo->flWindowAttr & CV_ICON)
    {
      /***************************************************************/
      /* If in icon view, text size of the icon text is calculated   */
      /* already.                                                    */
      /***************************************************************/
      textSize = preccnObject->slTextSize;
    }
    else
    {
      /***************************************************************/
      /* Calculate size of icon text                                 */
      /***************************************************************/
      IcnCalcTextSize (hps, iconSize, pccinfon,
                       preccnObject->preccX->pszIcon,
                       IS_BENEATHTEXT, &textSize);
    } /* endif */

    /*****************************************************************/
    /* Calculate icon rectl which is relative to the workplace       */
    /* origin. (in cnrcfunc.c )                                      */
    /*****************************************************************/
    GetIconTextRectInWorkplace(hwnd, preccnObject, &iconSize,
                               &textSize, &rctlIcon, &rctlText);
    if (flInsert) {
      /***************************************************************/
      /* Update for insertion of an item.                            */
      /*         (in cnrcfunc.c )                                    */
      /***************************************************************/
      ischanged = UpdateWorkplaceForInsert(hwnd, &rctlIcon, &rctlText);
    } else {
      /***************************************************************/
      /* Update for delrtion  of an item.                            */
      /*         (in cnrcfunc.c )                                    */
      /***************************************************************/
      ischanged = UpdateWorkplaceForDelete(hwnd,
                                       &(pccinfon->rctlWorkplaceItems),
                                       &rctlIcon, &rctlText);
    } /* endif */

  } else {
    /*****************************************************************/
    /* Get the new workplace.                                        */
    /*         (in cnrcfunc.c )                                      */
    /*****************************************************************/
    ischanged = GetNewWorkplace(hwnd);
  } /* endif */
  return ischanged;

}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  GetIconTextRectInWorkplace                        */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Calculate Icon Text Rectangles in Icon View    */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculates the             */
/*   rectangles of the icon and icon text which is relative to       */
/*   the origin of the workplace.                                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwnd - handle of the cnr window                     */
/*         LPRECORDCORE preccObject - pointer to the record to be     */
/*              added or deleted.                                    */
/*         PPOINT piconSize - pointer to the size of an icon         */
/*         PPOINT ptextSize - pointer to the size of an icon text    */
/*                                                                   */
/* OUTPUT: LPRECT  prctlIcon - pointer to the rectangle of icon      */
/*              to be compared with prectlWorkplace                  */
/*         LPRECT  prctlIcon - pointer to the rectangle of text      */
/*              to be compared with prectlWorkplace                  */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR GetIconTextRectInWorkplace (hwnd, preccnObject, piconSize,
                                     ptextSize, prctlIcon, prctlText)

HWND           hwnd;                                                    /*DAB1*/
LPRECORDCOREN  preccnObject;
LPPOINT        piconSize;
PSIZEL         ptextSize;                                               /*DAB1*/
LPRECT         prctlIcon;
LPRECT         prctlText;
{

  LPCNRINFON  pccinfon;                 /* Internal CNR control data  */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

  /*******************************************************************/
  /* Calculate the icon rectangle                                    */
  /*******************************************************************/
  prctlIcon->left  = preccnObject->ptlIcon.x - ICN_EMPHSPACE;
  prctlIcon->top   = preccnObject->ptlIcon.y - ICN_EMPHSPACE;
  prctlIcon->right = preccnObject->ptlIcon.x + piconSize->x +
                     ICN_EMPHSPACE;
  prctlIcon->bottom = preccnObject->ptlIcon.y + piconSize->y +
                      ICN_EMPHSPACE;


  /*******************************************************************/
  /* Calculate the icon text rectangle.                              */
  /*******************************************************************/
  prctlText->left = preccnObject->ptlIcon.x  -
                    ((ptextSize->cx - piconSize->x +                    /*DAB1*/
                                TXT_SIDEMARGIN) / 2);
  prctlText->right = (prctlText->left + ptextSize->cx);                 /*DAB1*/
  prctlText->top = preccnObject->ptlIcon.y +
                   piconSize->y +
                   pccinfon->lExternalLeading +
                   ICN_EMPHSPACE + TXT_TOPMARGIN;
  prctlText->bottom = (prctlText->top + ptextSize->cy +                 /*DAB1*/
                      TXT_BTMMARGIN);

  return;
}

/*********************************************************************/
/* FUNCTION NAME:  GetMinInvalidateRect                              */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Invalidate Before First In View                */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  Checks to see if the record passed is    */
/*      before the first item in the current view.                   */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
VOID FAR GetMinInvalidateRect(HANDLE hwnd, LPRECT prctlClient)

{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT       rctlHScroll;
  RECT       rctlTitle;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwnd, NULL, &rctlTitle, prctlClient, NULL,
                   &rctlHScroll);

  if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
      (pccinfo->flWindowAttr & CV_FLOW) )
  {
    prctlClient->left = prctlClient->right - 2;
  }
  else
  {
    prctlClient->bottom = rctlTitle.bottom + 2;
  }

  return ;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  GetNewWorkplace                                   */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get New Workplace Considering All Items        */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculates the rectangle   */
/*   of the workplace in the icon view  considering all items        */
/*   in the list.                                                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwnd - handle of the Cnr window                    */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR GetNewWorkplace(HANDLE hwnd)
{
  BOOL          ischanged = FALSE; /* flag for the change */
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  preccnCurrent;     /* pointer to the current record */
  RECT          rctlViewport;      /* rectangle of the viewport */
  RECT          rctlIcon;
  RECT          rctlText;
  RECT          rctlTitle;
  POINT         iconSize;
  int           cyHscroll;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Calculate the rectangles of the viewport and H scroll bar       */
  /*******************************************************************/
  CnrQuerySizeRect(hwnd, &iconSize, &rctlTitle, &rctlViewport,
                   NULL, NULL);
  cyHscroll = GetSystemMetrics( SM_CYHSCROLL);

  /*******************************************************************/
  /* Initialize the workplace rectangle.                             */
  /* We set bottom as the top of the viewport so that even though   */
  /* we resize the window the V scroll bar is not activated if       */
  /* no item is in the list                                          */
  /*******************************************************************/
  pccinfon->rctlWorkplace.left   = rctlViewport.left +
                                   pccinfo->ptlOrigin.x;
  pccinfon->rctlWorkplace.right  = rctlViewport.left +
                                   pccinfo->ptlOrigin.x;
  pccinfon->rctlWorkplace.top    = pccinfo->ptlOrigin.y + rctlTitle.bottom;
  pccinfon->rctlWorkplace.bottom = pccinfon->rctlWorkplace.top;

  pccinfon->rctlWorkplaceItems.left   = rctlViewport.left +
                                        pccinfo->ptlOrigin.x;
  pccinfon->rctlWorkplaceItems.right  = rctlViewport.left +
                                        pccinfo->ptlOrigin.x;
  pccinfon->rctlWorkplaceItems.top    = pccinfon->rctlWorkplace.top;
  pccinfon->rctlWorkplaceItems.bottom = pccinfon->rctlWorkplace.bottom;

  if (!(pccinfon->preccItem))
  {
    return TRUE;
  } /* endif */

  /*******************************************************************/
  /* Set the current item as the first item in the list.             */
  /*******************************************************************/
  preccnCurrent = pccinfon->preccItem;

  while (preccnCurrent)
  {
    if (ISRECORDVISIBLE(preccnCurrent))
    {
      /***************************************************************/
      /* Calculate icon rectl which is relative to the workplace     */
      /* origin. (in cnrcfunc.c)                                     */
      /***************************************************************/
      GetIconTextRectInWorkplace (hwnd, preccnCurrent,
                                  &iconSize,
                                  &(preccnCurrent->slTextSize),
                                  &rctlIcon, &rctlText);

      /***************************************************************/
      /* Update the workplace for the current item.                  */
      /*         (in cnrcfunc.c)                                     */
      /***************************************************************/
      ischanged = UpdateWorkplaceForInsert(hwnd, &rctlIcon, &rctlText);
    } /* endif */
    preccnCurrent = preccnCurrent->preccNextItem;

  } /* endwhile */
  return ischanged;
}


/* *******************************************************************/
/*                                                                   */
/*  Function Name:  HeapSort              New Function               */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Sort Records Using Heap  Sort Algorithm      */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function sorts the records in a array by                 */
/*     using heap  sort algorithm.                                   */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR HeapSort(ptrArray, n, mp1, mp2)
PALPRECORDCOREN  ptrArray;
long             n;
DWORD            mp1;
DWORD            mp2;

{
  long           i;
  LPRECORDCOREN  preccnMax;

  /*******************************************************************/
  /* Construct Heap.                                                 */
  /*******************************************************************/
  for (i = (n / 2); i >= 1 ; i--)
  {
    FixHeap(ptrArray, i, (*ptrArray)[i - 1], n, mp1, mp2);
  } /* endfor */

  /*******************************************************************/
  /* Repeatedly remove the key at the root and rearrange the heap.   */
  /*******************************************************************/
  for (i = n ; i >= 2 ; i--)
  {
    preccnMax = (*ptrArray)[0];
    FixHeap(ptrArray, 1, (*ptrArray)[i - 1], i - 1, mp1, mp2);
    (*ptrArray)[i - 1] = preccnMax;
  } /* endfor */

}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  FixHeap                      New Function        */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/*  Descriptive Name:   Rearrange Heap                               */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function properly rearranges the heap with keys          */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
#if defined(__IBMC__) || defined(__IBMCPP__)
typedef int (* CALLBACK PFNCOMP)(LPRECORDCORE, LPRECORDCORE, LPVOID);
#else
typedef int (FAR PASCAL *PFNCOMP)(LPRECORDCORE, LPRECORDCORE, LPVOID);
#endif

VOID FAR FixHeap(ptrArray, lRoot, preccnKey, lBound, mp1, mp2)
PALPRECORDCOREN  ptrArray;
long             lRoot;
LPRECORDCOREN    preccnKey;
long             lBound;
DWORD            mp1;
DWORD            mp2;

{
  PFNCOMP pfnComp;
  LPVOID  pStorage;
  long    lVacant;
  long    lLargerChild;

  pfnComp = (PFNCOMP)mp1;

  pStorage = (LPVOID)mp2;

  lVacant = lRoot;
  while (2 * lVacant <= lBound)
  {
    lLargerChild = 2 * lVacant;

    if ((2 * lVacant < lBound) &&
        (((*pfnComp) (((*ptrArray)[2 * lVacant])->preccX,
                      ((*ptrArray)[2 * lVacant - 1])->preccX,
                      pStorage)) > 0))
    {
      lLargerChild = 2 * lVacant + 1;
    }

    if (((*pfnComp) (preccnKey->preccX,
                     ((*ptrArray)[lLargerChild - 1])->preccX,
                     pStorage)) < 0)
    {
      (*ptrArray)[lVacant - 1] = (*ptrArray)[lLargerChild - 1];
      lVacant = lLargerChild;
    }
    else break;
  } /* endwhile */
  (*ptrArray)[lVacant - 1] = preccnKey;
  return;
}


/* *******************************************************************/
/*                                                                   */
/*  Function Name:     CnrUpdateMaxHeightForDel                      */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Get Tallest Height of Item in the List       */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function provides the the height of the tallest item     */
/*     before deleting or invalidating the current tallest item.     */
/*                                                                   */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:  long UpdateMaxHeightForDel(             */
/*                               HANDLE       hwnd                   */
/*                               LPRECORDCORE preccInvalidated       */
/*                               )                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*           preccInvalidated - LPRECORDCORE to be deleted or        */
/*                              invalidated                          */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/

VOID FAR  UpdateMaxHeightForDel(hwnd, preccInvalidate)
HANDLE         hwnd;
LPRECORDCOREN preccInvalidate;
{
  LPRECORDCOREN preccSearch;
  LPCNRINFON    pccinfon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  if (preccInvalidate)
  {
    /*****************************************************************/
    /* If cy of the item to be invalidated is lMaxHeight             */
    /* we need a loop to find the height of the second               */
    /* tallest item.                                                 */
    /*****************************************************************/
    if (pccinfon->lMaxHeight = preccInvalidate->slTextSize.cy)          /*DAB1*/
    {
      pccinfon->lMaxHeight = 0;
      preccSearch = pccinfon->preccItem;
      while (preccSearch)
      {
        if (ISRECORDVISIBLE(preccSearch) &&
            (pccinfon->lMaxHeight < preccSearch->slTextSize.cy) &&      /*DAB1*/
             (preccSearch != preccInvalidate))
        {
          pccinfon->lMaxHeight = preccSearch->slTextSize.cy;            /*DAB1*/
        }

        preccSearch = preccSearch->preccNextItem;
      }
    }
  }
  else
  {
    pccinfon->lMaxHeight = 0;
  }

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  UpdateWorkplaceForInsert                          */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Update Workplace for Insertion                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function updates    the rectangle   */
/*   of the workplace in the icon view  for the possible change      */
/*   by adding one item. If workplace changed, return TRUE.          */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  LPRECT prctlWorkplace - pointer to the rectangle of the   */
/*             workplace of the icon view                            */
/*         LPRECT  prctlIcon - pointer to the rectangle of icon      */
/*              to be compared with prectlWorkplace                  */
/*         LPRECT  prctlIcon - pointer to the rectangle of text      */
/*              to be compared with prectlWorkplace                  */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR UpdateWorkplaceForInsert(hwnd, prctlIcon, prctlText)
HANDLE hwnd;
LPRECT prctlIcon;
LPRECT prctlText;
{
  BOOL ischanged = FALSE;
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  POINT     ptlOldWorkplace;
  int       cyHscroll;
  RECT      rctlClient;
  RECT      rctlTitle;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  cyHscroll = GetSystemMetrics( SM_CYHSCROLL);
  CnrQuerySizeRect(hwnd, NULL, &rctlTitle, &rctlClient, NULL, NULL);

  ptlOldWorkplace.x = pccinfon->rctlWorkplace.left;
  ptlOldWorkplace.y = pccinfon->rctlWorkplace.top;

  if (pccinfon->rctlWorkplace.left == pccinfon->rctlWorkplace.right)
  {
    pccinfon->rctlWorkplaceItems.left = min(prctlText->left,
                                             prctlIcon->left) -
                                         pccinfon->lAveCharWidth;
    pccinfon->rctlWorkplaceItems.right = max(prctlText->right,
                                              prctlIcon->right) +
                                              pccinfon->lAveCharWidth;
    pccinfon->rctlWorkplaceItems.top = prctlIcon->top -
                                       pccinfon->lAveCharWidth;
    pccinfon->rctlWorkplaceItems.bottom = prctlText->bottom +
                                          pccinfon->lAveCharWidth;

    ischanged = TRUE;
  }
  else
  {
    /*******************************************************************/
    /* Check the left of the item and the left of the workplace.       */
    /* The min of the cx of the text rect is cx of the icon size,      */
    /* so, we don't need to consider the case when the cx of the       */
    /* text rect is less than cx of the icon rect.                     */
    /*******************************************************************/
    if (pccinfon->rctlWorkplaceItems.left >
        (min(prctlText->left, prctlIcon->left) -
         pccinfon->lAveCharWidth))
    {
      pccinfon->rctlWorkplaceItems.left = min(prctlText->left,
                                               prctlIcon->left) -
                                           pccinfon->lAveCharWidth;
    } /* endif */

    /*******************************************************************/
    /* Check the right of the item and the left of the workplace.      */
    /* The min textRect.cx is iconRect.cx which doesn't include        */
    /* sel emph space. So, for the cace we have min textRect.cx        */
    /* we add ICN_SELSPACE                                             */
    /*******************************************************************/
    if (pccinfon->rctlWorkplaceItems.right <
        (max(prctlText->right, prctlIcon->right) +
         pccinfon->lAveCharWidth))
    {
      pccinfon->rctlWorkplaceItems.right = max(prctlText->right,
                                                prctlIcon->right) +
                                            pccinfon->lAveCharWidth;

      ischanged = TRUE;
    } /* endif */

    /*******************************************************************/
    /* Check the top of the item and the left of the workplace.        */
    /*******************************************************************/
    if (pccinfon->rctlWorkplaceItems.top >
        (prctlIcon->top - pccinfon->lAveCharWidth))
    {
      pccinfon->rctlWorkplaceItems.top = prctlIcon->top -
                                         pccinfon->lAveCharWidth;
      ischanged = TRUE;
    } /* endif */

    /*******************************************************************/
    /* Check the bottom of the item and the left of the workplace.     */
    /*******************************************************************/
    if (pccinfon->rctlWorkplaceItems.bottom <
        (prctlText->bottom + pccinfon->lAveCharWidth))
    {
      pccinfon->rctlWorkplaceItems.bottom = prctlText->bottom +
                                            pccinfon->lAveCharWidth;
    } /* endif */
  }

  pccinfon->rctlWorkplace.right = pccinfon->rctlWorkplaceItems.right;
  pccinfon->rctlWorkplace.top = min(pccinfon->rctlWorkplaceItems.top,
                                    (pccinfo->ptlOrigin.y +
                                     rctlTitle.bottom));

  pccinfon->rctlWorkplace.left =
                              min(pccinfon->rctlWorkplaceItems.left,
                                  pccinfo->ptlOrigin.x);

  pccinfon->rctlWorkplace.bottom =
                              max(pccinfon->rctlWorkplaceItems.bottom,
                                  (pccinfo->ptlOrigin.y + rctlClient.bottom +
                                  rctlTitle.bottom));

  if ((ptlOldWorkplace.x != pccinfon->rctlWorkplace.left) ||
      (ptlOldWorkplace.y != pccinfon->rctlWorkplace.top))
  {
    ischanged = TRUE;
  } /* endif */

  return ischanged;

}
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  UpdateWorkplaceForDelete                          */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Calculate Workplace Rectangle        */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function updates    the rectangle   */
/*   of the workplace in the icon view  for the possible change      */
/*   by deleting one item. If workplace changed, return TRUE.        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  LPRECT prctlWorkplace - pointer to the rectangle of the   */
/*             workplace of the icon view                            */
/*         LPRECT  prctlIcon - pointer to the rectangle of icon      */
/*              to be compared with prectlWorkplace                  */
/*         LPRECT  prctlIcon - pointer to the rectangle of text      */
/*              to be compared with prectlWorkplace                  */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR UpdateWorkplaceForDelete(hwnd, prctlWorkplaceItems,
                                   prctlIcon, prctlText)
HANDLE   hwnd;
LPRECT prctlWorkplaceItems;
LPRECT prctlIcon;
LPRECT prctlText;
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  BOOL ischanged = FALSE;

  /*******************************************************************/
  /* Initialize pointers to control data                             */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Check if at least one of the boundaries of the item to          */
  /* be deleted is the boundary of the workplace. If so, get         */
  /* new workplace by calling GetNewWorkplace (in cnrcfunc.c)        */
  /* and set the flag that idicates the change of the workplace      */
  /* TRUE.  (GetNewWorkplace in cnrcfunc.c)                          */
  /*******************************************************************/
  if ((prctlWorkplaceItems->left >=
        (prctlText->left - pccinfon->lAveCharWidth)) ||
      (prctlWorkplaceItems->right <=
        (prctlText->right + pccinfon->lAveCharWidth)) ||
      (prctlWorkplaceItems->top >=
        (prctlIcon->top - pccinfon->lAveCharWidth)) ||
      (prctlWorkplaceItems->bottom <=
        (prctlText->bottom + pccinfon->lAveCharWidth)))
  {
    GetNewWorkplace(hwnd);
    ischanged = TRUE;
  }

  return(ischanged);

}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnCalcTileSize                                   */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME: Calculate Text Size                             */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculates tile            */
/*   size based on the icon size and text size.                      */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  POINT iconSize - icon size structure                      */
/*         POINT textSize - text size structure                      */
/*         DWORD iconStyle - icon style                              */
/*         PPOINT ptileSize - point to the tile size structure       */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR IcnCalcTileSize( piconSize, ptextSize, iconStyle, pccinfon,
                    ptileSize )
LPPOINT        piconSize;
PSIZEL         ptextSize;                                               /*DAB1*/
DWORD          iconStyle;
LPCNRINFON     pccinfon;
PSIZEL         ptileSize;                                               /*DAB1*/
{
  if (iconStyle & IS_BENEATHTEXT)
  {
     /****************************************************************/
     /* cx of the text size is always larger than or equal to the    */
     /* cx of the icon size. So, cx of the tile is cx of the text    */
     /****************************************************************/
     ptileSize->cx = ptextSize->cx;                                     /*DAB1*/

     /****************************************************************/
     /* cy of the tile size is sum of cy of the all possible spaces  */
     /****************************************************************/
     ptileSize->cy = ptextSize->cy + piconSize->y +                     /*DAB1*/
                     pccinfon->lExternalLeading + 2 * ICN_EMPHSPACE;

  } else {

     /****************************************************************/
     /*|________                    a: lAveCharWidth                 */
     /*||   c   || _______________  b: longest line in the column    */
     /*||       ||a|______________| c: cx of the icon                */
     /*||_______||      b           d: ICN_EMPHASIS                  */
     /*|---------|                  e: ICN_EMPHASIS                  */
     /* e       d                                                    */
     /****************************************************************/
     ptileSize->cx = piconSize->x + ptextSize->cx +                     /*DAB1*/
                     pccinfon->lAveCharWidth + 2 * ICN_EMPHSPACE;
     ptileSize->cy =                                                    /*DAB1*/
             max((piconSize->y + 2 * ICN_EMPHSPACE), ptextSize->cy);    /*DAB1*/
  } /* endif */
  return;

}
/* D7092 Next four functions rewritten */
/*********************************************************************/
/*                                                                   */
/* Function:  IcnDrawScaledIcon                                      */
/*                                                                   */
/* Description:  Draw the container icons for all views, either      */
/*               scaled or not.                                      */
/*                                                                   */
/*********************************************************************/
VOID FAR  IcnDrawScaledIcon (HDC hdcScreen, LPICONRESIZED pirsDraw, int xDst,
                             int yDst, LPCNRINFON pccinfon,
                             BOOL SourceEmph)
{
  BOOL       bMiniDrawn = FALSE;  /* Have we drawn mini icon yet? */
  HBITMAP    hMiniBitmap;         /* Mini bitmap handle */
  HBITMAP    hbmOld;              /* Old bitmap handle */
  HDC        hdcMemory;           /* handle to memory device context */
  LPCNRINFO  pccinfo;             /* Pointer to External container info. */

  /*----------------------------------------------------------------------*/
  /* If we are to draw a mini icon, first look for a mini in the original */
  /* bitmap.  If one exists, draw it.  Otherwise, proceed with the resized*/
  /* bitmaps.                                                             */
  /*----------------------------------------------------------------------*/
  pccinfo = PCCINFOXFROMN(pccinfon);
  if (pccinfo->flWindowAttr & CV_MINI)
  {
     hMiniBitmap = NULL; /* Replace with some function call */
     if (hMiniBitmap)
     {
        DrawIcon (hdcScreen, xDst, yDst, hMiniBitmap);
        bMiniDrawn = TRUE;
     }
  }
  /*----------------------------------------------------------------------*/
  /* If we didn't find a small icon, draw the resized one.                */
  /*----------------------------------------------------------------------*/
  if (!bMiniDrawn)
  {
     /*-------------------------------------------------------------------*/
     /* If there is a bitmap handle in the IRS metrics, use that.         */
     /* Otherwise, just draw the regular icon using DrawIcon.             */
     /*-------------------------------------------------------------------*/
     if (pirsDraw->hbmPointer)
     {

        /*-------------------------------------------------------------------*/
        /* Get a memory DC compatible with the screen to BLT the bitmap from.*/
        /*-------------------------------------------------------------------*/
        if (!(hdcMemory = CreateCompatibleDC (hdcScreen)))
        {
           DrawIcon (hdcScreen, xDst, yDst, pirsDraw->hptrOrigin);
           return;
        }

        /*-------------------------------------------------------------------*/
        /* Select the bitmap into the source DC.                             */
        /*-------------------------------------------------------------------*/
        hbmOld = SelectObject (hdcMemory, pirsDraw->hbmPointer);

        /*-------------------------------------------------------------------*/
        /* BLT it to the screen.                                             */
        /*-------------------------------------------------------------------*/
        CnrBltStretch (hdcMemory, hdcScreen, 0, 0, pirsDraw->slIcon.x,
                       pirsDraw->slIcon.y, xDst, yDst, pirsDraw->slIcon.x,
                       pirsDraw->slIcon.y, SRCAND, COLORONCOLOR);

        /*-------------------------------------------------------------------*/
        /* If there is a color component, XOR it on top of the mask.         */
        /*-------------------------------------------------------------------*/
        if (pirsDraw->hbmColor)
        {
           SelectObject (hdcMemory, pirsDraw->hbmColor);
           CnrBltStretch (hdcMemory, hdcScreen, 0, 0, pirsDraw->slIcon.x,
                          pirsDraw->slIcon.y, xDst, yDst, pirsDraw->slIcon.x,
                          pirsDraw->slIcon.y, SRCINVERT, COLORONCOLOR);
        }


        /*-------------------------------------------------------------------*/
        /* Select the old object back into the DC.                           */
        /*-------------------------------------------------------------------*/
        SelectObject (hdcMemory, hbmOld);
     }
     else
        DrawIcon (hdcScreen, xDst, yDst, pirsDraw->hptrOrigin);

     /*----------------------------------------------------------------------*/
     /* Delete the memory DC.                                                */
     /*----------------------------------------------------------------------*/
     DeleteDC(hdcMemory);
  }
  return;
}


/**********************************************************************/
/*                                                                    */
/*  Function:  CnrBltStretch                                          */
/*                                                                    */
/*  Description:  Copies a bitmap from a source DC to a target DC,    */
/*                stretching as necessary.                            */
/*                                                                    */
/**********************************************************************/
VOID FAR CnrBltStretch (HDC hdcSrc, HDC hdcDst, int xSrc, int ySrc,
                      int  cxSrc, int  cySrc, int xDst, int yDst,       /*DAB1*/
                      int  cxDst, int  cyDst, DWORD rop, DWORD mode)    /*DAB1*/
{
  DWORD       oldStretchMode1, oldStretchMode2;


  /*----------------------------------------------------------------------*/
  /* Change the stretch mode for the source and target DCs.               */
  /*----------------------------------------------------------------------*/
  oldStretchMode1 = SetStretchBltMode (hdcSrc,mode);
  oldStretchMode2 = SetStretchBltMode (hdcDst,mode);
  /*----------------------------------------------------------------------*/
  /* BLT from the source DC to the target, stretching as needed.          */
  /*----------------------------------------------------------------------*/
  StretchBlt( hdcDst, xDst, yDst,
                      cxDst, cyDst,
              hdcSrc, xSrc, ySrc,
                      cxSrc, cySrc,
              rop);

  /*----------------------------------------------------------------------*/
  /* Reset the stretch modes.                                             */
  /*----------------------------------------------------------------------*/
  SetStretchBltMode (hdcDst, oldStretchMode2);
  SetStretchBltMode (hdcSrc, oldStretchMode1);
  return;
}

/*********************************************************************/
/*                                                                   */
/* Function:  IcnResizeIcon                                          */
/*                                                                   */
/* Description:  Creates new resized bitmaps in the ICONRESIZED      */
/*               structure.  They are drawn later with               */
/*               IcnDrawScaledIcon.                                  */
/*********************************************************************/
VOID FAR IcnResizeIcon(LPICONRESIZED pirsNew, HWND hwndCnr, POINT slIcon)
{
  ICONINFO    iconInfo;
  SHORT       cxSrc;
  SHORT       cySrc;
  HDC         hdcScreen;

  /*----------------------------------------------------------------------*/
  /* Get icon info about the original.                                    */
  /*----------------------------------------------------------------------*/
  if (!GetIconInfo (pirsNew->hptrOrigin, &iconInfo))
  {
    return;
  }

  /*******************************************************************/
  /* If its an icon, use system icon size.  Otherwise, it is a cursor*/
  /*******************************************************************/
  if (iconInfo.fIcon)
  {
    cxSrc = (SHORT) GetSystemMetrics (SM_CXICON);
    cySrc = (SHORT) GetSystemMetrics (SM_CYICON);
  }
  else
  {
    cxSrc = (SHORT) GetSystemMetrics (SM_CXCURSOR);
    cySrc = (SHORT) GetSystemMetrics (SM_CYCURSOR);
  }

  if ((cxSrc == (SHORT)slIcon.x) && (cySrc == (SHORT)slIcon.y)) {
    /*****************************************************************/
    /* If the desired icon size is the same as source icon size      */
    /* don't stretch.                                                */
    /*****************************************************************/
    pirsNew->hbmPointer = NULL;
    pirsNew->hbmColor = NULL;
  } else {
    /*****************************************************************/
    /* Create a screen DC.                                           */
    /*****************************************************************/
    hdcScreen = GetDC (hwndCnr);

    /*****************************************************************/
    /* If there is a color component to the icon, save both the mask */
    /* and the color bitmaps.  Otherwise, just save the mask.        */
    /*****************************************************************/
    if (iconInfo.hbmColor)
    {
      /***************************************************************/
      /* Create the bitmaps, stretching as necessary.                */
      /***************************************************************/
      pirsNew->hbmPointer = CopyBitmap(iconInfo.hbmMask,
                                       (SHORT)slIcon.x,
                                       (SHORT)slIcon.y,
                                       COLORONCOLOR,
                                       hdcScreen);
      pirsNew->hbmColor = CopyBitmap(iconInfo.hbmColor,
                                     (SHORT)slIcon.x,
                                     (SHORT)slIcon.y,
                                     COLORONCOLOR,
                                     hdcScreen);

    }
    else
    {
      pirsNew->hbmPointer = CopyBitmap(iconInfo.hbmMask,
                                       (SHORT)slIcon.x,
                                       (SHORT)(slIcon.y * 2),
                                       BLACKONWHITE,
                                       hdcScreen);
      pirsNew->hbmColor = NULL;
    }

  /*----------------------------------------------------------------------*/
  /* Release the screen DC.                                               */
  /*----------------------------------------------------------------------*/
    ReleaseDC (hwndCnr,hdcScreen);
  } /* endif */

  /*------------------------------------------------------------------*/
  /* Free bitmap handles allocated by GetIconInfo call above          */
  /*------------------------------------------------------------------*/
  DeleteObject( iconInfo.hbmMask );
  DeleteObject( iconInfo.hbmColor );

  return;
}

/* =========================================================================
 *
 *  Function:  CopyBitmap
 *
 *  Description:
 *
 *     Creates a new bitmap, optionally shrinking or stretching in the process.
 *
 * ========================================================================= */
HBITMAP FAR CopyBitmap (HBITMAP hbm, WORD cx, WORD cy, DWORD flBlt,
                         HDC hdcScreen)
{
  BITMAP           bmp;
  HBITMAP          hbmNew;
  UINT             cxSrc, cySrc;                                        /*DAB1*/
  HBITMAP          hBitmapOld1;
  HBITMAP          hBitmapOld2;
  HDC              hdcSource,hdcTarget;

  /*--------------------------------------------------------------*/
  /* If no bitmap handle is passed in, return without success.    */
  /*--------------------------------------------------------------*/
  if (!hbm)
  {
    return (0);
  }

  /*--------------------------------------------------------------*/
  /* If there is no information in the bitmap handle, return      */
  /* without success.                                             */
  /*--------------------------------------------------------------*/
  if (!GetObject(hbm, sizeof(BITMAP), (LPSTR)&bmp))
  {
     return (0);
  }
  /*-------------------------------------------*/
  /* If cx and/or cy weren't supplied, use the */
  /* dimensions of the bitmap we're copying    */
  /*-------------------------------------------*/
  cxSrc = bmp.bmWidth;
  cySrc = bmp.bmHeight;
  bmp.bmWidth  = (cx) ? cx : bmp.bmWidth;
  bmp.bmHeight = (cy) ? cy : bmp.bmHeight;

  /*--------------------------------------------------------------*/
  /* Create a memory DC to be the source.                         */
  /*--------------------------------------------------------------*/
  if (!(hdcSource = CreateCompatibleDC(hdcScreen)))
    return (NULL);

  /*--------------------------------------------------------------*/
  /* Create the bitmap in the screen DC.  NOTE:  A memory DC does */
  /* not support color bitmaps in Windows NT.                     */
  /*--------------------------------------------------------------*/
  if (!(hbmNew = CreateCompatibleBitmap(hdcScreen,bmp.bmWidth, bmp.bmHeight)))
    return (NULL);

  /*--------------------------------------------------------------*/
  /* Create a target memory DC.                                   */
  /*--------------------------------------------------------------*/
  hdcTarget = CreateCompatibleDC(0);

  /*--------------------------------------------------------------*/
  /* Select the source bitmap into the source DC, and the new     */
  /* bitmap into the target DC.                                   */
  /*--------------------------------------------------------------*/
  hBitmapOld1 = SelectObject (hdcSource, hbm);
  hBitmapOld2 = SelectObject (hdcTarget, hbmNew);

  /*--------------------------------------------------------------*/
  /* BLT the bitmap from source to target, stretching as we go.   */
  /*--------------------------------------------------------------*/
  CnrBltStretch (hdcSource, hdcTarget, 0, 0, cxSrc, cySrc, 0, 0, cx, cy,
              SRCCOPY, flBlt);

  /*--------------------------------------------------------------*/
  /* Select the old objects back into the DCs.                    */
  /*--------------------------------------------------------------*/
  SelectObject (hdcTarget, hBitmapOld2);
  SelectObject (hdcSource, hBitmapOld1);

  /*--------------------------------------------------------------*/
  /* Delete the DCs.                                              */
  /*--------------------------------------------------------------*/
  DeleteDC(hdcTarget);
  DeleteDC(hdcSource);

  /*--------------------------------------------------------------*/
  /* Return the handle to the new resized bitmap.                 */
  /*--------------------------------------------------------------*/
  return (hbmNew);
}
