/* Revision: 14 1.2.2.2 source/ui/cclnt/cnr/cnrs2p3.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS2P3.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS2P3.C - Icon view routines for container control     */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* DAB1  02/09/94  DAB  Updated w/ fix for checking internal recordptr*/
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/* LJO1   8/06/96  LJO  Fix lack of column title refresh after insert.*/
/**********************************************************************/
#include <windows.h>
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef SEGMENTOR
#define CNRS2P3_C
#include "cnrpragw.h"
#endif

/*********************************************************************/
/* FUNCTION NAME:  CnrArrangeIconView                                */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Arrange Icon View            */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function arranges icon view         */
/*   records. Positioning will begin with the first record           */
/*   in the container, and all records are positioned in item        */
/*   order.  The z-order does not change due to this function        */
/*    since no records will overlap.                                 */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window             */
/*         HDC  hps                                                  */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL: TRUE - the records were positioned successfully.     */
/*                                                                   */
/* EXIT-ERROR: FALSE - an error occurred.                            */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR CnrArrangeIconView( hwndCnr, hps)
HANDLE        hwndCnr;
HDC           hps;
{
  long         cxVscroll;
  long         cyHscroll;
  long         cyBetween;           /* space between rows              */
  long         maxLength=0;         /* maximum length of the row       */
  long         maxHeight;           /* maximum height of the row       */
  long         xPrevRight;          /* right boundary of the prev item */
  long         right;               /* x boundary of the item          */
  long         xTempRight;          /* temporary x boundary of the item*/
  long         difference;
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  POINT        currentPos;          /* position of the current item   */
  LPRECORDCOREN precord;            /* record to be positioned        */
  LPRECORDCOREN preccCurrent;       /* pointer to prev record of      */
                                    /* the record to be positioned    */
  RECT         windowRect;          /* rectangle of the window        */
  RECT         rctlLTitle;          /* rectangle of the list title    */
  POINT        iconSize;            /* size of the icon               */
  SIZEL        textSize;            /* size of the text               */
  SIZEL        tileSize;            /* size of the tile               */

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = PCCINFONOF (hwndCnr);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if ((pccinfo->flWindowAttr & CV_ICON) && (pccinfon->preccItem))
  {
    PMGetClientRect (hwndCnr, &windowRect);

    /******************************************************************/
    /* Set icon dimensions.                                           */
    /******************************************************************/
    CnrCalcIconSize (hwndCnr, &iconSize);

    /******************************************************************/
    /* Set cyBetween                                                  */
    /******************************************************************/
    if (pccinfo->cyLineSpacing >= 0)
    {
      cyBetween = pccinfo->cyLineSpacing;
    }
    else
    {
      cyBetween = MINCY_BETWEENITEMS;
    }

    cxVscroll = GetSystemMetrics( SM_CXVSCROLL);
    cyHscroll = GetSystemMetrics( SM_CYHSCROLL);

    /******************************************************************/
    /* Get the size of the title rectangle                            */
    /******************************************************************/
    CnrQueryTitleRect(hwndCnr, &rctlLTitle, NULL);

    /*******************************************************************/
    /* Initialize the workplace rectangle.                             */
    /* We set bottom as the top of the viewport so that even though   */
    /* we resize the window the V scroll bar is not activated if       */
    /* no item is in the list                                          */
    /*******************************************************************/
    pccinfon->rctlWorkplace.left = pccinfon->rctlWorkplace.right = 0;
    pccinfon->rctlWorkplace.top =
                  pccinfon->rctlWorkplace.bottom = rctlLTitle.bottom;

    pccinfon->rctlWorkplaceItems.left =
                                pccinfon->rctlWorkplaceItems.right = 0;
    pccinfon->rctlWorkplaceItems.top =
                               pccinfon->rctlWorkplaceItems.bottom = 0;

    /******************************************************************/
    /* Before going into the loop, establish the y position and       */
    /* the rightmost point of the imaginary previous item of the first*/
    /* item.                                                          */
    /******************************************************************/
    currentPos.y = windowRect.top + rctlLTitle.bottom +
                   pccinfon->lAveCharWidth + ICN_EMPHSPACE;
    right = 0;

    precord = pccinfon->preccItem;

    /******************************************************************/
    /* Initialize the max height the rightmost point of the first row */
    /******************************************************************/
    maxHeight = 0;

    /******************************************************************/
    /* Initialize the origin of the viewport                          */
    /******************************************************************/
    pccinfo->ptlOrigin.x = pccinfo->ptlOrigin.y = 0;

    preccCurrent = NULL;

    /******************************************************************/
    /* Position all records.                                          */
    /******************************************************************/
    while (precord)
    {
      /****************************************************************/
      /* Calculate text size                                          */
      /****************************************************************/
      if (!(pccinfon->ulFlagSet & FL_TEXTCALC))
      {
        IcnCalcTextSize (hps, iconSize, pccinfon,
                         precord->preccX->pszIcon,
                         IS_BENEATHTEXT,
                         &(precord->slTextSize));
      }

      textSize = precord->slTextSize;
      maxLength = max(maxLength, textSize.cx);                          /*DAB1*/

      /****************************************************************/
      /* Position if the record is not filtered, else, ignore it.     */
      /****************************************************************/
      if (ISRECORDVISIBLE(precord))
      {
        xPrevRight = right;
        xTempRight = right + (pccinfon->lAveCharWidth + textSize.cx);   /*DAB1*/

        /**************************************************************/
        /* For the item that doesn't fit into the remaining area of   */
        /* the row and is not the first item of the row move          */
        /* to the next row.                                           */
        /**************************************************************/
        if ((xTempRight > (windowRect.right - windowRect.left -
             cxVscroll - pccinfon->lAveCharWidth)) &&
            (xPrevRight != 0))
        {
           currentPos.y += (maxHeight + cyBetween);

           /***********************************************************/
           /* Initialize the max height and the rightmost point       */
           /* of the new row.                                         */
           /***********************************************************/
           right = 0;
           maxHeight = 0;
        }

        /**************************************************************/
        /* Set x position according to the previous item's the        */
        /* rightmost position                                         */
        /**************************************************************/
        difference = textSize.cx - iconSize.x;                          /*DAB1*/
        currentPos.x = (int)(right + pccinfon->lAveCharWidth +
                       difference / 2 +
                       ((difference & 1) ? 1 : 0L));

        /**************************************************************/
        /* Set the max height of the row.                             */
        /**************************************************************/
        IcnCalcTileSize (&iconSize, &textSize, IS_BENEATHTEXT,
                         pccinfon, &tileSize);
        maxHeight = max(maxHeight, tileSize.cy);                        /*DAB1*/

        /**************************************************************/
        /* Save calculated position, update rightmost position of the */
        /* row and continue on to the next record.                    */
        /**************************************************************/
        precord->ptlIcon = currentPos;
        COPYNPTLTOX(precord);
        right += (3 * pccinfon->lAveCharWidth + textSize.cx);           /*DAB1*/

        CnrUpdateWorkplace(hwndCnr, hps, precord, TRUE);
        if (pccinfon->rctlWorkplaceItems.left <= 0)
        {
          pccinfon->rctlWorkplaceItems.left = 1;
          pccinfon->rctlWorkplace.left = 1;
        }
      }
      preccCurrent = precord;
      precord = precord->preccNextItem;
    }

    /******************************************************************/
    /* Find the last item unfiltered.                                 */
    /******************************************************************/
    while (preccCurrent && (!ISRECORDVISIBLE(preccCurrent)))
    {
      preccCurrent = preccCurrent->preccPrevItem;
    }

    if (preccCurrent)
    {
      currentPos = preccCurrent->ptlIcon;
    }

    /******************************************************************/
    /* Text calculation is done. Positioning is done.                 */
    /******************************************************************/
    pccinfon->ulFlagSet |= (FL_TEXTCALC | FL_SETWORKPLACE);

    pccinfon->preccFirstinAutoPosIV = FindFirstIteminRow(hwndCnr,
                                      pccinfon->preccFirstinAutoPosIV);
  }
  else if (!(pccinfo->flWindowAttr & CV_ICON))
  {
    /******************************************************************/
    /* If current view is not icon view, just set FL_NEEDARRANGE      */
    /* to rearrange before painting in icon view                      */
    /******************************************************************/
    pccinfon->ulFlagSet |= FL_NEEDARRANGE;
  }
  return TRUE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrTRVCollapse                                 */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:  Collapse Tree                                  */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input: mp1: LPRECORDCORE (RECORDCORE)                             */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrTRVExpandCollapse( HANDLE hwnd, UINT msg,                   /*DAB1*/
                               LPRECORDCOREN preccObj )
{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  preccOldCursor;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (msg == CM_COLLAPSETREE)
  {
    pccinfon->ulFlagSet &= ~FL_SETFIRSTINVIEW;

    preccOldCursor = pccinfon->preccCursor;

    /*****************************************************************/
    /* If preccCursor is descendant of the preccObj, move cursor to  */
    /* preccObj.                                                     */
    /*****************************************************************/
    if (preccObj &&
        TRVIsAncestor(preccObj, pccinfon->preccCursor) )
    {
      pccinfon->preccCursor = preccObj;
    }
    else
    {
      /***************************************************************/
      /* If preccCursor is not a root node, move cursor to  the root */
      /* of preccCursor.                                             */
      /***************************************************************/
      if (!preccObj && pccinfon->preccCursor->preccParent)
      {
        pccinfon->preccCursor = TRVGetRoot(pccinfon->preccCursor);
      }
    }
    if (preccOldCursor != pccinfon->preccCursor)
    {
      if (!SetEmphasisBits (hwnd, NULL, preccOldCursor,
                            CRA_CURSORED | CRA_SELECTED, SEB_CLEAR))
      {
        return (FALSE);
      }

      if (!SetEmphasisBits (hwnd, NULL, pccinfon->preccCursor,
                            CRA_CURSORED | CRA_SELECTED, SEB_SET))
      {
        return (FALSE);
      }
    }
  }

  if (preccObj)
  {
    /*****************************************************************/
    /* If preccCursor is decendent of the preccObj, move cursor to   */
    /* preccObj.                                                     */
    /*****************************************************************/
    if (preccObj->flRecordAttr &
        ((msg == CM_COLLAPSETREE) ? CRA_EXPANDED : CRA_COLLAPSED))
    {
      preccObj->flRecordAttr &=
            ~((msg == CM_COLLAPSETREE) ? CRA_EXPANDED : CRA_COLLAPSED);

      preccObj->flRecordAttr |=
             ((msg == CM_COLLAPSETREE) ? CRA_COLLAPSED : CRA_EXPANDED);

      COPYNDATATOX(preccObj);
      NotifyOwner( hwnd, (WORD)((msg == CM_COLLAPSETREE) ?              /*DAB1*/
                   CN_COLLAPSETREE : CN_EXPANDTREE),
                   (LPVOID)(preccObj->preccX));

      if (pccinfo->flWindowAttr & CV_TREE)
      {
        TRVInvalidateForExpCol(hwnd, preccObj);
      }
    }
  }
  else
  {
    DWORD       ulNewState;
    DWORD       ulOldState;
    WORD        usNotify;

    if (msg == CM_EXPANDTREE)
    {
      ulNewState = CRA_EXPANDED;
      ulOldState = CRA_COLLAPSED;
      usNotify = CN_EXPANDTREE;
    }
    else
    {
      ulNewState = CRA_COLLAPSED;
      ulOldState = CRA_EXPANDED;
      usNotify = CN_COLLAPSETREE;
    }
    TRVExpandCollapseAll(hwnd, ulOldState, ulNewState, usNotify);
  }
  return TRUE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     TRVExpandCollapseAll                           */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:  Collapse Tree                                  */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input: mp1: LPRECORDCORE (RECORDCORE)                             */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR TRVExpandCollapseAll(					/*RKK1*/
  HANDLE       hwnd,
  DWORD        ulOldState,
  DWORD        ulNewState,
  WORD         usNotify  )					/*RKK1*/
{
  LPCNRINFON    pccinfon;
  LPRECORDCOREN preccEnum;
  LPRECORDCOREN preccFirstInvalid = NULL;
  BOOL          flInvalidateAll = FALSE;
  BOOL          flMeetFirst = FALSE;


  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

  preccEnum = pccinfon->preccItem;

  if (pccinfon->preccItem)
  {
    if (!pccinfon->preccFirstinTRV)
    {
      pccinfon->preccFirstinTRV = pccinfon->preccItem;
    }

    while (preccEnum)
    {
      /***************************************************************/
      /* From here, we figure out that if an item before the first   */
      /* item in view has old state attribute and has children.      */
      /* (Can be expanded or collapsed only when it has children)    */
      /* If so, we invalidate whole window, else,                    */
      /* we only invalidate only after the first item which is       */
      /* visible and has old state.                                  */
      /***************************************************************/
      if (!flInvalidateAll && !flMeetFirst )
      {
        if (preccEnum == pccinfon->preccFirstinTRV)
        {
          flMeetFirst = TRUE;
        }
        else
        {
          if ((preccEnum->flRecordAttr & ulOldState) &&
              ISRECORDVISIBLE(preccEnum) &&
              TRVhasVisibleChild (preccEnum))
          {
            flInvalidateAll = TRUE;
          }
        }
      }

      if (flMeetFirst)
      {
        if (!preccFirstInvalid &&
            (preccEnum->flRecordAttr & ulOldState) &&
            ISRECORDVISIBLE(preccEnum))
        {
          preccFirstInvalid = preccEnum;
        }
      }

      /***************************************************************/
      /* Turn off old state and on new state.                        */
      /***************************************************************/
      if (preccEnum->flRecordAttr & ulOldState)
      {
        preccEnum->flRecordAttr &= ~ulOldState;
        preccEnum->flRecordAttr |= ulNewState;
        COPYNDATATOX(preccEnum);
        NotifyOwner(hwnd, usNotify, (LPVOID)preccEnum->preccX);
      }

      preccEnum = TRVGetNextItem(preccEnum);
    } /* end while */

    if (flInvalidateAll)
    {
      pccinfon->ulFlagSet &= ~(FL_CNRPOSITION | FL_SETSCROLL |
                          FL_NEWVIEWSCROLLSET );
      PMInvalidateRect (hwnd, NULL, TRUE);
    }
    else
    {
      HDC        hps;

      if (preccFirstInvalid)
      {
        hps = PMGetFlippedDC(hwnd);
        TRVInvalidateFrom(hwnd, hps, pccinfon, preccFirstInvalid);
        ReleaseDC(hwnd, hps);
      } /* endif */
    }
  }

  return TRUE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     TRVInvalidateFrom                              */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:  Collapse Tree                                  */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input: mp1: LPRECORDCORE (RECORDCORE)                             */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
void FAR TRVInvalidateFrom(hwnd, hps, pccinfon, preccInvalid)
HANDLE         hwnd;
HDC            hps;
LPCNRINFON     pccinfon;
LPRECORDCOREN  preccInvalid;
{
  RECT      rctlInvalid;
  int       sOldVscrollPos = pccinfon->sVscrollPos;
  int       sOldHscrollPos = pccinfon->sHscrollPos;

  pccinfon->ulFlagSet &= ~( FL_CNRPOSITION | FL_SETSCROLL );
  CnrTRVSetUpNewView(hwnd, hps);

  if ((sOldVscrollPos == pccinfon->sVscrollPos) &&
      (sOldHscrollPos == pccinfon->sHscrollPos))
  {
    TRVGetInvalidateRect(hwnd, preccInvalid, &rctlInvalid);
    PMInvalidateRect (hwnd, &rctlInvalid, TRUE);
  }
  else
  {
    PMInvalidateRect (hwnd, NULL, TRUE);
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     InvalidateForExpCol                            */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:  Collapse Tree                                  */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input: mp1: LPRECORDCORE (RECORDCORE)                              */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
void FAR TRVInvalidateForExpCol(hwnd, preccInvalid)
HANDLE         hwnd;
LPRECORDCOREN  preccInvalid;
{
  HDC         hps;
  LPCNRINFON  pccinfon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

  pccinfon->ulFlagSet &= ~( FL_CNRPOSITION | FL_SETSCROLL );

  /******************************************************************/
  /* If preccInvalid has children, we need repositon and repaint    */
  /* from preccInvalid.                                             */
  /* Else, just redraw precc only.                                  */
  /******************************************************************/
  if (TRVhasVisibleChild (preccInvalid) )
  {
    if (Before1stInView(hwnd, preccInvalid))
    {
      pccinfon->ulFlagSet &= ~FL_NEWVIEWSCROLLSET;
      PMInvalidateRect (hwnd, NULL, TRUE);
    }
    else
    {
      hps = PMGetFlippedDC(hwnd);
      TRVInvalidateFrom(hwnd, hps, pccinfon, preccInvalid);
      ReleaseDC(hwnd, hps);
    }
  }
  return;
}

/*********************************************************************/
/* FUNCTION NAME:  CnrFilter                                         */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Filter Container                               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function filters the contents       */
/*   of a container if mp1 is given and unfilters if mp1 is NULL.    */
/*   Filtering is done by setting the CRA_FILTER                     */
/*   attribute of records that are to be excluded from the           */
/*   viewable subset.                                                */
/*   Unfiltering is done by setting the CRA_FILTER attribute         */
/*   of records that are filtered off.                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:                                                            */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
#if defined(__IBMC__) || defined(__IBMCPP__)
typedef BOOL (* CALLBACK PFNFILTER)(LPRECORDCORE, LPVOID);
#else
typedef BOOL (FAR PASCAL *PFNFILTER)(LPRECORDCORE, LPVOID);
#endif

BOOL FAR CnrFilter(hwnd, mp1, mp2)
HANDLE   hwnd;
long     mp1;
long     mp2;

{
  PFNFILTER     pfnFilter;
  BOOL          rc = FALSE;
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccCurrent;
  LPVOID        pStorage;
  long          rcFilter;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (!pccinfon->preccItem)
  {
    /*****************************************************************/
    /* Unfiltering no record, then unfiltering is unsuccessful.      */
    /*****************************************************************/
    SETERROR(PMERR_NOFILTERED_ITEMS);
    return FALSE;
  }

   preccCurrent = pccinfon->preccItem;

  if (mp1)
  {
    /*****************************************************************/
    /* Get the application provided pointer to the function to       */
    /* determine if a record is to be included in the filtered       */
    /* subset.                                                       */
    /*****************************************************************/
    pfnFilter = (PFNFILTER)mp1;
    pStorage  = (LPVOID)mp2;

    while (preccCurrent)
    {
      rcFilter = (*pfnFilter)(preccCurrent->preccX, pStorage);

      if (rcFilter && !ISRECORDVISIBLE(preccCurrent))
      {
        SetEmphasisBits (hwnd, NULL, preccCurrent, CRA_FILTERED,
                         SEB_CLEAR);
        rc = TRUE;
      }
      else
      {
        if (!rcFilter && ISRECORDVISIBLE(preccCurrent))
        {
          /*************************************************************/
          /* If the current record is to be filtered, set CRA_FILTERED */
          /* attribute.                                                */
          /*************************************************************/
          SetEmphasisBits (hwnd, NULL, preccCurrent, CRA_FILTERED,
                           SEB_SET);

          /************************************************************/
          /* Check, and if necessary, update the cursored item.       */
          /************************************************************/
          UpdateCursorForDel(hwnd, preccCurrent);

          UpdateFirstItemInView(hwnd, preccCurrent);
          rc = TRUE;
        }
      }
      preccCurrent = TRVGetNextItem(preccCurrent);
    } /* endwhile */
  }
  else
  {
    /*****************************************************************/
    /* If mp1 is NULL, set CRA_FILTERED attrribute of the filtered   */
    /* records off.                                                  */
    /*****************************************************************/
    while (preccCurrent)
    {
      if (!(ISRECORDVISIBLE(preccCurrent)))
      {
        SetEmphasisBits (hwnd, NULL, preccCurrent, CRA_FILTERED,
                         SEB_CLEAR);
        rc = TRUE;
      }
      preccCurrent = TRVGetNextItem(preccCurrent);
    } /* endwhile */
  } /* endif */

  if (rc)
  {
    /*****************************************************************/
    /* Update cursored item.                                         */
    /*****************************************************************/
    if (!pccinfon->preccCursor)
    {
      LPRECORDCOREN  preccTemp;

      preccTemp = pccinfon->preccItem;

      while (preccTemp && !ISRECORDVISIBLE(preccTemp))
      {
        preccTemp = GetNextVisualItem(hwnd, preccTemp);
      }

      if (pccinfon->preccCursor = preccTemp)
      {
        SetEmphasisBits (hwnd, NULL, pccinfon->preccCursor, CRA_CURSORED,
                         SEB_SET);
      }
    }

    /*****************************************************************/
    /* If some records are filtered or unfileded we need reposition  */
    /* items, resetting scroll bars and resetting workplace.         */
    /*****************************************************************/
    pccinfon->ulFlagSet &= ~(FL_CNRPOSITION | FL_SETWORKPLACE |
               FL_SETSCROLL | FL_NEWVIEWSCROLLSET | FL_SIZECOLTITLES |
               FL_POSITIONCOLTITLES );

    /*****************************************************************/
    /* When change the view, initialize the noniconorigin            */
    /*****************************************************************/
    pccinfon->ptlNonIconOrg.x = pccinfon->ptlNonIconOrg.y = 0;

    /*****************************************************************/
    /* Clean the scroll range and scroll position.                   */
    /*****************************************************************/
    pccinfon->sVscrollPos = pccinfon->sHscrollPos = 0;
    pccinfon->sVscrollMax = pccinfon->sHscrollMax = 0;
    pccinfon->sHscrollMaxRight = 0;

    /*****************************************************************/
    /* Clean the scroll range and scroll position.                   */
    /*****************************************************************/
    pccinfon->preccFirstinNV = pccinfon->preccFirstinFNV = NULL;
    pccinfon->preccFirstinTV = pccinfon->preccFirstinFTV = NULL;
    pccinfon->preccFirstRowInDV = NULL;

    /*****************************************************************/
    /* If CCS_AUTOPOSITION is set on, we need rearrange the icons     */
    /* in icon view.                                                 */
    /*****************************************************************/
    if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
    {
      pccinfon->ulFlagSet |= FL_NEEDARRANGE;
    } /* endif */

    /*****************************************************************/
    /* If some records are filtered, redraw the window.              */
    /*****************************************************************/
#if 0
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       PMGetClientRect (hwnd, &CnrRect);

       if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
           ((long)pccinfo->xVertSplitbar > CnrRect.right))
       {
          CnrDetailsVCtlProc (hwnd, WM_PAINT, NULL, NULL);
       }
    }
#endif

    InvalidateChildren (hwnd, FALSE, FALSE);
  }

  if (mp1 || rc)
  {
    /*****************************************************************/
    /* Filtering operation doesn't have errors.(successful)          */
    /* Unfiltering at least one record, then unfiltering is          */
    /* successful.                                                   */
    /*****************************************************************/
    return TRUE;
  }
  else
  {
    /*****************************************************************/
    /* Unfiltering no record, then unfiltering is unsuccessful.      */
    /*****************************************************************/
    SETERROR(PMERR_NOFILTERED_ITEMS);
    return FALSE;
  }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrInsertRecord                                */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:  Insert Records                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input: mp1: pRecordCORE (LPRECORDCORE)                            */
/*              Pointer to record structure(s) to be inserted.        */
/*         mp2: pRecordInsert (LPRECORDINSERT)                        */
/*              Pointer to data structure containing:                 */
/*              pRecordOrder - position record relative to others.    */
/*                Can be: CMA_FIRST                                   */
/*                        CMA_END                                     */
/*                        Pointer to record before which to insert    */
/*                          record.                                   */
/*                                                                    */
/*              pRecordZOrder - position record in z-order.  Can be:  */
/*                CMA_TOP                                             */
/*                CMA_BOTTOM                                          */
/*              cRecordsInsert - number of records to insert          */
/*                                                                    */
/*                                                                    */
/*  Output:                                                           */
/*        Number of records in container.                             */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: Number of records in container.                      */
/*                                                                    */
/*  Exit-Error:  NULL                                                 */
/*                                                                    */
/*  Side Effects: None                                                */
/*                                                                    */
/**********************************************************************/
long FAR CnrInsertRecord(HANDLE hwnd, long mp1, long mp2)
{
  LPCNRINFON        pccinfon;
  LPCNRINFO         pccinfo;
  LPRECORDCORE      preccInsert;
  LPRECORDINSERT    pRecordInsert;
  LPRECORDCORE      preccNew;
  LPRECORDCOREN     preccnNew;
  LPRECORDCOREN     preccnParent;
  LPRECORDCOREN     preccnFirstInvalid;
  BOOL              flNoSiblings = FALSE;
  DWORD             flChangedAttr;
  DWORD             windowStyle;
  POINT             iconSize;
  HDC               hps;
  DWORD             i;

  if((!mp1) || (!mp2))
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return ((DWORD)0);
  }

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pRecordInsert = (LPRECORDINSERT)mp2;
  preccInsert = (LPRECORDCORE)mp1;
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);
  windowStyle = GetWindowLong( hwnd, GWL_STYLE );

  /*******************************************************************/
  /* Verify that we are inserting at least one record and the        */
  /* control block size is set correctly.                            */
  /*******************************************************************/
  if ((!(pRecordInsert)) || (!(preccInsert)) ||
      (!(pRecordInsert->pRecordOrder)) ||
      (pRecordInsert->cRecordsInsert <= 0) ||
      (pRecordInsert->cb != sizeof(RECORDINSERT)))
  {
     SETERROR(PMERR_INVALID_PARAMETERS);
     return ((DWORD)NULL);
  }

  /*******************************************************************/
  /* If Verify Pointers is set, check to see if any of the record    */
  /* structures the app is trying to insert are already inserted.    */
  /* If so, return NULL with the approriate error information.       */
  /*******************************************************************/
  if (GetWindowLong (hwnd, GWL_STYLE) & CCS_VERIFYPOINTERS)
  {
    BOOL             rc = FALSE;
    LPRECORDCORE      preccTemp = preccInsert;
    i = 0;
    while (!rc && (i < (DWORD)pRecordInsert->cRecordsInsert) &&
           preccTemp)
    {
      rc |= CnrVerifyPointers (hwnd, (LPVOID)(&preccTemp), 1L);
      preccTemp = preccTemp->preccNextRecord;
      i++;
    }

    if (rc)
    {
      SETERROR(PMERR_RECORD_CURRENTLY_INSERTED);
      return ((DWORD)0);
    }
  }

  /*******************************************************************/
  /* Convert pRecordInsert into pRecordInsertN and save it in        */
  /* pccinfon.                                                       */
  /*******************************************************************/
  if (!ConvertPRItoPRIN(hwnd, pRecordInsert))
  {
    return ((DWORD)0);
  }

  /*******************************************************************/
  /* Get the parent of the item to be inserted.                      */
  /*******************************************************************/
  preccnParent = pccinfon->pRecordInsertN->pRecordParentN;

  /*******************************************************************/
  /* Is there siblings?  If not let's keep the fact.                 */
  /*******************************************************************/
  if ((!(preccnParent) && !(pccinfon->preccItem)) ||
      (preccnParent && !(preccnParent->preccFirstChild)))
  {
    flNoSiblings = TRUE;
  } /* endif */

  /*******************************************************************/
  /* Insert the records in the tree.                                 */
  /*******************************************************************/
  preccnFirstInvalid = InsertRecordsInChain (hwnd,
                                             pccinfon->pRecordInsertN,
                                             preccInsert);

  /*******************************************************************/
  /* If this is initial root item insertion, need initialize         */
  /* a cursored record (/ selected record).                          */
  /*******************************************************************/
  if (!preccnParent)
  {
    /******************************************************************/
    /* Update the number of records before we send any notifications. */
    /******************************************************************/
    pccinfo->cRecords += pRecordInsert->cRecordsInsert;

    if (!pccinfon->preccCursor && preccnFirstInvalid)
    {
      pccinfon->preccCursor = preccnFirstInvalid;

      if ((pccinfo->flWindowAttr & CV_TREE) ||
          !(windowStyle & CCS_MULTIPLESEL))
      {
        flChangedAttr = CRA_SELECTED | CRA_CURSORED;
        /*************************************************************/
        /* Initialize the anchor point for range selection           */
        /*************************************************************/
        pccinfon->preccAnchor = preccnFirstInvalid;
      }
      else
      {
        flChangedAttr = CRA_CURSORED;
      }

      /****************************************************************/
      /* Notify the application that a record's attribute has changed.*/
      /* Return a pointer to the record and the emphasis attribute    */
      /* mask.                                                        */
      /****************************************************************/
      if (!SetEmphasisBits (hwnd, NULL, preccnFirstInvalid,
                            flChangedAttr, SEB_SET))
      {
        return (FALSE);
      }
    }
  }

  /********************************************************************/
  /* If inserted in the root level or in the tree view,...            */
  /********************************************************************/
  if ((!preccnParent) || (pccinfo->flWindowAttr & CV_TREE))
  {
    hps = GetDC(hwnd);

    /******************************************************************/
    /* Calculate the text size for the current view.  Later paint     */
    /* routines depend on this size being preset.                     */
    /******************************************************************/
    CnrCalcIconSize(hwnd, &iconSize);
    preccNew = preccInsert;

    for (i=0; i< pccinfon->pRecordInsertN->cRecordsInsert; i++)
    {
      /****************************************************************/
      /* Check for error in retrieving internal record for external   */
      /****************************************************************/
      if (!(preccnNew = GetRecordNFromX(hwnd, preccInsert)))            /*DAB2*/
      {                                                                 /*DAB2*/
        ReleaseDC( hwnd, hps );                                         /*DAB2*/
        return ((DWORD)0);                                              /*DAB2*/
      }                                                                 /*DAB2*/

      /****************************************************************/
      /* Calculate the text size for all views since paint routines   */
      /* depend on this being preset (otherwise no text will appear)  */
      /****************************************************************/
      CalcTextSize (hwnd, preccnNew, iconSize);

      /****************************************************************/
      /* Use bitmap when only CA_DRAWBITMAP is specified.             */
      /* Default is using Icon.                                       */
      /****************************************************************/
      if ((pccinfo->flWindowAttr & CA_DRAWICON) ||
          !(pccinfo->flWindowAttr & CA_DRAWBITMAP))
      {
        /***************************************************************/
        /* Resize the bitmaps and save them in the metrics.            */
        /* Resizing always takes too much time, so, it is better to    */
        /* save resized bitmaps when inserting the record and use      */
        /* those bitmaps when drawing.                                 */
        /***************************************************************/
        if (!(CnrInsertDeleteResizedIcon(hwnd, preccNew, TRUE, TRUE)))
        {
           ReleaseDC(hwnd, hps);
           return (0);                                                  /*DAB1*/
        }
        if (!preccnParent)
        {
          if (!(CnrInsertDeleteResizedIcon(hwnd, preccNew, TRUE, FALSE)))
          {
             ReleaseDC(hwnd, hps);
             return (0);                                                /*DAB1*/
          }
        }
        if (!(CnrTRVInsDelResizedIcon(hwnd, preccNew, TRUE)))
        {
           ReleaseDC(hwnd, hps);
           return (0);                                                  /*DAB1*/
        }

        /***************************************************************/
        /* If any FieldInfo's (columns) exist, we must store the       */
        /* resized Icon for each of them in the IRS Metrics.           */
        /* No need to process DV info for children records             */
        /***************************************************************/
        if ((pccinfon->pFirstFI) && (!preccnParent))
        {
          if (!(CnrProcessIconsForDV (hwnd, preccnNew, NULL,
                                      TRUE, TRUE)))
           {
              ReleaseDC(hwnd, hps);
              return (0);                                               /*DAB1*/
           }
        }
      } /* endif */

      /****************************************************************/
      /* If inserted in the root level and has at least one visible   */
      /* item is inserted.......                                      */
      /****************************************************************/
      if (!preccnParent && preccnFirstInvalid)
      {
        /*************************************************************/
        /* Update the workplace.                                     */
        /*************************************************************/
        if (CnrUpdateWorkplace(hwnd, hps, preccnNew, TRUE))
        {
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;
        }

        /*************************************************************/
        /* Update the lMaxHeight if the updated item is              */
        /* the tallest one.                                          */
        /*************************************************************/
        if (pccinfon->lMaxHeight < preccnNew->slTextSize.cy)            /*DAB1*/
        {
          pccinfon->lMaxHeight = preccnNew->slTextSize.cy;              /*DAB1*/
        }
      }

      /***************************************************************/
      /* Reposition the items for the insertion based on view        */
      /***************************************************************/
      if (pccinfo->flWindowAttr &
          (CV_TREE | CV_NAME | CV_TEXT | CV_DETAIL))
      {
        /*************************************************************/
        /* Initialize the position of the item to be inserted.       */
        /*************************************************************/
        if (preccnNew->preccPrevItem)
        {
          preccnNew->ptlNonIconPos =
                              preccnNew->preccPrevItem->ptlNonIconPos;
        }
        else if ((pccinfo->flWindowAttr & CV_TREE) &&
                 preccnNew->preccParent)
        {
          preccnNew->ptlNonIconPos =
                               preccnNew->preccParent->ptlNonIconPos;
        }
        else
        {
          preccnNew->ptlNonIconPos.x = preccnNew->ptlNonIconPos.y = 0;
        }
      }
      /****************************************************************/
      /* If icon view and autoposition                                */
      /****************************************************************/
      else if ((pccinfo->flWindowAttr & CV_ICON ) &&
         (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
      {
        /**************************************************************/
        /* Initialize the position of the item to be inserted.        */
        /**************************************************************/
        if (preccnNew->preccPrevItem)
        {
          preccnNew->ptlIcon = preccnNew->preccPrevItem->ptlIcon;
        }
        else
        {
          preccnNew->ptlIcon.x = 0;
          preccnNew->ptlIcon.y= pccinfon->rctlWorkplace.top;
        }
        COPYNPTLTOX(preccnNew);
      }

      preccNew = preccNew->preccNextRecord;
    } /* endfor */

    /*****************************************************************/
    /* If the flInvalidateRecords flag is set, invalidate the records*/
    /*****************************************************************/
    if ((pRecordInsert->fInvalidateRecord) && preccnFirstInvalid &&
        (pccinfon->ulFlagSet2 & FL2_INITPAINT))
    {
      if (!(InvalidateForInsert(hwnd, hps, preccInsert,
                              preccnFirstInvalid, pRecordInsert,
                              flNoSiblings)))
      {
         ReleaseDC(hwnd, hps);
         return (0);                                                    /*DAB1*/
      }
    }
    ReleaseDC(hwnd, hps);
  }

  /*******************************************************************/
  /* Return the number of records in the container.                  */
  /*******************************************************************/
  return (pccinfo->cRecords);
}

/**********************************************************************/
/* Procedure: CalcTextSize - This procedure will calculate text size. */
/*                                                                    */
/* Inputs: hwndCnr - Initial container window handle to update        */
/*         preccnew- Pointer to new record to calculate size of       */
/*         iconsize- Size of icon for record                          */
/*                                                                    */
/* Output: If TRUE, view was updated successfully                     */
/**********************************************************************/
VOID FAR CalcTextSize (HANDLE hwndCnr, LPRECORDCOREN preccnNew,
                        POINT iconSize)
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  HDC         hps;
  DWORD       iconStyle;
  LPSTR       pszString;

  /********************************************************************/
  /* Check if there are any other views open                          */
  /********************************************************************/
  pccinfon = PCCINFONOF (hwndCnr);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Set the icon style. It is necessary to set the values for       */
  /* text view because the IcnCalcTextSize function requires the     */
  /* icon style and size.                                            */
  /*******************************************************************/
  preccnNew->ptlNonIconPos.x = 0;
  preccnNew->ptlNonIconPos.y = 0;

  if (!(pccinfo->flWindowAttr & CV_DETAIL))
  {
    if (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT | CV_TREE) )
    {
      iconStyle = IS_SIDETEXT;
      if (pccinfo->flWindowAttr & CV_TREE)
      {
        pszString = GETLPSTRFORTRV (preccnNew->preccX);
      }
      else if (pccinfo->flWindowAttr & CV_NAME )
      {
        pszString = GETLPSTRFORNV (preccnNew->preccX);
      }
      else
      {
        pszString = GETLPSTRFORTV (preccnNew->preccX);
      }
    }
    else if (pccinfo->flWindowAttr & CV_ICON)
    {
      iconStyle = IS_BENEATHTEXT;
      pszString = preccnNew->preccX->pszIcon;
    }
    /*****************************************************************/
    /* Get the presentation space for that window                    */
    /*****************************************************************/
    hps = PMGetFlippedDC (hwndCnr);

    /*****************************************************************/
    /* Calculate size of text string and store in control info.      */
    /*****************************************************************/
    IcnCalcTextSize (hps, iconSize, pccinfon, pszString, iconStyle,
                     &(preccnNew->slTextSize));

    /*****************************************************************/
    /* and release that presentation space                           */
    /*****************************************************************/
    ReleaseDC (hwndCnr, hps);
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     ConvertPRItoPRIN                               */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/**********************************************************************/
BOOL FAR ConvertPRItoPRIN(HANDLE hwnd, LPRECORDINSERT pRecordInsert)
{
  BOOL              fCheckedpRecordOrder = FALSE;
  LPCNRINFON        pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  LPRECORDINSERTN   pRecordInsertN = pccinfon->pRecordInsertN;

  pRecordInsertN->fInvalidateRecord = pRecordInsert->fInvalidateRecord;
  pRecordInsertN->cRecordsInsert = pRecordInsert->cRecordsInsert;
  pRecordInsertN->cb = pRecordInsert->cb;

  /*******************************************************************/
  /* Get pRecordParentN.                                             */
  /*******************************************************************/
  if ((pRecordInsert->pRecordOrder == (LPRECORDCORE)CMA_FIRST) ||
      (pRecordInsert->pRecordOrder == (LPRECORDCORE)CMA_END))
  {
    pRecordInsertN->pRecordParentN = GetRecordNFromX(hwnd,
                                      pRecordInsert->pRecordParent);
  }
  else
  {
    /*******************************************************************/
    /* We need proper pRecordOrder to get pRecordParentN               */
    /*******************************************************************/
    if ((GetWindowLong (hwnd, GWL_STYLE) & CCS_VERIFYPOINTERS) &&
        (!CnrVerifyPointers (hwnd,
                             (LPVOID)(&(pRecordInsert->pRecordOrder)),
                             1L)))
    {
      SETERROR(PMERR_INVALID_PARAMETERS);
      return ((DWORD)NULL);
    }
    fCheckedpRecordOrder = TRUE;
    pRecordInsertN->pRecordParentN =
      (GetRecordNFromX(hwnd, pRecordInsert->pRecordOrder))->preccParent;
  }

  /*******************************************************************/
  /* Get pRecordOrderN.                                              */
  /*******************************************************************/
  if (pRecordInsert->pRecordOrder == (LPRECORDCORE)CMA_END)
  {
    if (pRecordInsert->pRecordParent)
    {
      /***************************************************************/
      /* If the records to be inserted are not the root items,       */
      /* get the last item in the linked list of the siblings        */
      /* using a loop.                                               */
      /***************************************************************/
      pRecordInsertN->pRecordOrderN =
                       pRecordInsertN->pRecordParentN->preccFirstChild;
      if (pRecordInsertN->pRecordOrderN)
      {
        while (pRecordInsertN->pRecordOrderN->preccNextItem)
        {
          pRecordInsertN->pRecordOrderN =
                         pRecordInsertN->pRecordOrderN->preccNextItem;
        }
      }
    }
    else
    {
      /***************************************************************/
      /* If the records to be inserted are the root items,           */
      /* get the last item in the linked list of the siblings.       */
      /***************************************************************/
      pRecordInsertN->pRecordOrderN = pccinfon->preccLastItem;
    }
  }
  else
  {
    if (pRecordInsert->pRecordOrder == (LPRECORDCORE)CMA_FIRST)
    {
      /***************************************************************/
      /* Insert items before the first item in item order of the     */
      /* siblings                                                    */
      /***************************************************************/
      pRecordInsertN->pRecordOrderN = NULL;
    }
    else
    {
      /*******************************************************************/
      /* We need proper pRecordOrder to get pRecordParentN               */
      /*******************************************************************/
      if ((GetWindowLong (hwnd, GWL_STYLE) & CCS_VERIFYPOINTERS) &&
          !fCheckedpRecordOrder &&
          (!CnrVerifyPointers (hwnd,
                               (LPVOID)(&(pRecordInsert->pRecordOrder)),
                               1L)))
      {
        SETERROR(PMERR_INVALID_PARAMETERS);
        return ((DWORD)NULL);
      }
      fCheckedpRecordOrder = TRUE;

      /***************************************************************/
      /* Insert items after  pRecordInsert->pRecordOrder.            */
      /***************************************************************/
      pRecordInsertN->pRecordOrderN = GetRecordNFromX(hwnd,
                                        pRecordInsert->pRecordOrder);
    }
  }

  /*******************************************************************/
  /* Get zOrderN.                                                    */
  /*******************************************************************/
  if (pRecordInsert->zOrder == (WORD)CMA_BOTTOM)
  {
    /*****************************************************************/
    /* Insert items before the first item in Z order of the siblings */
    /*****************************************************************/
    pRecordInsertN->zOrderN = NULL;
  }
  else
  {
    /*****************************************************************/
    /* Took following if statement out because it should be default  */
    /*****************************************************************/
      if (pRecordInsert->pRecordParent)
      {
        /***************************************************************/
        /* If the records to be inserted are not the root items,       */
        /* get the last item in the linked list of the siblings        */
        /* using a loop.                                               */
        /***************************************************************/
        pRecordInsertN->zOrderN =
                        pRecordInsertN->pRecordParentN->preccFirstChild;

        if (pRecordInsertN->zOrderN)
        {
          while (pRecordInsertN->zOrderN->preccNextZ)
          {
            pRecordInsertN->zOrderN =
                                    pRecordInsertN->zOrderN->preccNextZ;
          }
        }
      }
      else
      {
        /***************************************************************/
        /* If the records to be inserted are the root items,           */
        /* get the last item in the linked list of the siblings.       */
        /***************************************************************/
        pRecordInsertN->zOrderN = pccinfon->preccZTail;
      } /* endif */
  }
  return TRUE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     InsertRecordsInChain                           */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/

#if defined(__IBMC__) || defined(__IBMCPP__)
typedef int (* CALLBACK PFNCOMP)(LPRECORDCORE, LPRECORDCORE, LPVOID);
#else
typedef int (FAR PASCAL *PFNCOMP)(LPRECORDCORE, LPRECORDCORE, LPVOID);
#endif

LPRECORDCOREN InsertRecordsInChain(hwnd, pRecordInsertN, preccInsert)
HANDLE           hwnd;
LPRECORDINSERTN  pRecordInsertN;
LPRECORDCORE     preccInsert;
{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCORE   preccItem;
  LPRECORDCOREN  preccnItem;
  LPRECORDCOREN  preccnInsert;
  LPRECORDCOREN  preccnItemPrev;
  LPRECORDCOREN  preccnItemNext;
  LPRECORDCOREN  preccnNew;
  LPRECORDCOREN  preccnParent = NULL;
  LPRECORDCOREN  preccnLastInsert = NULL;
  LPRECORDCOREN  preccnFirstInvalid = NULL;
  LPRECORDCOREN  preccnOldChild;
  PFNCOMP       pfnComp;
  WORD          i;
  BOOL          flInitFirstInView = FALSE;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (!pccinfon->preccItem)
  {
    flInitFirstInView = TRUE;
  } /* endif */

  /*******************************************************************/
  /* Get the parent of the item to be inserted.                      */
  /*******************************************************************/
  preccnParent = pccinfon->pRecordInsertN->pRecordParentN;

  /*******************************************************************/
  /* Save the first child of the parent to be used as preccItem      */
  /* in the root level linked list.                                  */
  /*******************************************************************/
  if (preccnParent)
  {
    preccnOldChild = preccnParent->preccFirstChild;
  }

  if (!(preccnItem = preccnInsert = CnrAllocRecordN(hwnd, preccInsert)))
  {
    return (NULL);
  }

  if ((pccinfo->flWindowAttr & CV_ICON) &&
      (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
  {
    pccinfon->ulFlagSet |= FL_ITEMNUMCHANGED;
  }

  preccItem = preccInsert;

  /*******************************************************************/
  /* If there is a sort function specified, we need to insert        */
  /* in the proper position in the linked list.                      */
  /*******************************************************************/
  if (pccinfo->pSortRecord)
  {
    PALPRECORDCOREN apRecord;
//  LPVOID          pSatPtr;

    /*****************************************************************/
    /* If more than one items are to be inserted, sort them first    */
    /* so that we can insert all of them in a loop.                  */
    /*****************************************************************/
    if (pRecordInsertN->cRecordsInsert > 1)
    {
//    pSatPtr = (LPVOID)GetWindowLong (hwnd, QWL_STORAGEPTR);
//    if (!(apRecord = (PALPRECORDCOREN)StmAllocateStorage (pSatPtr,
      if (!(apRecord = (PALPRECORDCOREN)GlobalAlloc( GPTR,              /*DAB1*/
        ((DWORD)pRecordInsertN->cRecordsInsert * sizeof(LPRECORDCOREN)))))
      {
         SETERROR(PMERR_INSUFFICIENT_MEMORY);
         return (NULL);
      }

      preccnItemPrev = NULL;

      for (i=0; i< pRecordInsertN->cRecordsInsert; i++)
      {
        (*apRecord)[i] = preccnItem;

        /*************************************************************/
        /* The last time through the loop set nextitem to NULL       */
        /* and do not call CnrAllocRecordN                           */
        /*************************************************************/
        if (i == (pRecordInsertN->cRecordsInsert-1) )
        {
          preccnItemNext = NULL;
        }
        else if (!(preccnItemNext = CnrAllocRecordN(hwnd,
                                       preccItem->preccNextRecord)))
        {
          return (NULL);
        }

        /*************************************************************/
        /* If more than 1 record is inserted, chain z linked list    */
        /* together. Then insert the block of records in the         */
        /* specified location.                                       */
        /*************************************************************/
        preccnItem->preccNextZ = preccnItemNext;
        preccnItem->preccPrevZ = preccnItemPrev;

        /*************************************************************/
        /* Set the parent of the item to be inserted.                */
        /*************************************************************/
        preccnItem->preccParent = preccnParent;

        /*************************************************************/
        /* Clean the first child field.                              */
        /*************************************************************/
        preccnItem->preccFirstChild = NULL;

        preccnItemPrev = preccnItem;
        preccItem = preccItem->preccNextRecord;
        preccnItem = preccnItemNext;
      }
      preccnLastInsert = preccnItemPrev;

      HeapSort(apRecord,
               (LONG)(pRecordInsertN->cRecordsInsert),
               (DWORD)pccinfo->pSortRecord, (DWORD)NULL);
    }
    else
    {
      /***************************************************************/
      /* Set the first item to be invalidated to refresh.            */
      /***************************************************************/
      preccnLastInsert = preccnInsert;
      preccnInsert->preccParent = preccnParent;

      /***************************************************************/
      /* Clean the first child field.                                */
      /***************************************************************/
      preccnInsert->preccFirstChild = NULL;
    }

    /*******************************************************************/
    /* Insert the block of linked list of the items in Z order.        */
    /*******************************************************************/
    if (!InsertRecord(hwnd, preccnInsert, preccnLastInsert,
                      pRecordInsertN->zOrderN, preccnParent, FALSE))
    {
      return NULL;
    }

    if (preccnParent)
    {
      /***************************************************************/
      /* If the records to be inserted are not the root items,       */
      /* insert them in the proper position of the linked list of    */
      /* the children of the preccParent.                            */
      /***************************************************************/
      preccnItem = preccnParent->preccFirstChild;
    }
    else
    {
      /***************************************************************/
      /* If the records to be inserted are the root items,           */
      /* insert them in the proper position of the linked list of    */
      /* the container.                                              */
      /***************************************************************/
      preccnItem = pccinfon->preccItem;
    } /* endif */

    preccnItemPrev = NULL;

    /*****************************************************************/
    /* Insert records in the proper positions.                       */
    /*****************************************************************/
    for (i = 0; i < pRecordInsertN->cRecordsInsert ; i++ )
    {
      if (pRecordInsertN->cRecordsInsert > 1)
      {
        preccnNew = (*apRecord)[i];
      }
      else
      {
        preccnNew = preccnInsert;
      }

      /***************************************************************/
      /* Set the first item to be invalidated to refresh.            */
      /***************************************************************/
      if (!preccnFirstInvalid && ISRECORDVISIBLE(preccnNew))
      {
        preccnFirstInvalid = preccnNew;
      }

      pfnComp = (PFNCOMP)pccinfo->pSortRecord;

      while (preccnItem &&
             ((*pfnComp)(preccnNew->preccX,
                         preccnItem->preccX, NULL) > 0))
      {
        preccnItemPrev = preccnItem;
        preccnItem = preccnItem->preccNextItem;
      } /* endwhile */

      if (!InsertRecord(hwnd, preccnNew, preccnNew, preccnItemPrev,
                        preccnParent, TRUE))
      {
        return NULL;
      }

      preccnItemPrev = preccnNew;
    } /* endfor */

    if (pRecordInsertN->cRecordsInsert > 1)
    {
//    StmFreeStorage (pSatPtr, (PALPRECORDCOREN)apRecord);
      GlobalFree( (PALPRECORDCOREN)apRecord );                          /*DAB1*/
    }
  }
  else        /* pSortRecord is not specified */
  {
    /*****************************************************************/
    /* If more than 1 record is inserted, chain the internal pointers*/
    /* together. Then insert the block of records in the specified   */
    /* location, first, end, or after a given record.  In those      */
    /* blocks of code, fix the head and tail pointers as needed.     */
    /*****************************************************************/
    if (pRecordInsertN->cRecordsInsert > 1)
    {
      preccnItemPrev = NULL;

      for (i=0; i< pRecordInsertN->cRecordsInsert; i++)
      {
        if (i < (WORD)(pRecordInsertN->cRecordsInsert - (WORD)1))
        {
          if (!(preccnItemNext =
                CnrAllocRecordN(hwnd, preccItem->preccNextRecord)))
          {
            return (NULL);
          }
        }
        else
        {
          preccnItemNext = NULL;
        }

        /*************************************************************/
        /* If more than 1 record is inserted, chain   linked lists   */
        /* together. Then insert the block of records in the         */
        /* specified location.                                       */
        /*************************************************************/
        preccnItem->preccNextItem = preccnItemNext;
        preccnItem->preccNextZ = preccnItemNext;

        preccnItem->preccPrevItem = preccnItemPrev;
        preccnItem->preccPrevZ = preccnItemPrev;

        /*************************************************************/
        /* Set the parent of the item to be inserted.                */
        /*************************************************************/
        preccnItem->preccParent = preccnParent;

        /*************************************************************/
        /* Clean the first child field.                              */
        /*************************************************************/
        preccnItem->preccFirstChild = NULL;

        /*************************************************************/
        /* Set the first item to be invalidated to refresh.          */
        /*************************************************************/
        if (!preccnFirstInvalid && ISRECORDVISIBLE(preccnItem))
        {
          preccnFirstInvalid = preccnItem;
        }

        preccnItemPrev = preccnItem;
        preccItem = preccItem->preccNextRecord;
        preccnItem = preccnItemNext;
      } /* endfor */
      preccnLastInsert = preccnItemPrev;
    }
    else
    {
      preccnLastInsert = preccnInsert;
      preccnInsert->preccParent = preccnParent;

      /***************************************************************/
      /* Clean the first child field.                                */
      /***************************************************************/
      preccnInsert->preccFirstChild = NULL;

      /***************************************************************/
      /* Set the first item to be invalidated to refresh.            */
      /***************************************************************/
      if (!preccnFirstInvalid && ISRECORDVISIBLE(preccnInsert))
      {
        preccnFirstInvalid = preccnInsert;
      }
    }

    /*******************************************************************/
    /* Insert the block of linked list of the items in Z order.        */
    /*******************************************************************/
    if (!InsertRecord(hwnd, preccnInsert, preccnLastInsert,
                      pRecordInsertN->zOrderN, preccnParent, FALSE))
    {
      return NULL;
    }

    if (!InsertRecord(hwnd, preccnInsert, preccnLastInsert,
                      pRecordInsertN->pRecordOrderN, preccnParent,
                      TRUE))
    {
      return NULL;
    }
  }

#if 0
  /*******************************************************************/
  /* Insert the block of linked list of the items in Z order.        */
  /*******************************************************************/
  if (!InsertRecord(hwnd, preccnInsert, preccnLastInsert,
                    pRecordInsertN->zOrderN, preccnParent, FALSE))
  {
    return NULL;
  }
#endif

  if (flInitFirstInView)
  {
    CheckFirstInView(hwnd);
  }

  return preccnFirstInvalid;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     InsertRecord                                   */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL InsertRecord(hwnd, preccnBegin, preccnEnd, preccnPrev, preccnParent,
                  flItemOrder)
HANDLE             hwnd;
LPRECORDCOREN      preccnBegin;
LPRECORDCOREN      preccnEnd;
LPRECORDCOREN      preccnPrev;
LPRECORDCOREN      preccnParent;
BOOL               flItemOrder;

{
  LPCNRINFON     pccinfon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);


  if (flItemOrder)
  {
    /*****************************************************************/
    /* Insert the record  next to preccPrev in item                  */
    /* order. If preccPrev is NULL insert it at the beginning.       */
    /* Code itself is self-explanatory, So, no more comments.        */
    /*****************************************************************/
    preccnBegin->preccPrevItem = preccnPrev ;

    if (preccnPrev)
    {
      preccnEnd->preccNextItem = preccnPrev->preccNextItem;
      if (preccnPrev->preccNextItem)
      {
        preccnPrev->preccNextItem->preccPrevItem = preccnEnd;
      }
      else
      {
        if (!preccnParent)     /* If inserting in root level */
        {
          pccinfon->preccLastItem = preccnEnd;
        }
      } /* endif */
      preccnPrev->preccNextItem = preccnBegin;
    }
    else
    {
      if (preccnParent)
      {
        preccnEnd->preccNextItem = preccnParent->preccFirstChild;
        if (preccnParent->preccFirstChild)
        {
          preccnParent->preccFirstChild->preccPrevItem = preccnEnd;
        }
        preccnParent->preccFirstChild = preccnBegin;
      }
      else
      {
        preccnEnd->preccNextItem = pccinfon->preccItem ;
        if (pccinfon->preccItem)
        {
          pccinfon->preccItem->preccPrevItem = preccnEnd;
        }
        else
        {
          pccinfon->preccLastItem = preccnEnd;
        } /* endif */
        pccinfon->preccItem = preccnBegin;
      }
    }
  }
  else
  {
    /*****************************************************************/
    /* Insert the record to be moved to next to preccPrev in item    */
    /* order. If preccPrev is NULL insert it at the beginning.       */
    /* Code itself is self-explanatory, So, no more comments.        */
    /*****************************************************************/
    preccnBegin->preccPrevZ = preccnPrev;

    if (preccnPrev)
    {
      preccnEnd->preccNextZ = preccnPrev->preccNextZ;
      if (preccnPrev->preccNextZ)
      {
        preccnPrev->preccNextZ->preccPrevZ = preccnEnd;
      }
      else
      {
        if (!preccnParent)
        {
          pccinfon->preccZTail = preccnEnd;
        }
      } /* endif */
      preccnPrev->preccNextZ = preccnBegin;

    }
    else
    {
      if (!preccnParent)
      {
        preccnEnd->preccNextZ = pccinfon->preccZ;
        if (pccinfon->preccZ)
        {
          pccinfon->preccZ->preccPrevZ = preccnEnd;
        }
        else
        {
          pccinfon->preccZTail = preccnEnd;
        } /* endif */
        pccinfon->preccZ = preccnBegin;
      }
      else
      {
        LPRECORDCOREN  preccnFirstZChild = preccnParent->preccFirstChild;

        if (preccnFirstZChild)
        {
          while (preccnFirstZChild->preccPrevZ)
          {
            preccnFirstZChild = preccnFirstZChild->preccPrevZ;
          }
        }
        preccnEnd->preccNextZ = preccnFirstZChild;
        if (preccnFirstZChild)
        {
          preccnFirstZChild->preccPrevZ = preccnEnd;
        }
      }
    } /* endif */
  } /* endif */

  return TRUE;

}
/**********************************************************************/
/*                                                                    */
/*  Function Name:     InvalidateForInsert                            */
/*                                                                    */
/*  Descriptive Name:   Control Invalidate Record to Refresh          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR InvalidateForInsert(hwnd, hps, pRecordCore, preccnFirstInvalid,
                             pRecordInsert, flNoSiblings)
HANDLE         hwnd;
HDC            hps;
LPRECORDCORE   pRecordCore;
LPRECORDCOREN  preccnFirstInvalid;
LPRECORDINSERT pRecordInsert;
BOOL           flNoSiblings;
{
  LPCNRINFON       pccinfon;
  LPCNRINFO        pccinfo;
  BOOL             invalidateAll = FALSE;
  RECT             rctlInvalid;
  WORD             i;
  LPRECORDCOREN    precordn;          /* record to be invalidated       */
  LPRECORDCOREN    preccnFirstInLevel;
  LPRECORDCORE     precord;
  RECT             rctlText;
  RECT             rctlTextCur;
  POINT            iconSize;
  POINT            slTreeIcon;
  WORD             usOldLongestLine;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  pccinfon->ulFlagSet &= ~( FL_CNRPOSITION | FL_POSITIONCOLTITLES |
                       FL_SIZECOLTITLES | FL_SIZECHILDREN );

  if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
  {
    pccinfon->ulFlagSet |= FL_NEEDARRANGE;
  }

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    LPRECORDCOREN   preccnParent;

    /*****************************************************************/
    /* Get the parent of the item to be inserted.                    */
    /*****************************************************************/
    preccnParent = pccinfon->pRecordInsertN->pRecordParentN;

    if (preccnParent && !ISRECORDVISIBLE(preccnParent))
    {
      pccinfon->ulFlagSet |= (FL_CNRPOSITION | FL_SIZECHILDREN);
      return TRUE;
    }

    CnrCalcIconSize(hwnd, &iconSize);
    CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);

    /*****************************************************************/
    /*If the parent of the inserted item is visible, collapsed and   */
    /*didn't have children before, it need to be repainted to display*/
    /*collapsed icon.                                                */
    /*****************************************************************/
    if (preccnParent)
    {
      if (IsTreeVisible(preccnParent))
      {
        if (preccnParent->flRecordAttr & CRA_COLLAPSED)
        {
          /***********************************************************/
          /* We don't need to reposition nor resize children.        */
          /***********************************************************/
          pccinfon->ulFlagSet |= (FL_CNRPOSITION | FL_SIZECHILDREN);

          /***********************************************************/
          /* If this is the first child of the preccnParent,         */
          /* we need to paint tree icon for the tree icon/text view  */
          /* or icon for the tree name view.                         */
          /***********************************************************/
          if (flNoSiblings)
          {
            RECT        rctlIcon;
            RECT        rctlTreeIcon;
            RECT        rctlText;
            RECT        rctlErase;
            int         cxTreeLine;

            CnrTRVQueryIconTextRect(hwnd, preccnParent, FALSE,
                                    iconSize, slTreeIcon, &rctlIcon,
                                    &rctlTreeIcon, &rctlText);


            if (pccinfo->flWindowAttr & CV_NAME)
            {
              CnrEraseRect(hwnd, hps, rctlIcon);
            }
            else
            {
            /*********************************************************/
            /* If there was line before, we need to erase the line   */
            /* from the left of the tree icon to the left of the icon*/
            /* (for tree icon view) or the text (for the text view). */
            /* we don't need to erase any for the tree name view.    */
            /*********************************************************/
              if (pccinfo->flWindowAttr & CA_TREELINE)
              {
                cxTreeLine = (pccinfo->cxTreeLine < 0) ? 1 :
                                                   pccinfo->cxTreeLine;
                if (pccinfo->flWindowAttr & CV_TEXT)
                {
                  rctlErase.right = rctlText.left - ICN_SELSPACE;
                }
                else
                {
                  rctlErase.right = rctlIcon.left - ICN_EMPHSPACE;
                }

                rctlErase.left = rctlTreeIcon.left;
                rctlErase.bottom = (rctlTreeIcon.top +
                                     rctlTreeIcon.bottom) / 2 +
                                     cxTreeLine / 2;
                rctlErase.top = rctlErase.bottom - cxTreeLine;
                CnrEraseRect(hwnd, hps, rctlErase);
              }
            }

            /*********************************************************/
            /* Draw the icon for the tree name view or the tree icon */
            /* for the tree icon/text view.                          */
            /*********************************************************/
            if (!(CnrTRVDrawItem(hwnd, hps, preccnParent,
                                 rctlIcon, rctlTreeIcon, rctlText,
                          (WORD)((pccinfo->flWindowAttr & CV_NAME) ?    /*DAB1*/
                                          DE_ICON : DE_TREEICON))))     /*DAB1*/
            {
              return (0);
            }
          }
          return TRUE;
        }
      }
      else
      {
        /*************************************************************/
        /* Because parent is not visible, we don't need to do        */
        /* anything.                                                 */
        /*************************************************************/
        pccinfon->ulFlagSet |= (FL_CNRPOSITION | FL_SIZECHILDREN);
        return TRUE;
      }
    }

#if 0
    /*****************************************************************/
    /*If the parent of the inserted items is collapsed, we don't     */
    /*need refresh the window.                                       */
    /*****************************************************************/
    while (preccnParent)
    {
      if (preccnParent->flRecordAttr & CRA_COLLAPSED)
      {
        return TRUE;
      }
      preccnParent = preccnParent->preccParent;
    }
#endif

    /***************************************************************/
    /*If the item is after the first item in the view, we don't    */
    /*need refresh whole window. Instead, draw from the first item */
    /*inserted.                                                    */
    /***************************************************************/
    if (!(Before1stInView(hwnd, preccnFirstInvalid)))
    {
      if (preccnParent) /* not in the root level */
      {
        preccnFirstInLevel = preccnParent->preccFirstChild;
      }
      else  /* in the root level */
      {
        preccnFirstInLevel = pccinfon->preccItem;
      } /* endif */

      if (preccnFirstInLevel)
      {
        usOldLongestLine = preccnFirstInLevel->usLongestLine;
      }
      else
      {
        usOldLongestLine = 0;
      }

      if (!(CnrTRVSetUpNewView (hwnd, hps)))
      {
        return (FALSE);
      }

      if ((pccinfo->flWindowAttr & CA_TREELINE) &&
          (preccnFirstInvalid->preccParent) )
      {
        LPRECORDCOREN  preccnPrev;
        /************************************************************/
        /* We must go to the previous sibling or the parent and     */
        /* draw the tree line since this has not been painted yet   */
        /* down as far as this new child being inserted.            */
        /* This change optimizes the painting(flashing).            */
        /************************************************************/
        preccnPrev = GetPrevUnfilteredItem(preccnFirstInvalid);
        if ( preccnPrev )
        {
          preccnFirstInvalid = preccnPrev;
        }
        else
        {
          preccnFirstInvalid = preccnFirstInvalid->preccParent;
        }
      }

      TRVGetInvalidateRect(hwnd, preccnFirstInvalid, &rctlInvalid);

      /*************************************************************/
      /* If the items inserted causes the change of the longest    */
      /* line of the siblings in the tree view we need to          */
      /* redraw the selected or cursored items which are before    */
      /* the first inserted item if they are in viewport.          */
      /*************************************************************/
      if (preccnFirstInLevel && (usOldLongestLine !=
                                 preccnFirstInLevel->usLongestLine))
      {
        precordn = preccnFirstInLevel;
        while ((precordn != preccnFirstInvalid) &&
              (precordn->ptlNonIconPos.y < rctlInvalid.top))
        {
          if (ISRECORDVISIBLE(precordn))
          {
            CnrTRVQueryIconTextRect(hwnd, precordn, FALSE, iconSize,
                                    slTreeIcon, NULL, NULL, &rctlText);

            if (precordn->flRecordAttr & CRA_SELECTED)
            {
              CnrDrawTextOnly(hwnd, hps, precordn,
                              DRAW_TEXT,
                              rctlText, rctlText);
            }
          } /* endif */
          precordn = precordn->preccNextItem;

        } /* endwhile */
      }
    }
    else
    {
      pccinfon->ulFlagSet &= ~FL_NEWVIEWSCROLLSET;
      invalidateAll = TRUE;
    }
  }
  else
  {
    if ((pccinfo->flWindowAttr & CV_ICON) &&
        !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
    {
      pccinfon->ulFlagSet &= ~FL_CNRPOSITION;

      /***************************************************************/
      /*Set up the window (sizing add setting scroll bars updating   */
      /*workplaces, etc) if necessary.                               */
      /***************************************************************/
      CnrSetUpNewView(hwnd, hps);

      /***************************************************************/
      /*If the application wants to put new items under the existing */
      /*items refresh whole window.                                  */
      /***************************************************************/
      if (pRecordInsert->zOrder == (WORD)CMA_BOTTOM)
      {
        invalidateAll = TRUE;
      }
      else
      {
        /*************************************************************/
        /*If the application wants to put new items on the top of the*/
        /*existing items just draw new items only.                   */
        /*************************************************************/
        precordn = GetRecordNFromX (hwnd, pRecordCore);
        precord = pRecordCore;
        for (i = 0; i < pRecordInsert->cRecordsInsert ; i++)
        {
          EraseTextOnlyInIV(hwnd, hps, precordn);
          if (!(CnrDrawIcon( hwnd, hps, precordn, FALSE)))
          {
             return (FALSE);
          }
          precord = precord->preccNextRecord;
          precordn = GetRecordNFromX (hwnd, precord);
        } /* endfor */

        rctlInvalid.left = rctlInvalid.right =
        rctlInvalid.bottom = rctlInvalid.top = 0;
      }
    }
    else
    {
      /***************************************************************/
      /*If the item is after the first item in the view, we don't    */
      /*need refresh whole window. Instead, draw from the first item */
      /*inserted.                                                    */
      /***************************************************************/
      if ((pRecordInsert->pRecordOrder == (LPRECORDCORE)CMA_END) ||
          (pccinfon->pRecordInsertN->pRecordOrderN ==
           pccinfon->preccLastItem) ||
          !(Before1stInView(hwnd, preccnFirstInvalid)))
      {
        if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
            !(pccinfo->flWindowAttr & CV_FLOW) &&
            pccinfon->preccItem )
        {
          usOldLongestLine = pccinfon->preccItem->usLongestLine;
        }

        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
           if (!(CnrSetUpNewViewForDV (hwnd, hps)))
           {
              return (FALSE);
           }
        }
        else
        {
          CnrSetUpNewView (hwnd, hps);
        }

        GetInvalidateRect (hwnd, preccnFirstInvalid, &rctlInvalid);

        /*************************************************************/
        /* If the items inserted causes the change of the longest    */
        /* line of the column in non_flowed Name/Text view we need to*/
        /* redraw the selected or cursored items which are before    */
        /* the first inserted item if they are in viewport.          */
        /*************************************************************/
        if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
            !(pccinfo->flWindowAttr & CV_FLOW) &&
            pccinfon->preccItem &&
            (usOldLongestLine !=
             pccinfon->preccItem->usLongestLine))
        {
          CnrCalcIconSize(hwnd, &iconSize);
          precordn = GetFirstItemInView(hwnd);

          while ((precordn != preccnFirstInvalid) &&
                 (precordn->ptlNonIconPos.y <
                  rctlInvalid.top))
          {
            if (ISRECORDVISIBLE(precordn))
            {
              CnrQueryIconTextRect(hwnd, precordn, &iconSize,
                                   FALSE, NULL, &rctlText);

              rctlTextCur.left = rctlText.left - TXT_TOPMARGIN;
              rctlTextCur.right = rctlText.right + TXT_TOPMARGIN;
              rctlTextCur.bottom = rctlText.bottom + TXT_TOPMARGIN;
              rctlTextCur.top = rctlText.top;


              if ((precordn->flRecordAttr & CRA_SELECTED) &&
                  !((pccinfo->flWindowAttr & CV_TEXT) &&
                    (precordn->flRecordAttr & CRA_CURSORED)))
              {
                CnrDrawTextOnly(hwnd, hps, precordn,
                                DRAW_TEXT,
                                rctlText, rctlTextCur);
              }
              else
              {
                if ((pccinfo->flWindowAttr & CV_TEXT) &&
                    (precordn->flRecordAttr & CRA_CURSORED))
                {
                  CnrEraseRect(hwnd, hps, rctlTextCur);
                  CnrDrawTextOnly(hwnd, hps, precordn,
                                  DRAW_TEXT | DRAW_CURSOR,
                                  rctlText, rctlTextCur);
                }
              }
            }
            precordn = precordn->preccNextItem;

          } /* endwhile */
        }
      }
      else
      {
        pccinfon->ulFlagSet &= ~FL_NEWVIEWSCROLLSET;

        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
           if (!(CnrSetUpNewViewForDV (hwnd, hps)))
           {
              return (FALSE);
           }
        }
        else
        {
           CnrSetUpNewView (hwnd, hps);
        }

        invalidateAll = TRUE;
      }
    }
  }
  InvalidateContainer( hwnd, invalidateAll, &rctlInvalid );
  return (TRUE);
}


/**********************************************************************/
/* Procedure: InvalidateContainer - This procedure will invalidate    */
/*            the area specified depending on view of the container   */
/*            and region to be invalidated.                           */
/*                                                                    */
/* Inputs: hwndCnr - Initial container window handle to update        */
/*                                                                    */
/* Output: If TRUE, view was updated successfully                     */
/**********************************************************************/
VOID FAR InvalidateContainer (HANDLE hwnd, BOOL bInvalidateAll,
                              LPRECT prctlInvalid)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT       RightWndRect;
  RECT       CnrRect;

  pccinfon = PCCINFONOF( hwnd );
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (bInvalidateAll)
  {
    InvalidateChildren (hwnd, TRUE, FALSE);
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
      if (CnrAllColumnsFixedWidth( hwnd ))
      {
        PMInvalidateRect( GetDlgItem( hwnd, CID_LEFTDVWND ),
                           prctlInvalid, FALSE );
        UpdateWindow( GetDlgItem( hwnd, CID_LEFTDVWND ));

        /************************************************************/
        /* If there is a right window, we need to invalidate it too.*/
        /************************************************************/
        if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
        {
          PMGetClientRect( GetDlgItem( hwnd, CID_RIGHTDVWND ),
                              &RightWndRect);

          prctlInvalid->right = RightWndRect.right;

          PMInvalidateRect( GetDlgItem( hwnd, CID_RIGHTDVWND ),
                             prctlInvalid, FALSE);

          UpdateWindow( GetDlgItem( hwnd, CID_RIGHTDVWND ));
        }
      }
      /****************************************************************/
      /* We need to repaint the whole window because the inserted     */
      /* record could have changed the width of one of the columns    */
      /* which will affect everything.                                */
      /****************************************************************/
      else
      {
        PMGetClientRect (hwnd, &CnrRect);

        if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
            ((long)pccinfo->xVertSplitbar > CnrRect.right))
        {
           CnrDetailsVCtlProc (hwnd, WM_PAINT, 0, 0);                   /*DAB1*/
        }

        InvalidateRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                        NULL, FALSE);
        InvalidateRect (GetDlgItem (hwnd, CID_RIGHTDVWND),
                        NULL, FALSE);
        InvalidateRect (GetDlgItem (hwnd, CID_LEFTCOLTITLEWND),
                        NULL, FALSE);                                 /* LJO1 */
        InvalidateRect (GetDlgItem (hwnd, CID_RIGHTCOLTITLEWND),
                        NULL, FALSE);                                 /* LJO1 */
        InvalidateRect (hwnd, NULL, TRUE);  /* Splitbar update */
        UpdateWindow (hwnd);
      }
    }
    else
    {
      PMInvalidateRect (hwnd, prctlInvalid, TRUE);
      UpdateWindow (hwnd);
    }
  }
  return;
}

/*********************************************************************/
/* FUNCTION NAME:  GetInvalidateRect                                 */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
VOID FAR GetInvalidateRect(HANDLE hwnd, LPRECORDCOREN preccInvalid,
                           LPRECT prctlInvalid)

{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT       rctlHScroll;
  RECT       rctlTitle;
  POINT      iconSize;
  long       CyBetweenRows;
  long       ItemHeight;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (!(pccinfo->flWindowAttr & CV_DETAIL))
  {
    /******************************************************************/
    /* Get viewport rect, H scroll bar rect and icon size             */
    /* left-top point of a rectangle is (0, 0)                        */
    /******************************************************************/
    CnrQuerySizeRect(hwnd, &iconSize, &rctlTitle, prctlInvalid, NULL,
                     &rctlHScroll) ;

    prctlInvalid->top += rctlTitle.bottom;
    prctlInvalid->bottom += rctlTitle.bottom;
  }

  if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
      (pccinfo->flWindowAttr & CV_FLOW) )
  {
    prctlInvalid->left = preccInvalid->ptlNonIconPos.x -
                       ICN_EMPHSPACE -
                       ICN_SELSPACE - pccinfon->ptlNonIconOrg.x;
    if (prctlInvalid->left > prctlInvalid->right)
    {
      prctlInvalid->left = prctlInvalid->right;
    } /* endif */
  }
  else
  {
    if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
        !(pccinfo->flWindowAttr & CV_FLOW) )
    {
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        ItemHeight = max (iconSize.y,
                          preccInvalid->slTextSize.cy);                 /*DAB1*/
      }
      else
      {
        ItemHeight = preccInvalid->slTextSize.cy;                       /*DAB1*/
      }

      prctlInvalid->top = (int)(preccInvalid->ptlNonIconPos.y -
                   ICN_EMPHSPACE - ICN_SELSPACE - pccinfon->ptlNonIconOrg.y);

      if (prctlInvalid->bottom < prctlInvalid->top)
      {
        prctlInvalid->bottom = prctlInvalid->top;
      } /* endif */
    }
    else
    {
      if ((pccinfo->flWindowAttr & CV_ICON) &&
          (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
      {
        prctlInvalid->top = preccInvalid->ptlIcon.y -
                           ICN_EMPHSPACE - ICN_SELSPACE -
                           pccinfo->ptlOrigin.y;
        if (prctlInvalid->bottom < prctlInvalid->top)
        {
          prctlInvalid->bottom = prctlInvalid->top;
        } /* endif */
      }
      else
      {
        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
           /***********************************************************/
           /* Get the distance between the rows.                      */
           /***********************************************************/
           if (pccinfo->cyLineSpacing >= 0)
           {
              CyBetweenRows = pccinfo->cyLineSpacing;
           }
           else
           {
              CyBetweenRows = MINCY_BETWEENITEMS;
           }

           if (CyBetweenRows & NUM_ODD)
           {
              CyBetweenRows++;
           }

           PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                               prctlInvalid);

           /***********************************************************/
           /* Get the top of the record that we are invalidating and  */
           /* convert the value to be relative to the window.         */
           /***********************************************************/
           prctlInvalid->top = (int)(CnrQueryRowPosInDV (preccInvalid,
                  PT_TOP) - (CyBetweenRows / 2) - pccinfon->ptlNonIconOrg.y);

           if (prctlInvalid->bottom < prctlInvalid->top)
           {
             prctlInvalid->bottom = prctlInvalid->top;
           }
        }
      }
    }
  }
  return ;
}

/*********************************************************************/
/* FUNCTION NAME:  TRVGetInvalidateRect                              */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
VOID FAR TRVGetInvalidateRect(HANDLE hwnd, LPRECORDCOREN preccInvalid,
                              LPRECT prctlInvalid)

{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccPrev;
  RECT          rctlHscroll;
  RECT          rctlIcon;
  RECT          rctlTreeIcon;
  POINT         slIcon;
  POINT         slTreeIcon;
  RECT          rctlLTitle;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size               */
  /* left-bottom point of a rectangle is (0, 0)                       */
  /********************************************************************/
  CnrQuerySizeRect(hwnd, &slIcon,  &rctlLTitle, prctlInvalid, NULL,
                   &rctlHscroll) ;

  CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);

  prctlInvalid->top += rctlLTitle.bottom;
  prctlInvalid->bottom += rctlLTitle.bottom;

  preccPrev = GetPrevVisualItem( hwnd, preccInvalid);
  if (preccPrev)
  {
    CnrTRVQueryIconTextRect(hwnd, preccPrev, FALSE, slIcon,
                            slTreeIcon, &rctlIcon, &rctlTreeIcon,
                            NULL);
    prctlInvalid->top = (pccinfo->flWindowAttr & CV_NAME) ?
                               rctlIcon.top : rctlTreeIcon.top;
  }

  if (prctlInvalid->bottom < prctlInvalid->top)
  {
    prctlInvalid->bottom = prctlInvalid->top;
  }
  return ;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrSetUpNewView                                */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
VOID FAR CnrSetUpNewView (hwndCnr, hps)
HANDLE hwndCnr;
HDC    hps;
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  RECT       rctlWin;
//HANDLE     hwndFrame;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*---------------------------------------------*/
  /* if the container is in a frame, and the     */
  /* frame isn't minimized, reposition items now */
  /*---------------------------------------------*/
  if (!(IsIconic (hwndCnr)))
  {
    if (!(pccinfon->ulFlagSet & FL_CNRPOSITION))
    {
      if (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT))
      {
        CnrPositionNameTextView(hwndCnr, hps, NULL);

        /*************************************************************/
        /* Make sure FL_CNRWINOPEN is on(for the case of the         */
        /* initial painting without sending WM_SIZE                  */
        /*************************************************************/
        pccinfon->ulFlagSet &= ~(FL_SETSCROLL | FL_SIZECHILDREN);
        pccinfon->ulFlagSet |= FL_CNRPOSITION;
      }
      else
      {
        if (pccinfo->flWindowAttr & CV_ICON)
        {
          pccinfon->ulFlagSet |= FL_CNRPOSITION;
        }
      }
    } /* endif */

    if ((pccinfo->flWindowAttr & CV_ICON) &&
        (pccinfon->ulFlagSet & FL_NEEDARRANGE))
    {
      /****************************************************************/
      /* If FL_NEEDARRANGE is on arrange the icons by calling         */
      /* CnrArrangeIconView(in cnrgen.c).                             */
      /****************************************************************/
      CnrArrangeIconView(hwndCnr, hps);
      pccinfon->ulFlagSet &= ~(FL_SETSCROLL | FL_NEEDARRANGE );
      pccinfon->ulFlagSet |= ( FL_SETWORKPLACE | FL_TEXTCALC );
    } /* endif */

    if ((pccinfo->flWindowAttr & CV_ICON) &&
        !(pccinfon->ulFlagSet & FL_TEXTCALC))
    {
      CalcTextSizeInIV(hwndCnr, hps);
      pccinfon->ulFlagSet |= FL_TEXTCALC;
    }

    if ((pccinfo->flWindowAttr & CV_ICON) &&
        CheckNeedResize(hwndCnr))
    {
      pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
    }

    if (!(pccinfon->ulFlagSet & FL_SIZECHILDREN))
    {
      /*************************************************************/
      /* resize the scroll bars                                    */
      /*************************************************************/
      PMGetClientRect(hwndCnr, &rctlWin);

      CnrSize(hwndCnr, rctlWin.right, rctlWin.bottom);

      /*************************************************************/
      /* Set the scroll bar                                        */
      /*************************************************************/
      pccinfon->ulFlagSet &= ~FL_SETSCROLL;
      pccinfon->ulFlagSet |= FL_SIZECHILDREN;
    } /* endif */

    if (!(pccinfon->ulFlagSet & FL_SETSCROLL)) {
      if (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT))
      {
        /**************************************************************/
        /* Set the scroll bar                                         */
        /**************************************************************/
        CnrSetNameTextViewScroll(hwndCnr, FALSE);
        pccinfon->ulFlagSet |= FL_SETSCROLL;
      }
      else
      {
        if (pccinfo->flWindowAttr & CV_ICON)
        {
          /************************************************************/
          /* Set the scroll bar                                       */
          /************************************************************/
          CnrSetIconViewScroll(hwndCnr);
          pccinfon->ulFlagSet |= FL_SETSCROLL;
        }
      }
    } /* endif */
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrTRVSetUpNewView                             */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrTRVSetUpNewView (hwndCnr, hps)
HANDLE hwndCnr;
HDC    hps;
{
  LPCNRINFON  pccinfon;
  RECT       rctlWin;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);

  /*---------------------------------------------*/
  /* if the container is in a frame, and the     */
  /* frame isn't minimized, reposition items now */
  /*---------------------------------------------*/
  if (!(IsIconic(hwndCnr)))
  {
    if (!(pccinfon->ulFlagSet & FL_CNRPOSITION))
    {
      CnrPositionTreeView(hwndCnr, hps, NULL);
      pccinfon->ulFlagSet &= ~(FL_SIZECHILDREN | FL_SETSCROLL);
      pccinfon->ulFlagSet |= (FL_CNRPOSITION | FL_TEXTCALC);

    } /* endif */

    if (TRVCheckNeedResize(hwndCnr))
    {
      pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
    }

    if (!(pccinfon->ulFlagSet & FL_SIZECHILDREN))
    {
      /***************************************************************/
      /* resize the scroll bars                                      */
      /***************************************************************/
      PMGetClientRect(hwndCnr, &rctlWin);

      CnrSize(hwndCnr, rctlWin.right, rctlWin.bottom);

      /***************************************************************/
      /* Set the scroll bar                                          */
      /***************************************************************/
      pccinfon->ulFlagSet &= ~FL_SETSCROLL;
      pccinfon->ulFlagSet |= FL_SIZECHILDREN;
    } /* endif */

    if (!(pccinfon->ulFlagSet & FL_SETSCROLL))
    {
        /*************************************************************/
        /* Set the scroll bar                                        */
        /*************************************************************/
        CnrTRVSetScroll(hwndCnr);
        pccinfon->ulFlagSet |= FL_SETSCROLL;
    } /* endif */
  }
  return TRUE;
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrSetUpNewViewForDV                           */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/*  Programmer: Peter Haggar                                          */
/**********************************************************************/
BOOL FAR CnrSetUpNewViewForDV (hwndCnr, hps)

HANDLE  hwndCnr;
HDC   hps;

{
 LPCNRINFON  pccinfon;
 LPCNRINFO   pccinfo;
 RECT       Rect;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 PMGetClientRect (hwndCnr, &Rect);

 if (!(IsIconic(hwndCnr)))
 {
    if (!(pccinfon->ulFlagSet & FL_POSITIONCOLTITLES))
    {
       if (pccinfo->flWindowAttr & CA_DETAILSVIEWTITLES)
       {
          CnrPositionColumnTitles (hwndCnr, hps);
          pccinfon->ulFlagSet |= FL_POSITIONCOLTITLES;
          pccinfon->ulFlagSet &= ~(FL_SIZECOLTITLES | FL_CNRPOSITION );
       }
    }

    /*********************************************************************/
    /* If this flag is not set, we have to resize the column             */
    /* titles.                                                           */
    /*********************************************************************/
    if (!(pccinfon->ulFlagSet & FL_SIZECOLTITLES))
    {
       CnrSizeColumnTitleWindows (hwndCnr, Rect.right, Rect.bottom);

       /******************************************************************/
       /* Set the flag so it is off.  I know it's backwards,             */
       /* I hate it too, but I had no choice...I had to                  */
       /* follow the convention with the rest of the code.               */
       /******************************************************************/
       pccinfon->ulFlagSet |= FL_SIZECOLTITLES;
       pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
    }

    if (!(pccinfon->ulFlagSet & FL_CNRPOSITION))
    {
       if (!(CnrPositionDetailsView (GetDlgItem (hwndCnr,
                                         CID_LEFTDVWND),
                                         hps,
                                         NULL)))
       {
          return (FALSE);
       }

       pccinfon->ulFlagSet |= FL_CNRPOSITION;
       pccinfon->ulFlagSet &= ~FL_SETSCROLL;
    }

    /*********************************************************************/
    /* Set up the Scroll bars in the correct positions.                  */
    /*********************************************************************/
    if (!(pccinfon->ulFlagSet & FL_SETSCROLL))
    {
       CnrSetDetailsViewScroll (hwndCnr);
       pccinfon->ulFlagSet |= FL_SETSCROLL;
    }
 }

 return (TRUE);
}


/**********************************************************************/
/* Procedure: InvalidateChildren - This procedure will invalidate     */
/*            all child windows depending on view of the container    */
/*            and region to be invalidated.                           */
/*                                                                    */
/* Inputs: hwnd - Initial container window handle to update           */
/*         bEraseBackground - erase background flag                   */
/*         bSizingDetails   - sizing details view window              */
/*                                                                    */
/* Output: None                                                       */
/**********************************************************************/
VOID FAR InvalidateChildren (HANDLE hwnd, BOOL bEraseBackground,
                             BOOL bSizingDetails)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT       CnrRect;

  pccinfon = PCCINFONOF( hwnd );
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
    PMGetClientRect (hwnd, &CnrRect);

    /******************************************************************/
    /* If the window is unsplit, or the splitbar is not visible, ie.  */
    /* to the right of the window, we need to explicitly send the     */
    /* WM_PAINT message to the control proc.                          */
    /******************************************************************/
    if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
        ((long)pccinfo->xVertSplitbar > CnrRect.right) &&
        (!bSizingDetails))
    {
       CnrDetailsVCtlProc (hwnd, WM_PAINT, 0, 0);
    }

    /* Invalidate Splitbar */
    InvalidateRect (hwnd, NULL, bEraseBackground);
    InvalidateRect (GetDlgItem (hwnd,
                     CID_LEFTDVWND),
                     NULL,
                     bEraseBackground);
    InvalidateRect (GetDlgItem (hwnd,
                     CID_RIGHTDVWND),
                     NULL,
                     bEraseBackground);
    if (pccinfo->flWindowAttr & CA_DETAILSVIEWTITLES)
    {
      InvalidateRect (GetDlgItem (hwnd,
                       CID_LEFTCOLTITLEWND),
                       NULL,
                       bEraseBackground);
      InvalidateRect (GetDlgItem (hwnd,
                       CID_RIGHTCOLTITLEWND),
                       NULL,
                       bEraseBackground);
    }

  }
  else
  {
    InvalidateRect( hwnd, NULL, bEraseBackground );
  }

  if (pccinfo->flWindowAttr & CA_CONTAINERTITLE)
  {
    InvalidateRect( GetDlgItem( hwnd, CID_CNRTITLEWND ),
                    NULL, bEraseBackground);
  }

  InvalidateRect( GetDlgItem( hwnd, CID_BLANKBOX ),
                  NULL, bEraseBackground);

  if (bEraseBackground)
    UpdateWindow( hwnd );

  return;
}

