/* Revision: 17 1.2.2.2 source/ui/cclnt/cnr/cnrs2p6.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS2P6.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS2P6.C - Allocation routines for container control    */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/**********************************************************************/
#include <windows.h>
#include <string.h>            /* for memcpy                          */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef SEGMENTOR
#define CNRS2P6_C
#include "cnrpragw.h"
#endif

/**********************************************************************/
/*                                                                    */
/*  Function Name:  GetpRecordNFromX                                  */
/*                                                                    */
/*  Description:    Returns the internal record structure for the     */
/*                  given record for the container window specified   */
/*                  by hwndCnr.                                       */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                  pRecord  - record for which internal record is    */
/*                             requested.                             */
/*                                                                    */
/*  Output:         LPRECORDCOREN - pointer to the internal record for */
/*                  the external record that matches the container    */
/*                  handle passed in.                                 */
/*                                                                    */
/*  Programmer: Peter Haggar                                          */
/**********************************************************************/
LPRECORDCOREN FAR GetRecordNFromX (hwndCnr, pRecord)

HANDLE         hwndCnr;
LPRECORDCORE  pRecord;
{
 LPRECORDCOREN  preccN;

 if (!pRecord)
 {
   return (NULL);
 }

 /*********************************************************************/
 /* Get pointer to the (if any) existing internal recordcore.         */
 /*********************************************************************/
 preccN = PRECCFIRSTNFROMX(pRecord);

 /*********************************************************************/
 /* Loop through the internal recordcore control blocks until we find */
 /* one that has the same handle as the handle passed in.             */
 /*********************************************************************/
 while ((preccN) && (hwndCnr != preccN->hwndCnr))
 {
   preccN = preccN->preccnNext;
 }

 return (preccN);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrAllocRecordN                                   */
/*                                                                    */
/*  Description:    Allocates an internal record structure and puts   */
/*                  it in the linked list of internal records for     */
/*                  the external record passed in.  It also checks    */
/*                  to be sure the record being inserted is not       */
/*                  already inserted.                                 */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                  pRecord  - record for which internal record is    */
/*                             to be allocated and linked to.         */
/*                                                                    */
/*  Output:         LPRECORDCOREN - pointer to the internal record    */
/*                  allocated and linked in the list.                 */
/*                                                                    */
/*                  NULL - Error occured: Out of memory or record     */
/*                                        currently inserted.         */
/*                                                                    */
/*  Programmer: Peter Haggar                                          */
/**********************************************************************/
LPRECORDCOREN FAR CnrAllocRecordN (hwndCnr, pRecord)

HANDLE         hwndCnr;
LPRECORDCORE  pRecord;
{
 LPRECORDCOREN  preccN;

 if (!pRecord)
 {
   return (NULL);
 }

 /*******************************************************************/
 /* Get pointer to the first existing internal recordcore.          */
 /*******************************************************************/
 preccN = PRECCFIRSTNFROMX(pRecord);

 /*********************************************************************/
 /* If the record passed in already has internal blocks on it, we     */
 /* need to add the new internal block at the end.                    */
 /*********************************************************************/
 if (preccN)
 {
   if (preccN->hwndCnr == hwndCnr)
   {
     SETERROR(PMERR_RECORD_CURRENTLY_INSERTED);
     return (NULL);
   }

   /*******************************************************************/
   /* Find last block......then link it.                              */
   /*******************************************************************/
   while (preccN->preccnNext)
   {
     if (preccN->preccnNext->hwndCnr == hwndCnr)
     {
       SETERROR(PMERR_RECORD_CURRENTLY_INSERTED);
       return (NULL);
     }

     preccN = preccN->preccnNext;
   }

   return (preccN->preccnNext = AllocRecordN (hwndCnr, pRecord));
 }
 else
 /*********************************************************************/
 /* This is the first one so make the external record point to the    */
 /* internal one we just created.                                     */
 /*********************************************************************/
 {
   return (PRECCFIRSTNFROMX(pRecord) = AllocRecordN (hwndCnr, pRecord));
 }

 return (NULL);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  AllocRecordN                                      */
/*                                                                    */
/*  Description:    Allocates an internal record structure and        */
/*                  copies the container handle, record attributes    */
/*                  and ptlIcon from the external record into the new */
/*                  internal record structure.                        */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                  pRecord  - record for which internal record is    */
/*                             to be allocated.                       */
/*                                                                    */
/*  Output:         LPRECORDCOREN - pointer to the internal record    */
/*                  allocated and linked in the list.                 */
/*                                                                    */
/*                  NULL - Out of memory error.                       */
/*                                                                    */
/*  Programmer: Peter Haggar                                          */
/**********************************************************************/
LPRECORDCOREN FAR AllocRecordN (hwndCnr, pRecord)

HANDLE         hwndCnr;
LPRECORDCORE  pRecord;
{
// LPVOID         pSatPtr;
 LPRECORDCOREN  preccNNew;
 LPCNRINFON     pccinfon;
 LPCNRINFO      pccinfo;
 DWORD          ulAttrChange;

// pSatPtr = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
 pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* Allocate the storage for the new internal record structure.  If   */
 /* we're out of memory return the awful news.                        */
 /*********************************************************************/
// if (!(preccNNew = (LPRECORDCOREN)StmAllocateStorage (pSatPtr,
 if (!(preccNNew = (LPRECORDCOREN)GlobalAlloc( GPTR,                    /*DAB1*/
                                       (DWORD)(sizeof(RECORDCOREN)))))
 {
   SETERROR(PMERR_INSUFFICIENT_MEMORY);
   return ((LPRECORDCOREN)NULL);
 }

 /*********************************************************************/
 /* Since the allocation worked, copy the container handle this       */
 /* internal block was allocated for, then copy the flRecordAttr      */
 /* field and the ptlIcon field from the external structure to the    */
 /* new structure we just allocated.                                  */
 /*********************************************************************/
 preccNNew->hwndCnr = hwndCnr;
 preccNNew->flRecordAttr = pRecord->flRecordAttr;
 preccNNew->ptlIcon = pRecord->ptlIcon;

 /*********************************************************************/
 /* Ensure both COLLAPSED and EXPANDED are not set by turning off     */
 /* the one if the other is set                                       */
 /*********************************************************************/
 if (preccNNew->flRecordAttr & CRA_COLLAPSED)
 {
   preccNNew->flRecordAttr &= ~CRA_EXPANDED;
 }

 /*********************************************************************/
 /* Initialize CRA_COLLAPSED if CRA_EXPANDED not specified.           */
 /*********************************************************************/
 if (!(preccNNew->flRecordAttr & CRA_EXPANDED))
 {
   preccNNew->flRecordAttr |= CRA_COLLAPSED;
 }

 /*********************************************************************/
 /* Update X attr for the N attr changes                             */
 /*********************************************************************/
 pRecord->flRecordAttr =  preccNNew->flRecordAttr;

 /*********************************************************************/
 /* Assign the pointer in the internal structure to point to the      */
 /* external record.  Also, copy the internal attributes to make sure */
 /* they are accurate.                                                */
 /*********************************************************************/
 preccNNew->preccX = pRecord;

  /* If we are in Tree View or we are in Single selection,
   * turn CRA_SELECTED style bit off.
   */
 if ((pccinfo->flWindowAttr & CV_TREE) ||
     !(GetWindowLong (hwndCnr, GWL_STYLE) &
       (CCS_MULTIPLESEL | CCS_EXTENDSEL)))
 {
   ulAttrChange = CRA_SELECTED | CRA_CURSORED;
 }
 else
 {
   ulAttrChange = CRA_CURSORED;
 }
 SetEmphasisBits (hwndCnr, NULL, preccNNew,
                  ulAttrChange, SEB_CLEAR);
 return (preccNNew);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     DrawSelCurItemInView                           */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function is used in    InvalidateRecord internally.       */
/*     It erases or draws (depending on flDraw) the selected and/or   */
/*     cursored items in the viewport in the name/text view.          */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR DrawSelCurItemsInView(HANDLE hwnd, HDC hps, BOOL flDraw)
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccInView;
  RECT          rctlHScroll;
  RECT          rctlTitle;
  RECT          rctlClient;
  long          topRecord;
  long          yClientBottom;
  POINT         iconSize;

  preccInView = GetFirstItemInView(hwnd);
  if (preccInView)
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    /*******************************************************************/
    /* Get H scroll bar rect and icon size.                            */
    /* The left-top point of the rectangle is (0, 0)    {Windows}      */
    /*******************************************************************/
    CnrQuerySizeRect(hwnd, &iconSize, &rctlTitle, &rctlClient, NULL,
                     &rctlHScroll);

    topRecord = CnrQueryPtOfItemInNTV(hwnd, preccInView, &iconSize,
                                   PT_TOP);
    yClientBottom = rctlTitle.bottom + rctlClient.bottom +
                    pccinfon->ptlNonIconOrg.y;

    while (preccInView && (topRecord < yClientBottom))
    {
      if ((preccInView->flRecordAttr & CRA_SELECTED) ||
          ((preccInView->flRecordAttr & CRA_CURSORED) &&
           (pccinfo->flWindowAttr & CV_TEXT)))
      {
        if (flDraw)
        {
          if (!(CnrDrawRecord(hwnd, hps, preccInView, TRUE)))
          {
             return (FALSE);
          }
        }
        else
        {
          if (!(CnrEraseRecord(hwnd, hps, preccInView)))
          {
             return (FALSE);
          }
        }
      }
      preccInView = preccInView->preccNextItem;
      topRecord = CnrQueryPtOfItemInNTV(hwnd, preccInView, &iconSize,
                                     PT_TOP);
    } /* endwhile */
  } /* endif */

  return (TRUE);
}

/*********************************************************************/
/* FUNCTION NAME:  Before1stInView                                   */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Invalidate Before First In View                */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  Checks to see if the record passed is    */
/*      before the first item in the current view.                   */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR  Before1stInView(HANDLE hwnd, LPRECORDCOREN preccnInvalid)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  LPRECORDCOREN preccnCurrent;
  LPRECORDCOREN preccnFirstInView;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);
  preccnCurrent = pccinfon->preccItem;
  preccnFirstInView = GetFirstItemInView(hwnd);

  while ((preccnCurrent) && (preccnCurrent != preccnFirstInView))
  {
     if (preccnInvalid == preccnCurrent)
     {
        return(TRUE);
        break;
     }
     else
     {
        if (pccinfo->flWindowAttr & CV_TREE)
        {
          preccnCurrent = TRVGetNextVisualItem (preccnCurrent);
        }
        else
        {
          preccnCurrent = preccnCurrent->preccNextItem;
        } /* endif */
     }
  }

  return(FALSE);
}

/*********************************************************************/
/* FUNCTION NAME:  AfterLastInViewport                               */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR AfterLastInViewport(HANDLE hwnd, LPRECORDCOREN preccInvalid)

{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  POINT     iconSize;
  RECT      rctlHScroll;
  RECT      rctlClient;
  RECT      rctlTitle;
  long      ItemEnd;
  long      ClientEnd;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwnd, &iconSize, &rctlTitle, &rctlClient, NULL,
                   &rctlHScroll) ;


  if (pccinfo->flWindowAttr & CV_ICON)
  {
    ClientEnd = rctlClient.bottom + rctlTitle.bottom +
                pccinfo->ptlOrigin.y;
    ItemEnd = preccInvalid->ptlIcon.y;
    if (ItemEnd >= ClientEnd)
    {
      return TRUE;
    }
    else
    {
      return FALSE;
    } /* endif */
  }
  else
  {
    if (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT))
    {
      if (pccinfo->flWindowAttr & CV_FLOW)
      {
        ClientEnd = pccinfon->ptlNonIconOrg.x + rctlClient.right;
        ItemEnd = CnrQueryPtOfItemInNTV(hwnd, preccInvalid,
                                         &iconSize, PT_LEFT);
        if (ItemEnd >= ClientEnd)
        {
          return TRUE;
        }
        else
        {
          return FALSE;
        } /* endif */
      }
      else
      {
        ClientEnd = rctlTitle.bottom + rctlClient.bottom +
                    pccinfon->ptlNonIconOrg.y;
        ItemEnd = CnrQueryPtOfItemInNTV(hwnd, preccInvalid,
                                         &iconSize, PT_TOP);
        if (ItemEnd >= ClientEnd)
        {
          return TRUE;
        }
        else
        {
          return FALSE;
        }
      }
    }
    else
    {
      ClientEnd = pccinfon->ptlNonIconOrg.y +
                  rctlClient.bottom - rctlHScroll.bottom;

      ItemEnd = CnrQueryRowPosInDV (preccInvalid, PT_TOP);

      if (ItemEnd >= ClientEnd)
      {
        return(TRUE);
      }
      else
      {
        return(FALSE);
      }
    }
  }
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CallQuickSort                                    */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Copy To Array And Call QuickSort             */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function makes an array, copies LPRECORDCOREs in the     */
/*     linked list to the array and call QuickSort.                  */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR CallQuickSort(hwnd, mp1, mp2, preccFirstSibling, flRootLevel)
HANDLE          hwnd;
long            mp1;
long            mp2;
LPRECORDCOREN   preccFirstSibling;
BOOL            flRootLevel;
{
  LPCNRINFON       pccinfon;
  PALPRECORDCOREN  ptrArray;
  LPRECORDCOREN    preccEnum;
//LPVOID           pSatPtr;
  DWORD            i;
  DWORD            cRecords = 0;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

  if (preccFirstSibling)
  {
    preccEnum = preccFirstSibling;
    while (preccEnum)
    {
      cRecords++;
      CallQuickSort(hwnd, mp1, mp2, preccEnum->preccFirstChild, FALSE);
      preccEnum = preccEnum->preccNextItem;
    } /* endwhile */

//  pSatPtr = (LPVOID)GetWindowLong (hwnd, QWL_STORAGEPTR);
//  if (!(ptrArray = (PALPRECORDCOREN)StmAllocateStorage (pSatPtr,
    if (!(ptrArray = (PALPRECORDCOREN)GlobalAlloc( GPTR,                /*DAB1*/
                                   (cRecords * sizeof(LPRECORDCOREN)))))
    {
       SETERROR(PMERR_INSUFFICIENT_MEMORY);
       return (FALSE);                                                  /*DAB1*/
    }

    preccEnum = preccFirstSibling;

    /*****************************************************************/
    /* Copy the precords in the linked list to the array for the     */
    /* quick sort.                                                   */
    /*****************************************************************/
    for (i = 0; i < (cRecords - 1) ; i++ )
    {
      (*ptrArray)[i] = preccEnum;
      preccEnum = preccEnum->preccNextItem;
    }
    (*ptrArray)[i] = preccEnum;

    /*****************************************************************/
    /* Sort the items in the array                                   */
    /*****************************************************************/
    HeapSort(ptrArray, cRecords, mp1, mp2);

    /*****************************************************************/
    /* Update pccinfon->preccItem or the first child of the parent   */
    /* of the siblings.                                              */
    /*****************************************************************/
    if (flRootLevel)
    {
      pccinfon->preccItem = (*ptrArray)[0];
    }
    else
    {
      (*ptrArray)[0]->preccParent->preccFirstChild = (*ptrArray)[0];
    } /* endif */

    ((*ptrArray)[0])->preccPrevItem = NULL;

    for (i = 0; i < (cRecords - 1) ; i++ )
    {
      ((*ptrArray)[i])->preccNextItem = (*ptrArray)[i+1];
      ((*ptrArray)[i+1])->preccPrevItem = (*ptrArray)[i];
    } /* endfor */

    ((*ptrArray)[i])->preccNextItem = NULL;

    /*****************************************************************/
    /* Update pccinfon->preccLastItem when we are sorting the root   */
    /* level siblings.                                               */
    /*****************************************************************/
    if (flRootLevel)
    {
      pccinfon->preccLastItem = (*ptrArray)[i];
    }

//  if (!(StmFreeStorage (pSatPtr, (LPVOID)ptrArray)))
    if (GlobalFree( (LPVOID)ptrArray ))                                 /*DAB1*/
    {
       SETERROR (PMERR_MEMORY_DEALLOCATION_ERR);
       return (FALSE);                                                  /*DAB1*/
    }
  }
  return TRUE;

}

/*********************************************************************/
/* FUNCTION NAME:  CheckFirstInView                                  */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Check First Item In View                       */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function checks the first item      */
/*   in each view and if NULL initializes it.                        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwnd - handle to the container window              */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR CheckFirstInView(hwnd)
HANDLE    hwnd;
{
  LPCNRINFON   pccinfon;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

  if (!pccinfon->preccFirstinNV)
  {
    pccinfon->preccFirstinNV = pccinfon->preccItem;
  } /* endif */

  if (!pccinfon->preccFirstinFNV)
  {
    pccinfon->preccFirstinFNV = pccinfon->preccItem;
  } /* endif */
  if (!pccinfon->preccFirstinTV)
  {
    pccinfon->preccFirstinTV = pccinfon->preccItem;
  } /* endif */
  if (!pccinfon->preccFirstinFTV)
  {
    pccinfon->preccFirstinFTV = pccinfon->preccItem;
  } /* endif */

  if (!pccinfon->preccFirstinTRV)
  {
    pccinfon->preccFirstinTRV = pccinfon->preccItem;
  } /* endif */

  if (!pccinfon->preccFirstRowInDV)
  {
    pccinfon->preccFirstRowInDV = pccinfon->preccItem;
  } /* endif */

  return TRUE;

}

/**********************************************************************/
/* FUNCTION NAME:  CnrProcessDelta                    REWRITTEN       */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/* FUNCTIONAL DESCRIPTION:  Handle Query delta notification           */
/*                                                                    */
/* NOTES:                                                             */
/*    FORWARD DECLARATION:                                            */
/*                                                                    */
/* INPUT:  HANDLE hwnd - handle to the container window               */
/*         BOOL scrollDirection - Direction currently scrollling      */
/* OUTPUT:                                                            */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:                                                       */
/*                                                                    */
/* EXIT-ERROR:                                                        */
/*                                                                    */
/* SIDE EFFECTS:                                                      */
/**********************************************************************/
BOOL FAR  CnrProcessDelta(HANDLE hwndCnr, BOOL scrollDirection)
{
  LPCNRINFON    pccinfon;                  /* instance data            */
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccCheck=NULL;           /* front record for search  */
  LPRECORDCOREN preccCurrent = NULL;       /* back record for search   */
  NOTIFYDELTA   notifyDelta;               /* CN_QUERYDELTA structure  */
  DWORD         deltaIdx=0;                /* loop increment           */

  /********************************************************************/
  /* Initialize pointers to Container instance data                   */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Get pointer to first and last record in viewport.                */
  /********************************************************************/
  if (scrollDirection)
  {
    preccCheck = CnrGetLastItemInView(hwndCnr);
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
      preccCheck = pccinfon->preccFirstRowInDV;
    }
    else
    {
      if (pccinfo->flWindowAttr & CV_FLOW)
      {
        /************************************************************/
        /* Flowed Name View                                         */
        /************************************************************/
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          preccCheck = pccinfon->preccFirstinFNV;
        }
        /************************************************************/
        /* Flowed Text View                                         */
        /************************************************************/
        else
        if (pccinfo->flWindowAttr & CV_TEXT)
        {
          preccCheck = pccinfon->preccFirstinFTV;
        }
      }
      else    //non-flow view
      {
        /************************************************************/
        /* Non-Flowed Name View                                     */
        /************************************************************/
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          preccCheck = pccinfon->preccFirstinNV;
        }
        /************************************************************/
        /* Non-Flowed Text View                                     */
        /************************************************************/
        else
        if (pccinfo->flWindowAttr & CV_TEXT)
        {
          preccCheck = pccinfon->preccFirstinTV;
        }
      }
    }
  }

  /******************************************************************/
  /* Initialize records for indexing through the list via item order*/
  /******************************************************************/
  if (scrollDirection)
  {
    preccCurrent = pccinfon->preccLastItem;
    if (!ISRECORDVISIBLE(preccCurrent))
    {
      preccCurrent = GetPrevUnfilteredItem(preccCurrent);
    }
  }
  else
  {
    preccCurrent = pccinfon->preccItem;
    if (!ISRECORDVISIBLE(preccCurrent))
    {
      preccCurrent = GetNextUnfilteredItem(preccCurrent);
    }
  }

  /******************************************************************/
  /* Loop to see if we have scrolled to the delta set by the        */
  /* application. The loop continues until we reach our delta value */
  /* or the last  record in view (Scrolling down or right) is       */
  /* reached or the first record in view (Scrolling up or left) is  */
  /* reached.                                                       */
  /******************************************************************/
  while (preccCurrent &&
         (deltaIdx < pccinfo->cDelta) &&
         (preccCurrent != preccCheck))
  {
    if (ISRECORDVISIBLE(preccCurrent))
    {
      deltaIdx++;
    }

    if (scrollDirection)
    {
      preccCurrent = preccCurrent->preccPrevItem;
    }
    else
    {
      preccCurrent = preccCurrent->preccNextItem;
    }
  }

  /******************************************************************/
  /* If the delta value was not reached, then we have scrolled      */
  /* passed it and need to send a notification to the application   */
  /* to insert additional records.                                  */
  /******************************************************************/
  if (deltaIdx <= pccinfo->cDelta)
  {
    /****************************************************************/
    /* If we reached the first/last record in view.                 */
    /****************************************************************/
    if (preccCheck == preccCurrent)
    {
      /**************************************************************/
      /* If scroll down...                                          */
      /**************************************************************/
      if (scrollDirection)
      {
        /************************************************************/
        /* If we are at the last  record in item order set the flag */
        /* indicating this additionally.                            */
        /************************************************************/
        if (preccCurrent == pccinfon->preccLastItem)
        {
          notifyDelta.fDelta = CMA_DELTAEND;
        }
        else
        {
          /***********************************************************/
          /* Set a flag indicating that we have reached the delta at */
          /* the end of the item list.                               */
          /***********************************************************/
          notifyDelta.fDelta = CMA_DELTABOT;
        }
      }
      /***************************************************************/
      /* If scroll up.....                                           */
      /***************************************************************/
      else
      {
        /*************************************************************/
        /* If we are at the first record in item order set the flag  */
        /* indicating this additionally.                             */
        /*************************************************************/
        if (preccCurrent == pccinfon->preccItem)
        {
          notifyDelta.fDelta = CMA_DELTAHOME;
        }
        else
        {
          /***********************************************************/
          /* Set a flag indicating that we have reached the delta at */
          /* the front of the item list.                             */
          /***********************************************************/
          notifyDelta.fDelta = CMA_DELTATOP;
        }
      }
      notifyDelta.hwndCnr = hwndCnr;
      return (LOWORD (NotifyOwner (hwndCnr, CN_QUERYDELTA,
                                   (LPVOID)&notifyDelta)));
    }
  }
  /********************************************************************/
  /* Return TRUE to indicate no delta processing has been done.       */
  /********************************************************************/
  return TRUE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryCnrInfo                                */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:  Query container info structure                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:    WORD cbBuffer - maximum number of bytes to copy         */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:   LPCNRINFO LPCNRINFO - pointer to the buffer into which  */
/*            the CNRINFO structure is copied.                        */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: WORD fSuccess - Actual number of bytes copied        */
/*                                                                    */
/*  Exit-Error:  WORD fSuccess - NULL                                 */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
WORD FAR CnrQueryCnrInfo( HANDLE hwnd, long mp1, long mp2 )
{
  LPCNRINFON  pccinfon;                  /* Internal CNR control data */
  LPCNRINFO   pccinfo;
  WORD        cbBuffer;                  /* Number of bytes to copy   */
  LPCNRINFO   pCnrInfo;                  /* Buffer to copy into       */

  /********************************************************************/
  /* Initialize pointers to control data and local variables          */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);
  pCnrInfo = (LPCNRINFO)mp1;
  cbBuffer = LOWORD(mp2);

  /********************************************************************/
  /* Check if a valid pointer was passed                              */
  /********************************************************************/
  if (!pCnrInfo)
  {
    /******************************************************************/
    /* If not flag as an error and return.                            */
    /******************************************************************/
    SETERROR (PMERR_INVALID_PARAMETERS);
    return((WORD)0);
  }

  if (cbBuffer > sizeof(CNRINFO))
  {
    cbBuffer = sizeof(CNRINFO);
  }

  memcpy ((LPVOID)pCnrInfo, (LPVOID)pccinfo, cbBuffer);                 /*DAB1*/

  return(cbBuffer);
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryRecord                                 */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: Returns pointer to requested record.      */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:    LPRECORDCORE pRecord - Pointer for which data is        */
/*                                  requested                         */
/*                                                                    */
/*            PRECORDQUERY  pRecordQuery  - data structure            */
/*             WORD cmd - Flag indicating record to retrieve. May     */
/*                          be:                                       */
/*                          QCR_FIRST     first record                */
/*                          QCR_LAST      last record                 */
/*                          QCR_NEXT      next record after pointer   */
/*                          QCR_FIRST     first record after pointer  */
/*                          QCR_PARENT    find the parent             */
/*                          QCR_CHILD     find the first child        */
/*                                                                    */
/*             DWORD  fsSearch - Search control flags. May be:        */
/*                               One of:                              */
/*                               QCR_ITEMORDER - enumerate in item    */
/*                                               order  (default)     */
/*                               QCR_ZORDER - enumerate in z order    */
/*                               QCR_TREEORDER - enumerate in tree    */
/*                                               order                */
/*                                                                    */
/*                               When QCR_TREEORDER is set the        */
/*                               following flags may also be set:     */
/*                                                                    */
/*                               QCR_SIBLING - find the first, last,  */
/*                                             next, or previous      */
/*                                             sibling node of this   */
/*                                             node                   */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:   LPRECORDCORE pRecord - pointer to record requested      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: Pointer to record.                                   */
/*                                                                    */
/*  Exit-Error:  NULL - There are no records to satisfy query.        */
/*                                                                    */
/*  Side Effects:  None.                                              */
/*    xtai                                                            */
/**********************************************************************/
LPRECORDCORE FAR CnrQueryRecord(HANDLE hwnd, long mp1, long mp2)
{
  LPRECORDCOREN  preccQuery;
  LPRECORDCOREN  preccn;
  LPRECORDCORE   pRecord;
  LPCNRINFON     pccinfon;
  WORD           cmd;
  WORD           fsSearch;

  /*****************************************************************/
  /* Initialize pointers to control data and local variables       */
  /*****************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

//preccQuery = GetRecordNFromX (hwnd, (LPRECORDCORE)mp1);
  pRecord = (LPRECORDCORE)(LPVOID)mp1;
  cmd = LOWORD(mp2);
  fsSearch = HIWORD(mp2);

  /**************************************************************/
  /* If CCS_VERIFYPOINTERS is true and CnrVerifyPointers is false*/
  /* return an error condition                                  */
  /**************************************************************/
  if (GetWindowLong(hwnd, GWL_STYLE ) & CCS_VERIFYPOINTERS)
  {
    if ((cmd != CMA_FIRST) && (cmd != CMA_LAST) &&
        !(((cmd == CMA_FIRSTCHILD) || (cmd == CMA_LASTCHILD)) &&
          !pRecord) &&
        !(CnrVerifyPointers(hwnd, (LPVOID)&pRecord, (LONG)1)))
    {
      SETERROR (PMERR_INVALID_PARAMETERS);
      return((LPRECORDCORE)ERRORFLAG);
    }
  }

  /*******************************************************************/
  /* If the first record is requested, check the enumeration order   */
  /* and return the head pointer.  In zOrder, preccZTail is defined  */
  /* to be at the top of the list. preccZ is the first item in the   */
  /* list.                                                           */
  /*******************************************************************/
  if (cmd == CMA_FIRST)
  {
    if (fsSearch == CMA_ITEMORDER)
    {
      preccn = pccinfon->preccItem;
      goto ExitOK;
    }
    else
    {
      if (fsSearch == CMA_ZORDER)
      {
        preccn = pccinfon->preccZ;
        goto ExitOK;
      }
      else
      {
        goto ExitError;
      }
    } /* endif */
  }

  /*******************************************************************/
  /* If the last record in item order is requested, loop through     */
  /* the records and return the last one. If zOrder, return the      */
  /* tail pointer, preccZTail.                                       */
  /*******************************************************************/
  if (cmd == CMA_LAST)
  {
    if (fsSearch == CMA_ITEMORDER)
    {
      preccn = pccinfon->preccLastItem;
      goto ExitOK;
    }
    else
    {
      if (fsSearch == CMA_ZORDER)
      {
        preccn = pccinfon->preccZTail;
        goto ExitOK;
      }
      else
      {
        goto ExitError;
      }
    }
  }

  /********************************************************************/
  /* For the next set of queries, we need the internal record of the  */
  /* external one passed in.                                          */
  /********************************************************************/
  preccQuery = GetRecordNFromX (hwnd, pRecord);

  /*******************************************************************/
  /* If the next record is requested in either order, return         */
  /* the next record pointed to by preccNextItem or preccNextZ       */
  /*******************************************************************/
  if (cmd == CMA_NEXT)
  {
    if (fsSearch == CMA_ITEMORDER)
    {
      preccn = preccQuery->preccNextItem;
      goto ExitOK;
    }
    else
    {
      if (fsSearch == CMA_ZORDER)
      {
        preccn = preccQuery->preccNextZ;
        goto ExitOK;
      }
      else
      {
        goto ExitError;
      }
    }
  }

  /*******************************************************************/
  /* If the previous record is requested in either order,            */
  /* return the previous record pointed to by preccPrevItem          */
  /* or preccPrevZ.                                                  */
  /*******************************************************************/
  if (cmd == CMA_PREV)
  {
    if (fsSearch == CMA_ITEMORDER)
    {
      preccn = preccQuery->preccPrevItem;
      goto ExitOK;
    }
    else
    {
      if (fsSearch == CMA_ZORDER)
      {
        preccn = preccQuery->preccPrevZ;
        goto ExitOK;
      }
      else
      {
        goto ExitError;
      }
    }
  }

  /*******************************************************************/
  /* If the parent item is requested,                                */
  /* return the parent record of the preccQuery.                     */
  /*   Note : fsSearch is disregarded.                               */
  /*******************************************************************/
  if (cmd == CMA_PARENT)
  {
    preccn = preccQuery->preccParent;
    goto ExitOK;
  }

  /*******************************************************************/
  /* If the first child of the record is requested,                  */
  /* return the first child of preccQuery.                           */
  /*   Note : fsSearch is disregarded.                               */
  /*******************************************************************/
  if (cmd == CMA_FIRSTCHILD)
  {
    if (preccQuery)
    {
      preccn = preccQuery->preccFirstChild;
      goto ExitOK;
    }
    else
    {
      preccn = pccinfon->preccItem;
      goto ExitOK;
    }
  }

  /*******************************************************************/
  /* If the first child of the record is requested,                  */
  /* return the first child of preccQuery.                           */
  /*   Note : fsSearch is disregarded.                               */
  /*******************************************************************/
  if (cmd == CMA_LASTCHILD)
  {
    if (preccQuery)
    {
      LPRECORDCOREN   preccEnum = preccQuery->preccFirstChild;
      LPRECORDCOREN   preccLastChild = NULL;

      while (preccEnum)
      {
        preccLastChild = preccEnum;
        preccEnum = preccEnum->preccNextItem;
      }
      preccn = preccLastChild;
      goto ExitOK;
    }
    else
    {
      preccn = pccinfon->preccLastItem;
      goto ExitOK;
    }
  }

  /********************************************************************/
  /* If we get to here, no valid cmd value was specified indicating   */
  /* an error.                                                        */
  /********************************************************************/
  goto ExitError;

  /********************************************************************/
  /* Make sure we give the application the record back with the       */
  /* correct information.  Copy the internal info to the record, then */
  /* return it.                                                       */
  /********************************************************************/
ExitOK:
  if (preccn)
  {
    COPYNDATATOX(preccn);
    return (preccn->preccX);
  }
  return (NULL);

ExitError:
  SETERROR(PMERR_INVALID_PARAMETERS);
  return((LPRECORDCORE)ERRORFLAG);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrRemoveRecord                                */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
long FAR CnrRemoveRecord(HANDLE hwnd, long mp1, long mp2)
{
  HDC            hps;
  LPCNRINFON     pccinfon;                /* Internal CNR control data   */
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  preccEnumArray;
  LPRECORDCOREN  preccOldFirst;
  LPRECORDCOREN  preccOldCursor = NULL;
  DWORD          i = 0;
  WORD           cNumRecord;
  WORD           usFreeInvalidate;
  LPRECORDCORE   FAR *pRecordArray;     /* array of the records to be  */
                                        /* invalidated                 */
  LPRECORDCOREN  preccFirstInView;
  LPRECORDCOREN  preccFirstDel = NULL;
  BOOL           fInvalidateAll = FALSE;
  BOOL           flNoRefresh = FALSE;
  BOOL           flUpdateWorkplace = FALSE;
  RECT           CnrRect;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  cNumRecord = LOWORD(mp2);
  usFreeInvalidate = HIWORD(mp2);
  pRecordArray = (LPVOID)(mp1);

  if (!(pccinfon->preccItem))
  {
    return ((long)0);
  }

  if ((pccinfo->flWindowAttr & CV_ICON) &&
      (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
  {
    pccinfon->ulFlagSet |= FL_ITEMNUMCHANGED;
  }

  /*******************************************************************/
  /* If cNumRecord = 0 and flRemoveFree = TRUE, all records are to be*/
  /* removed and the memory freed.                                   */
  /*******************************************************************/
  if (!cNumRecord)
  {
    if (usFreeInvalidate & CMA_FREE)
    {
      CnrFreeAll (hwnd, TRUE);
    }
    else
    {
      CnrRemoveAllInternal (hwnd);
    } /* endif */

    CnrDeleteAllResizedIcon(hwnd);

    /*****************************************************************/
    /* If CMA_INVALIDATE is specified or in the regular icon view    */
    /* refresh whole window.                                         */
    /* NOTE: we erase the record deleted in the regular icon view    */
    /*       because the application can not use CM_INVALIDATERECORD */
    /*       as in the other view for the deleted items.             */
    /*****************************************************************/
    if ((usFreeInvalidate & CMA_INVALIDATE) ||
        (!(pccinfo->flWindowAttr & CV_TREE) &&
         (pccinfo->flWindowAttr & CV_ICON) &&
         !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)))
    {
      if (pccinfo->flWindowAttr & CV_DETAIL)
      {
        pccinfon->ulFlagSet &= ~FL_POSITIONCOLTITLES;

        PMGetClientRect (hwnd, &CnrRect);

        if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
            ((long)pccinfo->xVertSplitbar > CnrRect.right))
        {
           CnrDetailsVCtlProc (hwnd, WM_PAINT, 0, 0 );                  /*DAB1*/
        }
      }
      InvalidateContainer( hwnd, TRUE, NULL );
    }
    return(pccinfo->cRecords);
  }
  else
  {
    preccOldCursor = pccinfon->preccCursor;

    /**************************************************************/
    /* If CCS_VERIFYPOINTERS is set on, verify the pointers.       */
    /**************************************************************/
    if (GetWindowLong (hwnd, GWL_STYLE) & CCS_VERIFYPOINTERS)
    {
      if (!(CnrVerifyPointers(hwnd, pRecordArray, (long)cNumRecord)))
      {
        SETERROR (PMERR_INVALID_PARAMETERS);
        return(ERRORFLAG);
      }
    }

    hps = PMGetFlippedDC(hwnd);

    /*****************************************************************/
    /* Get the first visible item in the array.                      */
    /*****************************************************************/
    preccEnumArray = GetRecordNFromX (hwnd, pRecordArray[i]);
    while ((i < (DWORD)(cNumRecord - (WORD)1)) &&
           (!ISRECORDVISIBLE(preccEnumArray) ||
            (!(pccinfo->flWindowAttr & CV_TREE) &&
             preccEnumArray->preccParent)))
    {
      i++;
      preccEnumArray = GetRecordNFromX (hwnd, pRecordArray[i]);
    }

    if (i < cNumRecord)
    {
      preccOldFirst = GetFirstItemInView (hwnd);
      preccFirstDel = preccEnumArray;
    }
    else
    {
      flNoRefresh = TRUE;
    }

    /*****************************************************************/
    /* Remove records in the array.                                  */
    /*****************************************************************/
    for (i=0; i< cNumRecord; ++i)
    {
      preccEnumArray = GetRecordNFromX (hwnd, *(pRecordArray + i));
      if (!preccEnumArray)
      {
        ReleaseDC (hwnd, hps);
        SETERROR (PMERR_INVALID_PARAMETERS);
        return(ERRORFLAG);
      }

      /****************************************************************/
      /* If the record being removed happens to be the record we are  */
      /* maintaining as the anchor point or the previous end of range */
      /* for extended selection, reset those pointers.         -- MDS */
      /****************************************************************/
      if (preccEnumArray == pccinfon->preccAnchor) {
         pccinfon->preccAnchor = NULL;
      }
      if (preccEnumArray == pccinfon->preccPrevEndOfRange) {
         pccinfon->preccPrevEndOfRange = NULL;
      }

      /****************************************************************/
      /* Create our own linked list.                                  */
      /****************************************************************/
      pRecordArray[i]->preccNextRecord =
                            ((i < (DWORD)(cNumRecord - (WORD)1)) ?
                                          pRecordArray[i+1] : NULL);
      if (preccEnumArray)
      {
        /*************************************************************/
        /* Count down only when pRecord is a root node.              */
        /*************************************************************/
        if (!preccEnumArray->preccParent)
        {
          pccinfo->cRecords--;
        }

        if (ISRECORDVISIBLEINVIEW(pccinfo, preccEnumArray))
        {
          preccFirstDel = UpdateFirstDel(hwnd, preccEnumArray,
                                         preccFirstDel);

          if (!(preccEnumArray->preccParent) &&
              !(GetWindowLong (hwnd, GWL_STYLE) &
                CCS_AUTOPOSITION))
          {
            /*********************************************************/
            /* If the flag says "No need to reset workplace" check   */
            /* if removing preccEnumArray changes workplace.         */
            /* If so, change the flag to say "Need to reset it"      */
            /*********************************************************/
            if (pccinfon->ulFlagSet & FL_SETWORKPLACE)
            {
              NeedUpdateWorkplace(hwnd, hps, preccEnumArray);
            }
          }

          /***********************************************************/
          /* Update first item in view if necessary.                 */
          /***********************************************************/
          UpdateFirstItemInView(hwnd, preccEnumArray);

          /***********************************************************/
          /* If the record being removed is cursored, reset the      */
          /* pointer.                                                */
          /***********************************************************/
          UpdateCursorForDel(hwnd, preccEnumArray);
        }

        /*************************************************************/
        /* Delete the record in the linked lists (item order,        */
        /* z order)                                                  */
        /*************************************************************/
        DeleteRecord(hwnd, preccEnumArray, TRUE);
        DeleteRecord(hwnd, preccEnumArray, FALSE);

        /*************************************************************/
        /* Use bitmap when only CA_DRAWBITMAP is specified.          */
        /* Default is using Icon.                                    */
        /*************************************************************/
        if ((pccinfo->flWindowAttr & CA_DRAWICON) ||
            !(pccinfo->flWindowAttr & CA_DRAWBITMAP))
        {
          /***********************************************************/
          /* Delete the bitmaps for drawing the record from the      */
          /* resized icon metrics. (in cnrcfunc.c)                   */
          /***********************************************************/
          CnrInsertDeleteResizedIcon(hwnd, preccEnumArray->preccX,
                                     FALSE, TRUE);
          if (!preccEnumArray->preccParent)
          {
            CnrInsertDeleteResizedIcon(hwnd, preccEnumArray->preccX,
                                       FALSE, FALSE);
          }
          CnrTRVInsDelResizedIcon(hwnd, preccEnumArray->preccX, FALSE);

          /***********************************************************/
          /* If any FieldInfo's (columns) exist, we must remove the  */
          /* Resized Icon for each of them in the IRS Metrics.       */
          /* No need to process DV info for children records         */
          /***********************************************************/
          if ((pccinfon->pFirstFI) && (!preccEnumArray->preccParent))
          {
             CnrProcessIconsForDV (hwnd, preccEnumArray, NULL,
                                   FALSE, TRUE);
          }
        } /* endif */
      }
      else
      {
        ReleaseDC (hwnd, hps);
        SETERROR (PMERR_INVALID_PARAMETERS);
        return(ERRORFLAG);
      } /* endif */

    } /* endfor */
  }

  /*******************************************************************/
  /* If CMA_INVALIDATE is specified or in the regular icon view      */
  /* refresh the   window.                                           */
  /* NOTE: we erase the record deleted in the regular icon view      */
  /*       because the application can not use CM_INVALIDATERECORD   */
  /*       as in the other view for the deleted items.               */
  /*******************************************************************/
  if ((!(pccinfo->flWindowAttr & CV_TREE) &&
       (pccinfo->flWindowAttr & CV_ICON) &&
       !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)) ||
      ((usFreeInvalidate & CMA_INVALIDATE) && !flNoRefresh ))
  {
    if (!(pccinfo->flWindowAttr & CV_TREE) &&
        (pccinfo->flWindowAttr & CV_ICON) &&
        !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
    {
      fInvalidateAll = FALSE;
    }
    else
    {
      preccFirstInView = GetFirstItemInView(hwnd);
      if (preccFirstInView != preccOldFirst)
      {
        fInvalidateAll = TRUE;
      }
      else
      {
        if (IsBefore(hwnd, preccFirstDel, preccOldFirst))
        {
          fInvalidateAll = TRUE;
        }
      }
    } /* endif */

    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       PMGetClientRect (hwnd, &CnrRect);

      if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
          ((long)pccinfo->xVertSplitbar > CnrRect.right))
      {
         CnrDetailsVCtlProc (hwnd, WM_PAINT, 0, 0 );                    /*DAB1*/
      }
    }

    /*****************************************************************/
    /* If we need to refresh  and the cursored record is             */
    /* changed, redraw for that.                                     */
    /*****************************************************************/
    if (!flNoRefresh && pccinfon->preccCursor &&
        (preccOldCursor != pccinfon->preccCursor))
    {
      /***************************************************************/
      /* In icon view without CCS_AUTOPOSITION, we only need to draw  */
      /* the new cursored record.                                    */
      /***************************************************************/
      if (!(pccinfo->flWindowAttr & CV_TREE) &&
          (pccinfo->flWindowAttr & CV_ICON) &&
          !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
      {
        CnrDrawIcon(hwnd, hps, pccinfon->preccCursor, TRUE);
      }
      else
      {
        /*************************************************************/
        /* If other views, we update the first item to be            */
        /* to be invalidated.                                        */
        /*************************************************************/
        if (!fInvalidateAll &&
            (IsBefore(hwnd, pccinfon->preccCursor, preccFirstDel)))
        {
          preccFirstDel = pccinfon->preccCursor;
        }
      }
    }

    InvalidateForRemove (hwnd, hps, preccFirstDel, fInvalidateAll,
                         cNumRecord, pRecordArray);
  }
  /**************************************************************/
  /* Call function to remove the internal recordcores from the  */
  /* list.                                                      */
  /**************************************************************/
  CnrRemoveRecordN (hwnd, pRecordArray[0],
                    (WORD)(usFreeInvalidate & (WORD)CMA_FREE));

  if (usFreeInvalidate & CMA_FREE)
  {
    FreeRecords (hwnd);
  }

  ReleaseDC(hwnd, hps);
  return (pccinfo->cRecords);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  FreeRecords                                       */
/*                                                                    */
/*  Description:    Frees all the records in the linked list passed   */
/*                  in.                                               */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                                                                    */
/*  Output:         void                                              */
/*                                                                    */
/*  Programmer: Peter Haggar and Dr. Woo                              */
/**********************************************************************/
VOID FAR FreeRecords (hwndCnr)

HANDLE         hwndCnr;           /* Handle of Container window         */

{
  LPCNRINFON     pccinfon;
//LPVOID         pSatPtr;         /* Pointer to Storage Space.         */
  LPRECORDCORE   preccFree;
  LPRECORDCORE   preccNext;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);

  /********************************************************************/
  /* Get the pointer to the storage space.                            */
  /********************************************************************/
//pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
  preccFree = pccinfon->preccFreeFirst;

  while (preccFree)
  {
    /******************************************************************/
    /* Get next external record pointer.                              */
    /******************************************************************/
    preccNext = *(PLPRECORDCORE)LPVOIDFROMPRECCX(preccFree);

    /******************************************************************/
    /* Free the record.  Be sure to back up the pointer 4 bytes so    */
    /* we are sure to free the extra memory we allocated that the     */
    /* application was clueless about in the first place.             */
    /******************************************************************/
//  StmFreeStorage (pSatPtr, LPVOIDFROMPRECCX(preccFree));
    GlobalFree( LPVOIDFROMPRECCX(preccFree));                           /*DAB1*/

    preccFree = preccNext;
  }

   return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     IsBefore                                       */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR IsBefore(hwnd, preccBefore, preccAfter)
HANDLE           hwnd;
LPRECORDCOREN   preccBefore;
LPRECORDCOREN   preccAfter;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* If valid records passed in for both                              */
  /********************************************************************/
  if (preccBefore && preccAfter)
  {
    /******************************************************************/
    /* If view is tree, details or non flowed name/text view          */
    /******************************************************************/
    if ((pccinfo->flWindowAttr & CV_TREE) ||
        (pccinfo->flWindowAttr & CV_DETAIL) ||
        (!(pccinfo->flWindowAttr & CV_TREE)  &&
         (pccinfo->flWindowAttr & (CV_TEXT | CV_NAME)) &&
         !(pccinfo->flWindowAttr & CV_FLOW)))
    {
      /****************************************************************/
      /* If position of before record is higher than After record     */
      /****************************************************************/
      if (preccBefore->ptlNonIconPos.y <
          preccAfter->ptlNonIconPos.y)
      {
        return TRUE;
      }
      else
      {
        return FALSE;
      }
    }

    /******************************************************************/
    /* If view is flowed name/text view                               */
    /******************************************************************/
    else if ((pccinfo->flWindowAttr & (CV_TEXT | CV_NAME)) &&
        (pccinfo->flWindowAttr & CV_FLOW))
    {
      /****************************************************************/
      /* If position of before record is left of After record or if   */
      /*  they are at same x position and the before record is above  */
      /****************************************************************/
      if ((preccBefore->ptlNonIconPos.x <
           preccAfter->ptlNonIconPos.x) ||
          ((preccBefore->ptlNonIconPos.x ==
           preccAfter->ptlNonIconPos.x) &&
           (preccBefore->ptlNonIconPos.y <
           preccAfter->ptlNonIconPos.y)))
      {
        return TRUE;
      }
      else
      {
        return FALSE;
      }
    }

    /******************************************************************/
    /* If view is icon view and style is autoposition                 */
    /******************************************************************/
    else if ((pccinfo->flWindowAttr & CV_ICON) &&
        (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
    {
      /****************************************************************/
      /* If position of Before record is above of After record or if  */
      /*  they are at same height and Before record is left of After  */
      /****************************************************************/
      if ((preccBefore->ptlIcon.y < preccAfter->ptlIcon.y) ||
          ((preccBefore->ptlIcon.y == preccAfter->ptlIcon.y) &&
           (preccBefore->ptlIcon.x < preccAfter->ptlIcon.x)))
      {
        return TRUE;
      }
    }
    /******************************************************************/
    /* Else it must be messy desk icon view.                          */
    /* We must search through item order.                             */
    /******************************************************************/
    else
    {
      while( preccBefore )
      {
        if ( preccBefore->preccNextItem == preccAfter )
        {
          return TRUE;
        }
        preccBefore = preccBefore->preccNextItem;
      }
    }
  }

  return FALSE;
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     UpdateFirstDel                                 */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR UpdateFirstDel(hwnd, preccEnumArray, preccFirstDel)
HANDLE          hwnd;
LPRECORDCOREN  preccEnumArray;
LPRECORDCOREN  preccFirstDel;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* It the current view is not the tree view and preccEnum is       */
  /* not the root node, it is invisible. So disregard it.            */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_TREE) && preccEnumArray->preccParent)
  {
    return (preccFirstDel);
  }

  /*******************************************************************/
  /* Auto-positioned Icon view                                       */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_TREE) && (pccinfo->flWindowAttr & CV_ICON)
     && (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
  {
    return (preccEnumArray->ptlIcon.y < preccFirstDel->ptlIcon.y) ?
                                      preccEnumArray : preccFirstDel;
  }

  /*******************************************************************/
  /* Flowed Name / Text View                                         */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_TREE) &&
      (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
      (pccinfo->flWindowAttr & CV_FLOW))
  {
    if ((preccEnumArray->ptlNonIconPos.x <
         preccFirstDel->ptlNonIconPos.x) ||
        ((preccEnumArray->ptlNonIconPos.x ==
          preccFirstDel->ptlNonIconPos.x) &&
         (preccEnumArray->ptlNonIconPos.y <
          preccFirstDel->ptlNonIconPos.y)))
    {
      return (preccEnumArray);
    }
    else
    {
      return (preccFirstDel);
    }
  }

  /*******************************************************************/
  /* Tree view or Non_Flowed Name/Text view                          */
  /*******************************************************************/
  if ((pccinfo->flWindowAttr & CV_TREE) ||
      (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT | CV_DETAIL)))
  {
    return (preccEnumArray->ptlNonIconPos.y <
            preccFirstDel->ptlNonIconPos.y) ?
                         preccEnumArray : preccFirstDel;
  }

  return (preccFirstDel);
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     NeedUpdateWorkplace                            */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR NeedUpdateWorkplace(hwnd, hps, preccObject)
HANDLE         hwnd;
HDC            hps;
LPRECORDCOREN  preccObject;
{
  LPCNRINFON  pccinfon;                 /* Internal CNR control data  */
  LPCNRINFO   pccinfo;
  RECT        rctlIcon;                 /* rectangle of the icon */
  RECT        rctlText;                 /* rectangle of the icon text */
  POINT       slIcon;                  /* size of an icon */
  SIZEL       slText;                  /* size of the icon text */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
  {
    return FALSE;
  }

  if (preccObject)
  {
    CnrCalcIconSize(hwnd, &slIcon);

    if (!(pccinfo->flWindowAttr & CV_TREE) &&
         (pccinfo->flWindowAttr & CV_ICON))
    {
      /***************************************************************/
      /* If in icon view, text size of the icon text is calculated   */
      /* already.                                                    */
      /***************************************************************/
      slText = preccObject->slTextSize;
    }
    else
    {
      /***************************************************************/
      /* Calculate size of icon text                                 */
      /***************************************************************/
      IcnCalcTextSize (hps, slIcon, pccinfon,
                       preccObject->preccX->pszIcon,
                       IS_BENEATHTEXT, &slText);
    }

    /*****************************************************************/
    /* Calculate icon rectl which is relative to the workplace       */
    /* origin. (in cnrcfunc.c )                                      */
    /*****************************************************************/
    GetIconTextRectInWorkplace(hwnd, preccObject, &slIcon,
                               &slText, &rctlIcon, &rctlText);

    /*****************************************************************/
    /* Check if at least one of the boundaries of the item to        */
    /* be deleted is the boundary of the workplace.                  */
    /*****************************************************************/
    if ((pccinfon->rctlWorkplaceItems.left >=
                                      (rctlText.left - WIN_MARGIN)) ||
        (pccinfon->rctlWorkplaceItems.right <=
                                     (rctlText.right + WIN_MARGIN)) ||
        (pccinfon->rctlWorkplaceItems.top >=
                                       (rctlIcon.top - WIN_MARGIN)) ||
        (pccinfon->rctlWorkplaceItems.bottom <=
                                       (rctlText.bottom + WIN_MARGIN)))
    {
      pccinfon->ulFlagSet &= ~FL_SETWORKPLACE;
      return TRUE;
    }
  }
  return FALSE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     UpdateCursorForDel                             */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
void FAR UpdateCursorForDel(hwnd, preccDel)
HANDLE          hwnd;
LPRECORDCOREN   preccDel;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  DWORD         ulAttribute;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /************************************************************/
  /* If the record being removed is cursored, reset the       */
  /* pointer.                                                 */
  /************************************************************/
  if ((pccinfon->preccCursor == preccDel) ||
      TRVIsAncestor(preccDel, pccinfon->preccCursor) )
  {
    LPRECORDCOREN  preccOldCursor = pccinfon->preccCursor;

    pccinfon->preccCursor = GetPrevUnfilteredItem (preccDel);

    if (!pccinfon->preccCursor)
    {
      pccinfon->preccCursor = GetNextUnfilteredItem (preccDel);
    }

    if (!pccinfon->preccCursor)
    {
      pccinfon->preccCursor = preccDel->preccParent;
    }

    ulAttribute = CRA_CURSORED;

    /***************************************************************/
    /* If container is single selection, apply selection           */
    /* emphasis to the cursored record.                            */
    /***************************************************************/
    if ((pccinfo->flWindowAttr & CV_TREE) ||
        (!(GetWindowLong (hwnd, GWL_STYLE) &
           (CCS_EXTENDSEL | CCS_MULTIPLESEL)))  )
    {
       ulAttribute |= CRA_SELECTED;
    }

    /***************************************************************/
    /* Change the emphasis bits for the old cursored record and    */
    /* the new cursored record.                                    */
    /***************************************************************/
    if (!SetEmphasisBits (hwnd, NULL, preccOldCursor,
                          ulAttribute , SEB_CLEAR))
    {
      return;
    }

    if (pccinfon->preccCursor)
    {
      if (!SetEmphasisBits (hwnd, NULL, pccinfon->preccCursor,
                            ulAttribute , SEB_SET))
      {
        return;
      }
    }
  }
  return;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  DeleteRecord                                     */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Delete Record                                */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function deletes a record from the link lists            */
/*     (item order/Z order). If flItemOrder is TRUE,                 */
/*     delete from item order linked list, else, delete from         */
/*     Z order linked list.                                          */
/*                                                                   */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration: void DeleteRecord(                       */
/*                               HANDLE        hwnd                  */
/*                               LPRECORDCORE preccObject            */
/*                               BOOL        flItemOrder             */
/*                               )                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*           preccObject - item to be deleted                        */
/*           flItemOrder - flag to indicate a specific order         */
/*                         TRUE : item order                         */
/*                         FALSE : Z order                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
void FAR DeleteRecord(hwnd, preccObject, flItemOrder)
HANDLE           hwnd;
LPRECORDCOREN    preccObject;
BOOL             flItemOrder;

{
  LPCNRINFON     pccinfon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  if (flItemOrder)
  {
    /*****************************************************************/
    /* Delete the record in item order linked list                   */
    /* Code itself is self-explanatory, So, no more comments.        */
    /*****************************************************************/
    if (preccObject->preccPrevItem)
    {
      preccObject->preccPrevItem->preccNextItem =
                                  preccObject->preccNextItem;
    }
    else
    {
      if (preccObject->preccParent)
      {
        preccObject->preccParent->preccFirstChild =
                                  preccObject->preccNextItem;
      }
      else
      {
        pccinfon->preccItem = preccObject->preccNextItem;
      }
    }

    if (preccObject->preccNextItem)
    {
      preccObject->preccNextItem->preccPrevItem =
                                  preccObject->preccPrevItem;
    }
    else
    {
      if (!preccObject->preccParent)
      {
        pccinfon->preccLastItem = preccObject->preccPrevItem;
      }
    } /* endif */

  }
  else
  {
    /*****************************************************************/
    /* Delete the record in Z order linked list                      */
    /* Code itself is self-explanatory, So, no more comments.        */
    /*****************************************************************/
    if (preccObject->preccPrevZ)
    {
      preccObject->preccPrevZ->preccNextZ = preccObject->preccNextZ;
    }
    else
    {
      if (!preccObject->preccParent)
      {
        pccinfon->preccZ = preccObject->preccNextZ;
      }
    } /* endif */
    if (preccObject->preccNextZ)
    {
      preccObject->preccNextZ->preccPrevZ = preccObject->preccPrevZ;
    }
    else
    {
      if (!preccObject->preccParent)
      {
        pccinfon->preccZTail = preccObject->preccPrevZ;
      }
    } /* endif */
  } /* endif */

  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrRemoveRecordN                                  */
/*                                                                    */
/*  Description:    Removes itself and all childrens internal         */
/*                  records and all associated sub-trees.             */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                  preccLink - record for which internal record is   */
/*                             to be removed.                         */
/*                                                                    */
/*  Output:         void                                              */
/*                                                                    */
/*  Programmer: Peter Haggar and Dr. Woo                              */
/**********************************************************************/
void FAR CnrRemoveRecordN (					/*RKK1*/
  HANDLE        hwndCnr,
  LPRECORDCORE  preccLink,
  WORD          flFree  )					/*RKK1*/
{
  LPCNRINFON    pccinfon;
  LPRECORDCOREN preccnRemove;
  LPRECORDCORE  preccPrev;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  if (flFree)
  {
    pccinfon->preccFreeFirst = NULL;
    pccinfon->preccFreeLast  = NULL;
  }

  if (preccnRemove = GetRecordNFromX(hwndCnr, preccLink))
  {
    if (preccnRemove->preccFirstChild)
    {
      RemoveChildrenRecordN(hwndCnr, preccnRemove->preccFirstChild,
                            flFree);
    }

    RemoveRecordN(hwndCnr, preccnRemove, flFree);
  }

  preccPrev = preccLink;
  preccLink = preccLink->preccNextRecord;

  while (preccLink)
  {
    if (preccnRemove = GetRecordNFromX(hwndCnr, preccLink))
    {
      if (preccnRemove->preccFirstChild)
      {
        RemoveChildrenRecordN(hwndCnr, preccnRemove->preccFirstChild,
                              flFree);
      }
      RemoveRecordN(hwndCnr, preccnRemove, flFree);
      preccPrev = preccLink;
    }
    else
    {
      preccPrev->preccNextRecord = preccLink->preccNextRecord;
    }
    preccLink = preccLink->preccNextRecord;
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  RemoveChildrenRecordN                             */
/*                                                                    */
/*  Description:    Removes all children records from the record      */
/*                  passed in.                                        */
/*                                                                    */
/*  Input:          hwndCnr - container window handle                 */
/*                  preccnRemove - Internal record for which children */
/*                             and siblings are removed.              */
/*                                                                    */
/*  Output:         much confusion because this function is recursive */
/*                                                                    */
/*  Programmer: Peter Haggar and Dr. Woo                              */
/**********************************************************************/
void FAR RemoveChildrenRecordN(HANDLE hwndCnr, LPRECORDCOREN preccnRemove,
                                WORD flFree)
{
  LPRECORDCOREN  preccnNext;

  while (preccnRemove)
  {
    preccnNext = preccnRemove->preccNextItem;
    if (preccnRemove->preccFirstChild)
    {
      RemoveChildrenRecordN(hwndCnr, preccnRemove->preccFirstChild,
                           flFree);
    }
    RemoveRecordN(hwndCnr, preccnRemove, flFree);
    preccnRemove = preccnNext;
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  RemoveRecordN                                     */
/*                                                                    */
/*  Description:    Removes the internal record structure for the     */
/*                  given record for the container window specified   */
/*                  by hwndCnr.                                       */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                  preccnRemove - record for which internal record   */
/*                                 is to be removed.                  */
/*                                                                    */
/*  Output:         void                                              */
/*                                                                    */
/*  Programmer: Peter Haggar and Dr. Woo                              */
/**********************************************************************/
void FAR RemoveRecordN(HANDLE hwndCnr, LPRECORDCOREN preccnRemove,
                        WORD flFree)
{
  LPCNRINFON     pccinfon;
//LPVOID         pSatPtr;
  LPRECORDCOREN  preccnPrev;
  LPRECORDCORE   preccx;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
//pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);

  /*********************************************************************/
  /* Get first internal block for this record.                         */
  /*********************************************************************/
  preccnPrev = PRECCFIRSTNFROMX(preccnRemove->preccX);

  /*********************************************************************/
  /* Check to see if first one in list.                                */
  /*********************************************************************/
  if (preccnPrev == preccnRemove)
  {
    preccx = preccnRemove->preccX;
    PRECCFIRSTNFROMX(preccx) = preccnRemove->preccnNext;
//  StmFreeStorage (pSatPtr, (LPVOID)preccnRemove);
    GlobalFree( (LPVOID)preccnRemove );                                 /*DAB1*/

    if (!(PRECCFIRSTNFROMX(preccx)) && flFree)
    {
      if (pccinfon->preccFreeFirst)
      {
        *(PLPRECORDCORE)(LPVOIDFROMPRECCX(pccinfon->preccFreeLast)) =
                                                            preccx;
        pccinfon->preccFreeLast = preccx;
      }
      else
      {
        pccinfon->preccFreeFirst = preccx;
        pccinfon->preccFreeLast  = preccx;
      }
    }
  }
  else
  {
    while ((preccnPrev) && (preccnPrev->preccnNext != preccnRemove))
    {
      preccnPrev = preccnPrev->preccnNext;
    }

    if (preccnPrev)
    {
      preccnPrev->preccnNext = preccnRemove->preccnNext;
//    StmFreeStorage (pSatPtr, (LPVOID)preccnRemove);
      GlobalFree( (LPVOID)preccnRemove );                               /*DAB1*/
    }
  }
  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     GetNextVisualItem                              */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR GetNextVisualItem (hwnd, preccPrev)
HANDLE         hwnd;
LPRECORDCOREN  preccPrev;
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;

  pccinfon = PCCINFONOF( hwnd );
  pccinfo  = PCCINFOXFROMN(pccinfon);

  return (pccinfo->flWindowAttr & CV_TREE) ?
    TRVGetNextVisualItem (preccPrev) :
     GetNextUnfilteredItem (preccPrev);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     GetPrevVisualItem                              */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR GetPrevVisualItem (hwnd, preccNext)
HANDLE         hwnd;
LPRECORDCOREN  preccNext;
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;

  pccinfon = PCCINFONOF( hwnd );
  pccinfo  = PCCINFOXFROMN(pccinfon);

  return (pccinfo->flWindowAttr & CV_TREE) ?
    TRVGetPrevVisualItem(hwnd,preccNext) :
      GetPrevUnfilteredItem (preccNext);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     GetNextItem                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR GetNextItem (pccinfon, preccPrev)
LPCNRINFON     pccinfon;
LPRECORDCOREN  preccPrev;
{
  LPCNRINFO  pccinfo;

  pccinfo = PCCINFOXFROMN(pccinfon);

  return (pccinfo->flWindowAttr & CV_TREE) ?
     TRVGetNextItem(preccPrev) : preccPrev->preccNextItem;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     GetPrevItem             xtai                   */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR GetPrevItem (pccinfon, preccNext)
LPCNRINFON     pccinfon;
LPRECORDCOREN  preccNext;
{
  LPCNRINFO  pccinfo;

  pccinfo = PCCINFOXFROMN(pccinfon);

  return (pccinfo->flWindowAttr & CV_TREE) ?
     TRVGetPrevItem(preccNext) : preccNext->preccPrevItem;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     InvalidateForRemove                            */
/*                                                                    */
/*  Descriptive Name:   Control Invalidate Record to Refresh          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*   xtai                                                             */
/**********************************************************************/
BOOL FAR InvalidateForRemove(					/*RKK1*/
  HANDLE          hwnd,
  HDC             hps,
  LPRECORDCOREN   preccInvalid,
  BOOL            invalidateAll,
  WORD            cNumRecords,
  LPRECORDCORE FAR *pRecordArray    /* array of the records to remove */
			)					/*RKK1*/
{
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  RECT            rctlInvalid;
  WORD            i;
  LPRECORDCOREN   precord;          /* record to be invalidated       */
  RECT            rctlText;
  RECT            rctlIcon;
  RECT            rctlTextCur;
  RECT            rctlIconCur;
  POINT           iconSize;
  WORD            usOldLongestLine;
  int             sOldHscrollPos;
  int             sOldVscrollPos;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    return (TRVInvalidateForRemove (hwnd, hps, preccInvalid,
                                     invalidateAll));
  }

  if (invalidateAll)
  {
    pccinfon->ulFlagSet &= ~( FL_CNRPOSITION | FL_NEWVIEWSCROLLSET |
               FL_SETSCROLL | FL_POSITIONCOLTITLES | FL_SIZECOLTITLES);

    if (GetWindowLong (hwnd, GWL_STYLE) & CCS_AUTOPOSITION)
    {
      pccinfon->ulFlagSet |= FL_NEEDARRANGE;
    }

    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       if (!(CnrSetUpNewViewForDV (hwnd, hps)))
       {
          return (FALSE);
       }
    }
    else
    {
      CnrSetUpNewView (hwnd, hps);
    }
  }
  else
  {
    if ((pccinfo->flWindowAttr & CV_ICON) &&
        !(GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION))
    {
      pccinfon->ulFlagSet &= ~FL_CNRPOSITION;
      CnrSetUpNewView(hwnd, hps);

      /*****************************************************************/
      /*If we need to search more than 500   items refresh whole       */
      /*window, else erase each item.                                  */
      /*****************************************************************/
      if (!((cNumRecords == 1) || (pccinfo->cRecords <= 25) ||
            ((pccinfo->cRecords <= 50) && (cNumRecords > 1) &&
            (cNumRecords <= 10)) || ((pccinfo->cRecords <= 100) &&
             (cNumRecords > 1) && (cNumRecords <= 5)) ||
            ((pccinfo->cRecords <= 250) && (cNumRecords > 1) &&
            (cNumRecords <= 2)) ))
      {
        invalidateAll = TRUE;
      }
      else
      {
        /***************************************************************/
        /* Erase the record, and redraw the items which were under     */
        /* the item erased.                                            */
        /***************************************************************/
        for (i = 0; i < cNumRecords ; i++)
        {
          precord = GetRecordNFromX (hwnd, *(pRecordArray + i));
          CnrEraseIcon( hwnd, hps, precord, TRUE);
        }
        return TRUE;
      }
    }
    else
    {
      if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
          !(pccinfo->flWindowAttr & CV_FLOW) &&
          pccinfon->preccItem )
      {
        usOldLongestLine = pccinfon->preccItem->usLongestLine;
      }

      pccinfon->ulFlagSet &= ~( FL_CNRPOSITION | FL_POSITIONCOLTITLES |
                           FL_SIZECOLTITLES | FL_SETSCROLL );
      sOldVscrollPos = pccinfon->sVscrollPos;
      sOldHscrollPos = pccinfon->sHscrollPos;

      if (GetWindowLong (hwnd, GWL_STYLE) & CCS_AUTOPOSITION)
      {
        pccinfon->ulFlagSet |= FL_NEEDARRANGE;
      }

      if (pccinfo->flWindowAttr & CV_DETAIL)
      {
         if (!(CnrSetUpNewViewForDV (hwnd, hps)))
         {
            return (FALSE);
         }
      }
      else
      {
         CnrSetUpNewView(hwnd, hps);
      }

      if ((sOldVscrollPos == pccinfon->sVscrollPos) &&
          (sOldHscrollPos == pccinfon->sHscrollPos))
      {
        GetInvalidateRect(hwnd, preccInvalid, &rctlInvalid);

        /**************************************************************/
        /* If the items inserted causes the change of the longest     */
        /* line of the column in non_flowed Name/Text view we need to */
        /* redraw the selected or cursored items which are before     */
        /* the first inserted item if they are in viewport.           */
        /**************************************************************/
        if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
            !(pccinfo->flWindowAttr & CV_FLOW) &&
            pccinfon->preccItem &&
            (usOldLongestLine != pccinfon->preccItem->usLongestLine))
        {
          precord = GetFirstItemInView(hwnd);
          preccInvalid = GetPrevUnfilteredItem (precord);
          CnrCalcIconSize(hwnd, &iconSize);

          while (preccInvalid && (precord != preccInvalid) &&
                 (precord->ptlNonIconPos.y <
                  rctlInvalid.top))
          {
            CnrQueryIconTextRect(hwnd, precord, &iconSize,
                                 FALSE, &rctlIcon, &rctlText);
            rctlTextCur.left = rctlText.left - TXT_TOPMARGIN;
            rctlTextCur.right = rctlText.right + TXT_TOPMARGIN;
            rctlTextCur.bottom = rctlText.bottom - TXT_TOPMARGIN;
            rctlTextCur.top = rctlText.top;


            if ((precord->flRecordAttr & CRA_SELECTED) &&
                !((pccinfo->flWindowAttr & CV_TEXT) &&
                  (precord->flRecordAttr & CRA_CURSORED)))
            {
              CnrDrawTextOnly(hwnd, hps, precord,
                              DRAW_TEXT,
                              rctlText, rctlTextCur);
            }
            else
            {
              if ((pccinfo->flWindowAttr & CV_TEXT) &&
                  (precord->flRecordAttr & CRA_CURSORED))
              {
                CnrEraseRect(hwnd, hps, rctlTextCur);
                CnrDrawTextOnly(hwnd, hps, precord,
                                DRAW_TEXT | DRAW_CURSOR,
                                rctlText, rctlTextCur);
              }
            }
            if ((pccinfo->flWindowAttr & CV_NAME) &&
                (precord->flRecordAttr & CRA_CURSORED))
            {
              rctlIconCur.left = rctlIcon.left - ICN_EMPHSPACE;
              rctlIconCur.right = rctlIcon.right + ICN_EMPHSPACE;
              rctlIconCur.bottom = rctlIcon.bottom + ICN_EMPHSPACE;
              rctlIconCur.top = rctlIcon.top - ICN_EMPHSPACE +
                                ICN_SIDEMARGIN;
              CnrDrawIconOnly(hwnd, hps, precord,
                              DRAW_CURSOR,
                              rctlIcon, rctlIconCur);
            }
            precord = GetNextUnfilteredItem (precord);
          } /* endwhile */
        }
      }
      else
      {
        invalidateAll = TRUE;
      } /* endif */
    }
  } /* endif */

  InvalidateContainer( hwnd, invalidateAll, &rctlInvalid );
  return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     TRVInvalidateForRemove                         */
/*                                                                    */
/*  Descriptive Name:   Control Invalidate Record to Refresh          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR TRVInvalidateForRemove(hwnd, hps, preccInvalid, invalidateAll)
HANDLE         hwnd;
HDC            hps;
LPRECORDCOREN  preccInvalid;
BOOL           invalidateAll;

{
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  RECT            rctlInvalid;
  LPRECORDCOREN   preccFirstInView;
  LPRECORDCOREN   preccEnum;
  RECT            rctlText;
  RECT            rctlIcon;
  RECT            rctlTextCur;
  RECT            rctlIconCur;
  POINT           slIcon;
  POINT           slTreeIcon;
  WORD            usOldLongestLine;
  int             sOldHscrollPos;
  int             sOldVscrollPos;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  pccinfon->ulFlagSet &= ~( FL_CNRPOSITION | FL_SETSCROLL |
                       FL_POSITIONCOLTITLES | FL_SIZECOLTITLES );
  if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
  {
    pccinfon->ulFlagSet |= FL_NEEDARRANGE;
  }

  if (invalidateAll || !preccInvalid)
  {
    pccinfon->ulFlagSet &= ~FL_NEWVIEWSCROLLSET;
  }
  else
  {
    usOldLongestLine = preccInvalid->usLongestLine;

    sOldVscrollPos = pccinfon->sVscrollPos;
    sOldHscrollPos = pccinfon->sHscrollPos;

    CnrTRVSetUpNewView(hwnd, hps);

    if ((sOldVscrollPos == pccinfon->sVscrollPos) &&
        (sOldHscrollPos == pccinfon->sHscrollPos))
    {
      TRVGetInvalidateRect(hwnd, preccInvalid, &rctlInvalid);

      preccInvalid = GetPrevUnfilteredItem (preccInvalid);

      /****************************************************************/
      /* If the items removed  causes the change of the longest       */
      /* line of the siblings,                                        */
      /* redraw the selected or cursored items which are before       */
      /* the first inserted item if they are in viewport.             */
      /****************************************************************/
      if (preccInvalid && (usOldLongestLine !=
                           preccInvalid->usLongestLine))
      {
        preccFirstInView = GetFirstItemInView(hwnd);
        preccEnum = preccInvalid;
        while (preccEnum)
        {
          CnrCalcIconSize( hwnd, &slIcon );
          CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);
          CnrTRVQueryIconTextRect(hwnd, preccEnum, FALSE,
                                  slIcon, slTreeIcon,
                                  &rctlIcon, NULL, &rctlText);

          rctlTextCur.left = rctlText.left - TXT_TOPMARGIN;
          rctlTextCur.right = rctlText.right + TXT_TOPMARGIN;
          rctlTextCur.bottom = rctlText.bottom - TXT_TOPMARGIN;
          rctlTextCur.top = rctlText.top;


          if ((preccEnum->flRecordAttr & CRA_SELECTED) &&
              !((pccinfo->flWindowAttr & CV_TEXT) &&
                (preccEnum->flRecordAttr & CRA_CURSORED)))
          {
            CnrDrawTextOnly(hwnd, hps, preccEnum,
                            DRAW_TEXT,
                            rctlText, rctlTextCur);
          }
          else
          {
            if ((pccinfo->flWindowAttr & CV_TEXT) &&
                (preccEnum->flRecordAttr & CRA_CURSORED))
            {
              CnrEraseRect(hwnd, hps, rctlTextCur);
              CnrDrawTextOnly(hwnd, hps, preccEnum,
                              DRAW_TEXT | DRAW_CURSOR,
                              rctlText, rctlTextCur);
            }
          }
          if ((pccinfo->flWindowAttr & (CV_NAME | CV_ICON)) &&
              (preccEnum->flRecordAttr & CRA_CURSORED))
          {
            rctlIconCur.left = rctlIcon.left - ICN_EMPHSPACE;
            rctlIconCur.right = rctlIcon.right + ICN_EMPHSPACE;
            rctlIconCur.bottom = rctlIcon.bottom + ICN_EMPHSPACE;
            rctlIconCur.top = rctlIcon.top - ICN_EMPHSPACE +
                              ICN_SIDEMARGIN;
            CnrDrawIconOnly(hwnd, hps, preccEnum,
                            DRAW_CURSOR,
                            rctlIcon, rctlIconCur);
          }
          if (preccEnum == preccFirstInView)
          {
            preccEnum = NULL;
          }
          else
          {
            preccEnum = GetPrevUnfilteredItem (preccEnum);
          } /* endif */
        } /* endwhile */
      }
    }
    else
    {
      invalidateAll = TRUE;
    }
  } /* endif */

  if (invalidateAll)
  {
    PMInvalidateRect(hwnd, NULL, TRUE);
  }
  else
  {
    PMInvalidateRect(hwnd, &rctlInvalid, TRUE);
  }
  UpdateWindow (hwnd);

  return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     ChangeItemHeight                               */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function is used in    InvalidateRecord internally.       */
/*     It checks if the new text with the 'slNewTextSize' of 'precord'*/
/*     causes the change of the height of 'precord' in the            */
/*     name/text view.                                                */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR ChangeItemHeight(hwnd, precord, slNewTextSize, slIcon)
HANDLE         hwnd;
LPRECORDCOREN  precord;
SIZEL          slNewTextSize;
POINT          slIcon;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  int           cyCurrent;
  int           cyUpdate;

  if (precord)
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    CnrCalcIconSize( hwnd, &slIcon );
    /*****************************************************************/
    /* Get the old height and the new height of 'precord'.           */
    /* In name view, the height is maximum of the cy of the icon     */
    /* and cy of the text, and in the text view, the height is       */
    /* the text height.                                              */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_TREE)
    {
      POINT        slTreeIcon;

      CnrTRVCalcTreeIconSize (hwnd, &slTreeIcon);
      cyCurrent = TRVGetCy(hwnd, precord, slIcon, slTreeIcon);

      if (pccinfo->flWindowAttr & CV_NAME)
      {
        cyUpdate = max( slIcon.y, slNewTextSize.cy );                   /*DAB1*/
      }
      else
      {
        cyUpdate = max( slNewTextSize.cy, slTreeIcon.y );               /*DAB1*/
        if (!(pccinfo->flWindowAttr & CV_TEXT))
        {
          cyUpdate = max(slIcon.y, cyUpdate);
        }
      }
    }
    else
    {
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        cyCurrent = max(slIcon.y + 2 * ICN_EMPHSPACE,
                        precord->slTextSize.cy);                        /*DAB1*/
        cyUpdate = max(slIcon.y + 2 * ICN_EMPHSPACE,
                       slNewTextSize.cy);                               /*DAB1*/
      }
      else
      {
        cyCurrent = precord->slTextSize.cy;                             /*DAB1*/
        cyUpdate = slNewTextSize.cy;                                    /*DAB1*/
      }
    }
    /*****************************************************************/
    /* Check if the height of 'precord' changed enough (more than    */
    /* 2 pixels)                                                     */
    /*****************************************************************/
    if ((cyUpdate >= cyCurrent + 1) ||
        (cyUpdate <= cyCurrent - 1))
    {
      return TRUE;
    }
    else
    {
      return FALSE;
    }
  }
  else
  {
    return FALSE;
  }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     ChangeRowHeight                                */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function is used in    InvalidateRecord internally.       */
/*     It checks if the new text with the 'slNewTextSize' of 'precord'*/
/*     causes the change of the max height of the row, in which       */
/*     'precord' is, in autopositioned icon view.                     */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR  ChangeRowHeight(hwnd, precord, slNewTextSize)
HANDLE          hwnd;
LPRECORDCOREN   precord;
SIZEL           slNewTextSize;
{
  LPCNRINFON     pccinfon;
  LPRECORDCOREN  preccPrev;
  LPRECORDCOREN  preccNext;
  int           cyHighest;

  /********************************************************************/
  /* If a valid record was passed                                     */
  /********************************************************************/
  if (precord)
  {
    /******************************************************************/
    /* if cy of the text is changed enough(more than 2 pixels)        */
    /* check if it causes the change of the height of the row.        */
    /******************************************************************/
    if ((precord->slTextSize.cy >= slNewTextSize.cy + 1) ||             /*DAB1*/
        (precord->slTextSize.cy <= slNewTextSize.cy - 1))               /*DAB1*/
    {
      /****************************************************************/
      /* Get ptr to Container instance data                           */
      /****************************************************************/
      pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
      preccPrev = precord->preccPrevItem;
      preccNext = precord->preccNextItem;

      /****************************************************************/
      /* Initialize the cyHighest (cyHighest is the tallest height    */
      /* of the row excluding the height of 'precord').               */
      /****************************************************************/
      if (preccPrev && (preccPrev->ptlIcon.y == precord->ptlIcon.y))
      {
        /**************************************************************/
        /* Initialize the cyHighest as the height of the previous     */
        /* item of 'precord' in the row if any.                       */
        /**************************************************************/
        cyHighest = preccPrev->slTextSize.cy;                           /*DAB1*/
      }
      else
      {
        if (preccNext &&
            (preccNext->ptlIcon.y == precord->ptlIcon.y))
        {
          /************************************************************/
          /* Initialize the cyHighest as the height of the next       */
          /* item of 'precord' in the row if any.                     */
          /************************************************************/
          cyHighest = preccNext->slTextSize.cy;                         /*DAB1*/
        }
        else
        {
          /************************************************************/
          /* If 'precord' is the only item in the row, decide the     */
          /* change of the height of the row by comparing the new     */
          /* text height of 'precord' with the old text height.       */
          /************************************************************/
          if ((precord->slTextSize.cy <= (slNewTextSize.cy - 1)) ||     /*DAB1*/
              (precord->slTextSize.cy >= (slNewTextSize.cy + 1)))       /*DAB1*/
          {
            return TRUE;
          }
          else
          {
            return FALSE;
          }
        }
      } /* endif */

      /****************************************************************/
      /* Get cyHighest in the row.(cyHighest is the tallest height    */
      /* of the row excluding the height of 'precord').               */
      /****************************************************************/
      while (preccPrev &&
             (preccPrev->ptlIcon.y == precord->ptlIcon.y))
      {
        if ( preccPrev->slTextSize.cy >= cyHighest )                    /*DAB1*/
        {
          cyHighest = preccPrev->slTextSize.cy;                         /*DAB1*/
        }

        preccPrev = preccPrev->preccPrevItem;
      }

      while (preccNext &&
             (preccNext->ptlIcon.y == precord->ptlIcon.y))
      {
        if ( preccNext->slTextSize.cy >= cyHighest )                    /*DAB1*/
        {
          cyHighest = preccNext->slTextSize.cy;                         /*DAB1*/
        }

        preccNext = preccNext->preccNextItem;
      }

      /****************************************************************/
      /* If the old height of 'precord' was not the tallest in the    */
      /* row, and its new height is not the tallest either, the       */
      /* change of the height of 'precord' doesn't affect the         */
      /* height of the row.                                           */
      /****************************************************************/
      if ((cyHighest >= precord->slTextSize.cy) &&                      /*DAB1*/
          (cyHighest >= slNewTextSize.cy))                              /*DAB1*/
      {
        return FALSE;
      }
      else
      {
        return TRUE;
      }
    }
  }
  return FALSE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     ChangeColumnLength                             */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function is used in    InvalidateRecord internally.       */
/*     It checks if the new text with the 'slNewTextSize' of 'precord'*/
/*     causes the change of the length of the column, in which        */
/*     'precord' is, in the name/text view and returns the new        */
/*     length of the column if it changed, else, return 0.            */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
WORD FAR ChangeColumnLength (precord, slNewTextSize)

LPRECORDCOREN  precord;
SIZEL          slNewTextSize;
{
  LPRECORDCOREN  preccPrev;
  LPRECORDCOREN  preccNext;
  BOOL           isLongest = TRUE;
  int            cxLongest;

  if (precord)
  {
    /*****************************************************************/
    /* If the new cx of 'precord' is longer than the longest cx in   */
    /* the column, the column width  changes and the new cx of '     */
    /* 'precord' is the longest.                                     */
    /*****************************************************************/
    if (precord->usLongestLine < (WORD)slNewTextSize.cx)                /*DAB1*/
    {
      return (WORD)slNewTextSize.cx;                                    /*DAB1*/
    }
    else
    {
      /***************************************************************/
      /* Initialize the cxLongest (cxLongest is the longest length   */
      /* of the column excluding the length of 'precord').           */
      /***************************************************************/
      preccPrev = precord->preccPrevItem;
      preccNext = precord->preccNextItem;

      if (preccPrev &&
          (preccPrev->ptlNonIconPos.x == precord->ptlNonIconPos.x))
      {
        /*************************************************************/
        /* Initialize the cxLongest as the length of the previous    */
        /* item of 'precord' in the column if any.                   */
        /*************************************************************/
        cxLongest = preccPrev->slTextSize.cx;                           /*DAB1*/
      }
      else
      {
        if (preccNext &&
           (preccNext->ptlNonIconPos.x == precord->ptlNonIconPos.x))
        {
          /***********************************************************/
          /* Initialize the cxLongest as the length of the next      */
          /* item of 'precord' in the column if any.                 */
          /***********************************************************/
          cxLongest = preccNext->slTextSize.cx;                         /*DAB1*/
        }
        else
        {
          if ((precord->slTextSize.cx <= (slNewTextSize.cx - 1)) ||     /*DAB1*/
              (precord->slTextSize.cx >= (slNewTextSize.cx + 1)))       /*DAB1*/
          {
            return (WORD)(slNewTextSize.cx);                            /*DAB1*/
          }
          else
          {
            return (WORD)0;
          }
        }
      } /* endif */

      /***************************************************************/
      /* Get cyLongest in the column.                                */
      /***************************************************************/
      while (preccPrev && (preccPrev->ptlNonIconPos.x ==
             precord->ptlNonIconPos.x))
      {
        if ( preccPrev->slTextSize.cx > cxLongest)                      /*DAB1*/
        {
          cxLongest = preccPrev->slTextSize.cx;                         /*DAB1*/
        }

        preccPrev = preccPrev->preccPrevItem;
      }

      while (preccNext &&
             (preccNext->ptlNonIconPos.x == precord->ptlNonIconPos.x))
      {
        if ( preccNext->slTextSize.cx > cxLongest)                      /*DAB1*/
        {
          cxLongest = preccNext->slTextSize.cx;                         /*DAB1*/
        }

        preccNext = preccNext->preccNextItem;
      }

      /***************************************************************/
      /* If the old length of 'precord' was not the longest in the   */
      /* row, the width of the column doesn't change. Else, the      */
      /* longest text length is the max of cxLongest and the new cx  */
      /* of the text of 'precord'                                    */
      /***************************************************************/
      if (cxLongest >= precord->slTextSize.cx)                          /*DAB1*/
      {
        return (WORD)0;
      }
      else
      {
        return (WORD)(max(cxLongest, slNewTextSize.cx));                /*DAB1*/
      }
    }
  }

  return (WORD)0;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     UpdateColumnLongest                            */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function is used in    InvalidateRecord internally.       */
/*     It updates usLongestLine of the item in the column in          */
/*     which 'precord' is.                                            */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
VOID FAR UpdateColumnLongest(					/*RKK1*/
  HANDLE          hwnd,
  LPRECORDCOREN   precord,
  WORD            usLongestLine  )				/*RKK1*/
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccPrev;
  LPRECORDCOREN preccNext;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (!precord && !(pccinfo->flWindowAttr & CV_TREE))
  {
    precord = pccinfon->preccItem;
  }

  if (precord)
  {
    preccPrev = precord;
    preccNext = precord->preccNextItem;

    /******************************************************************/
    /* If there is a previous record and it is in the same column     */
    /******************************************************************/
    while (preccPrev &&
           (preccPrev->ptlNonIconPos.x == precord->ptlNonIconPos.x))
    {
      /****************************************************************/
      /* Update its longest line value to the one passed in for column*/
      /****************************************************************/
      preccPrev->usLongestLine = usLongestLine;
      preccPrev = preccPrev->preccPrevItem;
    }

    /******************************************************************/
    /* If there is a following record and it is in the same column    */
    /******************************************************************/
    while (preccNext &&
           (preccNext->ptlNonIconPos.x == precord->ptlNonIconPos.x))
    {
      /****************************************************************/
      /* Update its longest line value to the one passed in for column*/
      /****************************************************************/
      preccNext->usLongestLine = usLongestLine;
      preccNext = preccNext->preccNextItem;
    }
  }
  return;
}
