/* Revision: 21 1.3.1.2 source/ui/cclnt/cnr/cnrs3p4.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS3P4.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS3P4.C - Query info routines for container control    */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* DAB2  09/26/95  DAB  Corrected logic to calc details viewport rect */
/* DAB3  11/26/95  DAB  Adding logic for CRA_SOURCE emphasis          */
/* D6386  1/16/95  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* PPB1  01/29/96  PPB  Ensure scroll position > 0 defect 6948        */
/* PPB2  02/01/96  PPB  Adjust record rect by column titles.          */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/**********************************************************************/
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef SEGMENTOR
#define CNRS3P4_C
#include "cnrpragw.h"
#endif

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryRecordEmphasis                         */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: Queries the record with specified emphasis*/
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:    LPRECORDCORE pSearchAfter - Pointer to start record     */
/*                                                                    */
/*            WORD fEmphasisMask - specifies emphasis attribute       */
/*                                   of record.  May be:              */
/*                          CRA_CURSORED                              */
/*                          CRA_DISABLED                              */
/*                          CRA_INUSE                                 */
/*                          CRA_SELECTED                              */
/*                                                                    */
/*            WORD fSearchType - specifies search type.               */
/*                          CMA_SIBLINGS                              */
/*                          CMA_TREE                                  */
/*                                                                    */
/*  Output:   LPRECORDCORE pRecord - pointer to record requested      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: Pointer to record.                                   */
/*                                                                    */
/*  Exit-Error:  NULL - There are no records to satisfy query.        */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCORE FAR CnrQueryRecordEmphasis(hwnd, mp1, mp2)
HANDLE     hwnd;
long       mp1;
long       mp2;
{
  LPCNRINFON     pccinfon;              /* Internal CNR control data */
  LPCNRINFO      pccinfo;
  LPRECORDCORE   preccAfter;            /* Ptr to record, mp1        */
  LPRECORDCOREN  preccn;                /* Ptr to record, mp1        */
  DWORD          fEmphasisMask;         /* Attribute mask, mp2       */

  /*****************************************************************/
  /* Initialize pointers to control data and local variables       */
  /*****************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  preccAfter = (LPRECORDCORE)(LPVOID)mp1;
  fEmphasisMask = (DWORD)mp2;

  /*******************************************************************/
  /* If CCS_VERIFYPOINTERS is true and CnrVerifyPointers is false     */
  /* return an error condition                                       */
  /*******************************************************************/
  if (preccAfter != (LPRECORDCORE)CMA_FIRST)
  {
    if (GetWindowLong (hwnd, GWL_STYLE) & CCS_VERIFYPOINTERS)
    {
      if (((pccinfo->flWindowAttr & CV_TREE) &&
           !(CnrVerifyPointers(hwnd, (LPVOID)(&preccAfter), (long)1))) ||
          (!(pccinfo->flWindowAttr & CV_TREE) &&
           !(IsInRoot(hwnd, preccAfter))))
      {
        SETERROR (PMERR_INVALID_PARAMETERS);
        return((LPRECORDCORE)ERRORFLAG);
      }
    }
  }

  if (preccAfter == ((LPRECORDCORE)CMA_FIRST))
  {
    preccn = pccinfon->preccItem;
  }
  else
  {
    preccn = GetRecordNFromX( hwnd, preccAfter );
    if (preccn)
    {
      preccn = (pccinfo->flWindowAttr & CV_TREE) ?
                TRVGetNextItem(preccn) : preccn->preccNextItem;
    }
    else
    {
      SETERROR(PMERR_INVALID_PARAMETERS);
      return((LPRECORDCORE)ERRORFLAG);
    }
  }

#if 0
  /*******************************************************************/
  /* In the single selection mode and in the tree view, only one     */
  /* item is selected and cursored. So, if we query an item which    */
  /* has selection emph or cursor emph, we look at only one item,    */
  /* preccCursor.                                                    */
  /*******************************************************************/
  if ( (!(GetWindowLong(hwnd, GWL_STYLE) &
          (CCS_EXTENDSEL | CCS_MULTIPLESEL)) ||
        (pccinfo->flWindowAttr & CV_TREE)) &&
       (fEmphasisMask & (CRA_SELECTED | CRA_CURSORED)) )
  {
    /************************************************************/
    /* If item is filtered, get the next visual item so         */
    /* the function IsBefore will work properly.                */
    /************************************************************/
    if (!(ISRECORDVISIBLE(preccn)))
    {
      preccn = GetNextVisualItem (hwnd, preccn);
    }
    if (((pccinfon->preccCursor == preccn) ||
         (IsBefore(hwnd, preccn, pccinfon->preccCursor))) &&
       ((pccinfon->preccCursor->flRecordAttr & fEmphasisMask) ==
           fEmphasisMask))
    {
      COPYNDATATOX (pccinfon->preccCursor);
      return(pccinfon->preccCursor->preccX);
    }
    else
    {
      return NULL;
    }
  }
#endif

  /*******************************************************************/
  /* Beginning with the given record, compare each record with the   */
  /* given emphasis mask.  If the result of the & of the record's    */
  /* attributes and the mask is equal to the mask, the record        */
  /* satisfies the request.  The comparison is necessary because     */
  /* multiple attributes can be OR'd together in the mask and for    */
  /* the record to match the mask, all the attributes specified must */
  /* match.                                                          */
  /*******************************************************************/
  while (preccn)
  {
    if ((preccn->flRecordAttr & fEmphasisMask) == fEmphasisMask)
    {
      COPYNDATATOX (preccn);
      return(preccn->preccX);
    }
     preccn = (pccinfo->flWindowAttr & CV_TREE) ?
               TRVGetNextItem(preccn) : preccn->preccNextItem;
  }
  return NULL;
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     IsInRoot                                       */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR IsInRoot(hwnd, preccObj)
HANDLE        hwnd;
LPRECORDCORE  preccObj;
{
  LPCNRINFON    pccinfon;
  LPRECORDCOREN preccEnum;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  preccEnum = pccinfon->preccItem;
  while (preccEnum)
  {
    if (preccEnum->preccX == preccObj)
    {
      return TRUE;
    }
    preccEnum = preccEnum->preccNextItem;
  }
  return FALSE;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrQueryRecordInfo                                */
/*                                                                    */
/*  Description:    Returns the record information for the record     */
/*                  passed in.                                        */
/*                                                                    */
/*  Input:          hwndCnr - container window                        */
/*                  mp1     - pointer to an array of records that     */
/*                            information is requested.               */
/*                  mp2     - number of records in array.             */
/*                                                                    */
/*  Output:         BOOL - TRUE:  Success                             */
/*                         FALSE: Failure                             */
/*                                                                    */
/*  Author: Peter Haggar                                              */
/**********************************************************************/
BOOL FAR CnrQueryRecordInfo (hwndCnr, mp1, mp2)

HANDLE  hwndCnr;
long    mp1;
long    mp2;
{
 LPCNRINFON     pccinfon;
 LPRECORDCOREN  preccn;
 LPRECORDCORE   FAR *pRecordArray;
 WORD           cNumRecord;
 WORD           i;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Get array of the records and the  number of the records           */
 /* to be updated.                                                    */
 /*********************************************************************/
 pRecordArray = (LPVOID)mp1;
 cNumRecord   = LOWORD(mp2);

 /*********************************************************************/
 /* If verify pointers is set, run the list through this function.    */
 /*********************************************************************/
 if (GetWindowLong (hwndCnr, GWL_STYLE) & CCS_VERIFYPOINTERS)
 {
   if (!(CnrVerifyPointers (hwndCnr, pRecordArray, (long)cNumRecord)))
   {
     SETERROR(PMERR_INVALID_PARAMETERS);
     return (FALSE);
   }
 }

 /*********************************************************************/
 /* If they specified a number to be updated, loop though them and    */
 /* update them as we go.                                             */
 /*********************************************************************/
 if (cNumRecord)
 {
   for (i = 0; i < cNumRecord; i++)
   {
     /*****************************************************************/
     /* Get the internal record for the external one.                 */
     /*****************************************************************/
     preccn = GetRecordNFromX (hwndCnr, pRecordArray[i]);
     if (preccn)
     {
       /*****************************************************************/
       /* Copy the data for the 2 fields that are common to both the    */
       /* external record and the internal record.....from the internal */
       /* to the external.                                              */
       /*****************************************************************/
       COPYNDATATOX(preccn);
     }
     else
     {
       SETERROR(PMERR_INVALID_PARAMETERS);
       return (FALSE);
     }
   }
 }
 else
 {
   /*******************************************************************/
   /* Otherwise, they want to update all records inserted in this     */
   /* container.  Start with the first one, and blast through them    */
   /* all utill we're finished.                                       */
   /*******************************************************************/
   if (pccinfon->preccItem)
   {
     preccn = GetRecordNFromX (hwndCnr, pccinfon->preccItem->preccX);

     while (preccn)
     {
       /*****************************************************************/
       /* Copy the 2 fields that are common between the external record */
       /* and the internal record.                                      */
       /*****************************************************************/
       COPYNDATATOX(preccn);

       /*****************************************************************/
       /* Get the next record in the container.                         */
       /*****************************************************************/
       preccn = GetNextItem (pccinfon, preccn);
     }
   }
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryRecordFromRect                         */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:    LPRECORDCORE pSearchAfter - Pointer to start record     */
/*                                                                    */
/*            PRECORDSEARCH pRecordSearch - data structure            */
/*             RECT rect - Rectangle to query, in                     */
/*             virtual coordinates                                    */
/*             DWORD  fsSearch - Search control flags. May be:        */
/*                               One of:                              */
/*                               CMA_COMPLETE - wholly bounded        */
/*                               (default)                            */
/*                               CMA_PARTIAL  - partially bounded     */
/*                               One of:                              */
/*                               CMA_ITEMORDER - enumerate in item    */
/*                                               order  (default)     */
/*                               CMA_ZORDER - enumerate in z order    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:   LPRECORDCORE pRecord - pointer to record within bounding*/
/*                                   rectangle.                       */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: Pointer to record within bounding rectangle.         */
/*                                                                    */
/*  Exit-Error:  NULL - There are no records within bounding rectangle*/
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR CnrQueryRecordFromRect(HANDLE hwnd, long mp1, long mp2)
{
  LPCNRINFON    pccinfon;               /* Internal CNR control data */
  LPCNRINFO     pccinfo;
  LPQUERYRECFROMRECT pQueryRecFromRect; /* mp2 record search struct  */
  POINT         slIcon;                 /* Icon size for view        */
  POINT         slTreeIcon;
  RECT          rctlIcon;
  RECT          rctlTreeIcon;
  RECT          rctlText;
  RECT          rctlRow;
  RECT          rctlDest;               /* Text rectangle            */
  LPRECORDCORE  preccEnum;
  LPRECORDCOREN preccn;
  BOOL          found = FALSE;

  /*****************************************************************/
  /* Initialize pointers to control data and local variables       */
  /*****************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  preccEnum = (LPRECORDCORE)(LPVOID)mp1;
  pQueryRecFromRect = (LPQUERYRECFROMRECT)(LPVOID)mp2;

  if ((!preccEnum) || (!pQueryRecFromRect) ||
      (pQueryRecFromRect->cb != sizeof (QUERYRECFROMRECT)))
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return((LPRECORDCOREN)ERRORFLAG);
  }

  if (preccEnum == ((LPRECORDCORE)CMA_FIRST))
  {
    /*****************************************************************/
    /* In tree, icon, name and text views, we search the records     */
    /* in the rectangle in the item order.                           */
    /*****************************************************************/
    if ((pccinfo->flWindowAttr & (CV_TREE | CV_NAME | CV_TEXT | CV_DETAIL))
        || (pQueryRecFromRect->fsSearch & CMA_ITEMORDER))
    {
      preccn = pccinfon->preccItem;
    }
    else
    {
      if(pQueryRecFromRect->fsSearch & CMA_ZORDER)
      {
        preccn = pccinfon->preccZ;
      }
      else
      {
        SETERROR(PMERR_INVALID_PARAMETERS);
        return((LPRECORDCOREN)ERRORFLAG);
      }
    } /* endif */
  }
  else
  {
    if (GetWindowLong(hwnd, GWL_STYLE ) & CCS_VERIFYPOINTERS)
    {
      if (!(CnrVerifyPointers(hwnd, (LPVOID)(&preccEnum), (long)1)))
      {
        SETERROR (PMERR_INVALID_PARAMETERS);
        return((LPRECORDCOREN)ERRORFLAG);
      }
      if (!(pccinfo->flWindowAttr & CV_TREE) && preccn->preccParent)
      {
        return NULL;
      } /* endif */
    }

    /******************************************************************/
    /* Get internal record from the external one passed in.           */
    /******************************************************************/
    preccn = GetRecordNFromX (hwnd, preccEnum);
    if (preccn)
    {
      if ((pccinfo->flWindowAttr &
           (CV_TREE | CV_NAME | CV_TEXT | CV_DETAIL)) ||
          (pQueryRecFromRect->fsSearch & CMA_ITEMORDER))
      {
        preccn = GetNextVisualItem(hwnd, preccn);
      }
      else
      {
        if(pQueryRecFromRect->fsSearch & CMA_ZORDER)
        {
          preccn = preccn->preccNextZ;
        }
        else
        {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return((LPRECORDCOREN)ERRORFLAG);
        }
      } /* endif */
    }
    else
    {
      SETERROR(PMERR_INVALID_PARAMETERS);
      return((LPRECORDCOREN)ERRORFLAG);
    } /* endif */
  } /* endif */

  CnrCalcIconSize(hwnd, &slIcon);

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    CnrTRVCalcTreeIconSize (hwnd, &slTreeIcon);
  }

  /*******************************************************************/
  /* Loop through the records until either one is found which        */
  /* satisfies the requirements of being completely or partially in  */
  /* the rectangle or until the end of the records is reached.       */
  /*******************************************************************/
  while (preccn)
  {
    if (ISRECORDVISIBLE(preccn))
    {
      if (pccinfo->flWindowAttr & CV_DETAIL)
      {
        CnrQueryRowRectForDV (hwnd, preccn, &rctlRow);

        /*************************************************************/
        /* Since we ignore the x coordinates of the apps rectangle,  */
        /* let's set them to some arbitrary values so that           */
        /* IntersectRect doesn't fail.                               */
        /*************************************************************/
        pQueryRecFromRect->rect.left  = 0;
        pQueryRecFromRect->rect.right = 10;
      }
      else
      {
        /*************************************************************/
        /* Calculate the text and icon rectangles.                   */
        /*************************************************************/
        if (pccinfo->flWindowAttr & CV_TREE)
        {
          CnrTRVQueryIconTextRect(hwnd, preccn, FALSE, slIcon,
                                  slTreeIcon, &rctlIcon, &rctlTreeIcon,
                                  &rctlText);
        }
        else
        {
          CnrQueryIconTextRect(hwnd, preccn, &slIcon, FALSE,
                               &rctlIcon, &rctlText);
        }
      }

      /***************************************************************/
      /* If the record is partially within the given rectangle       */
      /* and the view is icon or name, the intersection of either the*/
      /* icon or text rectangles will satisfy the requirement.       */
      /***************************************************************/
      if (pQueryRecFromRect->fsSearch & CMA_PARTIAL)
      {
        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          if (PMIntersectRect ( &rctlDest,
                                &pQueryRecFromRect->rect,
                                &rctlRow))
          {
             COPYNDATATOX (preccn);
             return (preccn);
          }
        }
        else
        {
          if (PMIntersectRect( &rctlDest, &pQueryRecFromRect->rect,
              &rctlText))
          {
            COPYNDATATOX (preccn);
            return (preccn);
          } /* endif */

          if (pccinfo->flWindowAttr & (CV_ICON | CV_NAME))
          {
            if (PMIntersectRect( &rctlDest,
                                 &pQueryRecFromRect->rect,
                                 &rctlIcon))
            {
              COPYNDATATOX (preccn);
              return (preccn);
            } /* endif */
          }

          if ((pccinfo->flWindowAttr & CV_TREE) &&
              !(pccinfo->flWindowAttr & CV_NAME))
          {
            if (PMIntersectRect( &rctlDest,
                                 &pQueryRecFromRect->rect,
                                 &rctlTreeIcon))
            {
              COPYNDATATOX (preccn);
              return (preccn);
            }
          }
        }
      }
      else
      {
        /**************************************************************/
        /* The text rectangle is compared to the destination rectangle*/
        /* first because all views have a text rectangle.  If this    */
        /* rectangle intersects the destination rectangle and if the  */
        /* the rectangles are equal, the found flag is                */
        /* set.  If in Icon or Name View, the icon rectangle is       */
        /* compared. If the icon rectangles are equal and the text    */
        /* rectangle was equal, the Icon or Name View record is within*/
        /* the rectangle.                                             */
        /**************************************************************/
        if (pQueryRecFromRect->fsSearch & CMA_COMPLETE)
        {
          if (pccinfo->flWindowAttr & CV_DETAIL)
          {
            /*********************************************************/
            /* Since we ignore the x values of the apps. rectangle,  */
            /* we just compare the y values.                         */
            /*********************************************************/
            if ((pQueryRecFromRect->rect.top <= rctlRow.top) &&
                (pQueryRecFromRect->rect.bottom >= rctlRow.bottom))
            {
               found = TRUE;
            }
          }
          else
          {
            if (PMIntersectRect( &rctlDest, &pQueryRecFromRect->rect,
                                 &rctlText))
            {
              if (PMEqualRect( &rctlText, &rctlDest))
              {
                found = TRUE;

                if (found && ((pccinfo->flWindowAttr & CV_ICON) ||
                              (pccinfo->flWindowAttr & CV_NAME)))
                {
                  if (PMIntersectRect( &rctlDest,
                                       &pQueryRecFromRect->rect,
                                       &rctlIcon))
                  {
                    if (!(PMEqualRect( &rctlIcon, &rctlDest)))
                    {
                      found = FALSE;
                    }
                  }
                  else
                  {
                     found = FALSE;
                  }
                }

                if (found && ((pccinfo->flWindowAttr & CV_TREE) &&
                              !(pccinfo->flWindowAttr & CV_NAME)))
                {
                  if (PMIntersectRect( &rctlDest,
                                       &pQueryRecFromRect->rect,
                                       &rctlTreeIcon))
                  {
                    if (!(PMEqualRect( &rctlTreeIcon, &rctlDest)))
                    {
                      found = FALSE;
                    }
                  }
                  else
                  {
                     found = FALSE;
                  }
                }
              }
            }
          }
          if (found)
          {
            COPYNDATATOX (preccn);
            return (preccn);
          }
        }
        else
        {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return((LPRECORDCOREN)ERRORFLAG);
        } /* endif */
      } /* endif */
    }
    /***************************************************************/
    /* Find next record based on enumeration order.                */
    /***************************************************************/
    if ((pccinfo->flWindowAttr &
         (CV_TREE | CV_NAME | CV_TEXT | CV_DETAIL)) ||
        (pQueryRecFromRect->fsSearch & CMA_ITEMORDER))
    {
      preccn = GetNextVisualItem (hwnd, preccn);
    }
    else
    {
      if(pQueryRecFromRect->fsSearch & CMA_ZORDER)
      {
        preccn = preccn->preccNextZ;
      }
    }
  } /* endwhile */

  return(NULL);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryRecordRect                             */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: Find rectangle for specified record.      */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:    LPRECT prclItem - Rectangle coordinates.                */
/*                                                                    */
/*            LPQUERYRECORDRECT pQueryRecordRect - data structure     */
/*            LPRECORDCORE - pRecord - Pointer to specified record.   */
/*            DWORD  fsExtent - Search control flags. May be:         */
/*                               One of:                              */
/*                               CMA_ICON - returns icon rectangle    */
/*                               CMA_TEXT - returns text rectangle    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:   BOOL fSuccess - Success indicator.                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: TRUE - A rectangle is returned.                      */
/*                                                                    */
/*  Exit-Error:  NULL - The rectangle is empty.                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrQueryRecordRect(HANDLE hwnd, long mp1, long mp2)
{
  LPCNRINFON          pccinfon;          /* Internal CNR control data */
  LPCNRINFO           pccinfo;
  LPRECT              prclItem;          /* mp1, rectangle returned   */
  RECT                textRect;          /* Text rectangle            */
  RECT                iconRect;          /* Icon rectangle            */
  RECT                treeIconRect;
  LPQUERYRECORDRECT   pQueryRecordRect;  /* mp2 data structure        */
  POINT               iconSize;          /* Icon size for current view*/
  POINT               slTreeIcon;
  long                CxBetweenCols;
  long                CyBetweenRows;
  RECT                ViewPortLeft;
  RECT                rctlTitle;
  LPRECORDCOREN       preccn;

  /*****************************************************************/
  /* Initialize pointers to control data and local variables       */
  /*****************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  prclItem = (LPRECT)(LPVOID)mp1;
  pQueryRecordRect = (LPQUERYRECORDRECT)(LPVOID)mp2;

  if ((!prclItem) || (!pQueryRecordRect) ||
      (!pQueryRecordRect->pRecord) ||
      (pQueryRecordRect->cb != sizeof (QUERYRECORDRECT)))
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return(FALSE);                                                      /*DAB1*/
  }

  /**************************************************************/
  /* If CCS_VERIFYPOINTERS is true and CnrVerifyPointers is false*/
  /* return an error condition                                  */
  /**************************************************************/
  if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_VERIFYPOINTERS)
  {
     if (!(CnrVerifyPointers(hwnd, (LPVOID)(&pQueryRecordRect->pRecord),
          (long)1)))
     {
       SETERROR (PMERR_INVALID_PARAMETERS);
       return(FALSE);                                                   /*DAB1*/
     }
  }

  /********************************************************************/
  /* Get the internal record from the external record.                */
  /********************************************************************/
  preccn = GetRecordNFromX (hwnd, pQueryRecordRect->pRecord);

  /*******************************************************************/
  /* Check if pRecord is visible. If not, return NULL.               */
  /*******************************************************************/
  if (!preccn || !ISRECORDVISIBLE(preccn) ||
      ((pccinfo->flWindowAttr & CV_TREE) && !IsTreeVisible (preccn)) ||
      (!(pccinfo->flWindowAttr & CV_TREE) && preccn->preccParent))
  {
    SETERROR (PMERR_INVALID_PARAMETERS);
    return(FALSE);                                                      /*DAB1*/
  }

  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
     /*****************************************************************/
     /* Get the distance between the columns.                         */
     /*****************************************************************/
     CxBetweenCols = 3 * (pccinfon->lAveCharWidth);

     /*****************************************************************/
     /* Get the distance between the rows.                            */
     /*****************************************************************/
     if (pccinfo->cyLineSpacing >= 0)
     {
        CyBetweenRows = pccinfo->cyLineSpacing;
     }
     else
     {
        CyBetweenRows = MINCY_BETWEENITEMS;
     }

     /*****************************************************************/
     /* Get the title window rectangle.                               */
     /*****************************************************************/
     CnrQuerySizeRect(hwnd, NULL, &rctlTitle, NULL, NULL, NULL);

     /*****************************************************************/
     /* Check to see if there are any columns to process.             */
     /*****************************************************************/
     if (pccinfon->pFirstFI)
     {
        if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
            (pQueryRecordRect->fRightSplitWindow))
        {
          if (CnrVerifyFieldInfoPtrs (hwnd,
                                      pccinfon->pFieldInfoLast,
                                      1))
          {
             if (NEXTVISIBLEFI(pccinfon->pFieldInfoLast))
             {
                prclItem->left = 0;

                prclItem->right = CnrQueryColumnPosInDV (hwnd,
                        NEXTVISIBLEFI(pccinfon->pFieldInfoLast),
                                                         NULL,
                                                         FALSE,
                                                         TRUE);

                prclItem->top = CnrQueryRowPosInDV (preccn, PT_TOP);

                prclItem->bottom = CnrQueryRowPosInDV (preccn,
                                                        PT_BOTTOM);

                prclItem->right  += (CxBetweenCols / 2);
                prclItem->bottom += (CyBetweenRows / 2);
                prclItem->top    -= (CyBetweenRows / 2);

                prclItem->left   -= pccinfon->RightWndxOrg;
                prclItem->right  -= pccinfon->RightWndxOrg;
                prclItem->bottom -= pccinfon->ptlNonIconOrg.y;
                prclItem->top    -= pccinfon->ptlNonIconOrg.y;

                PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                                 &ViewPortLeft);

                prclItem->left += ViewPortLeft.right +
                                   GetSystemMetrics ( SM_CXFRAME);

                prclItem->right += ViewPortLeft.right +
                                    GetSystemMetrics ( SM_CXFRAME);

                prclItem->top += rctlTitle.bottom;

                prclItem->bottom += rctlTitle.bottom;

                // Adjust for the columnTitles.                         /*PPB2*/
                PMGetClientRect (GetDlgItem (hwnd, CID_LEFTCOLTITLEWND),/*PPB2*/
                                 &rctlTitle);                           /*PPB2*/
                prclItem->top += rctlTitle.bottom - rctlTitle.top;      /*PPB2*/
                prclItem->bottom += rctlTitle.bottom - rctlTitle.top;   /*PPB2*/

                return (TRUE);
             }
             else
             {
                prclItem->left   = 0;
                prclItem->right  = 0;
                prclItem->bottom = 0;
                prclItem->top    = 0;

                return (TRUE);
             }
          }
        }
        else
        {
           if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
       (CnrVerifyFieldInfoPtrs (hwnd, pccinfon->pFieldInfoLast, 1)))
           {
              prclItem->right = CnrQueryColumnPosInDV (hwnd,
                                           pccinfon->pFirstVisibleFI,
                                            pccinfon->pFieldInfoLast,
                                                       FALSE,
                                                       FALSE);
           }
           else
           {
              prclItem->right = CnrQueryColumnPosInDV (hwnd,
                                           pccinfon->pFirstVisibleFI,
                                                        NULL,
                                                        FALSE,
                                                        TRUE);
           }

           prclItem->left = 0;

           prclItem->top = CnrQueryRowPosInDV (preccn, PT_TOP);

           prclItem->bottom = CnrQueryRowPosInDV (preccn, PT_BOTTOM);

           prclItem->right  += (CxBetweenCols / 2);
           prclItem->bottom += (CyBetweenRows / 2);
           prclItem->top    -= (CyBetweenRows / 2);

           prclItem->left   -= pccinfon->ptlNonIconOrg.x;
           prclItem->right  -= pccinfon->ptlNonIconOrg.x;
           prclItem->bottom -= pccinfon->ptlNonIconOrg.y;
           prclItem->top    -= pccinfon->ptlNonIconOrg.y;

           prclItem->top += rctlTitle.bottom;
           prclItem->bottom += rctlTitle.bottom;

           // Adjust for the columnTitles.                              /*PPB2*/
           PMGetClientRect (GetDlgItem (hwnd, CID_LEFTCOLTITLEWND),     /*PPB2*/
                            &rctlTitle);                                /*PPB2*/
           prclItem->top += rctlTitle.bottom - rctlTitle.top;           /*PPB2*/
           prclItem->bottom += rctlTitle.bottom - rctlTitle.top;        /*PPB2*/

           return (TRUE);
        }
     }
     return (FALSE);
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_TREE)
    {
      CnrCalcIconSize(hwnd, &iconSize);

      /****************************************************************/
      /* If the little Icon to the left of the record exists, get its */
      /* size too.  This little Icon exists in the Text and Icon      */
      /* views of Tree View.                                          */
      /****************************************************************/
      if (pccinfo->flWindowAttr & (CV_TEXT | CV_ICON))
      {
        CnrTRVCalcTreeIconSize (hwnd, &slTreeIcon);
      }

      /****************************************************************/
      /* Get all of the rectangles that we may need in the comming    */
      /* moments of execution.                                        */
      /****************************************************************/
      CnrTRVQueryIconTextRect (hwnd, preccn, FALSE, iconSize,
                               slTreeIcon, &iconRect, &treeIconRect,
                               &textRect);

      /****************************************************************/
      /* First, handle Icon View of Tree View.                        */
      /****************************************************************/
      if (pccinfo->flWindowAttr & CV_ICON)
      {
        if ((pQueryRecordRect->fsExtent & CMA_TREEICON) &&
            (pQueryRecordRect->fsExtent & CMA_ICON) &&
            (pQueryRecordRect->fsExtent & CMA_TEXT))
        {
          /************************************************************/
          /* Union the text rectangle and the big icon rectangle      */
          /* first.                                                   */
          /************************************************************/
          prclItem->left = min(textRect.left, iconRect.left);
          prclItem->right = max(textRect.right, iconRect.right);
          prclItem->top = min(textRect.top, iconRect.top);
          prclItem->bottom = max(textRect.bottom, iconRect.bottom);

          /************************************************************/
          /* Now union the little icon rectangle with the resulting   */
          /* rectangle from the first union.                          */
          /************************************************************/
          prclItem->left = min(prclItem->left, treeIconRect.left);
          prclItem->right = max(prclItem->right, treeIconRect.right);
          prclItem->top = min(prclItem->top, treeIconRect.top);
          prclItem->bottom = max(prclItem->bottom, treeIconRect.bottom);
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TREEICON) &&
                 (pQueryRecordRect->fsExtent & CMA_ICON))
        {
          /************************************************************/
          /* Union the little icon rectangle and the big icon         */
          /* rectangle.                                               */
          /************************************************************/
          prclItem->left = min(treeIconRect.left, iconRect.left);
          prclItem->right = max(treeIconRect.right, iconRect.right);
          prclItem->top = min(treeIconRect.top, iconRect.top);
          prclItem->bottom = max(treeIconRect.bottom, iconRect.bottom);
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TEXT) &&
                 (pQueryRecordRect->fsExtent & CMA_ICON))
        {
          /************************************************************/
          /* Union the big icon rectangle and the text rectangle.     */
          /************************************************************/
          prclItem->left = min(textRect.left, iconRect.left);
          prclItem->right = max(textRect.right, iconRect.right);
          prclItem->top = min(textRect.top, iconRect.top);
          prclItem->bottom = max(textRect.bottom, iconRect.bottom);
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TREEICON) &&
                 (!(pQueryRecordRect->fsExtent & CMA_TEXT)))
        {
          *prclItem = treeIconRect;
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TEXT) &&
                 (!(pQueryRecordRect->fsExtent & CMA_TREEICON)))
        {
          *prclItem = textRect;
        }
        else if (pQueryRecordRect->fsExtent & CMA_ICON)
        {
          *prclItem = iconRect;
        }
        else
        {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return(FALSE);
        }
      }
      /****************************************************************/
      /* Otherwise we are in the text view version of tree view.      */
      /****************************************************************/
      else if (pccinfo->flWindowAttr & CV_TEXT)
      {
        if ((pQueryRecordRect->fsExtent & CMA_TREEICON) &&
            (pQueryRecordRect->fsExtent & CMA_TEXT) &&
            (!(pQueryRecordRect->fsExtent & CMA_ICON)))
        {
          /************************************************************/
          /* Union the little icon rectangle and the text rectangle.  */
          /************************************************************/
          prclItem->left = min(textRect.left, treeIconRect.left);
          prclItem->right = max(textRect.right, treeIconRect.right);
          prclItem->top = min(textRect.top, treeIconRect.top);
          prclItem->bottom = max(textRect.bottom, treeIconRect.bottom);
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TREEICON) &&
                (!(pQueryRecordRect->fsExtent & CMA_ICON)))
        {
          *prclItem = treeIconRect;
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TEXT) &&
                (!(pQueryRecordRect->fsExtent & CMA_ICON)))
        {
          *prclItem = textRect;
        }
        else
        {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return(FALSE);
        }
      }
      /****************************************************************/
      /* Otherwise we are in the name view version of tree view.      */
      /****************************************************************/
      else if (pccinfo->flWindowAttr & CV_NAME)
      {
        if ((pQueryRecordRect->fsExtent & CMA_TEXT) &&
            (pQueryRecordRect->fsExtent & CMA_ICON) &&
            (!(pQueryRecordRect->fsExtent & CMA_TREEICON)))
        {
          /************************************************************/
          /* Union the big icon rectangle and the text rectangle.     */
          /************************************************************/
          prclItem->left = min(textRect.left, iconRect.left);
          prclItem->right = max(textRect.right, iconRect.right);
          prclItem->top = min(textRect.top, iconRect.top);
          prclItem->bottom = max(textRect.bottom, iconRect.bottom);
        }
        else if ((pQueryRecordRect->fsExtent & CMA_ICON) &&
                (!(pQueryRecordRect->fsExtent & CMA_TREEICON)))
        {
          *prclItem = iconRect;
        }
        else if ((pQueryRecordRect->fsExtent & CMA_TEXT) &&
                (!(pQueryRecordRect->fsExtent & CMA_TREEICON)))
        {
          *prclItem = textRect;
        }
        else
        {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return(FALSE);
        }
      }
    }
    /******************************************************************/
    /* Otherwise, it is not Details or Tree View but Icon, Name, or   */
    /* Text View.                                                     */
    /******************************************************************/
    else
    {
      if ((pccinfo->flWindowAttr & CV_ICON) ||
           (pccinfo->flWindowAttr & CV_NAME))
      {
        CnrCalcIconSize(hwnd, &iconSize);
      }
      else
      {
        iconSize.x = 0;                                                 /*DAB1*/
        iconSize.y = 0;                                                 /*DAB1*/
      }
      /*******************************************************************/
      /* If rectangle is to contain the icon and text rectangles,        */
      /* calculate them depending on the view and union the rectangles.  */
      /*******************************************************************/
      if ((pQueryRecordRect->fsExtent & CMA_ICON) &&
          (pQueryRecordRect->fsExtent & CMA_TEXT))
      {
         if ((pccinfo->flWindowAttr & CV_ICON) ||
             (pccinfo->flWindowAttr & CV_NAME))
         {
           CnrQueryIconTextRect(hwnd, preccn, &iconSize,
                                FALSE, &iconRect, &textRect);
           /****************************************************************/
           /* Union two rectangles                                         */
           /****************************************************************/
           prclItem->left = min(textRect.left, iconRect.left);
           prclItem->right = max(textRect.right, iconRect.right);
           prclItem->top = min(textRect.top, iconRect.top);
           prclItem->bottom = max(textRect.bottom, iconRect.bottom);

         }
         else
         {
           CnrQueryIconTextRect(hwnd, preccn, NULL, FALSE, NULL,
                                prclItem);
         }
      }
      else
      {
         /****************************************************************/
         /* If only the icon rectangle is required, calculate it by view */
         /* and return the rectangle. If Text View, a NULL rectangle is  */
         /* returned.                                                    */
         /****************************************************************/
         if (pQueryRecordRect->fsExtent & CMA_ICON)
         {
           if ((pccinfo->flWindowAttr & CV_ICON) ||
               (pccinfo->flWindowAttr & CV_NAME))
           {
             CnrQueryIconTextRect(hwnd, preccn,
                                  &iconSize, FALSE, prclItem, NULL);
           }
           else
           {
             /*********************************************************/
             /* Current view does not have an icon, set rectl to NULL */
             /*********************************************************/
             prclItem->left = 0;                                        /*DAB1*/
             prclItem->right = 0;                                       /*DAB1*/
             prclItem->bottom = 0;                                      /*DAB1*/
             prclItem->top = 0;                                         /*DAB1*/

           } /* endif */
         }
         else
         {
            /*************************************************************/
            /* If only the text rectangle is required, calculate and     */
            /* return.                                                   */
            /*************************************************************/
            if (pQueryRecordRect->fsExtent & CMA_TEXT)
            {
                 CnrQueryIconTextRect(hwnd, preccn,
                                      &iconSize, FALSE, NULL, prclItem);
            }
            else
            {
              SETERROR(PMERR_INVALID_PARAMETERS);
              return(FALSE);
            } /* endif */
         } /* endif */
      } /* endif */
    }
  }

  return(TRUE);
}


/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryViewportRect                           */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: This function calculates a rectangle      */
/*     containing the coordinates of the container's viewport         */
/*     rectangle.                                                     */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrQueryViewportRect(hwnd, mp1, mp2)
HANDLE    hwnd;
long      mp1;
long      mp2;

{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  LPRECT       prctlViewport;
  RECT         rctlHscroll;
  RECT         rctlTitle;
  RECT         ViewPortLeft;
  LPLONG       WindowOrgx;


  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  prctlViewport = (LPRECT)(LPVOID)mp1;

  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
     if ((pccinfon->ulFlagSet & FL_UNSPLIT) || (!(HIWORD(mp2))))
     {
        PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                            prctlViewport);

        WindowOrgx = (LPLONG)&pccinfon->ptlNonIconOrg.x;
     }
     else if (HIWORD(mp2))
     {
        PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                            &ViewPortLeft);

        PMGetClientRect (GetDlgItem (hwnd, CID_RIGHTDVWND),
                            prctlViewport);

        /*************************************************************/
        /* Only add the left window factor in for CMA_WINDOW case.   */
        /*************************************************************/
        if (LOWORD(mp2) == CMA_WINDOW)
        {
          prctlViewport->left  += ViewPortLeft.right +
                                   GetSystemMetrics ( SM_CXFRAME);

          prctlViewport->right += ViewPortLeft.right +
                                   GetSystemMetrics ( SM_CXFRAME);
        }

        WindowOrgx = &pccinfon->RightWndxOrg;
     }
     else
     {
        SETERROR(PMERR_INVALID_PARAMETERS);
        return FALSE;
     }

     if (LOWORD(mp2) == CMA_WINDOW)
     {
        /*************************************************************/
        /* The details view window is on top of the container window */
        /* and its viewport needs to be adjusted for either/both of  */
        /* the container title area or the container details column  */
        /* title area                                                */
        /*************************************************************/
        RECT rctlLeftTitle;                                             /*DAB2*/

        if (pccinfo->flWindowAttr & CA_DETAILSVIEWTITLES)               /*DAB2*/
        {                                                               /*DAB2*/
          PMGetClientRect( GetDlgItem( hwnd, CID_LEFTCOLTITLEWND),      /*DAB2*/
                           &rctlLeftTitle);                             /*DAB2*/
          prctlViewport->top     += rctlLeftTitle.bottom;               /*DAB2*/
          prctlViewport->bottom  += rctlLeftTitle.bottom;               /*DAB2*/
        }                                                               /*DAB2*/
        if (pccinfo->flWindowAttr & CA_CONTAINERTITLE)                  /*DAB2*/
        {                                                               /*DAB2*/
          PMGetClientRect( GetDlgItem( hwnd, CID_CNRTITLEWND),          /*DAB2*/
                           &rctlTitle);                                 /*DAB2*/
          prctlViewport->top     += rctlTitle.bottom;                   /*DAB2*/
          prctlViewport->bottom  += rctlTitle.bottom;                   /*DAB2*/
        }                                                               /*DAB2*/
        return (TRUE);
     }
     else if (LOWORD(mp2) == CMA_WORKSPACE)
     {
        /*************************************************************/
        /* We should be adding the window origin , not subtracting.  */
        /*************************************************************/
        prctlViewport->left   += *WindowOrgx;
        prctlViewport->right  += *WindowOrgx;
        prctlViewport->bottom += pccinfon->ptlNonIconOrg.y;
        prctlViewport->top    += pccinfon->ptlNonIconOrg.y;

        return (TRUE);
     }
     else
     {
        SETERROR(PMERR_INVALID_PARAMETERS);
        return FALSE;
     }
  }
  /*******************************************************************/
  /* Calculate the rectangles of the viewport and H scroll bar       */
  /*   prctlViewport->left = 0;                                      */
  /*   prctlViewport->bottom = 0;                                    */
  /*   prctlViewport->right = cx of viewport;                        */
  /*   prctlViewport->top = cy of viewport;                          */
  /*******************************************************************/
  CnrQuerySizeRect(hwnd, NULL, &rctlTitle, prctlViewport, NULL,
                   &rctlHscroll);

  if (LOWORD(mp2) == CMA_WINDOW) {
    /*****************************************************************/
    /* Calculate the rectangles of the viewport relative to the      */
    /* window origin by adding cyHscroll for both y coordinates.     */
    /*****************************************************************/
    prctlViewport->bottom += rctlTitle.bottom;
    prctlViewport->top += rctlTitle.bottom;
    return TRUE;
  } else {
    /*****************************************************************/
    /* Calculate the rectangles of the viewport relative to the      */
    /* workplace origin by adding ptlOrigin (the coordinates of      */
    /* the window relative to the workplace) and cyHscroll.          */
    /*****************************************************************/
    if (LOWORD(mp2) == CMA_WORKSPACE) {
      if (pccinfo->flWindowAttr & CV_ICON)
      {
        prctlViewport->bottom += (rctlTitle.bottom +
                                   pccinfo->ptlOrigin.y);
        prctlViewport->top += (rctlTitle.bottom +
                                pccinfo->ptlOrigin.y);
        prctlViewport->left += pccinfo->ptlOrigin.x;
        prctlViewport->right += pccinfo->ptlOrigin.x;
      }
      else
      {
        prctlViewport->bottom += (rctlTitle.bottom +
                                   pccinfon->ptlNonIconOrg.y);
        prctlViewport->top += (rctlTitle.bottom +
                                pccinfon->ptlNonIconOrg.y);
        prctlViewport->left += pccinfon->ptlNonIconOrg.x;
        prctlViewport->right += pccinfon->ptlNonIconOrg.x;
      }
      return TRUE;
    } else {
      SETERROR(PMERR_INVALID_PARAMETERS);
      return FALSE;
    } /* endif */
  } /* endif */

}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrQueryRowRectForDV (hwndCnr, pRecord, RowRect)

HANDLE         hwndCnr;
LPRECORDCOREN  pRecord;
LPRECT         RowRect;

{
 LPCNRINFON  pccinfon;
 LPCNRINFO   pccinfo;
 RECT        CnrRect;
 int         CyBetweenRows;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /******************************************************************/
 /* Set the distance between the rows.  If the application has     */
 /* specified it, use it...otherwise use our constant.             */
 /******************************************************************/
 if (pccinfo->cyLineSpacing >= 0)
 {
    CyBetweenRows = pccinfo->cyLineSpacing;
 }
 else
 {
    CyBetweenRows = MINCY_BETWEENITEMS;
 }

 if (CyBetweenRows & NUM_ODD)
 {
    CyBetweenRows++;
 }

 /*********************************************************************/
 /* Calculate a rectangle that represents the row for this record.    */
 /*********************************************************************/
 if (pRecord)
 {
    PMGetClientRect (hwndCnr, &CnrRect);

    RowRect->left  = CnrRect.left;
    RowRect->right = CnrRect.right - (GetSystemMetrics ( SM_CXVSCROLL));

    /******************************************************************/
    /* Add CyBetweenRows to the top of the row and subtract it from   */
    /* the bottom in order not to have any white space in Details     */
    /* View.  This will have to change later when we add the option   */
    /* for the user to specify the distance between the rows.         */
    /******************************************************************/
    RowRect->top = CnrQueryRowPosInDV (pRecord, PT_TOP) -
                    (CyBetweenRows / 2) - pccinfon->ptlNonIconOrg.y;

    RowRect->bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM) +
                       (CyBetweenRows / 2) - pccinfon->ptlNonIconOrg.y;
 }

 return;
}

/*********************************************************************/
/* FUNCTION NAME:  CnrScrollWindow                                   */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Scrolls Window               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function scrolls the entire         */
/*   containr window in the direction given as parameter1 by the     */
/*   amount given by the parameter2. If tne amount is less than      */
/*   zero the the window will be scrolled in the negative direction. */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         long mp1                                                  */
/*         long mp2                                                  */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL: TRUE - the window is scrolled   successfully.        */
/*                                                                   */
/* EXIT-ERROR: FALSE - an error occurred.                            */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR CnrScrollWindow( hwndCnr, mp1, mp2)
HANDLE      hwndCnr;
long        mp1;
long        mp2;

{
  BOOL        flReturn = FALSE;
  long        lScroll;
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  int         sScrollPos;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (mp1)
  {
    if ((DWORD)mp1 == CMA_VERTICAL)
    {
      /***************************************************************/
      /* Because CMA_VERTICAL is specified, scrolls the  window      */
      /* vertically and return TRUE.                                 */
      /***************************************************************/
      if (mp2 % (long)(pccinfon->sVscrollFactor))
      {
        lScroll = mp2 / (long)(pccinfon->sVscrollFactor) + 1;
      }
      else
      {
        lScroll = mp2 / (long)(pccinfon->sVscrollFactor);
      }

      if (lScroll >
          ((long)pccinfon->sVscrollMax - (long)pccinfon->sVscrollPos))
      {
        sScrollPos = pccinfon->sVscrollMax;
      }
      else
      {
        if (lScroll < -(long)pccinfon->sVscrollPos )
        {
          sScrollPos = 0;
        }
        else
        {
          sScrollPos = pccinfon->sVscrollPos + lScroll;                 /*DAB1*/
        }
      } /* endif */

      if ( sScrollPos < 0 )                                             /*PPB1*/
        sScrollPos = 0;                                                 /*PPB1*/

      SENDMESSAGE( hwndCnr, WM_VSCROLL,
                   MAKELONG( SB_THUMBPOSITION, sScrollPos ),            /*DAB1*/
                   (LPARAM)GetDlgItem( hwndCnr, CID_VSCROLL ));         /*DAB1*/
      flReturn = TRUE;

    }
    else
    {
      if ((DWORD)mp1 == CMA_HORIZONTAL )
      /*************************************************************/
      /* Because CMA_HORIZONTAL is specified, scrolls the  window  */
      /* horizontally and return TRUE.                             */
      /*************************************************************/
      {
         /*************************************************************/
         /* First Check to see if we received this message while in   */
         /* Details View.                                             */
         /*************************************************************/
         if (pccinfo->flWindowAttr & CV_DETAIL)
         {
            /**********************************************************/
            /* Then check to make sure the splitbar is not there, and */
            /* Details View is being displayed in 1 window only.  We  */
            /* check this because if Details View is split, then to   */
            /* horizontally scroll it, the app. is supposed to use    */
            /* CM_HORZSCROLLSPLITWINDOW.  If it is not split, this    */
            /* message will work, because everything is in the left   */
            /* window in an unsplit Details View.                     */
            /**********************************************************/
            if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
                (CnrVerifyFieldInfoPtrs (hwndCnr,
                                         pccinfon->pFieldInfoLast, 1)))
            {
               SETERROR(PMERR_INVALID_PARAMETERS);
               return (FALSE);                                          /*DAB1*/
            }
         }

         if ((DWORD)mp2 % (long)(pccinfon->sHscrollFactor))
         {
           lScroll = mp2 / (long)(pccinfon->sHscrollFactor) + 1;
         }
         else
         {
           lScroll = mp2 / (long)(pccinfon->sHscrollFactor);
         }

         if (lScroll >
             ((long)pccinfon->sHscrollMax - (long)pccinfon->sHscrollPos))
         {
           sScrollPos = pccinfon->sHscrollMax;
         }
         else
         {
           if (lScroll < -(long)pccinfon->sHscrollPos )
           {
             sScrollPos = 0;
           }
           else
           {
             sScrollPos = pccinfon->sHscrollPos + lScroll;              /*DAB1*/
           }
         } /* endif */

        if ( sScrollPos < 0 )                                           /*PPB1*/
          sScrollPos = 0;                                               /*PPB1*/

        SENDMESSAGE( hwndCnr, WM_HSCROLL,
                     MAKELONG( SB_THUMBPOSITION, sScrollPos ),          /*DAB1*/
                     (LPARAM)GetDlgItem( hwndCnr, CID_HSCROLL ));       /*DAB1*/

        flReturn = TRUE;
      }
      else
      {
        SETERROR(PMERR_INVALID_PARAMETERS);
      } /* endif */
    } /* endif */
  }
  else
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
  } /* endif */

  return flReturn;

}

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrProcessTimer                                */
/*                                                                    */
/*  Descriptive Name:   Process WM_TIMER                              */
/*                                                                    */
/*  Functional Description: This function processes WM_TIMER          */
/*     message.                                                       */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd    - container control handle                       */
/*           mp1 -                                                    */
/*           mp2 -                                                    */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
LONG FAR CnrProcessTimer( HWND hwnd, UINT msg, WPARAM wParam,           /*DAB1*/
                          LPARAM lParam )                               /*DAB1*/
{
  HDC         hps;
  LPCNRINFON  pccinfon;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  /*******************************************************************/
  /* If the ID of the timer is ID_TIMER, blink the cursored          */
  /* emphasis.                                                       */
  /*******************************************************************/
  if (wParam == ID_TIMER)
  {
    hps = PMGetFlippedDC(hwnd);

    if (pccinfon->ulFlagSet & FL_BLINK)
    {
      /***************************************************************/
      /* If the box for the cursored emphasis is drawn already       */
      /* (FL_BLINK is on), erase it by calling BlinkCursored...      */
      /* (in curcfunc.c) with the last parameter FALSE.              */
      /***************************************************************/
      BlinkCursoredEmph(hwnd, hps, pccinfon->preccCursor, FALSE);
      pccinfon->ulFlagSet &= ~FL_BLINK;
    }
    else
    {
      /***************************************************************/
      /* If the box for the cursored emphasis is not drawn yet,      */
      /* (FL_BLINK is off), draw it by calling BlinkCursored...      */
      /* (in curcfunc.c) with the last parameter TRUE.               */
      /***************************************************************/
      BlinkCursoredEmph(hwnd, hps, pccinfon->preccCursor, TRUE);
      pccinfon->ulFlagSet |= FL_BLINK;
    } /* endif */
    ReleaseDC (hwnd, hps);
    return (long)0;

  }
  else
  {
    return(long)(DefWindowProc(hwnd, msg, wParam, lParam));
  }
}

#ifdef DRAG_DONE
/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryRecordTargetEmphPos                    */
/*                                                                    */
/*  Descriptive Name:  Set target emphasis position for ordered emph. */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/**********************************************************************/
DWORD FAR CnrQueryRecordTargetEmphPos(HANDLE hwnd, LPRECORDCOREN pRecord,
                                       long yPos)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT     rctlRecord;
  POINT     iconSize;
  int       CyBetweenRows;
  long      cyRecordEdge;
  long      cyRecord;
  long      yUpper;
  long      yLower;

  /********************************************************************/
  /* Get pointer to Container internal data.                          */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* If a NULL record pointer is passed return.                       */
  /********************************************************************/
  if (!pRecord || (pccinfo->flWindowAttr & (CV_TREE | CV_ICON)))
  {
    return (0);
  }

  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
     /*****************************************************************/
     /* Set the distance between the rows.  If the application has    */
     /* specified it, use it...otherwise use our constant.            */
     /*****************************************************************/
     if (pccinfo->cyLineSpacing >= 0)
     {
        CyBetweenRows = pccinfo->cyLineSpacing;
     }
     else
     {
        CyBetweenRows = MINCY_BETWEENITEMS;
     }

     rctlRecord.bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM) +
                          (CyBetweenRows / 2);
     rctlRecord.top    = CnrQueryRowPosInDV (pRecord, PT_TOP) -
                          (CyBetweenRows / 2);
  }
  else
  /*******************************************************************/
  /* Query for the icon size                                         */
  /*******************************************************************/
  {
    CnrCalcIconSize(hwnd, &iconSize);

    rctlRecord.bottom = CnrQueryPtOfItemInNTV(hwnd, pRecord,
                                               &iconSize, PT_BOTTOM);
    rctlRecord.top    = CnrQueryPtOfItemInNTV(hwnd, pRecord,
                                               &iconSize, PT_TOP);
  }

  PMOffsetRect (&rctlRecord, 0, (int)-(pccinfon->ptlNonIconOrg.y));

  /*---------------------------------------------------------------------*/
  /* compute what we will consider to be the "edge" of a record:         */
  /* if we're using mixed target emphasis, it will be the top and        */
  /* bottom quarter of each record; otherwise, we're using ordered       */
  /* target emphasis, so the entire record will be considered the "edge" */
  /*---------------------------------------------------------------------*/
  cyRecordEdge = (pccinfo->flWindowAttr & CA_MIXEDTARGETEMPH) ? 4 : 2;

  /*-----------------------------------------------------------*/
  /* make the record position and mouse position zero-relative */
  /*-----------------------------------------------------------*/
  cyRecord = rctlRecord.bottom - rctlRecord.top;
  yPos    += rctlRecord.bottom;

  yLower   = (cyRecord * (cyRecordEdge-1)) / cyRecordEdge;
  yUpper   = cyRecord / cyRecordEdge;

  /*---------------------------------------------------*/
  /* if the mouse is in the bottom 1/cyRecordEdge of   */
  /* the record, draw target emphasis below the record */
  /*---------------------------------------------------*/
  if (yPos >= yLower)
  {
    return (DRAGTARGETEMPHBELOW);
  }

  /*------------------------------------------------------*/
  /* otherwise, if the mouse is in the top 1/cyRecordEdge */
  /* of the record, draw target emphasis above the record */
  /*------------------------------------------------------*/
  else if (yPos <= yUpper)
  {
    return (DRAGTARGETEMPHABOVE);
  }

  /*--------------------------------------------------*/
  /* otherwise, the mouse is in the middle somewhere, */
  /* so draw target emphasis on the record            */
  /*--------------------------------------------------*/
  else
  {
    return (1);
  }
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrDrawDragTargetEmph                          */
/*                                                                    */
/*  Descriptive Name:  Draw target emphasis for dragging.             */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrDrawDragTargetEmph(HANDLE hwnd, LPRECORDCOREN pRecord,
                        DWORD dragEmphFlag)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  HDC       hps;                         /* Drag Presentation space   */

  /********************************************************************/
  /* Get pointer to Container internal data.                          */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Get presentation space for drawing cursor emphasis if we are not */
  /* in Details View.  Details View does not use the hps created with */
  /* hwndCnr, but that of the left and right windows.                 */
  /********************************************************************/
//if (pccinfo->flWindowAttr & (CV_TREE | CV_ICON | CV_NAME | CV_TEXT))
//{
//   hps = DrgGetPS(hwnd);
//}

  /********************************************************************/
  /* If window target emphasis flag then draw it.                     */
  /********************************************************************/
  if (dragEmphFlag & DRAGTARGETEMPHWINDOW)
  {
    if (dragEmphFlag & DRAGTARGETEMPHON)
    {
      CnrDrawWinTargetEmph(hwnd, hps, TRUE);
      pccinfon->ulFlagSet |= FL_WINTARGETEMPH;
    }
    else
    {
      CnrDrawWinTargetEmph(hwnd, hps, FALSE);
      pccinfon->ulFlagSet &= ~FL_WINTARGETEMPH;
    }

  }
  /********************************************************************/
  /* Otherwise, draw record target emphasis                           */
  /********************************************************************/
  else
  {
    if (dragEmphFlag & DRAGTARGETEMPHON)
    {
      pccinfon->preccTarget = pRecord;

      /*------------------------------------------------------------*/
      /* if we're in text, name, or details view, and either we're  */
      /* using ordered target emphasis, or we're using mixed target */
      /* emphasis and the emphasis will be between the records,     */
      /* then draw the emphasis between the records                 */
      /*------------------------------------------------------------*/
      if (IsEmphasisBetweenRecords(pccinfon))
      {
        if (!(pccinfo->flWindowAttr & CV_TREE) &&
           (pccinfo->flWindowAttr & CV_DETAIL))
        {
          CnrDrawTargetEmphBInDV (hwnd,
                                  pRecord,
                                  pccinfon->dragRecordEmphPos);
        }
        else
        {
          CnrDrawTargetEmphB(hwnd, hps, pRecord,
                             pccinfon->dragRecordEmphPos);
        }
      }

      /*-----------------------------------------------------*/
      /* otherwise, add target emphasis to the target record */
      /*-----------------------------------------------------*/
      else
      {
        SetEmphasisBits (hwnd, NULL, pRecord, CRA_TARGET, SEB_SET);
        CnrDrawRecTargetEmph(hwnd, hps, pRecord);
      }
    }
    else
    /********************************************************************/
    /* Turn the target emphasis off the record which is no longer under */
    /* the mouse pointer.                                               */
    /********************************************************************/
    {
      /*------------------------------------------------------------*/
      /* if we're in text, name, or details view, and either we're  */
      /* using ordered target emphasis, or we're using mixed target */
      /* emphasis and the emphasis is currently between the records,*/
      /* then remove the emphasis from between the records          */
      /*------------------------------------------------------------*/
      if (IsEmphasisBetweenRecords(pccinfon))
      {
        if (!(pccinfo->flWindowAttr & CV_TREE) &&
           (pccinfo->flWindowAttr & CV_DETAIL))
        {
          CnrDrawTargetEmphBInDV (hwnd,
                                  pccinfon->preccTarget,
                                  pccinfon->dragRecordEmphPos);
        }
        else
        {
          CnrDrawTargetEmphB(hwnd, hps, pccinfon->preccTarget,
                             pccinfon->dragRecordEmphPos);
        }
      }

      /*----------------------------------------------------------*/
      /* otherwise, remove target emphasis from the target record */
      /*----------------------------------------------------------*/
      else
      {
        SetEmphasisBits (hwnd, NULL, pRecord, CRA_TARGET, SEB_CLEAR);
        CnrDrawRecTargetEmph(hwnd, hps, pRecord);
      }

      pccinfon->preccTarget = NULL;
    }
  }
  /********************************************************************/
  /* Release presentation space                                       */
  /********************************************************************/
//if (pccinfo->flWindowAttr & (CV_TREE | CV_ICON | CV_NAME | CV_TEXT))
//{
//   DrgReleasePS(hps);
//}

  return (TRUE);
}

/*---------------------------------------------------------------------*/
/* Function Name: IsEmphasisBetweenRecords                             */
/*                                                                     */
/* Description:   Determines whether target emphasis should be drawn   */
/*                on the target record or between records              */
/*                                                                     */
/* Input:   pccinfo  - public CNRINFO for the container                */
/*          pccinfon - private CNRINFO for the container               */
/*                                                                     */
/* Output:  TRUE  - target emphasis should be drawn between records    */
/*          FALSE - target emphasis should be drawn on the record      */
/*---------------------------------------------------------------------*/
BOOL FAR IsEmphasisBetweenRecords(LPCNRINFON pccinfon)
{
  LPCNRINFO  pccinfo;

  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (!(pccinfo->flWindowAttr & CV_TREE) &&
      (pccinfo->flWindowAttr & (CV_TEXT | CV_NAME | CV_DETAIL)))
  {
    if (pccinfo->flWindowAttr & CA_MIXEDTARGETEMPH)
    {
      return ((BOOL)(pccinfon->dragRecordEmphPos == DRAGTARGETEMPHABOVE ||
              pccinfon->dragRecordEmphPos == DRAGTARGETEMPHBELOW));
    }
    else
    {
      return ((BOOL)((pccinfo->flWindowAttr & CA_ORDEREDTARGETEMPH) ?
                      TRUE : FALSE));
    }
  }
  else
  {
    return (FALSE);
  }
}

#endif

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrDrawBoxOutline                              */
/*                                                                    */
/*  Descriptive Name:  Draws an outline box with given start and end  */
/*                     points.                                        */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd    - container control handle                       */
/*           pptlStart - starting point for box.                      */
/*           pptlEnd - end point for box.                             */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
VOID FAR CnrDrawBoxOutline(HANDLE hwnd, HDC hps, POINT FAR *pptlStart,
                       POINT FAR *pptlEnd)
{
   LPCNRINFON  pccinfon;                /* Internal CNR control data  */
   LPCNRINFO   pccinfo;
   HPEN       hpenOld;
   HBRUSH     hbrushOld;
   int        oldFgndMode = 0;         /* save old foreground mode */   /*DAB1*/

   pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
   pccinfo  = PCCINFOXFROMN(pccinfon);

   if ((pccinfo->flWindowAttr & CV_ICON) && (pccinfon->ulFlagSet & FL_CAPTURE)
       && (!(pccinfon->ulFlagSet & FL_DRAGSELECT)))
   {
     oldFgndMode = SetROP2 (hps, R2_NOT);
     hbrushOld = SelectObject (hps, GetStockObject (NULL_BRUSH));
     hpenOld   = SelectObject (hps, GetStockObject (BLACK_PEN));
     Rectangle (hps, pptlStart->x, pptlStart->y, pptlEnd->x, pptlEnd->y);
     SelectObject (hps, hbrushOld);
     SelectObject (hps, hpenOld);

     /*****************************************************************/
     /* Select the old pen color and free resource.                   */
     /*****************************************************************/
//   DeleteObject (SelectObject (hps, hpenOld));
   }
   return;
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrAllocRecord                                 */
/*                                                                    */
/*  Descriptive Name:  Allocate a Record or a list of records         */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This routine will Allocate one or more records and return a       */
/*  pointer to the first record in the list.  If more than one record */
/*  is requested, the records are linked together in a singly linked  */
/*  list.  The allocation is done through a common Storage Manager.   */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  ExtraBytes - Number of extra bytes the application wants  */
/*                       to allocate with each record.                */
/*                                                                    */
/*          NumRecords - The number of records to be allocated.       */
/*                                                                    */
/*  Output:  A pointer to the first record in the list of records     */
/*           allocated.                                               */
/*                                                                    */
/*  Exit-Normal:  Pointer to a RECORDCORE Structure                   */
/*                                                                    */
/*  Exit-Error:   NULL with WinSetErrorInfo message specified -       */
/*                PMERR_INSUFFICIENT_MEMORY                           */
/*                PMERR_INVALID_PARAMETERS                            */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/* Author: Peter Haggar                                               */
/**********************************************************************/
LPRECORDCORE FAR CnrAllocRecord (				/*RKK1*/
  HANDLE  hwnd,
  DWORD   ExtraBytes,                                                     /*DAB1*/
  WORD    NumRecords  )						/*RKK1*/
{
  LPCNRINFON  pccinfon;
  LPVOID      pHead;        /* Pointer to the first allocated       */
                            /* record in the list.                  */
  LPVOID      pLead;        /* Pointer used in creating linked list */
                            /* of records.                          */
  LPVOID      pTrail=NULL;  /* Pointer used in creating linked list */
                            /* of records.                          */
//LPVOID      pSatPtr;      /* Pointer to Storage Space.            */
  WORD        i;
  DWORD       recordSize;
  DWORD       ExtRecordSize;  /* Size of RECORDCORE                  */

  /********************************************************************/
  /* Get the pointer to the storage space.                            */
  /********************************************************************/
//pSatPtr = (LPCNRINFON)GetWindowLong (hwnd, QWL_STORAGEPTR);
  pccinfon = PCCINFONOF (hwnd);

  /********************************************************************/
  /* Make sure that we are trying to allocate something of a size we  */
  /* can handle, and that we are trying to allocate at least one of   */
  /* them.  If not, return an error.                                  */
  /********************************************************************/
  if ((ExtraBytes > MAX_EXTRA_BYTES) || (NumRecords == 0))
  {
    SETERROR (PMERR_INVALID_PARAMETERS);
    return ((LPRECORDCORE)NULL);
  }

  /********************************************************************/
  /* Setup the recordcore size to be allocated based on the style     */
  /********************************************************************/
  if (GetWindowLong (hwnd, GWL_STYLE) & CCS_MINIRECORDCORE)
  {
    ExtRecordSize = (DWORD)(sizeof(MINIRECORDCORE));
  }
  else
  {
    ExtRecordSize = (DWORD)(sizeof(RECORDCORE));
  }

  /********************************************************************/
  /* Call Storage Manager and allocate the size of a RecordCore       */
  /* structure, plus the number of extra bytes the application wants, */
  /* plus a RECORDCOREN structure.  This is done because for every    */
  /* record the application requests, we add on our internal record   */
  /* structure. Therefore, we have to allocate enough memory for our  */
  /* stuff and theirs. In addition, our internal record abstractly    */
  /* sits on top of their external record.  Therefore to access the   */
  /* external structure, we add 1 to the pointer when it is pointing  */
  /* to the internal structure, and subtract 1 in the reverse case.   */
  /* So, by adding 1 to pHead, we are accessing the External record,  */
  /* then we set its next pointer to NULL.                            */
  /*                                                                  */
  /********************************************************************/
  recordSize = (ExtraBytes + ExtRecordSize + 4);

  if (NumRecords > 1)
  {
    /******************************************************************/
    /* Allocate a block for each record requested, chaining them      */
    /******************************************************************/
    for (i = NumRecords; i > 0; i--)
    {
//    if (pLead = CnrAllocateStorage (pSatPtr, recordSize))
      pLead = (LPVOID)GlobalAlloc( GPTR, recordSize);                   /*DAB1*/
      if (pLead)
      {
        if (pTrail)
        {
          PRECCXFROMLPVOID(pTrail)->preccNextRecord =                   /*DAB1*/
                                              PRECCXFROMLPVOID(pLead);  /*DAB1*/
        }
        else
        {
          pHead = pLead;
        }

        PRECCXFROMLPVOID(pLead)->cb = ExtRecordSize;
        pTrail = pLead;
      }
      else
      {
        SETERROR(PMERR_INSUFFICIENT_MEMORY);
        return ((LPRECORDCORE)NULL);
      }
    }
  }
  else
  {
    /******************************************************************/
    /* Allocate the record, adding in the additional bytest the app   */
    /* requested, and the 4 bytes we need for our incognito pointer.  */
    /******************************************************************/
//  if (pHead = CnrAllocateStorage (pSatPtr, ExtraBytes + ExtRecordSize + 4))
    pHead = (LPVOID)GlobalAlloc( GPTR, recordSize );                    /*DAB1*/
    if (pHead)
    {
      /****************************************************************/
      /* Initialize the external record size to the size of the record*/
      /****************************************************************/
      PRECCXFROMLPVOID(pHead)->cb = ExtRecordSize;
    }
    else
    /******************************************************************/
    /* The Allocation failed so set the error info and return NULL.   */
    /******************************************************************/
    {
       SETERROR(PMERR_INSUFFICIENT_MEMORY);
       return ((LPRECORDCORE)NULL);
    }
  }

  /********************************************************************/
  /* If there wasn't an error in allocating the memory, return the    */
  /* pointer to the first record.  Return the pointer to the External */
  /* portion of the record.  This is done because the pointer is      */
  /* returned to the application and the application only knows about */
  /* the external record.                                             */
  /********************************************************************/
  return (PRECCXFROMLPVOID(pHead));
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrRefreshColors                               */
/*                                                                    */
/*  Descriptive Name:  Refresh container colors                       */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This routine will update the color which has been modified.       */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr    - Window handle of the container               */
/*          pccinfon   - Pointer to internal container Info           */
/*          wCnrAttr   - color attribute to change                    */
/*          crColor    - color reference                              */
/*                                                                    */
/*  Output:  TRUE/FALSE                                               */
/*                                                                    */
/*  Exit-Normal: TRUE                                                 */
/*                                                                    */
/*  Exit-Error: FALSE                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrRefreshColors (HWND hwndCnr, LPCNRINFON pccinfon, WORD wCnrAttr,
                           COLORREF crColor)
{
  /********************************************************************/
  /* Process color attribute                                          */
  /********************************************************************/
  switch (wCnrAttr)
  {
    case CMA_FOREGROUNDCOLOR:
    case CMA_BACKGROUNDCOLOR:
    case CMA_SELECTFOREGROUNDCOLOR:
    case CMA_SELECTBACKGROUNDCOLOR:
    case CMA_SEPARATORCOLOR:
    {
      /****************************************************************/
      /* Save the new color reference for the specified attribute     */
      /****************************************************************/
      pccinfon->ulColorPPSupport[wCnrAttr] = crColor;

      /****************************************************************/
      /* Update the container with the new color                      */
      /****************************************************************/
      InvalidateContainer (hwndCnr, TRUE, NULL);
    }
    break;

    default:
    {
      /****************************************************************/
      /* Invalid color attribute                                      */
      /****************************************************************/
      return (FALSE);
    }
  }

  return (TRUE);
} /* end CnrRefreshColors */

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrQueryColors                                 */
/*                                                                    */
/*  Descriptive Name:  Query container colors                         */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This routine will return the current color.                       */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr    - Window handle of the container               */
/*          pccinfon   - Pointer to internal container Info           */
/*          wCnrAttr   - color attribute to change                    */
/*                                                                    */
/*  Output:  LONG (COLORREF)                                          */
/*                                                                    */
/*  Exit-Normal: LONG                                                 */
/*                                                                    */
/*  Exit-Error: NULL                                                  */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
LONG FAR CnrQueryColors (HWND hwndCnr, LPCNRINFON pccinfon, WORD wCnrAttr)
{
  LONG lColorRef;
  /********************************************************************/
  /* Process color attribute                                          */
  /********************************************************************/
  switch (wCnrAttr)
  {
    case CMA_FOREGROUNDCOLOR:
    case CMA_BACKGROUNDCOLOR:
    case CMA_SELECTFOREGROUNDCOLOR:
    case CMA_SELECTBACKGROUNDCOLOR:
    case CMA_SEPARATORCOLOR:
    {
      /****************************************************************/
      /* Save the new color reference for the specified attribute     */
      /****************************************************************/
      lColorRef = pccinfon->ulColorPPSupport[wCnrAttr];
    }
    break;

    default:
    {
      /****************************************************************/
      /* Invalid color attribute                                      */
      /****************************************************************/
      lColorRef = 0;
    }
  }

  return (long)(lColorRef);
} /* end CnrQueryColors */

/*********************************************************************/
/* FUNCTION NAME:  ResetWindowForFontChange                          */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:  Reset Window For Font Change.                  */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function sets the flags to          */
/*   reset the window.(repositioning, recalculation of the text      */
/*   size, and reset and paintingof the title window.                */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwnd - handle to the container window              */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
VOID FAR ResetWindowForFontChange(HANDLE hwnd)
{
  LPCNRINFON   pccinfon;

  /*******************************************************************/
  /* Initialize pointers to control data and local variables         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong( hwnd, QWL_CONTAINER );

  /*******************************************************************/
  /* Get font metrics for this window.                               */
  /*******************************************************************/
  CnrQueryFontMetrics( hwnd, hwnd, FALSE, NULL, NULL );

  /*******************************************************************/
  /* Reset flags for the text size calculation, repositioning, scroll*/
  /*  set, view scroll bar, title, workplace, column titles and      */
  /*  positioning to force a repaint and reposition for new font     */
  /*******************************************************************/
  pccinfon->ulFlagSet &= ~(FL_TEXTCALC | FL_CNRPOSITION | FL_SETSCROLL |
                FL_NEWVIEWSCROLLSET | FL_SETCNRTITLE | FL_SETWORKPLACE |
                FL_SIZECOLTITLES | FL_POSITIONCOLTITLES);
  pccinfon->ulFlagSet2 &= ~FL2_TITLESIZECALC;

  /*******************************************************************/
  /* If CCS_AUTOPOSITION is set on, we need rearrange the icons      */
  /* in icon view.                                                   */
  /*******************************************************************/
  if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
  {
    pccinfon->ulFlagSet |= FL_NEEDARRANGE;
  } /* endif */

  return;
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrSetRecordEmphasis                           */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: Sets the emphasis attribute of the        */
/*                          specified record.                         */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input: mp1: pRecord (LPRECORDCORE) - ptr to specified record      */
/*                                                                    */
/*         mp2: pRecordEmphasis (PRECORDEMPHASIS) data structure      */
/*              which contains:                                       */
/*              fChangeEmphasis (BOOL) change emphasis attribute flag */
/*                TRUE - set attribute ON, if not current state       */
/*                FALSE - set attribute OFF, if not current state     */
/*              fEmphasisAttribute (DWORD) record's emphasis          */
/*                attribute.                                          */
/*                                                                    */
/*  Output:  fSuccess (BOOL)                                          */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal: TRUE                                                 */
/*                                                                    */
/*  Exit-Error:  FALSE                                                */
/*                                                                    */
/*  Side Effects: None                                                */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrSetRecordEmphasis(HANDLE hwnd, long mp1, long mp2)
{
  HDC            hps;
  DWORD          windowStyle;
  BOOL           fChangeEmphasis;
  BOOL           flVisible = FALSE;
  BOOL           rc = TRUE;
  WORD           fEmphasisAttribute;
  LPCNRINFON     pccinfon;                /* Container instance data  */
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  pRecord;
  LPRECORDCOREN  preccOldCursor;
  LPRECORDCORE   pRecX;

  /*******************************************************************/
  /* Get window style bits                                           */
  /*******************************************************************/
  windowStyle = GetWindowLong( hwnd, GWL_STYLE );

  /*******************************************************************/
  /* Get ptr to Container instance data                              */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Set local variables                                             */
  /*******************************************************************/
  fChangeEmphasis = LOWORD(mp2);
  fEmphasisAttribute = HIWORD(mp2);
//pRecord = GetRecordNFromX (hwnd, ((LPRECORDCORE)(LPVOID)mp1));

  if (!(fEmphasisAttribute &
       (CRA_SELECTED | CRA_CURSORED | CRA_INUSE | CRA_SOURCE)))         /*DAB3*/
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return(FALSE);
  }

  pRecX = (LPRECORDCORE)mp1;

  /*******************************************************************/
  /* If CCS_VERIFYPOINTERS is true and CnrVerifyPointers is false    */
  /* return an error condition                                       */
  /*******************************************************************/
  if (windowStyle & CCS_VERIFYPOINTERS)
  {
     if (!(CnrVerifyPointers(hwnd, (LPVOID)&pRecX, (long)1)))
     {
       SETERROR (PMERR_INVALID_PARAMETERS);
       return(FALSE);
     }
  }

  pRecord = GetRecordNFromX (hwnd, pRecX);

  if (pRecord)
  {
    DWORD  ulAttr = CRA_CURSORED;

    /*****************************************************************/
    /* ulAttr includes CV_SELECTED, in the single selection mode     */
    /*****************************************************************/
    if ((!(windowStyle & (CCS_EXTENDSEL | CCS_MULTIPLESEL))) ||
        (pccinfo->flWindowAttr & CV_TREE))
    {
      ulAttr |= CRA_SELECTED;
    }

    /*****************************************************************/
    /* Check if pRecord is visible.                                  */
    /*****************************************************************/
    if (ISRECORDVISIBLE(pRecord) &&
        (((pccinfo->flWindowAttr & CV_TREE) &&
        IsTreeVisible (pRecord)) ||
         (!(pccinfo->flWindowAttr & CV_TREE) &&
          !pRecord->preccParent)))
    {
      flVisible = TRUE;
    }

    /*****************************************************************/
    /* Get presentation space for drawing emphasis.                  */
    /*****************************************************************/
    hps = PMGetFlippedDC(hwnd);

    /*****************************************************************/
    /* If the change emphasis flag is TRUE, set the emphasis         */
    /* atttribute state to ON.                                       */
    /*****************************************************************/
    if (fChangeEmphasis)
    {
      /***************************************************************/
      /* If the attribute mask includes ulAttr and the record with   */
      /* ulAttr is not the new record, remove ulAttr from the        */
      /* current record with ulAttr and place on the new one.        */
      /* Cursor and selection movement should happen when the        */
      /* item that receives the emphaisis should be visible.         */
      /***************************************************************/
      if ((fEmphasisAttribute & ulAttr) && flVisible)
      {
        fEmphasisAttribute |= ulAttr;

        if (pccinfon->preccCursor != pRecord)
        {
          /***********************************************************/
          /*  Remove ulAttr from current                             */
          /*  cursor record, and assign new cursor record,           */
          /***********************************************************/
          preccOldCursor = pccinfon->preccCursor;
          pccinfon->preccCursor = pRecord;

          if (!SetEmphasisBits (hwnd, hps, preccOldCursor, ulAttr,
                                SEB_CLEAR))
          {
             ReleaseDC (hwnd, hps);
             return (FALSE);
          }
        }
      }
      else
      {
        if (fEmphasisAttribute & ulAttr)
        {
          fEmphasisAttribute &= ~ulAttr;
          rc = FALSE;
        } /* endif */
      }

      /***************************************************************/
      /* Set the new attributes for the record and notify the owner  */
      /***************************************************************/
      if (!SetEmphasisBits (hwnd, (flVisible) ? hps : NULL, pRecord,
                            (DWORD)fEmphasisAttribute, SEB_SET))
      {
        ReleaseDC (hwnd, hps);
        return (FALSE);
      }
    }
    else
    {
      /***************************************************************/
      /* We don't change ulAttr for pRecord if it has.               */
      /***************************************************************/
      if ((fEmphasisAttribute & ulAttr) &&
          (pRecord->flRecordAttr & ulAttr))
      {
        fEmphasisAttribute &= ~ulAttr;
        rc = FALSE;
      } /* endif */

      /***************************************************************/
      /* Clear the  attributes for the record and notify the owner   */
      /***************************************************************/
      if (!SetEmphasisBits (hwnd, (flVisible) ? hps : NULL, pRecord,
                            (DWORD)fEmphasisAttribute, SEB_CLEAR))
      {
        ReleaseDC (hwnd, hps);
        return (FALSE);
      }
    }
  }
  else
  {
    rc = FALSE;
  }

  if (!rc)
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
  }
  ReleaseDC (hwnd, hps);
  return(rc);
}

/*--------------------------------------------------------------------*/
/* Function Name:  SetEmphasisBits                                    */
/*                                                                    */
/* Description:    Sets or clears emphasis flags for a record and     */
/*                 notifies the owner of the change                   */
/*                                                                    */
/* Input:          hwnd        - container window                     */
/*                 preccChange - record to change                     */
/*                 flMask      - mask of bits to set or clear         */
/*                 fsControl   - Control flag:                        */
/*                    SEB_SET      set the flags in flMask            */
/*                    SEB_CLEAR    clear the flags in flMask          */
/*                    SEB_TOGGLE   toggles the flags in flMask        */
/*                    SEB_REPLACE  replace the flags with flMask      */
/*                                                                    */
/* Output:         None                                               */
/*                                                                    */
/*--------------------------------------------------------------------*/
BOOL FAR SetEmphasisBits (HANDLE hwnd, HDC hps, LPRECORDCOREN preccnChange,
                          DWORD flMask, WORD fsControl)
{
  DWORD                 flOldAttr;
  NOTIFYRECORDEMPHASIS  nre;

  flOldAttr = preccnChange->flRecordAttr;

  /*------------------------------------------------------------------*/
  /* modify the flRecordAttr bits according to the operator           */
  /*------------------------------------------------------------------*/
  switch (fsControl)
  {
    case SEB_SET:
      preccnChange->flRecordAttr |= flMask;
      break;

    case SEB_CLEAR:
      preccnChange->flRecordAttr &= ~flMask;
      break;

    case SEB_TOGGLE:
      preccnChange->flRecordAttr ^= flMask;
      break;

    case SEB_REPLACE:
      preccnChange->flRecordAttr = flMask;
      break;
  }

  /*------------------------------------------------------------------*/
  /* if the attribute hasn't changed, or the only thing that's        */
  /* changed is CRA_FILTERED, don't bother sending a notification     */
  /*------------------------------------------------------------------*/
  if (flOldAttr != preccnChange->flRecordAttr)
  {
    COPYNDATATOX(preccnChange);
    if (!(flMask & CRA_FILTERED))
    {
      nre.hwndCnr       = hwnd;
//    COPYNDATATOX(preccnChange);
      nre.pRecord       = preccnChange->preccX;
      nre.fEmphasisMask = (WORD)(flOldAttr ^ preccnChange->flRecordAttr);

      NotifyOwner (hwnd, CN_EMPHASIS, (LPVOID)&nre);

      /*------------------------------------------------------------*/
      /* if we were given a PS, draw the new emphasis on the record */
      /*------------------------------------------------------------*/
      if (hps)
      {
        if (nre.fEmphasisMask & (CRA_INUSE | CRA_SOURCE))               /*DAB3*/
        {
          InvalidateRecord (hwnd, &preccnChange,
                            1, (CMA_NOREPOSITION | CMA_ERASE));
        }
        else if (!CnrDrawEmphChange (hwnd, hps, preccnChange, flOldAttr))
        {
          return (FALSE);
        }
      }
    }
  }

  return (TRUE);
}
