/* Revision: 24 1.4.1.2 source/ui/cclnt/cnr/cnrs4p1.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS4P1.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS4P1.C - Tree view routines for container control     */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* DAB2  10/11/95  DAB  Fixed problem with ownerdraw cnr id in mp1    */
/* D6386  1/16/96  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/**********************************************************************/
#define  OEMRESOURCE
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

#ifdef SEGMENTOR
#define CNRS4P1_C
#include "cnrpragw.h"
#endif

extern HANDLE  vhCNRInstance;                                           /*DAB3*/

/**********************************************************************/
/*                                                                    */
/* FUNCTION NAME:  CnrTreeVCtlProc                                    */
/*                                                                    */
/* USAGES : External                                                  */
/*                                                                    */
/* DESCRIPTIVE NAME:  Container Control   Tree View Control Proc      */
/*                                                                    */
/* FUNCTIONAL DESCRIPTION:  This procedure will handle all messages   */
/*   for the tree view of the container.  Any messages that           */
/*   require special handling for the name/text view will be handled  */
/*   here                                                             */
/*                                                                    */
/* NOTES:                                                             */
/*    FORWARD DECLARATION:                                            */
/*                                                                    */
/* INPUT:  HANDLE hwndCnr - window handle for the container.          */
/*         WORD   msg - message.                                      */
/*         WORD   wParam - message parameter 1.                       */
/*         DWORD  lParam - message parameter 2.                       */
/*                                                                    */
/* OUTPUT:                                                            */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:                                                       */
/*                                                                    */
/* EXIT-ERROR:                                                        */
/*                                                                    */
/* SIDE EFFECTS:                                                      */
/*                                                                    */
/**********************************************************************/
LRESULT CALLBACK CnrTreeVCtlProc( HWND hwndCnr, UINT msg,               /*DAB1*/
                                  WPARAM wParam, LPARAM lParam)         /*DAB1*/
{
  HDC          hps;
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  RECT         rctlWin;
  RECT         rctlInvalid;
  PAINTSTRUCT  paintStruct;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Process the messages sent to this view                           */
  /********************************************************************/
  switch (msg)
  {
    case WM_CREATE:
      return FALSE;

    case WM_SIZE:
      pccinfon->ulFlagSet2 |= FL2_WINDOWSIZED;

      if (wParam == SIZEICONIC)
      {
        pccinfon->ulFlagSet2 |= FL2_MINIMIZED;
        pccinfon->ulFlagSet &= ~FL_CNRWINOPEN;
      }
      else
      {
        if (pccinfon->ulFlagSet2 & FL2_MINIMIZED)
        {
          pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;
          pccinfon->ulFlagSet &= ~( FL_CNRWINOPEN | FL_CNRPOSITION |
                              FL_SETSCROLL | FL_NEWVIEWSCROLLSET |
                              FL_SETCNRTITLE);
        } /* endif */
      } /* endif */

      /***************************************************************/
      /* Need resize child window.                                   */
      /***************************************************************/
      pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;

      /***************************************************************/
      /* If window is already open....                               */
      /***************************************************************/
      if ((pccinfon->ulFlagSet & FL_CNRWINOPEN))
      {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
          } /* endif */

          pccinfon->ulFlagSet &= ~FL_SETCNRTITLE;

      } else {
      /***************************************************************/
      /* If window is not open, this msg is not for the resizing     */
      /* of window. So, don't change origin and set flag on so       */
      /* that next time we get this msg, we update the origin        */
      /***************************************************************/
        pccinfon->ulFlagSet |= FL_CNRWINOPEN;
      } /* endif */

      PMInvalidateRect( hwndCnr, NULL, TRUE );

      return 0;

    case WM_VSCROLL:
    {                                                                   /*DAB1*/
      HWND hwndScroll = (HWND)(UINT)lParam;                             /*DAB1*/
      int  nThumbPos  = HIWORD(wParam);                                 /*DAB1*/
      WORD wNotifyCode= LOWORD(wParam);                                 /*DAB1*/

      CnrINTViewVScroll(hwndCnr, hwndScroll, nThumbPos, wNotifyCode);   /*DAB1*/
      return 0;
    }                                                                   /*DAB1*/

    case WM_HSCROLL:
    {                                                                   /*DAB1*/
      HWND hwndScroll = (HWND)(UINT)lParam;                             /*DAB1*/
      int  nThumbPos  = HIWORD(wParam);                                 /*DAB1*/
      WORD wNotifyCode= LOWORD(wParam);                                 /*DAB1*/

      CnrINTViewHScroll(hwndCnr, hwndScroll, nThumbPos, wNotifyCode);   /*DAB1*/
      return 0;
    }                                                                   /*DAB1*/

    case WM_PAINT:
      hps = BeginPaint (hwndCnr, &paintStruct);
      rctlInvalid = paintStruct.rcPaint;

      PMFlipDC(hwndCnr, hps);
      PMFlipRect((LPRECT)&rctlInvalid);

      pccinfon->ulFlagSet |= FL_CNRWINOPEN;

      if (!(IsIconic(hwndCnr)))
      {
        pccinfon->ulFlagSet |= FL_CNRWINOPEN;
        pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;


        if (!(pccinfon->ulFlagSet & FL_SETCNRTITLE))
        {

          PMGetClientRect(hwndCnr, &rctlWin);

          /*************************************************************/
          /* We need reset Cnr Title Window. (The cnr window is not    */
          /* initialized or the font of the Title Window is changed)   */
          /*************************************************************/
          CnrSizeCnrTitle (hwndCnr, hps, rctlWin);

          /*************************************************************/
          /* Because Cnr Title Window is reset, the size of the        */
          /* client area might change, so, we need reposition of       */
          /* the items.                                                */
          /*************************************************************/
          if (GetWindowLong( hwndCnr, GWL_STYLE ) & CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
          } /* endif */
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_TEXTCALC))
        {
          CalcTextSizeInIV(hwndCnr, hps);
          pccinfon->ulFlagSet |= FL_TEXTCALC;
        }

        /***************************************************************/
        /* If repositioning is necessary, reposition items.            */
        /***************************************************************/
        if (!(pccinfon->ulFlagSet & FL_CNRPOSITION))
        {
          CnrPositionTreeView(hwndCnr, hps, NULL);

          /*************************************************************/
          /* Make sure FL_CNRWINOPEN is on(for the case of the         */
          /* initial painting without sending WM_SIZE                  */
          /*************************************************************/
          pccinfon->ulFlagSet &= ~(FL_SIZECHILDREN | FL_SETSCROLL );
          pccinfon->ulFlagSet |= ( FL_CNRPOSITION | FL_TEXTCALC );
        } /* endif */

        if (TRVCheckNeedResize(hwndCnr))
        {
          pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
        }

        if (!(pccinfon->ulFlagSet & FL_SIZECHILDREN))
        {
          /*************************************************************/
          /* resize the scroll bars                                    */
          /*************************************************************/
          PMGetClientRect(hwndCnr, &rctlWin);

          CnrSize(hwndCnr, rctlWin.right, rctlWin.bottom);
          /*************************************************************/
          /* Set the scroll bar                                        */
          /*************************************************************/
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;
          pccinfon->ulFlagSet |= FL_SIZECHILDREN;
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_SETSCROLL))
        {
          /*************************************************************/
          /* Set the scroll bar                                        */
          /*************************************************************/
          CnrTRVSetScroll(hwndCnr);
          pccinfon->ulFlagSet |= FL_SETSCROLL;
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_SETFIRSTINVIEW))
        {
          CnrTRVSetFirstItem(hwndCnr, pccinfon->preccItem, TRUE);
          pccinfon->ulFlagSet |= FL_SETFIRSTINVIEW;
        }

        CnrEraseRect(hwndCnr, hps, paintStruct.rcPaint);
        if (!(CnrPaintTreeView( hwndCnr, hps, paintStruct.rcPaint)))
        {
           EndPaint (hwndCnr, &paintStruct);
           return ((long)NULL);
        }
        pccinfon->ulFlagSet2 |= FL2_INITPAINT;
        EndPaint (hwndCnr, &paintStruct);

        if (pccinfon->ulFlagSet2 & FL2_WINDOWSIZED)
        {
           /************************************************************/
           /* Windows will call CnrEnumWindowsProc for each child      */
           /************************************************************/
           EnumChildWindows(hwndCnr, CnrEnumWindowsProc, 0 );           /*DAB1*/

           pccinfon->ulFlagSet2 &= ~FL2_WINDOWSIZED;
        }
        else
        {
          PMInvalidateRect (GetDlgItem (hwndCnr, CID_VSCROLL),
                             NULL, FALSE);

          PMInvalidateRect (GetDlgItem (hwndCnr, CID_HSCROLL),
                             NULL, FALSE);
        }
      }
      else
      {
        EndPaint (hwndCnr, &paintStruct);
      }
      return 0;

    /*****************************************************************/
    /* All other messages are sent on to the superclass              */
    /*****************************************************************/
    default:
      return ((long)CnrGateProc( hwndCnr, msg, wParam, lParam));
  }
}
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrPositionTreeView                               */
/*                                                                   */
/* USAGES : External                                                 */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Position Tree View           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function positions tree view        */
/*   records. The precord parameter is used as a starting point to   */
/*   begin positioning.  If the starting record is NULL, then        */
/*   positioning will begin with the first record in the container.  */
/*   Otherwise, the next record after the specified record is        */
/*   positioned based on the location of the specified record.       */
/*   Records are then positioned, textSize fields are filled until   */
/*   the current position goes to the next group of the sibilings    */
/*   and fill the longestLine fields of the sibilings.               */
/*   Positioning the sibilings continues until no records available. */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle of the container window           */
/*         LPRECORDCORE precord - pointer to external record core     */
/*             data structure                                        */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrPositionTreeView (hwndCnr, hps, precord)
HANDLE       hwndCnr;
HDC          hps;
LPRECORDCOREN precord;
{
  WORD          cxIndent;
  WORD          usNodeLevel;
  WORD          cyBetween;        /* space between rows */
  WORD          usLeftMargin;
  int           yPrevTop;         /* Top of an item */
  int           cyHscroll;        /* height of the H scroll bar */
  int           cyPrevText;       /* height of the previous text      */
  int           cyPrevItem;       /* height of the previous item      */
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccParent;
  RECT          rectlWindow;      /* rectangle of the window */
  RECT          rctlLTitle;       /* rectangle of the list title */
  RECT          rctlIcon;
  RECT          rctlTreeIcon;
  RECT          rctlText;
  POINT         slIcon;
  POINT         slTreeIcon;

  /******************************************************************/
  /* @left margin = lAveCharWidth                                   */
  /* @top margin = lAveCharWidth                                    */
  /* @cy margin between two items = max(9pixels, cyLineSpacing)     */
  /* @cx margin between icon & text = lAveCharWidth                 */
  /* @cx margin between icon & icon = lAveCharWidth                 */
  /******************************************************************/
  PMGetClientRect (hwndCnr, &rectlWindow);
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  cyHscroll = GetSystemMetrics (SM_CYHSCROLL);

  CnrCalcIconSize (hwndCnr, &slIcon);
  CnrTRVCalcTreeIconSize (hwndCnr, &slTreeIcon);

  /*******************************************************************/
  /* Set cyBetween                                                   */
  /*******************************************************************/
  cyBetween = (pccinfo->cyLineSpacing >= 0) ?
          (WORD)pccinfo->cyLineSpacing : (WORD)MINCY_BETWEENITEMS;
  cyBetween += 2 * ICN_EMPHSPACE;

  usLeftMargin = max((WORD)pccinfon->lAveCharWidth,
                     (WORD)MINCX_TREELEFTMARGIN);

  /*******************************************************************/
  /* Set cxIndent                                                    */
  /*******************************************************************/
  cxIndent = (pccinfo->cxTreeIndent >= 0) ?
            (WORD)pccinfo->cxTreeIndent : (WORD)slIcon.x;

  /*******************************************************************/
  /* Get bottom and x position of the previous item of the first item*/
  /* to be positioned. If no previous is available, set the bottom   */
  /* higher than top of the first item by cyBetween.                 */
  /*******************************************************************/
  if (precord)
  {
    /*****************************************************************/
    /* Find bottommost point of the current record                   */
    /*****************************************************************/
    CnrTRVQueryIconTextRect(hwndCnr, precord, TRUE, slIcon, slTreeIcon,
                            &rctlIcon, &rctlTreeIcon, &rctlText);
    yPrevTop = CnrTRVQueryPtOfItem(hwndCnr, rctlIcon, rctlTreeIcon,
                                   rctlText, PT_TOP);
    precord = TRVGetNextVisualItem (precord);
  }
  else
  {
    /*******************************************************************/
    /* Get List title box size                                         */
    /*******************************************************************/
    CnrQueryTitleRect ( hwndCnr, &rctlLTitle, NULL);

    pccinfon->xTreeRight = 0;

    /*******************************************************************/
    /* Imaginary top of the window which the first item in a column    */
    /* is positioned relative to.  The top of the first item in a      */
    /* column will be (top - cyBetween) which is                       */
    /* lAveCharWidth below  than top of the client area                */
    /*******************************************************************/
    yPrevTop = rectlWindow.top + rctlLTitle.bottom +
               (cyBetween / 2) + pccinfon->lAveCharWidth;
    cyPrevText = cyPrevItem = 0;

    precord = pccinfon->preccItem;
  }

  if (precord)
  {
    preccParent = precord;
    usNodeLevel = TRVNodeLevel(precord);
    /****************************************************************/
    /* Position precord, its down_siblings and their children.      */
    /* Then, if there are, position  parent's down_siblings and     */
    /* their children, grandparent's down_siblings and their        */
    /* children .... and root level grandparent's down_siblings     */
    /* and their children.                                          */
    /****************************************************************/
    while (preccParent)
    {
      if (precord)
      {
        yPrevTop = TRVPositionRecords(hwndCnr, hps, precord,
                           yPrevTop, usNodeLevel, cxIndent,
                           usLeftMargin,
                           cyBetween, slIcon, slTreeIcon,
                           &cyPrevText, &cyPrevItem);
      }
      preccParent = preccParent->preccParent;
      if (preccParent)
      {
        precord = preccParent->preccNextItem;
      }
      else
      {
        precord = NULL;
      }
      usNodeLevel--;
    }
    pccinfon->yTreeBottom = yPrevTop +
                            pccinfon->lAveCharWidth +
                            ((cyPrevText > cyPrevItem)  ?
                              cyPrevText  :
                              cyPrevItem);
  }
  /*******************************************************************/
  /* It is reposioned relative to the (0,0), so, we need move        */
  /* the window by the amount of the distance by which the           */
  /* is already scrolled                                             */
  /*******************************************************************/
  pccinfon->ptlNonIconOrg.y =  (int)((long)pccinfon->sVscrollPos *
                               (long)pccinfon->sVscrollFactor);
  pccinfon->ptlNonIconOrg.x =  (int)((long)pccinfon->sHscrollPos *
                               (long)pccinfon->sHscrollFactor);

  /*******************************************************************/
  /* Text is calculated, and list is positioned. So, shange the flag */
  /* accordingly.                                                    */
  /*******************************************************************/
  pccinfon->ulFlagSet |= FL_CNRPOSITION;

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVPositionRecords                                */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Position the Tree View Records                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function positions the records      */
/*     using recursion                                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
int FAR TRVPositionRecords(                                    /*RKK1*/
  HANDLE         hwnd,
  HDC            hps,
  LPRECORDCOREN  preccFrom,
  long           yPrevTop,
  WORD           usLevel,
  WORD           usIndent,
  WORD           usLeftMargin,
  WORD           cyBetween,
  POINT          slIcon,
  POINT          slTreeIcon,
  LPINT          lpcyPrevText, /* Text height of the previous record */
  LPINT          lpcyPrevItem  /* Height of the previous item        */
                        )                                      /*RKK1*/
{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  preccEnum = preccFrom;
  LPRECORDCOREN  preccTemp;
  WORD           cyItem;
  int            right;
  int            cxLongest = 0;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  while (preccEnum)
  {
    /****************************************************************/
    /* If text is not calculated yet, do it.                        */
    /****************************************************************/
    if (!(pccinfon->ulFlagSet & FL_TEXTCALC))
    {
      IcnCalcTextSize( hps, slIcon, pccinfon,
                       GETLPSTRFORTRV(preccEnum->preccX), IS_SIDETEXT,
                       &(preccEnum->slTextSize));
    }

    /****************************************************************/
    /* Remove when supporting multiple selection.                   */
    /****************************************************************/
    preccEnum->usLongestLine = (WORD)preccEnum->slTextSize.cx;          /*DAB1*/

    if (ISRECORDVISIBLE(preccEnum) ||
        (!(preccEnum->preccPrevItem || preccEnum->preccParent)))
    {
      /**************************************************************/
      /* Get the height of the item.                                */
      /**************************************************************/
      cyItem = TRVGetCy(hwnd, preccEnum, slIcon, slTreeIcon);

      if (pccinfo->flWindowAttr & CV_TEXT)      /* tree text view */
      {
        if (*lpcyPrevText)
        {
          yPrevTop += cyBetween + (*lpcyPrevItem + *lpcyPrevText) / 2;
        }
      }
      else             /* not tree text view */
      {
        if (*lpcyPrevItem)
        {
          yPrevTop += cyBetween + (*lpcyPrevItem + slIcon.y) / 2;
        }
      }
           /**********************************************************/
           /* Calculate CurrentPos.y from the top of the item        */
           /**********************************************************/
      if (pccinfo->flWindowAttr & CV_TEXT)      /* tree name view */
           {
             preccEnum->ptlNonIconPos.y = yPrevTop;                     /*DAB1*/
           }
           else
           {
             if (slIcon.y < preccEnum->slTextSize.cy)                   /*DAB1*/
             {
              preccEnum->ptlNonIconPos.y = (yPrevTop +                  /*DAB1*/
                             (preccEnum->slTextSize.cy - slIcon.y) / 2);/*DAB1*/

             }
             else
             {
               preccEnum->ptlNonIconPos.y = yPrevTop;                   /*DAB1*/
             }
           }

      if (pccinfo->flWindowAttr & CV_NAME)      /* tree name view */
      {
        preccEnum->ptlNonIconPos.x = (usLevel - 1) * usIndent +
                                      usLeftMargin;
      }
      else    /* tree icon view */
      {
        preccEnum->ptlNonIconPos.x = (usLevel - 1) *
                                     usIndent + 2 *
                                     usLeftMargin + slTreeIcon.x;
      }
    }

    if (ISRECORDVISIBLE(preccEnum)) /* if the record is visible */
    {
      yPrevTop = preccEnum->ptlNonIconPos.y;

      /****************************************************************/
      /* Store heights of the item and text for usage in calculating  */
      /* the position of the next item ...                            */
      /****************************************************************/
      *lpcyPrevItem = cyItem;
      *lpcyPrevText = preccEnum->slTextSize.cy;                         /*DAB1*/

      if ((preccEnum->flRecordAttr & CRA_EXPANDED) &&
           preccEnum->preccFirstChild)
      {
        yPrevTop = TRVPositionRecords(hwnd, hps,
                                      preccEnum->preccFirstChild,
                                      yPrevTop,
                                      (WORD)(usLevel + (WORD)1),
                                      (WORD)usIndent,
                                      (WORD)usLeftMargin,
                                      (WORD)cyBetween,
                                      slIcon,
                                      slTreeIcon,
                                      lpcyPrevText,
                                      lpcyPrevItem);
      }

      cxLongest = max(cxLongest, preccEnum->slTextSize.cx);             /*DAB1*/
    }
    else    /* if not visible */
    {
      /**************************************************************/
      /* If an item before preccEnum, use it's pos for preccEnum.   */
      /**************************************************************/
      if (preccEnum->preccPrevItem)
      {
        /**************************************************************/
        /* Get the view information for the record to position        */
        /**************************************************************/
        preccTemp = preccEnum->preccPrevItem;

        preccEnum->ptlNonIconPos.x = preccTemp->ptlNonIconPos.x;
        preccEnum->ptlNonIconPos.y = preccTemp->ptlNonIconPos.y;
      }
      /****************************************************************/
      /* If preccEnum has no previous item and has the parent, use    */
      /*  the parent's position for preccEnum                         */
      /****************************************************************/
      else if (preccEnum->preccParent)
      {
        preccEnum->ptlNonIconPos.x =
                               preccEnum->preccParent->ptlNonIconPos.x;
        preccEnum->ptlNonIconPos.y =
                               preccEnum->preccParent->ptlNonIconPos.y;
      }
    }
    preccEnum = preccEnum->preccNextItem;
  } /*endwhile */

  /**************************************************************/
  /* Update the rightmost point of the tree.                    */
  /**************************************************************/
  if (pccinfo->flWindowAttr & CV_TEXT)      /* tree text view */
  {
    right = preccFrom->ptlNonIconPos.x + cxLongest;
  }
  else
  {
    right = preccFrom->ptlNonIconPos.x + slIcon.x +
             usLeftMargin + cxLongest;
  }

  right += pccinfon->lAveCharWidth;
  pccinfon->xTreeRight = max(pccinfon->xTreeRight, right);
  return (yPrevTop);                                                    /*DAB1*/
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVCheckNeedResize                                */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR TRVCheckNeedResize(hwnd)
HANDLE    hwnd;
{
#define NEED_H          0x0001
#define NEED_HwithV     0x0002
#define NEED_V          0x0004
#define NEED_VwithH     0x0008


  LPCNRINFON     pccinfon;
  RECT          rctlCnr;
  RECT          rctlVscroll;
  RECT          rctlHscroll;
  RECT          rctlLTitle;
  int           sNeed = 0;
  int           cyHscroll;
  int           cxVscroll;
  int           cyResized;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  PMGetClientRect(hwnd, &rctlCnr);
  CnrQuerySizeRect(hwnd, NULL, &rctlLTitle, NULL,
                   &rctlVscroll, &rctlHscroll);

  cyHscroll = GetSystemMetrics( SM_CYHSCROLL);
  cxVscroll = GetSystemMetrics( SM_CXVSCROLL);
  cyResized = pccinfon->ptlNonIconOrg.y + pccinfon->sVscrollPos *
              pccinfon->sVscrollFactor;

  if (pccinfon->yTreeBottom >
      (rctlCnr.bottom - rctlLTitle.bottom - cyHscroll - cyResized))
  {
    sNeed |= NEED_VwithH;
  }

  if (pccinfon->yTreeBottom >
      (rctlCnr.bottom - rctlLTitle.bottom  - cyResized))
  {
    sNeed |= NEED_V;
  }

  if (pccinfon->xTreeRight > (rctlCnr.right - cxVscroll))
  {
    sNeed |= NEED_HwithV;
  } /* endif */

  if (pccinfon->xTreeRight > rctlCnr.right)
  {
    sNeed |= NEED_H;
  } /* endif */

  if ((sNeed & NEED_H) && (sNeed & NEED_VwithH) && !(sNeed & NEED_V))
  {
    sNeed |= NEED_V;
  }

  if ((sNeed & NEED_V) && (sNeed & NEED_HwithV) && !(sNeed & NEED_H))
  {
    sNeed |= NEED_H;
  }

  if ((rctlHscroll.right == 0) && (sNeed & NEED_H) ||
      (rctlVscroll.top == 0) && (sNeed & NEED_V) ||
      (rctlVscroll.top > 0) && !(sNeed & NEED_V) ||
      (rctlHscroll.right > 0) && !(sNeed & NEED_H))
  {
    return TRUE;
  }
  else
  {
    return FALSE;
  } /* endif */
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVSetScroll                                   */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set Scroll Bar Range and Pos                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the tree view                         */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrTRVSetScroll(HANDLE hwnd)
{
  HANDLE        hwndHscroll;         /* handle of H scroll window */
  HANDLE        hwndVscroll;         /* handle of V scroll window */
  int           bottom;             /* bottom of the certain range */
  int           top;                /* top of the certain range */
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN preccFirstinView;    /* first record in the current view */
  RECT          rctlClient;          /* rectangle of the client area */
  RECT          rctlHScroll;         /* rectangle of the H scroll bar */
  RECT          rctlIcon;
  RECT          rctlTreeIcon;
  RECT          rctlText;
  RECT          rctlTitle;
  POINT         slIcon;              /* size of the icon */
  POINT         slTreeIcon;
  int           sRBPos;              /* range of the scroll down or right */
  int           sLTPos;              /* range of the scroll up or left */


  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-top point of a rectangle is (0, 0)                         */
  /*******************************************************************/
  CnrQuerySizeRect(hwnd, NULL, &rctlTitle, &rctlClient, NULL,
                   &rctlHScroll) ;

  if (!(pccinfo->flWindowAttr & CV_TEXT))      /* not tree text view */
  {
    CnrCalcIconSize (hwnd, &slIcon);
  }

  if (!(pccinfo->flWindowAttr & CV_NAME))      /* not tree name view */
  {
    CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);
  }

  /*******************************************************************/
  /* Get handle of the scroll bar                                    */
  /*******************************************************************/
  hwndHscroll = GetDlgItem (hwnd, CID_HSCROLL);
  hwndVscroll = GetDlgItem (hwnd, CID_VSCROLL);

  /*******************************************************************/
  /* Set the first item in the viewport in the current view          */
  /*******************************************************************/
  preccFirstinView = pccinfon->preccFirstinTRV;

  if (pccinfon->ulFlagSet & FL_NEWVIEWSCROLLSET )
  {
    /*****************************************************************/
    /* If this is not the first setting of the scroll bar in the     */
    /* current view, just consider the scroll pos to get top         */
    /* scroll range in pixels.                                       */
    /*****************************************************************/
    sLTPos = pccinfon->sVscrollPos * pccinfon->sVscrollFactor;
  }
  else
  {
    sLTPos = 0;
    /*****************************************************************/
    /* If the first item in the view is not specified, assign        */
    /* the first item in the list as the first item in the view.     */
    /* And no more scroll up (sLTPos = 0 )                           */
    /*****************************************************************/
    if (!preccFirstinView)
    {
      preccFirstinView = pccinfon->preccItem;
    }
    else
    {
      /***************************************************************/
      /* Get the distance from top of list and top of the first      */
      /* item in viewport. This is the current pos of slider,        */
      /* and shows the possible distance to scroll up(sLTPos)        */
      /***************************************************************/
      if (pccinfon->preccItem &&
         (pccinfon->preccItem != preccFirstinView))
      {
        CnrTRVQueryIconTextRect(hwnd, pccinfon->preccItem, TRUE,
                                slIcon, slTreeIcon,
                                &rctlIcon, &rctlTreeIcon, &rctlText);

        top = CnrTRVQueryPtOfItem(hwnd, rctlIcon, rctlTreeIcon,
                                   rctlText, PT_TOP);

        CnrTRVQueryIconTextRect(hwnd, preccFirstinView, TRUE,
                                slIcon, slTreeIcon,
                                &rctlIcon, &rctlTreeIcon, &rctlText);

        bottom = CnrTRVQueryPtOfItem(hwnd, rctlIcon, rctlTreeIcon,
                                      rctlText, PT_TOP);

        sLTPos = bottom - top;
      }
    }
    /*****************************************************************/
    /* Move the window by the amount the slider moved down           */
    /* (sLTPos)                                                      */
    /*****************************************************************/
    pccinfon->ptlNonIconOrg.y = sLTPos;

    pccinfon->ulFlagSet |= FL_NEWVIEWSCROLLSET;
  } /* endif */


  if (pccinfon->preccItem)
  {
    sRBPos = pccinfon->yTreeBottom - pccinfon->ptlNonIconOrg.y -
             rctlClient.bottom - rctlTitle.bottom;
  }
  else
  {
    sRBPos = 0;
  } /* endif */

  SetVscrollInNFView(hwnd, hwndVscroll, sRBPos, sLTPos, rctlClient);

  /*******************************************************************/
  /* Set sHscrollFactor as 1 so that the minimum scrolling unit      */
  /* can be a pixels.                                                */
  /*******************************************************************/
  pccinfon->sHscrollFactor = 1;

  if (pccinfon->xTreeRight > rctlClient.right)
  {
    /*****************************************************************/
    /* Calculate sLBPos and sRTPos.                                  */
    /*****************************************************************/
    sLTPos = pccinfon->ptlNonIconOrg.x;
    sRBPos = pccinfon->xTreeRight - pccinfon->ptlNonIconOrg.x -
             rctlClient.right;

    SetHscrollInNFView(hwndHscroll, pccinfon, sLTPos, sRBPos,
                       rctlClient);
  }
  else
  {
    pccinfon->sHscrollPos = 0;
    pccinfon->sHscrollMax = 0;
    pccinfon->ptlNonIconOrg.x = 0;

  } /* endif */

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrPaintTreeView                                  */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:  Paint Tree View                                */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR  CnrPaintTreeView( hwndCnr, hps, rctlWindow)
HANDLE       hwndCnr;
HDC          hps;
RECT         rctlWindow;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  preccDraw;         /* the record to be drawn */
  LPRECORDCOREN  preccParent;
  POINT         slIcon;            /* size of the icon */
  POINT         slTreeIcon;
  int           fsContinue = 1L;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (!(pccinfo->flWindowAttr & CV_TEXT))      /* not tree text view */
  {
    CnrCalcIconSize (hwndCnr, &slIcon);
  }

  if (!(pccinfo->flWindowAttr & CV_NAME))      /* not tree name view */
  {
    CnrTRVCalcTreeIconSize( hwndCnr, &slTreeIcon);
  }
  /*******************************************************************/
  /* Find the first item to be drawn and set icon dimensions.        */
  /*******************************************************************/
  preccDraw = pccinfon->preccFirstinTRV;

  if (!preccDraw)
  {
    preccDraw = pccinfon->preccItem;
  } /* endif */

  if (preccDraw)
  {
    preccParent = preccDraw;
    /****************************************************************/
    /* Paint    precord, its down_siblings and their children.      */
    /* Then, if there are, position  parent's down_siblings and     */
    /* their children, grandparent's down_siblings and their        */
    /* children .... and root level grandparent's down_siblings     */
    /* and their children.                                          */
    /****************************************************************/
    while (preccParent)
    {
      if (preccDraw)
      {
        fsContinue = TRVDrawRecords(hwndCnr, hps, preccDraw,
                                    rctlWindow, slIcon, slTreeIcon);
      }

      /**************************************************************/
      /* If fsContinue is -1, don't need to draw the rest of the    */
      /* records because all of them are below the viewport.        */
      /**************************************************************/
      if (fsContinue == -1L)
      {
        return TRUE;
        break;
      }

      /**************************************************************/
      /* If fsContinue is 0, Error occured. Stop process.           */
      /**************************************************************/
      if (fsContinue == 0)
      {
        return FALSE;
        break;
      }

      preccParent = preccParent->preccParent;
      if (preccParent)
      {
        preccDraw = preccParent->preccNextItem;
      }
      else
      {
        preccDraw = NULL;
      }

    } /* endwhile */
  }
  return TRUE;
}
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVDrawRecords                                    */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:  Paint Tree View                                */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
int FAR TRVDrawRecords(hwnd, hps, preccEnum, rctlClient,
                          slIcon, slTreeIcon)
HANDLE        hwnd;
HDC           hps;
LPRECORDCOREN  preccEnum;
RECT          rctlClient;
POINT         slIcon;
POINT         slTreeIcon;
{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  RECT        rctlIconCur;
  RECT        rctlTreeIcon;
  RECT        rctlText;
  POINT       ptlLineTop;
  int         yItemBottom;
  int         yItemTop;
  int         left;
  int         fsContinue = 1L;
  BOOL        fsEndNext = FALSE;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if ((pccinfo->flWindowAttr & CA_TREELINE))
  {
    /*****************************************************************/
    /* Get the top of the line for the preccEnum and left-most       */
    /* point of the painting for the siblings of     preccEnum.      */
    /*****************************************************************/
    if (preccEnum->preccParent)
    {
      GetLineTop(hwnd, pccinfo, preccEnum, slIcon, slTreeIcon,
                 &ptlLineTop);

      left = ptlLineTop.x;
    }
    else
    {
      left = 0;
    }
  }

  while (preccEnum)
  {
    if (ISRECORDVISIBLE(preccEnum))
    {
      /* relative to window */
      CnrTRVQueryIconTextRect(hwnd, preccEnum, TRUE, slIcon, slTreeIcon,
                              &rctlIconCur, &rctlTreeIcon, &rctlText);

      /* relative to  window */
      yItemBottom = CnrTRVQueryPtOfItem(hwnd, rctlIconCur, rctlTreeIcon,
                                        rctlText, PT_BOTTOM) -
                    pccinfon->ptlNonIconOrg.y;

      /* relative to  window */
      yItemTop = CnrTRVQueryPtOfItem(hwnd, rctlIconCur, rctlTreeIcon,
                                     rctlText, PT_TOP) -
                 pccinfon->ptlNonIconOrg.y;

      /***************************************************************/
      /* Set the leftmost point. If it is name view leftmost         */
      /* point is left of the icon rect and if it is text/icon       */
      /* view left of the tree icon rect is the leftmost.            */
      /***************************************************************/
      if (!(pccinfo->flWindowAttr & CA_TREELINE))
      {
        if ((pccinfo->flWindowAttr & CV_NAME) ||
            (!TRVhasVisibleChild (preccEnum)))
        {
          if (pccinfo->flWindowAttr & CV_TEXT)
          {
            left = rctlText.left;
          }
          else
          {
            left = rctlIconCur.left;
          }
        }
        else
        {
          left = rctlTreeIcon.left;
        }
      }

      if ((pccinfo->flWindowAttr & CA_TREELINE) &&
           preccEnum->preccParent)
      {
        /***************************************************************/
        /* Draw the line for the item.                                 */
        /***************************************************************/
        ptlLineTop.y = CnrTRVDrawLine(hwnd, hps, preccEnum,
                                      ptlLineTop, rctlIconCur,
                                      rctlTreeIcon, rctlText);
      }

      /***************************************************************/
      /* If a part of the item is in the viewport, draw it.          */
      /***************************************************************/
      if ((rctlText.right > rctlClient.left ) &&
          (left < rctlClient.right) &&
          (yItemTop < (rctlClient.bottom)) &&
          (yItemBottom > (rctlClient.top)) )
      {
        RECT       rctlIcon;

        /*************************************************************/
        /* To draw the icon, we need rectl without emph space.       */
        /*************************************************************/
        rctlIcon.left = rctlIconCur.left + ICN_EMPHSPACE;
        rctlIcon.right = rctlIconCur.right - ICN_EMPHSPACE;
        rctlIcon.bottom = rctlIconCur.bottom - ICN_EMPHSPACE;
        rctlIcon.top = rctlIconCur.top + ICN_EMPHSPACE;

        if (!(CnrTRVDrawItem(hwnd, hps, preccEnum,
                             rctlIcon, rctlTreeIcon, rctlText,
                             DE_TREEICON | DE_ICON | DE_TEXT)))
        {
          return (0L);
        }
      } /* endif */

      /***************************************************************/
      /* If the item is below of (out of)the client area of the      */
      /* window, exit from the while loop and return -1 so that      */
      /* any external loop that calls this function terminates       */
      /* because we don't need to paint any item out of the client   */
      /* area.                                                       */
      /***************************************************************/
      if (yItemBottom >= rctlClient.bottom)
      {
        if ((pccinfo->flWindowAttr & CA_TREELINE) && !fsEndNext)
        {
          fsEndNext = TRUE;        /* want to draw next item even though */
                                   /* next item is not visible because */
                                   /* it still has line to be drawn */
        }
        else
        {
          fsContinue = -1L;

          /**********************************************************/
          /* We need to draw the lines of the next visible siblings */
          /* of the parent, grandparent, ...,because they are still */
          /* visible even though preccEnum is out of the viewport.  */
          /**********************************************************/
          if ((pccinfo->flWindowAttr & CA_TREELINE) &&
              preccEnum->preccParent)
          {
            LPRECORDCOREN  preccTemp = preccEnum->preccParent;

            while (preccTemp && preccTemp->preccParent)
            {
              LPRECORDCOREN  preccTempNext;

              if (preccTempNext = GetNextUnfilteredItem(preccTemp))
              {
                GetLineTop(hwnd, pccinfo, preccTempNext, slIcon,
                           slTreeIcon, &ptlLineTop);

                CnrTRVQueryIconTextRect(hwnd, preccTempNext, TRUE,
                                        slIcon, slTreeIcon,
                                        &rctlIconCur, &rctlTreeIcon,
                                        &rctlText);

                /*****************************************************/
                /* Draw the line for the item.                       */
                /*****************************************************/
                CnrTRVDrawLine(hwnd, hps, preccEnum, ptlLineTop,
                               rctlIconCur, rctlTreeIcon, rctlText);
              }
              preccTemp = preccTemp->preccParent;
            }
          }
        } /* endif */
      } /* endif */

      /***************************************************************/
      /* If the item is to the right of (out of)the invalidated      */
      /* rectangle or a bottom of the current item is below of the   */
      /* client area, don't need to check the children of the item   */
      /* because all of them are to the right or below of the window.*/
      /***************************************************************/
      if (!(left >= rctlClient.right) &&
          (fsContinue != -1L))
      {
        if ((preccEnum->flRecordAttr & CRA_EXPANDED) &&
            preccEnum->preccFirstChild)
        {
          fsContinue = TRVDrawRecords(hwnd, hps,
                                      preccEnum->preccFirstChild,
                                      rctlClient, slIcon, slTreeIcon);
        } /* endif */
      } /* endif */

      if (fsContinue <= 0L)
      {
        return (fsContinue);
        break;
      }
    } /* endif */

    preccEnum = preccEnum->preccNextItem;
  } /* endwhile */

  return (1L);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  GetLineTop                                        */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get Top of the Line of the Item                */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
void FAR GetLineTop(HANDLE hwnd, LPCNRINFO pccinfo, LPRECORDCOREN preccItem,
                     POINT slIcon, POINT slTreeIcon, LPPOINT pptlLineTop)
{
  RECT       rctlIconCur;
  RECT       rctlTreeIcon;
  RECT       rctlText;

  CnrTRVQueryIconTextRect(hwnd, preccItem->preccParent, TRUE,
                          slIcon, slTreeIcon,
                          &rctlIconCur, &rctlTreeIcon, &rctlText);

  if (pccinfo->flWindowAttr & CV_NAME)
  {
    pptlLineTop->x = (rctlIconCur.left + rctlIconCur.right) / 2;
    pptlLineTop->y = rctlIconCur.bottom;
  }
  else
  {
    pptlLineTop->x = (rctlTreeIcon.left + rctlTreeIcon.right) / 2;
    pptlLineTop->y = rctlTreeIcon.bottom;
  }
  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVDrawItem                                    */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:  Paint Tree View Item                           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrTRVDrawItem(                                       /*RKK1*/
  HANDLE          hwndCnr,
  HDC             hps,
  LPRECORDCOREN   precord,
  RECT            rctlIcon,
  RECT            rctlTreeIcon,
  RECT            rctlText,
  WORD            usWhat  )                                    /*RKK1*/
{
  BOOL            blOwnerdrawIcon;     /* Flag for the owner draw style */
  BOOL            blOwnerdrawTreeIcon; /* Flag for the owner draw style */
  BOOL            blOwnerdrawText;     /* Flag for the owner draw style */
  HANDLE          hwndOwner;           /* handle of the owner window */
  HBITMAP         hbmDraw=NULL;        /* handle of the bitmap to be drawn*/
  DRAWITEMSTRUCT  DrawItem;            /* structure passed to owner window*/
                                       /* for the owner draw */
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  LPICONRESIZED   pirsDraw=NULL;
  RECT            rctlIconCur;
  RECT            rctlTextCur;
  UINT            wWinID;                                               /*DAB1*/
  CNRDRAWITEMINFO CnrDrawItemInfo;
  BOOL            bDefault = FALSE;
  HFONT           hOldFont = NULL;     /* save old font                 */
  int             nSavedDC;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if ((pccinfo->flWindowAttr & CV_TEXT) && (usWhat & DE_TEXT))
  {
    rctlTextCur.left = rctlText.left - TXT_TOPMARGIN;
    rctlTextCur.right = rctlText.right + TXT_TOPMARGIN;
    rctlTextCur.bottom = rctlText.bottom + TXT_TOPMARGIN;
    rctlTextCur.top = rctlText.top - TXT_TOPMARGIN;
  }
  else
  {
    if (!(pccinfo->flWindowAttr & CV_TEXT) && (usWhat & DE_ICON))
    {
      rctlIconCur.left = rctlIcon.left - ICN_EMPHSPACE;
      rctlIconCur.right = rctlIcon.right + ICN_EMPHSPACE;
      rctlIconCur.bottom = rctlIcon.bottom + ICN_EMPHSPACE;
      rctlIconCur.top = rctlIcon.top - ICN_EMPHSPACE +
                        ICN_SIDEMARGIN;
    }
  }

  /*******************************************************************/
  /* If the container has the OWNERDRAW style, then send the         */
  /* WM_DRAWITEM message to the owner.  If, either the owner did     */
  /* not draw the record (returned FALSE), or the container was not  */
  /* OWNERDRAW, then we will paint the record.                       */
  /*******************************************************************/
  blOwnerdrawIcon = FALSE;
  blOwnerdrawTreeIcon = FALSE;
  blOwnerdrawText = FALSE;

  hwndOwner = PARENTOF (hwndCnr);
  wWinID = (UINT)getWindowId( hwndCnr );                              /*DAB1*/

  if (pccinfo->flWindowAttr & CA_OWNERDRAW)
  {
    /****************************************************************/
    /* Set the user specified font - If NULL use default (System)   */
    /****************************************************************/
    if (pccinfon->hCnrFont)
      hOldFont = SelectObject (hps, pccinfon->hCnrFont);

    /******************************************************************/
    /* Set DrawItem data by calling CnrSetDrawItem                    */
    /******************************************************************/
    CnrSetDrawItem (hwndCnr, hps, &DrawItem, 0);

    /******************************************************************/
    /* Set container control ID.                                      */
    /******************************************************************/
    DrawItem.CtlID = wWinID;

    /******************************************************************/
    /* Set up the CnrDrawItemInfo structure, and pass it back in the  */
    /* itemData field.                                                */
    /******************************************************************/
    COPYNDATATOX(precord);
    CnrDrawItemInfo.pRecord = precord->preccX;
    CnrDrawItemInfo.pFieldInfo = NULL;
    DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

    /*****************************************************************/
    /* If in the name view, draw the icon and the text. If in the    */
    /* text view draw the tree icon and the text. If in the icon     */
    /* view, draw the tree icon, the icon and the text.              */
    /*****************************************************************/
    if (!(pccinfo->flWindowAttr & CV_NAME) &&  /* tree icon or text view */
        TRVhasVisibleChild (precord) &&        /* the node is not a leaf */
        (usWhat & DE_TREEICON))
    {
      nSavedDC = SaveDC (hps);

      /****************************************************************/
      /* Set rcItem as the rectangle of the icon.                     */
      /****************************************************************/
      DrawItem.rcItem = rctlTreeIcon;

      /****************************************************************/
      /* Set the idItem field.                                        */
      /****************************************************************/
      DrawItem.itemID = CMA_TREEICON;

      /****************************************************************/
      /* Send WM_DRAWITEM for the owner drawing for the tree icon.    */
      /****************************************************************/
      blOwnerdrawTreeIcon = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/

      if (nSavedDC)
        RestoreDC (hps, nSavedDC);
    }

    nSavedDC = SaveDC (hps);

    if ((pccinfo->flWindowAttr & CV_TEXT) &&
        (usWhat & DE_TEXT))
    {
      /***************************************************************/
      /* Set rcItem as the rectangle of the text.                    */
      /* In text view, we need to provide the space for the cursored */
      /* emphasis and target emphasis.                               */
      /***************************************************************/
      DrawItem.rcItem = rctlTextCur;

      /****************************************************************/
      /* Set the itemID field.                                        */
      /****************************************************************/
      DrawItem.itemID = CMA_TEXT;

      /****************************************************************/
      /* Send WM_DRAWITEM for the owner drawing if the text.          */
      /****************************************************************/
      blOwnerdrawText = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
    }
    else /* tree icon view or tree name view */
    {
      /* for both tree icon view and name view */

      if (usWhat & DE_ICON)
      {
        /*************************************************************/
        /* Set rcItem as the rectangle of the icon.                  */
        /*************************************************************/
        DrawItem.rcItem = rctlIcon;

        /**************************************************************/
        /* Set the itemID field.                                      */
        /**************************************************************/
        DrawItem.itemID = CMA_ICON;

        /**************************************************************/
        /* Send WM_DRAWITEM for the owner drawing if the icon.        */
        /**************************************************************/
        blOwnerdrawIcon = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
      }

      if (usWhat & DE_TEXT)
      {
        /**************************************************************/
        /* Set rcItem as the rectangle of the text.                   */
        /**************************************************************/
        DrawItem.rcItem = rctlText;

        /**************************************************************/
        /* Set the itemID field.                                      */
        /**************************************************************/
        DrawItem.itemID = CMA_TEXT;

        /**************************************************************/
        /* Send WM_DRAWITEM for the owner drawing if the text.        */
        /**************************************************************/
        blOwnerdrawText = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
      }
    } /* endif */

    if (nSavedDC)
      RestoreDC (hps, nSavedDC);

    if (hOldFont)
      SelectObject (hps, hOldFont);
  }

  if (!(pccinfo->flWindowAttr & CV_NAME) &&  /* tree icon or text view */
      TRVhasVisibleChild (precord))          /* the node is not a leaf*/
  {
    if (!blOwnerdrawTreeIcon && (usWhat & DE_TREEICON))
    {
      DWORD      flTempAttr;

      /***************************************************************/
      /* Don't draw selection emph for the tree node. (+, -)         */
      /***************************************************************/
      flTempAttr = precord->flRecordAttr & ~(CRA_SELECTED | CRA_INUSE);

      /***************************************************************/
      /* Check if a custom +/- symbol was specified                  */
      /***************************************************************/
      if (!((pccinfo->hbmCollapsed) && (pccinfo->hbmExpanded)) &&
          !((pccinfo->hptrCollapsed) && (pccinfo->hptrExpanded)))
      {
        bDefault = TRUE;
#ifdef IC_WU
        {                                                       /*RKK1*/
        // With Wind/U need to do a LoadLibrary() before the LoadBitmap()
        HANDLE hLib = LoadLibrary( ICL_CONTAINER_DLL );
        if (precord->flRecordAttr & CRA_EXPANDED)
          hbmDraw = LoadBitmap( hLib, MAKEINTRESOURCE(TREEMINUS) );
        else
          hbmDraw = LoadBitmap( hLib, MAKEINTRESOURCE(TREEPLUS) );
        FreeLibrary( hLib );
        }                                                       /*RKK1*/
#else
        if (precord->flRecordAttr & CRA_EXPANDED)
        {
          hbmDraw = LoadBitmap( vhCNRInstance,                          /*DAB3*/
                                MAKEINTRESOURCE(TREEMINUS));
        }
        else
        {
          hbmDraw = LoadBitmap( vhCNRInstance,                          /*DAB3*/
                                MAKEINTRESOURCE(TREEPLUS));
        }
#endif
      }
      else if ((pccinfo->flWindowAttr & CA_DRAWBITMAP) &&
               !(pccinfo->flWindowAttr & CA_DRAWICON))
      {
        if (precord->flRecordAttr & CRA_EXPANDED)
        {
          hbmDraw = pccinfo->hbmExpanded;
        }
        else
        {
          if (precord->flRecordAttr & CRA_COLLAPSED)
          {
            hbmDraw = pccinfo->hbmCollapsed;
          }
        }
      }
      else       /* draw icon */
      {
        pirsDraw = TRVFindTreePirs (hwndCnr, precord);
      }

      if (hbmDraw || pirsDraw)
      {
        IcnDrawIcon (hps, pirsDraw, hbmDraw, flTempAttr,
                     rctlTreeIcon, pccinfon);
      }
      if (bDefault)
      {
        DeleteObject (hbmDraw);
      }
    }
  }

  if (!(pccinfo->flWindowAttr & CV_TEXT)) /* tree name view or icon view */
  {
    if (!blOwnerdrawIcon && (usWhat & DE_ICON))
    {
      /****************************************************************/
      /* Get the information to use for drawing                       */
      /****************************************************************/
      CnrSetupDrawInfo( hwndCnr, precord, &hbmDraw, &pirsDraw );

      /****************************************************************/
      /* If no icon or bitmap to draw exists and WM_DRAWITEM is not   */
      /* sent before, send WM_DRAWITEM to the owner of the container  */
      /* for the app to provide a bitmap or icon or to draw it.       */
      /* (James Taylor requirement)                                   */
      /****************************************************************/
      if (!(hbmDraw || pirsDraw) &&
          !(pccinfo->flWindowAttr & CA_OWNERDRAW))
      {
        CnrSetDrawItem (hwndCnr, hps, &DrawItem, CMA_ICON);

        /**************************************************************/
        /* Set container control ID.                                  */
        /**************************************************************/
        DrawItem.CtlID = wWinID;

        /*************************************************************/
        /* Set rcItem as the rectangle of the icon.                  */
        /*************************************************************/
        DrawItem.rcItem = rctlIcon;

        /**************************************************************/
        /* Set up the CnrDrawItemInfo structure, and pass it back in  */
        /* the itemData field.                                        */
        /**************************************************************/
        COPYNDATATOX(precord);
        CnrDrawItemInfo.pRecord = precord->preccX;
        CnrDrawItemInfo.pFieldInfo = NULL;
        DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

        blOwnerdrawIcon = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
        if (!blOwnerdrawIcon)
        {
          CnrSetupDrawInfo( hwndCnr, precord, &hbmDraw, &pirsDraw );
        }
      }

      if ((hbmDraw || pirsDraw ) && (!blOwnerdrawIcon))
      {
        /**************************************************************/
        /* Draw the icon (or bitmap)                                  */
        /**************************************************************/
        IcnDrawIcon (hps, pirsDraw, hbmDraw,
                     (precord->flRecordAttr & CRA_INUSE) ?
                     (precord->flRecordAttr | CRA_INUSE) :
                     precord->flRecordAttr, rctlIcon, pccinfon);
      }

      if ((precord->flRecordAttr & CRA_CURSORED) &&
          (!blOwnerdrawIcon))
      {
        /**************************************************************/
        /* Draw a box around the icon if necessary.                   */
        /**************************************************************/
        CnrDrawCursorBox(hwndCnr, hps, precord, rctlIconCur, TRUE);
      } /* endif */
    }
  }

  if (!blOwnerdrawText && (usWhat & DE_TEXT))
  {
    IcnDrawText (hps, GETLPSTRFORTRV(precord->preccX), rctlText,
                 (precord->flRecordAttr & CRA_INUSE) ?
                 (precord->flRecordAttr | CRA_INUSE) :
                 precord->flRecordAttr, IS_SIDETEXT, pccinfon);

    if ((pccinfo->flWindowAttr & CV_TEXT) &&
        (precord->flRecordAttr & CRA_CURSORED))
    {
      /***************************************************************/
      /* In text view, we need to provide the cursored emphasis,     */
      /* and target emphasis on the text by drawing proper box       */
      /* around.                                                     */
      /***************************************************************/
      CnrDrawCursorBox(hwndCnr, hps, precord, rctlTextCur, TRUE);
    }
  } /* endif ! Owner draw */

  return (TRUE);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVEraseItem                                   */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:  Erase Tree View Item                           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrTRVEraseItem(                                      /*RKK1*/
  HANDLE         hwnd,
  HDC            hps,
  LPRECORDCOREN  preccItem,
  RECT           rctlIcon,
  RECT           rctlTreeIcon,
  RECT           rctlText,
  WORD           usWhat  )                                     /*RKK1*/
{
                                   /* for the owner draw */
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if ((usWhat & DE_ICON) && !(pccinfo->flWindowAttr & CV_TEXT))
  {
    rctlIcon.left -= ICN_EMPHSPACE;
    rctlIcon.right += ICN_EMPHSPACE;
    rctlIcon.bottom += ICN_EMPHSPACE;
    rctlIcon.top -= ICN_EMPHSPACE;
    CnrEraseRect(hwnd, hps, rctlIcon);
  }

  if ((usWhat & DE_TREEICON) && !(pccinfo->flWindowAttr & CV_NAME) &&
      TRVhasVisibleChild (preccItem))
  {
    CnrEraseRect(hwnd, hps, rctlTreeIcon);
  }

  if (usWhat & DE_TEXT)
  {
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      rctlText.left -= 2 * TXT_SIDEMARGIN;
      rctlText.right += 2 * TXT_SIDEMARGIN;
      rctlText.bottom += TXT_TOPMARGIN;
      rctlText.top -= TXT_TOPMARGIN;
    }
    CnrEraseRect(hwnd, hps, rctlText);
  }
  return TRUE;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVDrawLine                                    */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:  Draw  Tree View Item Line                      */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
int  FAR CnrTRVDrawLine(hwnd, hps, preccItem, ptlLine, rctlIcon,
                         rctlTreeIcon, rctlText)
HANDLE        hwnd;
HDC           hps;
LPRECORDCOREN preccItem;
POINT         ptlLine;
RECT          rctlIcon;
RECT          rctlTreeIcon;
RECT          rctlText;
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  RECT        rctlLine;
  int         cxTreeLine;
  HBRUSH      hBrush;
  RECT        rclCnr;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  cxTreeLine = (pccinfo->cxTreeLine < 0) ? (int)1 : pccinfo->cxTreeLine;

  /*******************************************************************/
  /* Draw Vertical Line.                                             */
  /*******************************************************************/
  /*******************************************************************/
  /* Put this fix in cause it seems that the FillRect                */
  /* call does not process large numbers properly                    */
  /*******************************************************************/
  GetClientRect(hwnd, &rclCnr);
  if (rclCnr.top > ptlLine.y)
    rctlLine.top = rclCnr.top;
  else
    rctlLine.top = ptlLine.y;

  if (pccinfo->flWindowAttr & CV_NAME)
  {
    rctlLine.bottom = (rctlIcon.top + rctlIcon.bottom) / 2;
  }
  else
  {
    rctlLine.bottom = (rctlTreeIcon.top + rctlTreeIcon.bottom) / 2;
  }
  rctlLine.left = ptlLine.x - cxTreeLine / 2;
  rctlLine.right = rctlLine.left + cxTreeLine;

  if (hBrush = CreateSolidBrush (
               pccinfon->ulColorPPSupport[CMA_FOREGROUNDCOLOR]))
  {
    PMFillRect (hps, &rctlLine, hBrush);
    DeleteObject(hBrush);
  }

  /*******************************************************************/
  /* Draw Horizontal Line.                                           */
  /*******************************************************************/
  if (pccinfo->flWindowAttr & CV_NAME)
  {
    rctlLine.right = rctlIcon.left;
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      rctlLine.right = (TRVhasVisibleChild (preccItem)) ?
                                              rctlTreeIcon.left:
                                                rctlText.left - ICN_SELSPACE;
    }
    else          /* tree icon view */
    {
      rctlLine.right = (TRVhasVisibleChild (preccItem)) ?
                                              rctlTreeIcon.left:
                                                  rctlIcon.left;
    }
  }


  rctlLine.bottom += cxTreeLine / 2;
  rctlLine.top = rctlLine.bottom - cxTreeLine;

  if (hBrush = CreateSolidBrush (
               pccinfon->ulColorPPSupport[CMA_FOREGROUNDCOLOR]))
  {
    PMFillRect (hps, &rctlLine, hBrush);
    DeleteObject(hBrush);
  }

  return (rctlLine.bottom - 1);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVNodeLevel                                      */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get Level Of Node                              */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
WORD FAR TRVNodeLevel(preccObj)
LPRECORDCOREN    preccObj;
{
  WORD   usNodeLevel = 0;

  while (preccObj)
  {
    usNodeLevel++;
    preccObj = preccObj->preccParent;
  }
  return usNodeLevel;
}
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVIsAncestor                                     */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get Root of Node                               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR TRVIsAncestor(preccAncestor, preccDescendant)
LPRECORDCOREN   preccAncestor;
LPRECORDCOREN   preccDescendant;
{
  if (!preccDescendant)
  {
    return FALSE;
  } /* endif */

  preccDescendant = preccDescendant->preccParent;
  while (preccDescendant)
  {
    if (preccAncestor == preccDescendant)
    {
      return TRUE;
    }
    preccDescendant = preccDescendant->preccParent;
  }
  return FALSE;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVGetRoot                                        */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get Root of Node                               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LPRECORDCOREN FAR TRVGetRoot(preccObj)
LPRECORDCOREN    preccObj;
{
  LPRECORDCOREN  preccRoot = NULL;

  while (preccObj)
  {
    preccRoot = preccObj;
    preccObj = preccObj->preccParent;
  }
  return (preccRoot);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  TRVGetCy                                          */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get cy of the item given.                      */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
WORD FAR TRVGetCy(hwnd, preccObj, slIcon, slTreeIcon)
HANDLE          hwnd;
LPRECORDCOREN  preccObj;
POINT         slIcon;
POINT         slTreeIcon;
{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  WORD        cy;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & (CV_NAME))      /* tree name view */
  {
    cy =(WORD)max (slIcon.y, preccObj->slTextSize.cy);                  /*DAB1*/
  }
  else
  {
    if (!(TRVhasVisibleChild (preccObj)))
    {
      slTreeIcon.y = 0;
    }

    cy = (WORD)max (slTreeIcon.y, preccObj->slTextSize.cy);             /*DAB1*/
    if (!(pccinfo->flWindowAttr & CV_TEXT))      /* tree icon view */
    {
      cy = max(cy, (WORD)slIcon.y);
    }
  }
  return (cy);
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVQueryPtOfItem                               */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Query the Pt  of Item in Tree View             */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculate pt  of the item  */
/*     specified in the tree view                                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
int FAR CnrTRVQueryPtOfItem(hwndCnr, rctlIcon, rctlTreeIcon,
                            rctlText, lDir)
HANDLE        hwndCnr;
RECT          rctlIcon;
RECT          rctlTreeIcon;
RECT          rctlText;
long          lDir;               /* flag for the desired y pos */
                                  /* can be either PT_TOP or PT_BOTTOM */

{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*****************************************************************/
  /* Get the topmost point of the item                             */
  /*****************************************************************/
  if (lDir == PT_TOP)
  {
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      /*************************************************************/
      /* The top of the item in the text view is the top of the    */
      /* text rect or the tree icon rect whichever has higher      */
      /* height.                                                   */
      /*************************************************************/
      return (min(rctlTreeIcon.top, rctlText.top) +
              pccinfon->ptlNonIconOrg.y);
    }

    if (pccinfo->flWindowAttr & CV_NAME)
    {
      /*************************************************************/
      /* The top of the item in the name view is the top of the    */
      /* text rect or the      icon rect whichever has higher      */
      /* height.                                                   */
      /*************************************************************/
      return (min(rctlIcon.top, rctlText.top) +
              pccinfon->ptlNonIconOrg.y);
    }

    /***************************************************************/
    /* The top of the item in the icon view is the top of the      */
    /* text rect, the icon rect of tree icon rect whichever        */
    /* has higher height.                                          */
    /***************************************************************/
    return (min(rctlTreeIcon.top,
                min(rctlIcon.top, rctlText.top) ) +
            pccinfon->ptlNonIconOrg.y) ;
  }

  /*****************************************************************/
  /* Get the bottommost point of the item                          */
  /*****************************************************************/
  if (lDir == PT_BOTTOM)
  {
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      /*************************************************************/
      /* The bottom of the item in the text view is the bottom     */
      /* of the text rect or the tree icon rect whichever has      */
      /* higher height.                                            */
      /*************************************************************/
      return (max(rctlTreeIcon.bottom, rctlText.bottom) +
              pccinfon->ptlNonIconOrg.y);
    }

    if (pccinfo->flWindowAttr & CV_NAME)
    {
      /*************************************************************/
      /* The bottom of the item in the name view is the bottom     */
      /* of the text rect or the icon rect whichever has higher    */
      /* height.                                                   */
      /*************************************************************/
      return (max(rctlIcon.bottom, rctlText.bottom) +
              pccinfon->ptlNonIconOrg.y);
    }
    /***************************************************************/
    /* The bottom of the item in the icon view is the bottom       */
    /* of the text rect, the icon rect or tree icon rect           */
    /* whichever has higher height.                                */
    /***************************************************************/
    return (max(rctlTreeIcon.bottom,
                max(rctlIcon.bottom, rctlText.bottom) ) +
            pccinfon->ptlNonIconOrg.y);
  }

  if (lDir == PT_LEFT)
  {
    /***************************************************************/
    /* Get the leftmost point. If it is name view leftmost         */
    /* point is left of the icon rect and if it is text view or   */
    /* icon view the point is left of the rctlTreeIcon.           */
    /***************************************************************/
    if (pccinfo->flWindowAttr & CV_NAME)
    {
      return (rctlIcon.left + pccinfon->ptlNonIconOrg.x);
    }

    return (rctlTreeIcon.left + pccinfon->ptlNonIconOrg.x);
  }
  /***********************************************************/
  /* Get the rightmost point. In all views the right         */
  /* boundary of the text is the rightmost point.            */
  /***********************************************************/
  if (lDir == PT_RIGHT)
  {
    return (rctlText.right + pccinfon->ptlNonIconOrg.x);
  } /* endif */
  return 0;

}

/*********************************************************************/
/*                                                                   */
/*  Function Name:     CnrTRVQueryIconTextRect                       */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Control Query Icon Text Rectangles           */
/*                                                                   */
/*  Functional Description:                                          */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR  CnrTRVQueryIconTextRect(hwnd, precord, flIncEmph, slIcon,
                     slTreeIcon, prctlIcon, prctlTreeIcon, prctlText)
HANDLE       hwnd;
LPRECORDCOREN precord;
BOOL         flIncEmph;          /* ask if icon rect include emph space */
POINT        slIcon;
POINT        slTreeIcon;
LPRECT       prctlIcon;
LPRECT       prctlTreeIcon;
LPRECT       prctlText;
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  WORD       usLeftMargin;

  /*******************************************************************/
  /* Get pointer to locals                                           */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  usLeftMargin = max((WORD)pccinfon->lAveCharWidth,
                     (WORD)MINCX_TREELEFTMARGIN);

  /*******************************************************************/
  /* If the pointer to the icon rect is given get icon rect          */
  /*******************************************************************/
  if (prctlIcon && !(pccinfo->flWindowAttr & CV_TEXT))
  {
    /*****************************************************************/
    /* Calculate icon rectl which is relative to the window origin.  */
    /*****************************************************************/
    prctlIcon->left = precord->ptlNonIconPos.x -
                       pccinfon->ptlNonIconOrg.x;
    prctlIcon->right = prctlIcon->left + slIcon.x;
    prctlIcon->top = precord->ptlNonIconPos.y -
                     pccinfon->ptlNonIconOrg.y;
    prctlIcon->bottom = prctlIcon->top + slIcon.y;

    if (flIncEmph)
    {
      /***************************************************************/
      /* Calculate icon rectangle including selection space.         */
      /***************************************************************/
      prctlIcon->left -= ICN_EMPHSPACE;
      prctlIcon->bottom += ICN_EMPHSPACE;
      prctlIcon->right += ICN_EMPHSPACE;
      prctlIcon->top -= ICN_EMPHSPACE;
    }
  }

  /*******************************************************************/
  /* if pointer to the text rect is given get the value              */
  /*******************************************************************/
  if (prctlText)
  {
    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      /***************************************************************/
      /* In text view position of the record is the origin of the    */
      /* text rectangle. Make the pos to be relative to the          */
      /* Cnr window by subtractin ptlOrigin                          */
      /***************************************************************/
      prctlText->left = precord->ptlNonIconPos.x -
                         pccinfon->ptlNonIconOrg.x;
      prctlText->right = (prctlText->left +                             /*DAB1*/
                          max((long)precord->usLongestLine,
                              precord->slTextSize.cx));                 /*DAB1*/
      prctlText->top = precord->ptlNonIconPos.y -
                       pccinfon->ptlNonIconOrg.y;
      prctlText->bottom = prctlText->top + precord->slTextSize.cy;      /*DAB1*/
    }
    else
    {
      /***************************************************************/
      /* Position of the record is position of the icon. So, we need */
      /* adjust to center the text relative to the icon.             */
      /*  ______           a:from Bottom of the icon to the bottom   */
      /* |      |  _____d_____    of the text (iconSize.cy -         */
      /* |      | b|_________|c   textSize.cy/2                      */
      /* |______|   a             b:the space between text and icon */
      /*                            lAveCharWidth                    */
      /*      c: textSize.cy                                         */
      /*      d: textSize.cx                                         */
      /***************************************************************/
      prctlText->left = precord->ptlNonIconPos.x -
                         pccinfon->ptlNonIconOrg.x + slIcon.x +
                         usLeftMargin;
      prctlText->right = (prctlText->left +                             /*DAB1*/
                          max((long)precord->usLongestLine,
                              precord->slTextSize.cx));                 /*DAB1*/
      prctlText->top = (precord->ptlNonIconPos.y -                      /*DAB1*/
                       pccinfon->ptlNonIconOrg.y -
                       (precord->slTextSize.cy - slIcon.y) / 2);        /*DAB1*/
      prctlText->bottom = prctlText->top + precord->slTextSize.cy;      /*DAB1*/
    }
  }

  if (prctlTreeIcon && !(pccinfo->flWindowAttr & CV_NAME))
  {
    BOOL  flHasTreeIcon = TRVhasVisibleChild (precord);

    prctlTreeIcon->right = precord->ptlNonIconPos.x -
                            pccinfon->ptlNonIconOrg.x - usLeftMargin;
    prctlTreeIcon->left = prctlTreeIcon->right - slTreeIcon.x;

    if (pccinfo->flWindowAttr & CV_TEXT)
    {
      prctlTreeIcon->top = (precord->ptlNonIconPos.y -                  /*DAB1*/
                               pccinfon->ptlNonIconOrg.y -
                               ((flHasTreeIcon) ?
                                ((slTreeIcon.y -
                                  precord->slTextSize.cy) / 2) : 0));   /*DAB1*/
      prctlTreeIcon->bottom = ((flHasTreeIcon) ?                        /*DAB1*/
                               (prctlTreeIcon->top + slTreeIcon.y) :
                               (prctlTreeIcon->top +                    /*DAB1*/
                               precord->slTextSize.cy));                /*DAB1*/

    }
    else         /* tree icon view */
    {
      prctlTreeIcon->top = precord->ptlNonIconPos.y -
                               pccinfon->ptlNonIconOrg.y -
                               ((flHasTreeIcon) ?
                                ((slTreeIcon.y - slIcon.y) / 2) : 0);
      prctlTreeIcon->bottom = ((flHasTreeIcon) ?
                               (prctlTreeIcon->top + slTreeIcon.y) :
                               (prctlTreeIcon->top + slIcon.y));
    }
  }

  return;

}
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVCalcTreeIconSize                            */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Common Function Calculate Icon Size  */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculates the icon size   */
/*   based on the user provided information.  This function is used  */
/*   by the Tree View.                                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:                                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR  CnrTRVCalcTreeIconSize( hwndCnr, pslTreeIcon)
HANDLE   hwndCnr;
LPPOINT  pslTreeIcon;
{
  LPCNRINFON pccinfon;
  LPCNRINFO pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if ((pccinfo->slTreeBitmapOrIcon.x <= 0) &&
       (pccinfo->slTreeBitmapOrIcon.y <= 0) )
  {
    pslTreeIcon->x = pccinfon->lMaxBaselineExt;
    pslTreeIcon->y = pccinfon->lMaxBaselineExt;
  }
  else
  {
    pslTreeIcon->x = pccinfo->slTreeBitmapOrIcon.x;
    pslTreeIcon->y = pccinfo->slTreeBitmapOrIcon.y;
  }
  return;
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrTRVSetFirstItem                                */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set the First Item in the Tree      Viewport   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in    */
/*     viewport searching backward or forward depending on the flag. */
/*     If preccCurrent is NULL, search from the last item            */
/*     (flFromabove = FALSE) or from the first item (flFromAbove =   */
/*     TRUE)  of the list.                                           */
/*     Then, set the first item in the cnrinfon.                     */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrTRVSetFirstItem(hwndCnr, preccCurrent, flFromAbove)
HANDLE       hwndCnr;
LPRECORDCOREN preccCurrent;
BOOL         flFromAbove;

{
  LPCNRINFON    pccinfon;
  RECT         rctlHscroll;        /* H scroll bar rectangle */
  RECT         rctlViewport;       /* rectangle of the viewport */
  RECT         rctlIcon;
  RECT         rctlTreeIcon;
  RECT         rctlText;
  RECT         rctlTitle;
  POINT        slIcon;             /* size of the icon */
  POINT        slTreeIcon;         /* size of the icon */
  LPRECORDCOREN preccNext;          /* next item of the current item */
  LPRECORDCOREN preccFirstinView;   /* first item in the viewport */
  long         top;                /* top of the certain range */
  long         bottom;             /* bottom of the certain range */
  BOOL         found = FALSE;

  pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &slIcon, &rctlTitle, &rctlViewport, NULL,
                   &rctlHscroll);

  CnrTRVCalcTreeIconSize (hwndCnr, &slTreeIcon);

  if (flFromAbove)
  {
    /*******************************************************************/
    /* If preccCurrent is NULL set it as as the first item in the      */
    /* list.                                                           */
    /*******************************************************************/
    if (!preccCurrent)
    {
      preccCurrent = pccinfon->preccItem;
    } /* endif */

    while (preccCurrent && !found)
    {
      /***************************************************************/
      /* Get the bottom of the item to check if it is in the         */
      /* viewport                                                    */
      /***************************************************************/
      CnrTRVQueryIconTextRect(hwndCnr, preccCurrent, TRUE, slIcon,
                              slTreeIcon, &rctlIcon, &rctlTreeIcon,
                              &rctlText);
      bottom = CnrTRVQueryPtOfItem(hwndCnr, rctlIcon, rctlTreeIcon,
                                    rctlText, PT_BOTTOM);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
           (bottom < (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom)))
      {
        preccCurrent = TRVGetNextVisualItem (preccCurrent);
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    preccFirstinView = (preccCurrent) ? preccCurrent:
                                        pccinfon->preccItem;
  }
  else
  {
    preccNext = pccinfon->preccFirstinTRV;
    /*****************************************************************/
    /* Find the first item of which top  boundary is out of the      */
    /* viewport.                                                     */
    /*****************************************************************/
    while (preccCurrent && !found)
    {
      /***************************************************************/
      /* Get the bottom of the item to check if it is in the         */
      /* viewport                                                    */
      /***************************************************************/
      CnrTRVQueryIconTextRect(hwndCnr, preccCurrent, TRUE, slIcon,
                              slTreeIcon, &rctlIcon, &rctlTreeIcon,
                              &rctlText);
      top = CnrTRVQueryPtOfItem(hwndCnr, rctlIcon, rctlTreeIcon,
                                 rctlText, PT_TOP);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
           (top > (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom)))
      {
        /*************************************************************/
        /* Save preccCurrent as preccPrev for the case that          */
        /* preccCurrent is totally out of viewport.(Above the        */
        /* viewport.)                                                */
        /*************************************************************/
        if (ISRECORDVISIBLE(preccCurrent))
        {
          preccNext = preccCurrent;
        }
        preccCurrent = TRVGetPrevVisualItem(hwndCnr,preccCurrent);
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    if (preccCurrent)
    {
      /***************************************************************/
      /* If the first item whose top is above the viewport has       */
      /* bottom in the viewport, it is the first item.(Patially      */
      /* shown) Else, the first item in the viewport is the          */
      /* next item of the current item.                              */
      /***************************************************************/
      bottom = CnrTRVQueryPtOfItem(hwndCnr, rctlIcon, rctlTreeIcon,
                                    rctlText, PT_BOTTOM);
      if (bottom > (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom) )
      {
        preccFirstinView = preccCurrent;
      }
      else
      {
        preccFirstinView = preccNext;
      } /* endif*/
    }
    else
    {
      /***************************************************************/
      /* No item is above the viewport.                              */
      /* The first item in viewport is the first item in the         */
      /* list.                                                       */
      /***************************************************************/
      preccFirstinView = preccNext;
    } /* endif*/
  } /* endif */

  pccinfon->preccFirstinTRV = preccFirstinView;
  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  GetNextUnfilteredItem                             */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get Next Unfiltered Item                       */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LPRECORDCOREN FAR GetNextUnfilteredItem (preccObj)

LPRECORDCOREN preccObj;
{
  preccObj = preccObj->preccNextItem;
  while (preccObj && !(ISRECORDVISIBLE(preccObj)))
  {
    preccObj = preccObj->preccNextItem;
  }

  return (preccObj);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  GetPrevUnfilteredItem                             */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Get Prev Unfiltered Item                       */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LPRECORDCOREN FAR GetPrevUnfilteredItem (preccObj)

LPRECORDCOREN preccObj;
{
  preccObj = preccObj->preccPrevItem;
  while (preccObj && !(ISRECORDVISIBLE(preccObj)))
  {
    preccObj = preccObj->preccPrevItem;
  }

  return (preccObj);
}

/*********************************************************************/
/* FUNCTION NAME:  TRVGetNextVisualItem                              */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
LPRECORDCOREN FAR TRVGetNextVisualItem (LPRECORDCOREN preccPrev)

{
  LPRECORDCOREN  preccReturn;

  if (!preccPrev)
  {
    return NULL;
  }

  /*******************************************************************/
  /* If preccPrev is not filtered out, then the first visible        */
  /* child of the preccPrev is the next visual item.                 */
  /*******************************************************************/
  if (ISRECORDVISIBLE(preccPrev) &&
      (preccPrev->flRecordAttr & CRA_EXPANDED) &&
      preccPrev->preccFirstChild)
  {
    if (ISRECORDVISIBLE(preccPrev->preccFirstChild))
    {
      return (preccPrev->preccFirstChild);
    }

    if (preccReturn = GetNextUnfilteredItem (preccPrev->preccFirstChild))
    {
      return preccReturn;
    }
  }

  /*******************************************************************/
  /* Else, next visible item of the siblings is....                  */
  /* Else, next visible item of the parent   is....                  */
  /* Else, next visible item of the granparent   is....              */
  /*         :                                                       */
  /*         :                                                       */
  /*******************************************************************/
  while (preccPrev)
  {
    if (preccReturn = GetNextUnfilteredItem (preccPrev))
    {
      return preccReturn;
      break;
    }
    preccPrev = preccPrev->preccParent;
  }
  return NULL;
}

/*********************************************************************/
/* FUNCTION NAME:  TRVGetPrevVisualItem                              */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
LPRECORDCOREN FAR TRVGetPrevVisualItem (HANDLE hwnd,LPRECORDCOREN preccNext)

{
  LPRECORDCOREN   preccPrev;
  LPRECORDCOREN   preccParent;
  LPRECORDCOREN   preccReturn = NULL;

  if (!preccNext)
  {
    return NULL;
  }

  preccParent = preccNext->preccParent;

  /*******************************************************************/
  /* If preccNext has unfiltered previous item(of the siblings)      */
  /* the previous visual item of the preccNext is the bottom_most    */
  /* item of the subtree of the previous item.                       */
  /*******************************************************************/
  if (preccPrev = GetPrevUnfilteredItem (preccNext))
  {
    return (TRVFindLastUnfiltered(hwnd,preccPrev));
  }

  /*******************************************************************/
  /* If preccNext has no unfiltered previous item(of the siblings)   */
  /* the previous visual item of the preccNext is the its parent     */
  /* if it is visible. If it is not visible, the bottom_most item    */
  /* of the unfiltered previous item of the parent.                  */
  /*******************************************************************/
  while (preccParent)
  {
    if (ISRECORDVISIBLE(preccParent))
    {
      return (preccParent);
    }

    if (preccPrev = GetPrevUnfilteredItem (preccNext))
    {
      return (TRVFindLastUnfiltered(hwnd,preccPrev));
    } /* endif */

    preccParent = preccParent->preccParent;
  }
  return NULL;
}

/*********************************************************************/
/* FUNCTION NAME:  TRVGetNextItem                                    */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
LPRECORDCOREN FAR TRVGetNextItem(LPRECORDCOREN preccPrev)
{
  if (!preccPrev)
  {
    return NULL;
  }

  /*******************************************************************/
  /* If there are children, the first child is the next item in the  */
  /* tree.                                                           */
  /*******************************************************************/
  if (preccPrev->preccFirstChild)
  {
    return (preccPrev->preccFirstChild);
  }

  /*******************************************************************/
  /* Else, next item of the siblings is....                          */
  /* Else, next item of the parent   is....                          */
  /* Else, next item of the granparent   is....                      */
  /*         :                                                       */
  /*         :                                                       */
  /*******************************************************************/
  while (preccPrev)
  {
    if (preccPrev->preccNextItem)
    {
      return (preccPrev->preccNextItem);
      break;
    }
    preccPrev = preccPrev->preccParent;
  }

  return NULL;
}

/*********************************************************************/
/* FUNCTION NAME:  TRVGetPrevItem                                    */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
LPRECORDCOREN FAR TRVGetPrevItem(LPRECORDCOREN preccNext)

{
  if (!preccNext)
  {
    return NULL;
  }

  /*******************************************************************/
  /* If preccNext has          a previous item(of the siblings)      */
  /* the previous visual item of the preccNext is the bottom_most    */
  /* item of the subtree of the previous item.                       */
  /*******************************************************************/
  if (preccNext->preccPrevItem)
  {
    return (TRVFindLast(preccNext->preccPrevItem));
  }

  /*******************************************************************/
  /* If preccNext doesn't have a    previous item(of the siblings)   */
  /* the previous visual item of the preccNext is the its parent.    */
  /*******************************************************************/
  return (preccNext->preccParent);
}

/*********************************************************************/
/* FUNCTION NAME:  TRVFindLastUnfiltered                             */
/*                                                                   */
/*  Usage:                                                           */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION: This function finds the bottom_most       */
/*     visible item in the subtree that has preccParent as the root. */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
LPRECORDCOREN FAR TRVFindLastUnfiltered(HANDLE hwnd,
                                       LPRECORDCOREN preccParent)

{
  LPRECORDCOREN  preccReturn = NULL;
  LPRECORDCOREN  preccNext;
  LPRECORDCOREN  preccChild;

  if (preccParent && ISRECORDVISIBLE(preccParent))
  {
    preccReturn = preccParent;
  }

  if (preccParent && (preccParent->flRecordAttr & CRA_EXPANDED) &&
      ISRECORDVISIBLE(preccParent) && preccParent->preccFirstChild)
  {
    /******************************************************************/
    /* Get first visible item in the child level.                     */
    /******************************************************************/
    if (ISRECORDVISIBLE(preccParent->preccFirstChild))
    {
      preccNext = preccParent->preccFirstChild;
    }
    else
    {
      preccNext = GetNextUnfilteredItem (preccParent->preccFirstChild);
    }

    /*****************************************************************/
    /* Get the last visible item in the same level.                  */
    /*****************************************************************/
    while (preccNext)
    {
      preccReturn = preccNext;
      preccNext = GetNextUnfilteredItem (preccReturn);
    }

    /*****************************************************************/
    /* If there is preccReturn (the last sibling in the level),      */
    /* find bottom_most item of the subtree that has preccReturn as  */
    /* its root.                                                     */
    /*****************************************************************/
    if (preccReturn && (preccReturn != preccParent) &&
        (preccReturn->flRecordAttr & CRA_EXPANDED) &&
        preccReturn->preccFirstChild)
    {
      if (preccChild = TRVFindLastUnfiltered(hwnd,preccReturn))
      {
        preccReturn = preccChild;
      }
    }
  }
  return (preccReturn);
}

/*********************************************************************/
/* FUNCTION NAME:  TRVFindLast                                       */
/*                                                                   */
/*  Usage:                                                           */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION: This function finds the bottom_most       */
/*             item in the subtree that has preccParent as the root. */
/*                                                                   */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
LPRECORDCOREN FAR TRVFindLast(LPRECORDCOREN preccParent)

{
  LPRECORDCOREN   preccReturn = preccParent;
  LPRECORDCOREN   preccNext;

  if (preccParent && preccParent->preccFirstChild)
  {
    /*****************************************************************/
    /* Intialize the return value.                                   */
    /*****************************************************************/
    preccReturn = preccParent->preccFirstChild;

    /*****************************************************************/
    /* Get next visible item in the same level.                      */
    /*****************************************************************/
    preccNext = preccReturn->preccNextItem;

    /*****************************************************************/
    /* Get the last item of the siblings.                            */
    /*****************************************************************/
    while (preccNext)
    {
      preccReturn = preccNext;
      preccNext = preccNext->preccNextItem;
    }

    /*****************************************************************/
    /* If there is preccReturn (the last sibling in the level),      */
    /* find bottom_most item of the subtree that has preccReturn as  */
    /* its root.                                                     */
    /*****************************************************************/
    if (preccReturn->preccFirstChild)
    {
      preccReturn = TRVFindLast(preccReturn);
    }
  }
  return (preccReturn);
}


/*********************************************************************/
/* FUNCTION NAME:  TRVMoveCursor                                     */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR TRVMoveCursor(hwnd, hps, preccNewCursor)
HANDLE         hwnd;
HDC          hps;
LPRECORDCOREN preccNewCursor;
{
  LPRECORDCOREN    preccOldCursor;
  LPCNRINFON       pccinfon;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  preccOldCursor = pccinfon->preccCursor;
  pccinfon->preccCursor = preccNewCursor;

  if (preccOldCursor != pccinfon->preccCursor)
  {
    if (!SetEmphasisBits (hwnd, hps, preccOldCursor,
                          CRA_CURSORED | CRA_SELECTED, SEB_CLEAR))
    {
      return (FALSE);
    }

    if (!SetEmphasisBits (hwnd, hps, pccinfon->preccCursor,
                          CRA_CURSORED | CRA_SELECTED, SEB_SET))
    {
      return (FALSE);
    }
  }
  return TRUE;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  TRVFindPirs                                      */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Find LPICONRESIZED for The Record            */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function searchs for the LPICONRESIZED in the pirsMetrics*/
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
LPICONRESIZED FAR TRVFindPirs(hwnd, preccObject)
HANDLE         hwnd;
LPRECORDCOREN preccObject;

{
  BOOL           found = FALSE;
  HICON          hptrFind = NULL;
  LPCNRINFON     pccinfon;
  LPICONRESIZED  pirsCurrent;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  /*******************************************************************/
  /* If looking for the regular size, search in pirsMetrics and ...  */
  /*******************************************************************/
  pirsCurrent = pccinfon->pirsMetrics;
  if ((preccObject->flRecordAttr & CRA_EXPANDED) &&
       TRVhasVisibleChild (preccObject))
  {
    /*****************************************************************/
    /* If preccObject is expanded get hprExpanded.                   */
    /*****************************************************************/
    if (preccObject->preccX->pTreeItemDesc)
    {
      hptrFind = preccObject->preccX->pTreeItemDesc->hptrExpanded;
    }
  }
  else
  {
    if ((preccObject->flRecordAttr & CRA_COLLAPSED) &&
        TRVhasVisibleChild (preccObject))
    {
      /***************************************************************/
      /* If preccObject is collapsed get hprExpanded.                */
      /***************************************************************/
      if (preccObject->preccX->pTreeItemDesc)
      {
        hptrFind = preccObject->preccX->pTreeItemDesc->hptrCollapsed;
      }
    }
    else
    {
      /***************************************************************/
      /* If preccObject is Leaf,     get hprIcon.                    */
      /***************************************************************/
      hptrFind = preccObject->preccX->hptrIcon;
    }
  } /* endif */

  /*******************************************************************/
  /* Search for the hptrFind in the metrics.                         */
  /*******************************************************************/
  while (hptrFind && pirsCurrent)
  {
    if (pirsCurrent->hptrOrigin == hptrFind)
    {
      found = TRUE;
      return pirsCurrent;
    }
    pirsCurrent = pirsCurrent->pirsNext;
  }

  if (!found && hptrFind)
  {
    if (!(CnrTRVInsDelResizedIcon (hwnd,
                                   preccObject->preccX,
                                   TRUE)))
    {
       return ((LPICONRESIZED)NULL);
    }

    return TRVFindPirs(hwnd, preccObject);
  } /* endif */
  return 0;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  TRVFindTreePirs                                  */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Find LPICONRESIZED for The Record            */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function searchs for the LPICONRESIZED in the            */
/*     pirsTreeMetrics.                                              */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
LPICONRESIZED FAR TRVFindTreePirs(hwnd, preccObject)
HANDLE         hwnd;
LPRECORDCOREN preccObject;

{
  BOOL           found = FALSE;
  HICON          hptrFind;
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPICONRESIZED  pirsCurrent;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* If looking for the regular size, search in pirsMetrics and ...  */
  /*******************************************************************/
  pirsCurrent = pccinfon->pirsTreeMetrics;
  if (preccObject->flRecordAttr & CRA_EXPANDED)
  {
    /*****************************************************************/
    /* If preccObject is expanded get hprExpanded.                   */
    /*****************************************************************/
    hptrFind = pccinfo->hptrExpanded;
  }
  else
  {
    if (preccObject->flRecordAttr & CRA_COLLAPSED)
    {
      /***************************************************************/
      /* If preccObject is collapsed get hprExpanded.                */
      /***************************************************************/
      hptrFind = pccinfo->hptrCollapsed;
    }
  } /* endif */

  /*******************************************************************/
  /* Search for the hptrFind in the metrics.                         */
  /*******************************************************************/
  while (hptrFind && pirsCurrent)
  {
    if (pirsCurrent->hptrOrigin == hptrFind)
    {
      found = TRUE;
      return pirsCurrent;
    }
    pirsCurrent = pirsCurrent->pirsNext;
  }

  if (!found && hptrFind)
  {
    if (!(CnrTRVInsDelResizedTreeIcon (hwnd, TRUE)))
    {
       return ((LPICONRESIZED)NULL);
    }

    return TRVFindTreePirs(hwnd, preccObject);
  }
  return 0;
}

/**********************************************************************/
/* Procedure: IsTreeVisible  - This procedure will check to see if    */
/*            tree is visible(not filtered out and not collapsed)     */
/*            up to the root for the current record.                  */
/*            If so then the record is visible in the tree and the    */
/*            routine will return TRUE.                               */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
BOOL FAR IsTreeVisible (LPRECORDCOREN precord)
{

  if (precord)
  {
    /******************************************************************/
    /* Now search for parent and if one, check if it is expanded and  */
    /* not filtered out.                                              */
    /******************************************************************/
    precord = precord->preccParent;
    while (precord)
    {
      if (!((precord->flRecordAttr & CRA_EXPANDED) &&
            ISRECORDVISIBLE(precord)) )
      {
        return FALSE;
      }
      precord = precord->preccParent;
    }
    return TRUE;
  }
  else
  {
    return FALSE;
  }
}

/**********************************************************************/
/* Procedure: TRVhasVisibleChild                                      */
/*                Checks if the record has a visible child.           */
/*            If it has a visible child, the function returns TRUE.   */
/*                                                                    */
/*                                                                    */
/* Inputs:                                                            */
/*                                                                    */
/* Output:                                                            */
/**********************************************************************/
BOOL FAR TRVhasVisibleChild (LPRECORDCOREN precord)
{
  precord = precord->preccFirstChild;

  /********************************************************************/
  /* Now enumerate the children and if a visible child is found       */
  /* return TRUE.                                                     */
  /********************************************************************/
  while (precord)
  {
    if (ISRECORDVISIBLE(precord))
    {
      return TRUE;
    }
    precord = precord->preccNextItem;
  }

  return (FALSE);
}
