/* Revision: 25 1.2.2.2 source/ui/cclnt/cnr/cnrwin.c, cclcnr, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*              Licensed Materials - Property of IBM                  */
/*              20G0400 (C) Copyright IBM Corp. 1992, 1997.           */
/*                    See Copyright Instructions.                     */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/* CUA Controls Library/2 Version 1.0                       (Windows) */
/*                                                                    */
/* Sample Application Code                                            */
/* Container Control                                                  */
/*                                                                    */
/* File:  cnrwin.c                                                    */
/*                                                                    */
/* This application creates a large container window holding various  */
/* people and departments.  The user may open a department for        */
/* viewing by double-clicking on the appropriate icon.  People and    */
/* departments may be dragged within the same window, to another open */
/* department, or onto an unopened department icon.                   */
/*                                                                    */
/**********************************************************************/

#define OEMRESOURCE
#include <windows.h>
#include <stdio.h>                     /* sprintf                     */
#include <stdlib.h>                    /* itoa, atoi                  */
#include <string.h>                    /* lstrlen                     */
#include "iclcnrw.h"                   /* container include...        */
#include "cnrwin.h"                    /* private application include */
                                       /* file                        */

  /********************************************************************/
  /* globals                                                          */
  /********************************************************************/

static char szClientClass[] = "Container Sample Application";
static HICON hptrIcons[2];             /* icons for departments and   */
                                       /* people                      */
static HICON hptrMiniIcons[2];         /* mini icons                  */
static HBITMAP hbmTree[2];             /* tree view bitmaps           */
HANDLE vhInstance;                     /* global instance             */

  /********************************************************************/
  /* These menu names must correspond with the menu id's and the      */
  /* function calls in viewFunctions                                  */
  /********************************************************************/

static char *menuNames[] =
{
  NULL,          "Icon view",   "Details view",
  "Name view",   "Text view",   "Tree icon view"
} ;

  /********************************************************************/
  /* the following array contains pointers to constructors and        */
  /* destructors for the various views                                */
  /********************************************************************/

static VOID(*(viewFunctions[]))(HWND) =
{
  createIconView,      destroyIconView,   createDetailsView,
  destroyDetailsView,  createNameView,    destroyNameView,
  createTextView,      destroyTextView,   createTreeIconView,
  destroyTreeIconView,
} ;

  /********************************************************************/
  /* The following string identifies drag objects belonging to this   */
  /* application                                                      */
  /********************************************************************/

static char *application_name = "Container Demo";

  /********************************************************************/
  /* The following are the titles of the details view columns         */
  /********************************************************************/

static char *fieldNames[] =
{
  NULL,          "Name",        "Task",        "Nickname",
  "Social Security #",          "Hourly Wage", "Employees"
} ;

#define  COUNT_FIELDS  (sizeof (fieldNames) / sizeof (fieldNames [0]))

  /********************************************************************/
  /* The following array defines the objects in our company. The      */
  /* number of objects is static...objects cannot be added while the  */
  /* program is executing. In a real application, this list would be  */
  /* implemented dynamically as a linked list. the default [top       */
  /* level] department must be defined before any of its descendants. */
  /********************************************************************/

static DATAINFO dataObjects[] =
{
  /********************************************************************/
  /* type, name, task, nickname, parent, social security, hourly wage */
  /********************************************************************/

  {
    ITEM_TYPE_DEPT,
    "XYZ Software Development Corporation\r\n Developing Interfaces, \
       not just code",
    "",
    "",
    0,                                 // [0]
    NULL,
    0,
    0
  } ,
  {
    ITEM_TYPE_DEPT,                    // [1]
    "PM Extensions\r\nCary, NC",
    "OS/2 and Windows CUA Controls",
    "",
    0,
    NULL,
    0,
    0
  } ,
  {
    ITEM_TYPE_DEPT,                    // [2]
    "Marketing",
    "",
    "",
    0,
    NULL,
    0,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [3]
    "Johnny Bluebird",
    "",
    "",
    0,
    "153-53-4224",
    1534,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [4]
    "Laura Green",
    "",
    "",
    0,
    "790-83-5329",
    2125,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [5]
    "Harvey Smathers",
    "",
    "",
    0,
    "331-91-9103",
    850,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [6]
    "Denise Slater",
    "",
    "",
    0,
    "955-32-1436",
    2330,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [7]
    "Harry Sledgehammer",
    "",
    "",
    2,
    "991-32-4252",
    3263,
    0
  } ,
  {
    ITEM_TYPE_DEPT,                    // [8]
    "Quality Assurance",
    "",
    "",
    2,
    NULL,
    0,
    0
  } ,
  {
    ITEM_TYPE_DEPT,                    // [9]
    "Planning",
    "",
    "",
    2,
    NULL,
    0,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [10]
    "Ronald Walters",
    "",
    "",
    8,
    "132-82-1150",
    1550,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [11]
    "Gilligan Williams",
    "",
    "",
    8,
    "759-91-2032",
    1812,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [12]
    "Larry Roberts\r\nOffice Manager",
    "",
    "",
    0,
    "790-27-5649",
    4500,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [13]
    "Tom Swifte\r\nVice President\r\nSales",
    "",
    "",
    0,
    "831-44-6218",
    4325,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [14]
    "Brenda Singleton\r\nTraining Manager",
    "",
    "",
    0,
    "012-75-3829",
    3810,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [15]
    "Mark Starr\r\nSafety Resources Manager",
    "",
    "",
    0,
    "874-02-9214",
    2560,
    0
  } ,
  {
    ITEM_TYPE_DEPT,                    // [16]
    "Development",
    "",
    "",
    9,
    NULL,
    0,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [17]
    "Janet Robart",
    "",
    "",
    16,
    "329-82-1953",
    1825,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [18]
    "Henry Leverton",
    "",
    "",
    2,
    "631-73-2461",
    1960,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [19]
    "Mark Benge",
    "Windows Controls",
    "Banzai",
    1,
    "135-79-2468",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [20]
    "David Bernath",
    "Valueset,\r\nSlider,\r\nContainer,\r\nFile Dialog,\r\nFontDialog",
    "Mr. Dew it",
    1,
    "246-80-1357",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [21]
    "Peter Brightbill",
    "Container",
    "Billbright",
    1,
    "555-55-5555",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [22]
    "Teresa Geohagan",
    "QuickHelp",
    "Blondie",
    1,
    "445-44-4455",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [23]
    "Peter Haggar",
    "Container",
    "Golfman",
    1,
    "123-45-6789",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [24]
    "Jon Holliday",
    "Windows Controls",
    "Mr. Windows",
    1,
    "222-22-2222",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [25]
   "Morgan Kinne",
    "Department Tech Lead",
    "Uncle Morgie",
    1,
    "222-22-2222",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [26]
    "Diana Mack",
    "Notebook",
    "I'm a little teapot",
    1,
    "543-21-2345",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [27]
    "Tai Woo Nam",
    "Container",
    "Dr. Woo",
    1,
    "987-65-4321",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [28]
    "Jeff Robison",
    "Direct Manipulation",
    "Little Jeffy",
    1,
    "777-77-7777",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [29]
    "Ruth Anne Taylor",
    "Container",
    "RAT",
    1,
    "901-23-4567",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [30]
    "Tim Wolf",
    "Wolfman",
    "",
    1,
    "345-67-8888",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [31]
    "Sandeep K. Singhal",
    "Sample Applications",
    "",
    1,
    "444-44-4444",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [32]
    "Tom Weyer",
    "Demos",
    "Mr. IU",
    1,
    "999-99-9999",
    58,
    0
  } ,
  {
    ITEM_TYPE_PERSON,                  // [33]
    "Steve Robinson",
    "Boss",
    "The Chief",
    0,
    "333-33-3333",
    85,
    0
  } ,
} ;

#define  COUNT_OBJECT (sizeof(dataObjects) / sizeof(dataObjects[0]))
#define  INDEX_CONTAINER_TOP 0
#define  INDEXFIRST (1 + INDEX_CONTAINER_TOP)

INT APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR lpCmdLine, int nCmdShow );

#if defined(__IBMC__)
int main( int argc, char** argv )
{
   HANDLE hInstance = GetModuleHandle(0);
   return WinMain( hInstance, 0, 0, SW_SHOWNORMAL );
}
#endif

/**********************************************************************/
/*  Function:  WinMain                                                */
/*  Inputs:    hInstance        instance handle of execution          */
/*             hPrevInstance    instance handle of previous execution */
/*             lpCmdLine        command line arguments                */
/*             nCmdShow         startup mode (iconized/maximized)     */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function registers the window class and contains the message */
/*  loop. It loads the application icons and cursors and is           */
/*  responsible for message dispatching.                              */
/**********************************************************************/
INT APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR lpCmdLine, int nCmdShow )

  /********************************************************************/
  /* windows application entry point                                  */
  /********************************************************************/

{
  HANDLE    hLibrary;
  HWND      hwnd;                      /* client window handle        */
  MSG       msg;                       /* message number              */
  WNDCLASS  wndClass;                  /* window class for            */
                                       /* registration                */
  HBRUSH    hBrush;                    /* brush for client window     */
  static char szClientClass[] = "Container Sample Application";
  POINT       screenSize;              /* screen size                 */

  lpCmdLine;                           /* prevent unused warning      */

  /********************************************************************/
  /* register window class for client...a basic window with no fancy  */
  /* aspects...we use a colored brush for its background              */
  /********************************************************************/
  if (!hPrevInstance)
  {
    /******************************************************************/
    /* create a background brush                                      */
    /******************************************************************/
    hBrush = CreateSolidBrush( COLOR_CLIENT_BACKGROUND );
    wndClass.lpszClassName = (LPCSTR)szClientClass;
    wndClass.hInstance     = hInstance;
    wndClass.lpfnWndProc   = ClientWndProc;
    wndClass.hCursor       = LoadCursor( 0, IDC_ARROW);
    wndClass.hIcon         = LoadIcon( 0, IDI_APPLICATION);
    wndClass.lpszMenuName  = MAKEINTRESOURCE(ID_CLIENT_RESOURCES);
    wndClass.hbrBackground = hBrush;
    wndClass.style         = (UINT)(CS_HREDRAW | CS_VREDRAW);
    wndClass.cbClsExtra    = 0;
    wndClass.cbWndExtra    = WINDATA_LENGTH;

    RegisterClass (&wndClass);
  }

  /********************************************************************/
  /* register the container control                                   */
  /********************************************************************/
  if (!(hLibrary = LoadLibrary(ICL_CONTAINER_DLL)))
    return 0;

  screenSize.x = GetSystemMetrics (SM_CXFULLSCREEN); /* Get screen   */
  screenSize.y = GetSystemMetrics (SM_CYFULLSCREEN); /* size         */

  /********************************************************************/
  /* create a big client window                                       */
  /********************************************************************/
  hwnd = CreateWindow (szClientClass,          /* window class        */
                       "Personnel Records",    /* window title        */
                       WS_OVERLAPPEDWINDOW,    /* wnd style           */
                       CW_USEDEFAULT,          /* x position          */
                       CW_USEDEFAULT,          /* y position          */
                       (screenSize.x < 788)    /* cx size             */
                           ?  screenSize.x
                           :  788,
                        (screenSize.y < 500)   /* cy size             */
                           ?  screenSize.y
                           :  500,
                       NULL,                   /* no parent           */
                       NULL,                   /* class menu          */
                       hInstance,              /* creator             */
                       NULL);                  /* params              */

  /********************************************************************/
  /* load icons and bitmaps for objects                               */
  /********************************************************************/
  hptrIcons[0] = LoadIcon (hInstance, MAKEINTRESOURCE(ID_ICON_PERSON));
  hptrIcons[1] = LoadIcon (hInstance, MAKEINTRESOURCE(ID_ICON_DEPT));

  hptrMiniIcons[0] = LoadIcon (hInstance, MAKEINTRESOURCE(ID_ICON_PERSMINW));
  hptrMiniIcons[1] = LoadIcon (hInstance, MAKEINTRESOURCE(ID_ICON_DEPTMINW));

  hbmTree[0] = LoadBitmap (NULL, MAKEINTRESOURCE(OBM_REDUCE));
  hbmTree[1] = LoadBitmap (NULL, MAKEINTRESOURCE(OBM_ZOOM));

  /********************************************************************/
  /* create top-level container                                       */
  /********************************************************************/
  vhInstance = hInstance;

  if (!(createContainer (hwnd, INDEX_CONTAINER_TOP)))
     return FALSE;

  SendMessage (hwnd,                 /* have client initialize itself */
               WM_CLIENT_INIT,
               0,                                                       /*DAB1*/
               (LONG)INDEX_CONTAINER_TOP);


  ShowWindow (hwnd, nCmdShow);
  UpdateWindow (hwnd);

  /********************************************************************/
  /* message loop                                                     */
  /********************************************************************/

  while (GetMessage (&msg, NULL, 0, 0 ))
  {
    TranslateMessage (&msg);           /* keyboard interface          */
    DispatchMessage (&msg);            /* to client window procedure  */
  }                                    /* end of while loop           */

  FreeLibrary (hLibrary);

  /********************************************************************/
  /* deallocate brush resource                                        */
  /********************************************************************/

  if (!hPrevInstance)
  {
    DeleteObject (hbmTree[0]);
    DeleteObject (hbmTree[1]);
    DestroyIcon (hptrIcons[0]);
    DestroyIcon (hptrIcons[1]);
    DestroyIcon (hptrMiniIcons[0]);
    DestroyIcon (hptrMiniIcons[1]);
  }

  return (msg.wParam);
}                                      /* end of winMain              */




/**********************************************************************/
/*  Function:  ClientWndProc                                          */
/*  Inputs:    hwnd                 client window handle              */
/*             msg                  message id                        */
/*             mp1                  first parameter                   */
/*             mp2                  second parameter                  */
/*  Outputs:   return values appropriate to message                   */
/*                                                                    */
/*  This function processes messages sent to the client window,       */
/*  including system messages and messages from controls.             */
/**********************************************************************/
LRESULT CALLBACK ClientWndProc (HWND hwnd, UINT msg,
                                WPARAM wParam, LPARAM lParam)
{
  /********************************************************************/
  /* The following structure defines the menu items in the system     */
  /* menu                                                             */
  /********************************************************************/
  static WORD fMenuItems[] =
  {
      MF_BYPOSITION | MF_SEPARATOR,
      MF_BYPOSITION | MF_STRING | MF_CHECKED,
      MF_BYPOSITION | MF_STRING,
      MF_BYPOSITION | MF_STRING,
      MF_BYPOSITION | MF_STRING,
      MF_BYPOSITION | MF_STRING
  };

  static WORD idMenuItems[] =
  {
     0,
     IDM_SUBICON,
     IDM_SUBDETAIL,
     IDM_SUBNAME,
     IDM_SUBTEXT,
     IDM_SUBTREEICON
  };

  switch (msg)
  {

    /******************************************************************/
    /* WM_CREATE is sent when the window is created. We get and store */
    /* the window size and add items to the system menu.              */
    /******************************************************************/

    case WM_CREATE:
      {
        RECT  rclWindowSize;          /* size of client window        */
        HWND  hwndSysMenu;            /* handle of system menu bitmap */
        int   nMenuItemIndex;         /* loop index                   */

        /**************************************************************/
        /* get and store initial window size in window longs          */
        /**************************************************************/

        GetClientRect (hwnd, &rclWindowSize);
        SetWindowLong (hwnd,
                       WINDATA_SIZE,
                       MAKELONG(rclWindowSize.right, rclWindowSize.bottom));

        /**************************************************************/
        /* now, since we only have a few menu items, add them to the  */
        /* system menu so the menu bar does not eat up space Get the  */
        /* system bitmap handle                                       */
        /**************************************************************/

        hwndSysMenu = GetSystemMenu (hwnd, FALSE);

        /**************************************************************/
        /* insert the menu items in the system sub menu               */
        /**************************************************************/

        for (nMenuItemIndex = 0; nMenuItemIndex < 6; nMenuItemIndex++)
        {
          InsertMenu (hwndSysMenu,
                      0xFFFFFFFF,                                       /*DAB1*/
                      fMenuItems[nMenuItemIndex],
                      idMenuItems[nMenuItemIndex],
                      menuNames[nMenuItemIndex]);
        }
      }                                /* end of case WM_CREATE       */
      break;


    /******************************************************************/
    /* The WM_CLIENT_INIT is a user-defined message. It is sent to    */
    /* the client when all external initialization of the window is   */
    /* complete; by this point, the container has been created and    */
    /* filled. All that remains is for the client window to store     */
    /* its pointers and attributes for later: wParam = parent         */
    /* container handle lParam = index of object client container     */
    /* represents                                                     */
    /******************************************************************/

    case WM_CLIENT_INIT:
      initClient (hwnd,                           /* client window    */
                  GetDlgItem(hwnd, ID_CONTAINER), /* client container */
                  (HWND)wParam,                   /* parent container */
                  LOWORD(lParam));                /* item index       */
      break;


    /******************************************************************/
    /* The WM_SIZE message is received whenever the window size       */
    /* changes. lParam contains the new size (length in low word,     */
    /* height in high word). We extract the new size, store it, and   */
    /* adjust the container window to fit the client                  */
    /******************************************************************/

    case WM_SIZE:
      {
        WORD wWinXLen;               /* length of window              */
        WORD wWinYLen;               /* height of window              */

        /**************************************************************/
        /* get window size from message parameters                    */
        /**************************************************************/

        wWinXLen = LOWORD(lParam);
        wWinYLen = HIWORD(lParam);

        /**************************************************************/
        /* store window size                                          */
        /**************************************************************/

        SetWindowLong(hwnd, WINDATA_SIZE, MAKELONG(wWinXLen, wWinYLen));

        /**************************************************************/
        /* move container                                             */
        /**************************************************************/

        MoveWindow (GetDlgItem (hwnd, ID_CONTAINER),
                    CNR_MARGIN_XLEFT,
                    CNR_MARGIN_YTOP,
                    (int)(wWinXLen
                            - CNR_MARGIN_XLEFT
                            - CNR_MARGIN_XRIGHT),
                    (int)(wWinYLen
                            - CNR_MARGIN_YBOTTOM
                            - CNR_MARGIN_YTOP),
                    TRUE);
      }                                /* end of case WM_SIZE         */
      break;

    /******************************************************************/
    /* The WM_SETFOCUS message is received whenever the window is     */
    /* receiving the focus.  In certain scenarios, the parent will be */
    /* the only window receiving the focus.  In order to ensure that  */
    /* the control receives the focus, you must issue a SetFocus.     */
    /******************************************************************/

    case WM_SETFOCUS:
      {
        HWND hwndCnr;              /* hwnd of this window's container */

        if ((hwndCnr = GetDlgItem(hwnd, ID_CONTAINER)))
          SetFocus (hwndCnr);
      }                                /* end of case WM_SETFOCUS     */
      break;


  /********************************************************************/
  /* The client window receives a WM_COMMAND message to inform it     */
  /* of important events in the child windows. wParam                 */
  /* is the child window id, and high word of lParam is the           */
  /* notification code                                                */
  /********************************************************************/

  case WM_COMMAND:
    switch (HIWORD(wParam))                                             /*DAB1*/
    {
        /**************************************************************/
        /* The CN_ENTER message is sent by the container when the     */
        /* user double-clicks the mouse button or presses the         */
        /* enter key. lParam points to the recordCore structure over  */
        /* which action occurred. This message indicates that we      */
        /* should open the item, if it is a department.               */
        /**************************************************************/

        case CN_ENTER:
          switch (LOWORD(wParam))                                       /*DAB1*/
          {
          /************************************************************/
          /* based on ID of message                                   */
          /************************************************************/
            case ID_CONTAINER:
            {
              WORD                 wIndex;   /* index of item to open */
              LPNOTIFYRECORDENTER  lpNotify;


              /********************************************************/
              /* get the notify structure given the hwndCnr           */
              /********************************************************/
              lpNotify = (LPNOTIFYRECORDENTER)lParam;                   /*DAB1*/

              /********************************************************/
              /* if nothing was clicked upon then return              */
              /********************************************************/
              if (lpNotify->pRecord == NULL)
                break;

              /********************************************************/
              /* first locate record index for selected item          */
              /********************************************************/

              wIndex = getRecordIndex(lpNotify->pRecord);

              /********************************************************/
              /* if it is a department, then create new child         */
              /* window for its container; the newLevel function      */
              /* requires the current window handle, the index of     */
              /* the item to open, and the item's parent              */
              /********************************************************/

              if ((dataObjects[wIndex].wItemType) == ITEM_TYPE_DEPT)
              {
                newLevel (hwnd,
                          wIndex,
                          (WORD)GetWindowLong(hwnd, WINDATA_ITEM_INDEX));
              }

            }                            /* end of case ID_CONTAINER  */
            break;

          default:                       /* other messages            */
            break;
          }                              /* end of case CN_ENTER      */
          break;

        /**************************************************************/
        /* The CN_REALLOCPSZ message is sent by the container         */
        /* when direct edit is completing on an icon title. lParam    */
        /* points to a CNREDITDATA field. We must allocate            */
        /* memory for the new string; return TRUE if memory is        */
        /* available                                                  */
        /**************************************************************/

        case CN_REALLOCPSZ:
        {
          LPCNREDITDATA    lpEditData;

          /************************************************************/
          /* get the edit data structure given the hwndCnr            */
          /************************************************************/
          lpEditData = (LPCNREDITDATA)lParam;                           /*DAB1*/

          switch (lpEditData->id)
          {
            case ID_CONTAINER:
              return ((DWORD)(handleReallocPsz (lpEditData,
                                                hwnd,
                                                FALSE,
                                                FALSE)));
              break;

            /**********************************************************/
            /* message from container title-window                    */
            /**********************************************************/

            case CID_CNRTITLEWND:
              return ((DWORD)(handleReallocPsz (lpEditData,
                                                hwnd,
                                                TRUE,
                                                FALSE)));
              break;

            /**********************************************************/
            /* message from container details window                  */
            /**********************************************************/

            case CID_LEFTDVWND:
            case CID_RIGHTDVWND:
              return ((DWORD)(handleReallocPsz (lpEditData,
                                                hwnd,
                                                FALSE,
                                                TRUE)));
              break;

            default:                   /* other messages              */
              break;
          }                            /* end of switch CN_REALLOCPSZ */
          break;
        }

        default:                       /* other messages              */
          break;

    }                          /* end of switch on WM_COMMAND */
    break;


  /********************************************************************/
  /* The WM_DRAWITEM message is sent to a window when it must         */
  /* draw an object specified with the owner draw attribute. lParam   */
  /* points to a DRAWITEMSTRUCT structure                             */
  /********************************************************************/

  case WM_DRAWITEM:
    {
        /**************************************************************/
        /* switch on origin of message                                */
        /**************************************************************/

        switch (((LPDRAWITEMSTRUCT)lParam)->CtlID)
        {
          /************************************************************/
          /* messages are sent from the container, its title window,  */
          /* details view windows, etc. The id in wParam represents   */
          /* the container itself. The id of the window in pOwnerItem */
          /* specifies the actual sub-window                          */
          /************************************************************/

          case ID_CONTAINER:
            return ((long)drawContainerItem((LPDRAWITEMSTRUCT)lParam));
            break;

          default:                     /* other sources of            */
                                       /* WM_DRAWITEM messages        */
            break;
        }                              /* end of switch on origin of  */
                                       /* WM_DRAWITEM message         */
        break;
    }                          /* end of case: WM_DRAWITEM    */
    break;


    /******************************************************************/
    /* The WM_COMMAND message is sent to the client window to         */
    /* inform it of significant events; wParam is the id              */
    /* of the message source                                          */
    /******************************************************************/

    case WM_SYSCOMMAND:
      switch (wParam)
      {
        /**************************************************************/
        /* if a new view selected                                     */
        /**************************************************************/

        case IDM_SUBICON:
        case IDM_SUBDETAIL:
        case IDM_SUBNAME:
        case IDM_SUBTEXT:
        case IDM_SUBTREEICON:
          {
            HWND hwndMenu;

            /**********************************************************/
            /* get old view                                           */
            /**********************************************************/

            int nOldView = (int)GetWindowLong (hwnd, WINDATA_CURR_VIEW);

            /**********************************************************/
            /* get new view from parameter                            */
            /**********************************************************/

            int nNewView = wParam;

            /**********************************************************/
            /* if view not changing, then forget it                   */
            /**********************************************************/

            if (nOldView == nNewView)
              break;

            /**********************************************************/
            /* destroy current view by calling destructor function    */
            /**********************************************************/

            (viewFunctions[2 * (nOldView - ID_FIRSTVIEWITEM) + 1])
               (GetDlgItem (hwnd, ID_CONTAINER));

            /**********************************************************/
            /* create new view by calling constructor function        */
            /**********************************************************/

            (viewFunctions[2*(nNewView - ID_FIRSTVIEWITEM)])
               (GetDlgItem (hwnd, ID_CONTAINER));

            /**********************************************************/
            /* get system menu handle                                 */
            /**********************************************************/

            hwndMenu = GetSystemMenu (hwnd, FALSE);

            /**********************************************************/
            /* finally uncheck old view, check new view, and and      */
            /* store new view                                         */
            /**********************************************************/

            CheckMenuItem (hwndMenu,
                           nOldView,
                           MF_BYCOMMAND | MF_UNCHECKED);

            CheckMenuItem (hwndMenu,
                           nNewView,
                           MF_BYCOMMAND | MF_CHECKED);

            SetWindowLong (hwnd, WINDATA_CURR_VIEW, (long)nNewView);
          }                            /* end of new view selected    */
          break;

        default:                       /* other commands              */
          break;
      }                                /* end of switch: WM_COMMAND   */
      break;


    /******************************************************************/
    /* We don't really have to do much painting since the container   */
    /* does it all for us. Just fill the background                   */
    /******************************************************************/

    case WM_PAINT:
      {
        PAINTSTRUCT ps;                /* invalid region              */

        BeginPaint (hwnd, &ps);
        EndPaint (hwnd, &ps);
      }
      break;


    /******************************************************************/
    /* The WM_CLOSE message is sent when the user tries to close      */
    /* the window by selecting close from the system menu. If this    */
    /* is allowed to continue, the default window procedure           */
    /* automatically closes down the entire application. However,     */
    /* we only want to allow that if this is the top level window     */
    /******************************************************************/

    case WM_CLOSE:
      {
        WORD wIndex;                   /* index of this container     */

        /**************************************************************/
        /* figure out which container this is                         */
        /**************************************************************/

        wIndex = (WORD)GetWindowLong (hwnd, WINDATA_ITEM_INDEX);

        /**************************************************************/
        /* If the system menu wants to shut down the application,     */
        /* then only give in if we are the top-level window. If we    */
        /* are a child client with a container, we simply destroy     */
        /* ourselves-- returning control to the parent.               */
        /**************************************************************/

        if (wIndex != INDEX_CONTAINER_TOP)
        {
          DestroyWindow (hwnd);
          return (FALSE);
        }                              /* end of if we are the top    */
                                       /* level window                */

        /**************************************************************/
        /* otherwise, if the application will be closing, just        */
        /* proceed as usual                                           */
        /**************************************************************/

      }                                /* end of WM_CLOSE processing  */
      break;


    /******************************************************************/
    /* The WM_DESTROY message is sent when the client window is       */
    /* about to be removed from memory. It is no longer visible and   */
    /* the child container has already been destroyed. Our only       */
    /* remaining task is to restore ourselves within our parent       */
    /* container                                                      */
    /******************************************************************/

    case WM_DESTROY:
      {
        WORD wIndex;                   /* index of this container     */

        destroyClient (hwnd, GetDlgItem (hwnd, ID_CONTAINER));

        /**************************************************************/
        /* figure out which container this is                         */
        /**************************************************************/

        wIndex = (WORD)GetWindowLong (hwnd, WINDATA_ITEM_INDEX);

        /**************************************************************/
        /* If the system menu wants to shut down the application,     */
        /* then only give in if we are the top-level window. If we    */
        /* are the top-level window, post a quit messge to end the    */
        /* sample application.                                        */
        /**************************************************************/

        if (wIndex == INDEX_CONTAINER_TOP)
        {
          PostQuitMessage (0);
        }
      }                                /* end of WM_CLOSE processing  */
      break;


    default:
      break;
  }                                    /* endswitch                   */

  return (DefWindowProc (hwnd, msg, wParam, lParam));
}                                      /* end of client window        */
                                       /* procedure                   */



/**********************************************************************/
/*  Function:  newLevel                                               */
/*  Inputs:    hwndParent           parent window handle              */
/*             wNewIdx              index of item to open             */
/*             wStartFromIdx        current window id                 */
/*  Outputs:   window handle of client window representing newIdx item*/
/*                                                                    */
/*  This function is called when a container must be created for the  */
/*  newIdx item.  It checks whether the parent container has yet been */
/*  opened; if not, then newLevel is called recursively to open the   */
/*  parent window first.  Next, a frame window is created for the     */
/*  given item, a container is created and filled for it, and a       */
/*  CLIENT_INIT message is sent to complete initialization.           */
/**********************************************************************/

HWND newLevel (HWND hwndParent, WORD wNewIdx, WORD wStartFromIdx)
{

  /********************************************************************/
  /* from a client window, create a new client window in which to put */
  /* a container; the newId variable is the index of the object       */
  /* within the dataObjects array. It is converted into a container   */
  /* id by using the IDFROMINDEX macro. Return handle of new client   */
  /* window.                                                          */
  /********************************************************************/

  /********************************************************************/
  /* note that since we have no max/min buttons on the frame, the     */
  /* system menu only allows move, size, close, and switch            */
  /********************************************************************/

  HWND  hwndChild;
  RECT  rclParent;
  DWORD dwStyle;

  /********************************************************************/
  /* if parent container does not yet exist, then create it           */
  /* recursively                                                      */
  /********************************************************************/

  if (dataObjects[wNewIdx].wParentIdx != wStartFromIdx)
  {
    hwndParent = newLevel (hwndParent,
                           dataObjects[wNewIdx].wParentIdx,
                           wStartFromIdx);
  }

  /********************************************************************/
  /* get parent window size                                           */
  /********************************************************************/

  GetWindowRect (hwndParent, &rclParent);

  /********************************************************************/
  /* create a client window for the new item                          */
  /********************************************************************/

  dwStyle = WS_OVERLAPPEDWINDOW;            /* window style           */

  /********************************************************************/
  /* make window size a function of parent window; make sure window   */
  /* upper-left is in upper-left quadrant of parent window (so its    */
  /* title bar does not extend beyond the top of the client window;   */
  /* note that WIN_RATIO_X and WIN_RATIO_Y are integer fractions.     */
  /********************************************************************/

  hwndChild =
     CreateWindow (szClientClass,
                   (LPSTR)NULL,                      /* titlebar text */
                   dwStyle,                          /* style flags   */
                   rclParent.left,                   /* x position    */
                   rclParent.top,                    /* y position    */
                   (rclParent.right * WIN_RATIO_X),
                                                     /* cx size       */
                   (rclParent.bottom * WIN_RATIO_Y),
                                                     /* cy size       */
                   hwndParent,                       /* parent window */
                   NULL,                             /* menu          */
                   vhInstance,                       /* Instance      */
                   NULL);                            /* param -lParam */

  /********************************************************************/
  /* build a child container for the object                           */
  /********************************************************************/

  if (!(createContainer (hwndChild, wNewIdx)))
     return FALSE;

  /********************************************************************/
  /* and tell the client that initialization is complete              */
  /********************************************************************/

  SendMessage (hwndChild,
               WM_CLIENT_INIT,
               (WPARAM)GetDlgItem(hwndParent, ID_CONTAINER),            /*DAB1*/
               (DWORD)wNewIdx);
  ShowWindow (hwndChild, SW_SHOW);
  return (hwndChild);
}                                      /* end of function newLevel    */



/**********************************************************************/
/*  Function:  handleReallocPsz                                       */
/*  Inputs:    lpCnrEditData   pointer to container edit notification */
/*                             structure                              */
/*             hwnd            window handle                          */
/*             bEditTitle      TRUE if container title being edited;  */
/*                             FALSE otherwise                        */
/*             bEditDetails    TRUE if details view column being      */
/*                             edited; FALSE otherwise                */
/*  Outputs:   TRUE if memory available, FALSE otherwise              */
/*                                                                    */
/*  This function is called when the client window receives a         */
/*  CN_REALLOCPSZ message from the container window.  It attempts to  */
/*  allocate memory for the new string and set the internal pointers  */
/*  to the new text;  if it is successful, TRUE is returned.          */
/**********************************************************************/

BOOL handleReallocPsz (LPCNREDITDATA lpCnrEditData, HWND hwnd,
                       BOOL bEditTitle, BOOL bEditDetails)
{
  char *pNewString;                    /* new text memory             */
  WORD  wIndex;                        /* item being edited           */

  /********************************************************************/
  /* see if we can allocate space for new string try allocating       */
  /* space: lpCnrEditData->cbText is the length of the new string     */
  /********************************************************************/

  if (pNewString = (char *)malloc(sizeof(char)*(lpCnrEditData->cbText)))
  {

    /******************************************************************/
    /* container text is being edited; we have enough memory...       */
    /* figure out what we are working with if container title, then   */
    /* item is the container itself                                   */
    /******************************************************************/

    if (bEditTitle)
      wIndex = (WORD)GetWindowLong(hwnd, WINDATA_ITEM_INDEX);
    else
    {
      /****************************************************************/
      /* otherwise, item is the record itself                         */
      /****************************************************************/

      wIndex = getRecordIndex(lpCnrEditData->pRecord);
    }

    /******************************************************************/
    /* if not editing details view, then the only thing possible is   */
    /* to edit the name                                               */
    /******************************************************************/

    if (!bEditDetails)
    {

      /****************************************************************/
      /* free old string memory                                       */
      /****************************************************************/

      if (dataObjects[wIndex].fDirectEdit & DE_NAME_EDITED)
        free(dataObjects[wIndex].pItemName);

      /****************************************************************/
      /* set name pointer to new string                               */
      /****************************************************************/

      dataObjects[wIndex].pItemName = pNewString;
      dataObjects[wIndex].fDirectEdit |= DE_NAME_EDITED;

      /****************************************************************/
      /* copy string address to other views if editing a record       */
      /****************************************************************/

      if (!bEditTitle)
        setViewNames(lpCnrEditData->pRecord, wIndex);
    }                                  /* end of if editing a name in */
                                       /* record or title             */
    else
    {
      WORD wVar1 = FLDOFFSET(USERDATA, recordCore.pszIcon);
      WORD wVar2 = FLDOFFSET(USERDATA, lpSocialSec);
      WORD wVar3 = FLDOFFSET(USERDATA, lpTask);
      WORD wVar4 = FLDOFFSET(USERDATA, lpNickName);

      /****************************************************************/
      /* if we are in details view switch on which column is being    */
      /* edited                                                       */
      /****************************************************************/

      if (lpCnrEditData->pFieldInfo->offStruct == wVar1)
      {
        if (dataObjects[wIndex].fDirectEdit & DE_NAME_EDITED)
          free(dataObjects[wIndex].pItemName);

        /**************************************************************/
        /* store it internally                                        */
        /**************************************************************/

        dataObjects[wIndex].pItemName = pNewString;
        dataObjects[wIndex].fDirectEdit |= DE_NAME_EDITED;

        /**************************************************************/
        /* store in container                                         */
        /**************************************************************/

        setViewNames(lpCnrEditData->pRecord, wIndex);
      }
      else
        if (lpCnrEditData->pFieldInfo->offStruct == wVar2)
        {

          /************************************************************/
          /* editing social security number lpUserData is structure    */
          /* where SS# is stored                                      */
          /************************************************************/

          LPUSERDATA lpUserData = (LPUSERDATA)(lpCnrEditData->pRecord);

          /************************************************************/
          /* if this is a department, disallow edit ...departments do */
          /* not have SS# also, disallow if text is not null and it   */
          /* is not a valid length                                    */
          /************************************************************/

          if (((dataObjects[wIndex].wItemType) == ITEM_TYPE_DEPT) ||
             (((lpCnrEditData->cbText) != 1)                      &&
             ((lpCnrEditData->cbText) != 12)))
          {
            free(pNewString);
            return (FALSE);
          }                            /* end of if trying to edit    */
                                       /* department SS#, or trying   */
                                       /* to add an invalid SS#       */

          /************************************************************/
          /* ok, everything is valid: free old SS# and store the new  */
          /* one                                                      */
          /************************************************************/

          if (dataObjects[wIndex].fDirectEdit & DE_SOCSEC_EDITED)
            free(dataObjects[wIndex].pSocialSec);

          dataObjects[wIndex].pSocialSec = pNewString;
          dataObjects[wIndex].fDirectEdit |= DE_SOCSEC_EDITED;
        }
        else
          if (lpCnrEditData->pFieldInfo->offStruct == wVar3)
          {

            /**********************************************************/
            /* editing task lpUserData is structure where task is      */
            /* stored                                                 */
            /**********************************************************/

            LPUSERDATA lpUserData = (LPUSERDATA)(lpCnrEditData->pRecord);

            /**********************************************************/
            /* free old task and store the new one                    */
            /**********************************************************/

            if (dataObjects[wIndex].fDirectEdit & DE_TASK_EDITED)
              free(dataObjects[wIndex].pTask);

            dataObjects[wIndex].pTask = pNewString;
            dataObjects[wIndex].fDirectEdit |= DE_TASK_EDITED;
          }
          else
            if (lpCnrEditData->pFieldInfo->offStruct == wVar4)
            {

              /********************************************************/
              /* editing nickName lpUserData is structure where task   */
              /* is stored                                            */
              /********************************************************/

              LPUSERDATA lpUserData = (LPUSERDATA)(lpCnrEditData->pRecord);

              /********************************************************/
              /* free old task and store the new one                  */
              /********************************************************/

              if (dataObjects[wIndex].fDirectEdit & DE_NICKNAME_EDITED)
                free(dataObjects[wIndex].pNickName);

              dataObjects[wIndex].pNickName = pNewString;
              dataObjects[wIndex].fDirectEdit |= DE_NICKNAME_EDITED;
            }
            else
            {
              free(pNewString);
              return (FALSE);
            }
    }                                  /* end of else: details view   */

    /******************************************************************/
    /* if we are here, then the whole thing was successful. Just      */
    /* store the new string pointer in the container structure so     */
    /* that it may set things on the display                          */
    /******************************************************************/

    *(lpCnrEditData->ppszText) = pNewString;

    /******************************************************************/
    /* return value indicates success                                 */
    /******************************************************************/

    return (TRUE);
  }                                    /* end of if we have enough    */
                                       /* memory                      */
  else                                 /* not enough memory           */
    return (FALSE);
}                                      /* end of case CN_REALLOCPSZ   */



/**********************************************************************/
/*  Function:  drawContainerItem                                      */
/*  Inputs:    pDrawItem           pointer to draw item structure     */
/*  Outputs:   TRUE if item drawn, FALSE if item not drawn            */
/*                                                                    */
/*  This function is called when the client window receives a         */
/*  WM_DRAWITEM message from the container window.  It determines     */
/*  whether to draw the item, draws it if necessary, and returns an   */
/*  appropriate value.                                                */
/**********************************************************************/

BOOL drawContainerItem(LPDRAWITEMSTRUCT pDrawItem)
{

  /********************************************************************/
  /* messages are sent from the container, its title window, details  */
  /* view windows, etc. The id of the window in pDrawItem specifies   */
  /* the actual sub-window switch on actual container window source   */
  /********************************************************************/

  switch (getWindowId(pDrawItem->hwndItem))                             /*DAB1*/
  {

    /******************************************************************/
    /* from the container itself                                      */
    /******************************************************************/

    case ID_CONTAINER:
      {
        LPRECORDCORE  lpRecord;         /* record contains attributes */
        HPEN          hpenOld = NULL;   /* save old pen               */
        HBRUSH        hbrushOld = NULL; /* save old brush             */
        BITMAP        BitmapHdr;
        HBITMAP       hbmOld;           /* Old bitmap handle          */
        HDC           hDCMemory;        /* handle to mem dev context  */

        /**************************************************************/
        /* obtain pointer to record to get necessary attributes       */
        /**************************************************************/

        lpRecord = ((LPCNRDRAWITEMINFO)(pDrawItem->itemData))->pRecord;

        /**************************************************************/
        /* if we are drawing the tree icon                            */
        /**************************************************************/

        if (pDrawItem->itemID == CMA_TREEICON)
        {
          /************************************************************/
          /* Set the foreground color                                 */
          /************************************************************/
          hpenOld = SelectObject (pDrawItem->hDC,
                                  GetStockObject (WHITE_PEN));

          /************************************************************/
          /* Set the background color                                 */
          /************************************************************/
          hbrushOld = SelectObject (pDrawItem->hDC,
                                    GetStockObject (BLACK_BRUSH));

          /************************************************************/
          /* Create a memory DC that is compatible with the device    */
          /************************************************************/
          hDCMemory = CreateCompatibleDC (pDrawItem->hDC);

          /************************************************************/
          /* if it is expanded, then draw the bitmap                  */
          /************************************************************/

          if (lpRecord->flRecordAttr & CRA_EXPANDED)
          {
            /**********************************************************/
            /* Select the expanded bitmap                             */
            /**********************************************************/
            hbmOld = SelectObject (hDCMemory, hbmTree[1]);

            /**********************************************************/
            /* Get the expanded bitmap's header                       */
            /**********************************************************/
            GetObject (hbmTree[1], sizeof(BITMAP), (LPSTR)&BitmapHdr);

            /**********************************************************/
            /* Draw the expanded bitmap                               */
            /**********************************************************/
            StretchBlt (pDrawItem->hDC, pDrawItem->rcItem.left,
                        pDrawItem->rcItem.top,
                        (pDrawItem->rcItem.right - pDrawItem->rcItem.left),
                        (pDrawItem->rcItem.bottom - pDrawItem->rcItem.top),
                        hDCMemory, 0, 0, BitmapHdr.bmWidth,
                        BitmapHdr.bmHeight, SRCCOPY);
          }
          else
          {
            /**********************************************************/
            /* otherwise, it must be collapsed                        */
            /**********************************************************/

            /**********************************************************/
            /* Select the collasped bitmap                            */
            /**********************************************************/
            hbmOld = SelectObject (hDCMemory, hbmTree[0]);

            /**********************************************************/
            /* Get the collapsed bitmap's header                      */
            /**********************************************************/
            GetObject (hbmTree[0], sizeof(BITMAP), (LPSTR)&BitmapHdr);

            /**********************************************************/
            /* Draw the collasped bitmap                               */
            /**********************************************************/
            StretchBlt (pDrawItem->hDC, pDrawItem->rcItem.left,
                        pDrawItem->rcItem.top,
                        (pDrawItem->rcItem.right - pDrawItem->rcItem.left),
                        (pDrawItem->rcItem.bottom - pDrawItem->rcItem.top),
                        hDCMemory, 0, 0, BitmapHdr.bmWidth,
                        BitmapHdr.bmHeight, SRCCOPY);
          }

          /************************************************************/
          /* Select the old bitmap                                    */
          /************************************************************/
          SelectObject (hDCMemory, hbmOld);

          /************************************************************/
          /* Destroy the memory DC                                    */
          /************************************************************/
          DeleteDC (hDCMemory);

          /************************************************************/
          /* Select the old pen color.                                */
          /************************************************************/
          if (hpenOld)
            SelectObject (pDrawItem->hDC, hpenOld);

          /************************************************************/
          /* Select the old brush color.                              */
          /************************************************************/
          if (hbrushOld)
            SelectObject (pDrawItem->hDC, hbrushOld);

          /************************************************************/
          /* return TRUE if we drew it                                */
          /************************************************************/
          return (TRUE);
        }                              /* end of if drawing tree icon */
      }                                /* end of if message came from */
                                       /* container itself            */
      break;

    /******************************************************************/
    /* container window in details view                               */
    /******************************************************************/

    case CID_LEFTDVWND:
    case CID_RIGHTDVWND:
      {
        char         buffer[20];       /* temporary string buffer     */
        LPUSERDATA   lpUserData;       /* what we are drawing         */
        WORD         wIndex;           /* current item index          */
        POINT        ptl;              /* where to draw               */
        TEXTMETRIC   textMetrics;      /* text metrics                */
        LPFIELDINFO  lpFieldInfo;      /* field info pointer          */
        HBRUSH       hBrush;
        int          oldBkMode = 0;    /* save old background mode    *//*DAB1*/
        WORD         wVar5 = FLDOFFSET(USERDATA, wHourlyWage);

        /**************************************************************/
        /* find out about our font                                    */
        /**************************************************************/

        GetTextMetrics (pDrawItem->hDC, (LPTEXTMETRIC)&textMetrics);

        /**************************************************************/
        /* find out what we are drawing                               */
        /**************************************************************/

        lpUserData =
          (LPUSERDATA)((LPCNRDRAWITEMINFO)pDrawItem->itemData)->pRecord;

        lpFieldInfo = ((LPCNRDRAWITEMINFO)pDrawItem->itemData)->pFieldInfo;

        /**************************************************************/
        /* retrieve object in global data                             */
        /**************************************************************/

        wIndex = getRecordIndex((LPRECORDCORE)lpUserData);

        /**************************************************************/
        /* if we must draw selection emphasis                         */
        /**************************************************************/

        if (lpUserData->recordCore.flRecordAttr & CRA_SELECTED)
        {
          /************************************************************/
          /* set text color                                           */
          /************************************************************/

          SetTextColor (pDrawItem->hDC, GetSysColor (COLOR_HIGHLIGHTTEXT));
          SetBkColor (pDrawItem->hDC, GetSysColor (COLOR_HIGHLIGHT));
          oldBkMode = SetBkMode (pDrawItem->hDC, OPAQUE);

          /************************************************************/
          /* fill background with highlight                           */
          /************************************************************/
          hBrush = CreateSolidBrush (GetSysColor (COLOR_HIGHLIGHT));
          FillRect (pDrawItem->hDC, &(pDrawItem->rcItem), hBrush);
          DeleteObject(hBrush);
        }                              /* end of if we need selected  */
                                       /* state emphasis              */

        /**************************************************************/
        /* initialize y-position for single-line text                 */
        /**************************************************************/

        ptl.y = (pDrawItem->rcItem.top + pDrawItem->rcItem.bottom -
                 textMetrics.tmHeight) / 2;


        /**************************************************************/
        /* now depending on which column to draw                      */
        /**************************************************************/

        if (lpFieldInfo->offStruct == wVar5)
        {
          /************************************************************/
          /* hourly wage if a person                                  */
          /************************************************************/

          if ((dataObjects[wIndex].wItemType) == ITEM_TYPE_PERSON)
          {
            /**********************************************************/
            /* place dollar sign to the left of the wage              */
            /**********************************************************/

            ptl.x = pDrawItem->rcItem.left +
                    (3 * textMetrics.tmMaxCharWidth);

            TextOut(pDrawItem->hDC,
                    ptl.x,
                    ptl.y,
                    buffer,
                    sprintf(buffer, "$"));

            /**********************************************************/
            /* move over and right justify the actual value           */
            /**********************************************************/

            ptl.x += 4 * textMetrics.tmMaxCharWidth;
            sprintf(buffer,
                     (((lpUserData->wHourlyWage)%100) < 10) ?
                       "%5d.0%1d"  :
                       "%5d.%2d",
                     (lpUserData->wHourlyWage)/100,
                     (lpUserData->wHourlyWage)%100);

            rtJustCharStringAt(pDrawItem->hDC,
                               &ptl,
                               (long)lstrlen(buffer),
                               buffer);

          }                            /* end of if this is a         */
                                       /* person's record             */
        }
        else
        {
          if (lpFieldInfo->offStruct == 0 )                             /*DAB1*/
          {
            if ((dataObjects[wIndex].wItemType) == ITEM_TYPE_DEPT)
            {
              /********************************************************/
              /* right justify the data in middle of column           */
              /********************************************************/

              ptl.x = pDrawItem->rcItem.left +
                      (4 * textMetrics.tmMaxCharWidth);

              rtJustCharStringAt(pDrawItem->hDC,
                                 &ptl,
                                 (long)sprintf(buffer,
                                               "%d",
                                               childCount(wIndex)),
                                 buffer);
            }                          /* end of if this is a         */
                                       /* department's record         */
          }
        }
        if (lpUserData->recordCore.flRecordAttr & CRA_SELECTED)
          SetBkMode (pDrawItem->hDC, oldBkMode);

        return (TRUE);                 /* successful redraw           */
      }                                /* end of case: draw details   */
                                       /* view column                 */
    default:                           /* other items inside          */
                                       /* container                   */
      break;
  }                                    /* end of switch on container  */
                                       /* internal origin             */

  /********************************************************************/
  /* if nothing drawn, then return FALSE                              */
  /********************************************************************/

  return (FALSE);
}                                      /* end of function             */
                                       /* drawContainerItem           */


/**********************************************************************/
/*  Function:  initClient                                             */
/*  Inputs:    hwnd                 client window handle              */
/*             hwndCnr              container window handle           */
/*             hwndParentContainer  parent container handle           */
/*             wIndex               index of item hwndCnr represents  */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called in response to a WM_CLIENT_INIT message.  */
/*  The container has been created; all that remains is for the       */
/*  client window to initialize its internal data.                    */
/**********************************************************************/

void initClient (HWND hwnd, HWND hwndCnr, HWND hwndParentContainer,
                 WORD wIndex)
{
  LPRECORDCORE   lpRecordCurrentIcon;  /* current icon in parent      */

  /********************************************************************/
  /* This message is sent by the parent window when the container has */
  /* been created. We first store the index of this container in the  */
  /* window's reserved data field                                     */
  /********************************************************************/

  SetWindowLong (hwnd, WINDATA_ITEM_INDEX, (DWORD)wIndex);

  /********************************************************************/
  /* now store the parent container handle                            */
  /********************************************************************/

  SetWindowLong (hwnd, WINDATA_PARENT_CNR, (DWORD)hwndParentContainer);

  /********************************************************************/
  /* set the current view in memory                                   */
  /********************************************************************/

  SetWindowLong (hwnd, WINDATA_CURR_VIEW, (DWORD)IDM_SUBICON);

  /********************************************************************/
  /* call the view constructor function for the initial view          */
  /********************************************************************/

  (viewFunctions[2 * (IDM_SUBICON - ID_FIRSTVIEWITEM)])(hwndCnr);

  /********************************************************************/
  /* If this is the top level container client then we are done with  */
  /* initialization                                                   */
  /********************************************************************/

  if (wIndex == INDEX_CONTAINER_TOP)
    return;

  /********************************************************************/
  /* otherwise, we must remove icon from parent container... query    */
  /* container for matching record                                    */
  /********************************************************************/

  lpRecordCurrentIcon = getContainerRecord (hwndParentContainer,
                                            NULL,
                                            wIndex);

  /********************************************************************/
  /* save the icon's original position in memory                      */
  /********************************************************************/

  SetWindowLong (hwnd, WINDATA_CURR_ICON_X,
                 (DWORD)(lpRecordCurrentIcon->ptlIcon.x));
  SetWindowLong (hwnd, WINDATA_CURR_ICON_Y,
                 (DWORD)(lpRecordCurrentIcon->ptlIcon.y));

  /********************************************************************/
  /* now that we have found the record, remove it; we free its memory */
  /* too, since its children may change. When this window is closed,  */
  /* we will rebuild the item's tree into the parent's container the  */
  /* CM_REMOVERECORD message expects to receive an array of pointers  */
  /* to RECORDCORE structures. However, since we only have one        */
  /* structure to delete, there is no need to create an "array."      */
  /* Instead, we just pass a pointer to a pointer first call the      */
  /* destructor for container items                                   */
  /********************************************************************/

  destroyContainerItemTree (hwndParentContainer, lpRecordCurrentIcon);

  /********************************************************************/
  /* now remove the item from the parent container                    */
  /********************************************************************/
  SendMessage (hwndParentContainer,
               CM_REMOVERECORD,
               (WPARAM)&lpRecordCurrentIcon,                            /*DAB3*/
               (LPARAM)MAKELONG( 1, CMA_INVALIDATE | CMA_FREE));        /*DAB3*/

  /********************************************************************/
  /* when a container is closed, it automatically frees all memory    */
  /* allocated for it; we must be able to replace our record inside   */
  /* the parent when the window is destroyed: two things can happen:  */
  /* 1) the child is closed--> in which case we simply rebuild        */
  /* ourselves inside the parent containers, or 2) a parent is        */
  /* closed, in which case, our parent no longer exists anyway--the   */
  /* closest child of a living parent replaces the entire tree in the */
  /* living parent                                                    */
  /********************************************************************/

}                                      /* end of function initClient  */



/**********************************************************************/
/*  Function:  destroyClient                                          */
/*  Inputs:    hwnd                 client window handle              */
/*             hwndCnr              container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called when a client window must destroy itself. */
/*  The window has already been hidden from view.  It must deallocate */
/*  any resources belonging to the child container and restore itself */
/*  as an icon in its parent container.                               */
/**********************************************************************/

void destroyClient (HWND hwnd, HWND hwndCnr)

  /********************************************************************/
  /* we must restore the icon to the parent window before leaving     */
  /********************************************************************/

{
  WORD               wIndex;               /* data object index of    */
                                           /* this client             */
  HWND               hwndParentContainer;  /* parent container handle */
  RECORDINSERT       recordInsert;         /* record insert structure */
  POINT              ptPosition;           /* where to place icon in  */
                                           /* parent                  */
  WORD               wCurrentView;         /* which view to destroy   */
  LPRECORDCORE       lpRecord;

  /********************************************************************/
  /* this window is closing                                           */
  /********************************************************************/

  wIndex = (WORD)GetWindowLong (hwnd, WINDATA_ITEM_INDEX);

  /********************************************************************/
  /* destroy current view by calling destructor                       */
  /********************************************************************/

  wCurrentView = (int)GetWindowLong (hwnd, WINDATA_CURR_VIEW);
  (viewFunctions[2 * (wCurrentView - ID_FIRSTVIEWITEM) + 1])(hwndCnr);

  /********************************************************************/
  /* deallocate container resources                                   */
  /********************************************************************/

  destroyContainer (hwndCnr);

  /********************************************************************/
  /* If this is the top level container client then we are done with  */
  /* deallocation                                                     */
  /********************************************************************/

  if (wIndex == INDEX_CONTAINER_TOP)
    return;

  /********************************************************************/
  /* find out which container this we belong to                       */
  /********************************************************************/

  hwndParentContainer = (HWND)GetWindowLong (hwnd, WINDATA_PARENT_CNR);

  /********************************************************************/
  /* now, we just replace the child inside the parent container; we   */
  /* simply ask to recreate the child's tree in its original location */
  /* initialize insert structure for container: size of structure, at */
  /* front of list, no parent, top of z-order, one record, draw it    */
  /* immediately                                                      */
  /********************************************************************/

  recordInsert.cb = sizeof(RECORDINSERT);
  recordInsert.pRecordOrder = (LPRECORDCORE)CMA_FIRST;
  recordInsert.pRecordParent = NULL;
  recordInsert.zOrder = (WORD)CMA_TOP;
  recordInsert.cRecordsInsert = 1;
  recordInsert.fInvalidateRecord = TRUE;

  /********************************************************************/
  /* find the object's original position                              */
  /********************************************************************/

  ptPosition.x = (int)GetWindowLong (hwnd, WINDATA_CURR_ICON_X);
  ptPosition.y = (int)GetWindowLong (hwnd, WINDATA_CURR_ICON_Y);

  /********************************************************************/
  /* get record to set emphasis of                                    */
  /********************************************************************/
  lpRecord = createContainerItemTree (hwndParentContainer,
                                      recordInsert,
                                      wIndex,
                                      &ptPosition);

  /********************************************************************/
  /* the icon is restored to its original screen location; however,   */
  /* in single selection containers, we now have two icons with       */
  /* emphasis (this one--which received the emphasis when it was      */
  /* double-clicked in the first place), and another one, which       */
  /* received the emphasis after this one was removed. By sending the */
  /* CM_SETRECORDEMPHASIS message, we restore sole emphasis to this   */
  /* object.                                                          */
  /********************************************************************/

  SendMessage (hwndParentContainer,
               CM_SETRECORDEMPHASIS,
               (WPARAM)lpRecord,                                        /*DAB3*/
               (LPARAM)MAKELONG( TRUE, CRA_SELECTED ));                 /*DAB3*/

  SetFocus (hwndParentContainer);      /* Set the focus to the parent */
                                       /* container, since Windows    */
                                       /* seems to have problems with */
                                       /* the parent child chain      */

}                                      /* end of destroyClient        */
                                       /* function                    */


/**********************************************************************/
/*  Function:  createContainer                                        */
/*  Inputs:    hwndClient         client window handle                */
/*             wNewIdx            index of item container represents  */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called to create and fill a container for the    */
/*  given client window.  It creates a container window and then calls*/
/*  createContainerItemTree to fill it.                               */
/**********************************************************************/

BOOL createContainer (HWND hwndClient, WORD wNewIdx)

  /********************************************************************/
  /* create child container                                           */
  /********************************************************************/

{
  HWND          hwndContainer;         /* container window handle     */
  RECT          rcl;                   /* parent window size          */
  DWORD         dwCnrStyle;            /* container window style      */
  CNRINFO       cnrinfo;               /* container info. structure   */
  RECORDINSERT  recordInsert;          /* insert structure            */
  WORD          wIndex;                /* loop index                  */

  dwCnrStyle = CCS_SINGLESEL |         /* single selection            */
               CNR_REQUIREDSTYLES |    /* required styles for control */
               WS_VISIBLE;             /* visible                     */

  /********************************************************************/
  /* create a container in client window; all child containers have   */
  /* the id ID_CONTAINER. The actual object index is stored in the    */
  /* WORD memory allocated for the client window; the client window   */
  /* sets this upon receiving a WM_CLIENT_INIT message get client     */
  /* window size                                                      */
  /********************************************************************/

  GetClientRect (hwndClient, &rcl);
  hwndContainer = CreateWindow (ICL_CONTAINER,
                                NULL,
                                dwCnrStyle,          /* style         */
                                CNR_MARGIN_XLEFT,    /* origin        */
                                CNR_MARGIN_YTOP,
                                (int)(rcl.right      /* width         */
                                       - CNR_MARGIN_XLEFT
                                       - CNR_MARGIN_XRIGHT),
                                (int)(rcl.bottom     /* height        */
                                       - CNR_MARGIN_YBOTTOM
                                       - CNR_MARGIN_YTOP),
                                hwndClient,          /* parent        */
                                (HMENU)ID_CONTAINER, /* id            *//*DAB1*/
                                vhInstance,
                                (LPVOID)NULL);                          /*DAB1*/


  /********************************************************************/
  /* set container appearance                                         */
  /********************************************************************/

  cnrinfo.cb = sizeof(CNRINFO);        /* structure size              */

  /********************************************************************/
  /* the pSortRecord points to a function that can sort records as    */
  /* they are placed in the container                                 */
  /********************************************************************/

  if (!(cnrinfo.pSortRecord =
         (LPVOID)MakeProcInstance ((FARPROC)RecordCoreCompare, vhInstance)))
  {
     return (FALSE);
  }

  cnrinfo.flWindowAttr =
     CV_ICON           |               /* icon view                   */
     CA_CONTAINERTITLE |               /* container has a title       */
     CA_TITLESEPARATOR |               /* a line separates title      */
     CA_TITLECENTER    |               /* title is centered           */
     CA_DRAWICON;                      /* icons used in all views     */

  /********************************************************************/
  /* the container title is the name of the object it represents      */
  /********************************************************************/

  cnrinfo.pszCnrTitle = dataObjects[wNewIdx].pItemName;

  /********************************************************************/
  /* set these attributes in the container                            */
  /********************************************************************/

  SendMessage (hwndContainer,
               CM_SETCNRINFO,
               (WPARAM)&cnrinfo,                                        /*DAB3*/
               CMA_CNRTITLE | CMA_FLWINDOWATTR | CMA_PSORTRECORD );     /*DAB3*/

  /********************************************************************/
  /* initialize insert structure for container                        */
  /********************************************************************/

  recordInsert.cb = sizeof(RECORDINSERT);         /* size of struct   */
  recordInsert.pRecordOrder =
                          (LPRECORDCORE)CMA_END;  /* end of list      */
  recordInsert.pRecordParent = NULL;              /* root level       */
  recordInsert.zOrder = (WORD)CMA_TOP;            /* z-order          */
  recordInsert.cRecordsInsert = 1;                /* one record       */
  recordInsert.fInvalidateRecord = TRUE;          /* draw immediately */

  /********************************************************************/
  /* now construct the objects for the container by searching for all */
  /* data whose parent is this id                                     */
  /********************************************************************/

  for (wIndex = INDEXFIRST; wIndex < COUNT_OBJECT; wIndex++)
  {

    /******************************************************************/
    /* if this object's parent is this index then put it in           */
    /* container; we do not specify an initial position since the     */
    /* objects will be arranged later                                 */
    /******************************************************************/

    if (dataObjects[wIndex].wParentIdx == wNewIdx)
      createContainerItemTree (hwndContainer, recordInsert, wIndex, NULL);
  }                                    /* end of loop to initialize   */
                                       /* objects                     */

  SetFocus (hwndContainer);

  /********************************************************************/
  /* container objects are arranged when icon view is initialized     */
  /********************************************************************/

  return (TRUE);

}                                      /* end of function             */
                                       /* createContainer             */


/**********************************************************************/
/*  Function:  createContainerItemTree                                */
/*  Inputs:    hwndContainer        container window handle           */
/*             recordInsert         record insert structure           */
/*             wIndex               index of item to insert           */
/*             pptPosition          initial location of icon; NULL    */
/*                                  if don't care                     */
/*  Outputs:   record core of index inside container                  */
/*                                                                    */
/*  This function is called to insert a tree into the container.  The */
/*  specified item is inserted.  The object database is searched for  */
/*  any descendants, and they are inserted recursively.               */
/**********************************************************************/

LPRECORDCORE createContainerItemTree (HWND hwndContainer,
                                      RECORDINSERT recordInsert,
                                      WORD wIndex,
                                      LPPOINT lpptPosition)

  /********************************************************************/
  /* inserts the specified object into the given container; during    */
  /* insertion, we treat the data as a USERDATA structure so that the */
  /* user-defined data may be initialized; afterward, we can ignore   */
  /* the user data and treat all pointers as pointers to recordCore   */
  /* structures--the user data is always there; the inserted record   */
  /* is returned                                                      */
  /********************************************************************/

{
  WORD          wLoop;                 /* loop index                  */
  LPUSERDATA    lpUserData;            /* user data                   */
  LPRECORDCORE  lpRecordCore;          /* record core being inserted  */

  /********************************************************************/
  /* get memory for record                                            */
  /********************************************************************/

  lpUserData =
     (LPUSERDATA)SendMessage (
                     hwndContainer,               /* Window handle    */
                     CM_ALLOCRECORD,              /* Memory request   */
                     sizeof(USERDATA)             /* additional data  */
                       - sizeof(RECORDCORE),
                     1L);                         /* Record count     */

  /********************************************************************/
  /* initialize the record core                                       */
  /********************************************************************/

  lpRecordCore = &(lpUserData->recordCore);
  lpRecordCore->cb = sizeof(RECORDCORE);          /* structure size   */

  /********************************************************************/
  /* set strings for icon, name, text, and tree views                 */
  /********************************************************************/

  setViewNames (lpRecordCore, wIndex);

  /********************************************************************/
  /* set up object display depending on type: load appropriate icons  */
  /* and set attributes. Note that only a department may be dropped   */
  /* upon because only departments may contain other objects          */
  /********************************************************************/

  switch (dataObjects[wIndex].wItemType)
  {
    case  ITEM_TYPE_PERSON :
      lpRecordCore->hptrIcon = hptrIcons[0];
      lpRecordCore->hptrMiniIcon = hptrMiniIcons[0];
      break;

    case  ITEM_TYPE_DEPT :
      lpRecordCore->hptrIcon = hptrIcons[1];
      lpRecordCore->hptrMiniIcon = hptrMiniIcons[1];
      break;
  }                                    /* end of switch               */

  /********************************************************************/
  /* if we have an initial position, then set it                      */
  /********************************************************************/

  if (lpptPosition != NULL)
    lpRecordCore->ptlIcon = *lpptPosition;

  /********************************************************************/
  /* initialize user-defined data                                     */
  /********************************************************************/

  lpUserData->wIndex = wIndex;                          /* item index */
  lpUserData->lpSocialSec =
                    dataObjects[wIndex].pSocialSec;     /* Soc. Sec.# */
  lpUserData->wHourlyWage =
                    dataObjects[wIndex].wHourlyWage;    /* hourly wage*/
  lpUserData->lpTask = dataObjects[wIndex].pTask;       /* task       */
  lpUserData->lpNickName =
                    dataObjects[wIndex].pNickName;      /* nick name  */

  /********************************************************************/
  /* tell container about our new object                              */
  /********************************************************************/
  SendMessage (hwndContainer,
               CM_INSERTRECORD,
               (WPARAM)lpUserData,                                      /*DAB3*/
               (LPARAM)&recordInsert );                                 /*DAB3*/

  /********************************************************************/
  /* a person has no child nodes, so we are done                      */
  /********************************************************************/

  if (dataObjects[wIndex].wItemType == ITEM_TYPE_PERSON)
    return (lpRecordCore);

  /********************************************************************/
  /* now construct the child objects for the department by searching  */
  /* for all data whose parent is this id                             */
  /********************************************************************/

  recordInsert.pRecordParent = lpRecordCore;
  for (wLoop = INDEXFIRST; wLoop < COUNT_OBJECT; wLoop++)
  {

    /******************************************************************/
    /* if this object's parent is this index then put it in           */
    /* container; we do not specify an initial position since the     */
    /* objects will be arranged later                                 */
    /******************************************************************/

    if (dataObjects[wLoop].wParentIdx == wIndex)
      createContainerItemTree (hwndContainer, recordInsert, wLoop, NULL);
  }                                    /* end of loop to initialize   */
                                       /* objects                     */
  return (lpRecordCore);
}                                      /* end of function             */
                                       /* createContainerItemTree     */


void destroyContainer (HWND hwndCnr)

  /********************************************************************/
  /* destroy any allocated resources for objects in container: any    */
  /* deallocation of resources belonging to the entire container      */
  /* would go here                                                    */
  /********************************************************************/

{
  destroyContainerItemTree (hwndCnr, NULL);
}


/**********************************************************************/
/*  Function:  destroyContainerItemTree                               */
/*  Inputs:    hwndCnr              container window handle           */
/*             lpRecordCore         root of tree to destroy           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called to destroy all items that are children    */
/*  of specified item is inserted.  Memory is freed by the container. */
/*  We would destroy here any resources (icons, etc.) that are        */
/*  allocated for each record.                                        */
/**********************************************************************/

void destroyContainerItemTree (HWND hwndCnr, LPRECORDCORE lpRecordCore)

  /********************************************************************/
  /* performs any necessary deinitializations of the lpRecordCore     */
  /* structure and its children                                       */
  /********************************************************************/
{
  LPRECORDCORE  lpRecordCoreChild;     /* record being cleaned up     */

  /********************************************************************/
  /* get the parent's first child                                     */
  /********************************************************************/
  lpRecordCoreChild =
     (LPRECORDCORE)SendMessage (hwndCnr,
                                CM_QUERYRECORD,
                                (WPARAM)lpRecordCore,                   /*DAB3*/
                                (LPARAM)MAKELONG( CMA_FIRSTCHILD,       /*DAB3*/
                                                  CMA_ITEMORDER ));     /*DAB3*/

  /********************************************************************/
  /* while there are more children to deallocate                      */
  /********************************************************************/
  while (lpRecordCoreChild != NULL)
  {
    /******************************************************************/
    /* destroy this one recursively                                   */
    /******************************************************************/
    destroyContainerItemTree (hwndCnr, lpRecordCoreChild);

    /******************************************************************/
    /* any necessary record deallocation routines would go here (for  */
    /* lpRecordCoreChild end of deallocation move to next child       */
    /******************************************************************/
    lpRecordCoreChild =
      (LPRECORDCORE)SendMessage (hwndCnr,
                                 CM_QUERYRECORD,
                                 (WPARAM)lpRecordCoreChild,             /*DAB3*/
                                 (LPARAM)MAKELONG( CMA_NEXT,            /*DAB3*/
                                                   CMA_ITEMORDER ));    /*DAB3*/
  }                                    /* end of loop through objects */
                                       /* in container                */

  /********************************************************************/
  /* the final step would be to deallocate anything for the parent    */
  /* (lpRecordCore) end of deallocation                               */
  /********************************************************************/

}                                      /* end of function             */
                                       /* destroyContainerItemTree    */


/**********************************************************************/
/*  Function:  getRecordIndex                                         */
/*  Inputs:    lpSeekRecord         record to find internal index for */
/*  Outputs:   index corresponding to seekRecord                      */
/*                                                                    */
/*  This function receives a pointer to a recordCore structure and    */
/*  returns the index of the corresponding object within the          */
/*  dataObjects array.                                                */
/**********************************************************************/
WORD getRecordIndex (LPRECORDCORE lpSeekRecord)

  /********************************************************************/
  /* In this implementation, we save the array index as a WORD        */
  /* inside the RecordCore user-defined data. If this had utilized    */
  /* dynamic object allocation, a pointer could have been used; the   */
  /* objects would be stored in memory using a linked list            */
  /********************************************************************/

{
  return (((LPUSERDATA)lpSeekRecord)->wIndex);
}                                      /* end of function             */
                                       /* getRecordIndex              */


/**********************************************************************/
/*  Function:  getContainerRecord                                     */
/*  Inputs:    hwndCnr              container handle                  */
/*             lpRecordParent       root of tree                      */
/*             wIndex               index of record to seek           */
/*  Outputs:   pointer to recordCore representing this object         */
/*                                                                    */
/*  This function does the opposite of getRecordIndex:  it receives   */
/*  the container handle and the index of the object in the           */
/*  dataArray.  It queries the specified container until it finds     */
/*  a pRecordCore matching the data object index; CM_SEARCHSTRING     */
/*  cannot be used in this case since we are really searching         */
/*  for a specific record, rather than one just having the same name  */
/**********************************************************************/

LPRECORDCORE getContainerRecord (HWND hwndCnr, LPRECORDCORE lpRecordParent,
                                 WORD wIndex)
{
  LPRECORDCORE  lpRecordPreviousIcon;  /* previous record queried     */
  LPRECORDCORE  lpRecordCurrentIcon;   /* current record being        */
                                       /* queried                     */

  /********************************************************************/
  /* start with the parent's first child                              */
  /********************************************************************/
  lpRecordPreviousIcon = NULL;
  lpRecordCurrentIcon =
     (LPRECORDCORE)SendMessage (hwndCnr,
                                CM_QUERYRECORD,
                                (WPARAM)lpRecordParent,                 /*DAB3*/
                                (LPARAM)MAKELONG( CMA_FIRSTCHILD,       /*DAB3*/
                                                  CMA_ITEMORDER ));     /*DAB3*/

  /********************************************************************/
  /* while the current record name pointer does not match our index,  */
  /* keep searching                                                   */
  /********************************************************************/
  while ((lpRecordCurrentIcon) &&
         (getRecordIndex(lpRecordCurrentIcon) != wIndex))
  {

    /******************************************************************/
    /* search its children                                            */
    /******************************************************************/
    lpRecordPreviousIcon = lpRecordCurrentIcon;
    lpRecordCurrentIcon = getContainerRecord (hwndCnr,
                                              lpRecordPreviousIcon,
                                              wIndex);

    /******************************************************************/
    /* search its siblings                                            */
    /******************************************************************/
    if (!lpRecordCurrentIcon)
      lpRecordCurrentIcon =
        (LPRECORDCORE)SendMessage (hwndCnr,
                                   CM_QUERYRECORD,
                                   (WPARAM)lpRecordPreviousIcon,        /*DAB3*/
                                   (LPARAM)MAKELONG( CMA_NEXT,          /*DAB3*/
                                                     CMA_ITEMORDER ));  /*DAB3*/

  }                                    /* end of loop searching for   */
                                       /* matching record             */
  return (lpRecordCurrentIcon);
}                                      /* end of function             */
                                       /* getContainerRecord          */

/**********************************************************************/
/*  Function:  setViewNames                                           */
/*  Inputs:    lpUpdateRecord       record core to update             */
/*             wIndex               index of this record within data  */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called to set the icon, name, text, and tree     */
/*  view name strings to be the current name of the item within       */
/*  the global data.  It simply sets all of the strings to point to   */
/*  the strings within the global data array.                         */
/**********************************************************************/

void setViewNames (LPRECORDCORE lpUpdateRecord, WORD wIndex)

  /********************************************************************/
  /* updates the view name pointers for the given recordCore          */
  /* structure                                                        */
  /********************************************************************/

{
  lpUpdateRecord->pszText =
                  dataObjects[wIndex].pItemName;        /* text view  */
  lpUpdateRecord->pszName =
                  dataObjects[wIndex].pItemName;        /* name view  */
  lpUpdateRecord->pszIcon =
                  dataObjects[wIndex].pItemName;        /* icon view  */
  lpUpdateRecord->pszTree =
                  dataObjects[wIndex].pItemName;        /* tree view  */
}                                      /* end of function to reset    */
                                       /* string pointers inside      */
                                       /* record                      */


/**********************************************************************/
/*  Function:  ancestorOf                                             */
/*  Inputs:    wParent              index of possible parent item     */
/*             wChild               index of possible child item      */
/*  Outputs:   TRUE if parent is a direct ancestor of child;          */
/*             FALSE otherwise                                        */
/*                                                                    */
/*  This function searches each of the child's direct ancestors and   */
/*  returns TRUE if parent is one of them; otherwise, if returns FALSE*/
/**********************************************************************/

BOOL ancestorOf (WORD wParent, WORD wChild)
{

  /********************************************************************/
  /* while none of child's ancestors is the parent                    */
  /********************************************************************/

  while (wChild != INDEX_CONTAINER_TOP)
  {

    /******************************************************************/
    /* if child is same as parent, then parent counts as ancestor     */
    /******************************************************************/

    if (wChild == wParent)
      return (TRUE);

    /******************************************************************/
    /* move to child's parent and check again                         */
    /******************************************************************/

    wChild = dataObjects[wChild].wParentIdx;
  }                                    /* end of while loop: while we */
                                       /* haven't searched to top of  */
                                       /* child hierarchy             */

  /********************************************************************/
  /* if none of child's ancestor's was the parent, then the parent is */
  /* not an ancestor of the child                                     */
  /********************************************************************/

  return (FALSE);
}                                      /* end of function ancestorOf  */



/**********************************************************************/
/*  Function:  createIconView                                         */
/*  Inputs:    hwndContainer        container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This constructor function is called when a container is being     */
/*  changed into icon view.                                           */
/**********************************************************************/

void createIconView (HWND hwndContainer)

  /********************************************************************/
  /* creates icon view for the container                              */
  /********************************************************************/

{
  CNRINFO  cnrinfo;                    /* container info structure    */

  cnrinfo.cb = sizeof(CNRINFO);        /* structure size              */
  cnrinfo.flWindowAttr =
     CV_ICON           |               /* icon view                   */
     CA_CONTAINERTITLE |               /* has a title                 */
     CA_TITLESEPARATOR |               /* a line below title          */
     CA_TITLECENTER    |               /* title centered              */
     CA_DRAWICON;                      /* icons instead of bitmaps    */

  SendMessage (hwndContainer,          /* set the window attributes   */
               CM_SETCNRINFO,
               (WPARAM)&cnrinfo,                                        /*DAB3*/
               (LPARAM)CMA_FLWINDOWATTR );                              /*DAB3*/

  /********************************************************************/
  /* finally, rearrange the container objects                         */
  /********************************************************************/
  SendMessage (hwndContainer,
               CM_ARRANGE,
               0L,                                                      /*DAB1*/
               0L);
}                                      /* end of function             */
                                       /* createIconView              */


/**********************************************************************/
/*  Function:  createDetailsView                                      */
/*  Inputs:    hwndContainer        container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This constructor function is called when a container is being     */
/*  changed into details view.  It must perform any view-specific     */
/*  initializations and allocations.  Whenever a container is leaving */
/*  details view, destroyDetailsView will be called to deallocate any */
/*  view-specific resources.  For details view, we must create the    */
/*  FieldInfo structures to define each column in the container.      */
/**********************************************************************/

void createDetailsView (HWND hwndContainer)

  /********************************************************************/
  /* creates the details view for the container                       */
  /********************************************************************/

{
  CNRINFO           cnrinfo;           /* container info. structure   */
  FIELDINFOINSERT   fieldInsert;       /* FieldInsert structure       */
  LPFIELDINFO       lpFieldInfo;       /* first column fieldinfo      */
                                       /* structure                   */
  RECT              rcl;               /* size of container           */

  /********************************************************************/
  /* get size of container                                            */
  /********************************************************************/

  GetClientRect (hwndContainer, &rcl);

  /********************************************************************/
  /* initialize field insert structure for container: size of         */
  /* structure, end of column list, COUNT_FIELDS fields, display      */
  /* immediately                                                      */
  /********************************************************************/

  fieldInsert.cb = sizeof(FIELDINFOINSERT);
  fieldInsert.pFieldInfoOrder = (LPFIELDINFO)CMA_END;
  fieldInsert.cFieldInfoInsert = COUNT_FIELDS;
  fieldInsert.fInvalidateFieldInfo = TRUE;

  /********************************************************************/
  /* now construct the fields for the details view                    */
  /********************************************************************/

  lpFieldInfo = createDetailsViewItems (hwndContainer, &fieldInsert);

  /********************************************************************/
  /* set container view                                               */
  /********************************************************************/

  cnrinfo.cb = sizeof(CNRINFO);        /* structure size              */
  cnrinfo.flWindowAttr =
     CV_DETAIL            |            /* details view                */
     CA_CONTAINERTITLE    |            /* container title             */
     CA_TITLESEPARATOR    |            /* line below it               */
     CA_TITLECENTER       |            /* title centered              */
     CA_DETAILSVIEWTITLES |            /* column titles               */
     CA_DRAWICON;                      /* use icons, not bitmaps      */

  cnrinfo.pFieldInfoLast =
     lpFieldInfo->pNextFieldInfo;      /* put vertical splitbar after */
                                       /* second column               */
  cnrinfo.xVertSplitbar =
     (WORD)rcl.right *2/5;             /* put splitbar a bit left of  */
                                       /*  center          */
  SendMessage (hwndContainer,          /* ... set the view            */
               CM_SETCNRINFO,
               (WPARAM)&cnrinfo,                                        /*DAB3*/
               (LPARAM)CMA_FLWINDOWATTR | CMA_PFIELDINFOLAST |          /*DAB3*/
                       CMA_XVERTSPLITBAR );                             /*DAB3*/
}                                      /* end of function             */
                                       /* createDetailsView           */


/**********************************************************************/
/*  Function:  createDetailsViewItems                                 */
/*  Inputs:    hwndContainer        container window handle           */
/*             lpFieldInsert        pointer to field info. insert     */
/*                                  structure                         */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called when details view is created.  It sets    */
/*  up column information and inserts each field info. structure into */
/*  the container.                                                    */
/**********************************************************************/

LPFIELDINFO createDetailsViewItems (HWND hwndContainer,
                                    LPFIELDINFOINSERT lpFieldInsert)

  /********************************************************************/
  /* inserts the specified field into details view                    */
  /********************************************************************/

{
  LPFIELDINFO  lpFieldInfoHead;        /* first field                 */
  LPFIELDINFO  lpFieldInfo;            /* current column to           */
                                       /* initialize                  */
  WORD         column;                 /* loop index                  */

  /********************************************************************/
  /* get memory for record                                            */
  /********************************************************************/

  lpFieldInfoHead =
     (LPFIELDINFO)SendMessage (
                     hwndContainer,           /* Window handle        */
                     CM_ALLOCDETAILFIELDINFO,
                     COUNT_FIELDS,            /* inserting all fields */
                     0L);                     /* reserved             */
  lpFieldInfo = lpFieldInfoHead;              /* start at head of list*/

  /********************************************************************/
  /* now loop through columns                                         */
  /********************************************************************/

  for (column = 0 ; column < COUNT_FIELDS ; column ++)
  {
    lpFieldInfo->cb = sizeof(FIELDINFO);         /* size of structure */
    lpFieldInfo->pTitleData = fieldNames[column];/* field title       */

    /******************************************************************/
    /* title is read-only, vertically and horizontally centered       */
    /******************************************************************/

    lpFieldInfo->flTitle = CFA_FITITLEREADONLY |
                           CFA_VCENTER         |
                           CFA_CENTER;

    /******************************************************************/
    /* set up column display depending on index                       */
    /******************************************************************/

    switch (column)
    {
      case 0:

        /**************************************************************/
        /* column of icons we want icons, vertically centered and     */
        /* horizonally centered in column                             */
        /**************************************************************/

        lpFieldInfo->flData = CFA_BITMAPORICON | CFA_VCENTER | CFA_CENTER;

        /**************************************************************/
        /* The column data is the item's icon in the recordCore       */
        /**************************************************************/

        lpFieldInfo->offStruct = FLDOFFSET(USERDATA,
                                             recordCore.hptrIcon);
        break;

      case 1:

        /**************************************************************/
        /* column of item titles items are strings, there is a line   */
        /* below the column title, the strings are left justified and */
        /* vertically centered in column                              */
        /**************************************************************/

        lpFieldInfo->flData = CFA_STRING        |
                              CFA_HORZSEPARATOR |
                              CFA_VCENTER       |
                              CFA_LEFT;

        /**************************************************************/
        /* The data in this column is the icon-view text stored in    */
        /* the record core                                            */
        /**************************************************************/

        lpFieldInfo->offStruct = FLDOFFSET(USERDATA,
                                             recordCore.pszIcon);
        break;

      case 2:

        /**************************************************************/
        /* column of object tasks: /* items are strings, there is a   */
        /* line below the column title, the strings are left          */
        /* justified and vertically centered in column                */
        /**************************************************************/

        lpFieldInfo->flData = CFA_STRING        |
                              CFA_HORZSEPARATOR |
                              CFA_VCENTER       |
                              CFA_LEFT;

        /**************************************************************/
        /* The data in this column is the task name text stored in    */
        /* the user data                                              */
        /**************************************************************/

        lpFieldInfo->offStruct = FLDOFFSET(USERDATA, lpTask);
        break;

      case 3:

        /**************************************************************/
        /* column of nicknames: /* items are strings, there is a line */
        /* below the column title, the strings are left justified and */
        /* vertically centered in column                              */
        /**************************************************************/

        lpFieldInfo->flData = CFA_STRING         |
                              CFA_HORZSEPARATOR  |
                              CFA_VCENTER        |
                              CFA_LEFT;

        /**************************************************************/
        /* The data in this column is the task name text stored in    */
        /* the user data                                              */
        /**************************************************************/

        lpFieldInfo->offStruct = FLDOFFSET(USERDATA, lpNickName);
        break;

      case 4:

        /**************************************************************/
        /* column of social security numbers This is a column of      */
        /* strings, vertically and horizontally centered, with a      */
        /* title separator line                                       */
        /**************************************************************/

        lpFieldInfo->flData = CFA_STRING        |
                              CFA_HORZSEPARATOR |
                              CFA_VCENTER       |
                              CFA_CENTER;

        /**************************************************************/
        /* The column data is stored in the USERDATA structure        */
        /**************************************************************/

        lpFieldInfo->offStruct = FLDOFFSET(USERDATA, lpSocialSec);
        break;

      case 5:

        /**************************************************************/
        /* column of hourly wages: read-only, owner-draw vertically   */
        /* centered, right-justified, formatted numbers               */
        /**************************************************************/

        lpFieldInfo->flData = CFA_HORZSEPARATOR |
                              CFA_OWNER         |
                              CFA_VCENTER       |
                              CFA_RIGHT;

        /**************************************************************/
        /* The data is stored in the USERDATA structure               */
        /**************************************************************/

        lpFieldInfo->offStruct = FLDOFFSET(USERDATA, wHourlyWage);
        break;

      case 6:

        /**************************************************************/
        /* column of department member counts: read-only, owner-draw  */
        /* numbers, vertically centered, right-justified              */
        /**************************************************************/

        lpFieldInfo->flData = CFA_HORZSEPARATOR |
                              CFA_OWNER         |
                              CFA_VCENTER       |
                              CFA_RIGHT;

        /**************************************************************/
        /* This field is not stored in our structure since it is      */
        /* calculated "on the fly"                                    */
        /**************************************************************/

        lpFieldInfo->offStruct = 0;                                     /*DAB1*/
        break;

      default:
        break;
    }                                  /* end of switch               */

    /******************************************************************/
    /* advance pointer to next field                                  */
    /******************************************************************/

    lpFieldInfo = lpFieldInfo->pNextFieldInfo;
  }                                    /* end of loop through columns */

  /********************************************************************/
  /* tell container about our new column                              */
  /********************************************************************/

  SendMessage (hwndContainer,
               CM_INSERTDETAILFIELDINFO,
               (WPARAM)lpFieldInfoHead,                                 /*DAB3*/
               (LPARAM)lpFieldInsert );        /* insert structure    *//*DAB3*/

  /********************************************************************/
  /* return head of list                                              */
  /********************************************************************/

  return (lpFieldInfoHead);
}                                      /* end of function             */
                                       /* createDetailsViewItems      */


/**********************************************************************/
/*  Function:  destroyDetailsView                                     */
/*  Inputs:    hwndContainer        container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This destructor function is called when a container is being      */
/*  changed out of details view.  It must perform any view-specific   */
/*  deallocations corresponding to work done in createDetailsView.    */
/*  For details view, we must deallocate all of the fieldInfo         */
/*  structures.                                                       */
/**********************************************************************/

void destroyDetailsView (HWND hwndContainer)

  /********************************************************************/
  /* eliminates the details field structures when user switches out   */
  /* of details view                                                  */
  /********************************************************************/

{
  /********************************************************************/
  /* remove all details field structures                              */
  /********************************************************************/
  SendMessage (hwndContainer,
               CM_REMOVEDETAILFIELDINFO,
               (WPARAM)0L,                                              /*DAB3*/
               (LPARAM)MAKELONG( 0, CMA_FREE ));                        /*DAB3*/
}                                      /* end of function             */
                                       /* destroyDetailsView          */


/**********************************************************************/
/*  Function:  createNameView                                         */
/*  Inputs:    hwndContainer        container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This constructor function is called when a container is being     */
/*  changed into name view.                                           */
/**********************************************************************/

void createNameView (HWND hwndContainer)

  /********************************************************************/
  /* creates icon view for the container                              */
  /********************************************************************/

{
  CNRINFO cnrinfo;                     /* container info structure    */

  cnrinfo.cb = sizeof(CNRINFO);        /* structure size              */
  cnrinfo.flWindowAttr =
     CV_NAME           |               /* name view                   */
     CV_MINI           |               /* small icons                 */
     CV_FLOW           |               /* flow columns                */
     CA_CONTAINERTITLE |               /* title                       */
     CA_TITLESEPARATOR |               /* separator line              */
     CA_TITLECENTER    |               /* center title                */
     CA_DRAWICON;                      /* use icons, not bitmaps      */

  SendMessage (hwndContainer,          /* ... set the view            */
               CM_SETCNRINFO,
               (WPARAM)&cnrinfo,                                        /*DAB3*/
               (LPARAM)CMA_FLWINDOWATTR );                              /*DAB3*/
}                                      /* end of function             */
                                       /* createNameView              */


void destroyNameView (HWND hwndCnr)
{

/* Function not needed                                                */

  hwndCnr;
  return ;
}



void destroyTextView (HWND hwndCnr)
{

/* Function not needed                                                */

  hwndCnr;
  return ;
}



void destroyIconView (HWND hwndCnr)
{

/* Function not needed                                                */

  hwndCnr;
  return ;
}



void destroyTreeIconView (HWND hwndCnr)
{

/* Function not needed                                                */

  hwndCnr;
  return ;
}



/**********************************************************************/
/*  Function:  createTextView                                         */
/*  Inputs:    hwndContainer        container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This constructor function is called when a container is being     */
/*  changed into text view.                                           */
/**********************************************************************/

void createTextView (HWND hwndContainer)

  /********************************************************************/
  /* creates text view for the container                              */
  /********************************************************************/

{
  CNRINFO cnrinfo;                     /* container info structure    */

  cnrinfo.cb = sizeof(CNRINFO);        /* structure size              */
  cnrinfo.flWindowAttr =
     CV_TEXT           |               /* text view                   */
     CV_FLOW           |               /* flow columns                */
     CA_CONTAINERTITLE |               /* title                       */
     CA_TITLESEPARATOR |               /* horizontal line             */
     CA_TITLECENTER    |               /* center title                */
     CA_DRAWICON;                      /* use icons, not bitmaps      */

  SendMessage (hwndContainer,          /* ... set view                */
               CM_SETCNRINFO,
               (WPARAM)&cnrinfo,                                        /*DAB3*/
               (LPARAM)CMA_FLWINDOWATTR );                              /*DAB3*/
}                                      /* end of function             */
                                       /* createTextView              */


/**********************************************************************/
/*  Function:  createTreeIconView                                     */
/*  Inputs:    hwndContainer        container window handle           */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This constructor function is called when a container is being     */
/*  changed into text view.                                           */
/**********************************************************************/

void createTreeIconView (HWND hwndContainer)

  /********************************************************************/
  /* creates tree view for the container                              */
  /********************************************************************/

{
  CNRINFO cnrinfo;                     /* container info. structure   */

  cnrinfo.cb = sizeof(CNRINFO);        /* structure size              */
  cnrinfo.flWindowAttr =
     CV_ICON           |               /* CV_ICON+CV_TREE             */
     CV_TREE           |               /* == tree-icon view           */
     CA_CONTAINERTITLE |               /* container title             */
     CA_TITLESEPARATOR |               /* horizontal line             */
     CA_TITLECENTER    |               /* center title                */
     CA_DRAWICON       |               /* use icons, not bitmaps      */
     CA_OWNERDRAW      |               /* so we draw tree bitmaps     */
     CA_TREELINE;                      /* vertical line to left of    */
                                       /* hierarchy                   */

  /********************************************************************/
  /* The tree line is the vertical line extending from the parent     */
  /* down to each of its child nodes. If CV_DRAWICON is specified,    */
  /* causing container to draw icons for each item, then it will only */
  /* use icons when drawing the tree expanded/collapsed items.        */
  /* However, since we want to use bitmaps for these, we must owner   */
  /* draw them ourselves                                              */
  /********************************************************************/

  SendMessage (hwndContainer,          /* ... set view                */
               CM_SETCNRINFO,
               (WPARAM)&cnrinfo,                                        /*DAB3*/
               (LPARAM)CMA_FLWINDOWATTR );                              /*DAB3*/
}                                      /* end of function             */
                                       /* createTreeIconView          */


/**********************************************************************/
/*  Function:  rtJustCharStringAt                                     */
/*  Inputs:    hDC             device context for page                */
/*             lpptEnd         point to justify against               */
/*             lLength         number of characters in text string    */
/*             pText           pointer to string to justify           */
/*  Outputs:   return value from TextOut                              */
/*                                                                    */
/*  This function right justifies text at a given point.              */
/**********************************************************************/

long rtJustCharStringAt (HDC hDC, LPPOINT lpptEnd, long lLength, char *pText)

  /********************************************************************/
  /* computes the start point for text and right justifies it         */
  /********************************************************************/

{
  POINT  ptStart;                      /* where text should start     */
  SIZE   dwExtentText;                 /* width and height of text    */

  /********************************************************************/
  /* find out how long the text will be when printed and calculate    */
  /* the appropriate start position                                   */
  /********************************************************************/
//dwExtentText = GetTextExtent (hDC, pText, (int)lLength);
  GetTextExtentPoint( hDC, (LPSTR)pText, (int)lLength, &dwExtentText ); /*DAB1*/

  /********************************************************************/
  /* start point is end point moved to the left a distance equal to   */
  /* the length of the text                                           */
  /********************************************************************/
  ptStart.x = lpptEnd->x - dwExtentText.cx;                             /*DAB1*/
  ptStart.y = lpptEnd->y;

  /********************************************************************/
  /* and draw the text                                                */
  /********************************************************************/

  return (TextOut (hDC, ptStart.x, ptStart.y, pText, (int)lLength));
}                                      /* end of function             */
                                       /* rtJustCharStringAt          */


/**********************************************************************/
/*  Function:  childCount                                             */
/*  Inputs:    wIndex               index of item in global data      */
/*  Outputs:   number of people in the given department; -1 if this   */
/*             item is not a department                               */
/*                                                                    */
/*  This function searches the global data for members of this        */
/*  department and returns a count.                                   */
/**********************************************************************/

int childCount (WORD wIndex)
{
  WORD  wCount;                        /* child count                 */
  WORD  wLoop;                         /* loop index                  */

  /********************************************************************/
  /* if this is not a department, return -1                           */
  /********************************************************************/

  if (dataObjects[wIndex].wItemType != ITEM_TYPE_DEPT)
    return (-1L);

  wCount = 0;                          /* initialize counter          */

  /********************************************************************/
  /* loop through objects and search for a child that is a person     */
  /********************************************************************/

  for (wLoop = 0; wLoop < COUNT_OBJECT; wLoop++)
  {

    /******************************************************************/
    /* count this object if its parent is the given item and if it is */
    /* a person                                                       */
    /******************************************************************/

    if ((dataObjects[wLoop].wParentIdx == wIndex) &&
        (dataObjects[wLoop].wItemType == ITEM_TYPE_PERSON))
      wCount++;

  }                                    /* end of loop through records */

  return (wCount);                     /* return number of people in  */
                                       /* department                  */
}                                      /* end of function childCount  */



/**********************************************************************/
/*  Function:  RecordCoreCompare                                      */
/*  Inputs:    lp1                  first record to compare           */
/*             lp2                  second record to compare          */
/*             null                 pointer to storage (not used)     */
/*  Outputs:   -1 if lp1 <  lp2                                       */
/*             0  if lp1 == lp2                                       */
/*             1  if lp1 >  lp2                                       */
/*                                                                    */
/*  This function is a record sorting function.  It is called by      */
/*  container whenever it needs to sort records...the sort-records    */
/*  attribute is set when the container is created, and this function */
/*  is given as the comparison function.  The comparison here is      */
/*  simple:  we want to place departments before people inside the    */
/*  container; if both items are the same type, then place them in    */
/*  index order.                                                      */
/**********************************************************************/

int FAR PASCAL RecordCoreCompare (LPRECORDCORE lp1, LPRECORDCORE lp2,
                                  LPVOID null)

  /********************************************************************/
  /* sorting function for the container recordcore structures:        */
  /* departments are placed before people                             */
  /********************************************************************/

{
  WORD wIndex1 = getRecordIndex (lp1); /* first item index            */
  WORD wIndex2 = getRecordIndex (lp2); /* second item index           */

  null = 0;                            /* dummy statement             */

  /********************************************************************/
  /* if first item is department and second one is a person, then     */
  /* first record precedes second                                     */
  /********************************************************************/

  if ((dataObjects[wIndex1].wItemType == ITEM_TYPE_DEPT) &&
      (dataObjects[wIndex2].wItemType == ITEM_TYPE_PERSON))
    return (-1L);

  /********************************************************************/
  /* if second item is department and first one is a person, then     */
  /* second record precedes first                                     */
  /********************************************************************/

  if ((dataObjects[wIndex2].wItemType == ITEM_TYPE_DEPT) &&
      (dataObjects[wIndex1].wItemType == ITEM_TYPE_PERSON))
    return (1L);

  /********************************************************************/
  /* otherwise, base it on index order                                */
  /********************************************************************/

  if (wIndex1 < wIndex2)
    return (-1L);
  else
    return (1L);
}                                      /* end of RecordCoreCompare    */
                                       /* sorting function            */
