// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
/* Revision: 31 1.2.2.3 source/ui/cclnt/cslider/dial.c, cclcslid, ioc.v400, 000919  */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*                           Windows Modifications                          */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#define DRAW_MINUS       1
#define DRAW_PLUS        2
#define DRO_OUTLINE      1
#define DRO_FILL         2
#define DRO_OUTLINEFILL  3
#define DBM_NORMAL       0x0000
#define DBM_INVERT       0x0001
#define DBM_HALFTONE     0x0002
#define DBM_STRETCH      0x0004
#define DBM_IMAGEATTRS   0x0008

#include <math.h>
#include <limits.h>
#include <stdlib.h>
#include <malloc.h>
#include <windows.h>
#include <iclccl.h>
#include "iclcsldw.h"
#include "dial.h"

//**********************************************************************
// Function:  DialWndProc
//            Dial control main proc.
//
// Inputs: Window handle
//         message
//         parameter 1
//         parameter 2
//
// Output: MPARAM
//
//**********************************************************************
LRESULT CALLBACK DialWndProc (HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
{
   PDIAL         pDial;
   POINT         ptMouse;
   PCSBITMAPDATA pCSBitmapData;
   PCHAR         p;
   HDC           hdc;
   LONG          rc = 0;

   // Get dial's instance data
   pDial = (PDIAL) GetWindowLong(hwnd, GWL_CSLIDER);

   switch (msg)
   {


     case WM_CREATE:
       //  Create the Dial Control
       rc = DialCreate (hwnd, mp1, mp2);
       break;

     case WM_DESTROY:
       //  Destroy Dial control
       rc = DialDestroy (hwnd, pDial, mp2);
       break;

     case WM_KILLFOCUS:
       pDial->fState &= ~DL_FOCUS;
       DialDrawCursor (hwnd, pDial, FALSE);

       ReleaseDC( hwnd, hdc );

       // Send notification message
       SENDMESSAGE (GetParent( hwnd ),
                    WM_COMMAND,
                    MAKEWPARAM(GetWindowLong( hwnd, GWL_ID ), CSN_KILLFOCUS),
                    (LPARAM)hwnd);
       break;


     case WM_SETFOCUS:
       pDial->fState |= DL_FOCUS;

       DialDrawCursor (hwnd, pDial, TRUE);

       // Send notification message
       SENDMESSAGE (GetParent( hwnd ),
                    WM_COMMAND,
                    MAKEWPARAM(GetWindowLong( hwnd, GWL_ID ), CSN_SETFOCUS),
                    (LPARAM)hwnd);
       break;


     case WM_ENABLE:

       // Enable/disable this dial control
       if ((BOOL)mp1)
       {         // Enabled
         pDial->lStyle &= ~WS_DISABLED;
       }
       else
       {                         // Disabled
         pDial->lStyle |= WS_DISABLED;
       }

       // Change visual State
       InvalidateRect (hwnd, 0, FALSE);
       break;

     case WM_SIZE:

       // Calc Dial Size
       DialSize (hwnd, pDial);
       break;

     case WM_SETTEXT:
       {
         LPSTR newText = (LPSTR)mp2;
         if (newText)
         {
           // Initialize text
           strncpy (pDial->szText, (PCHAR)newText, DL_TEXTLEN);

           // Force end of string
           pDial->szText[DL_TEXTLEN-1] = 0;

           InvalidateRect( hwnd, 0, FALSE );
         }
         rc = TRUE;
       }
       break;


     case WM_PAINT:

       // Paint dial control
       rc = DialPaint (hwnd, pDial);
       break;

     case WM_LBUTTONDOWN:
     case WM_RBUTTONDOWN:
     case WM_LBUTTONDBLCLK:
     case WM_RBUTTONDBLCLK:
     case WM_LBUTTONUP:
     case WM_RBUTTONUP:
     case WM_MOUSEMOVE:

       // All mouse moves are handled the same way
       if (pDial->lStyle & WS_DISABLED)
       {
         MessageBeep( 0xFFFFFFFF );
         rc = FALSE;
       }
       else
       {
         ptMouse.x = LOWORD( mp2 );
         ptMouse.y = HIWORD( mp2 );
         rc = DialButton1 (hwnd, msg, pDial, &ptMouse);
       }
       break;

     case WM_TIMER:
       // All mouse moves are handled the same way
       if (pDial->lStyle & WS_DISABLED)
       {
         MessageBeep( 0xFFFFFFFF );
         rc = FALSE;
       }
       else
       {
         GetCursorPos( &ptMouse );
         MapWindowPoints( HWND_DESKTOP, hwnd, &ptMouse, 1 );
         rc = DialButton1 (hwnd, msg, pDial, &ptMouse);
       }
       break;


     case WM_KEYDOWN:
       // Process dial character input
       rc = DialChar (hwnd, pDial, mp1, mp2);
       break;

#if 0
     case WM_MATCHMNEMONIC:

       // Does a typed char match a mnemonic in the window text?
       p = (PCHAR)CTLstrchr(pDial->szText, '~');
       if (p)
       {
         //
         if (toupper(*(p+1)) == toupper((CHAR)(SHORT1FROMMP(mp1))))
         {
           rc = ((MRESULT)TRUE);
         }
       }
       break;
#endif


     case CSM_QUERYRANGE:

       // Query the value range of the dial control
       if ((mp1 == 0) || (mp2 == 0) )
       {
         rc = FALSE;
       }

       *((PSHORT) mp1) = (SHORT)pDial->Low;
       *((PSHORT) mp2) = (SHORT)pDial->High;
       rc = TRUE;
       break;


     case CSM_SETRANGE:
       {
       SHORT lowRange = mp1;
       SHORT highRange = mp2;
       // Sets the logical ranges of values for this dial control
       if ((lowRange == 0 && highRange == 0) ||
           (lowRange == highRange) ||
           (highRange < lowRange))
       {
         rc = FALSE;
       }
       else
       {
         pDial->Low = lowRange;
         pDial->High = highRange;
         InvalidateRect (hwnd, 0, FALSE);
         rc = TRUE;
       }
       }
       break;


      case CSM_QUERYVALUE:

         // Query the current dial value
         if (((PSHORT)(mp1)) == 0)
           {
             rc = FALSE;
           }

         *((PSHORT)  mp1) = (SHORT) pDial->Value;
         rc = TRUE;
         break;


      case CSM_SETVALUE:

         // Set the value of the circular slider control
         if ( ((LONG)mp1 > pDial->High) ||
              ((LONG)mp1 <  pDial->Low))
           {
             rc = FALSE;
           }
         else
           {
             CSDIALPOS posDial;

             pDial->Value = (LONG)mp1;
             hdc = GetDC(hwnd);

             // Notification
             DialDrawValue (hdc, pDial, 0);
             ReleaseDC(hwnd, hdc);

             posDial.position    = pDial->Value;
             posDial.hwndCSlider = pDial->hwnd;

             // Notify owner of change
             SENDMESSAGE (GetParent (pDial->hwnd),
                         WM_COMMAND,
                         MAKEWPARAM(GetWindowLong( hwnd, GWL_ID ), CSN_CHANGED),
                         (LPARAM)&posDial );

             // removed sOldValue from here per VISTA results dead code
             // InvalidateRect (hwnd, 0, FALSE);
             rc = TRUE;
           }
         break;


      case CSM_QUERYRADIUS:

         // Queries the current radius of the circular slider
         if ((PSHORT)mp1 == 0)
           {
             rc = FALSE;
           }
          *((PUSHORT) mp1) = (SHORT)pDial->uRadius;
          rc = TRUE;
         break;



      case CSM_SETINCREMENT:
          {
          LONG scrollIncr = (LONG)mp1;
          LONG tickIncr = (LONG)mp2;
          // Sets the scroll and tick mark increments of the control
          if ((scrollIncr <= 0) || (tickIncr <= 0) ||
              (tickIncr > ((SHORT)(pDial->High - pDial->Low))) ||
              (scrollIncr > ((SHORT)(pDial->High - pDial->Low))))
            {
              rc = FALSE;
            }
          else
            {
              pDial->Incre = scrollIncr;
              pDial->Tick  = tickIncr;
              InvalidateRect (hwnd, 0, FALSE);
              rc = TRUE;
            }
         }
         break;



      case CSM_QUERYINCREMENT:

         // Queries the increments used to scroll the value and
         // draws the tick marks
         if (((PSHORT)mp1 == 0) || ((PSHORT)mp2 == 0) )
           {
             rc = FALSE;
           }

         *((PUSHORT) mp1) = (USHORT) pDial->Incre;
         *((PUSHORT) mp2) = (USHORT) pDial->Tick;
         rc = TRUE;
         break;


      case CSM_SETBITMAPDATA:

         // Replaceable bit maps used for the plus/minus buttons
         // Optimal size for these bitmaps is 10x10 pels
         if ((PVOID)mp1 == 0)
           {
             rc = FALSE;
           }

           else
           {
             pCSBitmapData = (PCSBITMAPDATA) mp1;

             pDial->hbmLeftUp    = pCSBitmapData->hbmLeftUp;
             pDial->hbmLeftDown  = pCSBitmapData->hbmLeftDown;
             pDial->hbmRightUp   = pCSBitmapData->hbmRightUp;
             pDial->hbmRightDown = pCSBitmapData->hbmRightDown;
             pDial->fState      |= DL_BITMAPS;

             InvalidateRect (hwnd, 0, FALSE);

             rc = TRUE;
           }
         break;

   default:

     // Otherwise, default window procedure
     return DefWindowProc (hwnd, msg, mp1, mp2);
   }

   return(rc);

}

//**********************************************************************
// Function:  DialCreate
//            Create the dial control
//
// Inputs: Window handle
//         parameter 1
//         parameter 2
//
// Output: ULONG
//
//**********************************************************************
LONG DialCreate (HWND hwnd, WPARAM mp1, LPARAM mp2)
{
  HWND  hwndOwner;                            // Window Owner
  PDIAL pDial;                                // Dial Instance Data
  ULONG i;                                    // Index
  LPCREATESTRUCT pCS;                         // Create Struct

   //mp1;     unreferenced

   // Allocate control data for the dial window

   if ( !(pDial = (PDIAL)malloc(sizeof(DIAL))) )
   {
     // Sorry can't allocate memory for this control
     return (TRUE);
   }

   // Initialize data block to NULL
   memset((PUCHAR)pDial, '\0', sizeof(DIAL));

   SetWindowLong(hwnd, GWL_CSLIDER, (LONG)pDial);

   // Set Defaults

   pDial->lStyle = GetWindowLong (hwnd, GWL_STYLE);
   pDial->hwnd  = hwnd;           // Window Handle
   pDial->Incre = 1;              // Scroll Increment
   pDial->Tick = 3;               // Tick Mark Increment
   pDial->Low = 0;                // Low Bound

   if (pDial->lStyle & CSS_360)
   {
      pDial->lStyle |= CSS_NOBUTTON;
      pDial->lStyle |= CSS_NONUMBER;
      pDial->nPoints = 60;
      pDial->High = 60;
      pDial->Low = 0;
      pDial->Value = 30;
   }
   else if (pDial->lStyle & CSS_NOBUTTON)
   {
      pDial->nPoints = 42;                // Physical Points
      pDial->High = 42;                   // High Bound
      pDial->Value = 21;                  // Value
   }
   else if (pDial->lStyle & CSS_NOTICKS && pDial->lStyle & CSS_CIRCULARVALUE)
   {
      pDial->nPoints = 42;                // Physical Points
      pDial->High = 42;                   // High Bound
      pDial->Value = 21;                  // Value
   }
   else
   {
      pDial->High = 30;
      pDial->nPoints = 30;
      pDial->Value = 15;
   }

   // Load Pointers
// pDial->hptrArrow = LoadCursor (0, IDC_ARROW);

   // Get Font Info

   DialQueryFontMetrics (pDial, hwnd);

   // Calculate Dial Size
   DialSize (hwnd, pDial);    // @BJA

   // Get Text

   if (mp2) {
      pCS = (LPCREATESTRUCT) mp2;
      if (pCS->lpszName != NULL)
        {
          // Initialize text
          strncpy (pDial->szText, (PCHAR)pCS->lpszName, DL_TEXTLEN);

          // Force end of string
          pDial->szText[DL_TEXTLEN-1] = 0;
        }
   }

   return (FALSE);    // Success

}

//**********************************************************************
// Function:  DialDestroy
//            Destroy the dial control
//
// Inputs: Window handle
//         pointer to dial structure
//         parameter 2
//
// Output: ULONG
//
//**********************************************************************
LONG DialDestroy (HWND hwnd, PDIAL pDial, LPARAM mp2)
{

   KillTimer( hwnd, 1 );

   // Release allocated memory
   free( pDial );

   return (FALSE);

}

//**********************************************************************
// Function:  DialSize
//            Size of the dial control
//
// Inputs: Window handle
//         pointer to dial structure
//
// Output: ULONG
//
//**********************************************************************
LONG DialSize (HWND hwnd, PDIAL pDial)
{
  LONG  cx,cy,radius;
  RECT  rect;
  POINT newPoint;

  // Calc Metrics

  GetWindowRect( hwnd, &rect );

  cx = rect.right - rect.left;
  cy = rect.bottom - rect.top;

  if (!(pDial->lStyle & CSS_NOTEXT))
  {
    cy -= pDial->cyFont;                // Calculate Dial Area
  }

  // Which is the Minimum Dimension ?
  cx = min (cx, cy);

  if (pDial->lStyle & CSS_PROPORTIONALTICKS)
  {

    // Calculate the porportional tick marks

    pDial->DialTick = (cx/2)/3;

    if (pDial->DialTick < DIALTICKMIN )
    {
      pDial->DialTick = DIALTICKMIN;
    }

    if (pDial->DialTick > DIALTICKMAX)
    {
      pDial->DialTick = DIALTICKMAX;
    }

    pDial->DialTickLength = pDial->DialTick/2;

    if (pDial->DialTickLength < DIALTICKMINLEN)
    {
      pDial->DialTickLength = DIALTICKMINLEN;
    }
  }

  else
  {
    pDial->DialTick = DIALTICK;
  }

  radius = (cx / 2) - pDial->DialTick - 1;

  // Make sure it is positive

  if (radius < 1) radius = 1;

  pDial->uRadius = (ULONG) radius;

  return (TRUE);
}

//**********************************************************************
// Function:  DialPaint
//            Paint the dial control
//
// Inputs: Window handle
//         pointer to dial structure
//
// Output: ULONG
//
// 22-Aug-94  Graham Perks  Draw the ticks as lines not as individual pixels.
//**********************************************************************
LONG DialPaint (HWND hwnd, PDIAL pDial)
{
   HDC         hdc;
   PAINTSTRUCT ps;
   ULONG       j;
   LONG        i;
   POINT       pt;
   POINT       ptLine[2];
   ULONG       ulColor = 0;
   LONG        TWO = 2;
   LONG        THREE = 3;
   COLORREF    clrDial,
               clrShadow,
               clrLight,
               clrBorder,
               clrTextBackground,
               clrTextForeground;
   ULONG       usTickLen,
               usTick;
   LINEARCH    LineArch;
   static      WORD wSolid    [] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } ;
   static      WORD wHalftone [] = { 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA } ;
   HBITMAP     hbmSolid;
   HBITMAP     hbmHalftone;
   HBRUSH      solidBrush;
   HBRUSH      halftoneBrush;
   HBRUSH      oldBrush;
   HBRUSH      background = 0;
   HPEN        oldPen;
   HPEN        solidPen;

   hbmSolid      = CreateBitmap (8, 8, 1, 1, (LPSTR) wSolid) ;
   hbmHalftone   = CreateBitmap (8, 8, 1, 1, (LPSTR) wHalftone ) ;
   solidBrush    = CreatePatternBrush (hbmSolid);
   halftoneBrush = CreatePatternBrush (hbmHalftone);


   // Force Cursor off
   DialDrawCursor (hwnd, pDial, FALSE);

   // If dial is disabled, set it to these colors

   if (pDial->lStyle & WS_DISABLED)
   {
      clrDial   = RGB(192,192,192);
      clrShadow = RGB(192,192,192);
      clrLight  = RGB(255,255,255);
      clrBorder = RGB(128,128,128);
   }
   else  // Dial not disable. Use these colors instead
   {
      clrDial   = RGB(192,192,192);
      clrShadow = RGB(128,128,128);
      clrLight  = RGB(255,255,255);
      clrBorder = RGB(0,0,0);
   }

  hdc = BeginPaint (hwnd, &ps);


  // Get dial size

  GetWindowRect (hwnd, &pDial->rclWindow);
  pDial->rclWindow.bottom = pDial->rclWindow.bottom - pDial->rclWindow.top;
  pDial->rclWindow.right  = pDial->rclWindow.right - pDial->rclWindow.left;
  pDial->rclWindow.top    = 0;
  pDial->rclWindow.left   = 0;

  pDial->ptCenter.x = (pDial->rclWindow.right - pDial->rclWindow.left) / 2;
  pDial->ptCenter.y = pDial->rclWindow.bottom - pDial->DialTick - pDial->uRadius;

  if (!(pDial->lStyle & CSS_NOTEXT))
    pDial->ptCenter.y -= pDial->cyFont;

  pDial->ptValue = pDial->ptCenter;

  // Paint Background
  background = (HBRUSH)SendMessage( hwnd, WM_CTLCOLORCSLIDER, hdc, hwnd );

  if (background)
  {
    FillRect( hdc, &pDial->rclWindow, background );
    clrTextBackground = GetBkColor( hdc );
    clrTextForeground = GetTextColor( hdc );
  }
  else
  {
    FillRect( hdc, &pDial->rclWindow, (HBRUSH)(COLOR_WINDOW+1) );
    clrTextBackground = GetSysColor( COLOR_WINDOW );
    clrTextForeground = GetSysColor( COLOR_WINDOWTEXT );
  }
  pDial->backgroundColor = clrTextBackground;

  // Paint Dial Background
  solidPen = CreatePen( PS_SOLID, 1, clrDial );

  MoveToEx(hdc, pDial->ptCenter.x, pDial->ptCenter.y, 0 );

  if (pDial->lStyle & WS_DISABLED)
  {      // Fill Center
    oldBrush = SelectObject( hdc, halftoneBrush );
  }
  else
  {
    oldBrush = SelectObject( hdc, solidBrush );
  }

  SetTextColor( hdc, clrDial );
  oldPen = SelectObject( hdc, solidPen );

  // Center
  FullArc (hdc, DRO_FILL, pDial->uRadius+1);

  // Draw 3-Dimension Stuff : Dark Shadow low-right

  // Position to point on radious and draw

  LineArch.fxMultiplier1 = pDial->uRadius;
  LineArch.fxStartAngle1 = 235;
  LineArch.flColor       = TRUE;
  LineArch.iColor        = clrShadow;
  LineArch.fxMultiplier2 = pDial->uRadius;
  LineArch.fxStartAngle2 = 235;

  SetTextColor( hdc, clrShadow );
  SelectObject( hdc, oldPen );
  DeleteObject( solidPen );

  solidPen = CreatePen( PS_SOLID, 2, clrShadow );
  oldPen = SelectObject( hdc, solidPen );

  DialLineArch (hdc, &pDial->ptCenter, &pt, &LineArch);

  MoveToEx (hdc, pt.x, pt.y, 0);

  PartialArc (hdc, &pDial->ptCenter, pDial->uRadius-1, 245, 150);

  SelectObject( hdc, oldPen );
  DeleteObject( solidPen );
  solidPen = CreatePen( PS_SOLID, 1, clrShadow );
  oldPen = SelectObject( hdc, solidPen );

  MoveToEx (hdc, pt.x, pt.y, 0);

  PartialArc (hdc, &pDial->ptCenter, pDial->uRadius-2, 255, 140);

  // Draw 3-Dimension Stuff : White Light upper-left

  // Position to point on radious and draw

  LineArch.fxStartAngle1 = 55;
  LineArch.iColor        = clrLight;
  LineArch.fxStartAngle2 = 55;

  SetTextColor( hdc, clrLight );
  SelectObject( hdc, oldPen );
  DeleteObject( solidPen );
  solidPen = CreatePen( PS_SOLID, 2, clrLight );
  oldPen = SelectObject( hdc, solidPen );

  DialLineArch (hdc, &pDial->ptCenter, &pt, &LineArch );

  MoveToEx (hdc, pt.x, pt.y, 0);
  PartialArc (hdc, &pDial->ptCenter, pDial->uRadius-1, 65, 150);

  SelectObject( hdc, oldPen );
  DeleteObject( solidPen );
  solidPen = CreatePen( PS_SOLID, 1, clrLight );
  oldPen = SelectObject( hdc, solidPen );

  MoveToEx (hdc, pt.x, pt.y, 0);

  PartialArc (hdc, &pDial->ptCenter, pDial->uRadius-2, 75, 140);

  // Boundary

  MoveToEx (hdc, pDial->ptCenter.x, pDial->ptCenter.y, 0);
  SelectObject( hdc, solidBrush );
  SetTextColor( hdc, clrBorder );

  SelectObject( hdc, oldPen );
  DeleteObject( solidPen );
  solidPen = CreatePen( PS_SOLID, 2, clrBorder );
  oldPen = SelectObject( hdc, solidPen );


  // Specify boundary
  FullArc (hdc, DRO_OUTLINE, pDial->uRadius+1);

  SelectObject( hdc, oldPen );
  DeleteObject( solidPen );

//solidPen = CreatePen( PS_SOLID, 1, clrBorder );

  solidPen = CreatePen( PS_SOLID, 1, clrTextForeground );

  oldPen = SelectObject( hdc, solidPen );

  // Draw Tick Marks

  if (pDial->lStyle & CSS_PROPORTIONALTICKS)
  {
    usTick = (pDial->DialTick - pDial->DialTickLength);
    usTickLen = pDial->DialTick;
  }
  else
  {
    usTick = DIALTICK - 8;
    usTickLen = DIALTICK;
  }

  // Dial uses tick marks
  if (!(pDial->lStyle & CSS_NOTICKS))
  {
    if (pDial->lStyle & CSS_MIDPOINT)
    {
      for (i=pDial->Low; i<=pDial->High; i+=pDial->Tick)
      {
        // Check to see what tick your at.  If you are at the
        // left or right or middle make the tick a little longer

        // Bad assumption: If range has an odd number of increments
        // then the mid point won't draw!

        // Get coordinate of first end-point.
        DialGetPoint (pDial, &pDial->ptCenter, &ptLine[0], i, pDial->uRadius + usTick);

        // Get coordinate of the other end-point.
        if ((i == pDial->Low) || (i == pDial->High) ||
             i == ((pDial->High + pDial->Low)/TWO))
        {
          DialGetPoint (pDial, &pDial->ptCenter, &ptLine[0], i, pDial->uRadius + usTick);
          DialGetPoint (pDial, &pDial->ptCenter, &ptLine[1], i, pDial->uRadius + usTickLen + THREE);
        }
        else
        {
          DialGetPoint (pDial, &pDial->ptCenter, &ptLine[0], i, pDial->uRadius + usTick);
          DialGetPoint (pDial, &pDial->ptCenter, &ptLine[1], i, pDial->uRadius + usTickLen);
        }

        // Draw the line.
        MoveToEx( hdc, ptLine[0].x, ptLine[0].y, 0 );
        LineTo( hdc, ptLine[1].x, ptLine[1].y );
      }
    }
    else   // Dial does not use tick marks
    {
      for (i=pDial->Low; i<=pDial->High; i+=pDial->Tick)
      {
        // Calculate line end-points and join them up.
        DialGetPoint (pDial, &pDial->ptCenter, &ptLine[0], i, pDial->uRadius + usTick);
        DialGetPoint (pDial, &pDial->ptCenter, &ptLine[1], i, pDial->uRadius + usTickLen);
        MoveToEx( hdc, ptLine[0].x, ptLine[0].y, 0 );
        LineTo( hdc, ptLine[1].x, ptLine[1].y );
      }
    }
  }

  // Draw the Value Mark

  DialDrawValue (hdc, pDial, 0);          // Draw Value
  DialDrawSign (hdc, pDial, (ULONG)DRAW_MINUS);  // Draw Left
  DialDrawSign (hdc, pDial, (ULONG)DRAW_PLUS);   // Draw Right

  // Reset the text background color
  SetBkColor( hdc, clrTextBackground );
  SetTextColor( hdc, clrTextForeground );
  DialDrawText (hdc, pDial);

  SelectObject( hdc, oldPen );
  SelectObject( hdc, oldBrush );
  DeleteObject( solidBrush );
  DeleteObject( halftoneBrush );;
  DeleteObject( solidPen );;
  DeleteObject( hbmSolid );
  DeleteObject( hbmHalftone );

  EndPaint (hwnd, &ps);

  // Force cursor on
  DialDrawCursor (hwnd, pDial, TRUE);

  return (0L);
}



//**********************************************************************
// Function:  DialDrawSign
//            Draw user defined bitmap or draw plus/minus sign in the
//            correct position or don't draw them at all
//
// Inputs: Handle to picture space
//         pointer to dial structure
//         Right/Left indicator
//
// Output: ULONG
//
//**********************************************************************
LONG DialDrawSign(HDC hdc, PDIAL pDial, ULONG ulSign)
{
  RECT rcl, rclSign;
  HBITMAP hbm = 0;

  // Relative size of button
  USHORT d = pDial->DialTick-1;

  if ((d & 1) == 0)
  {
    d++;
  }

  //  Check to see if there are user defined bitmaps
  if (pDial->fState & DL_BITMAPS)
  {
    // Left sign (minus)
    if (ulSign == (ULONG)DRAW_MINUS)
    {

      if (pDial->fState & DL_LEFTDOWN)
      {
        hbm = pDial->hbmLeftDown;
      }
      else
      {
        hbm = pDial->hbmLeftUp;
      }
    }

    // Right sign (plus)
    if (ulSign == (ULONG)DRAW_PLUS)
    {

      if (pDial->fState & DL_RIGHTDOWN)
      {
        hbm = pDial->hbmRightDown;
      }
      else
      {
        hbm = pDial->hbmRightUp;
      }
    }

    if (pDial->lStyle & CSS_NOBUTTON)
    {
      rcl.left = rcl.right = rcl.bottom = rcl.top = 0L;
    }
    else
    {

      if (ulSign == (ULONG)DRAW_MINUS)
        rcl.left  = pDial->ptCenter.x - pDial->uRadius - pDial->DialTick;
      else
        rcl.left  = pDial->ptCenter.x + pDial->uRadius + pDial->DialTick - d +1;

      rcl.right = rcl.left + d;
      rcl.bottom = pDial->ptCenter.y + pDial->uRadius + (pDial->DialTick/2);

      rcl.top   = rcl.bottom - d;
      DrawBitmap (hdc, hbm, 0, &rcl, DBM_NORMAL | DBM_STRETCH);
    }
  }
       // There are no bitmaps so we will draw the plus and minus signs
  else
  {
    if (pDial->lStyle & CSS_NOBUTTON)
    {
      rcl.left = rcl.right = rcl.bottom = rcl.top = 0L;
    }
    else
    {
      if (ulSign == (ULONG)DRAW_MINUS)
        rcl.left  = pDial->ptCenter.x - pDial->uRadius - pDial->DialTick;
      else
        rcl.left  = pDial->ptCenter.x + pDial->uRadius + pDial->DialTick - d +1;

      rcl.right = rcl.left + d;
      rcl.bottom = pDial->ptCenter.y + pDial->uRadius + (pDial->DialTick/2);
      rcl.top   = rcl.bottom - d;

      rclSign = rcl;

      if ( ((pDial->fState & DL_LEFTDOWN) && ( ulSign == (ULONG)DRAW_MINUS)) ||
           ((pDial->fState & DL_RIGHTDOWN) && ( ulSign == (ULONG)DRAW_PLUS)) )
      {
        // Draw the plus/minus sign in the down position

        FillRect (hdc, &rcl, GetStockObject( LTGRAY_BRUSH ));

        rclSign = rcl;
        rclSign.right = rclSign.left+1;
        FillRect (hdc, &rclSign, GetStockObject( DKGRAY_BRUSH ));

        rclSign = rcl;
        rclSign.bottom = rclSign.top+1;
        FillRect (hdc, &rclSign, GetStockObject( DKGRAY_BRUSH ));

        rclSign = rcl;

        if (ulSign == (ULONG)DRAW_MINUS)
          rclSign.top = ((rclSign.bottom -2) + (rclSign.top + 1))/2;
        else
          rclSign.top = ((rclSign.bottom -2) + (rclSign.top + 2))/2;
        rclSign.top = rclSign.top + 1;

        rclSign.bottom = rclSign.top + 1;
        rclSign.left = rclSign.left+2;
        rclSign.right = rclSign.right-2;
        FillRect (hdc, &rclSign, GetStockObject( BLACK_BRUSH ));

        if (ulSign == (ULONG)DRAW_PLUS)
        {
          rclSign = rcl;
          rclSign.right = ((rclSign.right - 2) + (rclSign.left + 2))/2 +1;
          rclSign.left = rclSign.right -1;
          rclSign.bottom = rclSign.bottom - 1;
          rclSign.top = rclSign.top + 3;
          FillRect (hdc, &rclSign, GetStockObject( BLACK_BRUSH ));
        }
      }
      else
      {
        // Draw the plus/minus sign in the up position

        FillRect (hdc, &rcl, GetStockObject( LTGRAY_BRUSH ));

        rclSign = rcl;
        rclSign.right = rclSign.left+1;
        FillRect (hdc, &rclSign, GetStockObject( WHITE_BRUSH ));

        rclSign = rcl;
        rclSign.bottom = rclSign.top + 1;
        FillRect (hdc, &rclSign, GetStockObject( WHITE_BRUSH ));

        rclSign = rcl;
        rclSign.top = rclSign.bottom - 1;
        FillRect (hdc, &rclSign, GetStockObject( DKGRAY_BRUSH ));

        rclSign = rcl;
        rclSign.left = rclSign.right-1;
        FillRect (hdc, &rclSign, GetStockObject( DKGRAY_BRUSH ));

        rclSign = rcl;
        rclSign.bottom = ((rclSign.bottom -2) + (rclSign.top + 2))/2 +1;
        rclSign.top = rclSign.bottom - 1;
        rclSign.left = rclSign.left+2;
        rclSign.right = rclSign.right-2;
        FillRect (hdc, &rclSign, GetStockObject( BLACK_BRUSH ));

        if (ulSign == (ULONG)DRAW_PLUS)
        {

          rclSign = rcl;
          rclSign.right = ((rclSign.right - 2) + (rclSign.left + 2))/2 +1;
          rclSign.left = rclSign.right -1;
          rclSign.bottom = rclSign.bottom - 2;
          rclSign.top = rclSign.top + 2;
          FillRect (hdc, &rclSign, GetStockObject( BLACK_BRUSH ));
        }
      }
    }
  }

  if (ulSign == (ULONG)DRAW_MINUS)
    pDial->rcl1 = rcl;
  else
    pDial->rcl2 = rcl;

  return (0);
}


//**********************************************************************
// Function:  DialDrawValue
//            Re-draw dial with new value
//
// Inputs: Handle to picture space
//         pointer to dial structure
//         increment
//
// Output: ULONG
//
//**********************************************************************
LONG DialDrawValue (HDC hdc, PDIAL pDial, LONG incre)
{
  RECT     rcl;
  CHAR     szValue[6];
  ULONG    cxmax,cxmin;
  POINT    pt;
  POINT    ptNewValue;
  ULONG    x;
  COLORREF lineColor;
  LONG     ulForeClr;
  LINEARCH LineArch;
  HPEN     oldPen = 0;
  HPEN     newPen;
#ifdef BIDI
  ULONG    ulBidiAttr = 0L;
#endif
  COLORREF  clrDial   = RGB(192,192,192);
  static   WORD wSolid [] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } ;
  HBITMAP  hbmSolid;
  HBRUSH   solidBrush;
  HBRUSH   oldBrush;

  hbmSolid      = CreateBitmap (8, 8, 1, 1, (LPSTR) wSolid) ;
  solidBrush    = CreatePatternBrush (hbmSolid);

  // Assume No Number
  pDial->fNumber = FALSE;

  pDial->Value += incre;

  // Check Range
  if (pDial->Value < pDial->Low)
    pDial->Value = pDial->Low;
  if (pDial->Value > pDial->High)
    pDial->Value = pDial->High;

  // Notify Owner

  if (incre)
  {
    CSDIALPOS posDial;
    posDial.position    = pDial->Value;
    posDial.hwndCSlider = pDial->hwnd;

    // Non-Zero delta
    SENDMESSAGE (GetParent(pDial->hwnd),
                 WM_COMMAND,
                 MAKEWPARAM(GetWindowLong(pDial->hwnd, GWL_ID), CSN_TRACKING),
                 (LPARAM)&posDial );
  }

  // Draw a circular thumb or a line for value indicator

  if (pDial->lStyle & CSS_CIRCULARVALUE )
  {
    DialGetPoint (pDial,
                  &pDial->ptCenter,
                  &ptNewValue,
                  (LONG) pDial->Value,                  // Value
                  (LONG) ((3*(pDial->uRadius))/4));     // Value Marker


    // This control data will not change for function

    LineArch.fxMultiplier1 =  (pDial->uRadius)/8;
    LineArch.fxMultiplier2 =  (pDial->uRadius)/8;

    oldBrush = SelectObject( hdc, solidBrush );
    if ((ptNewValue.x != pDial->ptValue.x || ptNewValue.y != pDial->ptValue.y) &&
        (pDial->ptValue.x != pDial->ptCenter.x || pDial->ptValue.y != pDial->ptCenter.y))
    {

      // Erase Circle
      MoveToEx( hdc, pDial->ptValue.x, pDial->ptValue.y, 0 );
      SetTextColor( hdc, clrDial );
      newPen = CreatePen( PS_SOLID, 1, clrDial );
      oldPen = SelectObject( hdc, newPen );

      // Specify boundary
      FullArc (hdc, DRO_OUTLINEFILL, (pDial->uRadius)/8 );

      LineArch.fxMultiplier1 =  (pDial->uRadius)/8 - 1;
      LineArch.fxMultiplier2 =  (pDial->uRadius)/8 - 1;

      // Erase highlight low-right

      LineArch.fxStartAngle1 =  235;
      LineArch.flColor       =  FALSE;
      LineArch.iColor        =  0;
      LineArch.fxStartAngle2 =  235;

      SelectObject( hdc, oldPen );
      DeleteObject( newPen );
      newPen = CreatePen( PS_SOLID, 2, clrDial );
      oldPen = SelectObject( hdc, newPen );

      DialLineArch (hdc, &pDial->ptValue, &pt, &LineArch );

      // Erase shadow upper-left

      LineArch.fxStartAngle1 = 55;
      LineArch.fxStartAngle2 = 55;


      DialLineArch (hdc, &pDial->ptValue, &pt, &LineArch);
    }

    // Draw New Circle
    LineArch.fxMultiplier1 =  (pDial->uRadius)/8;
    LineArch.fxMultiplier2 =  (pDial->uRadius)/8;

    pDial->ptValue = ptNewValue;

    // Set Modes
    MoveToEx( hdc, pDial->ptValue.x, pDial->ptValue.y, 0 );

    if (pDial->lStyle & WS_DISABLED)
    {
      lineColor = RGB( 128, 128, 128 );
    }
    else
    {
      lineColor = RGB( 192, 192, 192 );
    }

    SetTextColor( hdc, lineColor );
    if (oldPen)
    {
      SelectObject( hdc, oldPen );
      DeleteObject( newPen );
    }

    newPen = CreatePen( PS_SOLID, 1, lineColor );
    oldPen = SelectObject( hdc, newPen );

    // Specify boundary
    FullArc (hdc, DRO_OUTLINEFILL, (pDial->uRadius)/8 );

    if (pDial->lStyle & WS_DISABLED)
    {
      lineColor = RGB( 128, 128, 128 );
    }
    else
    {
      lineColor = PALETTERGB( 96, 96, 96 );
    }

    SelectObject( hdc, oldPen );
    DeleteObject( newPen );
    newPen = CreatePen( PS_SOLID, 1, lineColor );
    oldPen = SelectObject( hdc, newPen );

    // Specify boundary
    FullArc (hdc, DRO_OUTLINE, (pDial->uRadius)/8);

    // Draw New 3D indent on thumb
    LineArch.fxMultiplier1 =  (pDial->uRadius)/8 - 1;
    LineArch.fxMultiplier2 =  (pDial->uRadius)/8 - 1;

    // Draw highlight low-right

    LineArch.fxStartAngle1 = 235;
    LineArch.flColor       = TRUE;
    LineArch.fxStartAngle2 = 235;


    lineColor = RGB( 255, 255, 255 );
    SelectObject( hdc, oldPen );
    DeleteObject( newPen );
    newPen = CreatePen( PS_SOLID, 2, lineColor );
    oldPen = SelectObject( hdc, newPen );

    DialLineArch (hdc, &pDial->ptValue, &pt, &LineArch );

    // Draw shadow upper-left

    LineArch.fxStartAngle1 = 55;
    LineArch.fxStartAngle2 = 55;

    lineColor = RGB( 128, 128, 128 );
    SelectObject( hdc, oldPen );
    DeleteObject( newPen );
    newPen = CreatePen( PS_SOLID, 2, lineColor );
    oldPen = SelectObject( hdc, newPen );

    DialLineArch (hdc, &pDial->ptValue, &pt, &LineArch );

    SelectObject( hdc, oldPen );
    DeleteObject( newPen );
    SelectObject( hdc, oldBrush );
    DeleteObject( solidBrush );
    DeleteObject( hbmSolid );
  }
  else      // line value indicator
  {

    DialGetPoint (pDial,
                  &pDial->ptCenter,
                  &ptNewValue,
                  (LONG) pDial->Value,                       // Value
                  (LONG) (pDial->uRadius-(ULONG)3));        // Value Marker

    if (ptNewValue.x != pDial->ptValue.x || ptNewValue.y != pDial->ptValue.y)
    {
      newPen = CreatePen( PS_SOLID, 1, RGB(192,192,192) );
      oldPen = SelectObject( hdc, newPen );
      MoveToEx(hdc, pDial->ptCenter.x, pDial->ptCenter.y, 0 );
      LineTo(hdc, pDial->ptValue.x, pDial->ptValue.y );
      SelectObject( hdc, oldPen );
      DeleteObject( newPen );
    }

    pDial->ptValue = ptNewValue;

    if (pDial->lStyle & WS_DISABLED)
    {
      lineColor = RGB( 128, 128, 128 );
    }
    else
    {
      // NOTE: can't draw the line in the foreground pres param color
      // because it may be a dithered color is the screen is in 16-color
      // mode and this is problems drawing a line of a dithered color
      // sometimes it doesn't draw at all, and sometimes you get the
      // wrong color
      lineColor = RGB( 0, 0, 0 );
    }

    newPen = CreatePen( PS_SOLID, 1, lineColor );
    oldPen = SelectObject( hdc, newPen );
    MoveToEx(hdc, pDial->ptCenter.x, pDial->ptCenter.y, 0 );
    LineTo(hdc, pDial->ptValue.x, pDial->ptValue.y );
    SelectObject( hdc, oldPen );
    DeleteObject( newPen );
  }
  // Draw Numeric Value

  if (pDial->lStyle & CSS_NONUMBER)
    return (0);

#ifdef BIDI
  if (fIsSysBidi)
  {
    ulBidiAttr = Call_WinQueryLangInfo(pDial->hwnd, LI_BD_WND_ATTR,
                                      LIF_NO_SENDMSG, 0L);
    Bidi_InitHps(hps);
  }
#endif

  // Field width
  _itoa (pDial->High, szValue, 10);

#ifdef BIDI
  if (fIsSysBidi && ulBidiAttr)
    ConvertBidiString(szValue, szValue, CTLstrlen(szValue),ulBidiAttr,NULL);
#endif //bidi

  cxmax = DialGetTextLength (hdc, (PSZ)szValue, 0);
  _itoa (pDial->Low, szValue, 10);

#ifdef BIDI
  if (fIsSysBidi && ulBidiAttr)
    ConvertBidiString(szValue, szValue, CTLstrlen(szValue),ulBidiAttr,NULL);
#endif //bidi

  cxmin = DialGetTextLength (hdc, (PSZ)szValue, 0);
  if (cxmin > cxmax)
    cxmax = cxmin;

  // Value
  _itoa (pDial->Value, szValue, 10);
#ifdef BIDI
  if (fIsSysBidi && ulBidiAttr)
     ConvertBidiString(szValue, szValue, CTLstrlen(szValue),ulBidiAttr,NULL);
#endif //bidi


  // Note: some comments have been made that developers don't like the
  // fact that if the text can't be drawn on the face of the dial that
  // it doesn't draw the text.  We can't change it because, if someone
  // has an app with a dial that doesn't use CSS_NONUMBER and it is small
  // and we decided to start drawing the number then they're app would
  // look bad.

  if (pDial->lStyle & CSS_CIRCULARVALUE)
  {

    // inside edge of value circle minus fudge
    x = 3*pDial->uRadius/4 - pDial->uRadius/8 - 4;

    // Calculate rectangle
    rcl.left  = pDial->ptCenter.x - (cxmax / 2);
    rcl.right = rcl.left + cxmax;
    rcl.bottom = pDial->ptCenter.y + (pDial->cyFont / 2);
    rcl.top = rcl.bottom - pDial->cyFont;

    if ( (cxmax/2) > x )
      return (0);    // No room for text

  }
  else
  {

    // No room for text
    if (pDial->uRadius-(ULONG)6 < (ULONG)pDial->cyFont)
      return (0);

    // Caculate rectangle
    rcl.left = pDial->ptCenter.x - (cxmax / 2);
    rcl.right = rcl.left + cxmax;
    rcl.bottom = pDial->ptCenter.y + pDial->uRadius -
                 ((pDial->uRadius - pDial->cyFont) / 2);
    rcl.top = rcl.bottom - pDial->cyFont;

  }

  FillRect (hdc, &rcl, GetStockObject( LTGRAY_BRUSH ));

  pDial->rclNumber = rcl;
  pDial->fNumber = TRUE;

  if (pDial->lStyle & WS_DISABLED)
    SetTextColor( hdc, RGB(128,128,128 ));
  else
    SetTextColor( hdc, GetSysColor( COLOR_BTNTEXT ));
  SetBkColor( hdc, RGB( 192, 192, 192 ));
  DrawText (hdc, (LPSTR)szValue, lstrlen(szValue), &rcl, DT_CENTER | DT_VCENTER );

  return (0);

}

//**********************************************************************
// Function:  DialDrawText
//            Draw dial text
//
// Inputs: Handle to picture space
//         pointer to dial structure
//
// Output: ULONG
//
//**********************************************************************
LONG DialDrawText (HDC hdc, PDIAL pDial)
{
   LONG   cch;
   LONG   cxText;
   LONG   cyText;
   LONG   cy;
   PSZ    pszText;

#ifdef BIDI
   #define MAX_CONV_TEXT 31
   CHAR   achConvText[MAX_CONV_TEXT+1];
   ULONG  ulBidiAttr;
#endif    //bidi

   pDial->fState &= ~DL_TEXT;                      /* Clear Text Flag */

   if (pDial->lStyle & CSS_NOTEXT)
     return (0);

   cch = lstrlen(pDial->szText);
   if (!cch)
     return (0);

   pDial->fState |= DL_TEXT;                       /* Set Text Flag */

   pszText = (PSZ)pDial->szText;                   /* Point to text */

#ifdef BIDI
   if (fIsSysBidi && cch < MAX_CONV_TEXT)
   {
      Bidi_InitHps(hps);
      ulBidiAttr = Call_WinQueryLangInfo(pDial->hwnd, LI_BD_WND_ATTR,
                                       LIF_NO_SENDMSG, 0L);
      ConvertBidiString(pszText, achConvText, cch, ulBidiAttr, NULL);
      pszText = achConvText;
   }
#endif //bidi

   cxText = DialGetTextLength (hdc, (PSZ)pszText, &cyText);

   cy = (LONG) (pDial->ptCenter.y + pDial->uRadius);
   if (pDial->lStyle & CSS_360)
   {
      pDial->rclText.bottom = 0;
   } else {
      pDial->rclText.bottom = max (0, (
                (pDial->rclWindow.bottom-(cy+cyText))/3));  /* lower third */
   }
   pDial->rclText.bottom = pDial->rclWindow.bottom - pDial->rclText.bottom;
   pDial->rclText.top  = pDial->rclText.bottom - cyText;
   pDial->rclText.left = pDial->ptCenter.x - (cxText / 2);
   pDial->rclText.right = pDial->rclText.left + cxText;

   /* use foreground color */

// Set the foreground and background color

   if (pDial->lStyle & WS_DISABLED)
     SetTextColor( hdc, RGB(128,128,128 ));
// else
//   SetTextColor( hdc, GetSysColor( COLOR_WINDOWTEXT ));

   DrawText (hdc, (LPSTR)pszText, cch,  &pDial->rclText,
             DT_CENTER | DT_BOTTOM | DT_SINGLELINE );

   return (0);
}

//**********************************************************************
// Function:  DialButton1
//            Handle new user input for dial
//
// Inputs: Handle to picture space
//         mesage
//         pointer to dial structure
//         pointer to pointer to mouse
//
// Output: ULONG
//
//**********************************************************************
LONG DialButton1 (HWND hwnd, ULONG msg, PDIAL pDial, PPOINT pptMouse)
{
  RECT     rcl3;
  HDC      hdc;
  HCURSOR  hptr;
  LONG     iDelta;

  rcl3.left   = pDial->ptCenter.x - pDial->uRadius;    // dial
  rcl3.right  = rcl3.left + pDial->uRadius * 2;
  rcl3.bottom = pDial->ptCenter.y + pDial->uRadius;
  rcl3.top    = rcl3.bottom - pDial->uRadius * 2;

  switch (msg)
  {
    case WM_LBUTTONDOWN:
    case WM_RBUTTONDOWN:
    case WM_LBUTTONDBLCLK:
    case WM_RBUTTONDBLCLK:
      pDial->fDrag1 = FALSE;
      pDial->fDrag2 = FALSE;

      if (((msg == WM_RBUTTONDOWN) || (msg == WM_RBUTTONDBLCLK)))
        pDial->fDrag2 = TRUE;
      if (((msg == WM_LBUTTONDOWN) || (msg == WM_LBUTTONDBLCLK)))
        pDial->fDrag1 = TRUE;

      /* Determine if the point is over a Tick Mark */
      if (DialTickSelect (hwnd, pDial, pptMouse))
      {
        pDial->fState &= ~(DL_LEFTDOWN | DL_RIGHTDOWN | DL_TRACKDIAL | DL_POINTSELECT);
        if (pDial->lStyle & CSS_POINTSELECT)
        {
          /* set the value immediately to the position */
          pDial->TrackValue = DialPointSelect (hwnd, pDial, pptMouse, TRUE);
        }
        else
        {
          /* start scrolling to the position */
          pDial->fState |= DL_TRACKDIAL;
          SetCapture (hwnd);
          pDial->TrackValue = DialPointSelect (hwnd, pDial, pptMouse, FALSE);

          /* Start Timer */

          pDial->uTimer = 0;
          pDial->uTimerPause = 0;
          SetTimer (hwnd, 1, 50, 0); /* 1/20 second */
          pDial->fState |= DL_TIMER;

        }
                                           /* Left Button */
      }
      else if (pptMouse->x >= pDial->rcl1.left &&
               pptMouse->x <pDial->rcl1.right &&
               pptMouse->y <=pDial->rcl1.bottom &&
               pptMouse->y > pDial->rcl1.top)
      {
        SetCapture (hwnd);
        pDial->fState &= ~(DL_RIGHTDOWN | DL_TRACKDIAL);
        pDial->fState |= DL_LEFTDOWN;
        hdc = GetDC (hwnd);
        DialDrawValue (hdc, pDial, -1);
        DialDrawSign  (hdc, pDial, (ULONG)DRAW_MINUS);
        ReleaseDC(hwnd, hdc);

        pDial->uTimer = 0;
        pDial->uTimerPause = 3;
        SetTimer(hwnd, 1, 100, 0); /* 1/10 Second */
        pDial->fState |= DL_TIMER;
                                           /* Right Button */
      }
      else if (pptMouse->x >= pDial->rcl2.left &&
               pptMouse->x < pDial->rcl2.right &&
               pptMouse->y <= pDial->rcl2.bottom &&
               pptMouse->y > pDial->rcl2.top)
      {

        SetCapture (hwnd);
        pDial->fState &= ~(DL_LEFTDOWN | DL_TRACKDIAL);
        pDial->fState |= DL_RIGHTDOWN;
        hdc = GetDC (hwnd);
        DialDrawValue (hdc, pDial, 1);
        DialDrawSign (hdc, pDial, (ULONG)DRAW_PLUS);
        ReleaseDC (hwnd, hdc);
        pDial->uTimer = 0;
        pDial->uTimerPause = 3;
        SetTimer (hwnd, 1, 100, 0); /* 1/10 Second */
        pDial->fState |= DL_TIMER;

                                           /* Dial */
      }
      else if (pptMouse->x >= rcl3.left &&
               pptMouse->x < rcl3.right &&
               pptMouse->y <= rcl3.bottom &&
               pptMouse->y > rcl3.top)
      {

        pDial->fState &= ~(DL_LEFTDOWN | DL_RIGHTDOWN | DL_POINTSELECT | DL_TRACKDIAL);
        if (pDial->lStyle & CSS_POINTSELECT)
        {
          pDial->fState |= DL_POINTSELECT;
          SetCapture (hwnd);
          pDial->TrackValue = DialPointSelect (hwnd, pDial, pptMouse, TRUE);

        }
        else
        {

          pDial->fState |= DL_TRACKDIAL;
          SetCapture (hwnd);
          pDial->TrackValue = DialPointSelect (hwnd, pDial, pptMouse, FALSE);

          /* Start Timer */

          pDial->uTimer = 0;
          pDial->uTimerPause = 0;
          SetTimer (hwnd, 1, 50, 0); /* 1/20 second */
          pDial->fState |= DL_TIMER;

        }

      }
      else
      {
        pDial->fState &= ~(DL_LEFTDOWN | DL_RIGHTDOWN | DL_TRACKDIAL | DL_POINTSELECT);
      }

      SetFocus (hwnd);                /* Take the focus */
      break;

    case WM_LBUTTONUP:
    case WM_RBUTTONUP:
    {
      CSDIALPOS posDial;

      if ((msg == WM_LBUTTONUP) && (pDial->fDrag2))
        break;
      if ((msg == WM_RBUTTONUP) && (pDial->fDrag1))
        break;

      // Turn off timer
      pDial->fState &= ~DL_TIMER;
      KillTimer (hwnd, 1);

//    WinSetPointer (HWND_DESKTOP, pDial->hptrArrow);
      ReleaseCapture ();

      hdc = GetDC (hwnd);

      if (pDial->fState & DL_LEFTDOWN)
      {
        pDial->fState &= ~DL_LEFTDOWN;
        DialDrawSign (hdc, pDial, (ULONG)DRAW_MINUS);
      }
      else if (pDial->fState & DL_RIGHTDOWN)
      {
        pDial->fState &= ~DL_RIGHTDOWN;
        DialDrawSign (hdc, pDial, (ULONG)DRAW_PLUS);
      }
      else if (pDial->fState & DL_TRACKDIAL)
      {
      }
      else if (pDial->fState & DL_POINTSELECT)
      {
      }
      pDial->fState &= ~(DL_LEFTDOWN | DL_RIGHTDOWN | DL_TRACKDIAL | DL_POINTSELECT);

      ReleaseDC (hwnd, hdc);

      posDial.position    = pDial->Value;
      posDial.hwndCSlider = pDial->hwnd;

      // Notify Owner of Final Value
      SENDMESSAGE (GetParent (pDial->hwnd ),
                   WM_COMMAND,
                   MAKEWPARAM(GetWindowLong(pDial->hwnd, GWL_ID), CSN_CHANGED),
                   (LPARAM)&posDial );

      break;
    }

    case WM_MOUSEMOVE:
      if (pDial->fState & DL_TRACKDIAL)
      {
        // Scroll to Position
        pDial->TrackValue = DialPointSelect (hwnd, pDial, pptMouse, FALSE);
      }
      else if (pDial->fState & DL_POINTSELECT)
      {
        pDial->TrackValue = DialPointSelect (hwnd, pDial, pptMouse, TRUE);
      }
#if 0
      // Determine the Pointer

      if (!(pDial->lStyle & WS_DISABLED))
      {

        // Send notification message
        hptr = (HPOINTER)SENDMESSAGE (WinQueryWindow (hwnd, QW_OWNER),
                                      WM_CONTROLPOINTER,
                                      MPFROMSHORT(WinQueryWindowUShort(hwnd, QWS_ID)),
                                      (MPARAM)pDial->hptrArrow);
        if (hptr)
        {
          WinSetPointer (HWND_DESKTOP, hptr);
        }
      }
#endif
      break;

    case WM_TIMER:

      // SetTimer has timed out and has sent this message
      if (pDial->fState & DL_TIMER && pDial->uTimer++ > pDial->uTimerPause)
      {

        if (pDial->fState & DL_TRACKDIAL)
        {
          if (pDial->Value < pDial->TrackValue)
          {
            iDelta = min ((pDial->TrackValue - pDial->Value), pDial->Incre);
            hdc = GetDC (hwnd);

            // Scroll right
            DialDrawValue (hdc, pDial, (LONG)iDelta);
            ReleaseDC (hwnd, hdc);
          }
          else if (pDial->Value > pDial->TrackValue)
          {
            iDelta = min ((pDial->Value - pDial->TrackValue), pDial->Incre);
            hdc = GetDC (hwnd);

            // Scroll left
            DialDrawValue (hdc, pDial, (LONG)(-iDelta));
            ReleaseDC (hwnd, hdc);
          }
        }
        else if (pDial->fState & DL_LEFTDOWN)
        {
          hdc = GetDC (hwnd);

          // Scroll Left
          DialDrawValue (hdc, pDial, (LONG)(-pDial->Incre));
          ReleaseDC (hwnd, hdc);
        }
        else if (pDial->fState & DL_RIGHTDOWN)
        {
          hdc = GetDC (hwnd);

          // Scroll Right
          DialDrawValue (hdc, pDial, (LONG)(pDial->Incre));
          ReleaseDC (hwnd, hdc);
        }

      }
      break;
  }
  return (0L);
}

//**********************************************************************
// Function:  DialChar
//            Process character input
//
// Inputs: Handle to picture space
//         mesage
//         pointer to dial structure
//         pointer to pointer to mouse
//
// Output: ULONG
//
//**********************************************************************
LONG DialChar (HWND hwnd, PDIAL pDial, WPARAM mp1, LPARAM mp2)
{
  LONG  direction = 0;
  LONG  increment = 0;
  HDC   hdc;

  if (!(GetKeyState(VK_CONTROL) & 0x8000) &&
      !(GetKeyState(VK_SHIFT) & 0x8000))
  {
    if (mp1 == VK_LEFT)
    {
      direction = -1;
    }
    else if (mp1 == VK_RIGHT)
    {
      direction = 1;
    }

    if (direction != 0)
    {
      CSDIALPOS posDial;

      hdc = GetDC (hwnd);
      increment = direction * (pDial->Incre);  //@112799
      DialDrawValue (hdc, pDial, increment);   //@112799
      ReleaseDC( hwnd, hdc );

      posDial.position    = pDial->Value;
      posDial.hwndCSlider = pDial->hwnd;

      // Notify Owner of Final Value
      SENDMESSAGE (GetParent(pDial->hwnd),
                   WM_COMMAND,
                   MAKEWPARAM(GetWindowLong(pDial->hwnd, GWL_ID), CSN_CHANGED),
                   (LPARAM)&posDial );
      return (TRUE);
    }
    else
    {
      return (DefWindowProc (hwnd, WM_KEYDOWN, mp1, mp2));
    }
  }
  return (DefWindowProc (hwnd, WM_KEYDOWN, mp1, mp2));
}

//**********************************************************************
// Function:  DialGetPoint
//            Get a point by converting hardcoded angular coordinates
//            to cartesian coordinates
//
// Inputs:  pointer to dial structure
//          pointer to point structure center
//          pointer to point structure point
//          value used to calculate increment
//          radius value used for cartesian coordinates
//
// Output: VOID
//
//**********************************************************************
VOID DialGetPoint (PDIAL pDial, PPOINT pcenter, PPOINT ppt, LONG Value, LONG radius)
{

  // Table of Circle Point Indices (60 Ticks, Like a clock) based as 6:00
  static LONG sin[61] = {
             0,-10,-20,-30,-40,-50,-58,-67,-74,-80,-86,-91,-95,-97,-99,
          -100,-99,-97,-95,-91,-86,-80,-74,-67,-58,-50,-40,-30,-20,-10,
             0,10,20,30,40,50,58,67,74,80,86,91,95,97,99,
           100,99,97,95,91,86,80,74,67,58,50,40,30,20,10,0
                        };

  static LONG cos[61] = {
         -100,-99,-97,-95,-91,-86,-80,-74,-67,-58,-50,-40,-30,-20,-10,
            0,10,20,30,40,50,58,67,74,80,86,91,95,97,99,
          100,99,97,95,91,86,80,74,67,58,50,40,30,20,10,
            0,-10,-20,-30,-40,-50,-58,-67,-74,-80,-86,-91,-95,-97,-99,-100
                        };

  register LONG tx;
  register LONG ty;
  register ULONG offset;
  register ULONG incre;

  // Calc Scroll Angle

  offset = (ULONG)30 - (pDial->nPoints / (ULONG)2);

  if (pDial->Low == pDial->High)
  {
    incre = offset + 0;
  }
  else
  {
    incre = offset + ( ((Value - pDial->Low) * pDial->nPoints) /
                          (pDial->High - pDial->Low));

    tx = sin[incre];
    ty = cos[incre];

    // Return angle as cartesian coordinates
    ppt->x = (tx*radius)/100 + pcenter->x;
    ppt->y = pcenter->y - (ty*radius)/100;
  }
}

//**********************************************************************
// Function:  DialDrawCursor
//            Handle the cursor
//
// Inputs: window handle
//         pointer to dial structure
//         cursor on/off flag
//
// Output: ULONG
//
//**********************************************************************
LONG DialDrawCursor (HWND hwnd, PDIAL pDial, BOOL fCursorOn)
{
  LONG x,y,cx,cy;
  HPEN newPen;
  HPEN oldPen;
  HDC  hdc;
  HBRUSH oldBrush;
  HBRUSH newBrush;

  hdc = GetDC( hwnd );

  if (fCursorOn)
  {
    if (pDial->fState & DL_FOCUS)
    {
      if (pDial->fState & DL_TEXT)
      {
        // Create new cursor outline
        // Give a little space on left side
        x  = pDial->rclText.left - 2;
        y  = pDial->rclText.top;
        cx = pDial->rclText.right - x + 2;
        cy = pDial->rclText.bottom - y;

        pDial->rclCursor.top = y;
        pDial->rclCursor.left = x;
        pDial->rclCursor.bottom = y + cy;
        pDial->rclCursor.right = x + cx;
        pDial->fAroundNumber = FALSE;
      }
      else if (!(pDial->lStyle & CSS_NONUMBER) && pDial->fNumber)
      {
        // create new cursor outline
        // Give a little space on left side
        x  = pDial->rclNumber.left - 2;
        y  = pDial->rclNumber.top - 1;
        cx = pDial->rclNumber.right - x + 2;
        cy = pDial->rclNumber.bottom - y + 2;

        pDial->rclCursor.top = y;
        pDial->rclCursor.left = x;
        pDial->rclCursor.bottom = y + cy;
        pDial->rclCursor.right = x + cx;
        pDial->fAroundNumber = TRUE;
      }
      else
      {
        x = pDial->rclWindow.left;
        y = pDial->rclWindow.top;
        cx = pDial->rclWindow.right - x;
        cy = pDial->rclWindow.bottom - y;

        pDial->rclCursor.top = y;
        pDial->rclCursor.left = x;
        pDial->rclCursor.bottom = y + cy;
        pDial->rclCursor.right = x + cx;
        pDial->fAroundNumber = TRUE;
      }


      newPen = CreatePen( PS_DOT, 1, RGB( 96,96,96 ));
      oldPen = SelectObject( hdc, newPen );
      oldBrush = SelectObject( hdc, GetStockObject( NULL_BRUSH ));
      if (pDial->fAroundNumber)
        SetBkColor( hdc, RGB( 192, 192, 192 ));
      else
        SetBkColor( hdc, GetSysColor( COLOR_WINDOW ));
      Rectangle( hdc,
                 pDial->rclCursor.left,
                 pDial->rclCursor.top,
                 pDial->rclCursor.right,
                 pDial->rclCursor.bottom );

      SelectObject( hdc, oldPen );
      SelectObject( hdc, oldBrush );
      DeleteObject( newPen );

//      CreateCaret( hwnd, (HBITMAP)1, cx, cy );
//      SetCaretPos( x, y );
//      ShowCaret( hwnd );
//      SetCaretBlinkTime(INT_MAX);
    }
  }
  else
  {
    if (pDial->fAroundNumber)
      newPen = CreatePen( PS_SOLID, 1, RGB( 192, 192, 192 ));
    else
      newPen = CreatePen( PS_SOLID, 1, pDial->backgroundColor );

    oldPen = SelectObject( hdc, newPen );
    oldBrush = SelectObject( hdc, GetStockObject( NULL_BRUSH ));
    Rectangle( hdc,
               pDial->rclCursor.left,
               pDial->rclCursor.top,
               pDial->rclCursor.right,
               pDial->rclCursor.bottom );
    SelectObject( hdc, oldPen );
    SelectObject( hdc, oldBrush );
    DeleteObject( newPen );

    // Kill cursor ... now
//  DestroyCaret();
  }
  ReleaseDC( hwnd, hdc );

  return (FALSE);
}


//**********************************************************************
// Function:  DialGetTextLength
//            Calculate the text length
//
// Inputs:  handle to presentation space
//          pointer to character string
//          height
//
//
// Output: LONG
//
//**********************************************************************
LONG DialGetTextLength (HDC hdc, PSZ achText, PLONG pHeight)
{
    int  cch;
    SIZE textSize;

    cch = lstrlen((PCHAR)achText);

    GetTextExtentPoint( hdc, (LPSTR)achText, cch, &textSize );

    if (pHeight)
    {
       *pHeight = textSize.cy;
    }

    // calc length - Handle italic fonts

    return (textSize.cx);
}

//**********************************************************************
// Function:  DialPointSelected
//            Look for the closest value to the one selected
//
// Inputs: window handle
//         pointer to dial structure
//         pointer to point structure for mouse
//         draw flag
//
// Output: LONG
//
//**********************************************************************
LONG DialPointSelect (HWND hwnd, PDIAL pDial, PPOINT pptMouse, BOOL fDraw)
{
  LONG  i;
  LONG  iBest = 0;
  LONG  disp,cx,cy;
  LONG  bestdisp = 0x7fffffff;
  POINT pt;
  HDC   hdc;

  for (i=pDial->Low; i<=pDial->High; i++)
  {   // Enum Values

    DialGetPoint (pDial, &pDial->ptCenter, &pt, i, pDial->uRadius);

    cx = pptMouse->x - pt.x;
    cy = pptMouse->y - pt.y;

    // This calculates the displacement ** 2.  Don't call sqrt
    // to get real disp hear because the sqrt routine only estimates.
    // it causes the dial indicator to jump occasionally.  This
    // routine is looking for the closest value so it doesn't have to
    // be the real distance.

    disp = ( cx*cx + cy*cy );

    if (disp < bestdisp)
    {
      iBest = i;
      bestdisp = disp;
    }
  }

  if (fDraw)
  {
    CSDIALPOS posDial;

    // Draw Value
    pDial->Value = iBest;
    hdc = GetDC (hwnd);
    DialDrawValue (hdc, pDial, 0);
    ReleaseDC (hwnd, hdc);

    posDial.position    = pDial->Value;
    posDial.hwndCSlider = pDial->hwnd;

    SENDMESSAGE (GetParent(pDial->hwnd),
                 WM_COMMAND,
                 MAKEWPARAM(GetWindowLong(pDial->hwnd, GWL_ID), CSN_TRACKING),
                 (LPARAM)&posDial );
  }

  return (iBest);   // Return Value
}

//**********************************************************************
// Function:  DialTickSelect
//            Selec a given tick mark
//
// Inputs: window handle
//         pointer to dial structure
//         pointer to point structure for mouse
//
//
// Output: ULONG
//
//**********************************************************************
LONG DialTickSelect (HWND hwnd, PDIAL pDial, PPOINT pptMouse)
{
  LONG cx,cy;
  LONG rad;
  POINT ptMin;
  POINT ptMax;

  cx = pDial->ptCenter.x - pptMouse->x;  /* Radius of Mouse */
  cy = pDial->ptCenter.y - pptMouse->y;
  rad = Dialsqrt (cx*cx + cy*cy);

  DialGetPoint (pDial, &pDial->ptCenter, &ptMin, pDial->Low, pDial->uRadius);
  DialGetPoint (pDial, &pDial->ptCenter, &ptMax, pDial->High, pDial->uRadius);

  if (rad >= (LONG)(pDial->uRadius) &&
      rad <= (LONG)(pDial->uRadius+(ULONG)pDial->DialTick))
  {

    /* Make sure it is within tick range */

    if (pDial->nPoints == 30 && pptMouse->y >= pDial->ptCenter.y)
      return (FALSE);

    if (pDial->nPoints == 42 &&
        pptMouse->y >= pDial->ptCenter.y &&
        (pptMouse->x > ptMin.x && pptMouse->x < ptMax.x))
      return (FALSE);

    /* Make sure not on top of a button */

    if (pptMouse->x >= pDial->rcl1.left &&       /* left button */
        pptMouse->x <  pDial->rcl1.right &&
        pptMouse->y <= pDial->rcl1.bottom &&
        pptMouse->y > pDial->rcl1.top)
      return (FALSE);

    if (pptMouse->x >= pDial->rcl2.left &&       /* right button */
        pptMouse->x < pDial->rcl2.right &&
        pptMouse->y <= pDial->rcl2.bottom &&
        pptMouse->y > pDial->rcl2.top)
      return (FALSE);

    return (TRUE);

  }

  return (FALSE);

}

//**********************************************************************
// Function:  DialSqrt
//            Estimate integer square root
//
// Inputs:  a long integer value.
//
//
// Output: LONG
//
//**********************************************************************
LONG Dialsqrt (LONG lValue)
{
  LONG LowRoot  = 0;
  LONG HighRoot = lValue;
  LONG Root     = 0;
  LONG LastRoot = 0xffffffff;

  while (Root != LastRoot)
  {
    LastRoot = Root;
    Root = (LowRoot + HighRoot) / 2;    // bisect
    if (Root > 46000 || Root * Root > lValue)
    {
      // 46000 ~= Dialsqrt(max long)
      HighRoot = Root;
    }
    else
    {
      LowRoot = Root;
    }
  }
  return (Root);
}

//**********************************************************************
// Function:  DialQueryFontMetrics
//            Return font information
//
// Inputs: pointer to dial structure
//         window handle
//
//
//
// Output: ULONG
//
//**********************************************************************
LONG DialQueryFontMetrics (PDIAL pDial, HWND hwnd)
{
   HDC hdc;
   TEXTMETRIC font;     // Font info

   hdc = GetDC(hwnd);

   // Ask for a record containing font metrics for this font
   GetTextMetrics( hdc, &font );

   // Height of Font
   pDial->cyFont = (LONG) font.tmHeight;

   // Width of Font
   pDial->cxFont = (LONG) font.tmAveCharWidth;

   ReleaseDC(hwnd, hdc);

   return (0);
}

//**********************************************************************
// Function:  DialLineArch
//
//
// Inputs: pointer to dial structure
//         window handle
//
//
//
// Output: none
//
//**********************************************************************
VOID DialLineArch (HDC hdc, PPOINT pptValue,
                   PPOINT pptPoint, PLINEARCH pLineArch)

{
  HPEN currentPen;
  HPEN invisiblePen;

  invisiblePen = GetStockObject( NULL_PEN );
  currentPen = SelectObject( hdc, invisiblePen );
  MoveToEx( hdc, pptValue->x, pptValue->y, 0 );

  PartialArc( hdc, pptValue, pLineArch->fxMultiplier1, pLineArch->fxStartAngle1, 0 );

  SelectObject( hdc, currentPen );

  GetCurrentPositionEx( hdc, pptPoint );

  PartialArc( hdc, pptValue, pLineArch->fxMultiplier2, pLineArch->fxStartAngle2, 160 );
}

VOID PartialArc( HDC hdc, PPOINT center, int radius, int startAngle, int sweepAngle )
{
  RECT boundingRect;
  POINT firstRadial;
  POINT secondRadial;

  boundingRect.left   = center->x - radius;
  boundingRect.bottom = center->y + radius;
  boundingRect.right  = center->x + radius;
  boundingRect.top    = center->y - radius;

  RadialPoint( &boundingRect, center, startAngle, &firstRadial );
  RadialPoint( &boundingRect, center, (startAngle+sweepAngle), &secondRadial );

  Arc( hdc,
       boundingRect.left,
       boundingRect.top,
       boundingRect.right,
       boundingRect.bottom,
       firstRadial.x,
       firstRadial.y,
       secondRadial.x,
       secondRadial.y );
  MoveToEx( hdc, secondRadial.x, secondRadial.y, 0 );
}

VOID FullArc (HDC hdc, int control, int radius)
{
  POINT currentPos;
  RECT  boundingRect;
  HPEN  currentPen;
  HBRUSH currentBrush;


  GetCurrentPositionEx( hdc, &currentPos );
  boundingRect.left = currentPos.x - radius;
  boundingRect.bottom = currentPos.y + radius;
  boundingRect.right = currentPos.x + radius;
  boundingRect.top = currentPos.y - radius;

  switch (control)
  {
    case DRO_OUTLINE:
      currentBrush = SelectObject( hdc, GetStockObject( HOLLOW_BRUSH ));
    break;
    case DRO_FILL:
      currentPen = SelectObject( hdc, GetStockObject( NULL_PEN ));
    break;
  }

  Ellipse( hdc,
           boundingRect.left,
           boundingRect.top,
           boundingRect.right,
           boundingRect.bottom );

  switch (control)
  {
    case DRO_OUTLINE:
      SelectObject( hdc, currentBrush );
    break;
    case DRO_FILL:
      SelectObject( hdc, currentPen );
    break;
  }
}

VOID RadialPoint( LPRECT rect, PPOINT center, int angle, PPOINT radial )
{
  long radialX;
  long radialY;
  float tanOfAngle;

  if (angle < 0)
  {
    angle = -angle;
    angle = 360.0 - (angle % 360);
  }
  else
  {
    angle = angle % 360;
  } /* endif */

  tanOfAngle = tan( (double)angle/(double)57.29577951 );

  if (angle == 90)
  {
    radialX = center->x;
    radialY = rect->top;
  }
  else if (angle == 270)
  {
    radialX = center->x;
    radialY = rect->bottom;
  }
  else if (angle < 45)
  {
    float adj = rect->right - center->x;
    radialX = rect->right;
    radialY = center->y - (int)(adj * tanOfAngle);
  }
  else if (angle < 135 )
  {
    float opp = center->y - rect->top;
    radialY = rect->top;
    radialX = center->x + (int)(opp / tanOfAngle);
  }
  else if (angle < 225 )
  {
    float adj = center->x - rect->left;
    radialX = rect->left;
    radialY = center->y + (int)(adj * tanOfAngle);
  }
  else if (angle < 315)
  {
    float opp = rect->bottom - center->y;
    radialY = rect->bottom;
    radialX = center->x - (int)(opp / tanOfAngle);
  }
  else
  {
    float adj = rect->right - center->x;
    radialX = rect->right;
    radialY = center->y - (int)(adj * tanOfAngle);
  }
  radial->x = radialX;
  radial->y = radialY;
}

//    The function below provides an equivalent function in Windows for WinDrawBitmap
//    Note the differences:
//         1. Does not support DBM_HALFTONE.
//         2. Colors are given in COLORREF. Refer to the Windows SDK for details.
//         3. Vertical coordinates in Windows: 0 is the top and the numbers increase going down.

BOOL DrawBitmap(HDC dcDst, HBITMAP hbm, LPRECT prclSrc, LPRECT prclDst, int fs)
{
  HDC           hdcMemory;
  HBITMAP       hbmpOld;
  BITMAP        bm;
  DWORD         dwRop;
  int           xSrc, ySrc, nWidthSrc, nHeightSrc;
  int           xDst, yDst, nWidthDst, nHeightDst;
  BOOL          RetVal;

   //   Get the bitmap dimensions.
   GetObject(hbm, sizeof(BITMAP), (LPSTR) &bm);

   // Select the bitmap into a memory DC we can BitBlt from.
   hdcMemory = CreateCompatibleDC(dcDst);
   hbmpOld = SelectObject(hdcMemory, hbm);

   // Translate flags
   dwRop = SRCCOPY;
   if (fs & DBM_INVERT)
     dwRop |= DSTINVERT;

   // If using colors save old colors in order to set them again after the operation.

   // Calculate coordinates. Remember, in Windows, y =0 is top of screen.
   if ( prclSrc )
   {
     xSrc = prclSrc->left;
     ySrc = prclSrc->bottom;
     nWidthSrc = prclSrc->right - prclSrc->left;
     nHeightSrc = prclSrc->bottom - prclSrc->top;
   }
   else
   {
     xSrc = ySrc = 0;
     nWidthSrc = bm.bmWidth;
     nHeightSrc = bm.bmHeight;
   }

   // According to the fs value use BitBlt or StretchBlt.
   if (fs & DBM_STRETCH)
   {
     if ( prclDst )
     {
       xDst = prclDst->left;
       yDst = prclDst->top;
       nWidthDst = prclDst->right - prclDst->left;
       nHeightDst = prclDst->bottom - prclDst->top;

       SetStretchBltMode( dcDst,     COLORONCOLOR );
       SetStretchBltMode( hdcMemory, COLORONCOLOR );

       RetVal = StretchBlt(dcDst,
                           xDst,
                           yDst,
                           nWidthDst,
                           nHeightDst,
                           hdcMemory,
                           xSrc,
                           ySrc,
                           nWidthSrc,
                           nHeightSrc,
                           dwRop);
     }
     else
     {
       RetVal = FALSE;
     }

   }
   else
   {
     RetVal = BitBlt(dcDst,
                     prclDst->left,
                     prclDst->top,
                     nWidthSrc,
                     nHeightSrc,
                     hdcMemory,
                     xSrc,
                     ySrc,
                     dwRop);
   }

   SelectObject(hdcMemory, hbmpOld);
   DeleteDC(hdcMemory);

   // So long...
   return (RetVal);
}
