// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#ifndef _DIAL_
#define _DIAL_

/* Revision: 32 1.3.2.3 source/ui/cclnt/cslider/dial.h, cclcslid, ioc.v400, 000919  */

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

// Memory allocation/deallocation functions -- defined in ctlinit.c

#define DialAllocMem          AllocHeapMem
#define DialFreeMem           FreeHeapMem

//  Local Defines/structures

#define DL_TEXTLEN    128

typedef struct _DIAL
{
   HWND      hwnd;                            // hwnd
   ULONG     fState;                          // Current State
   ULONG     lStyle;                          // Style Bits
   LONG      Low;                             // Low Range
   LONG      High;                            // High Range
   LONG      Value;                           // value
   LONG      TrackValue;                      // track value
   LONG      Incre;                           // Value Granularity
   LONG      Tick;                            // Tick Mark Granularity
   LONG      nPoints;                         // n Physical Value Points
   POINT     ptCenter;                        // Center
   POINT     ptValue;                         // Value Point
   LONG      cyFont;                          // Font Height
   LONG      cxFont;                          // Font Width
   ULONG     uRadius;                         // Radius value
   ULONG     uTimer;                          // Timer count
   ULONG     uTimerPause;                     // Timer Pause
   HCURSOR   hptrArrow;                       // hptrArrow
   COLORREF  backgroundColor;                 // background color
   RECT      rclWindow;                       // Window
   RECT      rcl1;                            // Left Button
   RECT      rcl2;                            // Right Button
   RECT      rclText;                         // Text Window
   RECT      rclCursor;
   BOOL      fNumber;                         // fNumber
   BOOL      fAroundNumber;
   RECT      rclNumber;                       // rclNumber
   HBITMAP   hbmLeftUp;                       // Bitmaps
   HBITMAP   hbmLeftDown;
   HBITMAP   hbmRightUp;
   HBITMAP   hbmRightDown;
   CHAR      szText[DL_TEXTLEN];              // Window Text
   BOOL      fDrag1;                          // Flag for mouse button
   BOOL      fDrag2;                          // Flag for mouse button
   LONG      DialTick;                        // Distance from the outer
                                              // circle to outer tick mark
   LONG      DialTickLength;                  // Distance from outer circle
                                              // to the start of ticks
} DIAL, *PDIAL;



typedef struct _LINEARCH    /* LineArch */
{
  int     fxMultiplier1;
  int     fxStartAngle1;
  ULONG   flColor;
  LONG    iColor;
  int     fxMultiplier2;
  int     fxStartAngle2;
} LINEARCH;
typedef LINEARCH *PLINEARCH;



// Status Bits to indicate where to draw what.

#define VHAB               ((HAB) 0)
#define DL_LEFTDOWN        0x0001
#define DL_RIGHTDOWN       0x0002
#define DL_TRACKDIAL       0x0004
#define DL_FOCUS           0x0008
#define DL_TEXT            0x0010
#define DL_TIMER           0x0020
#define DL_BITMAPS         0x0040
#define DL_POINTSELECT     0x0080

// Use these to calculate proportional tick marks

#define DIALPAD  3
#define DIALTICK 16
#define DIALTICKMAX 20        // Maximum calculated dialtick
#define DIALTICKMIN 8         // Minimum calculated dialtick
#define DIALTICKMINLEN 3      // Minimum calculated dialtick Length

// Max/min macros

#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))

// Dial colors

#define CLR_DIAL               SYSCLR_BUTTONMIDDLE   // CLR_PALEGRAY
#define CLR_DIALSHADOW         CLR_DARKGRAY
#define CLR_DIALLIGHT          CLR_WHITE
#define CLR_DIALBORDER         CLR_BLACK

#define CLR_DISABLEDDIAL       SYSCLR_BUTTONMIDDLE   // CLR_PALEGRAY
#define CLR_DISABLEDDIALSHADOW SYSCLR_BUTTONMIDDLE   // CLR_PALEGRAY
#define CLR_DISABLEDDIALLIGHT  CLR_WHITE
#define CLR_DISABLEDDIALBORDER CLR_DARKGRAY

/******************************************************************************/
/* Window word offset definitions (Starts at 0 in Windows)                    */
/******************************************************************************/
#define GWL_CSLIDER     0


//=========================================
//  Function Prototypes form DIAL.C
//=========================================

LRESULT CALLBACK DialWndProc (HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);

LONG DialCreate (HWND hwnd, WPARAM mp1, LPARAM mp2);
LONG DialDestroy (HWND hwnd, PDIAL pDial, LPARAM mp2);
LONG DialPaint (HWND hwnd, PDIAL pDial);
LONG DialButton1 (HWND hwnd, ULONG msg, PDIAL pDial, PPOINT pptMouse);
LONG DialChar (HWND hwnd, PDIAL pDial, WPARAM mp1, LPARAM mp2);
LONG DialSize (HWND hwnd, PDIAL pDial);
LONG DialQueryFontMetrics (PDIAL pDial, HWND hwnd);


LONG  DialDrawSign(HDC hdc, PDIAL pDial, ULONG ulSign);
LONG  DialDrawValue (HDC hdc, PDIAL pDial, LONG incre);
LONG  DialDrawText (HDC hdc, PDIAL pDial);
LONG  DialDrawCursor (HWND hwnd, PDIAL pDial, BOOL fCursorOn);

VOID  DialGetPoint (PDIAL pDial, PPOINT pcenter, PPOINT ppt, LONG Value, LONG radius);
LONG  DialPointSelect (HWND hwnd, PDIAL pDial, PPOINT pptMouse, BOOL fDraw);
LONG  DialTickSelect (HWND hwnd, PDIAL pDial, PPOINT pptMouse);
LONG  DialGetTextLength (HDC hdc, PSZ achText, PLONG pHeight);
LONG  Dialsqrt (LONG Value);
VOID  DialLineArch(HDC hdc, PPOINT pptValue, PPOINT pptPoint, PLINEARCH pLineArch);
VOID  RadialPoint  ( LPRECT rect, PPOINT center, int angle, PPOINT radial );
BOOL  DrawBitmap(HDC dcDst, HBITMAP hbm, LPRECT prclSrc, LPRECT pptDst, int fs);
VOID  FullArc (HDC hdc, int control, int radius);
VOID  PartialArc( HDC hdc, PPOINT center, int radius, int startAngle, int sweepAngle );

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#ifdef BIDI
  //@bidi_pi
  #include <ctlbidi.h>
#endif //bidi

#endif // DIAL header guard
