/* Revision: 38 1.7 source/ui/cclnt/iclnbw.h, cclbook, ioc.v400, 000919  */
/* NOSHIP */
#ifndef _ICLNBW_
#define _ICLNBW_
/*******************************************************************************
* FILE NAME: iclnbw.h                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the header information for the 32-bit version of        *
*    the notebook control.                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(4)

#ifndef CCL_MESSAGE_BASE
  #define CCL_MESSAGE_BASE ( 0xF000 )
#endif

#ifndef _ICLCLB_
	#include <iclclb.h>
#endif

/**********************************************************************/
/* Notebook Window Class                                              */
/**********************************************************************/
#define ICL_NOTEBOOK      "ICL Notebook"

#if defined(CCL_SEPARATE_DLL)
#define ICL_NOTEBOOK_DLL  "ICLNBW.DLL"
#else
#define ICL_NOTEBOOK_DLL  ICL_DLL
#endif


/**********************************************************************/
/* Ownerdraw id                                                       */
/**********************************************************************/
#define ODT_NOTEBOOK 101

/**********************************************************************/
/* Message ids                                                        */
/**********************************************************************/
/**********************************************************************/
/* Note that these message are in a specific range.  When adding a    */
/* new message, be sure to add within the CCL message ranges defined  */
/* in iclccl.h.  For Notebook, limit the message to the range         */
/* CCL_MESSAGE_BASE+0x0070 to CCL_MESSAGE_BASE+0x008F.  Other CCL     */
/* controls use other sections of the entire range defined in         */
/* iclccl.h                                                           */
/**********************************************************************/
#define BKM_CALCPAGERECT         (CCL_MESSAGE_BASE+0x0070) /* Calc book/page rectangle  */
#define BKM_DELETEPAGE           (CCL_MESSAGE_BASE+0x0071) /* Delete page(s)            */
#define BKM_INSERTPAGE           (CCL_MESSAGE_BASE+0x0072) /* Insert page               */
#define BKM_INVALIDATETABS       (CCL_MESSAGE_BASE+0x0073) /* Invalidate tab area       */
#define BKM_TURNTOPAGE           (CCL_MESSAGE_BASE+0x0074) /* Turn to page              */
#define BKM_QUERYPAGECOUNT       (CCL_MESSAGE_BASE+0x0075) /* Query number of pages     */
#define BKM_QUERYPAGEID          (CCL_MESSAGE_BASE+0x0076) /* Query page identifier     */
#define BKM_QUERYPAGEDATA        (CCL_MESSAGE_BASE+0x0077) /* Query page user data      */
#define BKM_QUERYPAGEWINDOWHWND  (CCL_MESSAGE_BASE+0x0078) /* Query page window handle  */
#define BKM_QUERYTABBITMAP       (CCL_MESSAGE_BASE+0x0079) /* Query tab bitmap handle   */
#define BKM_QUERYTABTEXT         (CCL_MESSAGE_BASE+0x007a) /* Query tab text pointer    */
#define BKM_SETDIMENSIONS        (CCL_MESSAGE_BASE+0x007b) /* Set tab/dogear dimensions */
#define BKM_SETPAGEDATA          (CCL_MESSAGE_BASE+0x007c) /* Set page user data        */
#define BKM_SETPAGEWINDOWHWND    (CCL_MESSAGE_BASE+0x007d) /* Set page window handle    */
#define BKM_SETSTATUSLINETEXT    (CCL_MESSAGE_BASE+0x007e) /* Set status line text      */
#define BKM_SETTABBITMAP         (CCL_MESSAGE_BASE+0x007f) /* Set tab bitmap            */
#define BKM_SETTABTEXT           (CCL_MESSAGE_BASE+0x0080) /* Set tab text              */
#define BKM_SETNOTEBOOKCOLORS    (CCL_MESSAGE_BASE+0x0081) /* Set Notebook colors       */
#define BKM_QUERYPAGESTYLE       (CCL_MESSAGE_BASE+0x0082) /* Query page style          */
#define BKM_QUERYSTATUSLINETEXT  (CCL_MESSAGE_BASE+0x0083) /* Query status line text    */

/**********************************************************************/
/* Notification codes                                                 */
/**********************************************************************/
#define BKN_PAGESELECTED         130     /* New page selected by user */
#define BKN_NEWPAGESIZE          131     /* App page size changed     */
#define BKN_HELP                 132     /* Help notification         */
#define BKN_PAGEDELETED          133     /* Page deleted notification */
#define BKN_PAGESELECTEDPENDING  134     /* New page selected by user */

/**********************************************************************/
/* Page deletion flags (usDeleteFlag)                                 */
/**********************************************************************/
#define BKA_ALL                  0x0001  /* all pages                 */
#define BKA_SINGLE               0x0002  /* single page               */
#define BKA_TAB                  0x0004  /* minor/major section       */

/**********************************************************************/
/* Page insertion/query order (usPageOrder, usQueryOrder)             */
/**********************************************************************/
#define BKA_LAST                 0x0002  /* Insert/Query last page    */
#define BKA_FIRST                0x0004  /* Insert/Query first page   */
#define BKA_NEXT                 0x0008  /* Insert/Query after page   */
#define BKA_PREV                 0x0010  /* Insert/Query before page  */
#define BKA_TOP                  0x0020  /* Query topmost page        */
#define BKA_DELETED              0x0040  /* Query deleted page        */
#define BKA_PREVSELECTED         0x0080  /* Query prev selected page  */

/**********************************************************************/
/* Notebook region types (usBookRegion, usType)                       */
/**********************************************************************/
#define BKA_MAJORTAB             0x0001  /* Major Tab                 */
#define BKA_MINORTAB             0x0002  /* Minor Tab                 */
#define BKA_PAGEBUTTON           0x0100  /* Page Turning Button       */

/**********************************************************************/
/* Page insertion/query styles (usPageStyle,usQueryEnd)               */
/**********************************************************************/
#define BKA_STATUSTEXTON         0x0001  /* status area text          */
#define BKA_MAJOR                0x0040  /* Major Tab                 */
#define BKA_MINOR                0x0080  /* Minor Tab                 */
#define BKA_AUTOPAGESIZE         0x0100  /* Page window position/size */
#define BKA_END                  0x0200  /* Query to end of book      */

/**********************************************************************/
/* Tab window contents (usTabDisplay)                                 */
/**********************************************************************/
#define BKA_TEXT                 0x0400  /* text data                 */
#define BKA_BITMAP               0x0800  /* bitmap                    */

/**********************************************************************/
/* Notebook window styles (ulNotebookStyles)                          */
/**********************************************************************/

/**********************************************************************/
/* Backpage Orientation                                               */
/**********************************************************************/
#define BKS_BACKPAGESBR          0x00000001  /* Bottom right          */
#define BKS_BACKPAGESBL          0x00000002  /* Bottom left           */
#define BKS_BACKPAGESTR          0x00000004  /* Top right             */
#define BKS_BACKPAGESTL          0x00000008  /* Top left              */

/**********************************************************************/
/* Major Tab Side                                                     */
/**********************************************************************/
#define BKS_MAJORTABRIGHT        0x00000010  /* Major tabs right      */
#define BKS_MAJORTABLEFT         0x00000020  /* Major tabs left       */
#define BKS_MAJORTABTOP          0x00000040  /* Major tabs top        */
#define BKS_MAJORTABBOTTOM       0x00000080  /* Major tabs bottom     */

/**********************************************************************/
/* Tab Type                                                           */
/**********************************************************************/
#define BKS_SQUARETABS           0x00000000  /* Square edged tabs     */
#define BKS_ROUNDEDTABS          0x00000100  /* Round  edged tabs     */
#define BKS_POLYGONTABS          0x00000200  /* Polygon edged tabs    */

/**********************************************************************/
/* Binding type                                                       */
/**********************************************************************/
#define BKS_SOLIDBIND            0x00000000  /* Solid binding         */
#define BKS_SPIRALBIND           0x00000400  /* Spiral binding        */

/**********************************************************************/
/* Status line text justification                                     */
/**********************************************************************/
#define BKS_STATUSTEXTLEFT       0x00000000  /* left justify text     */
#define BKS_STATUSTEXTRIGHT      0x00001000  /* right justify text    */
#define BKS_STATUSTEXTCENTER     0x00002000  /* center text           */

/**********************************************************************/
/* Tab text justification                                             */
/**********************************************************************/
#define BKS_TABTEXTLEFT          0x00000000  /* Left justify tab text */
#define BKS_TABTEXTRIGHT         0x00004000  /* Right justify tab text*/
#define BKS_TABTEXTCENTER        0x00008000  /* Center tab text       */

/**********************************************************************/
/* Notebook color attributes                                          */
/**********************************************************************/
#define BKA_BACKGROUNDCOLOR           0x0001
#define BKA_FOREGROUNDCOLOR           0x0002
#define BKA_BORDERCOLOR               0x0003
#define BKA_CURSORCOLOR               0x0004
#define BKA_BACKGROUNDPAGECOLOR       0x0005
#define BKA_BACKGROUNDMAJORCOLOR      0x0006
#define BKA_BACKGROUNDMINORCOLOR      0x0007
#define BKA_FOREGROUNDMAJORCOLOR      0x0008
#define BKA_FOREGROUNDMINORCOLOR      0x0009

/**********************************************************************/
/* Error message ids                                                  */
/**********************************************************************/
#define BOOKERR_INVALID_PARAMETERS   -1L /* Invalid parameters        */

/**********************************************************************/
/* BKM_QUERYTABTEXT message structure                                 */
/**********************************************************************/
typedef struct _PBOOKTEXT {  /* ptabtxt */
  LPSTR  pString;                        /* ptr to string buffer      */
  WORD   textLen;                        /* length of string to query */
} BOOKTEXT;
typedef BOOKTEXT FAR *PBOOKTEXT;

/**********************************************************************/
/* BKN_PAGEDELETED notify message structure                           */
/**********************************************************************/
typedef struct _DELETENOTIFY     /* delntfy */
{
   HWND    hwndBook;                     /* Notebook window handle    */
   HWND    hwndPage;                     /* App. page window handle   */
   DWORD   ulAppPageData;                /* App. page data            */
   HBITMAP hbmTab;                       /* App. tab bitmap handle    */
} DELETENOTIFY;
typedef DELETENOTIFY FAR *PDELETENOTIFY;

/**********************************************************************/
/* BKN_PAGESELECTED notify message structure                          */
/**********************************************************************/
typedef struct _PAGESELECTNOTIFY     /* pgsntfy */
{
   HWND    hwndBook;                     /* Notebook window handle    */
   DWORD   ulPageIdCur;                  /* Previous top page id      */
   DWORD   ulPageIdNew;                  /* New top Page id           */
} PAGESELECTNOTIFY;
typedef PAGESELECTNOTIFY FAR *PPAGESELECTNOTIFY;

#pragma pack()

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICLNBW_

