/* Revision: 39 1.9 source/ui/cclnt/iclsldw.h, cclslid, ioc.v400, 000919  */
/* NOSHIP */
#ifndef _ICLSLDW_
#define _ICLSLDW_
/*******************************************************************************
* FILE NAME: iclsldw.h                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the header information for the 32-bit version of        *
*    the slider control.                                                       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   "Restricted Materials of IBM"                                              *
*                                                                              *
*******************************************************************************/

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(4)

#ifndef CCL_MESSAGE_BASE
  #define CCL_MESSAGE_BASE ( 0xF000 )
#endif

#ifndef _ICLCLB_
	#include <iclclb.h>
#endif

/**********************************************************************/
/* Slider Window Class                                                */
/**********************************************************************/
#define ICL_SLIDER     "ICL Slider"

#if defined(CCL_SEPARATE_DLL)
#define ICL_SLIDER_DLL "ICLSLDW.DLL"
#else
#define ICL_SLIDER_DLL ICL_DLL
#endif

/**********************************************************************/
/* Ownerdraw id                                                       */
/**********************************************************************/
#define ODT_SLIDER      102              /* ownerdraw id for Slider   */

/**********************************************************************/
/* Window word offset definitions                                     */
/**********************************************************************/
#define QWL_SLDBRUSH   8             /* 8 byte offset from QWL_USER     */

/**********************************************************************/
/* Define messages for the slider control                             */
/**********************************************************************/
/**********************************************************************/
/* Note that these message are in a specific range.  When adding a    */
/* new message, be sure to add within the CCL message ranges defined  */
/* in iclccl.h.  For Slider, limit the message to the range           */
/* CCL_MESSAGE_BASE+0x0090 to CCL_MESSAGE_BASE+0x009f.  Other CCL     */
/* controls use other sections of the entire range defined in         */
/* iclccl.h                                                           */
/**********************************************************************/
#define SLM_ADDDETENT       (CCL_MESSAGE_BASE+0x0090) /* Add detent niche      */
#define SLM_QUERYDETENTPOS  (CCL_MESSAGE_BASE+0x0091) /* Query detent position */
#define SLM_QUERYSCALETEXT  (CCL_MESSAGE_BASE+0x0092) /* Query tick text       */
#define SLM_QUERYSLIDERINFO (CCL_MESSAGE_BASE+0x0093) /* Query slider info     */
#define SLM_QUERYTICKPOS    (CCL_MESSAGE_BASE+0x0094) /* Query position of tick*/
#define SLM_QUERYTICKSIZE   (CCL_MESSAGE_BASE+0x0095) /* Query size of tick    */
#define SLM_REMOVEDETENT    (CCL_MESSAGE_BASE+0x0096) /* Remove detent niche   */
#define SLM_SETSCALETEXT    (CCL_MESSAGE_BASE+0x0097) /* Set text above tick   */
#define SLM_SETSLIDERINFO   (CCL_MESSAGE_BASE+0x0098) /* Set slider parameters */
#define SLM_SETTICKSIZE     (CCL_MESSAGE_BASE+0x0099) /* Set size of tick      */
#define SLM_SETSLIDERCOLORS (CCL_MESSAGE_BASE+0x009a) /* Set colors of a slider*/
#define SLM_QUERYSLIDERCOLORS (CCL_MESSAGE_BASE+0x009b) /* Query colors of a slider*/
#define SLM_RESETSLIDERCOLORS (CCL_MESSAGE_BASE+0x009c) /* Reset colors of a slider*/
#define WM_CTLCOLORSLIDER   (CCL_MESSAGE_BASE+0x009d)  /* Query fore/background color   */

/**********************************************************************/
/* Define notification messages for the slider control                */
/**********************************************************************/
/**********************************************************************/
/* Note that these notification codes are in a specific range.  When  */
/* adding a new code, be sure to add within the CCL ranges defined    */
/* in iclccl.h.  For Slider, limit the notification code to the range */
/* 140 to 149.  Other CCL controls use other sections of the entire   */
/* range defined in iclccl.h.                                         */
/**********************************************************************/
#define SLN_CHANGE          140          /* Slider position changed   */
#define SLN_KILLFOCUS       141          /* Slider losing focus       */
#define SLN_SETFOCUS        142          /* Slider gaining focus      */
#define SLN_SLIDERTRACK     143          /* Slider dragged by user    */

/**********************************************************************/
/* Slider control data structure                                      */
/**********************************************************************/
typedef struct _SLDCDATA {
  DWORD   cbSize;                /* Size of control block             */
  WORD    usScale1Increments;    /* # of divisions on scale           */
  WORD    usScale1Spacing;       /* Space in pels between increments  */
  WORD    usScale2Increments;    /* # of divisions on scale           */
  WORD    usScale2Spacing;       /* Space in pels between increments  */
} SLDCDATA;

typedef SLDCDATA FAR *LPSLDCDATA;

/**********************************************************************/
/* Slider control style flag definition                               */
/**********************************************************************/
#define SLS_HORIZONTAL       0x00000000L /* Orient slider horizontally*/
#define SLS_VERTICAL         0x00000001L /* Orient slider vertically  */
#define SLS_CENTER           0x00000000L /* Center shaft in window    */
#define SLS_BOTTOM           0x00000002L /* Offset shaft to bottom (H)*/
#define SLS_TOP              0x00000004L /* Offset shaft to top (H)   */
#define SLS_LEFT             0x00000002L /* Offset shaft to left (V)  */
#define SLS_RIGHT            0x00000004L /* Offset shaft to right (V) */
#define SLS_SNAPTOINCREMENT  0x00000008L /* Snap to nearest increment */
#define SLS_BUTTONSBOTTOM    0x00000010L /* Add buttons at shaft bot. */
#define SLS_BUTTONSTOP       0x00000020L /* Add buttons at shaft top  */
#define SLS_BUTTONSLEFT      0x00000010L /* Add buttons left of shaft */
#define SLS_BUTTONSRIGHT     0x00000020L /* Add buttons right of shaft*/
#define SLS_OWNERDRAW        0x00000040L /* Owner draw some fields    */
#define SLS_READONLY         0x00000080L /* Provide a read only slider*/
#define SLS_RIBBONSTRIP      0x00000100L /* Provide a ribbon strip    */
#define SLS_HOMEBOTTOM       0x00000000L /* Set home position at bot. */
#define SLS_HOMETOP          0x00000200L /* Set home position at top  */
#define SLS_HOMELEFT         0x00000000L /* Set home position at left */
#define SLS_HOMERIGHT        0x00000200L /* Set home position at right*/
#define SLS_PRIMARYSCALE1    0x00000000L /* Scale 1 is primary scale  */
#define SLS_PRIMARYSCALE2    0x00000400L /* Scale 2 is primary scale  */

/**********************************************************************/
/* Message attributes for setting and querying slider components      */
/**********************************************************************/
#define SMA_SCALE1               0x0001
#define SMA_SCALE2               0x0002
#define SMA_SHAFTDIMENSIONS      0x0000
#define SMA_SHAFTPOSITION        0x0001
#define SMA_SLIDERARMDIMENSIONS  0x0002
#define SMA_SLIDERARMPOSITION    0x0003
#define SMA_RANGEVALUE           0x0000
#define SMA_INCREMENTVALUE       0x0001
#define SMA_SLIDERSCALEINFO      0x0004
#define SMA_SLIDERTICKCOUNT      0x0000
#define SMA_SLIDERTICKSPACING    0x0001
#define SMA_SETALLTICKS          0xFFFF

/**********************************************************************/
/* Ownerdraw flag definitions                                         */
/**********************************************************************/
#define SDA_RIBBONSTRIP          0x0001
#define SDA_SLIDERSHAFT          0x0002
#define SDA_BACKGROUND           0x0003
#define SDA_SLIDERARM            0x0004

/**********************************************************************/
/* Attributes for setting the colors of a slider                      */
/**********************************************************************/
#define SDA_FOREGROUNDCOLOR      0x0001
#define SDA_BACKGROUNDCOLOR      0x0002

/**********************************************************************/
/* Error return codes                                                 */
/**********************************************************************/
#define SLDERR_INVALID_PARAMETERS     -1

#pragma pack()

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICLSLDW_

