/* Revision: 46 1.4 source/ui/cclnt/notebook/bookdtab.c, cclbook, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992 (Unpublished)         */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                          N O T E B O O K                           */
/*                                                                    */
/*                       Filename: BOOKDTAB.C                         */
/*                                                                    */
/*             ( Windows NT Notebook Control Component )              */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: BOOKDTAB.C - Tab drawing routines for the Notebook       */
/*                        control                                     */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* JEH1  06/22/93  JEH  Initial modifications for NT execution        */
/* DAB1  12/01/93  DAB  Additional modifications for NT execution     */
/* DAB2  01/19/94  DAB  Corrected rounded tab cursor bug              */
/* DAB3  10/11/95  DAB  Fixed problem with ownerdraw cnr id in mp1    */
/**********************************************************************/
#include <windows.h>
#include <memory.h>
#include "iclnbw.h"
#include "bookp.h"

extern USHORT vcyScreen;


/*---------------------------------------------------------------------------*
 * DrawTabBitmap
 *
 * Draws a bitmap inside a tab.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawTabBitmap (
  HDC     hdcDest,
  HBITMAP hbm,
  LPRECT  lprc)
{
  if (hbm)  BltStretch (hdcDest, hbm, lprc, FALSE);
}


/*---------------------------------------------------------------------------*
 * DrawTabBorder
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawTabBorder(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprc,
  PPAGE  ppg)
{
  USHORT   usSide;
  COLORREF crTabColor;

  /*-----------------------------------------------------------*/
  /* Determine which edge of the book the tabs are to be drawn */
  /*-----------------------------------------------------------*/
  usSide = DetermineSide (pbk, ppg, 0);                                 /*JEH1*/

  /*--------------------------------------*/
  /* select the color to paint the tab in */
  /*--------------------------------------*/
  if (ppg->fsAttr & BKA_MAJOR)
  {
    crTabColor = pbk->crColor[(ppg == pbk->ppgTopMajor) ? CLR_PAGEBG :
                                                          CLR_MAJORBG];
  }
  else
  {
    crTabColor = pbk->crColor[(ppg == pbk->ppgTopMinor) ? CLR_PAGEBG :
                                                          CLR_MINORBG];
  }

  /*--------------*/
  /* polygon tabs */
  /*--------------*/
  if (pbk->flStyle & BKS_POLYGONTABS)
  {
    DrawPolygonTabBorder (pbk, hdc, lprc, ppg, usSide, crTabColor);
  }

  /*--------------*/
  /* rounded tabs */
  /*--------------*/
  else if (pbk->flStyle & BKS_ROUNDEDTABS)
  {
    DrawRoundTabBorder (pbk, hdc, lprc, ppg, usSide, crTabColor);
  }

  /*-----------------------*/
  /* Square tabs (default) */
  /*-----------------------*/
  else
  {
    DrawSquareTabBorder (pbk, hdc, lprc, ppg, usSide, crTabColor);
  }
}


/*---------------------------------------------------------------------------*
 * DrawSquareTabBorder
 *
 * History:
 * 13-Sep-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawSquareTabBorder (
  PBOOK    pbk,                             /* book instance data (locked)   */
  HDC      hdc,
  LPRECT   lprc,
  PPAGE    ppg,
  USHORT   usSide,
  COLORREF crTabColor)
{
  /*---------------------------------------*/
  /* Fill the tab with the specified color */
  /*---------------------------------------*/
  BookFillRect (hdc, lprc, crTabColor);

  /*-----------------------------------------------------------*/
  /* Draw a square tab border using the notebook outline color */
  /*-----------------------------------------------------------*/
  BookFrameRect (hdc, lprc, pbk->crColor[CLR_OUTLINE]);

  /*--------------------------------------------*/
  /* if this tab is the top major or minor tab, */
  /* draw the line where the tab meets the page */
  /*--------------------------------------------*/
  if ((ppg == pbk->ppgTopMajor) || (ppg == pbk->ppgTopMinor))
  {
    DrawTopTabLine (hdc, lprc, usSide, crTabColor, BKS_SQUARETABS);
  }

  /*-------------------------------------------------------------*/
  /* Adjust the tab window border for the cursor emphasis. The   */
  /* border is adjusted by 2 on each side to account for the     */
  /* cursor emphasis and a space between the cursor emphasis and */
  /* the tab border.                                             */
  /*-------------------------------------------------------------*/
  InflateRect (lprc, -2, -2);

  /*-------------------------------------------------------------------*/
  /* If the tab has the cursor, draw a dashed border to indicate this. */
  /*-------------------------------------------------------------------*/
  if (ppg == pbk->ppgCursor)
  {
    DrawTabCursor (pbk, hdc, lprc, NULL, usSide, 0, 0);
  }

  /*---------------------------------------------------------------*/
  /* Adjust the tab window border for the inside text/bitmap. The  */
  /* border is adjusted by 1 on each side for the cursor emphasis. */
  /*---------------------------------------------------------------*/
  InflateRect (lprc, -1, -1);
}


/*---------------------------------------------------------------------------*
 * DrawPolygonTabBorder
 *
 * History:
 * 13-Sep-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawPolygonTabBorder (
  PBOOK    pbk,                             /* book instance data (locked)   */
  HDC      hdc,
  LPRECT   lprc,
  PPAGE    ppg,
  USHORT   wSide,                                                       /*JEH1*/
  COLORREF crTabColor)
{
  SHORT    sHRound;
  SHORT    sVRound;
  HBRUSH   hbrshTab;
  HPEN     hpenTab;
  static WORD   wLastSide = 0xFFFF;
  static RECT   rcLast = {0, 0, 0, 0};
  static POINT  pt[7];

  /*---------------------------------------------------------------*/
  /* fill the outer edge of the tab which, visibly, is not the tab */
  /*---------------------------------------------------------------*/
  BookFillRect (hdc, lprc, pbk->crColor[CLR_NOTEBOOK]);

  lprc->right--;
  lprc->bottom--;

  /*-------------------------------------------------------------------*/
  /* if the rectangle passed in and the side is the same as last time, */
  /* don't bother recalculating, just offset what we used last time    */
  /*-------------------------------------------------------------------*/
  if ((wSide == wLastSide)                                 &&
      (rcLast.right-rcLast.left == lprc->right-lprc->left) &&
      (rcLast.bottom-rcLast.top == lprc->bottom-lprc->top))
  {
    int  dx = lprc->left - rcLast.left;
    int  dy = lprc->top  - rcLast.top;
    WORD i;

    /*---------------------------------------------------------------*/
    /* if this is not the identical rectangle from last time, offset */
    /*---------------------------------------------------------------*/
    if (dx || dy)
    {
      for (i = 0; i < 7; i++)
      {
        pt[i].x += dx;  pt[i].y += dy;
      }
    }
  }

  /*--------------------------------------------------------*/
  /* otherwise, we don't have valid coordinates - recompute */
  /*--------------------------------------------------------*/
  else
  {
    /*----------------------------*/
    /* Determine type of display. */
    /*----------------------------*/
    if (vcyScreen > VGAY)
    {
      sHRound = BGALROUND;
      sVRound = BGALROUND;
    }
    else
    {
      sHRound = VGALROUND;
      sVRound = VGALROUND;
    }

    /*------------------------------------------------------------------*/
    /* Determine the horizontal and vertical length of the full axis    */
    /* of the ellipse used for rounding the tab edge. A fixed length is */
    /* used unless the tab cx or cy is smaller than this.               */
    /*------------------------------------------------------------------*/
    if (((ppg->fsAttr & BKA_MAJOR) && ((pbk->cxMajorTab < sHRound) ||
                                       (pbk->cyMajorTab < sVRound)))  ||
        ((ppg->fsAttr & BKA_MINOR) && ((pbk->cxMinorTab < sHRound) ||
                                       (pbk->cyMinorTab < sVRound))))
    {
      sVRound = (lprc->bottom - lprc->top) * 2 / 5;
      sHRound = (lprc->right - lprc->left) * 2 / 5;
    }

    /*--------------------------------------------------------------------*/
    /* Adjust the starting coordinates based on the Notebook orientation. */
    /*--------------------------------------------------------------------*/
    switch (wSide)
    {
      case BKV_RIGHT:
      case BKV_LEFT:
        PolyVertical ((PPOINTL)pt, lprc, wSide, sVRound, 5);            /*JEH1*/
        break;

      case BKV_BOTTOM:
      case BKV_TOP:
        PolyHorizontal ((PPOINTL)pt, lprc, wSide, sHRound, 5);          /*JEH1*/
        break;
    }

    /*------------------------*/
    /* remember for next time */
    /*------------------------*/
    wLastSide = wSide;
  }

  /*------------------------*/
  /* remember for next time */
  /*------------------------*/
  rcLast = *lprc;

  /*--------------*/
  /* draw the tab */
  /*--------------*/
  if ((hpenTab = CreatePen (PS_SOLID, 1, pbk->crColor[CLR_OUTLINE])) &&
      (hbrshTab = CreateSolidBrush (crTabColor)))
  {
    hpenTab  = SelectObject (hdc, hpenTab);
    hbrshTab = SelectObject (hdc, hbrshTab);
    Polygon (hdc, pt, 7);
    DeleteObject (SelectObject (hdc, hpenTab));
    DeleteObject (SelectObject (hdc, hbrshTab));
  }
  else if (hpenTab)   DeleteObject (hpenTab);

  lprc->right++;
  lprc->bottom++;

  /*--------------------------------------------*/
  /* if this tab is the top major or minor tab, */
  /* draw the line where the tab meets the page */
  /*--------------------------------------------*/
  if ((ppg == pbk->ppgTopMajor) || (ppg == pbk->ppgTopMinor))
  {
    DrawTopTabLine (hdc, lprc, wSide, crTabColor, BKS_POLYGONTABS);
  }

  /*----------------------------------------------------*/
  /* If the tab has the cursor, draw cursored emphasis. */
  /*----------------------------------------------------*/
  if (ppg == pbk->ppgCursor)
  {
    DrawTabCursor (pbk, hdc, NULL, pt, wSide, 0, 0);
  }

  /*----------------------------------------------------------*/
  /* Adjust the tab window border for the inside text/bitmap. */
  /*----------------------------------------------------------*/
  switch (wSide)
  {
    case BKV_RIGHT:
    case BKV_BOTTOM:
      SetRect (lprc, pt[0].x+3, pt[0].y+2, pt[4].x-3, pt[4].y-4);
      break;

    case BKV_LEFT:
    case BKV_TOP:
      SetRect (lprc, pt[1].x+4, pt[1].y+3, pt[5].x-2, pt[5].y-3);
      break;
  }
}


/*---------------------------------------------------------------------------*
 * DrawRoundTabBorder
 *
 * History:
 * 13-Sep-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawRoundTabBorder (
  PBOOK    pbk,                             /* book instance data (locked)   */
  HDC      hdc,
  LPRECT   lprc,
  PPAGE    ppg,
  USHORT   usSide,
  COLORREF crTabColor)
{
  HBRUSH   hbrshTab;
  HPEN     hpenTab;
  RECT     rcFill1, rcFill2;
  SHORT    xRadius = 7, yRadius = 7;

  /*------------------------------------------------------------------*/
  /* Determine the horizontal and vertical length of the full axis    */
  /* of the ellipse used for rounding the tab edge. A fixed length is */
  /* used unless the tab cx or cy is smaller than this.               */
  /*------------------------------------------------------------------*/
  if (((ppg->fsAttr & BKA_MAJOR) && ((pbk->cxMajorTab < xRadius) ||
                                     (pbk->cyMajorTab < yRadius)))  ||
      ((ppg->fsAttr & BKA_MINOR) && ((pbk->cxMinorTab < xRadius) ||
                                     (pbk->cyMinorTab < yRadius))))
  {
    yRadius = (lprc->bottom-lprc->top) * 2 / 5;
    xRadius = (lprc->right-lprc->left) * 2 / 5;
  }

  /*----------------------------------------------*/
  /* compute the rectangle we'll need to fill out */
  /* the round corners adjacent to the notebook   */
  /*----------------------------------------------*/
  rcFill1 = *lprc;

  switch (usSide)
  {
    case BKV_LEFT:
      rcFill1.left  = rcFill1.right - xRadius;
      rcFill2       = rcFill1;
      rcFill2.right = rcFill2.left;

      InflateRect (&rcFill2, 1, -1);
      break;

    case BKV_RIGHT:
      rcFill1.right = rcFill1.left + xRadius;
      rcFill2       = rcFill1;
      rcFill2.left  = rcFill2.right;

      InflateRect (&rcFill2, 1, -1);
      break;

    case BKV_TOP:
      rcFill1.top    = rcFill1.bottom - yRadius;
      rcFill2        = rcFill1;
      rcFill2.bottom = rcFill2.top;

      InflateRect (&rcFill2, -1, 1);
      break;

    case BKV_BOTTOM:
      rcFill1.bottom = rcFill1.top + yRadius;
      rcFill2        = rcFill1;
      rcFill2.top    = rcFill2.bottom;

      InflateRect (&rcFill2, -1, 1);
      break;
  }

  /*--------------*/
  /* draw the tab */
  /*--------------*/
  if ((hpenTab = CreatePen (PS_SOLID, 1, pbk->crColor[CLR_OUTLINE])) &&
      (hbrshTab = CreateSolidBrush (crTabColor)))
  {
    hpenTab = SelectObject (hdc, hpenTab);
    hbrshTab = SelectObject (hdc, hbrshTab);

    BookFillRect (hdc, lprc, pbk->crColor[CLR_NOTEBOOK]);
    RoundRect (hdc, lprc->left, lprc->top, lprc->right, lprc->bottom,
               2 * xRadius, 2 * yRadius);
    Rectangle (hdc, rcFill1.left, rcFill1.top, rcFill1.right, rcFill1.bottom);
    BookFillRect (hdc, &rcFill2, crTabColor);

    DeleteObject (SelectObject (hdc, hpenTab));
    DeleteObject (SelectObject (hdc, hbrshTab));
  }
  else if (hpenTab)  DeleteObject (hpenTab);

  /*-----------------------------------------------------------------*/
  /* if this is the page with the cursor, draw the cursor on the tab */
  /*-----------------------------------------------------------------*/
  if (ppg == pbk->ppgCursor)
  {
    DrawTabCursor (pbk, hdc, lprc, NULL, usSide, xRadius, yRadius);
  }

  /*--------------------------------------------*/
  /* if this tab is the top major or minor tab, */
  /* draw the line where the tab meets the page */
  /*--------------------------------------------*/
  if ((ppg == pbk->ppgTopMajor) || (ppg == pbk->ppgTopMinor))
  {
    DrawTopTabLine (hdc, lprc, usSide, crTabColor, BKS_ROUNDEDTABS);
  }

  /*----------------------------------------------------------*/
  /* Adjust the tab window border for the inside text/bitmap. */
  /*----------------------------------------------------------*/
  switch (usSide)
  {
    case BKV_LEFT:
      lprc->left   += xRadius/2;   lprc->right  -= 4;
      lprc->top    += 4;           lprc->bottom -= 4;
      break;

    case BKV_RIGHT:
      lprc->left   += 4;           lprc->right  -= xRadius/2 + 1;
      lprc->top    += 4;           lprc->bottom -= 4;
      break;

    case BKV_TOP:
      lprc->left   += 4;           lprc->right  -= 4;
      lprc->top    += yRadius/2;   lprc->bottom -= 4;
      break;

    case BKV_BOTTOM:
      lprc->left   += 4;           lprc->right  -= 4;
      lprc->top    += 4;           lprc->bottom -= yRadius/2;
      break;
  }
}


/*---------------------------------------------------------------------------*
 * DrawTabCursor
 *
 * Draws cursored emphasis on a tab.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawTabCursor(
  PBOOK   pbk,                              /* book instance data (locked)   */
  HDC     hdc,
  LPRECT  lprc,
  LPPOINT lppt,
  USHORT  usSide,                                                       /*JEH1*/
  WORD    xRadius,
  WORD    yRadius)
{
  HPEN  hpenCursor;

  /*-------------------------------------------------------------*/
  /* if the notebook doesn't have the focus, don't draw anything */
  /*-------------------------------------------------------------*/
  if (pbk->hwndBook != GetFocus () ||
      !(hpenCursor = CreatePen (PS_SOLID, 1, pbk->crColor[CLR_CURSOR])))
  {
    return;
  }

  hpenCursor = SelectObject (hdc, hpenCursor);

  /*--------------*/
  /* polygon tabs */
  /*--------------*/
  if (pbk->flStyle & BKS_POLYGONTABS)
  {
    DrawPolygonTabCursor (hdc, lppt, usSide);                           /*JEH1*/
  }

  /*--------------*/
  /* rounded tabs */
  /*--------------*/
  else if (pbk->flStyle & BKS_ROUNDEDTABS)
  {
    DrawRoundTabCursor (hdc, lprc, usSide, xRadius, yRadius);           /*JEH1*/
  }

  /*-------------*/
  /* Square tabs */
  /*-------------*/
  else
  {
    DrawSquareTabCursor (hdc, lprc);
  }

  DeleteObject (SelectObject (hdc, hpenCursor));
}


/*---------------------------------------------------------------------------*
 * DrawSquareTabCursor
 *
 * Draws the cursor on a square tab.
 *
 * History:
 * 14-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawSquareTabCursor (
  HDC    hdc,
  LPRECT lprc)
{
  /*---------------------------------------------------------*/
  /* draw the cursor a line at a time (don't use Rectangle() */
  /* because we don't want the interior to be filled)        */
  /*---------------------------------------------------------*/
  MoveToEx (hdc, lprc->left,    lprc->top, NULL);                       /*JEH1*/
  LineTo (hdc, lprc->right-1, lprc->top);
  LineTo (hdc, lprc->right-1, lprc->bottom-1);
  LineTo (hdc, lprc->left,    lprc->bottom-1);
  LineTo (hdc, lprc->left,    lprc->top);
}


/*---------------------------------------------------------------------------*
 * DrawRoundTabCursor
 *
 * Draws the cursor on a rounded tab.
 *
 *     1       0          3       2        5     4        2         1
 *        5      4  Ŀ       0    3       Ŀ
 *                                                          
 *                                                          
 *                                                          
 *        4      5                 3    0
 *     2       3          0       1      1         2        4     5
 *
 *     Tab on right     Tab on left       Tab on top     Tab on bottom
 *
 * History:
 * 14-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawRoundTabCursor (
  HDC     hdc,
  LPRECT  lprc,
  WORD    wSide,
  WORD    xRadius,
  WORD    yRadius)
{
  RECT          rcCursor;
  static RECT   rcArc3to4, rcArc5to0, rcLast;
  static POINT  pt[6];
  static WORD   wLastSide;

  /*-------------------------------------------------------------------*/
  /* if the rectangle passed in and the side is the same as last time, */
  /* don't bother recalculating, just offset what we used last time    */
  /*-------------------------------------------------------------------*/
  if ((wSide == wLastSide)                                 &&
      (rcLast.right-rcLast.left == lprc->right-lprc->left) &&
      (rcLast.bottom-rcLast.top == lprc->bottom-lprc->top))
  {
    int  dx = lprc->left - rcLast.left;
    int  dy = lprc->top  - rcLast.top;
    WORD i;

    /*---------------------------------------------------------------*/
    /* if this is not the identical rectangle from last time, offset */
    /*---------------------------------------------------------------*/
    if (dx || dy)
    {
      /*-------------------------------------------------------------*/
      /* Offset all 6 points in the rectangle (NOT 7 as in polygon)  */
      /*-------------------------------------------------------------*/
      for (i = 0; i < 6; i++)                                           /*DAB2*/
      {
        pt[i].x += dx;  pt[i].y += dy;
      }

      OffsetRect (&rcArc3to4, dx, dy);
      OffsetRect (&rcArc5to0, dx, dy);
    }
  }

  /*--------------------------------------------------------*/
  /* otherwise, we don't have valid coordinates - recompute */
  /*--------------------------------------------------------*/
  else
  {
    /*--------------------------------------*/
    /* copy the rectangle for the tab and   */
    /* inset it to make room for the cursor */
    /*--------------------------------------*/
    rcCursor = *lprc,

    xRadius -= CX_CURSORINSET;
    yRadius -= CX_CURSORINSET;
    InflateRect (&rcCursor, -CX_CURSORINSET, -CX_CURSORINSET);

    rcCursor.right--;
    rcCursor.bottom--;

    /*---------------------------------------*/
    /* set up the points that define the tab */
    /*---------------------------------------*/
    switch (wSide)
    {
      case BKV_RIGHT:
        pt[0].x = rcCursor.right - xRadius;  pt[0].y = rcCursor.top;
        pt[1].x = rcCursor.left;             pt[1].y = pt[0].y;
        pt[2].x = pt[1].x;                   pt[2].y = rcCursor.bottom;
        pt[3].x = pt[0].x;                   pt[3].y = pt[2].y;
        pt[4].x = rcCursor.right;            pt[4].y = rcCursor.bottom - yRadius;
        pt[5].x = pt[4].x;                   pt[5].y = rcCursor.top    + yRadius;

        SetRect (&rcArc3to4, pt[3].x-xRadius, pt[4].y-yRadius, pt[4].x, pt[3].y);
        SetRect (&rcArc5to0, pt[0].x-xRadius, pt[0].y, pt[5].x, pt[5].y+yRadius);
        OffsetRect (&rcArc5to0, 0, 1);
        break;

      case BKV_LEFT:
        pt[0].x = rcCursor.left  + xRadius;  pt[0].y = rcCursor.bottom;
        pt[1].x = rcCursor.right;            pt[1].y = pt[0].y;
        pt[2].x = pt[1].x;                   pt[2].y = rcCursor.top;
        pt[3].x = pt[0].x;                   pt[3].y = pt[2].y;
        pt[4].x = rcCursor.left;             pt[4].y = rcCursor.top    + yRadius;
        pt[5].x = pt[4].x;                   pt[5].y = rcCursor.bottom - yRadius;

        SetRect (&rcArc3to4, pt[4].x, pt[3].y, pt[3].x+xRadius, pt[4].y+yRadius);
        SetRect (&rcArc5to0, pt[5].x, pt[5].y-yRadius, pt[0].x+xRadius, pt[0].y);
        OffsetRect (&rcArc3to4, 1, 1);
        OffsetRect (&rcArc5to0, 1, 0);
        break;

      case BKV_TOP:
        pt[0].x = rcCursor.left;             pt[0].y = rcCursor.top    + yRadius;
        pt[1].x = pt[0].x;                   pt[1].y = rcCursor.bottom;
        pt[2].x = rcCursor.right;            pt[2].y = pt[1].y;
        pt[3].x = pt[2].x;                   pt[3].y = pt[0].y;
        pt[4].x = rcCursor.right - xRadius;  pt[4].y = rcCursor.top;
        pt[5].x = rcCursor.left  + xRadius;  pt[5].y = pt[4].y;

        SetRect (&rcArc3to4, pt[4].x-xRadius, pt[4].y, pt[3].x, pt[3].y+yRadius);
        SetRect (&rcArc5to0, pt[0].x, pt[5].y, pt[5].x+xRadius, pt[0].y+yRadius);
        OffsetRect (&rcArc3to4, 0, 1);
        OffsetRect (&rcArc5to0, 1, 1);
        break;

      case BKV_BOTTOM:
        pt[0].x = rcCursor.right;            pt[0].y = rcCursor.bottom - yRadius;
        pt[1].x = pt[0].x;                   pt[1].y = rcCursor.top;
        pt[2].x = rcCursor.left;             pt[2].y = pt[1].y;
        pt[3].x = pt[2].x;                   pt[3].y = pt[0].y;
        pt[4].x = rcCursor.left  + xRadius;  pt[4].y = rcCursor.bottom;
        pt[5].x = rcCursor.right - xRadius;  pt[5].y = pt[4].y;

        SetRect (&rcArc3to4, pt[3].x, pt[3].y-yRadius, pt[4].x+xRadius, pt[4].y);
        SetRect (&rcArc5to0, pt[5].x-xRadius, pt[0].y-yRadius, pt[0].x, pt[5].y);
        OffsetRect (&rcArc3to4, 1, 0);
        break;
    }

    /*------------------------*/
    /* remember for next time */
    /*------------------------*/
    wLastSide = wSide;
  }

  /*------------------------*/
  /* remember for next time */
  /*------------------------*/
  rcLast = *lprc;

  /*--------------------------------------------------------------*/
  /* draw the tab by drawing the three contiguous sides, then the */
  /* disjoint side, followed by the 3-to-4 arc and the 5-to-0 arc */
  /*--------------------------------------------------------------*/
  BookPolyline    (hdc, pt,   4);
  BookPolyline    (hdc, pt+4, 2);
  DrawRoundTabArc (hdc, &rcArc3to4, pt[3].x, pt[3].y, pt[4].x, pt[4].y);
  DrawRoundTabArc (hdc, &rcArc5to0, pt[5].x, pt[5].y, pt[0].x, pt[0].y);
}


/*---------------------------------------------------------------------------*
 * DrawRoundTabArc
 *
 * Draws the 90-degree arc that forms a corner of a rounded tab.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawRoundTabArc (
  HDC    hdc,
  LPRECT lprcArcBound,
  int  xStart,                                                          /*JEH1*/
  int  yStart,                                                          /*JEH1*/
  int  xStop,                                                           /*JEH1*/
  int  yStop)                                                           /*JEH1*/
{
  Arc (hdc, lprcArcBound->left, lprcArcBound->top, lprcArcBound->right,
       lprcArcBound->bottom, xStart, yStart, xStop, yStop);
}


/*---------------------------------------------------------------------------*
 * DrawPolygonTabCursor
 *
 * Draws the cursor on a polygon tab.
 *
 * History:
 * 14-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawPolygonTabCursor (
  HDC     hdc,
  LPPOINT lppt,
  USHORT  usSide)                                                       /*JEH1*/
{
  POINT pt[7];

  memcpy (pt, lppt, 7 * sizeof (POINT));

  switch (usSide)                                                       /*JEH1*/
  {
    case BKV_RIGHT:
        pt[0].x += CX_CURSORINSET;       pt[0].y += CX_CURSORINSET;
        pt[1].x -= CX_CURSORINSET / 2;   pt[1].y += CX_CURSORINSET;
        pt[2].x -= CX_CURSORINSET;       pt[2].y += CX_CURSORINSET / 2;
        pt[3].x -= CX_CURSORINSET;       pt[3].y -= CX_CURSORINSET / 2;
        pt[4].x -= CX_CURSORINSET / 2;   pt[4].y -= CX_CURSORINSET;
        pt[5].x += CX_CURSORINSET;       pt[5].y -= CX_CURSORINSET;
        pt[6].x += CX_CURSORINSET;       pt[6].y += CX_CURSORINSET;
        break;

    case BKV_LEFT:
        pt[0].x -= CX_CURSORINSET;       pt[0].y += CX_CURSORINSET;
        pt[1].x += CX_CURSORINSET / 2;   pt[1].y += CX_CURSORINSET;
        pt[2].x += CX_CURSORINSET;       pt[2].y += CX_CURSORINSET / 2;
        pt[3].x += CX_CURSORINSET;       pt[3].y -= CX_CURSORINSET / 2;
        pt[4].x += CX_CURSORINSET / 2;   pt[4].y -= CX_CURSORINSET;
        pt[5].x -= CX_CURSORINSET;       pt[5].y -= CX_CURSORINSET;
        pt[6].x -= CX_CURSORINSET;       pt[6].y += CX_CURSORINSET;
        break;

    case BKV_TOP:
        pt[0].x += CX_CURSORINSET;       pt[0].y -= CX_CURSORINSET;
        pt[1].x += CX_CURSORINSET;       pt[1].y += CX_CURSORINSET / 2;
        pt[2].x += CX_CURSORINSET / 2;   pt[2].y += CX_CURSORINSET;
        pt[3].x -= CX_CURSORINSET / 2;   pt[3].y += CX_CURSORINSET;
        pt[4].x -= CX_CURSORINSET;       pt[4].y += CX_CURSORINSET / 2;
        pt[5].x -= CX_CURSORINSET;       pt[5].y -= CX_CURSORINSET;
        pt[6].x += CX_CURSORINSET;       pt[6].y -= CX_CURSORINSET;
        break;

    case BKV_BOTTOM:
        pt[0].x += CX_CURSORINSET;       pt[0].y += CX_CURSORINSET;
        pt[1].x += CX_CURSORINSET;       pt[1].y -= CX_CURSORINSET / 2;
        pt[2].x += CX_CURSORINSET / 2;   pt[2].y -= CX_CURSORINSET;
        pt[3].x -= CX_CURSORINSET / 2;   pt[3].y -= CX_CURSORINSET;
        pt[4].x -= CX_CURSORINSET;       pt[4].y -= CX_CURSORINSET / 2;
        pt[5].x -= CX_CURSORINSET;       pt[5].y += CX_CURSORINSET;
        pt[6].x += CX_CURSORINSET;       pt[6].y += CX_CURSORINSET;
        break;
  }

  BookPolyline (hdc, pt, 7);
}


/*---------------------------------------------------------------------------*
 * DrawTabText
 *
 * Draws the text string a tab.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawTabText(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppg,
  HDC    hdc,
  LPRECT lprc)
{
  USHORT   fsCmd = DT_VCENTER | DT_SINGLELINE;
  COLORREF crFore, crBack;
  BOOL     fSetFont;

  /*---------------------------------------*/
  /* if there is no tab text, don't bother */
  /*---------------------------------------*/
  if (!ppg->ptab->npszText)  return;

  /*----------------------------------------------------------*/
  /* Determine the tab text justification based on the window */
  /*----------------------------------------------------------*/
  fsCmd |= (pbk->flStyle & BKS_TABTEXTRIGHT)  ? DT_RIGHT :
           (pbk->flStyle & BKS_TABTEXTCENTER) ? DT_CENTER  : DT_LEFT;

  /*--------------------*/
  /* Set tab text color */
  /*--------------------*/
  crFore = pbk->crColor[(ppg->fsAttr & BKA_MAJOR) ? CLR_MAJORFG : CLR_MINORFG];
  crBack = pbk->crColor[(ppg->fsAttr & BKA_MAJOR) ? CLR_MAJORBG : CLR_MINORBG];

  /*----------------------------------------------------------------------*/
  /* if this is the top tab, the text background should be the page color */
  /*----------------------------------------------------------------------*/
  if ((ppg == pbk->ppgTopMajor) || (ppg == pbk->ppgTopMinor))
  {
    crBack = pbk->crColor[CLR_PAGEBG];
  }

  /*---------------*/
  /* Draw tab text */
  /*---------------*/
  crFore = SetTextColor (hdc, crFore);
  crBack = SetBkColor (hdc, crBack);

  if (fSetFont = (BOOL)pbk->hFont)                                      /*JEH1*/
    pbk->hFont = SelectObject (hdc, pbk->hFont);
  DrawText (hdc, ppg->ptab->npszText, -1, lprc, fsCmd);
  if (fSetFont)   pbk->hFont = SelectObject (hdc, pbk->hFont);

  SetTextColor (hdc, crFore);
  SetBkColor   (hdc, crBack);
}


/*---------------------------------------------------------------------------*
 * DrawTopTabLine
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID DrawTopTabLine (
  HDC      hdc,                                                         /*JEH1*/
  LPRECT   lprc,
  USHORT   usSide,                                                      /*JEH1*/
  COLORREF crColor,
  USHORT   usTabType)
{
  POINT pt[2];
  HPEN  hpenTab;

  switch (usSide)                                                       /*JEH1*/
  {
    case BKV_RIGHT:
      pt[0].x = lprc->left;        pt[0].y = lprc->top    + 1;
      pt[1].x = lprc->left;        pt[1].y = lprc->bottom - 1;
      break;

    case BKV_LEFT:
      pt[0].x = lprc->right - 1;   pt[0].y = lprc->top    + 1;
      pt[1].x = lprc->right - 1;   pt[1].y = lprc->bottom - 1;
      break;

    case BKV_TOP:
      pt[0].x = lprc->left  + 1;   pt[0].y = lprc->bottom - 1;
      pt[1].x = lprc->right - 1;   pt[1].y = lprc->bottom - 1;
      break;

    case BKV_BOTTOM:
      pt[0].x = lprc->left  + 1;   pt[0].y = lprc->top;
      pt[1].x = lprc->right - 1;   pt[1].y = lprc->top;
      break;
  }

  if (hpenTab = CreatePen (PS_SOLID, 1, crColor))
  {
    hpenTab = SelectObject (hdc, hpenTab);
    Polyline (hdc, pt, 2);
    DeleteObject (SelectObject (hdc, hpenTab));
  }
}


/*---------------------------------------------------------------------------*
 * DrawUpdateRect
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID DrawUpdateRect(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgTabStart,
  PPAGE  ppgTabEnd,
  LPRECT lprcUpdate,
  HDC    hdc)
{
  BOOL    fReleaseDC = (hdc == NULL);
  RECT    rcUpdate, rcTopPage;
  HBITMAP hbmMem;

  /*----------------------------------------*/
  /* if the notebook isn't visible, or the  */
  /* update rectangle is empty, just return */
  /*----------------------------------------*/
  if (!IsWindowVisible (pbk->hwndBook) || IsRectEmpty (lprcUpdate))
  {
    return;
  }

  /*------------------------------------------------------------------------*/
  /* offset a rect to 0 so we'll draw at the proper coords in the memory DC */
  /*------------------------------------------------------------------------*/
  rcUpdate  = *lprcUpdate;
  rcTopPage = pbk->rcTopPage;
  OffsetRect (&rcUpdate,  1-lprcUpdate->left, 1-lprcUpdate->top);
  OffsetRect (&rcTopPage,  -lprcUpdate->left,  -lprcUpdate->top);

  InflateRect (&rcUpdate, 1, 1);

  /*----------------------------------------------------------------*/
  /* if we weren't given a DC and we couldn't get one ourselves, or */
  /* we couldn't create a compatible memory bitmap, bail out here   */
  /*----------------------------------------------------------------*/
  if ((fReleaseDC && !(hdc = GetDC (pbk->hwndBook))) ||
      !(hbmMem = CreateCompatibleBitmap (hdc, rcUpdate.right, rcUpdate.bottom)))
  {
    if (fReleaseDC && hdc)  ReleaseDC (pbk->hwndBook, hdc);
    return;
  }

  hbmMem = SelectObject (pbk->hdcMem, hbmMem);

  /*--------------------------------------------*/
  /* redraw the invalid portion of the notebook */
  /*--------------------------------------------*/
  BookFillRect (pbk->hdcMem, &rcUpdate, pbk->crColor[CLR_NOTEBOOK]);
  DrawBookBackpages (pbk, pbk->hdcMem, &rcTopPage, &rcUpdate);

#define MEMORY_DC
#ifndef MEMORY_DC
  BitBlt (hdc, lprcUpdate->left, lprcUpdate->top,
          rcUpdate.right-2, rcUpdate.bottom-2, pbk->hdcMem, 0, 0, SRCCOPY);
  DrawTabs (pbk, hdc, ppgTabStart, ppgTabEnd, lprcUpdate);
#else
  DrawTabs (pbk, pbk->hdcMem, ppgTabStart, ppgTabEnd, lprcUpdate);
  BitBlt (hdc, lprcUpdate->left, lprcUpdate->top,
          rcUpdate.right-2, rcUpdate.bottom-2, pbk->hdcMem, 0, 0, SRCCOPY);
#endif

  DeleteObject (SelectObject (pbk->hdcMem, hbmMem));
  if (fReleaseDC)  ReleaseDC (pbk->hwndBook, hdc);
}


/*---------------------------------------------------------------------------*
 * DrawTabs
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawTabs (
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  PPAGE  ppgFirstTab,
  PPAGE  ppgLastTab,
  LPRECT lprcDraw)
{
  DRAWITEMSTRUCT dis;
  PPAGE  ppgTab;
  RECT   rcDraw;
  USHORT usType;

  if (ppgTab = ppgFirstTab)
  {
    if      ((ppgTab->fsAttr & BKA_MAJOR) && pbk->cxMajorTab) usType = BKA_MAJOR;
    else if ((ppgTab->fsAttr & BKA_MINOR) && pbk->cxMinorTab) usType = BKA_MINOR;
    else                                                      return;
  }

  while (ppgTab)
  {
    if (IntersectRect (&rcDraw, &ppgTab->ptab->rc, lprcDraw))
    {
      /*---------------------------------*/
      /* Calculate tab drawing rectangle */
      /*---------------------------------*/
      rcDraw = ppgTab->ptab->rc;
#ifdef MEMORY_DC
      OffsetRect (&rcDraw, -lprcDraw->left, -lprcDraw->top);
#endif

      /*---------------------*/
      /* Draw the tab border */
      /*---------------------*/
      DrawTabBorder (pbk, hdc, &rcDraw, ppgTab);

      /*--------------------------*/
      /* Draw the tab text/bitmap */
      /*--------------------------*/
      if (ppgTab->fsAttr & BKA_TEXT)
      {
        DrawTabText (pbk, ppgTab, hdc, &rcDraw);
      }

      else if (ppgTab->fsAttr & BKA_BITMAP)
      {
        DrawTabBitmap (hdc, ppgTab->ptab->hbmTab, &rcDraw);
      }

      else
      {
        HRGN hrgnClip;

        /*---------------------------------------------------*/
        /* Set up the owner item structure to pass to owner. */
        /*---------------------------------------------------*/
        dis.CtlType    = ODT_NOTEBOOK;
        dis.CtlID      = IDOF (pbk->hwndBook);
        dis.itemID     = (UINT) ppgTab;                                 /*DAB1*/
        dis.itemAction = ODA_DRAWENTIRE;
        dis.itemState  = 0;                                             /*JEH1*/
        dis.hwndItem   = pbk->hwndBook;
        dis.hDC        = hdc;
        dis.rcItem     = rcDraw;
        dis.itemData   = 0;                                             /*JEH1*/

        /*-------------------------------------------------------------------*/
        /* clip the DC to the tab so the app can't draw outside the tab area */
        /*-------------------------------------------------------------------*/
        SaveDC (hdc);
        if (hrgnClip = CreateRectRgnIndirect (&rcDraw))
        {
          SelectClipRgn (hdc, hrgnClip);
          DeleteObject (hrgnClip);
        }

        /*------------------------------------------------------------*/
        /* Now send the draw item message to the parent, passing      */
        /* the information to him for drawing in the structure        */
        /*------------------------------------------------------------*/
        SendMessage( PARENTOF(pbk->hwndBook), WM_DRAWITEM,
                    (WPARAM)dis.CtlID, (DWORD) (LPDRAWITEMSTRUCT) &dis);/*DAB3*/

        /*---------------------------------------*/
        /* restore the previous clipping region  */
        /*---------------------------------------*/
        RestoreDC (hdc, -1);
      }
    }

    ppgTab = (ppgTab == ppgLastTab) ?
             NULL : GetTabPtr (ppgTab->ptab->ppgNextTab, usType, BKA_NEXT);
  }
}
