/* Revision: 47 1.4 source/ui/cclnt/notebook/bookinit.c, cclbook, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992 (Unpublished)         */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                          N O T E B O O K                           */
/*                                                                    */
/*                       Filename: BOOKINIT.C                         */
/*                                                                    */
/*             ( Windows NT Notebook Control Component )              */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/******************************************************************************/
/* Product: ICLUINT.DLL                                                       */
/* Filename: BOOKINIT.C - Initialization code for the Notebook control        */
/*                                                                            */
/* Flag    Date    Defect  Description                                        */
/* ----  --------  -----  --------------------------------------------------- */
/* MAB1  01/16/96  6614   Backpages spacing not calculated properly           */
/* MAB2  01/23/96  6714   Initialization of cppwot3.dll failing sporaticly.   */
/******************************************************************************/
#include <windows.h>
#include "iclnbw.h"
#include "bookp.h"

// Segment definitions
#ifdef IC_PAGETUNE
  #define _BOOKINIT_C_
  #include "icltune.h"
#endif

HANDLE    vhInstance  = 0;
USHORT    vcxScreen   = 0;
USHORT    vcyScreen   = 0;
USHORT    vsDevIndX   = 0;       /* horizontal screen dev factor */
USHORT    vsDevIndY   = 0;       /* vertical screen dev factor   */
USHORT    vcxIcon     = 0;
USHORT    vcyIcon     = 0;
HHOOK     vlpNextHook = 0;

#if !defined(CCL_SEPARATE_DLL)
#define LibMain NotebookLibMain
#endif

INT  APIENTRY LibMain(HANDLE hInstance,
                      DWORD  ul_reason_being_called,
                      LPVOID lpReserved) ;

#if defined(CCL_SEPARATE_DLL)

#if defined(__IBMC__)

/* _CRT_init is the C run-time environment initialization function.         */
/* It will return 0 to indicate success and -1 to indicate failure.         */
int _CRT_init(void);

/* _DLL_InitTerm is the function that gets called by the operating system   */
/* loader when it loads and frees this DLL for each process that accesses   */
/* this DLL.  However, it only gets called the first time the DLL is loaded */
/* and the last time it is freed for a particular process.  The system      */
/* linkage convention MUST be used because the operating system loader is   */
/* calling this function.                                                   */
/* See DllEntryPoint in SDK documentation                                   */

INT APIENTRY          _DLL_InitTerm(HANDLE hInstance,
                                    DWORD  reasonCalled,
                                    LPVOID reserved );

INT APIENTRY          _DLL_InitTerm(HANDLE hInstance,
                                    DWORD  reasonCalled,
                                    LPVOID reserved )
{
   INT  rc = FALSE;     /* failed */
   switch (reasonCalled)
      {
      case  DLL_PROCESS_ATTACH:
         /*******************************************************************/
         /* The C run-time environment initialization function must be      */
         /* called before any calls to C run-time functions that are not    */
         /* inlined.                                                        */
         /*******************************************************************/

         if (_CRT_init() != -1)
            {
            /* initialize the controls */
            rc = LibMain( hInstance, reasonCalled, reserved );
            }
         break;

      case  DLL_THREAD_ATTACH:
      case  DLL_THREAD_DETACH:
      case  DLL_PROCESS_DETACH:
         /* We currently don't do anything with these */
         break;
      }
   return rc;
}

#endif

#endif  //CCL_SEPARATE_DLL

/****************************************************************************
   FUNCTION: LibMain(HANDLE, DWORD, LPVOID)

   PURPOSE:  LibMain is called by Windows when
             the DLL is initialized, Thread Attached, and other times.
             Refer to SDK documentation, as to the different ways this
             may be called.

             The LibMain function should perform additional initialization
             tasks required by the DLL.  In this example, no initialization
             tasks are required.  LibMain should return a value of 1 if
             the initialization is successful.

*******************************************************************************/
INT  APIENTRY LibMain(HANDLE hInstance,
                      DWORD  ul_reason_being_called,
                      LPVOID lpReserved)
{
  HCURSOR hcur;
  USHORT  i;
  LOGFONT logfont;                                                     //->@MAB1
  HFONT   systemFont;
  LONG    lDevX = 0;
  LONG    lDevY = 0;                                                   //@MAB1<-
  BOOL    rc;

#define CLASS_STYLES   CS_HREDRAW | CS_VREDRAW | CS_GLOBALCLASS

  static WNDCLASS wcls[] = {
    { CLASS_STYLES, BookWndProc,       0, 8, 0, 0, 0,
                    (HBRUSH)(COLOR_3DFACE+1), 0, ICL_NOTEBOOK},
    { CLASS_STYLES, PageWndProc,       0, 0, 0, 0, 0,
                    (HBRUSH)(COLOR_3DFACE+1), 0, WC_PAGE},
    { CLASS_STYLES, PageButtonWndProc, 0, 0, 0, 0, 0,
                    (HBRUSH)(COLOR_3DFACE+1), 0, WC_PAGEBUTTON},
    { CLASS_STYLES, ScrollWndProc,     0, 0, 0, 0, 0,
                    (HBRUSH)(COLOR_3DFACE+1), 0, WC_SCROLLARROW}};

  if ( (ul_reason_being_called == DLL_PROCESS_ATTACH)  &&  !vhInstance )
  {
    hcur = LoadCursor (NULL, IDC_ARROW);

    /*---------------------------------------------------*/
    /* register each of the classes used by the notebook */
    /*---------------------------------------------------*/
    for (i=0; i < sizeof (wcls)/sizeof (WNDCLASS); i++)
    {
      wcls[i].hInstance = hInstance;
      wcls[i].hCursor   = hcur;

      rc = RegisterClass(wcls + i);

      if (!rc)
      {
        return (FALSE);
      }
    }

    vhInstance = hInstance;
    vcxScreen  = GetSystemMetrics (SM_CXSCREEN);
    vcyScreen  = GetSystemMetrics (SM_CYSCREEN);
    vcxIcon    = GetSystemMetrics (SM_CXICON);
    vcyIcon    = GetSystemMetrics (SM_CYICON);

    /*-----------------------------------------------------*/          //->@MAB1
    /* The following calculation emulates the PM API,      */
    /* WinMapDlgPoints, so the device factor is calculated */
    /* properly for Hi-Res displays.                       */
    /*-----------------------------------------------------*/
    systemFont = GetStockObject( SYSTEM_FONT );
    GetObject( systemFont, sizeof(LOGFONT), &logfont );

    lDevX = ( (VGAX * 4) + (logfont.lfWidth/2) ) / logfont.lfWidth;
    lDevY = ( (VGAY * 8) + (logfont.lfHeight/2) ) / logfont.lfHeight;

    /*-----------------------------------------------------*/
    /* Store horizontal and vertical screen device factors */
    /* in globals which are used during the drawing of     */
    /* various parts of the notebook.                      */
    /*-----------------------------------------------------*/
    vsDevIndX  = max ((lDevX/VGAX), 1);
    vsDevIndY  = max ((lDevY/VGAY), 1);

//    vsDevIndX  = max (vcxScreen / VGAX, 1);
//    vsDevIndY  = max (vcyScreen / VGAY, 1);                          //@MAB1<-

    /*-----------------------------------------------*/
    /* set a hook to catch focus changing keystrokes */
    /*-----------------------------------------------*/
    vlpNextHook = SetWindowsHookEx (WH_KEYBOARD,
                                    (HOOKPROC)BookKeyboardHookProc,
                                    hInstance,
                                    GetCurrentThreadId() );

    if ( !vlpNextHook )                                                //->@MAB2
      return (FALSE);
  }

  /*--------------------------------------------------------*/
  /* unhook the hook that catches focus changing keystrokes */
  /*--------------------------------------------------------*/
  if (ul_reason_being_called == DLL_PROCESS_DETACH)
  {
    INT rc = FALSE;
    if ( vlpNextHook )
    {
      rc = UnhookWindowsHookEx( vlpNextHook );
      vlpNextHook = 0;
      vhInstance  = 0;
    }
    return( rc );
  }                                                                    //@MAB2<-

  return (TRUE);

  lpReserved;
}
