/* Revision: 49 1.6 source/ui/cclnt/notebook/bookproc.c, cclbook, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992 (Unpublished)         */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                          N O T E B O O K                           */
/*                                                                    */
/*                       Filename: BOOKPROC.C                         */
/*                                                                    */
/*             ( Windows NT Notebook Control Component )              */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/******************************************************************************/
/* Product: ICLUINT.DLL                                                       */
/* Filename: BOOKPROC.C - Window procedures and supporting functions          */
/*                        for all the classes of the Notebook control         */
/*                                                                            */
/* Flag    Date    Defect  Description                                        */
/* ----  --------  -----  --------------------------------------------------- */
/* JEH1  06/22/93         Initial modifications for NT execution              */
/* DAB1  12/01/93         Additional modifications for NT execution           */
/* DAB2  05/20/94         Modify message interface to match OS/2 API          */
/* DAB3  10/04/94         Updated notifications to return information         */
/*                        supported in PM which was left out of Win3.1        */
/* DAB4  10/14/95         Corrected logic error in page count when empty      */
/* DAB5  10/23/95         Added WM_HELP processing since VK_F1 is hooked      */
/* MAB3  01/13/96  6468   <Alt> UpArrow not working - cannot use keyboard to  */
/*                        move from page window to tab.                       */
/* MAB4  01/19/96  6557   Scroll arrow drawn over first tab, and not          */
/*                        repainting properly.                                */
/* MAB5  01/27/96  6830   QueryPageCount returns inaccurate count             */
/* MAB6  01/28/96  6681   Help notification not returning page handle         */
/* JKS1  02/09/96  6555   No longer need to look for VK_F1                    */
/* MAB7  03/08/96  7505   Non-top application page windows drawn on top page  */
/******************************************************************************/
#include <windows.h>
#include <ctype.h>
#include "iclnbw.h"
#include "bookp.h"

HANDLE    vhBookResource[CRESOURCES] = { 0 };
USHORT    vcNotebooks = 0;
PPAGE     vppgDeleted = 0;
PPAGE     vppgPrevSelected = 0;

#ifdef DEBUG
USHORT    _aDBused = 0;
#endif

extern HANDLE  vhInstance;
extern USHORT  vcxScreen;
extern USHORT  vcyScreen;
extern USHORT  vcxIcon;
extern USHORT  vcyIcon;
extern USHORT  vsDevIndX;
extern USHORT  vsDevIndY;
extern HHOOK   vlpNextHook;

/*---------------------------------------------------------------------------*
 * BookWndProc
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

WNDPROC BookWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) /*JEH1*/
{
  PPAGE ppg;
  RECT  rcUpdate;
  LONG  rc;
  PBOOK pbk = (PBOOK) GetWindowLong (hwnd, QWL_NOTEBOOK);               /*JEH1*/

  switch (msg)
  {
    case WM_CREATE:
      return (Initialize (hwnd));
      break;

    case WM_DESTROY:
      if (pbk)  TerminateNotebook (pbk);
      break;

      /****************************************************************/
      /* Calculate the regions of the book                            */
      /****************************************************************/
    case WM_SIZE:
      ResetNotebookRegions (pbk);
      break;

    case WM_GETDLGCODE:
      return (DLGC_WANTALLKEYS);
      break;

    case WM_PAINT:
      PaintBook (pbk);
      break;

    case WM_ERASEBKGND:
      return (TRUE);
      break;

      /******************************************************************/
      /* Process the help message                                       */
      /******************************************************************/
    case WM_HELP:                                                       /*DAB5*/
      if (pbk->ppgCursor)                                               /*DAB5*/
      {                                                                 /*DAB5*/
        NotifyParent( pbk->hwndBook, BKN_HELP, TRUE,                    /*DAB5*/
                      (LRESULT)(pbk->ppgCursor) );                      /*DAB5*/
      }                                                                 /*DAB5*/
      break;                                                            /*DAB5*/

    case WM_LBUTTONDOWN:
    {                                                                   /*JEH1*/
      POINT pt;                                                         /*JEH1*/

      BookSetFocus (pbk->hwndBook);
      SendSetCursor (hwnd, msg);
      pt.x = LOWORD(lParam);                                            /*JEH1*/
      pt.y = HIWORD(lParam);                                            /*JEH1*/
      /*--------------------------------------------------------------------*/
      /* If the mouse is over a tab, bring this page to the top of the book */
      /*--------------------------------------------------------------------*/
      if (ppg = ProcessMouseButtonDown (pbk, pt))                       /*JEH1*/
        GotoPage (pbk, ppg, 0, TRUE);                                   /*JEH1*/
      break;
    }                                                                   /*JEH1*/
    case WM_MOUSEMOVE:
    case WM_LBUTTONUP:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_MBUTTONDOWN:
    case WM_MBUTTONUP:
      SendSetCursor (hwnd, msg);
      break;

    case WM_KEYDOWN:
    case WM_SYSKEYDOWN:
      return (ProcessKeyDown (pbk, msg, wParam, lParam));
      break;

    case WM_SETFOCUS:
      /*---------------------------------------------------------*/
      /* if we didn't set the focus to ourselves by processing a */
      /* WM_LBUTTONDOWN or Alt+Up, move the cursor to a major    */
      /* tab, if it exists                                       */
      /*---------------------------------------------------------*/
      if (!(pbk->flState & BKV_SELFFOCUS) && pbk->ppgCursorMajor &&
          IsSibling (hwnd, (HWND) wParam))
      {
        pbk->ppgCursor = pbk->ppgCursorMajor;
      }
      /* flow through */

    case WM_KILLFOCUS:
      if (pbk->ppgCursor && IsTabVisible (pbk, pbk->ppgCursor))
      {
        CalcInvalidateRect (pbk, pbk->ppgCursor, pbk->ppgCursor, &rcUpdate);
        DrawUpdateRect (pbk, pbk->ppgCursor, pbk->ppgCursor, &rcUpdate, NULL);
      }
      else
      {
        InvalidateRect (hwnd, &pbk->rcAppPageWnd, FALSE);
      }
      break;

    case WM_SETFONT:
      pbk->hFont = (HFONT)wParam;                                       /*JEH1*/
      if (lParam)  InvalidateRect (hwnd, NULL, FALSE);
      break;

    case WM_GETFONT:
      return (LRESULT)(pbk->hFont);                                     /*JEH1*/
      break;


/*---------------------------------------------------------------------------*/
/*---------------------------< Notebook Messages >---------------------------*/
/*---------------------------------------------------------------------------*/

    case BKM_INVALIDATETABS:
      /*--------------------------------------------------------------*/
      /* Redraw the tabs of the notebook.                             */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - reserved                                          */
      /*   lParam - reserved                                          */
      /*--------------------------------------------------------------*/
      if (pbk->ppgFirstTab)
      {
        if (pbk->ppgFirstVisMajor)
        {
          CalcInvalidateRect (pbk, pbk->ppgFirstVisMajor,
                              pbk->ppgLastVisMajor, &rcUpdate);
          DrawUpdateRect (pbk, pbk->ppgFirstVisMajor,
                          pbk->ppgLastVisMajor,&rcUpdate, NULL);
        }
        if (pbk->ppgFirstVisMinor)
        {
          CalcInvalidateRect (pbk, pbk->ppgFirstVisMinor,
                              pbk->ppgLastVisMinor, &rcUpdate);
          DrawUpdateRect (pbk, pbk->ppgFirstVisMinor,
                          pbk->ppgLastVisMinor,&rcUpdate, NULL);
        }

        return (TRUE);
      }
      else  return (FALSE);

      break;

    case BKM_DELETEPAGE:
      /*---------------------------------------------------------------*/
      /* Delete the specified pages from the list.                     */
      /*                                                               */
      /* Parameters:                                                   */
      /*   wParam - ulPageId (ULONG)                                   */
      /*     Page to be deleted. Ignored if BKV_ALL.                   */
      /*   lParam - usDeleteFlag  (USHORT)                             */
      /*     BKV_SINGLE - Delete a single page                         */
      /*     BKV_TAB    - Delete the page and all subset pages         */
      /*     BKV_ALL    - Delete all pages in a section                */
      /*---------------------------------------------------------------*/
      return (DeletePage (pbk, (PPAGE) wParam, LOWORD (lParam), TRUE));
      break;

    case BKM_INSERTPAGE:
      /*---------------------------------------------------------------*/
      /* Insert the specified page into the Notebook list.             */
      /*                                                               */
      /* Parameters:                                                   */
      /*   wParam - ulPageId (ULONG)                                   */
      /*     Page identifier used for insert placement.                */
      /*                                                               */
      /*   lParam                                                      */
      /*     usPageStyle (USHORT)   - style for inserted page          */
      /*       BKA_VISIBILITYON (d) - display page with status text    */
      /*       BKA_VISIBILITYOFF    - display page with no status text */
      /*       BKA_MAJOR (d) - inserted page is a major tab            */
      /*       BKA_MINOR     - inserted page is a minor tab            */
      /*     usPageOrder (USHORT) - style for inserted page            */
      /*       BKA_LAST  - insert page at the end of the list          */
      /*       BKA_FIRST - insert page at the front of the list        */
      /*       BKA_NEXT  - insert page after ulPageId.                 */
      /*       BKA_PREV  - insert page before ulPageId.                */
      /*                                                               */
      /*---------------------------------------------------------------*/
      if ((ppg = MakePage (pbk, LOWORD (lParam))) &&
          !InsertPage (pbk, ppg, (PPAGE) wParam, HIWORD (lParam)))
      {
        /*----------------------------------------------------------*/
        /* If the page was not inserted, free the allocated memory. */
        /*----------------------------------------------------------*/
        FreePageData (pbk, ppg, FALSE);
        ppg = NULL;
      }
      return ((ULONG)ppg);                                              /*JEH1*/
      break;

    case BKM_QUERYPAGEID:
      /*--------------------------------------------------------------*/
      /* Query the id for the specified page                          */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for insert placement.               */
      /*   lParam                                                     */
      /*     usQueryOrder (USHORT) - order in which to query pageid   */
      /*       BKA_LAST (d) - last page in Notebook                   */
      /*       BKA_FIRST    - first page in Notebook                  */
      /*       BKA_PREV     - page previous to ulPageId               */
      /*       BKA_NEXT     - page after ulPageId                     */
      /*       BKA_TOP      - current topmost visible page            */
      /*       BKA_DELETED  - just deleted page (Windows only)        */
      /*       BKA_SELECTED - just selected page (Windows only)       */
      /*     usPageStyle  (USHORT) - page attribute to query pageid   */
      /*       BKA_MAJOR    - major tab page                          */
      /*       BKA_MINOR    - minor tab page                          */
      /*                                                              */
      /*--------------------------------------------------------------*/
      return (QueryPageId (pbk, (PPAGE) wParam, LOWORD (lParam),
                           HIWORD (lParam)));
      break;

    case BKM_CALCPAGERECT:
      /*--------------------------------------------------------------*/
      /* Calculate the the application page rectangle from a Notebook */
      /* rectangle or calculate a Notebook rectangle from a page      */
      /* rectangle.                                                   */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - lprc (LPRECT)                                     */
      /*     address of buffer for storage of the returning           */
      /*     rectangle coordinates                                    */
      /*   lParam - fPage                                             */
      /*     specifies whether the window to calculate is a page      */
      /*     window or a Notebook window. If TRUE, a page window      */
      /*     window is calculated. If FALSE, a Notebook window is     */
      /*     calculated.                                              */
      /*                                                              */
      /*--------------------------------------------------------------*/
      return (CalcPageRect (pbk, (LPRECT) wParam, lParam));             /*DAB2*/
      break;

    case BKM_QUERYPAGECOUNT:
      /*--------------------------------------------------------------*/
      /* Query the number of page in the book                         */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for insert placement.               */
      /*   lParam - usQueryEnd (ULONG)                                */
      /*     BKA_MAJOR - query up to the next major                   */
      /*     BKA_MINOR - query up to the next minor                   */
      /*     BKA_END   - query up to the end of the book.             */
      /*                                                              */
      /*--------------------------------------------------------------*/
      return (QueryPageCount (pbk, (PPAGE) wParam, LOWORD (lParam)));
      break;

    case BKM_QUERYPAGEDATA:
      /*--------------------------------------------------------------*/
      /* Get the 4 bytes of data set with BKM_SETPAGEULONG            */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam - reserved                                          */
      /*--------------------------------------------------------------*/
      ppg = (PPAGE) wParam;
      return (VerifyPageId (pbk, ppg) ? ppg->ulAppData :
                                        BOOKERR_INVALID_PARAMETERS);
      break;

    case BKM_QUERYPAGESTYLE:
      /*--------------------------------------------------------------*/
      /* Get the 4 page style                                         */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam - reserved                                          */
      /*--------------------------------------------------------------*/
      ppg = (PPAGE) wParam;
      return (VerifyPageId (pbk, ppg) ? ppg->fsAttr :
                                        BOOKERR_INVALID_PARAMETERS);
      break;

    case BKM_QUERYPAGEWINDOWHWND:
      /*--------------------------------------------------------------*/
      /* Get the window handle associated with the specified page.    */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam - reserved                                          */
      /*--------------------------------------------------------------*/
      ppg = (PPAGE) wParam;
      return (LRESULT)(VerifyPageId (pbk, ppg) ? (long)ppg->hwndPage :  /*JEH1*/
                                        BOOKERR_INVALID_PARAMETERS);
      break;

    case BKM_QUERYSTATUSLINETEXT:
      /*--------------------------------------------------------------*/
      /* Get the status line text associated with the specified page. */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam -                                                   */
      /*     pBookText (PBOOKTEXT)                                    */
      /*       pString (PSZ)    - pointer to text.                    */
      /*       textLen (USHORT) - text length.                        */
      /*                                                              */
      /* Note: If no status string pointer is associated with the     */
      /*       specified page or the page does not exist, FALSE is    */
      /*       returned. The size of the tab text string may be       */
      /*       queried by passing 0 for the textLen in lParam.        */
      /*--------------------------------------------------------------*/
      return (QueryStatusLineText (pbk, (PPAGE) wParam, (PBOOKTEXT) lParam));
      break;

    case BKM_QUERYTABBITMAP:
      /*--------------------------------------------------------------*/
      /* Get the bitmap handle associated with the specified page.    */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam - reserved                                          */
      /*--------------------------------------------------------------*/
      ppg = (PPAGE) wParam;
      return (LRESULT)(VerifyPageId (pbk, ppg) &&                       /*JEH1*/
              (ppg->fsAttr & (BKA_MAJOR | BKA_MINOR)) ?
                  (long)ppg->ptab->hbmTab : BOOKERR_INVALID_PARAMETERS);/*JEH1*/
      break;

    case BKM_QUERYTABTEXT:
      /*--------------------------------------------------------------*/
      /* Get the text pointer  associated with the specified page.    */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam -                                                   */
      /*     pBookText  (PBOOKTEXT)                                   */
      /*       pString (PSZ)    - pointer to text.                    */
      /*       textLen (USHORT) - text length.                        */
      /*                                                              */
      /* Note: If no string pointer is associated with the specified  */
      /*       page or the page does not exist, FALSE is returned.    */
      /*       The size of the tab text string may be queried by      */
      /*       passing 0 for the textLen in lParam.                   */
      /*--------------------------------------------------------------*/
      return (QueryTabText (pbk, (PPAGE) wParam, (PBOOKTEXT) lParam));

    case BKM_SETDIMENSIONS:
      /*--------------------------------------------------------------*/
      /* Set the dimensions of the page buttons, major tabs, or       */
      /* minor tabs.                                                  */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam -                                                   */
      /*     usWidth  (USHORT) - width of the pageButton window       */
      /*     usHeight (USHORT) - height of the pageButton window      */
      /*   lParam -                                                   */
      /*     usType                                                   */
      /*       BKA_MAJORTAB   - major tab                             */
      /*       BKA_MINORTAB   - minor tab                             */
      /*       BKA_PAGEBUTTON - page turning arrow                    */
      /*                                                              */
      /*--------------------------------------------------------------*/
      rc = SetDimensions( pbk, LOWORD(wParam), HIWORD(wParam),          /*DAB2*/
                          LOWORD(lParam) );                             /*DAB2*/

      /*---------------------------------------------*/
      /* the dimensions changed, redraw the Notebook */
      /*---------------------------------------------*/
      if (rc)  InvalidateRect (hwnd, NULL, FALSE);
      return (rc);
      break;

    case BKM_SETNOTEBOOKCOLORS:
      /*--------------------------------------------------------------*/
      /* Set Notebook window styles                                   */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - lColor (ULONG) - color to paint specified region  */
      /*                                                              */
      /*   lParam - usBookAttr (USHORT) - region to paint             */
      /*         BKA_BACKGROUNDPAGECOLOR                              */
      /*         BKA_BACKGROUNDMAJORCOLOR                             */
      /*         BKA_BACKGROUNDMINORCOLOR                             */
      /*         BKA_FOREGROUNDMAJORCOLOR                             */
      /*         BKA_FOREGROUNDMINORCOLOR                             */
      /*                                                              */
      /*--------------------------------------------------------------*/
      return (SetNotebookColors (pbk, wParam, LOWORD(lParam)));         /*DAB2*/
      break;

    case BKM_SETPAGEWINDOWHWND:
      /*--------------------------------------------------------------*/
      /* Set the window handle associated with the passed page.       */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier whose window handle is to be set.        */
      /*   lParam - hwndWindow (HWND)                                 */
      /*     Window handle to be associated with upPageId.            */
      /*                                                              */
      /*--------------------------------------------------------------*/
      return (SetPageWindowHwnd (pbk, (PPAGE) wParam, (HWND) lParam));
      break;

    case BKM_SETPAGEDATA:
      /*--------------------------------------------------------------*/
      /* Set the 4 bytes of page data for the app.                    */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier used for retrieval.                      */
      /*   lParam - ulPageData (ULONG)                                */
      /*     Data to store.                                           */
      /*--------------------------------------------------------------*/
      ppg = (PPAGE) wParam;
      if (VerifyPageId (pbk, ppg))
      {
        ppg->ulAppData = lParam;
        return (TRUE);
      }
      break;

    case BKM_SETTABBITMAP:
      /*--------------------------------------------------------------*/
      /* Associate a bitmap with the tab window                       */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier whose bitmap is to be set                */
      /*   lParam - hbm (HBM)                                         */
      /*     Bitmap handle.                                           */
      /*--------------------------------------------------------------*/
      if (SetTabBitmap (pbk, (PPAGE) wParam, (HWND) lParam))
      {
        /*--------------------------------*/
        /* Get pointer to the page record */
        /*--------------------------------*/
        ppg = (PPAGE) wParam;

        /*------------------------------------------------*/
        /* Redraw the updated tab window if it is visible */
        /*------------------------------------------------*/
        if (ppg && IsTabVisible (pbk, ppg))
        {
          CalcInvalidateRect (pbk, ppg, ppg, &rcUpdate);
          DrawUpdateRect (pbk, ppg, ppg, &rcUpdate, NULL);
        }

        /*--------------------------*/
        /* Return success indicator */
        /*--------------------------*/
        return (TRUE);
      }
      else
        return (FALSE);
      break;

    case BKM_SETTABTEXT:
      /*--------------------------------------------------------------*/
      /* Associate a text string with the tab window                  */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier whose bitmap is to be set                */
      /*   lParam - pString (PSZ)                                     */
      /*     Pointer to null terminated text string.                  */
      /*--------------------------------------------------------------*/
      if (SetTabText (pbk, (PPAGE) wParam, (PSZ) lParam))
      {
        /*--------------------------------*/
        /* Get pointer to the page record */
        /*--------------------------------*/
        ppg = (PPAGE) wParam;

        /*------------------------------------------------*/
        /* Redraw the updated tab window if it is visible */
        /*------------------------------------------------*/
        if (ppg && IsTabVisible (pbk, ppg))
        {
          CalcInvalidateRect (pbk, ppg, ppg, &rcUpdate);
          InvalidateRect (hwnd, &rcUpdate, FALSE);
//        DrawUpdateRect (pbk, ppg, ppg, &rcUpdate, NULL);
        }

        /*--------------------------*/
        /* Return success indicator */
        /*--------------------------*/
        return (TRUE);
      }
      else
        /*---------------------------------------------------------*/
        /* If unable to set the tab text, return failure indicator */
        /*---------------------------------------------------------*/
        return (FALSE);

      break;

    case BKM_SETSTATUSLINETEXT:
      /*--------------------------------------------------------------*/
      /* Associate a status string with the specifed page             */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier whose bitmap is to be set                */
      /*   lParam - pString (PSZ)                                     */
      /*     Pointer to null terminated text string.                  */
      /*--------------------------------------------------------------*/
      return (SetStatusLineText (pbk, (PPAGE) wParam, (PSZ) lParam));
      break;

    case BKM_TURNTOPAGE:
      /*--------------------------------------------------------------*/
      /* Diplay page at the top of the Notebook.                      */
      /*                                                              */
      /* Parameters:                                                  */
      /*   wParam - ulPageId (ULONG)                                  */
      /*     Page identifier whose bitmap is to be set                */
      /*   lParam - Reserved                                          */
      /*--------------------------------------------------------------*/
      ppg = (PPAGE) wParam;
      return (VerifyPageId (pbk, ppg) ?
                                  GotoPage (pbk, ppg, 0, TRUE) : FALSE);/*JEH1*/
      break;

    default:
      return (DefWindowProc (hwnd, msg, wParam, lParam));
      break;
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * PageWndProc
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

WNDPROC PageWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) /*JEH1*/
{
  PBOOK pbk;
  HWND  hwndParent;

  switch (msg)
  {
    case WM_PAINT:
    {
      PAINTSTRUCT ps;

      BeginPaint (hwnd, &ps);
      pbk = (PBOOK) GetWindowLong (PARENTOF (hwnd), QWL_NOTEBOOK);      /*JEH1*/
      BookFillRect (ps.hdc, &ps.rcPaint, pbk->crColor[CLR_PAGEBG]);
      EndPaint (hwnd, &ps);
      break;
    }

    case WM_ERASEBKGND:
      return (TRUE);
      break;

    case WM_LBUTTONDOWN:
      hwndParent = PARENTOF (hwnd);
      BookSetFocus (hwndParent);
      SendSetCursor (hwndParent, msg);
      break;

    case WM_MOUSEMOVE:
    case WM_LBUTTONUP:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_MBUTTONDOWN:
    case WM_MBUTTONUP:
      SendSetCursor (PARENTOF (hwnd), msg);
      break;

    case WM_COMMAND:
    {
      PBOOK pbk = (PBOOK) GetWindowLong (PARENTOF (hwnd), QWL_NOTEBOOK);/*JEH1*/
      PPAGE ppgTop, ppgT;
      HWND  hwndParent, hwndSender = (HWND)(UINT)lParam;                /*JEH1*/

      /*--------------------------------------------------------*/
      /* it is most likely that the top page is the one sending */
      /* messages to its parent, so check that one first        */
      /*--------------------------------------------------------*/
      if ((ppgTop = QueryTopPage (pbk)) && (ppgTop->hwndPage == hwndSender))
      {
        hwndParent = ppgTop->hwndPageParent;
      }

      /*--------------------------------------------------------*/
      /* otherwise, walk the list of pages to find the original */
      /* parent of the page that's sending this message         */
      /*--------------------------------------------------------*/
      else
      {
        for (ppgT = pbk->ppgFirst;
             ppgT && (ppgT->hwndPage == hwndSender);
             ppgT = ppgT->ppgNext);

        hwndParent = (ppgT) ? ppgT->hwndPageParent : NULL;
      }

      /*-----------------------------------------------------------*/
      /* if this message came from a page window in this notebook, */
      /* forward it on to the page's original owner                */
      /*-----------------------------------------------------------*/
      if (hwndParent)
      {

        return (SendMessage (hwndParent, msg, wParam, lParam));
      }

      /*--------------------------------------------*/
      /* otherwise, give DefWindowProc a shot at it */
      /*--------------------------------------------*/
      else
      {
        return (DefWindowProc (hwnd, msg, wParam, lParam));
      }
      break;
    }

    default:
      return (DefWindowProc (hwnd, msg, wParam, lParam));
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * PageButtonWndProc
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

WNDPROC PageButtonWndProc( HWND hwnd, UINT msg, WPARAM wParam,          /*JEH1*/
                           LPARAM lParam )                              /*JEH1*/
{
  PBOOK pbk;
  HWND  hwndParent;
  LONG  id  = IDOF (hwnd);                                              /*JEH1*/
  RECT  rc;

  switch (msg)
  {
    case WM_ENABLE:
      InvalidateRect (hwnd, NULL, FALSE);
      break;

    case WM_LBUTTONDOWN:
      /*-----------------------------------*/
      /* Set focus to Notebook, if not set */
      /*-----------------------------------*/
      hwndParent = PARENTOF (hwnd);
      pbk = (PBOOK) GetWindowLong (hwndParent, QWL_NOTEBOOK);           /*JEH1*/
      BookSetFocus (hwndParent);

      /*-----------------------------------------------*/
      /* If the book has inserted pages, turn the page */
      /*-----------------------------------------------*/
      if (IsWindowEnabled (hwnd))
      {
        /*-----------------------------------------------------------*/
        /* Capture the mouse to ensure we get the button1 up message */
        /*-----------------------------------------------------------*/
        SetCapture (hwnd);

        /*-------------------------------------------------------------*/
        /* Set a flag to cause the page turning arrow to be painted in */
        /* the pressed position. Save the active window.               */
        /*-------------------------------------------------------------*/
        pbk->flState |= (id == ID_PAGEBUTTONLEFT) ? BKV_LPAGEBUTTONDOWN :
                                                    BKV_RPAGEBUTTONDOWN;

        InvalidateRect (hwnd, NULL, FALSE);
        UpdateWindow (hwnd);
      }
      SendSetCursor (hwndParent, msg);
      break;

   case WM_LBUTTONUP:
      hwndParent = PARENTOF (hwnd);
      pbk = (PBOOK) GetWindowLong (hwndParent, QWL_NOTEBOOK);           /*JEH1*/

      if (hwnd == GetCapture())
      {
        /*----------------------------------------------*/
        /* Reset the mouse captured on the button1 down */
        /*----------------------------------------------*/
        ReleaseCapture ();

        if (pbk->flState & BKV_PAGEBUTTONDOWN)
        {
          /*------------------------------------*/
          /* Repaint the page arrow non-pressed */
          /*------------------------------------*/
          pbk->flState &= ~BKV_PAGEBUTTONDOWN;
          InvalidateRect (hwnd, NULL, FALSE);
          UpdateWindow (hwnd);

          /*------------------------------*/
          /* turn to the appropriate page */
          /*------------------------------*/
          GotoPage (pbk, NULL, (id==ID_PAGEBUTTONLEFT) ? BKA_PREV : BKA_NEXT,
                    TRUE);
        }
      }
      SendSetCursor (hwndParent, msg);
      break;

   case WM_MOUSEMOVE:
      hwndParent = PARENTOF (hwnd);
      pbk = (PBOOK) GetWindowLong (hwndParent, QWL_NOTEBOOK);           /*JEH1*/

      /*-------------------------------------------------------------*/
      /* If a button1down transition is in progress, ensure that the */
      /* page turning arrows are painted in the correct state.       */
      /*-------------------------------------------------------------*/
      if (hwnd == GetCapture())
      {
        POINT pt;                                                       /*JEH1*/
        pt.x = LOWORD(lParam);                                          /*JEH1*/
        pt.y = HIWORD(lParam);                                          /*JEH1*/

        GetClientRect (hwnd, &rc);

        if (PtInRect (&rc, pt))                                         /*JEH1*/
        {
          /*-------------------------------------------------------*/
          /* If the mouse is currently over the page arrow window, */
          /* set a flag to cause it to be painted depressed.       */
          /*-------------------------------------------------------*/
          if (!(pbk->flState & BKV_PAGEBUTTONDOWN))
          {
            pbk->flState |= (id == ID_PAGEBUTTONLEFT) ?
                             BKV_LPAGEBUTTONDOWN : BKV_RPAGEBUTTONDOWN;
            InvalidateRect (hwnd, NULL, FALSE);
            UpdateWindow (hwnd);
          }
        }
        else
        {
          /*------------------------------------------------------*/
          /* If the mouse is outside the page arrow window, set a */
          /* flag to cause it to be painted non-pressed.          */
          /*------------------------------------------------------*/
          if (pbk->flState & BKV_PAGEBUTTONDOWN)
          {
            pbk->flState &= (id == ID_PAGEBUTTONLEFT) ?
                            ~BKV_LPAGEBUTTONDOWN : ~BKV_RPAGEBUTTONDOWN;
            InvalidateRect (hwnd, NULL, FALSE);
            UpdateWindow (hwnd);
          }
        }
      }
      SendSetCursor (hwndParent, msg);
      break;

    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_MBUTTONDOWN:
    case WM_MBUTTONUP:
      SendSetCursor (PARENTOF (hwnd), msg);
      break;

    case WM_ERASEBKGND:
      return (TRUE);
      break;

    case WM_PAINT:
    {
      USHORT      idBitmap;
      HWND        hwndCapture = GetCapture ();
      PAINTSTRUCT ps;
      RECT        rcDst;

      pbk = (PBOOK) GetWindowLong (PARENTOF (hwnd), QWL_NOTEBOOK);      /*JEH1*/

      switch (id)
      {
        case ID_PAGEBUTTONLEFT:
          idBitmap = ((pbk->flState & BKV_LPAGEBUTTONDOWN) &&
                      (hwnd == hwndCapture)) ? ID_PAGEBUTTONLDOWN :
                                               ID_PAGEBUTTONLUP;
          break;

        case ID_PAGEBUTTONRIGHT:
          idBitmap = ((pbk->flState & BKV_RPAGEBUTTONDOWN) &&
                      (hwnd == hwndCapture)) ? ID_PAGEBUTTONRDOWN :
                                               ID_PAGEBUTTONRUP;
          break;
      }

      if (!IsWindowEnabled (hwnd))
      {
        idBitmap += DISABLE_DELTA;
      }

      BeginPaint (hwnd, &ps);

      GetClientRect (hwnd, &rcDst);
      BookFrameRect (ps.hdc, &rcDst, pbk->crColor[CLR_OUTLINE]);
      InflateRect (&rcDst, -1, -1);

      BltStretch (ps.hdc, QueryBitmapResource (idBitmap), &rcDst, TRUE);
      EndPaint (hwnd, &ps);
      break;
    }

    default:
      return (DefWindowProc (hwnd, msg, wParam, lParam));
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * ScrollWndProc
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

WNDPROC ScrollWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)/*JEH1*/
{
  switch (msg)
  {
    case WM_LBUTTONDOWN:
    {
      RECT   rcInvalid, rcInvalid1, rcUpdate;
      PPAGE  ppgFirstVis, ppgLastVis;
      USHORT usOrder, usTabType;
      HWND   hwndParent = PARENTOF (hwnd);
      PBOOK  pbk = (PBOOK) GetWindowLong (hwndParent, QWL_NOTEBOOK);    /*JEH1*/
      LONG   id  = IDOF (hwnd);                                         /*JEH1*/

      /*-------------------------*/
      /* Set scrolling direction */
      /*-------------------------*/
      usOrder = ((id == ID_HORIZONTALLEFT) || (id == ID_VERTICALTOP)) ?
                BKA_PREV : BKA_NEXT;

      /*------------------------------*/
      /* Reset scrolling start record */
      /*------------------------------*/
      if (pbk->flStyle & (BKS_MAJORTABRIGHT | BKS_MAJORTABLEFT))
      {
        if ((id == ID_VERTICALTOP) || (id == ID_VERTICALBOTTOM))
        {
          usTabType = BKA_MAJOR;
          CalcInvalidateRect (pbk, pbk->ppgFirstVisMajor,
                              pbk->ppgLastVisMajor, &rcInvalid);
        }
        else
        {
          usTabType = BKA_MINOR;
          CalcInvalidateRect (pbk, pbk->ppgFirstVisMinor,
                              pbk->ppgLastVisMinor, &rcInvalid);
        }

        ResetVisPagePtrs (pbk, NULL, usTabType, usOrder);

      }
      else if (pbk->flStyle & (BKS_MAJORTABTOP | BKS_MAJORTABBOTTOM))
      {
        if ((id == ID_VERTICALTOP) || (id == ID_VERTICALBOTTOM))
        {
          usTabType = BKA_MINOR;
          CalcInvalidateRect (pbk, pbk->ppgFirstVisMinor,
                              pbk->ppgLastVisMinor, &rcInvalid);
        }
        else
        {
          usTabType = BKA_MAJOR;
          CalcInvalidateRect (pbk, pbk->ppgFirstVisMajor,
                              pbk->ppgLastVisMajor, &rcInvalid);
        }

        ResetVisPagePtrs (pbk, NULL, usTabType, usOrder);
      }

      if (usTabType == BKA_MAJOR)
      {
        ppgFirstVis = pbk->ppgFirstVisMajor;
        ppgLastVis  = pbk->ppgLastVisMajor;
        ResetTabData (pbk, pbk->ppgFirstVisMajor,
                      QueryStartTabLevel (pbk, BKA_MAJOR));
      }
      else
      {
        ppgFirstVis = pbk->ppgFirstVisMinor;
        ppgLastVis  = pbk->ppgLastVisMinor;
        ResetTabData (pbk, pbk->ppgFirstVisMinor,
                      QueryStartTabLevel (pbk, BKA_MINOR));
      }

      CalcInvalidateRect (pbk, ppgFirstVis, ppgLastVis, &rcInvalid1);

      if (UnionRect (&rcUpdate, &rcInvalid, &rcInvalid1))
      {
        DrawUpdateRect (pbk, ppgFirstVis, ppgLastVis, &rcUpdate, NULL);
      }

      BookSetFocus (pbk->hwndBook);
      SendSetCursor (hwndParent, msg);
      break;
    }

    case WM_MOUSEMOVE:
    case WM_LBUTTONUP:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_MBUTTONDOWN:
    case WM_MBUTTONUP:
      SendSetCursor (PARENTOF (hwnd), msg);
      break;

    case WM_ENABLE:
      InvalidateRect (hwnd, NULL, FALSE);
      break;

    case WM_ERASEBKGND:
      return (TRUE);
      break;

    case WM_PAINT:
    {
      BOOL        fDisabled = !IsWindowEnabled (hwnd);
      PAINTSTRUCT ps;
      RECT        rcClient;                                              //@MAB4
      SHORT       xIcon, yIcon, idIcon, idxColor;
      PBOOK       pbk = (PBOOK) GetWindowLong (PARENTOF (hwnd),         /*JEH1*/
                                               QWL_NOTEBOOK);
      LONG        id  = IDOF (hwnd);                                    /*JEH1*/

      /*----------------------------------------------*/
      /* if major scroll arrow, fill with major back- */
      /* ground, otherwise fill with minor background */
      /*----------------------------------------------*/
      idxColor = (((pbk->flStyle & (BKS_MAJORTABRIGHT | BKS_MAJORTABLEFT)) &&
                   ((id == ID_VERTICALTOP) || (id == ID_VERTICALBOTTOM)))  ||
                  ((pbk->flStyle & (BKS_MAJORTABTOP | BKS_MAJORTABBOTTOM)) &&
                   ((id == ID_HORIZONTALLEFT) || (id == ID_HORIZONTALRIGHT))))
               ? CLR_MAJORBG : CLR_MINORBG;

      switch (id)
      {
        case ID_VERTICALTOP:
        case ID_VERTICALBOTTOM:
          xIcon  = ((pbk->cxScrollArrow - CX_VERTARROWIMAGE) / 2) & ~1;
          yIcon  = (pbk->cxScrollArrow - CY_VERTARROWIMAGE) / 2;
          idIcon = (id == ID_VERTICALTOP) ? ID_TABUP : ID_TABDWN;
          break;

        case ID_HORIZONTALLEFT:
        case ID_HORIZONTALRIGHT:
          xIcon  = (pbk->cxScrollArrow - CX_HORZARROWIMAGE) / 2;
          yIcon  = ((pbk->cxScrollArrow - CY_HORZARROWIMAGE) / 2) & ~1;
          idIcon = (id == ID_HORIZONTALLEFT) ? ID_TABLFT : ID_TABRGT;
          break;
      }

      if (fDisabled)
        idIcon += DISABLE_DELTA;

      /*************************************************************/  //->@MAB4
      /* Get the coordinates of the window, not the hps.           */
      /*************************************************************/
      GetClientRect( hwnd, &rcClient );

      BeginPaint (hwnd, &ps);
      BookFillRect  (ps.hdc, &rcClient, pbk->crColor[idxColor]);
      BookFrameRect (ps.hdc, &rcClient, pbk->crColor[CLR_OUTLINE]);    //@MAB4<-
      DrawIcon (ps.hdc, xIcon, yIcon, QueryIconResource (idIcon));
      EndPaint (hwnd, &ps);
      break;
    }

    default:
      return (DefWindowProc (hwnd, msg, wParam, lParam));
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * Initialize
 *
 * Allocates and initializes a BOOK structure for an instance of the
 * notebook.  The return value from this function is returned directly to
 * the system in response to WM_CREATE, so -1 indicates failure, 0
 * indicates success.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL Initialize (
  HWND hwndBook)
{
  PBOOK pbk;
  WORD  cxPageButton, cyPageButton;
  HDC   hdc;

  /*-----------------------------------------------------------------*/
  /* add a node to describe this process in the process list and     */
  /* allocate memory for the notebook instance data. If any of these */
  /* fail, return failure indicator                                  */
  /*-----------------------------------------------------------------*/
  if (!(pbk = BookAllocMem (sizeof (BOOK))))  return (-1);

  pbk->hwndBook = hwndBook;

  /*-----------------------------------------------------------------*/
  /* Save notebook window instance data pointer for future retrieval */
  /*-----------------------------------------------------------------*/
  SetWindowLong (hwndBook, QWL_NOTEBOOK, (LONG)pbk);                    /*JEH1*/

  /*------------------------------------------*/
  /* initialize the notebook's color settings */
  /*------------------------------------------*/
  pbk->crColor[CLR_STATUSTEXT] = GetSysColor (COLOR_WINDOWTEXT);
  pbk->crColor[CLR_NOTEBOOK]   = GetSysColor (COLOR_3DFACE);
  pbk->crColor[CLR_OUTLINE]    = GetSysColor (COLOR_WINDOWFRAME);
  pbk->crColor[CLR_CURSOR]     = GetSysColor (COLOR_WINDOWFRAME);
  pbk->crColor[CLR_PAGEBG]     = GetSysColor (COLOR_3DFACE);
  pbk->crColor[CLR_MAJORBG]    = GetSysColor (COLOR_3DFACE);
  pbk->crColor[CLR_MINORBG]    = GetSysColor (COLOR_3DFACE);
  pbk->crColor[CLR_MAJORFG]    = GetSysColor (COLOR_WINDOWTEXT);
  pbk->crColor[CLR_MINORFG]    = GetSysColor (COLOR_WINDOWTEXT);

  /*--------------------*/
  /* Set tab dimensions */
  /*--------------------*/
  SetTabDimensions (pbk, BKD_MAJORTABWIDTH, BKD_MAJORTABHEIGHT, BKA_MAJORTAB);
  SetTabDimensions (pbk, BKD_MINORTABWIDTH, BKD_MINORTABHEIGHT, BKA_MINORTAB);

  /*-----------------------------------*/
  /* Set page button window dimensions */
  /*-----------------------------------*/
  cxPageButton = BKD_PAGEBUTTONWIDTH;
  cyPageButton = BKD_PAGEBUTTONHEIGHT;

  if (vsDevIndX > 1)
  {
    cxPageButton++;
    cyPageButton--;
  }

  SetPageButtonDimensions (pbk, cxPageButton, cyPageButton);

  /*--------------------------------------------------------------------*/
  /* create the page turning windows, page clipping window, and scroll  */
  /* arrow windows; if any of this fails, abort creation of this window */
  /*--------------------------------------------------------------------*/
  if (!MakePageButton (hwndBook)                        ||
      !(pbk->hwndBookPage = MakeBookPageWnd (hwndBook)) ||
      !InitializeScrollArrows (pbk)                     ||
      !InitializeResources ()                           ||
      !(hdc = GetDC (hwndBook))                         ||
      !(pbk->hdcMem = CreateCompatibleDC (hdc)))
  {
    if (hdc)  ReleaseDC (hwndBook, hdc);
    BookFreeMem (pbk);
    SetWindowLong (hwndBook, QWL_NOTEBOOK, 0);                          /*JEH1*/
    return (-1);
  }

  /*-------------------------------------*/
  /* Set the Notebook window style bits. */
  /*-------------------------------------*/
  SetNotebookStyle (pbk, TRUE);

  ReleaseDC (hwndBook, hdc);
  ++vcNotebooks;
  return (0);
}


/*---------------------------------------------------------------------------*
 * InitializeResources
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL InitializeResources()
{
  USHORT   i;

  /*--------------------------------------------------*/
  /* Load the pageButton bitmaps. If an error occurs, */
  /* destroy all those that have been loaded.         */
  /*--------------------------------------------------*/
  for (i = ID_PAGEBUTTONRUP; i < CRESOURCES; i++)
  {
    QueryBitmapResource(i);

    if (!vhBookResource[i])
    {
      DestroyBookResources ();
      return (FALSE);
    }
  }

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * InitializeScrollArrows
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL InitializeScrollArrows (
  PBOOK pbk)
{
  USHORT id;

  pbk->cxScrollArrow = BKD_SCROLLARROWSIZE * vsDevIndX;

  for (id=ID_VERTICALTOP; id<=ID_HORIZONTALRIGHT; id++)
  {
    if (!CreateWindow (WC_SCROLLARROW, NULL, WS_DISABLED | WS_CHILD, 0, 0,
                       BKD_SCROLLARROWSIZE, BKD_SCROLLARROWSIZE, pbk->hwndBook,
                       (HMENU)id, vhInstance, NULL))                    /*JEH1*/
    {
      return (FALSE);
    }
  }

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * MakeBookPageWnd
 *
 * Creates the notebook page window that is parent to the application page
 * windows.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

HWND MakeBookPageWnd (
  HWND hwndBook)
{
  return (CreateWindow (WC_PAGE, NULL, WS_CHILD | WS_CLIPCHILDREN, 0, 0, 0, 0,
                        hwndBook, (HMENU)ID_BOOKPAGE, vhInstance, 0));  /*JEH1*/
}


/*---------------------------------------------------------------------------*
 * MakePage
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

PPAGE MakePage (
  PBOOK  pbk,
  USHORT usPageStyle)
{
  PPAGE ppgNew;

  /*-----------------------------------------------------*/
  /* Get memory from the heap created at initialization. */
  /*-----------------------------------------------------*/
  if (!(ppgNew = BookAllocMem (sizeof(PAGE))))  return NULL;

  /*-----------------------*/
  /* Initialize page type. */
  /*-----------------------*/
  ppgNew->fsAttr = (usPageStyle & BKA_MINOR) ? BKA_MINOR :
                   (usPageStyle & BKA_MAJOR) ? BKA_MAJOR : 0;

  /*------------------------------------------------------------------*/
  /* Initialize status text visibility. This determines if the status */
  /* text is diplayed or not. The default is no status text.          */
  /*------------------------------------------------------------------*/
  if (usPageStyle & BKA_STATUSTEXTON)  ppgNew->fsAttr |= BKA_STATUSTEXTON;

  /*---------------------------------------------------------------*/
  /* Initialize autopagezize flag. This determines if the Notebook */
  /* performs the sizing and positioning of the page window. The   */
  /* default is autopagesize on.                                   */
  /*---------------------------------------------------------------*/
  if (usPageStyle & BKA_AUTOPAGESIZE)  ppgNew->fsAttr |= BKA_AUTOPAGESIZE;

  /*---------------------------------------------------*/
  /* make a tab structure for major or minor tab pages */
  /*---------------------------------------------------*/
  if ((ppgNew->fsAttr & (BKA_MAJOR | BKA_MINOR)) &&
      !(ppgNew->ptab = BookAllocMem (sizeof(TAB))))
  {
    ppgNew = BookFreeMem (ppgNew);
  }

  return (ppgNew);
}


/*---------------------------------------------------------------------------*
 * MakePageButton
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL MakePageButton (
  HWND hwndBook)
{
  return (CreateWindow( WC_PAGEBUTTON, NULL, WS_DISABLED | WS_CHILD,
                        0, 0, 0, 0, hwndBook, (HMENU)ID_PAGEBUTTONLEFT, /*JEH1*/
                        vhInstance, 0) &&
          CreateWindow( WC_PAGEBUTTON, NULL, WS_DISABLED | WS_CHILD,
                        0, 0, 0, 0, hwndBook, (HMENU)ID_PAGEBUTTONRIGHT,/*JEH1*/
                        vhInstance, 0));
}


/*---------------------------------------------------------------------------*
 * ProcessKeyDown
 *
 * Manages WM_KEYDOWN and WM_SYSKEYDOWN messages.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

LONG ProcessKeyDown( PBOOK pbk, UINT msg, WPARAM wParam, LPARAM lParam) /*JEH1*/
{
  BOOL fCtrl  = (GetKeyState (VK_CONTROL) & 0x8000) ? TRUE : FALSE;
  BOOL fShift = (GetKeyState (VK_SHIFT)   & 0x8000) ? TRUE : FALSE;
  BOOL fAlt   = (HIWORD (lParam) & 0x2000)          ? TRUE : FALSE;

  /*-------------------------------------------------*/
  /* if more than one accent key is pressed, or an   */
  /* accent was pressed in conjunction with a key    */
  /* that doesn't use one, just pass on this message */
  /*-------------------------------------------------*/
  if ((fCtrl + fShift + fAlt > 1)        ||
      (fAlt   && (wParam != VK_DOWN)  &&
                 (wParam != VK_PRIOR) &&
                 (wParam != VK_NEXT))    ||
      (fCtrl  && (wParam != VK_TAB))     ||
      (fShift && (wParam != VK_TAB))     ||
      ((wParam == VK_PRIOR) && !fAlt &&
          !IsWindowEnabled (GetDlgItem (pbk->hwndBook, ID_PAGEBUTTONLEFT))) ||
      ((wParam == VK_NEXT)  && !fAlt &&
          !IsWindowEnabled (GetDlgItem (pbk->hwndBook, ID_PAGEBUTTONRIGHT))))
  {
    return (DefWindowProc (pbk->hwndBook, msg, wParam, lParam));
  }

  /*------------------------------------------------*/
  /* If the notebook is empty, pass this message on */
  /*------------------------------------------------*/
  if (!QueryPageCount( pbk, 0, BKA_END ))
  {
     return (DefWindowProc (pbk->hwndBook, msg, wParam, lParam));
  }

  switch (wParam)
  {
    /*----------------------------------------------*/
    /* Up or Left:  set cursor emphasis to prev tab */
    /*----------------------------------------------*/
    case VK_UP:
    case VK_LEFT:
      ResetCursorPagePtr (pbk, IsReverseNavigation (pbk) ? BKA_NEXT : BKA_PREV,
                          pbk->ppgCursor->fsAttr);
      break;

    /*---------------------------------------------------*/
    /* Alt+Down:  set the focus to the app's page window */
    /*---------------------------------------------------*/
    case VK_DOWN:
      if (fAlt)
      {
        PPAGE ppgTop = QueryTopPage (pbk);
        if (ppgTop && ppgTop->hwndPage)  SetFocus (ppgTop->hwndPage);
        break;
      }
      /* flow through */

    /*---------------------------------------------*/
    /* Down or Right:  set move cursor to next tab */
    /*---------------------------------------------*/
    case VK_RIGHT:
      ResetCursorPagePtr (pbk, IsReverseNavigation (pbk) ? BKA_PREV : BKA_NEXT,
                          pbk->ppgCursor->fsAttr);
      break;

    /*----------------------------------------------------------*/
    /* PageUp or Alt+PageUp:  bring the prev page to to the top */
    /*----------------------------------------------------------*/
    case VK_PRIOR:
      GotoPage (pbk, NULL, BKA_PREV, TRUE);
      break;

    /*----------------------------------------------------------*/
    /* PageDn or Alt+PageDn:  bring the next page to to the top */
    /*----------------------------------------------------------*/
    case VK_NEXT:
      GotoPage (pbk, NULL, BKA_NEXT, TRUE);
      break;

    /*-------------------------------------------------------*/
    /* Home:  bring the first page of the book to to the top */
    /*-------------------------------------------------------*/
    case VK_HOME:
      GotoPage (pbk, pbk->ppgFirst, 0, TRUE);                           /*JEH1*/
      break;

    /*-----------------------------------------------------*/
    /* End:  bring the last page of the book to to the top */
    /*-----------------------------------------------------*/
    case VK_END:
      GotoPage (pbk, pbk->ppgLast, 0, TRUE);                            /*JEH1*/
      break;

    /*--------------------------------------------------------------*/
    /* Enter or Space:  bring the currently cursored tab to the top */
    /*--------------------------------------------------------------*/
    case VK_RETURN:
    case VK_SPACE:
      GotoPage (pbk, pbk->ppgCursor, 0, TRUE);                          /*JEH1*/
      break;

    /*--------------------------------------------*/
    /* F1:  send a help notification to the owner */
    /*--------------------------------------------*/
//  case VK_F1:                                             //->JKS1   //->@MAB6
//    if (pbk->ppgCursor)
//    {
//      NotifyParent( pbk->hwndBook, BKN_HELP, TRUE,
//                    (LRESULT)(pbk->ppgCursor) );
//    }
//    break;                                                //JKS1<-   //@MAB6<-

    /*----------------*/
    /* Tab:  pick one */
    /*----------------*/
    case VK_TAB:
    {
      HWND hwndTabstop;

      /*-----------------------------------------------*/
      /* Ctrl+Tab:  move the focus to the next tabstop */
      /*-----------------------------------------------*/
      if (fCtrl)
      {
        if (hwndTabstop = FindTabstop (pbk->hwndBook, GW_HWNDNEXT))
        {
          SetFocus (hwndTabstop);
        }
      }

      /*-------------------------------------------------------------------*/
      /* Shift+Tab:  1.  If a minor tab has the cursor and there are major */
      /*                 tabs displayed, move the cursor to the major tab. */
      /*             2.  If there are no tabs, or a major has the cursor,  */
      /*                 or a minor has the cursor and there are no majors */
      /*                 displayed, move the focus to the previous tabstop */
      /*             3.  If a minor tab has the cursor and there are no    */
      /*                 sibling tabstop windows, move the cursor to the   */
      /*                 major tab, if it exists.                          */
      /*-------------------------------------------------------------------*/
      else if (fShift)
      {
        /*-------------------------------------*/
        /* case 1:  cursor from minor to major */
        /*-------------------------------------*/
        if (pbk->ppgCursor                      &&
           (pbk->ppgCursor->fsAttr & BKA_MINOR) &&
           (pbk->ppgCursorMajor))
        {
          pbk->ppgCursor = pbk->ppgCursorMajor;
          RedrawCursoredTabs (pbk);
        }

        /*--------------------------------------------------*/
        /* case 2:  focus from notebook to previous tabstop */
        /*--------------------------------------------------*/
        else if (hwndTabstop = FindTabstop (pbk->hwndBook, GW_HWNDPREV))
        {
          SetFocus (hwndTabstop);
        }

        /*-------------------------------------*/
        /* case 3:  cursor from major to minor */
        /*-------------------------------------*/
        else if (pbk->ppgCursor                      &&
                (pbk->ppgCursor->fsAttr & BKA_MAJOR) &&
                (pbk->ppgCursorMinor))
        {
          pbk->ppgCursor = pbk->ppgCursorMinor;
          RedrawCursoredTabs (pbk);
        }
      }

      /*-------------------------------------------------------------*/
      /* Tab:  1.  If a major tab has the cursor and there are minor */
      /*           tabs displayed, move the cursor to the minor tab. */
      /*       2.  If there are no tabs, or a minor has the cursor,  */
      /*           or a major has the cursor and there are no minors */
      /*           displayed, move the focus to the next tabstop     */
      /*       3.  If a minor tab has the cursor and there are no    */
      /*           sibling tabstop windows, move the cursor to the   */
      /*           major tab, if it exists.                          */
      /*-------------------------------------------------------------*/
      else
      {
        /*-------------------------------------*/
        /* case 1:  cursor from major to minor */
        /*-------------------------------------*/
        if (pbk->ppgCursor                      &&
           (pbk->ppgCursor->fsAttr & BKA_MAJOR) &&
           (pbk->ppgCursorMinor))
        {
          pbk->ppgCursor = pbk->ppgCursorMinor;
          RedrawCursoredTabs (pbk);
        }

        /*----------------------------------------------*/
        /* case 2:  focus from notebook to next tabstop */
        /*----------------------------------------------*/
        else if (hwndTabstop == FindTabstop (pbk->hwndBook, GW_HWNDNEXT))
        {
          SetFocus (hwndTabstop);
        }

        /*-------------------------------------*/
        /* case 3:  cursor from minor to major */
        /*-------------------------------------*/
        else if (pbk->ppgCursor                      &&
                (pbk->ppgCursor->fsAttr & BKA_MINOR) &&
                (pbk->ppgCursorMajor))
        {
          pbk->ppgCursor = pbk->ppgCursorMajor;
          RedrawCursoredTabs (pbk);
        }
      }
      break;
    }

    /*------------------------------------*/
    /* Anything else:  check for mnemonic */
    /*------------------------------------*/
    default:
    {
      PPAGE ppg = NULL, ppgStart;

      if (isalnum (wParam))
      {
#if 0      /*-------------------------< JDR >--------------------------------*/
        for (ppg = pbk->ppgFirstTab, wParam |= 0x20;
             ppg && (ppg->ptab->chMnemonic != (CHAR) wParam);
             ppg = ppg->ptab->ppgNextTab);
#endif     /*-------------------------< JDR >--------------------------------*/

        if (ppgStart = QueryTopPage (pbk))
        {
          ppgStart = GetTabPtr (ppgStart->ppgNext, BKA_MAJOR | BKA_MINOR,
                                BKA_NEXT);
        }

        /*----------------------------------------*/
        /* scan tabs from the current page onward */
        /*----------------------------------------*/
        for (ppg = ppgStart, wParam |= 0x20;
             ppg && (ppg->ptab->chMnemonic != (CHAR) wParam);
             ppg = ppg->ptab->ppgNextTab);

        /*-----------------------------------------------------------*/
        /* if we didn't find a match, start from the top of the book */
        /*-----------------------------------------------------------*/
        if (!ppg)
        {
          for (ppg = pbk->ppgFirstTab;
               ppg && (ppg != ppgStart) &&
                      (ppg->ptab->chMnemonic != (CHAR) wParam);
               ppg = ppg->ptab->ppgNextTab);

          /*-------------------------------------------------------------*/
          /* if we wound up where we started, don't bother with GotoPage */
          /*-------------------------------------------------------------*/
          if (ppg == ppgStart)  ppg = NULL;
        }
      }

      /*----------------------------------------------------------------*/
      /* If a page is found matching the mnemonic, bring it to the top. */
      /*----------------------------------------------------------------*/
      if (ppg)  GotoPage (pbk, ppg, 0, TRUE);                           /*JEH1*/
      else  return (DefWindowProc (pbk->hwndBook, msg, wParam, lParam));
      break;
    }
  }

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * FindTabstop
 *
 * Finds the next or previous window in the tabbing order, starting from
 * a given window.
 *
 * History:
 * 17-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

HWND FindTabstop (
  HWND hwndStart,
  WORD wCmd)
{
  HWND hwndT, hwndFocus = GetFocus();

  if (!hwndStart)  return (NULL);

  /*-----------------------------------------------------------------*/
  /* loop through the siblings until we find one that's a WS_TABSTOP */
  /*-----------------------------------------------------------------*/
  for (hwndT = GetWindow (hwndStart, wCmd);
       hwndT && (hwndT != hwndStart) && (hwndT != hwndFocus);
       hwndT = GetWindow (hwndT, wCmd))
  {
    if (GetWindowLong (hwndT, GWL_STYLE) & WS_TABSTOP)  break;
  }

  /*--------------------------------------------------------------*/
  /* if we reached the end of the sibling chain without finding a */
  /* tabstop, go to the front of the sibling chain a start over   */
  /*--------------------------------------------------------------*/
  if (!hwndT)
  {
    WORD wNextCmd      = (wCmd == GW_HWNDNEXT) ? GW_HWNDFIRST : GW_HWNDLAST;
    HWND hwndNextStart = GetWindow (hwndStart, wNextCmd);

    if (hwndNextStart != hwndStart)
    {
      hwndT = FindTabstop (hwndNextStart, wCmd);
    }
  }

  /*-------------------------------------------------------*/
  /* otherwise if we're back where we started, return NULL */
  /* so we don't bother setting the focus to ourselves     */
  /*-------------------------------------------------------*/
  else if (hwndT == hwndStart)
  {
    hwndT = NULL;
  }

  return (hwndT);
}


/*---------------------------------------------------------------------------*
 * RedrawCursoredTabs
 *
 * Redraws the cursored major and minor tabs if they're visible.
 *
 * History:
 * 19-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void RedrawCursoredTabs (
  PBOOK pbk)
{
  RECT rcUpdate;

  if (IsTabVisible (pbk, pbk->ppgCursorMajor))
  {
    CalcInvalidateRect (pbk, pbk->ppgCursorMajor, pbk->ppgCursorMajor,
                        &rcUpdate);
    DrawUpdateRect     (pbk, pbk->ppgCursorMajor, pbk->ppgCursorMajor,
                        &rcUpdate, NULL);
  }

  if (IsTabVisible (pbk, pbk->ppgCursorMinor))
  {
    CalcInvalidateRect (pbk, pbk->ppgCursorMinor, pbk->ppgCursorMinor,
                        &rcUpdate);
    DrawUpdateRect     (pbk, pbk->ppgCursorMinor, pbk->ppgCursorMinor,
                        &rcUpdate, NULL);
  }
}


/*---------------------------------------------------------------------------*
 * IsReverseNavigation
 *
 * Determines whether cursor keys should be handled in a reverse manner in
 * order to present a visually correct cursor navigation.
 *
 * History:
 * 19-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL IsReverseNavigation (
  PBOOK pbk)
{
  USHORT usSide = DetermineSide (pbk, pbk->ppgCursor, 0);               /*JEH1*/

  return (((pbk->flStyle & BKS_BACKPAGESTL) ||
           ((pbk->flStyle & (BKS_BACKPAGESBL | BKS_BACKPAGESTR)) &&
             ((usSide == BKV_RIGHT) || (usSide == BKV_BOTTOM)))));      /*JEH1*/
}


/*---------------------------------------------------------------------------*
 * QueryBitmapResource
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

HBITMAP QueryBitmapResource ( USHORT idxResource)                       /*JEH1*/
{
  /*--------------------------------*/
  /* Resource identifer is a bitmap */
  /*--------------------------------*/
  if (!vhBookResource[idxResource])
  {
    vhBookResource[idxResource] = LoadBitmap (vhInstance,
                                              MAKEINTRESOURCE (idxResource));
  }

  return (vhBookResource[idxResource]);
}


/*---------------------------------------------------------------------------*
 * QueryIconResource
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

HICON  QueryIconResource ( USHORT idxResource)                          /*JEH1*/
{
  /*-------------------------------*/
  /* Resource identifer is an icon */
  /*-------------------------------*/
  if (!vhBookResource[idxResource])
  {
    vhBookResource[idxResource] = LoadIcon (vhInstance,
                                            MAKEINTRESOURCE (idxResource));
  }

  return (vhBookResource[idxResource]);
}


/*---------------------------------------------------------------------------*
 * QueryPageCount
 *
 * Returns the number of pages in the book.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

LONG QueryPageCount(                                                    /*JEH1*/
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgQuery,
  USHORT usQueryEnd)
{
  LONG cPages;                                                          /*JEH1*/
  PPAGE ppgPage;                                                         //@MAB5

  /*----------------------------------------------------------------*/
  /* If a NULL page identifier was passed, start at the first page. */
  /*----------------------------------------------------------------*/
  if (!ppgQuery)
    ppgQuery = pbk->ppgFirst;

  /*----------------------------------------------------------------*/
  /* If first page identifier is NULL, then book is empty (0 pages) */
  /*----------------------------------------------------------------*/
  if (!ppgQuery)                                                        /*DAB4*/
    return (0);                                                         /*DAB4*/

  /*------------------------------------------------------------*/
  /* if an invalid stopping point was specified, return failure */
  /*------------------------------------------------------------*/
  if (!(usQueryEnd & (BKA_MAJOR | BKA_MINOR | BKA_END)) ||
      !VerifyPageId (pbk, ppgQuery))
  {
    return (BOOKERR_INVALID_PARAMETERS);
  }

  /*---------------------------*/                                      //->@MAB5
  /* Count the number of pages */
  /*---------------------------*/
  cPages = 1;
  ppgPage = ppgQuery->ppgNext;
  while (ppgPage && !(ppgPage->fsAttr & usQueryEnd))
  {
    cPages++;
    ppgPage = ppgPage->ppgNext;
  }                                                                    //@MAB5<-

  return (cPages);
}


/*---------------------------------------------------------------------------*
 * QueryPageId
 *
 * Returns the page identifier for a given page.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

LONG QueryPageId(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgQuery,
  USHORT usQueryOrder,
  USHORT usPageStyle)
{
  PPAGE ppgRet = (PPAGE) BOOKERR_INVALID_PARAMETERS;

  switch (usQueryOrder)
  {
    case BKA_FIRST:
      if (usPageStyle & BKA_MAJOR)
        ppgRet = GetTabPtr (pbk->ppgFirstTab, usPageStyle, BKA_NEXT);
      else if (usPageStyle & BKA_MINOR)
        ppgRet = GetTabPtr (pbk->ppgFirst, usPageStyle, BKA_NEXT);
      else
        ppgRet = pbk->ppgFirst;
      break;

    case BKA_TOP:
      ppgRet = QueryTopPage (pbk);
      break;

    case BKA_LAST:
      ppgRet = (usPageStyle & (BKA_MAJOR | BKA_MINOR)) ?
                GetTabPtr (pbk->ppgLastTab, usPageStyle, BKA_LAST) :
                pbk->ppgLast;
      break;

    case BKA_PREV:
      if (VerifyPageId (pbk, ppgQuery))
      {
        ppgRet = (usPageStyle & (BKA_MAJOR | BKA_MINOR)) ?
                  GetTabPtr (ppgQuery->ppgPrev, usPageStyle, BKA_PREV) :
                  ppgQuery->ppgPrev;
      }
      break;

    case BKA_NEXT:
      if (VerifyPageId (pbk, ppgQuery))
      {
        ppgRet = (usPageStyle & (BKA_MAJOR | BKA_MINOR)) ?
                  GetTabPtr (ppgQuery->ppgNext, usPageStyle, BKA_NEXT) :
                  ppgQuery->ppgNext;
      }
      break;

    case BKA_DELETED:
      ppgRet = vppgDeleted;
      break;

    case BKA_PREVSELECTED:
      ppgRet = vppgPrevSelected;
      break;
  }
  return ((LONG)ppgRet);                                                /*DAB1*/
}


/*---------------------------------------------------------------------------*
 * QueryStatusLineText
 *
 * Gets the status text for a given page.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

int QueryStatusLineText(                                                /*JEH1*/
  PBOOK     pbk,                            /* book instance data (locked)   */
  PPAGE     ppgQuery,
  PBOOKTEXT pbktxt)
{
  /*---------------------------------------------------------------*/
  /* if the status structure is NULL or an invalid pageid , return */
  /*---------------------------------------------------------------*/
  if (!pbktxt || !VerifyPageId (pbk, ppgQuery) ||
      !(ppgQuery->fsAttr & BKA_STATUSTEXTON))
    return (BOOKERR_INVALID_PARAMETERS);

  /*---------------------------------------------*/
  /* If no status text has been set, return NULL */
  /*---------------------------------------------*/
  if (!ppgQuery->npszStatus)
    return (0);

  /*--------------------------------------*/
  /* if a 0 buffer length was passed in,  */
  /* return the length of the status text */
  /*--------------------------------------*/
  if (!pbktxt->textLen)
    return (lstrlen (ppgQuery->npszStatus));

  if (pbktxt->pString)
  {
    /*-----------------*/
    /* mimic a strncpy */
    /*-----------------*/
    USHORT i;

    for (i=0; i<pbktxt->textLen-1 && ppgQuery->npszStatus[i]; i++)
    {
      pbktxt->pString[i] = ppgQuery->npszStatus[i];
    }
    pbktxt->pString[i] = '\0';

    return (i);
  }
  else
    return (BOOKERR_INVALID_PARAMETERS);
}


/*---------------------------------------------------------------------------*
 * QueryTabText
 *
 * Gets the text for the tab on a given page.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

int QueryTabText(                                                       /*JEH1*/
  PBOOK     pbk,                            /* book instance data (locked)   */
  PPAGE     ppgQuery,
  PBOOKTEXT pbktxt)
{
  /*------------------------------------------------------------*/
  /* if the tab structure is NULL or an invalid pageid , return */
  /*------------------------------------------------------------*/
  if ((!pbktxt) || (!VerifyPageId (pbk, ppgQuery)) ||
      !(ppgQuery->fsAttr & (BKA_MAJOR | BKA_MINOR)))
    return (BOOKERR_INVALID_PARAMETERS);

  /*------------------------------------------*/
  /* If no tab text has been set, return NULL */
  /*------------------------------------------*/
  if (!ppgQuery->ptab->npszText)
    return 0;                                                           /*JEH1*/

  /*------------------------------------------------------------------*/
  /* If passed tab text length is 0, return the length of the current */
  /* tab text string. The NULL terminator is not included in length.  */
  /*------------------------------------------------------------------*/
  if (pbktxt->textLen == 0)
    return (lstrlen (ppgQuery->ptab->npszText));

  /*------------------------------------------------------------------*/
  /* Otherwise, copy the current tab text string to the passed buffer */
  /* area and return the length of the copied text string.            */
  /*------------------------------------------------------------------*/
  if (pbktxt->pString)
  {
    /*-----------------*/
    /* mimic a strncpy */
    /*-----------------*/
    USHORT i;

    for (i=0; i<pbktxt->textLen-1 && ppgQuery->ptab->npszText[i]; i++)
    {
      pbktxt->pString[i] = ppgQuery->ptab->npszText[i];
    }
    pbktxt->pString[i] = '\0';

    return (i);
  }
  else
    return (BOOKERR_INVALID_PARAMETERS);
}


/*---------------------------------------------------------------------------*
 * QueryTopPage
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

PPAGE QueryTopPage(
  PBOOK pbk)                                /* book instance data (locked)   */
{
  return ((pbk->ppgTop)      ? pbk->ppgTop      :
          (pbk->ppgTopMinor) ? pbk->ppgTopMinor :
          (pbk->ppgTopMajor) ? pbk->ppgTopMajor : NULL);
}


/*---------------------------------------------------------------------------*
 * SetDimensions
 *
 * Sets the width and height of the major tab, minor tab, or page buttons.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetDimensions(
  PBOOK  pbk,                               /* book instance data (locked)    */
  USHORT usWidth,
  USHORT usHeight,
  USHORT usType)
{
  BOOL rc = FALSE;

  if ((usType == BKA_MAJORTAB) || (usType ==  BKA_MINORTAB))
  {
    /*-----------------------------------------------------*/
    /* Set the new tab dimensions and the book dimensions. */
    /*-----------------------------------------------------*/
    rc = SetTabDimensions (pbk, usWidth, usHeight, usType);
  }

  else if (usType == BKA_PAGEBUTTON)
  {
    /*-------------------------------------*/
    /* Set the new page button dimensions. */
    /*-------------------------------------*/
    rc = SetPageButtonDimensions (pbk, usWidth, usHeight);
  }

  /*----------------------------*/
  /* Reset the Notebook regions */
  /*----------------------------*/
  if (rc)  ResetNotebookRegions (pbk);
  return (rc);
}


/*---------------------------------------------------------------------------*
 * SetNotebookColors
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetNotebookColors (
  PBOOK  pbk,                               /* book instance data (locked)   */
  LONG   crColor,
  USHORT usRegion)
{
  RECT   rcUpdate;
  USHORT idForward, idBackward;

  /*-------------------------------------------------------*/
  /* if the app didn't give us a valid index, return error */
  /*-------------------------------------------------------*/
  if (usRegion > CCOLORS)  return (FALSE);

  pbk->crColor[usRegion-1] = crColor;

  switch (usRegion)
  {
    case BKA_BACKGROUNDPAGECOLOR:
      InvalidateRect (pbk->hwndBookPage, NULL, FALSE);
      /* flow through */

    case BKA_FOREGROUNDCOLOR:
    case BKA_BACKGROUNDCOLOR:
      InvalidateRect (pbk->hwndBook, NULL, FALSE);
      break;

    case BKA_BORDERCOLOR:
      InvalidateRect (pbk->hwndBook, NULL, FALSE);
      InvalidateRect (GetDlgItem (pbk->hwndBookPage, ID_PAGEBUTTONLEFT),
                      NULL, FALSE);
      InvalidateRect (GetDlgItem (pbk->hwndBookPage, ID_PAGEBUTTONRIGHT),
                      NULL, FALSE);
      break;

    case BKA_BACKGROUNDMAJORCOLOR:
    case BKA_FOREGROUNDMAJORCOLOR:
      CalcInvalidateRect (pbk, pbk->ppgFirstVisMajor,
                          pbk->ppgLastVisMajor, &rcUpdate);
      DrawUpdateRect (pbk, pbk->ppgFirstVisMajor,
                      pbk->ppgLastVisMajor, &rcUpdate, NULL);

      if (pbk->flStyle & (BKS_MAJORTABTOP | BKS_MAJORTABBOTTOM))
      {
        idBackward = ID_HORIZONTALLEFT;
        idForward  = ID_HORIZONTALRIGHT;
      }
      else if (pbk->flStyle & (BKS_MAJORTABRIGHT | BKS_MAJORTABLEFT))
      {
        idBackward = ID_VERTICALTOP;
        idForward  = ID_VERTICALBOTTOM;
      }

      InvalidateRect (GetDlgItem (pbk->hwndBook, idBackward), NULL, FALSE);
      InvalidateRect (GetDlgItem (pbk->hwndBook, idForward),  NULL, FALSE);
      break;

    case BKA_BACKGROUNDMINORCOLOR:
    case BKA_FOREGROUNDMINORCOLOR:
      CalcInvalidateRect (pbk, pbk->ppgFirstVisMinor,
                          pbk->ppgLastVisMinor, &rcUpdate);
      DrawUpdateRect (pbk, pbk->ppgFirstVisMinor,
                      pbk->ppgLastVisMinor, &rcUpdate, NULL);

      if (pbk->flStyle & (BKS_MAJORTABRIGHT | BKS_MAJORTABLEFT))
      {
        idBackward = ID_HORIZONTALLEFT;
        idForward  = ID_HORIZONTALRIGHT;
      }
      else if (pbk->flStyle & (BKS_MAJORTABTOP | BKS_MAJORTABBOTTOM))
      {
        idBackward = ID_VERTICALTOP;
        idForward  = ID_VERTICALBOTTOM;
      }

      InvalidateRect (GetDlgItem (pbk->hwndBook, idBackward), NULL, FALSE);
      InvalidateRect (GetDlgItem (pbk->hwndBook, idForward),  NULL, FALSE);
      break;

    case BKA_CURSORCOLOR:
      /*---------------------------------------------------*/
      /* if the notebook has the focus, invalidate the     */
      /* tabs so the cursor will redraw in the right color */
      /*---------------------------------------------------*/
      if (GetFocus () == pbk->hwndBook)
      {
        InvalidateRect (pbk->hwndBook, NULL, FALSE);
      }
      break;
  }

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * SetNotebookStyle
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

#define WS_REQUIRED   (WS_CHILD | WS_CLIPCHILDREN)
//#define WS_REQUIRED   WS_CHILD

VOID SetNotebookStyle (
  PBOOK pbk,                                /* book instance data (locked)   */
  BOOL  fInitialize)
{
  ULONG flStyle;
  ULONG flNewStyle = GetWindowLong (pbk->hwndBook, GWL_STYLE);

  /*--------------------------------------------*/
  /* make sure the notebook has required styles */
  /*--------------------------------------------*/
  if ((flNewStyle & WS_REQUIRED) != WS_REQUIRED)
  {
    SetWindowLong (pbk->hwndBook, GWL_STYLE, flNewStyle | WS_REQUIRED);
  }

  /*-----------------------*/
  /* Backpages bottom left */
  /*-----------------------*/
  if (flNewStyle & BKS_BACKPAGESBL)
  {
    flStyle = BKS_BACKPAGESBL | ((flNewStyle & BKS_MAJORTABLEFT) ?
                                  BKS_MAJORTABLEFT : BKS_MAJORTABBOTTOM);
  }

  /*---------------------*/
  /* Backpages top right */
  /*---------------------*/
  else if (flNewStyle & BKS_BACKPAGESTR)
  {
    flStyle = BKS_BACKPAGESTR | ((flNewStyle & BKS_MAJORTABRIGHT) ?
                                  BKS_MAJORTABRIGHT : BKS_MAJORTABTOP);
  }

  /*--------------------*/
  /* Backpages top left */
  /*--------------------*/
  else if (flNewStyle & BKS_BACKPAGESTL)
  {
    flStyle = BKS_BACKPAGESTL | ((flNewStyle & BKS_MAJORTABTOP) ?
                                  BKS_MAJORTABTOP : BKS_MAJORTABLEFT);
  }

  /*----------------------------------*/
  /* Backpages bottom right (default) */
  /*----------------------------------*/
  else
  {
    flStyle = BKS_BACKPAGESBR | ((flNewStyle & BKS_MAJORTABBOTTOM) ?
                                  BKS_MAJORTABBOTTOM : BKS_MAJORTABRIGHT);
  }

  /*------------------*/
  /* Set the tab type */
  /*------------------*/
  flStyle |= (flNewStyle & BKS_ROUNDEDTABS) ? BKS_ROUNDEDTABS :
             (flNewStyle & BKS_POLYGONTABS) ? BKS_POLYGONTABS :
                                              BKS_SQUARETABS;

  /*----------------------------------------*/
  /* Set the status line text justification */
  /*----------------------------------------*/
  flStyle |= (flNewStyle & BKS_STATUSTEXTCENTER) ? BKS_STATUSTEXTCENTER :
             (flNewStyle & BKS_STATUSTEXTRIGHT)  ? BKS_STATUSTEXTRIGHT  :
                                                   BKS_STATUSTEXTLEFT;

  /*--------------------------------*/
  /* Set the tab text justification */
  /*--------------------------------*/
  flStyle |= (flNewStyle & BKS_TABTEXTRIGHT)  ? BKS_TABTEXTRIGHT   :
             (flNewStyle & BKS_TABTEXTCENTER) ? BKS_TABTEXTCENTER  :
                                                BKS_TABTEXTLEFT;

  /*----------------------*/
  /* set the binding type */
  /*----------------------*/
  if (flNewStyle & BKS_SPIRALBIND)  flStyle |= BKS_SPIRALBIND;

  /*------------------------------------------------------------*/
  /* if the style has changed, recalculate the notebook regions */
  /*------------------------------------------------------------*/
  if (fInitialize || (LOWORD (flStyle) != LOWORD (pbk->flStyle)))
  {
    pbk->flStyle = flStyle;
    ResetNotebookRegions (pbk);
  }
}


/*---------------------------------------------------------------------------*
 * SetPageButtonDimensions
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetPageButtonDimensions(
  PBOOK pbk,                                /* book instance data (locked)   */
  SHORT cxPageButton,
  SHORT cyPageButton)
{
  if ((cxPageButton >= 0) && (cxPageButton < (SHORT) vcxScreen) &&
      (cyPageButton >= 0) && (cyPageButton < (SHORT) vcyScreen))
  {
    pbk->cxPageButton = cxPageButton;
    pbk->cyPageButton = cyPageButton;
    return (TRUE);
  }
  else
    return (FALSE);
}


/*---------------------------------------------------------------------------*
 * SetPageWindowHwnd
 *
 * Associates a window with a page.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetPageWindowHwnd (
  PBOOK pbk,                                /* book instance data (locked)   */
  PPAGE ppg,
  HWND  hwndPage)
{
  /*----------------------------------*/
  /* Verify that the page id is valid */
  /*----------------------------------*/
  if (!VerifyPageId (pbk, ppg))  return (FALSE);

  /*------------------------------------------------------------*/
  /* if there was a window on this page before, hide it.  Don't */
  /* use BookShowWindow here because it won't hide the window   */
  /* if the notebook is invisible.                              */
  /*------------------------------------------------------------*/
  if (IsWindow (ppg->hwndPage))
    ShowWindow( ppg->hwndPage, SW_HIDE );                                //@MAB7

  /*------------------------------------------------------------*/
  /* Save the application page handle in the pages record data. */
  /*------------------------------------------------------------*/
  ppg->hwndPage = hwndPage;

  if (IsWindow (hwndPage))
  {
    /*----------------------------------------------------------*/
    /* save the old parent of the page window so we can forward */
    /* messages we don't care about to it from PageWndProc      */
    /*----------------------------------------------------------*/
    ppg->hwndPageParent = PARENTOF (hwndPage);

    /*-------------------------------------------------------------*/
    /* If this page window's current parent is the notebook page,  */
    /* then the page has been inserted in the book before.  Find   */
    /* a (any) page that this page window has been associated with */
    /* and use its hwndPageParent for this page as well.           */
    /*-------------------------------------------------------------*/
    if (ppg->hwndPageParent == pbk->hwndBookPage)
    {
      PPAGE ppgT;

      /*----------------------*/
      /* find a matching page */
      /*----------------------*/
      for (ppgT = pbk->ppgFirst;
           ppgT && ppgT->hwndPage != hwndPage;
           ppgT = ppgT->ppgNext);

      /*-----------------------------------------*/
      /* if we found one, use its hwndPageParent */
      /*-----------------------------------------*/
      if (ppgT)  ppg->hwndPageParent = ppgT->hwndPageParent;
    }

    /*-------------------------------------------------------------------*/
    /* Otherwise, the page window hasn't been inserted in the notebook   */
    /* before, so set the parent handle for the application page window. */
    /*-------------------------------------------------------------------*/
    else
    {
      SetParent (hwndPage, pbk->hwndBookPage);
    }
  }

  /*---------------------------------------------------------------------*/
  /* if this page window is set into the top page, show it; else hide it */
  /*---------------------------------------------------------------------*/
  if (ppg == QueryTopPage (pbk))                                       //->@MAB7
    ShowTopPage (pbk, NULL);
  else
  {
    /*-------------------------------------------------------------------*/
    /* Don't use BookShowWindow here because it won't hide the window if */
    /* the notebook is invisible.                                        */
    /*-------------------------------------------------------------------*/
    if (IsWindow (hwndPage))
      ShowWindow( hwndPage, SW_HIDE );
  }                                                                    //@MAB7<-

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * SetPageWndPos
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID SetPageWndPos(
  PBOOK  pbk,                               /* book instance data (locked)   */
  LPRECT lprcPage)
{
  *lprcPage = pbk->rcTopPage;
  CalcPageWnd (pbk, lprcPage, TRUE);

  SetWindowPos (pbk->hwndBookPage, NULL,
                lprcPage->left,
                lprcPage->top,
                lprcPage->right - lprcPage->left,
                lprcPage->bottom - lprcPage->top,
                SWP_NOACTIVATE);
}


/*---------------------------------------------------------------------------*
 * SetTabBitmap
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetTabBitmap(
  PBOOK   pbk,                              /* book instance data (locked)   */
  PPAGE   ppg,
  HBITMAP hbm)
{
  /*------------------------------------------------------------------*/
  /* If a NULL record is passed or the page is not a tab page, return */
  /*------------------------------------------------------------------*/
  if (!VerifyPageId (pbk, ppg) || !(ppg->fsAttr & (BKA_MAJOR | BKA_MINOR)))
  {
    return (FALSE);
  }

  /*-----------------------------------------------------------*/
  /* If a tab text string currenty exists, free the associated */
  /* storage and null the pointer.                             */
  /*-----------------------------------------------------------*/
  if (ppg->ptab->npszText)
  {
    ppg->ptab->npszText = BookFreeMem (ppg->ptab->npszText);
  }

  /*----------------------------------*/
  /* bitmap tabs don't have mnemonics */
  /*----------------------------------*/
  ppg->ptab->chMnemonic = 0;                                            /*JEH1*/

  /*-------------------------*/
  /* Set tab bitmap and type */
  /*-------------------------*/
  ppg->fsAttr &= ~BKA_TEXT;
  ppg->fsAttr |= BKA_BITMAP;
  ppg->ptab->hbmTab = hbm;

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * SetTabDimensions
 *
 * Sets the Tab dimensions.  If a parameter is passed in that is 0 then the
 * parameter is not reset.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetTabDimensions(
  PBOOK  pbk,                               /* book instance data (locked)   */
  USHORT cxTab,
  USHORT cyTab,
  USHORT usTabType)
{
  /*-------------------------------------------------------------------*/
  /* The tab width and height must be include space for the tab border */
  /* and cursor emphasis                                               */
  /*-------------------------------------------------------------------*/
  if ((((cxTab >= BKV_TABSIZEMIN) && (cyTab >= BKV_TABSIZEMIN)) ||
       ((cxTab == 0) && (cyTab == 0)))  &&
       (cxTab < vcxScreen) && (cyTab < vcyScreen))
  {
    /*------------------------------------------------------------*/
    /* Reset the tab dimension if the passed values are valid and */
    /* the tab type is valid.                                     */
    /*------------------------------------------------------------*/
    switch (usTabType)
    {
      /*-----------------------------------------------------------*/
      /* If the tab type is major, reset the major tab dimensions. */
      /*-----------------------------------------------------------*/
      case BKA_MAJORTAB:
        pbk->cxMajorTab = cxTab;
        pbk->cyMajorTab = cyTab;
        break;

      /*-----------------------------------------------------------*/
      /* If the tab type is minor, reset the minor tab dimensions. */
      /*-----------------------------------------------------------*/
      case BKA_MINORTAB:
        pbk->cxMinorTab = cxTab;
        pbk->cyMinorTab = cyTab;
        break;
    }

    return (TRUE);
  }

  /*---------------------------------------------------------------*/
  /* If the tab dimensions are not large enough to include the tab */
  /* border and cursor emphasis, return a failure indicator.       */
  /*---------------------------------------------------------------*/
  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * SetTabText
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetTabText(
  PBOOK pbk,                                /* book instance data (locked)   */
  PPAGE ppg,
  PSZ   psz)
{
  PSZ  pchMnemonic;
  PSZ  npszNew;                                                         /*JEH1*/

  /*-----------------------------------------------------------------*/
  /* if the page isn't valid, or the page is doesn't have a tab, or  */
  /* we couldn't allocate memory for the new string, return an error */
  /*-----------------------------------------------------------------*/
  if (!VerifyPageId (pbk, ppg) || !(ppg->fsAttr & (BKA_MAJOR | BKA_MINOR)) ||
      (psz && !(npszNew = BookAllocMem (lstrlen (psz) + 1))))
  {
    return (FALSE);
  }

  /*--------------------------------------------------------------------*/
  /* If a tab text string currenty exists, free the associated storage. */
  /*--------------------------------------------------------------------*/
  if (ppg->ptab->npszText)
  {
    ppg->ptab->npszText = BookFreeMem (ppg->ptab->npszText);
  }

  /*-----------------------------------------------------------*/
  /* Set the record data to indicate the current data is text. */
  /*-----------------------------------------------------------*/
  ppg->ptab->hbmTab = NULL;
  ppg->fsAttr &= ~BKA_BITMAP;
  ppg->fsAttr |= BKA_TEXT;

  if (psz)
  {
    /*------------------------------------------------------*/
    /* get the accelerator from the tab string (if any) and */
    /* convert it to lowercase to speed later comparisons   */
    /*------------------------------------------------------*/
    for (pchMnemonic = psz;
        *pchMnemonic && *pchMnemonic != MNEMONIC_CHAR;
         pchMnemonic++);

    if (*pchMnemonic)
    {
      ppg->ptab->chMnemonic = (CHAR) (*(pchMnemonic + 1) | 0x20);
    }

    /*------------------------------*/
    /* Copy the new tab text string */
    /*------------------------------*/
    ppg->ptab->npszText = npszNew;
    lstrcpy (ppg->ptab->npszText, psz);
  }

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * SetStatusLineText
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL SetStatusLineText(
  PBOOK pbk,                                /* book instance data (locked)   */
  PPAGE ppg,
  PSZ   pszStatus)
{
  PSZ  npszNew;                                                         /*JEH1*/

  /*------------------------------------------*/
  /* Verify that a valid record id was passed */
  /*------------------------------------------*/
  if (!VerifyPageId (pbk, ppg) || !(ppg->fsAttr & BKA_STATUSTEXTON) ||
      (pszStatus && !(npszNew = BookAllocMem (lstrlen (pszStatus) + 1))))
    return FALSE;

  /*----------------------------------------------------------*/
  /* If status text currently set free the associated memory. */
  /*----------------------------------------------------------*/
  if (ppg->npszStatus)
  {
    ppg->npszStatus = BookFreeMem (ppg->npszStatus);
  }

  if (pszStatus)
  {
    /*------------------------------------------------*/
    /* Copy the status string to the allocated memory */
    /*------------------------------------------------*/
    ppg->npszStatus = npszNew;
    lstrcpy (ppg->npszStatus, pszStatus);
  }

  /*----------------------------------------------------------------------*/
  /* If the status text is associated with the top page, redraw the text. */
  /*----------------------------------------------------------------------*/
  if (ppg == QueryTopPage (pbk))
    DrawStatusLineText (pbk, ppg, NULL, TRUE);

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * TerminateNotebook
 *
 * Cleans up after a notebook during a WM_DESTROY.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 * 28-Jul-92 JeffR    Added call to SubclassTopPage to fix local heap
 *                    corruption problem.
 *---------------------------------------------------------------------------*/

VOID TerminateNotebook(
  PBOOK pbk)                                /* book instance data (locked)   */
{
  USHORT i;

  /*------------------------------------------------*/
  /* unsubclass the top page so TopPageSubclassProc */
  /* won't see the WM_NCDESTROY message             */
  /*------------------------------------------------*/
  SubclassTopPage (pbk, NULL, FALSE);

  /*-------------------------------------------------------------*/
  /* decrement the notebook count; if this is the last one,      */
  /* delete all of the resources we loaded before, cause Windows */
  /* won't clean up for us (!)                                   */
  /*-------------------------------------------------------------*/
  if (!--vcNotebooks)
  {
    for (i=ID_TABDWN; i<=ID_BINDBOTTOMSPIRAL; i++)
    {
      if (vhBookResource[i])
      {
        DestroyIcon (vhBookResource[i]);
        vhBookResource[i] = NULL;
      }
    }

    for (i=ID_PAGEBUTTONRUP; i<=ID_PAGEBUTTONLDOWN_D; i++)
    {
      if (vhBookResource[i])
      {
        DeleteObject (vhBookResource[i]);
        vhBookResource[i] = NULL;
      }
    }
  }

  /*------------------------------------------------*/
  /* get rid of the memory DC used to draw the tabs */
  /*------------------------------------------------*/
  DeleteDC (pbk->hdcMem);

  /*-------------------------------------------*/
  /* Delete all pages for the current instance */
  /*-------------------------------------------*/
  DeletePage (pbk, NULL, BKA_ALL, FALSE);

  /*---------------------------------*/
  /* Free the internal instance data */
  /*---------------------------------*/
  BookFreeMem (pbk);
}


/*---------------------------------------------------------------------------*
 * ProcessMouseButtonDown
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

PPAGE ProcessMouseButtonDown(
  PBOOK pbk,                                /* book instance data (locked)   */
  POINT ptMouse)
{
  PPAGE ppgTab;

  /*--------------------------------------------------*/
  /* Determine if mouse is over a major tab rectangle */
  /*--------------------------------------------------*/
  ppgTab = pbk->ppgFirstVisMajor;

  while (ppgTab)
  {
    if (PtInRect (&ppgTab->ptab->rc, ptMouse))
      return ppgTab;

    ppgTab = (ppgTab == pbk->ppgLastVisMajor) ? NULL :
                 GetTabPtr (ppgTab->ptab->ppgNextTab, BKA_MAJOR, BKA_NEXT);
  }

  /*--------------------------------------------------*/
  /* Determine if mouse is over a minor tab rectangle */
  /*--------------------------------------------------*/
  ppgTab = pbk->ppgFirstVisMinor;

  while (ppgTab)
  {
    if (PtInRect (&ppgTab->ptab->rc, ptMouse))
      return ppgTab;

    ppgTab = (ppgTab == pbk->ppgLastVisMinor) ? NULL :
                 GetTabPtr (ppgTab->ptab->ppgNextTab, BKA_MINOR, BKA_NEXT);
  }

  return (ppgTab);
}


/*---------------------------------------------------------------------------*
 * PaintBook
 *
 * Draws the notebook in response to the WM_PAINT message.
 *
 * History:
 * 15-Sep-91 JeffR    Created
 *---------------------------------------------------------------------------*/

VOID PaintBook (
  PBOOK pbk)                                /* book instance data (locked)   */
{
  PAINTSTRUCT ps;
  RECT        rcInvalid, rcIntersect;
  HRGN        hrgnPaint;

  /*---------------------------------------------------------*/
  /* Check to see if the application has changes any of the  */
  /* window style bits. If so verify and reset global style. */
  /*---------------------------------------------------------*/
  SetNotebookStyle (pbk, FALSE);

  /*-------------------------------------*/
  /* Get presentation space for painting */
  /*-------------------------------------*/
  if (!(hrgnPaint = CreateRectRgn (0, 0, 0, 0)))  return;

  GetUpdateRgn (pbk->hwndBook, hrgnPaint, FALSE);
  BeginPaint (pbk->hwndBook, &ps);

  /*-----------------------------------*/
  /* Fill in Notebook background color */
  /*-----------------------------------*/
  BookFillRect (ps.hdc, &ps.rcPaint, pbk->crColor[CLR_NOTEBOOK]);

  /*--------------------------------------------------------*/
  /* If the book page area is invalid, fill with page color */
  /*--------------------------------------------------------*/
  if (IntersectRect (&rcInvalid, &ps.rcPaint, &pbk->rcBookPages))
  {
    BookFillRect (ps.hdc, &rcInvalid, pbk->crColor[CLR_PAGEBG]);
  }

  /*---------------------------------------------------------*/
  /* Draw the back pages based on the window style settings. */
  /*---------------------------------------------------------*/
  DrawBookBackpages (pbk, ps.hdc, &pbk->rcTopPage, &ps.rcPaint);

  /*---------------------------------------------*/
  /* If the binding area is invalid, redraw them */
  /*---------------------------------------------*/
  if (IntersectRect (&rcInvalid, &ps.rcPaint, &pbk->rcBinding))
  {
    DrawBookBinding (pbk, ps.hdc, &rcInvalid);
  }

  /*--------------------------------*/
  /* if major tabs exist, draw them */
  /*--------------------------------*/
  if (pbk->ppgFirstVisMajor)
  {
    CalcInvalidateRect (pbk, pbk->ppgFirstVisMajor, pbk->ppgLastVisMajor,
                        &rcInvalid);

    /*-----------------------------------------------------------*/
    /* If the calculated invalid rectangle intersects with the   */
    /* rectangle that Windows invalidated, reduce the clipping   */
    /* region to the intersection of the current clipping region */
    /* and the calculated invalid rectangle.  This will keep us  */
    /* from overwriting things we've already drawn.              */
    /*-----------------------------------------------------------*/
    if (IntersectRect (&rcIntersect, &ps.rcPaint, &rcInvalid))
    {
      IntersectClipRect (ps.hdc, rcInvalid.left,  rcInvalid.top,
                                 rcInvalid.right, rcInvalid.bottom);
      DrawUpdateRect (pbk, pbk->ppgFirstVisMajor, pbk->ppgLastVisMajor,
                      &rcIntersect, ps.hdc);
      SelectClipRgn (ps.hdc, hrgnPaint);
    }
  }

  /*--------------------------------*/
  /* if minor tabs exist, draw them */
  /*--------------------------------*/
  if (pbk->ppgFirstVisMinor)
  {
    CalcInvalidateRect (pbk, pbk->ppgFirstVisMinor, pbk->ppgLastVisMinor,
                        &rcInvalid);

    /*-----------------------------------------------------------*/
    /* If the calculated invalid rectangle intersects with the   */
    /* rectangle that Windows invalidated, reduce the clipping   */
    /* region to the intersection of the current clipping region */
    /* and the calculated invalid rectangle.  This will keep us  */
    /* from overwriting things we've already drawn.              */
    /*-----------------------------------------------------------*/
    if (IntersectRect (&rcIntersect, &ps.rcPaint, &rcInvalid))
    {
      IntersectClipRect (ps.hdc, rcInvalid.left,  rcInvalid.top,
                                 rcInvalid.right, rcInvalid.bottom);
      DrawUpdateRect (pbk, pbk->ppgFirstVisMinor, pbk->ppgLastVisMinor,
                      &rcIntersect, ps.hdc);
      SelectClipRgn (ps.hdc, hrgnPaint);
    }
  }

  /*-------------------------------------------*/
  /* if there are no tabs to draw a focus      */
  /* indicator on, draw one on the page window */
  /*-------------------------------------------*/
  if (pbk->ppgCursor && !IsTabVisible (pbk, pbk->ppgCursor) &&          /*DAB1*/
      IntersectRect (&rcIntersect, &ps.rcPaint, &pbk->rcAppPageWnd) &&
      GetFocus () == pbk->hwndBook)
  {
    DrawFocusRect (ps.hdc, &pbk->rcAppPageWnd);
  }

  /*------------------------------------*/
  /* Show the application's page window */
  /*------------------------------------*/
  ShowTopPage (pbk, ps.hdc);
  EndPaint (pbk->hwndBook, &ps);
  DeleteObject (hrgnPaint);
}


/*---------------------------------------------------------------------------*
 * SendSetCursor
 *
 * Sends a WM_SETCURSOR to the notebook's parent window.
 *
 * History:
 * 10-Dec-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

VOID SendSetCursor (
  HWND hwndBook,
  UINT msg)                                                             /*JEH1*/
{
  SendMessage (PARENTOF (hwndBook), WM_SETCURSOR, (WPARAM)hwndBook,     /*JEH1*/
               MAKELONG (HTNOWHERE, LOWORD(msg)));                      /*JEH1*/
}


/*---------------------------------------------------------------------------*
 * TopPageSubclassProc
 *
 * Processes input messages for top page windows.  This insures that focus
 * processing on Alt-PageUp and Alt-PageDown work properly with no effort
 * on the application's part.
 *
 * History:
 * 09-Dec-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

WNDPROC TopPageSubclassProc (HWND hwnd, UINT msg, WPARAM wParam,        /*JEH1*/
                             LPARAM lParam)                             /*JEH1*/
{
  PBOOK pbk = (PBOOK) GetProp (hwnd, ICL_NOTEBOOK);

  /******************************************************************//*D4123*/
  /* Let CCL messages fall through to the actual CCL proc.          *//*D4123*/
  /* CallWindowProc will lose the HIWORD too, so don't do the       *//*D4123*/
  /* table lookup until in the actual CCL control's window proc.    *//*D4123*/
  /* In other words, don't do it in this subclass proc.             *//*D4123*/
  /* The messages processed in this proc do not need a table        *//*D4123*/
  /* lookup.                                                        *//*D4123*/
  /******************************************************************//*D4123*/

  switch (msg)
  {
    case WM_KEYDOWN:
    case WM_SYSKEYDOWN:
    {
      BOOL fCtrl  = GetKeyState (VK_CONTROL) & 0x8000;
      BOOL fShift = GetKeyState (VK_SHIFT)   & 0x8000;
      BOOL fAlt   = HIWORD (lParam) & 0x2000;

      /*---------------------------------------------------------------------*/
      /* if Alt+PageUp or Alt+PageDn was pressed, let the notebook handle it */
      /*---------------------------------------------------------------------*/
      if (((wParam == VK_PRIOR) || (wParam == VK_NEXT)) &&
           fAlt && !fShift && !fCtrl)
      {
        HWND hwndOldFocus = GetFocus ();

        SendMessage (pbk->hwndBook, WM_KEYDOWN, wParam, lParam);

        /*--------------------------------------------------------*/
        /* if the focus hasn't changed after the page-turn, the   */
        /* app left it on the old page, so put it on the notebook */
        /*--------------------------------------------------------*/
        if (GetFocus () == hwndOldFocus)  BookSetFocus (pbk->hwndBook);

        return (FALSE);
      }

      /*-----------------------------------------------------------*/
      /* if Alt+UpArrow was pressed, set the focus to the notebook */
      /*-----------------------------------------------------------*/
      else if ((wParam == VK_UP) && fAlt && !fShift && !fCtrl)
      {
        BookSetFocus (pbk->hwndBook);
        return (FALSE);
      }
      break;
    }

    case WM_NCDESTROY:
      /*---------------------------------------------------------------------*/
      /* If the window is destroyed while it's subclassed by the notebook,   */
      /* remove the property we added when it was subclassed.  If the window */
      /* is not destroyed while subclassed, the code that unsubclasses the   */
      /* window will remove the property.                                    */
      /*---------------------------------------------------------------------*/
      RemoveProp (hwnd, ICL_NOTEBOOK);
      pbk->hwndTopPage = NULL;
      break;
  }

  return (CallWindowProc (pbk->lpfnTopPageWndProc, hwnd, msg, wParam, lParam));
}


/*---------------------------------------------------------------------------*
 * BookKeyboardHookProc
 *
 * Processes keyboard messages for page windows.  This insures that focus
 * processing on Alt+PageUp, Alt+PageDn, and Alt+Up work properly with no
 * effort on the application's part.
 *
 * History:
 * 09-Dec-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

LRESULT CALLBACK BookKeyboardHookProc (int nCode, WPARAM wParam,        /*JEH1*/
                                       LPARAM lParam)                   /*JEH1*/
{
  HWND hwndTo;

  /*---------------------------------------------------------------*/
  /* if this is a keydown for a key we need to process, look at it */
  /*---------------------------------------------------------------*/
  if (IsDialogPageMessage (nCode, wParam, lParam, &hwndTo))
  {
    SendMessage (hwndTo, WM_KEYDOWN, wParam, lParam);
    return (1);
  }

  /*----------------------------------------*/
  /* otherwise, forward it to the next hook */
  /*----------------------------------------*/
  else
  {
//    return ((int) DefHookProc (nCode, wParam, lParam, &vlpNextHook));
    return  CallNextHookEx (vlpNextHook, nCode, wParam, lParam);
  }
}


/*---------------------------------------------------------------------------*
 * IsDialogPageMessage
 *
 * Determines if this keyboard message is one that needs to go to the
 * notebook.  It is a notebook message if:
 *
 *   1.  Windows says the keyboard hook can look and the message, AND
 *   2.  It is a key down (not a key up), AND
 *   3.  It is Alt+PageUp, Alt+PageDn, or Alt+Up, AND
 *   4.  The focus window's parent is subclassed by the notebook (this will
 *       be the case when a dialog is placed on a page)
 *
 * History:
 * 09-Dec-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL IsDialogPageMessage (int nCode, WPARAM wParam, LPARAM lParam,      /*JEH1*/
                          LPHANDLE phwndTo)                             /*JEH1*/
{
  #define fKeyDown  (!(lParam & 0x80000000))

  if (fKeyDown && (nCode >= 0))
  {
    BOOL fCtrl  = GetKeyState (VK_CONTROL) & 0x8000;
    BOOL fShift = GetKeyState (VK_SHIFT)   & 0x8000;
    BOOL fAlt   = HIWORD (lParam) & 0x2000;

    if (fAlt && !fShift && !fCtrl &&
        ((wParam == VK_PRIOR) || (wParam == VK_NEXT) || (wParam == VK_UP)))
    {
      *phwndTo = PARENTOF (GetFocus());                                //->@MAB3
      while ( *phwndTo && (*phwndTo != GetDesktopWindow()) )
      {
        if ( (LONG)TopPageSubclassProc ==
                      GetWindowLong( *phwndTo, GWL_WNDPROC ) )
          return( TRUE );

        *phwndTo = PARENTOF (*phwndTo);
      }                                                                //@MAB3<-
    }
  }

  return (FALSE);

  #undef fKeyDown
}


/*---------------------------------------------------------------------------*
 * BookSetFocus
 *
 * Sets the focus to the specified notebook window.  The BKV_SELFFOCUS flag
 * is OR-ed into the flState field of the instance data so the notebook will
 * not move the tab cursor to the major tab if it wasn't already there.
 *
 * History:
 * 20-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

void BookSetFocus (
  HWND hwndBook)
{
  PBOOK pbk = (PBOOK) GetWindowLong (hwndBook, QWL_NOTEBOOK);           /*JEH1*/

  pbk->flState |=  BKV_SELFFOCUS;
  SetFocus (hwndBook);
  pbk->flState &= ~BKV_SELFFOCUS;                                        //@MAB3
}


/*---------------------------------------------------------------------------*
 * IsSibling
 *
 * Determines if two windows are siblings (i.e. have the same parent)
 *
 * History:
 * 29-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL IsSibling (
  HWND hwnd1,
  HWND hwnd2)
{
  for (hwnd1 = GetWindow (PARENTOF (hwnd1), GW_CHILD);
       hwnd1 && hwnd2 && (hwnd1 != hwnd2);
       hwnd1 = GetWindow (hwnd1, GW_HWNDNEXT));

  return (hwnd1 == hwnd2);
}
