#ifndef _BOOKWIN_
#define _BOOKWIN_

/* Revision: 52 1.5 source/ui/cclnt/notebook/bookwin.h, cclbook, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*              Licensed Materials - Property of IBM                  */
/*              20G0400 (C) Copyright IBM Corp. 1992, 1997            */
/*                    See Copyright Instructions.                     */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/* CUA Controls Library/2 Version 1.0                       (Windows) */
/*                                                                    */
/* Sample Application Code                                            */
/* Notebook Control                                                   */
/*                                                                    */
/* File:  bookwin.h                                                   */
/*                                                                    */
/* This header file includes macro includes for the notebook sample   */
/* application (bookwin.c).                                           */
/*                                                                    */
/**********************************************************************/

/* control id's                                                       */

#define  ID_BOOK_CALENDAR       1      /* calendar                    */
#define  ID_WIN_CALENDAR_PAGE   2      /* calendar page               */
#define  ID_BOX_ERROR           3      /* error message box           */
#define  ID_CLIENT_RESOURCES  100      /* client window resources     */
#define  IDM_FILE             110
#define  IDM_SUBEXIT          111      /* exit                        */
#define  IDM_PAGE             120
#define  IDM_INSERT_AFTER     121      /* insert after current page   */
#define  IDM_INSERT_BEFORE    122      /* insert before current page  */
#define  IDM_DELETE_CURRENT   123      /* delete current page         */
#define  IDM_JUMP             130
#define  IDM_JUMP_TO_PAGE     131      /* jump to page in book        */
#define  ID_DLG_JUMP_PAGE     200      /* jump to page dialog box id  */
#define  IDD_BOX_MONTH        210      /* group box                   */

/**********************************************************************/
/* radio buttons are numbered IDD_BOX_MONTH + n                       */
/**********************************************************************/

#define  IDD_TEXT_TITLE 230            /* static text in box          */
#define  IDD_BOX_YEAR  240             /* group box                   */
#define  IDD_EDIT_YEAR 241             /* edit field for year         */
#define  IDD_BTN_OK    251             /* OK pushbutton               */
#define  IDD_BTN_CANCEL 252            /* cancel pushbutton           */
#define  BOOK_MARGIN_XLEFT 10          /* left margin of book in      */
                                       /* client                      */
#define  BOOK_MARGIN_XRIGHT 10         /* right margin of book in     */
                                       /* client                      */
#define  BOOK_MARGIN_YBOTTOM 10        /* bottom margin of book in    */
                                       /* client                      */
#define  BOOK_MARGIN_YTOP 10           /* top margin of book in       */
                                       /* client                      */
#define  BOOK_MAJOR_HEIGHT 30          /* major tab height            */
#define  BOOK_MAJOR_WIDTH 70           /* major tab width             */
#define  BOOK_MINOR_HEIGHT 25          /* minor tab height            */
#define  BOOK_MINOR_WIDTH 110          /* minor tab width             */
#define  BOOK_BUTTON_HEIGHT 25         /* page button height (1/25 of */
                                       /* client height               */
#define  BOOK_BUTTON_WIDTH 35          /* page button width (1/35 of  */
                                       /* client width                */
#define  BOOK_COL_LENGTH_THRESH 60     /* minimum column length       */
                                       /* before day names are        */
                                       /* collapsed                   */
#define  MAXTEXTLENGTH     100         /* maximum length of status txt*/

#define  CAL_MARGIN_LEFT 40            /* left margin of text on page */
#define  CAL_MARGIN_RIGHT 40           /* right margin of text on     */
                                       /* page                        */
#define  CAL_MARGIN_TOP 60             /* top margin of text on page  */
#define  CAL_MARGIN_BOTTOM 10          /* bottom margin of text on    */
                                       /* page                        */
#define  CAL_MARGIN_HEADER 35          /* top margin of month name    */

/**********************************************************************/
/* client window background                                           */
/**********************************************************************/

#define  COLOR_CLIENT_BACKGROUND RGB (128,128,128) /* PALE GREY       */

/**********************************************************************/
/* status text color                                                  */
/**********************************************************************/

#define  COLOR_STATUSTEXT RGB (0,0,255) /* DARK BLUE                  */

/**********************************************************************/
/* year title page background color                                   */
/**********************************************************************/

#define  COLOR_YEARTITLEPAGE RGB (0,0,255) /* DARK BLUE               */

/**********************************************************************/
/* year title page text color                                         */
/**********************************************************************/

#define  COLOR_YEARTITLETEXT RGB (0,0,255) /* DARK BLUE               */

/**********************************************************************/
/* calendar page background color                                     */
/**********************************************************************/

#define  COLOR_CALENDARPAGE RGB (255,255,255) /* WHITE                */

/**********************************************************************/
/* calendar page month header color                                   */
/**********************************************************************/

#define  COLOR_CALENDARHEADER RGB (0,0,255) /* DARK BLUE              */

/**********************************************************************/
/* calendar page days of week color                                   */
/**********************************************************************/

#define  COLOR_CALENDARDAYS RGB (128,128,128) /* DARK GREY            */

/**********************************************************************/
/* calendar page date numbers color                                   */
/**********************************************************************/

#define  COLOR_CALENDARDATES RGB (0,0,255) /* DARK BLUE               */

/**********************************************************************/
/* major tab color                                                    */
/**********************************************************************/

#define  COLOR_MAJORTABS RGB (255,255,255) /* WHITE                   */

/**********************************************************************/
/* minor tab color                                                    */
/**********************************************************************/

#define  COLOR_MINORTABS RGB (192,192,192) /* PALE GRAY               */
#define  YEAR_START    1990            /* first year                  */
#define  YEAR_STOP     1995            /* last year                   */

/**********************************************************************/
/* the number of years in the calendar                                */
/**********************************************************************/

#define  YEAR_COUNT    (YEAR_STOP - YEAR_START + 1)

/**********************************************************************/
/* this macro gives zero-based indices from a year                   */
/**********************************************************************/

#define  INDEXFROMYEAR(year) ((year) - YEAR_START)

/**********************************************************************/
/* the following are internal commands used to manage page id's       */
/**********************************************************************/

#define  PAGE_ID_CMD_STORE 1           /* store a new page id         */
#define  PAGE_ID_CMD_CONVERT 2         /* convert an id into a date   */
#define  PAGE_ID_CMD_RETRIEVE 3        /* retrieve the id for a date  */

/**********************************************************************/
/* internal flags to represent the type of page                       */
/**********************************************************************/

#define  YEAR_TITLEPAGE 0              /* title page for a year       */
#define  YEAR_INVALID  -1              /* invalid page                */

/**********************************************************************/
/* flag to indicate the page has been removed from the notebook       */
/**********************************************************************/

#define  PAGE_ID_FLAG_DELETED 0

/**********************************************************************/
/* structure to hold a date: month and year                           */
/**********************************************************************/

typedef struct
{
  int month;
  int year;
} DATE;

typedef DATE FAR *pDATE;

/**********************************************************************/
/* prototypes                                                         */
/**********************************************************************/

LRESULT CALLBACK ClientWndProc(HWND hwnd, UINT msg, WPARAM wParam,      /*JEH1*/
                               LPARAM lParam);                          /*JEH1*/
LRESULT CALLBACK PageWndProc  (HWND hwnd, UINT msg, WPARAM wParam,      /*JEH1*/
                               LPARAM lParam);                          /*JEH1*/
BOOL    CALLBACK JumpDlgProc  (HWND hwnd, UINT msg, WPARAM wParam,      /*JEH1*/
                               LPARAM lParam);                          /*JEH1*/

BOOL centerJustCharStringAt(HDC,POINT *,WORD,char *);
int  convertDateToDay(int,int);
BOOL createChildren(HANDLE,HANDLE);
VOID createPage(HANDLE,HANDLE,DATE,UINT,DWORD);                         /*DAB1*/
VOID drawMonthCalendar(HDC,RECT,int,int);
VOID drawYearTitlePage(HDC,RECT,int);
BOOL leapYear(int);
VOID menuInsertPage(HANDLE,WORD);
VOID newPageSelected(HANDLE,PPAGESELECTNOTIFY);                         /*DAB2*/
VOID pageRecord(int,UINT *,DATE *);                                     /*DAB1*/
BOOL rtJustCharStringAt(HDC,POINT *,WORD,char *);
#endif // BOOKWIN header guard
