/* Revision: 55 1.5 source/ui/cclnt/slider/sldinit.c, cclslid, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/*                           S L I D E R                              */
/*                                                                    */
/*                       Filename: SLDINIT.C                          */
/*                                                                    */
/*              ( Windows NT Slider Control Component )               */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: SLIDER.DLL                                                */
/* Filename: SLDINIT.C - Initialization routine for Slider control    */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* JEH1  06/21/93  JEH  Initial modifications for NT execution        */
/**********************************************************************/
#include <string.h>

#include <windows.h>
#include "iclsldw.h"               /* Slider control public includes  */
#include "sliderp.h"               /* Slider control private includes */

/**********************************************************************/
/* External variable declarations                                     */
/**********************************************************************/
extern HANDLE  hSliderInstance;   /* Module handle                    */
extern HBITMAP hbmScrSave;        /* Background preservation bitmap   */
extern HBITMAP hbmIcon;           /* Background preservation bitmap   */

/**********************************************************************/
/* Global handles for Arrow Bitmaps                                   */
/**********************************************************************/
extern HBITMAP hbmVIncr;
extern HBITMAP hbmVDecr;
extern HBITMAP hbmHIncr;
extern HBITMAP hbmHDecr;
extern HBITMAP hbmVIncrS;
extern HBITMAP hbmVDecrS;
extern HBITMAP hbmHIncrS;
extern HBITMAP hbmHDecrS;
extern HBITMAP hbmVIncrD;
extern HBITMAP hbmVDecrD;
extern HBITMAP hbmHIncrD;
extern HBITMAP hbmHDecrD;
extern HBITMAP hbmVIncrSD;
extern HBITMAP hbmVDecrSD;
extern HBITMAP hbmHIncrSD;
extern HBITMAP hbmHDecrSD;
extern HDC     hpsMem;
extern WORD    wUseCount;


WORD    slideCount = 0;

BOOL     vfBitmapsCreated = FALSE;
PTHDATA  vpthRoot = {0};

/**********************************************************************/
/* Procedure: CreateSliderBitMaps - This function is called when the  */
/*            slider window is created.  This routine creates and     */
/*            loads all of the bitmaps needed for use by the slider   */
/*            control for drawing the slider buttons and for moving   */
/*            the slider arm.                                         */
/*                                                                    */
/* Inputs: pth - pointer to the task handle node that the DC will be  */
/*               created for                                          */
/*                                                                    */
/* Output: If TRUE, bitmaps created successfully                      */
/*         if FALSE, an error occurred                                */
/**********************************************************************/
BOOL CreateSliderBitmaps( PTHDATA pth )
{
  BITMAP   bitmap;            /* Bitmap information used for create   */

  /********************************************************************/
  /* Create a new device context and presentation space to be used    */
  /* for the creation of the bitmaps                                  */
  /********************************************************************/
  pth->hdcMem = CreateCompatibleDC(NULL);
  /*------------------------------------------------------------*/
  /* remember the new memory DC for this task in the PTHDATA    */
  /*------------------------------------------------------------*/
  hpsMem = pth->hdcMem;

  /*------------------------------------------------*/
  /* if we've already created the bitmaps, we don't */
  /* need to do anything else, so just leave        */
  /*------------------------------------------------*/
  if (vfBitmapsCreated)
  {
    return (TRUE);
  }

  /********************************************************************/
  /* Load the slider bitmaps used for the arrow buttons               */
  /*   A different bitmap is used for the increment and decrement     */
  /*   buttons in both vertical and horizontal orientation - If the   */
  /*   shaft is very narrow, a small button arrow is used             */
  /********************************************************************/
  hbmHDecr  = LoadBitmap( hSliderInstance, MAKEINTRESOURCE(IDB_HORZ_DEC) );/*JEH1*/
  hbmHIncr  = LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_INC" );
  hbmVDecr  = LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_DEC" );
  hbmVIncr  = LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_INC" );
  hbmHDecrS = LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_DEC_S" );
  hbmHIncrS = LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_INC_S" );
  hbmVDecrS = LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_DEC_S" );
  hbmVIncrS = LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_INC_S" );
  hbmHDecrD = LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_DEC_D" );
  hbmHIncrD = LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_INC_D" );
  hbmVDecrD = LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_DEC_D" );
  hbmVIncrD = LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_INC_D" );
  hbmHDecrSD= LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_DEC_SD" );
  hbmHIncrSD= LoadBitmap( hSliderInstance, (LPSTR)"IDB_HORZ_INC_SD" );
  hbmVDecrSD= LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_DEC_SD" );
  hbmVIncrSD= LoadBitmap( hSliderInstance, (LPSTR)"IDB_VERT_INC_SD" );

  /********************************************************************/
  /* Get bitmap parameters of one of the bitmaps for use in creation  */
  /********************************************************************/
  GetObject(hbmHIncr, sizeof(BITMAP), (LPSTR)&bitmap);

  /********************************************************************/
  /* Default the dimensions of the bitmap to 2 times the button size  */
  /* If this is not big enough, it will be changed later.             */
  /********************************************************************/
  bitmap.bmWidth*=2;
  bitmap.bmHeight*=2;

  /********************************************************************/
  /* Create two bitmaps for use in blit routines in moving the arm    */
  /********************************************************************/
  hbmIcon    = CreateBitmap(bitmap.bmWidth, bitmap.bmHeight,
               bitmap.bmPlanes, bitmap.bmBitsPixel, (LPSTR)0L);
  hbmScrSave = CreateBitmap(bitmap.bmWidth, bitmap.bmHeight,
               bitmap.bmPlanes, bitmap.bmBitsPixel, (LPSTR)0L);

  /********************************************************************/
  /* If any of the bitmaps was not created, then delete all bitmaps   */
  /* that were created and terminate slider                           */
  /********************************************************************/
  if (!hbmHIncr   || !hbmVIncr   || !hbmHDecr   || !hbmVDecr   ||
      !hbmHIncrS  || !hbmVIncrS  || !hbmHDecrS  || !hbmVDecrS  ||
      !hbmHIncrD  || !hbmVIncrD  || !hbmHDecrD  || !hbmVDecrD  ||
      !hbmHIncrSD || !hbmVIncrSD || !hbmHDecrSD || !hbmVDecrSD ||
      !hbmIcon    || !hbmScrSave)
  {
    /******************************************************************/
    /* Delete only those bitmaps which were created                   */
    /******************************************************************/
    DeleteSliderBitmaps();
    return (FALSE);
  }

  vfBitmapsCreated = TRUE;

  return (TRUE);
}

/**********************************************************************/
/* Procedure: DeleteSliderBitmaps - This function is called to delete */
/*            all of the bitmap handles when the slider is destroyed  */
/*                                                                    */
/* Inputs: None                                                       */
/*                                                                    */
/* Output: None                                                       */
/**********************************************************************/
VOID DeleteSliderBitmaps (void)
{
  /********************************************************************/
  /* Check each bitmap to see if it exists and if so, delete it       */
  /********************************************************************/
  if (hbmHIncr) DeleteObject(hbmHIncr);
  if (hbmVIncr) DeleteObject(hbmVIncr);
  if (hbmHDecr) DeleteObject(hbmHDecr);
  if (hbmVDecr) DeleteObject(hbmVDecr);
  if (hbmHIncrS) DeleteObject(hbmHIncrS);
  if (hbmVIncrS) DeleteObject(hbmVIncrS);
  if (hbmHDecrS) DeleteObject(hbmHDecrS);
  if (hbmVDecrS) DeleteObject(hbmVDecrS);
  if (hbmHIncrD) DeleteObject(hbmHIncrD);
  if (hbmVIncrD) DeleteObject(hbmVIncrD);
  if (hbmHDecrD) DeleteObject(hbmHDecrD);
  if (hbmVDecrD) DeleteObject(hbmVDecrD);
  if (hbmHIncrSD) DeleteObject(hbmHIncrSD);
  if (hbmVIncrSD) DeleteObject(hbmVIncrSD);
  if (hbmHDecrSD) DeleteObject(hbmHDecrSD);
  if (hbmVDecrSD) DeleteObject(hbmVDecrSD);
  if (hbmScrSave) DeleteObject(hbmScrSave);
  if (hbmIcon) DeleteObject(hbmIcon);
  vfBitmapsCreated = FALSE;
  return;
}

/**********************************************************************/
/* Procedure: AddTaskNode - This routine will add a node to the linked*/
/*            list of task handles for the tasks which are using the  */
/*            slider.                                                 */
/*                                                                    */
/* Output: If TRUE, node was added successfully                       */
/*         if FALSE, error occurred                                   */
/**********************************************************************/
BOOL AddTaskNode (void)
{
  PTHDATA pthNew;

  /*--------------------------------------*/
  /* if we've already allocated a THDATA  */
  /* for this task, don't do it again     */
  /*--------------------------------------*/
  if (FindTaskData())   return (TRUE);

  /*-----------------------------------------------------------------*/
  /* allocate a block from the heap; if we couldn't, return an error */
  /*-----------------------------------------------------------------*/
  if (!(pthNew =
      (PTHDATA) LocalAlloc (LMEM_ZEROINIT, sizeof(THDATA))))
    return (FALSE);

  /*------------------------------------------------------------*/
  /* create the device context and bitmaps that will be used by */
  /* all sliders created by this task                           */
  /*------------------------------------------------------------*/
  if (!CreateSliderBitmaps (pthNew))
  {
    LocalFree ((HANDLE) pthNew);
    return (FALSE);
  }

  /*---------------------------------------------*/
  /* insert the new node at the head of the list */
  /*---------------------------------------------*/
  pthNew->pthPrev = NULL;
  pthNew->pthNext = vpthRoot;
  if (vpthRoot) vpthRoot->pthPrev = pthNew;
  vpthRoot = pthNew;

  /*------------------*/
  /* remember the TH  */
  /*------------------*/
  pthNew->hCurrentTask = GetCurrentProcess ();                          /*JEH1*/

  return (TRUE);
}

/**********************************************************************/
/* Procedure: DeleteTaskNode - This function is called from in the    */
/*            exit routine for the slider when all of the slider      */
/*            windows for that task have been destroyed.  This        */
/*            will remove the task handle node since it is no longer  */
/*            needed and free all memory associated with it.          */
/*                                                                    */
/* Output: If TRUE, node was deleted successfully                     */
/*         if FALSE, error occurred                                   */
/**********************************************************************/
VOID DeleteTaskNode (void)
{
  PTHDATA pthT;

  /*------------------------------------------------------------------*/
  /* find the THDATA for this process, if we can't don't do anything  */
  /*------------------------------------------------------------------*/
  if (pthT = FindTaskData ())
  {
    /*-------------------------------------------------------*/
    /* remove the link from the chain and free the resources */
    /*-------------------------------------------------------*/
    if (pthT->pthNext)     pthT->pthNext->pthPrev = pthT->pthPrev;
    if (pthT->pthPrev)     pthT->pthPrev->pthNext = pthT->pthNext;
    if (vpthRoot == pthT)  vpthRoot = pthT->pthNext;

    if (!--wUseCount)
       DeleteDC (pthT->hdcMem);
    LocalFree ((HANDLE) pthT);
  }
}


/**********************************************************************/
/* Procedure: FindTaskData - This function is called to find the      */
/*            PTHDATA structure corresponding to the current process. */
/*                                                                    */
/* Inputs: None.                                                      */
/*                                                                    */
/* Output: pth - matching PTHDATA        NULL if not found            */
/**********************************************************************/
PTHDATA FindTaskData (void)
{
  PTHDATA  pthT;
  HANDLE   th = GetCurrentProcess ();                                   /*JEH1*/

  for (pthT=vpthRoot;
      pthT && (pthT->hCurrentTask != th);
      pthT=pthT->pthNext);

  return (pthT);
}

