/* Revision: 63 1.4 source/ui/cclnt/slider/sliders.c, cclslid, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/*                           S L I D E R                              */
/*                                                                    */
/*                       Filename: SLIDERS.C                          */
/*                                                                    */
/*              ( Windows NT Slider Control Component )               */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1991                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: SLIDER.DLL                                                */
/* Filename: SLIDERS.C - Subroutines for the Slider control           */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* JEH1  06/21/93  JEH  Initial modifications for NT execution        */
/* DAB1  09/04/94  DAB  Updates to slider for dynamic setting/querying*/
/*                       tick information                             */
/* EVB1  10/26/94  EVB  Add SLDQueryPresParam for NT                  */
/**********************************************************************/
#include <string.h>

#include <windows.h>
#include "iclsldw.h"               /* Slider control public includes  */
#include "sliderp.h"               /* Slider control private includes */

/**********************************************************************/
/* Procedure: AddDetent - This function will add a detent linked list */
/*            node into the current linked list.  The list will be    */
/*            ordered by position from home (regardless of scale).    */
/*            It will return a pointer to this new node.              */
/*                                                                    */
/* Inputs: hwndSld     - handle of the slider window                  */
/*         pIntData    - Pointer to the slider internal control block */
/*         usDetentPos - position from home to draw detent at         */
/*                                                                    */
/* Output: If NULL, error on adding detent node                       */
/*         Otherwise returns pointer to new detent node (unique id)   */
/**********************************************************************/
PDETENTDATA AddDetent( HANDLE hwndSld, PISLDDATA pIntData,
                       USHORT usDetentPos )                             /*JEH1*/
{
  PDETENTDATA pDetentAdded=NULL;
  PDETENTDATA pDetentNext;
  ULONG       ulStyle;                                                  /*JEH1*/
  USHORT      usMaxRange;                                               /*JEH1*/

  /********************************************************************/
  /* Query for the maximum range for the primary scale on the slider  */
  /********************************************************************/
  usMaxRange = QueryMaxRange( hwndSld, pIntData );

  /********************************************************************/
  /* Check if position passed is within range of 0 to max scale range */
  /********************************************************************/
  if ( usDetentPos > usMaxRange )
  {
    /******************************************************************/
    /* If not, parameter out of range so set error and return error   */
    /******************************************************************/
    return ( NULL );
  }

  /********************************************************************/
  /* Allocate memory from the heap for detent node being added        */
  /********************************************************************/
  if (!(pDetentAdded = (PDETENTDATA)LocalAlloc( LMEM_FIXED | LMEM_ZEROINIT,
                                                sizeof(DETENTDATA))))
    return ( NULL );

  /********************************************************************/
  /* First check if the detent list is non-null                       */
  /********************************************************************/
  if ( pDetentNext = pIntData->pDetentData )
  {
    /******************************************************************/
    /* If non null, then check first item for position                */
    /******************************************************************/
    if ( pDetentNext->usDetentPos < usDetentPos )
    {
      /****************************************************************/
      /* If above first item, traverse list for each non-null item    */
      /****************************************************************/
      while ( pDetentNext->pNext )
      {
        /**************************************************************/
        /* Check if position of this detent is greater than one to add*/
        /* and if so stop searching through list                      */
        /**************************************************************/
        if (pDetentNext->pNext->usDetentPos > usDetentPos )
          break;

        /**************************************************************/
        /* otherwise goto next item & continue searching for position */
        /**************************************************************/
        pDetentNext = pDetentNext->pNext;
      }
      /****************************************************************/
      /* If here then we either reached end of list (so insert at end)*/
      /* or we found the node to insert after.  Save node pointed to  */
      /* by previous node as next node and make the new node the next */
      /****************************************************************/
      pDetentAdded->pNext = pDetentNext->pNext;
      pDetentNext->pNext = pDetentAdded;
    }
    else
    {
      /****************************************************************/
      /* This comes before the first node, so save pointer to first   */
      /* node as next node after node inserted, and make new first    */
      /* node the new one created                                     */
      /****************************************************************/
      pDetentAdded->pNext = pDetentNext;
      pIntData->pDetentData = pDetentAdded;
    }
  }
  /********************************************************************/
  /* Otherwise the list is null, so make this the first node in list  */
  /********************************************************************/
  else
    pIntData->pDetentData = pDetentAdded;

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Save the scale which is the current primary scale                */
  /********************************************************************/
  if (ulStyle & SLS_PRIMARYSCALE2)
    pDetentAdded->usScale = SMA_SCALE2;
  else
    pDetentAdded->usScale = SMA_SCALE1;

  /********************************************************************/
  /* Store the position on the scale for this detent                  */
  /********************************************************************/
  pDetentAdded->usDetentPos = usDetentPos;

  /********************************************************************/
  /* Finally invalidate control to update with new detent             */
  /********************************************************************/
  InvalidateRect( hwndSld, NULL, FALSE );

  /********************************************************************/
  /* Return pointer to the new node added (this is the unique id)     */
  /********************************************************************/
  return (pDetentAdded);
}


/**********************************************************************/
/* Procedure: AddTick - This function will add a tick linked list node*/
/*            to end of the current linked list.  It will return a    */
/*            pointer to this new node.                               */
/*                                                                    */
/* Inputs: pIntData   - Pointer to the slider internal control block  */
/*                                                                    */
/* Output: If NULL, error on adding tick node                         */
/*         Otherwise returns pointer to new tick node.                */
/**********************************************************************/
PTICKDATA AddTick( PISLDDATA pIntData )
{
  PTICKDATA pTickAdded=NULL;
  PTICKDATA pTickLoc;
  PTICKDATA pTickNext;

  /********************************************************************/
  /* Allocate memory from the heap for tick node being added          */
  /********************************************************************/
  if (!(pTickAdded = (PTICKDATA)LocalAlloc( LMEM_FIXED | LMEM_ZEROINIT,
                                            sizeof(TICKDATA))))
    return ( NULL );

  /********************************************************************/
  /* First check if the list is non-null                              */
  /********************************************************************/
  if ( pTickNext = pIntData->pTickData )
  {
    /******************************************************************/
    /* If not null, then traverse list until end of string            */
    /******************************************************************/
    while ( pTickLoc = pTickNext->pNext )
    {
      pTickNext = pTickLoc;
    }

    /******************************************************************/
    /* pTickNext points to last node in tick data list so add this    */
    /* as next node in the list after this one                        */
    /******************************************************************/
    pTickNext->pNext = pTickAdded;

  }
  /********************************************************************/
  /* Otherwise the list is null, so make this the first node in list  */
  /********************************************************************/
  else
    pIntData->pTickData = pTickAdded;

  /********************************************************************/
  /* Return pointer to the new node added                             */
  /********************************************************************/
  return (pTickAdded);
}


/**********************************************************************/
/* Procedure: QueryClosestTick - This function will return the        */
/*            tick mark for the current scale which is closest to     */
/*            the current position.  Returns 0 if invalid scale.      */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usArmPos   - Position to query closest tick from           */
/*                                                                    */
/* Output: usTickNum  - This is the tick number closest to the current*/
/*                      arm position.                                 */
/*         usArmPos   - This is the arm position at that tick number  */
/**********************************************************************/
ULONG QueryClosestTick( HANDLE hwndSld, PISLDDATA pIntData,             /*JEH1*/
                        USHORT usArmPos )                               /*JEH1*/
{
  USHORT usSpacing;                                                     /*JEH1*/
  USHORT usTickNum=0;                                                   /*JEH1*/

  /********************************************************************/
  /* Get the spacing between increments for the current scale         */
  /********************************************************************/
  usSpacing = QueryScaleSpacing( hwndSld, pIntData );

  /********************************************************************/
  /* Calculate which increment on the current scale is the            */
  /* closest to the current position                                  */
  /********************************************************************/
  if (usSpacing)
    usTickNum = (usArmPos+(usSpacing>>1)) / usSpacing;

  /********************************************************************/
  /* Return the closest tick number plus the arm position at that tick*/
  /********************************************************************/
  return ( MAKELONG( usTickNum, usTickNum * usSpacing ));
}


/**********************************************************************/
/* Procedure: QueryDetentPos - This function will return the position */
/*            of the unique detent id requested.                      */
/*                                                                    */
/* Inputs: pIntData  - Pointer to the slider internal control block   */
/*         pDetentId - This is the unique detent id to remove         */
/*                                                                    */
/* Output: usDetentPos - This is the number of pixels from home that  */
/*                       the detent specified is located.             */
/**********************************************************************/
LONG QueryDetentPos( PISLDDATA pIntData, PDETENTDATA pDetentId )        /*JEH1*/
{
  PDETENTDATA pDetentLoc=NULL;

  /********************************************************************/
  /* First check if detents exist and a non-null ID was passed        */
  /********************************************************************/
  if (( pDetentLoc = pIntData->pDetentData ) && pDetentId )
  {
    /******************************************************************/
    /* If so, then scan through list for this detent to check validity*/
    /******************************************************************/
    do
    {
      /****************************************************************/
      /* Each detent in the list should be checked for a match        */
      /****************************************************************/
      if ( pDetentLoc == pDetentId )
      {
        /**************************************************************/
        /* If match, return the position and the scale it is on       */
        /**************************************************************/
        return ( MAKELONG( pDetentId->usDetentPos, pDetentId->usScale));

      }
    /******************************************************************/
    /* otherwise go to next detent in list (if any)                   */
    /******************************************************************/
    } while ( pDetentLoc = pDetentLoc->pNext);
  }
  /********************************************************************/
  /* If not in list, no detents, or a null id, return an error        */
  /********************************************************************/
  return (SLDERR_INVALID_PARAMETERS);
}


/**********************************************************************/
/* Procedure: QueryMaxRange - This function is called to query the    */
/*            maximum value the arm can be moved on the current scale.*/
/*            This is governed by the minimum of either the farthest  */
/*            tick on the current scale, or by the end of the shaft   */
/*            which is based off of the primary scale set at creation.*/
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*                                                                    */
/* Output: usMaxRange - This is maximum value the arm can be set to   */
/**********************************************************************/
USHORT QueryMaxRange( HANDLE hwndSld, PISLDDATA pIntData )              /*JEH1*/
{
  ULONG  ulStyle;                                                       /*JEH1*/
  USHORT usMaxRange;                                                    /*JEH1*/
  USHORT usMaxRange1;                                                   /*JEH1*/
  USHORT usMaxRange2;                                                   /*JEH1*/

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Calculate maximum position arm can move to for scale 1           */
  /* and 2 which is maximum # of units - 1 (since 0 based)            */
  /********************************************************************/
  usMaxRange1 = ((pIntData->usScale1Increments - 1) * pIntData->
                 usScale1Spacing);
  usMaxRange2 = ((pIntData->usScale2Increments - 1) * pIntData->
                usScale2Spacing);

  /********************************************************************/
  /* Set maximum range for slider based on current scale              */
  /********************************************************************/
  if (ulStyle & SLS_PRIMARYSCALE2)
  {
    /******************************************************************/
    /* If scale 2 and it was used for sizing shaft, use its max       */
    /******************************************************************/
    if (pIntData->usStatusSld & FL_SLDINITIALSCALE2)
      usMaxRange = usMaxRange2;

    /******************************************************************/
    /* otherwise scale 1 was used to size shaft, so use smaller       */
    /* of two maximum extents (to prevent drawing past end)           */
    /******************************************************************/
    else
      usMaxRange = min( usMaxRange1, usMaxRange2 );
  }
  /********************************************************************/
  /* otherwise scale 1 is current scale                               */
  /********************************************************************/
  else
  {
    /******************************************************************/
    /* If scale 1 and it wasn't used to size shaft, use the smaller   */
    /* of the two maximum extents (to prevent drawing past end)       */
    /******************************************************************/
    if (pIntData->usStatusSld & FL_SLDINITIALSCALE2)
      usMaxRange = min( usMaxRange1, usMaxRange2 );

    /******************************************************************/
    /* otherwise scale 1 was used to size shaft so use its max        */
    /******************************************************************/
    else
      usMaxRange = usMaxRange1;
  }
  return (usMaxRange);
}


/**********************************************************************/
/* Procedure: QueryScaleSpacing - This function is called to query    */
/*            the spacing set for the current scale.                  */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/**********************************************************************/
USHORT QueryScaleSpacing( HANDLE hwndSld, PISLDDATA pIntData )          /*JEH1*/
{
  ULONG  ulStyle;                                                       /*JEH1*/
  USHORT usSpacing=0;                                                   /*JEH1*/

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Get the spacing between increments for the current scale         */
  /********************************************************************/
  if (ulStyle & SLS_PRIMARYSCALE2)

    /******************************************************************/
    /* If scale 2, use scale 2 spacing for calculating increment      */
    /******************************************************************/
    usSpacing  = pIntData->usScale2Spacing;

  else
    /******************************************************************/
    /* otherwise use scale 1 spacing for calculating increment        */
    /******************************************************************/
    usSpacing  = pIntData->usScale1Spacing;

  /********************************************************************/
  /* Return spacing value for current scale                           */
  /********************************************************************/
  return (usSpacing);
}


/**********************************************************************/
/* Procedure: QueryScaleText - This function is called to query the   */
/*            text which is displayed above a tick on the current     */
/*            scale.                                                  */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usTickNum  - This specifies the tick location to query     */
/*         usBufLen   - This is the size of the buffer pointed to     */
/*         pszTickText - pointer to buffer to copy text into          */
/*                                                                    */
/* Output: If >= 0, text was queried successfully                     */
/*         If SLDERR_INVALID_PARAMETERS, error getting text           */
/**********************************************************************/
LONG   QueryScaleText( HANDLE hwndSld, PISLDDATA pIntData,              /*JEH1*/
                       USHORT usTickNum, USHORT usBufLen,               /*JEH1*/
                       LPSTR pszTickText )
{
  PTICKDATA pTickLoc=NULL;

  /********************************************************************/
  /* Check the validity of the tick number passed in                  */
  /********************************************************************/
  if (!VerifyTickNum( hwndSld, pIntData, usTickNum, TRUE, 0 ))          /*JEH1*/

    /******************************************************************/
    /* If invalid tick, return an error                               */
    /******************************************************************/
    return ( SLDERR_INVALID_PARAMETERS );

  /********************************************************************/
  /* Search the tick linked list for this tick node                   */
  /********************************************************************/
  if ( !(pTickLoc = SearchForTick( hwndSld, pIntData, usTickNum )))
  {
    /******************************************************************/
    /* If no such tick, return an error                               */
    /******************************************************************/
    return ( SLDERR_INVALID_PARAMETERS );
  }
  else
  {
    /******************************************************************/
    /* Check if a text string exists at this tick                     */
    /******************************************************************/
    if ( pTickLoc->pszText )
    {
      /****************************************************************/
      /* If a valid string exists, check if a buffer of 0 was set     */
      /* which makes this a request for the string length             */
      /****************************************************************/
      if ( !usBufLen )
      {
        /**************************************************************/
        /* If so, then return the length of the item string           */
        /**************************************************************/
        return ( lstrlen( pTickLoc->pszText ));
      }
      /****************************************************************/
      /* If not a request for length, check if a valid pointer to a   */
      /* string buffer was passed.                                    */
      /****************************************************************/
      else if (pszTickText)
      {
        /**************************************************************/
        /* If so, then copy the item string to the provided string    */
        /* and place null at end of buffer since strncpy may not have */
        /**************************************************************/
        strncpy( pszTickText, pTickLoc->pszText, usBufLen-1 );
        pszTickText[usBufLen-1] = '\0';

        /**************************************************************/
        /* and return the length of the copied item string            */
        /**************************************************************/
        return ( lstrlen( pszTickText ));
      }
      else
      {
        /**************************************************************/
        /* An invalid pointer passed, set error information and return*/
        /**************************************************************/
        return (SLDERR_INVALID_PARAMETERS);
      }
    }
    /******************************************************************/
    /* Otherwise if no text, return NULL (0 length string)            */
    /******************************************************************/
    else
      return (0);                                                       /*JEH1*/
  }
}


/**********************************************************************/
/* Procedure: QuerySliderInfo - This function will return the         */
/*            information requested from the slider internal control  */
/*            block.  Any information which must be calculated will   */
/*            be done to obtain the desired result.                   */
/*                                                                    */
/* Inputs: hwndSld     - handle of the slider window                  */
/*         pIntData    - Pointer to the slider internal control block */
/*         usInfoType  - type of information to be returned           */
/*         usArmPosType - format of information for arm position      */
/*                        (only used if SMA_SLIDERARMPOSITION above)  */
/*                                                                    */
/* Output: ulInfo - information requested                             */
/*         If SLDERR_INVALID_PARAMETERS, invalid parameters passed    */
/**********************************************************************/
LONG QuerySliderInfo( HANDLE hwndSld, PISLDDATA pIntData,               /*JEH1*/
                       USHORT usInfoType, USHORT usArmPosType )         /*JEH1*/
{
  /********************************************************************/
  /* Check to see which parameter is queried for                      */
  /********************************************************************/
  switch ( usInfoType )
  {
    /******************************************************************/
    /* Check if the shaft dimensions are to be returned               */
    /******************************************************************/
    case SMA_SHAFTDIMENSIONS:

      /****************************************************************/
      /* If so, get the values from internal data and return them     */
      /****************************************************************/
      return ( MAKELONG( pIntData->usShaftLength,
                         pIntData->usShaftBreadth ));

    /******************************************************************/
    /* Check if the shaft position is to be returned                  */
    /******************************************************************/
    case SMA_SHAFTPOSITION:

      /****************************************************************/
      /* If so, get the values from internal data and return them     */
      /****************************************************************/
      return ( MAKELONG( pIntData->xShaft, pIntData->yShaft ));

    /******************************************************************/
    /* Check if the slider arm dimensions are to be returned          */
    /******************************************************************/
    case SMA_SLIDERARMDIMENSIONS:

      /****************************************************************/
      /* If so, get the values from internal data and return them     */
      /****************************************************************/
      return ( MAKELONG( pIntData->usArmLength,
                         pIntData->usArmBreadth ));

    /******************************************************************/
    /* Check if the slider arm position is to be returned             */
    /******************************************************************/
    case SMA_SLIDERARMPOSITION:

      /****************************************************************/
      /* Check to see which units are to be used for the position     */
      /****************************************************************/
      if ( usArmPosType == SMA_RANGEVALUE )
      {
        USHORT usMaxRange=0;                                            /*JEH1*/

        /**************************************************************/
        /* Calculate maximum number of positions on current scale     */
        /* which is the maximum position on slider plus one (for 0)   */
        /**************************************************************/
        usMaxRange = QueryMaxRange( hwndSld, pIntData ) + 1;

        /**************************************************************/
        /* Return the current position and the maximum number of units*/
        /**************************************************************/
        return ( MAKELONG( pIntData->usArmPos, usMaxRange ));
      }
      /****************************************************************/
      /* If units are in increments, then calculate as such           */
      /****************************************************************/
      else if ( usArmPosType == SMA_INCREMENTVALUE )
      {
        return ( MAKELONG( LOWORD ( QueryClosestTick( hwndSld,
                                  pIntData, pIntData->usArmPos )), 0 ));
      }
      /****************************************************************/
      /* otherwise invalid arm pos type parameter, so set error       */
      /****************************************************************/
      else
      {
        /**************************************************************/
        /* Set error since invalid parameter and return error flag    */
        /**************************************************************/
        return ( SLDERR_INVALID_PARAMETERS );
      }

    /******************************************************************/
    /* Check if the slider tick counts are to be returned             */
    /******************************************************************/
    case SMA_SLIDERSCALEINFO:                                           /*DAB1*/

      /****************************************************************/
      /* Check to see which information is be returned                */
      /****************************************************************/
      if ( usArmPosType == SMA_SLIDERTICKCOUNT )                        /*DAB1*/

        /**************************************************************/
        /* If so, get the values from internal data and return them   */
        /**************************************************************/
        return ( MAKELONG( pIntData->usScale1Increments,                /*DAB1*/
                           pIntData->usScale2Increments ));             /*DAB1*/

      /****************************************************************/
      /* Check if the slider tick spacings are to be returned         */
      /****************************************************************/
      else if ( usArmPosType == SMA_SLIDERTICKSPACING )                 /*DAB1*/

        /**************************************************************/
        /* If so, get the values from internal data and return them   */
        /**************************************************************/
        return ( MAKELONG( pIntData->usScale1Spacing,                   /*DAB1*/
                           pIntData->usScale2Spacing ));                /*DAB1*/

      /****************************************************************/
      /* otherwise invalid tick info type requested, so set error     */
      /****************************************************************/
      else                                                              /*DAB1*/
      {                                                                 /*DAB1*/
        /**************************************************************/
        /* Set error since invalid parameter and return error flag    */
        /**************************************************************/
        return ( SLDERR_INVALID_PARAMETERS );                           /*DAB1*/
      }                                                                 /*DAB1*/

    /******************************************************************/
    /* If none of the above, then an invalid request has been made    */
    /******************************************************************/
    default:

      /****************************************************************/
      /* Invalid Info type parameter, set error and return error      */
      /****************************************************************/
      return ( SLDERR_INVALID_PARAMETERS );
  }
}


/**********************************************************************/
/* Procedure: QueryTickPos - This function is called to query         */
/*            the position of a tick along the slider shaft for the   */
/*            current scale.  This is given as number of pixels from  */
/*            home and is the point of the tick closest to the shaft. */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usScale    - This specifies which scale the tick is on     */
/*         usTickNum  - This specifies the tick location to query     */
/*                      position of                                   */
/*                                                                    */
/* Output: x, y - x,y coordinates of tick mark along scale            */
/*                If NULL, invalid information passed                 */
/**********************************************************************/
ULONG QueryTickPos( HANDLE hwndSld, PISLDDATA pIntData, USHORT usScale, /*JEH1*/
                    USHORT usTickNum )                                  /*JEH1*/
{
  ULONG ulStyle;                                                        /*JEH1*/
  USHORT usSpacing;                                                     /*JEH1*/
  USHORT usTickSpace;                                                   /*JEH1*/
  USHORT usTickOddSpace;                                                /*JEH1*/
  USHORT usTickROSpace=0;                                               /*JEH1*/
  USHORT xTickPos=0;                                                    /*JEH1*/
  USHORT yTickPos=0;                                                    /*JEH1*/

  /********************************************************************/
  /* Check the validity of the tick number passed in                  */
  /********************************************************************/
  if (!VerifyTickNum( hwndSld, pIntData, usTickNum, FALSE, usScale ))

    /******************************************************************/
    /* If invalid tick, return an error                               */
    /******************************************************************/
    return ( 0 );                                                       /*JEH1*/

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Calculate spacing of ticks from shaft (just outside of arm)      */
  /* and check if there is an odd number in the pel differential      */
  /********************************************************************/
  usTickSpace = (( pIntData->usArmBreadth - pIntData->usShaftBreadth )
                   >> 1 );
  usTickOddSpace = ((pIntData->usArmBreadth - pIntData->usShaftBreadth)
                   & 1 );

  /********************************************************************/
  /* Set spacing offset if a one pel read only arm is in use - This is*/
  /* to adjust tick alignment since normal arm center is offset on arm*/
  /* and if only one pel wide, cannot draw with offset                */
  /********************************************************************/
  usTickROSpace = (pIntData->usArmLength == 1);

  /********************************************************************/
  /* Get the spacing between increments for the scale specified       */
  /********************************************************************/
  if (usScale == SMA_SCALE2)

    /******************************************************************/
    /* If scale 2, use scale 2 spacing for calculating increment      */
    /******************************************************************/
    usSpacing = pIntData->usScale2Spacing;

  else
    /******************************************************************/
    /* otherwise use scale 1 spacing for calculating increment        */
    /******************************************************************/
    usSpacing  = pIntData->usScale1Spacing;

  /********************************************************************/
  /* Check if vertical to determine which coordinate is which         */
  /********************************************************************/
  if (ulStyle & SLS_VERTICAL)
  {
    /******************************************************************/
    /* Calculate the y position from home on the slider               */
    /******************************************************************/
    if (ulStyle & SLS_HOMETOP)

      /****************************************************************/
      /* If home on top, calculate position as offset down from top   */
      /****************************************************************/
      yTickPos = pIntData->yShaft +
                 (BORDERWIDTH + (pIntData->usArmLength >> 1) +
                 usTickROSpace + (usTickNum * usSpacing));
    else
      /****************************************************************/
      /* otherwise home on bottom so calculate as offset up           */
      /****************************************************************/
      yTickPos = (pIntData->yShaft + pIntData->usShaftLength) -
                 (BORDERWIDTH + (pIntData-> usArmLength >> 1) +
                 (usTickNum * usSpacing));

    /******************************************************************/
    /* Check which scale this is on to determine x coordinate for tick*/
    /******************************************************************/
    if (usScale == SMA_SCALE2)

      /****************************************************************/
      /* If scale 2, x is calculated off of the left side             */
      /****************************************************************/
      xTickPos = pIntData->xShaft - (usTickSpace + 1);

    else
      /****************************************************************/
      /* otherwise scale 1 so it is calculated on the right side      */
      /* If odd differential, extra pel will be on right so add it in */
      /****************************************************************/
      xTickPos = pIntData->xShaft + pIntData->usShaftBreadth +
                 usTickSpace + usTickOddSpace;
  }
  /********************************************************************/
  /* Otherwise it is horizontal so calculate as such                  */
  /********************************************************************/
  else
  {
    /******************************************************************/
    /* Calculate the x position from home on the slider               */
    /******************************************************************/
    if (ulStyle & SLS_HOMERIGHT)

      /****************************************************************/
      /* If home to right, calculate position as offset from right    */
      /****************************************************************/
      xTickPos = (pIntData->xShaft + pIntData->usShaftLength) -
                 (BORDERWIDTH + (pIntData->usArmLength >> 1) + 1 +
                 (usTickNum * usSpacing));
    else
      /****************************************************************/
      /* otherwise home on left, so calculate as offset from left     */
      /****************************************************************/
      xTickPos = pIntData->xShaft + BORDERWIDTH + (pIntData->
                 usArmLength >> 1) - 1 + usTickROSpace +
                 (usTickNum * usSpacing);

    /******************************************************************/
    /* Check which scale this is on to determine y coordinate for tick*/
    /******************************************************************/
    if (usScale == SMA_SCALE2)

      /****************************************************************/
      /* If scale 2, y is calculated off of the bottom                */
      /****************************************************************/
      yTickPos = pIntData->yShaft + usTickSpace +
                 pIntData->usShaftBreadth;

    else
      /****************************************************************/
      /* otherwise scale 1 so it is calculated on the top             */
      /* If odd differential, extra pel will be on top so add it in   */
      /****************************************************************/
      yTickPos = pIntData->yShaft - ( usTickSpace + usTickOddSpace + 1);
  }
  /********************************************************************/
  /* Return the x and y position for the start of the tick            */
  /********************************************************************/
  return ( MAKELONG( xTickPos, yTickPos ));
}


/**********************************************************************/
/* Procedure: QueryTickSize - This function is called to query the    */
/*            size of a tick along the slider shaft for the current   */
/*            scale.                                                  */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usTickNum  - This specifies the tick location to set       */
/*                                                                    */
/* Output: usTickSize - Size of tick (in pixels)                      */
/*         If SLDERR_INVALID_PARAMETERS, invalid parameters passed    */
/**********************************************************************/
LONG QueryTickSize( HANDLE hwndSld, PISLDDATA pIntData,                 /*JEH1*/
                    USHORT usTickNum )                                  /*JEH1*/
{
  PTICKDATA pTickLoc=NULL;

  /********************************************************************/
  /* Check the validity of the tick number passed in                  */
  /********************************************************************/
  if (!VerifyTickNum( hwndSld, pIntData, usTickNum, TRUE, 0 ))          /*JEH1*/

    /******************************************************************/
    /* If invalid tick, return an error                               */
    /******************************************************************/
    return ( SLDERR_INVALID_PARAMETERS );

  /********************************************************************/
  /* Search the tick linked list for this tick node                   */
  /********************************************************************/
  if ( pTickLoc = SearchForTick( hwndSld, pIntData, usTickNum ))

    /******************************************************************/
    /* If found in list, return size of tick                          */
    /******************************************************************/
    return ( pTickLoc->usTickSize );

  /********************************************************************/
  /* otherwise if not in list, then tick not drawn (so size is 0)     */
  /********************************************************************/
  else
    return ( 0 );
}


/**********************************************************************/
/* Procedure: RemoveDetent - This function will traverse the detent   */
/*            detent linked list looking for a matching detent for    */
/*            the one passed.  If found, it will remove it from the   */
/*            list.                                                   */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usDetentId - This is a unique detent id to remove          */
/*                                                                    */
/* Output: TRUE, successfully found and removed                       */
/*         FALSE, could not remove because not found or bad id        */
/**********************************************************************/
BOOL RemoveDetent( HANDLE hwndSld, PISLDDATA pIntData,
                   PDETENTDATA pDetentId )
{
  PDETENTDATA pDetentLoc=NULL;

  /********************************************************************/
  /* First check if the list is non-null and a valid Id was passed    */
  /********************************************************************/
  if (( pDetentLoc = pIntData->pDetentData ) && pDetentId )
  {
    /******************************************************************/
    /* Check if the first node is the one to be removed               */
    /******************************************************************/
    if ( pIntData->pDetentData == pDetentId )
    {
      /****************************************************************/
      /* If so, set the new first node to the following node and exit */
      /****************************************************************/
      pIntData->pDetentData = pDetentId->pNext;

      /****************************************************************/
      /* Finally invalidate control to update w/out detent            */
      /****************************************************************/
      InvalidateRect( hwndSld, NULL, FALSE );
      return (TRUE);
    }

    /******************************************************************/
    /* Search each node for a detent which matches                    */
    /******************************************************************/
    do
    {
      /****************************************************************/
      /* Check if detent pointer matches the one to be removed        */
      /*  If so, pDetentLoc is pointing to previous detent            */
      /****************************************************************/
      if ( pDetentLoc->pNext == pDetentId )
      {
        /**************************************************************/
        /* Set previous detent next pointer to following node and exit*/
        /**************************************************************/
        pDetentLoc->pNext = pDetentId->pNext;

        /**************************************************************/
        /* Finally invalidate control to update w/out detent          */
        /**************************************************************/
        InvalidateRect( hwndSld, NULL, FALSE );
        return (TRUE);
      }
    /******************************************************************/
    /* Otherwise go to next detent in list if existing or exit if none*/
    /******************************************************************/
    } while ( pDetentLoc = pDetentLoc->pNext );
  }
  /********************************************************************/
  /* If no detent list, bad detent id, or not found in list set error */
  /* information and return False to show this                        */
  /********************************************************************/
  return ( FALSE );
}


/**********************************************************************/
/* Procedure: SearchForTick - This function will traverse the tick    */
/*            linked list looking for a matching tick on the current  */
/*            scale.                                                  */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usTickNum  - This specifies the tick location to set       */
/*                                                                    */
/* Output: If NULL, no match found                                    */
/*         Otherwise returns pointer to matching tick location        */
/**********************************************************************/
PTICKDATA SearchForTick( HANDLE hwndSld, PISLDDATA pIntData,
                         USHORT usTickNum )                             /*JEH1*/
{
  PTICKDATA pTickLoc=NULL;
  ULONG     ulStyle;                                                    /*JEH1*/
  USHORT    usScale;                                                    /*JEH1*/

  /********************************************************************/
  /* First check if the list is non-null                              */
  /********************************************************************/
  if ( pTickLoc = pIntData->pTickData )
  {
    /******************************************************************/
    /* Query for the style of the slider control                      */
    /******************************************************************/
    ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

    /******************************************************************/
    /* Save the scale which is the current primary scale              */
    /******************************************************************/
    if (ulStyle & SLS_PRIMARYSCALE2)
      usScale = SMA_SCALE2;
    else
      usScale = SMA_SCALE1;

    /******************************************************************/
    /* Search each node for a tick which matches                      */
    /******************************************************************/
    do
    {
      /****************************************************************/
      /* Check if tick number matches on current scale                */
      /*  If so, pTickLoc is pointing to matching tick                */
      /****************************************************************/
      if ( (pTickLoc->usTickNum == usTickNum) &&
           (pTickLoc->usScale == usScale))
        break;

    /******************************************************************/
    /* Otherwise go to next tick in list if existing or exit if none  */
    /******************************************************************/
    } while ( pTickLoc = pTickLoc->pNext );
  }
  /********************************************************************/
  /* Return matching tick location (or null if none exists)           */
  /********************************************************************/
  return ( pTickLoc );
}

/**********************************************************************/
/* Procedure: SetScaleText - This function is called to set text to   */
/*            be displayed above a tick on the current scale.         */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usTickNum  - This specifies the tick location to set       */
/*         pszTickText - pointer to buffer containing text to set     */
/*                                                                    */
/* Output: If TRUE, text was set successfully                         */
/*         If FALSE, error during set                                 */
/**********************************************************************/
BOOL SetScaleText( HANDLE hwndSld, PISLDDATA pIntData, USHORT usTickNum,/*JEH1*/
                   LPSTR pszTickText )
{
  PTICKDATA pTickLoc=NULL;
  ULONG     ulStyle;                                                    /*JEH1*/
  USHORT    usBufLen;                                                   /*JEH1*/

  /********************************************************************/
  /* Check the validity of the tick number passed in                  */
  /********************************************************************/
  if (!VerifyTickNum( hwndSld, pIntData, usTickNum, TRUE, 0 ))          /*JEH1*/

    /******************************************************************/
    /* If invalid tick, return an error                               */
    /******************************************************************/
    return ( FALSE );

  /********************************************************************/
  /* First check if the tick node exists                              */
  /********************************************************************/
  if ( !(pTickLoc = SearchForTick( hwndSld, pIntData, usTickNum )))
  {
    /******************************************************************/
    /* If it doesn't exist, then create one and add to list           */
    /******************************************************************/
    if ( !(pTickLoc = AddTick( pIntData )))
      return (FALSE);

    /******************************************************************/
    /* Place tick number into new tick node                           */
    /******************************************************************/
    pTickLoc->usTickNum = usTickNum;

    /******************************************************************/
    /* Query for the style of the slider control                      */
    /******************************************************************/
    ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

    /******************************************************************/
    /* Set scale on to current scale                                  */
    /******************************************************************/
    if (ulStyle & SLS_PRIMARYSCALE2)
      pTickLoc->usScale = SMA_SCALE2;
    else
      pTickLoc->usScale = SMA_SCALE1;
  }
  /********************************************************************/
  /* If tick exists, remove the current text and free the memory      */
  /********************************************************************/
  else
  {
    /******************************************************************/
    /* If tick text exists, free the text buffer                      */
    /******************************************************************/
    if ( pTickLoc->pszText )
      LocalFree( (HANDLE)pTickLoc->pszText );

    /******************************************************************/
    /* Null out the now invalid pointer                               */
    /******************************************************************/
    pTickLoc->pszText = NULL;
  }

  /********************************************************************/
  /* Check if a string was passed in                                  */
  /********************************************************************/
  if (pszTickText)
  {
    /******************************************************************/
    /* If so, get length of buffer to allocate for storing string     */
    /******************************************************************/
    usBufLen = lstrlen( pszTickText );

    /******************************************************************/
    /* Allocate memory for this string and place into node            */
    /******************************************************************/
    if (!(pTickLoc->pszText = (PBYTE)LocalAlloc(
                              LMEM_FIXED | LMEM_ZEROINIT, usBufLen+1 )))
      return ( FALSE );

    /******************************************************************/
    /* Then copy the string into the node                             */
    /******************************************************************/
    lstrcpy( pTickLoc->pszText, pszTickText );
  }

  /********************************************************************/
  /* Finally invalidate control to update with new detent             */
  /********************************************************************/
  InvalidateRect( hwndSld, NULL, FALSE );

  return (TRUE);
}


/**********************************************************************/
/* Procedure: SetSliderInfo - This function set specific slider       */
/*            parameters to the settings passed.  Values are stored   */
/*            in the slider internal control block.                   */
/*                                                                    */
/* Inputs: hwndSld     - handle of the slider window                  */
/*         pIntData    - Pointer to the slider internal control block */
/*         usInfoType  - parameter to be modified                     */
/*         usArmPosType - format of information for arm position      */
/*         ulInfo      - value of parameter to be set                 */
/*                                                                    */
/* Output: TRUE, successfully set                                     */
/*         FALSE, error in setting value                              */
/**********************************************************************/
BOOL SetSliderInfo( HANDLE hwndSld, PISLDDATA pIntData,                 /*JEH1*/
                    USHORT usInfoType,USHORT usArmPosType, ULONG ulInfo)/*JEH1*/
{
  USHORT usMaxRange;                                                    /*JEH1*/
  USHORT usSpacing;                                                     /*JEH1*/
  USHORT usNewArmPos;                                                   /*JEH1*/
  LONG   rc = 0;                                                        /*DAB1*/

  /********************************************************************/
  /* Before changing any value, check to insure drag is NOT in process*/
  /********************************************************************/
  if (pIntData->usStatusSld & FL_SLDDRAGGINGARM)
  {

    /******************************************************************/
    /* If drag in progress, don't change any values so set error      */
    /******************************************************************/
//  WinSetErrorInfo ((ULONG)PMERR_UPDATE_IN_PROGRESS, SEI_NOPROMPT);
    return (FALSE);
  }

  /********************************************************************/
  /* Check to see which parameter is to be changed                    */
  /********************************************************************/
  switch ( usInfoType )
  {
    /******************************************************************/
    /* Check if the shaft dimensions are to be set (only the breadth) */
    /******************************************************************/
    case SMA_SHAFTDIMENSIONS:

      /****************************************************************/
      /* Check to see if it is above the minimum size for a slider    */
      /****************************************************************/
      if ( LOWORD( ulInfo ) >= SLD_MINSHAFTBREADTH )
      {
        /**************************************************************/
        /* Set it if it is greater than or equal to the minimum height*/
        /**************************************************************/
        pIntData->usShaftBreadth = LOWORD( ulInfo );

        /**************************************************************/
        /* Now check to insure arm size is not less than shaft size   */
        /**************************************************************/
        if ( pIntData->usArmBreadth < pIntData->usShaftBreadth )

          /************************************************************/
          /* If so, then change arm size to be that of the shaft size */
          /* This must be done before shaft calculation is done       */
          /************************************************************/
          pIntData->usArmBreadth = pIntData->usShaftBreadth;

        /**************************************************************/
        /* Since the shaft breadth has changed, recalculate the shaft */
        /* size based on the number of increments and spacing         */
        /**************************************************************/
        CalcShaftSize( hwndSld, pIntData );

        /**************************************************************/
        /* Now that the shaft size has changed, recalculate the shaft */
        /* and possible button positions based on alignment and styles*/
        /**************************************************************/
        CalcShaftnButtonPos( hwndSld, pIntData );

        /**************************************************************/
        /* Now recalculate the arm rectangle with the new settings    */
        /* and recreate it because of a size change                   */
        /**************************************************************/
        CalcArmRectangle( hwndSld, pIntData,&(pIntData->rclSliderArm),
                          pIntData->usArmPos );
        pIntData->usStatusSld &= ~FL_SLDFOCUSCHANGE;
        CreateSliderArm( hwndSld, pIntData );

        /**************************************************************/
        /* Finally invalidate the entire control to update new setting*/
        /**************************************************************/
        InvalidateRect( hwndSld, NULL, FALSE );
      }

      /****************************************************************/
      /* Otherwise parameter is out of range so flag as error         */
      /****************************************************************/
      else
      {
        /**************************************************************/
        /* Parameter out of range so set error and return error flag  */
        /**************************************************************/
//      WinSetErrorInfo( (DWORD)PMERR_PARAMETER_OUT_OF_RANGE,
//                       SEI_NOPROMPT, NULL );
        return ( FALSE );
      }
      break;

    /******************************************************************/
    /* Check if the shaft position is to be changed within the slider */
    /* window (from its current position                              */
    /******************************************************************/
    case SMA_SHAFTPOSITION:

      /****************************************************************/
      /* Set the x,y position to that passed in                       */
      /****************************************************************/
      pIntData->xShaft = LOWORD( ulInfo );
      pIntData->yShaft = HIWORD( ulInfo );

      /****************************************************************/
      /* Set flag to show owner positioned it (don't change position) */
      /****************************************************************/
      pIntData->usStatusSld |= FL_SLDOWNERPOSITIONED;

      /****************************************************************/
      /* Now that the shaft position has changed, recalculate shaft   */
      /* and possible button positions based on alignment and styles  */
      /****************************************************************/
      CalcShaftnButtonPos( hwndSld, pIntData );

      /****************************************************************/
      /* Now recalculate the arm rectangle with the new settings      */
      /****************************************************************/
      CalcArmRectangle( hwndSld, pIntData, &(pIntData->rclSliderArm),
                        pIntData->usArmPos );

      /****************************************************************/
      /* Finally invalidate the entire control to update new setting  */
      /****************************************************************/
      InvalidateRect( hwndSld, NULL, FALSE );

      break;

    /******************************************************************/
    /* Check if the slider arm dimensions are to be changed           */
    /******************************************************************/
    case SMA_SLIDERARMDIMENSIONS:

      /****************************************************************/
      /* First check the width and height of the arm to be set        */
      /****************************************************************/
      if (( LOWORD( ulInfo ) >= SLD_MINARMLENGTH ) &&
          ( HIWORD( ulInfo ) >= pIntData->usShaftBreadth ))
      {
        /**************************************************************/
        /* Set it if it is greater than or equal to the minimum width */
        /* and height (height cannot be smaller than shaft height)    */
        /**************************************************************/
        pIntData->usArmBreadth = HIWORD( ulInfo );
        pIntData->usArmLength = LOWORD( ulInfo );

        /**************************************************************/
        /* Now that the arm length has changed, recalculate the shaft */
        /* size, taking the new arm length into account               */
        /**************************************************************/
        CalcShaftSize( hwndSld, pIntData );

        /**************************************************************/
        /* Now that the shaft length has changed, recalculate shaft   */
        /* and possible button positions based on alignment and styles*/
        /**************************************************************/
        CalcShaftnButtonPos( hwndSld, pIntData );

        /**************************************************************/
        /* Now recalculate the arm rectangle with the new settings    */
        /* and recreate it because of a size change                   */
        /**************************************************************/
        CalcArmRectangle( hwndSld, pIntData, &(pIntData->rclSliderArm),
                          pIntData->usArmPos );
        pIntData->usStatusSld &= ~FL_SLDFOCUSCHANGE;
        CreateSliderArm( hwndSld, pIntData );

        /**************************************************************/
        /* Finally invalidate the entire control to update new setting*/
        /**************************************************************/
        InvalidateRect( hwndSld, NULL, FALSE );
      }
      /****************************************************************/
      /* Otherwise parameter is out of range so flag as error         */
      /****************************************************************/
      else
      {
        /**************************************************************/
        /* Parameter out of range so set error and return error flag  */
        /**************************************************************/
//      WinSetErrorInfo( (DWORD)PMERR_PARAMETER_OUT_OF_RANGE,
//                       SEI_NOPROMPT, NULL );
        return ( FALSE );
      }
      break;

    /******************************************************************/
    /* Check if the slider arm position is to be moved                */
    /******************************************************************/
    case SMA_SLIDERARMPOSITION:

      /****************************************************************/
      /* Get spacing based on current scale                           */
      /****************************************************************/
      usSpacing = QueryScaleSpacing( hwndSld, pIntData );

      /****************************************************************/
      /* Query for the maximum range for the current slider           */
      /****************************************************************/
      usMaxRange = QueryMaxRange( hwndSld, pIntData );

      /****************************************************************/
      /* Check to see which units are to be used for the position     */
      /****************************************************************/
      if ( usArmPosType == SMA_RANGEVALUE )

        /**************************************************************/
        /* If range value passed, position is # of pixels from home   */
        /**************************************************************/
        usNewArmPos = LOWORD( ulInfo );

      /****************************************************************/
      /* If units are in increments, then calculate as such           */
      /****************************************************************/
      else if ( usArmPosType == SMA_INCREMENTVALUE )

        /**************************************************************/
        /* If increment value passed, position is increments from home*/
        /**************************************************************/
        usNewArmPos = usSpacing * LOWORD( ulInfo );

      /****************************************************************/
      /* otherwise invalid arm pos type parameter, so set error       */
      /****************************************************************/
      else
      {
        /**************************************************************/
        /* Set error since invalid parameter and return error flag    */
        /**************************************************************/
//      WinSetErrorInfo( (DWORD)PMERR_INVALID_PARAMETERS, SEI_NOPROMPT,
//                       NULL );
        return ( FALSE );
      }

      /****************************************************************/
      /* Check if the position passed is along the slider shaft       */
      /****************************************************************/
      if ( usNewArmPos <= usMaxRange )

        /**************************************************************/
        /* If in range, then set the new position of the arm          */
        /**************************************************************/
        MoveSliderArm( hwndSld, pIntData, usNewArmPos, TRUE, TRUE);

      /****************************************************************/
      /* Otherwise parameter is out of range so flag as error         */
      /****************************************************************/
      else
      {
        /**************************************************************/
        /* Parameter out of range so set error and return error flag  */
        /**************************************************************/
//      WinSetErrorInfo( (DWORD)PMERR_PARAMETER_OUT_OF_RANGE,
//                       SEI_NOPROMPT, NULL );
        return ( FALSE );
      }
      break;

    /******************************************************************//*DAB1*/
    /* Check if the slider scale information is to be changed         *//*DAB1*/
    /******************************************************************//*DAB1*/
    case SMA_SLIDERSCALEINFO:                                           /*DAB1*/

     rc = (SetupSliderFromCB( hwndSld, pIntData, (LPSLDCDATA)ulInfo ))  /*DAB1 MCS*/
            ? 0 : SLDERR_INVALID_PARAMETERS;                            /*DAB1*/
     return (rc);                                                       /*DAB1*/

    /******************************************************************/
    /* If none of the above, then an invalid request has been made    */
    /******************************************************************/
    default:

      /****************************************************************/
      /* Invalid Info type parameter, set error and return error      */
      /****************************************************************/
//    WinSetErrorInfo( (DWORD)PMERR_INVALID_PARAMETERS, SEI_NOPROMPT,
//                     NULL );
      return ( FALSE );
  }
  return ( TRUE );
}


/**********************************************************************/
/* Procedure: SetTickSize - This function is called to set the size   */
/*            of a tick along the slider shaft for the current scale. */
/*                                                                    */
/* Inputs: hwndSld    - handle of the slider window                   */
/*         pIntData   - Pointer to the slider internal control block  */
/*         usTickNum  - This specifies the tick location to set       */
/*                      If SMA_SETALLTICKS, it will set all ticks     */
/*                       on the current scale to the size specified   */
/*         usTickSize - Size of tick (in pixels)                      */
/*                                                                    */
/* Output: If TRUE, tick size was set successfully                    */
/*         If FALSE, error during set                                 */
/**********************************************************************/
BOOL SetTickSize( HANDLE hwndSld, PISLDDATA pIntData, USHORT usTickNum, /*JEH1*/
                  USHORT usTickSize )                                   /*JEH1*/
{
  PTICKDATA pTickLoc=NULL;
  ULONG     ulStyle;                                                    /*JEH1*/
  USHORT    usTickCur;                                                  /*JEH1*/
  USHORT    usFirstTick;                                                /*JEH1*/
  USHORT    usLastTick;                                                 /*JEH1*/
  BOOL      bNoTick;

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Check if all ticks are to be set and if so, set start and end pts*/
  /********************************************************************/
  if ( usTickNum == SMA_SETALLTICKS )
  {
    /******************************************************************/
    /* Set first tick to 0                                            */
    /******************************************************************/
    usFirstTick = 0;

    /******************************************************************/
    /* Check current scale and set last tick to that for current scale*/
    /* If set all ticks and no ticks on scale, return cause we're     */
    /* done                                                           */
    /******************************************************************/
    if (ulStyle & SLS_PRIMARYSCALE2)
    {
      if (pIntData->usScale2Increments)
        usLastTick = pIntData->usScale2Increments - 1;
      else
        return (TRUE);
    }
    else
    {
      if (pIntData->usScale1Increments)
        usLastTick = pIntData->usScale1Increments - 1;
      else
        return (TRUE);
    }
  }

  /********************************************************************/
  /* otherwise only 1 tick so check validity of tick number passed in */
  /********************************************************************/
  else if (!VerifyTickNum( hwndSld, pIntData, usTickNum, TRUE, 0 ))     /*JEH1*/

    /******************************************************************/
    /* If invalid tick, return an error                               */
    /******************************************************************/
    return ( FALSE );

  else
  {
    /******************************************************************/
    /* otherwise a valid tick so set start and end points to this     */
    /******************************************************************/
    usFirstTick = usTickNum;
    usLastTick = usTickNum;
  }

  /********************************************************************/
  /* For each tick to be set (one or more) do the following           */
  /********************************************************************/
  for ( usTickCur=usFirstTick; usTickCur <= usLastTick; usTickCur++ )
  {
    /******************************************************************/
    /* Reset no tick flag each time through loop (if needed)          */
    /******************************************************************/
    bNoTick = FALSE;

    /******************************************************************/
    /* First check if the tick node exists                            */
    /******************************************************************/
    if ( !(pTickLoc = SearchForTick( hwndSld, pIntData, usTickCur )))
    {
      /****************************************************************/
      /* Verify that a valid size is being specified.  If 0 passed,   */
      /* no tick exists so it is already 0, so set flag to do nothing */
      /****************************************************************/
      if (!usTickSize)

        /**************************************************************/
        /* Trying to zero non existent tick, set flag to skip set     */
        /**************************************************************/
        bNoTick = TRUE;

      /****************************************************************/
      /* If it doesn't exist, then create one and add to list         */
      /****************************************************************/
      if ( !(pTickLoc = AddTick( pIntData )))
        return (FALSE);

      /****************************************************************/
      /* Place tick number into new tick node                         */
      /****************************************************************/
      pTickLoc->usTickNum = usTickCur;

      /****************************************************************/
      /* Set scale on to current scale                                */
      /****************************************************************/
      if (ulStyle & SLS_PRIMARYSCALE2)
        pTickLoc->usScale = SMA_SCALE2;
      else
        pTickLoc->usScale = SMA_SCALE1;
    }
    /******************************************************************/
    /* Finally set the tick size to the new one passed in             */
    /******************************************************************/
    if (!bNoTick)
      pTickLoc->usTickSize = usTickSize;
  }

  /********************************************************************/
  /* Finally invalidate the entire control to update new tick marks   */
  /********************************************************************/
  InvalidateRect( hwndSld, NULL, FALSE );

  return (TRUE);
}


/**********************************************************************/
/* Procedure: VerifyTickNum - This function will verify that a tick   */
/*            number is valid for the current scale.                  */
/*                                                                    */
/* Inputs: hwndSld   - handle of the slider window                    */
/*         pIntData  - Pointer to the slider internal control block   */
/*         usTickNum - This specifies the tick number to verify       */
/*         bStyle    - If TRUE, use current style as scale indicator  */
/*                     If FALSE, use scale passed as scale indicator  */
/*         usScale   - This specifies which scale to verify tick on   */
/*                     if not the current scale (bStyle is FALSE)     */
/*                                                                    */
/* Output: If TRUE, valid tick number                                 */
/*         if FALSE, invalid tick number, error set                   */
/**********************************************************************/
BOOL VerifyTickNum( HANDLE hwndSld, PISLDDATA pIntData, USHORT usTickNum,/*JEH1*/
                    BOOL bStyle, USHORT usScale )                       /*JEH1*/
{
  ULONG     ulStyle;                                                    /*JEH1*/
  USHORT    usMaxTick;                                                  /*JEH1*/

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Check which scale is the current primary scale                   */
  /********************************************************************/
  if ( (bStyle && (ulStyle & SLS_PRIMARYSCALE2)) || (!bStyle &&
       (usScale == SMA_SCALE2)) )
    usMaxTick = pIntData->usScale2Increments - 1;
  else
    usMaxTick = pIntData->usScale1Increments - 1;

  /********************************************************************/
  /* Check if tick number is within range of 0 to max tick for scale  */
  /********************************************************************/
  if ( usTickNum > usMaxTick )
  {
    /******************************************************************/
    /* If not, parameter out of range so set error and return error   */
    /******************************************************************/
//  WinSetErrorInfo( (DWORD)PMERR_PARAMETER_OUT_OF_RANGE,
//                   SEI_NOPROMPT, NULL );
    return ( FALSE );
  }
  return (TRUE);
}

/**********************************************************************/
/* Procedure: CheckForEndOfScale - This routine will check if the     */
/*            current arm position is at the end of the scale (either */
/*            low or high).  If so, it will set a flag to denote      */
/*            which end we are at, otherwise it will reset the flags. */
/*                                                                    */
/* Inputs:  hwndSld    - Window handle of the slider control          */
/*          pIntData   - Pointer to the slider control data block     */
/*          usTickPos  - Tick number corresponding to arm position    */
/*                                                                    */
/* Output:  If TRUE, either end of scale reached or just left         */
/*          Otherwise in middle of scale and no button update needed  */
/*                                                                    */
/**********************************************************************/
BOOL CheckForEndOfScale (HWND hwndSld, PISLDDATA pIntData,
                         USHORT usTickPos)                              /*JEH1*/
{
  ULONG       ulStyle;                                                  /*JEH1*/
  USHORT        usMaxRange;                                             /*JEH1*/
  USHORT        usMaxTick;                                              /*JEH1*/
  USHORT        usOrigStatus;                                           /*JEH1*/
  USHORT        usLowFlag = FL_SLDARMATLOWEND;                          /*JEH1*/
  USHORT        usHighFlag = FL_SLDARMATHIGHEND;                        /*JEH1*/

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Query for the maximum range for the primary scale on the slider  */
  /* and reset both flags as default for starting check on limits     */
  /********************************************************************/
  usMaxRange = QueryMaxRange( hwndSld, pIntData );
  usOrigStatus = pIntData->usStatusSld;
  pIntData->usStatusSld &= ~(FL_SLDARMATLOWEND | FL_SLDARMATHIGHEND);

  /********************************************************************/
  /* If home is different, then setup flag equates swapped            */
  /********************************************************************/
  if ( ulStyle & SLS_HOMERIGHT )
  {
    usLowFlag  = FL_SLDARMATHIGHEND;
    usHighFlag = FL_SLDARMATLOWEND;
  }

  /********************************************************************/
  /* Check the position of the arm against the scale end limits       */
  /********************************************************************/
  if (ulStyle & SLS_SNAPTOINCREMENT)
  {
    /******************************************************************/
    /* If snap on, check tick position against tick limits - first    */
    /*  check if we are at the lowest tick end                        */
    /******************************************************************/
    if ( usTickPos == 0 )
      pIntData->usStatusSld |= usLowFlag;
    else
    {
      /****************************************************************/
      /* Check which scale is the current primary scale               */
      /****************************************************************/
      if ( ulStyle & SLS_PRIMARYSCALE2)
        usMaxTick = pIntData->usScale2Increments - 1;
      else
        usMaxTick = pIntData->usScale1Increments - 1;

      /****************************************************************/
      /* Check if upper limit reached for current scale               */
      /****************************************************************/
      if ( usTickPos >= usMaxTick )
        pIntData->usStatusSld |= usHighFlag;
    }
  }
  /********************************************************************/
  /* Otherwise use the arm position to determine if tick is at end    */
  /********************************************************************/
  else
  {
    /******************************************************************/
    /* Check if we are at the low end of the scale                    */
    /******************************************************************/
    if ( pIntData->usArmPos == 0 )
      pIntData->usStatusSld |= usLowFlag;

    /******************************************************************/
    /* otherwise check if we are at the high end of the scale         */
    /******************************************************************/
    else if ( pIntData->usArmPos >= usMaxRange )
      pIntData->usStatusSld |= usHighFlag;
  }

  return ( usOrigStatus != pIntData->usStatusSld );
}

/**********************************************************************/
/* Procedure: VSInitPresParam - This routine will initialize a        */
/*            presentation parameter with the system color index      */
/*            passed in and will update the valueset array with this  */
/*            index.                                                  */
/*                                                                    */
/* Inputs:  hwndVS     - Window handle of the value set control       */
/*          ulPp       - presentation parameter attribute identity    */
/*          ulSysIdx   - system color to map to presentation parameter*/
/**********************************************************************/
VOID SldInitPresParam( PISLDDATA pIntData )                             /*JEH1*/
{

  /********************************************************************/
  /* Set up all of the colors at one time                             */
  /********************************************************************/
  pIntData->ulColorVal[CLR_SLDBACKGROUND] =
                         GetSysColor(COLOR_WINDOW);

  pIntData->ulColorVal[CLR_SLDFOREGROUND] =
                         GetSysColor(COLOR_WINDOWTEXT);

  return;
}


/**********************************************************************/
/* Procedure: SLDSetPresParam - This routine will update the          */
/*            presentation parameter which has been modified for the  */
/*            control                                                 */
/*                                                                    */
/* Inputs:  hwndVS     - Window handle of the slider control          */
/*          pIntData   - Pointer to the slider control data block     */
/*          ulPp       - presentation parameter attribute identity    */
/**********************************************************************/
BOOL SldSetPresParam( PISLDDATA pIntData, USHORT wColorToChange,        /*JEH1*/
                      COLORREF crColor )
{

  /********************************************************************/
  /* Check if the foreground color has been changed                   */
  /********************************************************************/
  if ( wColorToChange == SDA_FOREGROUNDCOLOR )
  {
    pIntData->ulColorVal[CLR_SLDFOREGROUND] = crColor;
    return (TRUE);
  }
  /********************************************************************/
  /* Check if the background color has been changed                   */
  /********************************************************************/
  else if ( wColorToChange == SDA_BACKGROUNDCOLOR )
  {
    pIntData->ulColorVal[CLR_SLDBACKGROUND] = crColor;
    return (TRUE);
  }

  return (FALSE);
}

/**********************************************************************/
/* Procedure: SLDQueryPresParam - This routine will return the        */
/*            presentation parameter which currently exists for the   */
/*            control                                                 */
/*                                                                    */
/* Inputs:  hwndVS     - Window handle of the slider control          */
/*          pIntData   - Pointer to the slider control data block     */
/*          ulPp       - presentation parameter attribute identity    */
/**********************************************************************/
LONG SldQueryPresParam( PISLDDATA pIntData, USHORT wColorToReturn )       /*EVB1*/
{

  /********************************************************************/
  /* Return the current foreground color                              */
  /********************************************************************/
  if ( wColorToReturn == SDA_FOREGROUNDCOLOR )
  {
    return (long)(pIntData->ulColorVal[CLR_SLDFOREGROUND]);
  }
  /********************************************************************/
  /* Check if the background color has been changed                   */
  /********************************************************************/
  else if ( wColorToReturn == SDA_BACKGROUNDCOLOR )
  {
    return (long)(pIntData->ulColorVal[CLR_SLDBACKGROUND]);
  }
  else
    return (0);
}

/**********************************************************************/
/* Procedure: SLDResetPresParam - This routine will reset  the        */
/*            presentation parameter which currently exists for the   */
/*            control                                                 */
/*                                                                    */
/* Inputs:  hwndVS     - Window handle of the slider control          */
/*          pIntData   - Pointer to the slider control data block     */
/*          ulPp       - presentation parameter attribute identity    */
/**********************************************************************/
BOOL SldResetPresParam( PISLDDATA pIntData, USHORT wColorToReturn )       /*EVB1*/
{

  /********************************************************************/
  /* Return the current foreground color                              */
  /********************************************************************/
  if ( wColorToReturn == SDA_FOREGROUNDCOLOR )
  {
    pIntData->ulColorVal[CLR_SLDFOREGROUND] =
                         GetSysColor(COLOR_WINDOWTEXT);
    return (TRUE);
  }
  /********************************************************************/
  /* Check if the background color has been changed                   */
  /********************************************************************/
  else if ( wColorToReturn == SDA_BACKGROUNDCOLOR )
  {
    pIntData->ulColorVal[CLR_SLDBACKGROUND] =
                         GetSysColor(COLOR_WINDOW);
    return (TRUE);
  }
  else
    return (FALSE);
}


