/* Revision: 64 1.3 source/ui/cclnt/slider/slidewin.c, cclslid, ioc.v400, 000919  */
/**********************************************************************/
/*                                                                    */
/*              Licensed Materials - Property of IBM                  */
/*              20G0400 (C) Copyright IBM Corp. 1992, 1997.           */
/*                    See Copyright Instructions.                     */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/* CUA Controls Library/2 Version 1.0                       (Windows) */
/*                                                                    */
/* Sample Application Code                                            */
/* Slider Control                                                     */
/*                                                                    */
/* File:  slidewin.c                                                  */
/*                                                                    */
/* This application creates three sliders allowing the user to specify*/
/* the number of hours, minutes, and seconds.  When the start button  */
/* is pressed, a timer counts down the time, adjusting the hours,     */
/* minutes, and seconds sliders and displaying what percentage of     */
/* time has elapsed.                                                  */
/*                                                                    */
/**********************************************************************/

#include <windows.h>
#include <stdlib.h>                   /* C library for itoa           */
#include <string.h>                   /* C library for strcat         */
#include "iclsldw.h"                  /* Must include H file for CCL  */
#include "slidewin.h"                 /* Private H file               */

/**********************************************************************/
/*  Function:  WinMain                                                */
/*  Inputs:    hInstance            instance handle of execution      */
/*             hPrevInstance        instance handle of previous       */
/*                                  execution                         */
/*             lpCmdLine            command line arguments            */
/*             nCmdShow             startup mode (iconized/maximized) */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function registers the window class and contains the message */
/*  loop.                                                             */
/*  It loads the application icons and cursors and is responsible for */
/*  message dispatching.                                              */
/**********************************************************************/
INT APIENTRY WinMain (HANDLE hInstance,
                      HANDLE hPrevInstance,
                      LPSTR  lpCmdLine,
                      int    nCmdShow);

#if defined(__IBMC__)
int main( int argc, char** argv )
{
   HANDLE hInstance = GetModuleHandle(0);
   return WinMain( hInstance, 0, 0, SW_SHOWNORMAL );
}
#endif

INT APIENTRY WinMain (HANDLE hInstance,
                      HANDLE hPrevInstance,
                      LPSTR  lpCmdLine,
                      int    nCmdShow)
{
   HANDLE      hLibrary;            /* handle to slider DLL           */
   HANDLE      hwnd;                /* client window handle           */
   MSG         msg;                 /* message number                 */
   WNDCLASS    wndClass;            /* window class for registration  */
   static char szClientClass []  = "Slider Sample Application";
   POINT       screenSize;          /* screen size                    */

   lpCmdLine;                       /* prevent unused warning         */

   /*******************************************************************/
   /* register window class for client...a basic window with no fancy */
   /* aspects                                                         */
   /*******************************************************************/
   if (! hPrevInstance)
   {
      wndClass.lpszClassName = (LPCSTR)szClientClass;
      wndClass.hInstance     = hInstance;
      wndClass.lpfnWndProc   = ClientWndProc;
      wndClass.hCursor       = LoadCursor (0, IDC_ARROW);
      wndClass.hIcon         = LoadIcon (0, IDI_APPLICATION);
      wndClass.lpszMenuName  = MAKEINTRESOURCE (ID_RESOURCE);
      wndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
      wndClass.style         = (UINT)(CS_HREDRAW | CS_VREDRAW);
      wndClass.cbClsExtra    = (int)0;
      wndClass.cbWndExtra    =(int) 0;

      RegisterClass (&wndClass);

   }

  /********************************************************************/
  /* register the slider control                                      */
  /********************************************************************/
//if ((hLibrary = LoadLibrary(ICL_SLIDER_DLL)) < 32)
  if (!(hLibrary = LoadLibrary(ICL_SLIDER_DLL)))
    return 0;

   screenSize.x = GetSystemMetrics (SM_CXFULLSCREEN); /* Get screen   */
   screenSize.y = GetSystemMetrics (SM_CYFULLSCREEN); /* size         */

      /* create a big client window */
   hwnd = CreateWindow (szClientClass,                /* window class */
                        "Timer",                      /* window title */
                        WS_OVERLAPPEDWINDOW,          /* window style */
                        (screenSize.x < 550)          /* x-position   */
                           ?  0
                           :  (screenSize.x - 550) / 2,
                        (screenSize.y < 500)          /* y-position   */
                           ?  0
                           :  (screenSize.y - 500) / 2,
                        (screenSize.x < 550)          /* x-size       */
                           ?  screenSize.x
                           :  550,
                        (screenSize.y < 500)          /* y-size       */
                           ?  screenSize.y
                           :  500,
                        NULL,                         /* no parent    */
                        NULL,                         /* class menu   */
                        hInstance,                    /* creator      */
                        NULL);                        /* params       */

   ShowWindow (hwnd, nCmdShow);
   UpdateWindow (hwnd);



   while (GetMessage (&msg, NULL, 0, 0))      /* message loop   */
   {
      TranslateMessage (&msg);             /* keyboard interface to   */
      DispatchMessage (&msg);              /* client window procedure */
   }

   FreeLibrary(hLibrary);

   return 0;
}



/**********************************************************************/
/*  Function:  ClientWndProc                                          */
/*  Inputs:    hwnd                 client window handle              */
/*             msg                  message id                        */
/*             wParam               first parameter (word)            */
/*             lParam               second parameter (long)           */
/*  Outputs:   return values appropriate to message                   */
/*                                                                    */
/*  This function processes messages sent to the client window,       */
/*  including system messages and messages from controls.             */
/**********************************************************************/

LRESULT CALLBACK ClientWndProc (HWND hwnd, UINT msg,
                                WPARAM wParam, LPARAM lParam)
{
   static HFONT  hFontSld;       /* slider scale text font handle     */
   static RECT   rctlTime;       /* where to draw countdown clock     */
   static DWORD  ulTotalSecond;  /* how many seconds in the countdown */
   static DWORD  ulRemaining;    /* how many seconds are left         */
   DWORD         ulHour;         /* hour value                        */
   DWORD         ulMinute;       /* minute value                      */
   DWORD         ulSecond;       /* second value                      */


   switch (msg)
   {

      /****************************************************************/
      /* WM_CREATE is sent when the window is created.  It is here    */
      /* that we initialize the window by creating its children       */
      /****************************************************************/
      case WM_CREATE:
         {
            LOGFONT        logFont;         /* logical font structure */

            /**********************************************************/
            /* rctlTime is the region where the countdown clock is    */
            /* placed                                                 */
            /**********************************************************/
            rctlTime.right  = 500;
            rctlTime.left   = 365;
            rctlTime.bottom = 410;
            rctlTime.top    = 380;

            /**********************************************************/
            /* create a 12 pel height Tms Rmn font                    */
            /**********************************************************/
            memset (&logFont, 0, sizeof (LOGFONT));    /* initialize  */
            logFont.lfHeight      = 12;                /* height      */
            logFont.lfWeight      = 400;               /* normal      */
            logFont.lfOrientation = 900;               /* upright     */
            strcpy (logFont.lfFaceName, "Tms Rmn");    /* typeface    */

            hFontSld = CreateFontIndirect (&logFont);

            /**********************************************************/
            /* create child controls                                  */
            /**********************************************************/
            MakeChildren (hwnd,
                          ((LPCREATESTRUCT) lParam) ->hInstance,
                          hFontSld);
         }  /* end of WM_CREATE processing */

         break;


      /****************************************************************/
      /* The WM_TIMER message is sent whenever a timer interval has   */
      /* expired.  Hence, we decrement the sliders and update the     */
      /* countdown clock.  If the countdown is done, then we stop the */
      /* timer.                                                       */
      /****************************************************************/
      case WM_TIMER:
         {
            char              pszTime [9];      /* time left string   */
            char              *psz;             /* pointer to pszTime */
            HDC               hdc;              /* device context     */
            POINT             ptlStart;         /* where clocks drawn */
            WORD              idx;              /* loop index         */
            HBRUSH            hBrush;           /* brushes            */
            WORD              oldMode;          /* old background mode*/

            ulRemaining--;                     /* Decrement time left */

            /**********************************************************/
            /* convert the seconds remaining into hours, minutes, and */
            /* seconds                                                */
            /**********************************************************/
            ulHour   = ulRemaining / 3600;
            ulMinute = (ulRemaining % 3600) / 60;
            ulSecond = ulRemaining % 60;

            /**********************************************************/
            /* Send messages to sliders to set position               */
            /**********************************************************/
            setSliderPosition (hwnd, IDR_HOUR, ulHour);
            setSliderPosition (hwnd, IDR_MINUTE, ulMinute);
            setSliderPosition (hwnd, IDR_SECOND, ulSecond);

            /**********************************************************/
            /* only set the completion slider if we won't be dividing */
            /* by zero!...we calculate what percentage of the         */
            /* countdown is complete                                  */
            /**********************************************************/
            if (ulTotalSecond)
            {
               setSliderPosition (hwnd, IDR_COMPLETE,
                                  (ulTotalSecond - ulRemaining)
                                    * 100
                                    / ulTotalSecond);
            }

            /**********************************************************/
            /* Update clock:  Build clock text and then output it:    */
            /* text is in form "hh:mm:ss".  psz represents the        */
            /* endpoint of the string so far.                         */
            /**********************************************************/
            psz  = BuildTimeString (ulHour, pszTime);
            psz  = BuildTimeString (ulMinute, psz);
            psz  = BuildTimeString (ulSecond, psz);

            /**********************************************************/
            /* Set the clock position                                 */
            /**********************************************************/
            hdc        = GetDC (hwnd);
            ptlStart.x = rctlTime.left;
            ptlStart.y = rctlTime.bottom;

            /**********************************************************/
            /* Before outputting updated clock, erase previous clock  */
            /**********************************************************/
            hBrush = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
            FillRect (hdc, &rctlTime, hBrush);
            DeleteObject (hBrush);

            /**********************************************************/
            /* Now draw clock. Set text parameters first and then draw*/
            /**********************************************************/
            oldMode   = SetBkMode (hdc, TRANSPARENT);
            SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));
            SetTextAlign (hdc, TA_LEFT | TA_BASELINE);
            TextOut (hdc, ptlStart.x, ptlStart.y, pszTime, 8);
            SetBkMode (hdc, oldMode);

            /**********************************************************/
            /* if count down is complete, turn off timer and beep     */
            /**********************************************************/
            if ( ! ulRemaining )
            {
               SetUpTimer (hwnd, 1000, FALSE);

               for (idx = 1; idx < 5 ;idx ++ )
               {
                  DWORD idx2 = 100000;

                  MessageBeep (0);

                  while ( idx2-- ) ;   /* Add spacing between beeps   */

               } /* endfor */

               /*******************************************************/
               /* Erase clock                                         */
               /*******************************************************/
               hBrush = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
               FillRect (hdc, &rctlTime, hBrush);
               DeleteObject (hBrush);
            }

            ReleaseDC (hwnd, hdc);
            return 0;
         }


      /****************************************************************/
      /* The WM_COMMAND message is received when one of the buttons   */
      /* has been pressed or if a menu item has been selected.        */
      /* wParam - the id of the item selected.                        */
      /****************************************************************/
      case WM_COMMAND:

         switch (wParam)                /* See what item was selected */
         {
            /**********************************************************/
            /* menu item...exit:  to exit, we close the window        */
            /**********************************************************/
            case IDM_SUBEXIT:
               SendMessage (hwnd, WM_CLOSE, 0, 0);
               return FALSE;

            /**********************************************************/
            /* button..start clock:  to start, we get slider positions*/
            /* set timers, and begin countdown                        */
            /**********************************************************/
            case IDR_START:
               /*******************************************************/
               /* Send message to sliders to query position           */
               /*******************************************************/
               ulHour   = getSliderPosition (hwnd, IDR_HOUR);
               ulMinute = getSliderPosition (hwnd, IDR_MINUTE);
               ulSecond = getSliderPosition (hwnd, IDR_SECOND);

               /*******************************************************/
               /* Set Count in seconds                                */
               /*******************************************************/
               ulTotalSecond = ulHour   * 3600 +
                               ulMinute * 60 +
                               ulSecond;

               /*******************************************************/
               /* if countdown time exists, then begin coundown       */
               /*******************************************************/
               if (ulTotalSecond)
               {
                  ulRemaining   = ulTotalSecond;      /* store time   */
                  SetUpTimer (hwnd, 1000, TRUE);      /* begin timer  */
               }
               else
                  /****************************************************/
                  /* Otherwise, we cannot countdown from 00:00:00.    */
                  /* Warn user                                        */
                  /****************************************************/
                  MessageBeep (0);

               return 0;


            /**********************************************************/
            /* button...stop clock:  erase countdown clock, reset     */
            /* completion slider to 0%, and stop timer                */
            /**********************************************************/
            case IDR_STOP:
               {
                  HDC            hdc;        /* device context        */
                  HBRUSH         hBrush;     /* brush for filling rect*/

                  hdc = GetDC (hwnd);

                  /****************************************************/
                  /* Turn off timer, reset completion slider          */
                  /****************************************************/
                  SetUpTimer (hwnd, 1000, FALSE);
                  setSliderPosition (hwnd, IDR_COMPLETE, 0L);

                  /****************************************************/
                  /* erase clock                                      */
                  /****************************************************/
                  hBrush = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
                  FillRect (hdc, &rctlTime, hBrush);
                  DeleteObject (hBrush);

                  ReleaseDC (hwnd, hdc);
                  return 0;
               }

            default:                 /* other commands can be ignored */
               break;

         }


      /****************************************************************/
      /* on a paint message, simply draw the slider titles            */
      /****************************************************************/
      case WM_PAINT:
         {
            PAINTSTRUCT ps;                   /* paint structure      */
            RECT        rclInvalid;           /* invalid region       */
            POINT       ptlStart;             /* start point for text */
            HDC         hdc;                  /* device context       */
            WORD        oldBkMode;            /* old background mode  */

            /**********************************************************/
            /* set up for painting                                    */
            /**********************************************************/
            hdc         = BeginPaint (hwnd, &ps);
            rclInvalid  = ps.rcPaint;
            oldBkMode   = SetBkMode (hdc, TRANSPARENT);
            SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));

            /**********************************************************/
            /* output text                                            */
            /**********************************************************/
            SetTextAlign (hdc, TA_LEFT | TA_BASELINE);
            ptlStart.x = 120;
            ptlStart.y = 60;
            TextOut (hdc, ptlStart.x, ptlStart.y, "HOURS", 5);

            ptlStart.y = 170;
            TextOut (hdc, ptlStart.x, ptlStart.y, "MINUTES", 7);

            ptlStart.y = 280;
            TextOut (hdc, ptlStart.x, ptlStart.y, "SECONDS", 7);

            ptlStart.x = 400;
            ptlStart.y = 20;
            TextOut (hdc, ptlStart.x, ptlStart.y, "% COMPLETE", 10);

            SetBkMode (hdc, oldBkMode);
            EndPaint (hwnd, &ps);
         }  /* end of case WM_PAINT */

         break;


      /****************************************************************/
      /* Deallocate resources and remove time if still running        */
      /****************************************************************/
      case WM_DESTROY:
         SetUpTimer (hwnd, 1000, FALSE);
         DeleteObject (hFontSld);            /* remove font           */
         PostQuitMessage (0);
         break;

      default:
         break;

   }  /* end switch on message */

   /******************************************************************/
   /* other messages get passed on                                   */
   /******************************************************************/
   return (DefWindowProc (hwnd, msg, wParam, lParam));
}



/**********************************************************************/
/*  Function:  SetUpTimer                                             */
/*  Inputs:    hwnd       - client window handle                      */
/*             usDuration - how often timer should send messages      */
/*             flOn       - TRUE to start timer, FALSE to stop it     */
/*  Outputs:   returns TRUE if operation successful; FALSE otherwise  */
/*                                                                    */
/*  This function controls start and stop of timer.  If flOn is TRUE, */
/*  messages are sent to start a timer to send messages to hwnd at    */
/*  usDuration intervals.  If flOn is FALSE, the timer is killed.     */
/**********************************************************************/

BOOL SetUpTimer (HANDLE hwnd, WORD usDuration, BOOL flOn)
{

   if (flOn)
   {
      /****************************************************************/
      /* Start timer                                                  */
      /* If timer start failed, then tell user that there are too many*/
      /* clocks running and give the opportunity to try again         */
      /****************************************************************/
      while (! SetTimer (hwnd, ID_TIMER, usDuration, NULL))
      {
         if (IDCANCEL == MessageBox (hwnd,
                                     "Too many clocks or timers!",
                                     "Timer",
                                     MB_ICONEXCLAMATION | MB_RETRYCANCEL))
            return FALSE;
      }

   }
   else                              /* Stop timer                    */
   {
      if (! (KillTimer (hwnd, ID_TIMER) ) )
         return FALSE;
   }

   return TRUE;                      /* successful operation          */
}



/***********************************************************************/
/*  Function:  BuildTimeString                                         */
/*  Inputs:    Field                number to format                   */
/*             pszTime              string to place field into         */
/*  Outputs:   returns pointer to end of formatted string              */
/*                                                                     */
/*  This function formats Field into a two digit string, padding it    */
/*  with zeroes if necessary.  The result is placed in pszTime with a  */
/*  pointer to the end of the string being returned.                   */
/***********************************************************************/
char *BuildTimeString (LONG Field, char *pszTime)
{

   /*******************************************************************/
   /* if less than 10, then we must padd with a zero and go to the    */
   /* next character                                                  */
   /*******************************************************************/
   if (Field < 10)
      *pszTime++ = '0';

   /*******************************************************************/
   /* now convert the integer into a string and put in its place      */
   /*******************************************************************/
   ltoa (Field, pszTime, 10);

   /*******************************************************************/
   /* append a : to the end...remember that the format is hh:mm:ss    */
   /*******************************************************************/
   strcat (pszTime, ":");

   /*******************************************************************/
   /* Return a pointer to the end of the string, ie. to the           */
   /* terminating NULL.                                               */
   /*******************************************************************/
   while (*pszTime)
      pszTime++;

   return (pszTime);
}


/**********************************************************************/
/*  Function:  MakeChildren                                           */
/*  Inputs:    hwnd         - client window handle                    */
/*             hInstance    - executable instance handle              */
/*             hFont        - font handle for slider scale text       */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function creates four sliders and two buttons.  Sliders      */
/*  include those for hour, minute, and second, and completion        */
/*  percentage. Buttons for timer start and stop are also created.    */
/**********************************************************************/

VOID MakeChildren (HANDLE hwnd, HANDLE hInstance, HFONT hFont)
{
   DWORD    wndStyle;               /* window style                   */
   SLDCDATA sldcData;               /* slider control data            */

   /*******************************************************************/
   /* window style for hours, minutes, and seconds                    */
   /*******************************************************************/
   wndStyle = WS_CHILD          | /* child window                     */
              WS_VISIBLE        | /* visible                          */
              SLS_BUTTONSLEFT   | /* buttons to the left              */
              SLS_RIBBONSTRIP   | /* draw "progress indicator" strip  */
              SLS_HORIZONTAL    | /* left-to-right                    */
              SLS_HOMELEFT      | /* home to the left                 */
              SLS_SNAPTOINCREMENT;/* slider cannot go between         */
                                  /* increments                       */

   /*******************************************************************/
   /* Create Hour Slider                                              */
   /*                                                                 */
   /* Set up a sample slider with the top ruler setting defined       */
   /*******************************************************************/
   sldcData.cbSize             = sizeof (SLDCDATA);
   sldcData.usScale1Increments  = 25; /* 0-24 hours                   */
   sldcData.usScale1Spacing     = 10; /* automatic spacing of ticks   */
   sldcData.usScale2Increments  = 0;  /* not used                     */
   sldcData.usScale2Spacing     = 0;  /* not used                     */

   createSlider (hInstance,           /* instance handle              */
                 hwnd,                /* parent and owner             */
                 IDR_HOUR,            /* window id                    */
                 wndStyle,            /* window style                 */
                 SLD_POS_HOUR_X,      /* location                     */
                 SLD_POS_HOUR_Y,
                 SLD_LEN_HOUR_X,      /* size                         */
                 SLD_LEN_HOUR_Y,
                 &sldcData,           /* window data                  */
                 SLD_TEXTINCR_HOUR,   /* ticks to put text next to    */
                 FALSE,               /* all ticks same size          */
                 TRUE,                /* detent at 0 and halfway      */
                 hFont);              /* font handle                  */

   /*******************************************************************/
   /* Create Minute Slider                                            */
   /*                                                                 */
   /* Set up a sample slider with the top ruler setting defined       */
   /*******************************************************************/
   sldcData.cbSize             = sizeof (SLDCDATA);
   sldcData.usScale1Increments  = 60; /* 0-60 minutes                 */
   sldcData.usScale1Spacing     = 4;  /* automatic spacing of ticks   */
   sldcData.usScale2Increments  = 0;  /* not used                     */
   sldcData.usScale2Spacing     = 0;  /* not used                     */

   createSlider (hInstance,           /* instance handle              */
                 hwnd,                /* parent and owner             */
                 IDR_MINUTE,          /* window id                    */
                 wndStyle,            /* window style                 */
                 SLD_POS_MINUTE_X,    /* location                     */
                 SLD_POS_MINUTE_Y,
                 SLD_LEN_MINUTE_X,    /* size                         */
                 SLD_LEN_MINUTE_Y,
                 &sldcData,           /* window data                  */
                 SLD_TEXTINCR_MINUTE, /* ticks to put text next to    */
                 TRUE,                /* ticks with text are bigger   */
                 TRUE,                /* detent at 0 and halfway      */
                 hFont);              /* font handle                  */


   /*******************************************************************/
   /* Create Second Slider                                            */
   /*                                                                 */
   /* Set up a sample slider with the top ruler setting defined       */
   /*******************************************************************/
   sldcData.cbSize             = sizeof (SLDCDATA);
   sldcData.usScale1Increments  = 60; /* 0-60 seconds                 */
   sldcData.usScale1Spacing     = 4;  /* auto. spacing of ticks       */
   sldcData.usScale2Increments  = 0;  /* not used                     */
   sldcData.usScale2Spacing     = 0;  /* not used                     */

   createSlider (hInstance,           /* instance handle              */
                 hwnd,                /* parent and owner             */
                 IDR_SECOND,          /* window id                    */
                 wndStyle,            /* window style                 */
                 SLD_POS_SECOND_X,    /* location                     */
                 SLD_POS_SECOND_Y,
                 SLD_LEN_SECOND_X,    /* size                         */
                 SLD_LEN_SECOND_Y,
                 &sldcData,           /* window data                  */
                 SLD_TEXTINCR_SECOND, /* ticks to put text next to    */
                 TRUE,                /* ticks with text are bigger   */
                 TRUE,                /* detent at 0 and halfway      */
                 hFont);              /* font handle                  */


   /*******************************************************************/
   /* Create Completion Slider                                        */
   /*                                                                 */
   /* Set up a sample slider with the left ruler setting defined.     */
   /*                                                                 */
   /* Define style for completion slider                              */
   /*******************************************************************/
   wndStyle   = WS_CHILD          |   /* child window                 */
                WS_VISIBLE        |   /* visible                      */
                SLS_RIBBONSTRIP   |   /* progress indicator           */
                SLS_VERTICAL      |   /* up and down                  */
                SLS_HOMEBOTTOM    |   /* count up from bottom         */
                SLS_READONLY;         /* user cannot touch this       */


   /*******************************************************************/
   /* initialize control block for slider                             */
   /*******************************************************************/
   sldcData.cbSize             = sizeof (SLDCDATA);
   sldcData.usScale1Increments  = 101;/* 0-100 percent                */
   sldcData.usScale1Spacing     = 0;  /* auto. space the ticks        */
   sldcData.usScale2Increments  = 0;  /* not used                     */
   sldcData.usScale2Spacing     = 0;  /* not used                     */

   createSlider (hInstance,             /* instance handle            */
                 hwnd,                  /* parent and owner           */
                 IDR_COMPLETE,          /* window id                  */
                 wndStyle,              /* window style               */
                 SLD_POS_COMPLETE_X,    /* location                   */
                 SLD_POS_COMPLETE_Y,
                 SLD_LEN_COMPLETE_X,    /* size                       */
                 SLD_LEN_COMPLETE_Y,
                 &sldcData,             /* window data                */
                 SLD_TEXTINCR_COMPLETE, /* ticks to put text next to  */
                 TRUE,                  /* ticks with text are bigger */
                 FALSE,                 /* no detent at 0             */
                 hFont);                /* font handle                */


   /*******************************************************************/
   /* Create Buttons                                                  */
   /*******************************************************************/
   wndStyle = WS_CHILD   |              /* child window               */
              WS_VISIBLE |              /* visible                    */
              BS_PUSHBUTTON;            /* pushbutton                 */

   /*******************************************************************/
   /* Create the start button using this information                  */
   /*******************************************************************/
   CreateWindow ("button",                    /* class                */
                 "Start",                     /* title                */
                 wndStyle,                    /* style                */
                 BTN_POS_START_X, BTN_POS_START_Y,  /* position       */
                 BTN_LEN_START_X, BTN_LEN_START_Y,  /* size           */
                 hwnd,                        /* parent               */
                 (HMENU)IDR_START,                   /* class menu           */
                 hInstance,                   /* creator              */
                 (LPVOID) NULL);               /* data                 */

   /*******************************************************************/
   /* Create the cancel button using this information                 */
   /*******************************************************************/
   CreateWindow ("button",                    /* class                */
                 "Stop",                      /* title                */
                 wndStyle,                    /* style                */
                 BTN_POS_STOP_X, BTN_POS_STOP_Y,  /* positon          */
                 BTN_LEN_STOP_X, BTN_LEN_STOP_Y,  /* size             */
                 hwnd,                        /* parent               */
                 (HMENU)IDR_STOP,                    /* class menu           */
                 hInstance,                   /* creator              */
                 (LPVOID) NULL);               /* data                 */
}



/**********************************************************************/
/*  Function:  createSlider                                           */
/*  Inputs:    hInstance   - executable instance handle               */
/*             hwnd        - client window handle (slider owner/parent*/
/*             id          - slider id                                */
/*             wndStyle    - slider style                             */
/*             posX        - slider x-position                        */
/*             posY        - slider y-position                        */
/*             cx          - slider x-size                            */
/*             cy          - slider y-size                            */
/*             pSldcData   - pointer to slider data                   */
/*             textSpacing - which ticks to put text next to          */
/*             varyTickLen - whether ticks with text should be longer */
/*             detentAtZero- whether a detent should be placed at 0   */
/*             hFont       - font for slider scale text               */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function receives all parameters for creating and setting up */
/*  slider.  CreateWindow receives many of these parameters. Afterward*/
/*  slider tick marks, tick text, and detents are placed according to */
/*  the input parameters.  Scale font is changed to something readable*/
/**********************************************************************/

VOID createSlider (HANDLE     hInstance,
                   HANDLE     hwnd,
                   WORD       id,
                   DWORD      wndStyle,
                   WORD       posX,
                   WORD       posY,
                   WORD       cx,
                   WORD       cy,
                   LPSLDCDATA pSldcData,
                   WORD       textSpacing,
                   BOOL       varyTickLen,
                   BOOL       detentAtZero,
                   HFONT      hFont)

{
   HANDLE       hwndSld;                   /* slider handle          */
   WORD         idx;                       /* loop index             */
   char         buffer [5];                /* string buffer          */


   /******************************************************************/
   /* Create the slider using this information                       */
   /******************************************************************/
   hwndSld = CreateWindow (ICL_SLIDER,          /* class             */
                           NULL,                /* title             */
                           wndStyle,            /* style             */
                           posX, posY, cx, cy,  /* location and size */
                           hwnd,                /* parent            */
                           (HMENU)id,                  /* class menu        */
                           hInstance,           /* creator           */
                           (LPVOID) pSldcData);  /* data              */

   /********************************************************************/
   /* Place increments along the slider                                */
   /* If varyTickLen is TRUE, then make every text-marked              */
   /*    increment 3 pels larger; otherwise, make all ticks            */
   /*    equal.                                                        */
   /********************************************************************/
   if (! varyTickLen)
      SendMessage ( hwndSld,               /* set all ticks to 8 pels */
                    SLM_SETTICKSIZE,
                    MAKELONG (SMA_SETALLTICKS, 8),                      /*DAB1*/
                    (DWORD)0 );                                         /*DAB1*/
   else
   {
      WORD  wTickSize;                     /* length of tick to draw  */

      for ( idx = 0;
            idx <= (pSldcData->usScale1Increments);
            idx ++ )
      {
         /*************************************************************/
         /* tick size is 5 pels if not text-marked, 8 pels otherwise  */
         /*************************************************************/
         wTickSize = 5 + ( (idx % textSpacing) == 0) * 3 ;
         SendMessage ( hwndSld,                    /* set tick length */
                       SLM_SETTICKSIZE,
                       MAKELONG (idx, wTickSize),                       /*DAB1*/
                       (DWORD)0 );                                      /*DAB1*/
      }
   }

   /*******************************************************************/
   /* Place text along ruler at appropriate increments                */
   /*******************************************************************/
   for ( idx = 0;
         idx <= (pSldcData->usScale1Increments);
         idx += textSpacing)
   {
      itoa (idx, buffer, 10);           /* convert number to string   */
      SendMessage ( hwndSld,            /* and put into slider scale  */
                    SLM_SETSCALETEXT,
                    idx,
                    (DWORD) (VOID FAR *) ( buffer ) );
   }

   /*******************************************************************/
   /* If detent requested, then add detents on ruler at 0 and halfway */
   /* along scale.                                                    */
   /*******************************************************************/
   if (detentAtZero)
   {
      SendMessage ( hwndSld,
                    SLM_ADDDETENT,
                    0,
                    0L );
      SendMessage ( hwndSld,
                    SLM_ADDDETENT,
                    ((pSldcData->usScale1Increments) / 2)
                                * (pSldcData->usScale1Spacing),
                    0L );
   }

}


/**********************************************************************/
/*  Function:  setSliderPosition                                      */
/*  Inputs:    hwnd - client window handle (slider owner/parent)      */
/*             id   - slider id                                       */
/*             position             where slider arm should go        */
/*  Outputs:   TRUE if successful; FALSE otherwise                    */
/*                                                                    */
/*  This function simply sets the slider arm position to the increment*/
/*  given by the position parameter.  A message is sent to the slider */
/*  arm control to accomplish this.                                   */
/**********************************************************************/
BOOL setSliderPosition (HANDLE hwnd, WORD id, DWORD position)

{
   /*******************************************************************/
   /* initialize structure                                            */
   /* we want to set the slider arm position to be a specified        */
   /* increment value                                                 */
   /*******************************************************************/
   return (BOOL)(DWORD)SendDlgItemMessage ( hwnd, id, SLM_SETSLIDERINFO,/*DAB1*/
                  MAKELONG( SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE ),/*DAB1*/
                                            (DWORD)position );          /*DAB1*/
}


/**********************************************************************/
/*  Function:  getSliderPosition                                      */
/*  Inputs:    hwnd  - client window handle (slider owner/parent)     */
/*             id    - slider id                                      */
/*  Outputs:   current increment value for slider arm position        */
/*                                                                    */
/*  This function simply sends a message to the slider arm control to */
/*  determine where the arm position is currently set.                */
/**********************************************************************/
DWORD getSliderPosition (HANDLE hwnd, WORD id)

{
   return (DWORD) SendDlgItemMessage ( hwnd, id,
                                       SLM_QUERYSLIDERINFO,
                                       MAKELONG (SMA_SLIDERARMPOSITION, /*DAB1*/
                                                 SMA_INCREMENTVALUE),   /*DAB1*/
                                       (DWORD)0 );                      /*DAB1*/
}
