#ifndef _SPINBTNP_
#define _SPINBTNP_

/* Revision: 69 1.4.1.3 source/ui/cclnt/spinbtn/spinbtnp.h, cclspin, ioc.v400, 000919  */
/****************************************************************************/
/*                                                                          */
/*                         IBM CONFIDENTIAL                                 */
/*                                                                          */
/*        (IBM Confidential-Restricted when combined with the               */
/*         aggregated OCO source modules for this program)                  */
/*                                                                          */
/*                       OCO SOURCE MATERIALS                               */
/*                                                                          */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)         */
/*                                                                          */
/****************************************************************************/
/****************************************************************************/
/*                                                                          */
/* MODULE = SPINBTNP.H                                                      */
/* COMPONENT = SPIN BUTTON                                                  */
/* USAGE = SPIN BUTTON PRIVATE HEADER                                       */
/*                                                                          */
/*                                                                          */
/* NOTES =                                                                  */
/*     NONE.                                                                */
/*                                                                          */
/*   RESTRICTIONS =                                                         */
/*                                                                          */
/*     NONE.                                                                */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
/******************************************************************************/
/* Product: ICLUINT.DLL                                                       */
/* Filename: SPINBTNP.H - Internal control block header file for the          */
/*                        spin button control                                 */
/*                                                                            */
/* Flag    Date    Defect  Description                                        */
/* ----  --------  ------  -------------------------------------------------- */
/* W01   06/10/91          Initial pop!                                       */
/* L02   01/25/92  8500198 Spin Button disable support                        */
/* L03   02/13/92  8500425 Selection/Editing bug                              */
/* B07   08/26/92  8510066 Bug in numeric select/replace logic                */
/* EVB   10/31/95          Add color support for spinbtn                      */
/* MAB1  02/09/96  6613    Rename "CCL Xxxx" window classes to "ICL Xxxx"     */
/******************************************************************************/
#if defined(__IBMC__)
#pragma info(nocnd)
#endif

#include <iclccl.h>     /* CCL common include file */

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

/******************************************************************************/
/* Internal data structures.                                                  */
/******************************************************************************/

#define WC_ARROWS ((LPSTR) "ICL Spin Button Arrows")                     //@MAB1

typedef LPSTR FAR *LPPSTR;
typedef NPSTR NEAR *NPPSTR;
typedef VOID NEAR *NPVOID;


typedef struct _SPINDATA
{
   HANDLE    hInstance;          /* Identifies the instance of the app        */
   HWND      hwndArrows;         /* Handle to Spin Arrows                     */
   HWND      hwndSpinField;      /* Handle to Spin Field                      */
   HWND      hwndMaster;         /* Handle to this component's Master         */
                                 /*   If NULL upon creation, this is reset to */
                                 /*   the handle to itself. ie. it is it's    */
                                 /*   own master.                             */
   HWND     hwndCurrentlySelected;
                                 /* Who the spinarrows will send the up/down  */
                                 /*   arrow messages to.                      */
   HBITMAP  hbmUp;               /* hbm for the up arrow bitmap               */
   HBITMAP  hbmDown;             /* hbm for the down arrow bitmap             */
   HBITMAP  hbmUpDisabled;       /* hbm for the disabled up arrow bitmap      */ /* ->$L02 */
   HBITMAP  hbmDownDisabled;     /* hbm for the disabled down arrow bitmap    */ /* $L02<- */
   DWORD    ulWarpFactor;        /* Multiplier for spin up/down if button held*/
   DWORD    ulStyleChange;       /* Store styles to detect changes            */
   DWORD    fl;                  /* status flags                              */
   LONG     lUpperLimit;         /* Upper Limit: If Numeric, Upper Limit      */
                                 /*     If Char, Upper index. (#items -1)     */
   LONG     lLowerLimit;         /* Lower Limit: If Numeric, Lower Limit      */
                                 /*     If Char, Lower index. (0)             */
   LONG     lCurrentValue;       /* If Numeric, Current Value                 */
                                 /* If Char, Index for array of current string*/
   NPPSTR   anpszText;           /* CHARACTER DATA ONLY: Ptr to data          */
   WORD     cchMax;              /* Stores the number of chars in the largest */
                                 /*   string.                                 */
   WORD     usNumericFormatSize; /* Size of format for numeric output         */
   WORD     idTimer;             /* Id of the current timer (for stopping it) */
   WORD     usTextLimit;         /* Entryfield Text limit                     */
   WORD     cArrayItems;         /* count of items in text spin button array  */
   HFONT    hSpbFont;            /* Font for the spin button to use           */
   RECT     rctSpinField;        /* used to track size of edit field          */
   LONG     lPreviousValue;      /* In Windows EN_CHANGE notification is sent *///@B07
                                 /* twice for replacement of a selection:     */
                                 /* Once for deletion, Once for insertion.    */
                                 /* Save prev value for SPBQ_ALWAYSUPDATE to  */
                                 /* handle a numeric value that is out of     */
                                 /* range.  Not used for array values.        */
   ULONG    ulColorVal[2];       /* Color array for Spinbtn painting          *///@EVB
   HBRUSH   hBrush;              /* Background brush for Spinbtn painting     *///@EVB
} SPINDATA, NEAR *PSPINDATA;


typedef struct _ENTRYFDATA
{
    WORD  cb;
    WORD  cchEditLimit;
    WORD  ichMinSel;
    WORD  ichMaxSel;
} ENTRYFDATA, NEAR *PENTRYFDATA;

/******************************************************************************/
/* SPINBUTTON status flag definitions                                         */
/******************************************************************************/
#define SBF_ARRAYDATA      0x00000001   /* Data in this sbtn is an array      */
#define SBF_DATAEXIST      0x00000002   /* Data exists in this sbtn           */
#define SBF_OWNERCONTROL   0x00000004   /* Owner controls aspects of sbtn     */
#define SBF_TIMER          0x00000008   /* Timer is to be used                */
#define SBF_DOWNARROWPRESSED 0x00000010 /* Down arrow is pressed              */
#define SBF_UPARROWPRESSED 0x00000020   /* Up arrow is pressed                */
#define SBF_INITIALDOWNARROWPRESS 0x00000040 /* Initial down arrow press      */
#define SBF_INITIALUPARROWPRESS 0x00000080   /* Initial up arrow press        */
#define SBF_DELETEORREPLACE 0x80000000  /* Delete or replace contents         *///@B07

/******************************************************************************/
/* SPINBUTTON Miscellaneous definitions                                       */
/******************************************************************************/
#if 0      /*-------------------------< JDR >--------------------------------*/
#define SBF_ENTRYTOOSMALL      0x0040 /* Entryfield too small to send focus to*/
#define BMP_SIZE                    7 /* height and width of the arrow bitmap */
#define CONTINUE                  777
#define ENTRYFIELD_TOO_SMALL       25
#define INIT_DELAY                800 /* initial delay before spin on btn1dn  */
#endif     /*-------------------------< JDR >--------------------------------*/

#define CCHMAXEDIT                255 /* Default Maximum number of chars      */
#define CCHMAXLONG                 11 /* Maximum number of chars in a long val*/
#define ID_EDIT                0x0001 /* id of child edit control             *///@L03
#define TID_PRIMARY            0xFF01
#define TID_SECONDARY          0xFF02
#define WM_SETFOCUS_PRIVATE  WM_USER + 0x20A  /* Component Posts when WM_SETFOCUS */
#define CXARROW                    17 /* in pels                              */
#define INITIAL_TIMEDELAY         250 /* Delay 800 milliseconds               */
#define TIME_DURATION            2000 /* Delay 2000 milliseconds              */
// JEH #define MAXLONG            0x7FFFFFFF /* Maximum allowed for longs            */
#define HABX                ((HAB) 0)
#define GWL_USER                   0                                  /* @W01 */
#define DELAY                   6000
#define SPBERR_CREATE_FAILED      -1
#define SB_GOTBUTTONDOWN   0x7AD459B6

#if 0      /*-------------------------< MAB >--------------------------------*/
#define BYTESIN63K             0xFBFF /* Allow for Query overhead             */
#define COMPONENT_HEIGHT           31
#define CTLENTRYBYTES               8 /* 8 BYTES                              */
#define DLLNAME       CCL_SPINBTN_DLL /* name of the dll, used to load bitmaps*/
                                                                        /*WBB1*/
#define ID_TIMER      TID_USERMAX - 1
#define FIND                    FALSE /* value passed into 'Search' routine   */
#define ID_TIMER1                   1
#define ID_TIMER2                   2
#define PARTIAL_MATCH            TRUE /* value passed in to 'Search' for type */
#define RADIX                      10 /* Use base 10 for numeric conversions  */
#define SBI_COMPONENT_GOT_FOCUS   500 /* Component Posts when WM_SETFOCUS     */
#endif     /*-------------------------< MAB >--------------------------------*/
#define CLR_SPBBACKGROUND  0                                            //@EVB
#define CLR_SPBFOREGROUND  1                                            //@EVB

/******************************************************************************/
/* Window word offset definitions                                             */
/******************************************************************************/
#define GWL_SPIN       (GWL_USER + sizeof(LPLONG))

/******************************************************************************/
/* SPINBUTTON Function prototypes                                             */
/******************************************************************************/

LRESULT CALLBACK SpinButtonWndProc (HWND hwnd, UINT msg,
                                    WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK SpinSubclassEntryfield (HWND hwnd, UINT msg,
                                         WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK ArrowWndProc (HWND hwnd, UINT msg, WPARAM wParam,
                               LPARAM lParam);

VOID      AlignRange (PSPINDATA pSpinData);
NPVOID    AllocHeapMem (UINT cbSize);
PSPINDATA CreateSpinData (HWND hwnd);
VOID      DrawArrow (HDC hdc, LPRECT pRect, HBITMAP hbmArrow, BOOL bButtonDown);
VOID      DrawButtonArea (HDC hdc, LPRECT pRect, BOOL fButtonDown);
VOID      FastSpinOn (HWND hwnd, PSPINDATA pSpinData,
                      HWND hwndCurrent, int idTimer);
VOID      FastSpinOff (HWND hwnd, PSPINDATA pSpinData, int idTimer);
BOOL      FreeHeapMem (PBYTE nptr);
DWORD     GetSpinButtonStyle (HWND hwnd);
VOID      SendSpinKeybrdMsgs (PSPINDATA pSpinData, HWND hwnd, WORD SpinMsg,
                              WORD SpinNotify, BOOL bReleased);
long      SendSpinMouseMsgs (PSPINDATA pSpinData, HWND hwnd, WORD SpinMsg,
                             WORD SpinNotify);
BOOL      SpinAdjustWindowPos (PSPINDATA pSpinData, HWND hwnd, UINT fResize,
                               int sWidth, int sHeight);
long      SpinChar (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
                    PSPINDATA pSpinData);
VOID      SpinControl (HWND hwnd, PSPINDATA pSpinData, WORD idSender,
                       WORD usNotifyCode, HWND hwndSelected);
long      SpinCreate (HWND hwnd, LPCREATESTRUCT pCreate);
VOID      SpinDestroy (HWND hwnd, PSPINDATA pSpinData);
VOID      SpinPaint (HWND hwnd, PSPINDATA pSpinData);
long      SpinQueryValue (HWND hwnd, PSPINDATA pSpinData, LPVOID pBuf,
                          DWORD lOther);
long      SpinQueryPresParam (PSPINDATA pSpinData, USHORT wColorToReturn);   //@EVB
BOOL      SpinResetPresParam (PSPINDATA pSpinData, USHORT wColorToReturn);   //@EVB
int       SpinSearch (PSPINDATA pSpinData, LPSTR lpszSearchText, BOOL fPartial);
long      SpinSetArray (HWND hwnd, PSPINDATA psd, LPPSTR alpszNew, WORD cItems);
long      SpinSetCurrentValue (HWND hwnd, PSPINDATA pSpinData, LONG lNewValue);
HBRUSH    SpinSetEditCtrlBrush (HDC hdcEditCtrl, PSPINDATA pSpinData);       //@EVB
long      SpinSetLimits (HWND hwnd, PSPINDATA pSpinData, LONG lUpperLimit,
                         LONG lLowerLimit, BOOL fOverride);
long      SpinSetMaster (PSPINDATA pSpinData, HWND hwndMaster);
BOOL      SpinSetPresParam (PSPINDATA pSpinData, USHORT wColorToChange,      //@EVB
                            COLORREF crColor);                               //@EVB
long      SpinSetTextLimit (HWND hwnd, PSPINDATA pSpinData, WORD usTextLimit);
long      SpinSpinValue (HWND hwnd, PSPINDATA pSpinData, DWORD ulIncrement,
                         BOOL fUp);
BOOL      UpdateField (HWND hwnd, PSPINDATA pSpinData);

VOID      ltoa (HWND hwnd, PSPINDATA pSpinData, LPSTR pString);

#if 0
NPVOID    memset (NPVOID p, BYTE c, WORD cb);
int       strcmp (LPSTR lpsz1, LPSTR lpsz2);
LPSTR     strcpy (LPSTR lpszDest, LPSTR lpszSrc);
WORD      strlen (LPSTR lpsz);
VOID      strrev (LPSTR lpString);
#endif

LONG      strtol (LPSTR lpszValue, LPPSTR lppEndChar);

#if 0      /*-------------------------< MAB >--------------------------------*/
VOID SpinBinarySort (PINTERNALDATA pInternalData, PSHORT pMiddle,
                     LPWORD pIndex);
#endif     /*-------------------------< MAB >--------------------------------*/

/**********************************************************************/
/* macros                                                             */
/**********************************************************************/

 /****************************************************/
 /* Check to see if size of buffer is NULL or other. */
 /* If it is NULL, pBuf is assumed to be the address */
 /* of LONG variable.  If other, pBuf is assumed to  */
 /* be the address of a character string that is of  */
 /* size other.                                      */
 /*                                                  */
 /* If a LONG variable is to be returned, the current*/
 /* value is returned.  If a string is to be returned*/
 /* the string value is returned.                    */
 /****************************************************/
#define PBUFNOTNULL(var) if (pBuf)                                        \
                         {                                                \
                            if (lOther == 0)                              \
                               (*(LPLONG)pBuf) = (var);                   \
                            else                                          \
                               GetWindowText (pSpinData->hwndSpinField,   \
                                              (LPSTR)pBuf,                \
                                              lOther);                    \
                         }

#ifndef PARENTOF
#define PARENTOF(hwnd)  (HWND)GetWindowLong (hwnd, GWL_HWNDPARENT)
#endif
#ifndef STYLEOF
#define STYLEOF(hwnd)   GetWindowLong (hwnd, GWL_STYLE)
#endif
#ifndef IDOF
#define IDOF(hwnd)      GetWindowLong (hwnd, GWL_ID)
#endif

#define SENDCONTROL(hwnd,id,code,hwndctrl)                          \
                        SENDMESSAGE (hwnd, WM_COMMAND,              \
                                     MAKELONG(id,code), (LONG)hwndctrl)

#define POSTCONTROL(hwnd,id,code,hwndctrl)                          \
                        POSTMESSAGE ((HWND)hwnd, WM_COMMAND,              \
                                     MAKELONG(id,code), (LONG)hwndctrl)

#define MP1FROMMP2(mp2) { mp1 = *(LPDWORD)lParam;    \
                          mp2 = (DWORD)((LPDWORD)lParam+1); }

#define MP2ISVALUE(mp2) { mp2 = *(LPDWORD)mp2; }

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // SPINBTNP header guard
