// Revision: 84 1.7.1.5 source/ui/cnr/icnrctl3.cpp, container, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icnrctl3.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the view switching and            *
*   quering functions for the class IContainerControl declared                 *
*   in icnrctl.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDCNR

#include <icnrrec.hpp>  // Must be first for OS flags

#include <itrace.hpp>
#include <iexcept.hpp>
#include <icnrctl.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTL3_CPP_
  #include <ipagetun.h>
#endif

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: arrangeIconView

 Implementation: Flow the icons in the icon view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: arrangeIconView()
{
   unsigned fSuccess = sendEvent( CM_ARRANGE, 0, 0);             /* LJO */
   if(!fSuccess)                                                 /* LJO */
   {                                                             /* LJO */
      ITHROWGUIERROR("CM_ARRANGE");                              /* LJO */
   }                                                             /* LJO */
   return *this;                                                 /* LJO */
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: showMiniIcons

 Implementation: Show mini-icons in any non-text view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: showMiniIcons( bool mini)
{
   if (mini)
     setContainerAttributes( 0, miniIcons.asUnsignedLong() );
   else
     setContainerAttributes( miniIcons.asUnsignedLong(), 0 );

   // DEFECT 20101 : if PM, store new bmp size, and reset view...
   //  (pm-compatible does not always pick up the change when
   //   mini-size is turned OFF, so just force the icon size to be
   //   changed to the same sizes used by largeImages/smallImages)
   // NOTE : control does not "remember" what any prior size was,
   //   so if miniIcons gets turned off, then largeImages size gets
   //   used, no matter what the previous (possible scaled) size was.
#ifdef IC_WIN
   if ( isPMCompatible() ) 
   {
      ISize xymini( GetSystemMetrics(SM_CXSMICON), 
                    GetSystemMetrics(SM_CYSMICON) );
      ISize xynorm( GetSystemMetrics(SM_CXICON), 
                    GetSystemMetrics(SM_CYICON) );
      setIconSize( mini ? xymini : xynorm );
   }
#endif // IC_WIN

   return *this;
}


void IContainerControl :: showView(unsigned long ulViewFlags)
{
   setContainerAttributes((textView | treeView | nameView |
                          iconView | detailsView |
                          flowedView).asUnsignedLong(),
                          ulViewFlags);
}

IContainerControl& IContainerControl :: showTextView()
{
    showView(textView.asUnsignedLong());
    return *this;
}

IContainerControl& IContainerControl :: showFlowedTextView()
{
   showView((textView | flowedView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showNameView()
{
   showView(nameView.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showFlowedNameView()
{
   showView((nameView | flowedView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showDetailsView()
{
   showView(detailsView.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showIconView()
{
   showView(iconView.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showTreeIconView()
{
   showView((iconView | treeView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showTreeTextView()
{
   showView((textView | treeView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showTreeNameView()
{
   showView((nameView | treeView).asUnsignedLong());
   return *this;
}


IContainerControl& IContainerControl :: setNormalTargetEmphasis()
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   setContainerAttributes((mixedTargetEmphasis | orderedTargetEmphasis).asUnsignedLong()
                             , 0);
   return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

IContainerControl& IContainerControl :: setOrderedTargetEmphasis()
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   setContainerAttributes(mixedTargetEmphasis.asUnsignedLong(),
                          orderedTargetEmphasis.asUnsignedLong());
   return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

IContainerControl& IContainerControl :: setMixedTargetEmphasis()
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   setContainerAttributes(orderedTargetEmphasis.asUnsignedLong(),
                          mixedTargetEmphasis.asUnsignedLong());
   return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

/*-------------------------------------------------------------------
  Since there is no normal target emphasis bit(it is the default),
  we need to check that the other 2 (ordered, mixed) are not set.
-------------------------------------------------------------------*/
bool IContainerControl :: hasNormalTargetEmphasis() const
{
#ifdef IC_PMWIN
   return ( ( !(containerAttributes() & mixedTargetEmphasis.asUnsignedLong()) &&
	    !(containerAttributes() & orderedTargetEmphasis.asUnsignedLong()) )
	     ? true : false );
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

bool IContainerControl :: hasOrderedTargetEmphasis() const
{
#ifdef IC_PMWIN
   return ( (containerAttributes() & orderedTargetEmphasis.asUnsignedLong())
             ? true : false );
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

bool IContainerControl :: hasMixedTargetEmphasis() const
{
#ifdef IC_PMWIN
   return ( (containerAttributes() & mixedTargetEmphasis.asUnsignedLong())
             ? true : false );
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

bool IContainerControl :: isTextView(bool textOnly) const
{
  if (textOnly)  //text view and not tree-text view
    return( ( (containerAttributes()  & textView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) )
            ? true : false);
  else
    return((containerAttributes() & textView.asUnsignedLong()) ? true : false);
}

bool IContainerControl :: isNameView(bool nameOnly) const
{
  if (nameOnly)  //name view and not tree-name view
    return( ( (containerAttributes()  & nameView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) )
            ? true : false);
  else
    return((containerAttributes() & nameView.asUnsignedLong()) ? true : false);
}

bool IContainerControl :: isIconView(bool iconOnly) const
{
  if (iconOnly)  //icon view and not tree-icon view
    return( ( (containerAttributes()  & iconView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) )
            ? true : false);
  else
    return((containerAttributes() & iconView.asUnsignedLong()) ? true : false);
}

bool IContainerControl :: isDetailsView() const
{
   return ((containerAttributes() & detailsView.asUnsignedLong()) ? true : false);
}

bool IContainerControl :: isTreeView() const
{
   return ((containerAttributes() & treeView.asUnsignedLong()) ? true : false);
}

bool IContainerControl :: isFlowed() const
{
   return ((containerAttributes() & flowedView.asUnsignedLong()) ? true : false);
}

bool IContainerControl :: isFlowedTextView() const
{
   return ( ( (containerAttributes()  & textView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) &&
              (containerAttributes()  & flowedView.asUnsignedLong())  )
            ? true : false);
}

bool IContainerControl :: isFlowedNameView() const
{
   return ( ( (containerAttributes()  & nameView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) &&
              (containerAttributes()  & flowedView.asUnsignedLong())  )
            ? true : false);
}

bool IContainerControl :: isTreeIconView() const
{
   return ( ( (containerAttributes()  & treeView.asUnsignedLong()) &&
              (containerAttributes()  & iconView.asUnsignedLong())  )
            ? true : false);
}

bool IContainerControl :: isTreeNameView() const
{
   return ( ( (containerAttributes()  & treeView.asUnsignedLong()) &&
              (containerAttributes()  & nameView.asUnsignedLong())  )
            ? true : false);
}

bool IContainerControl :: isTreeTextView() const
{
   return ( ( (containerAttributes()  & treeView.asUnsignedLong()) &&
              (containerAttributes()  & textView.asUnsignedLong())  )
            ? true : false);
}

bool IContainerControl :: isShowingMiniIcons() const
{
   return ( (containerAttributes()  & miniIcons.asUnsignedLong())
            ? true : false);
}
