// Revision: 86 1.5.2.2 source/ui/cnr/icnrctl5.cpp, container, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: icnrctl5.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the Details View functions for    *
*    IContainerControl, declared in icnrctl.hpp.                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDCNR

#include <icnrrec.hpp>  // Must be first for OS flags


#include <itrace.hpp>
#include <iexcept.hpp>
#include <icnrctl.hpp>
#include <icnrcol.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTL5_CPP_
  #include <ipagetun.h>
#endif

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: showSplitBar

 Implementation: Show the Details View SpitBar.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: showSplitBar(bool fShow)
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   // if the boolean says to show the bar, show it
   if(fShow)
   {
     ICnrInfo cnrInfo;
     if(pcnrcolClSplitbar)
     {
       cnrInfo.pFieldInfoLast = pcnrcolClSplitbar->columnInfo();
     }
     cnrInfo.xVertSplitbar = (short)lClSplitbar;
     setContainerInfo(&cnrInfo, CMA_XVERTSPLITBAR | CMA_PFIELDINFOLAST);
     return *this;
   }     // end if
   // else, hide the bar
   return hideSplitBar();
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return *this;
#endif
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: hideSplitBar

 Implementation: Hide the Details View SpitBar.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: hideSplitBar()
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  ICnrInfo cnrInfo;
  containerInfo(&cnrInfo);
  lClSplitbar = cnrInfo.xVertSplitbar;
  cnrInfo.xVertSplitbar = -1;
  cnrInfo.pFieldInfoLast = 0;
  setContainerInfo(&cnrInfo, CMA_XVERTSPLITBAR | CMA_PFIELDINFOLAST);
  return *this;
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return *this;
#endif
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setDetailsViewSplit

 Implementation: Set the size and position of the SplitBar.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setDetailsViewSplit(
                      const IContainerColumn* lastColumnBeforeSplit,
                      unsigned long ulPixelsFromLeft)
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  IASSERTPARM(lastColumnBeforeSplit!=0);
  ICnrInfo cnrInfo;

  cnrInfo.pFieldInfoLast = lastColumnBeforeSplit->columnInfo();
  pcnrcolClSplitbar = (IContainerColumn*)lastColumnBeforeSplit;
  cnrInfo.xVertSplitbar = ulPixelsFromLeft;
  lClSplitbar = ulPixelsFromLeft;
  setContainerInfo(&cnrInfo, CMA_XVERTSPLITBAR | CMA_PFIELDINFOLAST);
  return *this;
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return *this;
#endif
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: splitBarOffset

 Implementation: Return the offset of the split bar in pels from the left
                 side of the container window.
------------------------------------------------------------------------------*/
unsigned long IContainerControl :: splitBarOffset( ) const
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  unsigned long offset;
  ICnrInfo cnrInfo;
  containerInfo(&cnrInfo);

  if (cnrInfo.xVertSplitbar != -1)
    offset =  (unsigned long)(cnrInfo.xVertSplitbar);
  else
    offset = 0;
  return offset;
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return 0;
#endif
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: detailsViewSplit

 Implementation: Return last column before splitbar.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl :: detailsViewSplit() const
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  ICnrInfo cnrInfo;
  containerInfo(&cnrInfo);
  if (cnrInfo.pFieldInfoLast)
    return (IContainerColumn*)(cnrInfo.pFieldInfoLast->pUserData);
  return 0;
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return 0;
#endif
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: addColumn

 Implementation: Add a column to the container.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: addColumn(const IContainerColumn* column,
                                    const IContainerColumn* pcnrcolAfter)
{
   IMODTRACE_DEVELOP("CnrCtl::addCol");
   IASSERTPARM(column!=0);
   IASSERTPARM(column->container()==0);
   ((IContainerColumn*)column)->setContainer(this);
   FIELDINFOINSERT fieldinfoinsert;
   PFIELDINFO pfldinfoNew = column->columnInfo();
   pfldinfoNew->pNextFieldInfo = 0;

   if(pcnrcolAfter)
     fieldinfoinsert.pFieldInfoOrder = pcnrcolAfter->columnInfo();
   else
     fieldinfoinsert.pFieldInfoOrder = (PFIELDINFO)CMA_END;

   fieldinfoinsert.cb = sizeof(FIELDINFOINSERT);

   /* insert 1 column header */
   fieldinfoinsert.cFieldInfoInsert = 1;

   /* Adjust the offset so it is based on the start of */
   /* IMiniCnrRecord.  Note that fields already in the */
   /* base have the value IFIELDINBASE added to them   */
   /* so that we can adjust them correctly             */
   if(pfldinfoNew->offStruct > IFIELDINBASE)
      pfldinfoNew->offStruct-=IFIELDINBASE;
   else
      pfldinfoNew->offStruct+=baseRecordSize();


   unsigned fSuccess = sendEvent( CM_INSERTDETAILFIELDINFO,
                                  MPFROMP(pfldinfoNew),
                                  MPFROMP(&fieldinfoinsert));

   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_INSERTDETAILFIELDINFO");
   }
   ulClColumnChanges++;
   if (fieldinfoinsert.fInvalidateFieldInfo==false)
      fSuccess = sendEvent( CM_INVALIDATEDETAILFIELDINFO,
                            MPFROMP(0),
                            MPFROMP(0));

   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: removeColumn

 Implementation: Remove a column from the container.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: removeColumn(const IContainerColumn* column)
{
   IMODTRACE_DEVELOP("CnrCtl::removeCol");
   IASSERTPARM(column!=0);
   unsigned long ulRemoveFlag = 0;
   PFIELDINFO pfldinfo = column->columnInfo();
   if(isVisible())
     ulRemoveFlag = CMA_INVALIDATE;

   long lCountRemaining = (long)sendEvent( CM_REMOVEDETAILFIELDINFO,
                                MPFROMP(&pfldinfo),
                                MPFROM2SHORT(1, (unsigned short)ulRemoveFlag));

   if(lCountRemaining==-1)
   {
      ITHROWGUIERROR("CM_REMOVEDETAILFIELDINFO");
   }
   ((IContainerColumn*)column)->setContainer(0);
   ulClColumnChanges++;
   /* Adjust the offset back to just the object */
   if(pfldinfo->offStruct < sizeof(IMiniCnrRecord))
      pfldinfo->offStruct+=IFIELDINBASE;
   else
      pfldinfo->offStruct-=baseRecordSize();
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: removeColumnAt

 Implementation: Remove a column from the container.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: removeColumnAt(IContainerControl::ColumnCursor& cursor)
{
   IASSERTSTATE(cursor.isValid());
   IContainerColumn* pcnrcol= cursor.current();
   IContainerColumn* pcnrcolNewCurrent = 0;
   // If there is a next item in the container, we want to update the cursor
   // to point to it.
   cursor.setToNext();
   if(cursor.isValid())
   {
     pcnrcolNewCurrent = cursor.current();
   }
   removeColumn(pcnrcol);
   // Note: remove will invalidate the cursor, setCurrent will revalidate
   if(pcnrcolNewCurrent)
     cursor.setCurrent(pcnrcolNewCurrent);
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: columnCount

 Implementation: Return the number of columns in the container.
------------------------------------------------------------------------------*/
unsigned long IContainerControl :: columnCount() const
{
  ICnrInfo cnrInfo;
  containerInfo(&cnrInfo);
  return cnrInfo.cFields;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: columnAt

 Implementation: Return the column at the zero-based index.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl :: columnAt(unsigned long ulIndex) const
{
  IContainerControl::ColumnCursor cnrcolcur(*this);
  IContainerColumn* pcnrcol;
  unsigned long ulCount = 0;
  while((pcnrcol=cnrcolcur.next())!=0)
  {
    if(ulCount==ulIndex)
      return(pcnrcol);
    ulCount++;
  }
  return 0;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: columnAt

 Implementation: Return the column at the passed cursor.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl :: columnAt(const ColumnCursor& cursor) const
{
  IASSERTSTATE(cursor.isValid());
  return cursor.current();
}



/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: isColumnRight

 Implementation: Determine if this column is in Right Split Details window.
------------------------------------------------------------------------------*/
bool IContainerControl :: isColumnRight(const IContainerColumn* column) const
{
#ifdef IC_PMWIN
  IASSERTPARM(column!=0);

  IContainerControl::ColumnCursor cnrcolcur(*this);
  IContainerColumn* pcnrcol = detailsViewSplit();

  if(pcnrcol!=0)
  {
    cnrcolcur.setCurrent(pcnrcol);
    for(cnrcolcur.setToNext();
        cnrcolcur.isValid();
        cnrcolcur.setToNext())
    {
      pcnrcol = columnAt(cnrcolcur);
      if(pcnrcol == column)
        return(true);
    }
  }
  return(false);
#endif  //IC_PMWIN
#ifdef IC_MOTIF
  return false;
#endif
}

