// Revision: 91 1.6.1.2 source/ui/cnr/icnrctla.cpp, container, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: icnrctla.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the text editing functions for    *
*   the class IContainerControl, declared in icnrctl.hpp.                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDCNR

#include <icnrrec.hpp>  // Must be first for OS flags


#include <itrace.hpp>
#include <iexcept.hpp>
#include <icnrctl.hpp>
#include <icnrcol.hpp>
#include <ithread.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTLA_CPP_
  #include <ipagetun.h>
#endif


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setEditMLE

 Implementation: Store the MLE used to edit a column.  Zero for end of edit.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setEditMLE(IMultiLineEdit* pmle)
{
  pmleClEdit = pmle;
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: currentEditMLE

 Implementation: Retrieve the MLE used to edit a column.  Zero for end of edit.
------------------------------------------------------------------------------*/
IMultiLineEdit* IContainerControl :: currentEditMLE()  const
{
   return pmleClEdit;
}



/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setEditColumn

 Implementation: Store the Column over which editing is occurring.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setEditColumn(IContainerColumn* pcnrcol)
{
  pcnrcolClEdit = pcnrcol;
   return *this;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: currentEditColumn

 Implementation: Retrieve the column being edited.  Zero for end of edit.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl :: currentEditColumn()  const
{
   return pcnrcolClEdit;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setEditObject

 Implementation: Store the Object over which editing is occurring.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setEditObject(IContainerObject* pcnrobj)
{
  pcnrobjClEdit = pcnrobj;
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: editObject

 Implementation: Retrieve the object being edited.  Zero for end of edit.
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl :: currentEditObject()  const
{
   return pcnrobjClEdit;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: editContainerTitle

 Implementation: Open an MLE on the Container Title.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: editContainerTitle()
{
#ifdef IC_PMWIN
   //We need to make sure any posted CM_CLOSEEDIT is processed before we
   //continue with opening a new edit field.  Remove any from the queue
   //and call CM_CLOSEEDIT now.  We post this message in closeEdit().
   QMSG  qmsg;
   bool closeFound = false;
   while ( IPEEKMSG( IThread::current().anchorBlock(), &qmsg, handle(),
                     CM_CLOSEEDIT, CM_CLOSEEDIT, PM_REMOVE ) )
   {
     closeFound = true;
   }
   if ( closeFound )
   {
     sendEvent(CM_CLOSEEDIT, 0, 0);
   }
#endif // IC_PMWIN

   CNREDITDATA editData;
   editData.cb = sizeof(CNREDITDATA);
   editData.hwndCnr = handle();
   editData.pRecord = 0;
   editData.ppszText=0;
   editData.cbText=0;
   editData.id=CID_CNRTITLEWND;
   editData.pFieldInfo = 0;
   unsigned fSuccess = sendEvent(CM_OPENEDIT,  &editData, 0);
   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_OPENEDIT");
   }
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: editColumnTitle

 Implementation: Open an MLE on a Column Title.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: editColumnTitle(IContainerColumn* pcnrcol)
{
#ifdef IC_PMWIN
   //We need to make sure any posted CM_CLOSEEDIT is processed before we
   //continue with opening a new edit field.  Remove any from the queue
   //and call CM_CLOSEEDIT now.  We post this message in closeEdit().
   QMSG  qmsg;
   bool closeFound = false;
   while ( IPEEKMSG( IThread::current().anchorBlock(), &qmsg, handle(),
                     CM_CLOSEEDIT, CM_CLOSEEDIT, PM_REMOVE ) )
   {
     closeFound = true;
   }
   if ( closeFound )
   {
     sendEvent(CM_CLOSEEDIT, 0, 0);
   }
#endif //IC_PMWIN

   pcnrcolClEdit = pcnrcol;
   CNREDITDATA editData;
   editData.cb = sizeof(CNREDITDATA);
   editData.hwndCnr = handle();
   editData.pRecord = 0;
   editData.ppszText=0;
   editData.cbText=0;
   if(isColumnRight(pcnrcol))
     editData.id=CID_RIGHTCOLTITLEWND;
   else
     editData.id=CID_LEFTCOLTITLEWND;
   editData.pFieldInfo = pcnrcol->columnInfo();
   unsigned fSuccess = sendEvent(CM_OPENEDIT,  &editData, 0);
   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_OPENEDIT");
   }
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: editObject

 Implementation: Open an edit field on an object.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: editObject(IContainerObject* pcnrobj, IContainerColumn* pcnrcolIn)
{
#ifdef IC_PMWIN
   //We need to make sure any posted CM_CLOSEEDIT is processed before we
   //continue with opening a new edit field.  Remove any from the queue
   //and call CM_CLOSEEDIT now.  We post this message in closeEdit().
   QMSG  qmsg;
   bool closeFound = false;
   while ( IPEEKMSG( IThread::current().anchorBlock(), &qmsg, handle(),
                     CM_CLOSEEDIT, CM_CLOSEEDIT, PM_REMOVE ) )
   {
     closeFound = true;
   }
   if ( closeFound )
   {
     sendEvent(CM_CLOSEEDIT, 0, 0);
   }
#endif //IC_PMWIN

   pcnrcolClEdit = pcnrcolIn;
   CNREDITDATA editData;
   editData.cb = sizeof(CNREDITDATA);
   editData.hwndCnr = handle();
   editData.pRecord = (PRECORDCORE)IRecFromObj(pcnrobj);
   editData.ppszText=0;
   editData.cbText=0;
#ifdef IC_MOTIF
   editData.id=0;           // for Motif, cannot have id = widget id
#else                       
   editData.id=id();
#endif
   if(pcnrcolIn!=0)
    {
      editData.pFieldInfo = pcnrcolIn->columnInfo();
      IContainerColumn* pcnrcol;
      IContainerControl::ColumnCursor colCursor(*this);
      editData.id=CID_RIGHTDVWND;

      while((pcnrcol=colCursor.next())!=0)
      {
        if(pcnrcol==pcnrcolIn)
           editData.id=CID_LEFTDVWND;
        if(pcnrcol==detailsViewSplit())
           break;
      }
   }          // end if pcnrcolln

   unsigned fSuccess = sendEvent(CM_OPENEDIT,  &editData, 0);
   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_OPENEDIT");
   }
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: closeEdit

 Implementation: Close an MLE.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: closeEdit()
{
  //We now post this message instead of sending it.  This is because
  //a user may have a keyboard handler attached to the MLE.  Sending
  //this event will cause the MLE to be deleted before we return to
  //the handler, where a possible trap may occur.
  postEvent(CM_CLOSEEDIT, 0, 0);

  return *this;
}
