// Revision: 92 1.12.1.3 source/ui/cnr/icnrctlb.cpp, container, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icnrctlb.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrnhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINSTDCNR
extern "C" {
   #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <icnrctl.hpp>
#include <icnrnhdr.hpp>
#include <ievent.hpp>
#include <ictlevt.hpp>
#include <iwcname.hpp>
#include <icnrevt.hpp>
#include <inotifev.hpp>
#include <icnrrec.hpp>
#include <icconst.h>
#ifdef IC_WIN
#include <iclcnrw.h>
#endif
#ifdef IC_MOTIF
#include <icnrm.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTLB_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const IContainerControl::enterId
                                      = "IContainerControl::enter";
INotificationId const IContainerControl::selectId
                                      = "IContainerControl::select";
INotificationId const IContainerControl::removeId
                                      = "IContainerControl::remove";
INotificationId const IContainerControl::addId
                                      = "IContainerControl::add";
INotificationId const IContainerControl::titleId
                                      = "IContainerControl::title";
INotificationId const IContainerControl::titleVisibleId
                                      = "IContainerControl::titleVisible";
INotificationId const IContainerControl::detailsViewTitlesId
                                      = "IContainerControl::detailsViewTitles";

/*------------------------------------------------------------------------------
| IContainerControl::enableNotification                                        |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IContainerControl, does not send notifications to observer objects until it  |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new IContainerControlNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IContainerControlNotifyHandler::IContainerControlNotifyHandler               |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerControlNotifyHandler::IContainerControlNotifyHandler( )
  : IContainerControlNotifyHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IContainerControlNotifyHandler::~IContainerControlNotifyHandler              |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerControlNotifyHandler::~IContainerControlNotifyHandler( )
{ }

/*------------------------------------------------------------------------------
| IContainerControlNotifyHandler::dispatchHandlerEvent                         |
|                                                                              |
| This notifies IContainerControl Windows of the following events:             |
|                                                                              |
| title(Id)                                                                    |
------------------------------------------------------------------------------*/
bool IContainerControlNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  switch (anEvent.eventId())
  {
    case CM_SETCNRINFO:

      if (anEvent.parameter2() == CMA_CNRTITLE)
      {
        if (thisWindow)
        {
          /************************************************************/
          /* Post private message to delay notification until after   */
          /* the selection state has been modified                    */
          /************************************************************/
           char* pTitleText ;
#ifdef IC_PMWIN
           CNRINFO *pcnrInfo = (CNRINFO*)(anEvent.parameter1().asUnsignedLong());
           pTitleText = (char *) malloc(strlen(pcnrInfo->pszCnrTitle)+1) ;
           strcpy(pTitleText, pcnrInfo->pszCnrTitle) ;
#endif // IC_PMWIN
#ifdef IC_MOTIF
           ICnrInfo *pcnrInfo = (ICnrInfo*)(anEvent.parameter1().asUnsignedLong()); /* LJO */
           pTitleText = (char *) malloc(strlen(pcnrInfo->strCnrTitle)+1);                                   /* LJO */
           strcpy(pTitleText, pcnrInfo->strCnrTitle) ;
#endif // IC_MOTIF
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( pTitleText ), 
                                 IEventParameter2( IC_NOTDEL_CNR_TITLE ));
        }
      }
      break;

    /****************************************************************/
    /* This is the delayed notification message.  Check to see which*/
    /* notification should be sent to the observers based on info   */
    /* stored in parameter 2 of the private message.  Parameter 1   */
    /* contains the information passed in the original event that   */
    /* needs to be sent with the notification                       */
    /****************************************************************/
    case IC_UM_DELAYNOTIFY:

      if (thisWindow && (anEvent.parameter2() == IC_NOTDEL_CNR_TITLE))
      {
         char* pTitleText = (char *) anEvent.parameter1() ;
         thisWindow->notifyObservers(INotificationEvent(
                                     IContainerControl::titleId,
                                     *thisWindow, true,
                                     pTitleText));
         free(pTitleText) ; 
      }
      break;

    case WM_CONTROL:
    {                              // Allocate class name string.
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
#ifdef IC_PM
        if (className == WC_CONTAINER)
#endif // IC_PM
#ifdef IC_WIN
        if (className == WC_CONTAINER || className == WC_NATIVECONTAINER)
#endif // IC_WIN
        {
          if (ulMsg == CN_ENTER)
          {
            ICnrEnterEvent enterEvt(ctlevt);
            thisWindow->notifyObservers(INotificationEvent(
                                        IContainerControl::enterId,
                                        *thisWindow, true,
                                        (void*)&enterEvt));
          }
          else if (ulMsg == CN_EMPHASIS)
          {
            ICnrEmphasisEvent emphasisEvt(ctlevt);
            if (emphasisEvt.changed(IContainerObject::selected))
            {
              // The following code is added to ensure that only one selection 
              // notification occurs using a single selection container.  Because
              // the container typically selects another object when one is 
              // deleted, selection notification may occur for many objects in 
              // the container during deleteAllObjects. We've added code to 
              // these functions to identify the selected object. Here, 
              // if the delete/removeAll object (pcnrobjClSingleSelectionNotification),
              // is set, we only send notification if it is the same as the object 
              // in the ICnrEmpasisEvent.

              IContainerControl* cnr = dynamic_cast<IContainerControl*>( thisWindow );
              if(cnr && (cnr->pcnrobjClSingleSelectionNotification == 0 ||
                 cnr->pcnrobjClSingleSelectionNotification == emphasisEvt.object()))
                thisWindow->notifyObservers(INotificationEvent(
                                          IContainerControl::selectId,
                                          *thisWindow, true,
                                          (void*)&emphasisEvt));
            }
          }
        }
      }
      break;
    }

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
/* LJO deleted NOP Motif version (merged with above )*/
