// Revision: 96 1.9.1.4 source/ui/cnr/icnrctlw.hpp, container, ioc.v400, 000919 
/* NOSHIP */
#ifndef _ICNRCTLW_
   #define _ICNRCTLW_
/*******************************************************************************
* FILE NAME: icnrctlw.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*                                                                              *
* Implementation Notes:                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ihandler.hpp>

#ifdef IC_WIN

#include <icnrctl.hpp>
#include <ikeyhdr.hpp>
#include <icnrrec.hpp>
#include <icontrol.hpp>
#include <icnrcol.hpp>
#include <ipoint.hpp>
#include <ikeyset2.h>
#include <iksssts2.h>


extern "C"
{
  #include <iwindefs.h>
  #include <commctrl.h>
}

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)
#pragma info(none)

class INativeContainerControl;
class IContainerControl;
class ICnrPointerSet;
class IColumnSet;
class IColDeleteSet;
class IStaticText;
class IMultiLineEdit;
class ITimer;

#define IC_NATIVE_LISTVIEW   0x3000
#define IC_NATIVE_TREEVIEW   0x3001

class ICnrControlData
{
public:

  ICnrControlData( );
  ICnrControlData( IContainerControl* cnr,
                   unsigned long      ulStyle,
                   IContainerControl::Attribute attribute );
 ~ICnrControlData( );

enum ActiveView {
  tree,
  list,
  none
  };

ICnrControlData
 &setCnrInfo              ( ICnrInfo*     pcnrInfo,
                            unsigned long setFlag               ),
 &setImageSize            ( ),
 &setWindowAttr           ( unsigned long flags                 ),
 &setActiveView           ( ActiveView    activeView            ),
 &setImageListWithView    ( unsigned long flags                 ),
 &queryCnrInfo            ( ICnrInfo*     pInfo                 ),
 &setSelection            ( unsigned long selection             ),
 &migrateContainerObjects ( bool          fromListToTree = true ),
 &layoutActiveView        ( const ISize&  parentSize            );

int
 addIconToList            ( const IPointerHandle& iconHandle    ),
 addInUseToList           ( const IPointerHandle& iconHandle    );

ActiveView
  activeView              ( ) const;

static void
  registerContainer       ( );

static unsigned long
  convertToNativeStyle    ( unsigned long cclStyle,
                            bool          fListView );
void
  timerPopped             ( unsigned long timerId );

static const char * const
  className;

IContainerControl
  *pCnr;

unsigned long
  cclStyle;

bool
  pmCompatible,
  showTitle,
  showTitleSeparator,
  doubleClickOccurred,
  bCheckingDragStart,
  collapseFirstColumn,
  needArrange,
  useScaledImages,
  expandNotificationDone;

INativeContainerControl
  *pActiveview,
  *pListview,
  *pTreeview;

IStaticText
  *pTitle;

IContainerColumn
  *pEditColumn;

IContainerObject
  *pEditObject;

IMultiLineEdit
  *pDetailsEdit;

ITimer
  *fTimer;

HIMAGELIST
  largeImages,
  scaledImages,
  smallImages,
  noImages,
  inUseImage,
  scaledInUseImage,
  smallInUseImage;

ICnrInfo
  cnrInfo;

ActiveView
  av;

ICnrPointerSet
 *pointerSet;

IColumnSet
 *columnSet;

IColDeleteSet
 *columnsToDelete;

IContainerObject
 *rootObject( IContainerObject* treeObj );

IPoint
  pointLButtonDown;
POINT
  editPoint;
};


class INativeContainerControl  : public IControl
{
typedef IControl
  Inherited;
/*******************************************************************************
* The INaticeContainerControl is a control wrapper class for the Windows       *
* list view and tree view controls.                                            *
*******************************************************************************/

public:

  INativeContainerControl( unsigned  long windowId,
                           const     IWindowHandle& parent,
                           const     IWindowHandle& owner,
                           const     IRectangle& initial,
                           unsigned  long style,
                           bool      fCreateListView = true );
virtual
 ~INativeContainerControl ( );

/*--------------------------------- Private ----------------------------------*/

  INativeContainerControl ( const INativeContainerControl& );
INativeContainerControl
 &operator=               ( const INativeContainerControl& );

};

class INativeContainerHandler : public IHandler
{
typedef IHandler
  Inherited;
/*******************************************************************************
* The INaticeContainerHandler class translates OS/2 container messages         *
* and notifications to the Windows equivalent where possible.                  *
*******************************************************************************/
public:

/*------------------------ Constructors ----------------------------------------
| The only way to construct an instance of this class is to use the default.   |
------------------------------------------------------------------------------*/
  INativeContainerHandler( );
 ~INativeContainerHandler( );

/*-------------------------------- Accessors ---------------------------------*/
static INativeContainerHandler
 *defaultHandler ( );

protected:
/*----------------------------- Implementation ---------------------------------
| This function is used to implement the class:                                |
|   dispatchHandlerEvent - Routes the edit events to the appropriate event     |
|                          functions.                                          |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent     ( IEvent& event       );

IWindowHandle
  containerHandleFromEvent ( const IEvent& event );

};

class IMessageHandler : public IHandler
{
typedef IHandler
  Inherited;

public:

/*------------------------ Constructors ----------------------------------------
| The only way to construct an instance of this class is to use the default.   |
------------------------------------------------------------------------------*/
  IMessageHandler( );
 ~IMessageHandler( );

/*-------------------------------- Accessors ---------------------------------*/
static IMessageHandler
 *defaultHandler ( );

protected:
/*----------------------------- Implementation ---------------------------------
| This function is used to implement the class:                                |
|   dispatchHandlerEvent - Routes the edit events to the appropriate event     |
|                          functions.                                          |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent     ( IEvent& event       );
};

class IDetailsEditHandler : public IKeyboardHandler
{
/*******************************************************************************
* This handler is used to dismiss the direct edit control when editing         *
* columns in details view.                                                     *
*******************************************************************************/
typedef IKeyboardHandler
  Inherited;

public:
/*------------------------ Constructors ----------------------------------------
| The only way to construct an instance of this class is to use the default.   |
------------------------------------------------------------------------------*/
  IDetailsEditHandler ( );
virtual
 ~IDetailsEditHandler ( );

/*-------------------------------- Accessors ---------------------------------*/
static IDetailsEditHandler
 *defaultHandler ( );

bool
  virtualKeyPress     ( IKeyboardEvent& event );
};


/*------------------------------------------------------------------------------
| The following key set is used to map pointer handles to indexs into          |
| an image list.  This is necessary so that we the same pointer need not       |
| be inserted into an image list multiple times.                               |
------------------------------------------------------------------------------*/

class IColumnElement  {
public:
  IColumnElement ( IContainerColumn* pCol,
                   int               subItemIdentifier,
                   int               columnIdentifier,
                   int               visibilityIdentifier );
  IColumnElement()
    : pColumn(0), subItemId(0),columnId(0), visibilityId(0) {}
 ~IColumnElement ( );

IColumnElement
  &operator =    ( const IColumnElement&   element );
bool
  operator ==    ( const IColumnElement&   element ) const;
bool
  operator <     ( const IColumnElement&   element ) const;

IContainerColumn
  *pColumn;
int
  subItemId;
int
  columnId;
int
  visibilityId;
};

// typedef ISetOnSortedTabularSequence< IColumnElement > IColumnSet;

class IColumnSet : public IKeySortedSetOnSortedTabularSequence< IColumnElement, int >
{
typedef IKeySortedSetOnSortedTabularSequence< IColumnElement, int >
  Inherited;
public:

  IColumnSet( );
 ~IColumnSet( );

bool
  add                         ( IContainerColumn* pCol,
                                int               subItemIdentifier,
                                int               columnIdentifier,
                                bool              visible);
bool
  remove                      ( IContainerColumn* pCol,
                                bool              collapseColumns=true );

IContainerColumn*
  containerColumnFromSubId    ( int subItemIdentifier ) const;

int
  newSubItemId                ( ) const;

int
  columnIdFromContainerColumn ( IContainerColumn* pCol ) const;

IContainerColumn*
  containerColumnFromColumnId ( int columnId           ) const;

int
  columnCount                 ( ) const;

int
  columnIdFromFieldInfo       ( PFIELDINFO        pFI  ) const;

int
  subItemIdFromFieldInfo      ( PFIELDINFO        pFI  ) const;

int
  lastSubItemId               ( ) const;

LPFIELDINFO
  firstFieldInfo              ( ) const;

LPFIELDINFO
  lastFieldInfo               ( ) const;

LPFIELDINFO
  nextFieldInfo               ( LPFIELDINFO       pFI  ) const;

LPFIELDINFO
  prevFieldInfo               ( LPFIELDINFO       pFI  ) const;

private:
int
  nextInvalidSubId;
};

inline int const& key (IColumnElement const& k)
{
  return k.subItemId;
}

class IColDeleteSet : public IKeySet< IColumnElement, int >
{
typedef IKeySet< IColumnElement, int >
  Inherited;
public:

  IColDeleteSet( );
 ~IColDeleteSet( );

Boolean
  add                         ( IContainerColumn* pCol ),
  remove                      ( IContainerColumn* pCol );

void
  deleteAll                   ( );
};


class IPointerElement {
public:
  IPointerElement  ( const IPointerHandle& pointer,
                     unsigned long         index    );
  ~IPointerElement ( );

  IPointerHandle pointerHandle;
  int  imageListIndex;
  int  inUseIndex;
  unsigned long lph;

private:
  friend unsigned long const& key ( IPointerElement* const& k );
  IPointerElement ( const IPointerElement& );
  IPointerElement
    &operator =   ( const IPointerElement& );
};

class ICnrPointerSet : public IKeySet < IPointerElement*, unsigned long >
{
public:
  ICnrPointerSet();
  ~ICnrPointerSet();

ICnrPointerSet
  &addPointerElement( const IPointerHandle& pointer,
                      int                   imageIndex );
};

inline unsigned long const& key (IPointerElement* const& k)
{
  return k->lph;
}

#pragma pack(pop)
#pragma enum(pop)

#else

#pragma enum(4)
#pragma pack(push,4)

class ICnrControlData
{
public:

  ICnrControlData( );
 ~ICnrControlData( );

bool
  pmCompatible;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif

#endif  /* _ICNRCTLW_ */
