// Revision: 98 1.4.1.2 source/ui/cnr/icnrdiev.cpp, container, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: icnrdiev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrdiev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <stdio.h>
  #define INCL_WINSTDCNR
  #define INCL_WINWINDOWMGR
  #define INCL_WINMENUS
  }

#include <icnrrec.hpp>  // Must be first for OS flags

#include <icnrdiev.hpp>
#include <icnrctl.hpp>
#include <irect.hpp>


// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICNRDIEV_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::~ICnrDrawItemEvent                                        |
------------------------------------------------------------------------------*/
ICnrDrawItemEvent  :: ~ICnrDrawItemEvent( ) {}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::~ICnrDrawBackgroundEvent                            |
------------------------------------------------------------------------------*/
ICnrDrawBackgroundEvent :: ~ICnrDrawBackgroundEvent( ) {}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::ICnrDrawItemEvent                                         |
------------------------------------------------------------------------------*/
ICnrDrawItemEvent ::  ICnrDrawItemEvent(const IEvent& evt)
    : IDrawItemEvent(evt)
{
  ITRACE_MOTIF_NOP();
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::ICnrDrawBackgroundEvent                             |
------------------------------------------------------------------------------*/
ICnrDrawBackgroundEvent :: ICnrDrawBackgroundEvent(const IEvent& evt)
    : IEvent(evt)
{
  ITRACE_MOTIF_NOP();
}


/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::container                                                 |
|                                                                              |
| Returns the container being drawn.                                           |
------------------------------------------------------------------------------*/
IContainerControl* ICnrDrawItemEvent :: container  ( ) const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   HWND hwndCtl =  IWindow::handleWithParent(parameter1(), handle());
   IContainerControl* pcnrctl =
                    (IContainerControl*)IWindow::windowWithHandle(
                                                     IWindowHandle(hwndCtl));
   return pcnrctl;
#endif
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::object                                                    |
|                                                                              |
| Returns the object in the container being drawn.                             |
| If a column title is being drawn in detail view, 0 is returned.              |
------------------------------------------------------------------------------*/
IContainerObject* ICnrDrawItemEvent :: object  ( ) const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
#ifdef IC_WIN
   LPCNRDRAWITEMINFO pditeminfo = (LPCNRDRAWITEMINFO)((LPDRAWITEMSTRUCT)
                                            (void*)parameter2())->itemData;
#endif
#ifdef IC_PM
   PCNRDRAWITEMINFO pditeminfo = (PCNRDRAWITEMINFO)((POWNERITEM)
                                                   (void*)parameter2())->hItem;
#endif

  IMiniCnrRecord* pcnrrec =
    ( pditeminfo == NULL ? NULL : (IMiniCnrRecord*)(pditeminfo->pRecord));

   return (IObjFromRec(pcnrrec));
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::column                                                    |
|                                                                              |
| If an object being drawn is in the details view, the column is returned.     |
| If the column title is being drawn in details view, the column is returned.  |
| If the view is other than a details view, 0 is returned.                     |
------------------------------------------------------------------------------*/
IContainerColumn* ICnrDrawItemEvent :: column ( ) const
{
  ITRACE_MOTIF_NOP();
  IContainerColumn* pcnrcol=0;

#ifdef IC_PMWIN
#ifdef IC_WIN
   LPCNRDRAWITEMINFO pditeminfo = (LPCNRDRAWITEMINFO)((LPDRAWITEMSTRUCT)
                                            (void*)parameter2())->itemData;
#endif
#ifdef IC_PM
  PCNRDRAWITEMINFO pditeminfo =
                   (PCNRDRAWITEMINFO)((POWNERITEM)(void*)parameter2())->hItem;
#endif

  if ( pditeminfo && pditeminfo->pFieldInfo )
    pcnrcol = (IContainerColumn*)(pditeminfo->pFieldInfo->pUserData);

  return (pcnrcol);
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::itemType                                                  |
|                                                                              |
| Returns the type of the item being drawn.                                    |
------------------------------------------------------------------------------*/
ICnrDrawItemEvent::ItemType ICnrDrawItemEvent :: itemType () const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
#ifdef IC_WIN
   LPDRAWITEMSTRUCT pitem = (LPDRAWITEMSTRUCT)(void*)parameter2();
   long itemId = pitem->itemID;
#endif
#ifdef IC_PM
   POWNERITEM pitem = (POWNERITEM)(void*)parameter2();
   long itemId = pitem->idItem;
#endif

   ItemType type;

   switch(itemId)
   {
     case CMA_TEXT :
     {
        type = text;
        break;
     }
     case CMA_ICON :
     {
        type = icon;
        break;
     }
     case CMA_TREEICON :
     {
        type = treeIcon;
        break;
     }
     case CMA_CNRTITLE :
     {
        type = title;
        break;
     }

     default :
     {
        type = detailsView;
        break;
     }
   }
   return type;
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return text;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::itemPresSpaceHandle                                       |
|                                                                              |
| Returns the item's presentation space handle.                                |
------------------------------------------------------------------------------*/
IPresSpaceHandle ICnrDrawItemEvent::itemPresSpaceHandle() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  if ( (void*)parameter2() )
  {
#ifdef IC_WIN
    LPDRAWITEMSTRUCT pitem = (LPDRAWITEMSTRUCT)(void*)parameter2();
    return IPresSpaceHandle( pitem->hDC, this->container()->handle() );
#endif
#ifdef IC_PM
    POWNERITEM pitem = (POWNERITEM)(void*)parameter2();
    return IPresSpaceHandle( pitem->hps, this->container()->handle() );
#endif
  }
  return IPresSpaceHandle();
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return IPresSpaceHandle();
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::itemRect                                                  |
|                                                                              |
| Returns the rectangle of the item.                                           |
------------------------------------------------------------------------------*/
IRectangle ICnrDrawItemEvent:: itemRect() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  if ( (void*)parameter2() )
  {
#ifdef IC_WIN
    LPDRAWITEMSTRUCT pitem = (LPDRAWITEMSTRUCT)(void*)parameter2();
    return IRectangle(pitem->rcItem);
#endif
#ifdef IC_PM
    POWNERITEM pitem = (POWNERITEM)(void*)parameter2();
    return IRectangle(pitem->rclItem);
#endif
  }
  return IRectangle();
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return IRectangle();
#endif
}


/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::container                                           |
|                                                                              |
| Returns the container being drawn.                                           |
------------------------------------------------------------------------------*/
IContainerControl* ICnrDrawBackgroundEvent :: container  ( ) const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   IContainerControl* pcnrctl =
        (IContainerControl*)IWindow::windowWithHandle(IWindowHandle(handle()));
   return pcnrctl;
#endif
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::itemPresSpaceHandle                                 |
|                                                                              |
| Returns the item's presentation space handle.                                |
------------------------------------------------------------------------------*/
IPresSpaceHandle ICnrDrawBackgroundEvent::itemPresSpaceHandle() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  if ( (void*)parameter1() )
  {
    POWNERBACKGROUND pback = (POWNERBACKGROUND)(void*)parameter1();
    return IPresSpaceHandle( pback->hps, this->container()->handle() );
  }
  return IPresSpaceHandle();
#endif
#ifdef IC_MOTIF
  return IPresSpaceHandle();
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::itemId                                              |
|                                                                              |
| Returns the item's id.                                                       |
------------------------------------------------------------------------------*/
unsigned long ICnrDrawBackgroundEvent::itemId() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  if ( (void*)parameter1() )
  {
    POWNERBACKGROUND pback = (POWNERBACKGROUND)(void*)parameter1();
    return (unsigned long) (pback->idWindow);
  }
  return 0;
#endif
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::itemRect                                            |
|                                                                              |
| Returns the rectangle of the item.                                           |
------------------------------------------------------------------------------*/
IRectangle ICnrDrawBackgroundEvent:: itemRect() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  if ( (void*)parameter1() )
  {
    POWNERBACKGROUND pback = (POWNERBACKGROUND)(void*)parameter1();
    return IRectangle(pback->rclBackground);
  }
  return IRectangle();
#endif
#ifdef IC_MOTIF
  return IRectangle();
#endif
}
