// Revision: 00 1.3.2.2 source/ui/cnr/icnrdihd.cpp, container, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: icnrdihd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrdihd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <stdio.h>
  #define INCL_WINSTDCNR
  #define INCL_WINWINDOWMGR
  #include <icconst.h>
  }

#include <icnrrec.hpp>  // Must be first for OS flags

#include <icnrdihd.hpp>
#include <icnrctl.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRDIHD_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ICnrDrawHandler::ICnrDrawHandler                                             |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
ICnrDrawHandler::ICnrDrawHandler()
{
   ITRACE_MOTIF_NOP();
   IMODTRACE_DEVELOP("CnrDrawHdr::CnrDrawHdr");
}

/*------------------------------------------------------------------------------
| ICnrDrawHandler::~ICnrDrawHandler                                            |
------------------------------------------------------------------------------*/
ICnrDrawHandler :: ~ICnrDrawHandler   ( ) {}

/*------------------------------------------------------------------------------
| ICnrDrawHandler::dispatchHandlerEvent                                        |
|                                                                              |
| Routes the draw events to the appropriate event functions.                   |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler::dispatchHandlerEvent(IEvent& evt)
{
   ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   bool fStopDispatch = false;
   if(evt.eventId() == WM_DRAWITEM)
   {
      ICnrDrawItemEvent dievt(evt);
      switch (dievt.itemType())
      {
          case ICnrDrawItemEvent::text:
          {
             fStopDispatch = drawText(dievt);
             break;
          }
          case ICnrDrawItemEvent::icon:
          {
             fStopDispatch = drawIcon(dievt);
             break;
          }
          case ICnrDrawItemEvent::treeIcon:
          {
             fStopDispatch = drawTreeIcon(dievt);
             break;
          }
          case ICnrDrawItemEvent::detailsView:
          {
             fStopDispatch = drawDetailsItem(dievt);
             break;
          }
          case ICnrDrawItemEvent::title:
          {
             fStopDispatch = drawTitle(dievt);
             break;
          }
      } /* endswitch */
      evt.setResult(dievt.result());
   }
   else if (evt.eventId() == CM_PAINTBACKGROUND)
   {
      ICnrDrawBackgroundEvent dbevt(evt);
      fStopDispatch = drawBackground (dbevt);
      evt.setResult(dbevt.result());
   } /* endelsif */

   return fStopDispatch;
#endif //IC_PMWIN
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawHandler::drawText                                                    |
|                                                                              |
| Provide default behavior for drawText.                                       |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler :: drawText(ICnrDrawItemEvent& evt)
       { return false; }

/*------------------------------------------------------------------------------
| ICnrDrawHandler::drawIcon                                                    |
|                                                                              |
| Provide default behavior for drawIcon.                                       |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler :: drawIcon(ICnrDrawItemEvent& evt)
       { return false; }

/*------------------------------------------------------------------------------
| ICnrDrawHandler::drawTreeIcon                                                |
|                                                                              |
| Provide default behavior for drawTreeIcon.                                   |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler :: drawTreeIcon(ICnrDrawItemEvent& evt)
       { return false; }

/*------------------------------------------------------------------------------
| ICnrDrawHandler::drawDetailsItem                                             |
|                                                                              |
| Provide default behavior for drawDetailsItem.                                |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler :: drawDetailsItem(ICnrDrawItemEvent& evt)
       { return false; }

/*------------------------------------------------------------------------------
| ICnrDrawHandler::drawTitle                                                   |
|                                                                              |
| Provide default behavior for drawTitle.                                      |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler :: drawTitle(ICnrDrawItemEvent& evt)
       { return false; }

/*------------------------------------------------------------------------------
| ICnrDrawHandler::drawBackground                                              |
|                                                                              |
| Provide default behavior for drawBackground.                                 |
------------------------------------------------------------------------------*/
bool ICnrDrawHandler :: drawBackground(ICnrDrawBackgroundEvent& evt)
       { return false; }


/*------------------------------------------------------------------------------
| ICnrDrawHandler::handleEventsFor                                             |
|                                                                              |
| Attaches the handler to the container control passed in the argument.        |
------------------------------------------------------------------------------*/
ICnrDrawHandler& ICnrDrawHandler :: handleEventsFor(
                                               IContainerControl* container )
{
  ITRACE_MOTIF_NOP();
  IASSERTPARM(container != 0);
#ifdef IC_PMWIN
  IHandler::handleEventsFor(container);
  return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawHandler::stopHandlingEventsFor                                       |
|                                                                              |
| Detaches the handler from the container control passed in the argument.      |
------------------------------------------------------------------------------*/
ICnrDrawHandler& ICnrDrawHandler :: stopHandlingEventsFor(
                                               IContainerControl* container )
{
  ITRACE_MOTIF_NOP();
  IASSERTPARM(container != 0);
#ifdef IC_PMWIN
  IHandler::stopHandlingEventsFor(container);
  return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

/*------------------------------------------------------------------------------
| ICnrDrawHandler::handleEventsFor                                             |
------------------------------------------------------------------------------*/
IHandler& ICnrDrawHandler :: handleEventsFor( IWindow* window )
{
    ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);

    return *this;
}

/*------------------------------------------------------------------------------
| ICnrDrawHandler::stopHandlingEventsFor                                       |
------------------------------------------------------------------------------*/
IHandler& ICnrDrawHandler :: stopHandlingEventsFor( IWindow* window )
{
    ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
    return *this;
}
