// Revision: 02 1.6.1.2 source/ui/cnr/icnreevt.cpp, container, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icnreevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*  This file contains the implementation of the following Edit Event classes:  *
*      ICnrBeginEditEvent                                                      *
*      ICnrEndEditEvent                                                        *
*      ICnrReallocStringEvent                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ilanglvl.hpp>

#define INCL_WINSTDCNR
#ifdef IC_MOTIF
  #include <cnr.h>
  #include <icong.h>
#endif
#include <icnrrec.hpp>  // Must be first for OS flags

#include <iexcept.hpp>
#include <itrace.hpp>
#include <icnrobj.hpp>
#include <icnreevt.hpp>
#include <icnrctl.hpp>
#include <iwindow.hpp>
#include <imle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNREEVT_CPP_
  #include <ipagetun.h>
#endif

// Stop the noise
ICnrEditEvent::~ICnrEditEvent  ( ) {}
ICnrReallocStringEvent::~ICnrReallocStringEvent ( ) {}
ICnrBeginEditEvent::~ICnrBeginEditEvent ( ) {}
ICnrEndEditEvent::~ICnrEndEditEvent ( ) {}

ICnrEditEvent:: ICnrEditEvent(const IControlEvent& cpszevt)
              : ICnrEvent(cpszevt) {}

ICnrReallocStringEvent::ICnrReallocStringEvent(const IControlEvent& cpszevt)
              : ICnrEditEvent(cpszevt) {}

ICnrBeginEditEvent::ICnrBeginEditEvent(const IControlEvent& evt)
                    : ICnrEditEvent(evt) {}

ICnrEndEditEvent::ICnrEndEditEvent(const IControlEvent& evt)
              : ICnrEditEvent(evt), strNewText(0) {}

/*-----------------------------------------------------------------------------
 ICnrReallocStringEvent::reallocText

 Allocate storage for new string.
------------------------------------------------------------------------------*/
void ICnrReallocStringEvent::reallocateText()
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if(editData)
  {
    delete(*editData->ppszText);
    *editData->ppszText = (PSZ)( new unsigned char[editData->cbText]);
  }
}

/*-----------------------------------------------------------------------------
 ICnrReallocStringEvent::reallocString

 Allocate storage for new IString.
------------------------------------------------------------------------------*/
void ICnrReallocStringEvent::reallocateString()
{
  IMODTRACE_ALL("CnrRealloc::reallocateString");

  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if(editData)
  {
    IString* pstr = (IString*)editData->ppszText;
    IString strNew(*editData->ppszText, textSize());
    *pstr = strNew;
  }
}

/*-----------------------------------------------------------------------------
 ICnrReallocStringEvent::newText

------------------------------------------------------------------------------*/
IString ICnrReallocStringEvent::newText()  const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if (editData)
  {
    IContainerControl* pcnrctl = (IContainerControl*)IWindow::windowWithHandle(editData->hwndCnr);
    if (pcnrctl)
    {
      IMultiLineEdit* pmle = pcnrctl->currentEditMLE();
      if(pmle)
         return  pmle->text();
    }
  }
  return IString();
}


/*-----------------------------------------------------------------------------
 ICnrEditEvent::isDetailsData

 Return true if editing column data (column exists).
------------------------------------------------------------------------------*/
bool ICnrEditEvent::isDetailsData() const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if(editData)
  {
    // If we are in the details view, have a column and are not in
    // the column heading, then return true to indicate we are in
    // the data of a column
    IContainerControl* pcnrctl = (IContainerControl*)IWindow::windowWithHandle(editData->hwndCnr);
    if(pcnrctl && pcnrctl->isDetailsView() &&
         editData->id != CID_RIGHTCOLTITLEWND  &&
         editData->id != CID_LEFTCOLTITLEWND  &&
         editData->pFieldInfo != 0)
      return true;
  }
  return false;

}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::isTitleWindow

------------------------------------------------------------------------------*/
bool ICnrEditEvent::isTitleWindow() const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if (editData && editData->id == CID_CNRTITLEWND)
    return true;
  else
    return false;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::isLeftDetailsHeading

------------------------------------------------------------------------------*/
bool ICnrEditEvent::isLeftDetailsHeading() const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if (editData && editData->id == CID_LEFTCOLTITLEWND)
    return true;
  else
    return false;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::isRightDetailsHeading

------------------------------------------------------------------------------*/
bool ICnrEditEvent::isRightDetailsHeading() const
{
#ifdef IC_PMWIN
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if (editData && editData->id == CID_RIGHTCOLTITLEWND)
    return true;
  else
    return false;
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::isLeftDetails

------------------------------------------------------------------------------*/
bool ICnrEditEvent::isLeftDetails() const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if (editData && editData->id == CID_LEFTDVWND)
    return true;
  else
    return false;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::isRightDetails

------------------------------------------------------------------------------*/
bool ICnrEditEvent::isRightDetails() const
{
#ifdef IC_PMWIN
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if (editData && editData->id == CID_RIGHTDVWND)
    return true;
  else
    return false;
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::textRef

------------------------------------------------------------------------------*/
char**    ICnrEditEvent::textRef()  const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  IASSERTSTATE(editData != 0 && editData->ppszText != 0);
  return editData->ppszText;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::textSize

------------------------------------------------------------------------------*/
unsigned long ICnrEditEvent::textSize() const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if(editData && editData->cbText>0)
    return editData->cbText - 1;
  else
    return 0;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::object

 Return the object being edited
------------------------------------------------------------------------------*/
IContainerObject* ICnrEditEvent::object()  const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if(editData && editData->pRecord)
    return IObjFromRec((IMiniCnrRecord*)editData->pRecord);
  return 0;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::column

 Return the column being edited
------------------------------------------------------------------------------*/
IContainerColumn* ICnrEditEvent::column()  const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  IContainerControl* pcnrctl = (IContainerControl*)IWindow::windowWithHandle(editData->hwndCnr);
  if(pcnrctl && pcnrctl->isDetailsView() && editData->pFieldInfo != 0)
    return (IContainerColumn*)(editData->pFieldInfo->pUserData);
  return 0;
}

/*-----------------------------------------------------------------------------
 ICnrEditEvent::container

 Return the container dispatching this event
------------------------------------------------------------------------------*/
IContainerControl* ICnrEditEvent::container()  const
{
  PCNREDITDATA editData = (PCNREDITDATA)(void*)parameter2();
  if(editData)
  {
    IContainerControl* container = IContainerControl::containerFromHandle(
                                   editData->hwndCnr);
    return container;
  }
  return 0;
}


/*-----------------------------------------------------------------------------
 ICnrReallocStringEvent::currentText

------------------------------------------------------------------------------*/
IString ICnrReallocStringEvent::currentText()  const
{
  return IString(*textRef());
}

/*-----------------------------------------------------------------------------
 ICnrBeginEditEvent::currentText

------------------------------------------------------------------------------*/
IString ICnrBeginEditEvent::currentText()
{
  return IString(*textRef());
}

/*-----------------------------------------------------------------------------
 ICnrEndEditEvent::newText

------------------------------------------------------------------------------*/
IString ICnrEndEditEvent::newText()
{
  // This returns the text that was entered in the MLE
  // which may not be the newText if the realloc was cancelled
  return strNewText;
}

/*-----------------------------------------------------------------------------
 ICnrEndEditEvent::currentText

------------------------------------------------------------------------------*/
IString ICnrEndEditEvent::currentText()
{
  return IString(*textRef());
}

/*-----------------------------------------------------------------------------
 ICnrReallocStringEvent::newTextSize

------------------------------------------------------------------------------*/
long ICnrReallocStringEvent::newTextSize() const
{
  return textSize();
}

