// Revision: 23 1.4.2.4 source/ui/cnr/icnrobjn.cpp, container, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icnrobjn.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the non-inlined, non-memory-debug implementations of    *
*   operators new and delete for IContainerObject (see icnrobj.hpp).           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <icnrobj.hpp>
#include <icconst.h>
#include <icnrrec.hpp>
#include <iexcept.hpp>

#ifndef __DEBUG_ALLOC__
// This file provides the non-inlined, non-__DEBUG_ALLOC__
// implementations of the operator new and delete for IContainerObject.
// The non-memory-debug version of the same operators is provided in
// icnrobjd.cpp.
// This file (icnrobjn.cpp) exists so that the application can build
// with /Tm-, with or without I_NO_INLINES, and link dynamically to the
// class library DLL without encountering an unresolved external, even
// if the class library code is built with /Tm+.
//
// This file must be compiled with /Tm-.  This compiler option is forced
// via the profile used to generate the makefile, which in turn is used
// used to compile this file.

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICNROBJN_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IContainerObject::operator new                                               |
|                                                                              |
| Called when an IContainerObject is "new-ed" using an ICnrAllocator.          |
------------------------------------------------------------------------------*/
void* IContainerObject::operator new ( size_t size,
                                       ICnrAllocator& allocator )
{
  void* pcnrRec = allocator.nextAvailable();
  IContainerObject* pcnrobj;

  if ( pcnrRec )
  {
     allocator.numInitialized++;
     pcnrobj = IObjFromRec( (IMiniCnrRecord*)(void*)pcnrRec );

     if ( size > sizeof( IContainerObject ))
     {
        pcnrobj->flClState = subclassed;
     }
  }
  else
  {
     ITHROWLIBRARYERROR( IC_CNRALLOCATOR_LIMIT_EXCEEDED,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  }
  return pcnrobj;
}

#else  // ! __DEBUG_ALLOC__
#ifdef IC_PMWIN
  #error Error: icnrobjn.cpp must be compiled with /Tm-. Check the makefile (or its profile) for a missing /Tm- option.
#endif
#ifdef IC_MOTIF
  #error Error: icnrobjn.cpp must be compiled with -qheapdebug on. Check the makefile (or its profile) for a missing /Tm- option.
#endif

#endif // __DEBUG_ALLOC__ 
