// Revision: 87 1.4.1.5 source/ui/cnr/icnrobjp.hpp, container, ioc.v400, 000919 
/* NOSHIP */
#ifndef _ICNROBJP_
#define _ICNROBJP_
/*******************************************************************************
* FILE NAME: icnrobjp.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*                                                                              *
* Implementation Notes:                                                        *
*   Private data classes for IContainerObject .                                *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#ifdef IC_WIN
#include <iset2.h>
extern "C"
{
  #include <iwindefs.h>
  #include <commctrl.h>
}
#endif // IC_WIN

#ifdef IC_MOTIF
#include <ikeyset.h>
#include <icnrrec.hpp>
#endif // IC_MOTIF

#include <icnrctl.hpp>
#include <ihandle.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)
#pragma info(none)

#ifdef IC_MOTIF
class IContainerControl;
class IContainerObject;

/*------------------------------------------------------------------------------
| ContainedInElement class                                                     |
------------------------------------------------------------------------------*/
class ContainedInElement
{
public:
  ContainedInElement (IContainerControl* aContainer,
                      IWindowHandle anIconGadget,
                      IContainerObject* pObject)   ;

  ~ContainedInElement ( ) ;

  IContainerControl* const& getKey() const ;
//  data
  IWindowHandle iconGadget;
  IContainerControl* containerKey;
  IContainerObject*   pCnrObject ;
  ContainedInElement* pParentElement;
  ContainedInElement* pPreviousElement;
  ContainedInElement* pNextElement;
  ContainedInElement* pFirstChildElement;
  ContainedInElement* pLastChildElement;

};

inline IContainerControl* const& key ( ContainedInElement* const& element )
{ return element->getKey(); }

typedef IKeySet<ContainedInElement*, IContainerControl*> ContainedInListType;
#endif // IC_MOTIF

#ifdef IC_WIN
class ICnrObjSetInfo  {
public:
  ICnrObjSetInfo ( IContainerControl* pCnrCtl,
                   HTREEITEM          pTreeViewItem,
                   int                imageListIndex );
 ~ICnrObjSetInfo ( );
  ICnrObjSetInfo ( const ICnrObjSetInfo& object      );

ICnrObjSetInfo
  &operator=     ( const ICnrObjSetInfo& object      );

ICnrObjSetInfo
  &setObjectState( unsigned long     stateOff,
                   unsigned long     stateOn,
                   IContainerObject* pCnrObj=0       );

IContainerControl
  *pContainer;
HTREEITEM
  pTVItem;
int
  imageIndex;
unsigned long
  objectState;
};

class ICnrObjSet : public ISet < ICnrObjSetInfo* > {
public:
  ICnrObjSet  ();
 ~ICnrObjSet  ();

HTREEITEM
  treeItemFromObject ( IContainerControl* pCnrCtl  );

bool
  containsObject     ( IContainerControl* pCnrCtl  ),
  remove             ( IContainerControl* pCnrCtl  ),
  isDropOnable       ( IContainerControl* pCnrCtl  ),
  isReadOnly         ( IContainerControl* pCnrCtl  ),
  isInUse            ( IContainerControl* pCnrCtl  );

ICnrObjSet&
  updateTreeItem     ( IContainerControl* pCnrCtl,
                       HTREEITEM treeItem          );

int
  imageIndexFromCnr  ( IContainerControl* pCnrCtl  );

ICnrObjSetInfo*
  objectFromCnr      ( IContainerControl* pCnrCtl  );
};
#endif // IC_WIN


class ICnrObjPrivateData {
public:
  // member functions
  ICnrObjPrivateData(IContainerObject* newObject);
  ~ICnrObjPrivateData();

#ifdef IC_WIN
  static int
     listItemFromCnr( const IContainerControl* pCnrCtl,
                      const IContainerObject*  pObject,
                      int   startFrom = -1              );
#endif // IC_WIN
#ifdef IC_MOTIF
  void addToCnr(IContainerControl* addTo,
                RECORDINSERT* precordinsert);

  void removeFromCnr(IContainerControl* removeFrom,
                     unsigned long flags);

  void updateEntryDetail(IContainerControl* activeCnr);

  void setRefreshOn(bool bRefreshState) ;

  void removeFromCollection(IContainerControl* pCnr) ;
  void appendToList(ContainedInElement* pObj, IContainerControl* pCnr) ;
  void insertToList(ContainedInElement* pObj, IContainerControl* pCnr) ;
  ContainedInElement* lastChild(IContainerControl* pCnr) ;
  ContainedInElement* firstChild(IContainerControl* pCnr) ;
  IContainerObject* nextObject(IContainerControl* pCnr) ;
  IContainerObject* previousObject(IContainerControl* pCnr) ;
  IContainerObject* firstChildObject(IContainerControl* pCnr) ;
  IContainerObject* lastChildObject(IContainerControl* pCnr) ;

#endif // IC_MOTIF

// data
IContainerObject*
  pObject;
IPointerHandle
  hptrInUseIcon ;
IPointerHandle
  hptrRegularIcon ;
#ifdef IC_WIN
ICnrObjSet
  *pCnrSet;
#endif // IC_WIN
#ifdef IC_MOTIF
ContainedInListType containedInList;
#endif // IC_MOTIF
};

#ifdef IC_MOTIF
// Return the IconGadget that represents pObj in pCnr
IWindowHandle IGadgetFromObj(IContainerObject*  pObj,
                             IContainerControl* pCnr,
                             bool throwIfNotFound = true);
ContainedInElement* IContainedInElementFromObj(IContainerObject* pCnrObj,
                                              IContainerControl* pCnr) ;

IContainerObject* IObjFromGadget(IWindowHandle iconG);
#endif // IC_MOTIF

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICNROBJP_



