// Revision: 28 1.3.2.3 source/ui/cnr/icnrocur.cpp, container, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: icnrocur.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrctl.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINSTDCNR
extern "C" {
  #include <string.h>
#ifdef IC_WIN
#ifndef IC_WU
  //Don't have this header
  //#include <commctl.h>
#endif  // !IC_WU
#endif
  }

#include <icnrrec.hpp>  // Must be first for OS flags

#include <icnrctl.hpp>
#ifdef IC_PMWIN
  #include <icnrctlw.hpp>
#endif
#include <icnrobj.hpp>
#include <iexcept.hpp>
#include <icnrrec.hpp>

#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNROCUR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::ObjectCursor                                |
|                                                                              |
| Constructor to create a cursor to iterate all records in the container or    |
| records meeting the emphasis selection criteria.  Records can be found in    |
| item order (the default) or z-order.                                         |
------------------------------------------------------------------------------*/
IContainerControl::ObjectCursor::ObjectCursor(
                         const IContainerControl& cnrctl,
                         IContainerObject::Emphasis emphasis,
                         IContainerControl::EnumerationOrder Enumeration)
                : ulClChanges(cnrctl.numberOfObjectChanges()),
                  pcnrctlClOwner ((IContainerControl*)&cnrctl),
                  pcnrobjClCurrent(0),
                  pcnrobjClParent(0),
                  flClEmphasis(0),
#ifdef IC_PMWIN
                  currentIndex(0),
#endif
                  flClState(all)
{
      /**************************************************/
      /* Normal or selected iterator                    */
      /* CRA_CURSORED, CRA_INUSE, and CRA_SELECTED are  */
      /* also valid for emphasis(default is all records)*/
      /* CMA_ZORDER is a valid enumeration in ICON View */
      /**************************************************/
  if(emphasis!=IContainerObject::none)
    flClState = IContainerControl::ObjectCursor::emphasis;

  if(emphasis & IContainerObject::cursored)
     flClEmphasis |= CRA_CURSORED;

  if(emphasis & IContainerObject::inuse)
     flClEmphasis |= CRA_INUSE;

  if(emphasis & IContainerObject::selected)
     flClEmphasis |= CRA_SELECTED;


  if(Enumeration==IContainerControl::zOrder)
     flClEnumeration = CMA_ZORDER;
  else
     flClEnumeration = CMA_ITEMORDER;

}


/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::ObjectCursor                                |
|                                                                              |
| Construct a cursor object to find all objects that have the passed object as |
| a parent.                                                                    |
------------------------------------------------------------------------------*/
IContainerControl::ObjectCursor::ObjectCursor(const IContainerControl& cnrctl,
                                       const IContainerObject* pcnrobjParent,
                                       bool                 allDescendents)
                : ulClChanges(cnrctl.numberOfObjectChanges()),
                  pcnrctlClOwner ((IContainerControl*)&cnrctl),
                  pcnrobjClCurrent(0),
                  pcnrobjClParent((IContainerObject*)pcnrobjParent),
                  flClEmphasis(0),
#ifdef IC_PMWIN
                  currentIndex(0),
#endif
                  flClState(child)
{
   flClEnumeration = CMA_ITEMORDER;
   if (allDescendents)
     flClState = allSubTree;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor :: ~ObjectCursor                             |
------------------------------------------------------------------------------*/
IContainerControl::ObjectCursor :: ~ObjectCursor ( )
{ }

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::current                                     |
|                                                                              |
| Return the current object of this cursor.                                    |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::current() const
{
   if (isValid())
      return(pcnrobjClCurrent);
   return 0;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::setCurrent                                  |
|                                                                              |
| Sets the cursor to point to a particular object.                             |
------------------------------------------------------------------------------*/
IContainerControl::ObjectCursor&
            IContainerControl::ObjectCursor::setCurrent(
                                         const IContainerObject* currentObject)
{
  IASSERTPARM(pcnrctlClOwner->containsObject(currentObject));
  ulClChanges = pcnrctlClOwner->numberOfObjectChanges();
  pcnrobjClCurrent=(IContainerObject*)currentObject;

#ifdef IC_WIN
  if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
  {
    LV_FINDINFO  lvFindInfo;

    lvFindInfo.flags  = LVFI_PARAM;
    lvFindInfo.lParam = (LPARAM)currentObject;

    currentIndex = IContainerControl::ListViewFindItem(
                                     pcnrctlClOwner->ppd->pListview->handle(),
                                     (-1), &lvFindInfo);
  }
#endif //IC_WIN

  return *this;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::setToFirst                                  |
|                                                                              |
| Sets the cursor to the first object in the container.                        |
------------------------------------------------------------------------------*/
bool IContainerControl::ObjectCursor::setToFirst()
{
  ulClChanges = pcnrctlClOwner->numberOfObjectChanges();

#ifdef IC_PMWIN
  if (flClEmphasis)
     pcnrobjClCurrent = selectedObject((IContainerObject*)CMA_FIRST,
                                       flClEmphasis);
#endif
#ifdef IC_MOTIF
  if (flClEmphasis)
  {
     IContainerControl::ObjectCursor iter(*pcnrctlClOwner);
     iter.setToFirst();
     pcnrobjClCurrent = 0;
     bool fFound =false;
     IContainerObject* pcnrobj = iter.current();

     while (pcnrobj!=0 && fFound==false)
     {
        unsigned long recordAttr = pcnrctlClOwner->attributes(pcnrobj);
        if((recordAttr & flClEmphasis) == flClEmphasis)
        {
            pcnrobjClCurrent = pcnrobj;
            fFound = true;
        }
        pcnrobj=iter.next();
     } /* endwhile */
  }
#endif//IC_MOTIF
  else if (pcnrobjClParent)
     pcnrobjClCurrent = object(pcnrobjClParent,CMA_FIRSTCHILD);
  else
     pcnrobjClCurrent = object(0,CMA_FIRST);

  if(pcnrobjClCurrent)
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::setToNext                                   |
|                                                                              |
| Points to the next or first object in the container.                         |
------------------------------------------------------------------------------*/
bool IContainerControl::ObjectCursor::setToNext()
{
  if(pcnrobjClCurrent == 0)
     return setToFirst();

#ifdef IC_PMWIN
  if (flClEmphasis)
     pcnrobjClCurrent = selectedObject(pcnrobjClCurrent, flClEmphasis);
#endif
#ifdef IC_MOTIF
  if (flClEmphasis)
  {
     IContainerControl::ObjectCursor iter(*pcnrctlClOwner);
     iter.setCurrent(pcnrobjClCurrent);
     pcnrobjClCurrent = 0;
     bool fFound =false;
     IContainerObject* pcnrobj = iter.next();

     while (pcnrobj!=0 && fFound==false)
     {
        unsigned long recordAttr = pcnrctlClOwner->attributes(pcnrobj);
        if((recordAttr & flClEmphasis) == flClEmphasis)
        {
            pcnrobjClCurrent = pcnrobj;
            fFound = true;
        }
        pcnrobj=iter.next();
     } /* endwhile */
  }
#endif  //IC_MOTIF

  else if (flClState == child || flClEnumeration == CMA_ZORDER)
     pcnrobjClCurrent = object(pcnrobjClCurrent, CMA_NEXT);

  else
     pcnrobjClCurrent = nextObject(pcnrobjClCurrent);

  if(pcnrobjClCurrent)
    return true;
  else
    return false;

}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::setToPrevious                               |
|                                                                              |
| Points to the previous or last object in the container.                      |
------------------------------------------------------------------------------*/
bool IContainerControl::ObjectCursor::setToPrevious()
{
  if(pcnrobjClCurrent == 0)
    return setToLast();

  if (flClEmphasis)
  {
     IContainerControl::ObjectCursor iter(*pcnrctlClOwner);
     iter.setCurrent(pcnrobjClCurrent);
     pcnrobjClCurrent = 0;
     bool fFound =false;
     IContainerObject* pcnrobj = iter.previous();

     while (pcnrobj!=0 && fFound==false)
     {
        unsigned long recordAttr = pcnrctlClOwner->attributes(pcnrobj);
        if((recordAttr & flClEmphasis) == flClEmphasis)
        {
            pcnrobjClCurrent = pcnrobj;
            fFound = true;
        }
        pcnrobj=iter.previous();
     } /* endwhile */
  }
  // Previous at current level
  else if (flClState == child || flClEnumeration == CMA_ZORDER)
     pcnrobjClCurrent = object(pcnrobjClCurrent, CMA_PREV);
  else
  {
     pcnrobjClCurrent = previousObject(pcnrobjClCurrent);
     if(pcnrobjClParent && (pcnrobjClCurrent == pcnrobjClParent))
        pcnrobjClCurrent = 0;
  }
  if(pcnrobjClCurrent!=0)
     return true;
  else
     return false;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::setToLast                                   |
|                                                                              |
| Point to the last object in the container.                                   |
------------------------------------------------------------------------------*/
bool IContainerControl::ObjectCursor::setToLast()
{
  ulClChanges = pcnrctlClOwner->numberOfObjectChanges();

  if (flClEmphasis)
  {
     IContainerControl::ObjectCursor iter(*pcnrctlClOwner);
     pcnrobjClCurrent = 0;
     bool fFound =false;
     IContainerObject* pcnrobj = iter.last();

     while (pcnrobj!=0 && fFound==false)
     {
        unsigned long recordAttr = pcnrctlClOwner->attributes(pcnrobj);
        if((recordAttr & flClEmphasis) == flClEmphasis)
        {
            pcnrobjClCurrent = pcnrobj;
            fFound = true;
        }
        pcnrobj=iter.previous();
     } /* endwhile */
  }
  else if (pcnrobjClParent)   // Child of specified parent
  {
    if (flClState==allSubTree)
    {
      pcnrobjClCurrent = lastObject(pcnrobjClParent);
      if(pcnrobjClCurrent == pcnrobjClParent)
        pcnrobjClCurrent = 0;
    }
    else
      pcnrobjClCurrent = object(pcnrobjClParent,CMA_LASTCHILD);
  }
  // Root level child
  else if (flClState == child || flClEnumeration == CMA_ZORDER)
     pcnrobjClCurrent = object(0,CMA_LAST);
  else
     pcnrobjClCurrent = lastObject();

  if (pcnrobjClCurrent!=0)
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::object                                      |
|                                                                              |
| Protected function to get an object.                                         |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::object(
                              const IContainerObject* pcnrobjAfter,
                                    unsigned long  ulCmd)
{
  unsigned short usItemOrder;
  usItemOrder = (unsigned short)flClEnumeration;

   IMiniCnrRecord* pcnrrecAfter = 0;
   if(pcnrobjAfter)
     pcnrrecAfter = IRecFromObj(pcnrobjAfter);

#ifdef IC_WIN
  if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
    pcnrrecAfter = (IMiniCnrRecord*)this;
#endif

  IMiniCnrRecord* pcnrrec =
            (IMiniCnrRecord*)(char*)
                      (pcnrctlClOwner->sendEvent(
                                 CM_QUERYRECORD,
                                 MPFROMP(pcnrrecAfter),
                                 MPFROM2SHORT((unsigned short)ulCmd,
                                              usItemOrder)));

   if (pcnrrec==0 || (long)pcnrrec==-1L)
     return 0;
   else
     return IObjFromRec(pcnrrec);
};


/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::nextObject                                  |
|                                                                              |
| Protected function to get the next object after a given object.              |
| Notes: This function has been updated to work in the Tree View.  The         |
|          strategy is:                                                        |
|                                                                              |
|    1) if CMA_FIRSTCHILD exists, return it (at A, returns B, at B returns C   |
|    2) else if CMA_NEXT exists, return it  (at E, return F)                   |
|    3) else call nextBranchObject  (at C return D, at F returns G             |
|                                                                              |
|   A                                                                          |
|   ^----B                                                                     |
|   |    ^---C                                                                 |
|   |----D                                                                     |
|   |    ^---E                                                                 |
|   |    ^---F                                                                 |
|   G                                                                          |
|   ^----H                                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::nextObject(
                              const IContainerObject* objectAfter)
{
  IASSERTPARM(objectAfter!=0);

  IMiniCnrRecord*  pcnrrecAfter = IRecFromObj(objectAfter);

  IMiniCnrRecord* pcnrrec;

#ifdef IC_WIN
  if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
    pcnrrecAfter = (IMiniCnrRecord*)this;
#endif

  pcnrrec =   (IMiniCnrRecord*)(char*)
                         (pcnrctlClOwner->sendEvent(
                                    CM_QUERYRECORD,
                                    MPFROMP(pcnrrecAfter),
                                    MPFROM2SHORT(CMA_FIRSTCHILD,
                                                 CMA_ITEMORDER)));

  if ((long)pcnrrec==-1L)
    return 0;
  else if (pcnrrec>0)
    return IObjFromRec(pcnrrec);

  pcnrrec = (IMiniCnrRecord*)(char*)(pcnrctlClOwner->sendEvent(
                                 CM_QUERYRECORD,
                                 MPFROMP(pcnrrecAfter),
                                 MPFROM2SHORT(CMA_NEXT,
                                         (unsigned short)flClEnumeration)));

   if ((long)pcnrrec==-1L)
     return 0;
   else if (pcnrrec>0)
     return IObjFromRec(pcnrrec);

  return nextBranchObject(objectAfter);

};

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::nextBranchObject                            |
|                                                                              |
| Protected function to go back up the tree and return the first object in the |
| next leg.  This occurs when we get to the bottom of any leg.                 |
| Notes:                                                                       |
|  1) if not CMA_PARENT, return 0 (at the top)                                 |
|  2) else if CMA_NEXT of parent, return it                                    |
|  3) else call ourselves with the parent (recursively)                        |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::nextBranchObject(
                              const IContainerObject* objectAfter)
{

  IMiniCnrRecord* pcnrrecAfter = IRecFromObj(objectAfter);

#ifdef IC_WIN
  if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
    pcnrrecAfter = (IMiniCnrRecord*)this;
#endif

   IMiniCnrRecord* pcnrrecParent =
             (IMiniCnrRecord*)(char*)
                       (pcnrctlClOwner->sendEvent(
                                         CM_QUERYRECORD,
                                         MPFROMP(pcnrrecAfter),
                                         MPFROM2SHORT(CMA_PARENT,
                                                      CMA_ITEMORDER)));

   if ( (long)pcnrrecParent==-1L || (pcnrrecParent==0)  ||
        ((flClState==allSubTree) &&
         (pcnrrecParent==IRecFromObj(pcnrobjClParent))) )
      return 0;

   IMiniCnrRecord* pcnrrec =
             (IMiniCnrRecord*)(char*)
                       (pcnrctlClOwner->sendEvent(
                                         CM_QUERYRECORD,
                                         MPFROMP(pcnrrecParent),
                                         MPFROM2SHORT(CMA_NEXT,
                                           (unsigned short)flClEnumeration)));
   if ((long)pcnrrec==-1L )
      return 0;
   else if (pcnrrec>0)
     return IObjFromRec(pcnrrec);
   else return nextBranchObject(IObjFromRec(pcnrrecParent));
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::previousObject                              |
|                                                                              |
| Private function to get the previous object in the container.                |
| Notes:                                                                       |
|    1) if CMA_PREV, return lastObject(CMA_PREV) F from G (via A, D), E from F |
|    2) else if CMA_PARENT, return it,  D from E, B from C, G from H           |
|    2) else return 0, at the top, from A                                      |
|   A                                                                          |
|   ^----B                                                                     |
|   |    ^---C                                                                 |
|   |----D                                                                     |
|   |    ^---E                                                                 |
|   |    ^---F                                                                 |
|   G                                                                          |
|   ^----H                                                                     |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::previousObject(
                              const IContainerObject* objectBefore)
{
  IMiniCnrRecord*   pcnrrecBefore = IRecFromObj(objectBefore);

#ifdef IC_WIN
  if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
    pcnrrecBefore = (IMiniCnrRecord*)this;
#endif

  IMiniCnrRecord* pcnrrecPrev =
            (IMiniCnrRecord*)(char*)
                      (pcnrctlClOwner->sendEvent(
                                        CM_QUERYRECORD,
                                        MPFROMP(pcnrrecBefore),
                                        MPFROM2SHORT(CMA_PREV,
                                           (unsigned short)flClEnumeration)));
  if ((long)pcnrrecPrev==-1L)
     return 0;


  if (pcnrrecPrev>0)
     return lastObject(IObjFromRec(pcnrrecPrev));

  pcnrrecPrev =  (IMiniCnrRecord*)(char*)(pcnrctlClOwner->sendEvent(
                                 CM_QUERYRECORD,
                                 MPFROMP(pcnrrecBefore),
                                 MPFROM2SHORT(CMA_PARENT,
                                              CMA_ITEMORDER)));

  if ( (long)pcnrrecPrev==-1L || (pcnrrecPrev==0) ||
       ((flClState==allSubTree) &&
        (pcnrrecPrev==IRecFromObj(pcnrobjClParent))) )
     return 0;
  else
     return IObjFromRec(pcnrrecPrev);
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::lastObject                                  |
|                                                                              |
| Protected function to get the last object in a leg of the container or the   |
| last object of the last leg of the container.                                |
| Notes:                                                                       |
|    1) if not CMA_LAST, return 0 (no records in the container)                |
|    2) Loop calling CMA_LASTCHILD until we get to the end, return it          |
|    note: if objectAfter passed then use it as the starting point otherwise   |
|          use the CMA_LAST (the last root item).                              |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::lastObject(
                              const IContainerObject* pcnrobjParent)
{
  IMiniCnrRecord*   pcnrrec=0;
  IMiniCnrRecord*   pcnrrecLast;
  IMiniCnrRecord*   pcnrrecParent;

  /*****************************************************************/
  /* If parent, we are looking for the last in it's leg, else we   */
  /* are looking for the last in the entire tree                   */
  /*****************************************************************/
  if(pcnrobjParent != 0)
     pcnrrecParent = IRecFromObj(pcnrobjParent);
  else
  {
#ifdef IC_WIN
    if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
      pcnrrec = (IMiniCnrRecord*)this;
#endif
    pcnrrecParent =  (IMiniCnrRecord*)(char*)(pcnrctlClOwner->sendEvent(
                                   CM_QUERYRECORD,
                                   MPFROMP(pcnrrec),
                                   MPFROM2SHORT(CMA_LAST,
                                                CMA_ITEMORDER)));
  }
  if ((long)pcnrrecParent==-1L || pcnrrecParent==0)
     return 0;

  pcnrrec = pcnrrecParent;
  pcnrrecLast = 0;

#ifdef IC_WIN
  //skip loop if only list view, no need to look for children
  if (!pcnrctlClOwner->ppd->pmCompatible && !pcnrctlClOwner->ppd->pTreeview)
    pcnrrec = 0;
#endif

  while(pcnrrec != 0)
  {
    pcnrrec =  (IMiniCnrRecord*)(char*)(pcnrctlClOwner->sendEvent(
                                   CM_QUERYRECORD,
                                   MPFROMP(pcnrrec),
                                   MPFROM2SHORT(CMA_LASTCHILD,
                                              CMA_ITEMORDER)));
    if ((long)pcnrrec==-1L)
       return 0;
    if(pcnrrec != 0)
      pcnrrecLast = pcnrrec;
  }

  if(pcnrrecLast)
     return IObjFromRec(pcnrrecLast);
  else
     return IObjFromRec(pcnrrecParent);
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::selectedObject                              |
|                                                                              |
| Protected function to return a selected object.                              |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::selectedObject(
                                    const IContainerObject* pcnrobjAfter,
                                    unsigned long  ulEmphasis)
{
// ITRACE_MOTIF_NOTYET();
#ifdef IC_PMWIN
   /* Check to be sure that we don't get something like CMA_NEXT 0x100 */
   IMiniCnrRecord* pcnrrecAfter;
   if((unsigned long)pcnrobjAfter > 200)
      pcnrrecAfter = IRecFromObj(pcnrobjAfter);
   else
      pcnrrecAfter = (IMiniCnrRecord*)pcnrobjAfter;

   IMiniCnrRecord*  pcnrrec =
             (IMiniCnrRecord*)(char*)
                       (pcnrctlClOwner->sendEvent(
                                 CM_QUERYRECORDEMPHASIS,
                                 MPFROMP(pcnrrecAfter),
                                 MPFROMLONG(ulEmphasis)));

   if ((long)pcnrrec<=0)      // 32 bit
     return 0;
   else
     return IObjFromRec(pcnrrec);
#endif  //IC_PMWIN
#ifdef IC_MOTIF
  ITRACE_DEVELOP(IString("ObjectCursor::selectedObject(): called on MOTIF - never should be the case !!!"));
  return 0;
#endif
};

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::isValid                                     |
|                                                                              |
| Is the cursor valid.                                                         |
------------------------------------------------------------------------------*/
bool IContainerControl::ObjectCursor::isValid () const
{
  if(ulClChanges == pcnrctlClOwner->numberOfObjectChanges()  &&
     pcnrobjClCurrent != 0)
      return true;
   else
      return false;
}
/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::invalidate                                  |
|                                                                              |
| Invalidate the cursor.                                                       |
------------------------------------------------------------------------------*/
IContainerControl::ObjectCursor& IContainerControl::ObjectCursor::invalidate()
{
  pcnrobjClCurrent=0;
  return *this;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::first                                       |
|                                                                              |
| Points to and returns the first object in the container.                     |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::first()
{
   setToFirst();
   return pcnrobjClCurrent;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::next                                        |
|                                                                              |
| Points to and returns the next or first object in the container.             |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::next()
{
   setToNext();
   return pcnrobjClCurrent;
}
/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::previous                                    |
|                                                                              |
| Points to and returns the previous object in the container.                  |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::previous()
{
   setToPrevious();
   return pcnrobjClCurrent;
}

/*------------------------------------------------------------------------------
| IContainerControl::ObjectCursor::last                                        |
|                                                                              |
| Points to and returns the last object in the container.                      |
------------------------------------------------------------------------------*/
IContainerObject* IContainerControl::ObjectCursor::last()
{
   setToLast();
   return pcnrobjClCurrent;
}


