// Revision: 30 1.3.1.4 source/ui/cnr/icnrrec.hpp, container, ioc.v400, 000919 
/* NOSHIP */
#ifndef _ICNRREC_
   #define _ICNRREC_
/*******************************************************************************
* FILE NAME: icnrrec.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the interface for IMiniCnrRecord and ICnrRecord which   *
*   are replacements for the Container Control structures MINIRECORDCORE and   *
*   RECORDCORE.                                                                *
*                                                                              *
* Implementation Notes:                                                        *
*   1) The OS/2 types HPOINTER & HBITMAP are replaced with IPointerHandle      *
*      and IBitmapHandle in order to reference count the use of these          *
*      resources. This is the primary reason for redefining these classes.     *
*   2) IMiniCnrRecord is an invisible (to the user) base of IContainerObject.  *
*      IContainerObject::new allocates storage for IMiniCnrRecord as well      *
*      as for IContainerObject.  IMiniCnrRecord::new usually returns a         *
*      pointer to the base portion of the allocation. This pointer is stored   *
*      in IContainerObject.                                                    *
*   3) Two Non-class functions exist for converting between IMiniCnrRecord     *
*      and IContainerObject.                                                   *
*   4) ICnrRecord exists to provide the additional capabilities of             *
*      RECORDCORE in the container.  At this time, this support is not         *
*      committed.  However, much of the juggling that goes on here has         *
*      been done to provide this support.                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"  {
  #include <iwindefs.h>
}

#include <ibhandle.hpp>
#include <ihandle.hpp>
#include <icnrobj.hpp>

#ifdef IC_MOTIF
 #include <icnrm.h>
#endif

#ifdef IC_WIN
  #ifndef _ICLCONTAINERW_
  extern "C"  {
    #include <iclcnrw.h>
  }
  #endif
#endif

// #include <inew.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)
#pragma info(none)

class IContainerBitmapObject;
struct _FIELDINFO;

struct IMiniCnrRecord
{

#ifdef __DEBUG_ALLOC__
   void* operator new ( size_t size,
                        const char * filename,
                        size_t linenum,
                        IContainerObject* pcnrobj)
         {
         return ( (pcnrobj!=0) ? ((char*)pcnrobj) - sizeof(IMiniCnrRecord) : 0);
         }
#else
   void* operator new(size_t size, IContainerObject* pcnrobj)
         {
         return ( (pcnrobj!=0) ? ((char*)pcnrobj) - sizeof(IMiniCnrRecord) : 0);
         }
#endif
   IMiniCnrRecord();
  ~IMiniCnrRecord();

#ifdef __DEBUG_ALLOC__
    void operator delete (void *,
                          const char * filename,
                          size_t linenum) { }
#else
    void operator delete(void* )  { }
#endif

  unsigned long   cb;
  unsigned long   flRecordAttr;          /* record attributes             */
  struct IPOINTL         /* ptl  */
  {
    long  x;
    long  y;
  } ;
  IPOINTL      ptlIcon;                  /* Position of CV_ICON item      */
  struct IMiniCnrRecord *preccNextRecord;/* ptr to next record   */
#ifdef IC_NOBINARYCOMP
           char*  cRecordText;           /* Text for record      */
           void*  pRecordIcon;           /* Icon for record      */
#endif
         IString  strIcon;               /* Text for Icon view   */
  IPointerHandle  hptrIcon;              /* Icon to display      */

private:
   void* operator new(size_t size);
};

struct ITreeItemDesc     /* treend */
{
   ITreeItemDesc() : hbmExpanded(0), hbmCollapsed(0),
                     hptrExpanded(0), hptrCollapsed(0) {}
      IBitmapHandle  hbmExpanded;
      IBitmapHandle  hbmCollapsed;
      IPointerHandle hptrExpanded;
      IPointerHandle hptrCollapsed;
};

struct ICnrRecord : public IMiniCnrRecord
{

#ifdef __DEBUG_ALLOC__
   void* operator new ( size_t size,
                        const char * filename, size_t linenum,
                        IContainerBitmapObject* pcnrobj)
          {
          return ( (pcnrobj!=0) ? ((char*)pcnrobj) - sizeof(ICnrRecord) : 0);
          }
#else
   void* operator new(size_t size, IContainerBitmapObject* pcnrobj)
          {
          return ( (pcnrobj!=0) ? ((char*)pcnrobj) - sizeof(ICnrRecord) : 0);
          }
#endif

  ICnrRecord() : strText(), strName(), strTree(),
                 hptrMiniIcon(0), hbmBitmap(0), hbmMiniBitmap(0),
                 pTreeItemDesc(0) {}
  IPointerHandle  hptrMiniIcon;        /* Icon to display for CV_MINI   */
   IBitmapHandle  hbmBitmap;           /* Bitmap to display for ~CV_MINI*/
   IBitmapHandle  hbmMiniBitmap;       /* Bitmap to display for CV_MINI */
   ITreeItemDesc* pTreeItemDesc;       /* Icons for the tree view       */
         IString  strText;             /* Text for CV_TEXT view         */
         IString  strName;             /* Text for CV_NAME view         */
         IString  strTree;             /* Text for CV_TREE view         */
private:
   void* operator new(size_t size);

};

struct ICnrInfo
{
   ICnrInfo();
  ~ICnrInfo();
    unsigned long cb;                  /* size of CNRINFO struct        */
      void*       pSortRecord;         /* ptr to sort function,         */
      /*                                    RECORDCORE                  */
      _FIELDINFO* pFieldInfoLast;      /* pointer to last column in     */
      /*                                    left pane of a split window.*/
      _FIELDINFO* pFieldInfoObject;    /* Pointer to a column to        */
      /*                                  represent an object.  This is */
      /*                                  the column which will receive */
      /*                                  IN-USE emphasis.              */
#ifdef IC_MOTIF
      IString     strCnrTitle;         /* text for container title. One */
#endif
#ifdef IC_PMWIN
      char*       strCnrTitle;         /* text for container title. One */
#endif
      /*                                    string separated by line    */
      /*                                    separators for multi-lines  */
    unsigned long flWindowAttr;        /* container attrs - CV_*, CA_*  */
    struct IPointl
    {
      IPointl(long xIn, long yIn) : x(xIn), y(yIn) {}
      long  x;
      long  y;
    } ;
    struct ISizel
    {
      ISizel(long cxIn, long cyIn) : cx(cxIn), cy(cyIn) {}
      long  cx;
      long  cy;
    } ;
     IPointl      ptlOrigin;           /* lower-left origin in virtual  */
      /*                                    coordinates. CV_ICON view   */
  unsigned long   cDelta;              /* Application defined threshold */
      /*                                    or number of records from   */
      /*                                    either end of the list.     */
  unsigned long   cRecords;            /* number of records in container*/
     ISizel       slBitmapOrIcon;      /* size of bitmap in pels        */
     ISizel       slTreeBitmapOrIcon;  /* size of tree bitmaps in pels  */
 IBitmapHandle    hbmExpanded;         /* bitmap  for tree node         */
 IBitmapHandle    hbmCollapsed;        /* bitmap  for tree node         */
 IPointerHandle   hptrExpanded;        /* icon    for tree node         */
 IPointerHandle   hptrCollapsed;       /* icon    for tree node         */
      long        cyLineSpacing;       /* space between two rows        */
      long        cxTreeIndent;        /* indent for children           */
      long        cxTreeLine;          /* thickness of the Tree Line    */
unsigned long     cFields;             /* number of fields  in container*/
      long        xVertSplitbar;       /* position relative to the      */
      /*                                    container (CV_DETAIL); if   */
      /*                                    0xFFFF then unsplit         */
};

/*------------------------------------------------------------------------------
| IRecFromObj                                                                  |
|                                                                              |
| Convert an IContainerObject pointer to an IMiniCnrRecord and an              |
| IContainerBitmapObject pointer to an ICnrRecord.                             |
|                                                                              |
| Note:                                                                        |
|   pcnrobj must be a valid pointer to a record and this must be checked by    |
|   the caller. IContainer(Bitmap)Object declares the function a friend        |
------------------------------------------------------------------------------*/
inline IMiniCnrRecord* IRecFromObj(const IContainerObject* pcnrobj)
  {
  if(pcnrobj)
    return ((IContainerObject*)pcnrobj)->baseRecord();
  return 0;
  }

inline ICnrRecord* IRecFromObj(const IContainerBitmapObject* pcnrobj)
  {
  if(pcnrobj)
    return (ICnrRecord*)IRecFromObj((IContainerObject*)pcnrobj);
  return 0;
  }

/*------------------------------------------------------------------------------
| IObjFromRec                                                                  |
|                                                                              |
| Convert an IMiniCnrRecord to an IContainerObject.                            |
------------------------------------------------------------------------------*/
inline  IContainerObject* IObjFromRec(const IMiniCnrRecord* pcnrrec)
  {
  if (pcnrrec)
    return  (IContainerObject*)(((char*)pcnrrec) + pcnrrec->cb);
  return 0;
  }

/*-----------------------------------------------------------------*/
/* The following value is used in the FIELDINFO field offstruct to */
/* identify a column offset that occurs in the ICnrMiniRecord part */
/* of the record.  When a IContainerColumn is constructed this     */
/* value is added to the offstruct value by iconOffset and         */
/* iconTextOffset() in IContainerObject.  User data reflects the   */
/* offset of the data in IContainerObject.  Upon addition to the   */
/* container these values are adjusted to the actual offset from   */
/* the start of IMiniCnrRecord.                                    */
/*-----------------------------------------------------------------*/
#define IFIELDINBASE 0XFFFFFF00

/*-----------------------------------------------------------------*/
/* The following value is used to interface to the native control  */
/* coordinate system differences that exist between environments   */
/*-----------------------------------------------------------------*/
#ifdef IC_MOTIFWIN
#define IC_CNR_COORDSYS  ICoordinateSystem::originLowerLeft
#endif
#ifdef IC_PM
#define IC_CNR_COORDSYS  ICoordinateSystem::originUpperLeft
#endif

#pragma pack(pop)
#pragma enum(pop)
#pragma info(restore)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#ifdef IC_MOTIF
// Following 2 files provide all the PM Toolkit declarations that the
// PM .cpp files used under Motif will need to compile.
#include <ixconst.h>
#include <icnrm.h>
#endif

#endif
