// Revision: 31 1.3.1.3 source/ui/cnr/icnrstat.hpp, container, ioc.v400, 000919  
/* NOSHIP */
#ifndef _ICNRSTAT_
#define _ICNRSTAT_
/*******************************************************************************
* FILE NAME: icnrstat.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ICnrStaticsPtr                                                           *
*     ICnrDestroyHandler                                                       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ihandler.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IContainerControl;
class IEvent;
class IWindow;
class ICnrControlList;
class IPrivateResource;

#pragma enum(4)
#pragma pack(push,4)

class ICnrDestroyHandler : public IHandler
{
typedef IHandler Inherited;
/*******************************************************************************
* The ICnrHandler class provides for removal of container objects and columns  *
* during a WM_DESTROY event.                                                   *
*******************************************************************************/
public:

/*------------------------ Constructors ----------------------------------------
| The only way to construct an instance of this class is to use the default.   |
------------------------------------------------------------------------------*/
  ICnrDestroyHandler();
 ~ICnrDestroyHandler();


/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to and from|
| a container window:                                                          |
|   handleEventsFor       - Attaches the handler to the container object passed|
|                           in the argument.                                   |
|   stopHandlingEventsFor - Detaches the handler from the container object     |
|                           passed in the argument.                            |
------------------------------------------------------------------------------*/
virtual ICnrDestroyHandler
 &handleEventsFor       ( IContainerControl* container ),
 &stopHandlingEventsFor ( IContainerControl* container );

protected:
/*----------------------------- Implementation ---------------------------------
| This function is used to implement the class:                                |
|   dispatchHandlerEvent - Routes the edit events to the appropriate event     |
|                          functions.                                          |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent(IEvent& event);

private:
/* Hidden functions */
virtual IHandler
 &handleEventsFor       ( IWindow* window  );
virtual IHandler
 &stopHandlingEventsFor ( IWindow* window  );

};

/*--------------------------------------------------------------*/
/* Static Construction/Destruction                              */
/*--------------------------------------------------------------*/
class ICnrControlStaticPtr 
{
  public:
    ~ICnrControlStaticPtr();
static ICnrControlList
 *pCnrSet;
static IPrivateResource
 *pCnrKey;
static ICnrDestroyHandler
 *pHandler;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICNRSTAT_
