// Revision: 45 1.5.1.3 source/ui/drag/idmcmdhr.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmcmdhr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMPopUpMenuHandler                                                      *
*     IDMPopUpCommandHandler                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IDMCMDHR_
  #include <idmcmdhr.hpp>
#endif

// Trace Facility
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IDMTGTOP_
  #include <idmtgtop.hpp>
#endif

#include <icconst.h>

// Segment definitions

#ifdef IC_PAGETUNE
  #define _IDMCMDHR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::IDMPopupMenuHandler                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IDMPopUpMenuHandler::IDMPopUpMenuHandler(IWindow* pWin) :
    fNonDefaultDrag(false)
    , pWindowToHandle(pWin)
{
} // IDMPopUpMenuHandler

IDMPopUpMenuHandler::IDMPopUpMenuHandler(const IDMPopUpMenuHandler&) {}

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::~IDMPopupMenuHandler                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IDMPopUpMenuHandler:: ~IDMPopUpMenuHandler()
{
   if (pWindowToHandle)
      stopHandlingEventsFor(pWindowToHandle) ;
} // ~IDMPopupMenuHandler

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::makePopUpMenu                                           |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDMPopUpMenuHandler :: makePopUpMenu(IMenuEvent& menuEvent)
{
  IMODTRACE_DEVELOP("IDMPopUpMenuHandler::makePopUpMenu");

  IPopUpMenu* popUp(new IPopUpMenu(menuEvent.window(), 0));

  IDMTargetOperation::Handle pTgtOp(IDMTargetOperation::targetOperation()) ;
  unsigned short
    supportedOperations = 0;

  if ( pTgtOp.valid() )
    {
    IDMItem::Handle pItemH(pTgtOp->item(1)) ;
    supportedOperations = pItemH->supportedOperations();
    }

  ITRACE_DEVELOP("Supported operations: " + IString(supportedOperations));
  // Create the popup menu based on the supported operations
  if ( supportedOperations & IDMItem::moveable )
    (*popUp).addText(IC_DM_MI_MOVE,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_MOVE));
  if ( supportedOperations & IDMItem::copyable )
    (*popUp).addText(IC_DM_MI_COPY,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_COPY));
  if ( supportedOperations & IDMItem::linkable )
    (*popUp).addText(IC_DM_MI_SHORTCUT,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_SHORTCUT));
  (*popUp)
    .addSeparator()
    .addText(IC_DM_MI_CANCEL,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_CANCEL));

  (*popUp).show(menuEvent.mousePosition());
  // By default, popup menus are autoDestroyObject where necessary (i.e., on PM and Motif)

  return true;
} // makePopUpMenu

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::dispatchHandlerEvent                                    |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDMPopUpMenuHandler::dispatchHandlerEvent(IEvent& evt)
{
  bool bProcessed = false;

  switch(evt.eventId())
    {
    case WM_RBUTTONUP:
    {
      if (fNonDefaultDrag) {
        ::ShowCursor(true);
        IMenuEvent mnEvt(evt);
        if (makePopUpMenu(mnEvt)) {
           evt.setResult(true);
           if (pWindowToHandle)
              stopHandlingEventsFor(pWindowToHandle) ;
        }
        bProcessed = true;
      }
      break;
    }
  } // switch

  return bProcessed;
} // dispatchHandlerEvent


/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::IDMpopupCommandHandler                               |
|                                                                              |
------------------------------------------------------------------------------*/
IDMPopUpCommandHandler::IDMPopUpCommandHandler(IWindow* pWin) :
    point()
    , dataObject(NULL)
    , pWindowToHandle(pWin)
{
   IMODTRACE_DEVELOP("IDMPopUpCommandHandler::IDMPopUpCommandHandler") ;
   memset(&point, 0, sizeof(POINTL)) ;
} // IDMPopupCommandHandler

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::~IDMpopupCommandHandler                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDMPopUpCommandHandler::~IDMPopUpCommandHandler()
{
   IMODTRACE_DEVELOP("IDMPopUpCommandHandler::~IDMPopUpCommandHandler") ;
   if (pWindowToHandle)
      stopHandlingEventsFor(pWindowToHandle) ;
} // ~IDMPopupCommandHandler

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::command                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDMPopUpCommandHandler::command( ICommandEvent& cmdEvent)
{
  IMODTRACE_DEVELOP("IDMPopUpCommandHandler::command");

  bool brc = false;
  DWORD dwOperation = 0;
  bool fValidOperation = false;

  // if dataObject is null, then we are in the wrong command handler. Ignore event
  if (dataObject == NULL)
     return fValidOperation ;

  switch (cmdEvent.commandId())
    {
    case IC_DM_MI_MOVE :
      {
      ITRACE_DEVELOP("Move selected");
      dwOperation = DROPEFFECT_MOVE;
      fValidOperation = true;
      brc = true;
      break;
      }
    case IC_DM_MI_COPY :
      {
      ITRACE_DEVELOP("Copy selected");
      dwOperation = DROPEFFECT_COPY;
      fValidOperation = true;
      brc = true;
      break;
      }
    case IC_DM_MI_SHORTCUT :
      {
      ITRACE_DEVELOP("Create Shortcut selected");
      dwOperation = DROPEFFECT_LINK;
      fValidOperation = true;
      brc = true;
      break;
      }
    case IC_DM_MI_CANCEL :
      {
      ITRACE_DEVELOP("Cancel selected");
      cmdEvent.window()->sendEvent( DM_DRAGLEAVE, 0, 0);
      // dataObject->Release();
      brc = true;
      // clean up on source side as well
      IDMSourceOperation::Handle pSrcOp(IDMSourceOperation::sourceOperation()) ;
      if ( pSrcOp.valid() ) {
         unsigned count = pSrcOp->numberOfItems();
         for (unsigned i = count; i; i--)
             pSrcOp->removeItem(i);
      }
      break;
    }
  } // switch

  // if we processed this event, the drop happened. No need to handle the popup menu anymore
  if (brc)
     if (pWindowToHandle)
        stopHandlingEventsFor(pWindowToHandle) ;

  if ( fValidOperation ) {
    IDMTargetOperation::Handle pTgtOp(IDMTargetOperation::targetOperation()) ;
    if ( pTgtOp.valid() )
       pTgtOp->setOperation( dwOperation );
    IEventResult evt =  cmdEvent.window()->sendEvent( DM_DROP,
                        IEventData( point.x, point.y ),
                        IEventData( (void *) dataObject ));
  }

  if (brc)
     dataObject->Release() ;

  return brc;
} // command

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::setPoint                                             |
|                                                                              |
------------------------------------------------------------------------------*/
void IDMPopUpCommandHandler::setPoint( POINTL pt)
{
  memcpy(&point, &pt, sizeof(POINTL)) ;
}

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::setDataObject                                        |
|                                                                              |
------------------------------------------------------------------------------*/
void IDMPopUpCommandHandler::setDataObject( IDataObject* dataObj)
{
  dataObject = dataObj;
}

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::startHandlingEvents                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IDMPopUpMenuHandler& IDMPopUpMenuHandler::startHandlingEvents()
{
   IMODTRACE_DEVELOP("IDMPopUpMenuHandler::startHandlingEvents") ;
   handleEventsFor(pWindowToHandle) ;
   return *this ;
}


/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::startHandlingEvents                                  |
|                                                                              |
------------------------------------------------------------------------------*/
IDMPopUpCommandHandler& IDMPopUpCommandHandler::startHandlingEvents()
{
   IMODTRACE_DEVELOP("IDMPopUpCommandHandler::startHandlingEvents") ;
   handleEventsFor(pWindowToHandle) ;
   return *this ;
}
