// Revision: 47 1.6.1.4 source/ui/drag/idmcnrit.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmcnrit.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMCnrItem                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
// include OS/2 definitions
  #define INCL_WINSTDDRAG
  #define INCL_WINSTDCNR
  #define INCL_DOSPROCESS
  #include <iwindefs.h>
}

#include <itrace.hpp>
#include <idmcnrit.hpp>
#include <icnrctl.hpp>
#include <icnrobj.hpp>
#include <istring.hpp>
#include <idmcomm.hpp>
#include <idmsrcop.hpp>
#include <idmtgtop.hpp>
#include <idmimage.hpp>
#include <iexcept.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMCNRIT_CPP_
  #include <ipagetun.h>
#endif

/* Forward Declarations */

/* Private classes      */
#pragma enum(4)
#pragma pack(push,4)

class IDMItemData
{
  friend class IDMCnrItem;
  static void setImageOffset( IDMItem* pItem, const ISize& imageOffset ) ;
};

#pragma enum(pop)
#pragma pack(pop)

void IDMItemData::setImageOffset( IDMItem* pItem, const ISize& imageOffset )
{
  pItem->sizeImageOffset = imageOffset;
}


/*------------------------------------------------------------------------------
| IDMCnrItem::IDMCnrItem                                                       |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMCnrItem :: IDMCnrItem(IContainerControl *pCnr,
                         IContainerObject *pObj,
                         IDMSourceOperation *pDMSrcOp,
                         const ISize &sizeOffset)  :
                         IDMItem(pDMSrcOp,
                                 IDM::containerObject,
                                 (IDMItem::moveable | IDMItem::copyable),
                                 none),
                         pContainerCtl(pCnr)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Set the image offset in the base class.                          */
  /********************************************************************/
  IDMItemData::setImageOffset( this, sizeOffset ) ;

  /********************************************************************/
  /* Save the container object that this item represents              */
  /********************************************************************/
  setObject(pObj);

  /********************************************************************/
  /* Set the native rendering mechanism and format for the item.      */
  /********************************************************************/
#ifdef IC_PMWIN 
  setRMFs( rmfFrom( IDM::rmLibrary, rfForThisProcess()
                       + "," + IDM::rfSharedMem) );
#endif // IC_PMWIN

#ifdef IC_MOTIF 
  setRMFs( rmfFrom( IDM::rmLibrary, rfForThisProcess() ) ) ;
#endif // IC_MOTIF

  /********************************************************************/
  /* Add a drag image for the item.                                   */
  /********************************************************************/
  IDMImage image = IDMImage(pObj->icon());
  setImage(image);
}

/*------------------------------------------------------------------------------
| IDMCnrItem::IDMCnrItem                                                       |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMCnrItem :: IDMCnrItem(const IDMItem::Handle &pCnrItem)  :
                         IDMItem(pCnrItem)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Get the pointer to the source container                          */
  /********************************************************************/
  this->pContainerCtl = (IContainerControl *)pCnrItem->sourceWindow();

  /********************************************************************/
  /* Verify that IDM::rfProcess=pid is one of the rendering formats.  */
  /********************************************************************/
  IString defaultRMF = rmfFrom(IDM::rmLibrary, rfForThisProcess());
  IString rmf = matchingRMFs (rmfsFrom( rmfs().change(' ', "") ),
                              defaultRMF);

  /******************************************************************/
  /* The only RMF that IDMCnrItem understands is                    */
  /* <IDM::rmLibrary,IDM::rfProcess=pid>.                           */
  /******************************************************************/
  setRMFs(rmf);
}

/*------------------------------------------------------------------------------
| IDMCnrItem::~IDMCnrItem                                                      |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMCnrItem :: ~IDMCnrItem()
{
  IFUNCTRACE_DEVELOP();
}

/*------------------------------------------------------------------------------
| IDMCnrItem::generateSourceItems                                              |
|                                                                              |
| Called to five IDMCnrItem opportunity to attach new IDMCnrItem's to the      |
| argument IDMSourceOperation object.  This function is called when a drag     |
| operation commences over an IContainerObject.  Returns true if items were    |
| provided, false otherwise.                                                   |
------------------------------------------------------------------------------*/
bool IDMCnrItem :: generateSourceItems(IDMSourceOperation *pSrcOp)
{
  IFUNCTRACE_DEVELOP();
  unsigned long ulCnt=0;

  /********************************************************************/
  /* Get the container object the drag started over (First if         */
  /* multiple were selected) and the container ...                    */
  /********************************************************************/
  IContainerObject *pFirstObj = pSrcOp->containerObject();
  IContainerControl *pSrcCnr  = (IContainerControl *)pSrcOp->sourceWindow();

  ISize sizeOffset = pSrcOp->pointerOffset();

  /********************************************************************/
  /* Allow drag only from container non-whitespace                    */
  /********************************************************************/
  if (pFirstObj)
  {
    /********************************************************************/
    /* Create container source item - throw exception if create failed. */
    /********************************************************************/
    IDMCnrItem::Handle pCnrItem(new IDMCnrItem(pSrcCnr,
                                          pFirstObj,
                                          pSrcOp,
                                          sizeOffset)) ;
    if ( ! pCnrItem.valid() )
      return(false);

    /********************************************************************/
    /* Add the item to the collection and increment item count.         */
    /********************************************************************/
    pSrcOp->addItem(pCnrItem);
    ulCnt++;

    /********************************************************************/
    /* Compute origin point of top item relative to cnr.                */
    /********************************************************************/
    IPoint ptOrigin =
           pSrcCnr->iconRectangle(pFirstObj).bottomLeft() - sizeOffset;

    /********************************************************************/
    /* If object is selected, query for others just in case multiple    */
    /* objects were selected.                                           */
    /********************************************************************/
    if (pSrcCnr->isSelected(pFirstObj))
    {
      /****************************************************************/
      /* Define cursor to process selected objects only.              */
      /****************************************************************/
      IContainerControl::ObjectCursor curs(*pSrcCnr,
                                           IContainerObject::selected);

      for (curs.setToFirst(); curs.isValid(); curs.setToNext())
      {
        IContainerObject *pObj;

        /**************************************************************/
        /* Avoid putting first object twice.                          */
        /**************************************************************/
        if ((pObj = pSrcCnr->objectAt(curs)) != pFirstObj)
        {
          /************************************************************/
          /* Create container source item - throw exception if        */
          /* create failed.                                           */
          /************************************************************/
          pCnrItem = new IDMCnrItem(pSrcCnr,
                                    pObj,
                                    pSrcOp,
                                (pSrcCnr->iconRectangle(pObj).bottomLeft() -
                                                              ptOrigin));
          if ( ! pCnrItem.valid() )
          {
               //Throw exception here!
          }

          /************************************************************/
          /* Add the item to the collection and increment item count. */
          /************************************************************/
          pSrcOp->addItem(pCnrItem);
          ulCnt++;

        }/* end if */
      }/* end for */
    }/* end if */
  }/* end if */

  /********************************************************************/
  /* Set the drag image style.                                        */
  /********************************************************************/
  pSrcOp->setImageStyle(IDM::allStacked);

  return((ulCnt!=0));
}

/*------------------------------------------------------------------------------
| IDMCnrItem::containerId                                                      |
|                                                                              |
| Returns the source container's ID.                                           |
------------------------------------------------------------------------------*/
unsigned long IDMCnrItem :: containerId() const
{
  return(sourceContainer()->id());
}

/*------------------------------------------------------------------------------
| IDMCnrItem::sourceContainer                                                  |
|                                                                              |
| Returns the source container.                                                |
------------------------------------------------------------------------------*/
IContainerControl* IDMCnrItem :: sourceContainer() const
{
  return(this->pContainerCtl);
}

/*------------------------------------------------------------------------------
| IDMCnrItem::targetDrop                                                       |
|                                                                              |
| Add the container object into the target container.                          |
------------------------------------------------------------------------------*/
bool IDMCnrItem :: targetDrop(IDMTargetDropEvent &event)
{
  IFUNCTRACE_DEVELOP();
  bool bSuccess = false;

  /********************************************************************/
  /* If we don't have a source object, then we can't do anything else.*/
  /********************************************************************/
  if (!object())
  {
    ITRACE_DEVELOP( "returning false - no source object supplied" );
    ITHROWLIBRARYERROR(IC_NO_SOURCE_OBJECT_SUPPLIED,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
    return(bSuccess);
  }

  /********************************************************************/
  /* Create a temp container item to make sure item is protected      */
  /* from destruction while we're in this routine.                    */
  /********************************************************************/
  IDMItem::Handle pDI(this) ;

  /********************************************************************/
  /* Get pointer to container target operation                        */
  /********************************************************************/
  IDMTargetOperation *pTgtOp = targetOperation();

  /********************************************************************/
  /* Get pointers to both the source and target containers            */
  /********************************************************************/
  IContainerControl *pSrcCnr = sourceContainer();
  IContainerControl *pTgtCnr = (IContainerControl *)pTgtOp->targetWindow();

  /********************************************************************/
  /* Query the operation, and handle accordingly                      */
  /********************************************************************/
  if ((pTgtOp->operation() == IDMOperation::move) ||
      (pTgtOp->operation() == IDMOperation::drag))
  {
    /******************************************************************/
    /* This is a move operation                                       */
    /* Note:  Move is the CUA default operation                       */
    /******************************************************************/

    /******************************************************************/
    /* Make sure target object is different than source               */
    /******************************************************************/
    if (object() != pTgtOp->containerObject())
    {
      /****************************************************************/
      /* Move it, noting the specific characteristics of each view    */
      /****************************************************************/
      if (pTgtCnr->isTreeView())
      {
        /**************************************************************/
        /* Move over node in tree view and add object as child of node*/
        /* over which it was dropped                                  */
        /**************************************************************/
        pSrcCnr->moveObjectTo((IContainerObject *)object(),
                              pTgtOp->containerObject(),
                              pTgtCnr,
                              0,
                              pTgtOp->dropPosition(pDI, event));
      }
      else
      {
        /**************************************************************/
        /* This is a non-tree view or white space                     */
        /**************************************************************/
        if ((pTgtCnr->isNameView() || pTgtCnr->isTextView() ||
             pTgtCnr->isDetailsView()) && pTgtOp->wasDragAfter())
        {
          /************************************************************/
          /* This is name, text or details view for a dragafter       */
          /* object ...  Move item after object ...                   */
          /************************************************************/
          pSrcCnr->moveObjectTo((IContainerObject *)object(),
                                0,
                                pTgtCnr,
                                pTgtOp->containerObject(),
                                pTgtOp->dropPosition(pDI, event));
        }
        else
        {
          /************************************************************/
          /* This is a drop on white space in icon view               */
          /************************************************************/
          if ((pSrcCnr == pTgtCnr)  &&  (pSrcCnr->isIconView()))
          {
            pSrcCnr->moveIconTo((IContainerObject *)object(),
                                 pTgtOp->dropPosition(pDI, event));
          }
          else
          {
            pSrcCnr->moveObjectTo((IContainerObject *)object(),
                                  0,
                                  pTgtCnr,
                                  0,
                                  pTgtOp->dropPosition(pDI, event));
          }
        }
      }

      bSuccess = true;
    }
  } /* end drag move */

  if (pTgtOp->operation() == IDMOperation::copy)
  {
    /******************************************************************/
    /* This is a copy operation                                       */
    /******************************************************************/

    /******************************************************************/
    /* Make sure target object is different than source               */
    /******************************************************************/
    if (object() != pTgtOp->containerObject())
    {
      IContainerObject *pCopyObj;

      /****************************************************************/
      /* Copy it, noting the specific characteristics of each view    */
      /****************************************************************/
      if (pTgtCnr->isTreeView())
      {
        /**************************************************************/
        /* Copy over node in tree view and add object as child of     */
        /* node over which it was dropped                             */
        /**************************************************************/
        pCopyObj = pSrcCnr->copyObjectTo((IContainerObject *)object(),
                                         pTgtOp->containerObject(),
                                         pTgtCnr,
                                         0,
                                         pTgtOp->dropPosition(pDI, event));
      }
      else
      {
        /**************************************************************/
        /* This is a non-tree view or white space                     */
        /**************************************************************/
        if ((pTgtCnr->isNameView() || pTgtCnr->isTextView() ||
             pTgtCnr->isDetailsView()) && pTgtOp->wasDragAfter())
        {
          /************************************************************/
          /* This is name, text or details view for a dragafter       */
          /* object ... Copy item after object ...                    */
          /************************************************************/
          pCopyObj = pSrcCnr->copyObjectTo((IContainerObject *)object(),
                                           0,
                                           pTgtCnr,
                                           pTgtOp->containerObject(),
                                           pTgtOp->dropPosition(pDI, event));
        }
        else
        {
          /************************************************************/
          /* This is a drop on white space in icon view               */
          /************************************************************/
          pCopyObj = pSrcCnr->copyObjectTo((IContainerObject *)object(),
                                           0,
                                           pTgtCnr,
                                           0,
                                           pTgtOp->dropPosition(pDI, event));
        }
      }

      /****************************************************************/
      /* Save a pointer to the copied object inside this drag item    */
      /****************************************************************/
      setObject( pCopyObj );
      bSuccess = true;
    }
  } /* end drag copy */

  return(bSuccess);
}

