// Revision: 51 1.6.1.2 source/ui/drag/idmdatob.cpp, dragdrop, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: idmdatob.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMDataObject                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <idmdatob.hpp>
#include <itrace.hpp>
#include <idmfmten.hpp>
#include <idmformt.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMDATOB_CPP_
  #include <ipagetun.h>
#endif

IDMDataObject::IDMDataObject( IWindow *renderer,
                              IDMEnumFormat *formatEnum,
                              IStream* pIStreamParam )
  : m_cRef(0)
  , m_cFormats( 0 )
  , m_hGlobal(NULL)
  , _renderer( renderer )
  , pIStream( pIStreamParam )

  {
  if (pIStream != NULL)
    pIStream->AddRef();

  if (SUCCEEDED(formatEnum->Clone( &m_aFormatEtc )))
    {
    FORMATETC rc;
    // Get the number of formats.
    m_aFormatEtc->Reset();
    // Do we really need to know how many formats there are.
//    while (m_aFormatEtc->Next(1, &rc, NULL) == S_OK)
//      m_cFormats++;
//    m_aFormatEtc->Reset();
    }
  ITRACE_DEVELOP( "Number of formats in IDMDataObject is " +
      IString( m_cFormats ));
  }

IDMDataObject::~IDMDataObject()
  {
  m_aFormatEtc->Release();
  if (pIStream != NULL)
    pIStream->Release();
  }

HRESULT __stdcall IDMDataObject::QueryInterface(  REFIID riid,
                                                  void **ppvoid )

  {
  IMODTRACE_DEVELOP("IDMDataObject::QueryInterface");
  ITRACE_DEVELOP(IString("\t...interface ") +
    (IsEqualIID( riid, IID_IDataObject ) ?
      IString("IDataObject") :
    (IsEqualIID( riid, IID_IUnknown ) ?
      IString("IUnknown") :
    IString("...not one of our interfaces")
    )) );

  *ppvoid = NULL;       // Always NULL our output parameter

  if ( IsEqualIID( riid, IID_IUnknown ) ||
       IsEqualIID( riid, IID_IDataObject ))
    {
    this->AddRef();
    *ppvoid = (void *) this;
    }
  if ( NULL != *ppvoid )
    {
    return NOERROR;
    }
  return ResultFromScode( E_NOINTERFACE);
  }

ULONG __stdcall IDMDataObject::AddRef( )

  {
  ++m_cRef;

  ITRACE_DEVELOP(IString("IDMDataObject::AddRef - exit count == ") +
                        (IString(m_cRef)));
  return m_cRef;
  }

ULONG __stdcall IDMDataObject::Release( )

  {
  ULONG cRefT;

  cRefT = --m_cRef;
  ITRACE_DEVELOP(IString("IDMDataObject::Release - exit count == ") +
                        (IString(cRefT)));

  if ( m_cRef == 0 )
    {
    ITRACE_DEVELOP("\tIDMDataObject::Release will delete self");
    delete this;
    }

  return cRefT;
  }

HRESULT __stdcall IDMDataObject::GetData ( FORMATETC *pformatetcIn,
                                 STGMEDIUM *pmedium)
{
  IMODTRACE_DEVELOP("IDMDataObject::GetData");

  SCODE sc = DATA_E_FORMATETC;

  if (S_OK == QueryGetData(pformatetcIn))
  {
    CLIPFORMAT clipFmt =
          CLIPFORMAT(::RegisterClipboardFormat(formatDragInfo()));
    if (clipFmt == pformatetcIn->cfFormat)
    {
      IStream* pClone = NULL;
      if (SUCCEEDED(pIStream->Clone(&pClone)))
      {
        ITRACE_DEVELOP("Data object returning medium for DRAGINFO format");
        pmedium->tymed          = TYMED_ISTREAM;
        pmedium->pstm           = pClone;
        pmedium->pUnkForRelease = NULL; // Caller owns medium
        sc = S_OK;
      }
    }
    else
    {
      ITRACE_DEVELOP("Data object sending DM_RENDER to source hwnd");
      _renderer->sendEvent( DM_RENDER,
                            IEventParameter1( pformatetcIn),
                            IEventParameter2( pmedium ) );
      sc = S_OK;
    }

  }
  return ResultFromScode( sc );
}

HRESULT __stdcall IDMDataObject::GetDataHere ( FORMATETC *pformatetc,
                                        STGMEDIUM *pmedium)

  {
  IMODTRACE_DEVELOP("IDMDataObject::GetDataHere");

  return ResultFromScode( E_NOTIMPL );
  }

HRESULT __stdcall IDMDataObject::QueryGetData ( FORMATETC *pformatetc)

  {
  IMODTRACE_DEVELOP( "IDMDataObject::QueryGetData" );

  FORMATETC formatEtc;
  ULONG count;

  m_aFormatEtc->Reset();
  while (m_aFormatEtc->Next(1, &formatEtc, &count) == S_OK)
    {
    if (( formatEtc.dwAspect & pformatetc->dwAspect) &&
        ( formatEtc.cfFormat == pformatetc->cfFormat ) &&
        ( formatEtc.tymed == pformatetc->tymed ) )
      {
      ITRACE_DEVELOP( "We support data");
      return ResultFromScode(S_OK);
      }
    }
  return ResultFromScode(DATA_E_FORMATETC);
  }

HRESULT __stdcall IDMDataObject::GetCanonicalFormatEtc( FORMATETC *pformatectIn,
                                              FORMATETC *pformatetcOut)
  {
  IMODTRACE_DEVELOP( "IDMDataObject::GetCanonicalFormatEtc" );
  pformatetcOut = NULL;
  return ResultFromScode( DATA_S_SAMEFORMATETC );
  }

HRESULT __stdcall IDMDataObject::SetData( FORMATETC *pformatetc,
                                          STGMEDIUM *pmedium,
                                          BOOL fRelease)
{
  IMODTRACE_DEVELOP( "IDMDataObject::SetData");

  return E_NOTIMPL;
}

HRESULT __stdcall IDMDataObject::EnumFormatEtc( DWORD dwDirection,
                                      IEnumFORMATETC **ppenumFormatEtc)
  {
  IMODTRACE_DEVELOP( "IDMDataObject::EnumFormatEtc" );
  SCODE sc = E_INVALIDARG;

  *ppenumFormatEtc = NULL;

  if (dwDirection == DATADIR_GET)
    {
    if (SUCCEEDED(m_aFormatEtc->Clone( ppenumFormatEtc )))
      {
      sc = E_OUTOFMEMORY;
      if (ppenumFormatEtc != NULL)
        {
        (*ppenumFormatEtc)->Reset( );
        sc = S_OK;
        }
      }
    }
  return ResultFromScode(sc);
  }

HRESULT __stdcall IDMDataObject::DAdvise( FORMATETC *pformatetc,
                                DWORD advf,
                                IAdviseSink *pAdvSink,
                                                                                                                DWORD *pdwConnection)
  {
  IMODTRACE_DEVELOP( "IDMDataObject::DAdvise" );
  return ResultFromScode(OLE_E_ADVISENOTSUPPORTED);
  }

HRESULT __stdcall IDMDataObject::DUnadvise( DWORD dwConnection)

  {
  IMODTRACE_DEVELOP( "IDMDataObject::DUnadvise" );
  return ResultFromScode( E_FAIL );
  }

HRESULT __stdcall IDMDataObject::EnumDAdvise( IEnumSTATDATA **ppenumAdvise)

  {
  IMODTRACE_DEVELOP( "IDMDataObject::EnumDAdvise" );
  return ResultFromScode(OLE_E_ADVISENOTSUPPORTED);
  }

/*------------------- Misc member functions ------------------------------------*/

const char* IDMDataObject::formatDragInfo()
{
  return "IOC_DRAGINFO";
}

