// Revision: 53 1.5.1.2 source/ui/drag/idmditm.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmditm.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDragItem                                                                *
*     IDragItemWrapper                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#ifdef IC_PM
extern "C" {
  // include PM headers
  #define INCL_WINSTDDRAG
  #define INCL_DOSPROCESS
  #include <os2.h>
  #include <string.h>
}
#endif

  #include <itrace.hpp>
  #include <iexcept.hpp>
  #include <idmditm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMDITM_CPP_
  #include <ipagetun.h>
#endif

//@@Rob.  Do we need this?

/*------------------------------------------------------------------------------
| IDragItem::IDragItem                                                         |
|                                                                              |
| Constructor.                                                                 |
|   Try to keep this light-weight, they are created frequently.                |
------------------------------------------------------------------------------*/
IDragItem :: IDragItem()
  : strContainerName(),
    strSourceName(),
    strTargetName(),
    strTypes(),
    strRMFs(),
    sizeImageOffset(0,0),
    hwndItem(0),
    ulItemID(0),
    fsControl(0),
    fsSupportedOps(0)
{ }

/*------------------------------------------------------------------------------
| IDRagItem::~IDragItem                                                        |
------------------------------------------------------------------------------*/
IDragItem :: ~IDragItem() {}

/*------------------------------------------------------------------------------
| IDragItem::marshall                                                          |
|                                                                              |
| Create a string representation of *this, starting at outStr[startPos].       |
| Return startPos + the length of the string representation.                   |
|                                                                              |
| The return value is calculated correctly even if outStr is 0.                |
------------------------------------------------------------------------------*/
unsigned long IDragItem :: marshall ( unsigned long startPos,
                                      char* outStr ) const
{
  unsigned long
  newPos = IDMMarshall::marshall( strTypes                    , startPos, outStr);
  newPos = IDMMarshall::marshall( strRMFs                       , newPos, outStr);
  newPos = IDMMarshall::marshall( strContainerName              , newPos, outStr);
  newPos = IDMMarshall::marshall( strSourceName                 , newPos, outStr);
  newPos = IDMMarshall::marshall( strTargetName                 , newPos, outStr);
  newPos = IDMMarshall::marshall( (long)sizeImageOffset.width() , newPos, outStr);
  newPos = IDMMarshall::marshall( (long)sizeImageOffset.height(), newPos, outStr);
 #ifdef IC_MOTIFWIN 
  ITRACE_DEVELOP(IString("marshalling hwndItem : ") + IString((long)(void*)hwndItem));
  newPos = IDMMarshall::marshall( (long)(IWindowHandle::Value)hwndItem   , newPos, outStr);
#else
  newPos = IDMMarshall::marshall( (long)(IHandle::Value)hwndItem , newPos, outStr);
#endif
  newPos = IDMMarshall::marshall( (long)ulItemID                , newPos, outStr);
  newPos = IDMMarshall::marshall( (long)fsControl               , newPos, outStr);
  newPos = IDMMarshall::marshall( (long)fsSupportedOps          , newPos, outStr);
  return newPos;
}

/*------------------------------------------------------------------------------
| IDragItem::unmarshall                                                        |
|                                                                              |
| Initialize *this from a string representation, previously created using      |
| marshall.                                                                    |
|                                                                              |
| Return inStr +                                                               |
| the number of characters used from the string representation.                |
------------------------------------------------------------------------------*/
const char* IDragItem :: unmarshall ( const char* inStr )
{
  long result;
  const char*
  newStr = IDMMarshall::unmarshall( strTypes        , inStr);
  newStr = IDMMarshall::unmarshall( strRMFs         , newStr);
  newStr = IDMMarshall::unmarshall( strContainerName, newStr);
  newStr = IDMMarshall::unmarshall( strSourceName   , newStr);
  newStr = IDMMarshall::unmarshall( strTargetName   , newStr);

  newStr = IDMMarshall::unmarshall( result          , newStr);
  sizeImageOffset.setWidth((ISize::Coord)result);

  newStr = IDMMarshall::unmarshall( result          , newStr);
  sizeImageOffset.setHeight((ISize::Coord)result);

  newStr = IDMMarshall::unmarshall( result          , newStr);
#ifdef IC_MOTIFWIN 
  hwndItem = IWindowHandle((IWindowHandle::Value)(long)result);
  ITRACE_DEVELOP(IString("unmarshalled hwndItem : ") + IString((long)(void*)hwndItem));
#else
  hwndItem = IWindowHandle((unsigned long)result);
#endif

  newStr = IDMMarshall::unmarshall( result          , newStr);
  ulItemID = (unsigned long)result;

  newStr = IDMMarshall::unmarshall( result          , newStr);
  fsControl = (unsigned short) result;

  newStr = IDMMarshall::unmarshall( result          , newStr);
  fsSupportedOps = (unsigned short) result;

  return newStr;
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::IDragItemWrapper                                           |
|                                                                              |
| Constructor.                                                                 |
|   Try to keep this light-weight, they are created frequently.                |
------------------------------------------------------------------------------*/
IDragItemWrapper :: IDragItemWrapper( _DRAGITEM& aDragItem )
  : theDragItem(aDragItem)
#ifdef IC_PM
   ,workDragItem( *new IDragItem )
#endif
{ }

/*------------------------------------------------------------------------------
| IDragItemWrapper::IDragItemWrapper                                           |
|                                                                              |
| Copy constructor.                                                            |
------------------------------------------------------------------------------*/
IDragItemWrapper :: IDragItemWrapper( const IDragItemWrapper& diw )
  : theDragItem(diw.theDragItem)
#ifdef IC_PM
   ,workDragItem( *new IDragItem ) // No need to copy this work drag item.
#endif
{ }

/*------------------------------------------------------------------------------
| IDragItemWrapper::~IDragItemWrapper                                          |
------------------------------------------------------------------------------*/
IDragItemWrapper :: ~IDragItemWrapper()
{
#ifdef IC_PM
  delete &workDragItem;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::asDragItemPtr                                              |
------------------------------------------------------------------------------*/
_DRAGITEM* IDragItemWrapper :: asDragItemPtr() const
{
  return &theDragItem;
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::sourceWindowHandle                                         |
------------------------------------------------------------------------------*/
IWindowHandle IDragItemWrapper :: sourceWindowHandle() const
{
  return theDragItem.hwndItem;
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::itemID                                                     |
------------------------------------------------------------------------------*/
unsigned long IDragItemWrapper :: itemID() const
{
  return theDragItem.ulItemID;
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::dragAttrs                                                  |
------------------------------------------------------------------------------*/
unsigned short IDragItemWrapper :: dragAttrs() const
{
  return theDragItem.fsControl;
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::dragSupOps                                                 |
------------------------------------------------------------------------------*/
unsigned short IDragItemWrapper :: dragSupOps() const
{
  return theDragItem.fsSupportedOps;
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::imageOffset                                                |
------------------------------------------------------------------------------*/
const ISize& IDragItemWrapper :: imageOffset() const
{
#ifdef IC_PM
  workDragItem.sizeImageOffset
         = ISize( theDragItem.cxOffset, theDragItem.cyOffset );
  return workDragItem.sizeImageOffset;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  return theDragItem.sizeImageOffset;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::types                                                      |
------------------------------------------------------------------------------*/
const IString& IDragItemWrapper :: types() const
{
#ifdef IC_PM
  workDragItem.strTypes = stringFromHandle( theDragItem.hstrType );
  return workDragItem.strTypes;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  return theDragItem.strTypes;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::rmfs                                                       |
------------------------------------------------------------------------------*/
const IString& IDragItemWrapper :: rmfs() const
{
#ifdef IC_PM
  workDragItem.strRMFs = stringFromHandle( theDragItem.hstrRMF );
  return workDragItem.strRMFs;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  return theDragItem.strRMFs;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::containerName                                              |
------------------------------------------------------------------------------*/
const IString& IDragItemWrapper :: containerName() const
{
#ifdef IC_PM
  workDragItem.strContainerName
         = stringFromHandle( theDragItem.hstrContainerName );
  return workDragItem.strContainerName;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  return theDragItem.strContainerName;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::sourceName                                                 |
------------------------------------------------------------------------------*/
const IString& IDragItemWrapper :: sourceName() const
{
#ifdef IC_PM
  workDragItem.strSourceName
         = stringFromHandle( theDragItem.hstrSourceName );
  return workDragItem.strSourceName;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  return theDragItem.strSourceName;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::targetName                                                 |
------------------------------------------------------------------------------*/
const IString& IDragItemWrapper :: targetName() const
{
#ifdef IC_PM
  workDragItem.strTargetName
         = stringFromHandle( theDragItem.hstrTargetName );
  return workDragItem.strTargetName;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  return theDragItem.strTargetName;
#endif
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setSourceWindowHandle                                      |
------------------------------------------------------------------------------*/
IDragItemWrapper&
IDragItemWrapper :: setSourceWindowHandle ( const IWindowHandle& window )
{
  theDragItem.hwndItem = window;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setItemID                                                  |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setItemID ( unsigned long ulID )
{
  theDragItem.ulItemID = ulID;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setTypes                                                   |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setTypes ( const IString& types )
{
#ifdef IC_PM
  setHSTR( theDragItem.hstrType, handleFromString( types ) );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  theDragItem.strTypes = types;
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setRMFs                                                    |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setRMFs ( const IString& rmfs )
{
#ifdef IC_PM
  setHSTR( theDragItem.hstrRMF, handleFromString( rmfs ) );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  theDragItem.strRMFs = rmfs;
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setContainerName                                           |
------------------------------------------------------------------------------*/
IDragItemWrapper&
IDragItemWrapper :: setContainerName ( const IString& containerName )
{
#ifdef IC_PM
  setHSTR( theDragItem.hstrContainerName, handleFromString(containerName) );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  theDragItem.strContainerName = containerName;
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setTargetName                                              |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setTargetName (const IString& targetName)
{
#ifdef IC_PM
  setHSTR( theDragItem.hstrTargetName, handleFromString(targetName) );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  theDragItem.strTargetName = targetName;
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setSourceName                                              |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setSourceName (const IString& sourceName)
{
#ifdef IC_PM
  setHSTR( theDragItem.hstrSourceName, handleFromString(sourceName) );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  theDragItem.strSourceName = sourceName;
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setImageOffset                                             |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setImageOffset ( const ISize& offset )
{
#ifdef IC_PM
  theDragItem.cxOffset = offset.width();
  theDragItem.cyOffset = offset.height();
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  theDragItem.sizeImageOffset = offset;
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setDragAttrs                                               |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setDragAttrs ( unsigned short attrs )
{
  theDragItem.fsControl = attrs;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::setDragSupOps                                              |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: setDragSupOps ( unsigned short supOps )
{
  theDragItem.fsSupportedOps = supOps;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragItemWrapper::clearStrings                                               |
------------------------------------------------------------------------------*/
IDragItemWrapper& IDragItemWrapper :: clearStrings ( )
{
#ifdef IC_PM
  setHSTR( theDragItem.hstrType         , handleFromString(types()) );
  setHSTR( theDragItem.hstrRMF          , handleFromString(rmfs()) );
  setHSTR( theDragItem.hstrContainerName, handleFromString(containerName()) );
  setHSTR( theDragItem.hstrSourceName   , handleFromString(sourceName()) );
  setHSTR( theDragItem.hstrTargetName   , handleFromString(targetName()) );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  IString nullString;
  theDragItem.strTypes         = nullString;
  theDragItem.strRMFs          = nullString;
  theDragItem.strContainerName = nullString;
  theDragItem.strSourceName    = nullString;
  theDragItem.strTargetName    = nullString;
#endif
  return (*this);
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDragItemWrapper::setHSTR                                    static, private |
|                                                                              |
| Set the HSTR to the new value.  If it previously was not null, release it.   |
------------------------------------------------------------------------------*/
void IDragItemWrapper :: setHSTR ( unsigned long& oldHSTR,
                                   const IStringHandle& newHSTR )
{
  HSTR old = oldHSTR;
  oldHSTR = newHSTR;

  if ( old )
  {
    if ( !DrgDeleteStrHandle( old ) )
      ITHROWGUIERROR( "DrgDeleteStrHandle" );
  }
}
#endif

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDragItemWrapper::stringFromHandle                           static, private |
|                                                                              |
| Create an IString from a PM HSTR.                                            |
------------------------------------------------------------------------------*/
IString IDragItemWrapper :: stringFromHandle ( const IStringHandle& hstr )
{
  char szBuf[256];
  // DrgQueryStrName returns a null string when hstr is null.
  DrgQueryStrName(hstr, sizeof(szBuf), (PSZ)szBuf);
  return(IString(szBuf));
}
#endif

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDragItemWrapper::handleFromString                           static, private |
|                                                                              |
| Create a PM HSTR from a char*, and return an IStringHandle to it.            |
| If str is null, return a null handle.                                        |
------------------------------------------------------------------------------*/
IStringHandle IDragItemWrapper :: handleFromString ( const IString& str )
{
  PSZ pStr = (PSZ)str;
  if ( strlen(pStr) )
   {
    IStringHandle sh = DrgAddStrHandle(pStr);
    if (sh)
      return (sh);
    else
     {
      ITHROWGUIERROR("DrgAddStrHandle");
     }
   }
  return (IStringHandle(0));
}
#endif
