// Revision: 56 1.7.1.3 source/ui/drag/idmdnfo.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmdnfo.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDragInfo                                                                *
*     IDragInfoWrapper                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif
#ifdef IC_PM
  // include PM headers
  #define INCL_WINSTDDRAG
  #define INCL_DOSPROCESS
  #include <os2.h>
#endif

#ifdef IC_WIN
  #include <shlobj.h>
  #include <wchar.h>
#endif
  #include <string.h>
}

  #include <itrace.hpp>
  #include <iexcept.hpp>
  #include <iapp.hpp>
  #include <idmoper.hpp>
  #include <idmditm.hpp>
  #include <idmdnfo.hpp>

#ifdef IC_WIN
 #include <idmformt.hpp>
 #include <idmdatob.hpp>
 #include <iwindefs.h>
#endif

#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMDNFO_CPP_
  #include <ipagetun.h>
#endif


//@@Rob.  Do we need this?

#if defined(IC_MOTIF) || defined(IC_WIN)
/*------------------------------------------------------------------------------
| IDragInfo::IDragInfo                                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDragInfo :: IDragInfo( unsigned short numDItems )
  : pointDrop(0,0),
    hwndSource(),
    pidSource( IApplication::currentPID() ),
    usDrop( IDM::notOk ),
    usDefaultOp( IDMOperation::drag ),
    usOperation( IDMOperation::drag ),
    cditem( numDItems ),
    pDragItems( 0 )
{
 if ( cditem )
   pDragItems = new IDragItem[cditem];
}

/*------------------------------------------------------------------------------
| IDragInfo::~IDragInfo                                                        |
------------------------------------------------------------------------------*/
IDragInfo :: ~IDragInfo()
{
 delete [] pDragItems;
}

/*------------------------------------------------------------------------------
| IDragInfo::marshall                                                          |
|                                                                              |
| Create a string representation of *this, starting at outStr[startPos].       |
| Return startPos + the length of the string representation.                   |
|                                                                              |
| The return value is calculated correctly even if outStr is 0.                |
------------------------------------------------------------------------------*/
unsigned long IDragInfo :: marshall ( unsigned long startPos,
                                      char* outStr ) const
{
 unsigned long
 newPos = IDMMarshall::marshall( (long)pointDrop.x()          , startPos, outStr); // 1
 newPos = IDMMarshall::marshall( (long)pointDrop.y()          ,   newPos, outStr); // 2
#ifdef IC_WIN
 newPos = IDMMarshall::marshall( (long)(IProcessId::Value)hwndSource   , newPos, outStr);      // 3
#else
 newPos = IDMMarshall::marshall( (long)hwndSource.asUnsigned(),   newPos, outStr); // 3
#endif
#ifdef IC_WIN
 newPos = IDMMarshall::marshall( (long)(IProcessId::Value)pidSource, newPos, outStr);// 4
#else
 newPos = IDMMarshall::marshall( (long)pidSource              ,   newPos, outStr); // 4
#endif
 newPos = IDMMarshall::marshall( (long)usDrop                 ,   newPos, outStr); // 5
 newPos = IDMMarshall::marshall( (long)usDefaultOp            ,   newPos, outStr); // 6
 newPos = IDMMarshall::marshall( (long)usOperation            ,   newPos, outStr); // 7
 newPos = IDMMarshall::marshall( (long)cditem                 ,   newPos, outStr); // 8
 ITRACE_DEVELOP(IString("marshall item count: ") + IString(cditem));
 for ( unsigned long i=0; i<cditem; ++i )
  {
   newPos = pDragItems[i].marshall( newPos, outStr );
  }
 return newPos;
}

/*------------------------------------------------------------------------------
| IDragInfo::unmarshall                                                        |
|                                                                              |
| Initialize *this from a string representation, previously created using      |
| marshall.                                                                    |
|                                                                              |
| Return inStr +                                                               |
| the number of characters used from the string representation.                |
------------------------------------------------------------------------------*/
const char* IDragInfo :: unmarshall ( const char* inStr )
{
 long result;
 const char*
 newStr = IDMMarshall::unmarshall( result          , inStr);  // 1
 pointDrop.setX( (IPoint::Coord)result );

 newStr = IDMMarshall::unmarshall( result          , newStr); // 2
 pointDrop.setY( (IPoint::Coord)result );

 newStr = IDMMarshall::unmarshall( result          , newStr); // 3
#ifdef IC_MOTIFWIN 
 hwndSource = IWindowHandle((IWindowHandle::Value)(long)result);
#else
 hwndSource = IWindowHandle((unsigned long)result);
#endif

 newStr = IDMMarshall::unmarshall( result          , newStr); // 4
#ifdef IC_WIN
 pidSource = IProcessId((IProcessId::Value)result);
#else
 pidSource = IProcessId((unsigned long)result);
#endif

 newStr = IDMMarshall::unmarshall( result          , newStr); // 5
 usDrop = (unsigned short)result;

 newStr = IDMMarshall::unmarshall( result          , newStr); // 6
 usDefaultOp = (unsigned short)result;

 newStr = IDMMarshall::unmarshall( result          , newStr); // 7
 usOperation = (unsigned short)result;

 newStr = IDMMarshall::unmarshall( result          , newStr); // 8
 cditem = (unsigned short)result;

 delete [] pDragItems;
 pDragItems = 0;
 if ( cditem )
   pDragItems = new IDragItem[cditem];
 for ( unsigned long i=0; i<cditem; ++i )
  {
   newStr = pDragItems[i].unmarshall( newStr );
  }

 return newStr;
}

/*------------------------------------------------------------------------------
| IDragInfo::dropIndicator                                                     |
------------------------------------------------------------------------------*/
IDM::DropIndicator IDragInfo :: dropIndicator ( ) const
{
 return (IDM::DropIndicator)usDrop;
}

/*------------------------------------------------------------------------------
| IDragInfo::dropOperation                                                     |
------------------------------------------------------------------------------*/
unsigned short IDragInfo :: dropOperation ( ) const
{
 return usDefaultOp;
}

/*------------------------------------------------------------------------------
| IDragInfo::setDragResult                                                     |
------------------------------------------------------------------------------*/
IDragInfo& IDragInfo :: setDragResult ( const IEventResult& result )
{
 //
 // This is from the PM programming reference under the DM_DRAGOVER message:
 //
 //   If usOperation in DRAGINFO is DO_DEFAULT or DO_UNKNOWN and the target
 //   returns DOR_DROP for usDrop, usDefaultOp should be set to reflect what
 //   the target defines as the default operation.  This information is
 //   used to provide the appropriate modification to the drag pointer,
 //   and the target's default operation will be passed in the usOperation
 //   field of DRAGINFO in the DM_DROP message.
 //
 // To emulate this behaviour:
 //
 //   If dragInfoWrapper.dragOperation() is IDMOperation::drag or
 //   >= IDMOperation::unknown and the dragOver result parameter1
 //   is IDM::ok, then the dragOver result parameter2 has been set to
 //   reflect what the target defines as the default operation.
 //   Otherwise, set the default operation to be the same as the
 //   current operation.  The default operation will be used to modify
 //   the drag pointer appearance to reflect the default operation, and
 //   it will be copied into usOperation just before the DM_DROP is sent.
 //
 usDrop = result.number1();
 usDefaultOp = result.number2();
 if ( ( usOperation==IDMOperation::drag || usOperation>=IDMOperation::unknown )
   && usDrop == IDM::ok )
  {
   //
   // The usDrop parameter is treated as DOR_NEVERDROP if we get
   // here and usDefaultOp is not one of the "defined values".
   //
   if ( usDefaultOp != IDMOperation::move
     && usDefaultOp != IDMOperation::copy
     && usDefaultOp != IDMOperation::link
     && usDefaultOp <  IDMOperation::unknown )
    {
     usDrop = IDM::neverOk;
     usDefaultOp = usOperation;
    }
  }
 else
   usDefaultOp = usOperation;
 return *this;
}
#endif

/*------------------------------------------------------------------------------
| IDragInfoWrapper::IDragInfoWrapper                                           |
|                                                                              |
| Constructor.                                                                 |
|   Try to keep this light-weight, they are created frequently.                |
------------------------------------------------------------------------------*/
IDragInfoWrapper :: IDragInfoWrapper( _DRAGINFO& aDragInfo )
  : theDragInfo(aDragInfo)
{ }

/*------------------------------------------------------------------------------
| IDragInfoWrapper::IDragInfoWrapper                                           |
|                                                                              |
| Constructor.                                                                 |
|   Try to keep this light-weight, they are created frequently.                |
------------------------------------------------------------------------------*/
IDragInfoWrapper :: IDragInfoWrapper( const IDMOperation& anIDMOperation )
  : theDragInfo( *anIDMOperation.dragInfo() )
{ }

/*------------------------------------------------------------------------------
| IDragInfoWrapper::IDragInfoWrapper                                           |
|                                                                              |
| Copy constructor.                                                            |
------------------------------------------------------------------------------*/
IDragInfoWrapper :: IDragInfoWrapper( const IDragInfoWrapper& diw )
  : theDragInfo(diw.theDragInfo)
{ }

/*------------------------------------------------------------------------------
| IDragInfoWrapper::~IDragInfoWrapper                                          |
------------------------------------------------------------------------------*/
IDragInfoWrapper :: ~IDragInfoWrapper() {}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::asDragInfoPtr                                              |
------------------------------------------------------------------------------*/
_DRAGINFO* IDragInfoWrapper :: asDragInfoPtr() const
{
 return &theDragInfo;
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::operator[]                                                 |
------------------------------------------------------------------------------*/
_DRAGITEM& IDragInfoWrapper :: operator[]( unsigned long ulItem ) const
{
#ifdef IC_PM
 _DRAGITEM *pPMDItem = DrgQueryDragitemPtr (&theDragInfo, ulItem);
 if (!pPMDItem)
  {
   ITHROWGUIERROR("DrgQueryDragitemPtr");
  }
 return *pPMDItem;
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
 if ( ulItem >= theDragInfo.cditem )
  {
   ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
  }
 return theDragInfo.pDragItems[ulItem];
#endif
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::dropPoint                                                  |
------------------------------------------------------------------------------*/
IPoint IDragInfoWrapper :: dropPoint() const
{
#ifdef IC_PM
 return IPoint( theDragInfo.xDrop, theDragInfo.yDrop );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
 return theDragInfo.pointDrop;
#endif
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::sourceWindowHandle                                         |
------------------------------------------------------------------------------*/
IWindowHandle IDragInfoWrapper :: sourceWindowHandle() const
{
 return theDragInfo.hwndSource;
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::sourcePID                                                  |
------------------------------------------------------------------------------*/
IProcessId IDragInfoWrapper :: sourcePID() const
{
#if defined(IC_MOTIF) || defined(IC_WIN)
 return theDragInfo.pidSource;
#endif
#ifdef IC_PM
  PID  srcPid;
  TID  tid;
  if ((!WinQueryWindowProcess (theDragInfo.hwndSource, &srcPid, &tid)) )
  {
    ITHROWGUIERROR("WinQueryWindowProcess");
  }
  return IProcessId( srcPid );
#endif
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::dragOperation                                              |
------------------------------------------------------------------------------*/
unsigned short IDragInfoWrapper :: dragOperation() const
{
 return theDragInfo.usOperation;
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::numDragItems                                               |
------------------------------------------------------------------------------*/
unsigned short IDragInfoWrapper :: numDragItems() const
{
 return theDragInfo.cditem;
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::setDropPoint                                               |
------------------------------------------------------------------------------*/
IDragInfoWrapper& IDragInfoWrapper :: setDropPoint ( const IPoint& dropPoint )
{
#ifdef IC_PM
 theDragInfo.xDrop = dropPoint.x();
 theDragInfo.yDrop = dropPoint.y();
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
 theDragInfo.pointDrop = dropPoint;
#endif
 return (*this);
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::setSourceWindowHandle                                      |
------------------------------------------------------------------------------*/
IDragInfoWrapper&
IDragInfoWrapper :: setSourceWindowHandle ( const IWindowHandle& window )
{
  theDragInfo.hwndSource = window;
#ifdef IC_WIN
  unsigned tidSource;
  IQUERYWINDOWPROCESS(window, &theDragInfo.pidSource, &tidSource);
#endif
  return (*this);
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::setDragOperation                                           |
------------------------------------------------------------------------------*/
IDragInfoWrapper& IDragInfoWrapper :: setDragOperation ( unsigned short dragOp )
{
 theDragInfo.usOperation = dragOp;
 return (*this);
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::deleteTargetResources                                      |
------------------------------------------------------------------------------*/
IDragInfoWrapper& IDragInfoWrapper :: deleteTargetResources ( )
{
#ifdef IC_PM
 DrgDeleteDraginfoStrHandles ( &theDragInfo );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
 delete &theDragInfo;
#endif
 return (*this);
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::deleteSourceResources                                      |
------------------------------------------------------------------------------*/
IDragInfoWrapper& IDragInfoWrapper :: deleteSourceResources ( )
{
#ifdef IC_PM
 DrgFreeDraginfo ( &theDragInfo );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
 // Do nothing. We're only a wrapper: _DRAGINFO has its own life span.
#endif
 return (*this);
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::access                                                     |
------------------------------------------------------------------------------*/
IDragInfoWrapper& IDragInfoWrapper :: access ( )
{
#ifdef IC_PM
 if (!DrgAccessDraginfo ( &theDragInfo ) )
   ITHROWGUIERROR("DrgAccessDraginfo");
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  // Do nothing.
#endif
 return (*this);
}

/*------------------------------------------------------------------------------
| IDragInfoWrapper::release                                                    |
|                                                                              |
| Note:                                                                        |
|   This member function used to be free, but free is actually a macro in this |
|   compiler, so the name had to be changed to protect the innocent.           |
------------------------------------------------------------------------------*/
IDragInfoWrapper& IDragInfoWrapper :: release ( )
{
#ifdef IC_PM
 DrgFreeDraginfo ( &theDragInfo );
#endif
#if defined(IC_MOTIF) || defined(IC_WIN)
  // Do nothing.
#endif
 return (*this);
}

#ifdef IC_PMWIN 
/*---------------------------------------------------------------
| Create and populate the drag info wrapper with all of its
| little item wrappers. Returns NULL if failure.
---------------------------------------------------------------*/
IDragInfoWrapper* IDragInfoWrapper::setDragInfoWrapper(
                   IDMSourceOperation *pSrcOp, _DRAGINFO *pDInfo )
{
  IMODTRACE_DEVELOP("IDMSourceOperationData::dragInfoWrapper");

  ULONG ulItemCnt = pSrcOp->numberOfItems();
  IDragInfoWrapper* pDInfoWrapper = new IDragInfoWrapper(*pDInfo);

  if (pDInfoWrapper)
  {
    /*---------------------------------------------------------------
    | Populate the drag info wrapper.
    ---------------------------------------------------------------*/
    pDInfoWrapper->setSourceWindowHandle (pSrcOp->sourceWindowHandle());
    pDInfoWrapper->setDragOperation ((unsigned short)pSrcOp->operation());

    /*-----------------------------------------------------------------
    | Loop through the collection of IDMItems. For each, copy
    | the correspoinding data, and set each one into the PM DragInfo
    | structure. (todo)Also, build the array of drag images to be
    | displayed
    -----------------------------------------------------------------*/
    for (ULONG ulItem = 0; ulItem < ulItemCnt; ulItem++)
    {
      IDMItem::Handle pItemHandle(pSrcOp->item((unsigned int)ulItem + 1)) ;

      /*---------------------------------------------------------------
      | Set the DRAGITEM in the DRAGINFO for this item
      ---------------------------------------------------------------*/
      _DRAGITEM& dItem = (*pDInfoWrapper)[ulItem];
      IDragItemWrapper dragItemWrapper(dItem);
      dragItemWrapper.setSourceWindowHandle (pItemHandle->sourceWindowHandle() ),
      dragItemWrapper.setItemID((unsigned)(IDMItem*)pItemHandle.getAlias());
      dragItemWrapper.setTypes( pItemHandle->strTypes );
      dragItemWrapper.setRMFs( pItemHandle->strRMFs );
      dragItemWrapper.setContainerName( pItemHandle->strContainerName );
      dragItemWrapper.setTargetName( pItemHandle->targetName() );
      dragItemWrapper.setSourceName( pItemHandle->sourceName() );
      dragItemWrapper.setImageOffset( pItemHandle->imageOffset() );
      dragItemWrapper.setDragAttrs ( (unsigned short)pItemHandle->dragAttrs );
      dragItemWrapper.setDragSupOps( (unsigned short)pItemHandle->dragSupOps );

      /*---------------------------------------------------------------
      | Tell the OpenClass drag item where the DRAGITEM is.
      ---------------------------------------------------------------*/
      pItemHandle->setPMDragItem( &dItem );
    }
  }
  return pDInfoWrapper;
}
#endif // IC_PMWIN


#ifdef IC_PMWIN 
/*-------------------------------------------------------------------
| Re-constitute a IDragInfoWrapper instance from the IDataObject.
| If no IOC_DRAGINFO advertisedin IDataObject, then make a 1-item
| skeleton IDragInfoWrapper containing types obtained from the
| advertised formats enumerated from IDataObject.
|
| Returns NULL if failed. This should be considered, uh, unusual.
-------------------------------------------------------------------*/
IDragInfoWrapper* IDragInfoWrapper::dragInfoWrapper(
                        IDataObject *dataObject )
{
  IMODTRACE_DEVELOP("IDragInfoWrapper::dragInfoWrapper");
  /*-----------------------------------------------------------------
  | This member function performs these actions:
  |
  | I. Query the dataobject for our private DRAGINFO format:
  |   1. Register format IOC_DRAGINFO
  |   2. Query dataObject for the format. If hit:
  |     a. Obtain the IStream reference for the marshalled _DRAGINFO
  |     b. Get the length of the stream
  |     d. Allocate a buffer for the stream.
  |     d. Read the entire stream into the buffer
  |     e. Create a IDragInfoWrapper
  |     f. IDragInfoWrapper unmarshall's self from the buffer
  | II. The IDragInfoWrapper has the re-constituted _DRAGINFO
  |     from the source operation.
  -----------------------------------------------------------------*/
  ITRACE_DEVELOP("Registering private clipboard format IOC_DRAGINFO");
  CLIPFORMAT clipFmt =
     CLIPFORMAT(::RegisterClipboardFormat(IDMDataObject::formatDragInfo()));
  IASSERT(clipFmt);

  /*-----------------------------------------------------------------
  | Query dataobject for DRAGINFO format
  -----------------------------------------------------------------*/
  FORMATETC formatEtc;
  SETDefFormatEtc( formatEtc, clipFmt, TYMED_ISTREAM );
  bool bFoundFormat = dataObject->QueryGetData ( &formatEtc) == S_OK;

  /*-----------------------------------------------------------------
  | Draginfo wrapper pointer to be re-constituted
  -----------------------------------------------------------------*/
  _DRAGINFO *pDInfo = NULL;

  IDragInfoWrapper* pDInfoWrapper = NULL;

  if (bFoundFormat)
  {

    /*---------------------------------------------------------------
    | Get draginfo IStream clone from OLE data object.
    | This clone belongs to us - we release it when we're through.
    ---------------------------------------------------------------*/
    ITRACE_DEVELOP("Getting draginfo data");
    STGMEDIUM stgMedium;
    if (S_OK == dataObject->GetData(&formatEtc, &stgMedium))
    {
      IStream* pIStream = stgMedium.pstm;

      STATSTG statstg;

      if (S_OK == pIStream->Stat(&statstg, STATFLAG_NONAME))
      {
        /*-------------------------------------------------------------
        | Get size of IStream.
        | Read rest of IStream into byte array.
        -------------------------------------------------------------*/
        ITRACE_DEVELOP("Read draginfo byte-stream");

        /*----------------- Begin hack ----------------------------*/
        // We simply want the size of the stream here.
        // But IBMCPPNT undefs ULARGE_INTEGER LowPart,
        // (todo: why?) we use this union hack.
        // memcpy would work, but isn't inline.
        union
        {
          ULARGE_INTEGER* puliSize;
          unsigned* pLowPart;
        } _sizeUnion;
        _sizeUnion.puliSize = &statstg.cbSize; // Size of stream

        unsigned cbBuffer = *(_sizeUnion.pLowPart);

        /*----------------- end of hack ---------------------------*/

        char* pBytes = new char [ cbBuffer];

        const LARGE_INTEGER ORIGIN = { 0, 0 };
        if ( S_OK != pIStream->Seek(ORIGIN, STREAM_SEEK_SET, NULL)
          || S_OK != pIStream->Read(pBytes, cbBuffer, NULL) )
          ITHROWGUIERROR("IStream");

        ITRACE_DEVELOP(IString(cbBuffer) + " bytes of marshall data");

        /*-------------------------------------------------------------
        | Re-constitute our empty IDragInfoWrapper instance
        | from the byte array passed through the OLE IStream.
        | Start with empty IDragInfo instance. Un-marshalling
        | creates correct number of DRAGITEMs within it.
        -------------------------------------------------------------*/
        ITRACE_DEVELOP("Unmarshalling draginfo");
        pDInfo = new IDragInfo( 0 );

        IASSERT(pDInfo != NULL);

        pDInfoWrapper = new IDragInfoWrapper(*pDInfo);
        pDInfoWrapper->asDragInfoPtr()->unmarshall(pBytes);
        delete [] pBytes;
        ITRACE_DEVELOP("Created draginfo wrapper");
      }
      pIStream->Release();
    }
  }
  else
  {
    ITRACE_DEVELOP("IOC_DRAGINFO not found");
        /*--------------------------------------------------------------
    | Initialize ICLUI drag item object from data object
    --------------------------------------------------------------*/
    IEnumFORMATETC *enumFormatEtc;
    IString types;
    int numberOfItems = 1;
    if ( SUCCEEDED(dataObject->EnumFormatEtc( DATADIR_GET, &enumFormatEtc)))
    {
      enumFormatEtc->Reset();
      FORMATETC formatEtc;

      while (enumFormatEtc->Next(1, &formatEtc, NULL) == S_OK)
      {
        IString type = IDMFormat::stringFromFormat( formatEtc.cfFormat );
        ITRACE_DEVELOP("Format: " + type);

        if ( ( type == IDM::file) &&
             ( type.length() == IString(IDM::file).length()) )
        {
          ITRACE_DEVELOP("We found IDM::file");
          //we have a file drop from Windows. We need to request CF_HDROP so we can
          // determine how many files are being dropped. Request the DROPFILE data structure and
          // count the number of null terminated strings. Then build a drag info wrapper for each
          // file.
          FORMATETC formatEtc;
          STGMEDIUM stgmedium;
          CLIPFORMAT cfFormat = CLIPFORMAT(IDMFormat::formatFromString(type));

          SETDefFormatEtc( formatEtc, cfFormat, TYMED_HGLOBAL );

          HRESULT hr = dataObject->GetData( &formatEtc, &stgmedium );

          if (SUCCEEDED(hr))
          {
            numberOfItems = 0;
            /*-------------------------------
            | Get drag data
            -------------------------------*/
            char FAR* pszData = (char FAR*)::GlobalLock(stgmedium.hGlobal);
            DROPFILES *dropFile = (DROPFILES *) pszData;
            if (dropFile->fWide )
            {
              WCHAR *pszString = (WCHAR *)((char *)(dropFile) + dropFile->pFiles);
              WCHAR *nullString = wcschr( pszString, '\0');
              while ( nullString )
              {
                if (nullString - pszString <= sizeof(WCHAR) )
                {
                  break;
                }
                pszString = nullString + 1;
                nullString = wcschr( pszString, '\0');
                numberOfItems++;
              }
            }
            else
            {
              char *pszString = (char *)(dropFile) + dropFile->pFiles;
              char *nullString = strchr( pszString, '\0');
              while ( nullString )
              {
                if (nullString - pszString <= sizeof(char) )
                {
                  break;
                }
                pszString = nullString + 1;
                nullString = strchr( pszString, '\0');
                numberOfItems++;
              }
            }
            ITRACE_DEVELOP("Number of items found is " + IString( numberOfItems ) );
          }
        }
        if (types.length())
          types += "," + type;
        else
          types = type;
      }

      ITRACE_DEVELOP("Types: " + types);
      enumFormatEtc->Release();
    }
    /*--------------------------------------------------------------
    | No drag info passed through OLE. Must be non-OpenClass
    | drag source. Build one item with the types (formats)
    | advertised by the IDataObject interface.
    |
    | If the type CF_HDROP is detected then we need to figure out
    | how many files are being dragged.
    --------------------------------------------------------------*/
    ITRACE_DEVELOP("Number of files is " + IString( numberOfItems));
    pDInfo = new IDragInfo( numberOfItems );
    pDInfoWrapper = new IDragInfoWrapper(*pDInfo);
    for (unsigned index = 0; index < numberOfItems; index++)
    {
      IDragItemWrapper diw((*pDInfoWrapper)[index] );


      diw.setTypes(types);
      //this needs to be fixed.
      diw.setDragSupOps(IDMItem::copyable | IDMItem::moveable);
      diw.setDragAttrs(IDMItem::none);
      diw.setItemID( index );
    }

    // get the offsets of the additional items
    FORMATETC formatEtc;
    STGMEDIUM stgmedium;
    CLIPFORMAT cfFormat =
        CLIPFORMAT(IDMFormat::formatFromString(CFSTR_SHELLIDLISTOFFSET));

    SETDefFormatEtc( formatEtc, cfFormat, TYMED_HGLOBAL );

    HRESULT hr = dataObject->GetData( &formatEtc, &stgmedium );

    if (SUCCEEDED(hr))
    {
      /*-------------------------------
      | Get drag data
      -------------------------------*/
      char FAR* pszData = (char FAR*)::GlobalLock(stgmedium.hGlobal);
      POINT *myPoints = (POINT* ) pszData;

      ITRACE_DEVELOP( "The point is " +
          IPoint( myPoints->x , myPoints->y ).asString() );
      //we only need the relative points so step past the first one.
      myPoints++;

      ITRACE_DEVELOP( "The point is " +
        IPoint( myPoints->x, myPoints->y ).asString() );

      for ( unsigned long itemIndex = 0;
            itemIndex < numberOfItems;
            itemIndex++ )
      {
        _DRAGITEM& dragItem = (*pDInfoWrapper)[itemIndex];
        IDragItemWrapper diw(dragItem);
        //relative offsets
        diw.setImageOffset( ISize( myPoints->x, myPoints->y ));
        ITRACE_DEVELOP( "The point is " +
            IPoint( myPoints->x, myPoints->y ).asString() );
        myPoints++;
      }
    }
  }
  return pDInfoWrapper;
}
#endif 

