// Revision: 59 1.3.1.2 source/ui/drag/idmdptgh.cpp, dragdrop, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: idmdptgh.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMDropTargetHandler                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif
  #include <iwindefs.h>
}

// Trace Facility
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Include our header
#ifndef _IDMDPTGH_
  #include <idmdptgh.hpp>
#endif

#ifndef _IDMDRPTG_
  #include <idmdrptg.hpp>
#endif

#ifndef _IDMDTPRV_
  #include <idmdtprv.hpp>
#endif

#ifndef _IDMDRAGH_
  #include <idmdragh.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMDPTGH_CPP_
  #include <ipagetun.h>
#endif

IDMDropTargetSet::IDMDropTargetSet()
{
} // IDMDropTargetSet

IDMDropTargetSet::~IDMDropTargetSet()
{
} // ~IDMDropTargetSet

IDropTargetElement :: IDropTargetElement( unsigned long handle,
                      IDMDropTarget* dropTarget )
  : handleKey( handle )
  , _dropTarget( dropTarget )
  {;}

IDropTargetElement::~IDropTargetElement( )
  {;}

#pragma enum(4)
#pragma pack(push,4)

class IDMDropTargetHandlerData
  {
  public:
    IDMDropTargetHandlerData()
    {;}
    ~IDMDropTargetHandlerData()
    {;}
    IDMDropTargetSet
     *dropTargetSet;
  private:
  };

#pragma pack(pop)
#pragma enum(pop)

IDMDropTargetHandler :: IDMDropTargetHandler ()
  : IHandler()
  , fDMDropTargetHandlerData( new IDMDropTargetHandlerData() )

  {
  fDMDropTargetHandlerData->dropTargetSet =
    new IDMDropTargetSet();
  }

IDMDropTargetHandler& IDMDropTargetHandler :: handleEventsFor( IWindow *pWin )
  {
  // create the IDM drop target and register the window as a drop
  // target with ole.
  IDMDropTarget *_dropTarget = new IDMDropTarget( pWin );
  _dropTarget->Register();
  fDMDropTargetHandlerData->dropTargetSet->add(
    new IDropTargetElement( pWin->handle().asUnsigned(), _dropTarget));

  Inherited::handleEventsFor( pWin );
  return *this;
  }

IDMDropTargetHandler :: ~IDMDropTargetHandler ()

  {
  if (fDMDropTargetHandlerData)
    {
    if (fDMDropTargetHandlerData->dropTargetSet)
      delete fDMDropTargetHandlerData->dropTargetSet;
    delete fDMDropTargetHandlerData;
    }
  }

/*------------------------------------------------------------------------------
| IDMDropTargetHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Overload of Handler's dispatcher.                                            |
------------------------------------------------------------------------------*/
bool IDMDropTargetHandler :: dispatchHandlerEvent(IEvent& evt)
{
  switch(evt.eventId())
    {
    case WM_DESTROY:
      {
      // Find the IDMDropTarget to revoke the OLE registration.
      IDropTargetElement* elementWithHandle(
          fDMDropTargetHandlerData->dropTargetSet->elementWithKey(
              evt.window()->handle().asUnsigned() ));

      fDMDropTargetHandlerData->dropTargetSet->removeElementWithKey(
              evt.window()->handle().asUnsigned() );

      // The following line will revoke the drop target.
      if ( elementWithHandle )
        {
        ITRACE_DEVELOP("Revoke the registration for this window");
        elementWithHandle->_dropTarget->Revoke();
        }
      this->stopHandlingEventsFor( evt.window() );
      // If the collection is empty we could then delete
      // this handler but this would be dangerous if a
      // drop site is added at a later time. The safest
      // bet is to leave this handler for the lifetime
      // of the application.

      IDMDragHandler::uninitOle();
      }
    }
  return false;
  }
