// Revision: 63 1.4.1.5 source/ui/drag/idmdrptg.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmdrptg.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMDropTarget                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IDMDRPTG_
  #include <idmdrptg.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IDMDATOB_
  #include <idmdatob.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IDMTGTOP_
  #include <idmtgtop.hpp>
#endif
#ifndef _IDMSRCOP_
  #include <idmsrcop.hpp>
#endif
#ifndef _IDMCMDHR_
  #include <idmcmdhr.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMDRPTG_CPP_
  #include <ipagetun.h>
#endif

#pragma enum(4)
#pragma pack(push,4)

class IDMDropTargetData
{
  public:
    IDMDropTargetData( IWindow *win ) ;
    ~IDMDropTargetData() ;

    HRESULT
      setEffect( DWORD grfKeyState,
                 POINTL pt,
                 LPDWORD pdwEffect,
                 unsigned long eventID );
    void
      setDefaultOperation ();

    IWindow
     *_window;
    IDataObject
      *_dataObject;
    bool
      _registered,
      _fRButtonDown;
    unsigned long
      _cRef;
    IWindowHandle
      _hwnd;
    DWORD
      _defaultOperation;
    unsigned short
      _supportedOperations;
    IDMPopUpCommandHandler
      commandHandler;
    IDMPopUpMenuHandler
      popUpHandler;

  private:
     IDMDropTargetData ( const IDMDropTargetData&);
     IDMDropTargetData& operator=( const IDMDropTargetData&) ;
}; // IDMDropTargetData

#pragma pack(pop)
#pragma enum(pop)

//-------------------------------------------
// Constructors and destructor
//-------------------------------------------

IDMDropTargetData::IDMDropTargetData( IWindow *win )
    : _dataObject(0)
    , _window( win )
    , _registered( false )
    , _cRef(0)
    , _hwnd(0)
    , _defaultOperation(0)
    , _supportedOperations(0)
    , _fRButtonDown(false)
    , commandHandler(win)
    , popUpHandler(win)
{
   _hwnd = _window->handle();
}  // IDMDropTargetData

IDMDropTargetData::~IDMDropTargetData()
{
   IMODTRACE_DEVELOP("IDMDropTargetData::~IDMDropTargetData") ;
} // ~IDMDropTargetData

IDMDropTarget::IDMDropTarget(IWindow *win )
  : fDMDropTargetData( new IDMDropTargetData(win) )
{
}

IDMDropTarget::~IDMDropTarget()
{
  IMODTRACE_DEVELOP("IDMDropTarget::~IDMDropTarget") ;
  if ( fDMDropTargetData )
    delete fDMDropTargetData;
}

//-------------------------------------------
// OLE IDropTarget C++ wrappers
//-------------------------------------------

HRESULT __stdcall IDMDropTarget::QueryInterface( REFIID riid,
                                       void **ppvoid )
  {
  ITRACE_DEVELOP("IDMDropTarget::QueryInterface");
  ITRACE_DEVELOP(IString("\t...interface ") +
    (IsEqualIID( riid, IID_IDropTarget ) ?
      IString("IDropTarget") :
    (IsEqualIID( riid, IID_IUnknown ) ?
      IString("IUnknown") :
    IString("...not one of our interfaces")
    )) );

  *ppvoid = NULL;       // Always null our output parameters

  if ( IsEqualIID( riid, IID_IUnknown ) ||
       IsEqualIID( riid, IID_IDropTarget ))
    {
    *ppvoid = (void *) this;

    AddRef();
    }
  if ( NULL != *ppvoid )
    return NOERROR;

  return ResultFromScode( E_NOINTERFACE);
  }

ULONG __stdcall IDMDropTarget::AddRef( )
  {
  ++fDMDropTargetData->_cRef;
  ITRACE_DEVELOP(IString("IDMDropTarget::AddRef - exit count == ") +
                        (IString(fDMDropTargetData->_cRef)));
  return fDMDropTargetData->_cRef;
  }

ULONG __stdcall IDMDropTarget::Release( )

  {
  ULONG cRefT;

  cRefT = --fDMDropTargetData->_cRef;
  ITRACE_DEVELOP(IString("IDMDropTarget::Release - exit count == ") +
                        (IString(cRefT)));
  if ( cRefT == 0 )
    {
    ITRACE_DEVELOP("IDMDropTarget::Release will delete self");
    delete this;
    }

  return cRefT;
  }


HRESULT __stdcall IDMDropTarget::DragEnter (LPDATAOBJECT pDataObject,
                                  DWORD grfKeyState,
                                  POINTL pt,
                                  LPDWORD pdwEffect)
  {
  ITRACE_DEVELOP("IDMDropTarget::DragEnter");
  fDMDropTargetData->_dataObject = pDataObject;

  if (grfKeyState & MK_RBUTTON)
    {
    fDMDropTargetData->_fRButtonDown = true;
    }


  HRESULT hResult = fDMDropTargetData->setEffect( grfKeyState,
                                       pt,
                                       pdwEffect,
                                       DM_DRAGOVER );

  fDMDropTargetData->setDefaultOperation();

  return hResult;

  }


HRESULT __stdcall IDMDropTarget::DragOver (DWORD grfKeyState,
                                 POINTL pt,
                                 LPDWORD pdwEffect)
  {
  ITRACE_DEVELOP("IDMDropTarget::DragOver");

  HRESULT hResult = fDMDropTargetData->setEffect( grfKeyState,
                                       pt,
                                       pdwEffect,
                                       DM_DRAGOVER );

  fDMDropTargetData->setDefaultOperation();

  return hResult;

  }

HRESULT __stdcall IDMDropTarget::DragLeave ()

  {
  ITRACE_DEVELOP("IDMDropTarget::DragLeave");
  fDMDropTargetData->_window->sendEvent( DM_DRAGLEAVE, 0, 0 );
  fDMDropTargetData->_fRButtonDown = false;

  return NOERROR;
  }

HRESULT __stdcall IDMDropTarget::Drop (LPDATAOBJECT pDataObject,
                             DWORD grfKeyState,
                             POINTL pt,
                             LPDWORD pdwEffect)
  {
  IMODTRACE_DEVELOP("IDMDropTarget::Drop");
  if (fDMDropTargetData->_fRButtonDown)
    {
    // Set the point in the command handler
    fDMDropTargetData->commandHandler.setPoint(pt);

    // Set the data object in the command handler
    fDMDropTargetData->commandHandler.setDataObject(fDMDropTargetData->_dataObject);
    fDMDropTargetData->_dataObject->AddRef();

    // Set flag in menu handler indicating non-default drag in progress
    fDMDropTargetData->popUpHandler.fNonDefaultDrag = true;

    // show a popup menu to ask for drag operation
    fDMDropTargetData->commandHandler.startHandlingEvents() ;
    fDMDropTargetData->popUpHandler.startHandlingEvents() ;
    fDMDropTargetData->_window->sendEvent( WM_RBUTTONUP, 0, 0);

    fDMDropTargetData->_fRButtonDown = false;
    fDMDropTargetData->popUpHandler.fNonDefaultDrag = false;

    return NOERROR;
    }

  HRESULT hResult = fDMDropTargetData->setEffect( grfKeyState,
                                       pt,
                                       pdwEffect,
                                       DM_DROP );

  return hResult;
  }


bool IDMDropTarget::Register ( )
  {
  IDropTarget* pIDropTarget;

  QueryInterface(IID_IDropTarget, (void**)&pIDropTarget);
  // Note: we don't AddRef(). Was done by QueryInterface

  ::CoLockObjectExternal(LPUNKNOWN(pIDropTarget), TRUE, FALSE);

  fDMDropTargetData->_registered =
    (::RegisterDragDrop (fDMDropTargetData->_window->handle(), pIDropTarget)
        == NOERROR);

  Release();

  return fDMDropTargetData->_registered;
  }

void IDMDropTarget::Revoke ()
  {
  ITRACE_DEVELOP("IDMDropTarget::Revoke");

  if ( fDMDropTargetData->_registered ) {
    if ( fDMDropTargetData->_window ){
      ITRACE_DEVELOP("IDMDropTarget::Revoke .. ::RevokeDragDrop");
      ::RevokeDragDrop( fDMDropTargetData->_hwnd );
      ::CoLockObjectExternal(LPUNKNOWN(this), FALSE, TRUE);
      }
    }
  }


HRESULT IDMDropTargetData::setEffect( DWORD grfKeyState,
                                      POINTL pt,
                                      LPDWORD pdwEffect,
                                      unsigned long eventID )
  {
  IMODTRACE_DEVELOP("IDMDropTargetData::setEffect");
  // get the default drag operation
  *pdwEffect = _defaultOperation;

  // Only set dropeffect based on key state if not right mouse button drag
  if (!_fRButtonDown)
    {
    if ( grfKeyState & MK_CONTROL )
      {
      if ( _supportedOperations & IDMItem::copyable )
        *pdwEffect=DROPEFFECT_COPY;
      else
        *pdwEffect=DROPEFFECT_NONE;
      if ( grfKeyState & MK_SHIFT )
        {
        if ( _supportedOperations & IDMItem::linkable )
          *pdwEffect=DROPEFFECT_LINK;
        else
          *pdwEffect=DROPEFFECT_NONE;
        }
      }
    else if ( grfKeyState & MK_SHIFT )
      {
      if ( _supportedOperations & IDMItem::moveable )
        *pdwEffect=DROPEFFECT_MOVE;
      else
        *pdwEffect=DROPEFFECT_NONE;
      }
    }

  IEventResult evt = _window->sendEvent( eventID,
                        IEventData( pt.x, pt.y),
                        IEventData( (void *) _dataObject ));

  if (eventID == DM_DRAGOVER)
    {
    // Disallow drop if dropindicator not OK
    if ((evt.number1() == IDM::neverOk)  ||
        (evt.number1() == IDM::notOk))
      *pdwEffect=DROPEFFECT_NONE;
    }

  IDMTargetOperation::Handle pTgtOp(IDMTargetOperation::targetOperation()) ; 
  if ( pTgtOp.valid() )
    pTgtOp->setOperation( *pdwEffect );

  return NOERROR;
  }

void IDMDropTargetData::setDefaultOperation ()

  {
  IMODTRACE_DEVELOP("IDMDropTarget::setDefaultOperation");

  // If right mouse button drag, then set DROPEFFECT to move.  Don't allow
  // key state to change DROPEFFECT
  if (_fRButtonDown)
    {
    _defaultOperation = DROPEFFECT_MOVE;
    return;
    }

  DWORD defOper = DROPEFFECT_COPY;

  IDMTargetOperation::Handle pTgtOp(IDMTargetOperation::targetOperation()) ;
  if ( pTgtOp.valid() )
    {
    IDMItem::Handle pItemH(pTgtOp->item(1)) ;

    _supportedOperations = pItemH->supportedOperations();

    ITRACE_DEVELOP("Supported operations as seen by drop target: " +
                   IString(_supportedOperations));

    // return default operation: order is move, copy, link
    if ( _supportedOperations & IDMItem::moveable )
      {
      ITRACE_DEVELOP( "Move supported");
      defOper = DROPEFFECT_MOVE;
      }

    else if ( _supportedOperations & IDMItem::copyable )
      {
      ITRACE_DEVELOP( "Copy supported");
      defOper = DROPEFFECT_COPY;
      }

    else if ( _supportedOperations & IDMItem::linkable )
      {
      ITRACE_DEVELOP( "Link supported");
      defOper = DROPEFFECT_LINK;
      }
    }

  _defaultOperation = defOper;
  }
