// Revision: 68 1.4.1.4 source/ui/drag/idmefit.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmefit.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMEFItem                                                                *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINPOINTERS
  #define INCL_DOSPROCESS
  #include <iwindefs.h>
}

  #include <itrace.hpp>
  #include <iexcept.hpp>
  #include <idmefit.hpp>
  #include <idmevent.hpp>
  #include <idmsrcop.hpp>
  #include <ientryfd.hpp>
  #include <idmimage.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMEFIT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMEFItem::IDMEFItem                                                         |
|                                                                              |
| Constructor with source item arguments.                                      |
------------------------------------------------------------------------------*/
IDMEFItem::IDMEFItem(IDMSourceOperation *pDMSrcOp)  :
                     IDMItem(pDMSrcOp,
                             IDM::text,
                             (IDMItem::moveable | IDMItem::copyable),
                             none)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Obtain pointer to entry field control from the source window     */
  /********************************************************************/
  IEntryField *pEF = (IEntryField *)pDMSrcOp->sourceWindow();

  /********************************************************************/
  /* Get the actual text, selected characters only if selection       */
  /* in use.                                                          */
  /********************************************************************/
  setContents( ((pEF->hasSelectedText()) ? pEF->selectedText()  :
                                          pEF->text()) );

  /********************************************************************/
  /* Set the native renderering mechanism and format for the item.    */
  /********************************************************************/
#ifdef IC_MOTIFPM  
  IString tempStr = generateSourceName();
#else
  ITRACE_DEVELOP("about to generateSourceName");
  IString tempStr;
#endif
  if (tempStr.size())
  {
    setSourceName(tempStr);
    setRMFs(rmfsFrom(IDM::rmLibrary,
                     rfForThisProcess() + "," + IDM::rfText));
  }
  else
  {
    setRMFs(rmfsFrom(IDM::rmLibrary,
                     rfForThisProcess() + "," + IDM::rfSharedMem));

    /******************************************************************/
    /* Use render prepare event to determine size of transfer buffer  */
    /******************************************************************/
    setRequiresPreparation(true);
  }

  /********************************************************************/
  /* Add a drag image for the item.                                   */
  /********************************************************************/
#ifdef IC_MOTIFPM  
#ifdef IC_PM
  IDMImage image = IDMImage(ISystemPointerHandle(
                            ISystemPointerHandle::text));
#endif
#ifdef IC_MOTIF
  IDMImage image = IDMImage() ;
#endif
  setImage(image);
#endif
}


/*------------------------------------------------------------------------------
| IDMEFItem::IDMEFItem                                                         |
|                                                                              |
| Constructor with target item arguments.                                      |
------------------------------------------------------------------------------*/
IDMEFItem::IDMEFItem(const IDMItem::Handle &pEFItem)  :
                     IDMItem(pEFItem)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Verify that one of these rmfs is supported by the source.        */
  /********************************************************************/
  IString defaultRMFs = rmfsFrom(IDM::rmLibrary,
                                      rfForThisProcess() + "," +
                                      IDM::rfText + "," +
                                      IDM::rfSharedMem);
  IString rmf = matchingRMFs (rmfsFrom( rmfs().change(' ', "") ),
                              defaultRMFs);

  /******************************************************************/
  /* The RMFs that IDMEFItem understands are                        */
  /* (IDM::rmLibrary)x                                              */
  /*           (IDM::rfProcess=pid,IDM::rfText,IDM::sharedMem)      */
  /******************************************************************/
  setRMFs(rmf);
}

/*------------------------------------------------------------------------------
| IDMEFItem::~IDMEFItem                                                        |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMEFItem :: ~IDMEFItem()
{
  IFUNCTRACE_DEVELOP();
}

/*------------------------------------------------------------------------------
| IDMEFItem::generateSourceItems                                               |
|                                                                              |
| Gives IDMEFItem an opportunity to attach new IDMEFItem's to the argument     |
| IDMSourceOperation object.  This function is called when a drag operation    |
| commences over an IEntryField object.  Returns true if items were provided,  |
| false otherwise.                                                             |
------------------------------------------------------------------------------*/
bool IDMEFItem ::
        generateSourceItems(IDMSourceOperation *pDMSrcOp)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Obtain pointer to entry field control from the source window     */
  /********************************************************************/
  IEntryField *pEF = (IEntryField *)pDMSrcOp->sourceWindow();

  /********************************************************************/
  /* Do not attempt drag if the entry field is empty or has no text   */
  /* selected                                                         */
  /********************************************************************/
  if (pEF->isEmpty())
    return(false);

  if (! pEF->hasSelectedText())
     return(false) ;

  /********************************************************************/
  /* Create the entry field drag item                                 */
  /********************************************************************/
  IDMEFItem::Handle pEFItem(new IDMEFItem(pDMSrcOp)) ;

  /********************************************************************/
  /* Add entry field drag item to the source operation.               */
  /********************************************************************/
  pDMSrcOp->addItem(pEFItem);

  /********************************************************************/
  /* Set the drag image style.                                        */
  /********************************************************************/
#ifdef IC_MOTIFPM 
  pDMSrcOp->setImageStyle(IDM::allStacked);
#endif

  /********************************************************************/
  /* Indicate that the entry field drag item was successfully created.*/
  /********************************************************************/
  return(true);
}

/*------------------------------------------------------------------------------
| IDMEFItem::object                                                            |
|                                                                              |
| Returns a pointer to the item text.                                          |
------------------------------------------------------------------------------*/
void* IDMEFItem :: object() const
{
  return((void *)&(this->strContents));
}

/*------------------------------------------------------------------------------
| IDMEFItem::targetDrop                                                        |
|                                                                              |
| Writes text into the target entry field.                                     |
------------------------------------------------------------------------------*/
bool IDMEFItem :: targetDrop(IDMTargetDropEvent &)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Cast IWindow pointer to entry field                              */
  /********************************************************************/
  IEntryField *pEF = (IEntryField *)this->targetOperation()->targetWindow();

  /********************************************************************/
  /* Add the text to the entry field control                          */
  /********************************************************************/
  pEF->setText(contents());

  return(true);
}

/*------------------------------------------------------------------------------
| IDMEFItem::sourceEnd                                                         |
|                                                                              |
| Remove selected text to complete the move operation.                         |
------------------------------------------------------------------------------*/
bool IDMEFItem :: sourceEnd(IDMSourceEndEvent &event)
{
  IFUNCTRACE_DEVELOP();
#ifdef IC_WIN
  if ( event.wasTargetSuccessful() &&
     ( this->sourceOperation()->operation() == IDMOperation::move ))
    {
    // Get the entry field
    IEntryField *pEF = (IEntryField *)this->sourceOperation()->sourceWindow();

    // NOTE:  When using non-default drag, the popup menu causes a problem
    // with obtaining the selected range.  EM_GETSEL returns the cursor
    // position as both the start and end values for the selected text.
    // I am able to work around this with the implementation below to determine
    // the selected text accurately.
    //
    // The above commented out code is the correct implementation for left mouse
    // button default drags, but would not suffice for non-default drags.  The
    // implementation below works for both drag operations.

    IString newText = pEF->text();
    bool fSelInRange = false;
    unsigned startPos = 0;

    IEventResult Sel;
    this->sourceOperation()->sourceWindowHandle().sendEvent(EM_GETSEL,
                     IEventParameter1(&Sel),
                     IEventParameter2(0));

    while (!fSelInRange)
      {
      startPos = newText.indexOf(contents(), startPos + 1);

      // If the selection position (could be start or end of selected text)
      // is within the range of the matching text string
      if (( (Sel.asUnsignedLong()+1) >= startPos ) &&
          ( (Sel.asUnsignedLong()+1) <= (startPos + contents().length()) ) )
        {
        // remove selected text from string
        newText.remove(startPos, contents().length());
        fSelInRange = true;
        }
      }

    pEF->setText(newText);
    }
#endif // IC_WIN
#ifdef IC_MOTIF
  if ( event.wasTargetSuccessful() &&
     ( this->sourceOperation()->operation() == IDMOperation::move ))
  {
    // Get the entry field
    IEntryField *pEF = (IEntryField *)this->sourceOperation()->sourceWindow();

    if (pEF->hasSelectedText())
    {
       IString newText = pEF->text();
   
       IRange selectedText = pEF->selectedRange() ;
   
       newText.remove(selectedText.lowerBound()+1, 
                      pEF->selectedTextLength() );
       pEF->setText(newText);
    }
  }
#endif // IC_MOTIF
  return( true );
}


