// Revision: 73 1.2.2.2 source/ui/drag/idmfmten.cpp, dragdrop, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: idmfmten.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMEnumFormat                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IDMFMTEN_
  #include <idmfmten.hpp>
#endif

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMFMTEN_CPP_
  #include <ipagetun.h>
#endif

//-------------------------------------------
// Constructors and destructor
//-------------------------------------------

IDMEnumFormat::IDMEnumFormat(ULONG cFE, LPFORMATETC prgFE )
  : m_cRef(0)
  , m_iCur(0)
  , m_cfe(cFE)

  {
  m_prgfe=new FORMATETC[cFE];

  if (NULL!=m_prgfe)
    {
    for (int i=0; i < cFE; i++)
      m_prgfe[i]=prgFE[i];
    }
  }


IDMEnumFormat::~IDMEnumFormat()

  {
  if (m_prgfe)
    delete [] m_prgfe;
  }

/*------------------------------------------------------------------------------
| IDMEnumFormat::QueryInterface                                                         |
|                                                                              |
| Parameters:
|
| Return Value:
------------------------------------------------------------------------------*/
HRESULT __stdcall IDMEnumFormat::QueryInterface( REFIID riid,
                                                 void **ppvoid )
  {
  *ppvoid=NULL;

  if (IID_IUnknown==riid || IID_IEnumFORMATETC==riid)
    *ppvoid=this;

  if (NULL!=*ppvoid)
    {
    // AddRef any returned interface.
    ((LPUNKNOWN)*ppvoid)->AddRef();
    return ResultFromScode( S_OK );
    }

  return ResultFromScode(E_NOINTERFACE);
  }

/*------------------------------------------------------------------------------
| IDMEnumFormat::AddRef                                                         |
|                                                                              |
| Parameters: None
|
| Return Value:
|  ULONG  refcount
------------------------------------------------------------------------------*/
ULONG __stdcall IDMEnumFormat::AddRef( )
  {
  ++m_cRef;
  ITRACE_DEVELOP(IString("IDMEnumFormat::AddRef - exit count == ") +
                        (IString(m_cRef)));
  return m_cRef;
  }

/*------------------------------------------------------------------------------
| IDMEnumFormat::Release                                                         |
|                                                                              |
| Parameters: None
|
| Return Value:
|  ULONG  refcount
------------------------------------------------------------------------------*/
ULONG __stdcall IDMEnumFormat::Release( )

  {
  ULONG cRefT;

  cRefT = --m_cRef;
  ITRACE_DEVELOP(IString("IDMEnumFormat::Release - exit count == ") +
                        (IString(cRefT)));
  if ( m_cRef == 0 )
    {
    ITRACE_DEVELOP("IDMEnumFormat::Release will delete self");
    delete this;
    }

  return cRefT;
  }


/*------------------------------------------------------------------------------
| IDMEnumFormat::Next                                                         |
|                                                                              |
| Parameters:
|  cFE             ULONG number of FORMATETCs to return.
|  pFE             LPFORMATETC in which to store the returned
|                  structures.
|  pulFE           ULONG * in which to return how many we
|                  enumerated.
|
| Return Value:
|  HRESULT         NOERROR if successful, S_FALSE otherwise,
------------------------------------------------------------------------------*/
HRESULT __stdcall IDMEnumFormat::Next(ULONG cFE,
                                    LPFORMATETC pFE,
                                    ULONG *pulFE)
  {
  IMODTRACE_DEVELOP( "IDMEnumFormat::Next");
  ULONG cReturn=0L;

  if (NULL == m_prgfe)
    return ResultFromScode(S_FALSE);

  if (NULL == pulFE)
    {
    if (1 != cFE)
      return ResultFromScode(E_POINTER);
    }
  else
    *pulFE=0;

  if (NULL == pFE || m_iCur >= m_cfe)
    return ResultFromScode(S_FALSE);

  while (m_iCur < m_cfe && cFE > 0)
    {
    *pFE++=m_prgfe[m_iCur++];
    cReturn++;
    cFE--;
    }

  if (NULL!=pulFE)
    *pulFE=cReturn;

  return ResultFromScode( S_OK );
  }


/*------------------------------------------------------------------------------
| IDMEnumFormat::Skip                                                          |
|                                                                              |
| Purpose:
|  Skips the next n elements in the enumeration.
|
| Parameters:
|  cSkip           ULONG number of elements to skip.
|
| Return Value:
|  HRESULT         NOERROR if successful, S_FALSE if we could not
|                  skip the requested number.
|
------------------------------------------------------------------------------*/
HRESULT __stdcall IDMEnumFormat::Skip(ULONG cSkip)

  {
  IMODTRACE_DEVELOP( "IDMEnumFormat::Skip");
  if (((m_iCur+cSkip) >= m_cfe) || NULL==m_prgfe)
    return ResultFromScode(S_FALSE);

  m_iCur+=cSkip;
  return ResultFromScode( S_OK);
  }


/*------------------------------------------------------------------------------
| IDMEnumFormat::Reset                                                         |
|                                                                              |
| Resets the current element index in the enumeration to zero.
|
| Parameters:
|  None
|
| Return Value:
|  HRESULT         NOERROR
|
------------------------------------------------------------------------------*/
HRESULT __stdcall IDMEnumFormat::Reset()
  {
  IMODTRACE_DEVELOP( "IDMEnumFormat::Reset");
  m_iCur=0;
  return ResultFromScode(S_OK);
  }

/*------------------------------------------------------------------------------
| IDMEnumFormat::Clone
|                                                         |
| Returns another IEnumFORMATETC with the same state as ourselves.
|
| Parameters:
|   ppEnum          LPENUMFORMATETC * in which to return the
|                  new object.
|
| Return Value:
|  HRESULT         NOERROR or a general error value.
|
------------------------------------------------------------------------------*/
HRESULT __stdcall IDMEnumFormat::Clone(LPENUMFORMATETC *ppEnum)

  {
  IMODTRACE_DEVELOP( "IDMEnumFormat::Clone");
  IDMEnumFormat *pNew;

  *ppEnum=NULL;

  //Create the clone
  pNew=new IDMEnumFormat(m_cfe, m_prgfe);

  if (NULL==pNew)
    return ResultFromScode(E_OUTOFMEMORY);

  pNew->AddRef();
  pNew->m_iCur=m_iCur;

  *ppEnum=pNew;
  return ResultFromScode(S_OK);
  }
