// Revision: 75 1.2.2.2 source/ui/drag/idmformt.cpp, dragdrop, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: idmformat.cpp                                                     *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMFormat                                                                *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #include <iwindefs.h>
}

// Tracing functions
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Include our classes's stuff
  #include <idmformt.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMFORMT_CPP_
  #include <ipagetun.h>
#endif

/*-----------------------------------------------------------------------------|
| IDMFormat::IDMFormat                                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMFormat::IDMFormat()
{
  IMODTRACE_DEVELOP( "IDMFormat::IDMFormat ctor" );

}

/*------------------------------------------------------------------------------
| IDMFormat::~IDMFormat                                                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMFormat::~IDMFormat()
{
  IMODTRACE_DEVELOP( "IDMFormat::~IDMFormat" );

}

/*------------------------------------------------------------------------------
| IDMFormat::formatFromString                                                  |
|                                                                              |
| Obtain the data format Id from a format string name.                         |
------------------------------------------------------------------------------*/
short IDMFormat::formatFromString( const char* string)

  {
  IMODTRACE_DEVELOP( "IDMFormat::formatFromString()" );

  IString strFormat( string );
  short formatID = 0;

  if ( strFormat == IDM::text )
    formatID = CF_TEXT;
  else if ( strFormat == IDM::plainText )
    formatID = CF_OEMTEXT;
  else if ( strFormat == IDM::bitmap )
    formatID = CF_BITMAP;
  else if ( strFormat == IDM::file )
    formatID = CF_HDROP;
  //----- add additional supported formats -----
  else
    {
    // not a match for predefined format, so register new format
    formatID = RegisterClipboardFormat((LPCTSTR)strFormat);
    }

  return formatID;

  }


/*------------------------------------------------------------------------------
| IDMFormat::stringFromFormat                                                  |
|                                                                              |
| Obtain the format string name from a data format Id.                         |
------------------------------------------------------------------------------*/
IString IDMFormat::stringFromFormat(short formatID)
{
  IMODTRACE_DEVELOP( "IDMFormat::stringFromFormat()" );

  IString formatString;

  switch (formatID)
    {
    case CF_TEXT:
      formatString = IDM::text;
      break;
    case CF_BITMAP:
      formatString = IDM::bitmap;
      break;
    case CF_OEMTEXT:
      formatString = IDM::plainText;
      break;
    case CF_HDROP:
      formatString = IDM::file;
      break;
    default:
      {
        char format[50];
        // assume format is a user-defined format that has been registered
        int rc = GetClipboardFormatName(formatID, format, sizeof(format));
        if (rc)
          {
          formatString = format;
          }
        else
          {
          formatString = IDM::unknown;
          }
//        }


      }

    }
/* possible format IDs to support - need corresponding IDM formats

    case CF_METAFILEPICT   :
    case CF_SYLK           :
    case CF_DIF            :
    case CF_TIFF           :
    case CF_OEMTEXT        :
    case CF_DIB            :
    case CF_PALETTE        :
    case CF_PENDATA        :
    case CF_RIFF           :
    case CF_WAVE           :
    case CF_UNICODETEXT    :
    case CF_ENHMETAFILE    :
    case CF_OWNERDISPLAY   :
    case CF_DSPTEXT        :
    case CF_DSPBITMAP      :
    case CF_DSPMETAFILEPICT:
    case CF_DSPENHMETAFILE :

*/

  return formatString;

}



