// Revision: 85 1.8.1.3 source/ui/drag/idmmenit.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmmenit.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMMenuItem                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINMENUS
  #include <iwindefs.h>
}

  #include <idmmenit.hpp>
  #include <idmevent.hpp>
  #include <idmsrcop.hpp>
  #include <isubmenu.hpp>
  #include <imnitem.hpp>
  #include <idmimage.hpp>
  #include <ireslib.hpp>
  #include <istring.hpp>
  #include <irect.hpp>
  #include <itrace.hpp>
  #include <iexcept.hpp>

#ifdef IC_PMWIN
#include <igrport.hpp>
#include <iimage.hpp>
#include <itbarbut.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMMENIT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMMenuItem::IDMMenuItem                                                     |
|                                                                              |
| Constructor with source item arguments.                                      |
------------------------------------------------------------------------------*/
IDMMenuItem::IDMMenuItem( IDMSourceOperation *pDMSrcOp )  :
                     IDMItem( pDMSrcOp,
                              IDM::menuItem,
                              IDMItem::linkable,
                              none ),
                     fNoDismiss(false)
{
  IMODTRACE_DEVELOP( "IDMMenuItem::ctor (source)" );

  /********************************************************************/
  /* Set the native rendering mechanism and format for the item.      */
  /********************************************************************/
  setRMFs( rmfFrom( IDM::rmLibrary, rfForThisProcess() ) );

#ifdef IC_MOTIFPM  
  /********************************************************************/
  /* Obtain pointer to the submenu from the source window             */
  /********************************************************************/
  IWindow *pWin = pDMSrcOp->sourceWindow();
  ISubmenu subMenu( pWin->handle() );
  ITRACE_DEVELOP( "submenu->" + IString( (unsigned long)pWin ) );

  /********************************************************************/
  /* Iterate thru the menu items to determine which one the drag      */
  /* started over.                                                    */
  /********************************************************************/
  ISubmenu::Cursor cursor( subMenu );
  IPoint pos = pDMSrcOp->position();
  ITRACE_DEVELOP( "Looking for menu item for " + pos.asString() );

  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
    IMenuItem menuItem = subMenu.elementAt( cursor );
    IRectangle itemRect = subMenu.itemRect( menuItem.id() );
    ITRACE_DEVELOP( "testing menuItem->" + IString( menuItem.id() ) );
    ITRACE_DEVELOP( "rectangle->" + itemRect.asString() );

    if (itemRect.contains( pDMSrcOp->position() ))
    {
      /****************************************************************/
      /* Store menuid in the object data member ...                   */
      /****************************************************************/
      ITRACE_DEVELOP( "Match!" );
      setObject( (void *)menuItem.id() );
      setContents( menuItem.text() );

      /****************************************************************/
      /* Do not allow the menu to be dismissed ...                    */
      /****************************************************************/
      if (!menuItem.isNoDismiss())
      {
        menuItem.setNoDismiss();
        subMenu.setItem( menuItem );
        this->fNoDismiss = true;
      }
      subMenu.selectItem( menuItem.id() );

      break;
    }
  } /* endfor */

#ifdef IC_PM  
  /********************************************************************/
  /* Add a drag image for the item.  Use the value of the resource    */
  /* id to determine if the bitmap is one of ours, or user defined.   */
  /* If we cannot load the resource, we use the default.              */
  /********************************************************************/
  IBitmapHandle hbitMap;
  unsigned long ulId = (unsigned long)this->object();

  // Always try to load the from the user resource first so that they
  // can override our images

  hbitMap =
      IApplication::current().userResourceLibrary().tryToLoadBitmap( ulId );

  if (( ulId >= IC_ID_BASE ) && !hbitMap )
  {
    hbitMap =
        IApplication::current().resourceLibrary().tryToLoadBitmap( ulId );
  }

  if (!hbitMap)
  {
    hbitMap =
     IApplication::current().resourceLibrary().tryToLoadBitmap( IC_ID_DEFAULT );
  }

  if (hbitMap)
  {
    IGImage         bitmap(hbitMap);
    ISize           bmpSize = ISize( bitmap.rasterBounds().width(),
                                     bitmap.rasterBounds().height() );
    IImage          memImage( IPresSpaceHandle(bitmap.deviceContext(), pWin->handle()),
                              bmpSize.width(),
                              bmpSize.height() );
    IGrafBundle
      bundle;
    bundle.setFrameColor( IColor(IColor::defaultButton) );
    bundle.setFillColor( IColor(IColor::buttonMiddle) );
    memImage.grafPort()->draw( IGRect2D(0,0,bmpSize.width()+9,bmpSize.height()+9), bundle );

    if (IToolBarButton::isDefaultTransparentColorSet())
      memImage.setTransparentColor(IToolBarButton::defaultTransparentColor());
    IGrafMatrix
      matrix;
    matrix.translateBy( IGPoint2D( 4, 4  ) );
    memImage.grafPort()->draw( bitmap, bundle, matrix );

    IDMImage image = IDMImage( memImage.image()->handle() );
    setImage( image );
// remember how we did it in 2d graphics just in case
//#else
//    IGBitmap        bitmap(hbitMap);
//    ISize           bmpSize = bitmap.size();
//    IGraphicContext gc(ISize(bmpSize.width()+5,bmpSize.height()+5));
//
//    gc.setPenColor(IGUIColor(IGUIColor::defaultButton));
//    gc.setFillColor(IGUIColor(IGUIColor::buttonMiddle));
//    gc.draw(IGRectangle(IRectangle(0,0,bmpSize.width()+9,bmpSize.height()+9)));
//
//    if (IToolBarButton::isDefaultTransparentColorSet())
//      bitmap.setTransparentColor(IToolBarButton::defaultTransparentColor());
//    bitmap.moveTo(IPoint(4,4)).drawOn(gc);
//
//    IGBitmap* newBitmap =
//         new IGBitmap(gc, IRectangle(0,0,bmpSize.width()+9,bmpSize.height()+9));
//
//    IDMImage image = IDMImage( newBitmap->handle() );
//    setImage( image );
//
//    delete newBitmap;
//#endif
  }
#endif // IC_PM

#ifdef IC_MOTIF 
  IDMImage tempimg;
  setImage(tempimg);
#endif // IC_MOTIF
#endif  // IC_MOTIFPM
}


/*------------------------------------------------------------------------------
| IDMMenuItem::IDMMenuItem                                                     |
|                                                                              |
| Constructor with target item arguments.                                      |
------------------------------------------------------------------------------*/
IDMMenuItem::IDMMenuItem( const IDMItem::Handle &pMenuItem )  :
                     IDMItem( pMenuItem ),
                     fNoDismiss(false)
{
  IMODTRACE_DEVELOP( "IDMMenuItem::ctor (target)" );
}

/*------------------------------------------------------------------------------
| IDMMenuItem::~IDMMenuItem                                                        |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMMenuItem :: ~IDMMenuItem( )
{
  IMODTRACE_DEVELOP( "IDMMenuItem::dtor" );

  /********************************************************************/
  /* If the menu hasn't been dismissed, we need to do so here.  This  */
  /* can happen if one of the following conditions occur:             */
  /*    - the operation was cancelled                                 */
  /*    - help was requested                                          */
  /*    - the drop was not allowed                                    */
  /********************************************************************/
  if (this->fNoDismiss)
    dismissMenu();
}

/*------------------------------------------------------------------------------
| IDMMenuItem::generateSourceItems                                             |
|                                                                              |
| Gives IDMMenuItem an opportunity to attach new IDMMenuItem's to the argument |
| IDMSourceOperation object.  This function is called when a drag operation    |
| commences over an IDMMenuItem object.  Returns true if items were provided,  |
| false otherwise.                                                             |
------------------------------------------------------------------------------*/
bool IDMMenuItem ::
        generateSourceItems( IDMSourceOperation *pDMSrcOp )
{
  IMODTRACE_DEVELOP( "IDMMenuItem::generateSourceItems" );

  /********************************************************************/
  /* Create the menu item drag item                                   */
  /********************************************************************/
  IDMMenuItem::Handle pMenuItem(new IDMMenuItem( pDMSrcOp )) ; 

  /********************************************************************/
  /* Add it to the source operation.                                  */
  /********************************************************************/
  pDMSrcOp->addItem( pMenuItem );

  /********************************************************************/
  /* Set the image style.                                             */
  /********************************************************************/
  pDMSrcOp->setImageStyle( IDM::allStacked );

  /********************************************************************/
  /* Set the default operation.                                       */
  /********************************************************************/
  pDMSrcOp->setOperation( IDMOperation::link );

  /********************************************************************/
  /* Indicate that the menu item drag item was successfully created.  */
  /********************************************************************/
  return( true );
}

/*------------------------------------------------------------------------------
| IDMMenuItem::supportedOperationsFor                                          |
|                                                                              |
| Returns IDMItem::linkable as the only operation this item supports.          |
------------------------------------------------------------------------------*/
unsigned long IDMMenuItem :: supportedOperationsFor( const IString & ) const
{
  return( IDMItem::linkable );
}

/*------------------------------------------------------------------------------
| IDMMenuItem::sourceEnd                                                       |
|                                                                              |
| Called by the source handler to allow the menu to be dismissed after the     |
| target has indicated that the drop  processing has completed.                |
------------------------------------------------------------------------------*/
bool IDMMenuItem :: sourceEnd( IDMSourceEndEvent & )
{
  dismissMenu();
  return( true );
}

/*------------------------------------------------------------------------------
| IDMMenuItem::dismissMenu                                         (private)   |
|                                                                              |
| Dismisses the menuitem.                                                      |
------------------------------------------------------------------------------*/
IDMMenuItem& IDMMenuItem :: dismissMenu( )
{
#ifdef IC_MOTIFPM  
  IWindow *pWin = sourceOperation()->sourceWindow();
  ISubmenu subMenu( pWin->handle() );
  IMenuItem menuItem = subMenu.menuItem( (unsigned long)object() );

  if (this->fNoDismiss)
  {
    menuItem.setNoDismiss( false );
    subMenu.setItem( menuItem );
    this->fNoDismiss = false;
  }

#ifdef IC_PM
  pWin->sendEvent( MM_ENDMENUMODE,
                   IEventData(true, 0),
                   IEventData(0) );
#endif
//@@Rob.  Need to look into this.

#endif // IC_MOTIFPM
  return( *this );
}

