// Revision: 87 1.5.1.4 source/ui/drag/idmmleit.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmmleit.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMMLEItem                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  // define what subsections of PM headers to include
  #define INCL_WINPOINTERS
  #define INCL_DOSPROCESS
  #include <iwindefs.h>
}

  #include <itrace.hpp>
  #include <iexcept.hpp>
  #include <idmmleit.hpp>
  #include <idmevent.hpp>
  #include <idmsrcop.hpp>
  #include <imle.hpp>
  #include <idmimage.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMMLEIT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMMLEItem::IDMMLEItem                                                       |
|                                                                              |
| Constructor using source item arguments.                                     |
------------------------------------------------------------------------------*/
IDMMLEItem::IDMMLEItem(IDMSourceOperation *pDMSrcOp)  :
                       IDMItem(pDMSrcOp,
                               IDM::text,
                               (IDMItem::moveable | IDMItem::copyable),
                               none)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Obtain pointer to entry field control from the source window     */
  /********************************************************************/
  IMultiLineEdit *pMLE = (IMultiLineEdit *)pDMSrcOp->sourceWindow();

  /********************************************************************/
  /* Get the actual text, selected characters only if selection       */
  /* in use.                                                          */
  /********************************************************************/
  setContents(((pMLE->hasSelectedText()) ? pMLE->selectedText()  :
                                           pMLE->text()));

  /********************************************************************/
  /* Set the native renderering mechanism and format for the item.    */
  /********************************************************************/
#ifdef IC_MOTIFPM 
  IString tempStr = generateSourceName();
#endif // IC_MOTIFPM
#ifdef IC_WIN
  ITRACE_DEVELOP("about to generateSourceName");
  IString tempStr;
#endif
  if (tempStr.size())
  {
    setSourceName(tempStr);
    setRMFs(rmfsFrom(IDM::rmLibrary,
                     rfForThisProcess() + "," + IDM::rfText));
  }
  else
  {
    setRMFs(rmfsFrom(IDM::rmLibrary,
                     rfForThisProcess() + "," + IDM::rfSharedMem));

    /******************************************************************/
    /* Use render prepare event to determine size of transfer buffer  */
    /******************************************************************/
    setRequiresPreparation(true);
  }

  /********************************************************************/
  /* Add a drag image for the item.                                   */
  /********************************************************************/
#ifdef IC_PM  
  IDMImage image = IDMImage(ISystemPointerHandle(
                            ISystemPointerHandle::text));
  setImage(image);
#endif // IC_PM
#ifdef IC_MOTIF 
   IDMImage image ;
   setImage(image);
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IDMMLEItem::IDMMLEItem                                                       |
|                                                                              |
| Constructor using target item arguments.                                     |
------------------------------------------------------------------------------*/
IDMMLEItem::IDMMLEItem(const IDMItem::Handle &pMLEItem)  :
                       IDMItem(pMLEItem)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Verify that one of these rmfs is supported by the source.        */
  /********************************************************************/
  IString defaultRMFs = rmfsFrom(IDM::rmLibrary,
                                      rfForThisProcess() + "," +
                                      IDM::rfText + "," +
                                      IDM::rfSharedMem);
  IString rmf = matchingRMFs (rmfsFrom( rmfs().change(' ', "") ),
                              defaultRMFs);

  /******************************************************************/
  /* The RMFs that IDMMLEItem understands are                       */
  /* (IDM::rmLibrary)x                                              */
  /*           (IDM::rfProcess=pid,IDM::rfText,IDM::sharedMem)      */
  /******************************************************************/
  setRMFs(rmf);
}

/*------------------------------------------------------------------------------
| IDMMLEItem::~IDMMLEItem                                                      |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMMLEItem :: ~IDMMLEItem()
{
  IFUNCTRACE_DEVELOP();
}

/*------------------------------------------------------------------------------
| IDMMLEItem::generateSourceItems                                              |
|                                                                              |
| Called to give IDMMLEItem opportunity to attach new IDMMLEItems to the       |
| argument IDMSourceOperation object.  This function is called when a drag     |
| operation commences over an IMLE object.  Returns true if items were         |
| provided, false otherwise.                                                   |
------------------------------------------------------------------------------*/
bool IDMMLEItem ::
        generateSourceItems(IDMSourceOperation *pDMSrcOp)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Obtain pointer to MLE control from the source window             */
  /********************************************************************/
  IMultiLineEdit *pMLE = (IMultiLineEdit *)pDMSrcOp->sourceWindow();

  /********************************************************************/
  /* Do not attempt drag if the MLE is empty or has no text selected  */
  /********************************************************************/
  if (!(pMLE->textLength()))
    return(false);

  if (! pMLE->hasSelectedText())
     return(false) ;

  /********************************************************************/
  /* Create the MLE drag item                                         */
  /********************************************************************/
  IDMMLEItem::Handle pMLEItem(new IDMMLEItem(pDMSrcOp)) ;

  /********************************************************************/
  /* Add MLE drag item to the source operation.                       */
  /********************************************************************/
  pDMSrcOp->addItem(pMLEItem);

  /********************************************************************/
  /* Set the drag image style.                                        */
  /********************************************************************/
#ifdef IC_MOTIFPM  
  pDMSrcOp->setImageStyle(IDM::allStacked);
#endif // IC_MOTIFPM

  /********************************************************************/
  /* Indicate that the MLE drag item was successfully created.        */
  /********************************************************************/
  return(true);
}

/*------------------------------------------------------------------------------
| IDMMLEItem::object                                                           |
|                                                                              |
| Returns a pointer to the item text.                                          |
------------------------------------------------------------------------------*/
void* IDMMLEItem :: object() const
{
  return((void *)&(this->strContents));
}

/*------------------------------------------------------------------------------
| IDMMLEItem::targetDrop                                                       |
|                                                                              |
| Write text into entry field from drag item.                                  |
------------------------------------------------------------------------------*/
bool IDMMLEItem :: targetDrop(IDMTargetDropEvent &)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Cast IWindow pointer to MLE                                      */
  /********************************************************************/
  IMultiLineEdit *pMLE =
                 (IMultiLineEdit *)this->targetOperation()->targetWindow();

  /********************************************************************/
  /* Add the text to the MLE control                                  */
  /********************************************************************/
  pMLE->add(contents(), contentsSize());

  return(true);
}

/*------------------------------------------------------------------------------
| IDMMLEItem::sourceEnd                                                        |
|                                                                              |
| Remove selected text to complete the move operation.                         |
------------------------------------------------------------------------------*/
bool IDMMLEItem :: sourceEnd(IDMSourceEndEvent &event)
{
  IFUNCTRACE_DEVELOP();
#ifdef IC_WIN

  if ( event.wasTargetSuccessful() &&
     ( this->sourceOperation()->operation() == IDMOperation::move ))
    {
    // Get the entry field
    IMultiLineEdit *pMLE = (IMultiLineEdit *)this->sourceOperation()->sourceWindow();


    // NOTE:  When using non-default drag, the popup menu causes a problem
    // with obtaining the selected range.  EM_GETSEL returns the cursor
    // position as both the start and end values for the selected text.
    // I am able to work around this with the implementation below to determine
    // the selected text accurately.
    //
    // The above commented out code is the correct implementation for left mouse
    // button default drags, but would not suffice for non-default drags.  The
    // implementation below works for both drag operations.

    IString newText = pMLE->text();
    bool fSelInRange = false;
    unsigned startPos = 0;

    IEventResult Sel;
    this->sourceOperation()->sourceWindowHandle().sendEvent(EM_GETSEL,
                     IEventParameter1(&Sel),
                     IEventParameter2(0));

    while (!fSelInRange)
      {
      startPos = newText.indexOf(contents(), startPos + 1);

      // If the selection position (could be start or end of selected text)
      // is within the range of the matching text string
      if (( (Sel.asUnsignedLong()+1) >= startPos ) &&
          ( (Sel.asUnsignedLong()+1) <= (startPos + contents().length()) ) )
        {
        // remove selected text from string
        newText.remove(startPos, contents().length());
        fSelInRange = true;
        }
      }

    pMLE->setText(newText);
    }
#endif // IC_WIN
#ifdef IC_MOTIF
  if ( event.wasTargetSuccessful() &&
     ( this->sourceOperation()->operation() == IDMOperation::move ))
  {
     // Get the entry field
     IMultiLineEdit *pMLE = (IMultiLineEdit *)this->sourceOperation()->sourceWindow();
     if (pMLE->hasSelectedText())
     {
        IString newText = pMLE->text();
    
        IRange selectedText = pMLE->selectedRange() ;
    
        newText.remove(selectedText.lowerBound()+1, 
                       pMLE->selectedTextLength() );
        pMLE->setText(newText);
     }
  }
#endif // IC_MOTIF
  return( true );
}


