// Revision: 83 1.14.1.6 source/ui/drag/idmmotf.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmmotf.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMOperationMarshaller                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ibase.hpp>

#ifdef IC_MOTIF
extern "C"
{
  #include <strings.h>
  #include <memory.h>
  #include <Xm/Xm.h>
  #include <cnr.h>
  #include <Xm/ScrolledW.h>
  #include <Xm/DragDrop.h>
  #include <Xm/AtomMgr.h>
  #include <Xm/DropSMgrP.h>
  #include <X11/Xmu/StdSel.h>
}
#endif

#include <ireslib.hpp>
// Tracing functions
#include <itrace.hpp>

// Exception handling
#include <iexcept.hpp>

#include <icconst.h>

// Include our classes's stuff
#include <idmmotf.hpp>

#include <icoordsy.hpp>


#ifdef IC_MOTIF
#include <icnrrec.hpp>
#include <idmsrcop.hpp>
#include <idmtgtop.hpp>
#include <idmdnfo.hpp>
#include <idmopma.hpp>
#include <idmevent.hpp>
#include <idmformt.hpp>
#include <ictlevt.hpp>
#include <icnr.hpp>
#include <ithread.hpp>      // For the calls to XtAppAddActions:
#include <idmimage.hpp>
#include <ibmpstat.hpp>
#include <igimage.hpp>
#include <ibmpdata.hpp>
#endif // IC_MOTIF

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMMOTF_CPP_
  #include <ipagetun.h>
#endif

// Use this var to remember the default XmNconvertProc function for a widget.
// Since we can only have one D/D operation going on at any one time, no point in
// saving the proc for every widget enabled for D/D. We just save the function for the
// current widget whenever a drag starts.
// Note that O'Reilly V6B says that the type of an XmNConvertProc is XtConverSelectionIncrProc
// which is WRONG !!! It's true only when XmNincremental == True. I was going to use
// XmNclientData to save the function address but then we can't get to it once we're
// in the XmNConvertProc routine
IDMDefaultConvertProc* IDMMotif::defaultConvertProc = 0 ;
IDMDefaultDropProc* IDMMotif::defaultDropProc = 0 ;

UsArray::UsArray( unsigned long numElems )
: numInArray( numElems ),
  pArray( 0 )
{
 if ( numInArray ) pArray = new unsigned long [ numInArray ];
 memset(pArray, 0, sizeof(unsigned long)*numInArray) ; 
}

#if IC_EXTENDED_RTTI_ENABLED
UsArray::UsArray( const UsArray& )
: numInArray( 0 ),
  pArray( 0 )
{
}
#endif // IC_EXTENDED_RTTI_ENABLED

UsArray::~UsArray()
{
 delete [] pArray;
}

unsigned long UsArray::length()
{
 return numInArray;
}

unsigned long& UsArray::operator[]( unsigned long index )
{
 if ( index >= numInArray )
  {
   ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
  }
 return pArray[ index ];
}

unsigned long UsArray::marshall( unsigned long startPos, char* outStr ) const
{
 unsigned long
 newPos = IDMMarshall::marshall( (long)numInArray, startPos, outStr );
 for (unsigned long i=0; i<numInArray; i++)
  {
   newPos = IDMMarshall::marshall( (long)pArray[i], newPos, outStr );
  }
 return newPos;
}

const char* UsArray::unmarshall( const char* inStr )
{
 long result;
 const char*
 newStr = IDMMarshall::unmarshall( result, inStr );
 numInArray = (unsigned long)result;

 delete [] pArray;
 pArray = 0;
 if ( numInArray ) pArray = new unsigned long [ numInArray ];
 memset(pArray, 0, sizeof(unsigned long)*numInArray) ; 

 for (unsigned long i=0; i<numInArray; i++)
  {
   newStr = IDMMarshall::unmarshall( result, newStr );
   pArray[i] = (unsigned long)result;
  }
 return newStr;
}

#ifdef IC_MOTIF

#pragma enum(4)
#pragma pack(push,4)
class dragDropRequestorCallbackInfo
 {
  public:
   virtual ~dragDropRequestorCallbackInfo();

   unsigned long numberOfLeaves() const;
   IWindow& dropSiteWindow() const;

   virtual IPoint windowPoint( const IWindow& iWin ) const=0;
   virtual IPoint motifPoint() const=0;
   virtual unsigned char motifOperation() const=0;
   virtual unsigned char motifOperations() const=0;
   virtual Time timeStamp() const=0;
   virtual dragDropRequestorCallbackInfo* clone() const=0;

  protected:
   dragDropRequestorCallbackInfo( unsigned long numLeaves,
                                  Widget wid );
   dragDropRequestorCallbackInfo( const dragDropRequestorCallbackInfo& info);

  private:
   unsigned long ulNumLeaves;
   IWindow* theIWindowPtr;
 };

class dragRequestorCallbackInfo : public dragDropRequestorCallbackInfo
 {
   public:
    dragRequestorCallbackInfo( unsigned long numLeaves,
                               Widget wid,
                               const XmDragProcCallbackStruct& cbs );
    dragRequestorCallbackInfo( const dragRequestorCallbackInfo& info);
    virtual ~dragRequestorCallbackInfo();
    virtual IPoint windowPoint( const IWindow& iWin ) const;
    virtual IPoint motifPoint() const;
    virtual unsigned char motifOperation() const;
    virtual unsigned char motifOperations() const;
    virtual Time timeStamp() const;
    virtual dragDropRequestorCallbackInfo* clone() const;
 
   private:
    Widget theWidget;
    XmDragProcCallbackStruct theCbs;
 };

class dropRequestorCallbackInfo : public dragDropRequestorCallbackInfo
 {
   public:
    dropRequestorCallbackInfo( unsigned long numLeaves,
                               Widget wid,
                               const XmDropProcCallbackStruct& cbs );
    dropRequestorCallbackInfo( const dropRequestorCallbackInfo& info);
    virtual ~dropRequestorCallbackInfo();
    virtual IPoint windowPoint( const IWindow& iWin ) const;
    virtual IPoint motifPoint() const;
    virtual unsigned char motifOperation() const;
    virtual unsigned char motifOperations() const;
    virtual Time timeStamp() const;
    virtual dragDropRequestorCallbackInfo* clone() const;
    virtual dropRequestorCallbackInfo& setTargetType(const Atom& atomValue) ;
    virtual Atom targetType() ;
 
    unsigned char dropAction() const;
 
   private:
    Widget                   theWidget;
    XmDropProcCallbackStruct theCbs;
    Atom                     targetAtom ;
 };
 
#pragma enum(pop)
#pragma pack(pop)
 
dragDropRequestorCallbackInfo::~dragDropRequestorCallbackInfo()
{}

dragDropRequestorCallbackInfo::dragDropRequestorCallbackInfo(
                               unsigned long numLeaves,
                               Widget wid )
: ulNumLeaves( numLeaves ),
  theIWindowPtr( IDMMotif::controlWindow( wid ) ) 
{}

dragDropRequestorCallbackInfo::dragDropRequestorCallbackInfo(
                                const dragDropRequestorCallbackInfo& info)
: ulNumLeaves( info.ulNumLeaves ),
  theIWindowPtr( info.theIWindowPtr )
{}

unsigned long dragDropRequestorCallbackInfo::numberOfLeaves() const
{
 return ulNumLeaves;
}

IWindow& dragDropRequestorCallbackInfo::dropSiteWindow() const
{
 return *theIWindowPtr;
}

dragRequestorCallbackInfo::~dragRequestorCallbackInfo()
{}

dragRequestorCallbackInfo::dragRequestorCallbackInfo(
                           unsigned long numLeaves,
                           Widget wid,
                           const XmDragProcCallbackStruct& cbs )
: dragDropRequestorCallbackInfo( numLeaves, wid ),
  theWidget( wid ),
  theCbs( cbs )
{}

dragRequestorCallbackInfo::dragRequestorCallbackInfo(
                            const dragRequestorCallbackInfo& info)
: dragDropRequestorCallbackInfo( info ),
  theWidget( info.theWidget ),
  theCbs( info.theCbs )
{}

IPoint dragRequestorCallbackInfo::windowPoint( const IWindow& iWin ) const
{
 return IDMMotif::motif2Window( theWidget, theCbs.x, theCbs.y, &iWin );
}

IPoint dragRequestorCallbackInfo::motifPoint() const
{
 return IPoint( theCbs.x, theCbs.y);
}

unsigned char dragRequestorCallbackInfo::motifOperation() const
{
 return theCbs.operation;
}

unsigned char dragRequestorCallbackInfo::motifOperations() const
{
 return theCbs.operations;
}

Time dragRequestorCallbackInfo::timeStamp() const
{
 return theCbs.timeStamp;
}

dragDropRequestorCallbackInfo* dragRequestorCallbackInfo::clone() const
{
 return new dragRequestorCallbackInfo(*this);
}

dropRequestorCallbackInfo::~dropRequestorCallbackInfo()
{}

dropRequestorCallbackInfo::dropRequestorCallbackInfo(
                           unsigned long numLeaves,
                           Widget wid,
                           const XmDropProcCallbackStruct& cbs )
: dragDropRequestorCallbackInfo( numLeaves, wid ),
  theWidget( wid )
  , theCbs( cbs )
  , targetAtom(0)
{}

dropRequestorCallbackInfo::dropRequestorCallbackInfo(
                            const dropRequestorCallbackInfo& info)
: dragDropRequestorCallbackInfo( info ),
  theWidget( info.theWidget ),
  theCbs( info.theCbs )
{}

dropRequestorCallbackInfo& dropRequestorCallbackInfo::setTargetType( const Atom& atomValue )
{
   targetAtom = atomValue ;
   return *this ;
}

Atom dropRequestorCallbackInfo::targetType(  )
{
   return targetAtom ;
}

IPoint dropRequestorCallbackInfo::windowPoint( const IWindow& iWin ) const
{
 return IDMMotif::motif2Window( theWidget, theCbs.x, theCbs.y, &iWin );
}

IPoint dropRequestorCallbackInfo::motifPoint() const
{
 return IPoint( theCbs.x, theCbs.y);
}

unsigned char dropRequestorCallbackInfo::motifOperation() const
{
 return theCbs.operation;
}

unsigned char dropRequestorCallbackInfo::motifOperations() const
{
 return theCbs.operations;
}

Time dropRequestorCallbackInfo::timeStamp() const
{
 return theCbs.timeStamp;
}

dragDropRequestorCallbackInfo* dropRequestorCallbackInfo::clone() const
{
 return new dropRequestorCallbackInfo(*this);
}

unsigned char dropRequestorCallbackInfo::dropAction() const
{
 return theCbs.dropAction;
}

static ::Boolean idragCompletionConvertProc(
        Widget          wid,
        Atom*           selection,
        Atom*           target,
        Atom*           type_return,
        XtPointer*      value_return,
        unsigned long*  length_return,
        int*            format_return)
{
 IMODTRACE_DEVELOP("idragCompletionConvertProc");
 if ( *target == XmInternAtom( XtDisplay(wid), "ICLUI_DM_COMPLETION", False )
   && IDMMotif::targetCompletions() )
  {
   ITRACE_DEVELOP( "Sending completion information." );
   //
   // This executes in the target process.  It provides completion information
   // to the source process.
   //
   *type_return = *target;
   *length_return = IDMMotif::targetCompletions()->marshall();
   *value_return = XtMalloc( sizeof(char)*(*length_return) );
   IDMMotif::targetCompletions()->marshall(0, (char*)(*value_return));
   *format_return = sizeof(char)*8;
   //
   // Get rid of the completion information and disown this selection.
   //
   XtDisownSelection(wid, *selection, CurrentTime);
   delete IDMMotif::targetCompletions();
   IDMMotif::targetCompletions() = 0;
   return True;
  }
 return False;
}

static ::Boolean idragLeaveConvertProc(
        Widget          wid,
        Atom*           selection,
        Atom*           target,
        Atom*           type_return,
        XtPointer*      value_return,
        unsigned long*  length_return,
        int*            format_return)
{
 IMODTRACE_DEVELOP("idragLeaveConvertProc");
 if ( *target == XmInternAtom(XtDisplay(wid), "ICLUI_DM_ALLOW_LEAVE", False) )
  {
   //
   // We got here because the drag started but has not finished yet.
   // (i.e. the button up event has not occurred yet).
   //
   // When a drop occurs, Motif sends a leave event, followed by a drop
   // event. To emulate PM behaviour, the leave event just before the
   // drop event needs to be ignored.
   //
   // The requestor has encountered a leave event, and wants to know
   // if it should be processed.  Since we got here, it can be processed.
   // So we return True in *value_return.
   //
   *type_return = *target;
   *value_return = XtMalloc(sizeof(char));
   *(char*)(*value_return) = True;
   *length_return = 1;
   *format_return = sizeof(char)*8;
   return True;
  }
 return False;
}

static void idragFinish(
        Widget          wid,
        XButtonEvent*   event,
        String*         params,
        Cardinal*       num_params)
{
 IMODTRACE_DEVELOP("idragFinish");
 if ( XmIsDragContext( wid ) )
  {
   //
   // Any target leave events from this point on should be ignored.
   //
   IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
   Widget sourceWidget = pSrcOpH->sourceWindow()->handleForChildCreation() ;
   Atom dm_leave
      = XmInternAtom(XtDisplay(sourceWidget), "ICLUI_DM_LEAVE", False);
   XtDisownSelection(sourceWidget, dm_leave, CurrentTime);
  }
 XtCallActionProc( wid, "FinishDrag", (XEvent*)event, params, *num_params );
}

static void idragCompletionRequestorCallbackProc(
        Widget          wid,
        XtPointer       /* client_data */,
        Atom*           /* selection */,
        Atom*           type,
        XtPointer       value,
        unsigned long*  length,
        int*            /* format */ )
{
 IMODTRACE_DEVELOP("idragCompletionRequestorCallbackProc");

 if ( value && *length
   && *type == XmInternAtom(XtDisplay(wid), "ICLUI_DM_COMPLETION", False) )
  {
   //
   // If we do not have a source operation there is nothing to do...
   //
   IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
   if ( pSrcOpH.valid() )
    {
     //
     // Obtain the source window from the source operation.
     //
     IWindow& iWin = *(pSrcOpH->sourceWindow());
     //
     // Unmarshall the array of completion information.
     //
     UsArray completionInformation(0);
     completionInformation.unmarshall( (char*)value );
     XtFree((char*)value); // Free asap to avoid a memory leak if exception.
     value = 0; // Remember we have freed it.
     //
     // completionInformation contains a pair of numbers for each item processed
     // followed by the usOperation for the dragInfo.
     //
     // The first number of each pair is the ulItemID, and the second is the
     // IDM::RenderCompletion completion status.
     //
     // For each such pair, dispatch a DM_ENDCONVERSATION message to the source
     // window.  Loop from high to low index, since that is the order
     // IDMTargetRenderer::informSourceOfCompletion adds them.
     //
     IEventResult result;
     unsigned long count = completionInformation.length();
     IDragInfoWrapper dragInfoWrapper( *pSrcOpH );
     dragInfoWrapper.setDragOperation( (unsigned short)
                                       completionInformation[ --count ] );
     for ( unsigned long i = count; i; i -= 2 )
      {
       IDMMotif::sendEndConversation( iWin,
                              completionInformation[i-2],
                              completionInformation[i-1],
                              result );
      }
    }
  }
 if ( value ) XtFree((char*)value);
 //
 // Clean up the source operation.
 //
 IDMMotif::cleanupSourceOperation();
}

static ::Boolean idragConvertProc(
        Widget          wid,
        Atom*           selection,
        Atom*           target,
        Atom*           type_return,
        XtPointer*      value_return,
        unsigned long*  length_return,
        int*            format_return)
{
 IMODTRACE_DEVELOP("idragConvertProc");
//@@ static unsigned nextItemNumberToComplete = 0;
 Display* dpy = XtDisplay( wid );

 if ( *target == XmInternAtom( dpy, "XA_TARGETS", False ) )
  {
   ITRACE_DEVELOP( IString("*target=") + IString("XA_TARGETS.") );
   //
   // Handle XA_TARGETS target, for ICCCM compliance.
   // See O'Reilley V4, Example 11-10
   //
   Atom* targetP;
   Atom* std_targets;
   unsigned long std_length;
   // Get the standard list of supported targets.
   XSelectionRequestEvent*
   req = XtGetSelectionRequest( wid, *selection, (XtRequestId)NULL );
   XmuConvertStandardSelection( wid, req->time, selection, target,
                                type_return,
                                (caddr_t*)(XtPointer*)&std_targets, // Compiler requires explicit cast of XtPointer* to caddr_t*.
                                &std_length, format_return);
   // Prefix our target to the list.
   *value_return = XtMalloc( sizeof(Atom)*(std_length+2) );
   targetP = *(Atom**)value_return;
   *length_return = std_length+4;
   *targetP++ = XmInternAtom( dpy, "ICLUI_DM_DRAGINFO", False );
   *targetP++ = XmInternAtom( dpy, "ICLUI_DM_TRANSFER", False );
   *targetP++ = XmInternAtom( dpy, "ICLUI_DM_READY", False );
   *targetP++ = XmInternAtom( dpy, "ICLUI_DM_NULL", False );
   bcopy( (char*)std_targets, (char*)targetP, sizeof(Atom)*std_length );
   XtFree( (char*)std_targets );
   // Return the list of supported targets.
   *type_return = XA_ATOM;
   *format_return = sizeof(Atom)*8;
  }
 else
 if ( *target == XmInternAtom(dpy, "ICLUI_DM_DRAGINFO", False) )
  {
   ITRACE_DEVELOP( IString("*target=") + IString("ICLUI_DM_DRAGINFO.") );
   IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
   if ( ! pSrcOpH.valid() ) return False;
   //
   // Return a string representing the IDragInfo data.
   //
   *type_return = *target;
   IDragInfo* pDragInfo = IDragInfoWrapper( *pSrcOpH ).asDragInfoPtr();
   *length_return = pDragInfo->marshall();
   *value_return = XtMalloc( sizeof(char)*(*length_return) );
   pDragInfo->marshall(0, (char*)(*value_return));
   *format_return = sizeof(char)*8;
  }
 else
 if ( *target == XmInternAtom(dpy, "ICLUI_DM_TRANSFER", False) )
  {
   ITRACE_DEVELOP( IString("*target=") + IString("ICLUI_DM_TRANSFER.") );
   IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
   if ( ! pSrcOpH.valid() ) return False;
//@@   nextItemNumberToComplete = pSrcOpH->numberOfItems();
   //
   // Return a string representing the "object" and "contents" of the items.
   //
   *type_return = *target;
   IDMOperation* pSrcOp = pSrcOpH.getAlias() ; 
   IDMOperationMarshaller opMarshaller( *pSrcOp );
   *length_return = opMarshaller.marshall();
   *value_return = XtMalloc( sizeof(char)*(*length_return) );
   opMarshaller.marshall(0, (char*)(*value_return));
   *format_return = sizeof(char)*8;
  }
 else
 if ( *target == XmInternAtom(dpy, "ICLUI_DM_READY", False)
   || *target == XmInternAtom(dpy, "ICLUI_DM_NULL", False) )
  {
   if ( *target == XmInternAtom(dpy, "ICLUI_DM_READY", False) )
    {
     ITRACE_DEVELOP( IString("*target=") + IString("ICLUI_DM_READY.") );
     //
     // The target process is ready to send the completion information.
     // Ask the target to send it.
     //
     IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
     if ( ! pSrcOpH.valid() ) return False; // Should never happen...
     Widget sourceWidget = pSrcOpH->sourceWindowHandle(); // N.B. In some cases, this
                                                          // is the top handle of a 
                                                          // composite widget. But,
                                                          // we don't care because
                                                          // it's only used as a 
                                                          // placeholder
     XtGetSelectionValue( sourceWidget,
       XmInternAtom( dpy, "ICLUI_DM_RESULT", False ),
       XmInternAtom( dpy, "ICLUI_DM_COMPLETION", False ),
       idragCompletionRequestorCallbackProc,
       0,
       XtLastTimestampProcessed( dpy ) );
    }
   else
    {
     ITRACE_DEVELOP( IString("*target=") + IString("ICLUI_DM_NULL.") );
    }
   //
   // Return a single character string so the requestor has something to free.
   //
   *type_return = *target;
   *length_return = 1;
   *value_return = XtMalloc( sizeof(char) );
   *format_return = sizeof(char)*8;
  }
//@@ else
//@@ if ( *target == XmInternAtom(dpy, "ICLUI_DM_SUCCESS", False)
//@@   || *target == XmInternAtom(dpy, "ICLUI_DM_FAILURE", False) )
//@@  {
//@@   unsigned long completion_flags;
//@@   if ( *target == XmInternAtom(dpy, "ICLUI_DM_SUCCESS", False) )
//@@    {
//@@     ITRACE_DEVELOP( IString("*target=") + IString("ICLUI_DM_SUCCESS.") );
//@@     completion_flags = IDM::targetSuccessful;
//@@    }
//@@   else
//@@    {
//@@     ITRACE_DEVELOP( IString("*target=") + IString("ICLUI_DM_FAILURE.") );
//@@     completion_flags = IDM::targetFailed;
//@@    }
//@@   IDMSourceOperation::Handle pSrcOpH = IDMSourceOperation::sourceOperation();
//@@   if ( !pSrcOpH ) return False; // Should never happen...
//@@   //
//@@   // Send an endConversation event to the source window.
//@@   //
//@@   IWindow& iWin = *pSrcOpH->sourceWindow();
//@@   IDMItem::Handle itemH = pSrcOpH->item( nextItemNumberToComplete-- );
//@@   IEventResult result;
//@@   IDMMotif::sendEndConversation( iWin,
//@@                              (unsigned long)(IDMItem*)itemH,
//@@                              completion_flags,
//@@                              result );
//@@   //
//@@   // Return a single character string so the requestor has something to free.
//@@   //
//@@   *type_return = *target;
//@@   *length_return = 1;
//@@   *value_return = XtMalloc( sizeof(char) );
//@@   *format_return = sizeof(char)*8;
//@@  }
 else
  {
   ITRACE_DEVELOP( "Unknown target." );

   Atom dd = XmInternAtom(dpy, "_MOTIF_DROP", False) ;

   ::Boolean bDefaultProc = False ;

   // handle the case where we are dropping from a Motif widget that's predefined
   // for Drag/Drop to a non ICLUI drop site
   if (*selection == dd && IDMMotif::defaultConvertProc != 0)
      bDefaultProc = (* IDMMotif::defaultConvertProc) (wid,
                                                       selection,
                                                       target,
                                                       type_return,
                                                       value_return,
                                                       length_return,
                                                       format_return ) ;
   if (! bDefaultProc) {
      // Handle the other standard targets, for ICCCM compliance.
      // See O'Reilley V4, Example 11-10
      XSelectionRequestEvent*
         req = XtGetSelectionRequest( wid, *selection, (XtRequestId)NULL );
      if ( !XmuConvertStandardSelection( wid, req->time, selection, target,
              type_return, (caddr_t*)value_return, length_return, format_return ) )
         return False;
   }
   ITRACE_DEVELOP( "Standard selection was processed." );
  } // else
 return True;
} // idragConvertProc

/*------------------------------------------------------------------------------
|  idropStartCallback is the XmNdropStartCallback for the Motif drag context.  |
------------------------------------------------------------------------------*/
extern void _System idropStartCallback(
               Widget      /* wid */,
               XtPointer   /* client_data */,
               XtPointer   call_data)
{
 IMODTRACE_DEVELOP("idropStartCallback");

 XmDropStartCallbackStruct* cbs = (XmDropStartCallbackStruct*) call_data;
 //
 // Update the dragInfo with the drag position from the drag context.
 //
 IDragInfoWrapper dragInfoWrapper( *IDMSourceOperation::sourceOperation() );
 dragInfoWrapper.setDropPoint(
       IDMMotif::motif2Window(0, cbs->x, cbs->y, IWindow::desktopWindow() ) );
//@@ //
//@@ // Update the dragInfo with the current operation from the drag context.
//@@ //
//@@ dragInfoWrapper.setDragOperation(
//@@                    IDMMotif::motifOperation2DragOperation( cbs->operation,
//@@                                                            cbs->operations ) );
} // end idropStartCallback

/*------------------------------------------------------------------------------
|  idropFinishCallback is the XmNdropFinishCallback for the Motif drag         |
|  context.  It is called when the drop completes, or is canceled.             |
------------------------------------------------------------------------------*/
extern void _System idropFinishCallback(
               Widget      /* wid */,
               XtPointer   client_data,
               XtPointer   call_data )
{
 IMODTRACE_DEVELOP("idropFinishCallback");
 //
 // Free the selection.
 //
 Widget sourceWidget = (Widget)client_data;
 Atom dm_selection
 = XmInternAtom(XtDisplay(sourceWidget), "ICLUI_DM_SELECTION", False);
 XtDisownSelection(sourceWidget, dm_selection, CurrentTime);
 //
 // Cleanup the source operation, unless it was a successful drop.
 // In the case of a successful drop, the source operation is cleaned up
 // from idragCompletionRequestorCallbackProc.
 //
 XmDropFinishCallbackStruct* cbs = (XmDropFinishCallbackStruct*)call_data;
 if ( cbs->completionStatus != XmDROP_SUCCESS || cbs->dropAction != XmDROP )
  {
   // Get rid of the source operation.
   IDMMotif::cleanupSourceOperation();
  }
} // end idropFinishCallback

/*------------------------------------------------------------------------------
|  idragDropFinishCallback is the XmNdragDropFinishCallback for the Motif drag |
|  context.                                                                    |
------------------------------------------------------------------------------*/
extern void _System idragDropFinishCallback(
               Widget      /* wid */,
               XtPointer   /* client_data */,
               XtPointer   /* call_data */ )
{
 IMODTRACE_DEVELOP("idragDropFinishCallback");
 IDMMotif::defaultConvertProc = 0 ;
} // end idragDropFinishCallback

static void idragStart(
        Widget          wid,
        XEvent*         event,
        String*         /* params */,
        Cardinal*       /* num_params */ )

{
 IMODTRACE_DEVELOP("idragStart");
 //
 // If we still have a source operation, the previous drag/drop has not finished
 // yet.  This can happen if the user clicks up and down quickly on a drag
 // source.  Just ignore this kind of start.
 //
  {
   IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
   if ( pSrcOpH.valid() ) return;
  }

 Display* dpy = event->xbutton.display;
 //
 // Set the selection proc to provide drag info to the drop site,
 // and the selection proc to allow target leave events.
 //
 Atom dm_selection = XmInternAtom(dpy, "ICLUI_DM_SELECTION", False);
 Atom dm_leave = XmInternAtom(dpy, "ICLUI_DM_LEAVE", False);
 if ( ! XtOwnSelection( wid, dm_selection, event->xbutton.time,
                        (XtConvertSelectionProc) idragConvertProc, NULL, NULL )
   || ! XtOwnSelection( wid, dm_leave, event->xbutton.time,
                        idragLeaveConvertProc, NULL, NULL ) )
  {
   XtDisownSelection(wid, dm_selection, CurrentTime);
   XtDisownSelection(wid, dm_leave, CurrentTime);
   return ;
  }
  //
  // Send appropriate beginDrag events to the window.
  //
 IWindow& iWin = *IDMMotif::controlWindow(wid) ; 
  IEventResult result;
  IDMMotif::sendBeginDrag( iWin, *event, result );

  // If the source operation exists and has some items, start the drag.
   IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
   if ( ! pSrcOpH.valid() )
      return ;
   unsigned long nItems = pSrcOpH->numberOfItems();
   if ( nItems == 0 ) {
      // If we have no items, clean up and return as if the drag never happened.
      IDMMotif::cleanupSourceOperation();
      return ;
   }

   Widget dragContext ;
   Arg dcArgs[10] ;            // resource array for DragContext
   int dcArgsIndex(0) ;
   Atom IDMExportTarget = XmInternAtom( dpy, "ICLUI_DM_TRANSFER", False);

   // set up our XmNConvertProc callback
   // this is the routine that Motif calls when the target asks for the
   // drag data
   XtSetArg(dcArgs[dcArgsIndex], XmNconvertProc, idragConvertProc)  ; dcArgsIndex++;

   //
   // Allow all possible operations.  Otherwise, Motif sets operation and
   // operations to zero when an invalid operation is specified, and
   // does not display the operation icon.
   //
   // As far as I can tell, Motif really should be displaying the operation
   // icon, even when it is an invalid operation.  The state icon is
   // supposed to be used to indicate drop site validity, even though
   // Motif's default is to show the same state icon for all 3 states
   // (valid drop site, invalid drop site, and no drop site).
   //
   // See O'Reilly V61 18.2.3 pg 629 for a discussion of these icons.
   //
   // We want to know the operation in order to emulate PM behaviour.
   //
   unsigned char dcOperations = ~0;
   XtSetArg(dcArgs[dcArgsIndex], XmNdragOperations, dcOperations ) ; dcArgsIndex++ ;


   /*---------------------------------------------------------------
      Set up the dragimage for this drag item.
   ---------------------------------------------------------------*/

   Widget dragIcon;
   Pixmap sourceIcon ;
   Pixmap shapeMask ;
   bool    bBitmap(true) ;

   // at this point, we know we have a valid , non-empty sourceOperation object 
   // following lines commented out until kMultipleFile is supported on Motif
   //if (pSrcOpH->numberOfItems() > 1) 
   //   sourceIcon = ISystemPointerHandle(ISystemPointerHandle::kMultipleFile) ;
   //else
   {
      IDMItem::Handle pItemHandle(pSrcOpH->item(1)) ;  
      IDMImage& image      = pItemHandle->image();
      sourceIcon = image.bitmap() ;
      if (! sourceIcon) {
         sourceIcon = image.pointer() ;
         bBitmap = false ;
      }
   }
                                                    

   //if ( (diStyle == IDM::allStacked) ||
   //     (diStyle == IDM::stack3AndFade))
   //{

        if ( sourceIcon )
        {
           Arg iconArgs[10] ;
           int n(0) ;
           unsigned int  iCursorWidth, iCursorHeight;
           int iDepth = DefaultDepth( dpy, XScreenNumberOfScreen(XtScreen(wid))) ;
           XtSetArg( iconArgs[n], XmNpixmap, sourceIcon ); n++;
           if (bBitmap)
           {
              IGImage iimage(sourceIcon) ;
              IGRect2D imageSize = iimage.rasterBounds() ;
              iCursorWidth         = imageSize.width() ;
              iCursorHeight        = imageSize.height() ;
           }
           else 
           {
              shapeMask = IBitmapStaticPtr::shapeMaskFor( (IPointerHandle) sourceIcon );
              if ( shapeMask )
                 XtSetArg( iconArgs[n], XmNmask, shapeMask );n++;
              ISize iconSize = IBitmapStaticPtr::pointerSet().find(sourceIcon)->size() ;
              iCursorWidth  = iconSize.width() ; 
              iCursorHeight = iconSize.height() ;
           }
           XtSetArg( iconArgs[n], XmNwidth,  iCursorWidth  );n++;
           XtSetArg( iconArgs[n], XmNheight, iCursorHeight );n++;
           XtSetArg( iconArgs[n], XmNdepth, iDepth );n++;
           dragIcon = XmCreateDragIcon( wid, "ICLUIDrag", iconArgs, n );
           XtSetArg(dcArgs[dcArgsIndex], XmNsourcePixmapIcon, dragIcon) ; dcArgsIndex++ ;
       }
   //}

   if (IDMMotif::isPreDefinedForDrag(wid)) {
      // for Motif built-in drag sources, we get called after the default
      // startDrag routine which already created the DragContext. We just extend
      // the DC with our values
      dragContext = XmGetDragContext(wid, event->xbutton.time) ;

      Arg args[10];
      int n(0) ;
      Cardinal numExportTargets ;
      Atom *currentExportTargets, *newExportTargets ;
      XtSetArg(args[n], XmNexportTargets, &currentExportTargets) ; n++ ;
      XtSetArg(args[n], XmNnumExportTargets, &numExportTargets)  ; n++ ;
      XtSetArg(args[n], XmNconvertProc, &(IDMMotif::defaultConvertProc))     ; n++ ;
      XtGetValues(dragContext, args, n) ;

      // add ourselves to the list of export types
      newExportTargets = (Atom *) XtMalloc (sizeof(Atom) * (numExportTargets + 1)) ;
      newExportTargets[0] = IDMExportTarget ;
      for (int i = 0; i < numExportTargets; i++)
         newExportTargets[i+1] = currentExportTargets[i] ;

      XtSetArg(dcArgs[dcArgsIndex], XmNexportTargets, newExportTargets)      ; dcArgsIndex++ ;
      XtSetArg(dcArgs[dcArgsIndex], XmNnumExportTargets, numExportTargets+1) ; dcArgsIndex++ ;
      XtSetValues(dragContext, dcArgs, dcArgsIndex) ;

      XtFree((char *) newExportTargets) ;
    } // isPreRegisteredForDragDrop
    else {
       // not a predefined drag source. We need to actually create the DC
       Atom exportList[1];
       exportList[0] = IDMExportTarget ;

       XtSetArg( dcArgs[dcArgsIndex], XmNexportTargets, exportList )    ; dcArgsIndex++;
       XtSetArg( dcArgs[dcArgsIndex], XmNnumExportTargets, 1 )          ; dcArgsIndex++;
       XtSetArg( dcArgs[dcArgsIndex], XmNclientData, (XtPointer) 0 )    ; dcArgsIndex++;

       dragContext = XmDragStart( wid, event, dcArgs, dcArgsIndex);   // Create the drag context.
    }

    IDMMotif::enableDragContext( dragContext );

    XtAddCallback( dragContext, XmNdropStartCallback     , idropStartCallback     , NULL);
    XtAddCallback( dragContext, XmNdropFinishCallback    , idropFinishCallback    , (XtPointer) wid );
    XtAddCallback( dragContext, XmNdragDropFinishCallback, idragDropFinishCallback, NULL );
 return ;
} // idragStart

static void idragInfoRequestorCallbackProc(
        Widget          wid,
        XtPointer       client_data,
        Atom*           /* selection */,
        Atom*           type,
        XtPointer       value,
        unsigned long*  length,
        int*            /* format */ )
{
 IMODTRACE_DEVELOP("idragInfoRequestorCallbackProc");

 dragDropRequestorCallbackInfo*
 pInfo = ( dragDropRequestorCallbackInfo* )client_data;

 if ( value && *length && pInfo
   && *type == XmInternAtom(XtDisplay(wid), "ICLUI_DM_DRAGINFO", False) )
  {
   unsigned long numLeaves = pInfo->numberOfLeaves();
   //
   // If we already have a target operation, we do not need to do anything.
   // This can happen if this callback is delayed, so that it gets called
   // twice in a row.
   //
   // And if we have left the drop site since requesting the dragInfo,
   // we must ignore it, since we must not be left with a target operation
   // after we have left the drop site.
   //
   if ( ! IDMTargetOperation::targetOperation().valid() &&
          numLeaves == IDMMotif::numberOfDropSiteLeaves() )
    {
     IDMMotif::numberOfDropSiteLeaves() = 0;  // Avoid incrementing forever.
     //
     // Create an IDragInfo from value.
     //
     IDragInfo* pDragInfo = new IDragInfo(0);
     pDragInfo->unmarshall( (char*)value );
     XtFree((char*)value); // Free asap to avoid a memory leak if exception.
     value = 0; // Remember we have freed it.
     IDragInfoWrapper dragInfoWrapper( *pDragInfo );
     //
     // Update the dragInfo with the pointer position and current operation.
     //
     dragInfoWrapper.setDropPoint(
                              pInfo->windowPoint( *IWindow::desktopWindow() ) );
     unsigned char cbsOperation = pInfo->motifOperation();
//@@     if ( cbsOperation != XmDROP_NOOP )
     dragInfoWrapper.setDragOperation(
           IDMMotif::motifOperation2DragOperation( cbsOperation,
                                                   pInfo->motifOperations() ) );
     //
     // Dispatch appropriate dragOver events to this widget.
     // This creates the IDMTargetOperation.
     //
     IWindow& iWin = pInfo->dropSiteWindow();
     IPoint motifPoint = pInfo->motifPoint();
     delete pInfo; pInfo = 0; // Free asap to avoid memory leak if exception.
     IEventResult result;
     IDMMotif::sendDragOver( iWin, dragInfoWrapper, motifPoint, result );
    }
  }
 if ( value ) XtFree((char*)value);
 delete pInfo;
}

static void idragLeaveRequestorCallbackProc(
        Widget          wid,
        XtPointer       client_data,
        Atom*           /* selection */,
        Atom*           type,
        XtPointer       value,
        unsigned long*  length,
        int*            /* format */ )
{
 IMODTRACE_DEVELOP("idragLeaveRequestorCallbackProc");
 Display* dpy = XtDisplay( wid );

 if ( value && *length
   && *type == XmInternAtom( dpy, "ICLUI_DM_ALLOW_LEAVE", False )
   && *(char*)value )
  {
   ITRACE_DEVELOP( "Received True for ICLUI_DM_ALLOW_LEAVE." );
   XtFree((char*)value); value=0; // Free asap to avoid a memory leak.
   //
   //  If we do not have a target operation, we have already done a drag leave.
   //  This can happen when the receipt of this callback is delayed.
   //
   if ( IDMTargetOperation::targetOperation().valid() )
    {
     //
     // Since we got True from idragLeaveConvertProc, we are still dragging.
     // Dispatch appropriate dragLeave events to this widget.
     //
     IDragInfo* pDragInfo = (IDragInfo*)client_data;
     IDragInfoWrapper dragInfoWrapper( *pDragInfo );
     IWindow& iWin = *IDMMotif::controlWindow( wid );
     IEventResult result;
     IDMMotif::sendDragLeave( iWin, dragInfoWrapper, result );
    }
  }
 if ( value ) XtFree((char*)value);
}

static void idropTransferRequestorCallbackProc(
        Widget          wid,
        XtPointer       client_data,
        Atom*           selection,
        Atom*           type,
        XtPointer       value,
        unsigned long*  length,
        int*            format )
{
 IMODTRACE_DEVELOP("idropTransferRequestorCallbackProc");
 //
 // Here is a rule:  Once you have a traget operation, you have to
 // eventually get rid of it.  There are only two ways to get rid of it:
 // send a targetDrop or send a targetLeave.
 //
 //ITRACE_DEVELOP("value = " + IString((unsigned long) value)) ;
 //ITRACE_DEVELOP("length = " + IString(*length)) ;
 if ( value && *length )
  {
   ITRACE_DEVELOP("value = " + IString((void *) value, *length)) ;
   Display* dpy = XtDisplay( wid );
   IString returnType = XmGetAtomName(dpy, *type) ;
   IString selectAtom = XmGetAtomName(dpy, *selection) ;
   ITRACE_DEVELOP("selection = " + selectAtom) ;
   ITRACE_DEVELOP("type      = " + returnType) ;
   XmDropTransferEntryRec transferEntries[1];

   Atom targetType(0) ;
   Atom transferType(0) ;
   dropRequestorCallbackInfo* cbInfo = (dropRequestorCallbackInfo*)client_data;
   if (cbInfo)
      targetType = cbInfo->targetType() ;
   {
   XmDropTransferEntryRec *pTransferEntries ;
   Arg args[3] ;
   int n(0) ;
   Cardinal numTransferEntries ;
   XtSetArg(args[n], XmNdropTransfers, &pTransferEntries) ; n ++ ;
   XtSetArg(args[n], XmNnumDropTransfers, &numTransferEntries) ; n++ ;
   XtGetValues(wid, args, n) ;
   ITRACE_DEVELOP("numTransferEntries = " + IString(numTransferEntries)) ;
   for (int i=0; i<numTransferEntries; i++) {
      ITRACE_DEVELOP("TransferTarget = " + IString(XmGetAtomName(dpy, pTransferEntries[i].target))) ;
      transferType = pTransferEntries[i].target ;
   }
   }


   if ( *type == XmInternAtom( dpy, "ICLUI_DM_DRAGINFO", False) )
    {
     ITRACE_DEVELOP( IString("*type=") + IString("ICLUI_DM_DRAGINFO.") );
     dropRequestorCallbackInfo* pInfo = (dropRequestorCallbackInfo*)client_data;
     dragDropRequestorCallbackInfo* pMyInfo = pInfo->clone();
     IWindow& iWin = pInfo->dropSiteWindow();
     unsigned char motifOperation = pInfo->motifOperation();
     unsigned char dropAction = pInfo->dropAction();
     IPoint motifPoint = pInfo->motifPoint();
     //
     // Get the dragInfo, and send a dragOver event.
     // This call frees value and deletes pInfo.
     //
     idragInfoRequestorCallbackProc( wid, client_data, selection, type,
                                     value, length, format );
     value = 0; // Remember we have freed it.
     pInfo = 0; // Remember we have deleted it.
     IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;
     if ( pTgtOpH.valid() ) // We got the dragInfo
      {
       IDragInfoWrapper dragInfoWrapper( *pTgtOpH );
       //
       // Set the usOperation field of the dragInfo to the usDefaultOp
       // from the most recent dragOver event.
       //
       dragInfoWrapper.setDragOperation(
                             dragInfoWrapper.asDragInfoPtr()->dropOperation() );
       //
       // If the dragInfo says ok, and we are requesting
       // a drop, and some operation is specified, do the drop.
       //
       // XmDROP_NOOP indicates that the operation the user is attempting is
       // not valid for the drag context, or is not valid for the drop site.
       // In this case, treat a drop as if the user canceled the drag.
       //
       IDM::DropIndicator
       dropIndicator = dragInfoWrapper.asDragInfoPtr()->dropIndicator();
       if ( dropIndicator == IDM::ok
         && motifOperation != XmDROP_NOOP
         && dropAction == XmDROP )
        {
         ITRACE_DEVELOP( "Adding ICLUI_DM_TRANSFER to dropSite targets." );
         //
         // We are going to continue with the drop.  Add a transfer target
         // to get the item contents into the target operation items.
         //
         transferEntries[0].target
         = XmInternAtom( dpy, "ICLUI_DM_TRANSFER",False );
         transferEntries[0].client_data = (XtPointer) pMyInfo;
         XmDropTransferAdd( wid, transferEntries, 1 );
         return;
        }
       else // We sent a dragOver event, but have decided not to drop.
        {
         delete pMyInfo; pMyInfo = 0; // Free asap to avoid a memory leak.
         IEventResult result;
         //
         // If help is requested, send a help event.
         //
         if (dropAction == XmDROP_HELP )
          {
           //
           // Emulate PM behaviour:  send dropHelp events.
           // Then send dragLeave events.
           //
           IDMMotif::sendDropHelp( iWin, dragInfoWrapper, motifPoint, result );
          }
         //
         // We (or someone) have decided we cannot drop here.
         // Emulate PM behaviour:  send dragLeave events to indicate cancel.
         //
         IDMMotif::sendDragLeave( iWin, dragInfoWrapper, result );
        }
      }
     delete pMyInfo;
     //
     // We decided not to drop.
     //
     ITRACE_DEVELOP( "Canceling drop." );
     XtVaSetValues( wid,
                    XmNtransferStatus, XmTRANSFER_FAILURE,
                    XmNnumDropTransfers, 0,
                    NULL );
    }
   else
   if ( *type == XmInternAtom( dpy, "ICLUI_DM_TRANSFER", False) )
    {
     ITRACE_DEVELOP( IString("*type=") + IString("ICLUI_DM_TRANSFER.") );
     dropRequestorCallbackInfo* pInfo = (dropRequestorCallbackInfo*)client_data;
     //
     // Obtain the window for this widget.
     //
     IWindow& iWin = pInfo->dropSiteWindow();
     //
     // Obtain the drop point in motif coordinates.
     //
     IPoint motifPoint = pInfo->motifPoint();
     //
     // Obtain the time of the drop.
     //
     Time dropTime = pInfo->timeStamp();
     delete pInfo; pInfo = 0; // Free asap to avoid memory leak if exception.
     //
     // If we do not have a target operation, there is nothing to do...
     //
     IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;
     if ( pTgtOpH.valid() )
      {
       //
       // Own the selection that provides access to the completion information.
       //
       int rc = XtOwnSelection( (Widget)iWin.handleForChildCreation(), 
                                XmInternAtom( dpy, "ICLUI_DM_RESULT", False),
                                dropTime,
                                idragCompletionConvertProc, NULL, NULL );
       ITRACE_DEVELOP( IString("XtOwnSelection ICLUI_DM_RESULT returned ") + IString(rc) );
       IEventResult result;
       IDragInfoWrapper dragInfoWrapper( *pTgtOpH );
       if ( !rc )
        {
         //
         // "This should never happen".
         // But it does. It is some kind of timing error.
         // Cancel the drop.  Make it look like a target leave.
         //
         ITRACE_DEVELOP( "Unable to own ICLUI_DM_RESULT selection." );
         //
         // Emulate PM behaviour:  send dragLeave events to indicate cancel.
         //
         IDMMotif::sendDragLeave( iWin, dragInfoWrapper, result );
         XtVaSetValues( wid,
                        XmNtransferStatus, XmTRANSFER_FAILURE,
                        XmNnumDropTransfers, 0,
                        NULL );
        }
       else
        {
         //
         // Allocate the array of completion information.
         // 2 for each item, plus one for usOperation for the dragInfo.
         //
         delete IDMMotif::targetCompletions();
         IDMMotif::targetCompletions()
         = new UsArray( 2*pTgtOpH->numberOfItems() + 1 );
         UsArray& completions = *IDMMotif::targetCompletions();
         completions[completions.length()-1]
         = (unsigned long) dragInfoWrapper.dragOperation();
         //
         // Update the target items with the transferred "object" and "contents".
         //
         IDMOperation *pTgtOp = pTgtOpH.getAlias() ;
         IDMOperationMarshaller opMarshaller( *pTgtOp );
         opMarshaller.unmarshall( (char*)value );
         XtFree((char*)value); // Free asap to avoid a memory leak if exception.
         value = 0; // Remember we have freed it.
         //
         // Emulate PM Behaviour:  send drop events.
         //
         IDMMotif::sendDrop( iWin, dragInfoWrapper, motifPoint, result );
         //
         // IDMMotif::targetCompletions() contains a pair of numbers for each
         // item processed.
         // The first number is the ulItemID, and the second is the
         // IDM::RenderCompletion completion status.
         //
         // The last element is usOperation for the dragInfo.
         //
         // Tell the source process the completion codes are ready.
         //
         transferEntries[0].target
         = XmInternAtom( dpy, "ICLUI_DM_READY", False);
         transferEntries[0].client_data = (XtPointer) 0;
         ITRACE_DEVELOP( "Adding ICLUI_DM_READY to dropSite targets." );
         XmDropTransferAdd( wid, transferEntries, 1 );

//@@       // The next step is to communicate the target completion codes
//@@       // back to the source.  We do this by adding adding targets
//@@       // to the dropTransfer.  Each target is either ICLUI_DM_SUCCESS
//@@       // or ICLUI_DM_FAILURE.  We start with the last completion code,
//@@       // since this is the order
//@@       // IDMTargetRenderer::informSourceOfCompletion adds them,
//@@       // and the order idragConvertProc expects them.
//@@       //
//@@       unsigned long index = IDMMotif::targetCompletions()->length()-1;
//@@       transferEntries[0].target
//@@       = (*IDMMotif::targetCompletions())[index] == IDM::targetSuccessful
//@@       ? XmInternAtom( dpy, "ICLUI_DM_SUCCESS", False)
//@@       : XmInternAtom( dpy, "ICLUI_DM_FAILURE", False);
//@@       transferEntries[0].client_data = (XtPointer) index;
//@@
//@@       ITRACE_DEVELOP( "Adding ICLUI_DM_SUCCESS or ICLUI_DM_FAILURE to dropSite targets." );
//@@       XmDropTransferAdd( wid, transferEntries, 1 );
        }
      }
     else
      {
       //
       // "This should never happen".  - Really.
       // We had a target operation just a while ago...
       //
       ITRACE_DEVELOP( "Target operation is missing." );
       XtVaSetValues( wid,
                      XmNtransferStatus, XmTRANSFER_FAILURE,
                      XmNnumDropTransfers, 0,
                      NULL );
      }
    }
   else
   if ( transferType == targetType) {
      ITRACE_DEVELOP( IString("*type=") + IString(transferType) );
      dropRequestorCallbackInfo* pInfo = (dropRequestorCallbackInfo*)client_data;
      IWindow& iWin = pInfo->dropSiteWindow();
      IPoint motifPoint = pInfo->motifPoint();
      Time dropTime = pInfo->timeStamp();
      delete pInfo; pInfo = 0; // Free asap to avoid memory leak if exception.
      IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;
      if ( pTgtOpH.valid() )
      {
        IEventResult result;
        IDragInfoWrapper dragInfoWrapper( *pTgtOpH );
        IDMItem::Handle pTgtDIH(pTgtOpH->item(1)) ;
        IString droppedBuffer = IString((void *) value, *length) ;
        pTgtDIH->setContents(droppedBuffer) ;
        XtFree((char*)value); // Free asap to avoid a memory leak if exception.
        value = 0; // Remember we have freed it.
        IDMMotif::sendDrop( iWin, dragInfoWrapper, motifPoint, result );
      }
      else
      {
        //
        // "This should never happen".  - Really.
        // We had a target operation just a while ago...
        //
        ITRACE_DEVELOP( "Target operation is missing." );
        XtVaSetValues( wid,
                       XmNtransferStatus, XmTRANSFER_FAILURE,
                       XmNnumDropTransfers, 0,
                       NULL );
      }
   }
   else
   if ( *type == XmInternAtom( dpy, "ICLUI_DM_READY", False)
     || *type == XmInternAtom( dpy, "ICLUI_DM_NULL", False) )
    {
     ITRACE_DEVELOP( IString("*type=") + IString("ICLUI_DM_READY or ICLUI_DM_NULL.") );
     //
     // If we still have the completion codes, add ICLUI_DM_NULL to the
     // dropSite targets.  Otherwise we are done, and can let the drop
     // complete.
     //
     // We loop in this way, waiting for the completion codes
     // to be sent, to avoid any complications which theoretically might occur
     // if the drop is complete, but we get another target enter (for a
     // different drag) before the completion codes have been sent.
     //
     // From the way Motif drop happens, it looks like this technique
     // is not preventing us from getting another drag and drop even while
     // this drop transfer is taking place.  But leave this here, to be
     // as well behaved as possible.  But we have to remember we still
     // can receive drag and drop events for a subsequent drag/drop even
     // while this drop transfer is in progress.....
     //
     if ( IDMMotif::targetCompletions() )
      {
       transferEntries[0].target
       = XmInternAtom( dpy, "ICLUI_DM_NULL", False);
       transferEntries[0].client_data = (XtPointer) 0;
       ITRACE_DEVELOP( "Adding ICLUI_DM_NULL to dropSite targets." );
       XmDropTransferAdd( wid, transferEntries, 1 );
      }
    }
//@@   else
//@@   if ( *type == XmInternAtom( dpy, "ICLUI_DM_SUCCESS", False)
//@@     || *type == XmInternAtom( dpy, "ICLUI_DM_FAILURE", False) )
//@@    {
//@@     ITRACE_DEVELOP( IString("*type=") + IString("ICLUI_DM_SUCCESS or ICLUI_DM_FAILURE.") );
//@@     unsigned long index = (unsigned long)client_data;
//@@     if ( index > 1 ) // There is more to be sent.
//@@      {
//@@       index = index - 2;
//@@       transferEntries[0].target
//@@       = (*IDMMotif::targetCompletions())[index] == IDM::targetSuccessful
//@@       ? XmInternAtom( dpy, "ICLUI_DM_SUCCESS", False)
//@@       : XmInternAtom( dpy, "ICLUI_DM_FAILURE", False);
//@@       transferEntries[0].client_data = (XtPointer) index;
//@@
//@@       ITRACE_DEVELOP( "Adding ICLUI_DM_SUCCESS or ICLUI_DM_FAILURE to dropSite targets." );
//@@       XmDropTransferAdd( wid, transferEntries, 1 );
//@@      }
//@@    }
  }
 else
  {
   //
   // It looks like we encountered a time out.
   // The best we can do is cancel.
   // The worst case is that the drop was done at the target,
   // but the completion codes were not processed at the source.
   //
   ITRACE_DEVELOP( "Timeout.  Abending drop." );
   XtVaSetValues( wid,
                  XmNtransferStatus, XmTRANSFER_FAILURE,
                  XmNnumDropTransfers, 0,
                  NULL );
   IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;
   if ( pTgtOpH.valid() )  // Not the worst case, the drop hasn't happened yet.
    {
     //
     // Emulate PM behaviour:  send dragLeave events to indicate cancel.
     //
     IWindow& iWin = *pTgtOpH->targetWindow();
     IEventResult result;
     IDragInfoWrapper dragInfoWrapper( *pTgtOpH );
     IDMMotif::sendDragLeave( iWin, dragInfoWrapper, result );
    }
  }
 if ( value ) XtFree((char*)value);
 IDMMotif::dropping = false;
}

/*------------------------------------------------------------------------------
|  idragProcCallback is the XmNdragProc for Motif drop sites.                  |
------------------------------------------------------------------------------*/
extern void _System idragProcCallback(
               Widget      wid,
               XtPointer   /* client_data */,
               XtPointer   call_data )
{
 IFUNCTRACE_DEVELOP();

 XmDragProcCallbackStruct* cbs = (XmDragProcCallbackStruct*) call_data;
 if ( IDMMotif::dropInProgress() )  // Previous drop not yet complete
  {
   ITRACE_DEVELOP("NO_DROP_SITE since drop in progress") ;
   cbs->dropSiteStatus = XmNO_DROP_SITE;
   return; // Ignore this drag until the previous drop completes.
  }

 if ( cbs->reason == XmCR_DROP_SITE_LEAVE_MESSAGE )
  {
   ++IDMMotif::numberOfDropSiteLeaves();
   //
   // Set the visual indication to not a drop site.
   // Otherwise, we can be stuck displaying a valid status over a
   // non-dropsite.
   //
   ITRACE_DEVELOP("NO_DROP_SITE in leave site processing") ;
   cbs->dropSiteStatus = XmNO_DROP_SITE;
  }

 IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;

 bool dragOverSent(false) ;
 Display *dpy = XtDisplay(wid) ;
 Atom icluiTarget = XmInternAtom(dpy, "ICLUI_DM_TRANSFER", False) ;

 if ( !pTgtOpH.valid() ) {   // We have no target operation.
   if ( cbs->reason != XmCR_DROP_SITE_LEAVE_MESSAGE ) { // We need a target op.
      bool bTargetMatch(false) ;
      Arg args[5] ;
      int n(0) ;
      Cardinal numExports ;
      Atom *pExports ;
      XtSetArg(args[n], XmNexportTargets, &pExports)      ; n++ ;
      XtSetArg(args[n], XmNnumExportTargets, &numExports) ; n++ ;
      XtGetValues(cbs->dragContext, args, n) ;
      int i(0) ;
      do {
         bTargetMatch = ( icluiTarget == pExports[i] ) ;
         i++ ;
      } while ( (!bTargetMatch) && (i<numExports) ) ;

      //
      // get the dragInfo.
      //
      if (!bTargetMatch) {
         // non ICLUI source.
         // create dragInfo from whatever we can gleam from the XmDragContext
         int numDragItems(1) ;
         IString types = IDMMotif::stringFromTargets(dpy, pExports, numExports) ;
         dragRequestorCallbackInfo pInfo(IDMMotif::numberOfDropSiteLeaves(), wid, *cbs) ;
         IDragInfo* pDragInfo = new IDragInfo(numDragItems) ;
         IDragInfoWrapper dragInfoWrapper (*pDragInfo) ;
         dragInfoWrapper.setDropPoint(pInfo.windowPoint( *IWindow::desktopWindow() ) );
         unsigned char cbsOperation = pInfo.motifOperation();
         dragInfoWrapper.setDragOperation(
            IDMMotif::motifOperation2DragOperation( pInfo.motifOperation(),
                                                    pInfo.motifOperations() ) );
         for (i=0; i< numDragItems; i++) {
            IDragItemWrapper diw(dragInfoWrapper[i] );
            diw.setTypes(types);
            diw.setDragSupOps(IDMItem::copyable | IDMItem::moveable);
            diw.setDragAttrs(IDMItem::none);
            diw.setItemID( i );
         } // for
      //
      // Dispatch appropriate dragOver events to this widget.
      // This creates the IDMTargetOperation.
      //
      IWindow& iWin = pInfo.dropSiteWindow();
      IPoint motifPoint = pInfo.motifPoint();
      IEventResult result;
      IDMMotif::sendDragOver( iWin, dragInfoWrapper, motifPoint, result );
      } // !bTargetMatch
      else {
         // Processing continues in idragInfoRequestorCallbackProc, either now
         // (for a local transfer within the same process), or after we return
         // (for a transfer between processes).
         dragRequestorCallbackInfo*
            pInfo = new dragRequestorCallbackInfo( IDMMotif::numberOfDropSiteLeaves(),
                                                   wid, *cbs ); // deleted in idragInfoRequestorCallbackProc
         XtGetSelectionValue( wid,
            XmInternAtom( dpy, "ICLUI_DM_SELECTION", False ),
            XmInternAtom( dpy, "ICLUI_DM_DRAGINFO", False ),
            idragInfoRequestorCallbackProc,
            pInfo,
            cbs->timeStamp );  // Next time we should have a target operation.
         ITRACE_DEVELOP("NO_DROP_SITE in 1st time through, about to get draginfo and selection") ;
         cbs->dropSiteStatus = XmNO_DROP_SITE;  // Until we find out otherwise.
         IDMMotif::dropSiteStatus() = cbs->dropSiteStatus;  // Remember for dropProc
      }
      // For a local transfer within the same process, we should have
      // a target operation now.  Otherwise we will some time in the future.
      pTgtOpH = IDMTargetOperation::targetOperation();
      dragOverSent = ( pTgtOpH != 0 );  // true for local case.
    } // ! XmCR_DROP_SITE_LEAVE_MESSAGE
  } // ! pTgtOph

 // handle case of a dragover from a non-ICLUI source.
 // check if our import targets intersect with the source's export targets
 // and if we support the operation. If everything works out,
 // let motif know that it's possible to drop here
 //if (!pTgtOpH && cbs->dragContext && cbs->reason != XmCR_DROP_SITE_LEAVE_MESSAGE ) {
 //   Arg args[5] ;
 //   int n(0) ;
 //   Cardinal numExports ;
 //   Atom *pExports ;
 //   XtSetArg(args[n], XmNexportTargets, &pExports)      ; n++ ;
 //   XtSetArg(args[n], XmNnumExportTargets, &numExports) ; n++ ;
 //   XtGetValues(cbs->dragContext, args, n) ;
 //   n = 0 ;
 //   Cardinal numImports ;
 //   Atom *pImports ;
 //   XtSetArg(args[n], XmNimportTargets, &pImports)      ; n++ ;
 //  XtSetArg(args[n], XmNnumImportTargets, &numImports) ; n++ ;
 //   XmDropSiteRetrieve(wid, args, n) ;
 //   bool bTypesMatch = XmTargetsAreCompatible(dpy,
 //                                             pExports, numExports,
 //                                             pImports, numImports) ;
    // TTD need to check operations too
 //   if (bTypesMatch)
 //      cbs->dropSiteStatus = XmDROP_SITE_VALID ;
 //}

 if ( pTgtOpH.valid() ) // We have a target operation.
  {
   IDragInfoWrapper dragInfoWrapper( *pTgtOpH );

   switch( cbs->reason )
    {
     case XmCR_DROP_SITE_LEAVE_MESSAGE:
      {
       //
       // Send appropriate dragLeave events, but only if we are still dragging.
       //
       // By the way, the debugger shows cbs->x, cbs->y, cbs->operation,
       // and cbs->operations are all zero at this point.
       //
       Display* dpy = XtDisplay( wid );
       XtGetSelectionValue( wid,
         XmInternAtom( dpy, "ICLUI_DM_LEAVE", False ),
         XmInternAtom( dpy, "ICLUI_DM_ALLOW_LEAVE", False ),
         idragLeaveRequestorCallbackProc,
         dragInfoWrapper.asDragInfoPtr(),
         cbs->timeStamp );
       break;
      }
     case XmCR_DROP_SITE_ENTER_MESSAGE:
     case XmCR_DROP_SITE_MOTION_MESSAGE:
     // No such message...   case XmCR_OPERATION_CHANGED_MESSAGE:
     // The debugger shows this one instead...
     case XmCR_OPERATION_CHANGED:
      {
       //
       // Determine the (possibly new) drag operation.
       //
       unsigned short
//@@       dragOp = dragInfoWrapper.dragOperation();
//@@       if ( cbs->operation != XmDROP_NOOP )
       dragOp = IDMMotif::motifOperation2DragOperation( cbs->operation,
                                                        cbs->operations );
       //
       // Emulate PM behaviour:
       //   If our _DRAGINFO indicates IDM::operationNotOk, only send a
       //   drag over event if the operation has changed.
       //   If it indicates IDM::neverOk, do not send a drag over event.
       //
       // It is possible that cbs->dropSiteStatus indicates XmNO_DROP_SITE
       // while dropIndicator does not indicate IDM::neverOk, due to the fact
       // that in order to obtain the _DRAGINFO we arbitrarily set
       // cbs->dropSiteStatus to XmNO_DROP_SITE since we had to exit the
       // drag proc callback before we knew the result of the first drag over
       // event.
       //
//@@       // XmDROP_NOOP indicates that the operation the user is attempting is
//@@       // not valid for the drag context, or is not valid for the drop site.
//@@       // So treat this as if the operation is not ok.  i.e. don't send an event.
//@@       //
       IDM::DropIndicator
       dropInd = dragInfoWrapper.asDragInfoPtr()->dropIndicator();
       if ( dropInd != IDM::neverOk
         && !( dropInd == IDM::operationNotOk
            && dragOp == dragInfoWrapper.dragOperation() ) )
//@@         && cbs->operation != XmDROP_NOOP )
        {
         //
         // Update the dragInfo with the current operation.
         //
         dragInfoWrapper.setDragOperation( dragOp );
         //
         // Update the dragInfo with the drag position.
         //
         dragInfoWrapper.setDropPoint( IDMMotif::motif2Window(
                            wid, cbs->x, cbs->y, IWindow::desktopWindow()  ) );
         //
         // Dispatch a DM_DRAGOVER event to the window.
         //
         IWindow& iWin = *IDMMotif::controlWindow( wid ); 
         if ( !dragOverSent ) // Avoid sending the same dragOver twice.
          {
           IEventResult result;
           IDMMotif::sendDragOver( iWin, dragInfoWrapper,
                                   IPoint( cbs->x, cbs->y), result );
          }
         dropInd = dragInfoWrapper.asDragInfoPtr()->dropIndicator();
         if ( dropInd == IDM::ok )
          {
           //
           // Update the Motif drag/drop operation field.
           //
           unsigned short
           usDefaultOp = dragInfoWrapper.asDragInfoPtr()->dropOperation();

           if ( usDefaultOp == IDMOperation::move )
             cbs->operation = XmDROP_MOVE;
           else
           if ( usDefaultOp == IDMOperation::copy )
             cbs->operation = XmDROP_COPY;
           else
           if ( usDefaultOp == IDMOperation::link )
             cbs->operation = XmDROP_LINK;
           else
           if ( usDefaultOp >= IDMOperation::unknown )
             cbs->operation = XmDROP_MOVE + XmDROP_COPY + XmDROP_LINK + 1
                            + (usDefaultOp - IDMOperation::unknown);
          }
        }
       //
       // Update the Motif drag/drop status field.
       //
       ITRACE_DEVELOP("NO_DROP_SITE after return from processing dragover") ;
       if (dropInd == IDM::ok) 
          ITRACE_DEVELOP("DROP_SITE ok") ;
       else
          ITRACE_DEVELOP("DROP_SITE not ok") ;
       cbs->dropSiteStatus = IDMMotif::dropIndicator2DropSiteStatus( dropInd );
       IDMMotif::dropSiteStatus() = cbs->dropSiteStatus;  // Remember for dropProc
       break;
      }
    } // end switch
  }
} // end idragProcCallback

/*------------------------------------------------------------------------------
|  idropProcCallback is the XmNdropProc for Motif drop sites.                  |
------------------------------------------------------------------------------*/
extern void _System idropProcCallback(
               Widget      wid,
               XtPointer   client_data,
               XtPointer   call_data )
{
 IMODTRACE_DEVELOP("idropProcCallback");
 XmDropTransferEntryRec transferEntries[1];
 Arg dropTransferArgs[5] ;
 int numDropTransferArgs(0) ;

 Display* dpy = XtDisplay( wid );
 XmDropProcCallbackStruct* cbs = (XmDropProcCallbackStruct*) call_data;

 if ( IDMMotif::dropInProgress() )  // Previous drop not yet complete
  {
   //
   // Yes it is possible that the previous drop has not yet completed,
   // since Motif gives the mouse pointer back to the user while the
   // drop transfer is in progress.
   //
   // Since we only have one target operation, we must be careful not
   // to get confused by trying to handle two drops at the same time.
   // Cancel this drop.
   //
   XtSetArg( dropTransferArgs[numDropTransferArgs], XmNtransferStatus, XmTRANSFER_FAILURE );
   numDropTransferArgs++;
   XtSetArg( dropTransferArgs[numDropTransferArgs], XmNnumDropTransfers, 0 );
   numDropTransferArgs++;
   XmDropTransferStart( cbs->dragContext, dropTransferArgs, numDropTransferArgs );
   return;
  }

 // TTD if not using ICLUI import target, let Motif default handle it
 //if (cbs->dragContext) {
 //   Atom icluiTarget = XmInternAtom( dpy, "ICLUI_DM_TRANSFER", False);
 //   Arg args[10] ;
 //   int n(0) ;
 //   Cardinal numExports ;
 //   Atom *pExports ;
 //  XtSetArg(args[n], XmNexportTargets, &pExports) ; n++ ;
 //  XtSetArg(args[n], XmNnumExportTargets, &numExports) ; n++ ;
 //   XtGetValues(cbs->dragContext, args, n) ;
 //   bool bMatch(false) ;
 //   for (int i=0; i<numExports; i++) {
 //      bMatch = pExports[i] == icluiTarget ;
 //      if (bMatch) break ;
 //   }
 //   if (! bMatch) {
 //      (* IDMMotif::defaultDropProc) (wid, client_data, call_data) ;
 //     return ;
 //  }
 //}

 IDMMotif::dropping = true;

 IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;
 if ( ! pTgtOpH.valid() )   // We have no target operation.
  {
   if ( cbs->dropSiteStatus != XmDROP_SITE_VALID
     || IDMMotif::dropSiteStatus() != XmDROP_SITE_VALID )
    {
     // The visual indication is no drop site, so cancel.
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNtransferStatus, XmTRANSFER_FAILURE );
     numDropTransferArgs++;
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNnumDropTransfers, 0 );
     numDropTransferArgs++;
     IDMMotif::dropping = false;
    }
   else
    {
     //
     // If we got here we have not issued the dragEnter, but are dropping now.
     //
     // We will attempt the drop, starting by getting the dragInfo.
     // Further processing is done in idropTransferRequestorCallbackProc.
     //
     transferEntries[0].target
     = XmInternAtom( dpy, "ICLUI_DM_DRAGINFO", False );
     transferEntries[0].client_data = (XtPointer) new
       dropRequestorCallbackInfo( IDMMotif::numberOfDropSiteLeaves(),wid,*cbs );
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNdropTransfers, transferEntries );
     numDropTransferArgs++;
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNnumDropTransfers, 1 );
     numDropTransferArgs++;
     XtSetArg( dropTransferArgs[numDropTransferArgs],XmNtransferProc,idropTransferRequestorCallbackProc );
     numDropTransferArgs++;
    }
  }
 else // We have a target operation.
  {
   IWindow& iWin = *IDMMotif::controlWindow( wid ); 
   IDragInfoWrapper dragInfoWrapper( *pTgtOpH );
   //
   // Set the usOperation field of the dragInfo to the usDefaultOp
   // from the most recent dragOver event.
   //
   dragInfoWrapper.setDragOperation(
                             dragInfoWrapper.asDragInfoPtr()->dropOperation() );
   //
   // If the dragInfo says ok, and Motif says valid, and we are requesting
   // a drop, and some operation is specified, do the drop.
   //
   // XmDROP_NOOP indicates that the operation the user is attempting is
   // not valid for the drag context, or is not valid for the drop site.
   // In this case, treat a drop as if the user canceled the drag.
   //
   // IDMMotif::dropSiteStatus() is the dropSiteStatus from the most
   // recent dragProc.  It represents the status displayed to the user.
   // cbs->dropSiteStatus at this point seems to be valid even when
   // the user sees an invalid status.  So we need to check
   // IDMMotif::dropSiteStatus() as well.  Otherwise the user would be
   // surprised to see an invalid status but still have the drop performed.
   //
   IDM::DropIndicator
   dropIndicator = dragInfoWrapper.asDragInfoPtr()->dropIndicator();
   if ( dropIndicator == IDM::ok
     // && cbs->dropSiteStatus == XmDROP_SITE_VALID
     && IDMMotif::dropSiteStatus() == XmDROP_SITE_VALID
     && cbs->operation != XmDROP_NOOP
     && cbs->dropAction == XmDROP )
    {
     //
     // We are going to do a drop.  Before we send the drop events,
     // we need to get the item contents into the target operation items.
     // So the drop events are sent from idropTransferRequestorCallbackProc.
     //
     Atom icluiTarget = XmInternAtom( dpy, "ICLUI_DM_TRANSFER", False);
     Arg args[5] ;
     int n(0) ;
     bool bMatch(false) ;
     Cardinal numExports ;
     Atom *pExports ;
     transferEntries[0].client_data = (XtPointer) new
       dropRequestorCallbackInfo( IDMMotif::numberOfDropSiteLeaves(), wid, *cbs );

     XtSetArg(args[n], XmNexportTargets, &pExports) ; n++ ;
     XtSetArg(args[n], XmNnumExportTargets, &numExports) ; n++ ;
     XtGetValues(cbs->dragContext, args, n) ;
     for (int i=0; i<numExports; i++) {
        bMatch = pExports[i] == icluiTarget ;
        if (bMatch) break ;
     }
     if (bMatch)
        transferEntries[0].target = icluiTarget ;
     else {
        IString firstSupportedType = IDMMotif::firstSupportedType(dpy, pExports, numExports) ;
        transferEntries[0].target
           = XmInternAtom( dpy, firstSupportedType, False );
        ((dropRequestorCallbackInfo *)(transferEntries[0].client_data))->setTargetType(transferEntries[0].target) ;
     }
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNdropTransfers, transferEntries );
     numDropTransferArgs++;
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNnumDropTransfers, 1 );
     numDropTransferArgs++;
     XtSetArg( dropTransferArgs[numDropTransferArgs],XmNtransferProc,idropTransferRequestorCallbackProc );
     numDropTransferArgs++;
    }
   else
    {
     IEventResult result;
     //
     // If help is requested, send a help event.
     //
     if ( cbs->dropAction == XmDROP_HELP )
      {
       //
       // Emulate PM behaviour:  send dropHelp events.
       // Then send dragLeave events.
       //
       IDMMotif::sendDropHelp( iWin, dragInfoWrapper,
                               IPoint( cbs->x, cbs->y ), result );
      }
     //
     // We (or someone) have decided we cannot drop here.
     // Emulate PM behaviour:  send dragLeave events to indicate cancel.
     //
     IDMMotif::sendDragLeave( iWin, dragInfoWrapper, result );
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNtransferStatus, XmTRANSFER_FAILURE );
     numDropTransferArgs++;
     XtSetArg( dropTransferArgs[numDropTransferArgs], XmNnumDropTransfers, 0 );
     numDropTransferArgs++;
    }
  }

  IDMMotif::dropSiteStatus() = XmDROP_SITE_VALID; // Reset for next d/d event.
  XmDropTransferStart( cbs->dragContext, dropTransferArgs, numDropTransferArgs );
} // end idropProcCallback

/*------------------------------------------------------------------------------
| IDMMotif::firstSupportedType                                                 |
|                                                                              |
| return first target type supported by targetOp                               |
|                                                                              |
------------------------------------------------------------------------------*/
IString IDMMotif::firstSupportedType(Display* pDisplay, const Atom* pAtomID, const int numAtoms)
{
   return IString("FILE_NAME") ; // TODO : finish this one day
}

/*------------------------------------------------------------------------------
| IDMMotif::stringFromTargets                                                  |
|                                                                              |
| translates list of Atom IDs to their names                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IString IDMMotif::stringFromTargets(Display* pDisplay, const Atom* pAtomID, const int numAtoms)
{
   IString listOfAtomNames ;
   IString atomName ;

   for (int i=0; i<numAtoms; i++) {
      if (i>0) listOfAtomNames += "," ;
      atomName = XmGetAtomName(pDisplay, pAtomID[i]) ;
      if (atomName == "STRING") atomName = IDM::text ;
      else if (atomName == "FILE_NAME") atomName = IDM::file ;
      listOfAtomNames += atomName ;
   }
   return listOfAtomNames ;
} // stringFromTargets

/*------------------------------------------------------------------------------
| IDMMotif::controlWindow                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IWindow* IDMMotif::controlWindow (const Widget widget)
{
   IWindow* pWindow = IWindow::windowWithHandle((IWindowHandle) widget) ;
   Widget parentWidget = XtParent(widget) ;
   while ( ! pWindow && ! XmIsMainWindow(parentWidget) ) 
   {
      pWindow = IWindow::windowWithHandle(parentWidget) ;
      parentWidget = XtParent(parentWidget) ;
   }

   return pWindow ;
} // controlWindow

/*------------------------------------------------------------------------------
| IDMMotif::isContainerControl                                                 |
|                                                                              |
| determines if the given window is a container                                |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDMMotif::isContainerControl(const IWindow* pCtrlWin)
{
   bool bYesOrNo(false) ;

   if (pCtrlWin != 0)
      bYesOrNo = XmIsContainer((Widget) pCtrlWin->handleForChildCreation()) ; 

   return bYesOrNo ;
} // isContainerControl
// 
//if ( XmIsContainer( (Widget) pCtrlWin->handle() ))
//  return true;
//   else
//     {
//     if (XmIsScrolledWindow( (Widget ) pCtrlWin->handle()) )
//       {
//       Widget child;
//       XtVaGetValues( (Widget ) pCtrlWin->handle(),
//          XmNworkWindow, &child,
//          NULL );
//       if ( XmIsContainer( child ) )
//         return True;
//       }
//     }
//@@Rob.  According to Mannie, this scrolled window check is not necessary and wrong.
//@@Rob. Adapted from Jim's prototype.

/*------------------------------------------------------------------------------
| IDMMotif::isPreDefinedForDrag                                                |
|                                                                              |
| determines if Motif already supports DragDrop for the given control.         |
| here's the current list of widgets already setup for DragDrop                |
|                                                                              |
|     widget          Drag Site          Drop Site                             |
|                                                                              |
|     XmText          process-bdrag          y                                 |
|     XmTextField     process-bdrag          y                                 |
|     XmLabel         ProcessDrag            n                                 |
|     XmList          ListProcessDrag        n                                 |
|                                                                              |
| Drag site column indicates action routine to be called for startDrag         |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDMMotif::isPreDefinedForDrag(const Widget widget) {
   bool bYesOrNo(false) ;
   if (widget)
      bYesOrNo = XmIsText(widget) | XmIsTextField(widget) | XmIsList(widget) | XmIsLabel(widget) ;
   return bYesOrNo ;
}

/*------------------------------------------------------------------------------
| IDMMotif::isPreDefinedForDrop                                                |
|                                                                              |
| determines if Motif already supports Drop for the given control.             |
| here's the current list of widgets already setup for DragDrop                |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDMMotif::isPreDefinedForDrop(const Widget widget) {
   bool bYesOrNo(false) ;
   if (widget)
      bYesOrNo = XmIsText(widget) | XmIsTextField(widget)  ;
   return bYesOrNo ;
}

/*------------------------------------------------------------------------------
| IDMMotif::preDefinedDragRoutine                                              |
|                                                                              |
| return predefined Motif drag start routine for given control                 |
|                                                                              |
------------------------------------------------------------------------------*/
IString IDMMotif::preDefinedDragRoutine(const Widget widget) {
   if (widget) {
      if (XmIsText(widget) | XmIsTextField(widget))
         return IString("process-bdrag()") ;
      else if (XmIsList(widget))
         return IString("ListProcessDrag") ;
      else if (XmIsLabel(widget))
         return IString("ProcessDrag") ;
   }
   return IString() ;
}

IPoint IDMMotif::motif2Window( Widget fromWidget,
                                 Position fromX,
                                 Position fromY,
                                 const IWindow* pToWin)
{
 //
 // Convert a point from motif coordinates relative to a widget,
 // to PM window style coordinates, relative to the bottom left
 // corner of the specified IWindow.
 //
 if ( !pToWin )
  {
   // Provide a default for pToWin:  same as fromWidget.
   if ( fromWidget)
      pToWin = IDMMotif::controlWindow( fromWidget );
   else // A null widget indicates the desktop.
     pToWin = IWindow::desktopWindow();
  }
 //
 // Get a widget whose Motif origin is the top left corner of the IWindow.
 //
 Widget toWidget = pToWin->handle();
 // if ( ! toWidget ) toWidget = pToWin->handle();  // Be defensive.
 //
 // Get the Motif origins of the two widgets.
 //
 Position fromWidgetX=0, fromWidgetY=0, toWidgetX=0, toWidgetY=0;
 if ( fromWidget != toWidget )
  {
   if ( fromWidget )
     XtTranslateCoords( fromWidget, 0, 0, &fromWidgetX, &fromWidgetY );
   if ( toWidget )
     XtTranslateCoords( toWidget, 0, 0, &toWidgetX, &toWidgetY );
  }
 //
 // fromX and fromY have the Motif mouse coordinates wrt fromWidget.
 // Adjust them to toWidget coordinates.  Then convert to IWindow coordinates.
 //

 if (ICoordinateSystem::applicationOrientation() == ICoordinateSystem::originUpperLeft)
    return IPoint(fromX + fromWidgetX - toWidgetX,
                  fromY + fromWidgetY - toWidgetY) ;
 else
    return IPoint( fromX + fromWidgetX - toWidgetX,
                   pToWin->size().height()
                      - ( fromY + fromWidgetY - toWidgetY ) );
}

bool IDMMotif::dropping = false;

unsigned char IDMMotif::dsStatus = XmDROP_SITE_VALID;

unsigned long IDMMotif::numDropSiteLeaves = 0;

UsArray* IDMMotif::targetCompletionArrayPtr = 0;

XtTranslations IDMMotif::parsedDragTranslations = 0;
IString lastTranslationString("") ;

void IDMMotif::enableDragFrom( Widget widget )
{
   IMODTRACE_DEVELOP("IDMMotif::enableDragFrom");

   Widget dragWidget = widget ;
   IWindow *pWindow = IDMMotif::controlWindow(widget) ;
      dragWidget = pWindow->handleForChildCreation() ;

   XtActionsRec dragActions[] = {{"IDMStartDrag", (XtActionProc)idragStart} };
   XtAppContext appContext = XtWidgetToApplicationContext( dragWidget ); 
   // XtAppAddActions( IThread::current().appContext(), dragActions,
   XtAppAddActions( appContext, dragActions,
                    XtNumber(dragActions));

   IString dragTranslations("#override <Btn2Down>: " +
                            IDMMotif::preDefinedDragRoutine(dragWidget) +
                            " IDMStartDrag()") ;
   if (dragTranslations != lastTranslationString)
   {
      parsedDragTranslations = XtParseTranslationTable( dragTranslations );
      lastTranslationString = dragTranslations ;
   }

   XtOverrideTranslations( dragWidget, parsedDragTranslations );
}
// 
// if ( !parsedDragTranslations ) // Initialize only the first time through.
//  {
//   XtActionsRec dragActions[] = {{"StartDrag", (XtActionProc)idragStart} };
//   XtAppAddActions( IThread::current().appContext(), dragActions,
//                    XtNumber(dragActions));
//
//   char dragTranslations[] = { "#override <Btn2Down>: StartDrag()" };
//   parsedDragTranslations = XtParseTranslationTable( dragTranslations );
// }
// XtOverrideTranslations( widget, parsedDragTranslations );

void IDMMotif::enableDropOn( Widget widget )
{
 IMODTRACE_DEVELOP("IDMMotif::enableDropOn");

 int numRegisterArgs(0) ;
 Arg registerArgs[10];

 Display* dpy = XtDisplay( widget );
 Atom IDMImportTarget = XmInternAtom( dpy, "ICLUI_DM_TRANSFER", False);

 // Allow all possible operations, even "unknown" ones.
 // That way we can distinguish the "default" case of no keys pressed
 // when operation != operations.  See IDMMotif::motifOperation2DragOperation
 // comments.
 XtSetArg( registerArgs[numRegisterArgs],
           XmNdropSiteOperations,
           (unsigned char)~0);
 numRegisterArgs++;

 // Motif animation does not work.  Motion events do not cause target
 // emphasis changes when the drop site status changes.  You can see this
 // in HandleMotion Motif source code.  It sets the callback reason to
 // XmCR_DROP_SITE_ENTER or XmCR_DROP_SITE_LEAVE, but it should be
 // XmCR_DROP_SITE_ENTER_MESSAGE or XmCR_DROP_SITE_LEAVE_MESSAGE.
 // I have experienced abends in AnimateLeave, which I suspect may be due
 // to this bug, causing Motif to get out of synch with the current animation.
 //
 // So, turn off Motif animation.  The item provider can arrange for
 // appropriate target emphasis, using provideEnterSupport/provideLeaveSupport
 // methods.
 //
 bool dbg_no_animate=1;
 if ( dbg_no_animate )
   XtSetArg( registerArgs[numRegisterArgs], XmNanimationStyle, XmDRAG_UNDER_NONE ); numRegisterArgs++;

 // point to our dragover and drop routines
 XtSetArg( registerArgs[numRegisterArgs], XmNdragProc, idragProcCallback ); numRegisterArgs++;
 XtSetArg( registerArgs[numRegisterArgs], XmNdropProc, idropProcCallback ); numRegisterArgs++;

 // set up the drop site object.
 // if the widget is already predefined for drop by Motif, it already has
 // an XmNDropSite so we should update the current one ( Motif gives us
 // an error if we try to reregister the drop site ...)
 if (IDMMotif::isPreDefinedForDrop(widget)) {
    Arg args[10] ;
    int n(0) ;
    int numImportTargets(0) ;
    Atom *currentImportTargets, *newImportTargets ;

    XtSetArg(args[n], XmNimportTargets, &currentImportTargets)   ; n++ ;
    XtSetArg(args[n], XmNnumImportTargets, &numImportTargets)    ; n++ ;
    XtSetArg(args[n], XmNdropProc, &(IDMMotif::defaultDropProc)) ; n++ ;
    XmDropSiteRetrieve(widget, args, n) ;

    newImportTargets = (Atom*) XtMalloc (sizeof(Atom) * (numImportTargets+1)) ;
    newImportTargets[0] = IDMImportTarget ;
    for (int i=0; i<numImportTargets; i++)
       newImportTargets[i+1] = currentImportTargets[i] ;

    XtSetArg( registerArgs[numRegisterArgs], XmNimportTargets, newImportTargets )      ;
    numRegisterArgs++;
    XtSetArg( registerArgs[numRegisterArgs], XmNnumImportTargets, numImportTargets+1 ) ;
    numRegisterArgs++;
    XmDropSiteUpdate(widget, registerArgs, numRegisterArgs) ;

    XtFree((char *) newImportTargets) ;
 }
 else {
    Atom importList[1];
    importList[0] = IDMImportTarget ;

    XtSetArg( registerArgs[numRegisterArgs], XmNimportTargets, importList ) ; numRegisterArgs++;
    XtSetArg( registerArgs[numRegisterArgs], XmNnumImportTargets, 1 )       ; numRegisterArgs++;

    XmDropSiteRegister( widget, registerArgs, numRegisterArgs );
 }

} // enableDropOn

XtTranslations IDMMotif::parsedDragContextTranslations = 0;

void IDMMotif::enableDragContext( Widget widget )
{
 IMODTRACE_DEVELOP("IDMMotif::enableDragContext");
 if ( !parsedDragContextTranslations )  // Initialize only the first time.
  {
   XtActionsRec dragActions[] = {{"ICLUIFinishDrag", (XtActionProc)idragFinish}};
   XtAppAddActions( IThread::current().appContext(), dragActions,
                    XtNumber(dragActions));

   char dragTranslations[] = { "#override <Btn2Up>: ICLUIFinishDrag()" };
   parsedDragContextTranslations = XtParseTranslationTable( dragTranslations );
  }
 XtOverrideTranslations( widget, parsedDragContextTranslations );
}

unsigned char
IDMMotif::dropIndicator2DropSiteStatus(const IDM::DropIndicator& aDropIndicator)
{
 unsigned char result=XmDROP_SITE_INVALID;
 if ( aDropIndicator == IDM::ok )
   result = XmDROP_SITE_VALID;
 else
 if ( aDropIndicator == IDM::neverOk )
   result = XmNO_DROP_SITE;
 return result;
}

unsigned short
IDMMotif::motifOperation2DragOperation( unsigned char aMotifOperation,
                                        unsigned char motifOperations )
{
 IMODTRACE_DEVELOP("motifOperation2DragOperation");
 //
 // When no key is held, Motif sets operations to all the allowable operations,
 // and sets operation to one of the allowable operations.
 //
 // When an explicit operation key is held, Motif sets operations
 // the same as operation.  If the keys held indicate an operation
 // that is not in the set of allowable operations, then both operation
 // and operations are set to XmDROP_NOOP (zero).
 //
 // Motif only defines operations for move, copy, and link.  We always
 // set the allowable operations to include other operations.
 // So if operation is not the same as operations, it means no key is held,
 // which we treat as "default", meaning the target can choose the operation.
 //
 // If operation is XmDROP_NOOP, Motif has decided the specified operation
 // is not valid.  We don't know what the specified operation is.
 // We treat it as the "unknown" case.
 //
 unsigned char adjustedMotifOperation
 = aMotifOperation == XmDROP_NOOP
 ? XmDROP_MOVE + XmDROP_COPY + XmDROP_LINK + 1
 : aMotifOperation;

 unsigned short dragOp = IDMOperation::unknown;
 if ( aMotifOperation != motifOperations ) // We have a choice of operations.
   dragOp = IDMOperation::drag;  // Let the target decide the operation.
 else
 if ( aMotifOperation == XmDROP_MOVE )
   dragOp = IDMOperation::move;
 else
 if ( aMotifOperation == XmDROP_COPY )
   dragOp = IDMOperation::copy;
 else
 if ( aMotifOperation == XmDROP_LINK )
   dragOp = IDMOperation::link;
 else
 if ( adjustedMotifOperation > XmDROP_MOVE + XmDROP_COPY + XmDROP_LINK )
   dragOp = IDMOperation::unknown
     + (adjustedMotifOperation - (XmDROP_MOVE + XmDROP_COPY + XmDROP_LINK + 1));
 ITRACE_DEVELOP( IString( "Converted (" ) + IString( (int)aMotifOperation ) + "," + IString( (int)motifOperations ) + ") to " + IString( dragOp ) + "." );
 return dragOp;
}

void IDMMotif::cleanupSourceOperation()
{
 //
 // Clean up the source operation.
 //
 IDMSourceOperation::Handle pSrcOpH(IDMSourceOperation::sourceOperation()) ;
 if ( pSrcOpH.valid() )
  {
   unsigned long nItems = pSrcOpH->numberOfItems();
   for ( unsigned long i = nItems; i; i-- )
     pSrcOpH->removeItem(i);
   IDMSourceOperation::setSourceOperation(0);
  }
}

bool IDMMotif::sendDragEvent( IWindow& iWin,
                             IDMEvent& iEvt,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendDragEvent");
 //
 // Dispatch the IDMEvent to the window.
 //
 bool bHandled = iWin.dispatch( iEvt );
 result = iEvt.result();
 return bHandled;
}

bool IDMMotif::sendCnrDragEvent( IContainerControl& iCnrWin,
                             IControlEvent& iCtlEvt,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendCnrDragEvent");
 //
 // Send the IControlEvent
 //
 // Emulate OS/2 ICLUI dispatch behaviour:
 // Dispatch to the container control window.
 // If the container control window does not handle it, then
 // dispatch again to the owner of the container control window.
 //
 bool bHandled = iCnrWin.dispatch( iCtlEvt );
 if ( !bHandled )
  {
   IWindow* pwinOwner = iCnrWin.owner();
   if ( pwinOwner ) bHandled = pwinOwner->dispatch( iCtlEvt );
  }
 if ( bHandled ) result = iCtlEvt.result();
 return bHandled;
}

bool IDMMotif::sendBeginDrag( IWindow& iWin,
                             const XEvent& event,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendBeginDrag");
 //
 // Dispatch a WM_BEGINDRAG to the widget
 //
 Widget wid ;
 //if (IDMMotif::isContainerControl(&iWin)) 
    wid = iWin.handleForChildCreation(); 
 //else
 //   wid = iWin.handle() ;
 IPoint winCoord(
     IDMMotif::motif2Window( wid, event.xbutton.x, event.xbutton.y, &iWin )) ;
 IDMEvent iEvt ( IEvent(&iWin,
                        WM_BEGINDRAG,
                        IEventParameter1((unsigned short)winCoord.x(), (unsigned short) winCoord.y() ),
                        IEventParameter2(0)
                       )
               ) ;

 bool bHandled = sendDragEvent( iWin, iEvt, result );

 return bHandled;
} // sendBeginDrag
// 
// if ( IDMMotif::isContainerControl(&iWin) )
//  {
//   IContainerControl& iCnrWin = (IContainerControl&)iWin;
   //
   // Emulate PM Container control behaviour:
   // Dispatch a CN_INITDRAG control notification to
   // the container control window.
   //
//   unsigned short ctlId = IControlEvent( iEvt ).controlId();
//   unsigned short ctlNotification = CN_INITDRAG;
//
//   _CNRDRAGINIT cnrDragInit;
//   cnrDragInit.hwndCnr = iCnrWin.handle();
//
//   IPoint desktopCoord( IDMMotif::motif2Window( cnrDragInit.hwndCnr,
//                                                event.xbutton.x,
//                                                event.xbutton.y,
//                                                IWindow::desktopWindow() ) );
//   cnrDragInit.pRecord = 0;
//   cnrDragInit.x = desktopCoord.x();
//   cnrDragInit.y = desktopCoord.y();
//   cnrDragInit.cx = 0;
//   cnrDragInit.cy = 0;
//
//   IContainerObject* pCnrObj = iCnrWin.objectUnderPoint( winCoord );
//   if ( pCnrObj ) // The pointer is over an object.
//    {
//     cnrDragInit.pRecord = IRecFromObj( pCnrObj );
//     IRectangle objRect;
//     if ( iCnrWin.isIconView() )
//       objRect = iCnrWin.iconRectangle( pCnrObj );
//     else
//     if ( iCnrWin.isTextView() )
//       objRect = iCnrWin.textRectangle( pCnrObj );
//     else
//     if ( iCnrWin.isDetailsView() )
//       objRect = iCnrWin.detailsObjectRectangle( pCnrObj );
//     cnrDragInit.cx = winCoord.x() - objRect.left();
//     cnrDragInit.cy = winCoord.y() - objRect.bottom();
//    }
//
//   IControlEvent iCtlEvt( IEvent( &iCnrWin, WM_CONTROL,
//                              IEventParameter1( ctlId, ctlNotification ),
//                              IEventParameter2( &cnrDragInit ) ) );
//   bHandled = sendCnrDragEvent( iCnrWin, iCtlEvt, result );
//  }

bool IDMMotif::sendDragOver( IWindow& iWin,
                             IDragInfoWrapper& dragInfoWrapper,
                             const IPoint& motifPoint,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendDragOver");
 //
 // Dispatch a DM_DRAGOVER event to the window.
 //
 IDMEvent iEvt( IEvent( &iWin, DM_DRAGOVER,
                IEventParameter1( (short)dragInfoWrapper.dropPoint().x(),
                                  (short)dragInfoWrapper.dropPoint().y()),
                IEventParameter2( dragInfoWrapper.asDragInfoPtr() ) ) ) ;

 bool bHandled = sendDragEvent( iWin, iEvt, result );
 if ( IDMMotif::isContainerControl(&iWin) )
  {
   IContainerControl& iCnrWin = (IContainerControl&)iWin;
   //
   // Emulate PM Container control behaviour:
   // Dispatch a CN_DRAGAFTER or CN_DRAGOVER control notification to
   // the container control window.
   //
   unsigned short ctlId = IControlEvent( iEvt ).controlId();

   unsigned short
   ctlNotification = ( iCnrWin.isIconView() || iCnrWin.isTreeView()
                    || iCnrWin.hasNormalTargetEmphasis() )
                   ? CN_DRAGOVER : CN_DRAGAFTER;

   _CNRDRAGINFO cnrDragInfo;
   cnrDragInfo.pDragInfo = dragInfoWrapper.asDragInfoPtr();
   cnrDragInfo.pRecord = 0;

   Widget wid = iWin.handleForChildCreation();
   IPoint winCoord
   = IDMMotif::motif2Window( wid, motifPoint.x(), motifPoint.y(), &iWin );

   IContainerObject* pCnrObj = iCnrWin.objectUnderPoint( winCoord );
   if ( pCnrObj )
     cnrDragInfo.pRecord = IRecFromObj( pCnrObj );
   //@@Rob.  We will never have CN_DRAGAFTER,
   //@@Rob.  for now, since ordered target emphasis is not supported.
   //@@Rob.  When we do, we need to point to the minirecordcore for the
   //@@Rob.  container object the highlight line is under, or CMA_FIRST
   //@@Rob.  if over first ordered item, or NULL if pointer over white space.

   IControlEvent iCtlEvt( IEvent( &iCnrWin, WM_CONTROL,
                              IEventParameter1( ctlId, ctlNotification ),
                              IEventParameter2( &cnrDragInfo ) ) );
   bHandled = sendCnrDragEvent( iCnrWin, iCtlEvt, result );
  }
 if ( bHandled )
  {
   // Save this away.  We will look at it the next event we get.
   dragInfoWrapper.asDragInfoPtr()->setDragResult( result );
  }
 return bHandled;
}

bool IDMMotif::sendDragLeave( IWindow& iWin,
                             const IDragInfoWrapper& dragInfoWrapper,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendDragLeave");
 //
 // Dispatch a DM_DRAGLEAVE event to the window.
 //
 IDMEvent iEvt( IEvent( &iWin, DM_DRAGLEAVE,
              IEventParameter1( dragInfoWrapper.asDragInfoPtr() ),
              IEventParameter2( 0 ) ) );
 bool bHandled = sendDragEvent ( iWin, iEvt, result );
 // Widget wid = iWin.handle();
 if ( IDMMotif::isContainerControl(&iWin) )
  {
   IContainerControl& iCnrWin = (IContainerControl&)iWin;
   //
   // Emulate PM Container control behaviour:
   // Dispatch a CN_DRAGLEAVE control notification to
   // the container control window.
   //
   unsigned short ctlId = IControlEvent( iEvt ).controlId();

   unsigned short ctlNotification = CN_DRAGLEAVE;

   _CNRDRAGINFO cnrDragInfo;
   cnrDragInfo.pDragInfo = dragInfoWrapper.asDragInfoPtr();
   cnrDragInfo.pRecord = 0;

   IControlEvent iCtlEvt( IEvent( &iCnrWin, WM_CONTROL,
                              IEventParameter1( ctlId, ctlNotification ),
                              IEventParameter2( &cnrDragInfo ) ) );

   bHandled = sendCnrDragEvent ( iCnrWin, iCtlEvt, result);
  }
 //
 // Make sure we forget about the target operation.  We will create a new one
 // if we enter this drop site again.
 //
 IDMTargetOperation::setTargetOperation(0);
 IDMMotif::dropping = false;
//@@ //
//@@ // Get rid of the completion information and disown the selection for it.
//@@ //
//@@ Display* dpy = XtDisplay( wid );
//@@ Atom dm_result = XmInternAtom( dpy, "ICLUI_DM_RESULT", False );
//@@ XtDisownSelection(wid, dm_result, CurrentTime);
//@@ delete IDMMotif::targetCompletions();
//@@ IDMMotif::targetCompletions() = 0;
 return bHandled;
}

bool IDMMotif::sendDrop( IWindow& iWin,
                             const IDragInfoWrapper& dragInfoWrapper,
                             const IPoint& motifPoint,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendDrop");
 //
 // Dispatch a DM_DROP event to the window.
 //
 IDMEvent iEvt( IEvent( &iWin, DM_DROP,
              IEventParameter1( dragInfoWrapper.asDragInfoPtr() ),
              IEventParameter2( 0 ) ) );
 bool bHandled = sendDragEvent( iWin, iEvt, result );
 if ( IDMMotif::isContainerControl(&iWin ) )
  {
   IContainerControl& iCnrWin = (IContainerControl&)iWin;
   //
   // Emulate PM Container control behaviour:
   // Dispatch a CN_DROP control notification to
   // the container control window.
   //
   unsigned short ctlId = IControlEvent( iEvt ).controlId();
   unsigned short ctlNotification = CN_DROP;

   _CNRDRAGINFO cnrDragInfo;
   cnrDragInfo.pDragInfo = dragInfoWrapper.asDragInfoPtr();
   cnrDragInfo.pRecord = 0;

   Widget wid = iWin.handleForChildCreation(); 
   IPoint winCoord
   = IDMMotif::motif2Window( wid, motifPoint.x(), motifPoint.y(), &iWin );

   IContainerObject* pCnrObj = iCnrWin.objectUnderPoint( winCoord );
   if ( pCnrObj )
     cnrDragInfo.pRecord = IRecFromObj( pCnrObj );

   IControlEvent iCtlEvt( IEvent( &iCnrWin, WM_CONTROL,
                              IEventParameter1( ctlId, ctlNotification ),
                              IEventParameter2( &cnrDragInfo ) ) );
   bHandled = sendCnrDragEvent( iCnrWin, iCtlEvt, result );
  }
 //
 // Make sure we forget about the target operation.
 //
 IDMTargetOperation::setTargetOperation(0);
 IDMMotif::dropping = false;
 return bHandled;
}

bool IDMMotif::sendDropHelp( IWindow& iWin,
                             const IDragInfoWrapper& dragInfoWrapper,
                             const IPoint& motifPoint,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendDropHelp");
 //
 // Dispatch a DM_DROPHELP event to the window.
 //
 IDMEvent iEvt( IEvent( &iWin, DM_DROPHELP,
              IEventParameter1( dragInfoWrapper.asDragInfoPtr() ),
              IEventParameter2( 0 ) ) );
 bool bHandled = sendDragEvent( iWin, iEvt, result );
 if ( IDMMotif::isContainerControl(&iWin) )
  {
   IContainerControl& iCnrWin = (IContainerControl&)iWin;
   //
   // Emulate PM Container control behaviour:
   // Dispatch a CN_DROPHELP control notification to
   // the container control window.
   //
   unsigned short ctlId = IControlEvent( iEvt ).controlId();
   unsigned short ctlNotification = CN_DROPHELP;

   _CNRDRAGINFO cnrDragInfo;
   cnrDragInfo.pDragInfo = dragInfoWrapper.asDragInfoPtr();
    cnrDragInfo.pRecord = 0;

   Widget wid = iWin.handleForChildCreation(); 
   IPoint winCoord
   = IDMMotif::motif2Window( wid, motifPoint.x(), motifPoint.y(), &iWin );

   IContainerObject* pCnrObj = iCnrWin.objectUnderPoint( winCoord );
   if ( pCnrObj )
     cnrDragInfo.pRecord = IRecFromObj( pCnrObj );

   IControlEvent iCtlEvt( IEvent( &iCnrWin, WM_CONTROL,
                              IEventParameter1( ctlId, ctlNotification ),
                              IEventParameter2( &cnrDragInfo ) ) );
   bHandled = sendCnrDragEvent( iCnrWin, iCtlEvt, result );
  }
 return bHandled;
}

bool IDMMotif::sendEndConversation( IWindow& iWin,
                             unsigned long ulItemID,
                             unsigned long completionFlags,
                             IEventResult& result )
{
 IMODTRACE_DEVELOP("IDMMotif::sendEndConversation");
 //
 // Dispatch a DM_ENDCONVERSATION event to the window.
 //
 IDMEvent iEvt( IEvent( &iWin, DM_ENDCONVERSATION,
              IEventParameter1( ulItemID ),
              IEventParameter2( completionFlags ) ) );
 bool bHandled = sendDragEvent( iWin, iEvt, result );
 return bHandled;
}
#endif
