// Revision: 84 1.8.1.3 source/ui/drag/idmmotf.hpp, dragdrop, ioc.v400, 000919 
/* NOSHIP */
#ifndef _IDMMOTF_
#define _IDMMOTF_
/*******************************************************************************
* FILE NAME: idmmotf.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IDMMotif                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <imarsha.hpp>
#include <ievtdata.hpp>
#include <idmcomm.hpp>
#ifdef IC_MOTIF
extern "C"
{
  #include <Xm/Xm.h>
}
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

class UsArray : public IDMMarshall
 {
  public:
/*------------------------------- Constructors -------------------------------*/
   UsArray( unsigned long numElems );
   virtual ~UsArray();

   unsigned long length();  // Number of elements in the array.
   unsigned long& operator[]( unsigned long index );

   virtual unsigned long
     marshall              ( unsigned long startPos=0, char* outStr=0 ) const;

   virtual const char
    *unmarshall            ( const char* inStr );

  private:
/*--------------------------------- Private ----------------------------------*/
   // Dissallow copy and assignment.
   UsArray( const UsArray& );
   UsArray& operator=(const UsArray& );

   unsigned long  numInArray;  // Number of elements in the array.
   unsigned long* pArray;  // Pointer to array of unsigned long.
 };

#ifdef IC_MOTIF
class IPoint;
class IWindowHandle;
class IWindow;
class IDMEvent;
class IContainerControl;
class IControlEvent;
class IDragInfoWrapper;

typedef Boolean
           (_System IDMDefaultConvertProc)(Widget,
                                   Atom*,
                                   Atom*,
                                   Atom*,
                                   XtPointer*,
                                   unsigned long*,
                                   int* ) ;

typedef void
          (_System IDMDefaultDropProc) (Widget, XtPointer, XtPointer) ;

class IDMMotif
 {
  public:
   static IPoint motif2Window( Widget widget, Position x, Position y,
                                 const IWindow* pWin=0 );
   static bool isContainerControl(const IWindow* pCtrlWin) ;
   static IWindow* controlWindow(const Widget widget) ; 
   static bool isPreDefinedForDrag( const Widget widget );
   static bool isPreDefinedForDrop( const Widget widget );
   static IString preDefinedDragRoutine( const Widget widget );
   static void enableDragFrom( Widget widget );
   static void enableDropOn( Widget widget );
   static void enableDragContext ( Widget widget );
   static bool dropInProgress();
   static unsigned char& dropSiteStatus();
   static unsigned long& numberOfDropSiteLeaves();
   static UsArray*& targetCompletions();
   static unsigned char
       dropIndicator2DropSiteStatus( const IDM::DropIndicator& aDropIndicator );
   static unsigned short
       motifOperation2DragOperation( unsigned char aMotifOperation,
                                     unsigned char motifOperations );
   static IString
       stringFromTargets(Display* pDisplay,
                         const Atom* pAtomID,
                         const int numAtoms) ;
   static IString
       firstSupportedType(Display* pDisplay,
                         const Atom* pAtomID,
                         const int numAtoms) ;
   static void cleanupSourceOperation();
   static bool sendDragEvent( IWindow& iWin,
                             IDMEvent& iEvt,
                             IEventResult& result );
   static bool sendCnrDragEvent( IContainerControl& iCnrWin,
                             IControlEvent& iCtlEvt,
                             IEventResult& result );
   static bool sendBeginDrag( IWindow& iWin,
                             const XEvent& event,
                             IEventResult& result );
   static bool sendDragOver( IWindow& iWin,
                                       IDragInfoWrapper& dragInfoWrapper,
                                       const IPoint& motifPoint,
                                       IEventResult& result );
   static bool sendDragLeave( IWindow& iWin,
                                        const IDragInfoWrapper& dragInfoWrapper,
                                        IEventResult& result );
   static bool sendDrop( IWindow& iWin,
                                   const IDragInfoWrapper& dragInfoWrapper,
                                   const IPoint& motifPoint,
                                   IEventResult& result );
   static bool sendDropHelp( IWindow& iWin,
                                       const IDragInfoWrapper& dragInfoWrapper,
                                       const IPoint& motifPoint,
                                       IEventResult& result );
   static bool sendEndConversation( IWindow& iWin,
                                              unsigned long ulItemID,
                                              unsigned long completionFlags,
                                              IEventResult& result );
   static bool dropping;
   static IDMDefaultConvertProc *defaultConvertProc ;
   static IDMDefaultDropProc *defaultDropProc ;
  private:
/*--------------------------------- Private ----------------------------------*/
   static XtTranslations parsedDragTranslations;
   static XtTranslations parsedDragContextTranslations;
   static UsArray* targetCompletionArrayPtr;
   static unsigned long numDropSiteLeaves;
   static unsigned char dsStatus; // Our latest known value of dropSiteStatus.
 };
#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <idmmotf.inl>

#endif // _IDMMOTF_
