// Revision: 96 1.13.1.4 source/ui/drag/idmrendr.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmrendr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMRenderer                                                              *
*       IDMSourceRenderer                                                      *
*       IDMTargetRenderer                                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif

// include PM headers
  #define INCL_WINSTDDRAG       // Standard drag stuff
  #include <iwindefs.h>
  #ifdef IC_WIN
    #include <shlobj.h>
    #include <wchar.h>
  #endif
}

#include <idmitem.hpp>
#include <idmrendr.hpp>
#include <idmsrcrn.hpp>
#include <idmtgtrn.hpp>
#include <idmoper.hpp>
#include <idmevent.hpp>
#include <ictlevt.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>
#include <icconst.h>

#ifdef IC_MOTIF
   #include <idmmotf.hpp>
#endif // IC_MOTIF

#ifdef IC_MOTIFWIN 
   #include <idmditm.hpp>
#endif // IC_MOTIFWIN

#ifdef IC_WIN
   #include <idmformt.hpp>
   #include <idmdrptg.hpp>
   #include <itemid.hpp>
   #include <iplatfrm.hpp>
   #include <idmdatob.hpp>
   #include <idmdnfo.hpp>
#endif // IC_WIN


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMRENDR_CPP_
  #include <ipagetun.h>
#endif

#pragma enum(4)
#pragma pack(push,4)

#ifdef IC_WIN
class IDMSourceRendererData
{
  public:
    IDMSourceRendererData()
    {;}

    virtual
      ~IDMSourceRendererData()
    {;}

    IDMSourceRendererData
     &renderData( IDMSourceRenderEvent &event ),
     &renderFileData( IDMSourceRenderEvent &event );
};
#endif


class IDMTargetRendererData
{
  public:
    IDMTargetRendererData()
#ifdef IC_PM
    : pCurrentDragTransfer (0),
      ulDragTransferCount (0)
#endif
    {;}

    virtual
      ~IDMTargetRendererData()
    {;}

#ifdef IC_WIN
    bool getTheData( IDMTargetDropEvent &event,
                     const IDMItem::Handle &pTgtDIH);

    IString firstSupportedType( const IDMItem::Handle &pTgtDIH );
#endif

#ifdef IC_PM
    unsigned long
      ulDragTransferCount;

    _DRAGTRANSFER
      *pCurrentDragTransfer;
#endif
};


typedef IDM::DropIndicator DropIndicator;

/*------------------------------------------------------------------------------
| Static objects to ensure default renderers get deleted at shutdown.          |
------------------------------------------------------------------------------*/
static struct IDMRenderer_Static {
static IDMSourceRenderer
 *pSrcRenderer;
static IDMTargetRenderer
 *pTgtRenderer;

  ~IDMRenderer_Static ( )
    {
    IMODTRACE_DEVELOP( "IDMRenderer_Static dtor" );
#if IC_STATIC_PRIORITY_SUPPORTED
    // Deletes are safe only when static init/term ordering is supported.
    delete pSrcRenderer;
    delete pTgtRenderer;
#endif
    }

} statics;

IDMSourceRenderer
 *IDMRenderer_Static::pSrcRenderer = 0;
IDMTargetRenderer
 *IDMRenderer_Static::pTgtRenderer = 0;

// IDMRenderer static members:
IDMSourceRenderer
 *IDMRenderer::pSource = 0;
IDMTargetRenderer
 *IDMRenderer::pTarget = 0;

#pragma enum(pop)
#pragma pack(pop)

/*-------------------------------- BASE RENDERER ----------------------------*/

/*------------------------------------------------------------------------------
| IDMRenderer::IDMRenderer                                                     |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMRenderer :: IDMRenderer(const char *pRMFs, const char *pTypes)
  : strRMFs( IDMItem::rmfsFrom( pRMFs ) ),
    strTypes(pTypes)
{
}

/*------------------------------------------------------------------------------
| IDMRenderer::IDMRenderer                                                     |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMRenderer :: IDMRenderer()
  : strTypes(0),
    strRMFs(0)
{
  setSupportedTypes(IDM::any);
}

/*------------------------------------------------------------------------------
| IDMRenderer::~IDMRenderer                                                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMRenderer :: ~IDMRenderer()
{
}

/*------------------------------------------------------------------------------
| IDMRenderer::supportedTypes                                                  |
|                                                                              |
| Returns the types of items supported by the renderer.                        |
------------------------------------------------------------------------------*/
IString IDMRenderer :: supportedTypes() const
{
  return(this->strTypes);
}

/*------------------------------------------------------------------------------
| IDMRenderer::supportedRMFs                                                   |
|                                                                              |
| Returns RMFs types of items supported by the renderer.                       |
------------------------------------------------------------------------------*/
IString IDMRenderer :: supportedRMFs() const
{
  return(this->strRMFs);
}

/*------------------------------------------------------------------------------
| IDMRenderer::setSupportedTypes                                               |
|                                                                              |
| Sets the types of items supported by the renderer.                           |
------------------------------------------------------------------------------*/
IDMRenderer& IDMRenderer :: setSupportedTypes(const char *pTypes)
{
  IMODTRACE_DEVELOP("IDMRenderer::setSupportedTypes");

  this->strTypes = pTypes;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMRenderer::setSupportedRMFs                                                |
|                                                                              |
| Sets the RMFs supported by this renderer.                                    |
------------------------------------------------------------------------------*/
IDMRenderer& IDMRenderer :: setSupportedRMFs(const char *pRMFs)
{
  IMODTRACE_DEVELOP("IDMRenderer::setSupportedRMFs");

  this->strRMFs = IDMItem::rmfsFrom( pRMFs );
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMRenderer::defaultSourceRenderer                                           |
|                                                                              |
| Return default source renderer, initializing if this is the first time.      |
------------------------------------------------------------------------------*/
IDMSourceRenderer &IDMRenderer :: defaultSourceRenderer ( )
  {
  if ( !IDMRenderer::pSource )
    {
    if ( !statics.pSrcRenderer )
      statics.pSrcRenderer = new IDMSourceRenderer;
    IDMRenderer::pSource = statics.pSrcRenderer;
    }
  return *IDMRenderer::pSource;
  }

/*------------------------------------------------------------------------------
| IDMRenderer::defaultTargetRenderer                                           |
|                                                                              |
| Return default target renderer, initializing if this is the first time.      |
------------------------------------------------------------------------------*/
IDMTargetRenderer &IDMRenderer :: defaultTargetRenderer ( )
  {
  if ( !IDMRenderer::pTarget )
    {
    if ( !statics.pTgtRenderer )
      statics.pTgtRenderer = new IDMTargetRenderer;
    IDMRenderer::pTarget = statics.pTgtRenderer;
    }
  return *IDMRenderer::pTarget;
  }

/*------------------------------------------------------------------------------
| IDMRenderer::setDefaultSourceRenderer                                        |
|                                                                              |
| Set default source renderer pointer to point to argument renderer.           |
------------------------------------------------------------------------------*/
void IDMRenderer :: setDefaultSourceRenderer ( IDMSourceRenderer &source )
  {
  IDMRenderer::pSource = &source;
  }

/*------------------------------------------------------------------------------
| IDMRenderer::setDefaultTargetRenderer                                        |
|                                                                              |
| Set default target renderer pointer to point to argument renderer.           |
------------------------------------------------------------------------------*/
void IDMRenderer :: setDefaultTargetRenderer ( IDMTargetRenderer &target )
  {
  IDMRenderer::pTarget = &target;
  }


/*-------------------------------- SOURCE RENDERER ---------------------------*/

/*------------------------------------------------------------------------------
| IDMSourceRenderer::IDMSourceRenderer                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMSourceRenderer :: IDMSourceRenderer(const char *pRMFs,
                                       const char *pTypes)
  : IDMRenderer(pRMFs, pTypes)
#ifdef IC_WIN
  , fDMSourceRendererData( new IDMSourceRendererData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceRenderer::ctor (rmfs, types)");
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::IDMSourceRenderer                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMSourceRenderer :: IDMSourceRenderer ()
  : IDMRenderer()
#ifdef IC_WIN
  , fDMSourceRendererData( new IDMSourceRendererData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceRenderer::ctor ()");

  setSupportedRMFs(IString("(")                 +
                   IDM::rmLibrary               +
                   ")x("                        +
                   IDMItem::rfForThisProcess()  +
                   ","                          +
                   IDM::rfText                  +
                   ","                          +
                   IDM::rfSharedMem             +
                   "),("                        +
                   IDM::rmDiscard               +
                   ","                          +
                   IDM::rmPrint                 +
                   ")x("                        +
                   IDM::rfUnknown               +
                   "),<"                        +
                   IDM::rmFile                  +
                   ","                          +
                   IDM::rfText                  +
                   ">,<"                        +
                   IDM::rmObject                +
                   ","                          +
                   IDM::rfObject                +
                   ">");
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::~IDMSourceRenderer                                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMSourceRenderer :: ~IDMSourceRenderer ()
{
  IMODTRACE_DEVELOP("IDMSourceRenderer::dtor ()");

#ifdef IC_WIN
  if (fDMSourceRendererData )
    delete fDMSourceRendererData;
#endif

}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::canRender                                                 |
|                                                                              |
| Returns true if the renderer can render items with the argument attribute.   |
| Used when the drag is initiated.                                             |
------------------------------------------------------------------------------*/
bool IDMSourceRenderer :: canRender(const IString &itemTypes)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::canRender(types)" );

  /**********************************************************************/
  /* Check if any type is supported - IDM::any allows any type ...      */
  /**********************************************************************/
  if (!(supportedTypes().indexOf(IDM::any)))
  {
    unsigned startPos = 1;
    unsigned count = itemTypes.occurrencesOf(',') + 1;

    for (unsigned i=0; i < count; i++)
    {
      unsigned index = itemTypes.indexOf(',', startPos);
      IString type = (index ? itemTypes.subString(startPos, index-startPos)  :
                                        itemTypes.subString(startPos));
      /******************************************************************/
      /* Verify the type ...                                            */
      /******************************************************************/
      if (supportedTypes().indexOf(type))
        return(true);

      startPos = index + 1;
    }

    /********************************************************************/
    /* Type not supported if we reach this point ...                    */
    /********************************************************************/
    return(false);
  }

  return(true);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::sourceRender                                              |
|                                                                              |
| Called when a target requests that a source renderer render an item.         |
------------------------------------------------------------------------------*/
IDMSourceRenderer& IDMSourceRenderer ::
                   sourceRender(IDMSourceRenderEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::sourceRender" );
#ifndef IC_MOTIF
#ifdef IC_PM
  /********************************************************************/
  /* Indicate source and target are not the same                      */
  /********************************************************************/
  event.dragItem()->bTargetTheSource = false;

  /********************************************************************/
  /* Check for our rendering mechanism first                          */
  /********************************************************************/
  if (event.selectedMechanism() == IDM::rmLibrary)
  {
    /******************************************************************/
    /* Check for shared memory format ...                             */
    /******************************************************************/
    if (event.selectedFormat().indexOf(IDM::rfSharedMem))
    {
      ITRACE_DEVELOP( "Using rmLibrary, rfSharedMem - Src" );
      /****************************************************************/
      /* Give source item opportunity to acquire data                 */
      /****************************************************************/
      event.dragItem()->sourceRender(event);

      /****************************************************************/
      /* Call default/derived renderer to make the data available     */
      /* for the target.                                              */
      /****************************************************************/
      doRender(event);

      /****************************************************************/
      /* Free the dragtransfer structure.                             */
      /****************************************************************/
      DrgFreeDragtransfer (event.dragTransfer());
    }
    else
    {
      /****************************************************************/
      /* Unknown rendering format ...                                 */
      /****************************************************************/
      ITRACE_DEVELOP( "Unknown format, rm=rmLibrary - Src" );
      ITHROWLIBRARYERROR(IC_UNKNOWN_RENDERING_FORMAT,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
    }
  }
  else
  {
    /******************************************************************/
    /* Check for file rendering mechanism (delayed file rendering)    */
    /* Do not check the format since it may be text or unknown ...    */
    /******************************************************************/
    if (event.selectedMechanism() == IDM::rmFile)
    {
      ITRACE_DEVELOP( "Using rmFile (delayed file rendering) - Src" );
      /****************************************************************/
      /* Give source item opportunity to acquire data                 */
      /****************************************************************/
      event.dragItem()->sourceRender( event );

      /****************************************************************/
      /* Let the target (probably the WPS) know we're finished ...    */
      /****************************************************************/
      informTargetOfCompletion( event );
    }
    else
    {
      /****************************************************************/
      /* Unknown rendering mechanism ...                              */
      /****************************************************************/
      ITHROWLIBRARYERROR(IC_UNKNOWN_RENDERING_MECHANISM,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
    }
  }
#endif //IC_PM
#ifdef IC_WIN
  // need to fix this but for now if we got here just render the
  // data
  fDMSourceRendererData->renderData(event);
#endif // IC_WIN

#endif // ~IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::sourcePrepare                                             |
|                                                                              |
| Called when an item indicates that it requires preparation.                  |
------------------------------------------------------------------------------*/
IDMSourceRenderer& IDMSourceRenderer ::
                   sourcePrepare(IDMSourcePrepareEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::sourceRenderPrepare" );

#ifndef IC_MOTIF

  /********************************************************************/
  /* Give source item opportunity to perform preparation.             */
  /********************************************************************/
  event.dragItem()->sourcePrepare(event);

  /********************************************************************/
  /* Call default/derived renderer to perform any required            */
  /* preparation.                                                     */
  /********************************************************************/
  doRenderPrepare(event);

#endif // ~IC_MOTIF
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::sourceEnd                                                 |
|                                                                              |
| Called when the target window informs the source window that it has finished |
| processing the data that was associated with the dropped item.               |
------------------------------------------------------------------------------*/
IDMSourceRenderer& IDMSourceRenderer :: sourceEnd(IDMSourceEndEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::sourceEnd" );

  /********************************************************************/
  /* Give item opportunity to perform clean-up                        */
  /********************************************************************/
#ifdef IC_PMWIN
  event.dragItem()->sourceEnd(event);

  /********************************************************************/
  /* Call default/derived renderer to do its source end event         */
  /* processing.                                                      */
  /********************************************************************/
  doRenderEnd(event);

  /********************************************************************/
  /* Obtain the drag operation from the item in order to access       */
  /* the operation's item collection per removal of item from         */
  /* the collection.  Also, reset the PM DRAGITEM's ulItemId field    */
  /* so we will not attempt to construct an IDMItem from it when we   */
  /* are processing DM_PRINTOBJECT or DM_DISCARDOBJECT.               */
  /********************************************************************/
  IDMSourceOperation *pSrcOp = event.dragItem()->sourceOperation();
#ifdef IC_PM
  event.dragItem()->pPMDragItem->ulItemID = 0;
#endif
  pSrcOp->removeItem(event.dragItem());
#endif // IC_PMWIN

#ifdef IC_MOTIF
  IDMItem::Handle hDI(event.dragItem()) ; 

  /********************************************************************/
  /* Give item opportunity to perform clean-up                        */
  /********************************************************************/
  hDI->sourceEnd(event);

  /********************************************************************/
  /* Obtain the drag operation from the item in order to access       */
  /* the operation's item collection per removal of item from         */
  /* the collection.  Also, reset the PM DRAGITEM's ulItemId field    */
  /* so we will not attempt to construct an IDMItem from it when we   */
  /* are processing DM_PRINTOBJECT or DM_DISCARDOBJECT.               */
  /********************************************************************/
  IDMSourceOperation *pSrcOp = hDI->sourceOperation();
  hDI->pDragItemWrapper->setItemID(0);

  pSrcOp->removeItem( hDI );

  /********************************************************************/
  /* Call default/derived renderer to do its source end event         */
  /* processing.                                                      */
  /********************************************************************/
  doRenderEnd(event);
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::sourceDiscard                                             |
|                                                                              |
| Called by the IDMSourceHandler::sourceDiscard() function when it was         |
| indicated that the source window has responsibility for discarding the item. |
------------------------------------------------------------------------------*/
IDMSourceRenderer& IDMSourceRenderer ::
                   sourceDiscard(IDMSourceDiscardEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::sourceDiscard" );

#ifndef IC_MOTIF

  /********************************************************************/
  /* Get the source operation                                         */
  /********************************************************************/
  IDMSourceOperation::Handle pSrcOpH(event.sourceOperation()) ;

  /********************************************************************/
  /* Get the source item from the collection.                         */
  /********************************************************************/
  IDMItem::Handle pDIH(pSrcOpH->item(1)) ; 

  /********************************************************************/
  /* Give source item opportunity to perform discard processing.      */
  /********************************************************************/
  pDIH->sourceDiscard(event);

  /********************************************************************/
  /* Call default/derived renderer to do its source discard event     */
  /* processing.                                                      */
  /********************************************************************/
  doDiscard(event);

#endif // ~IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::sourcePrint                                               |
|                                                                              |
| Called by the IDMSourceHandler::sourcePrint() function when it was           |
| indicated that the source window has responsibility for printing the item.   |
------------------------------------------------------------------------------*/
IDMSourceRenderer& IDMSourceRenderer ::
                   sourcePrint(IDMSourcePrintEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::sourcePrint" );

  /********************************************************************/
  /* Get the source operation                                         */
  /********************************************************************/
  IDMSourceOperation::Handle pSrcOpH(event.sourceOperation()) ;

  /********************************************************************/
  /* Get the source item from the collection.                         */
  /********************************************************************/
  IDMItem::Handle pDIH(pSrcOpH->item(1)) ;

  /********************************************************************/
  /* Give source item opportunity to perform print processing.        */
  /********************************************************************/
  pDIH->sourcePrint(event);

  /********************************************************************/
  /* Call default/derived renderer to do its source print event       */
  /* processing.                                                      */
  /********************************************************************/
  doPrint(event);

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::informTargetOfCompletion                                  |
|                                                                              |
| Informs the target window that source renderering has completed on the drag  |
| item.                                                                        |
------------------------------------------------------------------------------*/
IDMSourceRenderer& IDMSourceRenderer ::
                   informTargetOfCompletion(IDMSourceRenderEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::informTargetOfCompletion" );

#ifdef IC_PM
  if (!(event.dragTransfer()))
  {
    ITHROWLIBRARYERROR(IC_DRAGTRANSFER_INVALID,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }
#endif

#ifdef IC_PM
  /********************************************************************/
  /* Tell the target we have finished renderering the item and the    */
  /* data is available.                                               */
  /*                                                                  */
  /* Note:  Only valid for source renderering                         */
  /********************************************************************/
  ITRACE_DEVELOP( "Posting DM_RENDERCOMPLETE" );
  if (!DrgPostTransferMsg (event.targetWindowHandle(),
                           DM_RENDERCOMPLETE,
                           event.dragTransfer(),
                           (unsigned short)event.completion(),
                           0,
                           event.canRetry()))
  {
    ITHROWGUIERROR("DrgPostTransferMsg");
  }

  /********************************************************************/
  /* Free the dragtransfer structure.                                 */
  /********************************************************************/
  DrgFreeDragtransfer (event.dragTransfer());
#endif

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::doRender                                                  |
|                                                                              |
| Called when a render event occurs and this renderer must render a drag item. |
------------------------------------------------------------------------------*/
bool IDMSourceRenderer :: doRender(IDMSourceRenderEvent &event)
{
  void     *pBuffer,
           *pTempBuffer;

  IMODTRACE_DEVELOP( "IDMSourceRenderer::doRender" );

#ifndef IC_MOTIF

  /********************************************************************/
  /* Get the shared memory buffer address from the target information */
  /*                                                                  */
  /* NOTE:  The first 4 bytes of the buffer contain the buffer's      */
  /*        size, so we index around it.                              */
  /********************************************************************/
  pBuffer = (void *)event.targetInfo();
  pTempBuffer = (char *)pBuffer + 4;

  /********************************************************************/
  /* Get the data from the drag item and copy it to our transfer      */
  /* buffer.                                                          */
  /********************************************************************/
  IString tempStr = event.dragItem()->contents();
  memcpy(pTempBuffer, (char *)tempStr, (tempStr.length() + 1));

  /********************************************************************/
  /* Free the memory object and return true to indicate the render    */
  /* processed ...                                                    */
  /********************************************************************/
#ifdef IC_PM
  DosFreeMem (pBuffer);
#else
  free( pBuffer);
#endif
#endif // ~IC_MOTIF
  return(true);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::doRenderPrepare                                           |
|                                                                              |
| Called when a render prepate event occurs and this renderer must render a    |
| drag item.                                                                   |
------------------------------------------------------------------------------*/
bool IDMSourceRenderer :: doRenderPrepare(IDMSourcePrepareEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::doRenderPrepare" );

#ifndef IC_MOTIF

  /********************************************************************/
  /* Return the length of the source IString                          */
  /********************************************************************/
  event.setTargetInfo(event.dragItem()->contentsSize()+1);

#endif // ~IC_MOTIF

  return(true);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::doRenderEnd                                               |
|                                                                              |
| Called when an end event occurs and this renderer must end the render        |
| operation.                                                                   |
------------------------------------------------------------------------------*/
bool IDMSourceRenderer :: doRenderEnd(IDMSourceEndEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::doRenderEnd" );
  ITRACE_DEVELOP( "return false" );

  return(false);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::doDiscard                                                 |
|                                                                              |
| Called when a discard event occurs and this renderer must discard a drag     |
| item.                                                                        |
------------------------------------------------------------------------------*/
bool IDMSourceRenderer :: doDiscard(IDMSourceDiscardEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::doDiscard" );
  ITRACE_DEVELOP( "return false" );

  return(false);
}

/*------------------------------------------------------------------------------
| IDMSourceRenderer::doPrint                                                   |
|                                                                              |
| Called when a print event occurs and this renderer must print a drag item.   |
------------------------------------------------------------------------------*/
bool IDMSourceRenderer :: doPrint(IDMSourcePrintEvent &event)
{
  IMODTRACE_DEVELOP( "IDMSourceRenderer::doPrint" );
  ITRACE_DEVELOP( "return false" );

  return(false);
}


/*-------------------------------- TARGET RENDERER ---------------------------*/

/*------------------------------------------------------------------------------
| IDMTargetRenderer::IDMTargetRenderer                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMTargetRenderer :: IDMTargetRenderer(const char *pRMFs,
                                       const char *pTypes)
  : IDMRenderer(pRMFs, pTypes),
    pPMDragTransfer(0),
    fDMTargetRendererData( new IDMTargetRendererData() )
{
  IMODTRACE_DEVELOP("IDMTargetRenderer::ctor (rmfs, types)");
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::IDMTargetRenderer                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMTargetRenderer :: IDMTargetRenderer()
  : IDMRenderer(),
    pPMDragTransfer(0),
    fDMTargetRendererData( new IDMTargetRendererData() )
{
  IMODTRACE_DEVELOP("IDMTargetRenderer::ctor ()");

  setSupportedRMFs(IString("(")                 +
                   IDM::rmLibrary               +
                   ")x("                        +
                   IDMItem::rfForThisProcess()  +
                   ","                          +
                   IDM::rfText                  +
                   ","                          +
                   IDM::rfSharedMem             +
                   "),<"                        +
                   IDM::rmFile                  +
                   ","                          +
                   IDM::rfText                  +
                   ">,<"                        +
                   IDM::rmObject                +
                   ","                          +
                   IDM::rfObject                +
                   ">");
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::~IDMTargetRenderer                                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMTargetRenderer :: ~IDMTargetRenderer()
{
  IMODTRACE_DEVELOP("IDMTargetRenderer::dtor ()");
  if (fDMTargetRendererData )
    delete fDMTargetRendererData;
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::canRender                                                 |
|                                                                              |
| Returns a drop indicator with the renderer's ability to render the item with |
| the argument attributes.                                                     |
------------------------------------------------------------------------------*/
DropIndicator IDMTargetRenderer :: canRender(const IDMItem::Handle &pIDMItem)
{
  DropIndicator dropFlag = IDM::neverOk;

  IMODTRACE_DEVELOP( "IDMTargetRenderer::canRender(item)" );

  /**********************************************************************/
  /* Check if any type is supported - IDM::any allows any type ...      */
  /**********************************************************************/
  if (!(supportedTypes().indexOf(IDM::any)))
  {
    /********************************************************************/
    /* Pick thru the item's types to see if we support one of them.     */
    /********************************************************************/
    bool noTypesSupported = true;
    IString itemTypes = pIDMItem->types();
    unsigned count = itemTypes.occurrencesOf(',') + 1;
    unsigned startPos = 1;
    for (unsigned i=0; i < count; i++)
    {
      unsigned index = itemTypes.indexOf(',', startPos);
      IString type = (index ? itemTypes.subString(startPos, index-startPos)  :
                              itemTypes.subString(startPos));

      /******************************************************************/
      /* Verify the type                                                */
      /******************************************************************/
      if (supportedTypes().indexOf(type))
      {
        noTypesSupported = false;
        break;
      }
    startPos = index + 1;
    }

    /********************************************************************/
    /* If no types supported, indicate that there is no need in sending */
    /* us more drag over events.                                        */
    /********************************************************************/
    if (noTypesSupported)
      return(IDM::neverOk);
  }

  /**********************************************************************/
  /* Verify the source RMFs, whereas we have to have a match on         */
  /* on at least 1 of the source RMFs                                   */
  /**********************************************************************/
  IString
    selected = IDMItem::matchingRMFs( pIDMItem->rmfs(),
                                      this->supportedRMFs(),
                                      true );

  if ( selected.length() )
    {
    // Yes, then we can render it...
    ITRACE_DEVELOP( "Match on " + selected );
    pIDMItem->setSelectedRMF( selected );
    dropFlag = IDM::ok;
    }

  return(dropFlag);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::supportsOperation                                         |
|                                                                              |
| Returns true if the default operation (for example, move copy, or link)      |
| is supported by the item.                                                    |
------------------------------------------------------------------------------*/
bool IDMTargetRenderer ::
        supportsOperation(const IDMItem::Handle &pIDMItem)
{
  unsigned long defaultOp = pIDMItem->targetOperation()->operation();
  unsigned long supportedOps = pIDMItem->supportedOperations();

  IMODTRACE_DEVELOP( "IDMTargetRenderer::supportsOperation" );

  /**********************************************************************/
  /* Exit stage right if unknown drag operation defined (must derive    */
  /* and define corresponding behavior).                                */
  /**********************************************************************/
  if (defaultOp >= IDMOperation::unknown)
    return(false);

  /**********************************************************************/
  /* If the drag operation set by the source is IDMOperation::drag,     */
  /* use the item's supported operations to see if we can use the       */
  /* CUA defined default which is "copy" ...                            */
  /**********************************************************************/
  if (defaultOp == IDMOperation::drag)
  {
    if (supportedOps & IDMItem::moveable)
      pIDMItem->targetOperation()->setOperation(IDMOperation::move);
    else
    {
      if (supportedOps & IDMItem::copyable)
        pIDMItem->targetOperation()->setOperation(IDMOperation::copy);
      else
      {
        if (supportedOps & IDMItem::linkable)
          pIDMItem->targetOperation()->setOperation(IDMOperation::link);
        else
        {
          ITRACE_DEVELOP( "can't choose default" );
          return(false);
        }
      }
    }
    ITRACE_DEVELOP( "chose default="
                    +
                    IString( pIDMItem->targetOperation()->operation() ) );
    return(true);
  }

  /**********************************************************************/
  /* See if we can find a match with the item's supported operations,   */
  /* to allow a drop (ok) condition ...                                 */
  /*                                                                    */
  /* Note:  Source defined drag operation will be used here!            */
  /**********************************************************************/
  bool bOpOK = false;
  if ((defaultOp == IDMOperation::copy) && (supportedOps & IDMItem::copyable))
    bOpOK=true;

  if ((defaultOp == IDMOperation::move) && (supportedOps & IDMItem::moveable))
    bOpOK=true;

  if ((defaultOp == IDMOperation::link) && (supportedOps & IDMItem::linkable))
    bOpOK=true;

  ITRACE_DEVELOP( "non-default OK?=" + IString( bOpOK ) );
  return(bOpOK);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::targetRender                                              |
|                                                                              |
| Called when a drop event occurs for a drag item and this renderer must       |
| render the item.                                                             |
------------------------------------------------------------------------------*/
IDMTargetRenderer& IDMTargetRenderer ::
                   targetRender(IDMTargetDropEvent &event,
                                const IDMItem::Handle &pTgtDIH)
{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::targetRender" );

  /**********************************************************************/
  /* Get the selected RMF from the drag item                            */
  /**********************************************************************/

  IString
    rmf = pTgtDIH->selectedRMF(),
    rm  = IDMItem::rmFrom( rmf ),
    rf  = IDMItem::rfFrom( rmf );
  ITRACE_DEVELOP( "Selected RMF=" + rmf );

  /**********************************************************************/
  /* Check for our rendering mechanism first                            */
  /**********************************************************************/
  if (rm == IDM::rmLibrary)
  {
    IMODTRACE_DEVELOP( "Using rmLibrary..." );
    /********************************************************************/
    /* Next key off the format to determine our next step,              */
    /* first checking for inter-process format ...                      */
    /********************************************************************/
    if (rf.includes( IDM::rfProcess ))
    {
      if (rf == IDMItem::rfForThisProcess())
      {
        /****************************************************************/
        /* Same process - we can acquire the data at this point         */
        /****************************************************************/
#ifdef IC_PMWIN
         ITRACE_DEVELOP( "Using rfProcess" );
         IDMItem::Handle pSrcDIH;
         pSrcDIH = IDMItem::sourceItemFor(pTgtDIH);
         pTgtDIH->setContents(pSrcDIH->contents());
         pTgtDIH->setObject(pSrcDIH->object());
#endif // IC_PMWIN
#ifdef IC_MOTIF
         // This has already been done in idropTransferRequestorCallback
         // May wish to optimize later for same process transfer.
#endif // IC_MOTIF
      }
      else
      {
        //Throw exception
        ITRACE_DEVELOP( "Error - IDM::rfProcess used for different process" );
        ITHROWLIBRARYERROR(IC_RFPROCESS_FOR_SAME_PROCESS,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        return(*this);
      }
    }
    else
    {
      /******************************************************************/
      /* Check for the text format and acquire the data if text         */
      /******************************************************************/
      if (rf == IDM::rfText)
      {
#ifdef IC_PMWIN
        ITRACE_DEVELOP( "Using rfText" );
        pTgtDIH->setContents(pTgtDIH->sourceName());
#endif // IC_PMWIN
#ifdef IC_MOTIF
        // This has already been done in idropTransferRequestorCallback
        // May wish to optimize later.
#endif // IC_MOTIF
      }
      else
      {
        /****************************************************************/
        /* Check for shared memory format ...                           */
        /****************************************************************/
        if (rf == IDM::rfSharedMem)
        {
          ITRACE_DEVELOP( "Using rfSharedMem - Tgt" );
#ifdef IC_PMWIN
          /**************************************************************/
          /* This rmf requires source preparation...                    */
          /**************************************************************/
#ifdef IC_PM
          targetRenderPrepare(event, pTgtDIH);
#endif

          /**************************************************************/
          /* Attempt source rendering.                                  */
          /**************************************************************/
          if (renderAtSource(event, pTgtDIH))
          {
            /************************************************************/
            /* Store the transferred data that has been made available  */
            /* by the source.                                           */
            /************************************************************/
            renderComplete(pTgtDIH, (void *)event.targetInfo());
          }
          else
          {
            //Throw exception
            ITRACE_DEVELOP( "Error in renderAtSource!" );
            ITHROWLIBRARYERROR(IC_RENDER_AT_SOURCE_FAILED,
                               IBaseErrorInfo::invalidRequest,
                               IException::recoverable);
            return(*this);
          }
#endif // IC_PMWIN
#ifdef IC_MOTIF
          // This has already been done in idropTransferRequestorCallback
          // May wish to optimize later.
#endif // IC_MOTIF
        }
        else
        {
          /**************************************************************/
          /* Unknown rendering format ...                               */
          /**************************************************************/
          ITHROWLIBRARYERROR(IC_UNKNOWN_RENDERING_FORMAT,
                             IBaseErrorInfo::invalidRequest,
                             IException::recoverable);
          return(*this);
        }
      }
    }
  }
  else
  {
    /********************************************************************/
    /* Check for the file rendering mechanism                           */
    /********************************************************************/
    if (IDMItem::rmFrom(rmf) == IDM::rmFile)
    {
#ifdef IC_WIN
      /**************************************************************/
      /* Get the file data                                  */
      /**************************************************************/
      if (renderAtSource(event, pTgtDIH))
      {
        /************************************************************/
        /* Store the transferred data that has been made available  */
        /* by the source.                                           */
        /************************************************************/
        renderComplete(pTgtDIH, (void *)event.targetInfo());
      }
      else
      {
        //Throw exception
        ITRACE_DEVELOP( "Error in renderAtSource!" );
        ITHROWLIBRARYERROR(IC_RENDER_AT_SOURCE_FAILED,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        return(*this);
      }
#endif
      ITRACE_DEVELOP( "Using rmFile" );
    }
    else
    {
      /******************************************************************/
      /* Check for the WPS object rendering mechanism                   */
      /******************************************************************/
      if (IDMItem::rmFrom(rmf) == IDM::rmObject)
      {
        ITRACE_DEVELOP( "Using rmObject" );
      }
      else
      {
        /****************************************************************/
        /* Unknown rendering mechanism ...                              */
        /****************************************************************/
        //Throw exception
        ITHROWLIBRARYERROR(IC_UNKNOWN_RENDERING_MECHANISM,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        return(*this);
      }
    }
  }

  /**********************************************************************/
  /* Allow item to do any specific drop processing related to the       */
  /* derived IDMItem class.                                             */
  /**********************************************************************/
  bool bRc = pTgtDIH->targetDrop(event);

  /**********************************************************************/
  /* Signal source that we're done (unless processing                   */
  /* IDMTargetEndEvent)                                                 */
  /**********************************************************************/
  informSourceOfCompletion(pTgtDIH, ((bRc) ? IDM::targetSuccessful
                                           : IDM::targetFailed));

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::targetRenderPrepare                                       |
|                                                                              |
| Called when the source window requests that the target window generate a     |
| preparation event.                                                           |
------------------------------------------------------------------------------*/
IDMTargetRenderer& IDMTargetRenderer ::
                   targetRenderPrepare(IDMTargetDropEvent &event,
                                       const IDMItem::Handle &pTgtDIH)
{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::targetRenderPrepare" );

#ifndef IC_MOTIF

  /**********************************************************************/
  /* Request source rendering preparation ...                           */
  /**********************************************************************/
  prepareAtSource(event, pTgtDIH);

#endif // ~IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::targetRenderComplete                                      |
|                                                                              |
| Called when a completion event occurs and this renderer must render a drag   |
| item.                                                                        |
------------------------------------------------------------------------------*/
IDMTargetRenderer& IDMTargetRenderer ::
                   targetRenderComplete(IDMTargetEndEvent &event)
{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::targetRenderComplete" );

#ifndef IC_MOTIF

  IDMItem::Handle pTgtDIH(event.dragItem()) ;

#ifdef IC_PM
  /**********************************************************************/
  /* Store pointer to the current drag transfer, so we can retrieve     */
  /* later for freeing when we use threaded source rendering.           */
  /**********************************************************************/
  fDMTargetRendererData->pCurrentDragTransfer = event.dragTransfer();

  /**********************************************************************/
  /* Store the transferred data that has been made available by the     */
  /* source.                                                            */
  /**********************************************************************/
  renderComplete( pTgtDIH,
            (void *)fDMTargetRendererData->pCurrentDragTransfer->ulTargetInfo );
#else

  /**********************************************************************/
  /* Store the transferred data that has been made available by the     */
  /* source.                                                            */
  /**********************************************************************/
  renderComplete(pTgtDIH, (void *)(event.targetInfo()));
#endif

  /**********************************************************************/
  /* Give item opportunity to diddle ...                                */
  /**********************************************************************/
  bool bRc = pTgtDIH->targetEnd(event);

  /**********************************************************************/
  /* Signal source that we're done ...                                  */
  /**********************************************************************/
  informSourceOfCompletion(pTgtDIH, ((bRc) ? IDM::targetSuccessful
                                           : IDM::targetFailed));

#endif // ~IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::informSourceOfCompletion                                  |
|                                                                              |
| Informs the source window that the target window has finished processing the |
| data associated with the dropped item.  The source window is informed for    |
| both source and target rendering.  The render completion code is used to     |
| to indicate if the target was successful.                                    |
------------------------------------------------------------------------------*/
IDMTargetRenderer& IDMTargetRenderer ::
                   informSourceOfCompletion(const IDMItem::Handle &pTgtDIH,
                                            IDM::RenderCompletion code)
{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::informSourceOfCompletion" );

  IDMTargetOperation::Handle pTgtOpH(IDMTargetOperation::targetOperation()) ;

#ifdef IC_WIN
  pTgtDIH->sourceWindowHandle().sendEvent (DM_ENDCONVERSATION,
                            IEventParameter1(pTgtDIH->pDragItemWrapper->itemID()),
                            IEventParameter2(code,
                                             pTgtDIH->targetOperation()->operation()));

#endif

#ifdef IC_PM
  if (!(pTgtDIH->PMDragItem()))
  {
    ITHROWLIBRARYERROR(IC_DRAGITEM_INVALID,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  /**********************************************************************/
  /* Tell the source that we are done communicating for a particular    */
  /* drag item.                                                         */
  /**********************************************************************/
  pTgtDIH->sourceWindowHandle().sendEvent (DM_ENDCONVERSATION,
                                   MPFROMLONG(pTgtDIH->PMDragItem()->ulItemID),
                                   MPFROMLONG(code));

  /**********************************************************************/
  /* Free the dragtransfer structure.                                   */
  /**********************************************************************/
  if (fDMTargetRendererData->pCurrentDragTransfer)
  {
    DrgFreeDragtransfer( fDMTargetRendererData->pCurrentDragTransfer );
    fDMTargetRendererData->pCurrentDragTransfer = 0;
  }
#endif

#ifdef IC_MOTIF
     unsigned long numItems = pTgtOpH->numberOfItems();
     IDragItemWrapper diw( *pTgtDIH->PMDragItem() );
     unsigned long twiceNumItems = 2*numItems;
     // Save the completion information for later sending.
     UsArray& completionCodes = *IDMMotif::targetCompletions();
     completionCodes[ twiceNumItems-2 ] = diw.itemID();
     completionCodes[ twiceNumItems-1 ] = code;
#endif // IC_MOTIF

  /**********************************************************************/
  /* Remove the item from the target operation's collection ...         */
  /**********************************************************************/
  pTgtOpH->removeItem(pTgtDIH);
  if (!(pTgtOpH->numberOfItems()))
  {
    /********************************************************************/
    /* When the count of items reaches 0, reset the static target       */
    /* operation pointer, which causes the destructor for               */
    /* IDMTargetOperation to be called ...                              */
    /********************************************************************/
    IDMTargetOperation::setTargetOperation (0);

#ifdef IC_PM
    /********************************************************************/
    /* Reset the drag transfer count and set the pointer to the PM      */
    /* drag transfer structure to 0.                                    */
    /********************************************************************/
    fDMTargetRendererData->ulDragTransferCount = 0;
    this->pPMDragTransfer = 0;
#endif
  }

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::renderAtSource                                            |
|                                                                              |
| Sends a render event to the source window to request source rendering for    |
| the argument item.                                                           |
------------------------------------------------------------------------------*/
bool IDMTargetRenderer :: renderAtSource(IDMTargetDropEvent &event,
                                            const IDMItem::Handle &pTgtDIH)
{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::renderAtSource" );

#ifdef IC_PMWIN

#ifdef IC_WIN
  return fDMTargetRendererData->getTheData( event, pTgtDIH );
#endif

#ifdef IC_PM
  void         *pBuffer;
  unsigned long bufferSize;
  PID           spid;
  TID           tid;

  IString
    rmf = pTgtDIH->selectedRMF(),
    rm  = IDMItem::rmFrom( rmf ),
    rf  = IDMItem::rfFrom( rmf );

  bool
    bSharedMem = (rf == IDM::rfSharedMem) ? true : false;

  if (bSharedMem)
  {
    ITRACE_DEVELOP( "Rendering via <" + rm + ",rfSharedMem>" );

    /********************************************************************/
    /* Get the length of the text string so we know how large to make   */
    /* the (shared) buffer.                                             */
    /********************************************************************/
    bufferSize = event.targetInfo();

    /********************************************************************/
    /* Allocate a shared memory buffer and give the target process      */
    /* access to it.  First obtain the source process id.               */
    /* If either of the memory calls fail...bail.                       */
    /********************************************************************/
    if (!WinQueryWindowProcess (pTgtDIH->sourceWindowHandle(), &spid, &tid))
    {
      ITHROWGUIERROR("WinQueryWindowProcess");
    }

    unsigned long rc;
    rc = DosAllocSharedMem (&pBuffer, 0, bufferSize + 4,
                            (OBJ_GIVEABLE | PAG_READ | PAG_WRITE | PAG_COMMIT));
    if (rc)
    {
      ITHROWSYSTEMERROR(rc,
                        "DosAllocSharedMem",
                        IBaseErrorInfo::outOfMemory,
                        IException::recoverable);
    }

    rc = DosGiveSharedMem (pBuffer, spid, (PAG_READ | PAG_WRITE));
    if (rc)
    {
      ITHROWSYSTEMERROR(rc,
                        "DosGiveSharedMem",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
    }

    /********************************************************************/
    /* Save the pointer to the buffer that will be used to transfer the */
    /* data between the source and the target.                          */
    /*                                                                  */
    /* NOTE:  The first 4 bytes of the buffer contain the buffer's      */
    /*        size, so we store the size here.                          */
    /********************************************************************/
    *(unsigned long *)pBuffer = bufferSize + 4;
    event.setTargetInfo((unsigned long)pBuffer);
  } //rfSharedMem

  /**********************************************************************/
  /* Get the dragtransfer structure and allocate it if not allocated    */
  /**********************************************************************/
  _DRAGTRANSFER *pDXfer = allocDragTransfer(event, pTgtDIH);

  /**********************************************************************/
  /* Store pointer to shared memory buffer in the _DRAGTRANSFER         */
  /**********************************************************************/
  pDXfer->ulTargetInfo = event.targetInfo();

  ITRACE_DEVELOP( "Sending DM_RENDER" );
  if ( !DrgSendTransferMsg (pTgtDIH->sourceWindowHandle(),
                            DM_RENDER,
                            MPFROMP(pDXfer), 0) )
  {
    ITHROWGUIERROR("DrgSendTransferMsg");
    return(false);
  }

  return(true);
#endif // IC_PM
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return (false) ;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::prepareAtSource                                           |
|                                                                              |
| Sends a prepare event to the source window to request source preparation for |
| the argument item.                                                           |
------------------------------------------------------------------------------*/
bool IDMTargetRenderer :: prepareAtSource(IDMTargetDropEvent &event,
                                             const IDMItem::Handle &pTgtDIH)
{
#ifdef IC_PM
  IMODTRACE_DEVELOP( "IDMTargetRenderer::prepareAtSource" );

  /********************************************************************/
  /* Get the dragtransfer structure and allocate it if not allocated. */
  /********************************************************************/
  _DRAGTRANSFER *pDXfer = allocDragTransfer(event, pTgtDIH);

  ITRACE_DEVELOP( "Sending DM_RENDERPREPARE" );
  DrgSendTransferMsg (pTgtDIH->sourceWindowHandle(),
                      DM_RENDERPREPARE,
                      MPFROMP(pDXfer), 0);

  /********************************************************************/
  /* Save the target information in the event.                        */
  /********************************************************************/
  event.setTargetInfo(pDXfer->ulTargetInfo);

  /*************************************************************************/
  /* Update the source window handle in the drag item, so the target will  */
  /* be aware of an alternate source window for source rendering.          */
  /*************************************************************************/
  pTgtDIH->setSourceWindowHandle( pDXfer->pditem->hwndItem );
#endif
  return(true);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::renderComplete                                            |
|                                                                              |
| Called to process a completion event for a drag item.                        |
------------------------------------------------------------------------------*/
bool IDMTargetRenderer ::
                  renderComplete(const IDMItem::Handle &pTgtDIH,
                                 void* pBuffer)
{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::renderComplete" );

#ifndef IC_MOTIF

  IString
    rmf = pTgtDIH->selectedRMF(),
    rm  = IDMItem::rmFrom( rmf ),
    rf  = IDMItem::rfFrom( rmf );

  if (rf == IDM::rfSharedMem)
  {
    ITRACE_DEVELOP( "Completing rendering via <" + rm + ",rfSharedMem>" );

#ifdef IC_PM
    unsigned long ulBufferSize = *(unsigned long *)pBuffer;
    void *pTempBuffer = (char *)pBuffer + 4;

    /**********************************************************************/
    /* Store the transferred data in the target drag item                 */
    /**********************************************************************/
    pTgtDIH->setContents( IString(pTempBuffer, ulBufferSize ) );
#endif

    /**********************************************************************/
    /* Free the memory used to hold the dragged data.                     */
    /**********************************************************************/
#ifdef IC_PM
    DosFreeMem (pBuffer);
#else
    free(pBuffer);
#endif
  }

#endif // ~IC_MOTIF

  return (true);
}

/*------------------------------------------------------------------------------
| IDMTargetRenderer::renderToName                                              |
|                                                                              |
| Called to provide the name of the location where the data will be renderer   |
| to.  This name can be a filename, shared memory buffer, etc.                 |
------------------------------------------------------------------------------*/
IString IDMTargetRenderer :: renderToName (const IDMItem::Handle &pDI)
{
  return (pDI->targetName());
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDMTargetRenderer::allocDragTransfer                                         |
|                                                                              |
| Create a dragtransfer structure.                                             |
------------------------------------------------------------------------------*/
_DRAGTRANSFER* IDMTargetRenderer ::
                allocDragTransfer(IDMTargetDropEvent &dropEvent,
                                  const IDMItem::Handle &pDI)

{
  IMODTRACE_DEVELOP( "IDMTargetRenderer::allocDragTransfer" );


  /********************************************************************/
  /* Allocate a drag transfer array whose size is based on the        */
  /* number of drag items, unless the array already exists.           */
  /********************************************************************/
  if (!this->pPMDragTransfer)
  {
    this->pPMDragTransfer =
      DrgAllocDragtransfer( pDI->targetOperation()->numberOfItems() );

    ITRACE_DEVELOP( "Number of DRAGTRANSFERs allocated = " +
                    IString( pDI->targetOperation()->numberOfItems() ) );

    if (!this->pPMDragTransfer)
      ITHROWGUIERROR("DrgAllocDragtransfer");
  }

  /********************************************************************/
  /* Test to see if the drag transfer pointer is set, and if so,      */
  /* check to see if we have already alloted the DRAGTRANSFER         */
  /* structure from the drag transfer array for this drag item object.*/
  /* If we have, we are done and we return the drag transfer.         */
  /********************************************************************/
  if ( fDMTargetRendererData->pCurrentDragTransfer )
  {
    if ( fDMTargetRendererData->pCurrentDragTransfer->pditem ==
         pDI->PMDragItem() )
    {
      return( fDMTargetRendererData->pCurrentDragTransfer );
    }
  }

  /********************************************************************/
  /* If we reach this point we need to allot a DRAGTRANSFER structure */
  /* for this drag item object from the array.  Use the drag transfer */
  /* count to determine the location of the DRAGTRANSFER structure to */
  /* use.                                                             */
  /********************************************************************/
  unsigned long
    ulCnt = fDMTargetRendererData->ulDragTransferCount * sizeof(_DRAGTRANSFER);

  _DRAGTRANSFER
   *pdxfer = (_DRAGTRANSFER *)(((char *)this->pPMDragTransfer) + ulCnt);

  /********************************************************************/
  /* Increment the count to track how many DRAGTRANSFER structures we */
  /* have allotted.                                                   */
  /********************************************************************/
  fDMTargetRendererData->ulDragTransferCount++;

  /********************************************************************/
  /* Fill in the fields of the DRAGTRANSFER structure.                */
  /********************************************************************/
  pdxfer->cb = sizeof(_DRAGTRANSFER);
  pdxfer->hwndClient = dropEvent.window()->handle();
  pdxfer->pditem = pDI->PMDragItem();
  pdxfer->hstrSelectedRMF = pDI->handleFromString(pDI->selectedRMF());
  pdxfer->hstrRenderToName = pDI->handleFromString(renderToName (pDI));
  pdxfer->ulTargetInfo = 0;
  pdxfer->usOperation = (IDMTargetOperation::targetOperation())->operation();
  pdxfer->fsReply = 0;                      // Reply, used later

  /********************************************************************/
  /* Store pointer to the current drag transfer, so we can retrieve   */
  /* later for freeing when we use non-threaded source rendering.     */
  /********************************************************************/
  fDMTargetRendererData->pCurrentDragTransfer = pdxfer;

  /********************************************************************/
  /* Return the drag transfer structure we alloted.                   */
  /********************************************************************/
  return(pdxfer);
}/* end IDMTargetRenderer :: dragTransfer(...) */
#endif

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDMTargetRenderer::dragTransfer                                              |
|                                                                              |
| Returns the PM dragtransfer structure.                                       |
------------------------------------------------------------------------------*/
_DRAGTRANSFER* IDMTargetRenderer :: dragTransfer() const
{
  return (this->pPMDragTransfer);
}
#endif

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDMTargetRendererData::getTheData                                            |
|                                                                              |
| Returns the PM dragtransfer structure.                                       |
------------------------------------------------------------------------------*/
bool IDMTargetRendererData::getTheData( IDMTargetDropEvent &event,
                                            const IDMItem::Handle &pTgtDIH)
{
  IMODTRACE_DEVELOP("IDMTargetRenderer::getTheData");

  IDataObject *dataObject = (IDataObject *) event.parameter2().asUnsignedLong();

  IString type = firstSupportedType(pTgtDIH);
  ITRACE_DEVELOP("firstSupportedType(pTgtDIH) : " + type + " all types are "
      + pTgtDIH->types() );

  /*-----------------------------------------------------------
  | convert type to corresponding clipboard format
  -----------------------------------------------------------*/
  FORMATETC formatEtc;
  STGMEDIUM stgmedium;

  //Initialize unused STGMEDIUM fields.
  stgmedium.pUnkForRelease = NULL;

  CLIPFORMAT cfFormat =
    IDMFormat::formatFromString(IDMDataObject::formatDragInfo());
  SETDefFormatEtc( formatEtc, cfFormat, TYMED_ISTREAM );
  bool bFoundFormat = dataObject->QueryGetData ( &formatEtc) == S_OK;

  cfFormat = CLIPFORMAT(IDMFormat::formatFromString(type));
  ITRACE_DEVELOP("Format is text " + IString( cfFormat) );

  IString draggedText;

  SETDefFormatEtc( formatEtc, cfFormat, TYMED_HGLOBAL );

  /*-----------------------------------------------------------
  | Set the item ID (source item handle) in formatEtc.lindex;
  -----------------------------------------------------------*/
   unsigned long itemID = 0xFFFF;
  if (pTgtDIH->pDragItemWrapper)
  {
    // if the drag orginated from an Open Class app then get
    // the lindex from the item id. We can determine that we
    // came from an Open Class app if the sourceWindowHandle
    // is present.
    if ( pTgtDIH->pDragItemWrapper->sourceWindowHandle() ) {
       itemID = pTgtDIH->pDragItemWrapper->itemID();
       formatEtc.lindex = unsigned(itemID);
    }
  }

  ITRACE_DEVELOP("About to get the format " +
                 IString( (unsigned short)formatEtc.cfFormat));

  HRESULT hr = dataObject->GetData( &formatEtc, &stgmedium );
  if (SUCCEEDED(hr))
  {
    char FAR* pszData = (char FAR*)::GlobalLock(stgmedium.hGlobal);

    /*-------------------------------
    | Get drag data
    -------------------------------*/
    if ( ( type == IDM::file ) &&
         ( type.length() == IString( IDM::file ).length()) )
    {
      LPMALLOC lpMalloc;
      DROPFILES *dropFile = (DROPFILES *) pszData;
      // Use the item id to extract the corresponding file from the
      // the dropfile-pFiles.
      unsigned long itemIndex(0) ;
      if (pTgtDIH->pDragItemWrapper)
         itemIndex = pTgtDIH->pDragItemWrapper->itemID() ;
      if ( dropFile->fWide)
      {
        if (SUCCEEDED(SHGetMalloc(&lpMalloc) ))
        {
          // Use the item id to find the file that corresponds to the item. The
          // Dropfile data structure is a double null terminated list of null
          // terminated strings. On NT the file names are UNICODE and on 95
          // they are sbcs.
          WCHAR *pszString = (WCHAR *)((char *)(dropFile) + dropFile->pFiles);

          if ( !bFoundFormat )
          {
            // for ( unsigned long i = 0; i < itemID; i++)
            for ( unsigned long i = 0; i < itemIndex; i++)
            {
              pszString = wcschr( pszString, '\0');
              pszString++;
            }
          }

          int cch = WideCharToMultiByte(CP_ACP, 0,
                      pszString, -1, NULL, 0, NULL, NULL);
          LPSTR lpsz = (LPSTR) lpMalloc->Alloc( cch );

          if (lpsz != NULL )
          {
            cch = WideCharToMultiByte( CP_ACP, 0,
                      pszString, -1, lpsz, cch, NULL, NULL );
            draggedText = IString( (void *) lpsz, cch);

            lpMalloc->Free( lpsz );
          }
        }
      }
      else
      {
        char *pszString = (char *)(dropFile) + dropFile->pFiles;

        if ( !bFoundFormat )
        {
          // for ( unsigned long i = 0; i < itemID; i++)
          for ( unsigned long i = 0; i < itemIndex ; i++)
          {
            pszString = strchr( pszString, '\0');
            pszString++;
          }
        }

        draggedText = IString( (char *) pszString);
      }

      if (draggedText.length())
      {
        int lastSlash = draggedText.lastIndexOf('\\');

        pTgtDIH->setContainerName( draggedText.subString( 1, lastSlash ) );
        pTgtDIH->setSourceName( draggedText.subString( lastSlash + 1,
          draggedText.length() - lastSlash ));
      }
      pTgtDIH->setTrueType( IDM::file );
    }
    else if ( type == CFSTR_FILENAME)
    {
      draggedText = IString( (char *) pszData);
    }
    else if ( ( type == CFSTR_SHELLIDLIST) &&
              ( type.length() == IString(CFSTR_SHELLIDLIST).length() ))
    {
      LPIDA pIDA = (LPIDA) pszData;
      LPITEMIDLIST pidl = (LPITEMIDLIST)((char *)pIDA + pIDA->aoffset[pIDA->cidl]);
      ItemID lastItem( pidl );
      int itemSize = lastItem.size() + sizeof( CIDA) +
          (pIDA->cidl * sizeof(UINT));
      draggedText = IString((void *)pszData, itemSize );
      if (draggedText.length())
      {
        int lastSlash = draggedText.lastIndexOf('\\');

        pTgtDIH->setContainerName( draggedText.subString( 1, lastSlash ) );
        pTgtDIH->setSourceName( draggedText.subString( lastSlash + 1,
          draggedText.length() - lastSlash ));
      }
      pTgtDIH->setTrueType( IString(CFSTR_SHELLIDLIST) );
    }
    else
    {
      unsigned long ulLength = ::GlobalSize(stgmedium.hGlobal) ;
      draggedText = IString((void *)(pszData), ulLength);
    }

    ::GlobalUnlock(stgmedium.hGlobal);

    ReleaseStgMedium(&stgmedium);
    ITRACE_DEVELOP("\tDrop succeeded");
  }
  else
  {
    ITRACE_DEVELOP("\tDrop failed");
  }

  pTgtDIH->setContents( draggedText );

  return true;
}

/*------------------------------------------------------------------------------
| IDMSourceRendererData::renderData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IDMSourceRendererData& IDMSourceRendererData::
                                renderData( IDMSourceRenderEvent &event )
{
  IMODTRACE_DEVELOP( "IDMSourceRendererData::renderData");

  /*-----------------------------------------------------------
  | Retrieve handle to source operation
  -----------------------------------------------------------*/
  IDMSourceOperation::Handle pSrcOp(IDMSourceOperation::sourceOperation()) ; 

  IDMItem::Handle item(pSrcOp->item( 1 )) ;

  /*----------------------------------------------------------------
  | If the format has an item number augmentation, resolve the
  | requested item from it.
  ----------------------------------------------------------------*/
  FORMATETC* pFormat = (FORMATETC*)event.parameter1().asUnsignedLong();
  if (pFormat->lindex != -1)
    item = (IDMItem*)(unsigned)pFormat->lindex;

  ITRACE_DEVELOP(IString("rendering item ") + IString(pFormat->lindex));

  /********************************************************************/
  /* Indicate source and target are not the same                      */
  /********************************************************************/
  item->bTargetTheSource = false;

  /*-----------------------------------------------------------
  | Get the contents from IDMItem
  -----------------------------------------------------------*/
  IString contents = item->contents();

  if ( item->supportsRMF( IDM::rmFile, IDM::rfText ) )
  {
    renderFileData( event );
  }
  else
  {
    ITRACE_DEVELOP("IDMSourceRendererData::renderData  item contents " +
                   contents);

    STGMEDIUM *pmedium = (STGMEDIUM *) event.parameter2().asUnsignedLong();

    pmedium->tymed = TYMED_HGLOBAL;

    pmedium->hGlobal = ::GlobalAlloc(GHND, contents.length() + 1);
    char FAR* pszTarget = (char FAR*)::GlobalLock(pmedium->hGlobal);

    /*-----------------------------------------------------------
    | Copy inbound data to our new data area
    -----------------------------------------------------------*/
    ::memcpy(pszTarget, (char *)contents , contents.length() );
    ITRACE_DEVELOP(IString('\"') + pszTarget + IString('\"'));
    ::GlobalUnlock(pmedium->hGlobal);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IDMTargetRendererData::firstSupportedType                                    |
|                                                                              |
| Returns the first supported type in the target drop item                     |
------------------------------------------------------------------------------*/
IString IDMTargetRendererData::firstSupportedType( const IDMItem::Handle &pTgtDIH )

  {
  IMODTRACE_DEVELOP("IDMTargetRenderer::firstSupportedType");

  IString type;
  IString itemTypes = pTgtDIH->types();

  IDMTargetRenderer* pTgtRender = &IDMRenderer::defaultTargetRenderer();
  IString supportedTypes = pTgtRender->supportedTypes();

  ITRACE_DEVELOP("Item types: " + itemTypes);
  ITRACE_DEVELOP("Supported types: " + supportedTypes);

  unsigned startPos = 1;
  unsigned count = itemTypes.occurrencesOf(',') + 1;

  for (unsigned i=0; i < count; i++)
    {
    unsigned index = itemTypes.indexOf(',', startPos);
    type = (index ? itemTypes.subString(startPos, index-startPos)  :
                                      itemTypes.subString(startPos));
    /******************************************************************/
    /* Verify the type ...                                            */
    /******************************************************************/
    if (supportedTypes.indexOf(type)  || supportedTypes.indexOf(IDM::any))
      break;

    startPos = index + 1;
    }

  return (type);
  }

/*------------------------------------------------------------------------------
| IDMSourceRendererData::renderFileData                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IDMSourceRendererData& IDMSourceRendererData::renderFileData(
     IDMSourceRenderEvent &event )
{
  IMODTRACE_DEVELOP("IDMSourceRendererData::renderFileData");
  FORMATETC* pFormat = (FORMATETC*)event.parameter1().asUnsignedLong();
  IDMSourceOperation::Handle pSrcOp(IDMSourceOperation::sourceOperation()) ;

  unsigned long itemCount = pSrcOp->numberOfItems();

  IDMItem::Handle item;

  if (pFormat->cfFormat == CF_HDROP )
  {
    IMODTRACE_DEVELOP( "Looking for CF_HDROP");
    IString aNull( (void *) "\0", 1 );
    IString doubleNull( (void *) "\0\0", 2 );

    // Need to get the real point.
    POINT pt;
    pt.x = 0;
    pt.y = 0;

    DROPFILES dropFile;

    IString theFile;

    if (pFormat->lindex != -1)
    {
      item = (IDMItem*)(unsigned)pFormat->lindex; 
      theFile = item->containerName() + item->sourceName() + aNull;
    }
    else
    {
      for (unsigned itemIndex = 1; itemIndex <= itemCount; itemIndex++)
      {
        item = pSrcOp->item( itemIndex );
        // TODO: We should probably just see if type includes file??
        if ( item->trueType() == IDM::file )
        {
          theFile += item->containerName() + item->sourceName() + aNull;
        }
      }
    }

    theFile += doubleNull;

    ITRACE_DEVELOP("The full qualified file is " + theFile );

    dropFile.pFiles = sizeof( DROPFILES);
    dropFile.pt = pt;
    dropFile.fNC = false;
    if (IPlatform::isWinNT() )
    {
      dropFile.fWide = True;
    }
    else
    {
      dropFile.fWide = False;
    }

    char *lpsz;
    int fileNameLength;

    LPMALLOC lpMalloc = 0;
    if ( dropFile.fWide)
    {
      // double null terminated string
      if (SUCCEEDED(SHGetMalloc(&lpMalloc) ))
      {
        fileNameLength = theFile.length() * sizeof(WCHAR);
        lpsz = (char *) lpMalloc->Alloc( fileNameLength );

        if (lpsz != NULL )
        {
          int cch2 = MultiByteToWideChar( CP_ACP, 0,
                    (char *)theFile, theFile.length(),
                    (LPWSTR) lpsz, theFile.length());
        }
      }
      else
        lpMalloc = 0;
    }
    else
    {
      fileNameLength = theFile.length();
      lpsz = (char *)theFile;
    }
    STGMEDIUM *pmedium = (STGMEDIUM *) event.parameter2().asUnsignedLong();
    pmedium->tymed = TYMED_HGLOBAL;

    pmedium->hGlobal = ::GlobalAlloc(GHND,
                           sizeof( DROPFILES ) + fileNameLength );
    char FAR* pszTarget = (char FAR*)::GlobalLock(pmedium->hGlobal);

    memcpy( pszTarget,
            (void *) &dropFile,
            sizeof(DROPFILES) );

    memcpy( pszTarget + sizeof(DROPFILES),
            (void *) lpsz,
            fileNameLength );

    // if we needed a wide character buffer then free it.
    if ( dropFile.fWide && lpMalloc)
    {
      lpMalloc->Free( lpsz );
    }

    ::GlobalUnlock(pmedium->hGlobal);
  }

  else if (pFormat->cfFormat ==
              CLIPFORMAT( IDMFormat::formatFromString(CFSTR_SHELLIDLIST)) )
  {
    IMODTRACE_DEVELOP( "Looking for ShellIDList");
    CIDA myCIDA;
    myCIDA.cidl = itemCount;
    myCIDA.aoffset[0] = sizeof( CIDA) + ( itemCount * sizeof(UINT));

    // First pass will calculate the size of the items.
    int totalItemSize = 0;
    for (unsigned itemIndex = 1; itemIndex <= itemCount; itemIndex++)
    {
      item = pSrcOp->item( itemIndex );
      if ( item->trueType() == IDM::file )
      {
        IString displayName = item->containerName() + item->sourceName();
        ItemID otherID = ItemID::itemIDFromDisplayName( displayName );
        totalItemSize += otherID.size();
      }
    }

    STGMEDIUM *pmedium = (STGMEDIUM *) event.parameter2().asUnsignedLong();
    pmedium->tymed = TYMED_HGLOBAL;

    // Allocate the global memory that we need for the CIDA structure.

    pmedium->hGlobal = ::GlobalAlloc(GHND,   sizeof( CIDA )
                                           + (itemCount * sizeof( myCIDA.aoffset[0]))
                                           + sizeof( USHORT )
                                           + totalItemSize );

    char FAR* pszTarget = (char FAR*)::GlobalLock(pmedium->hGlobal);
    memcpy( pszTarget,
            (void *) &myCIDA,
            sizeof(CIDA) );
    pszTarget += sizeof(CIDA);

    // The item area is after the CIDA structure and the additional aoffsets
    char FAR* pszItems = pszTarget + ( itemCount * sizeof( myCIDA.aoffset[0] ) );

    USHORT theCb = 0;
    memcpy( pszItems, (void *) &theCb, sizeof(USHORT) );
    pszItems += sizeof(USHORT);

    UINT offset = myCIDA.aoffset[0] + sizeof(USHORT);

    for (itemIndex = 1; itemIndex <= itemCount; itemIndex++)
    {
      item = pSrcOp->item( itemIndex );
      if ( item->trueType() == IDM::file )
      {
        IString displayName = item->containerName() + item->sourceName();
        ItemID otherID = ItemID::itemIDFromDisplayName( displayName );

        // Copy the item list to global memory.
        memcpy( pszItems, otherID.asLPITEMIDLIST(), otherID.size() );
        pszItems += otherID.size();

        // ...and update the offset to the item.
        memcpy( pszTarget, &offset, sizeof( offset ) );
        pszTarget += sizeof( myCIDA.aoffset[0] );

        // calculate the offset for the next item
        offset += otherID.size();
      }
    }

    ::GlobalUnlock(pmedium->hGlobal);
  }
  else if (pFormat->cfFormat ==
            CLIPFORMAT( IDMFormat::formatFromString(CFSTR_SHELLIDLISTOFFSET)) )
  {
    ITRACE_DEVELOP("Shell IDList Offset");
    STGMEDIUM *pmedium = (STGMEDIUM *) event.parameter2().asUnsignedLong();
    pmedium->tymed = TYMED_HGLOBAL;

    pmedium->hGlobal = ::GlobalAlloc(GHND, sizeof( POINT) * ( itemCount + 1) );
    char FAR* pszTarget = (char FAR*)::GlobalLock(pmedium->hGlobal);

    POINT *myPoints = (POINT* ) pszTarget;

    // This one isn't that important.
    myPoints->x = 40;
    myPoints->y = 50;

    myPoints++;
    int xoffset,yoffset;
    for ( unsigned long itemIndex = 0;
          itemIndex < itemCount;
          itemIndex++ )
    {
      item = pSrcOp->item( itemIndex + 1 );
      ISize imageOffset = item->imageOffset( );
      //relative offsets
      myPoints->x = imageOffset.width();
      myPoints->y = imageOffset.height();
        ITRACE_DEVELOP("myPoint is " +
      IPoint(myPoints->x,myPoints->y).asString());
      myPoints++;
    }

    ::GlobalUnlock(pmedium->hGlobal);
  }
  else if (pFormat->cfFormat ==
              CLIPFORMAT( IDMFormat::formatFromString("FileName")) )
  {
    STGMEDIUM *pmedium = (STGMEDIUM *) event.parameter2().asUnsignedLong();
    pmedium->tymed = TYMED_HGLOBAL;

    IString displayName = item->containerName() + item->sourceName();

    pmedium->hGlobal = ::GlobalAlloc(GHND, displayName.length() + 1 );

    char FAR* pszTarget = (char FAR*)::GlobalLock(pmedium->hGlobal);
    memcpy( pszTarget,
            (void *) (char *) displayName,
            displayName.length() + 1 );

    ::GlobalUnlock(pmedium->hGlobal);
  }
  else
    ITRACE_DEVELOP("What format is it?");
  return *this;
}
#endif
