// Revision: 04 1.10.1.6 source/ui/drag/idmtbbit.cpp, dragdrop, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: idmtbbit.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMTBarButtonItem                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #include <iwindefs.h>
}

#include <idmtbbit.hpp>
#include <idmevent.hpp>
#include <idmsrcop.hpp>
#include <idmimage.hpp>
#include <itbar.hpp>
#include <itbarbut.hpp>
#include <iimage.hpp>
#include <igrport.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMTBBIT_CPP_
  #include <ipagetun.h>
#endif

#pragma enum(4)
#pragma pack(push,4)

class IDMTBarButtonItemData
{
public:
   IDMTBarButtonItemData() ;
   ~IDMTBarButtonItemData() ;
   IImage
      *pIImage ;
private:
   IDMTBarButtonItemData ( const IDMTBarButtonItemData& privateItemData );
   IDMTBarButtonItemData
      &operator = ( const IDMTBarButtonItemData&  );
} ;

#pragma enum(pop)
#pragma pack(pop)

IDMTBarButtonItemData::IDMTBarButtonItemData()
   : pIImage(0)
{
}

IDMTBarButtonItemData::~IDMTBarButtonItemData()
{
   if (pIImage)
      delete pIImage ;
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::IDMTBarButtonItem                                         |
|                                                                              |
| Constructor for toolbar button source items.                                 |
------------------------------------------------------------------------------*/
IDMTBarButtonItem::IDMTBarButtonItem( IDMSourceOperation* pDMSrcOp )  :
                          IDMItem(pDMSrcOp,
                                  IDM::toolBarButton,
                                  IDMItem::moveable,
                                  none),
                          pTargetBtn(0),
                          fDMTBarButtonItemData(0)
{
  IFUNCTRACE_DEVELOP();

  fDMTBarButtonItemData = new IDMTBarButtonItemData() ;

  /********************************************************************/
  /* Obtain pointer to the toolbar button from the source window      */
  /********************************************************************/
  this->pSourceBtn = (IToolBarButton *)pDMSrcOp->sourceWindow();

  /********************************************************************/
  /* Set the native rendering mechanism and format for the item.      */
  /* Add the WPS shredder RMF, unless disableDragDelete style is      */
  /* specified.                                                       */
  /********************************************************************/
  setRMFs( rmfFrom( IDM::rmLibrary, rfForThisProcess() ) );
  if (this->pSourceBtn->allowsDragDelete())
    addRMF( rmfFrom( IDM::rmDiscard, IDM::rfUnknown ) );

  /********************************************************************/
  /* Add a drag image for this item.  Use the bitmap handle from the  */
  /* toolbar button for the drag image.                               */
  /********************************************************************/
  IBitmapHandle hbitMap = this->pSourceBtn->bitmap();

  if (hbitMap)
  {
    IGImage        bitmap(hbitMap);
    ISize          bmpSize( bitmap.rasterBounds().width(),
                            bitmap.rasterBounds().height() );

    //Create an IImage to retrieve the grafport.  This allows all drawing
    //to be cached in the image's presentation space.
    //Cast to (long) as required by IImage Contructor
    fDMTBarButtonItemData->pIImage = new IImage( (long)bmpSize.width()+4,
                                                 (long)bmpSize.height()+4,
                                                 IGImage::k256Color8Bit,
                                                 ICoordinateSystem::kOriginUpperLeft);

    IGrafBundle    bundle;
    bundle.setFrameColor( IBasicColor( IColor::kDefaultButton ) );
    bundle.setFillColor( this->pSourceBtn->backgroundColor() );
    bundle.setDrawingOperation( IAttributeState::kFillAndFrame );
    bundle.setFramePen(IPen(IPen::kHairline, 1));

    fDMTBarButtonItemData->pIImage->grafPort()->draw( IGRect2D(0,0,
                                                               bmpSize.width()+4,
                                                               bmpSize.height()+4),
                                                      bundle );

    //Next, draw the bitmap onto the solid color square area.  Shift it to
    //the lower right area, leaving 4 pixels of background color on top and
    //left.
    //if style is set and button has a transparentcolor
    if (this->pSourceBtn->hasTransparentColor())
      fDMTBarButtonItemData->pIImage->setTransparentColor(this->pSourceBtn->transparentColor());
    IGrafMatrix    matrix;
    matrix.translateBy( IGPoint2D(4,4) );
    fDMTBarButtonItemData->pIImage->grafPort()->draw( bitmap, bundle, matrix );

    IDMImage image =
       IDMImage( fDMTBarButtonItemData->pIImage->image()->handle() );

    setImage( image );
  }
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::IDMTBarButtonItem                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IDMTBarButtonItem::IDMTBarButtonItem( const IDMItem::Handle& pTBarItem )  :
                          IDMItem(pTBarItem),
                          fDMTBarButtonItemData(0)
{
  IFUNCTRACE_DEVELOP();
  this->pSourceBtn = (IToolBarButton *)sourceWindow();
  this->pTargetBtn = (IToolBarButton *)targetOperation()->targetWindow();
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::~IDMTBarButtonItem                                        |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMTBarButtonItem :: ~IDMTBarButtonItem( )
{
  IFUNCTRACE_DEVELOP();
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::generateSourceItems                                       |
|                                                                              |
| Gives IDMTBarButtonItem an opportunity to attach new IDMTBarButtonItem's to  |
| the argument IDMSourceOperation object.  This function is called when a drag |
| operation commences over an IDMTBarButtonItem object.  Returns true if items |
| were provided, false otherwise.                                              |
------------------------------------------------------------------------------*/
bool IDMTBarButtonItem ::
               generateSourceItems( IDMSourceOperation* pDMSrcOp )
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Create the toolbar button item                                   */
  /********************************************************************/
  IDMTBarButtonItem::Handle pTBarBtnItem(new IDMTBarButtonItem( pDMSrcOp )) ;

  /********************************************************************/
  /* Add it to the source operation.                                  */
  /********************************************************************/
  pDMSrcOp->addItem( pTBarBtnItem );

  /********************************************************************/
  /* Set the image style.                                             */
  /********************************************************************/
  pDMSrcOp->setImageStyle( IDM::allStacked );

  /********************************************************************/
  /* Set the default operation.                                       */
  /********************************************************************/
  pDMSrcOp->setOperation( IDMOperation::move );

  /********************************************************************/
  /* Indicate that the toolbar button drag item was successfully      */
  /* created.                                                         */
  /********************************************************************/
  return( true );
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::supportedOperationsFor                                    |
|                                                                              |
| Returns operations this item supports using the specified RMFs.              |
------------------------------------------------------------------------------*/
unsigned long IDMTBarButtonItem ::
              supportedOperationsFor( const IString& rmf ) const
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* If using file rendering mechanism, then only copy is supported   */
  /* Else, whatever the base class supports is used.                  */
  /********************************************************************/
  if (rmf == IDMItem::rmfFrom( IDM::rmFile, IDM::rfText ))
    return( IDMItem::copyable & supportedOperations() );
  else
    return( Inherited::supportedOperationsFor( rmf ) );
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::targetDrop                                                |
|                                                                              |
| Sets button bitmap and text into the target toolbar button,  or moves the    |
| toolbar button to a new position within the toolbar.                         |
------------------------------------------------------------------------------*/
bool IDMTBarButtonItem :: targetDrop( IDMTargetDropEvent& evt )
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Get the target button and the toolbar it resides in              */
  /********************************************************************/
  IToolBarButton *pTgtBtn = targetToolBarButton();
  IToolBar *pTgtTBar = targetToolBar();

  /********************************************************************/
  /* If type includes "toolBarButton" then process the move request   */
  /********************************************************************/
  if (types().includes( IDM::toolBarButton ))
  {
    /******************************************************************/
    /* Get the source button and the toolbar it resides in            */
    /******************************************************************/
    IToolBarButton *pSrcBtn = sourceToolBarButton();
    IToolBar *pSrcTBar = sourceToolBar();
    IPoint pt= IWindow::mapPoint(evt.dropPosition(),
                                 IWindow::desktopWindow()->handle(),
                                 pTgtBtn->handle());

    /******************************************************************/
    /* If we're in the same toolbar ...                               */
    /******************************************************************/
    if (pSrcTBar == pTgtTBar)
    {
      /****************************************************************/
      /* Move it after or before the button that was dropped on ...   */
      /****************************************************************/
      if ( pTgtTBar->deckOrientation() == ISetCanvas::vertical )
      {
        if ( pt.y() > (pTgtBtn->size().height()/2) )
          pTgtTBar->moveBefore( pSrcBtn, pTgtBtn );
        else
          pTgtTBar->moveAfter( pSrcBtn, pTgtBtn );
      }
      else
      {
        if ( pt.x() < (pTgtBtn->size().width()/2) )
          pTgtTBar->moveBefore( pSrcBtn, pTgtBtn );
        else
          pTgtTBar->moveAfter( pSrcBtn, pTgtBtn );
      }
    }
#ifdef IC_PMWIN
    else
    {
      /****************************************************************/
      /* Add it to the new toolbar after the button it was dropped on */
      /* and remove it from the old one ...                           */
      /****************************************************************/
      pSrcTBar->remove( pSrcBtn );
      pSrcBtn->setParent( pTgtTBar );
      pSrcBtn->setOwner( pTgtTBar );

      if ( pTgtTBar->deckOrientation() == ISetCanvas::vertical )
      {
        if ( pt.y() > (pTgtBtn->size().height()/2) )
          pTgtTBar->addAsPrevious( pSrcBtn, pTgtBtn );
        else
          pTgtTBar->addAsNext( pSrcBtn, pTgtBtn );
      }
      else
      {
        if ( pt.x() < (pTgtBtn->size().width()/2) )
          pTgtTBar->addAsPrevious( pSrcBtn, pTgtBtn );
        else
          pTgtTBar->addAsNext( pSrcBtn, pTgtBtn );
      }
    }
#endif // IC_PMWIN

    return( true );
  } //end toolBarButton processing

  /********************************************************************/
  /* If type includes "menuItem" then process the menuitem link       */
  /* request.                                                         */
  /********************************************************************/
  if (types().includes( IDM::menuItem ))
  {
    /******************************************************************/
    /* Create the toolbar button and add it after the button that was */
    /* dropped on ...                                                 */
    /******************************************************************/
    IToolBarButton *pButton = new IToolBarButton( (unsigned long)object(),
                                                  pTgtTBar,
                                                  pTgtTBar );
    pButton->setAutoDeleteObject();
    IPoint pt= IWindow::mapPoint(evt.dropPosition(),
                                 IWindow::desktopWindow()->handle(),
                                 pTgtBtn->handle());

    if ( pTgtTBar->deckOrientation() == ISetCanvas::vertical )
    {
      if ( pt.y() > (pTgtBtn->size().height()/2) )
        pTgtTBar->addAsPrevious( pButton, pTgtBtn );
      else
        pTgtTBar->addAsNext( pButton, pTgtBtn );
    }
    else
    {
      if ( pt.x() < (pTgtBtn->size().width()/2) )
        pTgtTBar->addAsPrevious( pButton, pTgtBtn );
      else
        pTgtTBar->addAsNext( pButton, pTgtBtn );
    }

    return( true );
  } //end menuItem processing

  /********************************************************************/
  /* If type includes "bitmap" or "plainText" then process the bitmap */
  /* file copy request ...                                            */
  /********************************************************************/
  if ((types().includes( IDM::bitmap ))  ||
      (types().includes( IDM::plainText )))
  {
    /******************************************************************/
    /* Attempt to load the bitmap from a system file                  */
    /******************************************************************/
    IString fname = containerName() + sourceName();
    // IImage  image( fname, true, IImage::kNoDither );
    // const IGImage &bitMap = *image.image();
    IGImage bitMap( IGImagePixelAccessor::loadFromFile( fname ) );

    /******************************************************************/
    /* Set the new bitmap into the button that was dropped on ...     */
    /******************************************************************/
    pTgtBtn->setBitmap( bitMap.handle() );

    return( true );
  } //end bitmap file processing

  return( false );
}


/*------------------------------------------------------------------------------
| IDMTBarButtonItem::sourceDiscard                                             |
|                                                                              |
| Called by the source handler to allow deletion of the toolbar button after   |
| it has been dropped on the shredder.                                         |
------------------------------------------------------------------------------*/
bool IDMTBarButtonItem :: sourceDiscard( IDMSourceDiscardEvent& evt )
{
  IFUNCTRACE_DEVELOP();

#ifdef IC_PM
  IToolBarButton *pTBarBtn = sourceToolBarButton();
  sourceToolBar()->remove( pTBarBtn );
  IDESTROYWINDOW( pTBarBtn->handle() );
#endif // IC_PM

  return( true );
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::sourceToolBar                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar* IDMTBarButtonItem :: sourceToolBar( ) const
{
  IFUNCTRACE_DEVELOP();

  return( (IToolBar *)sourceToolBarButton()->parent() );
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::targetToolBar                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar* IDMTBarButtonItem :: targetToolBar( ) const
{
  IFUNCTRACE_DEVELOP();

  return( (IToolBar *)targetToolBarButton()->parent() );
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::sourceToolBarButton                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarButton* IDMTBarButtonItem :: sourceToolBarButton( ) const
{
  IFUNCTRACE_DEVELOP();

  return( this->pSourceBtn );
}

/*------------------------------------------------------------------------------
| IDMTBarButtonItem::targetToolBarButton                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarButton* IDMTBarButtonItem :: targetToolBarButton( ) const
{
  IFUNCTRACE_DEVELOP();

  return( this->pTargetBtn );
}

