// Revision: 17 1.3.2.2 source/ui/drag/imarsha.cpp, dragdrop, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: imarsha.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMMarshall                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// Tracing functions
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Exception handling
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Include our classes's stuff
#ifndef _IMARSHA_
  #include <imarsha.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMARSHA_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IDMMarshall::~IDMMarshall                                                    |
------------------------------------------------------------------------------*/
IDMMarshall :: ~IDMMarshall ()
{}

/*------------------------------------------------------------------------------
| IDMMarshall::marshall                                                        |
|                                                                              |
| Create a string representation of a string, starting at outStr[startPos].    |
| Return startPos + the length of the string representation.                   |
|                                                                              |
| Strings are represented by a string representing the length                  |
| of the string, followed by the contents of the string.                       |
| Strings can contain embedded zero chars.                                     |
|                                                                              |
| The return value is calculated correctly even if outStr is 0.                |
------------------------------------------------------------------------------*/
unsigned long IDMMarshall :: marshall ( const IString& istr,
                                      unsigned long startPos,
                                      char* outStr )
{
 unsigned long
 newStartPos = IDMMarshall::marshall( (long) istr.length(), startPos, outStr );
 if ( outStr && istr.length() )
   memcpy( &outStr[newStartPos], (const char*)istr , istr.length() );
 return newStartPos + istr.length();
}

/*------------------------------------------------------------------------------
| IDMMarshall::marshall                                                        |
|                                                                              |
| Create a string representation of a number, starting at outStr[startPos].    |
| Return startPos + the length of the string representation.                   |
|                                                                              |
| Numbers are represented by an IString representation, followed by a zero     |
| char.                                                                        |
|                                                                              |
| The return value is calculated correctly even if outStr is 0.                |
------------------------------------------------------------------------------*/
unsigned long IDMMarshall :: marshall ( long num,
                                      unsigned long startPos,
                                      char* outStr )
{
 IString strNum( num );
 if ( outStr )
   memcpy( &outStr[startPos], (const char*)strNum, strNum.length()+1 );
 return startPos + strNum.length() + 1;
}

/*------------------------------------------------------------------------------
| IDMMarshall::unmarshall                                                  |
|                                                                              |
| Initialize an IString from a string representation previously created        |
| using marshall.                                                              |
|                                                                              |
| Return inStr +                                                               |
| the number of characters used from the string representation.                |
------------------------------------------------------------------------------*/
const char* IDMMarshall :: unmarshall ( IString& outStr, const char* inStr )
{
 long result;
 const char*
 newStr = IDMMarshall::unmarshall( result, inStr );
 unsigned long strLen = (unsigned long)result;

 if ( strLen )
  { // This technique avoids constructing a temp IString, for performance.
   outStr.overlayWith( "x", strLen);
   if ( outStr.length() > strLen )
     outStr.remove( strLen+1 );
   memcpy( (char*)outStr, newStr, strLen );
  }
 return newStr + strLen;
}

/*------------------------------------------------------------------------------
| IDMMarshall::unmarshall                                                      |
|                                                                              |
| Initialize a number from a string representation previously created          |
| using marshall.                                                              |
|                                                                              |
| Return inStr +                                                               |
| the number of characters used from the string representation.                |
------------------------------------------------------------------------------*/
const char* IDMMarshall :: unmarshall ( long& outNum, const char* inStr )
{
 IString strNum( inStr );
 outNum = strNum.asInt();
 return &inStr[strNum.length() + 1];
}
