// Revision: 22 1.11.1.3 source/ui/extapp/ianimbut.cpp, buttonctls, ioc.v400, 000919 
/*******************************************************************************
* FILE NAME: ianimbut.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ianimbut.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define INCL_WINBUTTONS
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#include <ianimbut.hpp>
#include <icustbhd.hpp>
#include <icustbev.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <igrport.hpp>
#include <iimage.hpp>
#include <iinhratt.hpp>
#include <ireslib.hpp>
#include <ifont.hpp>
#include <itimer.hpp>
#include <istring.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IANIMBUT_CPP_
  #include <ipagetun.h>
#endif

class IAnimatedButtonData : public ICustomButtonDrawHandler {
typedef ICustomButtonDrawHandler
  Inherited;
public:
  IAnimatedButtonData ( IAnimatedButton* aButton );
  ~IAnimatedButtonData ( );
IAnimatedButton
 *button;
unsigned long
  rate,
  index,
  count;
IBitmapHandle
 *bitmaps;
ITimer
 *timer;
void
  timerExpired ( unsigned long timerId );
IPoint
  bitmapPos;
protected:
void
  drawForeground ( ICustomButtonDrawEvent& event );
};

IAnimatedButtonData :: IAnimatedButtonData ( IAnimatedButton *aButton )
                     : button ( aButton ),
                       rate ( 1000 ),
                       index ( 0 ),
                       count ( 0 ),
                       bitmaps ( 0 ),
                       timer ( 0 )
{
}

IAnimatedButtonData :: ~IAnimatedButtonData ( )
{
}

void IAnimatedButtonData :: drawForeground ( ICustomButtonDrawEvent& event )
{
  IRectangle rect = event.drawingArea();

  // Draw the current bitmap
  if ( bitmaps != 0 )
  {
    IImage
      bmp( bitmaps[ index ] );

    IGRect2D
      bitmapSize = bmp.geometricBounds();

    // Center bitmap
    bitmapPos.setX(rect.left()+((rect.width()-bitmapSize.width())/2));
    if ( event.customButton()->text() != "" )
      bitmapPos.setY(rect.top()-bitmapSize.height());
    else
      bitmapPos.setY(rect.bottom()+((rect.height()-bitmapSize.height())/2));

    // Draw the bitmap
    bmp.moveTo( IGPoint2D( bitmapPos.x(), bitmapPos.y() ) );
    bmp.draw( event.grafPort() );

    // Remove the bitmap area from the drawing area
    event.setDrawingArea ( IRectangle ( rect.bottomLeft(),
                           ISize ( rect.width(), bitmapPos.y() )));
  }

  // Draw text (if it exists)
  if ( event.customButton()->text() != "" )
  {
    Inherited::drawForeground(event);
  }

  // Restore drawing area rectangle (since we may have changed it)
  event.setDrawingArea ( rect );
}

void IAnimatedButtonData :: timerExpired ( unsigned long timerId )
{
  index++;
  if ( index >= count )
    index = 0;

  if ( button )
  {
    IManagedPresSpaceHandle
      presSpace( button );
    IRootGrafPort
      gc( presSpace );
    IImage
      bmp( bitmaps[ index ] );

    bmp.moveTo( IGPoint2D( bitmapPos.x(), bitmapPos.y() ) );
    bmp.draw( gc );
  }
}

/*------------------------------------------------------------------------------
| IAnimatedButton styles                                                       |
------------------------------------------------------------------------------*/
const IAnimatedButton::Style
  IAnimatedButton::animateWhenLatched ( 0, IBS_ANIMATEDLATCHED ),
  IAnimatedButton::classDefaultStyle  ( WS_VISIBLE);

  IAnimatedButton::Style IAnimatedButton::fgCurrentDefaultStyle ( WS_VISIBLE);

/*------------------------------------------------------------------------------
| IAnimatedButton ctor                                                         |
------------------------------------------------------------------------------*/
IAnimatedButton :: IAnimatedButton (unsigned long id,
                                    IWindow* parent,
                                    IWindow* owner,
                                    const IRectangle& initial,
                                    const Style& style )
                 : fAnimatedButtonData(0)
{
  IASSERTPARM(parent!=0);

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  fAnimatedButtonData = new IAnimatedButtonData(this);

  IWindowHandle hOwner = (owner == 0) ? IWindowHandle(0)
                                       : owner->handle();

  IWindowHandle whButton =
      this -> create( id,
                      0,
                      convertToGUIStyle( style ),
                      WC_BUTTON,
                      parent->handle(),
                      hOwner,
                      initial,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( style, true ) );

  startHandlingEventsFor(whButton);

  // Add handler
  fAnimatedButtonData->handleEventsFor(this);

#ifdef IC_MOTIFWIN
  // Add the inheritColor attribute to the window.  IAnimatedButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );
#endif // IC_MOTIFWIN
#ifdef IC_WIN
  setColor( PP_BACKGROUNDCOLOR, IColor(IColor::kButtonMiddle));
#endif // IC_WIN
}

/*------------------------------------------------------------------------------
| ICustomButton (wrapper) ctor                                                 |
------------------------------------------------------------------------------*/
IAnimatedButton :: IAnimatedButton (const IWindowHandle& handle )
                 : fAnimatedButtonData(0)
{
  setAutoDestroyWindow ( false );
  fAnimatedButtonData = new IAnimatedButtonData(this);
  reserveUserWindowWord( false );
  startHandlingEventsFor(handle);

#ifdef IC_MOTIFWIN
  // Add the inheritColor attribute to the window.  IAnimatedButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );

#endif // IC_MOTIFWIN
}


/*------------------------------------------------------------------------------
| IAnimatedButton dtor                                                         |
------------------------------------------------------------------------------*/
IAnimatedButton :: ~IAnimatedButton()
{
  if (fAnimatedButtonData)
  {
    fAnimatedButtonData->stopHandlingEventsFor(this);
    if ( fAnimatedButtonData->timer)
    {
      fAnimatedButtonData->timer->stop();
      delete fAnimatedButtonData->timer;
    }
    if ( fAnimatedButtonData->bitmaps )
    {
      delete [] fAnimatedButtonData->bitmaps;
    }
    delete fAnimatedButtonData;
  }
}

/*------------------------------------------------------------------------------
| IAnimatedButton::defaultStyle                                                |
------------------------------------------------------------------------------*/
IAnimatedButton::Style IAnimatedButton :: defaultStyle()
{
  return fgCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IAnimatedButton::setDefaultStyle                                             |
------------------------------------------------------------------------------*/
void IAnimatedButton :: setDefaultStyle(const Style& style)
{
  fgCurrentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IAnimatedButton::convertToGUIStyle                                           |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IAnimatedButton :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   bool bExtOnly) const
{
  // Obtain the style from the class (ICustomButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: setBitmaps ( const IResourceId& firstBitmap,
                                                 unsigned long count )
{
  if ( fAnimatedButtonData->bitmaps )
    delete [] fAnimatedButtonData->bitmaps;

  fAnimatedButtonData->count = 0;
  fAnimatedButtonData->index = 0;
  if ( count )
    fAnimatedButtonData->bitmaps = new IBitmapHandle[count];
  else
    fAnimatedButtonData->bitmaps = 0;

  int x;
  for ( x = 0; x < count; x++ )
  {
    fAnimatedButtonData->bitmaps[x] = firstBitmap.resourceLibrary()
                                     .loadBitmap(firstBitmap+x);
  }

  fAnimatedButtonData->count = count;
  setLayoutDistorted(IWindow::minimumSizeChanged,0);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: setBitmaps ( AnimatedBitmaps bitmaps )
{
  switch (bitmaps)
  {
    case play:
         setBitmaps(IResourceId(IC_PLAY1,
                                IApplication::current().resourceLibrary()),
                    5);
         break;
    case pause:
         setBitmaps(IResourceId(IC_PAUSE,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case rewind:
         setBitmaps(IResourceId(IC_REWIND,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case stop:
         setBitmaps(IResourceId(IC_STOP,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case fastForward:
         setBitmaps(IResourceId(IC_FF,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case record:
         setBitmaps(IResourceId(IC_RECORD,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case mute:
         setBitmaps(IResourceId(IC_MUTE,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case volumeUp:
         setBitmaps(IResourceId(IC_VOLUP,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case volumeDown:
         setBitmaps(IResourceId(IC_VOLDN,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case trackAdvance:
         setBitmaps(IResourceId(IC_TRKA,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case trackReverse:
         setBitmaps(IResourceId(IC_TRKR,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case stepForward:
         setBitmaps(IResourceId(IC_STEPF,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case stepBackward:
         setBitmaps(IResourceId(IC_STEPB,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    case scanForward:
         setBitmaps(IResourceId(IC_SCANF1,
                                IApplication::current().resourceLibrary()),
                    6);
         break;
    case scanBackward:
         setBitmaps(IResourceId(IC_SCANB1,
                                IApplication::current().resourceLibrary()),
                    6);
         break;
    case eject:
         setBitmaps(IResourceId(IC_EJECT,
                                IApplication::current().resourceLibrary()),
                    1);
         break;
    default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
  } /* endswitch */
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IBitmapHandle IAnimatedButton :: bitmap ( unsigned long index ) const
{
  if ( index < fAnimatedButtonData->count )
    return fAnimatedButtonData->bitmaps[index];
  else
    return 0;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

unsigned long IAnimatedButton :: bitmapCount () const
{
  return fAnimatedButtonData->count;
}


/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

unsigned long IAnimatedButton :: currentBitmapIndex () const
{
  return fAnimatedButtonData->index;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: setCurrentBitmapIndex ( unsigned long index )
{
  if ( index < fAnimatedButtonData->count )
  {
    fAnimatedButtonData->index = index;
  }
  else
  {
    ITHROWLIBRARYERROR1(IC_INVALID_INDEX,
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable,
                        IString(index));
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

unsigned long IAnimatedButton :: animationRate () const
{
  return fAnimatedButtonData->rate;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: setAnimationRate ( unsigned long newRate )
{
  fAnimatedButtonData->rate = newRate;
  if ( fAnimatedButtonData->timer )
  {
    fAnimatedButtonData->timer->setInterval(fAnimatedButtonData->rate);
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: startAnimation ( unsigned long index )
{
  if ( isAnimationStarted() )
    stopAnimation();

  if ( index < fAnimatedButtonData->count )
  {
    fAnimatedButtonData->index = index;
    fAnimatedButtonData->timer = new ITimer ( new ITimerMemberFn<IAnimatedButtonData>
                                            ( *fAnimatedButtonData,
                                              &IAnimatedButtonData::timerExpired ),
                                              fAnimatedButtonData->rate );
  }
  else
  {
    ITHROWLIBRARYERROR1(IC_INVALID_INDEX,
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable,
                        IString(index));
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: stopAnimation ()
{
  if ( fAnimatedButtonData->timer != 0 )
  {
    fAnimatedButtonData->timer->stop();
    delete fAnimatedButtonData->timer;
    fAnimatedButtonData->timer = 0;
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IAnimatedButton::isAnimationStarted                                          |
------------------------------------------------------------------------------*/
bool IAnimatedButton :: isAnimationStarted () const
{
  return( (fAnimatedButtonData->timer) ? true : false );
}

/*------------------------------------------------------------------------------
| IAnimatedButton::isAnimateLatched                                            |
------------------------------------------------------------------------------*/
bool IAnimatedButton :: isAnimatedWhenLatched () const
{
  ITRACE_DEVELOP(IString("Extended style =")+IString(extendedStyle()).d2x());
  return( (extendedStyle() & animateWhenLatched.asExtendedUnsignedLong())
                           ? true : false );
}

/*------------------------------------------------------------------------------
| IAnimatedButton::enableAnimateWhenLatched                                    |
------------------------------------------------------------------------------*/
IAnimatedButton& IAnimatedButton :: enableAnimateWhenLatched ( bool enable )
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  if ( enable )
  {
    ulExtStyle |= animateWhenLatched.asExtendedUnsignedLong();
    if ( isLatched() && !isAnimationStarted() )
      startAnimation ( currentBitmapIndex() );
  }
  else
  {
    ulExtStyle &= ~animateWhenLatched.asExtendedUnsignedLong();
    if ( isLatched() && isAnimationStarted() )
      stopAnimation ( );
  }

  ITRACE_DEVELOP(IString("Old extended style =")+IString(ulOldExtStyle).d2x());
  ITRACE_DEVELOP(IString("New extended style =")+IString(ulExtStyle).d2x());
  if ( ulExtStyle != ulOldExtStyle )
  {
    setExtendedStyle(ulExtStyle);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IAnimatedButton::disableAnimateWhenLatched                                   |
------------------------------------------------------------------------------*/
IAnimatedButton& IAnimatedButton :: disableAnimateWhenLatched ( )
{
  enableAnimateWhenLatched( false );
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IAnimatedButton& IAnimatedButton :: latch ( bool latched,
                                            bool refresh )
{
  Inherited::latch( latched, refresh );
  if ( isAnimatedWhenLatched() )
  {
    if ( latched && !isAnimationStarted() )
      startAnimation ( currentBitmapIndex() );
    if ( !latched && isAnimationStarted() )
    {
      stopAnimation ();
      setCurrentBitmapIndex ( 0 );
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

ISize IAnimatedButton :: calcMinimumSize () const
{
  ISize sz;

  // Get border sizes to be used later

  if ( text() != "" )
  {
    sz = Inherited::calcMinimumSize();
  }
  else
  {
    unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
    unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);
    sz.setWidth ( cxBorder * 5 );
    sz.setHeight ( cyBorder * 5 );
  }

  if ( fAnimatedButtonData->bitmaps )
  {
    IGRect2D
      bounds = IImage( fAnimatedButtonData->bitmaps[0] ).geometricBounds();
    ISize
      bitmapSize( (long)bounds.width(), (long)bounds.height() );
    unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);

    if ( sz.width() < ( bitmapSize.width() + ( cxBorder * 5 )))
    {
      sz.setWidth ( bitmapSize.width() + ( cxBorder * 5 ));
    }
    sz.setHeight ( sz.height() + bitmapSize.height() );
  }

  return sz;
}

