// Revision: 31 1.9.1.3 source/ui/extapp/iclipimp.cpp, clipboard, ioc.v400, 000919  
/*******************************************************************************
* FILE NAME: iclipimp.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iclipimp.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

#include <iclipimp.hpp>
#include <iclipbrd.hpp>
#include <ithread.hpp>
#include <iexcept.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <icconst.h>
#include <iseq2.h>

  
#define INCL_WINATOM
#define INCL_DOSMEMMGR
#define INCL_WINCLIPBOARD
#define INCL_GPI
extern "C" {
   #include <iwindefs.h>
   #ifdef IC_PM
     #include <bsememf.h>
   #endif
}

#ifdef IC_WIN
  #include <string.h>
#endif

#ifdef IC_MOTIF
  #include <Xm/CutPaste.h>
  #include <ixevtids.h>
  #include <imstring.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICLIPIMP_CPP_
  #include <ipagetun.h>
#endif



/*------------------------------------------------------------------------------
| Constant Data                                                                |
------------------------------------------------------------------------------*/
#ifdef IC_PM
const char
          * const IClipboard::textFormat(SZFMT_TEXT),
          * const IClipboard::displayTextFormat(SZFMT_DSPTEXT),
          * const IClipboard::bitmapFormat(SZFMT_BITMAP),
          * const IClipboard::displayBitmapFormat(SZFMT_DSPBITMAP),
          * const IClipboard::metafileFormat(SZFMT_METAFILE),
          * const IClipboard::displayMetafileFormat(SZFMT_DSPMETAFILE),
          * const IClipboard::paletteFormat(SZFMT_PALETTE);
#endif

#ifdef IC_WIN
         // We use the character representation of the predefined formats
         // since there are no predefined atoms for these formats.
         // This design allows the user to use this class with predefined
         // formats we don't explictly support with code like
         //  IString format(CF_WAVE);
         //  clipboard.setData( format, data, databytes );
         // Warning:  need to keep values in sync with SDK !!!!
const char
          * const IClipboard::textFormat("1"),              //CF_TEXT
          * const IClipboard::displayTextFormat("129"),     //CF_DSPTEXT
          * const IClipboard::bitmapFormat("2"),            //CF_BITMAP
          * const IClipboard::displayBitmapFormat("130"),   //CF_DSPBITMAP
          * const IClipboard::metafileFormat("14"),         //CF_ENHMETAFILE
          * const IClipboard::displayMetafileFormat("142"), //CF_DSPENHMETAFILE
          * const IClipboard::paletteFormat("9");           //CF_PALETTE
#endif

#ifdef IC_MOTIF
const char
          * const IClipboard::textFormat("STRING"),
          * const IClipboard::bitmapFormat("PIXMAP");
#endif

#ifdef IC_MOTIFWIN
#pragma enum(4)
#pragma pack(push,4)

class IClipboardImpDeleteList : public ISequence<void*>
{
public:
  IClipboardImpDeleteList( );
  virtual ~IClipboardImpDeleteList( );
};

#pragma pack(pop)
#pragma enum(pop)

IClipboardImpDeleteList::IClipboardImpDeleteList( )
 : ISequence<void*>(5) 
{
  IFUNCTRACE_DEVELOP(); 
}

IClipboardImpDeleteList::~IClipboardImpDeleteList( ) {}

#endif

/*------------------------------------------------------------------------------
| IClipboardImp::IClipboardImp                                                 |
|                                                                              |
| Constructor for base implementation class.                                   |
------------------------------------------------------------------------------*/
IClipboardImp::IClipboardImp(const IWindowHandle& owner)
  : fOwnerHandle(owner),
    fOpened(false),
    fAutoOpened(false)
{
   IFUNCTRACE_DEVELOP();    
}


/*------------------------------------------------------------------------------
| IClipboardImp::!IClipboardImp                                                |
|                                                                              |
| Destructor for base implementation class.                                    |
------------------------------------------------------------------------------*/
IClipboardImp::~IClipboardImp( )
{ 
   IFUNCTRACE_DEVELOP();   
}

/*------------------------------------------------------------------------------
| IClipboardImp::autoOpen                                                      |
|                                                                              |
| If the clipboard is not open, open it and set the autoOpen flag so we        |
| know that we forced the clipboard open.                                      |
------------------------------------------------------------------------------*/
void IClipboardImp::autoOpen( )
{
  IFUNCTRACE_DEVELOP();
  if (!isOpen())
  {
    openClipboard();
    setAutoOpen(true);
  }
}
/*------------------------------------------------------------------------------
| IClipboardImp::open                                                          |
|                                                                              |
| If the clipboard is not open, open it.                                       |
------------------------------------------------------------------------------*/
void IClipboardImp::open( )
{
  IFUNCTRACE_DEVELOP();
  if (!isOpen())
  {
    openClipboard();
    setAutoOpen(false);
  }
}
/*------------------------------------------------------------------------------
| IClipboardImp::close                                                         |
|                                                                              |
| If the clipboard is open, close it.                                          |
------------------------------------------------------------------------------*/
void IClipboardImp::close( )
{
  IFUNCTRACE_DEVELOP();
  if (isOpen())
  {
    closeClipboard();
  }
}

/*------------------------------------------------------------------------------
| IClipboardImp::autoClose                                                     |
|                                                                              |
| If we forced the clipboard open, close it.                                   |
------------------------------------------------------------------------------*/
void IClipboardImp::autoClose( )
{
  IFUNCTRACE_DEVELOP();
  if(isAutoOpen())
    closeClipboard();
}


///////////////////// Windows /////////////////////////


#ifdef IC_WIN



/*------------------------------------------------------------------------------
| IWindowsClipboardImp::IWindowsClipboardImp                                   |
|                                                                              |
| Constructor for Windows implementation.                                      |
------------------------------------------------------------------------------*/
IWindowsClipboardImp::IWindowsClipboardImp(const IWindowHandle& owner )
      : IClipboardImp(owner),
        fDeleteList( new IClipboardImpDeleteList)
{
   IFUNCTRACE_DEVELOP();   
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::~IWindowsClipboardImp                                  |
|                                                                              |
| Destructor for Windows implementation.                                       |
------------------------------------------------------------------------------*/
IWindowsClipboardImp::~IWindowsClipboardImp( )
{
  IFUNCTRACE_DEVELOP();
  cleanUpDeleteList();
  delete fDeleteList;
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::addToDeleteList                                        |
|                                                                              |
| Add a memory buffer to the delete list so that we can delete the             |
| memory when the clipboard is closed.                                         |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::addToDeleteList( void* memoryToDelete)
{
  IFUNCTRACE_DEVELOP();
  fDeleteList->addAsLast(memoryToDelete);
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::cleanUpDeleteList                                      |
|                                                                              |
| Free the memory in the delete list and remove the items from the list.       |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::cleanUpDeleteList( )
{
  IFUNCTRACE_DEVELOP();
  if(fDeleteList->numberOfElements() != 0)
  {
     IClipboardImpDeleteList::Cursor cursor (*fDeleteList);
     for(cursor.setToFirst(); cursor.isValid(); cursor.setToFirst())
     {
        void* currentBuffer = fDeleteList->elementAt(cursor);
        fDeleteList->removeAt(cursor);
        ::GlobalUnlock(currentBuffer);
     }
  }
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::registerFormat                                         |
|                                                                              |
| Register a private format.                                                   |
------------------------------------------------------------------------------*/
unsigned long IWindowsClipboardImp::registerFormat( const char* formatName,
                                                    unsigned long )
{
  IFUNCTRACE_DEVELOP();
  unsigned long format = RegisterClipboardFormat(formatName);

  return format;
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::clipboardHasFormat                                     |
|                                                                              |
| Return true if the clipboard contains data with the format name.             |
------------------------------------------------------------------------------*/
bool IWindowsClipboardImp::clipboardHasFormat( const char* formatName )
{
  IFUNCTRACE_DEVELOP();

  if(formatName == 0)
      return (itemCount() != 0);

  unsigned long formatHandle = atomFromName(formatName);
  bool fExists = IsClipboardFormatAvailable( (UINT)formatHandle );

  return fExists;
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::openClipboard                                          |
|                                                                              |
| Open the clipboard.                                                          |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::openClipboard( )
{
  IFUNCTRACE_DEVELOP();
  bool successful = OpenClipboard(owner());
  if (!successful)
       ITHROWGUIERROR("OpenClipboard");
  setOpen(true);

}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::closeClipboard                                         |
|                                                                              |
| Close the clipboard.                                                         |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::closeClipboard( )
{
  IFUNCTRACE_DEVELOP();
  bool successful = CloseClipboard();
  cleanUpDeleteList();

  if (!successful)
  {
     IGUIErrorInfo errorString("CloseClipboard");
     ITRACE_RUNTIME(errorString);
  }

  setOpen(false);
  setAutoOpen(false);
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::emptyClipboard                                         |
|                                                                              |
| Clear the clipboard.                                                         |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::emptyClipboard( )
{
  IFUNCTRACE_DEVELOP();
  autoOpen();

  bool successful = EmptyClipboard();


  if (!successful)
     ITHROWGUIERROR("EmptyClipboard");

  autoClose();

}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::retrieveData                                           |
|                                                                              |
| Get data of the specified format from the clipboard.                         |
------------------------------------------------------------------------------*/
const void* IWindowsClipboardImp::retrieveData( const char*    formatName,
                                                unsigned long& bytesReturned)
{
  IFUNCTRACE_DEVELOP();
  unsigned long formatHandle = atomFromName(formatName);
  bytesReturned = 0;
  void* pData = 0;

  // Get the data from the clipboard
  HANDLE globalHandle = GetClipboardData( (UINT)formatHandle);

  // If we didn't succeed and the clipboard hasn't been opened,
  // open it and try again
  if (!globalHandle && !isOpen())
  {
    autoOpen();
    globalHandle = GetClipboardData( (UINT)formatHandle);
  }

  if(globalHandle)
  {
    bytesReturned = GlobalSize(globalHandle);
    ITRACE_DEVELOP(IString("Bytes returned: ")+IString(bytesReturned));
    pData = ::GlobalLock(globalHandle);
    addToDeleteList(pData);
  }

  return pData;
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::copyData                                               |
|                                                                              |
| Put data of the specified format onto the clipboard.                         |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::copyData( const char*   formatName,
                                     const void*   buffer,
                                     unsigned long bufferSize)
{
  IFUNCTRACE_DEVELOP();

  // Allocate shared memory for the data
  void* memory=0;
  HGLOBAL memoryHandle = 0;

  if(bufferSize !=0)
  {

    IASSERTPARM(buffer!=0);
    memoryHandle =
                ::GlobalAlloc(GMEM_DDESHARE | GMEM_MOVEABLE, bufferSize+1);
    if (!memoryHandle)
      ITHROWGUIERROR("GlobalAlloc");

    memory = ::GlobalLock(memoryHandle);

    // Copy to shared memory and ensure '0' termination
    memcpy(memory, buffer, (size_t)bufferSize);
    ((char*)memory)[bufferSize] = '\0';

    ::GlobalUnlock(memoryHandle);
  }


  unsigned long formatHandle = atomFromName(formatName);

  HANDLE  successful = SetClipboardData((UINT)formatHandle, memoryHandle);

  // If we didn't succeed and the clipboard hasn't been opened,
  // open it and try again
  if (!successful && !isOpen())
  {
     autoOpen();
     successful = SetClipboardData((UINT)formatHandle, memoryHandle);

     if ( (!successful) && (!memoryHandle) )
         successful = (void*)1;        // avoid throw on delayed rendering
  }

  autoClose();

  if(!successful)
    ITHROWGUIERROR("SetClipboardData");
}


/*------------------------------------------------------------------------------
| IWindowsClipboardImp::copyDelayedData                                        |
|                                                                              |
| Put data of the specified format onto the clipboard.                         |
|                                                                              |
| Note: There is a special case of getting data from the clipboard             |
| during the processing of the WM_RENDERFMT or WM_RENDERALLFMTS                |
| messages.  We don't want the clipboard open since it might already           |
| be open by the process requesting the data.  We work around this             |
| by first writting the data without opening the clipboard.  If                |
| this fails, we open and try again.                                           |
|                                                                              |
| Note: We've added support to IClipboardHandler to request a copy of the      |
| data during delayed rendering.  It is implemented here (and in OS/2) by      |
| a call to copyDelayedData.  In Tuscany, we will document that the handler    |
| function is the only way to copy the data from the handler.  In the          |
| follow-on release we can then change the logic in copyData/copyDataDelayed   |
| such that copyData doesn't need to worry about not opening the clipboard.    |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::copyDelayedData( const char*   formatName,
                                            const void*   buffer,
                                            unsigned long bufferSize,
                                            IEvent&       event)
{
   IFUNCTRACE_DEVELOP();
   copyData(formatName, buffer, bufferSize);
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::copyHandle                                             |
|                                                                              |
| Puts the handle of the specified format on the clipboard.                    |
|                                                                              |
------------------------------------------------------------------------------*/
void IWindowsClipboardImp::copyHandle( const char*   formatName,
                                       unsigned long dataHandle)
{
  IFUNCTRACE_DEVELOP();

  // Try to set the clipboard data
  unsigned long formatHandle = atomFromName(formatName);
  HANDLE  successful = SetClipboardData((UINT)formatHandle,
                                        (HANDLE)dataHandle );

  // If we didn't succeed and the clipboard hasn't been opened,
  // open it and try again
  if (!successful && !isOpen())
  {
    autoOpen();
    successful = SetClipboardData((UINT)formatHandle,
                                  (HANDLE)dataHandle );
  }

  // Close the clipboard if we opened it.
  autoClose();

  if(!successful)
    ITHROWGUIERROR("SetClipboardData");
}


/*------------------------------------------------------------------------------
| IWindowsClipboardImp::formatForItem                                          |
|                                                                              |
| Return the format name of the passed item.  The itemHandle is how we         |
| identify the data on the clipboard.  On Windows and OS/2, this it is the     |
| same as the atom for the clipboard format.  On Motif, it is the index of the |
| item in the clipboard. We use the handle for returning the data via a cursor.|
------------------------------------------------------------------------------*/
IString IWindowsClipboardImp::formatForItem( unsigned long itemHandle)
{
  IFUNCTRACE_DEVELOP();
  char buffer[256];

  int count = GetClipboardFormatName((UINT)itemHandle, buffer, sizeof(buffer));
  /*--------------------------------------------------
  | If zero length, we must have a pre-defined format.
  | Convert the handle to its displayable
  | equivalent and return that as the format name.
  --------------------------------------------------*/
  if (!count)
    return publicFormat( itemHandle);

  return IString(buffer);
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::formatForAtom                                          |
|                                                                              |
| Return the format name of the passed atom. On Windows, OS/2, and Motif       |
| formats are registered and stored as atoms. Windows and OS/2 externalize     |
| the atom as the format handle needed for clipboard operations. Motif         |
| use the format name for clipboard operations.                                |
------------------------------------------------------------------------------*/
IString IWindowsClipboardImp::formatForAtom( unsigned long atom )
{
  // On Windows, the handle is the atom
  return formatForItem(atom);
}


/*------------------------------------------------------------------------------
| IWindowsClipboardImp::atomFromName                                           |
|                                                                              |
| Return the atom for the passed format name.  Windows and OS/2 externalize    |
| the atom as the format handle needed for clipboard operations. Motif         |
| use the format name for clipboard operations.                                |
------------------------------------------------------------------------------*/
unsigned long IWindowsClipboardImp::atomFromName( const char* formatName)
{
  // First, attempt to convert the format string to a handle and check it
  // for the predefined formats.  If one is specified return the
  // appropriate flag for use with the clipboard functions.   Otherwise,
  // look for a registered format.
  unsigned long formatHandle = IString(formatName).asUnsigned();

  switch ( formatHandle )
     {
     case CF_TEXT           :
     case CF_BITMAP         :
     case CF_METAFILEPICT   :
     case CF_SYLK           :
     case CF_DIF            :
     case CF_TIFF           :
     case CF_OEMTEXT        :
     case CF_DIB            :
     case CF_PALETTE        :
     case CF_PENDATA        :
     case CF_RIFF           :
     case CF_WAVE           :
     case CF_UNICODETEXT    :
     case CF_ENHMETAFILE    :
     case CF_OWNERDISPLAY   :
     case CF_DSPTEXT        :
     case CF_DSPBITMAP      :
     case CF_DSPMETAFILEPICT:
     case CF_DSPENHMETAFILE :
        // Handle predefined formats
        break;
     default:
        // If the handle is not in one of the reserved ranges assume its
        // non numeric or a numeric value which is a registered format name
        if (!(((formatHandle >= CF_PRIVATEFIRST) &&
               (formatHandle <= CF_PRIVATELAST ))
                           ||
              ((formatHandle >= CF_GDIOBJFIRST) &&
               (formatHandle <= CF_GDIOBJLAST )) ))
           {
           // In Windows, the registration function will return the
           // handle of a previously registered format.  If the format
           // is not registered, this has the effect of registering it.
           // Thats OK since a subsequent attempt at registration by
           // the app will get the same format handle.
           formatHandle = registerFormat( formatName );
           }        // if
     }      // switch

return formatHandle;
}

/*------------------------------------------------------------------------------
| IWindowsClipboardImp::itemCount                                              |
|                                                                              |
| Return the nunber of items on the clipboard.                                 |
------------------------------------------------------------------------------*/
unsigned long IWindowsClipboardImp::itemCount( )
{
  IFUNCTRACE_DEVELOP();

  autoOpen();

  /*-------------------------------------------------------
  | Enumerate the formats. Accumulate a total for the
  | function return value.
  -------------------------------------------------------*/
  unsigned long count = 0;
  unsigned long formatHandle = 0;
  do {

    formatHandle = EnumClipboardFormats((UINT)formatHandle);
   if (formatHandle)
      count++;
  }
  while (formatHandle!=0);

  autoClose();


  ITRACE_DEVELOP(IString("Item count is: ")+IString(count));

  return count;

}
/*------------------------------------------------------------------------------
| IWindowsClipboardImp::nextItemHandle                                         |
|                                                                              |
| Returns the item handle for the clipboard item following the passed item or  |
| 0 if no item follows this item on the clipboard.                             |
------------------------------------------------------------------------------*/
unsigned long IWindowsClipboardImp::nextItemHandle( unsigned long priorItemHandle)
{
  return (EnumClipboardFormats((UINT)priorItemHandle));
}


/*------------------------------------------------------------------------------
| IWindowsClipboardImp::realOwner                                              |
|                                                                              |
| Returns the real owner of the clipboard.                                     |
------------------------------------------------------------------------------*/
IWindowHandle IWindowsClipboardImp::realOwner( )
{
  return IWindowHandle(GetClipboardOwner());
}


/*-----------------------------------------------------------------------
| Private static helper. Used by format(...) member functions.
|
| Input: pre-defined Win32 clipboard format.
| Output: name of format rendered as handle-converted-to-string.
|
| Throws recoverable library IC_CLIPBOARDFORMATNOTFOUND exception
| if unknown format.
-----------------------------------------------------------------------*/
IString IWindowsClipboardImp::publicFormat( unsigned long formatHandle )
{
   IString result;

   switch ( formatHandle )
   {
      case CF_TEXT           :
      case CF_BITMAP         :
      case CF_METAFILEPICT   :
      case CF_SYLK           :
      case CF_DIF            :
      case CF_TIFF           :
      case CF_OEMTEXT        :
      case CF_DIB            :
      case CF_PALETTE        :
      case CF_PENDATA        :
      case CF_RIFF           :
      case CF_WAVE           :
      case CF_UNICODETEXT    :
      case CF_ENHMETAFILE    :
      case CF_OWNERDISPLAY   :
      case CF_DSPTEXT        :
      case CF_DSPBITMAP      :
      case CF_DSPMETAFILEPICT:
      case CF_DSPENHMETAFILE :
         // Process predefined formats
         result = IString( formatHandle );
         break;

      default:
         ITHROWLIBRARYERROR(IC_CLIPBOARDFORMATNOTFOUND,
                            IBaseErrorInfo::invalidRequest,
                            IException::recoverable);
         break;
    }
    return result;
}

#endif


//////////////////////  OS/2  /////////////////////////

#ifdef IC_PM

/////////////////// Known Problem
// For some reason there is a problem with delayed rendering within
// a single process.  It looks like, the setData() of the delayed renderer
// gives the data to the shell and never gets it back on the data() side.

IOS2ClipboardImp::IOS2ClipboardImp( const IWindowHandle& owner )
  : IClipboardImp(owner)
{
   IFUNCTRACE_DEVELOP();   
}

IOS2ClipboardImp::~IOS2ClipboardImp( )
{
   IFUNCTRACE_DEVELOP();
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::registerFormat                                             |
|                                                                              |
| Register a private format.                                                   |
------------------------------------------------------------------------------*/
unsigned long IOS2ClipboardImp::registerFormat( const char* formatName,
                                                unsigned long )
{
  IFUNCTRACE_DEVELOP();
  unsigned long format = WinAddAtom(WinQuerySystemAtomTable(), formatName);

  return format;
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::clipboardHasFormat                                         |
|                                                                              |
| Return true if the clipboard contains data with the format name.             |
------------------------------------------------------------------------------*/
bool IOS2ClipboardImp::clipboardHasFormat( const char* formatName )
{
  IFUNCTRACE_DEVELOP();

  if(formatName == 0)
     return (itemCount() != 0);

  unsigned long formatHandle = atomFromName(formatName);
  unsigned long usageFlags;
  bool fExists = (bool)WinQueryClipbrdFmtInfo(
                             IThread::current().anchorBlock(),
                             formatHandle,
                             &usageFlags);

  return fExists;
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::openClipboard                                              |
|                                                                              |
| Open the clipboard.                                                          |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::openClipboard   ( )
{
  IFUNCTRACE_DEVELOP();
  bool successful = (bool)WinOpenClipbrd( IThread::current().anchorBlock());

  if (!successful)
    ITHROWGUIERROR("WinOpenClipbrd");
  setOpen(true);
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::closeClipboard                                             |
|                                                                              |
| Close the clipboard.                                                         |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::closeClipboard( )
{
  IFUNCTRACE_DEVELOP();
  bool successful = (bool)WinCloseClipbrd(IThread::current().anchorBlock());

  if (!successful)
  {
     IGUIErrorInfo errorString("WinCloseClipbrd");
     ITRACE_RUNTIME(errorString);
  }

  setOpen(false);
  setAutoOpen(false);
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::emptyClipboard                                             |
|                                                                              |
| Clear the clipboard.                                                         |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::emptyClipboard( )
{
  IFUNCTRACE_DEVELOP();
  autoOpen();

  bool successful = (bool)WinEmptyClipbrd(IThread::current().anchorBlock());

  if (!successful)
     ITHROWGUIERROR("WinEmptyClipbrd");

  // Note: in Win32, the owner is automatically estabished on empty.  To keep
  // things the same, we also set the owner in PM
  successful = (bool)WinSetClipbrdOwner(
                            IThread::current().anchorBlock(),
                            owner());
  if (!successful)
     ITHROWGUIERROR("WinSetClipbrdOwner");

  autoClose();

}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::retrieveData                                               |
|                                                                              |
| Get data of the specified format from the clipboard.                         |
|                                                                              |
| Note:  On OS/2 we receive global access to the data until the clipboard      |
|        is closed.  It is the callers responsibility to copy the data         |
|        before closing the clipboard.  Here, we open the clipboard if we      |
|        need to and leave it open.                                            |
------------------------------------------------------------------------------*/
const void* IOS2ClipboardImp::retrieveData( const char*    formatName,
                                            unsigned long& bytesReturned)
{
  IFUNCTRACE_DEVELOP();
  unsigned long formatHandle = atomFromName(formatName);
   // Note: It looks like it might be possible to use DosQueryMem to
   //       find the length of the memory object returned.
  bytesReturned = 0;

  // Get the data from the clipboard
  void* pData = (void*)WinQueryClipbrdData
                        (IThread::current().anchorBlock(),
                         formatHandle);

  // If we didn't succeed and the clipboard hasn't been opened,
  // open it and try again
  if (!pData && !isOpen())
  {
    autoOpen();
    pData = (void*)WinQueryClipbrdData
                         (IThread::current().anchorBlock(),
                         formatHandle);
  }
  if(pData)
  {
    unsigned long memorySize = 0x3FFFFFFF;
    unsigned long memoryFlags = 0;
    DosQueryMem(pData, &memorySize, &memoryFlags);
    ITRACE_DEVELOP(IString("Bytes returned: ")+IString(memorySize));
  }

  return pData;
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::copyData                                                   |
|                                                                              |
| Put data of the specified format onto the clipboard.                         |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::copyData( const char*   formatName,
                                 const void*   buffer,
                                 unsigned long bufferSize)
{
  IFUNCTRACE_DEVELOP();

  // Allocate shared memory for the data
  void* memory=0;

  if(bufferSize !=0)
  {
    IASSERTPARM(buffer!=0);
    unsigned long rc = DosAllocSharedMem(
                          &memory, 
                          0,
                          bufferSize+1,
                          PAG_WRITE | PAG_COMMIT | OBJ_GIVEABLE);
    if (rc)
    {
      ITHROWSYSTEMERROR(rc, "DosAllocSharedMem",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
    }

    // Copy to shared memory and ensure '0' termination
    memcpy(memory, buffer, (size_t)bufferSize);
    ((char*)memory)[bufferSize] = '\0';
  }


  // Note there is a special case of getting data from the clipboard
  // during the processing of the WM_RENDERFMT or WM_RENDERALLFMTS
  // messages.  We don't want the clipboard open since it might already
  // be open by the process requesting the data.  We work around this
  // by first setting the data without opening the clipboard.  If
  // this fails, we open and try again.
  // Note that we added interface in Tuscany that should allow this
  // hack to be removed in the release after Tuscany since 
  // copyDelayedData should be called from the clipboard handler
  // not this routine.
  unsigned long formatHandle = atomFromName(formatName);

  bool successful = (bool)WinSetClipbrdData(
                             IThread::current().anchorBlock(),
                             (unsigned long)memory,
                             formatHandle,
                             CFI_POINTER);
  // If we didn't succeed and the clipboard hasn't been opened,
  // open it and try again
  if (!successful && !isOpen())
  {
    autoOpen();
    successful = (bool)WinSetClipbrdData(
                     IThread::current().anchorBlock(),
                     (unsigned long)memory,
                     formatHandle,
                     CFI_POINTER);
  }

  autoClose();

  if(!successful)
    ITHROWGUIERROR("SetClipboardData");
}


/*------------------------------------------------------------------------------
| IOS2ClipboardImp::copyDelayedData                                            |
|                                                                              |
| Put data of the specified format onto the clipboard.                         |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::copyDelayedData( const char*   formatName,
                                        const void*   buffer,
                                        unsigned long bufferSize,
                                        IEvent&       event)
{
   IFUNCTRACE_DEVELOP();
   copyData(formatName, buffer, bufferSize);
}


/*------------------------------------------------------------------------------
| IOS2ClipboardImp::copyHandle                                                 |
|                                                                              |
| Puts the handle of the specified format on the clipboard.                    |
|                                                                              |
| Note: there is a special case of getting data from the clipboard             |
| during the processing of the WM_RENDERFMT or WM_RENDERALLFMTS                |
| messages.  We don't want the clipboard open since it might already           |
| be open by the process requesting the data.  We work around this             |
| by first writting the data without opening the clipboard.  If                |
| this fails, we open and try again.                                           |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::copyHandle( const char*   formatName,
                                   unsigned long dataHandle)
{
  IFUNCTRACE_DEVELOP();

  // Try to set the clipboard data
  unsigned long formatHandle = atomFromName(formatName);
  bool successful = (bool)WinSetClipbrdData(
                             IThread::current().anchorBlock(),
                             dataHandle,
                             formatHandle,
                             CFI_HANDLE);

  // If we didn't succeed and the clipboard hasn't been opened,
  // open it and try again
  if (!successful && !isOpen())
  {
    autoOpen();
    successful = (bool)WinSetClipbrdData(
                             IThread::current().anchorBlock(),
                             dataHandle,
                             formatHandle,
                             CFI_HANDLE);
  }

  // If we opened the clipboard, close it.
  autoClose();

  if(!successful)
     ITHROWGUIERROR("WinSetClipbrdData");
}


/*------------------------------------------------------------------------------
| IOS2ClipboardImp::formatForItem                                              |
|                                                                              |
| Return the format name of the passed item.  The itemHandle is how we         |
| identify the data on the clipboard.  On Windows and OS/2, this it is the     |
| same as the atom for the clipboard format.  On Motif, it is the index of the |
| item in the clipboard. We use the handle for returning the data via a cursor.|
------------------------------------------------------------------------------*/
IString IOS2ClipboardImp::formatForItem( unsigned long itemHandle)
{
  IFUNCTRACE_DEVELOP();

  char buffer[256];
  unsigned long count = WinQueryAtomName
                ( WinQuerySystemAtomTable(),itemHandle, buffer, sizeof(buffer));
  if(!count)
      ITHROWLIBRARYERROR(IC_CLIPBOARDFORMATNOTFOUND,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);

  return IString(buffer);
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::formatForAtom                                              |
|                                                                              |
| Return the format name of the passed atom. On Windows, OS/2, and Motif       |
| formats are registered and stored as atoms. Windows and OS/2 externalize     |
| the atom as the format handle needed for clipboard operations. Motif         |
| use the format name for clipboard operations.                                |
------------------------------------------------------------------------------*/
IString IOS2ClipboardImp::formatForAtom( unsigned long atom )
{
  // On OS/2, the handle is the atom
  return formatForItem(atom);

}


/*------------------------------------------------------------------------------
| IOS2ClipboardImp::atomFromName                                               |
|                                                                              |
| Return the atom for the passed format name.  Windows and OS/2 externalize    |
| the atom as the format handle needed for clipboard operations. Motif         |
| use the format name for clipboard operations.                                |
| Note:  We duplicate the Windows side-effect of registering a format when     |
|        it is queried if not already done so.                                 |
------------------------------------------------------------------------------*/
unsigned long IOS2ClipboardImp::atomFromName( const char* formatName)
{
  unsigned long formatHandle = WinFindAtom(WinQuerySystemAtomTable(), formatName);
  if(!formatHandle )
     formatHandle = registerFormat( formatName, 8);

  return formatHandle;
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::itemCount                                              |
|                                                                              |
| Return the nunber of items on the clipboard.                                 |
------------------------------------------------------------------------------*/
unsigned long IOS2ClipboardImp::itemCount( )
{
  IFUNCTRACE_DEVELOP();

  autoOpen();

  /*-------------------------------------------------------
  | Enumerate the formats. Accumulate a total for the
  | function return value.
  -------------------------------------------------------*/
  unsigned long count = 0;
  unsigned long formatHandle = 0;
  do {

    formatHandle = WinEnumClipbrdFmts
                        (IThread::current().anchorBlock(),
                         formatHandle);
   if (formatHandle)
      count++;
  }
  while (formatHandle!=0);

  autoClose();

  ITRACE_DEVELOP(IString("Item count is: ")+IString(count));

  return count;

}
/*------------------------------------------------------------------------------
| IOS2ClipboardImp::nextItemHandle                                             |
|                                                                              |
| Returns the item handle for the clipboard item following the passed item or  |
| 0 if no item follows this item on the clipboard.                             |
------------------------------------------------------------------------------*/
unsigned long IOS2ClipboardImp::nextItemHandle( unsigned long priorItemHandle)
{
  return WinEnumClipbrdFmts(IThread::current().anchorBlock(), priorItemHandle);
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::realOwner                                                  |
|                                                                              |
| Returns the real owner of the clipboard.                                     |
------------------------------------------------------------------------------*/
IWindowHandle IOS2ClipboardImp::realOwner( )
{
  return IWindowHandle(WinQueryClipbrdOwner(IThread::current().anchorBlock()));
}

/*------------------------------------------------------------------------------
| IOS2ClipboardImp::setRealOwner                                               |
|                                                                              |
| OS2 only function to set the owner of the clipboard.                         |
------------------------------------------------------------------------------*/
void IOS2ClipboardImp::setRealOwner( )
{
  IFUNCTRACE_DEVELOP();
  bool successful = (bool)WinSetClipbrdOwner(
                          IThread::current().anchorBlock(),
                          owner());
  if(!successful)
     ITHROWGUIERROR("WinSetClipbrdOwner");
}

#endif  // IC_PM

////////////////////// Motif  /////////////////////////

#ifdef IC_MOTIF

IMotifClipboardImp::IMotifClipboardImp (const IWindowHandle& owner)
        : IClipboardImp(owner),
          fDeleteList( new IClipboardImpDeleteList),
          fStartItemId(0),
          fDisplay(XtDisplay( IThread::current().applicationShell())),
          fWindow(0)
{
   IFUNCTRACE_DEVELOP();   

   if(owner == 0)
   {
      if ( !XtIsRealized( IWindow::desktopWindow()->handle() ) )
         XtRealizeWidget( IWindow::desktopWindow()->handle() );
      this->setOwner(IWindow::objectWindow()->handle());
   }

   fWindow = XtWindow(this->owner());

   IASSERTSTATE(fWindow != 0);
   ITRACE_DEVELOP(IString("owner: ")+IString(this->owner().asUnsigned()).d2x()
                          +" fWindow: "+IString((unsigned long)fWindow).d2x());
}

IMotifClipboardImp::~IMotifClipboardImp ( )
{
  IFUNCTRACE_DEVELOP();
  cleanUpDeleteList();
  delete fDeleteList;
}



// The following function is to keep track of memory buffers that
// we need to delete when the clipboard is closed.
void IMotifClipboardImp::addToDeleteList ( void* memoryToDelete)
{
  IFUNCTRACE_DEVELOP();
  fDeleteList->addAsLast(memoryToDelete);
}

void IMotifClipboardImp::cleanUpDeleteList ( )
{
  IFUNCTRACE_DEVELOP();
  if(fDeleteList->numberOfElements() != 0)
  {
     IClipboardImpDeleteList::Cursor cursor (*fDeleteList);
     for(cursor.setToFirst(); cursor.isValid(); cursor.setToFirst())
     {
        char* currentBuffer = (char*)fDeleteList->elementAt(cursor);
        fDeleteList->removeAt(cursor);
        delete [] currentBuffer;
     }
  }

}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::clipboardHasFormat                                       |
|                                                                              |
| Return true if the clipboard contains data with the format name.             |
------------------------------------------------------------------------------*/
bool IMotifClipboardImp::clipboardHasFormat( const char* formatName )
{
  IFUNCTRACE_DEVELOP();

  if(formatName == 0)
     return (itemCount() > 0);

  bool hasFormat = false;
  unsigned long formatDataLength = 0;
  unsigned long rc = XmClipboardInquireLength(fDisplay, fWindow,
                       (char*)formatName,
                       &formatDataLength);
  ITRACE_DEVELOP(IString("XmClipboardInquireLength RC: ")+IString(rc)
                          +" formatName: "+formatName
                          +" formatLength: "+ IString(formatDataLength)
                          +" fWindow: "+IString((unsigned long)fWindow).d2x());

  // Note: The length of the data for a format will be zero if using
  //       delayed rendering and the data is not yet on the clipboard.
  if (formatDataLength > 0)
     hasFormat = true;
  else if (rc == XmClipboardSuccess) 
  {
     unsigned long itemCount = this->itemCount();
     if (itemCount > 0) 
     {
        char buffer[256];
        unsigned long copiedLength = 0;
        unsigned long itemHandle = 0;
        do
        {
           itemHandle++;
           rc = XmClipboardInquireFormat(fDisplay, fWindow,
                              itemHandle,
                              buffer,
                              sizeof(buffer),
                              &copiedLength);

           ITRACE_DEVELOP(IString("XmClipboardInquireFormat: RC: ")+IString(rc)
                          +" item: "+IString(itemHandle)
                          +" format: "+IString(buffer, copiedLength));
           if (copiedLength > 0 && 
               memcmp(buffer, formatName, copiedLength) == 0)
             hasFormat = true;
        }
        while(rc == XmClipboardSuccess && hasFormat == false &&
              itemHandle <= itemCount);
     }
  }
  return hasFormat;
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::registerFormat                                           |
|                                                                              |
| Register a private format.                                                   |
------------------------------------------------------------------------------*/
unsigned long IMotifClipboardImp::registerFormat( 
                                           const char* formatName,
                                           unsigned long elementSizeInBits)
{
  IFUNCTRACE_DEVELOP();
  unsigned long format = 0;

  Display* display = XtDisplay( IThread::current().applicationShell() );
  unsigned long rc = XmClipboardRegisterFormat( display, (char*)formatName, elementSizeInBits);
  ITRACE_DEVELOP(IString("XmClipboardRegisterFormat RC: ")+IString(rc)
                  + " format: "+IString(formatName));

  if (rc==XmClipboardSuccess)
  {
    format = (unsigned long)XInternAtom(
                            display,
                            formatName,
                            false);
    ITRACE_DEVELOP(IString("XInternAtom: format name: ")+formatName
                   +" format id: "+IString(format));
  }

  return format;

}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::openClipboard                                            |
|                                                                              |
| Open the clipboard.                                                          |
|                                                                              |
| Note:  Motif doesn't really support the concept of a generic clipboard       |
|        open.  Here, we just lock the clipboard.                              |
------------------------------------------------------------------------------*/
void IMotifClipboardImp::openClipboard( )
{
  IFUNCTRACE_DEVELOP();

  unsigned long rc = 0;

  // I've tried locking the clipboard on an open call to be more consistent
  // with the behavior in OS/2 and Windows.  Unfortunately, various Motif
  // calls like XmClipboardInquireCount don't work when the clipboard
  // has been explicitly locked.
  //  do
  //    rc = XmClipboardLock(fDisplay, fWindow);
  //  while (rc == ClipboardLocked);

  setOpen(true);
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::closeClipboard                                           |
|                                                                              |
| Close the clipboard.                                                         |
------------------------------------------------------------------------------*/
void IMotifClipboardImp::closeClipboard( )
{
  IFUNCTRACE_DEVELOP();


  if(fStartItemId)
  {
     int rc = 0;
     do
     {
       rc = XmClipboardEndCopy(fDisplay, fWindow, fStartItemId);
       ITRACE_DEVELOP(IString("XmClipboardEndCopy RC: ")+IString(rc)
                      +" fStartItemId: "+IString(fStartItemId).d2x()
                      +" fWindow: "+IString((unsigned long)fWindow).d2x());
     }
     while(rc == ClipboardLocked);

     fStartItemId = 0;
  }
  //  Note: See comments in openClipboard
  //    unsigned long rc = XmClipboardUnlock(fDisplay, fWindow,
  //                          true);

  cleanUpDeleteList();

  setOpen(false);
  setAutoOpen(false);
}

/*------------------------------------------------------------------------------
| iClipboardDelayedRenderingCallback                                           |
|                                                                              |
| Builds and sends an event for delayed rendering.                             |
------------------------------------------------------------------------------*/
void _System iClipboardDelayedRenderingCallback( Widget handle, 
                                                 long*  dataId,
                                                 long*  formatHandle, 
                                                 int*   reason)
{

   IFUNCTRACE_DEVELOP();

   Display* display = XtDisplay( IThread::current().applicationShell());
   IWindow* window = IWindow::windowWithHandle( handle );

   /*-----------------------------------------------------------------
      We create an IEvent as follows:
        handle - the widget parameter on the callback
        eventid - The Motif reason adjusted for motif offsets
        parameter1 - the formatHandle as a privateId 
        parameter2 - the dataId (returned first by XmClipboardStartCopy
        
   -----------------------------------------------------------------*/
   unsigned long eventId = *reason + motifEventOffset;
   ITRACE_DEVELOP(IString("eventId: ")+IString(eventId).d2x()
                  +" window: "+IString((unsigned long)(void*)window).d2x()
                  +" widget: "+IString((unsigned long)handle).d2x()
                  +" dataId: "+IString((unsigned long)*dataId).d2x()
                  +" formatHandle: "+IString((unsigned long)*formatHandle).d2x());
   if ( window != 0 && eventId != 0)
   {
     // Construct an IEvent from the system data.
     IEvent event( window,
                 eventId,
                 (unsigned long)*formatHandle,
                 (unsigned long)*dataId);
 
     // This event should not go up the owner chain
     event.setPassToOwner( false );
 
     // Dispatch the window's event handler
     if ( window->dispatch( event ) != false )
     {
       return;
     }
   } 
 
   return;


}


/*------------------------------------------------------------------------------
| IMotifClipboardImp::emptyClipboard                                           |
|                                                                              |
------------------------------------------------------------------------------*/
void IMotifClipboardImp::emptyClipboard( )
{
   IFUNCTRACE_DEVELOP();

   autoOpen();


   // Motif doesn't support the concept of clearing the clipboard.  Here, we
   // lock the clipboard for writing by calling XmClipboardStartCopy. 
   IMString clipboardLabel("IOC_CLIPBOARD");
   int rc = 0;
   do
   {
     rc = XmClipboardStartCopy
                      ( fDisplay,
                        fWindow,
                        clipboardLabel,
                        CurrentTime,
                        (Widget)owner(),
                        iClipboardDelayedRenderingCallback,
                        &fStartItemId);
     ITRACE_DEVELOP(IString("XmClipboardStartCopy RC: ")+IString(rc)
                    +" fStartItemId: "+IString(fStartItemId).d2x()
                    +" fWindow: "+IString((unsigned long)fWindow).d2x());
   }
   while (rc == ClipboardLocked);

   if (rc!=XmClipboardSuccess)
     ITHROWGUIERROR(IString("XmClipboardStartCopy: RC=")+IString(rc));
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::copyData                                                 |
|                                                                              |
| Put data of the specified format onto the clipboard.                         |
------------------------------------------------------------------------------*/
void IMotifClipboardImp::copyData( const char*   formatName,
                                   const void*   buffer,
                                   unsigned long bufferSize)
{
  IFUNCTRACE_DEVELOP();

  int rc = 0;
  
  // Note: Because we need to provide the format name on 
  // IClipboardHandler::renderFormat, and we don't get it in our
  // Motif callback routine, we add the format handle here in
  // the private data field.

  do
  {
     rc = XmClipboardCopy(  fDisplay,
                            fWindow,
                            fStartItemId,
                            (char*)formatName,
                            XtPointer(buffer),
                            bufferSize,
                            atomFromName(formatName),
                            0);
     ITRACE_DEVELOP(IString("XmClipboardCopy RC: ")+IString(rc)
                    +" formatName: "+formatName
                    +" bufferSize: "+IString(bufferSize)+
                    +" fStartItemId: "+IString(fStartItemId).d2x()
                    +" fWindow: "+IString((unsigned long)fWindow).d2x());
  }
  while (rc == ClipboardLocked);

  if (rc!=XmClipboardSuccess)
    ITHROWGUIERROR(IString("XmClipboardCopy: RC=")+IString(rc));
}


/*------------------------------------------------------------------------------
| IMotifClipboardImp::copyDelayedData                                          |
|                                                                              |
| Put data of the specified format onto the clipboard.                         |
------------------------------------------------------------------------------*/
void IMotifClipboardImp::copyDelayedData  ( const char*   formatName,
                                            const void*   buffer,
                                            unsigned long bufferSize,
                                            IEvent&       event)
{
   IFUNCTRACE_DEVELOP();

   unsigned long itemId = event.parameter2().asUnsignedLong();
   unsigned long  formatHandle = event.parameter1().asUnsignedLong();

   int rc = XmClipboardCopyByName( fDisplay,
                                   fWindow,
                                   itemId,
                                   XtPointer(buffer),
                                   bufferSize,
                                   (long)formatHandle);

   ITRACE_DEVELOP(IString("XmClipboardCopyByName RC: ")+IString(rc)
                  +" itemId: "+IString(itemId).d2x()
                  +" bufferSize: "+IString(bufferSize)+
                  +" formatHandle:: "+IString(formatHandle).d2x()
                  +" fWindow: "+IString((unsigned long)fWindow).d2x());
 
   if (rc!=XmClipboardSuccess)
     ITHROWGUIERROR(IString("XmClipboardCopyByName: RC=")+IString(rc));
   
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::copyHandle                                                 |
|                                                                              |
| Puts the handle of the specified format on the clipboard.                    |
------------------------------------------------------------------------------*/
void IMotifClipboardImp::copyHandle ( const char*   formatName,
                                      unsigned long dataHandle)
{
   IFUNCTRACE_DEVELOP();

   copyData(formatName, (void*)(&dataHandle), sizeof(dataHandle));

}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::retrieveData                                             |
|                                                                              |
| Get data of the specified format from the clipboard.                         |
------------------------------------------------------------------------------*/
const void* IMotifClipboardImp::retrieveData( const char*    formatName,
                                        unsigned long& bytesReturned)
{
   IFUNCTRACE_DEVELOP();
 
  char* pData = 0;
  unsigned long bytesAvailable = 0;
  bytesReturned = 0;
  unsigned long rc = 0;


  // Start clipboard retrieval by locking the clipboard
  do
  {
    rc = XmClipboardStartRetrieve(fDisplay, fWindow, CurrentTime);

    ITRACE_DEVELOP(IString("XmClipboardStartRetrieve RC: ")+IString(rc)
                   +" fWindow: "+IString((unsigned long)fWindow).d2x());
  }
  while (rc == ClipboardLocked);
  
  if(rc==XmClipboardSuccess)
  {
     // Retrieve a some data from the clipboard so that we can
     // inquire how much data is on the clipboard (if delayed rendering
     // is being used, you don't know the amount of data until after
     // you've requested it.  In other words, XmClipboardInquireLength
     // returns zero.).
     #define FIRST_BUFFER_SIZE 256
     char buffer[FIRST_BUFFER_SIZE];
     long privateId;                              
     bytesReturned = 0;
     unsigned long retrieveRC = XmClipboardRetrieve(fDisplay, fWindow,
                                  (char*)formatName,
                                  buffer,
                                  sizeof(buffer),
                                  &bytesReturned,
                                  &privateId);
     ITRACE_DEVELOP(IString("XmClipboardRetrieve RC: ")+IString(retrieveRC)
                           +" formatName: "+formatName
                           +" bufferSize: "+IString(sizeof(buffer))
                           +" bytesReturned: "+IString(bytesReturned)
                           +" fWindow: "+IString((unsigned long)fWindow).d2x());

     rc = XmClipboardInquireLength(fDisplay, fWindow,
                                    (char*)formatName,
                                    &bytesAvailable);

     ITRACE_DEVELOP(IString("XmClipboardInquireLength RC: ")+IString(rc)
                            +" formatName: "+formatName
                            +" bytesAvailable:"+IString(bytesAvailable));
     if(bytesReturned)
     {
        if(bytesAvailable)
        {
          pData = new char[bytesAvailable+1];
          pData[bytesAvailable] = '\0';

        }
        else
        {
          pData = new char[bytesReturned+1];
        }
        memcpy(pData, buffer, bytesReturned);
        pData[bytesReturned] = '\0';
     }
     
     if(retrieveRC == XmClipboardTruncate && bytesAvailable > 0)
     {
        char* pDataCurrent = pData+bytesReturned;
        bytesAvailable = bytesAvailable - bytesReturned;
        retrieveRC = XmClipboardRetrieve(fDisplay, 
                           fWindow,
                           (char*)formatName,
                           pDataCurrent,
                           bytesAvailable,
                           &bytesReturned,
                           &privateId);

        ITRACE_DEVELOP(IString("XmClipboardRetrieve2 RC: ")+IString(retrieveRC)
                              +" formatName: "+formatName
                              +" bytesAvailable: "+IString(bytesAvailable)
                              +" bytesReturned: "+IString(bytesReturned));
     }
     if(retrieveRC != XmClipboardSuccess)
     {
        delete pData;
        pData  = 0;
     }
     rc = XmClipboardEndRetrieve(fDisplay, fWindow);
     ITRACE_DEVELOP(IString("XmClipboardEndRetrieve RC: ")+IString(retrieveRC));
  }
     

  if(pData)
     addToDeleteList(pData);
  else
     ITHROWGUIERROR("XmClipboardRetrieve: no data on clipboard");

  return (void*)pData;
}


/*------------------------------------------------------------------------------
| IMotifClipboardImp::formatForItem                                            |
|                                                                              |
| Return the format name of the passed item.  The itemHandle is how we         |
| identify the data on the clipboard.  On Windows and OS/2, this it is the     |
| same as the atom for the clipboard format.  On Motif, it is the index of the |
| item in the clipboard. We use the handle for returning the data via a cursor.|
------------------------------------------------------------------------------*/
IString IMotifClipboardImp::formatForItem   ( unsigned long itemHandle)
{
  IFUNCTRACE_DEVELOP();

  char buffer[256];
  unsigned long copiedLength = 0;

  unsigned long rc = 0;
  do
  {
    rc = XmClipboardInquireFormat(fDisplay, fWindow,
                        itemHandle,
                        buffer,
                        sizeof(buffer),
                        &copiedLength);
    
    ITRACE_DEVELOP(IString("XmClipboardInquireFormat: RC: ")
                 +IString(rc)+" item: "+IString(itemHandle)
                 +" format: "+IString(buffer, copiedLength)
                 +" fWindow: "+IString((unsigned long)fWindow).d2x());
  }
  while (rc == ClipboardLocked);

  if(rc!=XmClipboardSuccess)
    ITHROWLIBRARYERROR(IC_CLIPBOARDFORMATNOTFOUND,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
  return IString(buffer, copiedLength);

}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::formatForAtom                                            |
|                                                                              |
| Return the format name of the passed atom. On Windows, OS/2, and Motif       |
| formats are registered and stored as atoms. Windows and OS/2 externalize     |
| the atom as the format handle needed for clipboard operations. Motif         |
| use the format name for clipboard operations.                                |
------------------------------------------------------------------------------*/
IString IMotifClipboardImp::formatForAtom   ( unsigned long atom )
{
  IFUNCTRACE_DEVELOP();

  char* atomName = XGetAtomName
                       ( XtDisplay( IThread::current().applicationShell()),
                         atom);

  ITRACE_DEVELOP(IString("XGetAtomName atom: ")+IString(atom).d2x()
                 +" atomName: "+atomName);
  return IString(atomName);

}


/*------------------------------------------------------------------------------
| IMotifClipboardImp::atomFromName                                             |
|                                                                              |
| Return the atom for the passed format name.  Windows and OS/2 externalize    |
| the atom as the format handle needed for clipboard operations. Motif         |
| uses the format name for clipboard operations.                               |
|                                                                              |
| Note:  We duplicate the Windows side-effect of registering a format when     |
|        it is queried if not already done so.                                 |
------------------------------------------------------------------------------*/
unsigned long IMotifClipboardImp::atomFromName    ( const char* formatName)
{
  IFUNCTRACE_DEVELOP();
  // On Motif, you get the atom by re-registering it.

  unsigned long atom = (unsigned long)XInternAtom(
                           fDisplay,
                           formatName,
                           false);

  if(!atom )
     atom = registerFormat( formatName, 8);

  ITRACE_DEVELOP(IString("XInternAtom formatName: ")+formatName
                 +" atom: "+IString(atom));
  return atom;
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::itemCount                                                |
|                                                                              |
| Return the nunber of items on the clipboard.                                 |
------------------------------------------------------------------------------*/
unsigned long IMotifClipboardImp::itemCount( )
{
  IFUNCTRACE_DEVELOP();

  int count = 0;
  unsigned long maxFormatLength = 0;

  unsigned long rc = 0;
  do
  {
    rc = XmClipboardInquireCount(fDisplay, fWindow,
                                   &count,
                                   &maxFormatLength);

    ITRACE_DEVELOP(IString("XmClipboardInquireCount RC: ")+IString(rc)
                   +" itemCount: "+IString(count)
                   +" maxFormatLength: "+IString(maxFormatLength)
                   +" fWindow: "+IString((unsigned long)fWindow).d2x());
  }
  while (rc == ClipboardLocked);


  return (unsigned long)count;
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::nextItemHandle                                           |
|                                                                              |
| Returns the item handle for the clipboard item following the passed item or  |
| 0 if no item follows this item on the clipboard.                             |
------------------------------------------------------------------------------*/
unsigned long IMotifClipboardImp::nextItemHandle ( unsigned long priorItemHandle)
{
  IFUNCTRACE_DEVELOP();

  unsigned long itemHandle = priorItemHandle + 1;
  int count = 0;
  unsigned long maxFormatLength = 0;

  unsigned long rc = 0;
  do
  {
    rc = XmClipboardInquireCount(fDisplay, fWindow,
                                   &count,
                                   &maxFormatLength);

    ITRACE_DEVELOP(IString("XmClipboardInquireCount RC: ")+IString(rc)
                   +" itemCount: "+IString(count)
                   +" maxFormatLength: "+IString(maxFormatLength)
                   +" fWindow: "+IString((unsigned long)fWindow).d2x());
  }
  while (rc == ClipboardLocked);

  if(rc == XmClipboardSuccess)
  {
     char buffer[256];
     buffer[0] = '\0';
     unsigned long copiedLength = 0;
     do
     {
       rc = XmClipboardInquireFormat(fDisplay, fWindow,
                        itemHandle,
                        buffer,
                        sizeof(buffer),
                        &copiedLength);

       ITRACE_DEVELOP(IString("XmClipboardInquireFormat: RC: ")
                  +IString(rc)+" item: "+IString(itemHandle)
                  +" format: "+IString(buffer));
     }
     while (rc == ClipboardLocked);

     if (rc==XmClipboardSuccess && copiedLength > 0)
     {
       return itemHandle;
     }
  }
  
  return 0;
}

/*------------------------------------------------------------------------------
| IMotifClipboardImp::realOwner                                                |
|                                                                              |
| Returns the real owner of the clipboard.                                     |
------------------------------------------------------------------------------*/
IWindowHandle IMotifClipboardImp::realOwner( )
{
  IFUNCTRACE_DEVELOP();

  Atom clipboardAtom = XInternAtom(fDisplay, "CLIPBOARD", False);
  Window window = XGetSelectionOwner(fDisplay, clipboardAtom);
  Widget widget = XtWindowToWidget(fDisplay, window);

  return IWindowHandle(widget);
}


#endif  // IC_MOTIF
