/* Revision: 16 1.3.1.3 source/ui/widgets/canvas.c, cuawidgets, ioc.v400, 000919  */
/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1997, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "canvasp.h"


/*
 * Resources
 */
#define defaultTranslations	_XmManager_defaultTranslations

#define offset(field) XtOffsetOf(XiclCanvasRec, canvas.field)


/* Private data */
static XtGeometryResult GeometryManager(Widget,
                      XtWidgetGeometry *, XtWidgetGeometry *);
static XtGeometryResult QueryGeometry(Widget,
                      XtWidgetGeometry *, XtWidgetGeometry *);
static void Redisplay( Widget, XEvent *, Region );
static void ChangeManaged( Widget );
static XmNavigability WidgetNavigable( 
                        Widget wid) ;
static void ClassInitialize( void ) ;
static void ClassPartInitialize( WidgetClass wc) ;
/****************************************************************
 * Full class record constant
 ****************************************************************/


static XmBaseClassExtRec baseClassExtRec = {
    NULL,
    NULLQUARK,
    XmBaseClassExtVersion,
    sizeof(XmBaseClassExtRec),
    NULL,				/* InitializePrehook	*/
    NULL,				/* SetValuesPrehook	*/
    NULL,				/* InitializePosthook	*/
    NULL,				/* SetValuesPosthook	*/
    NULL,				/* secondaryObjectClass	*/
    NULL,				/* secondaryCreate	*/
    NULL,               		/* getSecRes data	*/
    { 0 },      			/* fastSubclass flags	*/
    NULL,				/* getValuesPrehook	*/
    NULL,				/* getValuesPosthook	*/
    NULL,                               /* ClassPartInitPrehook */
    NULL,                               /* classPartInitPosthook*/
    NULL,                               /* ext_resources        */
    NULL,                               /* compiled_ext_resources*/
    0,                                  /* num_ext_resources    */
    FALSE,                              /* use_sub_resources    */
    WidgetNavigable,                    /* widgetNavigable      */
    NULL                                /* focusChange          */
};


#define Superclass (&xmManagerClassRec)

XiclCanvasClassRec xiclCanvasClassRec = {
  { /* core_class fields    */
    /* superclass       */      (WidgetClass) Superclass,
    /* class_name       */      "XiclCanvas",
    /* widget_size      */      sizeof(XiclCanvasRec),
    /* class_initialize */      ClassInitialize,
    /* class_part_init  */      ClassPartInitialize,
    /* class_inited     */      FALSE,
    /* initialize       */      NULL,
    /* initialize_hook  */      NULL,
    /* realize          */      XtInheritRealize,
    /* actions          */      NULL,
    /* num_actions      */      0,
    /* resources        */      NULL,
    /* num_resources    */      0,
    /* xrm_class        */      NULLQUARK,
    /* compress_motion  */      TRUE,
    /* compress_exposure*/      XtExposeCompressMaximal,
    /* compress_enterleave*/    TRUE,
    /* visible_interest */      FALSE,
    /* destroy          */      NULL,
    /* resize           */      XtInheritResize,
    /* expose           */      Redisplay,
    /* set_values       */      NULL,
    /* set_values_hook  */      NULL,
    /* set_values_almost*/      XtInheritSetValuesAlmost,
    /* get_values_hook  */      NULL,
    /* accept_focus     */      NULL,
    /* version          */      XtVersion,
    /* callback_private */      NULL,
    /* tm_table         */      defaultTranslations,
    /* query_geometry   */      QueryGeometry,
    /* display_accelerator*/    XtInheritDisplayAccelerator,
      (XtPointer)&baseClassExtRec,           /* extension             */
  },{ /* composite_class fields */
    /* geometry_manager */      GeometryManager,
    /* change_managed   */      ChangeManaged,
    /* insert_child     */      XtInheritInsertChild,
    /* delete_child     */      XtInheritDeleteChild,
    /* extension        */      NULL
  },{ /* constraint_class fields */
    /* resources        */      NULL,
    /* num_resources    */      0,
    /* constraint_size  */      0,
    /* initialize       */      NULL,
    /* destroy          */      NULL,
    /* set_values       */      NULL,
    /* extension        */      NULL,
  },{ /* manager_class fields */
    /* default translations */  XtInheritTranslations,
    /* syn_resources        */  NULL,
    /* num_syn_resources    */  0,
    /* syn_cont_resources   */  NULL,
    /* num_syn_cont_resources */ 0,
    /* parent_process       */  XmInheritParentProcess,
    /* extension            */  NULL,
  },{ /* canvas_class fields */
    /* nothing */                0,
  }
};

WidgetClass xiclCanvasWidgetClass =
                 (WidgetClass)&xiclCanvasClassRec;

/*********************************************************************
 *
 * ClassInitialize
 *
 *********************************************************************/
static void 
#ifdef _NO_PROTO
ClassInitialize()
#else
ClassInitialize( void )
#endif /* _NO_PROTO */
{
   _XmRegisterConverters();   /* Register Library Conversion Rtnes */
   _XmRegisterPixmapConverters();

   _XmInitializeExtensions();
   baseClassExtRec.record_type = XmQmotif;
}

/*********************************************************************
 *
 * ClassPartInitialize
 *
 *********************************************************************/
static void 
#ifdef _NO_PROTO
ClassPartInitialize( wc )
        WidgetClass wc ;
#else
ClassPartInitialize(
        WidgetClass wc )
#endif /* _NO_PROTO */
{
  _XmBaseClassPartInitialize(wc);
}

/****************************************************************
 * Private Routines
 ****************************************************************/
static XtGeometryResult
GeometryManager(Widget child              /* child making request */,
                XtWidgetGeometry *request /* requested geometry   */,
                XtWidgetGeometry *reply    /* geometry result      */
                )

{
  XtWidgetGeometry tmp;

  #define Requesting(flag)  (request->request_mode & flag)

  if (Requesting(CWStackMode))
    return (*Superclass->composite_class.geometry_manager)(child, request,reply);
        
  /* Initialize tmp with the current child values.*/
  tmp.x = child->core.x;
  tmp.y = child->core.y;
  tmp.width = child->core.width;
  tmp.height = child->core.height;
  tmp.border_width = child->core.border_width;

  /* Now update tmps values with the requested values. */
  if (Requesting( CWX ) )
  {
    tmp.x = request->x;
    /* printf("requested x %d\n", tmp.x ); */
  }
  if (Requesting( CWY ) )
  {
    tmp.y = request->y;
    /* printf("requested y %d\n", tmp.y ); */
  }
  if (Requesting( CWWidth ) )
  {
    tmp.width = request->width;
    /* printf("requested width %d\n", tmp.height ); */
  }
  if (Requesting( CWHeight ) )
  {
    tmp.height = request->height;
    /* printf("requested height %d\n", tmp.height ); */
  }
  if (Requesting( CWBorderWidth ) )
  {
    tmp.border_width = request->border_width;
    /* printf("requested border_width %d\n", tmp.border_width ); */
  }

  /* printf("\n"); */
  /* Now set the reply values. */
  reply->x = tmp.x;
  reply->y = tmp.y;
  reply->width = tmp.width;
  reply->height = tmp.height;
  reply->border_width = tmp.border_width;
  reply->request_mode = request->request_mode;

  if ( Requesting(XtCWQueryOnly ))
  {
    return XtGeometryYes;
  }
  else
  {    
    /* Seems like I have two choices here to move the child widget. 
       1) Set the child's fields and return XtGeometryYes ( normal way )
       2) Do not set child's fields and call XtConfigureWidget and return
          XtGeometryDone ( brute force )
    */

    child->core.x = tmp.x;
    child->core.y = tmp.y;
    child->core.width = tmp.width;
    child->core.height = tmp.height;
    child->core.border_width = tmp.border_width;
  
    return XtGeometryYes;                       
/*    
    XtConfigureWidget( child,
      tmp.x,
      tmp.y,
      tmp.width,
      tmp.height,
      tmp.border_width );

    return XtGeometryDone;
*/
  } 
  #undef Requesting
}

static void
Redisplay( Widget w, XEvent *event, Region region )
{
#if (XmVersion < 1002 )
  _XmRedisplayGadgets((CompositeWidget)w, (XExposeEvent)event, region );
#else
  _XmRedisplayGadgets(w, event, region);
#endif
}

static void
ChangeManaged( Widget w )
{                             
  /* for Motif keyboard focus navigation */
  _XmNavigChangeManaged(w);
}

static XtGeometryResult 
QueryGeometry(Widget w,
              XtWidgetGeometry *intended, 
              XtWidgetGeometry *preferred)
{
  return XtGeometryYes;
}

/*
 * public convenience Canvas creation function
 */
Widget XiclCreateCanvas(Widget   parent,
                      char*    name,
                      ArgList  arglist,
                      Cardinal argcount)
  {
  Widget canvas;

  canvas = XtCreateWidget(name,
                          xiclCanvasWidgetClass,
                          parent,
                          arglist,
                          argcount);
  return canvas;
  }

static XmNavigability
#ifdef _NO_PROTO
WidgetNavigable( wid)
        Widget wid ;
#else
WidgetNavigable(
        Widget wid)
#endif /* _NO_PROTO */
{   
//    printf("WidgetNavigable\n");
  // if we have children then use the version from Manager.c
  if ( ((XiclCanvasWidget)wid)->composite.num_children )
  {
    if ( XtIsSensitive(wid)
         &&  ((XiclCanvasWidget) wid)->manager.traversal_on    )
    {
      XmNavigationType nav_type = ((XiclCanvasWidget) wid)->manager.navigation_type ;
      if ( ( nav_type == XmSTICKY_TAB_GROUP) ||
	         ( nav_type == XmEXCLUSIVE_TAB_GROUP) ||
	         (( nav_type == XmTAB_GROUP )
  		        &&  !_XmShellIsExclusive( wid)))
      {
	     return XmDESCENDANTS_TAB_NAVIGABLE ;
      }
      return XmDESCENDANTS_NAVIGABLE ;
    }
    return XmNOT_NAVIGABLE ;
  }
  else
  // use a version from pushbutton
  {
    if(    XtIsSensitive(wid)
           &&  ((XiclCanvasWidget) wid)->manager.traversal_on    )
    {
	    XmNavigationType nav_type = ((XiclCanvasWidget) wid)->manager.navigation_type ;
	    if ( ( nav_type == XmSTICKY_TAB_GROUP) ||
	         ( nav_type == XmEXCLUSIVE_TAB_GROUP) ||
	         ( (nav_type == XmTAB_GROUP) &&
		         !_XmShellIsExclusive( wid)) )
  		{
        return XmTAB_NAVIGABLE ;
		  }
  	  return XmCONTROL_NAVIGABLE ;
	  }
    return XmNOT_NAVIGABLE ;
  }
  return XmNOT_NAVIGABLE ;
}


