/* Revision: 22 1.6.1.6 source/ui/widgets/icong.c, cuawidgets, ioc.v400, 000919 
/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1997, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
/*
 * (c) Copyright 1989, 1990, 1991, 1992, 1993 OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * Motif top of 2.0 tree as of Tue Jun  1 05:18:29 EDT 1993
 */
/*
 * IconG.c: The IconGadget widget methods.
 */

#include "icongp.h"
#include <Xm/XmosP.h>
#include <Xm/RepType.h>
#include "cnr.h"

#ifndef MIN
#define MIN(x,y)        ((x) < (y) ? (x) : (y))
#endif /* MIN */
#ifndef MAX
#define MAX(x,y)  ((x) > (y) ? (x) : (y))
#endif /* MAX */

/********    Static Function Declarations    ********/
        /* RectObj methods */
static  void                    ClassPartInitialize(
                                        WidgetClass     wc);
static  void                    Initialize(
                                        Widget          rw,
                                        Widget          nw,
                                        ArgList         args,
                                        Cardinal        *num_args);
static  void                    Destroy(
                                        Widget  wid);
static  void                    Redisplay(
                                        Widget          wid,
                                        XEvent          *event,
                                        Region          region);
static  Boolean                 SetValues(
                                        Widget          cw,
                                        Widget          rw,
                                        Widget          nw,
                                        ArgList         args,
                                        Cardinal        *num_args);
static  XtGeometryResult        QueryGeometry(
                                        Widget          wid,
                                        XtWidgetGeometry *intended,
                                        XtWidgetGeometry *reply);
        /* XmGadget methods */
static  Boolean                 VisualChange(
                                        Widget          wid,
                                        Widget          cmw,
                                        Widget          nmw);
static  void                    InputDispatch(
                                        Widget          wid,
                                        XEvent          *event,
                                        Mask            event_mask);
static  Boolean                 XmIconGadgetGetBaselines(
                                        Widget          wid,
                                        Dimension       **baselines,
                                        int             *line_count);
static  Boolean                 XmIconGadgetGetDisplayRect(
                                        Widget          wid,
                                        XRectangle      *displayrect);
        /* XmIconGadget methods */
        /* Action procs */
        /* Internal functions */
static  void                    GetLabelString(
                                        Widget          wid,
                                        int             offset,
                                        XtArgVal        *value);
static  Boolean                 GetLargeIcon(
                                        Widget          wid);
static  Boolean                 GetSmallIcon(
                                        Widget          wid);
static  void                    UpdateGCs(
                                        Widget          wid);
static  void                    GetLabelSize(
                                        Widget          wid);
static  void                    GetLargeIconSize(
                                        Widget          wid);
static  void                    GetSmallIconSize(
                                        Widget          wid);
static  void                    Layout(
                                        Widget          wid,
                                        Dimension       *ideal_width,
                                        Dimension       *ideal_height);
static  void                    Help(
                                        Widget          wid,
                                        XEvent          *event);

static Boolean                 ShowLabel(
                                        Widget          wid);
static Boolean                 ShowIcon (
                                        Widget          wid);
static  void                   InUseEmphasisCB(
                                        Widget          wid,
                                        XEvent          *event,
                                        unsigned char   inuse_change_type) ;

        /* External function declarations are in IconG.h */

/********    End Static Function Declarations    ********/

static  XtResource              resources[] =
        {
                {
                XmNfontList,XmCFontList,XmRFontList,
                        sizeof(XmFontList),
                XtOffset(XmIconGadget,icong.font_list),
                        XmRImmediate,(XtPointer)NULL},
                {
                XmNlabelString,XmCXmString,XmRXmString,sizeof(_XmString),
                XtOffset(XmIconGadget,icong._label),
                        XmRImmediate,(XtPointer)NULL},
                {
                XmNlargeIcon,XmCIcon,XmRString,sizeof(String),
                XtOffset(XmIconGadget,icong.large_icon),
                        XmRImmediate,(XtPointer)NULL},
                {
                XmNlargeIconMask,XmCIconMask,XmRPixmap,sizeof(Pixmap),
                XtOffset(XmIconGadget,icong.large_icon_mask),
                        XmRImmediate,(XtPointer)XmUNSPECIFIED_PIXMAP},
                {
                XmNlargeIconPixmap,XmCIconPixmap,XmRPixmap,sizeof(Pixmap),
                XtOffset(XmIconGadget,icong.large_icon_pixmap),
                        XmRImmediate,(XtPointer)XmUNSPECIFIED_PIXMAP},
                {
                XmNrecomputeSize,XmCRecomputeSize,XmRBoolean,sizeof(Boolean),
                XtOffset(XmIconGadget,icong.recompute_size),
                        XmRImmediate,(XtPointer)True},
                {
                XmNrefreshGadget,XmCRefreshGadget,XmRBoolean,sizeof(Boolean),
                XtOffset(XmIconGadget,icong.refreshOn),
		          XmRImmediate,(XtPointer)True},
                {
                XmNsmallIcon,XmCIcon,XmRString,sizeof(String),
                XtOffset(XmIconGadget,icong.small_icon),
                        XmRImmediate,(XtPointer)NULL},
                {
                XmNsmallIconMask,XmCIconMask,XmRPixmap,sizeof(Pixmap),
                XtOffset(XmIconGadget,icong.small_icon_mask),
                        XmRImmediate,(XtPointer)XmUNSPECIFIED_PIXMAP},
                {
                XmNsmallIconPixmap,XmCIconMask,XmRPixmap,sizeof(Pixmap),
                XtOffset(XmIconGadget,icong.small_icon_pixmap),
                        XmRImmediate,(XtPointer)XmUNSPECIFIED_PIXMAP},
                {
                XmNviewType,XmCViewType,XmRViewType,sizeof(unsigned char),
                XtOffset(XmIconGadget,icong.viewtype),
                        XmRImmediate,(XtPointer)NULL},
                {
                XmNvisualEmphasis,XmCVisualEmphasis,XmRVisualEmphasis,
                sizeof(unsigned char),
                        XtOffset(XmIconGadget,icong.visual_emphasis),
                        XmRImmediate,(XtPointer)XmNOT_SELECTED},
                {
                XmNcursorEmphasis,XmCCursorEmphasis,XmRCursorEmphasis,
                sizeof(unsigned char),
                        XtOffset(XmIconGadget,icong.cursor_emphasis),
                        XmRImmediate,(XtPointer)XmNOT_CURSORED},
                {
                XmNinUseEmphasis,XmCInUseEmphasis,XmRInUseEmphasis,
                sizeof(unsigned char),
                        XtOffset(XmIconGadget,icong.inuse_emphasis),
                        XmRImmediate,(XtPointer)XmNOT_INUSE},
                {
                XmNinUseEmphasisCallback,XmCCallback, XmRCallback,
                sizeof(XtCallbackList),
                        XtOffset(XmIconGadget,icong.inuse_emphasis_cb),
                        XmRImmediate, (XtPointer) NULL},
        };

static  XmSyntheticResource     syn_resources[] =
        {
                {
                XmNlabelString,sizeof(_XmString),
                XtOffset(XmIconGadget,icong._label),GetLabelString,NULL},
        };
static  String  ViewTypeNames[] =
{       "large_icon", "small_icon"
        };
XmRepTypeId     XmRID_VIEW_TYPE;

static  String  VisualEmphasisNames[] =
{       "selected", "not_selected"
        };
XmRepTypeId     XmRID_VISUAL_EMPHASIS;

static  String  CursorEmphasisNames[] =
      {       "cursored", "not_cursored"
        };
XmRepTypeId     XmRID_CURSOR_EMPHASIS;
static  String  InUseEmphasisNames[] =
      {       "inuse", "not_inuse"
        };
XmRepTypeId     XmRID_INUSE_EMPHASIS;

/* This is the class record that gets set at compile/link time  */
/* this is what is passed to the widget create routine as the   */
/* the class.  All fields must be inited at compile time.       */

XmGadgetClassExtRec _XmIconGadClassExtRec = {
    NULL,
    NULLQUARK,
    XmGadgetClassExtVersion,
    sizeof(XmGadgetClassExtRec),
    XmIconGadgetGetBaselines,                   /* widget_baseline */
    XmIconGadgetGetDisplayRect,                 /* widget_display_rect */
};


externaldef( xmicongadgetclassrec) XmIconGadgetClassRec xmIconGadgetClassRec =
{       /* RectObjClassPart */
        {
                (WidgetClass) &xmGadgetClassRec, /* superclass          */
                "XmIconGadget",                 /* class_name           */
                sizeof (XmIconGadgetRec),       /* widget_size          */
                NULL,                           /* class_initialize     */
                ClassPartInitialize,            /* class_part_initialize*/
                False,                          /* class_inited         */
                Initialize,                     /* initialize           */
                NULL,                           /* initialize_hook      */
                NULL,                           /* realize              */
                NULL,                           /* actions              */
                0,                              /* num_actions          */
                resources,                      /* resources            */
                XtNumber (resources),           /* num_resources        */
                NULLQUARK,                      /* xrm_class            */
                True,                           /* compress_motion      */
                True,                           /* compress_exposure    */
                True,                           /* compress_enterleave  */
                False,                          /* visible_interest     */
                Destroy,                        /* destroy              */
                NULL,                           /* resize               */
                Redisplay,                      /* expose               */
                SetValues,                      /* set_values           */
                NULL,                           /* set_values_hook      */
                XtInheritSetValuesAlmost,       /* set_values_almost    */
                NULL,                           /* get_values_hook      */
                NULL,                           /* accept_focus         */
                XtVersion,                      /* version              */
                NULL,                           /* callback private     */
                NULL,                           /* tm_table             */
                QueryGeometry,                  /* query_geometry       */
                NULL,                           /* display_accelerator  */
                NULL,                           /* extension            */
        },

        /* XmGadget Class Part */
        {
        XmInheritBorderHighlight,               /* border_highlight     */
        XmInheritBorderUnhighlight,             /* border_unhighlight   */
        NULL,                                   /* arm_and_activate     */
        InputDispatch,                          /* input_dispatch       */
        VisualChange,                           /* visual_change        */
        syn_resources,                          /* get_resources        */
        XtNumber(syn_resources),                /* num_get_resources    */
        NULL,                                   /* class_cache_part     */
        (XtPointer)&_XmIconGadClassExtRec,      /* extension            */
        },
};

externaldef(xmicongadgetclass) WidgetClass
        xmIconGadgetClass=(WidgetClass)&xmIconGadgetClassRec;

/*----------------
| RectObj methods |
----------------*/
/************************************************************************
 * ClassInitialize              - NULL
 ************************************************************************/

/************************************************************************
 * ClassPartInitialize
 *      Parms(IconGadgetClass)
 *              returns void
 *
 *      Set Motif Fast subclass initialize bit.
 ************************************************************************/
static void
ClassPartInitialize(
        WidgetClass     wc)
{
        XmRID_VIEW_TYPE = XmRepTypeRegister(XmRViewType,
                ViewTypeNames,NULL,XtNumber(ViewTypeNames));
        XmRID_VISUAL_EMPHASIS = XmRepTypeRegister(XmRVisualEmphasis,
                VisualEmphasisNames,NULL,XtNumber(VisualEmphasisNames));
        XmRID_CURSOR_EMPHASIS = XmRepTypeRegister(XmRCursorEmphasis,
                CursorEmphasisNames,NULL,XtNumber(CursorEmphasisNames));
        XmRID_INUSE_EMPHASIS = XmRepTypeRegister(XmRInUseEmphasis,
                InUseEmphasisNames,NULL,XtNumber(InUseEmphasisNames));
}

/************************************************************************
 * Initialize
 ************************************************************************/
static  void
Initialize(
        Widget          rw,
        Widget          nw,
        ArgList         args,
        Cardinal        *num_args)
{
        XmIconGadget    req_ig = (XmIconGadget)rw;
        XmIconGadget    new_ig = (XmIconGadget)nw;
        unsigned char   ancestor_view_type = XmLARGE_ICON;
        Dimension       ideal_width, ideal_height;

#ifdef DEBUG
printf("Initialize\n");
#endif /* DEBUG */


        XtVaGetValues(XtParent(nw),XmNentryViewType,&ancestor_view_type,NULL);
        if (ancestor_view_type != XmDETAIL)
                IG_ViewType(nw) = ancestor_view_type;

        /* XmNviewType */
        if (!XmRepTypeValidValue(XmRID_VIEW_TYPE,IG_ViewType(nw),nw))
                IG_ViewType(nw) = XmLARGE_ICON;
        /* XmNvisualEmphasis */
        if (!XmRepTypeValidValue(XmRID_VISUAL_EMPHASIS,
                                                IG_VisualEmphasis(nw),nw))
                IG_VisualEmphasis(nw) = XmNOT_SELECTED;
        /* XmNcursoredEmphasis */
        if (!XmRepTypeValidValue(XmRID_CURSOR_EMPHASIS,
                                                IG_CursorEmphasis(nw),nw))
                IG_CursorEmphasis(nw) = XmNOT_CURSORED;
        /* XmNinUseEmphasis  */
        if (!XmRepTypeValidValue(XmRID_INUSE_EMPHASIS,
                                                IG_InUseEmphasis(nw),nw))
                IG_InUseEmphasis(nw) = XmNOT_INUSE;

        /* XmNrenderTable */
        if (IG_FontList(nw) == NULL)
                {
                XmFontList      defaultFont = NULL;

                XtVaGetValues(XtParent(nw),XmNfontList,&defaultFont,NULL);
                if (defaultFont == NULL)
                        defaultFont = _XmGetDefaultFontList(nw,
                                                XmLABEL_FONTLIST);
                IG_FontList(nw) = XmFontListCopy(defaultFont);
                }
        else
                IG_FontList(nw) = XmFontListCopy(IG_FontList(nw));

        /* XmNlabelString */
        if (_XmStringIsXmString((XmString)IG__LabelString(nw)))
                IG__LabelString(nw) = _XmStringCreate((XmString)
                                                IG__LabelString(nw));
        else
                {
                XmString        string;

                string = _XmOSGetLocalizedString((char *)NULL, /* reserved */
                                nw,XmNlabelString,
                                XrmQuarkToString(IG_Xrm_Name(nw)));
                IG__LabelString(nw) = _XmStringCreate(string);
                XmStringFree(string);
                }

        /* XmNlargeIcon */
        GetLargeIcon(nw);

        /* XmNsmallIcon */
        GetSmallIcon(nw);


        Layout(nw,&ideal_width,&ideal_height);

        if (IG_RecomputeSize(nw) || req_ig->rectangle.width == 0)
                new_ig->rectangle.width = MAX(1,ideal_width);
        else
                new_ig->rectangle.width = req_ig->rectangle.width;
        if (IG_RecomputeSize(nw) || req_ig->rectangle.height == 0)
                new_ig->rectangle.height = MAX(1,ideal_height);
        else
                new_ig->rectangle.height = req_ig->rectangle.height;

        IG_NormalGC(nw) = NULL;
        IG_InverseGC(nw) = NULL;
        UpdateGCs(nw);
}

/************************************************************************
 * InitializeHook               - NULL
 ************************************************************************/

/************************************************************************
 * Realize                      - NULL
 ************************************************************************/

/************************************************************************
 * Destroy
 ************************************************************************/
static  void
Destroy(
        Widget  wid)
{
#ifdef DEBUG
printf("Destroy\n");
#endif /* DEBUG */

        if (IG_FontList(wid) != NULL) XmFontListFree(IG_FontList(wid));
        if (IG__LabelString(wid) != NULL) _XmStringFree(IG__LabelString(wid));
        if (IG_OwnLargePixmap(wid))
                {
                if (IG_LargeIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                        XmDestroyPixmap(XtScreen(wid),IG_LargeIconPixmap(wid));
                if (IG_LargeIconMask(wid) != XmUNSPECIFIED_PIXMAP)
                        XmDestroyPixmap(XtScreen(wid),IG_LargeIconMask(wid));
                }
        if (IG_OwnSmallPixmap(wid))
                {
                if (IG_SmallIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                        XmDestroyPixmap(XtScreen(wid),IG_SmallIconPixmap(wid));
                if (IG_SmallIconMask(wid) != XmUNSPECIFIED_PIXMAP)
                        XmDestroyPixmap(XtScreen(wid),IG_SmallIconMask(wid));
                }
        XtReleaseGC(XtParent(wid),IG_NormalGC(wid));
        XtReleaseGC(XtParent(wid),IG_InverseGC(wid));
}

/************************************************************************
 * Resize                       - NULL
 ************************************************************************/

/************************************************************************
 * Redisplay
 ************************************************************************/
static  void
Redisplay(
        Widget  wid,
        XEvent  *event,
        Region  region)
{
        XmIconGadget    ig = (XmIconGadget)wid;
        GC              gc;
        GC              background_gc;
        Dimension       edge;
        Dimension       needed_width,needed_height;
        Dimension       pm_width,pm_height;
        XRectangle      clip_rect;
        Boolean         clip_set = False;
        int             depth;
        Dimension       labelWidth, labelHeight ;
        Boolean         bIsSelected = False ;

#ifdef DEBUG
printf("IconGadget Redisplay\n");
#endif /* DEBUG */

        if (! ig->icong.refreshOn)
           return ;

        if (IG_VisualEmphasis(wid) == XmSELECTED)
                {
                gc = IG_InverseGC(wid);
                background_gc = IG_NormalGC(wid);
                bIsSelected = True ;
                }
        else
                {
                gc = IG_NormalGC(wid);
                background_gc = IG_InverseGC(wid);
                }
        edge = ig->gadget.highlight_thickness + ig->gadget.shadow_thickness;

        /*XFillRectangle(XtDisplay(wid),XtWindow(wid),background_gc,
                        ig->rectangle.x + (Position)edge,
                        ig->rectangle.y + (Position)edge,
                        ig->rectangle.width - 2 * edge,
                        ig->rectangle.height - 2 * edge); */

        if (IG_ViewType(wid) == XmLARGE_ICON)
                {
                needed_width = (2 * edge) + MAX(IG_LargeIconRectRV(wid).width,
                                                IG_LabelRectRV(wid).width);
                needed_height = (2 * edge) + IG_Spacing(wid)
                                        + IG_LargeIconRectRV(wid).height
                                        + IG_LabelRectRV(wid).height;
                pm_width = IG_LargeIconRectRV(wid).width;
                pm_height = IG_LargeIconRectRV(wid).height;
                }
        else
                {
                needed_width = (2 * edge) + IG_Spacing(wid)
                                        + IG_SmallIconRectRV(wid).width
                                        + IG_LabelRectRV(wid).width;
                needed_height = (2 * edge) + MAX(IG_SmallIconRectRV(wid).height,
                                                IG_LabelRectRV(wid).height);
                pm_width = IG_SmallIconRectRV(wid).width;
                pm_height = IG_SmallIconRectRV(wid).height;
                }

        if (ig->rectangle.width < needed_width)
                pm_width = ig->rectangle.width - (2 * edge);
        if (ig->rectangle.height < needed_height)
                pm_height = ig->rectangle.height - (2 * edge);

        if ((IG_ViewType(wid) == XmLARGE_ICON) &&
            (IG_LargeIconPixmap(wid) != XmUNSPECIFIED_PIXMAP) &&
            (ShowIcon(wid)) &&
            (pm_width > 0) && (pm_height > 0))
                {
                if (IG_LargeIconMask(wid) != XmUNSPECIFIED_PIXMAP)
                        {
                        XSetClipMask(XtDisplay(wid),gc,
                                        IG_LargeIconMask(wid));
                        XSetClipOrigin(XtDisplay(wid),gc,
                                ig->rectangle.x + IG_LargeIconRectRV(wid).x,
                                ig->rectangle.y + IG_LargeIconRectRV(wid).y);
                             clip_set = True;
                        }
                XCopyArea(XtDisplay(wid),IG_LargeIconPixmap(wid),
                        XtWindow(wid),gc,0,0,
                        pm_width,pm_height,
                        ig->rectangle.x + IG_LargeIconRectRV(wid).x,
                        ig->rectangle.y + IG_LargeIconRectRV(wid).y);
                }
        if ((IG_ViewType(wid) == XmSMALL_ICON) &&
            (IG_SmallIconPixmap(wid) != XmUNSPECIFIED_PIXMAP) &&
            (ShowIcon(wid)) &&
            (pm_width > 0) && (pm_height > 0))
                {
                if (IG_SmallIconMask(wid) != XmUNSPECIFIED_PIXMAP)
                        {
                        XSetClipMask(XtDisplay(wid),gc,
                                        IG_SmallIconMask(wid));
                        XSetClipOrigin(XtDisplay(wid),gc,
                                ig->rectangle.x + IG_SmallIconRectRV(wid).x,
                                ig->rectangle.y + IG_SmallIconRectRV(wid).y);
                        clip_set = True;
                        }
                XCopyArea(XtDisplay(wid),IG_SmallIconPixmap(wid),
                        XtWindow(wid),gc,0,0,
                        pm_width,pm_height,
                        ig->rectangle.x + IG_SmallIconRectRV(wid).x,
                        ig->rectangle.y + IG_SmallIconRectRV(wid).y);
                }
        if (clip_set)
                {
                XSetClipMask(XtDisplay(wid),gc,None);
                clip_set = False;
                }

        if ((ig->rectangle.height < needed_height) ||
            (ig->rectangle.width < needed_width))
                {
                clip_rect.x = ig->rectangle.x + (Position)edge;
                clip_rect.y = ig->rectangle.y + (Position)edge;
                clip_rect.width = ig->rectangle.width - (2 * edge);
                clip_rect.height = ig->rectangle.height - (2 * edge);
                XSetClipRectangles(XtDisplay(wid),gc,0,0,&clip_rect,1,Unsorted);
                clip_set = True;
                }
        if (IG__LabelString(wid) != NULL && ShowLabel(wid))
        {
           if (bIsSelected)
           {
              /* handle special case of having a bitmap as the         *
               * container background.                                 *
               * draw the text label with foreground and background    *
               * flipped to indicate selection emphasis                */
              _XmStringExtent(IG_FontList(wid),
                             IG__LabelString(wid),
                             &labelWidth,
                             &labelHeight) ;
              XFillRectangle(XtDisplay(wid),XtWindow(wid),background_gc,
                      ig->rectangle.x + IG_LabelRectRV(wid).x,
                      ig->rectangle.y + IG_LabelRectRV(wid).y,
                      IG_LabelRectRV(wid).width,
                      labelHeight); 
           }
           _XmStringDraw(XtDisplay(wid),XtWindow(wid),
                   IG_FontList(wid),IG__LabelString(wid),gc,
                   ig->rectangle.x + IG_LabelRectRV(wid).x,
                   ig->rectangle.y + IG_LabelRectRV(wid).y,
                   IG_LabelRectRV(wid).width,
                   XmALIGNMENT_BEGINNING,IG_StringDirection(wid),NULL);
        }
        if (clip_set)
                XSetClipMask(XtDisplay(wid),gc,None);
}

/************************************************************************
 * SetValues                    - Design Complete
 ************************************************************************/
static  Boolean
SetValues(
        Widget          cw,
        Widget          rw,
        Widget          nw,
        ArgList         args,
        Cardinal        *num_args)
{
        Boolean                 Relayout = False;
        Boolean                 Redraw = False;
        Boolean                 bCnrRefresh = True ;
#ifdef DEBUG
printf("IconGadget SetValues\n");
#endif /* DEBUG */

        if (IG_ViewType(nw) != IG_ViewType(cw))
                {
                if (XmRepTypeValidValue(XmRID_VIEW_TYPE,IG_ViewType(nw),nw))
                        Relayout = Redraw = True;
                else
                        IG_ViewType(nw) = IG_ViewType(cw);
                }
        if (IG_CursorEmphasis(nw) != IG_CursorEmphasis(cw))
                {
                if (XmRepTypeValidValue(XmRID_CURSOR_EMPHASIS,
                                                IG_CursorEmphasis(nw),nw))
                        Redraw = True;
                else
                        IG_CursorEmphasis(nw) = IG_CursorEmphasis(cw);
                }
        if (IG_VisualEmphasis(nw) != IG_VisualEmphasis(cw))
                {
                if (XmRepTypeValidValue(XmRID_VISUAL_EMPHASIS,
                                                IG_VisualEmphasis(nw),nw))
                        Redraw = True;
                else
                        IG_VisualEmphasis(nw) = IG_VisualEmphasis(cw);
                }
        if (IG_InUseEmphasis(nw) != IG_InUseEmphasis(cw))
                {
                if (XmRepTypeValidValue(XmRID_INUSE_EMPHASIS,
                                                IG_InUseEmphasis(nw),nw))
                {
                   Redraw = True;
                   InUseEmphasisCB(nw, NULL, IG_InUseEmphasis(nw)) ;
                }
                else
                        IG_InUseEmphasis(nw) = IG_InUseEmphasis(cw);
                }

        if (IG_FontList(nw) != IG_FontList(cw))
                {
                XmFontListFree(IG_FontList(cw));
                IG_FontList(nw) = XmFontListCopy(IG_FontList(nw));
                UpdateGCs(nw);
                Relayout = Redraw = True;
                }
        if (IG__LabelString(nw) != IG__LabelString(cw))
                {
                _XmStringFree(IG__LabelString(cw));
                IG__LabelString(nw) = _XmStringCreate((XmString)
                                        IG__LabelString(nw));
                Relayout = Redraw = True;
                }
        if (IG_LargeIcon(nw) != IG_LargeIcon(cw))
                {
                if (IG_OwnLargePixmap(cw))
                        {
                        if (IG_LargeIconPixmap(cw) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(cw),
                                                IG_LargeIconPixmap(cw));
                        if (IG_LargeIconMask(cw) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(cw),
                                                IG_LargeIconMask(cw));
                        }
                GetLargeIcon(nw);
                Relayout = True;
                if (IG_ViewType(nw) == XmLARGE_ICON)
                        Redraw = True;
                }
        if (!IG_OwnLargePixmap(cw))
                {
                if (IG_LargeIconPixmap(nw) != IG_LargeIconPixmap(cw))
                        {
                        Relayout = True;
                        if (IG_ViewType(nw) == XmLARGE_ICON)
                                Redraw = True;
                        }
                if (IG_LargeIconMask(nw) != IG_LargeIconMask(cw))
                        {
                        Relayout = True;
                        if (IG_ViewType(nw) == XmLARGE_ICON)
                                Redraw = True;
                        }
                }
        if (IG_SmallIcon(nw) != IG_SmallIcon(cw))
                {
                if (IG_OwnSmallPixmap(cw))
                        {
                        if (IG_SmallIconPixmap(cw) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(cw),
                                                IG_SmallIconPixmap(cw));
                        if (IG_SmallIconMask(cw) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(cw),
                                                IG_SmallIconMask(cw));
                        }
                GetSmallIcon(nw);
                Relayout = True;
                if (IG_ViewType(nw) == XmSMALL_ICON)
                        Redraw = True;
                }
        if (!IG_OwnSmallPixmap(cw))
                {
                if (IG_SmallIconPixmap(nw) != IG_SmallIconPixmap(cw))
                        {
                        Relayout = True;
                        if (IG_ViewType(nw) == XmSMALL_ICON)
                                Redraw = True;
                        }
                if (IG_SmallIconMask(nw) != IG_SmallIconMask(cw))
                        {
                        Relayout = True;
                        if (IG_ViewType(nw) == XmSMALL_ICON)
                                Redraw = True;
                        }
                }

        Redraw = Redraw && ( (XmIconGadget) nw)->icong.refreshOn ;

        if (Redraw)
	     {
           XtVaGetValues(XtParent(nw), XmNrefreshCnr, &bCnrRefresh, NULL) ;
           Redraw = bCnrRefresh ;
        }

        if (Relayout)
          XuiclIconGadgetLayout(nw);

        return(Redraw);
}

/************************************************************************
 * SetValuesHook                - NULL
 ************************************************************************/

/************************************************************************
 * SetValuesAlmost              - Inherit
 ************************************************************************/

/************************************************************************
 * GetValuesHook                - NULL
 ************************************************************************/

/************************************************************************
 * QueryGeometry
 ************************************************************************/
static  XtGeometryResult
QueryGeometry(
        Widget                  wid,
        XtWidgetGeometry        *intended,
        XtWidgetGeometry        *reply)
{
        Dimension       ideal_width,ideal_height;

#ifdef DEBUG
printf("QueryGeometry\n");
#endif /* DEBUG */
        reply->request_mode = 0;
        if ((intended->request_mode) & (~(CWWidth | CWHeight)))
                return (XtGeometryNo);
        if (IG_RecomputeSize(wid) == False)
                return (XtGeometryNo);

        Layout(wid,&ideal_width,&ideal_height);
        reply->request_mode = (CWWidth | CWHeight);
        reply->width = MAX(1,ideal_width);
        reply->height = MAX(1,ideal_height);

        if (((intended->request_mode & CWWidth) &&
                                        (reply->width != intended->width)) ||
            ((intended->request_mode & CWHeight) &&
                                        (reply->height != intended->height)) ||
            (intended->request_mode != reply->request_mode))
                return (XtGeometryAlmost);
        else
                {
                reply->request_mode = 0;
                return (XtGeometryYes);
                }
}


/*-----------------
| XmGadget methods |
-----------------*/
/************************************************************************
 * HighlightBorder              - Inherit
 ************************************************************************/

/************************************************************************
 * UnhighlightBorder            - Inherit
 ************************************************************************/

/************************************************************************
 * ArmAndActivate               - NULL
 ************************************************************************/

/************************************************************************
 * InputDispatch                - NULL
 ************************************************************************/

/************************************************************************
 * VisualChange
 ************************************************************************/
static  Boolean
VisualChange(
        Widget  wid,
        Widget  cmw,
        Widget  nmw)
{
        XmManagerWidget curmw = (XmManagerWidget)cmw;
        XmManagerWidget newmw = (XmManagerWidget)nmw;

#ifdef DEBUG
printf("VisualChange\n");
#endif /* DEBUG */

        if (curmw->manager.foreground != newmw->manager.foreground ||
            curmw->core.background_pixel != newmw->core.background_pixel)
                {
                UpdateGCs(wid);
                if (IG_OwnLargePixmap(wid))
                        {
                        if (IG_LargeIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(wid),
                                                IG_LargeIconPixmap(wid));
                        if (IG_LargeIconMask(wid) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(wid),
                                                IG_LargeIconMask(wid));
                        GetLargeIcon(wid);
                        }
                if (IG_OwnSmallPixmap(wid))
                        {
                        if (IG_SmallIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(wid),
                                                IG_SmallIconPixmap(wid));
                        if (IG_SmallIconMask(wid) != XmUNSPECIFIED_PIXMAP)
                                XmDestroyPixmap(XtScreen(wid),
                                                IG_SmallIconMask(wid));
                        GetSmallIcon(wid);
                        }
                return(True);
                }
        return(False);
}

/************************************************************************
 * InputDispatch
 ************************************************************************/
static  void
InputDispatch(
        Widget  wid,
        XEvent  *event,
        Mask    event_mask)
{
#ifdef DEBUG
printf("InputDispatch\n");
#endif /* DEBUG */

        if (event_mask & XmHELP_EVENT) Help(wid,event);
}

/************************************************************************
 * Help
 ************************************************************************/
static  void
Help(
        Widget  wid,
        XEvent  *event)
{
#ifdef DEBUG
printf("Help\n");
#endif /* DEBUG */

        _XmSocorro(wid,event,NULL,NULL);
}

/************************************************************************
 * XmIconGadgetGetBaselines
 ************************************************************************/
static  Boolean
XmIconGadgetGetBaselines(
        Widget          wid,
        Dimension       **baselines,
        int             *line_count)
{
        Dimension *     base_array;

#ifdef DEBUG
printf("IconGadgetBaseline\n");
#endif /* DEBUG */

        if ((IG_ViewType(wid) == XmLARGE_ICON) &&
            (IG_LargeIconPixmap(wid) != XmUNSPECIFIED_PIXMAP))
                {
                *line_count = 1;
                base_array = (Dimension *)XtMalloc(sizeof(Dimension));
                base_array[0] = ((XmIconGadget)wid)->rectangle.y
                                + (Dimension)IG_LargeIconRectRV(wid).y
                                + IG_LargeIconRectRV(wid).height;
                *baselines = base_array;
                return(True);
                }
        /* XmSMALL_ICON or XmNlabelString only */
        if (IG__LabelString(wid) == NULL)
                return(False);
        *line_count = 1;
        base_array = (Dimension *)XtMalloc(sizeof(Dimension));
        base_array[0] = ((XmIconGadget)wid)->rectangle.y
                                + (Dimension)IG_LabelRectRV(wid).y
                                + _XmStringBaseline(IG_FontList(wid),
                                                        IG__LabelString(wid));
        *baselines = base_array;
        return(True);
}

/************************************************************************
 * XmIconGadgetGetDisplayRect
 ************************************************************************/
static  Boolean
XmIconGadgetGetDisplayRect(
        Widget          wid,
        XRectangle      *displayrect)
{
        XmIconGadget    ig = (XmIconGadget)wid;

#ifdef DEBUG
printf("XmIconGadgetGetDisplayRect\n");
#endif /* DEBUG */

        if (IG_ViewType(wid) == XmLARGE_ICON)
                {
                (*displayrect).x = ig->rectangle.x +
                        MIN(IG_LargeIconRectRV(wid).x,IG_LabelRectRV(wid).x);
                (*displayrect).y = ig->rectangle.y + IG_LargeIconRectRV(wid).y;
                (*displayrect).width = MAX(IG_LargeIconRectRV(wid).width,
                                                IG_LabelRectRV(wid).width);
                (*displayrect).height = IG_LargeIconRectRV(wid).height
                                        + IG_Spacing(wid)
                                        + IG_LabelRectRV(wid).height;
                }
        else    /* XmSMALL_ICON */
                {
                (*displayrect).x = ig->rectangle.x +
                        MIN(IG_SmallIconRectRV(wid).x,IG_LabelRectRV(wid).x);
                (*displayrect).y = ig->rectangle.y +
                                        MIN(IG_SmallIconRectRV(wid).y,
                                            IG_LabelRectRV(wid).y);
                (*displayrect).width = IG_SmallIconRectRV(wid).width
                                        + IG_Spacing(wid)
                                        + IG_LabelRectRV(wid).width;
                (*displayrect).height = MAX(IG_SmallIconRectRV(wid).height,
                                                IG_LabelRectRV(wid).height);
                }
  return(True);
}

/*---------------------
| XmIconGadget methods |
---------------------*/

/*------------
| ActionProcs |
------------*/

/*-------------------
| Internal functions |
-------------------*/
/************************************************************************
 * GetLabelString
 ************************************************************************/
static  void
GetLabelString(
        Widget          wid,
        int             offset,
        XtArgVal        *value)
{
        XmString        string;

#ifdef DEBUG
printf("GetLabelString\n");
#endif /* DEBUG */

        string = _XmStringCreateExternal
                                (IG_FontList(wid),IG__LabelString(wid));
        *value = (XtArgVal)string;
}

/************************************************************************
 * GetLargeIcon
 ************************************************************************/
static  Boolean
GetLargeIcon(
        Widget  wid)
{
        char    *mask_name;

#ifdef DEBUG
printf("GetLargeIcon\n");
#endif /* DEBUG */

        if (!IG_LargeIcon(wid))
                return(IG_OwnLargePixmap(wid) = False);
        IG_LargeIconPixmap(wid) = XmGetPixmapByDepth(XtScreen(wid),
                                        IG_LargeIcon(wid),
                                        IG_Foreground(wid),IG_Background(wid),
                                        IG_Depth(wid));
        if (IG_LargeIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                {
                mask_name = XtMalloc(strlen(IG_LargeIcon(wid))+3);
                strcpy(mask_name,IG_LargeIcon(wid));
                strcat(mask_name,"_m");
                IG_LargeIconMask(wid) = XmGetPixmapByDepth(XtScreen(wid),
                                mask_name,
                                IG_Foreground(wid),IG_Background(wid),1);
                XtFree(mask_name);
                }
        return(IG_OwnLargePixmap(wid) = True);
}

/************************************************************************
 * GetSmallIcon
 ************************************************************************/
static  Boolean
GetSmallIcon(
        Widget  wid)
{
        char    *mask_name;

#ifdef DEBUG
printf("GetSmallIcon\n");
#endif /* DEBUG */

        if (!IG_SmallIcon(wid))
                return(IG_OwnSmallPixmap(wid) = False);
        IG_SmallIconPixmap(wid) = XmGetPixmapByDepth(XtScreen(wid),
                                        IG_SmallIcon(wid),
                                        IG_Foreground(wid),IG_Background(wid),
                                        IG_Depth(wid));
        if (IG_SmallIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                {
                mask_name = XtMalloc(strlen(IG_SmallIcon(wid))+3);
                strcpy(mask_name,IG_SmallIcon(wid));
                strcat(mask_name,"_m");
                IG_SmallIconMask(wid) = XmGetPixmapByDepth(XtScreen(wid),
                                mask_name,
                                IG_Foreground(wid),IG_Background(wid),1);
                XtFree(mask_name);
                }
        return(IG_OwnSmallPixmap(wid) = True);
}

/************************************************************************
 * UpdateGCs
 ************************************************************************/
static  void
UpdateGCs(
        Widget  wid)
{
        XGCValues       values;
        XtGCMask        valueMask;
        XFontStruct     *fs = (XFontStruct *)NULL;

#ifdef DEBUG
printf("UpdateGCs\n");
#endif /* DEBUG */

        if (IG_NormalGC(wid))
                XtReleaseGC(XtParent(wid),IG_NormalGC(wid));
        if (IG_InverseGC(wid))
                XtReleaseGC(XtParent(wid),IG_InverseGC(wid));

        valueMask = GCForeground | GCBackground | GCGraphicsExposures;
        values.foreground = IG_Foreground(wid);
        values.background = IG_Background(wid);
        values.graphics_exposures = FALSE;

        _XmFontListGetDefaultFont(IG_FontList(wid),&fs);
        if (fs != NULL)
                {
                values.font = fs->fid;
                valueMask |= GCFont;
                }

        IG_NormalGC(wid) = XtGetGC(XtParent(wid),valueMask,&values);
        values.foreground = IG_Background(wid);
        values.background = IG_Foreground(wid);
        IG_InverseGC(wid) = XtGetGC(XtParent(wid),valueMask,&values);
}

/************************************************************************
 * GetLabelSize
 ************************************************************************/
static  void
GetLabelSize(
        Widget  wid)
{
        Dimension       w = 0, h = 0;

#ifdef DEBUG
printf("GetLabelSize\n");
#endif /* DEBUG */

        if (!_XmStringEmpty(IG__LabelString(wid)))
                _XmStringExtent(IG_FontList(wid),IG__LabelString(wid),&w,&h);
        IG_LabelRect(wid).width  = ShowLabel(wid) ? (unsigned short)w : 0;
        IG_LabelRect(wid).height = ShowLabel(wid) ? (unsigned short)h : 0;
}

/************************************************************************
 * GetLargeIconSize
 ************************************************************************/
static  void
GetLargeIconSize(
        Widget  wid)
{
        unsigned int    junk, w = 0, h = 0, d;

#ifdef DEBUG
printf("GetLargeIconSize\n");
#endif /* DEBUG */

        if (IG_LargeIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                XGetGeometry(XtDisplay(wid),IG_LargeIconPixmap(wid),
                                (Window *)&junk,(int *)&junk,(int *)&junk,
                                &w,&h,&junk,&d);

        IG_LargeIconRect(wid).width  = ShowIcon(wid) ? (unsigned short)w : 0;
        IG_LargeIconRect(wid).height = ShowIcon(wid) ? (unsigned short)h : 0;
}

/************************************************************************
 * GetSmallIconSize
 ************************************************************************/
static  void
GetSmallIconSize(
        Widget  wid)
{
        unsigned int    junk, w = 0, h = 0, d;

#ifdef DEBUG
printf("GetSmallIconSize\n");
#endif /* DEBUG */

        if (IG_SmallIconPixmap(wid) != XmUNSPECIFIED_PIXMAP)
                XGetGeometry(XtDisplay(wid),IG_SmallIconPixmap(wid),
                                (Window *)&junk,(int *)&junk,(int *)&junk,
                                &w,&h,&junk,&d);

        IG_SmallIconRect(wid).width  = ShowIcon(wid) ? (unsigned short)w : 0;
        IG_SmallIconRect(wid).height = ShowIcon(wid) ? (unsigned short)h : 0;
}

/************************************************************************
 * Layout
 ************************************************************************/
static  void
Layout(
        Widget          wid,
        Dimension *     ideal_width,
        Dimension *     ideal_height)
{
        Dimension       edge;

        XmIconGadget     new_ig = (XmIconGadget)wid;

#ifdef DEBUG
printf("IconGadget Layout\n");
#endif /* DEBUG */

        /*if (! new_ig->icong.refreshOn)
           return ; */

        /* ICLUI: following statements used to be in Initialize before its
         *  call to Layout(). Moved here to handle changing labels and icons.
         */
        GetLabelSize(wid);
        GetLargeIconSize(wid);
        GetSmallIconSize(wid);
        IG_Spacing(wid) = 4;
        IG_LayoutDirection(wid) = XmLEFT_TO_RIGHT;
        XtVaGetValues(XtParent(wid),
                XmNlayoutDirection,&IG_LayoutDirection(wid),NULL);

        /* ICLUI: end of stuff from Initialize */

        edge = IG_HLThickness(wid) + IG_ShadowThickness(wid);

        if (IG_ViewType(wid) == XmLARGE_ICON)
                {
                *ideal_height = (2 * edge) + IG_LargeIconRectRV(wid).height
                                        + IG_Spacing(wid)
                                        + IG_LabelRectRV(wid).height;
                IG_LargeIconRect(wid).y = (Position)edge;
                IG_LabelRect(wid).y = (Position)(edge + IG_Spacing(wid)
                                        + IG_LargeIconRectRV(wid).height);
                *ideal_width = (2 * edge) + MAX(IG_LargeIconRectRV(wid).width,
                                                IG_LabelRect(wid).width);
                IG_LargeIconRect(wid).x = (Position)edge;
                IG_LabelRect(wid).x = (Position)edge;
                if ((IG_LayoutDirection(wid) == XmRIGHT_TO_LEFT)
                                        &&
                    (IG_LargeIconRectRV(wid).width < IG_LabelRectRV(wid).width))
                        IG_LargeIconRect(wid).x = (Position)
                                        (edge + IG_LabelRectRV(wid).width
                                                - IG_LargeIconRectRV(wid).width);
                if ((IG_LayoutDirection(wid) == XmRIGHT_TO_LEFT)
                                        &&
                    (IG_LargeIconRectRV(wid).width > IG_LabelRectRV(wid).width))
                        IG_LabelRect(wid).x = (Position)
                                        (edge + IG_LargeIconRectRV(wid).width
                                                - IG_LabelRectRV(wid).width);
                }
        else
        {
        /* XmSMALL_ICON */
        *ideal_width = (2 * edge) + IG_SmallIconRectRV(wid).width
                                + IG_Spacing(wid)
                                + IG_LabelRectRV(wid).width;
        *ideal_height = (2 * edge) + MAX(IG_SmallIconRectRV(wid).height,
                                IG_LabelRectRV(wid).height);
        if (IG_LayoutDirection(wid) == XmLEFT_TO_RIGHT)
                {
                IG_SmallIconRect(wid).x = (Position)edge;
                IG_LabelRect(wid).x = (Position)(edge + IG_Spacing(wid)
                                        + IG_SmallIconRectRV(wid).width);
                }
        else
                {
                IG_LabelRect(wid).x = (Position)edge;
                IG_SmallIconRect(wid).x = (Position)(edge + IG_Spacing(wid)
                                        + IG_LabelRectRV(wid).width);
                }
        IG_SmallIconRect(wid).y = (Position)edge;
        IG_LabelRect(wid).y = (Position)edge;
        if (IG_SmallIconRectRV(wid).height > IG_LabelRectRV(wid).height)
                IG_LabelRect(wid).y = (Position)(edge
                                        + ((IG_SmallIconRectRV(wid).height
                                        - IG_LabelRectRV(wid).height) / 2));
        if (IG_SmallIconRectRV(wid).height < IG_LabelRectRV(wid).height)
                IG_SmallIconRect(wid).y = (Position)(edge
                                        + ((IG_LabelRectRV(wid).height
                                        - IG_SmallIconRectRV(wid).height) / 2));
        }

}

/*-------------------
| External functions |
-------------------*/
/************************************************************************
 * XmCreateIconGadget
 *
 * Create an instance of a xmIconGadgetClass widget and
 * return it's id.
 ************************************************************************/
Widget
XmCreateIconGadget(
        Widget          parent,
        char            *name,
        ArgList         arglist,
        Cardinal        argcount)
{
#ifdef DEBUG
printf("XmCreateIconGadget\n");
#endif /* DEBUG */

        return(XtCreateWidget(name,xmIconGadgetClass,parent,arglist,argcount));
}

/************************************************************************
 * XuiclIconGadgetLayout
 *
 *  Called by Container Widget's SetValues for each IconGadget that is
 *  a child of the Container. Does this when IconGadget's geometry changes.
 *
 *  This function is also called from IconGadget's Layout routine.
 ************************************************************************/
void XuiclIconGadgetLayout(Widget iconG)
{
  Dimension               ideal_width,ideal_height;
  XtWidgetGeometry        geo_desired;

  Layout(iconG,&ideal_width,&ideal_height);
  if (IG_RecomputeSize(iconG))
          {
          XmIconGadget    ig = (XmIconGadget)iconG;

          geo_desired.request_mode = 0;
          if (ig->rectangle.width != ideal_width)
                  {
                  geo_desired.request_mode |= CWWidth;
                  geo_desired.width = ideal_width;
                  ig->rectangle.width = ideal_width;
                  }
          if (ig->rectangle.height != ideal_height)
                  {
                  geo_desired.request_mode |= CWHeight;
                  geo_desired.height = ideal_height;
                  ig->rectangle.height = ideal_height;
                  }
          if (geo_desired.request_mode != 0)
                  _XmMakeGeometryRequest(iconG,&geo_desired);
          }
}

/************************************************************************
 * XuiclIsGadgetBeingDestroyed
 *
 ************************************************************************/
Boolean XuiclIsGadgetBeingDestroyed(Widget gid)
{
   return gid->core.being_destroyed ;
}

/************************************************************************
 * XuiclIconGadgetLabelCoords
 *
 *  Called by Container Widget's XiuclLocation for a non-details view
 *  gadget to locate the label x and y coordinates.
 ************************************************************************/
Boolean XuiclIconGadgetLabelCoords(
    Widget   w,
    Position *x,
    Position *y)
{
  XmIconGadget ig = (XmIconGadget)w;

  if ( (! IG__LabelString(w)) || (! ShowLabel(w)) ) return False;

  if(x)
  {
      *x = ig->rectangle.x + IG_LabelRect(w).x;
  }
  if(y)
  {
      *y = ig->rectangle.y + IG_LabelRect(w).y;
  }
  return True;
}

/************************************************************************
 * XuiclIconGadgetRectangle
 *
 *  Called by Container Widget's XiuclRectangle for a non-details view
 *  gadget to locate the label/text rectangle.
 ************************************************************************/
XRectangle XuiclIconGadgetRectangle(
              Widget    w,
              unsigned  long extent) /* XmICON_EXTENT | XmTEXT_EXTENT */
{
   XmIconGadget ig = (XmIconGadget)w;
   XRectangle rect;

   if( extent == (XmICON_EXTENT | XmTEXT_EXTENT) ) /* label and icon */
   {
       Dimension edge = IG_HLThickness(w) + IG_ShadowThickness(w);

       rect.x      = ig->rectangle.x + edge;
       rect.y      = ig->rectangle.y + edge;
       rect.width  = ig->rectangle.width - 2 * edge;
       rect.height = ig->rectangle.height - 2 * edge;

       return rect;
   }
   else if( extent == XmICON_EXTENT )            /* icon only */
   {
       if( IG_ViewType(w) == XmLARGE_ICON )
       {
            rect.x      = ig->rectangle.x + IG_LargeIconRectRV(w).x;
            rect.y      = ig->rectangle.y + IG_LargeIconRectRV(w).y;
            rect.width  = IG_LargeIconRectRV(w).width;
            rect.height = IG_LargeIconRectRV(w).height;
       }
       else               /* XmSMALL_ICON */
       {
            rect.x      = ig->rectangle.x + IG_SmallIconRectRV(w).x;
            rect.y      = ig->rectangle.y + IG_SmallIconRectRV(w).y;
            rect.width  = IG_SmallIconRectRV(w).width;
            rect.height = IG_SmallIconRectRV(w).height;
       }
       return rect;
   }
   else if( extent == XmTEXT_EXTENT )            /* label only */
   {
       rect.x      = ig->rectangle.x + IG_LabelRectRV(w).x;
       rect.y      = ig->rectangle.y + IG_LabelRectRV(w).y;
       rect.width  = IG_LabelRectRV(w).width;
       rect.height = IG_LabelRectRV(w).height;
       return rect;
   }
   else if( extent == 0 ) /*empty rectangle positioned at lower left of record*/
   {
       rect.x      = ig->rectangle.x;
       rect.y      = ig->rectangle.y + ig->rectangle.height;
       rect.width  = rect.height = 0;
       return rect;
   }
   else                  /* badparms: unpositioned empty rectangle */
   {
       rect.x = rect.y = -1;
       rect.width  = rect.height = 0;
       return rect;
   }
}


/**********************************************************************
 * static functions added to support ICLUI
 *********************************************************************/

/************************************************************************
 * ShowLabel
 ************************************************************************/
static Boolean ShowLabel(Widget wid)
{
  Boolean showIconInDetail;
  unsigned char     viewType;

  XtVaGetValues( XtParent(wid),
                 XmNentryViewType, &viewType,
                 NULL );

  /* Never show the label in details view, if they want it it will be
   * added as a column.
   */
  return ( !(viewType == XmDETAIL) );
}

/************************************************************************
 * ShowIcon
 ************************************************************************/
static Boolean ShowIcon(Widget wid)
{
  Boolean textView, showIconInDetail;
  unsigned char viewType;

  XtVaGetValues( XtParent(wid),
                 XmNshowTextView, &textView,
                 XmNentryViewType, &viewType,
                 XmNshowIconInDetail, &showIconInDetail,
                 NULL);

  if (textView && (viewType != XmDETAIL) )
    return False;
  else if (viewType == XmDETAIL && !showIconInDetail)
    return False;
  else
    return True;
}

/************************************************************************
 * InUseEmphasisCB
 ************************************************************************/
static  void
InUseEmphasisCB(
        Widget        wid,
        XEvent        *event,
        unsigned char inuse_change_type)
{
        XmIconGadget gwid = (XmIconGadget) wid ;
        XmIconGadgetInuseEmphasisCallbackStruct cbs;

        if (!(XtHasCallbacks((Widget) gwid, XmNinUseEmphasisCallback) == XtCallbackHasSome))
           return ;

        cbs.reason = XmCR_INUSE_CHANGE;
        cbs.event = event;
        cbs.aGadget = (Widget) gwid ;
        cbs.emphasis = inuse_change_type ;
        XtCallCallbackList((Widget)gwid, gwid->icong.inuse_emphasis_cb,&cbs);
}



