/* Revision: 27 1.3.2.2 source/ui/widgets/notebkp.h, cuawidgets, ioc.v400, 000919  */
/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1997, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
/*
 * (c) Copyright 1989, 1990, 1991, 1992, 1993 OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * Motif top of 2.0 tree as of Tue Jun  1 05:18:29 EDT 1993
 */
#ifndef _NOTEBKP_
#define _NOTEBKP_

#include <Xm/XmP.h>
#include <Xm/ManagerP.h>
#include <Xm/XmStrDefs.h>
#include "notebk.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Notebook's constraint info. fields */
typedef struct _XmNotebookConstraintPart
{
    /* resources */
    unsigned char child_type;           /* notebook child type */
    int page_number;                    /* page number */
    Boolean resizable;                  /* is this child resizable? */

    /* private variables */
    Boolean active;                     /* True if the child is active */
    Dimension preferred_width;          /* the child's preferred width */
    Dimension preferred_height;         /* the child's preferred height */
    Dimension shadow_thickness;         /* to save child's shadow thickness */
    Boolean recompute_size;             /* to save child's recompute size */
    Boolean in_setshadow;               /* True if processing set shadowThick */
} XmNotebookConstraintPart, *XmNotebookConstraint;

typedef struct _XmNotebookConstraintRec
{
    XmManagerConstraintPart manager;
    XmNotebookConstraintPart notebook;
} XmNotebookConstraintRec, *XmNotebookConstraintPtr;





/* New fields for the Notebook widget class record */
typedef struct _XmNotebookClassPart
{
    XtPointer unused;
} XmNotebookClassPart;

/* Full class record declaration */
typedef struct _NotebookClassRec
{
    CoreClassPart core_class;
    CompositeClassPart composite_class;
    ConstraintClassPart constraint_class;
    XmManagerClassPart manager_class;
    XmNotebookClassPart notebook_class;
} XmNotebookClassRec;

extern XmNotebookClassRec xmNotebookClassRec;





/* New fields for the Notebook widget instance record */
typedef struct _XmNotebookPart
{
    /* resources */
    int current_page_number;            /* the current page number */
    int first_page_number;              /* the first page number */
    int last_page_number;               /* the last page number */
    unsigned char orientation;          /* notebook orientation */
    unsigned char back_page_pos;        /* position of back pages */
    Cardinal back_page_number;          /* the of back page lines */
    Dimension back_page_size;           /* the thickness of back pages */
    Pixel back_page_foreground;         /* foreground color for bk pgs */
    Pixel back_page_background;         /* background color for bk pgs */
    Pixel frame_background;             /* background color for frame */
    unsigned char binding_type;         /* binding type */
    Pixmap binding_pixmap;              /* pixmap for the binding */
    Dimension binding_width;            /* the width of the binding */
    Dimension margin_width;             /* horizontal margin between widgets */
    Dimension margin_height;            /* vertical margin between widgets */
    int major_spacing;                  /* gap between major tabs */
    int minor_spacing;                  /* gap between minor tabs */
    Dimension shadow_thickness;         /* notebook frame shadow thickness */
    XtCallbackList page_change_callback;/* the page change callback */

    /* child widgets */
    Widget scroller;                    /* the page scroller widget */
    Widget next_page;                   /* the next page default button */
    Widget prev_page;                   /* the prev page default button */
    Widget next_major;                  /* next major tab scroll button */
    Widget prev_major;                  /* prev major tab scroll button */
    Widget next_minor;                  /* next minor tab scroll button */
    Widget prev_minor;                  /* prev minor tab scroll button */

    /* preferred children sizes */
    Dimension real_binding_width;       /* real binding width */
    Dimension real_back_page_number;    /* real back page number */
    Dimension page_width;               /* width of page widgets */
    Dimension page_height;              /* height of page widgets */
    Dimension status_width;             /* width of the status areas */
    Dimension status_height;            /* height of the status  areas */
    Dimension major_width;              /* width of major tabs */
    Dimension major_height;             /* height of major tabs */
    Dimension minor_width;              /* width of minor tabs */
    Dimension minor_height;             /* height of minor tabs */
    Dimension scroller_width;           /* width of the page scroller */
    Dimension scroller_height;          /* height of the page scroller */
    Dimension major_scroller_width;     /* width of major scrollers */
    Dimension major_scroller_height;    /* height of major scroller */
    Dimension minor_scroller_width;     /* width of minor scrollers */
    Dimension minor_scroller_height;    /* height of minor scrollers */
    Dimension frame_width;              /* width of the frame */
    Dimension frame_height;             /* height of the frame */
    Dimension nb_width;                 /* width of the notebook */
    Dimension nb_height;                /* height of the notebook */

    /* for layouting tabs */
    Widget first_major;                 /* the first major tab */
    Widget old_top_major;               /* the old top major tab */
    Widget top_major;                   /* the top major tab */
    Widget last_major;                  /* the last major tab */
    Widget first_minor;                 /* the first minor tab */
    Widget old_top_minor;               /* the old top minor tab */
    Widget top_minor;                   /* the top minor tab */
    Widget last_minor;                  /* the last minor tab */

    /* extra position information */
    unsigned char major_pos;            /* position of major tabs */
    unsigned char minor_pos;            /* position of minor tabs */
    unsigned char binding_pos;          /* binding position */

    /* other misc. variables */
    unsigned char which_tab;            /* currently active tab type */
    int last_alloc_num;                 /* lastly allocated page number */
    unsigned char scroller_status;      /* status of the page scroller */
    unsigned short need_scroller;       /* need for tab scrollers */
    Boolean dynamic_last_page_num;      /* True if using dynamic last page# */
    Boolean in_callback;                /* True if processing a callback */
    GC back_page_gc;                    /* GC for drawing backpages */
    GC frame_gc;                        /* GC for drawing frame */
    GC binding_gc;                      /* GC for drawing binding */
    GC binding2_gc;                     /* GC for drawing binding */
    GC foreground_gc;                   /* GC for drawing foreground */
    GC background_gc;                   /* GC for drawing background */
    GC background2_gc;                  /* GC for drawing background */

} XmNotebookPart;

/* Full instance record declaration */
typedef struct _XmNotebookWidgetRec
{
    CorePart core;
    CompositePart composite;
    ConstraintPart constraint;
    XmManagerPart manager;
    XmNotebookPart notebook;
} XmNotebookWidgetRec;





#ifdef __cplusplus
}
#endif

#endif // _NOTEBKP_

